/*
 * Polar Coordinates
 * by Javantea
 * May 11, 2016
 * Based on polar1.js Polar Coordinates
 * by Javantea
 * Jan 25, 2016
 * Based on polar1.py Polar Coordinates
 * by Javantea
 * Nov 10, 2015
 * Based on AltSci Algorithm Spherical Coordinates Take 2
 * by Javantea
 * Oct 26, 2014
 *
 */
#include <cmath>
#include "sww.h"
//SMALL = 1e-12
#define SMALL_SQ 1e-24


PolarVector::PolarVector(float r_init, float theta_init)
{
	r = r_init;
	theta = theta_init;
}

PolarVector::PolarVector(Vector2 *p)
{
	PolarVector i = cartesianToPolar(p);
	r = i.r;
	theta = i.theta;
}

/* *shrug* */
PolarVector::PolarVector()
{
}

PolarVector::~PolarVector()
{}

/*
 * cartesianToPolar(v)
	Input vector (x, y)
	Output vector (r, theta)
	r = |v|
	phi = atan2(y, x)
 */
PolarVector cartesianToPolar(Vector2 *v)
{
	//# r = |v|
	float r = sqrt(v->x*v->x + v->y*v->y);

	//theta is the angle made by x axis and v.
	float theta = atan2(v->y, v->x);
	
	return PolarVector(r, theta);
} //#end def cartesianToPolar(v)

/*
 * cartesianToPolar(v)
	Input vector (r, theta)
	Output vector (x, y)
	See cartesianToPolar for formulae or the source.
 */ 
Vector2 polarToCartesian(PolarVector *v)
{
	Vector2 r;
	r.x = v->r * cos(v->theta);
	r.y = v->r * sin(v->theta);
	return r;
} //#end def polarToCartesian(v)

/*
 * polar_sub(v1, v2)
	NOTE: Don't use this function, use cartesian coordinates from the get 
	go, then convert to polar.
 */
PolarVector polar_sub(PolarVector *v1, PolarVector *v2)
{
	Vector2 vc1 = polarToCartesian(v1);
	Vector2 vc2 = polarToCartesian(v2);
	Vector2 diff = vc1 - vc2;
	return cartesianToPolar(&diff);
} //#end def polar_sub(v1, v2)

/*
 * polar_add(v1, v2)
	NOTE: Don't use this function, use cartesian coordinates from the get 
	go, then convert to polar.
 */
PolarVector polar_add(PolarVector *v1, PolarVector *v2)
{
	Vector2 vc1 = polarToCartesian(v1);
	Vector2 vc2 = polarToCartesian(v2);
	Vector2 sum = vc1 + vc2;
	return cartesianToPolar(&sum);
} //#end def polar_add(v1, v2)

#ifdef TEST

// TODO: Finish converting the test.
/*
int main(int argc, char **argv)
{
	cartesian_vectors = [(random.random() * 10, random.random() * 10) for i in range(100)]
	polar_vectors = [cartesianToPolar(v) for v in cartesian_vectors]
	print('\n'.join(['%3.3f %3.3f' % v for v in polar_vectors]))
	dvs = vec_sub(polar_vectors[0], polar_vectors[1])
	dvc = vec_sub(cartesian_vectors[0], cartesian_vectors[1])
	diff1 = vec_sub(dvs, cartesianToPolar(dvc))
	diff2 = vec_sub(polarToCartesian(dvs), dvc)
	//# NOTE: Subtraction in Polar coordinates is not the same as 
	//# subtraction in cartesian coordinates!
	//# For example, (0, 0) - (r, theta) = (r, theta+pi)
	print("not same", diff1, diff2)
	
	//# This works:
	dvs = polar_sub(polar_vectors[0], polar_vectors[1])
	dvc = vec_sub(cartesian_vectors[0], cartesian_vectors[1])
	diff1 = polar_sub(dvs, cartesianToPolar(dvc))
	diff2 = vec_sub(polarToCartesian(dvs), dvc)
	print('same', diff1, diff2)
	assert(diff1[0] < SMALL) # r should be 0
	assert(diff2[0] < SMALL) # x should be 0
	assert(diff2[1] < SMALL) # y should be 0
}
*/
#endif // TEST
