#!/usr/bin/env python3
"""
Histogram 49 50 51 53 54 55
by Javantea
June 30, 2016

Draw a histogram instead of a graph.

42000 data points or something.
nodes conns branches b dt  b V g dt  grid V     s dt  spiral V  l dt   lattice V   n dt  n V p dt   loop V    r dt  right V    e dt  random V    f dt   FR V        seed
118   24    0        0.273 inf 5.349 11518542.0 6.721 1071763.0 10.754 1028196.125 0.245 inf 15.407 2337427.5 5.623 3574013.75 4.374 348288864.0 27.692 183576288.0 9999
"""
import numpy as np
import matplotlib.pyplot as plt
import sys

def histogram_high_range(col, fname='', bins=10):
	col2 = list(col[:])
	junk = True
	removed = []
	rounds = 0
	while junk:
		try:
			y, x = np.histogram(col2)
		except ValueError:
			print("%s failed." % (fname))
			return [], [], [], rounds
		#end try
		if sum(y[1:]) > (y[0]/200):
			junk = False
		else:
			if rounds > len(col)-3: break
			col2max = max(col2)
			removed.append(col2max)
			col2.remove(col2max)
		#end if
		rounds += 1
	#loop
	return y, x, removed, rounds
#end def histogram_high_range(col, fname='')

def histogram_by_half(col, fname='', bins=10):
	col2 = list(col[:])
	col2.sort()
	try:
		y, x = np.histogram(col2[:len(col2)>>1], bins)
	except ValueError:
		print("%s failed." % (fname))
		return [], [], [], []
	#end try
	try:
		y2, x2 = np.histogram(col2[len(col2)>>1:], bins)
	except ValueError:
		print("%s failed." % (fname))
		return y, x, [], []
	#end try
	return y, x, y2, x2
#end def histogram_by_half(col, fname='')


a = np.loadtxt('grid1testz49_50_51_53_54_55.txt.dat')
cols = [[c[i] for c in a] for i in range(len(a[0]))]

fields = 'nodes conns branches branch V grid V spiral V lattice V linear V loop V right V random V FruchtermanReingold V seed'.split(' ')

#print(cols[-1])

cols_plot = cols
field_i = 0
if len(sys.argv) > 1:
	field_i = fields.index(sys.argv[1])
	cols_plot = cols[field_i:field_i+2]
#end if

for i, col in enumerate(cols_plot):
	w = 1
	# Rectify if they skip to a certain pair of fields.
	i += field_i
	fname = fields[i]
	if fname == 'V':
		fname = fields[i-1]
	else:
		print("%s:" % (fields[i]))
	if fields[i] == 'V':
		y, x, y2, x2 = histogram_by_half(col, fname, 20)
		if len(x) > 1: w = x[1]-x[0]
		print(x)
		print(y)
		plt.bar(x[:-1], y, w)
		plt.show()
		if i == 10:
			# Don't compare lattice to lattice. We know it is 1.
			continue
		#end if
		# Compare this potential to lattice
		y, x, y2, x2 = histogram_by_half(np.array(col)/np.array(cols[10]), fname, 20)
		if len(x) > 1: w = x[1]-x[0]
		if len(y) > 0:
			plt.bar(x[:-1], y, w)
			plt.show()
	else:
		y, x = np.histogram(col, 20)
		print(x)
		print(y)
		if len(x) > 1: w = x[1]-x[0]
		plt.bar(x[:-1], y, w)
		plt.show()
	#end if
#next i, col

"""
nodes:
[  12.    25.8   39.6   53.4   67.2   81.    94.8  108.6  122.4  136.2
  150. ]
[296 291 291 315 290 300 299 282 311 325]
[  1.    3.9   6.8   9.7  12.6  15.5  18.4  21.3  24.2  27.1  30. ]
[293 299 292 287 309 286 318 318 293 305]
conns:
[  0.   6.  12.  18.  24.  30.  36.  42.  48.  54.  60.]
[287 284 309 334 248 276 301 325 310 326]
[  3.00000000e-03   3.43770000e+00   6.87240000e+00   1.03071000e+01
   1.37418000e+01   1.71765000e+01   2.06112000e+01   2.40459000e+01
   2.74806000e+01   3.09153000e+01   3.43500000e+01]
[2879   40   18   18   16    9    9    9    1    1]
branches:
[  3.16027374e+01   1.00000000e+35   2.00000000e+35   3.00000000e+35
   4.00000000e+35   5.00000000e+35   6.00000000e+35   7.00000000e+35
   8.00000000e+35   9.00000000e+35   1.00000000e+36]
[ 192    0    0    0    0    0    0    0    0 2808]
[  0.047    4.4082   8.7694  13.1306  17.4918  21.853   26.2142  30.5754
  34.9366  39.2978  43.659 ]
[1271  676  405  259  144   96   66   47   26   10]
branch:
[  2.08520374e+01   1.14979995e+33   2.29959989e+33   3.44939984e+33
   4.59919979e+33   5.74899973e+33   6.89879968e+33   8.04859962e+33
   9.19839957e+33   1.03481995e+34   1.14979995e+34]
[2999    0    0    0    0    0    0    0    0    1]
[  0.065    4.3238   8.5826  12.8414  17.1002  21.359   25.6178  29.8766
  34.1354  38.3942  42.653 ]
[1195  641  378  270  174  122  110   64   35   11]
grid:
[  1.58269943e+02   9.33676187e+21   1.86735237e+22   2.80102856e+22
   3.73470475e+22   4.66838093e+22   5.60205712e+22   6.53573331e+22
   7.46940949e+22   8.40308568e+22   9.33676187e+22]
[2999    0    0    0    0    0    0    0    0    1]
[  0.054    4.2152   8.3764  12.5376  16.6988  20.86    25.0212  29.1824
  33.3436  37.5048  41.666 ]
[1038  687  413  288  201  136  106   68   49   14]
lattice:
[  1.57958618e+02   8.96882190e+17   1.79376438e+18   2.69064657e+18
   3.58752876e+18   4.48441095e+18   5.38129314e+18   6.27817533e+18
   7.17505752e+18   8.07193971e+18   8.96882190e+18]
[2999    0    0    0    0    0    0    0    0    1]
[ 0.003   0.1169  0.2308  0.3447  0.4586  0.5725  0.6864  0.8003  0.9142
  1.0281  1.142 ]
[1014  544  450  274  268  180  113   86   47   24]
/usr/lib64/python3.3/site-packages/numpy/lib/function_base.py:215: RuntimeWarning: divide by zero encountered in double_scalars
  norm = bins / (mx - mn)
/usr/lib64/python3.3/site-packages/numpy/lib/function_base.py:237: RuntimeWarning: invalid value encountered in multiply
  tmp_a *= norm
Traceback (most recent call last):
  File "histogram_35.py", line 10, in <module>
    y, x = np.histogram(col)
  File "/usr/lib64/python3.3/site-packages/numpy/lib/function_base.py", line 249, in histogram
    n += np.bincount(indices, weights=tmp_w, minlength=bins).astype(ntype)
ValueError: The first argument of bincount must be non-negative
"""
