#!/usr/bin/env python3
"""
K-Means on a Dataset
by Javantea
Aug 13, 2016

Based loosely on http://scikit-learn.org/stable/auto_examples/cluster/plot_kmeans_digits.html
"""
import scipy
import numpy as np
import matplotlib.pyplot as plt

filename = 'grid1testz49_50_51_53_54_55.txt.dat'

a = np.loadtxt(filename)
cols = [[c[i] for c in a] for i in range(len(a[0]))]

fields = 'nodes conns branches branch V grid V spiral V lattice V linear V loop V right V random V FruchtermanReingold V seed'.split(' ')

M = np.matrix([np.array(cols[f('nodes')])+np.array(cols[f('branches')]), np.array(cols[f('conns')]), cols[f('grid')+1]])
obs = M.T

obsw = scipy.cluster.vq.whiten(obs)

km = scipy.cluster.vq.kmeans(obsw, 8)

# Find out the scaling on obsw
mult = obsw[0] / obs.A[0]

h = 0.02
x_min, x_max = obsw[:, 0].min() - 1, obsw[:, 0].max() + 1
y_min, y_max = obsw[:, 2].min() - 1, obsw[:, 2].max() + 1
xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))


def predict(centroids, v):
    """
    Get the correct value based on the input.
    """
    lensqs = np.zeros(len(centroids))
    for i, x in enumerate(centroids):
        dx = (x-v)
        lensq = dx.dot(dx)
        lensqs[i] = lensq
    #next x
    #min_index, min_value = min(enumerate(lensqs), key=operator.itemgetter(1))
    min_index = np.argmax(lensqs)
    return min_index
#end def predict(centroids, v)

avg_obsw1 = sum(obsw[:, 1])/len(obsw[:, 1])

color = np.zeros(xx.shape[0] * xx.shape[1])
i = 0
for v in np.c_[xx.ravel(), yy.ravel()]:
    color[i] = predict(km[0], np.array([v[0], avg_obsw1, v[1]]))
    i += 1
#next v

color = color.reshape(xx.shape)

plt.figure(1)
plt.clf()
plt.imshow(color, interpolation='nearest',
           extent=(xx.min(), xx.max(), yy.min(), yy.max()),
           cmap=plt.cm.Paired,
           aspect='auto', origin='lower')
plt.plot(obsw[:, 0], obsw[:, 2], 'k.', markersize=2)
# Plot the centroids as a white X
centroids = km[0]
plt.scatter(centroids[:, 0], centroids[:, 2],
            marker='x', s=169, linewidths=3,
            color='w', zorder=10)
plt.title('K-means clustering on the digits dataset (PCA-reduced data)\n'
          'Centroids are marked with white cross')
plt.xlim(x_min, x_max)
plt.ylim(y_min, y_max)
plt.xticks(())
plt.yticks(())
plt.show()

