#!/usr/bin/env python3
"""
Loop Sort
by Javantea
Dec 31, 2015

sortLoop and sortLinear originated from here, but were merged into mapmake1.Map 
so that any script can use them. Now this is a utility script.

Very fast operation, could be faster.
real    0m0.160s
user    0m0.151s
sys     0m0.009s

"""
from __future__ import print_function
import os
from sww_optimize1 import createMap4
import mapmake1

def usage():
	"Print the usage info."
	print(main.__doc__)
	return 1
#end def usage()

def main():
	"""
	Small Wide World Loop Sort Interface
This program organizes a graph by loop detection algorithm.
Usage:
loopsort1.py [-v] [-h] [-n nodes] [-c conns] [-b branches] [filename]
-v       | verbose
-h       | This usage.
-n nodes | How many nodes in the backbone [default 20]
-c conns | How many connections between the backbone [default 1]
-b branches | How many branches to add to the backbone [default 0]
-l       | Output the linear sort instead of loop sort.
-m       | Run Metropolis for 0.1s on the result.
filename | JSON file optional.
	"""
	import sys
	import getopt
	nodes = 20
	conns = 1
	branches = 0
	seed = 9999
	verbose = False
	linear = False
	fix = False
	metropolis = False
	try:
		flags, extra_args = getopt.getopt(sys.argv[1:], 'vhdn:c:b:x:lfm', ['verbose', 'help', 'nodes=', 'conns=', 'branches=', 'seed=', 'linear', 'fix', 'metropolis'])
	except getopt.GetoptError as goe:
		print(goe)
		return 1
	#end try
	for flag in flags:
		if flag[0] == '-v' or flag[0] == '--verbose':
			verbose = True
		elif flag[0] == '-h' or flag[0] == '--help':
			usage()
			return 1
		elif flag[0] == '-n' or flag[0] == '--nodes':
			nodes = int(flag[1])
		elif flag[0] == '-c' or flag[0] == '--conns':
			conns = int(flag[1])
		elif flag[0] == '-b' or flag[0] == '--branches':
			branches = int(flag[1])
		elif flag[0] == '-x' or flag[0] == '--seed':
			seed = int(flag[1])
		elif flag[0] == '-l' or flag[0] == '--linear':
			linear = True
		elif flag[0] == '-f' or flag[0] == '--fix':
			fix = True
		elif flag[0] == '-m' or flag[0] == '--metropolis':
			metropolis = True
		#end if
	#next flag
	filename = None
	if len(extra_args) > 0:
		filename = extra_args[0]
	#end if
	mapmake1.seed(seed)
	#end if
	if filename:
		map1 = mapmake1.Map()
		map1.loadJson(open(filename, 'r').read())
		nodes = len(map1.points)
		# If we need conns at some point:
		# All connections minus the backbone connections (nodes - 1)
		#conns = sum([len(n.conns) for n in map1.points]) - (nodes - 1)
		map1.desc = "SWW Loop Sort map with {} nodes from {} [results]".format(nodes, filename)
	else:
		# Don't sort please, faster.
		map1 = createMap4(nodes, conns, branches, seed, False)
		map1.desc = ("SWW Loop Sort map with {} nodes, {} conns, and " +
			   "{} branches, seed: {} [results]").format(nodes, conns, branches, seed)
	#end if
	map1.sortLinear()
	#if fix or not linear:
	#	# Fix the problem with ...
	#	map1.sortFix()
	#end if
	# Linear switch turns off the whole loop thing...
	if not linear:
		map1.sortLoop()
	#end if
	if metropolis:
		import metropolis1
		# Very short time, medium temperature, small randomization constant.
		metropolis1.Metropolis(map1, 0.1, 100, 40, 5)
	#end if
	map1.centerBound()
	if verbose:
		for p in map1.points:
			for q in p.conns:
				dr = p - q
				print("%s -- %s distance: %3.3f" % (p.name, q.name, dr.length()))
			#next q
		#next p
	#end if
	file_num = 0
	while os.path.exists('loopsort1_%i.json' % file_num):
		file_num += 1
	#loop
	open('loopsort1_%i.json' % file_num, 'w').write(map1.saveJson())
	print("Wrote loopsort1_%i.json" % file_num)
#end def main()

if __name__ == '__main__':
	main()
#end if
