#!/usr/bin/env python3
"""
Random Theta Sort
by Javantea
June 13, 2016
Based on Branch Sort
by Javantea
Feb 15, 2016
Based on Packing Unconnected Nodes According to Lennard-Jones 6-12 Potential
by Javantea
Feb 11, 2016

Branch Sort Algorithm was originally developed Nov 8, 2015, major improvements January 2016 and Feb 15, 2016.

Given a graph, assuming that there are no loops, grow branches from trunks using trigonometry.
"""
from __future__ import print_function
import sys
import os
import sww_optimize1
from sww_optimize1 import createMap4, LJ_Potential, getNonConns, V_SmallWideWorld, ConnList
mapmake1 = sww_optimize1.mapmake1

def usage():
	"Print the usage info."
	print(main.__doc__)
	return 1
#end def usage()

def main():
	"""
	Small Wide World Random Theta Sort Interface
This program organizes a graph by Random Theta algorithm.
Usage:
randthetasort1.py [-v] [-h] [-n nodes] [-c conns] [-b branches] [filename]
-v       | verbose
-h       | This usage.
-n nodes | How many nodes in the backbone [default 20]
-c conns | How many connections between the backbone [default 1]
-b branches | How many branches to add to the backbone [default 0]
filename | JSON file optional.
	"""
	import getopt
	nodes = 20
	conns = 1
	branches = 0
	seed = 9999
	verbose = False
	b0 = 40
	shuffle = False
	try:
		flags, extra_args = getopt.getopt(sys.argv[1:], 'vhdn:c:b:x:s', ['verbose', 'help', 'nodes=', 'conns=', 'branches=', 'shuffle'])
	except getopt.GetoptError as goe:
		print(goe)
		return 1
	#end try
	for flag in flags:
		if flag[0] == '-v' or flag[0] == '--verbose':
			verbose = True
		elif flag[0] == '-h' or flag[0] == '--help':
			usage()
			return 1
		elif flag[0] == '-n' or flag[0] == '--nodes':
			nodes = int(flag[1])
		elif flag[0] == '-c' or flag[0] == '--conns':
			conns = int(flag[1])
		elif flag[0] == '-b' or flag[0] == '--branches':
			branches = int(flag[1])
		elif flag[0] == '-x' or flag[0] == '--seed':
			seed = int(flag[1])
		elif flag[0] == '-s' or flag[0] == '--shuffle':
			shuffle = True
		#end if
	#next flag
	filename = None
	if len(extra_args) > 0:
		filename = extra_args[0]
	#end if
	mapmake1.seed(seed)
	lj_conf = LJ_Potential()
	
	if filename:
		map1 = mapmake1.Map()
		map1.loadJson(open(filename, 'r').read())
		nodes = len(map1.points)
		# If we need conns at some point:
		# All connections minus the backbone connections (nodes - 1)
		#conns = sum([len(n.conns) for n in map1.points]) - (nodes - 1)
		map1.desc = "SWW Random Theta Sort map with {} nodes from {} [results]".format(nodes, filename)
		out_filename = filename.replace('.json', 'randtheta.json')
	else:
		# Don't sort please, faster.
		map1 = createMap4(nodes, conns, branches, seed, False)
		map1.desc = ("SWW Random Theta Sort map with {} nodes, {} conns, and " +
			   "{} branches, seed: {} [results]").format(nodes, conns, branches, seed)
		file_num = 0
		while os.path.exists('randthetasort1_%i.json' % file_num):
			file_num += 1
		#loop
		out_filename = 'randthetasort1_%i.json' % file_num
	#end if

	map1.sortRandomTheta(b0)
	map1.centerBound()
	x1, conns = map1.getX0Conns(True)
	conns_cpp = ConnList(conns)
	nonConns = ConnList(getNonConns(conns, True))
	Vmin = V_SmallWideWorld(x1, conns_cpp, b0, lj_conf, nonConns)
	print('Energy:', Vmin)
	open(out_filename, 'w').write(map1.saveJson())
#end def main()

if __name__ == '__main__':
	main()
#end if
