#!/usr/bin/env python3
"""
rsvg_map2.py
by Javantea

Draw maps with rsvg.
This cool library can easily draw any svg from file or memory.
This is the proper way to make dynamic, editable svg systems in Python.
Eventually this frame will be used to create many cool projects such as AltSci 
Composer.
"""
from __future__ import print_function
import sys
from time import sleep, time
from threading import Thread
import cairo
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Rsvg', '2.0')
from gi.repository import Gtk, Gdk, Rsvg
from sww_optimize1 import createMap4, LJ_Potential, getNonConns, V_SmallWideWorld, V_SmallWideWorld_verbose, ConnList
import numpy as np
from metropolis1 import MetropolisAccept

class DisplaySVG(Gtk.Window):
	
	BORDER_WIDTH = 1
	#pixmap = None
	
	clx, cly = -2, -2
	
	def __init__(self, name='rsvg Map Renderer in Python', object_init=None, object_args=None):
		Gtk.Window.__init__(self)
		self.connect("delete-event", self.delete_cb)
		self.set_title(name)
		#self.set_icon_from_file('bldg1_icon32.png')
		
		self.svg_file = 'mapmake1_out.svg'
		self.data = ''
		self.svg = None
		self.seed = '222'
		self.object_init = object_init
		args1 = list(object_args)
		# The second one is always seed.
		if len(object_args) >= 2:
			self.seed = args1[1]
		#end if
		self.object_args = tuple(args1)
		if object_init == None:
			if len(object_args) >= 1:
				filename = object_args[0]
			else:
				print("Looking for a file or an object.")
				return
			#end if
			self.svg = Rsvg.Handle(file=filename)
		else:
			# rsvg can be written to from memory.
			self.map1 = object_init(*object_args)
			self.data = str(self.map1)
			self.svg = Rsvg.Handle()
			self.svg.write(self.data.encode('utf-8'))
			self.svg.close()
			#filename = 'temp1.svg'
			#file(filename, 'w').write(s)
			#raise SystemExit("need svg file")
		#end if
		
		#win.connect("expose-event", expose_cairo, svg)
		
		hbox1 = Gtk.HBox()
		vbox1 = Gtk.VBox()
		drawingarea = Gtk.DrawingArea()
		self.svgview1 = Gtk.TextView()
		buffer_1 = self.svgview1.get_buffer()
		#print buffer_1
		sw = Gtk.ScrolledWindow()
		sw.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
		sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
		sw.add(self.svgview1)
		self.svgview1.set_size_request(300, 600)
		self.attr_table = Gtk.Table(8, 3)
		self.row = {}
		self.addRow('Seed', self.seed, '', single=True)
		"""self.attr_table.attach(gtk.Label('Width'), 1, 2, 0, 1)
		self.attr_table.attach(gtk.Label('Height'), 2, 3, 0, 1)
		self.row = {}
		self.addRow('Pixel', bldg.pixels_w, bldg.pixels_h)
		self.addRow('Floor', bldg.a, bldg.h)
		self.addRow('Window', bldg.ww, bldg.wh)
		self.addRow('Door', bldg.dw, bldg.dh)
		self.addRow('Padding', bldg.pw, bldg.ph)
		self.addRow('W_m / n_i', bldg.m, bldg.ni)
		self.addRow('Floors', '', bldg.f)"""
		
		viewb1 = Gtk.Button("Test SVG")
		viewb1.connect('button_press_event', self.redraw)
		genb1 = Gtk.Button("Gen")
		genb1.connect('button_press_event', self.regen)
		savesb1 = Gtk.Button("Save SVG")
		savesb1.connect('button_press_event', self.saveSvg)
		savejb1 = Gtk.Button("Save JSON")
		savejb1.connect('button_press_event', self.saveJson)
		animateb1 = Gtk.Button("Animate")
		animateb1.connect('button_press_event', self.animate)
		centerb1 = Gtk.Button("Center")
		centerb1.connect('button_press_event', self.center)
		sortb2 = Gtk.Button("Sort Grid")
		sortb2.connect('button_press_event', self.sortGrid)
		sortbm = Gtk.Button("Metropolis")
		sortbm.connect('button_press_event', self.metropolis)
		sortb4 = Gtk.Button("Sort Best")
		sortb4.connect('button_press_event', self.sortBest)
		sortb5 = Gtk.Button("Sort Spiral")
		sortb5.connect('button_press_event', self.sortSpiral)
		sortbr = Gtk.Button("Sort Branch")
		sortbr.connect('button_press_event', self.sortBranch)
		sortbl = Gtk.Button("Sort Loop")
		sortbl.connect('button_press_event', self.sortLoop)
		sortlt = Gtk.Button("Sort Lattice")
		sortlt.connect('button_press_event', self.sortLattice)
		sortrl = Gtk.Button("Sort Right Loop")
		sortrl.connect('button_press_event', self.sortRightLoop)
		potentialb1 = Gtk.Button("Potential")
		potentialb1.connect('button_press_event', self.potential)
		but_box2 = Gtk.HButtonBox()
		but_box3 = Gtk.HButtonBox()
		but_box4 = Gtk.HButtonBox()
		but_box5 = Gtk.HButtonBox()
		but_box6 = Gtk.HButtonBox()
		but_box1 = Gtk.VBox()
		but_box2.add(viewb1)
		but_box2.add(genb1)
		but_box2.add(savesb1)
		but_box3.add(animateb1)
		but_box3.add(centerb1)
		but_box3.add(sortb2)
		but_box4.add(sortbm)
		but_box4.add(sortb4)
		but_box4.add(sortb5)
		but_box5.add(sortbr)
		but_box5.add(sortbl)
		but_box5.add(savejb1)
		but_box6.add(sortlt)
		but_box6.add(potentialb1)
		but_box6.add(sortrl)
		but_box1.add(but_box2)
		but_box1.add(but_box3)
		but_box1.add(but_box4)
		but_box1.add(but_box5)
		but_box1.add(but_box6)
		
		vbox1.add(self.attr_table)
		vbox1.add(sw)
		vbox1.add(but_box1)
		hbox1.add(drawingarea)
		hbox1.add(vbox1)
		self.add(hbox1)
		self.set_default_size(1024, 768)
		
		drawingarea.connect('draw', self.expose_cairo)
		drawingarea.connect('button_press_event', self.click_screen)
		drawingarea.connect('motion_notify_event', self.drag_screen)
		drawingarea.set_events(Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.POINTER_MOTION_MASK)
		drawingarea.set_size_request(self.svg.props.width, self.svg.props.height)
		
		print(self.svg.props.width, self.svg.props.height, self.svg.props.em, self.svg.props.ex)
		self.create_tags(buffer_1)
		self.insert_text(buffer_1, self.data)
		
		self.connect("destroy", lambda w: Gtk.main_quit())
		self.show_all()
		#print('da', sorted(dir(drawingarea)))
	#end def __init__()
	
	def getAttr(self):
		"""a = {'pixels_w': 'Pixel_width', 'pixels_h': 'Pixel_height', 
			'a': 'Floor_width', 'h': 'Floor_height', 
			'ww': 'Window_width',  'wh': 'Window_height', 
			'dw': 'Door_width',  'dh': 'Door_height', 
			'pw': 'Padding_width',  'ph': 'Padding_height', 
			'm': 'W_m / n_i_width', 'ni': 'W_m / n_i_height', 
			'f': 'Floors_height'
		}"""
		print(self.row)
		a = {'seed': 'Seed_width'}
		b = {}
		for (k, v) in list(a.items()):
			b[k] = self.row[v].get_text()
		#end if
		return b
	#end def getAttr()
	
	def addRow(self, name, default_width='', default_height='', single=False):
		row = len(self.row) + 1
		self.attr_table.attach(Gtk.Label(name), 0, 1, row, row+1)
		pw = Gtk.Entry()
		pw.set_width_chars(5)
		self.row[name + '_width'] = pw
		self.attr_table.attach(pw, 1, 2, row, row+1)
		pw.set_text(str(default_width))
		if single == False:
			ph = Gtk.Entry()
			ph.set_width_chars(5)
			self.row[name + '_height'] = ph
			self.attr_table.attach(ph, 2, 3, row, row+1)
			ph.set_text(str(default_height))
		#end if
	#end addRow(self, name)
	
	def delete_cb(self, win, event):
		Gtk.main_quit()
	
	def expose_cairo(self, win, event):
	
		alloc = win.get_allocation()
		w, h = alloc.width, alloc.height
		#print('dawh', w, h)
		if w != self.map1.width or h != self.map1.height:
			self.map1.width = w
			self.map1.height = h
			self.center(win, event)
		#end if
		#style = win.get_style().copy()
		#style.bg[gtk.STATE_NORMAL] = style.white
		#win.set_style(style)
		cr = win.get_window().cairo_create()
		
		if self.svg != None:
			self.svg.render_cairo(cr)
		
		#cr.set_source_color(win.style.fg[win.state])
		cr.rectangle(self.BORDER_WIDTH, self.BORDER_WIDTH,
				w - 2*self.BORDER_WIDTH, h - 2*self.BORDER_WIDTH)
		cr.set_line_width(2.0)
		cr.set_line_join(cairo.LINE_JOIN_ROUND)
		cr.stroke()
		
		# Draw the click.
		cr.rectangle(self.clx-1, self.cly-1, 2, 2)
		cr.stroke()
		
		return True
	
	def drag_screen(self, win, event):
		if event.state & Gdk.ModifierType.BUTTON1_MASK == 0:
			return False
		#end if
		#print 'dragging!'
		x, y, = event.x, event.y
		if self.map1.drag(x, y):
			self.drawData(win, self.map1)
		#end if
		return True
	
	def click_screen(self, win, event):
		#print('clicked!')
		if event.button == 1:
			self.clx, self.cly = event.x, event.y
			self.map1.click(event.x, event.y)
			#win.window.invalidate_rect(None, True)
			self.drawData(win, self.map1)
		#end if
		return True
	
	def saveSvg(self, win, event):
		#print 'clicked redraw!'
		# rsvg can be written to from memory.
		#a = bldgs3d.bldg3d()
		#s = str(a)
		d = Gtk.FileChooserDialog(title="Save SVG", parent=self, 
			action=Gtk.FileChooserAction.SAVE, 
			buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
				Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
		d.show()
		response = d.run()
		if response == Gtk.ResponseType.CANCEL:
			d.destroy()
			return True
		#end if
		svg_file = d.get_filename()
		d.destroy()
		buffer1 = self.svgview1.get_buffer()
		start, end = buffer1.get_bounds()
		self.svg_data = buffer1.get_text(start, end, True)
		open(svg_file, 'w').write(self.svg_data)
		print("Saved", svg_file, '.')
		return True
	#end def saveSvg(win)
	
	def saveJson(self, win, event):
		#print 'clicked redraw!'
		# rsvg can be written to from memory.
		#a = bldgs3d.bldg3d()
		#s = str(a)
		d = Gtk.FileChooserDialog(title="Save JSON", parent=self, 
			action=Gtk.FileChooserAction.SAVE, 
			buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
				Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
		d.show()
		response = d.run()
		if response == Gtk.ResponseType.CANCEL:
			d.destroy()
			return True
		#end if
		json_file = d.get_filename()
		d.destroy()
		open(json_file, 'w').write(self.map1.saveJson())
		print("Saved", json_file, '.')
		return True
	#end def saveSvg(win)
	
	def redraw(self, win, event):
		"""
		This doesn't mean redraw map1, it means redraw the svg text box.
		"""
		#print('clicked redraw!')
		# rsvg can be written to from memory.
		#a = bldgs3d.bldg3d()
		#s = str(a)
		buffer_1 = self.svgview1.get_buffer()
		start, end = buffer_1.get_bounds()
		self.data = buffer_1.get_text(start, end, True)
		self.svg = Rsvg.Handle()
		self.svg.write(self.data.encode('utf-8'))
		self.svg.close()
		win.get_window().invalidate_rect(None, True)
		return True
	
	def center(self, win, event):
		#print('clicked center!')
		#b = self.getAttr()
		#print b
		# rsvg can be written to from memory.
		#self.map1 = mapmake1.createMap(b['seed'])
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sort2(self, win, event):
		#print('sort clicked!')
		#b = self.getAttr()
		#print b
		# rsvg can be written to from memory.
		#self.map1 = mapmake1.createMap(b['seed'])
		# TODO: b0 = avg(bond length) like sww1.js
		self.map1.sort2(40)
		self.drawData(win, self.map1)
	
	def sort3(self, win, event):
		self.map1.sort3()
		self.drawData(win, self.map1)
	
	def sort4(self, win, event):
		self.map1.sort4()
		self.drawData(win, self.map1)
	
	def sort5(self, win, event):
		self.map1.sort5()
		self.drawData(win, self.map1)
	
	def sortSpiral(self, win, event):
		self.map1.sortSpiral()
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sortBranch(self, win, event):
		self.map1.sortBranch()
		self.drawData(win, self.map1)
	
	def sortLoop(self, win, event):
		self.map1.sortLinear()
		self.map1.sortLoop()
		self.drawData(win, self.map1)
	
	def sortGrid(self, win, event):
		self.map1.sortGrid()
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sortLattice(self, win, event):
		self.map1.sortLattice()
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sortRightLoop(self, win, event):
		self.map1.sortRightLoop()
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sortBest(self, win, event):
		self.map1.sortBest()
		self.drawData(win, self.map1)
	
	def regen(self, win, event):
		print('clicked regen!')
		b = self.getAttr()
		print(b)
		self.seed = b['seed']
		# rsvg can be written to from memory.
		
		args1 = list(self.object_args)
		# The second one is always seed.
		if len(self.object_args) >= 2:
			args1[1] = self.seed
		#end if
		self.object_args = tuple(args1)
		self.map1 = self.object_init(*self.object_args)
		self.drawData(win, self.map1)
		
	def drawData(self, win, map1):
		self.data = str(map1)
		buffer_1 = self.svgview1.get_buffer()
		self.clear_text(buffer_1)
		self.insert_text(buffer_1, self.data)
		#self.data = buffer_1.get_text(start, end)
		self.svg = Rsvg.Handle()
		self.svg.write(self.data.encode('utf-8'))
		self.svg.close()
		win = win.get_window()
		if win == None: return False
		win.invalidate_rect(None, True)
		return True
	#end def regen(win, event)
	
	def animate(self, win, event=None):
		self.win_anim = win
		self.anim_thread = Thread(target=self.animateLoop)
		self.anim_thread.start()
		return True
	#end def animate(win, event)
	
	def animateLoop(self):
		#amount = 10
		k = 24.
		t = k * 0.15
		for i in range(80):
			#self.map1.randomize(amount)
			#self.map1.sort2(40)
			self.map1.sortFruchtermanReingold(k, t)
			t *= 0.99
			Gdk.threads_enter()
			self.drawData(self.win_anim, self.map1)
			Gdk.threads_leave()
			sleep(0.01)
		#next i
		"""sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)"""
		return True
	#end def animateLoop()
	
	def metropolis(self, win, event=None):
		"""
		Metropolis Monte-Carlo threaded
		"""
		self.win_anim = win
		self.anim_thread = Thread(target=self.metropolisLoop)
		self.anim_thread.start()
		return True
	#end def metropolis(win, event=None)
	
	def metropolisLoop(self, temperature=200, b0=40):
		"""
		Metropolis Monte-Carlo from metropolis1.py
		Jan 24, 2016
		"""
		time_limit = 4
		lj_conf = LJ_Potential()
		x0, conns = self.map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		nonConns = ConnList(getNonConns(conns, True))
		V0 = V_SmallWideWorld(x0, conns_cpp, b0, lj_conf, nonConns)
		x_min = x0
		V_min = V0
		x_in = x0
		beta = 1 / temperature
		start = time()
		dt = 0
		while dt < time_limit:
			x_t = np.array(x_in)
			for i in range(len(x_t)):
				x_t[i] += (np.random.rand() - 0.5) * b0
			#next i
			V_t = V_SmallWideWorld(x_t, conns_cpp, b0, lj_conf, nonConns)
			dt = time() - start
			resetPos = True
			if V_t < V_min:
				V_min = V_t
				x_min = x_t
				print("Min found at t=%3.6f %3.6f" % (dt, V_t))
				resetPos = False
				m = np.matrixlib.matrix(x_min)
				m = m.reshape(len(x_min)//2, 2)
				self.map1.setPos([np.array(xa)[0] for xa in m])
				Gdk.threads_enter()
				if not self.drawData(self.win_anim, self.map1): break
				Gdk.threads_leave()
				sleep(0.01)
			elif MetropolisAccept(V_t, V_min, beta):
				# Keep this x_t if random chance tells us to.
				resetPos = False
				m = np.matrixlib.matrix(x_min)
				m = m.reshape(len(x_min)//2, 2)
				self.map1.setPos([np.array(xa)[0] for xa in m])
				Gdk.threads_enter()
				if not self.drawData(self.win_anim, self.map1): break
				Gdk.threads_leave()
				sleep(0.01)
			#end if
			if resetPos:
				# Reset to x_min
				x_in = x_min
			#end if
		#loop
		m = np.matrixlib.matrix(x_min)
		m = m.reshape(len(x_min)//2, 2)
		self.map1.setPos([np.array(xa)[0] for xa in m])
		Gdk.threads_enter()
		self.drawData(self.win_anim, self.map1)
		Gdk.threads_leave()
		# End of Metropolis algorithm
		return True
	#end def metropolisLoop()
	
	def potential(self, win, event=None):
		"""
		"""
		b0 = 40
		lj_conf = LJ_Potential()
		x0, conns = self.map1.getX0Conns(True)
		conns_cpp = ConnList(conns)
		nonConns = ConnList(getNonConns(conns, True))
		V0 = V_SmallWideWorld(x0, conns_cpp, b0, lj_conf, nonConns)
		print('Potential:', V0)
	#end def potential(win, event=None)
	
	def clear_text(self, text_buffer):
		start, end = text_buffer.get_bounds()
		text_buffer.delete(start, end)
	#end def clear_text(text_buffer)
	
	def create_tags(self, text_buffer):
		text_buffer.create_tag("word_wrap", wrap_mode=Gtk.WrapMode.WORD)
	#end def create_tags(text_buffer)
	
	def insert_text(self, text_buffer, data):
		iter = text_buffer.get_iter_at_offset(0)
		text_buffer.insert(iter, data)
		start, end = text_buffer.get_bounds()
		text_buffer.apply_tag_by_name("word_wrap", start, end)
	#end def insert_text(text_buffer)
	
#end class DisplaySVG

def main():
	from sys import path
	path.append('.')
	import mapmake1
	seed = '222'
	filename = 'file.json'
	if len(sys.argv) > 1:
		filename = sys.argv[1]
	#end if
	a = DisplaySVG('Small Wide World Renderer', mapmake1.JsonMap, (filename, seed, 1))
	Gdk.threads_init()
	Gtk.main()
#end def main()

if __name__ == '__main__':
    main()
#end if
