#!/usr/bin/env python3.4
"""
Semantics Graph
by Javantea
July 24, 2016
http://www.nltk.org/howto/semantics.html

"""
import sys
import os
import mapmake1
from nltk.sem import Valuation, Model, Assignment, Expression

def main():
	# Semantic
	if len(sys.argv) > 1:
		filename = sys.argv[1]
		val = Valuation.fromstring(open(filename, 'r').read())
		query = False
	else:
		v = [('adam', 'b1'), ('betty', 'g1'), ('fido', 'd1'),
		('girl', set(['g1', 'g2'])), ('boy', set(['b1', 'b2'])),
		('dog', set(['d1'])),
		('love', set([('b1', 'g1'), ('b2', 'g2'), ('g1', 'b1'), ('g2', 'b1')]))]
		val = Valuation(v)
		query = True
	#end if
	dom = val.domain
	m = Model(dom, val)
	g = Assignment(dom)
	if query:
		# All boys are not girls.
		query = 'all x.(boy(x) -> - girl(x))'
		print(query)
		print(m.evaluate(query, g))
		# True
		# There exists a girl who loves adam.
		query = 'exists x. (girl(x) & love(x, adam))'
		print(query)
		print(m.evaluate(query, g))
		# True
		query = r'\x. dog(x)'
		print(query)
		print(m.evaluate(query, g))
		#{'d1': True, 'b1': False, 'g1': False, 'g2': False, 'b2': False}
		
		query = r'\x. \y. (boy(x) & love(x, y))'
		print(query)
		print(m.evaluate(query, g))
		# {'d1': {'d1': False, 'b1': False, 'g1': False, 'g2': False, 'b2': False}, 'b1': {'d1': False, 'b1': False, 'g1': True, 'g2': False, 'b2': False}, 'g1': {'d1': False, 'b1': False, 'g1': False, 'g2': False, 'b2': False}, 'g2': {'d1': False, 'b1': False, 'g1': False, 'g2': False, 'b2': False}, 'b2': {'d1': False, 'b1': False, 'g1': False, 'g2': True, 'b2': False}}
		
		tests = ['(boy(x) | girl(x))',
				'!(boy(x) | girl(x))',
				'love(x, adam)',
				'love(adam, x)']
		for test in tests:
			print(test)
			print(m.satisfiers(Expression.fromstring(test), 'x', g))
		#next test

	#end if

	# Create a list of names.
	names = set()
	for x, y in val.items():
		names.add(x)
		if isinstance(y, str):
			names.add(y)
		else:
			for i in y:
				if isinstance(i, str):
					names.add(i)
				else:
					names.update(i)
				#end if
			#next i
		#end if
	#next x, y

	# Graph names
	gdl = mapmake1.Map()
	points = [mapmake1.Point(name) for name in names]
	points_map = {}
	for p in points:
		points_map[p.name] = p
	#next p

	for x, y in val.items():
		#print('x y:', x, y)
		conns = []
		if isinstance(y, str):
			conns.append(y)
		else:
			for i in y:
				if isinstance(i, str):
					conns.append(i)
				else:
					#print('cli:', conns, list(i))
					conns.extend(list(i))
					if len(i) == 2:
						il = list(i)
						points_map[il[0]].addConn(points_map[il[1]])
					#end if
				#end if
			#next i
		#end if
		for conn in conns:
			points_map[x].addConn(points_map[conn])
		#next conn
	#next x, y

	gdl.addPoints(points)
	try:
		gdl.sortBest()
	except ImportError:
		gdl.sortRightLoop()
	#end try
	file_num = 0
	while os.path.exists('semgraph1_%i.json' % file_num):
		file_num += 1
	#loop
	open('semgraph1_%i.json' % file_num, 'w').write(gdl.saveJson())
	print("Wrote semgraph1_%i.json" % file_num)
#end def main()

if __name__ == '__main__':
	main()
#end if
