#!/usr/bin/env python3
"""
Small Wide World Potential Energy Calculator
by Javantea
Jan 2, 2016

Based on sww_optimize1.py aka. SWW Minimize 1
by Javantea
Sept 30, 2015
Based on N Atom molecule using Basin-Hopping Method (aka L-BFGS-B)
by Javantea
July 23, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014

"""
from __future__ import print_function
import sww_optimize1
import mapmake1

def usage():
	print(main.__doc__)
	return 1
#end def usage()

def main():
	"""Small Wide World Potential Energy Calculator
This program computes the energy of a graph.
Usage:
sww_energy1.py [-h] [-v] filename
-h       | This usage.
-v       | Verbose Potential calculation.
filename | JSON file.
"""
	lj_conf = sww_optimize1.LJ_Potential()
	import sys
	import time
	
	filename = None
	V_SmallWideWorld = sww_optimize1.V_SmallWideWorld
	ConnList = sww_optimize1.ConnList
	verbose = False
	if '-h' in sys.argv:
		return usage()
	#end if
	if '-v' in sys.argv:
		verbose = True
		V_SmallWideWorld = sww_optimize1.V_SmallWideWorld_verbose
		ConnList = lambda x: x
		sys.argv.remove('-v')
	#end if
	if len(sys.argv) > 1:
		filename = sys.argv[1]
	#end if
	
	if filename is None:
		return usage()
	#end if
	map1 = mapmake1.Map()
	map1.loadJson(open(filename, 'r').read())
	nodes = len(map1.points)
	# If we need conns at some point:
	# All connections minus the backbone connections (nodes - 1)
	#conns = sum([len(n.conns) for n in map1.points]) - (nodes - 1)
	map1.desc = "SWW Optimize map with {} nodes from {} [results]".format(nodes, filename)
	x0, conns = map1.getX0Conns(True)
	print("Nodes:", nodes)
	print("Conns:", sum([len(cl) for cl in conns]))
	conns_cpp = ConnList(conns)
	non_conns = ConnList(sww_optimize1.getNonConns(conns, True))
	if verbose:
		print("Conns:", conns)
		print("Non-Conns:", sww_optimize1.getNonConns(conns, True))
	#end if
	#print("Configuration:")
	start = time.time()
	potential = V_SmallWideWorld(x0, conns_cpp, 40, lj_conf, non_conns)
	print("Potential:", potential)
	print('dt:', (time.time()-start))
#end def main()

if __name__ == '__main__':
	main()
#end if
