#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
SWW Minimize 1
by Javantea
Sept 30, 2015
Based on N Atom molecule using Basin-Hopping Method (aka L-BFGS-B)
by Javantea
July 23, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014

TODO: Better AcceptPotential
"""
from __future__ import print_function
import math
import numpy as np
from scipy.optimize import basinhopping
import mapmake1
import time
Vector2 = mapmake1.Vector2
getNonConns = mapmake1.getNonConns
verbose = True
try:
	import sww_cpp
except:
	if verbose: print("Slow Python version of V_SmallWideWorld in use.")
	sww_cpp = None
#end try
#sww_cpp = None

def nameIncremental(i):
	"""
	Deprecated, use nameIncremental2 unless you're trying to name things like SWW paper.
	a b c ... ab bb cb db eb
	"""
	alphabet = 'abcdefghijklmnopqrstuvwxyz'
	output = ''
	while i >= 26:
		i, p = divmod(i, 26)
		output += alphabet[p]
	#loop
	output += alphabet[i]
	return output
#end def nameIncremental(i)

def nameIncremental2(i):
	"""
	a b c ... aa ab ac ad ae ... ba bb bc bd ... ca cb cc ...
	aaa aab aac aad ... baa bab bac bad ...
	aaaa aaab aaac aaad ... baaa baab baac baad ....
	"""
	if i == 0: return 'a'
	alphabet = 'abcdefghijklmnopqrstuvwxyz'
	output = ''
	#verbose = False
	#if i >= 26*26: verbose = True
	p = i
	digit = 0
	while i > 0:
		#if verbose: print('i in', i)
		i, p = divmod(i, 26)
		if digit > 0: p -= 1
		#if verbose: print('i, p out', i, p)
		output = alphabet[p] + output
		if p < 0:
			if i == 1: break
			i -= 1
		#end if
		digit += 1
	#loop
	#output = alphabet[p] + output
	#if verbose: print('output', output)
	return output
#end def nameIncremental2(i)

def testNameIncremental2():
	alphabet = 'abcdefghijklmnopqrstuvwxyz'
	first26 = ''.join([nameIncremental2(i) for i in range(26)])
	if first26 != alphabet:
		print(0, 'a', first26)
		return False
	#end if
	for j in range(1, 27):
		next_26 = [nameIncremental2(j*26+i) for i in range(26)]
		if [len(x) for x in next_26] != [2]*26:
			print(j, 'c', next_26)
			return False
		#end if
		next_26a = ''.join([v[1] for v in next_26])
		if next_26a != alphabet:
			print(j, 'a', next_26)
			return False
		#end if
		next_26b = ''.join([v[0] for v in next_26])
		if next_26b != alphabet[j-1]*26:
			print(j, 'b', next_26, alphabet[j-1])
			return False
		#end if
	#next j
	
	for j in range(27, 27+(26*26)):
		#print(j)
		next_26 = [nameIncremental2(j*26+i) for i in range(26)]
		if [len(x) for x in next_26] != [3]*26:
			print(j, 'd', next_26)
			return False
		#end if
		next_26a = ''.join([v[2] for v in next_26])
		if next_26a != alphabet:
			print(j, 'a', next_26)
			return False
		#end if
		next_26b = ''.join([v[1] for v in next_26])
		if next_26b != alphabet[(j-27)%26]*26:
			print(j, 'b', next_26, alphabet[(j-27)%26])
			return False
		#end if
		next_26c = ''.join([v[0] for v in next_26])
		if next_26c != alphabet[(j-27)//26]*26:
			print(j, 'c', next_26, alphabet[(j-27)//26])
			return False
		#end if
	#next j
	
	if [nameIncremental2((27+(26*26))*26+i) for i in range(26)] != ['aaa'+c for c in alphabet]:
		print("The first 26 4 letter names are not correct")
		return False
	#end if
	
	if len(set([nameIncremental2(i) for i in range(100000)])) != 100000:
		print("Not unique up to 100k")
		return False
	#end if
	if len(set([nameIncremental2(i) for i in range(1000000)])) != 1000000:
		print("Not unique up to 1M")
		return False
	#end if
	if len(set([nameIncremental2(i) for i in range(10000000)])) != 10000000:
		print("Not unique up to 10M")
		return False
	#end if
	
	return True
#end def testNameIncremental2()

def sum_range(x):
	"""
	Solution for sum(range(x)) for all x.
	"""
	return (x-1)*(x>>1)+((x&1)*(x>>1))
#end def sum_range(x)

def createMap2(nodes=8, conns=7, seed_id=3243):
	connsAvailable = sum_range(nodes) - (nodes - 1)
	if conns > connsAvailable: raise ValueError("Too many connections, conns >= sum(range(nodes)) - (nodes - 1)")
	mapmake1.seed(seed_id)
	a = mapmake1.Map()
	b = []
	for i in range(nodes):
		b.append(mapmake1.Point(nameIncremental2(i)))
	#next i
	# Connect them all end to end
	for i in range(nodes-1):
		b[i].addConn(b[i+1])
	#next i
	for i in range(conns):
		addConn1 = mapmake1.randint(0,len(b)-1)
		# Fully connected!
		while len(b[addConn1].conns) >= (nodes-1):
			addConn1 = mapmake1.randint(0,len(b)-1)
		#end if
		addConn2 = addConn1
		while addConn2 == addConn1 or b[addConn2] in b[addConn1].conns:
			addConn2 = mapmake1.randint(0,len(b)-1)
		#loop
		#print b, addConn1, addConn2
		b[addConn1].addConn(b[addConn2])
	#next i
	a.addPoints(b)
	# Sort it in the way of the branch.
	a.sortBranch()
	# Only 2 pixels of randomization
	a.randomize(2)
	a.center()
	return a
#end def createMap2([nodes], [conns], [seed_id])

def connsAvailable(nodes):
	"""
	How many connections can you make given a graph already connected once only.
	Usage: connsAvailable(nodes + branches)
	"""
	return sum_range(nodes) - (nodes - 1)
#end def connsAvailable(nodes)

def createMap4(nodes=8, conns=7, branches=5, seed_id=3243, runSorts=True):
	"""
	Create a map with nodes in a backbone and random branches (which may or may not extend the backbone) using a random seed. Then connect it more between nodes to create loops conns times.
	runSorts runs sortBranch, randomize(2), and center, otherwise the graph is all set at position (0, 0).
	"""
	# sum(range(nodes+branches))
	connsAvailable1 = connsAvailable(nodes + branches)
	#print('cca', nodes, conns, branches, conns, connsAvailable1)
	if conns > connsAvailable1: raise ValueError("Too many connections, conns >= sum(range(nodes)) - (nodes - 1)")
	mapmake1.seed(seed_id)
	a = mapmake1.Map()
	b = []
	for i in range(nodes):
		b.append(mapmake1.Point(nameIncremental2(i)))
	#next i
	# Connect them all end to end
	for i in range(nodes-1):
		b[i].addConn(b[i+1])
	#next i
	# Add connections to the backbone
	connsBackbone = min(conns, sum_range(nodes) - (nodes - 1))
	for i in range(connsBackbone):
		addConn1 = mapmake1.randint(0,len(b)-1)
		# Fully connected!
		while len(b[addConn1].conns) >= (nodes-1):
			addConn1 = mapmake1.randint(0,len(b)-1)
		#end if
		addConn2 = addConn1
		while addConn2 == addConn1 or b[addConn2] in b[addConn1].conns:
			addConn2 = mapmake1.randint(0,len(b)-1)
		#loop
		#print b, addConn1, addConn2
		b[addConn1].addConn(b[addConn2])
	#next i
	for i in range(branches):
		addBranch1 = mapmake1.randint(0,len(b)-1)
		b.append(mapmake1.Point(nameIncremental2(len(b))))
		b[addBranch1].addConn(b[-1])
	#next i
	# Add any extra connections to the branches
	for i in range(conns-connsBackbone):
		addConn1 = mapmake1.randint(0,len(b)-1)
		# Fully connected!
		#print('debug cm4', [len(c.conns) for c in b], '>=', ((nodes+branches)-1))
		while len(b[addConn1].conns) >= ((nodes+branches)-1):
			addConn1 = mapmake1.randint(0,len(b)-1)
		#end if
		addConn2 = addConn1
		while addConn2 == addConn1 or b[addConn2] in b[addConn1].conns:
			addConn2 = mapmake1.randint(0,len(b)-1)
		#loop
		#print b, addConn1, addConn2
		b[addConn1].addConn(b[addConn2])
	#loop
	a.addPoints(b)
	if runSorts:
		# Sort it in the way of the branch.
		import time
		start = time.time()
		a.sortBranch()
		print("sortBranch in %3.9f seconds" % (time.time()-start))
		# Only 2 pixels of randomization
		a.randomize(2)
		a.center()
	#end if
	return a
#end def createMap4([nodes], [conns], [branches], [seed_id])

def ConnList(x):
	return x
#end def ConnList(x)
	
class LJ_Potential:
	"""
	Proper Lennard-Jones 6 12 Potential.
	"""
	def __init__(self, r0=40, scale_factor=0.055):
		self.r0 = r0
		self.scale_factor = scale_factor
		self.A = 2 * ((r0)**6) * scale_factor
		self.B = ((r0)**12) * scale_factor
	#end def __init__()
	
	def V_LJ(self, r):
		"""
		Attempt at a Lennard-Jones 6 12 potential
		The potential is very high when the two nodes are close.
		The potential is low when two nodes are far away.
		The potential is lowest when two nodes are at the r0 distance, not too 
		close, not too far away.
		Non-Connected nodes should have a LJ potential.
		r0 is the value for which this LJ potential is optimized.
		"""
		# This is a divide by zero protection.
		if r < 1e-3:
			return self.V_LJ(1e-3)
		#end if
		r6 = 1/(r ** 6)
		return ((-self.A*r6) + (self.B*r6*r6))
	#end def V_LJ(r)
#end class LJ_Potential

def toFloatArray(x):
	return x
#end def toFloatArray(x)

def V_Harmonic(r, b0):
	"""
	Attempt at a Harmonic potential
	Connected nodes should have a harmonic potential.
	b0 is the value for which this Harmonic potential is optimized.
	A is the scale of potential.
	"""
	A = 1
	return A * (r - b0)**2
#end def V_Harmonic(r, b0)

def V_Harmonic_Angle(theta, theta0):
	"""
	Attempt at a Harmonic potential for the angle between two connected items
	Connected nodes should have a harmonic potential.
	theta0 is the value for which this Harmonic potential is optimized.
	A is the scale of potential.
	"""
	A = 2
	return A * (theta - theta0)**2
#end def V_Harmonic_Angle(theta, theta0)

def V_SmallWideWorld(x, conns, b0, lj_conf, nonConns):
	"""
	Just like a molecule but in 2D.
	"""
	Potential = 0.0
	dps_all = []
	for i in range(len(conns)):
		dps_all.append([])
	#next i
	# Generate conns list for each node
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dps_all[i].append(dp)
			dps_all[j].append(dp * -1.)
		#next j
	#next i
	# Harmonic potential with length b0.
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		dps = dps_all[i]
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			Potential += V_Harmonic(dp_len, b0)
		#next j
		if len(dps) >= 2:
			# Angular Harmonic calculation
			theta0 = 2 * math.pi / len(dps)
			# theta is computed for axb, bxc, cxa
			thetas = []
			for k in range(len(dps)):
				ax = dps[k]
				thetas.append(math.atan2(ax.y, ax.x) % (2*math.pi))
			#next k
			thetas.sort()
			for k in range(len(thetas)):
				b = (k + 1) % len(thetas)
				theta = (thetas[b] - thetas[k]) % (2*math.pi)
				Potential += V_Harmonic_Angle(theta, theta0)
			#next k
		#end if
	#next i
	# Lennard-Jones for non-connected nodes.
	for i in range(len(nonConns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in nonConns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			Potential += lj_conf.V_LJ(dp_len)
		#next j
	#next i
	return Potential
#end def V_SmallWideWorld(x, conns, b0, lj_conf)

def V_SmallWideWorld_verbose(x, conns, b0, lj_conf, nonConns):
	"""
	Just like a molecule but in 2D.
	"""
	Potential = 0.0
	HP = 0.0
	HPA = 0.0
	dps_names = []
	dps_all = []
	for i in range(len(conns)):
		dps_names.append([])
		dps_all.append([])
	#next i
	# Generate conns list for each node
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dps_all[i].append(dp)
			dps_all[j].append(dp * -1.)
			dps_names[i].append('%i --> %i' % (i, j))
			dps_names[j].append('%i --> %i' % (j, i))
		#next j
	#next i
	print(dps_names)
	# Harmonic potential with length b0.
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		dps = dps_all[i]
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			v = V_Harmonic(dp_len, b0)
			print("r: %3.3f, V: %3.3f" % (dp_len, v))
			Potential += v
			HP += v
		#next j
		if len(dps) >= 2:
			# Angular Harmonic calculation
			theta0 = 2 * math.pi / len(dps)
			# theta is computed for axb, bxc, cxa
			thetas = []
			for k in range(len(dps)):
				ax = dps[k]
				thetas.append(math.atan2(ax.y, ax.x) % (2*math.pi))
			#next k
			thetas.sort()
			for k in range(len(thetas)):
				b = (k + 1) % len(thetas)
				theta = (thetas[b] - thetas[k]) % (2*math.pi)
				v = V_Harmonic_Angle(theta, theta0)
				print("theta: %3.3f, theta0: %3.3f, V: %3.3f" % (theta, theta0, v))
				Potential += v
				HPA += v
			#next k
		#end if
	#next i
	print("Harmonic Potential: %3.3f" % (HP))
	print("Angular Harmonic Potential: %3.3f" % (HPA))
	# Lennard-Jones for non-connected nodes.
	for i in range(len(nonConns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in nonConns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			v = lj_conf.V_LJ(dp_len)
			print("r: %3.3f, V: %3.3f" % (dp_len, v))
			Potential += v
		#next j
	#next i
	print("LJ Potential: %3.3f" % (Potential-(HP+HPA)))
	return Potential
#end def V_SmallWideWorld_verbose(x, conns, b0, lj_conf)

def OptimizeBasinHopping(map1, lj_conf, b0 = 40, steps=60, stepsize=50, disp=True, callback=None, gtol=1e-5):
	"""
	Optimize using SciPy BasinHopping and L-BFGS-B
	One step is fairly fast and usually helps, so why not try a step?
	Optimizes in place.
	"""
	x0, conns = map1.getX0Conns(True)
	conns_cpp = ConnList(conns)
	nonConns = ConnList(getNonConns(conns, True))
	#res = minimize(V_molecular_mult, molecule.x0, args=(molecule.conns,), method='L-BFGS-B',
	#	options={'disp': True}) #, 'T0': 0.1, 'schedule':verbose_fast_sa
	minimizer_kwargs = {'args':(conns_cpp, b0, lj_conf, nonConns), 'method':'L-BFGS-B', 'options':{'gtol':gtol, 'eps':1e-4}}
	res = basinhopping(V_SmallWideWorld, x0, minimizer_kwargs=minimizer_kwargs, 
			disp=disp, niter=steps, stepsize=stepsize, callback=callback)

	m = np.matrixlib.matrix(res.x)
	m = m.reshape(len(map1.points), 2)
	map1.setPos([np.array(xa)[0] for xa in m])
	map1.centerBound()
	return
#end def OptimizeBasinHopping(map1, lj_conf, b0 = 40, steps=60, stepsize=50, disp=True, callback=None, gtol=1e-5)

def drawLJ():
	"""
	Draw A Lennard-Jones 6-12 Potential Graph
	"""
	from matplotlib import pyplot
	lj_conf = LJ_Potential()
	data = [lj_conf.V_LJ(x) for x in range(33, 70)]
	xdata = [x for x in range(33, 70)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		fig = pyplot.figure()
		ax = fig.add_subplot(111)
		ax.set_xlabel('distance (pixels)')
		ax.set_ylabel('Potential (arbitrary magnitude)')
		ax.plot(xdata, data)
		#ax.axis([0.7, 2, 0, None])
		pyplot.show()
	#end if
#end def drawLJ()

def drawHarmonic():
	"""
	Draw A Harmonic Potential Graph
	"""
	from matplotlib import pyplot
	b0 = 40
	data = [V_Harmonic(x, b0) for x in range(0, 80)]
	xdata = [x for x in range(0, 80)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		fig = pyplot.figure()
		ax = fig.add_subplot(111)
		ax.set_xlabel('distance (pixels)')
		ax.set_ylabel('Potential (arbitrary magnitude)')
		ax.plot(xdata, data)
		#ax.axis([0.7, 2, 0, None])
		pyplot.show()
	#end if
#end def drawHarmonic()

class AcceptPotential(object):
	def __init__(self, V):
		self.V = V
	def __call__(self, x=None, f=None, accept=None):
		return (f <= self.V)
#end class AcceptPotential(object)

def usage():
	print(main.__doc__)
	return 1
#end def usage()

if sww_cpp:
	def ConnList(x):
		"""
		C++ vector vector Wrapper
		"""
		r = []
		for j in x:
			k = sww_cpp.Conns()
			k.indices = sww_cpp.IntVector(j)
			r.append(k)
		#next j
		return sww_cpp.ConnListV(r)
	#end def ConnList(x)
	
	LJ_Potential = sww_cpp.LJ_Potential
	
	def toFloatArray(x):
		"""
		C++ Float array wrapper
		TODO: Numpy support
		"""
		r = sww_cpp.floatArray(len(x))
		for i in range(len(x)):
			r[i] = float(x[i])
		#next i
		return r
	#end def toFloatArray(x)
	
	def V_SmallWideWorld(x, conns, b0, lj_conf, nonConns):
		"""
		C++ V_SmallWideWorld Wrapper
		"""
		x_conv = toFloatArray(x)
		return sww_cpp.V_SmallWideWorld(x_conv, conns, b0, lj_conf, nonConns)
	#end def V_SmallWideWorld(x, conns, b0, lj_conf, nonConns)
#end if

def main():
	"""Small Wide World Optimize 1
This program creates or optimizes an already created graph using basin-hopping global optimization.
Usage:
sww_optimize1.py [-f filename] [-lin] [nodes] [conns] [steps]
-f filename | Load a JSON file.
-lin        | Take all nodes and line them up from first to last 
              horizontally instead of sortBranch + random or how they were 
              loaded.
nodes       | If you aren't loading from file, how many nodes on the 
              graph. Default: 20
conns       | How many random connections to make between nodes on the 
              backbone. Default: 4
steps       | How many steps to take in the basin-hopping algorithm, 
              determines the quality. Default: 20
"""
	lj_conf = LJ_Potential()
	"""
	#This is how to test the LJ and the Harmonic potentials.
	#aconf = LJConf(A, B)
	#data = [E_Harmonic(x*0.1, a) for x in range(7, 20)]
	data = [V_LJ(x*1.0) for x in range(30, 80)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		pyplot.plot(data)
		pyplot.show()
	#end if
	data2 = [V_Harmonic(x*1.0, 40) for x in range(30, 80)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		pyplot.plot(data2)
		pyplot.show()
	#end if
	"""
	import sys
	import os
	
	nodes = 20
	conns = 4
	branches = 0
	steps = 20
	# This determines how long the L-BFGS-B spends trying to minimize gradient. 
	# 1-e5 gives very small improvement at a very high time cost. 1e-2 is much better at a potential cost.
	gtol = 1e-2
	filename = None
	lin = False
	actualRandom = False
	# When you finish a round and it failed to find the lowest potential, running again is as easy as:
	# mkdir redo && cd redo
	# for i in 10 13 15 16; do python3 ../../sww_optimize1.py -f ../sww_optimize1out2"$i".json; done
	if '-h' in sys.argv:
		return usage()
	#end if
	if '-t' in sys.argv:
		r = testNameIncremental2()
		if r:
			sys.exit(0)
		#end if
		sys.exit(1)
		return r
	#end if
	if '-f' in sys.argv:
		fpos = sys.argv.index('-f')+1
		if fpos >= len(sys.argv): return usage()
		filename = sys.argv[fpos]
		sys.argv.pop(fpos)
		sys.argv.remove('-f')
	#end if
	if '-lin' in sys.argv:
		lin = True
		sys.argv.remove('-lin')
	#end if
	if '-r' in sys.argv:
		actualRandom = True
		sys.argv.remove('-r')
	#end if
	if '-g' in sys.argv:
		gpos = sys.argv.index('-g')+1
		if gpos >= len(sys.argv): return usage()
		gtol = float(sys.argv[gpos])
		sys.argv.pop(gpos)
		sys.argv.remove('-g')
	#end if
	if len(sys.argv) > 1:
		nodes = int(sys.argv[1])
		if len(sys.argv) > 2:
			conns = int(sys.argv[2])
			if len(sys.argv) > 3:
				steps = int(sys.argv[3])
				if len(sys.argv) > 4:
					branches = int(sys.argv[4])
				#end if
			#end if
		#end if
	#end if
	
	if filename:
		if not actualRandom:
			mapmake1.seed(9999)
		map1 = mapmake1.Map()
		map1.loadJson(open(filename, 'r').read())
		nodes = len(map1.points)
		# If we need conns at some point:
		# All connections minus the backbone connections (nodes - 1)
		#conns = sum([len(n.conns) for n in map1.points]) - (nodes - 1)
		map1.desc = "SWW Optimize map with {} nodes from {} [results]".format(nodes, filename)
	else:
		seed = 9999
		if actualRandom:
			# TODO: Reduces the entropy to very low.
			seed = mapmake1.randint(0, 0xffffffff)
		#end if
		map1 = createMap4(nodes, conns, branches, seed)
		map1.desc = "SWW Optimize map with {} nodes and {} conns [results]".format(nodes, conns)
	#end if
	dri = map1.points[0] - map1.points[1]
	x0, conns = map1.getX0Conns(True)
	conns_cpp = ConnList(conns)
	if lin:
		# Perfectly horizontal spaces 40 apart.
		x0a = []
		j = 0
		curr_x = 10
		for x in x0:
			if j == 0:
				x0a.append(curr_x)
				curr_x += 40
				j = 1
			else:
				x0a.append(10)
				j = 0
			#end if
		#next x
		x0 = x0a
		m = np.matrixlib.matrix(x0)
		m = m.reshape(len(x0)//2, 2)
		map1.setPos([np.array(xa)[0] for xa in m])
	#end if
	nonConns = ConnList(getNonConns(conns, True))

	# FIXME: Race condition is very likely due to multiple programs running at once.
	file_num = 0
	while os.path.exists('sww_optimize1out2%ia.svg' % file_num):
		file_num += 1
	#loop
	open('sww_optimize1out2%ia.svg' % file_num, 'w').write(str(map1))
	if sum([len(c) for c in conns]) < nodes:
		# There are unconnected nodes, so we should expect lower potential
		callback = AcceptPotential(nodes*nodes*-0.055)
	else:
		# TODO: Improve this.
		callback = AcceptPotential(nodes*nodes*-0.025)
	#end if
	start = time.time()
	OptimizeBasinHopping(map1, lj_conf, 40, steps, 50, True, callback, gtol)
	print('dt: %3.3f' % (time.time()-start))
	
	print('sww_optimize1out2%i.svg' % file_num)
	print('sww_optimize1out2%i.json' % file_num)
	print("Configuration:")
	x_end = map1.getX0Conns()[0]
	print(x_end)
	Vmin = V_SmallWideWorld(x_end, conns_cpp, 40, lj_conf, nonConns)
	print("Minimum potential:", Vmin)
	dr = Vector2(*(x_end[:2])) - Vector2(*(x_end[2:4]))
	print("Distance:", dr.length())
	#x0 = []
	#for v in map1.points:
	#	x0.extend(v)
	#next v
	#x0 = np.array(x0)
	x0 = list(map(float, x0))
	V0 = V_SmallWideWorld(x0, conns_cpp, 40, lj_conf, nonConns)
	print("Initial Potential:", V0)
	print("Initial Distance:", dri.length())
	#print('debug', np.array(m[0])[0], np.array(m[1])[0])
	s = str(map1)
	results = '\nConfiguration:\n{}\nMinimum potential:{}\nDistance: {}\nInitial potential:{}\nInitial Distance: {}\n'.format(x_end, Vmin, dr.length(), V0, dri.length())
	s = s.replace('[results]', results)
	open('sww_optimize1out2%i.svg' % file_num, 'w').write(s)
	open('sww_optimize1out2%i.json' % file_num, 'w').write(map1.saveJson())
	#print("Minimum potential breakdown:")
	#V_SmallWideWorld_verbose(res.x, conns, 40, lj_conf)
#end def main()

if __name__ == '__main__':
    main()
#end if
