#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
SWW Minimize 2 (Polar with bounds)
by Javantea
Mar 3, 2016
SWW Minimize 1
by Javantea
Sept 30, 2015
Based on N Atom molecule using Basin-Hopping Method (aka L-BFGS-B)
by Javantea
July 23, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014

TODO: Better AcceptPotential

Basically, this doesn't seem to work as is. However, it helps me to learn about 
the pitfalls of transformations. If I can tame L-BFGS-B and/or basinhopping, it 
might be much better than sww_optimize1.py (which is the current best 
algorithm).
"""
from __future__ import print_function
import math
import numpy as np
from scipy.optimize import basinhopping
import mapmake1
Vector2 = mapmake1.Vector2
from sww_optimize1 import createMap4, LJ_Potential, V_SmallWideWorld, V_Harmonic, V_Harmonic_Angle, getNonConns, AcceptPotential, ConnList

def V_SmallWideWorld_polar(xpol, conns, map1, b0, lj_conf, nonConns):
	"""
	Just like a molecule but in 2D.
	"""
	# FIXME: This easy conversion method is very inefficient:
	#print(xpol)
	m = np.matrixlib.matrix(xpol)
	m = m.reshape(len(map1.points), 2)
	map1.setCartesian([np.array(xa)[0] for xa in m])
	#print([np.array(xa)[0] for xa in m])
	x = map1.getX0Conns()[0]
	#open('sww_optimize2out_test.json', 'w').write(map1.saveJson())
	#V_SmallWideWorld_polar_verbose(xpol, conns, map1, b0, lj_conf, nonConns)
	#exit(1)
	#print(x)

	Potential = 0.0
	# Harmonic potential with length b0.
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		verts = [pt_i]
		dps = []
		dp_lens = []
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			Potential += V_Harmonic(dp_len, b0)
			verts.append(pt_j)
			dps.append(dp)
			dp_lens.append(dp_len)
		#next j
		if len(verts) > 2:
			# Angular Harmonic calculation
			theta0 = 2 * math.pi / len(conns[i])
			# theta is computed for axb, bxc, cxa
			# FIXME: There was a bug here where I was using i. Now I'm confused.
			for k in range(len(conns[i])):
				b = (k+1) % len(dps)
				# Theta between axb is arccos((ax·xb)/(|ax|*|xb|))
				ax = dps[k]
				xb = dps[b]
				#print('xb', '(%3.3f,%3.3f)' % (xb.x, xb.y), '(%3.3f,%3.3f)' % (dps[b].x, dps[b].y), xb.length(), dp_lens[b], dps[b].length())
				top = ax.dot(xb)
				bot = dp_lens[k]*dp_lens[b]
				if bot == 0:
					theta = 0
				else:
					intermediate = top / bot
					# A very stupid floating point bug.
					if intermediate < -1.0:
						theta = math.pi
					elif intermediate > 1.0:
						theta = 0.0
					else:
						theta = math.acos(intermediate)
					#end if
				#end if
				Potential += V_Harmonic_Angle(theta, theta0)
			#next i
		#end if
	#next i
	# Lennard-Jones for non-connected nodes.
	for i in range(len(nonConns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in nonConns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			Potential += lj_conf.V_LJ(dp_len)
		#next j
	#next i
	return Potential
#end def V_SmallWideWorld_polar(x, conns, b0, lj_conf)

def V_SmallWideWorld_polar_verbose(xpol, conns, map1, b0, lj_conf, nonConns):
	"""
	Just like a molecule but in 2D.
	"""
	# FIXME: This easy conversion method is very inefficient:
	m = np.matrixlib.matrix(xpol)
	m = m.reshape(len(map1.points), 2)
	map1.setCartesian([np.array(xa)[0] for xa in m])
	x, conns = map1.getX0Conns(True)

	Potential = 0.0
	HP = 0.0
	HPA = 0.0
	# Harmonic potential with length b0.
	for i in range(len(conns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		verts = [pt_i]
		dps = []
		dp_lens = []
		for j in conns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			v = V_Harmonic(dp_len, b0)
			print("r: %3.3f, V: %3.3f" % (dp_len, v))
			Potential += v
			HP += v
			verts.append(pt_j)
			dps.append(dp)
			dp_lens.append(dp_len)
		#next j
		if len(verts) > 2:
			# Angular Harmonic calculation
			theta0 = 2 * math.pi / len(conns[i])
			# theta is computed for axb, bxc, cxa
			for k in range(len(conns[i])):
				b = (k+1) % len(dps)
				# Theta between axb is arccos((ax·xb)/(|ax|*|xb|))
				ax = dps[k]
				xb = dps[b]
				#print('xb', '(%3.3f,%3.3f)' % (xb.x, xb.y), '(%3.3f,%3.3f)' % (dps[b].x, dps[b].y), xb.length(), dp_lens[b], dps[b].length())
				top = ax.dot(xb)
				bot = dp_lens[k]*dp_lens[b]
				if bot == 0:
					theta = 0
				else:
					#print(top, bot)
					theta = math.acos(top/bot)
				#end if
				v = V_Harmonic_Angle(theta, theta0)
				print("theta: %3.3f, V: %3.3f" % (theta, v))
				Potential += v
				HPA += v
			#next i
		#end if
	#next i
	print("Harmonic Potential: %3.3f" % (HP))
	print("Angular Harmonic Potential: %3.3f" % (HPA))
	# Lennard-Jones for non-connected nodes.
	for i in range(len(nonConns)):
		pt_i = Vector2(*x[2*i:2*i+2])
		for j in nonConns[i]:
			pt_j = Vector2(*x[2*j:2*j+2])
			dp = pt_i - pt_j
			dp_len = dp.length()
			v = lj_conf.V_LJ(dp_len)
			print("r: %3.3f, V: %3.3f" % (dp_len, v))
			Potential += v
		#next j
	#next i
	print("LJ Potential: %3.3f" % (Potential-(HP+HPA)))
	return Potential
#end def V_SmallWideWorld_polar_verbose(x, conns, b0, lj_conf)

def OptimizeBasinHopping_polar(map1, lj_conf, b0 = 40, steps=60, stepsize=50, disp=True, callback=None, gtol=1e-5):
	"""
	Optimize using SciPy BasinHopping and L-BFGS-B
	One step is fairly fast and usually helps, so why not try a step?
	Optimizes in place.
	"""
	x0cart, conns = map1.getX0Conns(True)
	conns_cpp = ConnList(conns)
	x0, atom_hash = map1.toPolar()
	nonConns = ConnList(getNonConns(conns, True))
	#res = minimize(V_molecular_mult, molecule.x0, args=(molecule.conns,), method='L-BFGS-B',
	#	options={'disp': True}) #, 'T0': 0.1, 'schedule':verbose_fast_sa
	# TODO: Create boundary conditions
	# r is bound by 10 for now, theta is bound to all valid values.
	# the first r is bound to nothing because...
	bounds = [[0,50],[-mapmake1.pi, mapmake1.pi]] + [[30, 50],[-mapmake1.pi, mapmake1.pi]] * (len(map1.points)-1)
	minimizer_kwargs = {'args':(conns_cpp, map1, b0, lj_conf, nonConns), 'method':'L-BFGS-B', 'options':{'gtol':gtol, 'eps':1e-4}, 'bounds':bounds}
	res = basinhopping(V_SmallWideWorld_polar, x0, minimizer_kwargs=minimizer_kwargs, 
			disp=disp, niter=steps, stepsize=stepsize, callback=callback)

	m = np.matrixlib.matrix(res.x)
	m = m.reshape(len(map1.points), 2)
	map1.setCartesian([np.array(xa)[0] for xa in m])
	map1.centerBound()
	return
#end def OptimizeBasinHopping_polar(map1, lj_conf, steps=60, stepsize=50, disp=True, callback=None, gtol=1e-5)

def usage():
	print(main.__doc__)
	return 1
#end def usage()

def main():
	"""Small Wide World Optimize 2
This program creates or optimizes an already created graph using basin-hopping global optimization.
Usage:
sww_optimize2.py [-f filename] [-lin] [nodes] [conns] [steps]
-f filename | Load a JSON file.
-lin        | Take all nodes and line them up from first to last 
              horizontally instead of sortBranch + random or how they were 
              loaded.
nodes       | If you aren't loading from file, how many nodes on the 
              graph. Default: 20
conns       | How many random connections to make between nodes on the 
              backbone. Default: 4
steps       | How many steps to take in the basin-hopping algorithm, 
              determines the quality. Default: 20
"""
	lj_conf = LJ_Potential()
	"""
	#This is how to test the LJ and the Harmonic potentials.
	#aconf = LJConf(A, B)
	#data = [E_Harmonic(x*0.1, a) for x in range(7, 20)]
	data = [V_LJ(x*1.0) for x in range(30, 80)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		pyplot.plot(data)
		pyplot.show()
	#end if
	data2 = [V_Harmonic(x*1.0, 40) for x in range(30, 80)]
	#print('\n'.join(['%3.3f %3.3f' % ((x*0.01) + 1, E) for (x, E) in enumerate(data)]))
	
	if pyplot:
		# TODO: x axis.
		pyplot.plot(data2)
		pyplot.show()
	#end if
	"""
	import sys
	import os
	mapmake1.loadPolar()
	
	nodes = 20
	conns = 4
	branches = 0
	steps = 20
	# This determines how long the L-BFGS-B spends trying to minimize gradient. 
	# 1-e5 gives very small improvement at a very high time cost. 1e-2 is much better at a potential cost.
	gtol = 1e-2
	filename = None
	lin = False
	actualRandom = False
	# When you finish a round and it failed to find the lowest potential, running again is as easy as:
	# mkdir redo && cd redo
	# for i in 10 13 15 16; do python3 ../../sww_optimize1.py -f ../sww_optimize1out2"$i".json; done
	if '-h' in sys.argv:
		return usage()
	#end if
	if '-f' in sys.argv:
		fpos = sys.argv.index('-f')+1
		if fpos >= len(sys.argv): return usage()
		filename = sys.argv[fpos]
		sys.argv.pop(fpos)
		sys.argv.remove('-f')
	#end if
	if '-lin' in sys.argv:
		lin = True
		sys.argv.remove('-lin')
	#end if
	if '-r' in sys.argv:
		actualRandom = True
		sys.argv.remove('-r')
	#end if
	if '-g' in sys.argv:
		gpos = sys.argv.index('-g')+1
		if gpos >= len(sys.argv): return usage()
		gtol = float(sys.argv[gpos])
		sys.argv.pop(gpos)
		sys.argv.remove('-g')
	#end if
	if len(sys.argv) > 1:
		nodes = int(sys.argv[1])
		if len(sys.argv) > 2:
			conns = int(sys.argv[2])
			if len(sys.argv) > 3:
				steps = int(sys.argv[3])
				if len(sys.argv) > 4:
					branches = int(sys.argv[4])
				#end if
			#end if
		#end if
	#end if
	
	if filename:
		if not actualRandom:
			mapmake1.seed(9999)
		map1 = mapmake1.Map()
		map1.loadJson(open(filename, 'r').read())
		nodes = len(map1.points)
		# If we need conns at some point:
		# All connections minus the backbone connections (nodes - 1)
		#conns = sum([len(n.conns) for n in map1.points]) - (nodes - 1)
		map1.desc = "SWW Optimize map with {} nodes from {} [results]".format(nodes, filename)
	else:
		seed = 9999
		if actualRandom:
			# TODO: Reduces the entropy to very low.
			seed = mapmake1.randint(0, 0xffffffff)
		#end if
		map1 = createMap4(nodes, conns, branches, seed)
		map1.desc = "SWW Optimize map with {} nodes and {} conns [results]".format(nodes, conns)
	#end if
	dri = map1.points[0] - map1.points[1]
	# Cartesian coordinates, we want the polar coordinates.
	x0cart, conns = map1.getX0Conns(True)
	conns_cpp = ConnList(conns)
	if lin:
		# Perfectly horizontal spaces 40 apart.
		x0carta = []
		j = 0
		curr_x = 10
		for x in x0cart:
			if j == 0:
				x0carta.append(curr_x)
				curr_x += 40
				j = 1
			else:
				x0carta.append(10)
				j = 0
			#end if
		#next x
		x0cart = x0carta
		m = np.matrixlib.matrix(x0cart)
		m = m.reshape(len(x0cart)//2, 2)
		map1.setPos([np.array(xa)[0] for xa in m])
	#end if
	nonConns = ConnList(getNonConns(conns, True))

	# FIXME: Race condition is very likely due to multiple programs running at once.
	file_num = 0
	while os.path.exists('sww_optimize2out%ia.svg' % file_num):
		file_num += 1
	#loop
	open('sww_optimize2out%ia.svg' % file_num, 'w').write(str(map1))
	x0, atom_hash = map1.toPolar()
	# Fixed a bug in toPolar and getPolarConn with this, so I'm leaving it.
	#print(atom_hash.hexdigest(), len(x0), len(map1.points))
	#m = np.matrixlib.matrix(x0)
	#m = m.reshape(len(map1.points), 2)
	#map1.setCartesian([np.array(xa)[0] for xa in m])
	#print([np.array(xa)[0] for xa in m])
	#open('sww_optimize2out%ib.json' % file_num, 'w').write(map1.saveJson())
	if sum([len(c) for c in conns]) < nodes:
		# There are unconnected nodes, so we should expect lower potential
		# TODO: account for number of connections lerp or something.
		# nodes | pot
		# 20    | -6
		# 10    | -4
		# 30    | -8
		callback = AcceptPotential(nodes*-0.33)
	else:
		# See r0_40_nodes_potential.dat
		callback = AcceptPotential(0.11 * nodes * nodes)
	#end if
	OptimizeBasinHopping_polar(map1, lj_conf, 40, steps, 10, True, callback, gtol)
	
	print('sww_optimize2out%i.svg' % file_num)
	print('sww_optimize2out%i.json' % file_num)
	print("Configuration:")
	x_end = map1.getX0Conns()[0]
	print(x_end)
	Vmin = V_SmallWideWorld(x_end, conns_cpp, 40, lj_conf, nonConns)
	print("Minimum potential:", Vmin)
	dr = Vector2(*(x_end[:2])) - Vector2(*(x_end[2:4]))
	print("Distance:", dr.length())
	#x0 = []
	#for v in map1.points:
	#	x0.extend(v)
	#next v
	#x0 = np.array(x0)
	V0 = V_SmallWideWorld(x0cart, conns_cpp, 40, lj_conf, nonConns)
	print("Initial Potential:", V0)
	print("Initial Distance:", dri.length())
	#print('debug', np.array(m[0])[0], np.array(m[1])[0])
	s = str(map1)
	results = '\nConfiguration:\n{}\nMinimum potential:{}\nDistance: {}\nInitial potential:{}\nInitial Distance: {}\n'.format(x_end, Vmin, dr.length(), V0, dri.length())
	s = s.replace('[results]', results)
	open('sww_optimize2out%i.svg' % file_num, 'w').write(s)
	open('sww_optimize2out%i.json' % file_num, 'w').write(map1.saveJson())
	#print("Minimum potential breakdown:")
	#V_SmallWideWorld_polar_verbose(res.x, conns, 40, lj_conf)
#end def main()

if __name__ == '__main__':
    main()
#end if
