// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
if (Module.hasOwnProperty(key)) {
moduleOverrides[key] = Module[key];
}
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_WEB = typeof window === 'object';
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function' && !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_WORKER;
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
// Expose functionality in the same simple way that the shells work
// Note that we pollute the global namespace here, otherwise we break in node
if (!Module['print']) Module['print'] = function print(x) {
process['stdout'].write(x + '\n');
};
if (!Module['printErr']) Module['printErr'] = function printErr(x) {
process['stderr'].write(x + '\n');
};

var nodeFS = require('fs');
var nodePath = require('path');

Module['read'] = function read(filename, binary) {
filename = nodePath['normalize'](filename);
var ret = nodeFS['readFileSync'](filename);
// The path is absolute if the normalized version is the same as the resolved.
if (!ret && filename != nodePath['resolve'](filename)) {
filename = path.join(__dirname, '..', 'src', filename);
ret = nodeFS['readFileSync'](filename);
}
if (ret && !binary) ret = ret.toString();
return ret;
};

Module['readBinary'] = function readBinary(filename) {
var ret = Module['read'](filename, true);
if (!ret.buffer) {
ret = new Uint8Array(ret);
}
assert(ret.buffer);
return ret;
};

Module['load'] = function load(f) {
globalEval(read(f));
};

if (!Module['thisProgram']) {
if (process['argv'].length > 1) {
Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
} else {
Module['thisProgram'] = 'unknown-program';
}
}

Module['arguments'] = process['argv'].slice(2);

if (typeof module !== 'undefined') {
module['exports'] = Module;
}

process['on']('uncaughtException', function(ex) {
// suppress ExitStatus exceptions from showing an error
if (!(ex instanceof ExitStatus)) {
throw ex;
}
});

Module['inspect'] = function () { return '[Emscripten Module object]'; };
}
else if (ENVIRONMENT_IS_SHELL) {
if (!Module['print']) Module['print'] = print;
if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

if (typeof read != 'undefined') {
Module['read'] = read;
} else {
Module['read'] = function read() { throw 'no read() available (jsc?)' };
}

Module['readBinary'] = function readBinary(f) {
if (typeof readbuffer === 'function') {
return new Uint8Array(readbuffer(f));
}
var data = read(f, 'binary');
assert(typeof data === 'object');
return data;
};

if (typeof scriptArgs != 'undefined') {
Module['arguments'] = scriptArgs;
} else if (typeof arguments != 'undefined') {
Module['arguments'] = arguments;
}

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
Module['read'] = function read(url) {
var xhr = new XMLHttpRequest();
xhr.open('GET', url, false);
xhr.send(null);
return xhr.responseText;
};

if (typeof arguments != 'undefined') {
Module['arguments'] = arguments;
}

if (typeof console !== 'undefined') {
if (!Module['print']) Module['print'] = function print(x) {
console.log(x);
};
if (!Module['printErr']) Module['printErr'] = function printErr(x) {
console.log(x);
};
} else {
// Probably a worker, and without console.log. We can do very little here...
var TRY_USE_DUMP = false;
if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
dump(x);
}) : (function(x) {
// self.postMessage(x); // enable this if you want stdout to be sent as messages
}));
}

if (ENVIRONMENT_IS_WORKER) {
Module['load'] = importScripts;
}

if (typeof Module['setWindowTitle'] === 'undefined') {
Module['setWindowTitle'] = function(title) { document.title = title };
}
}
else {
// Unreachable because SHELL is dependant on the others
throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
Module['load'] = function load(f) {
globalEval(Module['read'](f));
};
}
if (!Module['print']) {
Module['print'] = function(){};
}
if (!Module['printErr']) {
Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
Module['arguments'] = [];
}
if (!Module['thisProgram']) {
Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
if (moduleOverrides.hasOwnProperty(key)) {
Module[key] = moduleOverrides[key];
}
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
setTempRet0: function (value) {
tempRet0 = value;
},
getTempRet0: function () {
return tempRet0;
},
stackSave: function () {
return STACKTOP;
},
stackRestore: function (stackTop) {
STACKTOP = stackTop;
},
getNativeTypeSize: function (type) {
switch (type) {
case 'i1': case 'i8': return 1;
case 'i16': return 2;
case 'i32': return 4;
case 'i64': return 8;
case 'float': return 4;
case 'double': return 8;
default: {
if (type[type.length-1] === '*') {
return Runtime.QUANTUM_SIZE; // A pointer
} else if (type[0] === 'i') {
var bits = parseInt(type.substr(1));
assert(bits % 8 === 0);
return bits/8;
} else {
return 0;
}
}
}
},
getNativeFieldSize: function (type) {
return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
},
STACK_ALIGN: 16,
prepVararg: function (ptr, type) {
if (type === 'double' || type === 'i64') {
// move so the load is aligned
if (ptr & 7) {
assert((ptr & 7) === 4);
ptr += 4;
}
} else {
assert((ptr & 3) === 0);
}
return ptr;
},
getAlignSize: function (type, size, vararg) {
// we align i64s and doubles on 64-bit boundaries, unlike x86
if (!vararg && (type == 'i64' || type == 'double')) return 8;
if (!type) return Math.min(size, 8); // align structures internally to 64 bits
return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
},
dynCall: function (sig, ptr, args) {
if (args && args.length) {
assert(args.length == sig.length-1);
if (!args.splice) args = Array.prototype.slice.call(args);
args.splice(0, 0, ptr);
assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
return Module['dynCall_' + sig].apply(null, args);
} else {
assert(sig.length == 1);
assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
return Module['dynCall_' + sig].call(null, ptr);
}
},
functionPointers: [],
addFunction: function (func) {
for (var i = 0; i < Runtime.functionPointers.length; i++) {
if (!Runtime.functionPointers[i]) {
Runtime.functionPointers[i] = func;
return 2*(1 + i);
}
}
throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
},
removeFunction: function (index) {
Runtime.functionPointers[(index-2)/2] = null;
},
warnOnce: function (text) {
if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
if (!Runtime.warnOnce.shown[text]) {
Runtime.warnOnce.shown[text] = 1;
Module.printErr(text);
}
},
funcWrappers: {},
getFuncWrapper: function (func, sig) {
assert(sig);
if (!Runtime.funcWrappers[sig]) {
Runtime.funcWrappers[sig] = {};
}
var sigCache = Runtime.funcWrappers[sig];
if (!sigCache[func]) {
sigCache[func] = function dynCall_wrapper() {
return Runtime.dynCall(sig, func, arguments);
};
}
return sigCache[func];
},
getCompilerSetting: function (name) {
throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
},
stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret;  return 0; } }; return ret; },
alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
GLOBAL_BASE: 8,
QUANTUM_SIZE: 4,
__dummy__: 0
}



Module["Runtime"] = Runtime;



//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
if (!condition) {
abort('Assertion failed: ' + text);
}
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
var func = Module['_' + ident]; // closure exported function
if (!func) {
try {
func = eval('_' + ident); // explicit lookup
} catch(e) {}
}
assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
return func;
}

var cwrap, ccall;
(function(){
var JSfuncs = {
// Helpers for cwrap -- it can't refer to Runtime directly because it might
// be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
// out what the minified function name is.
'stackSave': function() {
Runtime.stackSave()
},
'stackRestore': function() {
Runtime.stackRestore()
},
// type conversion from js to c
'arrayToC' : function(arr) {
var ret = Runtime.stackAlloc(arr.length);
writeArrayToMemory(arr, ret);
return ret;
},
'stringToC' : function(str) {
var ret = 0;
if (str !== null && str !== undefined && str !== 0) { // null string
// at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
ret = Runtime.stackAlloc((str.length << 2) + 1);
writeStringToMemory(str, ret);
}
return ret;
}
};
// For fast lookup of conversion functions
var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

// C calling interface. 
ccall = function ccallFunc(ident, returnType, argTypes, args, opts) {
var func = getCFunc(ident);
var cArgs = [];
var stack = 0;
assert(returnType !== 'array', 'Return type should not be "array".');
if (args) {
for (var i = 0; i < args.length; i++) {
var converter = toC[argTypes[i]];
if (converter) {
if (stack === 0) stack = Runtime.stackSave();
cArgs[i] = converter(args[i]);
} else {
cArgs[i] = args[i];
}
}
}
var ret = func.apply(null, cArgs);
if ((!opts || !opts.async) && typeof EmterpreterAsync === 'object') {
assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling ccall');
}
if (opts && opts.async) assert(!returnType, 'async ccalls cannot return values');
if (returnType === 'string') ret = Pointer_stringify(ret);
if (stack !== 0) {
if (opts && opts.async) {
EmterpreterAsync.asyncFinalizers.push(function() {
Runtime.stackRestore(stack);
});
return;
}
Runtime.stackRestore(stack);
}
return ret;
}

var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
function parseJSFunc(jsfunc) {
// Match the body and the return value of a javascript function source
var parsed = jsfunc.toString().match(sourceRegex).slice(1);
return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
}
var JSsource = {};
for (var fun in JSfuncs) {
if (JSfuncs.hasOwnProperty(fun)) {
// Elements of toCsource are arrays of three items:
// the code, and the return value
JSsource[fun] = parseJSFunc(JSfuncs[fun]);
}
}

cwrap = function cwrap(ident, returnType, argTypes) {
argTypes = argTypes || [];
var cfunc = getCFunc(ident);
// When the function takes numbers and returns a number, we can just return
// the original function
var numericArgs = argTypes.every(function(type){ return type === 'number'});
var numericRet = (returnType !== 'string');
if ( numericRet && numericArgs) {
return cfunc;
}
// Creation of the arguments list (["$1","$2",...,"$nargs"])
var argNames = argTypes.map(function(x,i){return '$'+i});
var funcstr = "(function(" + argNames.join(',') + ") {";
var nargs = argTypes.length;
if (!numericArgs) {
// Generate the code needed to convert the arguments from javascript
// values to pointers
funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
for (var i = 0; i < nargs; i++) {
var arg = argNames[i], type = argTypes[i];
if (type === 'number') continue;
var convertCode = JSsource[type + 'ToC']; // [code, return]
funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
funcstr += convertCode.body + ';';
funcstr += arg + '=' + convertCode.returnValue + ';';
}
}


// When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
// Call the function
funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
if (!numericRet) { // Return type can only by 'string' or 'number'
// Convert the result to a string
var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
funcstr += 'ret = ' + strgfy + '(ret);';
}
funcstr += "if (typeof EmterpreterAsync === 'object') { assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling cwrap') }";
if (!numericArgs) {
// If we had a stack, restore it
funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
}
funcstr += 'return ret})';
return eval(funcstr);
};
})();
Module["ccall"] = ccall;
Module["cwrap"] = cwrap;

function setValue(ptr, value, type, noSafe) {
type = type || 'i8';
if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
switch(type) {
case 'i1': HEAP8[((ptr)>>0)]=value; break;
case 'i8': HEAP8[((ptr)>>0)]=value; break;
case 'i16': HEAP16[((ptr)>>1)]=value; break;
case 'i32': HEAP32[((ptr)>>2)]=value; break;
case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
case 'float': HEAPF32[((ptr)>>2)]=value; break;
case 'double': HEAPF64[((ptr)>>3)]=value; break;
default: abort('invalid type for setValue: ' + type);
}
}
Module["setValue"] = setValue;


function getValue(ptr, type, noSafe) {
type = type || 'i8';
if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
switch(type) {
case 'i1': return HEAP8[((ptr)>>0)];
case 'i8': return HEAP8[((ptr)>>0)];
case 'i16': return HEAP16[((ptr)>>1)];
case 'i32': return HEAP32[((ptr)>>2)];
case 'i64': return HEAP32[((ptr)>>2)];
case 'float': return HEAPF32[((ptr)>>2)];
case 'double': return HEAPF64[((ptr)>>3)];
default: abort('invalid type for setValue: ' + type);
}
return null;
}
Module["getValue"] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module["ALLOC_NORMAL"] = ALLOC_NORMAL;
Module["ALLOC_STACK"] = ALLOC_STACK;
Module["ALLOC_STATIC"] = ALLOC_STATIC;
Module["ALLOC_DYNAMIC"] = ALLOC_DYNAMIC;
Module["ALLOC_NONE"] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
var zeroinit, size;
if (typeof slab === 'number') {
zeroinit = true;
size = slab;
} else {
zeroinit = false;
size = slab.length;
}

var singleType = typeof types === 'string' ? types : null;

var ret;
if (allocator == ALLOC_NONE) {
ret = ptr;
} else {
ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
}

if (zeroinit) {
var ptr = ret, stop;
assert((ret & 3) == 0);
stop = ret + (size & ~3);
for (; ptr < stop; ptr += 4) {
HEAP32[((ptr)>>2)]=0;
}
stop = ret + size;
while (ptr < stop) {
HEAP8[((ptr++)>>0)]=0;
}
return ret;
}

if (singleType === 'i8') {
if (slab.subarray || slab.slice) {
HEAPU8.set(slab, ret);
} else {
HEAPU8.set(new Uint8Array(slab), ret);
}
return ret;
}

var i = 0, type, typeSize, previousType;
while (i < size) {
var curr = slab[i];

if (typeof curr === 'function') {
curr = Runtime.getFunctionIndex(curr);
}

type = singleType || types[i];
if (type === 0) {
i++;
continue;
}
assert(type, 'Must know what type to store in allocate!');

if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

setValue(ret+i, curr, type);

// no need to look up size unless type changes, so cache it
if (previousType !== type) {
typeSize = Runtime.getNativeTypeSize(type);
previousType = type;
}
i += typeSize;
}

return ret;
}
Module["allocate"] = allocate;

// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
function getMemory(size) {
if (!staticSealed) return Runtime.staticAlloc(size);
if ((typeof _sbrk !== 'undefined' && !_sbrk.called) || !runtimeInitialized) return Runtime.dynamicAlloc(size);
return _malloc(size);
}
Module["getMemory"] = getMemory;

function Pointer_stringify(ptr, /* optional */ length) {
if (length === 0 || !ptr) return '';
// TODO: use TextDecoder
// Find the length, and check for UTF while doing so
var hasUtf = 0;
var t;
var i = 0;
while (1) {
assert(ptr + i < TOTAL_MEMORY);
t = HEAPU8[(((ptr)+(i))>>0)];
hasUtf |= t;
if (t == 0 && !length) break;
i++;
if (length && i == length) break;
}
if (!length) length = i;

var ret = '';

if (hasUtf < 128) {
var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
var curr;
while (length > 0) {
curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
ret = ret ? ret + curr : curr;
ptr += MAX_CHUNK;
length -= MAX_CHUNK;
}
return ret;
}
return Module['UTF8ToString'](ptr);
}
Module["Pointer_stringify"] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
var str = '';
while (1) {
var ch = HEAP8[((ptr++)>>0)];
if (!ch) return str;
str += String.fromCharCode(ch);
}
}
Module["AsciiToString"] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
return writeAsciiToMemory(str, outPtr, false);
}
Module["stringToAscii"] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
var u0, u1, u2, u3, u4, u5;

var str = '';
while (1) {
// For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
u0 = u8Array[idx++];
if (!u0) return str;
if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
u1 = u8Array[idx++] & 63;
if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
u2 = u8Array[idx++] & 63;
if ((u0 & 0xF0) == 0xE0) {
u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
} else {
u3 = u8Array[idx++] & 63;
if ((u0 & 0xF8) == 0xF0) {
u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
} else {
u4 = u8Array[idx++] & 63;
if ((u0 & 0xFC) == 0xF8) {
u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
} else {
u5 = u8Array[idx++] & 63;
u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
}
}
}
if (u0 < 0x10000) {
str += String.fromCharCode(u0);
} else {
var ch = u0 - 0x10000;
str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
}
}
}
Module["UTF8ArrayToString"] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
return UTF8ArrayToString(HEAPU8,ptr);
}
Module["UTF8ToString"] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(e,t,n,r){if(r>0){var i=n,s=n+r-1;for(var o=0;o<e.length;++o){var u=e.charCodeAt(o);u>=55296&&u<=57343&&(u=65536+((u&1023)<<10)|e.charCodeAt(++o)&1023);if(u<=127){if(n>=s)break;t[n++]=u}else if(u<=2047){if(n+1>=s)break;t[n++]=192|u>>6,t[n++]=128|u&63}else if(u<=65535){if(n+2>=s)break;t[n++]=224|u>>12,t[n++]=128|u>>6&63,t[n++]=128|u&63}else if(u<=2097151){if(n+3>=s)break;t[n++]=240|u>>18,t[n++]=128|u>>12&63,t[n++]=128|u>>6&63,t[n++]=128|u&63}else if(u<=67108863){if(n+4>=s)break;t[n++]=248|u>>24,t[n++]=128|u>>18&63,t[n++]=128|u>>12&63,t[n++]=128|u>>6&63,t[n++]=128|u&63}else{if(n+5>=s)break;t[n++]=252|u>>30,t[n++]=128|u>>24&63,t[n++]=128|u>>18&63,t[n++]=128|u>>12&63,t[n++]=128|u>>6&63,t[n++]=128|u&63}}return t[n]=0,n-i}return 0};
Module["stringToUTF8Array"] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
assert(typeof maxBytesToWrite == 'number', 'stringToUTF8(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
return stringToUTF8Array(str, HEAPU8,outPtr, maxBytesToWrite);
}
Module["stringToUTF8"] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
var len = 0;
for (var i = 0; i < str.length; ++i) {
// Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
// See http://unicode.org/faq/utf_bom.html#utf16-3
var u = str.charCodeAt(i); // possibly a lead surrogate
if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
if (u <= 0x7F) {
++len;
} else if (u <= 0x7FF) {
len += 2;
} else if (u <= 0xFFFF) {
len += 3;
} else if (u <= 0x1FFFFF) {
len += 4;
} else if (u <= 0x3FFFFFF) {
len += 5;
} else {
len += 6;
}
}
return len;
}
Module["lengthBytesUTF8"] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
var i = 0;

var str = '';
while (1) {
var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
if (codeUnit == 0)
return str;
++i;
// fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
str += String.fromCharCode(codeUnit);
}
}
Module["UTF16ToString"] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
assert(typeof maxBytesToWrite == 'number', 'stringToUTF16(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
// Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
if (maxBytesToWrite === undefined) {
maxBytesToWrite = 0x7FFFFFFF;
}
if (maxBytesToWrite < 2) return 0;
maxBytesToWrite -= 2; // Null terminator.
var startPtr = outPtr;
var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
for (var i = 0; i < numCharsToWrite; ++i) {
// charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
HEAP16[((outPtr)>>1)]=codeUnit;
outPtr += 2;
}
// Null-terminate the pointer to the HEAP.
HEAP16[((outPtr)>>1)]=0;
return outPtr - startPtr;
}
Module["stringToUTF16"] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
return str.length*2;
}
Module["lengthBytesUTF16"] = lengthBytesUTF16;

function UTF32ToString(ptr) {
var i = 0;

var str = '';
while (1) {
var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
if (utf32 == 0)
return str;
++i;
// Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
// See http://unicode.org/faq/utf_bom.html#utf16-3
if (utf32 >= 0x10000) {
var ch = utf32 - 0x10000;
str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
} else {
str += String.fromCharCode(utf32);
}
}
}
Module["UTF32ToString"] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
assert(typeof maxBytesToWrite == 'number', 'stringToUTF32(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
// Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
if (maxBytesToWrite === undefined) {
maxBytesToWrite = 0x7FFFFFFF;
}
if (maxBytesToWrite < 4) return 0;
var startPtr = outPtr;
var endPtr = startPtr + maxBytesToWrite - 4;
for (var i = 0; i < str.length; ++i) {
// Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
// See http://unicode.org/faq/utf_bom.html#utf16-3
var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
var trailSurrogate = str.charCodeAt(++i);
codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
}
HEAP32[((outPtr)>>2)]=codeUnit;
outPtr += 4;
if (outPtr + 4 > endPtr) break;
}
// Null-terminate the pointer to the HEAP.
HEAP32[((outPtr)>>2)]=0;
return outPtr - startPtr;
}
Module["stringToUTF32"] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
var len = 0;
for (var i = 0; i < str.length; ++i) {
// Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
// See http://unicode.org/faq/utf_bom.html#utf16-3
var codeUnit = str.charCodeAt(i);
if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
len += 4;
}

return len;
}
Module["lengthBytesUTF32"] = lengthBytesUTF32;

function demangle(func) {
var hasLibcxxabi = !!Module['___cxa_demangle'];
if (hasLibcxxabi) {
try {
var buf = _malloc(func.length);
writeStringToMemory(func.substr(1), buf);
var status = _malloc(4);
var ret = Module['___cxa_demangle'](buf, 0, 0, status);
if (getValue(status, 'i32') === 0 && ret) {
return Pointer_stringify(ret);
}
// otherwise, libcxxabi failed, we can try ours which may return a partial result
} catch(e) {
// failure when using libcxxabi, we can try ours which may return a partial result
} finally {
if (buf) _free(buf);
if (status) _free(status);
if (ret) _free(ret);
}
}
var i = 3;
// params, etc.
var basicTypes = {
'v': 'void',
'b': 'bool',
'c': 'char',
's': 'short',
'i': 'int',
'l': 'long',
'f': 'float',
'd': 'double',
'w': 'wchar_t',
'a': 'signed char',
'h': 'unsigned char',
't': 'unsigned short',
'j': 'unsigned int',
'm': 'unsigned long',
'x': 'long long',
'y': 'unsigned long long',
'z': '...'
};
var subs = [];
var first = true;
function dump(x) {
//return;
if (x) Module.print(x);
Module.print(func);
var pre = '';
for (var a = 0; a < i; a++) pre += ' ';
Module.print (pre + '^');
}
function parseNested() {
i++;
if (func[i] === 'K') i++; // ignore const
var parts = [];
while (func[i] !== 'E') {
if (func[i] === 'S') { // substitution
i++;
var next = func.indexOf('_', i);
var num = func.substring(i, next) || 0;
parts.push(subs[num] || '?');
i = next+1;
continue;
}
if (func[i] === 'C') { // constructor
parts.push(parts[parts.length-1]);
i += 2;
continue;
}
var size = parseInt(func.substr(i));
var pre = size.toString().length;
if (!size || !pre) { i--; break; } // counter i++ below us
var curr = func.substr(i + pre, size);
parts.push(curr);
subs.push(curr);
i += pre + size;
}
i++; // skip E
return parts;
}
function parse(rawList, limit, allowVoid) { // main parser
limit = limit || Infinity;
var ret = '', list = [];
function flushList() {
return '(' + list.join(', ') + ')';
}
var name;
if (func[i] === 'N') {
// namespaced N-E
name = parseNested().join('::');
limit--;
if (limit === 0) return rawList ? [name] : name;
} else {
// not namespaced
if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
var size = parseInt(func.substr(i));
if (size) {
var pre = size.toString().length;
name = func.substr(i + pre, size);
i += pre + size;
}
}
first = false;
if (func[i] === 'I') {
i++;
var iList = parse(true);
var iRet = parse(true, 1, true);
ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
} else {
ret = name;
}
paramLoop: while (i < func.length && limit-- > 0) {
//dump('paramLoop');
var c = func[i++];
if (c in basicTypes) {
list.push(basicTypes[c]);
} else {
switch (c) {
case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
case 'L': { // literal
i++; // skip basic type
var end = func.indexOf('E', i);
var size = end - i;
list.push(func.substr(i, size));
i += size + 2; // size + 'EE'
break;
}
case 'A': { // array
var size = parseInt(func.substr(i));
i += size.toString().length;
if (func[i] !== '_') throw '?';
i++; // skip _
list.push(parse(true, 1, true)[0] + ' [' + size + ']');
break;
}
case 'E': break paramLoop;
default: ret += '?' + c; break paramLoop;
}
}
}
if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
if (rawList) {
if (ret) {
list.push(ret + '?');
}
return list;
} else {
return ret + flushList();
}
}
var parsed = func;
try {
// Special-case the entry point, since its name differs from other name mangling.
if (func == 'Object._main' || func == '_main') {
return 'main()';
}
if (typeof func === 'number') func = Pointer_stringify(func);
if (func[0] !== '_') return func;
if (func[1] !== '_') return func; // C function
if (func[2] !== 'Z') return func;
switch (func[3]) {
case 'n': return 'operator new()';
case 'd': return 'operator delete()';
}
parsed = parse();
} catch(e) {
parsed += '?';
}
if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
}
return parsed;
}

function demangleAll(text) {
return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
var err = new Error();
if (!err.stack) {
// IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
// so try that as a special-case.
try {
throw new Error(0);
} catch(e) {
err = e;
}
if (!err.stack) {
return '(no stack trace available)';
}
}
return err.stack.toString();
}

function stackTrace() {
return demangleAll(jsStackTrace());
}
Module["stackTrace"] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;

function alignMemoryPage(x) {
if (x % 4096 > 0) {
x += (4096 - (x % 4096));
}
return x;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk


function abortOnCannotGrowMemory() {
abort('Cannot enlarge memory arrays. Either (1) compile with  -s TOTAL_MEMORY=X  with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with  -s ALLOW_MEMORY_GROWTH=1  which adjusts the size at runtime but prevents some optimizations, (3) set Module.TOTAL_MEMORY to a higher value before the program runs, or if you want malloc to return NULL (0) instead of this abort, compile with  -s ABORTING_MALLOC=0 ');
}

function enlargeMemory() {
abortOnCannotGrowMemory();
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
if (totalMemory < 16*1024*1024) {
totalMemory *= 2;
} else {
totalMemory += 16*1024*1024
}
}
if (totalMemory !== TOTAL_MEMORY) {
Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be compliant with the asm.js spec (and given that TOTAL_STACK=' + TOTAL_STACK + ')');
TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
'JS engine does not provide full typed array support');

var buffer;



buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);


// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
while(callbacks.length > 0) {
var callback = callbacks.shift();
if (typeof callback == 'function') {
callback();
continue;
}
var func = callback.func;
if (typeof func === 'number') {
if (callback.arg === undefined) {
Runtime.dynCall('v', func);
} else {
Runtime.dynCall('vi', func, [callback.arg]);
}
} else {
func(callback.arg === undefined ? null : callback.arg);
}
}
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;


function preRun() {
// compatibility - merge in anything from Module['preRun'] at this time
if (Module['preRun']) {
if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
while (Module['preRun'].length) {
addOnPreRun(Module['preRun'].shift());
}
}
callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
if (runtimeInitialized) return;
runtimeInitialized = true;
callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
callRuntimeCallbacks(__ATEXIT__);
runtimeExited = true;
}

function postRun() {
// compatibility - merge in anything from Module['postRun'] at this time
if (Module['postRun']) {
if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
while (Module['postRun'].length) {
addOnPostRun(Module['postRun'].shift());
}
}
callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
__ATPRERUN__.unshift(cb);
}
Module["addOnPreRun"] = addOnPreRun;

function addOnInit(cb) {
__ATINIT__.unshift(cb);
}
Module["addOnInit"] = addOnInit;

function addOnPreMain(cb) {
__ATMAIN__.unshift(cb);
}
Module["addOnPreMain"] = addOnPreMain;

function addOnExit(cb) {
__ATEXIT__.unshift(cb);
}
Module["addOnExit"] = addOnExit;

function addOnPostRun(cb) {
__ATPOSTRUN__.unshift(cb);
}
Module["addOnPostRun"] = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
var u8array = new Array(len);
var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
if (dontAddNull) u8array.length = numBytesWritten;
return u8array;
}
Module["intArrayFromString"] = intArrayFromString;

function intArrayToString(array) {
var ret = [];
for (var i = 0; i < array.length; i++) {
var chr = array[i];
if (chr > 0xFF) {
assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
chr &= 0xFF;
}
ret.push(String.fromCharCode(chr));
}
return ret.join('');
}
Module["intArrayToString"] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
var array = intArrayFromString(string, dontAddNull);
var i = 0;
while (i < array.length) {
var chr = array[i];
HEAP8[(((buffer)+(i))>>0)]=chr;
i = i + 1;
}
}
Module["writeStringToMemory"] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
for (var i = 0; i < array.length; i++) {
HEAP8[((buffer++)>>0)]=array[i];
}
}
Module["writeArrayToMemory"] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
for (var i = 0; i < str.length; ++i) {
assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
}
// Null-terminate the pointer to the HEAP.
if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module["writeAsciiToMemory"] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
if (value >= 0) {
return value;
}
return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
: Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
if (value <= 0) {
return value;
}
var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
: Math.pow(2, bits-1);
if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
// but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
// TODO: In i64 mode 1, resign the two parts separately and safely
value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
}
return value;
}


// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
var ah  = a >>> 16;
var al = a & 0xffff;
var bh  = b >>> 16;
var bl = b & 0xffff;
return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
x = x >>> 0;
for (var i = 0; i < 32; i++) {
if (x & (1 << (31 - i))) return i;
}
return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function getUniqueRunDependency(id) {
var orig = id;
while (1) {
if (!runDependencyTracking[id]) return id;
id = orig + Math.random();
}
return id;
}

function addRunDependency(id) {
runDependencies++;
if (Module['monitorRunDependencies']) {
Module['monitorRunDependencies'](runDependencies);
}
if (id) {
assert(!runDependencyTracking[id]);
runDependencyTracking[id] = 1;
if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
// Check for missing dependencies every few seconds
runDependencyWatcher = setInterval(function() {
if (ABORT) {
clearInterval(runDependencyWatcher);
runDependencyWatcher = null;
return;
}
var shown = false;
for (var dep in runDependencyTracking) {
if (!shown) {
shown = true;
Module.printErr('still waiting on run dependencies:');
}
Module.printErr('dependency: ' + dep);
}
if (shown) {
Module.printErr('(end of list)');
}
}, 10000);
}
} else {
Module.printErr('warning: run dependency added without ID');
}
}
Module["addRunDependency"] = addRunDependency;

function removeRunDependency(id) {
runDependencies--;
if (Module['monitorRunDependencies']) {
Module['monitorRunDependencies'](runDependencies);
}
if (id) {
assert(runDependencyTracking[id]);
delete runDependencyTracking[id];
} else {
Module.printErr('warning: run dependency removed without ID');
}
if (runDependencies == 0) {
if (runDependencyWatcher !== null) {
clearInterval(runDependencyWatcher);
runDependencyWatcher = null;
}
if (dependenciesFulfilled) {
var callback = dependenciesFulfilled;
dependenciesFulfilled = null;
callback(); // can add another dependenciesFulfilled
}
}
}
Module["removeRunDependency"] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data



var memoryInitializer = null;



// === Body ===

var ASM_CONSTS = [];




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 25104;
/* global initializers */  __ATINIT__.push();


/* memory initializer */ allocate([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,169,241,210,77,98,80,63,205,204,204,204,204,204,236,63,154,153,153,153,153,153,185,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,178,53,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,241,53,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,236,54,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,80,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,140,55,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,210,56,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,99,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,156,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,195,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,1,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,51,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,101,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,51,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,156,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,161,58,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,106,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,9,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,72,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,72,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,9,61,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,172,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,66,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,144,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,197,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,241,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,249,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,63,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,133,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,203,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,173,65,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,125,66,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,128,67,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,31,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,145,68,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,200,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,1,0,0,0,255,255,255,255,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,220,70,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,128,63,0,0,0,0,1], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);
/* memory initializer */ allocate([187,73,0,0,203,73,0,0,223,73,0,0,248,73,0,0,19,74,0,0,41,74,0,0,67,74,0,0,88,74,0,0,103,74,0,0,119,74,0,0,138,74,0,0,156,74,0,0,185,74,0,0,213,74,0,0,119,74,0,0,231,74,0,0,1,75,0,0,25,75,0,0,43,75,0,0,61,75,0,0,86,75,0,0,100,75,0,0,125,75,0,0,155,75,0,0,177,75,0,0,201,75,0,0,225,75,0,0,241,75,0,0,2,76,0,0,20,76,0,0,41,76,0,0,68,76], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+7624);
/* memory initializer */ allocate([158,76,0,0,161,76,0,0,163,76,0,0,166,76,0,0,168,76,0,0,172,76,0,0,144,1,0,0,175,76,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,108,50,0,0,108,50,0,0,220,50,0,0,76,51,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,243,91,0,0,0,4,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,235,91,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,4,0,0,0,227,87,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,84,65,82,84,0,40,47,44,2,65,116,32,105,116,101,114,97,116,101,2,44,105,53,44,52,120,44,2,102,61,32,2,44,49,112,44,100,49,50,46,53,44,52,120,44,2,124,112,114,111,106,32,103,124,61,32,2,44,49,112,44,100,49,50,46,53,41,0,40,47,47,44,2,73,84,69,82,65,84,73,79,78,32,2,44,105,53,41,0,40,47,44,2,32,83,105,110,103,117,108,97,114,32,116,114,105,97,110,103,117,108,97,114,32,115,121,115,116,101,109,32,100,101,116,101,99,116,101,100,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,47,44,2,32,78,111,110,112,111,115,105,116,105,118,101,32,100,101,102,105,110,105,116,101,110,101,115,115,32,105,110,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,47,44,2,32,66,97,100,32,100,105,114,101,99,116,105,111,110,32,105,110,32,116,104,101,32,108,105,110,101,32,115,101,97,114,99,104,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,2,32,32,121,115,61,2,44,49,112,44,101,49,48,46,51,44,2,32,32,45,103,115,61,2,44,49,112,44,101,49,48,46,51,44,2,32,66,70,71,83,32,117,112,100,97,116,101,32,83,75,73,80,80,69,68,2,41,0,40,47,44,2,32,78,111,110,112,111,115,105,116,105,118,101,32,100,101,102,105,110,105,116,101,110,101,115,115,32,105,110,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,116,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,0,0,0,101,0,45,45,45,0,69,82,82,79,82,0,70,71,95,76,78,0,78,69,87,95,88,0,70,71,95,83,84,0,83,84,79,80,0,67,80,85,0,70,71,95,83,84,65,82,84,0,67,79,78,86,69,82,71,69,78,67,69,58,32,78,79,82,77,95,79,70,95,80,82,79,74,69,67,84,69,68,95,71,82,65,68,73,69,78,84,95,60,61,95,80,71,84,79,76,0,65,66,78,79,82,77,65,76,95,84,69,82,77,73,78,65,84,73,79,78,95,73,78,95,76,78,83,82,67,72,0,82,69,83,84,65,82,84,95,70,82,79,77,95,76,78,83,82,67,72,0,67,79,78,86,69,82,71,69,78,67,69,58,32,82,69,76,95,82,69,68,85,67,84,73,79,78,95,79,70,95,70,95,60,61,95,70,65,67,84,82,42,69,80,83,77,67,72,0,40,47,44,2,65,116,32,88,48,32,2,44,105,57,44,2,32,118,97,114,105,97,98,108,101,115,32,97,114,101,32,101,120,97,99,116,108,121,32,97,116,32,116,104,101,32,98,111,117,110,100,115,2,41,0,84,104,101,32,105,110,105,116,105,97,108,32,88,32,105,115,32,105,110,102,101,97,115,105,98,108,101,46,32,32,82,101,115,116,97,114,116,32,119,105,116,104,32,105,116,115,32,112,114,111,106,101,99,116,105,111,110,46,0,84,104,105,115,32,112,114,111,98,108,101,109,32,105,115,32,117,110,99,111,110,115,116,114,97,105,110,101,100,46,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,32,67,65,85,67,72,89,32,101,110,116,101,114,101,100,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,41,0,40,2,67,97,117,99,104,121,32,88,32,61,32,32,2,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,40,47,44,2,80,105,101,99,101,32,32,32,32,2,44,105,51,44,2,32,45,45,102,49,44,32,102,50,32,97,116,32,115,116,97,114,116,32,112,111,105,110,116,32,2,44,49,112,44,50,40,49,120,44,100,49,49,46,52,41,41,0,40,2,68,105,115,116,97,110,99,101,32,116,111,32,116,104,101,32,110,101,120,116,32,98,114,101,97,107,32,112,111,105,110,116,32,61,32,32,2,44,49,112,44,100,49,49,46,52,41,0,40,2,68,105,115,116,97,110,99,101,32,116,111,32,116,104,101,32,115,116,97,116,105,111,110,97,114,121,32,112,111,105,110,116,32,61,32,32,2,44,49,112,44,100,49,49,46,52,41,0,40,2,80,105,101,99,101,32,32,32,32,2,44,105,51,44,2,32,45,45,102,49,44,32,102,50,32,97,116,32,115,116,97,114,116,32,112,111,105,110,116,32,2,44,49,112,44,50,40,49,120,44,100,49,49,46,52,41,41,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,32,101,120,105,116,32,67,65,85,67,72,89,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,83,117,98,103,110,111,114,109,32,61,32,48,46,32,32,71,67,80,32,61,32,88,46,0,84,104,101,114,101,32,97,114,101,32,0,32,32,98,114,101,97,107,112,111,105,110,116,115,32,0,86,97,114,105,97,98,108,101,32,32,0,32,32,105,115,32,102,105,120,101,100,46,0,71,67,80,32,102,111,117,110,100,32,105,110,32,116,104,105,115,32,115,101,103,109,101,110,116,0,69,82,82,79,82,58,32,78,32,46,76,69,46,32,48,0,69,82,82,79,82,58,32,77,32,46,76,69,46,32,48,0,69,82,82,79,82,58,32,70,65,67,84,82,32,46,76,84,46,32,48,0,69,82,82,79,82,58,32,73,78,86,65,76,73,68,32,78,66,68,0,69,82,82,79,82,58,32,78,79,32,70,69,65,83,73,66,76,69,32,83,79,76,85,84,73,79,78,0,86,97,114,105,97,98,108,101,32,0,32,108,101,97,118,101,115,32,116,104,101,32,115,101,116,32,111,102,32,102,114,101,101,32,118,97,114,105,97,98,108,101,115,0,32,101,110,116,101,114,115,32,116,104,101,32,115,101,116,32,111,102,32,102,114,101,101,32,118,97,114,105,97,98,108,101,115,0,32,118,97,114,105,97,98,108,101,115,32,108,101,97,118,101,59,32,0,32,118,97,114,105,97,98,108,101,115,32,101,110,116,101,114,0,32,118,97,114,105,97,98,108,101,115,32,97,114,101,32,102,114,101,101,32,97,116,32,71,67,80,32,0,32,97,115,99,101,110,116,32,100,105,114,101,99,116,105,111,110,32,105,110,32,112,114,111,106,101,99,116,105,111,110,32,103,100,32,61,32,0,67,79,78,86,0,87,65,82,78,0,70,71,95,76,78,83,82,67,72,0,40,2,82,85,78,78,73,78,71,32,84,72,69,32,76,45,66,70,71,83,45,66,32,67,79,68,69,2,44,47,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,44,47,44,47,44,2,77,97,99,104,105,110,101,32,112,114,101,99,105,115,105,111,110,32,61,2,44,49,112,44,100,49,48,46,51,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,78,32,61,32,0,32,32,32,32,77,32,61,32,0,76,32,61,0,88,48,32,61,0,85,32,61,0,40,47,44,2,65,116,32,105,116,101,114,97,116,101,2,44,105,53,44,52,120,44,2,102,61,32,2,44,49,112,44,100,49,50,46,53,44,52,120,44,2,124,112,114,111,106,32,103,124,61,32,2,44,49,112,44,100,49,50,46,53,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,99,111,110,0,98,110,100,0,84,78,84,0,76,73,78,69,32,83,69,65,82,67,72,0,32,116,105,109,101,115,59,32,110,111,114,109,32,111,102,32,115,116,101,112,32,61,32,0,88,32,61,0,71,32,61,0,40,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,44,47,44,47,44,2,84,105,116,32,32,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,105,116,101,114,97,116,105,111,110,115,2,44,47,44,2,84,110,102,32,32,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,102,117,110,99,116,105,111,110,32,101,118,97,108,117,97,116,105,111,110,115,2,44,47,44,2,84,110,105,110,116,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,115,101,103,109,101,110,116,115,32,101,120,112,108,111,114,101,100,32,100,117,114,105,110,103,2,44,2,32,67,97,117,99,104,121,32,115,101,97,114,99,104,101,115,2,44,47,44,2,83,107,105,112,32,32,61,32,110,117,109,98,101,114,32,111,102,32,66,70,71,83,32,117,112,100,97,116,101,115,32,115,107,105,112,112,101,100,2,44,47,44,2,78,97,99,116,32,32,61,32,110,117,109,98,101,114,32,111,102,32,97,99,116,105,118,101,32,98,111,117,110,100,115,32,97,116,32,102,105,110,97,108,32,103,101,110,101,114,97,108,105,122,101,100,2,44,2,32,67,97,117,99,104,121,32,112,111,105,110,116,2,44,47,44,2,80,114,111,106,103,32,61,32,110,111,114,109,32,111,102,32,116,104,101,32,102,105,110,97,108,32,112,114,111,106,101,99,116,101,100,32,103,114,97,100,105,101,110,116,2,44,47,44,2,70,32,32,32,32,32,61,32,102,105,110,97,108,32,102,117,110,99,116,105,111,110,32,118,97,108,117,101,2,44,47,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,41,0,40,47,44,51,120,44,2,78,2,44,52,120,44,2,84,105,116,2,44,53,120,44,2,84,110,102,2,44,50,120,44,2,84,110,105,110,116,2,44,50,120,44,2,83,107,105,112,2,44,50,120,44,2,78,97,99,116,2,44,53,120,44,2,80,114,111,106,103,2,44,56,120,44,2,70,2,41,0,40,105,53,44,50,40,49,120,44,105,54,41,44,40,49,120,44,105,54,41,44,40,50,120,44,105,52,41,44,40,49,120,44,105,53,41,44,49,112,44,50,40,50,120,44,100,49,48,46,51,41,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,40,47,44,97,54,48,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,49,115,116,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,50,115,116,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,116,104,101,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,116,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,68,101,114,105,118,97,116,105,118,101,32,62,61,32,48,44,32,98,97,99,107,116,114,97,99,107,105,110,103,32,108,105,110,101,32,115,101,97,114,99,104,32,105,109,112,111,115,115,105,98,108,101,46,2,44,47,44,2,32,32,32,80,114,101,118,105,111,117,115,32,120,44,32,102,32,97,110,100,32,103,32,114,101,115,116,111,114,101,100,46,2,44,47,44,2,32,80,111,115,115,105,98,108,101,32,99,97,117,115,101,115,58,32,49,32,101,114,114,111,114,32,105,110,32,102,117,110,99,116,105,111,110,32,111,114,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,59,2,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,50,32,114,111,117,110,100,105,110,103,32,101,114,114,111,114,115,32,100,111,109,105,110,97,116,101,32,99,111,109,112,117,116,97,116,105,111,110,46,2,41,0,40,47,44,2,32,87,97,114,110,105,110,103,58,32,32,109,111,114,101,32,116,104,97,110,32,49,48,32,102,117,110,99,116,105,111,110,32,97,110,100,32,103,114,97,100,105,101,110,116,2,44,47,44,2,32,32,32,101,118,97,108,117,97,116,105,111,110,115,32,105,110,32,116,104,101,32,108,97,115,116,32,108,105,110,101,32,115,101,97,114,99,104,46,32,32,84,101,114,109,105,110,97,116,105,111,110,2,44,47,44,2,32,32,32,109,97,121,32,112,111,115,115,105,98,108,121,32,98,101,32,99,97,117,115,101,100,32,98,121,32,97,32,98,97,100,32,115,101,97,114,99,104,32,100,105,114,101,99,116,105,111,110,46,2,41,0,40,47,44,2,32,84,104,101,32,116,114,105,97,110,103,117,108,97,114,32,115,121,115,116,101,109,32,105,115,32,115,105,110,103,117,108,97,114,46,2,41,0,40,47,44,2,32,76,105,110,101,32,115,101,97,114,99,104,32,99,97,110,110,111,116,32,108,111,99,97,116,101,32,97,110,32,97,100,101,113,117,97,116,101,32,112,111,105,110,116,32,97,102,116,101,114,32,50,48,32,102,117,110,99,116,105,111,110,2,44,47,44,2,32,32,97,110,100,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,115,46,32,32,80,114,101,118,105,111,117,115,32,120,44,32,102,32,97,110,100,32,103,32,114,101,115,116,111,114,101,100,46,2,44,47,44,2,32,80,111,115,115,105,98,108,101,32,99,97,117,115,101,115,58,32,49,32,101,114,114,111,114,32,105,110,32,102,117,110,99,116,105,111,110,32,111,114,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,59,2,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,50,32,114,111,117,110,100,105,110,103,32,101,114,114,111,114,32,100,111,109,105,110,97,116,101,32,99,111,109,112,117,116,97,116,105,111,110,46,2,41,0,40,47,44,2,32,67,97,117,99,104,121,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,2,32,83,117,98,115,112,97,99,101,32,109,105,110,105,109,105,122,97,116,105,111,110,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,2,32,76,105,110,101,32,115,101,97,114,99,104,32,32,32,32,32,32,32,32,32,32,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,41,0,40,47,44,2,32,84,111,116,97,108,32,85,115,101,114,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,41,0,32,70,32,61,0,32,73,110,112,117,116,32,110,98,100,40,0,41,32,105,115,32,105,110,118,97,108,105,100,46,0,32,108,40,0,41,32,62,32,117,40,0,41,46,32,32,78,111,32,102,101,97,115,105,98,108,101,32,115,111,108,117,116,105,111,110,46,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,83,85,66,83,77,32,101,110,116,101,114,101,100,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,101,120,105,116,32,83,85,66,83,77,32,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,32,80,111,115,105,116,105,118,101,32,100,105,114,32,100,101,114,105,118,97,116,105,118,101,32,105,110,32,112,114,111,106,101,99,116,105,111,110,32,0,32,85,115,105,110,103,32,116,104,101,32,98,97,99,107,116,114,97,99,107,105,110,103,32,115,116,101,112,32,0,69,82,82,79,82,58,32,83,84,80,32,46,76,84,46,32,83,84,80,77,73,78,0,69,82,82,79,82,58,32,83,84,80,32,46,71,84,46,32,83,84,80,77,65,88,0,69,82,82,79,82,58,32,73,78,73,84,73,65,76,32,71,32,46,71,69,46,32,90,69,82,79,0,69,82,82,79,82,58,32,70,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,71,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,88,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,83,84,80,77,73,78,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,83,84,80,77,65,88,32,46,76,84,46,32,83,84,80,77,73,78,0,70,71,0,87,65,82,78,73,78,71,58,32,82,79,85,78,68,73,78,71,32,69,82,82,79,82,83,32,80,82,69,86,69,78,84,32,80,82,79,71,82,69,83,83,0,87,65,82,78,73,78,71,58,32,88,84,79,76,32,84,69,83,84,32,83,65,84,73,83,70,73,69,68,0,87,65,82,78,73,78,71,58,32,83,84,80,32,61,32,83,84,80,77,65,88,0,87,65,82,78,73,78,71,58,32,83,84,80,32,61,32,83,84,80,77,73,78,0,67,79,78,86,69,82,71,69,78,67,69,0,68,71,69,83,86,32,0,68,71,69,84,82,70,0,32,0,68,71,69,84,82,83,0,76,101,102,116,0,76,111,119,101,114,0,78,111,32,116,114,97,110,115,112,111,115,101,0,85,110,105,116,0,85,112,112,101,114,0,78,111,110,45,117,110,105,116,0,84,114,97,110,115,112,111,115,101,0,69,0,66,0,80,0,77,0,79,0,40,47,47,2,32,87,65,82,78,73,78,71,46,32,84,104,101,32,118,97,108,117,101,32,69,77,73,78,32,109,97,121,32,98,101,32,105,110,99,111,114,114,101,99,116,58,45,2,44,2,32,32,69,77,73,78,32,61,32,2,44,105,56,44,47,2,32,73,102,44,32,97,102,116,101,114,32,105,110,115,112,101,99,116,105,111,110,44,32,116,104,101,32,118,97,108,117,101,32,69,77,73,78,32,108,111,111,107,115,2,44,2,32,97,99,99,101,112,116,97,98,108,101,32,112,108,101,97,115,101,32,99,111,109,109,101,110,116,32,111,117,116,32,2,44,47,2,32,116,104,101,32,73,70,32,98,108,111,99,107,32,97,115,32,109,97,114,107,101,100,32,119,105,116,104,105,110,32,116,104,101,32,99,111,100,101,32,111,102,32,114,111,117,116,105,110,101,2,44,2,32,68,76,65,77,67,50,44,2,44,47,2,32,111,116,104,101,114,119,105,115,101,32,115,117,112,112,108,121,32,69,77,73,78,32,101,120,112,108,105,99,105,116,108,121,46,2,44,47,41,0,71,69,0,84,82,70,0,81,82,70,0,82,81,70,0,76,81,70,0,81,76,70,0,72,82,68,0,66,82,68,0,84,82,73,0,80,79,0,83,89,0,84,82,68,0,71,83,84,0,72,69,0,79,82,0,81,82,0,82,81,0,76,81,0,81,76,0,72,82,0,84,82,0,66,82,0,85,78,0,71,66,0,80,66,0,76,65,0,85,85,77,0,83,84,0,69,66,90,0,68,71,69,84,70,50,0,83,0,117,110,107,110,111,119,110,32,99,111,100,101,32,105,110,32,100,111,95,102,105,111,58,32,37,100,10,37,115,10,0,100,111,95,102,105,111,0,102,109,116,0,98,97,100,32,115,116,114,105,110,103,0,102,111,114,109,97,116,32,116,111,111,32,99,111,109,112,108,105,99,97,116,101,100,58,10,0,115,102,101,0,115,116,97,114,116,105,111,0,119,114,105,116,101,32,115,116,97,114,116,0,108,105,115,116,32,111,117,116,112,117,116,32,115,116,97,114,116,0,37,115,10,0,101,110,100,102,105,108,101,0,37,115,58,32,105,108,108,101,103,97,108,32,101,114,114,111,114,32,110,117,109,98,101,114,32,37,100,10,0,37,115,58,32,101,110,100,32,111,102,32,102,105,108,101,10,0,37,115,58,32,37,115,10,0,97,112,112,97,114,101,110,116,32,115,116,97,116,101,58,32,117,110,105,116,32,37,100,32,0,110,97,109,101,100,32,37,115,10,0,40,117,110,110,97,109,101,100,41,10,0,97,112,112,97,114,101,110,116,32,115,116,97,116,101,58,32,105,110,116,101,114,110,97,108,32,73,47,79,10,0,108,97,115,116,32,102,111,114,109,97,116,58,32,37,115,10,0,114,101,97,100,105,110,103,0,119,114,105,116,105,110,103,0,115,101,113,117,101,110,116,105,97,108,0,100,105,114,101,99,116,0,102,111,114,109,97,116,116,101,100,0,117,110,102,111,114,109,97,116,116,101,100,0,101,120,116,101,114,110,97,108,0,105,110,116,101,114,110,97,108,0,108,97,116,101,108,121,32,37,115,32,37,115,32,37,115,32,37,115,0,32,73,79,0,101,114,114,111,114,32,105,110,32,102,111,114,109,97,116,0,105,108,108,101,103,97,108,32,117,110,105,116,32,110,117,109,98,101,114,0,102,111,114,109,97,116,116,101,100,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,117,110,102,111,114,109,97,116,116,101,100,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,100,105,114,101,99,116,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,115,101,113,117,101,110,116,105,97,108,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,99,97,110,39,116,32,98,97,99,107,115,112,97,99,101,32,102,105,108,101,0,110,117,108,108,32,102,105,108,101,32,110,97,109,101,0,99,97,110,39,116,32,115,116,97,116,32,102,105,108,101,0,117,110,105,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,111,102,102,32,101,110,100,32,111,102,32,114,101,99,111,114,100,0,116,114,117,110,99,97,116,105,111,110,32,102,97,105,108,101,100,32,105,110,32,101,110,100,102,105,108,101,0,105,110,99,111,109,112,114,101,104,101,110,115,105,98,108,101,32,108,105,115,116,32,105,110,112,117,116,0,111,117,116,32,111,102,32,102,114,101,101,32,115,112,97,99,101,0,114,101,97,100,32,117,110,101,120,112,101,99,116,101,100,32,99,104,97,114,97,99,116,101,114,0,98,97,100,32,108,111,103,105,99,97,108,32,105,110,112,117,116,32,102,105,101,108,100,0,98,97,100,32,118,97,114,105,97,98,108,101,32,116,121,112,101,0,98,97,100,32,110,97,109,101,108,105,115,116,32,110,97,109,101,0,118,97,114,105,97,98,108,101,32,110,111,116,32,105,110,32,110,97,109,101,108,105,115,116,0,110,111,32,101,110,100,32,114,101,99,111,114,100,0,118,97,114,105,97,98,108,101,32,99,111,117,110,116,32,105,110,99,111,114,114,101,99,116,0,115,117,98,115,99,114,105,112,116,32,102,111,114,32,115,99,97,108,97,114,32,118,97,114,105,97,98,108,101,0,105,110,118,97,108,105,100,32,97,114,114,97,121,32,115,101,99,116,105,111,110,0,115,117,98,115,116,114,105,110,103,32,111,117,116,32,111,102,32,98,111,117,110,100,115,0,115,117,98,115,99,114,105,112,116,32,111,117,116,32,111,102,32,98,111,117,110,100,115,0,99,97,110,39,116,32,114,101,97,100,32,102,105,108,101,0,99,97,110,39,116,32,119,114,105,116,101,32,102,105,108,101,0,39,110,101,119,39,32,102,105,108,101,32,101,120,105,115,116,115,0,99,97,110,39,116,32,97,112,112,101,110,100,32,116,111,32,102,105,108,101,0,110,111,110,45,112,111,115,105,116,105,118,101,32,114,101,99,111,114,100,32,110,117,109,98,101,114,0,110,109,76,98,117,102,32,111,118,101,114,102,108,111,119,0,108,105,115,116,32,105,111,0,115,116,108,101,114,0,108,105,111,0,117,110,107,110,111,119,110,32,116,121,112,101,32,105,110,32,108,105,111,0,37,46,57,71,0,111,112,101,110,0,102,111,114,116,46,37,108,100,0,110,111,32,115,112,97,99,101,0,115,0,100,0,102,0,117,0,114,98,0,114,0,119,98,0,119,0,114,43,98,0,114,43,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,97,108,108,111,99,32,102,97,105,108,117,114,101,0,119,95,101,100,44,32,117,110,101,120,112,101,99,116,101,100,32,99,111,100,101,58,32,37,100,10,0,119,95,110,101,100,44,32,117,110,101,120,112,101,99,116,101,100,32,99,111,100,101,58,32,37,100,10,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,0,108,101,102,116,32,111,102,102,0,114,101,99,101,110,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,35,46,42,69,0,37,43,46,50,100,0,43,48,48,0,37,35,46,42,102,0,68,71,69,82,32,32,0,68,71,69,77,77,32,0,76,0,78,0,85,0,82,0,84,0,67,0,68,84,82,83,77,32,0,42,42,32,79,110,32,101,110,116,114,121,32,116,111,32,37,54,115,44,32,112,97,114,97,109,101,116,101,114,32,110,117,109,98,101,114,32,37,50,105,32,104,97,100,32,97,110,32,105,108,108,101,103,97,108,32,118,97,108,117,101,10,0,119,43,0,114,119,97,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,47,116,109,112,0,47,116,109,112,47,116,37,120,45,37,120,0,84,33,34,25,13,1,2,3,17,75,28,12,16,4,11,29,18,30,39,104,110,111,112,113,98,32,5,6,15,19,20,21,26,8,22,7,40,36,23,24,9,10,14,27,31,37,35,131,130,125,38,42,43,60,61,62,63,67,71,74,77,88,89,90,91,92,93,94,95,96,97,99,100,101,102,103,105,106,107,108,114,115,116,121,122,123,124,0,73,108,108,101,103,97,108,32,98,121,116,101,32,115,101,113,117,101,110,99,101,0,68,111,109,97,105,110,32,101,114,114,111,114,0,82,101,115,117,108,116,32,110,111,116,32,114,101,112,114,101,115,101,110,116,97,98,108,101,0,78,111,116,32,97,32,116,116,121,0,80,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,79,112,101,114,97,116,105,111,110,32,110,111,116,32,112,101,114,109,105,116,116,101,100,0,78,111,32,115,117,99,104,32,102,105,108,101,32,111,114,32,100,105,114,101,99,116,111,114,121,0,78,111,32,115,117,99,104,32,112,114,111,99,101,115,115,0,70,105,108,101,32,101,120,105,115,116,115,0,86,97,108,117,101,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,100,97,116,97,32,116,121,112,101,0,78,111,32,115,112,97,99,101,32,108,101,102,116,32,111,110,32,100,101,118,105,99,101,0,79,117,116,32,111,102,32,109,101,109,111,114,121,0,82,101,115,111,117,114,99,101,32,98,117,115,121,0,73,110,116,101,114,114,117,112,116,101,100,32,115,121,115,116,101,109,32,99,97,108,108,0,82,101,115,111,117,114,99,101,32,116,101,109,112,111,114,97,114,105,108,121,32,117,110,97,118,97,105,108,97,98,108,101,0,73,110,118,97,108,105,100,32,115,101,101,107,0,67,114,111,115,115,45,100,101,118,105,99,101,32,108,105,110,107,0,82,101,97,100,45,111,110,108,121,32,102,105,108,101,32,115,121,115,116,101,109,0,68,105,114,101,99,116,111,114,121,32,110,111,116,32,101,109,112,116,121,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,112,101,101,114,0,79,112,101,114,97,116,105,111,110,32,116,105,109,101,100,32,111,117,116,0,67,111,110,110,101,99,116,105,111,110,32,114,101,102,117,115,101,100,0,72,111,115,116,32,105,115,32,100,111,119,110,0,72,111,115,116,32,105,115,32,117,110,114,101,97,99,104,97,98,108,101,0,65,100,100,114,101,115,115,32,105,110,32,117,115,101,0,66,114,111,107,101,110,32,112,105,112,101,0,73,47,79,32,101,114,114,111,114,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,32,111,114,32,97,100,100,114,101,115,115,0,66,108,111,99,107,32,100,101,118,105,99,101,32,114,101,113,117,105,114,101,100,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,0,78,111,116,32,97,32,100,105,114,101,99,116,111,114,121,0,73,115,32,97,32,100,105,114,101,99,116,111,114,121,0,84,101,120,116,32,102,105,108,101,32,98,117,115,121,0,69,120,101,99,32,102,111,114,109,97,116,32,101,114,114,111,114,0,73,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,0,65,114,103,117,109,101,110,116,32,108,105,115,116,32,116,111,111,32,108,111,110,103,0,83,121,109,98,111,108,105,99,32,108,105,110,107,32,108,111,111,112,0,70,105,108,101,110,97,109,101,32,116,111,111,32,108,111,110,103,0,84,111,111,32,109,97,110,121,32,111,112,101,110,32,102,105,108,101,115,32,105,110,32,115,121,115,116,101,109,0,78,111,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,115,32,97,118,97,105,108,97,98,108,101,0,66,97,100,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,0,78,111,32,99,104,105,108,100,32,112,114,111,99,101,115,115,0,66,97,100,32,97,100,100,114,101,115,115,0,70,105,108,101,32,116,111,111,32,108,97,114,103,101,0,84,111,111,32,109,97,110,121,32,108,105,110,107,115,0,78,111,32,108,111,99,107,115,32,97,118,97,105,108,97,98,108,101,0,82,101,115,111,117,114,99,101,32,100,101,97,100,108,111,99,107,32,119,111,117,108,100,32,111,99,99,117,114,0,83,116,97,116,101,32,110,111,116,32,114,101,99,111,118,101,114,97,98,108,101,0,80,114,101,118,105,111,117,115,32,111,119,110,101,114,32,100,105,101,100,0,79,112,101,114,97,116,105,111,110,32,99,97,110,99,101,108,101,100,0,70,117,110,99,116,105,111,110,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,0,78,111,32,109,101,115,115,97,103,101,32,111,102,32,100,101,115,105,114,101,100,32,116,121,112,101,0,73,100,101,110,116,105,102,105,101,114,32,114,101,109,111,118,101,100,0,68,101,118,105,99,101,32,110,111,116,32,97,32,115,116,114,101,97,109,0,78,111,32,100,97,116,97,32,97,118,97,105,108,97,98,108,101,0,68,101,118,105,99,101,32,116,105,109,101,111,117,116,0,79,117,116,32,111,102,32,115,116,114,101,97,109,115,32,114,101,115,111,117,114,99,101,115,0,76,105,110,107,32,104,97,115,32,98,101,101,110,32,115,101,118,101,114,101,100,0,80,114,111,116,111,99,111,108,32,101,114,114,111,114,0,66,97,100,32,109,101,115,115,97,103,101,0,70,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,105,110,32,98,97,100,32,115,116,97,116,101,0,78,111,116,32,97,32,115,111,99,107,101,116,0,68,101,115,116,105,110,97,116,105,111,110,32,97,100,100,114,101,115,115,32,114,101,113,117,105,114,101,100,0,77,101,115,115,97,103,101,32,116,111,111,32,108,97,114,103,101,0,80,114,111,116,111,99,111,108,32,119,114,111,110,103,32,116,121,112,101,32,102,111,114,32,115,111,99,107,101,116,0,80,114,111,116,111,99,111,108,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,80,114,111,116,111,99,111,108,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,83,111,99,107,101,116,32,116,121,112,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,78,111,116,32,115,117,112,112,111,114,116,101,100,0,80,114,111,116,111,99,111,108,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,65,100,100,114,101,115,115,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,112,114,111,116,111,99,111,108,0,65,100,100,114,101,115,115,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,78,101,116,119,111,114,107,32,105,115,32,100,111,119,110,0,78,101,116,119,111,114,107,32,117,110,114,101,97,99,104,97,98,108,101,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,110,101,116,119,111,114,107,0,67,111,110,110,101,99,116,105,111,110,32,97,98,111,114,116,101,100,0,78,111,32,98,117,102,102,101,114,32,115,112,97,99,101,32,97,118,97,105,108,97,98,108,101,0,83,111,99,107,101,116,32,105,115,32,99,111,110,110,101,99,116,101,100,0,83,111,99,107,101,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,67,97,110,110,111,116,32,115,101,110,100,32,97,102,116,101,114,32,115,111,99,107,101,116,32,115,104,117,116,100,111,119,110,0,79,112,101,114,97,116,105,111,110,32,97,108,114,101,97,100,121,32,105,110,32,112,114,111,103,114,101,115,115,0,79,112,101,114,97,116,105,111,110,32,105,110,32,112,114,111,103,114,101,115,115,0,83,116,97,108,101,32,102,105,108,101,32,104,97,110,100,108,101,0,82,101,109,111,116,101,32,73,47,79,32,101,114,114,111,114,0,81,117,111,116,97,32,101,120,99,101,101,100,101,100,0,78,111,32,109,101,100,105,117,109,32,102,111,117,110,100,0,87,114,111,110,103,32,109,101,100,105,117,109,32,116,121,112,101,0,78,111,32,101,114,114,111,114,32,105,110,102,111,114,109,97,116,105,111,110,0,0,47,112,114,111,99,47,115,101,108,102,47,102,100,47], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+12656);
/* memory initializer */ allocate([17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,45,43,32,32,32,48,88,48,120,0,40,110,117,108,108,41,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,105,110,102,0,73,78,70,0,110,97,110,0,78,65,78,0,46,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+24555);





/* no memory initializer */
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

HEAP8[tempDoublePtr] = HEAP8[ptr];

HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

HEAP8[tempDoublePtr] = HEAP8[ptr];

HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}

// {{PRE_LIBRARY}}



Module["_i64Subtract"] = _i64Subtract;


function ___setErrNo(value) {
if (Module['___errno_location']) HEAP32[((Module['___errno_location']())>>2)]=value;
else Module.printErr('failed to set errno from JS');
return value;
}

var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};function _sysconf(name) {
// long sysconf(int name);
// http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
switch(name) {
case 30: return PAGE_SIZE;
case 85: return totalMemory / PAGE_SIZE;
case 132:
case 133:
case 12:
case 137:
case 138:
case 15:
case 235:
case 16:
case 17:
case 18:
case 19:
case 20:
case 149:
case 13:
case 10:
case 236:
case 153:
case 9:
case 21:
case 22:
case 159:
case 154:
case 14:
case 77:
case 78:
case 139:
case 80:
case 81:
case 82:
case 68:
case 67:
case 164:
case 11:
case 29:
case 47:
case 48:
case 95:
case 52:
case 51:
case 46:
return 200809;
case 79:
return 0;
case 27:
case 246:
case 127:
case 128:
case 23:
case 24:
case 160:
case 161:
case 181:
case 182:
case 242:
case 183:
case 184:
case 243:
case 244:
case 245:
case 165:
case 178:
case 179:
case 49:
case 50:
case 168:
case 169:
case 175:
case 170:
case 171:
case 172:
case 97:
case 76:
case 32:
case 173:
case 35:
return -1;
case 176:
case 177:
case 7:
case 155:
case 8:
case 157:
case 125:
case 126:
case 92:
case 93:
case 129:
case 130:
case 131:
case 94:
case 91:
return 1;
case 74:
case 60:
case 69:
case 70:
case 4:
return 1024;
case 31:
case 42:
case 72:
return 32;
case 87:
case 26:
case 33:
return 2147483647;
case 34:
case 1:
return 47839;
case 38:
case 36:
return 99;
case 43:
case 37:
return 2048;
case 0: return 2097152;
case 3: return 65536;
case 28: return 32768;
case 44: return 32767;
case 75: return 16384;
case 39: return 1000;
case 89: return 700;
case 71: return 256;
case 40: return 255;
case 2: return 100;
case 180: return 64;
case 25: return 20;
case 5: return 16;
case 6: return 6;
case 73: return 4;
case 84: {
if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
return 1;
}
}
___setErrNo(ERRNO_CODES.EINVAL);
return -1;
}




var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};

var PATH={splitPath:function (filename) {
var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
return splitPathRe.exec(filename).slice(1);
},normalizeArray:function (parts, allowAboveRoot) {
// if the path tries to go above the root, `up` ends up > 0
var up = 0;
for (var i = parts.length - 1; i >= 0; i--) {
var last = parts[i];
if (last === '.') {
parts.splice(i, 1);
} else if (last === '..') {
parts.splice(i, 1);
up++;
} else if (up) {
parts.splice(i, 1);
up--;
}
}
// if the path is allowed to go above the root, restore leading ..s
if (allowAboveRoot) {
for (; up--; up) {
parts.unshift('..');
}
}
return parts;
},normalize:function (path) {
var isAbsolute = path.charAt(0) === '/',
trailingSlash = path.substr(-1) === '/';
// Normalize the path
path = PATH.normalizeArray(path.split('/').filter(function(p) {
return !!p;
}), !isAbsolute).join('/');
if (!path && !isAbsolute) {
path = '.';
}
if (path && trailingSlash) {
path += '/';
}
return (isAbsolute ? '/' : '') + path;
},dirname:function (path) {
var result = PATH.splitPath(path),
root = result[0],
dir = result[1];
if (!root && !dir) {
// No dirname whatsoever
return '.';
}
if (dir) {
// It has a dirname, strip trailing slash
dir = dir.substr(0, dir.length - 1);
}
return root + dir;
},basename:function (path) {
// EMSCRIPTEN return '/'' for '/', not an empty string
if (path === '/') return '/';
var lastSlash = path.lastIndexOf('/');
if (lastSlash === -1) return path;
return path.substr(lastSlash+1);
},extname:function (path) {
return PATH.splitPath(path)[3];
},join:function () {
var paths = Array.prototype.slice.call(arguments, 0);
return PATH.normalize(paths.join('/'));
},join2:function (l, r) {
return PATH.normalize(l + '/' + r);
},resolve:function () {
var resolvedPath = '',
resolvedAbsolute = false;
for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
var path = (i >= 0) ? arguments[i] : FS.cwd();
// Skip empty and invalid entries
if (typeof path !== 'string') {
throw new TypeError('Arguments to path.resolve must be strings');
} else if (!path) {
return ''; // an invalid portion invalidates the whole thing
}
resolvedPath = path + '/' + resolvedPath;
resolvedAbsolute = path.charAt(0) === '/';
}
// At this point the path should be resolved to a full absolute path, but
// handle relative paths to be safe (might happen when process.cwd() fails)
resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
return !!p;
}), !resolvedAbsolute).join('/');
return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
},relative:function (from, to) {
from = PATH.resolve(from).substr(1);
to = PATH.resolve(to).substr(1);
function trim(arr) {
var start = 0;
for (; start < arr.length; start++) {
if (arr[start] !== '') break;
}
var end = arr.length - 1;
for (; end >= 0; end--) {
if (arr[end] !== '') break;
}
if (start > end) return [];
return arr.slice(start, end - start + 1);
}
var fromParts = trim(from.split('/'));
var toParts = trim(to.split('/'));
var length = Math.min(fromParts.length, toParts.length);
var samePartsLength = length;
for (var i = 0; i < length; i++) {
if (fromParts[i] !== toParts[i]) {
samePartsLength = i;
break;
}
}
var outputParts = [];
for (var i = samePartsLength; i < fromParts.length; i++) {
outputParts.push('..');
}
outputParts = outputParts.concat(toParts.slice(samePartsLength));
return outputParts.join('/');
}};

var TTY={ttys:[],init:function () {
// https://github.com/kripken/emscripten/pull/1555
// if (ENVIRONMENT_IS_NODE) {
//   // currently, FS.init does not distinguish if process.stdin is a file or TTY
//   // device, it always assumes it's a TTY device. because of this, we're forcing
//   // process.stdin to UTF8 encoding to at least make stdin reading compatible
//   // with text files until FS.init can be refactored.
//   process['stdin']['setEncoding']('utf8');
// }
},shutdown:function () {
// https://github.com/kripken/emscripten/pull/1555
// if (ENVIRONMENT_IS_NODE) {
//   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
//   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
//   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
//   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
//   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
//   process['stdin']['pause']();
// }
},register:function (dev, ops) {
TTY.ttys[dev] = { input: [], output: [], ops: ops };
FS.registerDevice(dev, TTY.stream_ops);
},stream_ops:{open:function (stream) {
var tty = TTY.ttys[stream.node.rdev];
if (!tty) {
throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
}
stream.tty = tty;
stream.seekable = false;
},close:function (stream) {
// flush any pending line data
stream.tty.ops.flush(stream.tty);
},flush:function (stream) {
stream.tty.ops.flush(stream.tty);
},read:function (stream, buffer, offset, length, pos /* ignored */) {
if (!stream.tty || !stream.tty.ops.get_char) {
throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
}
var bytesRead = 0;
for (var i = 0; i < length; i++) {
var result;
try {
result = stream.tty.ops.get_char(stream.tty);
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
if (result === undefined && bytesRead === 0) {
throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
}
if (result === null || result === undefined) break;
bytesRead++;
buffer[offset+i] = result;
}
if (bytesRead) {
stream.node.timestamp = Date.now();
}
return bytesRead;
},write:function (stream, buffer, offset, length, pos) {
if (!stream.tty || !stream.tty.ops.put_char) {
throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
}
for (var i = 0; i < length; i++) {
try {
stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
}
if (length) {
stream.node.timestamp = Date.now();
}
return i;
}},default_tty_ops:{get_char:function (tty) {
if (!tty.input.length) {
var result = null;
if (ENVIRONMENT_IS_NODE) {
// we will read data by chunks of BUFSIZE
var BUFSIZE = 256;
var buf = new Buffer(BUFSIZE);
var bytesRead = 0;

var fd = process.stdin.fd;
// Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
var usingDevice = false;
try {
fd = fs.openSync('/dev/stdin', 'r');
usingDevice = true;
} catch (e) {}

bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);

if (usingDevice) { fs.closeSync(fd); }
if (bytesRead > 0) {
result = buf.slice(0, bytesRead).toString('utf-8');
} else {
result = null;
}

} else if (typeof window != 'undefined' &&
typeof window.prompt == 'function') {
// Browser.
result = window.prompt('Input: ');  // returns null on cancel
if (result !== null) {
result += '\n';
}
} else if (typeof readline == 'function') {
// Command line.
result = readline();
if (result !== null) {
result += '\n';
}
}
if (!result) {
return null;
}
tty.input = intArrayFromString(result, true);
}
return tty.input.shift();
},put_char:function (tty, val) {
if (val === null || val === 10) {
Module['print'](UTF8ArrayToString(tty.output, 0));
tty.output = [];
} else {
if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
}
},flush:function (tty) {
if (tty.output && tty.output.length > 0) {
Module['print'](UTF8ArrayToString(tty.output, 0));
tty.output = [];
}
}},default_tty1_ops:{put_char:function (tty, val) {
if (val === null || val === 10) {
Module['printErr'](UTF8ArrayToString(tty.output, 0));
tty.output = [];
} else {
if (val != 0) tty.output.push(val);
}
},flush:function (tty) {
if (tty.output && tty.output.length > 0) {
Module['printErr'](UTF8ArrayToString(tty.output, 0));
tty.output = [];
}
}}};

var MEMFS={ops_table:null,mount:function (mount) {
return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
},createNode:function (parent, name, mode, dev) {
if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
// no supported
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
if (!MEMFS.ops_table) {
MEMFS.ops_table = {
dir: {
node: {
getattr: MEMFS.node_ops.getattr,
setattr: MEMFS.node_ops.setattr,
lookup: MEMFS.node_ops.lookup,
mknod: MEMFS.node_ops.mknod,
rename: MEMFS.node_ops.rename,
unlink: MEMFS.node_ops.unlink,
rmdir: MEMFS.node_ops.rmdir,
readdir: MEMFS.node_ops.readdir,
symlink: MEMFS.node_ops.symlink
},
stream: {
llseek: MEMFS.stream_ops.llseek
}
},
file: {
node: {
getattr: MEMFS.node_ops.getattr,
setattr: MEMFS.node_ops.setattr
},
stream: {
llseek: MEMFS.stream_ops.llseek,
read: MEMFS.stream_ops.read,
write: MEMFS.stream_ops.write,
allocate: MEMFS.stream_ops.allocate,
mmap: MEMFS.stream_ops.mmap,
msync: MEMFS.stream_ops.msync
}
},
link: {
node: {
getattr: MEMFS.node_ops.getattr,
setattr: MEMFS.node_ops.setattr,
readlink: MEMFS.node_ops.readlink
},
stream: {}
},
chrdev: {
node: {
getattr: MEMFS.node_ops.getattr,
setattr: MEMFS.node_ops.setattr
},
stream: FS.chrdev_stream_ops
}
};
}
var node = FS.createNode(parent, name, mode, dev);
if (FS.isDir(node.mode)) {
node.node_ops = MEMFS.ops_table.dir.node;
node.stream_ops = MEMFS.ops_table.dir.stream;
node.contents = {};
} else if (FS.isFile(node.mode)) {
node.node_ops = MEMFS.ops_table.file.node;
node.stream_ops = MEMFS.ops_table.file.stream;
node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
// When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
// for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
// penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
node.contents = null; 
} else if (FS.isLink(node.mode)) {
node.node_ops = MEMFS.ops_table.link.node;
node.stream_ops = MEMFS.ops_table.link.stream;
} else if (FS.isChrdev(node.mode)) {
node.node_ops = MEMFS.ops_table.chrdev.node;
node.stream_ops = MEMFS.ops_table.chrdev.stream;
}
node.timestamp = Date.now();
// add the new node to the parent
if (parent) {
parent.contents[name] = node;
}
return node;
},getFileDataAsRegularArray:function (node) {
if (node.contents && node.contents.subarray) {
var arr = [];
for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
return arr; // Returns a copy of the original data.
}
return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
},getFileDataAsTypedArray:function (node) {
if (!node.contents) return new Uint8Array;
if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
return new Uint8Array(node.contents);
},expandFileStorage:function (node, newCapacity) {
// If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
// instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
// increase the size.
if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
node.contents = MEMFS.getFileDataAsRegularArray(node);
node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
}

if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
// Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
// For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
// avoid overshooting the allocation cap by a very large margin.
var CAPACITY_DOUBLING_MAX = 1024 * 1024;
newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
var oldContents = node.contents;
node.contents = new Uint8Array(newCapacity); // Allocate new storage.
if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
return;
}
// Not using a typed array to back the file storage. Use a standard JS array instead.
if (!node.contents && newCapacity > 0) node.contents = [];
while (node.contents.length < newCapacity) node.contents.push(0);
},resizeFileStorage:function (node, newSize) {
if (node.usedBytes == newSize) return;
if (newSize == 0) {
node.contents = null; // Fully decommit when requesting a resize to zero.
node.usedBytes = 0;
return;
}
if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
var oldContents = node.contents;
node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
if (oldContents) {
node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
}
node.usedBytes = newSize;
return;
}
// Backing with a JS array.
if (!node.contents) node.contents = [];
if (node.contents.length > newSize) node.contents.length = newSize;
else while (node.contents.length < newSize) node.contents.push(0);
node.usedBytes = newSize;
},node_ops:{getattr:function (node) {
var attr = {};
// device numbers reuse inode numbers.
attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
attr.ino = node.id;
attr.mode = node.mode;
attr.nlink = 1;
attr.uid = 0;
attr.gid = 0;
attr.rdev = node.rdev;
if (FS.isDir(node.mode)) {
attr.size = 4096;
} else if (FS.isFile(node.mode)) {
attr.size = node.usedBytes;
} else if (FS.isLink(node.mode)) {
attr.size = node.link.length;
} else {
attr.size = 0;
}
attr.atime = new Date(node.timestamp);
attr.mtime = new Date(node.timestamp);
attr.ctime = new Date(node.timestamp);
// NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
//       but this is not required by the standard.
attr.blksize = 4096;
attr.blocks = Math.ceil(attr.size / attr.blksize);
return attr;
},setattr:function (node, attr) {
if (attr.mode !== undefined) {
node.mode = attr.mode;
}
if (attr.timestamp !== undefined) {
node.timestamp = attr.timestamp;
}
if (attr.size !== undefined) {
MEMFS.resizeFileStorage(node, attr.size);
}
},lookup:function (parent, name) {
throw FS.genericErrors[ERRNO_CODES.ENOENT];
},mknod:function (parent, name, mode, dev) {
return MEMFS.createNode(parent, name, mode, dev);
},rename:function (old_node, new_dir, new_name) {
// if we're overwriting a directory at new_name, make sure it's empty.
if (FS.isDir(old_node.mode)) {
var new_node;
try {
new_node = FS.lookupNode(new_dir, new_name);
} catch (e) {
}
if (new_node) {
for (var i in new_node.contents) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
}
}
}
// do the internal rewiring
delete old_node.parent.contents[old_node.name];
old_node.name = new_name;
new_dir.contents[new_name] = old_node;
old_node.parent = new_dir;
},unlink:function (parent, name) {
delete parent.contents[name];
},rmdir:function (parent, name) {
var node = FS.lookupNode(parent, name);
for (var i in node.contents) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
}
delete parent.contents[name];
},readdir:function (node) {
var entries = ['.', '..']
for (var key in node.contents) {
if (!node.contents.hasOwnProperty(key)) {
continue;
}
entries.push(key);
}
return entries;
},symlink:function (parent, newname, oldpath) {
var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
node.link = oldpath;
return node;
},readlink:function (node) {
if (!FS.isLink(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
return node.link;
}},stream_ops:{read:function (stream, buffer, offset, length, position) {
var contents = stream.node.contents;
if (position >= stream.node.usedBytes) return 0;
var size = Math.min(stream.node.usedBytes - position, length);
assert(size >= 0);
if (size > 8 && contents.subarray) { // non-trivial, and typed array
buffer.set(contents.subarray(position, position + size), offset);
} else {
for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
}
return size;
},write:function (stream, buffer, offset, length, position, canOwn) {
if (!length) return 0;
var node = stream.node;
node.timestamp = Date.now();

if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
if (canOwn) { // Can we just reuse the buffer we are given?
assert(position === 0, 'canOwn must imply no weird position inside the file');
node.contents = buffer.subarray(offset, offset + length);
node.usedBytes = length;
return length;
} else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
node.usedBytes = length;
return length;
} else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
node.contents.set(buffer.subarray(offset, offset + length), position);
return length;
}
}

// Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
MEMFS.expandFileStorage(node, position+length);
if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
else {
for (var i = 0; i < length; i++) {
node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
}
}
node.usedBytes = Math.max(node.usedBytes, position+length);
return length;
},llseek:function (stream, offset, whence) {
var position = offset;
if (whence === 1) {  // SEEK_CUR.
position += stream.position;
} else if (whence === 2) {  // SEEK_END.
if (FS.isFile(stream.node.mode)) {
position += stream.node.usedBytes;
}
}
if (position < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
return position;
},allocate:function (stream, offset, length) {
MEMFS.expandFileStorage(stream.node, offset + length);
stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
},mmap:function (stream, buffer, offset, length, position, prot, flags) {
if (!FS.isFile(stream.node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
}
var ptr;
var allocated;
var contents = stream.node.contents;
// Only make a new copy when MAP_PRIVATE is specified.
if ( !(flags & 2) &&
(contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
// We can't emulate MAP_SHARED when the file is not backed by the buffer
// we're mapping to (e.g. the HEAP buffer).
allocated = false;
ptr = contents.byteOffset;
} else {
// Try to avoid unnecessary slices.
if (position > 0 || position + length < stream.node.usedBytes) {
if (contents.subarray) {
contents = contents.subarray(position, position + length);
} else {
contents = Array.prototype.slice.call(contents, position, position + length);
}
}
allocated = true;
ptr = _malloc(length);
if (!ptr) {
throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
}
buffer.set(contents, ptr);
}
return { ptr: ptr, allocated: allocated };
},msync:function (stream, buffer, offset, length, mmapFlags) {
if (!FS.isFile(stream.node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
}
if (mmapFlags & 2) {
// MAP_PRIVATE calls need not to be synced back to underlying fs
return 0;
}

var bytesWritten = MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
// should we check if bytesWritten and length are the same?
return 0;
}}};

var IDBFS={dbs:{},indexedDB:function () {
if (typeof indexedDB !== 'undefined') return indexedDB;
var ret = null;
if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
assert(ret, 'IDBFS used, but indexedDB not supported');
return ret;
},DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
// reuse all of the core MEMFS functionality
return MEMFS.mount.apply(null, arguments);
},syncfs:function (mount, populate, callback) {
IDBFS.getLocalSet(mount, function(err, local) {
if (err) return callback(err);

IDBFS.getRemoteSet(mount, function(err, remote) {
if (err) return callback(err);

var src = populate ? remote : local;
var dst = populate ? local : remote;

IDBFS.reconcile(src, dst, callback);
});
});
},getDB:function (name, callback) {
// check the cache first
var db = IDBFS.dbs[name];
if (db) {
return callback(null, db);
}

var req;
try {
req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
} catch (e) {
return callback(e);
}
req.onupgradeneeded = function(e) {
var db = e.target.result;
var transaction = e.target.transaction;

var fileStore;

if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
} else {
fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
}

if (!fileStore.indexNames.contains('timestamp')) {
fileStore.createIndex('timestamp', 'timestamp', { unique: false });
}
};
req.onsuccess = function() {
db = req.result;

// add to the cache
IDBFS.dbs[name] = db;
callback(null, db);
};
req.onerror = function(e) {
callback(this.error);
e.preventDefault();
};
},getLocalSet:function (mount, callback) {
var entries = {};

function isRealDir(p) {
return p !== '.' && p !== '..';
};
function toAbsolute(root) {
return function(p) {
return PATH.join2(root, p);
}
};

var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));

while (check.length) {
var path = check.pop();
var stat;

try {
stat = FS.stat(path);
} catch (e) {
return callback(e);
}

if (FS.isDir(stat.mode)) {
check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
}

entries[path] = { timestamp: stat.mtime };
}

return callback(null, { type: 'local', entries: entries });
},getRemoteSet:function (mount, callback) {
var entries = {};

IDBFS.getDB(mount.mountpoint, function(err, db) {
if (err) return callback(err);

var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
transaction.onerror = function(e) {
callback(this.error);
e.preventDefault();
};

var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
var index = store.index('timestamp');

index.openKeyCursor().onsuccess = function(event) {
var cursor = event.target.result;

if (!cursor) {
return callback(null, { type: 'remote', db: db, entries: entries });
}

entries[cursor.primaryKey] = { timestamp: cursor.key };

cursor.continue();
};
});
},loadLocalEntry:function (path, callback) {
var stat, node;

try {
var lookup = FS.lookupPath(path);
node = lookup.node;
stat = FS.stat(path);
} catch (e) {
return callback(e);
}

if (FS.isDir(stat.mode)) {
return callback(null, { timestamp: stat.mtime, mode: stat.mode });
} else if (FS.isFile(stat.mode)) {
// Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
// Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
node.contents = MEMFS.getFileDataAsTypedArray(node);
return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
} else {
return callback(new Error('node type not supported'));
}
},storeLocalEntry:function (path, entry, callback) {
try {
if (FS.isDir(entry.mode)) {
FS.mkdir(path, entry.mode);
} else if (FS.isFile(entry.mode)) {
FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
} else {
return callback(new Error('node type not supported'));
}

FS.chmod(path, entry.mode);
FS.utime(path, entry.timestamp, entry.timestamp);
} catch (e) {
return callback(e);
}

callback(null);
},removeLocalEntry:function (path, callback) {
try {
var lookup = FS.lookupPath(path);
var stat = FS.stat(path);

if (FS.isDir(stat.mode)) {
FS.rmdir(path);
} else if (FS.isFile(stat.mode)) {
FS.unlink(path);
}
} catch (e) {
return callback(e);
}

callback(null);
},loadRemoteEntry:function (store, path, callback) {
var req = store.get(path);
req.onsuccess = function(event) { callback(null, event.target.result); };
req.onerror = function(e) {
callback(this.error);
e.preventDefault();
};
},storeRemoteEntry:function (store, path, entry, callback) {
var req = store.put(entry, path);
req.onsuccess = function() { callback(null); };
req.onerror = function(e) {
callback(this.error);
e.preventDefault();
};
},removeRemoteEntry:function (store, path, callback) {
var req = store.delete(path);
req.onsuccess = function() { callback(null); };
req.onerror = function(e) {
callback(this.error);
e.preventDefault();
};
},reconcile:function (src, dst, callback) {
var total = 0;

var create = [];
Object.keys(src.entries).forEach(function (key) {
var e = src.entries[key];
var e2 = dst.entries[key];
if (!e2 || e.timestamp > e2.timestamp) {
create.push(key);
total++;
}
});

var remove = [];
Object.keys(dst.entries).forEach(function (key) {
var e = dst.entries[key];
var e2 = src.entries[key];
if (!e2) {
remove.push(key);
total++;
}
});

if (!total) {
return callback(null);
}

var errored = false;
var completed = 0;
var db = src.type === 'remote' ? src.db : dst.db;
var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
var store = transaction.objectStore(IDBFS.DB_STORE_NAME);

function done(err) {
if (err) {
if (!done.errored) {
done.errored = true;
return callback(err);
}
return;
}
if (++completed >= total) {
return callback(null);
}
};

transaction.onerror = function(e) {
done(this.error);
e.preventDefault();
};

// sort paths in ascending order so directory entries are created
// before the files inside them
create.sort().forEach(function (path) {
if (dst.type === 'local') {
IDBFS.loadRemoteEntry(store, path, function (err, entry) {
if (err) return done(err);
IDBFS.storeLocalEntry(path, entry, done);
});
} else {
IDBFS.loadLocalEntry(path, function (err, entry) {
if (err) return done(err);
IDBFS.storeRemoteEntry(store, path, entry, done);
});
}
});

// sort paths in descending order so files are deleted before their
// parent directories
remove.sort().reverse().forEach(function(path) {
if (dst.type === 'local') {
IDBFS.removeLocalEntry(path, done);
} else {
IDBFS.removeRemoteEntry(store, path, done);
}
});
}};

var NODEFS={isWindows:false,staticInit:function () {
NODEFS.isWindows = !!process.platform.match(/^win/);
},mount:function (mount) {
assert(ENVIRONMENT_IS_NODE);
return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
},createNode:function (parent, name, mode, dev) {
if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
var node = FS.createNode(parent, name, mode);
node.node_ops = NODEFS.node_ops;
node.stream_ops = NODEFS.stream_ops;
return node;
},getMode:function (path) {
var stat;
try {
stat = fs.lstatSync(path);
if (NODEFS.isWindows) {
// On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so
// propagate write bits to execute bits.
stat.mode = stat.mode | ((stat.mode & 146) >> 1);
}
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
return stat.mode;
},realPath:function (node) {
var parts = [];
while (node.parent !== node) {
parts.push(node.name);
node = node.parent;
}
parts.push(node.mount.opts.root);
parts.reverse();
return PATH.join.apply(null, parts);
},flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
flags &= ~0100000 /*O_LARGEFILE*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
if (flags in NODEFS.flagsToPermissionStringMap) {
return NODEFS.flagsToPermissionStringMap[flags];
} else {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
},node_ops:{getattr:function (node) {
var path = NODEFS.realPath(node);
var stat;
try {
stat = fs.lstatSync(path);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
// node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
// See http://support.microsoft.com/kb/140365
if (NODEFS.isWindows && !stat.blksize) {
stat.blksize = 4096;
}
if (NODEFS.isWindows && !stat.blocks) {
stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
}
return {
dev: stat.dev,
ino: stat.ino,
mode: stat.mode,
nlink: stat.nlink,
uid: stat.uid,
gid: stat.gid,
rdev: stat.rdev,
size: stat.size,
atime: stat.atime,
mtime: stat.mtime,
ctime: stat.ctime,
blksize: stat.blksize,
blocks: stat.blocks
};
},setattr:function (node, attr) {
var path = NODEFS.realPath(node);
try {
if (attr.mode !== undefined) {
fs.chmodSync(path, attr.mode);
// update the common node structure mode as well
node.mode = attr.mode;
}
if (attr.timestamp !== undefined) {
var date = new Date(attr.timestamp);
fs.utimesSync(path, date, date);
}
if (attr.size !== undefined) {
fs.truncateSync(path, attr.size);
}
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},lookup:function (parent, name) {
var path = PATH.join2(NODEFS.realPath(parent), name);
var mode = NODEFS.getMode(path);
return NODEFS.createNode(parent, name, mode);
},mknod:function (parent, name, mode, dev) {
var node = NODEFS.createNode(parent, name, mode, dev);
// create the backing node for this in the fs root as well
var path = NODEFS.realPath(node);
try {
if (FS.isDir(node.mode)) {
fs.mkdirSync(path, node.mode);
} else {
fs.writeFileSync(path, '', { mode: node.mode });
}
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
return node;
},rename:function (oldNode, newDir, newName) {
var oldPath = NODEFS.realPath(oldNode);
var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
try {
fs.renameSync(oldPath, newPath);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},unlink:function (parent, name) {
var path = PATH.join2(NODEFS.realPath(parent), name);
try {
fs.unlinkSync(path);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},rmdir:function (parent, name) {
var path = PATH.join2(NODEFS.realPath(parent), name);
try {
fs.rmdirSync(path);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},readdir:function (node) {
var path = NODEFS.realPath(node);
try {
return fs.readdirSync(path);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},symlink:function (parent, newName, oldPath) {
var newPath = PATH.join2(NODEFS.realPath(parent), newName);
try {
fs.symlinkSync(oldPath, newPath);
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},readlink:function (node) {
var path = NODEFS.realPath(node);
try {
path = fs.readlinkSync(path);
path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
return path;
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
}},stream_ops:{open:function (stream) {
var path = NODEFS.realPath(stream.node);
try {
if (FS.isFile(stream.node.mode)) {
stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
}
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},close:function (stream) {
try {
if (FS.isFile(stream.node.mode) && stream.nfd) {
fs.closeSync(stream.nfd);
}
} catch (e) {
if (!e.code) throw e;
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
},read:function (stream, buffer, offset, length, position) {
if (length === 0) return 0; // node errors on 0 length reads
// FIXME this is terrible.
var nbuffer = new Buffer(length);
var res;
try {
res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
if (res > 0) {
for (var i = 0; i < res; i++) {
buffer[offset + i] = nbuffer[i];
}
}
return res;
},write:function (stream, buffer, offset, length, position) {
// FIXME this is terrible.
var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
var res;
try {
res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
return res;
},llseek:function (stream, offset, whence) {
var position = offset;
if (whence === 1) {  // SEEK_CUR.
position += stream.position;
} else if (whence === 2) {  // SEEK_END.
if (FS.isFile(stream.node.mode)) {
try {
var stat = fs.fstatSync(stream.nfd);
position += stat.size;
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES[e.code]);
}
}
}

if (position < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}

return position;
}}};

var WORKERFS={DIR_MODE:16895,FILE_MODE:33279,reader:null,mount:function (mount) {
assert(ENVIRONMENT_IS_WORKER);
if (!WORKERFS.reader) WORKERFS.reader = new FileReaderSync();
var root = WORKERFS.createNode(null, '/', WORKERFS.DIR_MODE, 0);
var createdParents = {};
function ensureParent(path) {
// return the parent node, creating subdirs as necessary
var parts = path.split('/');
var parent = root;
for (var i = 0; i < parts.length-1; i++) {
var curr = parts.slice(0, i+1).join('/');
if (!createdParents[curr]) {
createdParents[curr] = WORKERFS.createNode(parent, curr, WORKERFS.DIR_MODE, 0);
}
parent = createdParents[curr];
}
return parent;
}
function base(path) {
var parts = path.split('/');
return parts[parts.length-1];
}
// We also accept FileList here, by using Array.prototype
Array.prototype.forEach.call(mount.opts["files"] || [], function(file) {
WORKERFS.createNode(ensureParent(file.name), base(file.name), WORKERFS.FILE_MODE, 0, file, file.lastModifiedDate);
});
(mount.opts["blobs"] || []).forEach(function(obj) {
WORKERFS.createNode(ensureParent(obj["name"]), base(obj["name"]), WORKERFS.FILE_MODE, 0, obj["data"]);
});
(mount.opts["packages"] || []).forEach(function(pack) {
pack['metadata'].files.forEach(function(file) {
var name = file.filename.substr(1); // remove initial slash
WORKERFS.createNode(ensureParent(name), base(name), WORKERFS.FILE_MODE, 0, pack['blob'].slice(file.start, file.end));
});
});
return root;
},createNode:function (parent, name, mode, dev, contents, mtime) {
var node = FS.createNode(parent, name, mode);
node.mode = mode;
node.node_ops = WORKERFS.node_ops;
node.stream_ops = WORKERFS.stream_ops;
node.timestamp = (mtime || new Date).getTime();
assert(WORKERFS.FILE_MODE !== WORKERFS.DIR_MODE);
if (mode === WORKERFS.FILE_MODE) {
node.size = contents.size;
node.contents = contents;
} else {
node.size = 4096;
node.contents = {};
}
if (parent) {
parent.contents[name] = node;
}
return node;
},node_ops:{getattr:function (node) {
return {
dev: 1,
ino: undefined,
mode: node.mode,
nlink: 1,
uid: 0,
gid: 0,
rdev: undefined,
size: node.size,
atime: new Date(node.timestamp),
mtime: new Date(node.timestamp),
ctime: new Date(node.timestamp),
blksize: 4096,
blocks: Math.ceil(node.size / 4096),
};
},setattr:function (node, attr) {
if (attr.mode !== undefined) {
node.mode = attr.mode;
}
if (attr.timestamp !== undefined) {
node.timestamp = attr.timestamp;
}
},lookup:function (parent, name) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
},mknod:function (parent, name, mode, dev) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},rename:function (oldNode, newDir, newName) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},unlink:function (parent, name) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},rmdir:function (parent, name) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},readdir:function (node) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},symlink:function (parent, newName, oldPath) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
},readlink:function (node) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}},stream_ops:{read:function (stream, buffer, offset, length, position) {
if (position >= stream.node.size) return 0;
var chunk = stream.node.contents.slice(position, position + length);
var ab = WORKERFS.reader.readAsArrayBuffer(chunk);
buffer.set(new Uint8Array(ab), offset);
return chunk.size;
},write:function (stream, buffer, offset, length, position) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
},llseek:function (stream, offset, whence) {
var position = offset;
if (whence === 1) {  // SEEK_CUR.
position += stream.position;
} else if (whence === 2) {  // SEEK_END.
if (FS.isFile(stream.node.mode)) {
position += stream.node.size;
}
}
if (position < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
return position;
}}};

var _stdin=allocate(1, "i32*", ALLOC_STATIC);

var _stdout=allocate(1, "i32*", ALLOC_STATIC);

var _stderr=allocate(1, "i32*", ALLOC_STATIC);var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},filesystems:null,handleFSError:function (e) {
if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
return ___setErrNo(e.errno);
},lookupPath:function (path, opts) {
path = PATH.resolve(FS.cwd(), path);
opts = opts || {};

if (!path) return { path: '', node: null };

var defaults = {
follow_mount: true,
recurse_count: 0
};
for (var key in defaults) {
if (opts[key] === undefined) {
opts[key] = defaults[key];
}
}

if (opts.recurse_count > 8) {  // max recursive lookup of 8
throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
}

// split the path
var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
return !!p;
}), false);

// start at the root
var current = FS.root;
var current_path = '/';

for (var i = 0; i < parts.length; i++) {
var islast = (i === parts.length-1);
if (islast && opts.parent) {
// stop resolving
break;
}

current = FS.lookupNode(current, parts[i]);
current_path = PATH.join2(current_path, parts[i]);

// jump to the mount's root node if this is a mountpoint
if (FS.isMountpoint(current)) {
if (!islast || (islast && opts.follow_mount)) {
current = current.mounted.root;
}
}

// by default, lookupPath will not follow a symlink if it is the final path component.
// setting opts.follow = true will override this behavior.
if (!islast || opts.follow) {
var count = 0;
while (FS.isLink(current.mode)) {
var link = FS.readlink(current_path);
current_path = PATH.resolve(PATH.dirname(current_path), link);

var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
current = lookup.node;

if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
}
}
}
}

return { path: current_path, node: current };
},getPath:function (node) {
var path;
while (true) {
if (FS.isRoot(node)) {
var mount = node.mount.mountpoint;
if (!path) return mount;
return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
}
path = path ? node.name + '/' + path : node.name;
node = node.parent;
}
},hashName:function (parentid, name) {
var hash = 0;


for (var i = 0; i < name.length; i++) {
hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
}
return ((parentid + hash) >>> 0) % FS.nameTable.length;
},hashAddNode:function (node) {
var hash = FS.hashName(node.parent.id, node.name);
node.name_next = FS.nameTable[hash];
FS.nameTable[hash] = node;
},hashRemoveNode:function (node) {
var hash = FS.hashName(node.parent.id, node.name);
if (FS.nameTable[hash] === node) {
FS.nameTable[hash] = node.name_next;
} else {
var current = FS.nameTable[hash];
while (current) {
if (current.name_next === node) {
current.name_next = node.name_next;
break;
}
current = current.name_next;
}
}
},lookupNode:function (parent, name) {
var err = FS.mayLookup(parent);
if (err) {
throw new FS.ErrnoError(err, parent);
}
var hash = FS.hashName(parent.id, name);
for (var node = FS.nameTable[hash]; node; node = node.name_next) {
var nodeName = node.name;
if (node.parent.id === parent.id && nodeName === name) {
return node;
}
}
// if we failed to find it in the cache, call into the VFS
return FS.lookup(parent, name);
},createNode:function (parent, name, mode, rdev) {
if (!FS.FSNode) {
FS.FSNode = function(parent, name, mode, rdev) {
if (!parent) {
parent = this;  // root node sets parent to itself
}
this.parent = parent;
this.mount = parent.mount;
this.mounted = null;
this.id = FS.nextInode++;
this.name = name;
this.mode = mode;
this.node_ops = {};
this.stream_ops = {};
this.rdev = rdev;
};

FS.FSNode.prototype = {};

// compatibility
var readMode = 292 | 73;
var writeMode = 146;

// NOTE we must use Object.defineProperties instead of individual calls to
// Object.defineProperty in order to make closure compiler happy
Object.defineProperties(FS.FSNode.prototype, {
read: {
get: function() { return (this.mode & readMode) === readMode; },
set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
},
write: {
get: function() { return (this.mode & writeMode) === writeMode; },
set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
},
isFolder: {
get: function() { return FS.isDir(this.mode); }
},
isDevice: {
get: function() { return FS.isChrdev(this.mode); }
}
});
}

var node = new FS.FSNode(parent, name, mode, rdev);

FS.hashAddNode(node);

return node;
},destroyNode:function (node) {
FS.hashRemoveNode(node);
},isRoot:function (node) {
return node === node.parent;
},isMountpoint:function (node) {
return !!node.mounted;
},isFile:function (mode) {
return (mode & 61440) === 32768;
},isDir:function (mode) {
return (mode & 61440) === 16384;
},isLink:function (mode) {
return (mode & 61440) === 40960;
},isChrdev:function (mode) {
return (mode & 61440) === 8192;
},isBlkdev:function (mode) {
return (mode & 61440) === 24576;
},isFIFO:function (mode) {
return (mode & 61440) === 4096;
},isSocket:function (mode) {
return (mode & 49152) === 49152;
},flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
var flags = FS.flagModes[str];
if (typeof flags === 'undefined') {
throw new Error('Unknown file open mode: ' + str);
}
return flags;
},flagsToPermissionString:function (flag) {
var perms = ['r', 'w', 'rw'][flag & 3];
if ((flag & 512)) {
perms += 'w';
}
return perms;
},nodePermissions:function (node, perms) {
if (FS.ignorePermissions) {
return 0;
}
// return 0 if any user, group or owner bits are set.
if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
return ERRNO_CODES.EACCES;
} else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
return ERRNO_CODES.EACCES;
} else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
return ERRNO_CODES.EACCES;
}
return 0;
},mayLookup:function (dir) {
var err = FS.nodePermissions(dir, 'x');
if (err) return err;
if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
return 0;
},mayCreate:function (dir, name) {
try {
var node = FS.lookupNode(dir, name);
return ERRNO_CODES.EEXIST;
} catch (e) {
}
return FS.nodePermissions(dir, 'wx');
},mayDelete:function (dir, name, isdir) {
var node;
try {
node = FS.lookupNode(dir, name);
} catch (e) {
return e.errno;
}
var err = FS.nodePermissions(dir, 'wx');
if (err) {
return err;
}
if (isdir) {
if (!FS.isDir(node.mode)) {
return ERRNO_CODES.ENOTDIR;
}
if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
return ERRNO_CODES.EBUSY;
}
} else {
if (FS.isDir(node.mode)) {
return ERRNO_CODES.EISDIR;
}
}
return 0;
},mayOpen:function (node, flags) {
if (!node) {
return ERRNO_CODES.ENOENT;
}
if (FS.isLink(node.mode)) {
return ERRNO_CODES.ELOOP;
} else if (FS.isDir(node.mode)) {
if ((flags & 2097155) !== 0 ||  // opening for write
(flags & 512)) {
return ERRNO_CODES.EISDIR;
}
}
return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
},MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
fd_start = fd_start || 0;
fd_end = fd_end || FS.MAX_OPEN_FDS;
for (var fd = fd_start; fd <= fd_end; fd++) {
if (!FS.streams[fd]) {
return fd;
}
}
throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
},getStream:function (fd) {
return FS.streams[fd];
},createStream:function (stream, fd_start, fd_end) {
if (!FS.FSStream) {
FS.FSStream = function(){};
FS.FSStream.prototype = {};
// compatibility
Object.defineProperties(FS.FSStream.prototype, {
object: {
get: function() { return this.node; },
set: function(val) { this.node = val; }
},
isRead: {
get: function() { return (this.flags & 2097155) !== 1; }
},
isWrite: {
get: function() { return (this.flags & 2097155) !== 0; }
},
isAppend: {
get: function() { return (this.flags & 1024); }
}
});
}
// clone it, so we can return an instance of FSStream
var newStream = new FS.FSStream();
for (var p in stream) {
newStream[p] = stream[p];
}
stream = newStream;
var fd = FS.nextfd(fd_start, fd_end);
stream.fd = fd;
FS.streams[fd] = stream;
return stream;
},closeStream:function (fd) {
FS.streams[fd] = null;
},chrdev_stream_ops:{open:function (stream) {
var device = FS.getDevice(stream.node.rdev);
// override node's stream ops with the device's
stream.stream_ops = device.stream_ops;
// forward the open call
if (stream.stream_ops.open) {
stream.stream_ops.open(stream);
}
},llseek:function () {
throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
}},major:function (dev) {
return ((dev) >> 8);
},minor:function (dev) {
return ((dev) & 0xff);
},makedev:function (ma, mi) {
return ((ma) << 8 | (mi));
},registerDevice:function (dev, ops) {
FS.devices[dev] = { stream_ops: ops };
},getDevice:function (dev) {
return FS.devices[dev];
},getMounts:function (mount) {
var mounts = [];
var check = [mount];

while (check.length) {
var m = check.pop();

mounts.push(m);

check.push.apply(check, m.mounts);
}

return mounts;
},syncfs:function (populate, callback) {
if (typeof(populate) === 'function') {
callback = populate;
populate = false;
}

var mounts = FS.getMounts(FS.root.mount);
var completed = 0;

function done(err) {
if (err) {
if (!done.errored) {
done.errored = true;
return callback(err);
}
return;
}
if (++completed >= mounts.length) {
callback(null);
}
};

// sync all mounts
mounts.forEach(function (mount) {
if (!mount.type.syncfs) {
return done(null);
}
mount.type.syncfs(mount, populate, done);
});
},mount:function (type, opts, mountpoint) {
var root = mountpoint === '/';
var pseudo = !mountpoint;
var node;

if (root && FS.root) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
} else if (!root && !pseudo) {
var lookup = FS.lookupPath(mountpoint, { follow_mount: false });

mountpoint = lookup.path;  // use the absolute path
node = lookup.node;

if (FS.isMountpoint(node)) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
}

if (!FS.isDir(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
}
}

var mount = {
type: type,
opts: opts,
mountpoint: mountpoint,
mounts: []
};

// create a root node for the fs
var mountRoot = type.mount(mount);
mountRoot.mount = mount;
mount.root = mountRoot;

if (root) {
FS.root = mountRoot;
} else if (node) {
// set as a mountpoint
node.mounted = mount;

// add the new mount to the current mount's children
if (node.mount) {
node.mount.mounts.push(mount);
}
}

return mountRoot;
},unmount:function (mountpoint) {
var lookup = FS.lookupPath(mountpoint, { follow_mount: false });

if (!FS.isMountpoint(lookup.node)) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}

// destroy the nodes for this mount, and all its child mounts
var node = lookup.node;
var mount = node.mounted;
var mounts = FS.getMounts(mount);

Object.keys(FS.nameTable).forEach(function (hash) {
var current = FS.nameTable[hash];

while (current) {
var next = current.name_next;

if (mounts.indexOf(current.mount) !== -1) {
FS.destroyNode(current);
}

current = next;
}
});

// no longer a mountpoint
node.mounted = null;

// remove this mount from the child mounts
var idx = node.mount.mounts.indexOf(mount);
assert(idx !== -1);
node.mount.mounts.splice(idx, 1);
},lookup:function (parent, name) {
return parent.node_ops.lookup(parent, name);
},mknod:function (path, mode, dev) {
var lookup = FS.lookupPath(path, { parent: true });
var parent = lookup.node;
var name = PATH.basename(path);
if (!name || name === '.' || name === '..') {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
var err = FS.mayCreate(parent, name);
if (err) {
throw new FS.ErrnoError(err);
}
if (!parent.node_ops.mknod) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
return parent.node_ops.mknod(parent, name, mode, dev);
},create:function (path, mode) {
mode = mode !== undefined ? mode : 438 /* 0666 */;
mode &= 4095;
mode |= 32768;
return FS.mknod(path, mode, 0);
},mkdir:function (path, mode) {
mode = mode !== undefined ? mode : 511 /* 0777 */;
mode &= 511 | 512;
mode |= 16384;
return FS.mknod(path, mode, 0);
},mkdev:function (path, mode, dev) {
if (typeof(dev) === 'undefined') {
dev = mode;
mode = 438 /* 0666 */;
}
mode |= 8192;
return FS.mknod(path, mode, dev);
},symlink:function (oldpath, newpath) {
if (!PATH.resolve(oldpath)) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
var lookup = FS.lookupPath(newpath, { parent: true });
var parent = lookup.node;
if (!parent) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
var newname = PATH.basename(newpath);
var err = FS.mayCreate(parent, newname);
if (err) {
throw new FS.ErrnoError(err);
}
if (!parent.node_ops.symlink) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
return parent.node_ops.symlink(parent, newname, oldpath);
},rename:function (old_path, new_path) {
var old_dirname = PATH.dirname(old_path);
var new_dirname = PATH.dirname(new_path);
var old_name = PATH.basename(old_path);
var new_name = PATH.basename(new_path);
// parents must exist
var lookup, old_dir, new_dir;
try {
lookup = FS.lookupPath(old_path, { parent: true });
old_dir = lookup.node;
lookup = FS.lookupPath(new_path, { parent: true });
new_dir = lookup.node;
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
}
if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
// need to be part of the same mount
if (old_dir.mount !== new_dir.mount) {
throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
}
// source must exist
var old_node = FS.lookupNode(old_dir, old_name);
// old path should not be an ancestor of the new path
var relative = PATH.relative(old_path, new_dirname);
if (relative.charAt(0) !== '.') {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
// new path should not be an ancestor of the old path
relative = PATH.relative(new_path, old_dirname);
if (relative.charAt(0) !== '.') {
throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
}
// see if the new path already exists
var new_node;
try {
new_node = FS.lookupNode(new_dir, new_name);
} catch (e) {
// not fatal
}
// early out if nothing needs to change
if (old_node === new_node) {
return;
}
// we'll need to delete the old entry
var isdir = FS.isDir(old_node.mode);
var err = FS.mayDelete(old_dir, old_name, isdir);
if (err) {
throw new FS.ErrnoError(err);
}
// need delete permissions if we'll be overwriting.
// need create permissions if new doesn't already exist.
err = new_node ?
FS.mayDelete(new_dir, new_name, isdir) :
FS.mayCreate(new_dir, new_name);
if (err) {
throw new FS.ErrnoError(err);
}
if (!old_dir.node_ops.rename) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
}
// if we are going to change the parent, check write permissions
if (new_dir !== old_dir) {
err = FS.nodePermissions(old_dir, 'w');
if (err) {
throw new FS.ErrnoError(err);
}
}
try {
if (FS.trackingDelegate['willMovePath']) {
FS.trackingDelegate['willMovePath'](old_path, new_path);
}
} catch(e) {
console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
}
// remove the node from the lookup hash
FS.hashRemoveNode(old_node);
// do the underlying fs rename
try {
old_dir.node_ops.rename(old_node, new_dir, new_name);
} catch (e) {
throw e;
} finally {
// add the node back to the hash (in case node_ops.rename
// changed its name)
FS.hashAddNode(old_node);
}
try {
if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
} catch(e) {
console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
}
},rmdir:function (path) {
var lookup = FS.lookupPath(path, { parent: true });
var parent = lookup.node;
var name = PATH.basename(path);
var node = FS.lookupNode(parent, name);
var err = FS.mayDelete(parent, name, true);
if (err) {
throw new FS.ErrnoError(err);
}
if (!parent.node_ops.rmdir) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
if (FS.isMountpoint(node)) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
}
try {
if (FS.trackingDelegate['willDeletePath']) {
FS.trackingDelegate['willDeletePath'](path);
}
} catch(e) {
console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
}
parent.node_ops.rmdir(parent, name);
FS.destroyNode(node);
try {
if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
} catch(e) {
console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
}
},readdir:function (path) {
var lookup = FS.lookupPath(path, { follow: true });
var node = lookup.node;
if (!node.node_ops.readdir) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
}
return node.node_ops.readdir(node);
},unlink:function (path) {
var lookup = FS.lookupPath(path, { parent: true });
var parent = lookup.node;
var name = PATH.basename(path);
var node = FS.lookupNode(parent, name);
var err = FS.mayDelete(parent, name, false);
if (err) {
// POSIX says unlink should set EPERM, not EISDIR
if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
throw new FS.ErrnoError(err);
}
if (!parent.node_ops.unlink) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
if (FS.isMountpoint(node)) {
throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
}
try {
if (FS.trackingDelegate['willDeletePath']) {
FS.trackingDelegate['willDeletePath'](path);
}
} catch(e) {
console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
}
parent.node_ops.unlink(parent, name);
FS.destroyNode(node);
try {
if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
} catch(e) {
console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
}
},readlink:function (path) {
var lookup = FS.lookupPath(path);
var link = lookup.node;
if (!link) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
if (!link.node_ops.readlink) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
return PATH.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
},stat:function (path, dontFollow) {
var lookup = FS.lookupPath(path, { follow: !dontFollow });
var node = lookup.node;
if (!node) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
if (!node.node_ops.getattr) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
return node.node_ops.getattr(node);
},lstat:function (path) {
return FS.stat(path, true);
},chmod:function (path, mode, dontFollow) {
var node;
if (typeof path === 'string') {
var lookup = FS.lookupPath(path, { follow: !dontFollow });
node = lookup.node;
} else {
node = path;
}
if (!node.node_ops.setattr) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
node.node_ops.setattr(node, {
mode: (mode & 4095) | (node.mode & ~4095),
timestamp: Date.now()
});
},lchmod:function (path, mode) {
FS.chmod(path, mode, true);
},fchmod:function (fd, mode) {
var stream = FS.getStream(fd);
if (!stream) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
FS.chmod(stream.node, mode);
},chown:function (path, uid, gid, dontFollow) {
var node;
if (typeof path === 'string') {
var lookup = FS.lookupPath(path, { follow: !dontFollow });
node = lookup.node;
} else {
node = path;
}
if (!node.node_ops.setattr) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
node.node_ops.setattr(node, {
timestamp: Date.now()
// we ignore the uid / gid for now
});
},lchown:function (path, uid, gid) {
FS.chown(path, uid, gid, true);
},fchown:function (fd, uid, gid) {
var stream = FS.getStream(fd);
if (!stream) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
FS.chown(stream.node, uid, gid);
},truncate:function (path, len) {
if (len < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
var node;
if (typeof path === 'string') {
var lookup = FS.lookupPath(path, { follow: true });
node = lookup.node;
} else {
node = path;
}
if (!node.node_ops.setattr) {
throw new FS.ErrnoError(ERRNO_CODES.EPERM);
}
if (FS.isDir(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
}
if (!FS.isFile(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
var err = FS.nodePermissions(node, 'w');
if (err) {
throw new FS.ErrnoError(err);
}
node.node_ops.setattr(node, {
size: len,
timestamp: Date.now()
});
},ftruncate:function (fd, len) {
var stream = FS.getStream(fd);
if (!stream) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
if ((stream.flags & 2097155) === 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
FS.truncate(stream.node, len);
},utime:function (path, atime, mtime) {
var lookup = FS.lookupPath(path, { follow: true });
var node = lookup.node;
node.node_ops.setattr(node, {
timestamp: Math.max(atime, mtime)
});
},open:function (path, flags, mode, fd_start, fd_end) {
if (path === "") {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
if ((flags & 64)) {
mode = (mode & 4095) | 32768;
} else {
mode = 0;
}
var node;
if (typeof path === 'object') {
node = path;
} else {
path = PATH.normalize(path);
try {
var lookup = FS.lookupPath(path, {
follow: !(flags & 131072)
});
node = lookup.node;
} catch (e) {
// ignore
}
}
// perhaps we need to create the node
var created = false;
if ((flags & 64)) {
if (node) {
// if O_CREAT and O_EXCL are set, error out if the node already exists
if ((flags & 128)) {
throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
}
} else {
// node doesn't exist, try to create it
node = FS.mknod(path, mode, 0);
created = true;
}
}
if (!node) {
throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
}
// can't truncate a device
if (FS.isChrdev(node.mode)) {
flags &= ~512;
}
// if asked only for a directory, then this must be one
if ((flags & 65536) && !FS.isDir(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
}
// check permissions, if this is not a file we just created now (it is ok to
// create and write to a file with read-only permissions; it is read-only
// for later use)
if (!created) {
var err = FS.mayOpen(node, flags);
if (err) {
throw new FS.ErrnoError(err);
}
}
// do truncation if necessary
if ((flags & 512)) {
FS.truncate(node, 0);
}
// we've already handled these, don't pass down to the underlying vfs
flags &= ~(128 | 512);

// register the stream with the filesystem
var stream = FS.createStream({
node: node,
path: FS.getPath(node),  // we want the absolute path to the node
flags: flags,
seekable: true,
position: 0,
stream_ops: node.stream_ops,
// used by the file family libc calls (fopen, fwrite, ferror, etc.)
ungotten: [],
error: false
}, fd_start, fd_end);
// call the new stream's open function
if (stream.stream_ops.open) {
stream.stream_ops.open(stream);
}
if (Module['logReadFiles'] && !(flags & 1)) {
if (!FS.readFiles) FS.readFiles = {};
if (!(path in FS.readFiles)) {
FS.readFiles[path] = 1;
Module['printErr']('read file: ' + path);
}
}
try {
if (FS.trackingDelegate['onOpenFile']) {
var trackingFlags = 0;
if ((flags & 2097155) !== 1) {
trackingFlags |= FS.tracking.openFlags.READ;
}
if ((flags & 2097155) !== 0) {
trackingFlags |= FS.tracking.openFlags.WRITE;
}
FS.trackingDelegate['onOpenFile'](path, trackingFlags);
}
} catch(e) {
console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
}
return stream;
},close:function (stream) {
if (stream.getdents) stream.getdents = null; // free readdir state
try {
if (stream.stream_ops.close) {
stream.stream_ops.close(stream);
}
} catch (e) {
throw e;
} finally {
FS.closeStream(stream.fd);
}
},llseek:function (stream, offset, whence) {
if (!stream.seekable || !stream.stream_ops.llseek) {
throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
}
stream.position = stream.stream_ops.llseek(stream, offset, whence);
stream.ungotten = [];
return stream.position;
},read:function (stream, buffer, offset, length, position) {
if (length < 0 || position < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
if ((stream.flags & 2097155) === 1) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
if (FS.isDir(stream.node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
}
if (!stream.stream_ops.read) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
var seeking = true;
if (typeof position === 'undefined') {
position = stream.position;
seeking = false;
} else if (!stream.seekable) {
throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
}
var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
if (!seeking) stream.position += bytesRead;
return bytesRead;
},write:function (stream, buffer, offset, length, position, canOwn) {
if (length < 0 || position < 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
if ((stream.flags & 2097155) === 0) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
if (FS.isDir(stream.node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
}
if (!stream.stream_ops.write) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
if (stream.flags & 1024) {
// seek to the end before writing in append mode
FS.llseek(stream, 0, 2);
}
var seeking = true;
if (typeof position === 'undefined') {
position = stream.position;
seeking = false;
} else if (!stream.seekable) {
throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
}
var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
if (!seeking) stream.position += bytesWritten;
try {
if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
} catch(e) {
console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
}
return bytesWritten;
},allocate:function (stream, offset, length) {
if (offset < 0 || length <= 0) {
throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
}
if ((stream.flags & 2097155) === 0) {
throw new FS.ErrnoError(ERRNO_CODES.EBADF);
}
if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
}
if (!stream.stream_ops.allocate) {
throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
}
stream.stream_ops.allocate(stream, offset, length);
},mmap:function (stream, buffer, offset, length, position, prot, flags) {
// TODO if PROT is PROT_WRITE, make sure we have write access
if ((stream.flags & 2097155) === 1) {
throw new FS.ErrnoError(ERRNO_CODES.EACCES);
}
if (!stream.stream_ops.mmap) {
throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
}
return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
},msync:function (stream, buffer, offset, length, mmapFlags) {
if (!stream || !stream.stream_ops.msync) {
return 0;
}
return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
},munmap:function (stream) {
return 0;
},ioctl:function (stream, cmd, arg) {
if (!stream.stream_ops.ioctl) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
}
return stream.stream_ops.ioctl(stream, cmd, arg);
},readFile:function (path, opts) {
opts = opts || {};
opts.flags = opts.flags || 'r';
opts.encoding = opts.encoding || 'binary';
if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
throw new Error('Invalid encoding type "' + opts.encoding + '"');
}
var ret;
var stream = FS.open(path, opts.flags);
var stat = FS.stat(path);
var length = stat.size;
var buf = new Uint8Array(length);
FS.read(stream, buf, 0, length, 0);
if (opts.encoding === 'utf8') {
ret = UTF8ArrayToString(buf, 0);
} else if (opts.encoding === 'binary') {
ret = buf;
}
FS.close(stream);
return ret;
},writeFile:function (path, data, opts) {
opts = opts || {};
opts.flags = opts.flags || 'w';
opts.encoding = opts.encoding || 'utf8';
if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
throw new Error('Invalid encoding type "' + opts.encoding + '"');
}
var stream = FS.open(path, opts.flags, opts.mode);
if (opts.encoding === 'utf8') {
var buf = new Uint8Array(lengthBytesUTF8(data)+1);
var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
} else if (opts.encoding === 'binary') {
FS.write(stream, data, 0, data.length, 0, opts.canOwn);
}
FS.close(stream);
},cwd:function () {
return FS.currentPath;
},chdir:function (path) {
var lookup = FS.lookupPath(path, { follow: true });
if (!FS.isDir(lookup.node.mode)) {
throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
}
var err = FS.nodePermissions(lookup.node, 'x');
if (err) {
throw new FS.ErrnoError(err);
}
FS.currentPath = lookup.path;
},createDefaultDirectories:function () {
FS.mkdir('/tmp');
FS.mkdir('/home');
FS.mkdir('/home/web_user');
},createDefaultDevices:function () {
// create /dev
FS.mkdir('/dev');
// setup /dev/null
FS.registerDevice(FS.makedev(1, 3), {
read: function() { return 0; },
write: function(stream, buffer, offset, length, pos) { return length; }
});
FS.mkdev('/dev/null', FS.makedev(1, 3));
// setup /dev/tty and /dev/tty1
// stderr needs to print output using Module['printErr']
// so we register a second tty just for it.
TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
FS.mkdev('/dev/tty', FS.makedev(5, 0));
FS.mkdev('/dev/tty1', FS.makedev(6, 0));
// setup /dev/[u]random
var random_device;
if (typeof crypto !== 'undefined') {
// for modern web browsers
var randomBuffer = new Uint8Array(1);
random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
} else if (ENVIRONMENT_IS_NODE) {
// for nodejs
random_device = function() { return require('crypto').randomBytes(1)[0]; };
} else {
// default for ES5 platforms
random_device = function() { return (Math.random()*256)|0; };
}
FS.createDevice('/dev', 'random', random_device);
FS.createDevice('/dev', 'urandom', random_device);
// we're not going to emulate the actual shm device,
// just create the tmp dirs that reside in it commonly
FS.mkdir('/dev/shm');
FS.mkdir('/dev/shm/tmp');
},createSpecialDirectories:function () {
// create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the name of the stream for fd 6 (see test_unistd_ttyname)
FS.mkdir('/proc');
FS.mkdir('/proc/self');
FS.mkdir('/proc/self/fd');
FS.mount({
mount: function() {
var node = FS.createNode('/proc/self', 'fd', 16384 | 0777, 73);
node.node_ops = {
lookup: function(parent, name) {
var fd = +name;
var stream = FS.getStream(fd);
if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
var ret = {
parent: null,
mount: { mountpoint: 'fake' },
node_ops: { readlink: function() { return stream.path } }
};
ret.parent = ret; // make it look like a simple root node
return ret;
}
};
return node;
}
}, {}, '/proc/self/fd');
},createStandardStreams:function () {
// TODO deprecate the old functionality of a single
// input / output callback and that utilizes FS.createDevice
// and instead require a unique set of stream ops

// by default, we symlink the standard streams to the
// default tty devices. however, if the standard streams
// have been overwritten we create a unique device for
// them instead.
if (Module['stdin']) {
FS.createDevice('/dev', 'stdin', Module['stdin']);
} else {
FS.symlink('/dev/tty', '/dev/stdin');
}
if (Module['stdout']) {
FS.createDevice('/dev', 'stdout', null, Module['stdout']);
} else {
FS.symlink('/dev/tty', '/dev/stdout');
}
if (Module['stderr']) {
FS.createDevice('/dev', 'stderr', null, Module['stderr']);
} else {
FS.symlink('/dev/tty1', '/dev/stderr');
}

// open default streams for the stdin, stdout and stderr devices
var stdin = FS.open('/dev/stdin', 'r');
assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');

var stdout = FS.open('/dev/stdout', 'w');
assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');

var stderr = FS.open('/dev/stderr', 'w');
assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
},ensureErrnoError:function () {
if (FS.ErrnoError) return;
FS.ErrnoError = function ErrnoError(errno, node) {
//Module.printErr(stackTrace()); // useful for debugging
this.node = node;
this.setErrno = function(errno) {
this.errno = errno;
for (var key in ERRNO_CODES) {
if (ERRNO_CODES[key] === errno) {
this.code = key;
break;
}
}
};
this.setErrno(errno);
this.message = ERRNO_MESSAGES[errno];
if (this.stack) this.stack = demangleAll(this.stack);
};
FS.ErrnoError.prototype = new Error();
FS.ErrnoError.prototype.constructor = FS.ErrnoError;
// Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
[ERRNO_CODES.ENOENT].forEach(function(code) {
FS.genericErrors[code] = new FS.ErrnoError(code);
FS.genericErrors[code].stack = '<generic error, no stack>';
});
},staticInit:function () {
FS.ensureErrnoError();

FS.nameTable = new Array(4096);

FS.mount(MEMFS, {}, '/');

FS.createDefaultDirectories();
FS.createDefaultDevices();
FS.createSpecialDirectories();

FS.filesystems = {
'MEMFS': MEMFS,
'IDBFS': IDBFS,
'NODEFS': NODEFS,
'WORKERFS': WORKERFS,
};
},init:function (input, output, error) {
assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
FS.init.initialized = true;

FS.ensureErrnoError();

// Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
Module['stdin'] = input || Module['stdin'];
Module['stdout'] = output || Module['stdout'];
Module['stderr'] = error || Module['stderr'];

FS.createStandardStreams();
},quit:function () {
FS.init.initialized = false;
// force-flush all streams, so we get musl std streams printed out
var fflush = Module['_fflush'];
if (fflush) fflush(0);
// close all of our streams
for (var i = 0; i < FS.streams.length; i++) {
var stream = FS.streams[i];
if (!stream) {
continue;
}
FS.close(stream);
}
},getMode:function (canRead, canWrite) {
var mode = 0;
if (canRead) mode |= 292 | 73;
if (canWrite) mode |= 146;
return mode;
},joinPath:function (parts, forceRelative) {
var path = PATH.join.apply(null, parts);
if (forceRelative && path[0] == '/') path = path.substr(1);
return path;
},absolutePath:function (relative, base) {
return PATH.resolve(base, relative);
},standardizePath:function (path) {
return PATH.normalize(path);
},findObject:function (path, dontResolveLastLink) {
var ret = FS.analyzePath(path, dontResolveLastLink);
if (ret.exists) {
return ret.object;
} else {
___setErrNo(ret.error);
return null;
}
},analyzePath:function (path, dontResolveLastLink) {
// operate from within the context of the symlink's target
try {
var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
path = lookup.path;
} catch (e) {
}
var ret = {
isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
parentExists: false, parentPath: null, parentObject: null
};
try {
var lookup = FS.lookupPath(path, { parent: true });
ret.parentExists = true;
ret.parentPath = lookup.path;
ret.parentObject = lookup.node;
ret.name = PATH.basename(path);
lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
ret.exists = true;
ret.path = lookup.path;
ret.object = lookup.node;
ret.name = lookup.node.name;
ret.isRoot = lookup.path === '/';
} catch (e) {
ret.error = e.errno;
};
return ret;
},createFolder:function (parent, name, canRead, canWrite) {
var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
var mode = FS.getMode(canRead, canWrite);
return FS.mkdir(path, mode);
},createPath:function (parent, path, canRead, canWrite) {
parent = typeof parent === 'string' ? parent : FS.getPath(parent);
var parts = path.split('/').reverse();
while (parts.length) {
var part = parts.pop();
if (!part) continue;
var current = PATH.join2(parent, part);
try {
FS.mkdir(current);
} catch (e) {
// ignore EEXIST
}
parent = current;
}
return current;
},createFile:function (parent, name, properties, canRead, canWrite) {
var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
var mode = FS.getMode(canRead, canWrite);
return FS.create(path, mode);
},createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
var mode = FS.getMode(canRead, canWrite);
var node = FS.create(path, mode);
if (data) {
if (typeof data === 'string') {
var arr = new Array(data.length);
for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
data = arr;
}
// make sure we can write to the file
FS.chmod(node, mode | 146);
var stream = FS.open(node, 'w');
FS.write(stream, data, 0, data.length, 0, canOwn);
FS.close(stream);
FS.chmod(node, mode);
}
return node;
},createDevice:function (parent, name, input, output) {
var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
var mode = FS.getMode(!!input, !!output);
if (!FS.createDevice.major) FS.createDevice.major = 64;
var dev = FS.makedev(FS.createDevice.major++, 0);
// Create a fake device that a set of stream ops to emulate
// the old behavior.
FS.registerDevice(dev, {
open: function(stream) {
stream.seekable = false;
},
close: function(stream) {
// flush any pending line data
if (output && output.buffer && output.buffer.length) {
output(10);
}
},
read: function(stream, buffer, offset, length, pos /* ignored */) {
var bytesRead = 0;
for (var i = 0; i < length; i++) {
var result;
try {
result = input();
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
if (result === undefined && bytesRead === 0) {
throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
}
if (result === null || result === undefined) break;
bytesRead++;
buffer[offset+i] = result;
}
if (bytesRead) {
stream.node.timestamp = Date.now();
}
return bytesRead;
},
write: function(stream, buffer, offset, length, pos) {
for (var i = 0; i < length; i++) {
try {
output(buffer[offset+i]);
} catch (e) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
}
if (length) {
stream.node.timestamp = Date.now();
}
return i;
}
});
return FS.mkdev(path, mode, dev);
},createLink:function (parent, name, target, canRead, canWrite) {
var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
return FS.symlink(target, path);
},forceLoadFile:function (obj) {
if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
var success = true;
if (typeof XMLHttpRequest !== 'undefined') {
throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
} else if (Module['read']) {
// Command-line.
try {
// WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
//          read() will try to parse UTF8.
obj.contents = intArrayFromString(Module['read'](obj.url), true);
obj.usedBytes = obj.contents.length;
} catch (e) {
success = false;
}
} else {
throw new Error('Cannot load without read() or XMLHttpRequest.');
}
if (!success) ___setErrNo(ERRNO_CODES.EIO);
return success;
},createLazyFile:function (parent, name, url, canRead, canWrite) {
// Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
function LazyUint8Array() {
this.lengthKnown = false;
this.chunks = []; // Loaded chunks. Index is the chunk number
}
LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
if (idx > this.length-1 || idx < 0) {
return undefined;
}
var chunkOffset = idx % this.chunkSize;
var chunkNum = (idx / this.chunkSize)|0;
return this.getter(chunkNum)[chunkOffset];
}
LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
this.getter = getter;
}
LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
// Find length
var xhr = new XMLHttpRequest();
xhr.open('HEAD', url, false);
xhr.send(null);
if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
var datalength = Number(xhr.getResponseHeader("Content-length"));
var header;
var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
var chunkSize = 1024*1024; // Chunk size in bytes

if (!hasByteServing) chunkSize = datalength;

// Function to get a range from the remote URL.
var doXHR = (function(from, to) {
if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");

// TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
var xhr = new XMLHttpRequest();
xhr.open('GET', url, false);
if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);

// Some hints to the browser that we want binary data.
if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
if (xhr.overrideMimeType) {
xhr.overrideMimeType('text/plain; charset=x-user-defined');
}

xhr.send(null);
if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
if (xhr.response !== undefined) {
return new Uint8Array(xhr.response || []);
} else {
return intArrayFromString(xhr.responseText || '', true);
}
});
var lazyArray = this;
lazyArray.setDataGetter(function(chunkNum) {
var start = chunkNum * chunkSize;
var end = (chunkNum+1) * chunkSize - 1; // including this byte
end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
lazyArray.chunks[chunkNum] = doXHR(start, end);
}
if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
return lazyArray.chunks[chunkNum];
});

this._length = datalength;
this._chunkSize = chunkSize;
this.lengthKnown = true;
}
if (typeof XMLHttpRequest !== 'undefined') {
if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
var lazyArray = new LazyUint8Array();
Object.defineProperty(lazyArray, "length", {
get: function() {
if(!this.lengthKnown) {
this.cacheLength();
}
return this._length;
}
});
Object.defineProperty(lazyArray, "chunkSize", {
get: function() {
if(!this.lengthKnown) {
this.cacheLength();
}
return this._chunkSize;
}
});

var properties = { isDevice: false, contents: lazyArray };
} else {
var properties = { isDevice: false, url: url };
}

var node = FS.createFile(parent, name, properties, canRead, canWrite);
// This is a total hack, but I want to get this lazy file code out of the
// core of MEMFS. If we want to keep this lazy file concept I feel it should
// be its own thin LAZYFS proxying calls to MEMFS.
if (properties.contents) {
node.contents = properties.contents;
} else if (properties.url) {
node.contents = null;
node.url = properties.url;
}
// Add a function that defers querying the file size until it is asked the first time.
Object.defineProperty(node, "usedBytes", {
get: function() { return this.contents.length; }
});
// override each stream op with one that tries to force load the lazy file first
var stream_ops = {};
var keys = Object.keys(node.stream_ops);
keys.forEach(function(key) {
var fn = node.stream_ops[key];
stream_ops[key] = function forceLoadLazyFile() {
if (!FS.forceLoadFile(node)) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
return fn.apply(null, arguments);
};
});
// use a custom read function
stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
if (!FS.forceLoadFile(node)) {
throw new FS.ErrnoError(ERRNO_CODES.EIO);
}
var contents = stream.node.contents;
if (position >= contents.length)
return 0;
var size = Math.min(contents.length - position, length);
assert(size >= 0);
if (contents.slice) { // normal array
for (var i = 0; i < size; i++) {
buffer[offset + i] = contents[position + i];
}
} else {
for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
buffer[offset + i] = contents.get(position + i);
}
}
return size;
};
node.stream_ops = stream_ops;
return node;
},createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
Browser.init();
// TODO we should allow people to just pass in a complete filename instead
// of parent and name being that we just join them anyways
var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
function processData(byteArray) {
function finish(byteArray) {
if (preFinish) preFinish();
if (!dontCreateFile) {
FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
}
if (onload) onload();
removeRunDependency(dep);
}
var handled = false;
Module['preloadPlugins'].forEach(function(plugin) {
if (handled) return;
if (plugin['canHandle'](fullname)) {
plugin['handle'](byteArray, fullname, finish, function() {
if (onerror) onerror();
removeRunDependency(dep);
});
handled = true;
}
});
if (!handled) finish(byteArray);
}
addRunDependency(dep);
if (typeof url == 'string') {
Browser.asyncLoad(url, function(byteArray) {
processData(byteArray);
}, onerror);
} else {
processData(url);
}
},indexedDB:function () {
return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
},DB_NAME:function () {
return 'EM_FS_' + window.location.pathname;
},DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
onload = onload || function(){};
onerror = onerror || function(){};
var indexedDB = FS.indexedDB();
try {
var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
} catch (e) {
return onerror(e);
}
openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
console.log('creating db');
var db = openRequest.result;
db.createObjectStore(FS.DB_STORE_NAME);
};
openRequest.onsuccess = function openRequest_onsuccess() {
var db = openRequest.result;
var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
var files = transaction.objectStore(FS.DB_STORE_NAME);
var ok = 0, fail = 0, total = paths.length;
function finish() {
if (fail == 0) onload(); else onerror();
}
paths.forEach(function(path) {
var putRequest = files.put(FS.analyzePath(path).object.contents, path);
putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
});
transaction.onerror = onerror;
};
openRequest.onerror = onerror;
},loadFilesFromDB:function (paths, onload, onerror) {
onload = onload || function(){};
onerror = onerror || function(){};
var indexedDB = FS.indexedDB();
try {
var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
} catch (e) {
return onerror(e);
}
openRequest.onupgradeneeded = onerror; // no database to load from
openRequest.onsuccess = function openRequest_onsuccess() {
var db = openRequest.result;
try {
var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
} catch(e) {
onerror(e);
return;
}
var files = transaction.objectStore(FS.DB_STORE_NAME);
var ok = 0, fail = 0, total = paths.length;
function finish() {
if (fail == 0) onload(); else onerror();
}
paths.forEach(function(path) {
var getRequest = files.get(path);
getRequest.onsuccess = function getRequest_onsuccess() {
if (FS.analyzePath(path).exists) {
FS.unlink(path);
}
FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
ok++;
if (ok + fail == total) finish();
};
getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
});
transaction.onerror = onerror;
};
openRequest.onerror = onerror;
}};var SYSCALLS={DEFAULT_POLLMASK:5,mappings:{},umask:511,calculateAt:function (dirfd, path) {
if (path[0] !== '/') {
// relative path
var dir;
if (dirfd === -100) {
dir = FS.cwd();
} else {
var dirstream = FS.getStream(dirfd);
if (!dirstream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
dir = dirstream.path;
}
path = PATH.join2(dir, path);
}
return path;
},doStat:function (func, path, buf) {
try {
var stat = func(path);
} catch (e) {
if (e && e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
// an error occurred while trying to look up the path; we should just report ENOTDIR
return -ERRNO_CODES.ENOTDIR;
}
throw e;
}
HEAP32[((buf)>>2)]=stat.dev;
HEAP32[(((buf)+(4))>>2)]=0;
HEAP32[(((buf)+(8))>>2)]=stat.ino;
HEAP32[(((buf)+(12))>>2)]=stat.mode;
HEAP32[(((buf)+(16))>>2)]=stat.nlink;
HEAP32[(((buf)+(20))>>2)]=stat.uid;
HEAP32[(((buf)+(24))>>2)]=stat.gid;
HEAP32[(((buf)+(28))>>2)]=stat.rdev;
HEAP32[(((buf)+(32))>>2)]=0;
HEAP32[(((buf)+(36))>>2)]=stat.size;
HEAP32[(((buf)+(40))>>2)]=4096;
HEAP32[(((buf)+(44))>>2)]=stat.blocks;
HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
HEAP32[(((buf)+(52))>>2)]=0;
HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
HEAP32[(((buf)+(60))>>2)]=0;
HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
HEAP32[(((buf)+(68))>>2)]=0;
HEAP32[(((buf)+(72))>>2)]=stat.ino;
return 0;
},doMsync:function (addr, stream, len, flags) {
var buffer = new Uint8Array(HEAPU8.subarray(addr, addr + len));
FS.msync(stream, buffer, 0, len, flags);
},doMkdir:function (path, mode) {
// remove a trailing slash, if one - /a/b/ has basename of '', but
// we want to create b in the context of this function
path = PATH.normalize(path);
if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
FS.mkdir(path, mode, 0);
return 0;
},doMknod:function (path, mode, dev) {
// we don't want this in the JS API as it uses mknod to create all nodes.
switch (mode & 61440) {
case 32768:
case 8192:
case 24576:
case 4096:
case 49152:
break;
default: return -ERRNO_CODES.EINVAL;
}
FS.mknod(path, mode, dev);
return 0;
},doReadlink:function (path, buf, bufsize) {
if (bufsize <= 0) return -ERRNO_CODES.EINVAL;
var ret = FS.readlink(path);
ret = ret.slice(0, Math.max(0, bufsize));
writeStringToMemory(ret, buf, true);
return ret.length;
},doAccess:function (path, amode) {
if (amode & ~7) {
// need a valid mode
return -ERRNO_CODES.EINVAL;
}
var node;
var lookup = FS.lookupPath(path, { follow: true });
node = lookup.node;
var perms = '';
if (amode & 4) perms += 'r';
if (amode & 2) perms += 'w';
if (amode & 1) perms += 'x';
if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
return -ERRNO_CODES.EACCES;
}
return 0;
},doDup:function (path, flags, suggestFD) {
var suggest = FS.getStream(suggestFD);
if (suggest) FS.close(suggest);
return FS.open(path, flags, 0, suggestFD, suggestFD).fd;
},doReadv:function (stream, iov, iovcnt, offset) {
var ret = 0;
for (var i = 0; i < iovcnt; i++) {
var ptr = HEAP32[(((iov)+(i*8))>>2)];
var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
var curr = FS.read(stream, HEAP8,ptr, len, offset);
if (curr < 0) return -1;
ret += curr;
if (curr < len) break; // nothing more to read
}
return ret;
},doWritev:function (stream, iov, iovcnt, offset) {
var ret = 0;
for (var i = 0; i < iovcnt; i++) {
var ptr = HEAP32[(((iov)+(i*8))>>2)];
var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
var curr = FS.write(stream, HEAP8,ptr, len, offset);
if (curr < 0) return -1;
ret += curr;
}
return ret;
},varargs:0,get:function (varargs) {
SYSCALLS.varargs += 4;
var ret = HEAP32[(((SYSCALLS.varargs)-(4))>>2)];
return ret;
},getStr:function () {
var ret = Pointer_stringify(SYSCALLS.get());
return ret;
},getStreamFromFD:function () {
var stream = FS.getStream(SYSCALLS.get());
if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
return stream;
},getSocketFromFD:function () {
var socket = SOCKFS.getSocket(SYSCALLS.get());
if (!socket) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
return socket;
},getSocketAddress:function (allowNull) {
var addrp = SYSCALLS.get(), addrlen = SYSCALLS.get();
if (allowNull && addrp === 0) return null;
var info = __read_sockaddr(addrp, addrlen);
if (info.errno) throw new FS.ErrnoError(info.errno);
info.addr = DNS.lookup_addr(info.addr) || info.addr;
return info;
},get64:function () {
var low = SYSCALLS.get(), high = SYSCALLS.get();
if (low >= 0) assert(high === 0);
else assert(high === -1);
return low;
},getZero:function () {
assert(SYSCALLS.get() === 0);
}};function ___syscall63(which, varargs) {SYSCALLS.varargs = varargs;
try {
// dup2
var old = SYSCALLS.getStreamFromFD(), suggestFD = SYSCALLS.get();
if (old.fd === suggestFD) return suggestFD;
return SYSCALLS.doDup(old.path, old.flags, suggestFD);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}


Module["_memset"] = _memset;

var _BDtoILow=true;


Module["_bitshift64Shl"] = _bitshift64Shl;

function _abort() {
Module['abort']();
}

function ___syscall195(which, varargs) {SYSCALLS.varargs = varargs;
try {
// SYS_stat64
var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
return SYSCALLS.doStat(FS.stat, path, buf);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall194(which, varargs) {SYSCALLS.varargs = varargs;
try {
// ftruncate64
var fd = SYSCALLS.get(), zero = SYSCALLS.getZero(), length = SYSCALLS.get64();
FS.ftruncate(fd, length);
return 0;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall197(which, varargs) {SYSCALLS.varargs = varargs;
try {
// SYS_fstat64
var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get();
return SYSCALLS.doStat(FS.stat, stream.path, buf);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___lock() {}

function ___unlock() {}


Module["_i64Add"] = _i64Add;

var _floor=Math_floor;

var _sqrt=Math_sqrt;



function _emscripten_set_main_loop_timing(mode, value) {
Browser.mainLoop.timingMode = mode;
Browser.mainLoop.timingValue = value;

if (!Browser.mainLoop.func) {
console.error('emscripten_set_main_loop_timing: Cannot set timing mode for main loop since a main loop does not exist! Call emscripten_set_main_loop first to set one up.');
return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
}

if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setTimeout() {
setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
};
Browser.mainLoop.method = 'timeout';
} else if (mode == 1 /*EM_TIMING_RAF*/) {
Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_rAF() {
Browser.requestAnimationFrame(Browser.mainLoop.runner);
};
Browser.mainLoop.method = 'rAF';
} else if (mode == 2 /*EM_TIMING_SETIMMEDIATE*/) {
if (!window['setImmediate']) {
// Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
var setImmediates = [];
var emscriptenMainLoopMessageId = '__emcc';
function Browser_setImmediate_messageHandler(event) {
if (event.source === window && event.data === emscriptenMainLoopMessageId) {
event.stopPropagation();
setImmediates.shift()();
}
}
window.addEventListener("message", Browser_setImmediate_messageHandler, true);
window['setImmediate'] = function Browser_emulated_setImmediate(func) {
setImmediates.push(func);
window.postMessage(emscriptenMainLoopMessageId, "*");
}
}
Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setImmediate() {
window['setImmediate'](Browser.mainLoop.runner);
};
Browser.mainLoop.method = 'immediate';
}
return 0;
}function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg, noSetTiming) {
Module['noExitRuntime'] = true;

assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');

Browser.mainLoop.func = func;
Browser.mainLoop.arg = arg;

var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;

Browser.mainLoop.runner = function Browser_mainLoop_runner() {
if (ABORT) return;
if (Browser.mainLoop.queue.length > 0) {
var start = Date.now();
var blocker = Browser.mainLoop.queue.shift();
blocker.func(blocker.arg);
if (Browser.mainLoop.remainingBlockers) {
var remaining = Browser.mainLoop.remainingBlockers;
var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
if (blocker.counted) {
Browser.mainLoop.remainingBlockers = next;
} else {
// not counted, but move the progress along a tiny bit
next = next + 0.5; // do not steal all the next one's progress
Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
}
}
console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
Browser.mainLoop.updateStatus();
setTimeout(Browser.mainLoop.runner, 0);
return;
}

// catch pauses from non-main loop sources
if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;

// Implement very basic swap interval control
Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
// Not the scheduled time to render this frame - skip.
Browser.mainLoop.scheduler();
return;
}

// Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
// VBO double-buffering and reduce GPU stalls.

if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
Browser.mainLoop.method = ''; // just warn once per call to set main loop
}

Browser.mainLoop.runIter(function() {
if (typeof arg !== 'undefined') {
Runtime.dynCall('vi', func, [arg]);
} else {
Runtime.dynCall('v', func);
}
});

// catch pauses from the main loop itself
if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;

// Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
// to queue the newest produced audio samples.
// TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
//       do not need to be hardcoded into this function, but can be more generic.
if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();

Browser.mainLoop.scheduler();
}

if (!noSetTiming) {
if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)

Browser.mainLoop.scheduler();
}

if (simulateInfiniteLoop) {
throw 'SimulateInfiniteLoop';
}
}var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
Browser.mainLoop.scheduler = null;
Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
},resume:function () {
Browser.mainLoop.currentlyRunningMainloop++;
var timingMode = Browser.mainLoop.timingMode;
var timingValue = Browser.mainLoop.timingValue;
var func = Browser.mainLoop.func;
Browser.mainLoop.func = null;
_emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg, true /* do not set timing and call scheduler, we will do it on the next lines */);
_emscripten_set_main_loop_timing(timingMode, timingValue);
Browser.mainLoop.scheduler();
},updateStatus:function () {
if (Module['setStatus']) {
var message = Module['statusMessage'] || 'Please wait...';
var remaining = Browser.mainLoop.remainingBlockers;
var expected = Browser.mainLoop.expectedBlockers;
if (remaining) {
if (remaining < expected) {
Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
} else {
Module['setStatus'](message);
}
} else {
Module['setStatus']('');
}
}
},runIter:function (func) {
if (ABORT) return;
if (Module['preMainLoop']) {
var preRet = Module['preMainLoop']();
if (preRet === false) {
return; // |return false| skips a frame
}
}
try {
func();
} catch (e) {
if (e instanceof ExitStatus) {
return;
} else {
if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
throw e;
}
}
if (Module['postMainLoop']) Module['postMainLoop']();
}},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers

if (Browser.initted) return;
Browser.initted = true;

try {
new Blob();
Browser.hasBlobConstructor = true;
} catch(e) {
Browser.hasBlobConstructor = false;
console.log("warning: no blob constructor, cannot create blobs with mimetypes");
}
Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
Module.noImageDecoding = true;
}

// Support for plugins that can process preloaded files. You can add more of these to
// your app by creating and appending to Module.preloadPlugins.
//
// Each plugin is asked if it can handle a file based on the file's name. If it can,
// it is given the file's raw data. When it is done, it calls a callback with the file's
// (possibly modified) data. For example, a plugin might decompress a file, or it
// might create some side data structure for use later (like an Image element, etc.).

var imagePlugin = {};
imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
};
imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
var b = null;
if (Browser.hasBlobConstructor) {
try {
b = new Blob([byteArray], { type: Browser.getMimetype(name) });
if (b.size !== byteArray.length) { // Safari bug #118630
// Safari's Blob can only take an ArrayBuffer
b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
}
} catch(e) {
Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
}
}
if (!b) {
var bb = new Browser.BlobBuilder();
bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
b = bb.getBlob();
}
var url = Browser.URLObject.createObjectURL(b);
assert(typeof url == 'string', 'createObjectURL must return a url as a string');
var img = new Image();
img.onload = function img_onload() {
assert(img.complete, 'Image ' + name + ' could not be decoded');
var canvas = document.createElement('canvas');
canvas.width = img.width;
canvas.height = img.height;
var ctx = canvas.getContext('2d');
ctx.drawImage(img, 0, 0);
Module["preloadedImages"][name] = canvas;
Browser.URLObject.revokeObjectURL(url);
if (onload) onload(byteArray);
};
img.onerror = function img_onerror(event) {
console.log('Image ' + url + ' could not be decoded');
if (onerror) onerror();
};
img.src = url;
};
Module['preloadPlugins'].push(imagePlugin);

var audioPlugin = {};
audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
};
audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
var done = false;
function finish(audio) {
if (done) return;
done = true;
Module["preloadedAudios"][name] = audio;
if (onload) onload(byteArray);
}
function fail() {
if (done) return;
done = true;
Module["preloadedAudios"][name] = new Audio(); // empty shim
if (onerror) onerror();
}
if (Browser.hasBlobConstructor) {
try {
var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
} catch(e) {
return fail();
}
var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
assert(typeof url == 'string', 'createObjectURL must return a url as a string');
var audio = new Audio();
audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
audio.onerror = function audio_onerror(event) {
if (done) return;
console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
function encode64(data) {
var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var PAD = '=';
var ret = '';
var leftchar = 0;
var leftbits = 0;
for (var i = 0; i < data.length; i++) {
leftchar = (leftchar << 8) | data[i];
leftbits += 8;
while (leftbits >= 6) {
var curr = (leftchar >> (leftbits-6)) & 0x3f;
leftbits -= 6;
ret += BASE[curr];
}
}
if (leftbits == 2) {
ret += BASE[(leftchar&3) << 4];
ret += PAD + PAD;
} else if (leftbits == 4) {
ret += BASE[(leftchar&0xf) << 2];
ret += PAD;
}
return ret;
}
audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
finish(audio); // we don't wait for confirmation this worked - but it's worth trying
};
audio.src = url;
// workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
Browser.safeSetTimeout(function() {
finish(audio); // try to use it even though it is not necessarily ready to play
}, 10000);
} else {
return fail();
}
};
Module['preloadPlugins'].push(audioPlugin);

// Canvas event setup

var canvas = Module['canvas'];
function pointerLockChange() {
Browser.pointerLock = document['pointerLockElement'] === canvas ||
document['mozPointerLockElement'] === canvas ||
document['webkitPointerLockElement'] === canvas ||
document['msPointerLockElement'] === canvas;
}
if (canvas) {
// forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
// Module['forcedAspectRatio'] = 4 / 3;

canvas.requestPointerLock = canvas['requestPointerLock'] ||
canvas['mozRequestPointerLock'] ||
canvas['webkitRequestPointerLock'] ||
canvas['msRequestPointerLock'] ||
function(){};
canvas.exitPointerLock = document['exitPointerLock'] ||
document['mozExitPointerLock'] ||
document['webkitExitPointerLock'] ||
document['msExitPointerLock'] ||
function(){}; // no-op if function does not exist
canvas.exitPointerLock = canvas.exitPointerLock.bind(document);


document.addEventListener('pointerlockchange', pointerLockChange, false);
document.addEventListener('mozpointerlockchange', pointerLockChange, false);
document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
document.addEventListener('mspointerlockchange', pointerLockChange, false);

if (Module['elementPointerLock']) {
canvas.addEventListener("click", function(ev) {
if (!Browser.pointerLock && canvas.requestPointerLock) {
canvas.requestPointerLock();
ev.preventDefault();
}
}, false);
}
}
},createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.

var ctx;
var contextHandle;
if (useWebGL) {
// For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
var contextAttributes = {
antialias: false,
alpha: false
};

if (webGLContextAttributes) {
for (var attribute in webGLContextAttributes) {
contextAttributes[attribute] = webGLContextAttributes[attribute];
}
}

contextHandle = GL.createContext(canvas, contextAttributes);
if (contextHandle) {
ctx = GL.getContext(contextHandle).GLctx;
}
// Set the background of the WebGL canvas to black
canvas.style.backgroundColor = "black";
} else {
ctx = canvas.getContext('2d');
}

if (!ctx) return null;

if (setInModule) {
if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');

Module.ctx = ctx;
if (useWebGL) GL.makeContextCurrent(contextHandle);
Module.useWebGL = useWebGL;
Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
Browser.init();
}
return ctx;
},destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
Browser.lockPointer = lockPointer;
Browser.resizeCanvas = resizeCanvas;
Browser.vrDevice = vrDevice;
if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;

var canvas = Module['canvas'];
function fullScreenChange() {
Browser.isFullScreen = false;
var canvasContainer = canvas.parentNode;
if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
document['fullScreenElement'] || document['fullscreenElement'] ||
document['msFullScreenElement'] || document['msFullscreenElement'] ||
document['webkitCurrentFullScreenElement']) === canvasContainer) {
canvas.cancelFullScreen = document['cancelFullScreen'] ||
document['mozCancelFullScreen'] ||
document['webkitCancelFullScreen'] ||
document['msExitFullscreen'] ||
document['exitFullscreen'] ||
function() {};
canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
if (Browser.lockPointer) canvas.requestPointerLock();
Browser.isFullScreen = true;
if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
} else {

// remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
canvasContainer.parentNode.removeChild(canvasContainer);

if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
}
if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
Browser.updateCanvasDimensions(canvas);
}

if (!Browser.fullScreenHandlersInstalled) {
Browser.fullScreenHandlersInstalled = true;
document.addEventListener('fullscreenchange', fullScreenChange, false);
document.addEventListener('mozfullscreenchange', fullScreenChange, false);
document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
document.addEventListener('MSFullscreenChange', fullScreenChange, false);
}

// create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
var canvasContainer = document.createElement("div");
canvas.parentNode.insertBefore(canvasContainer, canvas);
canvasContainer.appendChild(canvas);

// use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
canvasContainer['mozRequestFullScreen'] ||
canvasContainer['msRequestFullscreen'] ||
(canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);

if (vrDevice) {
canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
} else {
canvasContainer.requestFullScreen();
}
},nextRAF:0,fakeRequestAnimationFrame:function (func) {
// try to keep 60fps between calls to here
var now = Date.now();
if (Browser.nextRAF === 0) {
Browser.nextRAF = now + 1000/60;
} else {
while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
Browser.nextRAF += 1000/60;
}
}
var delay = Math.max(Browser.nextRAF - now, 0);
setTimeout(func, delay);
},requestAnimationFrame:function requestAnimationFrame(func) {
if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
Browser.fakeRequestAnimationFrame(func);
} else {
if (!window.requestAnimationFrame) {
window.requestAnimationFrame = window['requestAnimationFrame'] ||
window['mozRequestAnimationFrame'] ||
window['webkitRequestAnimationFrame'] ||
window['msRequestAnimationFrame'] ||
window['oRequestAnimationFrame'] ||
Browser.fakeRequestAnimationFrame;
}
window.requestAnimationFrame(func);
}
},safeCallback:function (func) {
return function() {
if (!ABORT) return func.apply(null, arguments);
};
},allowAsyncCallbacks:true,queuedAsyncCallbacks:[],pauseAsyncCallbacks:function () {
Browser.allowAsyncCallbacks = false;
},resumeAsyncCallbacks:function () { // marks future callbacks as ok to execute, and synchronously runs any remaining ones right now
Browser.allowAsyncCallbacks = true;
if (Browser.queuedAsyncCallbacks.length > 0) {
var callbacks = Browser.queuedAsyncCallbacks;
Browser.queuedAsyncCallbacks = [];
callbacks.forEach(function(func) {
func();
});
}
},safeRequestAnimationFrame:function (func) {
return Browser.requestAnimationFrame(function() {
if (ABORT) return;
if (Browser.allowAsyncCallbacks) {
func();
} else {
Browser.queuedAsyncCallbacks.push(func);
}
});
},safeSetTimeout:function (func, timeout) {
Module['noExitRuntime'] = true;
return setTimeout(function() {
if (ABORT) return;
if (Browser.allowAsyncCallbacks) {
func();
} else {
Browser.queuedAsyncCallbacks.push(func);
}
}, timeout);
},safeSetInterval:function (func, timeout) {
Module['noExitRuntime'] = true;
return setInterval(function() {
if (ABORT) return;
if (Browser.allowAsyncCallbacks) {
func();
} // drop it on the floor otherwise, next interval will kick in
}, timeout);
},getMimetype:function (name) {
return {
'jpg': 'image/jpeg',
'jpeg': 'image/jpeg',
'png': 'image/png',
'bmp': 'image/bmp',
'ogg': 'audio/ogg',
'wav': 'audio/wav',
'mp3': 'audio/mpeg'
}[name.substr(name.lastIndexOf('.')+1)];
},getUserMedia:function (func) {
if(!window.getUserMedia) {
window.getUserMedia = navigator['getUserMedia'] ||
navigator['mozGetUserMedia'];
}
window.getUserMedia(func);
},getMovementX:function (event) {
return event['movementX'] ||
event['mozMovementX'] ||
event['webkitMovementX'] ||
0;
},getMovementY:function (event) {
return event['movementY'] ||
event['mozMovementY'] ||
event['webkitMovementY'] ||
0;
},getMouseWheelDelta:function (event) {
var delta = 0;
switch (event.type) {
case 'DOMMouseScroll': 
delta = event.detail;
break;
case 'mousewheel': 
delta = event.wheelDelta;
break;
case 'wheel': 
delta = event['deltaY'];
break;
default:
throw 'unrecognized mouse wheel event: ' + event.type;
}
return delta;
},mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
if (Browser.pointerLock) {
// When the pointer is locked, calculate the coordinates
// based on the movement of the mouse.
// Workaround for Firefox bug 764498
if (event.type != 'mousemove' &&
('mozMovementX' in event)) {
Browser.mouseMovementX = Browser.mouseMovementY = 0;
} else {
Browser.mouseMovementX = Browser.getMovementX(event);
Browser.mouseMovementY = Browser.getMovementY(event);
}

// check if SDL is available
if (typeof SDL != "undefined") {
	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
} else {
	// just add the mouse delta to the current absolut mouse position
	// FIXME: ideally this should be clamped against the canvas size and zero
	Browser.mouseX += Browser.mouseMovementX;
	Browser.mouseY += Browser.mouseMovementY;
}        
} else {
// Otherwise, calculate the movement based on the changes
// in the coordinates.
var rect = Module["canvas"].getBoundingClientRect();
var cw = Module["canvas"].width;
var ch = Module["canvas"].height;

// Neither .scrollX or .pageXOffset are defined in a spec, but
// we prefer .scrollX because it is currently in a spec draft.
// (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
// If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
// and we have no viable fallback.
assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');

if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
var touch = event.touch;
if (touch === undefined) {
return; // the "touch" property is only defined in SDL

}
var adjustedX = touch.pageX - (scrollX + rect.left);
var adjustedY = touch.pageY - (scrollY + rect.top);

adjustedX = adjustedX * (cw / rect.width);
adjustedY = adjustedY * (ch / rect.height);

var coords = { x: adjustedX, y: adjustedY };

if (event.type === 'touchstart') {
Browser.lastTouches[touch.identifier] = coords;
Browser.touches[touch.identifier] = coords;
} else if (event.type === 'touchend' || event.type === 'touchmove') {
var last = Browser.touches[touch.identifier];
if (!last) last = coords;
Browser.lastTouches[touch.identifier] = last;
Browser.touches[touch.identifier] = coords;
} 
return;
}

var x = event.pageX - (scrollX + rect.left);
var y = event.pageY - (scrollY + rect.top);

// the canvas might be CSS-scaled compared to its backbuffer;
// SDL-using content will want mouse coordinates in terms
// of backbuffer units.
x = x * (cw / rect.width);
y = y * (ch / rect.height);

Browser.mouseMovementX = x - Browser.mouseX;
Browser.mouseMovementY = y - Browser.mouseY;
Browser.mouseX = x;
Browser.mouseY = y;
}
},xhrLoad:function (url, onload, onerror) {
var xhr = new XMLHttpRequest();
xhr.open('GET', url, true);
xhr.responseType = 'arraybuffer';
xhr.onload = function xhr_onload() {
if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
onload(xhr.response);
} else {
onerror();
}
};
xhr.onerror = onerror;
xhr.send(null);
},asyncLoad:function (url, onload, onerror, noRunDep) {
Browser.xhrLoad(url, function(arrayBuffer) {
assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
onload(new Uint8Array(arrayBuffer));
if (!noRunDep) removeRunDependency('al ' + url);
}, function(event) {
if (onerror) {
onerror();
} else {
throw 'Loading data file "' + url + '" failed.';
}
});
if (!noRunDep) addRunDependency('al ' + url);
},resizeListeners:[],updateResizeListeners:function () {
var canvas = Module['canvas'];
Browser.resizeListeners.forEach(function(listener) {
listener(canvas.width, canvas.height);
});
},setCanvasSize:function (width, height, noUpdates) {
var canvas = Module['canvas'];
Browser.updateCanvasDimensions(canvas, width, height);
if (!noUpdates) Browser.updateResizeListeners();
},windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
// check if SDL is available   
if (typeof SDL != "undefined") {
	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
}
Browser.updateResizeListeners();
},setWindowedCanvasSize:function () {
// check if SDL is available       
if (typeof SDL != "undefined") {
	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
}
Browser.updateResizeListeners();
},updateCanvasDimensions:function (canvas, wNative, hNative) {
if (wNative && hNative) {
canvas.widthNative = wNative;
canvas.heightNative = hNative;
} else {
wNative = canvas.widthNative;
hNative = canvas.heightNative;
}
var w = wNative;
var h = hNative;
if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
if (w/h < Module['forcedAspectRatio']) {
w = Math.round(h * Module['forcedAspectRatio']);
} else {
h = Math.round(w / Module['forcedAspectRatio']);
}
}
if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
document['fullScreenElement'] || document['fullscreenElement'] ||
document['msFullScreenElement'] || document['msFullscreenElement'] ||
document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
var factor = Math.min(screen.width / w, screen.height / h);
w = Math.round(w * factor);
h = Math.round(h * factor);
}
if (Browser.resizeCanvas) {
if (canvas.width  != w) canvas.width  = w;
if (canvas.height != h) canvas.height = h;
if (typeof canvas.style != 'undefined') {
canvas.style.removeProperty( "width");
canvas.style.removeProperty("height");
}
} else {
if (canvas.width  != wNative) canvas.width  = wNative;
if (canvas.height != hNative) canvas.height = hNative;
if (typeof canvas.style != 'undefined') {
if (w != wNative || h != hNative) {
canvas.style.setProperty( "width", w + "px", "important");
canvas.style.setProperty("height", h + "px", "important");
} else {
canvas.style.removeProperty( "width");
canvas.style.removeProperty("height");
}
}
}
},wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
var handle = Browser.nextWgetRequestHandle;
Browser.nextWgetRequestHandle++;
return handle;
}};

function ___syscall330(which, varargs) {SYSCALLS.varargs = varargs;
try {
// dup3
var old = SYSCALLS.getStreamFromFD(), suggestFD = SYSCALLS.get(), flags = SYSCALLS.get();
assert(!flags);
if (old.fd === suggestFD) return -ERRNO_CODES.EINVAL;
return SYSCALLS.doDup(old.path, old.flags, suggestFD);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}


function __exit(status) {
// void _exit(int status);
// http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
Module['exit'](status);
}function _exit(status) {
__exit(status);
}

function ___syscall6(which, varargs) {SYSCALLS.varargs = varargs;
try {
// close
var stream = SYSCALLS.getStreamFromFD();
FS.close(stream);
return 0;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall54(which, varargs) {SYSCALLS.varargs = varargs;
try {
// ioctl
var stream = SYSCALLS.getStreamFromFD(), op = SYSCALLS.get();
switch (op) {
case 21505: {
if (!stream.tty) return -ERRNO_CODES.ENOTTY;
return 0;
}
case 21506: {
if (!stream.tty) return -ERRNO_CODES.ENOTTY;
return 0; // no-op, not actually adjusting terminal settings
}
case 21519: {
if (!stream.tty) return -ERRNO_CODES.ENOTTY;
var argp = SYSCALLS.get();
HEAP32[((argp)>>2)]=0;
return 0;
}
case 21520: {
if (!stream.tty) return -ERRNO_CODES.ENOTTY;
return -ERRNO_CODES.EINVAL; // not supported
}
case 21531: {
var argp = SYSCALLS.get();
return FS.ioctl(stream, op, argp);
}
default: abort('bad ioctl syscall ' + op);
}
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}


Module["_bitshift64Lshr"] = _bitshift64Lshr;

function ___syscall33(which, varargs) {SYSCALLS.varargs = varargs;
try {
// access
var path = SYSCALLS.getStr(), amode = SYSCALLS.get();
return SYSCALLS.doAccess(path, amode);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

var _BDtoIHigh=true;

function _pthread_cleanup_push(routine, arg) {
__ATEXIT__.push(function() { Runtime.dynCall('vi', routine, [arg]) })
_pthread_cleanup_push.level = __ATEXIT__.length;
}

function ___syscall10(which, varargs) {SYSCALLS.varargs = varargs;
try {
// unlink
var path = SYSCALLS.getStr();
FS.unlink(path);
return 0;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function _pthread_cleanup_pop() {
assert(_pthread_cleanup_push.level == __ATEXIT__.length, 'cannot pop if something else added meanwhile!');
__ATEXIT__.pop();
_pthread_cleanup_push.level = __ATEXIT__.length;
}

function ___syscall5(which, varargs) {SYSCALLS.varargs = varargs;
try {
// open
var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get() // optional TODO
var stream = FS.open(pathname, flags, mode);
return stream.fd;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}


function _emscripten_memcpy_big(dest, src, num) {
HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
return dest;
} 
Module["_memcpy"] = _memcpy;

var _log=Math_log;

function _sbrk(bytes) {
// Implement a Linux-like 'memory area' for our 'process'.
// Changes the size of the memory area by |bytes|; returns the
// address of the previous top ('break') of the memory area
// We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
var self = _sbrk;
if (!self.called) {
DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
self.called = true;
assert(Runtime.dynamicAlloc);
self.alloc = Runtime.dynamicAlloc;
Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
}
var ret = DYNAMICTOP;
if (bytes != 0) {
var success = self.alloc(bytes);
if (!success) return -1 >>> 0; // sbrk failure code
}
return ret;  // Previous break location.
}


var __sigalrm_handler=0;function _signal(sig, func) {
if (sig == 14 /*SIGALRM*/) {
__sigalrm_handler = func;
} else {
Module.printErr('Calling stub instead of signal()');
}
return 0;
}

var _BItoD=true;

function ___syscall265(which, varargs) {SYSCALLS.varargs = varargs;
try {
// clock_nanosleep
return 0;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function _time(ptr) {
var ret = (Date.now()/1000)|0;
if (ptr) {
HEAP32[((ptr)>>2)]=ret;
}
return ret;
}

function _pthread_self() {
//FIXME: assumes only a single thread
return 0;
}

function ___syscall140(which, varargs) {SYSCALLS.varargs = varargs;
try {
// llseek
var stream = SYSCALLS.getStreamFromFD(), offset_high = SYSCALLS.get(), offset_low = SYSCALLS.get(), result = SYSCALLS.get(), whence = SYSCALLS.get();
var offset = offset_low;
assert(offset_high === 0);
FS.llseek(stream, offset, whence);
HEAP32[((result)>>2)]=stream.position;
if (stream.getdents && offset === 0 && whence === 0) stream.getdents = null; // reset readdir state
return 0;
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall146(which, varargs) {SYSCALLS.varargs = varargs;
try {
// writev
var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
return SYSCALLS.doWritev(stream, iov, iovcnt);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall221(which, varargs) {SYSCALLS.varargs = varargs;
try {
// fcntl64
var stream = SYSCALLS.getStreamFromFD(), cmd = SYSCALLS.get();
switch (cmd) {
case 0: {
var arg = SYSCALLS.get();
if (arg < 0) {
return -ERRNO_CODES.EINVAL;
}
var newStream;
newStream = FS.open(stream.path, stream.flags, 0, arg);
return newStream.fd;
}
case 1:
case 2:
return 0;  // FD_CLOEXEC makes no sense for a single process.
case 3:
return stream.flags;
case 4: {
var arg = SYSCALLS.get();
stream.flags |= arg;
return 0;
}
case 12:
case 12: {
var arg = SYSCALLS.get();
var offset = 0;
// We're always unlocked.
HEAP16[(((arg)+(offset))>>1)]=2;
return 0;
}
case 13:
case 14:
case 13:
case 14:
return 0; // Pretend that the locking is successful.
case 16:
case 8:
return -ERRNO_CODES.EINVAL; // These are for sockets. We don't have them fully implemented yet.
case 9:
// musl trusts getown return values, due to a bug where they must be, as they overlap with errors. just return -1 here, so fnctl() returns that, and we set errno ourselves.
___setErrNo(ERRNO_CODES.EINVAL);
return -1;
default: {
return -ERRNO_CODES.EINVAL;
}
}
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}

function ___syscall145(which, varargs) {SYSCALLS.varargs = varargs;
try {
// readv
var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
return SYSCALLS.doReadv(stream, iov, iovcnt);
} catch (e) {
if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
return -e.errno;
}
}
FS.staticInit();__ATINIT__.unshift(function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() });__ATMAIN__.push(function() { FS.ignorePermissions = false });__ATEXIT__.push(function() { FS.quit() });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;Module["FS_unlink"] = FS.unlink;
__ATINIT__.unshift(function() { TTY.init() });__ATEXIT__.push(function() { TTY.shutdown() });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function nullFunc_i(x) { Module["printErr"]("Invalid function pointer called with signature 'i'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_ii(x) { Module["printErr"]("Invalid function pointer called with signature 'ii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_iiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_vi(x) { Module["printErr"]("Invalid function pointer called with signature 'vi'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function invoke_i(index) {
try {
return Module["dynCall_i"](index);
} catch(e) {
if (typeof e !== 'number' && e !== 'longjmp') throw e;
asm["setThrew"](1, 0);
}
}

function invoke_ii(index,a1) {
try {
return Module["dynCall_ii"](index,a1);
} catch(e) {
if (typeof e !== 'number' && e !== 'longjmp') throw e;
asm["setThrew"](1, 0);
}
}

function invoke_iiii(index,a1,a2,a3) {
try {
return Module["dynCall_iiii"](index,a1,a2,a3);
} catch(e) {
if (typeof e !== 'number' && e !== 'longjmp') throw e;
asm["setThrew"](1, 0);
}
}

function invoke_iiiii(index,a1,a2,a3,a4) {
try {
return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
} catch(e) {
if (typeof e !== 'number' && e !== 'longjmp') throw e;
asm["setThrew"](1, 0);
}
}

function invoke_vi(index,a1) {
try {
Module["dynCall_vi"](index,a1);
} catch(e) {
if (typeof e !== 'number' && e !== 'longjmp') throw e;
asm["setThrew"](1, 0);
}
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };

Module.asmLibraryArg = { "abort": abort, "assert": assert, "nullFunc_i": nullFunc_i, "nullFunc_ii": nullFunc_ii, "nullFunc_iiii": nullFunc_iiii, "nullFunc_iiiii": nullFunc_iiiii, "nullFunc_vi": nullFunc_vi, "invoke_i": invoke_i, "invoke_ii": invoke_ii, "invoke_iiii": invoke_iiii, "invoke_iiiii": invoke_iiiii, "invoke_vi": invoke_vi, "_pthread_cleanup_pop": _pthread_cleanup_pop, "___syscall221": ___syscall221, "___syscall265": ___syscall265, "_log": _log, "___syscall63": ___syscall63, "___syscall6": ___syscall6, "___setErrNo": ___setErrNo, "_floor": _floor, "_signal": _signal, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "___syscall330": ___syscall330, "___syscall197": ___syscall197, "_emscripten_memcpy_big": _emscripten_memcpy_big, "___syscall194": ___syscall194, "__exit": __exit, "___syscall195": ___syscall195, "_pthread_self": _pthread_self, "_sqrt": _sqrt, "___syscall33": ___syscall33, "___syscall54": ___syscall54, "___unlock": ___unlock, "_emscripten_set_main_loop": _emscripten_set_main_loop, "___syscall10": ___syscall10, "_sysconf": _sysconf, "___lock": ___lock, "_abort": _abort, "___syscall5": ___syscall5, "_time": _time, "___syscall140": ___syscall140, "_exit": _exit, "_pthread_cleanup_push": _pthread_cleanup_push, "___syscall145": ___syscall145, "___syscall146": ___syscall146, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8 };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
'almost asm';


var HEAP8 = new global.Int8Array(buffer);
var HEAP16 = new global.Int16Array(buffer);
var HEAP32 = new global.Int32Array(buffer);
var HEAPU8 = new global.Uint8Array(buffer);
var HEAPU16 = new global.Uint16Array(buffer);
var HEAPU32 = new global.Uint32Array(buffer);
var HEAPF32 = new global.Float32Array(buffer);
var HEAPF64 = new global.Float64Array(buffer);


var STACKTOP=env.STACKTOP|0;
var STACK_MAX=env.STACK_MAX|0;
var tempDoublePtr=env.tempDoublePtr|0;
var ABORT=env.ABORT|0;
var cttz_i8=env.cttz_i8|0;

var __THREW__ = 0;
var threwValue = 0;
var setjmpId = 0;
var undef = 0;
var nan = global.NaN, inf = global.Infinity;
var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

var tempRet0 = 0;
var tempRet1 = 0;
var tempRet2 = 0;
var tempRet3 = 0;
var tempRet4 = 0;
var tempRet5 = 0;
var tempRet6 = 0;
var tempRet7 = 0;
var tempRet8 = 0;
var tempRet9 = 0;
var Math_floor=global.Math.floor;
var Math_abs=global.Math.abs;
var Math_sqrt=global.Math.sqrt;
var Math_pow=global.Math.pow;
var Math_cos=global.Math.cos;
var Math_sin=global.Math.sin;
var Math_tan=global.Math.tan;
var Math_acos=global.Math.acos;
var Math_asin=global.Math.asin;
var Math_atan=global.Math.atan;
var Math_atan2=global.Math.atan2;
var Math_exp=global.Math.exp;
var Math_log=global.Math.log;
var Math_ceil=global.Math.ceil;
var Math_imul=global.Math.imul;
var Math_min=global.Math.min;
var Math_clz32=global.Math.clz32;
var abort=env.abort;
var assert=env.assert;
var nullFunc_i=env.nullFunc_i;
var nullFunc_ii=env.nullFunc_ii;
var nullFunc_iiii=env.nullFunc_iiii;
var nullFunc_iiiii=env.nullFunc_iiiii;
var nullFunc_vi=env.nullFunc_vi;
var invoke_i=env.invoke_i;
var invoke_ii=env.invoke_ii;
var invoke_iiii=env.invoke_iiii;
var invoke_iiiii=env.invoke_iiiii;
var invoke_vi=env.invoke_vi;
var _pthread_cleanup_pop=env._pthread_cleanup_pop;
var ___syscall221=env.___syscall221;
var ___syscall265=env.___syscall265;
var _log=env._log;
var ___syscall63=env.___syscall63;
var ___syscall6=env.___syscall6;
var ___setErrNo=env.___setErrNo;
var _floor=env._floor;
var _signal=env._signal;
var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
var _sbrk=env._sbrk;
var ___syscall330=env.___syscall330;
var ___syscall197=env.___syscall197;
var _emscripten_memcpy_big=env._emscripten_memcpy_big;
var ___syscall194=env.___syscall194;
var __exit=env.__exit;
var ___syscall195=env.___syscall195;
var _pthread_self=env._pthread_self;
var _sqrt=env._sqrt;
var ___syscall33=env.___syscall33;
var ___syscall54=env.___syscall54;
var ___unlock=env.___unlock;
var _emscripten_set_main_loop=env._emscripten_set_main_loop;
var ___syscall10=env.___syscall10;
var _sysconf=env._sysconf;
var ___lock=env.___lock;
var _abort=env._abort;
var ___syscall5=env.___syscall5;
var _time=env._time;
var ___syscall140=env.___syscall140;
var _exit=env._exit;
var _pthread_cleanup_push=env._pthread_cleanup_push;
var ___syscall145=env.___syscall145;
var ___syscall146=env.___syscall146;
var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
size = size|0;
var ret = 0;
ret = STACKTOP;
STACKTOP = (STACKTOP + size)|0;
STACKTOP = (STACKTOP + 15)&-16;
if ((STACKTOP|0) >= (STACK_MAX|0)) abort();

return ret|0;
}
function stackSave() {
return STACKTOP|0;
}
function stackRestore(top) {
top = top|0;
STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
stackBase = stackBase|0;
stackMax = stackMax|0;
STACKTOP = stackBase;
STACK_MAX = stackMax;
}

function setThrew(threw, value) {
threw = threw|0;
value = value|0;
if ((__THREW__|0) == 0) {
__THREW__ = threw;
threwValue = value;
}
}
function copyTempFloat(ptr) {
ptr = ptr|0;
HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
ptr = ptr|0;
HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}

function setTempRet0(value) {
value = value|0;
tempRet0 = value;
}
function getTempRet0() {
return tempRet0|0;
}

    function _setulb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0;var E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0;return Rs=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),E=e,S=t,Cn=n,mi=r,ki=i,Fi=s,Ki=o,os=u,gs=a,Ls=f,x=l,P=c,X=h,nt=p,pt=d,Tt=v,Ht=m,Vt=g,rn=y,vn=b,kn=w,Fn=P,Kn=Fn+ -4|0,P=Kn,or=os,gr=or+ -8|0,os=gr,Lr=Fi,Ir=Lr+ -4|0,Fi=Ir,Qr=ki,ui=Qr+ -8|0,ki=ui,vi=mi,gi=vi+ -8|0,mi=gi,yi=Cn,bi=yi+ -8|0,Cn=bi,wi=x,Ei=wi+ -8|0,x=Ei,Si=Tt,xi=Si+ -4|0,Tt=xi,Ti=Ht,Ni=Ti+ -4|0,Ht=Ni,Ci=Vt,Li=Ci+ -8|0,Vt=Li,Ai=X,Oi=_s_cmp(Ai,13740,60,5)|0,Mi=(Oi|0)==0,Mi&&(_i=S,Di=HEAP32[_i>>2]|0,Pi=E,Hi=HEAP32[Pi>>2]|0,Bi=Math_imul(Di,Hi)|0,ji=Ht,Ii=ji+4|0,HEAP32[Ii>>2]=Bi,qi=S,Ri=HEAP32[qi>>2]|0,Is=Ri,Ui=Is,zi=Is,Wi=Math_imul(Ui,zi)|0,Xi=Ht,Vi=Xi+8|0,HEAP32[Vi>>2]=Wi,$i=S,Ji=HEAP32[$i>>2]|0,Is=Ji,Qi=Is,Gi=Is,Yi=Math_imul(Qi,Gi)|0,Zi=Yi<<2,es=Ht,ts=es+12|0,HEAP32[ts>>2]=Zi,ns=Ht,rs=ns+16|0,HEAP32[rs>>2]=1,is=Ht,ss=is+16|0,us=HEAP32[ss>>2]|0,as=Ht,fs=as+4|0,ls=HEAP32[fs>>2]|0,cs=us+ls|0,hs=Ht,ps=hs+20|0,HEAP32[ps>>2]=cs,ds=Ht,vs=ds+20|0,ms=HEAP32[vs>>2]|0,ys=Ht,bs=ys+4|0,ws=HEAP32[bs>>2]|0,Es=ms+ws|0,Ss=Ht,xs=Ss+24|0,HEAP32[xs>>2]=Es,Ts=Ht,Ns=Ts+24|0,Cs=HEAP32[Ns>>2]|0,ks=Ht,As=ks+8|0,Os=HEAP32[As>>2]|0,Ms=Cs+Os|0,_s=Ht,Ds=_s+28|0,HEAP32[Ds>>2]=Ms,Ps=Ht,Hs=Ps+28|0,Bs=HEAP32[Hs>>2]|0,js=Ht,Fs=js+8|0,T=HEAP32[Fs>>2]|0,N=Bs+T|0,C=Ht,k=C+32|0,HEAP32[k>>2]=N,L=Ht,A=L+32|0,O=HEAP32[A>>2]|0,M=Ht,_=M+8|0,D=HEAP32[_>>2]|0,H=O+D|0,B=Ht,j=B+36|0,HEAP32[j>>2]=H,F=Ht,I=F+36|0,q=HEAP32[I>>2]|0,R=Ht,U=R+12|0,z=HEAP32[U>>2]|0,W=q+z|0,V=Ht,$=V+40|0,HEAP32[$>>2]=W,J=Ht,K=J+40|0,Q=HEAP32[K>>2]|0,G=Ht,Y=G+12|0,Z=HEAP32[Y>>2]|0,et=Q+Z|0,tt=Ht,rt=tt+44|0,HEAP32[rt>>2]=et,it=Ht,st=it+44|0,ot=HEAP32[st>>2]|0,ut=E,at=HEAP32[ut>>2]|0,ft=ot+at|0,lt=Ht,ct=lt+48|0,HEAP32[ct>>2]=ft,ht=Ht,dt=ht+48|0,vt=HEAP32[dt>>2]|0,mt=E,gt=HEAP32[mt>>2]|0,yt=vt+gt|0,bt=Ht,wt=bt+52|0,HEAP32[wt>>2]=yt,Et=Ht,St=Et+52|0,xt=HEAP32[St>>2]|0,Nt=E,Ct=HEAP32[Nt>>2]|0,kt=xt+Ct|0,Lt=Ht,At=Lt+56|0,HEAP32[At>>2]=kt,Ot=Ht,Mt=Ot+56|0,_t=HEAP32[Mt>>2]|0,Dt=E,Pt=HEAP32[Dt>>2]|0,Bt=_t+Pt|0,jt=Ht,Ft=jt+60|0,HEAP32[Ft>>2]=Bt,It=Ht,qt=It+60|0,Rt=HEAP32[qt>>2]|0,Ut=E,zt=HEAP32[Ut>>2]|0,Wt=Rt+zt|0,Xt=Ht,$t=Xt+64|0,HEAP32[$t>>2]=Wt),Jt=Ht,Kt=Jt+16|0,Qt=HEAP32[Kt>>2]|0,HEAP32[240]=Qt,Gt=Ht,Yt=Gt+20|0,Zt=HEAP32[Yt>>2]|0,HEAP32[243]=Zt,en=Ht,tn=en+24|0,nn=HEAP32[tn>>2]|0,HEAP32[242]=nn,sn=Ht,on=sn+28|0,un=HEAP32[on>>2]|0,HEAP32[238]=un,an=Ht,fn=an+32|0,ln=HEAP32[fn>>2]|0,HEAP32[241]=ln,cn=Ht,hn=cn+36|0,pn=HEAP32[hn>>2]|0,HEAP32[237]=pn,dn=Ht,mn=dn+40|0,gn=HEAP32[mn>>2]|0,HEAP32[244]=gn,yn=Ht,bn=yn+44|0,wn=HEAP32[bn>>2]|0,HEAP32[235]=wn,En=Ht,Sn=En+48|0,xn=HEAP32[Sn>>2]|0,HEAP32[233]=xn,Tn=Ht,Nn=Tn+52|0,Ln=HEAP32[Nn>>2]|0,HEAP32[232]=Ln,An=Ht,On=An+56|0,Mn=HEAP32[On>>2]|0,HEAP32[234]=Mn,_n=Ht,Dn=_n+60|0,Pn=HEAP32[Dn>>2]|0,HEAP32[239]=Pn,Hn=Ht,Bn=Hn+64|0,jn=HEAP32[Bn>>2]|0,HEAP32[236]=jn,In=E,qn=S,Rn=Cn,Un=Rn+8|0,zn=mi,Wn=zn+8|0,Xn=ki,Vn=Xn+8|0,$n=Fi,Jn=$n+4|0,Qn=Ki,Gn=os,Yn=Gn+8|0,Zn=gs,er=Ls,tr=HEAP32[240]|0,nr=x,rr=nr+(tr<<3)|0,ir=HEAP32[243]|0,sr=x,ur=sr+(ir<<3)|0,ar=HEAP32[242]|0,fr=x,lr=fr+(ar<<3)|0,cr=HEAP32[238]|0,hr=x,pr=hr+(cr<<3)|0,dr=HEAP32[241]|0,vr=x,mr=vr+(dr<<3)|0,yr=HEAP32[237]|0,br=x,wr=br+(yr<<3)|0,Er=HEAP32[244]|0,Sr=x,xr=Sr+(Er<<3)|0,Tr=HEAP32[235]|0,Nr=x,Cr=Nr+(Tr<<3)|0,kr=HEAP32[233]|0,Ar=x,Or=Ar+(kr<<3)|0,Mr=HEAP32[232]|0,_r=x,Dr=_r+(Mr<<3)|0,Pr=HEAP32[234]|0,Hr=x,Br=Hr+(Pr<<3)|0,jr=HEAP32[239]|0,Fr=x,qr=Fr+(jr<<3)|0,Rr=HEAP32[236]|0,Ur=x,zr=Ur+(Rr<<3)|0,Wr=P,Xr=Wr+4|0,Vr=E,$r=HEAP32[Vr>>2]|0,Jr=$r+1|0,Kr=P,Gr=Kr+(Jr<<2)|0,Yr=E,Zr=HEAP32[Yr>>2]|0,ei=Zr<<1,ti=ei+1|0,ni=P,ri=ni+(ti<<2)|0,ii=X,si=nt,oi=pt,ai=Tt,fi=ai+4|0,li=Ht,ci=li+88|0,hi=Vt,pi=hi+8|0,di=rn,_mainlb_(In,qn,Un,Wn,Vn,Jn,Qn,Yn,Zn,er,rr,ur,lr,pr,mr,wr,xr,Cr,Or,Dr,Br,qr,zr,Xr,Gr,ri,ii,si,oi,fi,ci,pi,di,60,60)|0,STACKTOP=Rs,0}function _mainlb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x,T,N,C,k,L,A,O,M,_,D,P){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0,E|=0,S|=0,x|=0,T|=0,N|=0,C|=0,k|=0,L|=0,A|=0,O|=0,M|=0,_|=0,D|=0,P|=0;var H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0,Rd=0,Ud=0,zd=0,Wd=0,Xd=0,Vd=0,$d=0,Jd=0,Kd=0,Qd=0,Gd=0,Yd=0,Zd=0,ev=0,tv=0,nv=0,rv=0,iv=0,sv=0,ov=0,uv=0,av=0,fv=0,lv=0,cv=0,hv=0,pv=0,dv=0,vv=0,mv=0,gv=0,yv=0,bv=0,wv=0,Ev=0,Sv=0,xv=0,Tv=0,Nv=0,Cv=0,kv=0,Lv=0,Av=0,Ov=0,Mv=0,_v=0,Dv=0,Pv=0,Hv=0,Bv=0,jv=0,Fv=0,Iv=0,qv=0,Rv=0,Uv=0,zv=0,Wv=0,Xv=0,Vv=0,$v=0,Jv=0,Kv=0,Qv=0,Gv=0,Yv=0,Zv=0,em=0,tm=0,nm=0,rm=0,im=0,sm=0,om=0,um=0,am=0,fm=0,lm=0,cm=0,hm=0,pm=0,dm=0,vm=0,mm=0,gm=0,ym=0,bm=0,wm=0,Em=0,Sm=0,xm=0,Tm=0,Nm=0,Cm=0,km=0,Lm=0,Am=0,Om=0,Mm=0,_m=0,Dm=0,Pm=0,Hm=0,Bm=0,jm=0,Fm=0,Im=0,qm=0,Rm=0,Um=0,zm=0,Wm=0,Xm=0,Vm=0,$m=0,Jm=0,Km=0,Qm=0,Gm=0,Ym=0,Zm=0,eg=0,tg=0,ng=0,rg=0,ig=0,sg=0,og=0,ug=0,ag=0,fg=0,lg=0,cg=0,hg=0,pg=0,dg=0,vg=0,mg=0,gg=0,yg=0,bg=0,wg=0,Eg=0,Sg=0,xg=0,Tg=0,Ng=0,Cg=0,kg=0,Lg=0,Ag=0,Og=0,Mg=0,_g=0,Dg=0,Pg=0,Hg=0,Bg=0,jg=0,Fg=0,Ig=0,qg=0,Rg=0,Ug=0,zg=0,Wg=0,Xg=0,Vg=0,$g=0,Jg=0,Kg=0,Qg=0,Gg=0,Yg=0,Zg=0;Zg=STACKTOP,STACKTOP=STACKTOP+224|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Hg=Zg+16|0,B=e,qn=t,zi=n,$o=r,Qa=i,ec=s,rp=o,ov=u,fg=a,j=f,J=l,st=c,mt=h,kt=p,Ft=d,Kt=v,un=m,yn=g,An=y,Rn=b,Yn=w,fr=E,wr=S,Mr=x,Ur=T,Zr=N,li=C,Ei=k,_i=L,Wi=A,ts=O,hs=M,xs=_,Ps=D,Xs=P,no=Zr,po=no+ -4|0,Zr=po,No=Ur,Bo=No+ -4|0,Ur=Bo,Jo=Mr,su=Jo+ -4|0,Mr=su,mu=fr,ku=mu+ -8|0,fr=ku,Fu=Yn,Ku=Fu+ -8|0,Yn=Ku,oa=Rn,ga=oa+ -8|0,Rn=ga,La=An,Ia=La+ -8|0,An=Ia,Ga=yn,ff=Ga+ -8|0,yn=ff,wf=ov,Mf=wf+ -8|0,ov=Mf,Uf=ec,Zf=Uf+ -4|0,ec=Zf,ll=Qa,El=ll+ -8|0,Qa=El,_l=$o,zl=_l+ -8|0,$o=zl,tc=zi,hc=tc+ -8|0,zi=hc,xc=wr,Pc=xc+ -8|0,wr=Pc,Xc=qn,nh=HEAP32[Xc>>2]|0,ph=nh<<1,Fg=ph,Th=Fg,Hh=1+Th|0,Ig=Hh,Vh=Ig,ip=un,vp=0-Vh|0,Cp=ip+(vp<<3)|0,un=Cp,jp=qn,Jp=HEAP32[jp>>2]|0,sd=Jp<<1,Wg=sd,md=Wg,kd=1+md|0,Xg=kd,Fd=Xg,Kd=Kt,uv=0-Fd|0,yv=Kd+(uv<<3)|0,Kt=yv,Av=qn,qv=HEAP32[Av>>2]|0,Jg=qv,Gv=Jg,am=1+Gv|0,Kg=am,bm=Kg,Om=Ft,Rm=0-bm|0,Ym=Om+(Rm<<3)|0,Ft=Ym,lg=qn,Eg=HEAP32[lg>>2]|0,qg=Eg,Ng=qg,Cg=1+Ng|0,Rg=Cg,kg=Rg,Lg=kt,Ag=0-kg|0,Og=Lg+(Ag<<3)|0,kt=Og,Mg=qn,_g=HEAP32[Mg>>2]|0,Ug=_g,F=Ug,I=1+F|0,zg=I,q=zg,R=mt,U=0-q|0,z=R+(U<<3)|0,mt=z,W=B,X=HEAP32[W>>2]|0,Qg=X,V=Qg,$=1+V|0,Gg=$,K=Gg,Q=st,G=0-K|0,Y=Q+(G<<3)|0,st=Y,Z=B,et=HEAP32[Z>>2]|0,Vg=et,tt=Vg,nt=1+tt|0,$g=nt,rt=$g,it=J,ot=0-rt|0,ut=it+(ot<<3)|0,J=ut,at=Wi,ft=at+ -4|0,Wi=ft,lt=ts,ct=lt+ -4|0,ts=ct,ht=hs,pt=ht+ -8|0,hs=pt,dt=li,vt=_s_cmp(dt,13740,60,5)|0,gt=(vt|0)==0;e:do{if(gt){yt=+_dlamch_(14350),bt=yt*2,HEAPF64[20]=bt,_timer_(96)|0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,HEAP32[256]=0,HEAP32[259]=0,HEAP32[261]=0,HEAP32[250]=0,HEAP32[262]=0,HEAP32[267]=0,HEAPF64[9]=0,HEAPF64[16]=0,HEAPF64[7]=0,HEAPF64[1]=0,HEAPF64[18]=0,HEAPF64[22]=0,HEAPF64[6]=0,HEAPF64[14]=0,HEAPF64[4]=0,HEAP32[255]=0,HEAP32[253]=0,HEAP32[252]=0,HEAP32[268]=0,HEAP32[260]=0,wt=B,Et=HEAP32[wt>>2]|0,HEAP32[257]=Et,HEAP32[254]=0,St=fg,xt=+HEAPF64[St>>3],Tt=+HEAPF64[20],Nt=xt*Tt,HEAPF64[5]=Nt,HEAPF64[19]=0,HEAPF64[21]=0,HEAPF64[23]=0,_s_copy(14347,14352,3,3),HEAP32[251]=0,Ct=B,Lt=qn,At=fg,Ot=$o,Mt=Ot+8|0,_t=Qa,Dt=_t+8|0,Pt=ec,Ht=Pt+4|0,Bt=li,_errclb_(Ct,Lt,At,Mt,Dt,Ht,Bt,1004,984,60)|0,jt=li,It=_s_cmp(jt,14356,5,5)|0,qt=(It|0)==0,Rt=B;if(!qt){$t=qn,Jt=$o,Qt=Jt+8|0,Gt=Qa,Yt=Gt+8|0,Zt=zi,en=Zt+8|0,tn=Ei,_prn1lb_(Rt,$t,Qt,Yt,en,tn,160)|0,nn=B,rn=$o,sn=rn+8|0,on=Qa,an=on+8|0,fn=ec,ln=fn+4|0,cn=zi,hn=cn+8|0,pn=Ur,dn=pn+4|0,vn=Ei,_active_(nn,sn,an,ln,hn,dn,vn,1056,1064,1032)|0,Yg=11;break}return Ut=zi,zt=Ut+8|0,Wt=rp,Xt=li,Vt=Ei,_prn3lb_(Rt,zt,Wt,Xt,Vt,1004,1020,1012,1072,1040,1e3,176,192,1008,14347,1024,48,136,984,152,168,184,60,3)|0,H=0,Tg=H,STACKTOP=Zg,Tg|0}mn=Wi,gn=mn+4|0,bn=HEAP32[gn>>2]|0,HEAP32[264]=bn,wn=Wi,En=wn+8|0,Sn=HEAP32[En>>2]|0,HEAP32[266]=Sn,xn=Wi,Tn=xn+12|0,Nn=HEAP32[Tn>>2]|0,HEAP32[258]=Nn,Cn=Wi,kn=Cn+16|0,Ln=HEAP32[kn>>2]|0,HEAP32[263]=Ln,On=ts,Mn=On+4|0,_n=HEAP32[Mn>>2]|0,HEAP32[268]=_n,Dn=ts,Pn=Dn+16|0,Hn=HEAP32[Pn>>2]|0,HEAP32[256]=Hn,Bn=ts,jn=Bn+20|0,Fn=HEAP32[jn>>2]|0,HEAP32[260]=Fn,In=ts,Un=In+24|0,zn=HEAP32[Un>>2]|0,HEAP32[249]=zn,Wn=ts,Xn=Wn+28|0,Vn=HEAP32[Xn>>2]|0,HEAP32[247]=Vn,$n=ts,Jn=$n+32|0,Kn=HEAP32[Jn>>2]|0,HEAP32[259]=Kn,Qn=ts,Gn=Qn+36|0,Zn=HEAP32[Gn>>2]|0,HEAP32[255]=Zn,er=ts,tr=er+40|0,nr=HEAP32[tr>>2]|0,HEAP32[265]=nr,rr=ts,ir=rr+48|0,sr=HEAP32[ir>>2]|0,HEAP32[252]=sr,or=ts,ur=or+52|0,ar=HEAP32[ur>>2]|0,HEAP32[253]=ar,lr=ts,cr=lr+56|0,hr=HEAP32[cr>>2]|0,HEAP32[251]=hr,pr=ts,dr=pr+60|0,vr=HEAP32[dr>>2]|0,HEAP32[254]=vr,mr=ts,gr=mr+64|0,yr=HEAP32[gr>>2]|0,HEAP32[261]=yr,br=ts,Er=br+68|0,Sr=HEAP32[Er>>2]|0,HEAP32[257]=Sr,xr=ts,Tr=xr+72|0,Nr=HEAP32[Tr>>2]|0,HEAP32[250]=Nr,Cr=ts,kr=Cr+76|0,Lr=HEAP32[kr>>2]|0,HEAP32[262]=Lr,Ar=ts,Or=Ar+80|0,_r=HEAP32[Or>>2]|0,HEAP32[267]=_r,Dr=hs,Pr=Dr+8|0,Hr=+HEAPF64[Pr>>3],HEAPF64[15]=Hr,Br=hs,jr=Br+16|0,Fr=+HEAPF64[jr>>3],HEAPF64[9]=Fr,Ir=hs,qr=Ir+24|0,Rr=+HEAPF64[qr>>3],HEAPF64[5]=Rr,zr=hs,Wr=zr+32|0,Xr=+HEAPF64[Wr>>3],HEAPF64[16]=Xr,Vr=hs,$r=Vr+40|0,Jr=+HEAPF64[$r>>3],HEAPF64[20]=Jr,Kr=hs,Qr=Kr+48|0,Gr=+HEAPF64[Qr>>3],HEAPF64[7]=Gr,Yr=hs,ei=Yr+56|0,ti=+HEAPF64[ei>>3],HEAPF64[19]=ti,ni=hs,ri=ni+64|0,ii=+HEAPF64[ri>>3],HEAPF64[21]=ii,si=hs,oi=si+72|0,ui=+HEAPF64[oi>>3],HEAPF64[23]=ui,ai=hs,fi=ai+80|0,ci=+HEAPF64[fi>>3],HEAPF64[12]=ci,hi=hs,pi=hi+88|0,di=+HEAPF64[pi>>3],HEAPF64[1]=di,vi=hs,mi=vi+96|0,gi=+HEAPF64[mi>>3],HEAPF64[18]=gi,yi=hs,bi=yi+104|0,wi=+HEAPF64[bi>>3],HEAPF64[22]=wi,Si=hs,xi=Si+112|0,Ti=+HEAPF64[xi>>3],HEAPF64[6]=Ti,Ni=hs,Ci=Ni+120|0,ki=+HEAPF64[Ci>>3],HEAPF64[14]=ki,Li=hs,Ai=Li+128|0,Oi=+HEAPF64[Ai>>3],HEAPF64[4]=Oi,Mi=li,Di=_s_cmp(Mi,14362,5,5)|0,Pi=(Di|0)==0;do if(Pi)Yg=43;else{Hi=li,Bi=_s_cmp(Hi,14368,5,5)|0,ji=(Bi|0)==0;if(!ji){Fi=li,Ii=_s_cmp(Fi,14374,5,5)|0,qi=(Ii|0)==0;if(!qi){Ri=li,Ui=_s_cmp(Ri,14380,4,4)|0,Xi=(Ui|0)==0;if(!Xi){Yg=11;break e}Vi=li,$i=Vi+6|0,Ji=_s_cmp($i,14385,3,3)|0,Ki=(Ji|0)==0;if(!Ki){Yg=74;break}Qi=B,Gi=Yn,Yi=Gi+8|0,Zi=zi,es=Zi+8|0,_dcopy_(Qi,Yi,1236,es,1236)|0,ns=B,rs=An,is=rs+8|0,ss=ov,os=ss+8|0,_dcopy_(ns,is,1236,os,1236)|0,us=+HEAPF64[9],as=rp,HEAPF64[as>>3]=us,Yg=74;break}HEAP32[253]=1,ls=B,cs=$o,ps=cs+8|0,ds=Qa,vs=ds+8|0,ms=ec,gs=ms+4|0,ys=zi,bs=ys+8|0,ws=ov,Es=ws+8|0,_projgr_(ls,ps,vs,gs,bs,Es,176)|0,Ss=Ei,Ts=HEAP32[Ss>>2]|0,Ns=(Ts|0)>=1,Ns&&(_s_wsfe(1076)|0,_do_fio(1236,1020,4)|0,Cs=rp,_do_fio(1236,Cs,8)|0,_do_fio(1236,176,8)|0,_e_wsfe()|0),ks=+HEAPF64[22],Ls=j,As=+HEAPF64[Ls>>3],Os=ks<=As;if(!Os){Yg=16;break}Ms=li,_s_copy(Ms,14398,60,48),Yg=74;break}Sh=+HEAPF64[22],xh=j,Nh=+HEAPF64[xh>>3],Ch=Sh<=Nh;if(Ch){kh=li,_s_copy(kh,14398,60,48),Yg=74;break}Lh=+HEAPF64[9],Ah=Lh>=0,Oh=+HEAPF64[9],Mh=-Oh,_h=Ah?Oh:Mh,Dg=_h,Dh=rp,Ph=+HEAPF64[Dh>>3],Bh=Ph>=0,jh=rp,Fh=+HEAPF64[jh>>3],Ih=-Fh,qh=Bh?Fh:Ih,Pg=qh,Rh=Dg,Uh=Pg,zh=Rh>=Uh,Wh=Dg,Xh=Pg,$h=zh?Wh:Xh,Dg=$h,Jh=Dg,Kh=Jh>=1,Qh=Dg,Gh=Kh?Qh:1,HEAPF64[10]=Gh,Yh=+HEAPF64[9],Zh=rp,ep=+HEAPF64[Zh>>3],tp=Yh-ep,np=+HEAPF64[5],sp=+HEAPF64[10],op=np*sp,up=tp<=op;if(up){ap=li,_s_copy(ap,14498,60,47),fp=HEAP32[256]|0,lp=(fp|0)>=10;if(!lp){Yg=74;break}HEAP32[251]=-5,Yg=74;break}cp=B,hp=HEAP32[cp>>2]|0,HEAP32[Hg>>2]=hp,HEAP32[245]=1;for(;;){pp=HEAP32[245]|0,dp=HEAP32[Hg>>2]|0,mp=(pp|0)<=(dp|0);if(!mp)break;gp=HEAP32[245]|0,yp=ov,bp=yp+(gp<<3)|0,wp=+HEAPF64[bp>>3],Ep=HEAP32[245]|0,Sp=An,xp=Sp+(Ep<<3)|0,Tp=+HEAPF64[xp>>3],Np=wp-Tp,kp=HEAP32[245]|0,Lp=An,Ap=Lp+(kp<<3)|0,HEAPF64[Ap>>3]=Np,Op=HEAP32[245]|0,Mp=Op+1|0,HEAP32[245]=Mp}_p=B,Dp=An,Pp=Dp+8|0,Hp=An,Bp=Hp+8|0,Fp=+_ddot_(_p,Pp,1236,Bp,1236),HEAPF64[3]=Fp,Ip=+HEAPF64[6],qp=Ip==1,Rp=+HEAPF64[1],Up=+HEAPF64[14],zp=Rp-Up,qp?(HEAPF64[2]=zp,Wp=+HEAPF64[14],Xp=-Wp,HEAPF64[10]=Xp):(Vp=+HEAPF64[6],$p=zp*Vp,HEAPF64[2]=$p,Kp=B,Qp=Rn,Gp=Qp+8|0,_dscal_(Kp,48,Gp,1236)|0,Yp=+HEAPF64[14],Zp=-Yp,ed=+HEAPF64[6],td=Zp*ed,HEAPF64[10]=td),nd=+HEAPF64[2],rd=+HEAPF64[20],id=+HEAPF64[10],od=rd*id,ud=nd<=od;if(ud){ad=HEAP32[260]|0,fd=ad+1|0,HEAP32[260]=fd,HEAP32[263]=0,ld=Ei,cd=HEAP32[ld>>2]|0,hd=(cd|0)>=1;if(!hd){Yg=16;break}_s_wsfe(1196)|0,_do_fio(1236,16,8)|0,_do_fio(1236,80,8)|0,_e_wsfe()|0,Yg=16;break}HEAP32[263]=1,pd=HEAP32[265]|0,dd=pd+1|0,HEAP32[265]=dd,vd=B,gd=qn,yd=$g,bd=J,wd=bd+(yd<<3)|0,Ed=Gg,Sd=st,xd=Sd+(Ed<<3)|0,Td=zg,Nd=mt,Cd=Nd+(Td<<3)|0,Ld=Rg,Ad=kt,Od=Ad+(Ld<<3)|0,Md=Rn,_d=Md+8|0,Dd=An,Pd=Dd+8|0,_matupd_(vd,gd,wd,xd,Cd,Od,_d,Pd,1036,1060,988,996,120,24,16,48,32)|0,Hd=qn,Bd=Kg,jd=Ft,Id=jd+(Bd<<3)|0,qd=zg,Rd=mt,Ud=Rd+(qd<<3)|0,zd=Rg,Wd=kt,Xd=Wd+(zd<<3)|0,_formt_(Hd,Id,Ud,Xd,988,120,1004)|0,Vd=HEAP32[251]|0,$d=(Vd|0)!=0,$d?(Jd=Ei,Qd=HEAP32[Jd>>2]|0,Gd=(Qd|0)>=1,Gd&&(_s_wsfe(1216)|0,_e_wsfe()|0),HEAP32[251]=0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,Yg=16):Yg=16}while(0);t:for(;;){if((Yg|0)==16){Yg=0,_s=Ei,Ds=HEAP32[_s>>2]|0,Hs=(Ds|0)>=99,Hs&&(_s_wsfe(1096)|0,Bs=HEAP32[255]|0,js=Bs+1|0,HEAP32[Hg>>2]=js,_do_fio(1236,Hg,4)|0,_e_wsfe()|0),HEAP32[261]=-1,Fs=HEAP32[266]|0,Is=(Fs|0)==0,qs=HEAP32[247]|0,Rs=(qs|0)>0,Bg=Is&Rs;do{if(!Bg){_timer_(56)|0,Ks=B,Qs=zi,Gs=Qs+8|0,Ys=$o,Zs=Ys+8|0,eo=Qa,to=eo+8|0,ro=ec,io=ro+4|0,so=ov,oo=so+8|0,uo=Zr,ao=uo+4|0,fo=Ur,lo=fo+4|0,co=Yn,ho=co+8|0,vo=Rn,mo=vo+8|0,go=yn,yo=go+8|0,bo=qn,wo=Gg,Eo=st,So=Eo+(wo<<3)|0,xo=$g,To=J,Co=To+(xo<<3)|0,ko=zg,Lo=mt,Ao=Lo+(ko<<3)|0,Oo=Kg,Mo=Ft,_o=Mo+(Oo<<3)|0,Do=wr,Po=Do+8|0,Ho=qn,jo=HEAP32[Ho>>2]|0,Fo=jo<<1,Io=Fo+1|0,qo=wr,Ro=qo+(Io<<3)|0,Uo=qn,zo=HEAP32[Uo>>2]|0,Wo=zo<<2,Xo=Wo+1|0,Vo=wr,Ko=Vo+(Xo<<3)|0,Qo=qn,Go=HEAP32[Qo>>2]|0,Yo=Go*6|0,Zo=Yo+1|0,eu=wr,tu=eu+(Zo<<3)|0,nu=Ei,_cauchy_(Ks,Gs,Zs,to,io,oo,ao,lo,ho,mo,yo,bo,So,Co,Ao,_o,120,988,996,Po,Ro,Ko,tu,1008,nu,176,1004,160)|0,ru=HEAP32[251]|0,iu=(ru|0)!=0;if(!iu){_timer_(64)|0,du=+HEAPF64[19],vu=+HEAPF64[8],gu=du+vu,yu=+HEAPF64[7],bu=gu-yu,HEAPF64[19]=bu,wu=HEAP32[252]|0,Eu=HEAP32[268]|0,Su=Eu+wu|0,HEAP32[268]=Su,xu=B,Tu=Mr,Nu=Tu+4|0,Cu=Zr,Lu=Cu+4|0,Au=Ur,Ou=Au+4|0,Mu=Ei,_freev_(xu,1028,Nu,1068,1048,Lu,Ou,992,1052,1064,Mu,1020)|0,_u=B,Du=HEAP32[_u>>2]|0,Pu=HEAP32[257]|0,Hu=Du-Pu|0,HEAP32[250]=Hu;break}ou=Ei,uu=HEAP32[ou>>2]|0,au=(uu|0)>=1,au&&(_s_wsfe(1116)|0,_e_wsfe()|0),HEAP32[251]=0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,_timer_(64)|0,fu=+HEAPF64[19],lu=+HEAPF64[8],cu=fu+lu,hu=+HEAPF64[7],pu=cu-hu,HEAPF64[19]=pu,Yg=16;continue t}Us=B,zs=zi,Ws=zs+8|0,Vs=yn,$s=Vs+8|0,_dcopy_(Us,Ws,1236,$s,1236)|0,Js=HEAP32[263]|0,HEAP32[248]=Js,HEAP32[252]=0}while(0);Bu=HEAP32[257]|0,ju=(Bu|0)==0,Iu=HEAP32[247]|0,qu=(Iu|0)==0,jg=ju|qu;do if(!jg){_timer_(56)|0,Ru=HEAP32[248]|0,Uu=(Ru|0)!=0,Uu&&(zu=B,Wu=Mr,Xu=Wu+4|0,Vu=Zr,$u=Vu+4|0,Ju=Xg,Qu=Kt,Gu=Qu+(Ju<<3)|0,Yu=Ig,Zu=un,ea=Zu+(Yu<<3)|0,ta=qn,na=$g,ra=J,ia=ra+(na<<3)|0,sa=Gg,ua=st,aa=ua+(sa<<3)|0,fa=zg,la=mt,ca=la+(fa<<3)|0,_formk_(zu,1028,Xu,1068,1048,$u,1060,1052,Gu,ea,ta,ia,aa,ca,120,988,996,1004)|0),ha=HEAP32[251]|0,pa=(ha|0)!=0;if(pa){da=Ei,va=HEAP32[da>>2]|0,ma=(va|0)>=1,ma&&(_s_wsfe(1136)|0,_e_wsfe()|0),HEAP32[251]=0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,_timer_(64)|0,ya=+HEAPF64[21],ba=+HEAPF64[8],wa=ya+ba,Ea=+HEAPF64[7],Sa=wa-Ea,HEAPF64[21]=Sa,Yg=16;continue t}xa=B,Ta=qn,Na=zi,Ca=Na+8|0,ka=ov,Aa=ka+8|0,Oa=$g,Ma=J,_a=Ma+(Oa<<3)|0,Da=Gg,Pa=st,Ha=Pa+(Da<<3)|0,Ba=zg,ja=mt,Fa=ja+(Ba<<3)|0,qa=Kg,Ra=Ft,Ua=Ra+(qa<<3)|0,za=yn,Wa=za+8|0,Xa=An,Va=Xa+8|0,$a=wr,Ja=$a+8|0,Ka=Mr,Ya=Ka+4|0,_cmprlb_(xa,Ta,Ca,Aa,_a,Ha,Fa,Ua,Wa,Va,Ja,Ya,120,988,996,1028,1064,1004)|0,Za=HEAP32[251]|0,ef=(Za|0)!=0,ef||(tf=B,nf=qn,rf=Mr,sf=rf+4|0,of=$o,uf=of+8|0,af=Qa,lf=af+8|0,cf=ec,hf=cf+4|0,pf=yn,df=pf+8|0,vf=An,mf=vf+8|0,gf=fr,yf=gf+8|0,bf=$g,Ef=J,Sf=Ef+(bf<<3)|0,xf=Gg,Tf=st,Nf=Tf+(xf<<3)|0,Cf=zi,kf=Cf+8|0,Lf=ov,Af=Lf+8|0,Of=wr,_f=Of+8|0,Df=Xg,Pf=Kt,Hf=Pf+(Df<<3)|0,Bf=Ei,_subsm_(tf,nf,1028,sf,uf,lf,hf,df,mf,yf,Sf,Nf,120,kf,Af,988,996,1044,_f,Hf,Bf,1004)|0),jf=HEAP32[251]|0,Ff=(jf|0)!=0;if(!Ff){_timer_(64)|0,Jf=+HEAPF64[21],Kf=+HEAPF64[8],Qf=Jf+Kf,Gf=+HEAPF64[7],Yf=Qf-Gf,HEAPF64[21]=Yf;break}If=Ei,qf=HEAP32[If>>2]|0,Rf=(qf|0)>=1,Rf&&(_s_wsfe(1156)|0,_e_wsfe()|0),HEAP32[251]=0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,_timer_(64)|0,zf=+HEAPF64[21],Wf=+HEAPF64[8],Xf=zf+Wf,Vf=+HEAPF64[7],$f=Xf-Vf,HEAPF64[21]=$f,Yg=16;continue t}while(0);el=B,tl=HEAP32[el>>2]|0,HEAP32[Hg>>2]=tl,HEAP32[245]=1;for(;;){nl=HEAP32[245]|0,rl=HEAP32[Hg>>2]|0,il=(nl|0)<=(rl|0);if(!il)break;sl=HEAP32[245]|0,ol=yn,ul=ol+(sl<<3)|0,al=+HEAPF64[ul>>3],fl=HEAP32[245]|0,cl=zi,hl=cl+(fl<<3)|0,pl=+HEAPF64[hl>>3],dl=al-pl,vl=HEAP32[245]|0,ml=Rn,gl=ml+(vl<<3)|0,HEAPF64[gl>>3]=dl,yl=HEAP32[245]|0,bl=yl+1|0,HEAP32[245]=bl}_timer_(56)|0,Yg=43;continue}if((Yg|0)==43){Yg=0,wl=B,Sl=$o,xl=Sl+8|0,Tl=Qa,Nl=Tl+8|0,Cl=ec,kl=Cl+4|0,Ll=zi,Al=Ll+8|0,Ol=rp,Ml=ov,Dl=Ml+8|0,Pl=Rn,Hl=Pl+8|0,Bl=An,jl=Bl+8|0,Fl=Yn,Il=Fl+8|0,ql=yn,Rl=ql+8|0,Ul=li,Wl=_i,Xl=ts,Vl=Xl+88|0,$l=hs,Jl=$l+136|0,Kl=Ei,_lnsrlb_(wl,xl,Nl,kl,Al,Ol,72,8,112,Dl,Hl,jl,Il,Rl,48,128,32,136,144,1020,1016,1024,1012,1004,Ul,1032,1064,Wl,Vl,Jl,Kl,60,60)|0,Ql=HEAP32[251]|0,Gl=(Ql|0)!=0;if(!Gl){Yl=HEAP32[256]|0,Zl=xs,nc=HEAP32[Zl>>2]|0,rc=(Yl|0)>=(nc|0);if(!rc)break}ic=B,sc=Yn,oc=sc+8|0,uc=zi,ac=uc+8|0,_dcopy_(ic,oc,1236,ac,1236)|0,fc=B,lc=An,cc=lc+8|0,pc=ov,dc=pc+8|0,_dcopy_(fc,cc,1236,dc,1236)|0,vc=+HEAPF64[9],mc=rp,HEAPF64[mc>>3]=vc,gc=HEAP32[247]|0,yc=(gc|0)==0;if(yc){bc=HEAP32[251]|0,wc=(bc|0)==0,wc&&(HEAP32[251]=-9,Ec=HEAP32[253]|0,Sc=Ec+ -1|0,HEAP32[253]=Sc,Tc=HEAP32[254]|0,Nc=Tc+ -1|0,HEAP32[254]=Nc,Cc=HEAP32[256]|0,kc=Cc+ -1|0,HEAP32[256]=kc),Lc=li,_s_copy(Lc,14447,60,30),Ac=HEAP32[255]|0,Oc=Ac+1|0,HEAP32[255]=Oc,Yg=74;continue}Mc=Ei,_c=HEAP32[Mc>>2]|0,Dc=(_c|0)>=1,Dc&&(_s_wsfe(1176)|0,_e_wsfe()|0),Hc=HEAP32[251]|0,Bc=(Hc|0)==0,Bc&&(jc=HEAP32[253]|0,Fc=jc+ -1|0,HEAP32[253]=Fc),HEAP32[251]=0,HEAP32[247]=0,HEAP32[249]=1,HEAPF64[15]=1,HEAP32[265]=0,HEAP32[263]=0,Ic=li,_s_copy(Ic,14478,60,19),_timer_(64)|0,qc=+HEAPF64[23],Rc=+HEAPF64[8],Uc=qc+Rc,zc=+HEAPF64[7],Wc=Uc-zc,HEAPF64[23]=Wc,Yg=16;continue}if((Yg|0)==74){Yg=0,_timer_(104)|0,Yd=+HEAPF64[13],Zd=+HEAPF64[12],ev=Yd-Zd,HEAPF64[11]=ev,tv=B,nv=zi,rv=nv+8|0,iv=rp,sv=li,av=Ei,_prn3lb_(tv,rv,iv,sv,av,1004,1020,1012,1072,1040,1e3,176,88,1008,14347,1024,48,136,984,152,168,184,60,3)|0;break e}}Vc=li,$c=_s_cmp(Vc,14362,5,5)|0,Jc=($c|0)==0,Jc||(_timer_(64)|0,Kc=+HEAPF64[23],Qc=+HEAPF64[8],Gc=Kc+Qc,Yc=+HEAPF64[7],Zc=Gc-Yc,HEAPF64[23]=Zc,eh=HEAP32[255]|0,th=eh+1|0,HEAP32[255]=th,rh=B,ih=$o,sh=ih+8|0,oh=Qa,uh=oh+8|0,ah=ec,fh=ah+4|0,lh=zi,ch=lh+8|0,hh=ov,dh=hh+8|0,_projgr_(rh,sh,uh,fh,ch,dh,176)|0,vh=B,mh=zi,gh=mh+8|0,yh=rp,bh=ov,wh=bh+8|0,Eh=Ei,_prn2lb_(vh,gh,yh,wh,Eh,1020,1012,1e3,176,1008,14347,1044,1024,48,136,3)|0)}while(0);return(Yg|0)==11&&(fs=li,_s_copy(fs,14389,60,8)),fv=HEAP32[264]|0,lv=Wi,cv=lv+4|0,HEAP32[cv>>2]=fv,hv=HEAP32[266]|0,pv=Wi,dv=pv+8|0,HEAP32[dv>>2]=hv,vv=HEAP32[258]|0,mv=Wi,gv=mv+12|0,HEAP32[gv>>2]=vv,bv=HEAP32[263]|0,wv=Wi,Ev=wv+16|0,HEAP32[Ev>>2]=bv,Sv=HEAP32[268]|0,xv=ts,Tv=xv+4|0,HEAP32[Tv>>2]=Sv,Nv=HEAP32[256]|0,Cv=ts,kv=Cv+16|0,HEAP32[kv>>2]=Nv,Lv=HEAP32[260]|0,Ov=ts,Mv=Ov+20|0,HEAP32[Mv>>2]=Lv,_v=HEAP32[249]|0,Dv=ts,Pv=Dv+24|0,HEAP32[Pv>>2]=_v,Hv=HEAP32[247]|0,Bv=ts,jv=Bv+28|0,HEAP32[jv>>2]=Hv,Fv=HEAP32[259]|0,Iv=ts,Rv=Iv+32|0,HEAP32[Rv>>2]=Fv,Uv=HEAP32[255]|0,zv=ts,Wv=zv+36|0,HEAP32[Wv>>2]=Uv,Xv=HEAP32[265]|0,Vv=ts,$v=Vv+40|0,HEAP32[$v>>2]=Xv,Jv=HEAP32[252]|0,Kv=ts,Qv=Kv+48|0,HEAP32[Qv>>2]=Jv,Yv=HEAP32[253]|0,Zv=ts,em=Zv+52|0,HEAP32[em>>2]=Yv,tm=HEAP32[251]|0,nm=ts,rm=nm+56|0,HEAP32[rm>>2]=tm,im=HEAP32[254]|0,sm=ts,om=sm+60|0,HEAP32[om>>2]=im,um=HEAP32[261]|0,fm=ts,lm=fm+64|0,HEAP32[lm>>2]=um,cm=HEAP32[257]|0,hm=ts,pm=hm+68|0,HEAP32[pm>>2]=cm,dm=HEAP32[250]|0,vm=ts,mm=vm+72|0,HEAP32[mm>>2]=dm,gm=HEAP32[262]|0,ym=ts,wm=ym+76|0,HEAP32[wm>>2]=gm,Em=HEAP32[267]|0,Sm=ts,xm=Sm+80|0,HEAP32[xm>>2]=Em,Tm=+HEAPF64[15],Nm=hs,Cm=Nm+8|0,HEAPF64[Cm>>3]=Tm,km=+HEAPF64[9],Lm=hs,Am=Lm+16|0,HEAPF64[Am>>3]=km,Mm=+HEAPF64[5],_m=hs,Dm=_m+24|0,HEAPF64[Dm>>3]=Mm,Pm=+HEAPF64[16],Hm=hs,Bm=Hm+32|0,HEAPF64[Bm>>3]=Pm,jm=+HEAPF64[20],Fm=hs,Im=Fm+40|0,HEAPF64[Im>>3]=jm,qm=+HEAPF64[7],Um=hs,zm=Um+48|0,HEAPF64[zm>>3]=qm,Wm=+HEAPF64[19],Xm=hs,Vm=Xm+56|0,HEAPF64[Vm>>3]=Wm,$m=+HEAPF64[21],Jm=hs,Km=Jm+64|0,HEAPF64[Km>>3]=$m,Qm=+HEAPF64[23],Gm=hs,Zm=Gm+72|0,HEAPF64[Zm>>3]=Qm,eg=+HEAPF64[12],tg=hs,ng=tg+80|0,HEAPF64[ng>>3]=eg,rg=+HEAPF64[1],ig=hs,sg=ig+88|0,HEAPF64[sg>>3]=rg,og=+HEAPF64[18],ug=hs,ag=ug+96|0,HEAPF64[ag>>3]=og,cg=+HEAPF64[22],hg=hs,pg=hg+104|0,HEAPF64[pg>>3]=cg,dg=+HEAPF64[6],vg=hs,mg=vg+112|0,HEAPF64[mg>>3]=dg,gg=+HEAPF64[14],yg=hs,bg=yg+120|0,HEAPF64[bg>>3]=gg,wg=+HEAPF64[4],Sg=hs,xg=Sg+128|0,HEAPF64[xg>>3]=wg,H=0,Tg=H,STACKTOP=Zg,Tg|0};

function _errclb_(e,t,n,r,i,s,o,u,a,f){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0;var l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0;Nt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),l=e,c=t,S=n,D=r,W=i,tt=s,ht=o,wt=u,Et=a,St=f,h=tt,p=h+ -4|0,tt=p,d=W,v=d+ -8|0,W=v,m=D,g=m+ -8|0,D=g,y=l,b=HEAP32[y>>2]|0,w=(b|0)<=0,w&&(E=ht,_s_copy(E,15167,60,15)),x=c,T=HEAP32[x>>2]|0,N=(T|0)<=0,N&&(C=ht,_s_copy(C,15183,60,15)),k=S,L=+HEAPF64[k>>3],A=L<0,A&&(O=ht,_s_copy(O,15199,60,19)),M=l,_=HEAP32[M>>2]|0,xt=_,HEAP32[420]=1;for(;;){P=HEAP32[420]|0,H=xt,B=(P|0)<=(H|0);if(!B)break;j=HEAP32[420]|0,F=tt,I=F+(j<<2)|0,q=HEAP32[I>>2]|0,R=(q|0)<0,R?Tt=11:(U=HEAP32[420]|0,z=tt,X=z+(U<<2)|0,V=HEAP32[X>>2]|0,$=(V|0)>3,$&&(Tt=11)),(Tt|0)==11&&(Tt=0,J=ht,_s_copy(J,15219,60,18),K=wt,HEAP32[K>>2]=-6,Q=HEAP32[420]|0,G=Et,HEAP32[G>>2]=Q),Y=HEAP32[420]|0,Z=tt,et=Z+(Y<<2)|0,nt=HEAP32[et>>2]|0,rt=(nt|0)==2,rt&&(it=HEAP32[420]|0,st=D,ot=st+(it<<3)|0,ut=+HEAPF64[ot>>3],at=HEAP32[420]|0,ft=W,lt=ft+(at<<3)|0,ct=+HEAPF64[lt>>3],pt=ut>ct,pt&&(dt=ht,_s_copy(dt,15238,60,27),vt=wt,HEAP32[vt>>2]=-7,mt=HEAP32[420]|0,gt=Et,HEAP32[gt>>2]=mt)),yt=HEAP32[420]|0,bt=yt+1|0,HEAP32[420]=bt}return STACKTOP=Nt,0}function _prn3lb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0,E|=0,S|=0,x|=0;var T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0;wn=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),T=e,N=t,j=n,J=r,st=i,mt=s,kt=o,Ft=u,Kt=a,un=f,C=l,L=c,A=h,O=p,M=d,_=v,D=m,P=g,H=y,B=b,F=w,I=E,q=S,R=x,U=N,z=U+ -8|0,N=z,W=J,X=_s_cmp(W,14356,5,5)|0,V=(X|0)==0;if(!V){$=st,K=HEAP32[$>>2]|0,Q=(K|0)>=0;if(Q){_s_wsfe(2136)|0,_e_wsfe()|0,_s_wsfe(2156)|0,_e_wsfe()|0,_s_wsfe(2176)|0,G=T,_do_fio(1236,G,4)|0,Y=kt,_do_fio(1236,Y,4)|0,Z=Ft,_do_fio(1236,Z,4)|0,et=Kt,_do_fio(1236,et,4)|0,tt=un,_do_fio(1236,tt,4)|0,nt=C,_do_fio(1236,nt,4)|0,rt=L,_do_fio(1236,rt,8)|0,it=j,_do_fio(1236,it,8)|0,_e_wsfe()|0,ot=st,ut=HEAP32[ot>>2]|0,at=(ut|0)>=100;if(at){_s_wsfe(2196)|0,_do_fio(1236,15780,3)|0,ft=T,lt=HEAP32[ft>>2]|0,yn=lt,HEAP32[533]=1;for(;;){ct=HEAP32[533]|0,ht=yn,pt=(ct|0)<=(ht|0);if(!pt)break;dt=HEAP32[533]|0,vt=N,gt=vt+(dt<<3)|0,_do_fio(1236,gt,8)|0,yt=HEAP32[533]|0,bt=yt+1|0,HEAP32[533]=bt}_e_wsfe()|0}wt=st,Et=HEAP32[wt>>2]|0,St=(Et|0)>=1,St&&(_s_wsle(2216)|0,_do_lio(1308,1236,17485,4)|0,xt=j,_do_lio(1908,1236,xt,8)|0,_e_wsle()|0)}}return Tt=st,Nt=HEAP32[Tt>>2]|0,Ct=(Nt|0)>=0,Ct?(_s_wsfe(2236)|0,Lt=J,_do_fio(1236,Lt,60)|0,_e_wsfe()|0,At=mt,Ot=HEAP32[At>>2]|0,Mt=(Ot|0)!=0,Mt&&(_t=mt,Dt=HEAP32[_t>>2]|0,Pt=(Dt|0)==-1,Pt&&(_s_wsfe(2256)|0,_e_wsfe()|0),Ht=mt,Bt=HEAP32[Ht>>2]|0,jt=(Bt|0)==-2,jt&&(_s_wsfe(2276)|0,_e_wsfe()|0),It=mt,qt=HEAP32[It>>2]|0,Rt=(qt|0)==-3,Rt&&(_s_wsfe(2296)|0,_e_wsfe()|0),Ut=mt,zt=HEAP32[Ut>>2]|0,Wt=(zt|0)==-4,Wt&&(_s_wsfe(2316)|0,_e_wsfe()|0),Xt=mt,Vt=HEAP32[Xt>>2]|0,$t=(Vt|0)==-5,$t&&(_s_wsfe(2336)|0,_e_wsfe()|0),Jt=mt,Qt=HEAP32[Jt>>2]|0,Gt=(Qt|0)==-6,Gt&&(_s_wsle(2356)|0,_do_lio(1308,1236,17490,11)|0,Yt=H,_do_lio(1660,1236,Yt,4)|0,_do_lio(1308,1236,17502,13)|0,_e_wsle()|0),Zt=mt,en=HEAP32[Zt>>2]|0,tn=(en|0)==-7,tn&&(_s_wsle(2376)|0,_do_lio(1308,1236,17516,3)|0,nn=H,_do_lio(1660,1236,nn,4)|0,_do_lio(1308,1236,17520,6)|0,rn=H,_do_lio(1660,1236,rn,4)|0,_do_lio(1308,1236,17527,25)|0,_e_wsle()|0),sn=mt,on=HEAP32[sn>>2]|0,an=(on|0)==-8,an&&(_s_wsfe(2396)|0,_e_wsfe()|0),fn=mt,ln=HEAP32[fn>>2]|0,cn=(ln|0)==-9,cn&&(_s_wsfe(2416)|0,_e_wsfe()|0)),hn=st,pn=HEAP32[hn>>2]|0,dn=(pn|0)>=1,dn&&(_s_wsfe(2436)|0,vn=B,_do_fio(1236,vn,8)|0,mn=F,_do_fio(1236,mn,8)|0,gn=I,_do_fio(1236,gn,8)|0,_e_wsfe()|0),_s_wsfe(2456)|0,k=A,_do_fio(1236,k,8)|0,_e_wsfe()|0,STACKTOP=wn,0):(STACKTOP=wn,0)}function _prn1lb_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0;lt=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),u=e,a=t,b=n,O=r,R=i,Y=s,it=o,st=R,ot=st+ -8|0,R=ot,ut=O,f=ut+ -8|0,O=f,l=b,c=l+ -8|0,b=c,h=Y,p=HEAP32[h>>2]|0,d=(p|0)>=0;if(!d)return STACKTOP=lt,0;_s_wsfe(1924)|0,v=it,_do_fio(1236,v,8)|0,_e_wsfe()|0,_s_wsle(1944)|0,_do_lio(1308,1236,15598,4)|0,m=u,_do_lio(1660,1236,m,4)|0,_do_lio(1308,1236,15603,8)|0,g=a,_do_lio(1660,1236,g,4)|0,_e_wsle()|0,y=Y,w=HEAP32[y>>2]|0,E=(w|0)>=1;if(!E)return STACKTOP=lt,0;S=Y,x=HEAP32[S>>2]|0,T=(x|0)>100;if(!T)return STACKTOP=lt,0;_s_wsfe(1964)|0,_do_fio(1236,15612,3)|0,N=u,C=HEAP32[N>>2]|0,at=C,HEAP32[480]=1;for(;;){k=HEAP32[480]|0,L=at,A=(k|0)<=(L|0);if(!A)break;M=HEAP32[480]|0,_=b,D=_+(M<<3)|0,_do_fio(1236,D,8)|0,P=HEAP32[480]|0,H=P+1|0,HEAP32[480]=H}_e_wsfe()|0,_s_wsfe(1984)|0,_do_fio(1236,15616,4)|0,B=u,j=HEAP32[B>>2]|0,at=j,HEAP32[480]=1;for(;;){F=HEAP32[480]|0,I=at,q=(F|0)<=(I|0);if(!q)break;U=HEAP32[480]|0,z=R,W=z+(U<<3)|0,_do_fio(1236,W,8)|0,X=HEAP32[480]|0,V=X+1|0,HEAP32[480]=V}_e_wsfe()|0,_s_wsfe(2004)|0,_do_fio(1236,15621,3)|0,$=u,J=HEAP32[$>>2]|0,at=J,HEAP32[480]=1;for(;;){K=HEAP32[480]|0,Q=at,G=(K|0)<=(Q|0);if(!G)break;Z=HEAP32[480]|0,et=O,tt=et+(Z<<3)|0,_do_fio(1236,tt,8)|0,nt=HEAP32[480]|0,rt=nt+1|0,HEAP32[480]=rt}return _e_wsfe()|0,STACKTOP=lt,0}function _active_(e,t,n,r,i,s,o,u,a,f){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0;var l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0;ur=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),l=e,c=t,Tt=n,Ht=r,Vt=i,rn=s,vn=o,Cn=u,jn=a,Jn=f,h=rn,x=h+ -4|0,rn=x,P=Vt,X=P+ -8|0,Vt=X,nt=Ht,pt=nt+ -4|0,Ht=pt,wt=Tt,Et=wt+ -8|0,Tt=Et,St=c,xt=St+ -8|0,c=xt,HEAP32[311]=0,Nt=Cn,HEAP32[Nt>>2]=0,Ct=jn,HEAP32[Ct>>2]=0,kt=Jn,HEAP32[kt>>2]=1,Lt=l,At=HEAP32[Lt>>2]|0,sr=At,HEAP32[310]=1;for(;;){Ot=HEAP32[310]|0,Mt=sr,_t=(Ot|0)<=(Mt|0);if(!_t)break;Dt=HEAP32[310]|0,Pt=Ht,Bt=Pt+(Dt<<2)|0,jt=HEAP32[Bt>>2]|0,Ft=(jt|0)>0;do if(Ft){It=HEAP32[310]|0,qt=Ht,Rt=qt+(It<<2)|0,Ut=HEAP32[Rt>>2]|0,zt=(Ut|0)<=2;if(zt){Wt=HEAP32[310]|0,Xt=Vt,$t=Xt+(Wt<<3)|0,Jt=+HEAPF64[$t>>3],Kt=HEAP32[310]|0,Qt=c,Gt=Qt+(Kt<<3)|0,Yt=+HEAPF64[Gt>>3],Zt=Jt<=Yt;if(Zt){en=HEAP32[310]|0,tn=Vt,nn=tn+(en<<3)|0,sn=+HEAPF64[nn>>3],on=HEAP32[310]|0,un=c,an=un+(on<<3)|0,fn=+HEAPF64[an>>3],ln=sn<fn,ln&&(cn=Cn,HEAP32[cn>>2]=1,hn=HEAP32[310]|0,pn=c,dn=pn+(hn<<3)|0,mn=+HEAPF64[dn>>3],gn=HEAP32[310]|0,yn=Vt,bn=yn+(gn<<3)|0,HEAPF64[bn>>3]=mn),wn=HEAP32[311]|0,En=wn+1|0,HEAP32[311]=En;break}}Sn=HEAP32[310]|0,xn=Ht,Tn=xn+(Sn<<2)|0,Nn=HEAP32[Tn>>2]|0,kn=(Nn|0)>=2,kn&&(Ln=HEAP32[310]|0,An=Vt,On=An+(Ln<<3)|0,Mn=+HEAPF64[On>>3],_n=HEAP32[310]|0,Dn=Tt,Pn=Dn+(_n<<3)|0,Hn=+HEAPF64[Pn>>3],Bn=Mn>=Hn,Bn&&(Fn=HEAP32[310]|0,In=Vt,qn=In+(Fn<<3)|0,Rn=+HEAPF64[qn>>3],Un=HEAP32[310]|0,zn=Tt,Wn=zn+(Un<<3)|0,Xn=+HEAPF64[Wn>>3],Vn=Rn>Xn,Vn&&($n=Cn,HEAP32[$n>>2]=1,Kn=HEAP32[310]|0,Qn=Tt,Gn=Qn+(Kn<<3)|0,Yn=+HEAPF64[Gn>>3],Zn=HEAP32[310]|0,er=Vt,tr=er+(Zn<<3)|0,HEAPF64[tr>>3]=Yn),nr=HEAP32[311]|0,rr=nr+1|0,HEAP32[311]=rr))}while(0);ir=HEAP32[310]|0,p=ir+1|0,HEAP32[310]=p}d=l,v=HEAP32[d>>2]|0,sr=v,HEAP32[310]=1;for(;;){m=HEAP32[310]|0,g=sr,y=(m|0)<=(g|0);if(!y)break;b=HEAP32[310]|0,w=Ht,E=w+(b<<2)|0,S=HEAP32[E>>2]|0,T=(S|0)!=2,T&&(N=Jn,HEAP32[N>>2]=0),C=HEAP32[310]|0,k=Ht,L=k+(C<<2)|0,A=HEAP32[L>>2]|0,O=(A|0)==0;do if(O)M=HEAP32[310]|0,_=rn,D=_+(M<<2)|0,HEAP32[D>>2]=-1;else{H=jn,HEAP32[H>>2]=1,B=HEAP32[310]|0,j=Ht,F=j+(B<<2)|0,I=HEAP32[F>>2]|0,q=(I|0)==2;if(q){R=HEAP32[310]|0,U=Tt,z=U+(R<<3)|0,W=+HEAPF64[z>>3],V=HEAP32[310]|0,$=c,J=$+(V<<3)|0,K=+HEAPF64[J>>3],Q=W-K,G=Q<=0;if(G){Y=HEAP32[310]|0,Z=rn,et=Z+(Y<<2)|0,HEAP32[et>>2]=3;break}}tt=HEAP32[310]|0,rt=rn,it=rt+(tt<<2)|0,HEAP32[it>>2]=0}while(0);st=HEAP32[310]|0,ot=st+1|0,HEAP32[310]=ot}return ut=vn,at=HEAP32[ut>>2]|0,ft=(at|0)>=0,ft&&(lt=Cn,ct=HEAP32[lt>>2]|0,ht=(ct|0)!=0,ht&&(_s_wsle(1248)|0,_do_lio(1308,1236,14601,58)|0,_e_wsle()|0),dt=jn,vt=HEAP32[dt>>2]|0,mt=(vt|0)!=0,mt||(_s_wsle(1268)|0,_do_lio(1308,1236,14660,30)|0,_e_wsle()|0)),gt=vn,yt=HEAP32[gt>>2]|0,bt=(yt|0)>0,bt?(_s_wsfe(1288)|0,_do_fio(1236,1244,4)|0,_e_wsfe()|0,STACKTOP=ur,0):(STACKTOP=ur,0)}function _projgr_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0;zt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),u=e,a=t,b=n,O=r,R=i,Y=s,ft=o,wt=Y,Mt=wt+ -8|0,Y=Mt,Ft=R,f=Ft+ -8|0,R=f,l=O,c=l+ -4|0,O=c,h=b,p=h+ -8|0,b=p,d=a,v=d+ -8|0,a=v,m=ft,HEAPF64[m>>3]=0,g=u,y=HEAP32[g>>2]|0,Rt=y,HEAP32[619]=1;for(;;){w=HEAP32[619]|0,E=Rt,S=(w|0)<=(E|0);if(!S)break;x=HEAP32[619]|0,T=Y,N=T+(x<<3)|0,C=+HEAPF64[N>>3],HEAPF64[58]=C,k=HEAP32[619]|0,L=O,A=L+(k<<2)|0,M=HEAP32[A>>2]|0,_=(M|0)!=0;do if(_){D=+HEAPF64[58],P=D<0,H=HEAP32[619]|0,B=O,j=B+(H<<2)|0,F=HEAP32[j>>2]|0;if(P){I=(F|0)>=2;if(!I)break;q=HEAP32[619]|0,U=R,z=U+(q<<3)|0,W=+HEAPF64[z>>3],X=HEAP32[619]|0,V=b,$=V+(X<<3)|0,J=+HEAPF64[$>>3],K=W-J,It=K,Q=It,G=+HEAPF64[58],Z=Q>=G,et=It,tt=+HEAPF64[58],nt=Z?et:tt,HEAPF64[58]=nt;break}rt=(F|0)<=2;if(!rt)break;it=HEAP32[619]|0,st=R,ot=st+(it<<3)|0,ut=+HEAPF64[ot>>3],at=HEAP32[619]|0,lt=a,ct=lt+(at<<3)|0,ht=+HEAPF64[ct>>3],pt=ut-ht,It=pt,dt=It,vt=+HEAPF64[58],mt=dt<=vt,gt=It,yt=+HEAPF64[58],bt=mt?gt:yt,HEAPF64[58]=bt;break}while(0);Et=ft,St=+HEAPF64[Et>>3],It=St,xt=+HEAPF64[58],Tt=xt>=0,Nt=+HEAPF64[58],Ct=-Nt,kt=Tt?Nt:Ct,qt=kt,Lt=It,At=qt,Ot=Lt>=At,_t=It,Dt=qt,Pt=Ot?_t:Dt,Ht=ft,HEAPF64[Ht>>3]=Pt,Bt=HEAP32[619]|0,jt=Bt+1|0,HEAP32[619]=jt}return STACKTOP=zt,0}function _cauchy_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x,T,N,C,k){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0,E|=0,S|=0,x|=0,T|=0,N|=0,C|=0,k|=0;var L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0;qd=STACKTOP,STACKTOP=STACKTOP+176|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),kd=qd+12|0,Cd=qd,A=e,Dn=t,Bi=n,qo=r,za=i,$l=s,Qh=o,id=u,vd=a,O=f,R=l,Y=c,ft=h,wt=p,Mt=d,Ut=v,Zt=m,cn=g,Sn=y,Pn=b,Xn=w,nr=E,pr=S,Tr=x,Hr=T,Vr=N,ri=C,di=k,Ni=R,ji=Ni+ -8|0,R=ji,Ji=O,ss=Ji+ -8|0,O=ss,ms=vd,ks=ms+ -8|0,vd=ks,Fs=id,Ks=Fs+ -4|0,id=Ks,oo=Qh,yo=oo+ -4|0,Qh=yo,Ao=$l,Ro=Ao+ -8|0,$l=Ro,Yo=za,fu=Yo+ -4|0,za=fu,wu=qo,Mu=wu+ -8|0,qo=Mu,Uu=Bi,Zu=Uu+ -8|0,Bi=Zu,la=Dn,Ea=la+ -8|0,Dn=Ea,_a=pr,Wa=_a+ -8|0,pr=Wa,tf=nr,pf=tf+ -8|0,nr=pf,Tf=Xn,Hf=Tf+ -8|0,Xn=Hf,Vf=Pn,rl=Vf+ -8|0,Pn=rl,dl=Y,Nl=HEAP32[dl>>2]|0,Hd=Nl,Bl=Hd,Jl=1+Bl|0,Bd=Jl,sc=Bd,mc=Ut,kc=0-sc|0,Fc=mc+(kc<<3)|0,Ut=Fc,Kc=Y,oh=HEAP32[Kc>>2]|0,Md=oh,gh=Md,Lh=1+gh|0,_d=Lh,Ih=_d,Gh=Mt,ap=0-Ih|0,bp=Gh+(ap<<3)|0,Mt=bp,Op=A,Rp=HEAP32[Op>>2]|0,Dd=Rp,Yp=Dd,ed=1+Yp|0,Pd=ed,td=Pd,nd=wt,rd=0-td|0,sd=nd+(rd<<3)|0,wt=sd,od=A,ud=HEAP32[od>>2]|0,jd=ud,ad=jd,fd=1+ad|0,Fd=fd,ld=Fd,cd=ft,hd=0-ld|0,pd=cd+(hd<<3)|0,ft=pd,dd=Vr,md=+HEAPF64[dd>>3],gd=md<=0;if(gd)return yd=Hr,bd=HEAP32[yd>>2]|0,wd=(bd|0)>=0,wd&&(_s_wsle(1380)|0,_do_lio(1308,1236,15068,23)|0,_e_wsle()|0),Ed=A,Sd=Dn,xd=Sd+8|0,Td=R,Nd=Td+8|0,_dcopy_(Ed,xd,1236,Nd,1236)|0,L=0,Zp=L,STACKTOP=qd,Zp|0;HEAP32[337]=1,M=A,_=HEAP32[M>>2]|0,D=_+1|0,HEAP32[338]=D,HEAP32[340]=0,HEAP32[341]=0,HEAPF64[42]=0,P=cn,H=HEAP32[P>>2]|0,B=H<<1,HEAP32[335]=B,HEAPF64[26]=0,j=Hr,F=HEAP32[j>>2]|0,I=(F|0)>=99,I&&(_s_wsfe(1400)|0,_e_wsfe()|0),q=HEAP32[335]|0,HEAP32[kd>>2]=q,HEAP32[332]=1;for(;;){U=HEAP32[332]|0,z=HEAP32[kd>>2]|0,W=(U|0)<=(z|0);if(!W)break;X=HEAP32[332]|0,V=Pn,$=V+(X<<3)|0,HEAPF64[$>>3]=0,J=HEAP32[332]|0,K=J+1|0,HEAP32[332]=K}Q=A,G=HEAP32[Q>>2]|0,HEAP32[kd>>2]=G,HEAP32[332]=1;for(;;){Z=HEAP32[332]|0,et=HEAP32[kd>>2]|0,tt=(Z|0)<=(et|0);if(!tt)break;nt=HEAP32[332]|0,rt=$l,it=rt+(nt<<3)|0,st=+HEAPF64[it>>3],ot=-st,HEAPF64[41]=ot,ut=HEAP32[332]|0,at=id,lt=at+(ut<<2)|0,ct=HEAP32[lt>>2]|0,ht=(ct|0)!=3;do if(ht){pt=HEAP32[332]|0,dt=id,vt=dt+(pt<<2)|0,mt=HEAP32[vt>>2]|0,gt=(mt|0)!=-1;if(gt){yt=HEAP32[332]|0,bt=za,Et=bt+(yt<<2)|0,St=HEAP32[Et>>2]|0,xt=(St|0)<=2,xt&&(Tt=HEAP32[332]|0,Nt=Dn,Ct=Nt+(Tt<<3)|0,kt=+HEAPF64[Ct>>3],Lt=HEAP32[332]|0,At=Bi,Ot=At+(Lt<<3)|0,_t=+HEAPF64[Ot>>3],Dt=kt-_t,HEAPF64[30]=Dt),Pt=HEAP32[332]|0,Ht=za,Bt=Ht+(Pt<<2)|0,jt=HEAP32[Bt>>2]|0,Ft=(jt|0)>=2,Ft&&(It=HEAP32[332]|0,qt=qo,Rt=qt+(It<<3)|0,zt=+HEAPF64[Rt>>3],Wt=HEAP32[332]|0,Xt=Dn,Vt=Xt+(Wt<<3)|0,$t=+HEAPF64[Vt>>3],Jt=zt-$t,HEAPF64[31]=Jt),Kt=HEAP32[332]|0,Qt=za,Gt=Qt+(Kt<<2)|0,Yt=HEAP32[Gt>>2]|0,en=(Yt|0)<=2,tn=+HEAPF64[30],nn=tn<=0,rn=en?nn:0,sn=rn&1,HEAP32[343]=sn,on=HEAP32[332]|0,un=za,an=un+(on<<2)|0,fn=HEAP32[an>>2]|0,ln=(fn|0)>=2,hn=+HEAPF64[31],pn=hn<=0,dn=ln?pn:0,vn=dn&1,HEAP32[344]=vn,mn=HEAP32[332]|0,gn=id,yn=gn+(mn<<2)|0,HEAP32[yn>>2]=0,bn=HEAP32[343]|0,wn=(bn|0)!=0;if(wn){En=+HEAPF64[41],xn=En<=0;if(!xn)break;Tn=HEAP32[332]|0,Nn=id,Cn=Nn+(Tn<<2)|0,HEAP32[Cn>>2]=1;break}kn=HEAP32[344]|0,Ln=(kn|0)!=0,An=+HEAPF64[41],On=An>=0;if(Ln){if(!On)break;Mn=HEAP32[332]|0,_n=id,Hn=_n+(Mn<<2)|0,HEAP32[Hn>>2]=2;break}Bn=+HEAPF64[41],jn=-Bn,Fn=On?Bn:jn,In=Fn<=0;if(!In)break;qn=HEAP32[332]|0,Rn=id,Un=Rn+(qn<<2)|0,HEAP32[Un>>2]=-3;break}}while(0);zn=Sn,Wn=HEAP32[zn>>2]|0,HEAP32[342]=Wn,Vn=HEAP32[332]|0,$n=id,Jn=$n+(Vn<<2)|0,Kn=HEAP32[Jn>>2]|0,Qn=(Kn|0)!=0,Qn?(Gn=HEAP32[332]|0,Yn=id,Zn=Yn+(Gn<<2)|0,er=HEAP32[Zn>>2]|0,tr=(er|0)!=-1,tr?(rr=HEAP32[332]|0,ir=O,sr=ir+(rr<<3)|0,HEAPF64[sr>>3]=0):Id=29):Id=29;do if((Id|0)==29){Id=0,or=+HEAPF64[41],ur=HEAP32[332]|0,ar=O,fr=ar+(ur<<3)|0,HEAPF64[fr>>3]=or,lr=+HEAPF64[41],cr=+HEAPF64[41],hr=lr*cr,dr=+HEAPF64[26],vr=dr-hr,HEAPF64[26]=vr,mr=cn,gr=HEAP32[mr>>2]|0,Ld=gr,HEAP32[333]=1;for(;;){yr=HEAP32[333]|0,br=Ld,wr=(yr|0)<=(br|0),Er=HEAP32[332]|0;if(!wr)break;Sr=HEAP32[342]|0,xr=jd,Nr=Math_imul(Sr,xr)|0,Cr=Er+Nr|0,kr=ft,Lr=kr+(Cr<<3)|0,Ar=+HEAPF64[Lr>>3],Or=+HEAPF64[41],Mr=Ar*Or,_r=HEAP32[333]|0,Dr=Pn,Pr=Dr+(_r<<3)|0,Br=+HEAPF64[Pr>>3],jr=Br+Mr,HEAPF64[Pr>>3]=jr,Fr=HEAP32[332]|0,Ir=HEAP32[342]|0,qr=Dd,Rr=Math_imul(Ir,qr)|0,Ur=Fr+Rr|0,zr=wt,Wr=zr+(Ur<<3)|0,Xr=+HEAPF64[Wr>>3],$r=+HEAPF64[41],Jr=Xr*$r,Kr=cn,Qr=HEAP32[Kr>>2]|0,Gr=HEAP32[333]|0,Yr=Qr+Gr|0,Zr=Pn,ei=Zr+(Yr<<3)|0,ti=+HEAPF64[ei>>3],ni=ti+Jr,HEAPF64[ei>>3]=ni,ii=HEAP32[342]|0,si=Y,oi=HEAP32[si>>2]|0,ui=(ii|0)%(oi|0)&-1,ai=ui+1|0,HEAP32[342]=ai,fi=HEAP32[333]|0,li=fi+1|0,HEAP32[333]=li}ci=za,hi=ci+(Er<<2)|0,pi=HEAP32[hi>>2]|0,vi=(pi|0)<=2;if(vi){mi=HEAP32[332]|0,gi=za,yi=gi+(mi<<2)|0,bi=HEAP32[yi>>2]|0,wi=(bi|0)!=0,Ei=+HEAPF64[41],Si=Ei<0,Ad=wi&Si;if(Ad){xi=HEAP32[340]|0,Ti=xi+1|0,HEAP32[340]=Ti,Ci=HEAP32[332]|0,ki=HEAP32[340]|0,Li=Qh,Ai=Li+(ki<<2)|0,HEAP32[Ai>>2]=Ci,Oi=+HEAPF64[30],Mi=+HEAPF64[41],_i=-Mi,Di=Oi/_i,Pi=HEAP32[340]|0,Hi=vd,Fi=Hi+(Pi<<3)|0,HEAPF64[Fi>>3]=Di,Ii=HEAP32[340]|0,qi=(Ii|0)==1;if(!qi){Ri=HEAP32[340]|0,Ui=vd,zi=Ui+(Ri<<3)|0,Wi=+HEAPF64[zi>>3],Xi=+HEAPF64[42],Vi=Wi<Xi;if(!Vi)break}$i=HEAP32[340]|0,Ki=vd,Qi=Ki+($i<<3)|0,Gi=+HEAPF64[Qi>>3],HEAPF64[42]=Gi,Yi=HEAP32[340]|0,HEAP32[341]=Yi;break}}Zi=HEAP32[332]|0,es=za,ts=es+(Zi<<2)|0,ns=HEAP32[ts>>2]|0,rs=(ns|0)>=2,is=+HEAPF64[41],os=is>0,Od=rs&os;if(!Od){Ps=HEAP32[338]|0,Hs=Ps+ -1|0,HEAP32[338]=Hs,Bs=HEAP32[332]|0,js=HEAP32[338]|0,Is=Qh,qs=Is+(js<<2)|0,HEAP32[qs>>2]=Bs,Rs=+HEAPF64[41],Us=Rs>=0,zs=+HEAPF64[41],Ws=-zs,Xs=Us?zs:Ws,Vs=Xs>0;if(!Vs)break;HEAP32[337]=0;break}us=HEAP32[340]|0,as=us+1|0,HEAP32[340]=as,fs=HEAP32[332]|0,ls=HEAP32[340]|0,cs=Qh,hs=cs+(ls<<2)|0,HEAP32[hs>>2]=fs,ps=+HEAPF64[31],ds=+HEAPF64[41],vs=ps/ds,gs=HEAP32[340]|0,ys=vd,bs=ys+(gs<<3)|0,HEAPF64[bs>>3]=vs,ws=HEAP32[340]|0,Es=(ws|0)==1;if(!Es){Ss=HEAP32[340]|0,xs=vd,Ts=xs+(Ss<<3)|0,Ns=+HEAPF64[Ts>>3],Cs=+HEAPF64[42],Ls=Ns<Cs;if(!Ls)break}As=HEAP32[340]|0,Os=vd,Ms=Os+(As<<3)|0,_s=+HEAPF64[Ms>>3],HEAPF64[42]=_s,Ds=HEAP32[340]|0,HEAP32[341]=Ds}while(0);$s=HEAP32[332]|0,Js=$s+1|0,HEAP32[332]=Js}Qs=Zt,Gs=+HEAPF64[Qs>>3],Ys=Gs!=1,Ys&&(Zs=cn,eo=Zt,to=cn,no=HEAP32[to>>2]|0,ro=no+1|0,io=Pn,so=io+(ro<<3)|0,_dscal_(Zs,eo,so,1236)|0),uo=A,ao=Dn,fo=ao+8|0,lo=R,co=lo+8|0,_dcopy_(uo,fo,1236,co,1236)|0,ho=HEAP32[340]|0,po=(ho|0)==0;if(po){vo=HEAP32[338]|0,mo=A,go=HEAP32[mo>>2]|0,bo=go+1|0,wo=(vo|0)==(bo|0);if(wo){Eo=Hr,So=HEAP32[Eo>>2]|0,xo=(So|0)>100;if(xo){_s_wsfe(1420)|0,To=A,No=HEAP32[To>>2]|0,HEAP32[kd>>2]=No,HEAP32[332]=1;for(;;){Co=HEAP32[332]|0,ko=HEAP32[kd>>2]|0,Lo=(Co|0)<=(ko|0);if(!Lo)break;Oo=HEAP32[332]|0,Mo=R,_o=Mo+(Oo<<3)|0,_do_fio(1236,_o,8)|0,Do=HEAP32[332]|0,Po=Do+1|0,HEAP32[332]=Po}_e_wsfe()|0}return L=0,Zp=L,STACKTOP=qd,Zp|0}}Ho=HEAP32[335]|0,HEAP32[kd>>2]=Ho,HEAP32[333]=1;for(;;){Bo=HEAP32[333]|0,jo=HEAP32[kd>>2]|0,Fo=(Bo|0)<=(jo|0);if(!Fo)break;Io=HEAP32[333]|0,Uo=Xn,zo=Uo+(Io<<3)|0,HEAPF64[zo>>3]=0,Wo=HEAP32[333]|0,Xo=Wo+1|0,HEAP32[333]=Xo}Vo=Zt,$o=+HEAPF64[Vo>>3],Jo=-$o,Ko=+HEAPF64[26],Qo=Jo*Ko,HEAPF64[27]=Qo,Go=+HEAPF64[27],HEAPF64[43]=Go,Zo=cn,eu=HEAP32[Zo>>2]|0,tu=(eu|0)>0;do if(tu){nu=Y,ru=_d,iu=Mt,su=iu+(ru<<3)|0,ou=Bd,uu=Ut,au=uu+(ou<<3)|0,lu=cn,cu=Pn,hu=cu+8|0,pu=pr,du=pu+8|0,vu=ri,_bmv_(nu,su,au,lu,hu,du,vu)|0,mu=ri,gu=HEAP32[mu>>2]|0,yu=(gu|0)!=0;if(!yu){bu=pr,Eu=bu+8|0,Su=Pn,xu=Su+8|0,Tu=+_ddot_(1340,Eu,1236,xu,1236),Nu=+HEAPF64[27],Cu=Nu-Tu,HEAPF64[27]=Cu;break}return L=0,Zp=L,STACKTOP=qd,Zp|0}while(0);ku=+HEAPF64[26],Lu=-ku,Au=+HEAPF64[27],Ou=Lu/Au,HEAPF64[33]=Ou,HEAPF64[39]=0,_u=Tr,HEAP32[_u>>2]=1,Du=Hr,Pu=HEAP32[Du>>2]|0,Hu=(Pu|0)>=99,Hu&&(_s_wsle(1440)|0,_do_lio(1308,1236,15092,10)|0,_do_lio(1660,1236,1360,4)|0,_do_lio(1308,1236,15103,14)|0,_e_wsle()|0),Bu=HEAP32[340]|0,ju=(Bu|0)==0;e:do if(ju)Id=95;else{Fu=HEAP32[340]|0,HEAP32[339]=Fu,HEAP32[336]=1,HEAPF64[29]=0;for(;;){Iu=+HEAPF64[29],HEAPF64[32]=Iu,qu=HEAP32[336]|0,Ru=(qu|0)==1,Ru?(zu=+HEAPF64[42],HEAPF64[29]=zu,Wu=HEAP32[341]|0,Xu=Qh,Vu=Xu+(Wu<<2)|0,$u=HEAP32[Vu>>2]|0,HEAP32[334]=$u):(Ju=HEAP32[336]|0,Ku=(Ju|0)==2,Ku&&(Qu=HEAP32[341]|0,Gu=HEAP32[340]|0,Yu=(Qu|0)!=(Gu|0),Yu&&(ea=HEAP32[340]|0,ta=vd,na=ta+(ea<<3)|0,ra=+HEAPF64[na>>3],ia=HEAP32[341]|0,sa=vd,oa=sa+(ia<<3)|0,HEAPF64[oa>>3]=ra,ua=HEAP32[340]|0,aa=Qh,fa=aa+(ua<<2)|0,ca=HEAP32[fa>>2]|0,ha=HEAP32[341]|0,pa=Qh,da=pa+(ha<<2)|0,HEAP32[da>>2]=ca)),va=HEAP32[336]|0,ma=va-2|0,HEAP32[kd>>2]=ma,ga=vd,ya=ga+8|0,ba=Qh,wa=ba+4|0,_hpsolb_(1356,ya,wa,kd)|0,Sa=HEAP32[339]|0,xa=vd,Ta=xa+(Sa<<3)|0,Na=+HEAPF64[Ta>>3],HEAPF64[29]=Na,Ca=HEAP32[339]|0,ka=Qh,La=ka+(Ca<<2)|0,Aa=HEAP32[La>>2]|0,HEAP32[334]=Aa),Oa=+HEAPF64[29],Ma=+HEAPF64[32],Da=Oa-Ma,HEAPF64[28]=Da,Pa=+HEAPF64[28],Ha=Pa!=0,Ha&&(Ba=Hr,ja=HEAP32[Ba>>2]|0,Fa=(ja|0)>=100,Fa&&(_s_wsfe(1460)|0,Ia=Tr,_do_fio(1236,Ia,4)|0,_do_fio(1236,208,8)|0,_do_fio(1236,216,8)|0,_e_wsfe()|0,_s_wsfe(1480)|0,_do_fio(1236,224,8)|0,_e_wsfe()|0,_s_wsfe(1500)|0,_do_fio(1236,264,8)|0,_e_wsfe()|0)),qa=+HEAPF64[33],Ra=+HEAPF64[28],Ua=qa<Ra;if(Ua){Id=95;break e}Xa=+HEAPF64[28],Va=+HEAPF64[39],$a=Va+Xa,HEAPF64[39]=$a,Ja=HEAP32[339]|0,Ka=Ja+ -1|0,HEAP32[339]=Ka,Qa=HEAP32[336]|0,Ga=Qa+1|0,HEAP32[336]=Ga,Ya=HEAP32[334]|0,Za=O,ef=Za+(Ya<<3)|0,nf=+HEAPF64[ef>>3],HEAPF64[37]=nf,rf=HEAP32[334]|0,sf=O,of=sf+(rf<<3)|0,HEAPF64[of>>3]=0,uf=+HEAPF64[37],af=uf>0,ff=HEAP32[334]|0,af?(lf=qo,cf=lf+(ff<<3)|0,hf=+HEAPF64[cf>>3],df=HEAP32[334]|0,vf=Dn,mf=vf+(df<<3)|0,gf=+HEAPF64[mf>>3],yf=hf-gf,HEAPF64[38]=yf,bf=HEAP32[334]|0,wf=qo,Ef=wf+(bf<<3)|0,Sf=+HEAPF64[Ef>>3],xf=HEAP32[334]|0,Nf=R,Cf=Nf+(xf<<3)|0,HEAPF64[Cf>>3]=Sf,kf=HEAP32[334]|0,Lf=id,Af=Lf+(kf<<2)|0,HEAP32[Af>>2]=2):(Of=Bi,Mf=Of+(ff<<3)|0,_f=+HEAPF64[Mf>>3],Df=HEAP32[334]|0,Pf=Dn,Bf=Pf+(Df<<3)|0,jf=+HEAPF64[Bf>>3],Ff=_f-jf,HEAPF64[38]=Ff,If=HEAP32[334]|0,qf=Bi,Rf=qf+(If<<3)|0,Uf=+HEAPF64[Rf>>3],zf=HEAP32[334]|0,Wf=R,Xf=Wf+(zf<<3)|0,HEAPF64[Xf>>3]=Uf,$f=HEAP32[334]|0,Jf=id,Kf=Jf+($f<<2)|0,HEAP32[Kf>>2]=1),Qf=Hr,Gf=HEAP32[Qf>>2]|0,Yf=(Gf|0)>=100,Yf&&(_s_wsle(1520)|0,_do_lio(1308,1236,15118,10)|0,_do_lio(1660,1236,1336,4)|0,_do_lio(1308,1236,15129,11)|0,_e_wsle()|0),Zf=HEAP32[339]|0,el=(Zf|0)==0;if(el){tl=HEAP32[340]|0,nl=A,il=HEAP32[nl>>2]|0,sl=(tl|0)==(il|0);if(sl){Id=82;break}}ul=Tr,al=HEAP32[ul>>2]|0,fl=al+1|0,HEAP32[ul>>2]=fl,ll=+HEAPF64[37],HEAPF64[Cd>>3]=ll,cl=+HEAPF64[Cd>>3],hl=+HEAPF64[Cd>>3],pl=cl*hl,HEAPF64[40]=pl,vl=+HEAPF64[26],ml=+HEAPF64[28],gl=+HEAPF64[27],yl=ml*gl,bl=vl+yl,wl=+HEAPF64[40],El=bl+wl,Sl=Zt,xl=+HEAPF64[Sl>>3],Tl=+HEAPF64[37],Cl=xl*Tl,kl=+HEAPF64[38],Ll=Cl*kl,Al=El-Ll,HEAPF64[26]=Al,Ol=Zt,Ml=+HEAPF64[Ol>>3],_l=+HEAPF64[40],Dl=Ml*_l,Pl=+HEAPF64[27],Hl=Pl-Dl,HEAPF64[27]=Hl,jl=cn,Fl=HEAP32[jl>>2]|0,Il=(Fl|0)>0;if(Il){ql=Pn,Rl=ql+8|0,Ul=Xn,zl=Ul+8|0,_daxpy_(1340,224,Rl,1236,zl,1236)|0,Wl=Sn,Xl=HEAP32[Wl>>2]|0,HEAP32[342]=Xl,Vl=cn,Kl=HEAP32[Vl>>2]|0,HEAP32[kd>>2]=Kl,HEAP32[333]=1;for(;;){Ql=HEAP32[333]|0,Gl=HEAP32[kd>>2]|0,Yl=(Ql|0)<=(Gl|0);if(!Yl)break;Zl=HEAP32[334]|0,ec=HEAP32[342]|0,tc=jd,nc=Math_imul(ec,tc)|0,rc=Zl+nc|0,ic=ft,oc=ic+(rc<<3)|0,uc=+HEAPF64[oc>>3],ac=HEAP32[333]|0,fc=nr,lc=fc+(ac<<3)|0,HEAPF64[lc>>3]=uc,cc=Zt,hc=+HEAPF64[cc>>3],pc=HEAP32[334]|0,dc=HEAP32[342]|0,vc=Dd,gc=Math_imul(dc,vc)|0,yc=pc+gc|0,bc=wt,wc=bc+(yc<<3)|0,Ec=+HEAPF64[wc>>3],Sc=hc*Ec,xc=cn,Tc=HEAP32[xc>>2]|0,Nc=HEAP32[333]|0,Cc=Tc+Nc|0,Lc=nr,Ac=Lc+(Cc<<3)|0,HEAPF64[Ac>>3]=Sc,Oc=HEAP32[342]|0,Mc=Y,_c=HEAP32[Mc>>2]|0,Dc=(Oc|0)%(_c|0)&-1,Pc=Dc+1|0,HEAP32[342]=Pc,Hc=HEAP32[333]|0,Bc=Hc+1|0,HEAP32[333]=Bc}jc=Y,Ic=_d,qc=Mt,Rc=qc+(Ic<<3)|0,Uc=Bd,zc=Ut,Wc=zc+(Uc<<3)|0,Xc=cn,Vc=nr,$c=Vc+8|0,Jc=pr,Qc=Jc+8|0,Gc=ri,_bmv_(jc,Rc,Wc,Xc,$c,Qc,Gc)|0,Yc=ri,Zc=HEAP32[Yc>>2]|0,eh=(Zc|0)!=0;if(eh){Id=88;break}th=Xn,nh=th+8|0,rh=pr,ih=rh+8|0,sh=+_ddot_(1340,nh,1236,ih,1236),HEAPF64[34]=sh,uh=Pn,ah=uh+8|0,fh=pr,lh=fh+8|0,ch=+_ddot_(1340,ah,1236,lh,1236),HEAPF64[35]=ch,hh=nr,ph=hh+8|0,dh=pr,vh=dh+8|0,mh=+_ddot_(1340,ph,1236,vh,1236),HEAPF64[36]=mh,yh=+HEAPF64[37],bh=-yh,HEAPF64[Cd>>3]=bh,wh=nr,Eh=wh+8|0,Sh=Pn,xh=Sh+8|0,_daxpy_(1340,Cd,Eh,1236,xh,1236)|0,Th=+HEAPF64[37],Nh=+HEAPF64[34],Ch=Th*Nh,kh=+HEAPF64[26],Ah=kh+Ch,HEAPF64[26]=Ah,Oh=+HEAPF64[27],Mh=+HEAPF64[37],_h=Mh*2,Dh=+HEAPF64[35],Ph=_h*Dh,Hh=Oh+Ph,Bh=+HEAPF64[40],jh=+HEAPF64[36],Fh=Bh*jh,qh=Hh-Fh,HEAPF64[27]=qh}Rh=di,Uh=+HEAPF64[Rh>>3],zh=+HEAPF64[43],Wh=Uh*zh,HEAPF64[Cd>>3]=Wh,Xh=+HEAPF64[Cd>>3],Vh=+HEAPF64[27],$h=Xh>=Vh,Jh=+HEAPF64[Cd>>3],Kh=+HEAPF64[27],Yh=$h?Jh:Kh,HEAPF64[27]=Yh,Zh=HEAP32[339]|0,ep=(Zh|0)>0;if(!ep){Id=92;break}tp=+HEAPF64[26],np=-tp,rp=+HEAPF64[27],ip=np/rp,HEAPF64[33]=ip}if((Id|0)==82){ol=+HEAPF64[28],HEAPF64[33]=ol;break}if((Id|0)==88)return L=0,Zp=L,STACKTOP=qd,Zp|0;if((Id|0)==92){sp=HEAP32[337]|0,op=(sp|0)!=0;if(op){HEAPF64[26]=0,HEAPF64[27]=0,HEAPF64[33]=0,Id=95;break}up=+HEAPF64[26],fp=-up,lp=+HEAPF64[27],cp=fp/lp,HEAPF64[33]=cp,Id=95;break}}while(0);(Id|0)==95&&(hp=Hr,pp=HEAP32[hp>>2]|0,dp=(pp|0)>=99,dp&&(_s_wsle(1540)|0,_e_wsle()|0,_s_wsle(1560)|0,_do_lio(1308,1236,15141,25)|0,_e_wsle()|0,_s_wsfe(1580)|0,vp=Tr,_do_fio(1236,vp,4)|0,_do_fio(1236,208,8)|0,_do_fio(1236,216,8)|0,_e_wsfe()|0,_s_wsfe(1600)|0,_do_fio(1236,264,8)|0,_e_wsfe()|0),mp=+HEAPF64[33],gp=mp<=0,gp&&(HEAPF64[33]=0),yp=+HEAPF64[33],wp=+HEAPF64[39],Ep=wp+yp,HEAPF64[39]=Ep,Sp=A,xp=O,Tp=xp+8|0,Np=R,Cp=Np+8|0,_daxpy_(Sp,312,Tp,1236,Cp,1236)|0),kp=cn,Lp=HEAP32[kp>>2]|0,Ap=(Lp|0)>0,Ap&&(Mp=Pn,_p=Mp+8|0,Dp=Xn,Pp=Dp+8|0,_daxpy_(1340,264,_p,1236,Pp,1236)|0),Hp=Hr,Bp=HEAP32[Hp>>2]|0,jp=(Bp|0)>100;if(jp){_s_wsfe(1620)|0,Fp=A,Ip=HEAP32[Fp>>2]|0,HEAP32[kd>>2]=Ip,HEAP32[332]=1;for(;;){qp=HEAP32[332]|0,Up=HEAP32[kd>>2]|0,zp=(qp|0)<=(Up|0);if(!zp)break;Wp=HEAP32[332]|0,Xp=R,Vp=Xp+(Wp<<3)|0,_do_fio(1236,Vp,8)|0,$p=HEAP32[332]|0,Jp=$p+1|0,HEAP32[332]=Jp}_e_wsfe()|0}return Kp=Hr,Qp=HEAP32[Kp>>2]|0,Gp=(Qp|0)>=99,Gp&&(_s_wsfe(1640)|0,_e_wsfe()|0),L=0,Zp=L,STACKTOP=qd,Zp|0}function _freev_(e,t,n,r,i,s,o,u,a,f,l,c){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0;var h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0;tr=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Zn=tr,h=e,p=t,yt=n,At=r,qt=i,Gt=s,fn=o,wn=u,Mn=a,Un=f,d=l,N=c,B=fn,$=B+ -4|0,fn=$,it=Gt,pt=it+ -4|0,Gt=pt,dt=yt,vt=dt+ -4|0,yt=vt,mt=At,HEAP32[mt>>2]=0,gt=h,bt=HEAP32[gt>>2]|0,wt=bt+1|0,Et=qt,HEAP32[Et>>2]=wt,St=N,xt=HEAP32[St>>2]|0,Tt=(xt|0)>0;if(Tt){Nt=Un,Ct=HEAP32[Nt>>2]|0,kt=(Ct|0)!=0;if(kt){Lt=p,Ot=HEAP32[Lt>>2]|0,HEAP32[Zn>>2]=Ot,HEAP32[443]=1;for(;;){Mt=HEAP32[443]|0,_t=HEAP32[Zn>>2]|0,Dt=(Mt|0)<=(_t|0);if(!Dt)break;Pt=HEAP32[443]|0,Ht=yt,Bt=Ht+(Pt<<2)|0,jt=HEAP32[Bt>>2]|0,HEAP32[444]=jt,Ft=HEAP32[444]|0,It=fn,Rt=It+(Ft<<2)|0,Ut=HEAP32[Rt>>2]|0,zt=(Ut|0)>0,zt&&(Wt=qt,Xt=HEAP32[Wt>>2]|0,Vt=Xt+ -1|0,HEAP32[Wt>>2]=Vt,$t=HEAP32[444]|0,Jt=qt,Kt=HEAP32[Jt>>2]|0,Qt=Gt,Yt=Qt+(Kt<<2)|0,HEAP32[Yt>>2]=$t,Zt=d,en=HEAP32[Zt>>2]|0,tn=(en|0)>=100,tn&&(_s_wsle(1784)|0,_do_lio(1308,1236,15266,9)|0,_do_lio(1660,1236,1776,4)|0,_do_lio(1308,1236,15276,33)|0,_e_wsle()|0)),nn=HEAP32[443]|0,rn=nn+1|0,HEAP32[443]=rn}sn=h,on=HEAP32[sn>>2]|0,HEAP32[Zn>>2]=on,un=p,an=HEAP32[un>>2]|0,ln=an+1|0,HEAP32[443]=ln;for(;;){cn=HEAP32[443]|0,hn=HEAP32[Zn>>2]|0,pn=(cn|0)<=(hn|0);if(!pn)break;dn=HEAP32[443]|0,vn=yt,mn=vn+(dn<<2)|0,gn=HEAP32[mn>>2]|0,HEAP32[444]=gn,yn=HEAP32[444]|0,bn=fn,En=bn+(yn<<2)|0,Sn=HEAP32[En>>2]|0,xn=(Sn|0)<=0,xn&&(Tn=At,Nn=HEAP32[Tn>>2]|0,Cn=Nn+1|0,HEAP32[Tn>>2]=Cn,kn=HEAP32[444]|0,Ln=At,An=HEAP32[Ln>>2]|0,On=Gt,_n=On+(An<<2)|0,HEAP32[_n>>2]=kn,Dn=d,Pn=HEAP32[Dn>>2]|0,Hn=(Pn|0)>=100,Hn&&(_s_wsle(1804)|0,_do_lio(1308,1236,15266,9)|0,_do_lio(1660,1236,1776,4)|0,_do_lio(1308,1236,15310,33)|0,_e_wsle()|0)),Bn=HEAP32[443]|0,jn=Bn+1|0,HEAP32[443]=jn}Fn=d,In=HEAP32[Fn>>2]|0,qn=(In|0)>=99,qn&&(_s_wsle(1824)|0,Rn=h,zn=HEAP32[Rn>>2]|0,Wn=zn+1|0,Xn=qt,Vn=HEAP32[Xn>>2]|0,$n=Wn-Vn|0,HEAP32[Zn>>2]=$n,_do_lio(1660,1236,Zn,4)|0,_do_lio(1308,1236,15344,18)|0,Jn=At,_do_lio(1660,1236,Jn,4)|0,_do_lio(1308,1236,15363,16)|0,_e_wsle()|0)}}Kn=qt,Qn=HEAP32[Kn>>2]|0,Gn=h,Yn=HEAP32[Gn>>2]|0,v=Yn+1|0,m=(Qn|0)<(v|0),m?T=1:(g=At,y=HEAP32[g>>2]|0,b=(y|0)>0,b?T=1:(w=Mn,E=HEAP32[w>>2]|0,S=(E|0)!=0,T=S)),x=T&1,C=wn,HEAP32[C>>2]=x,k=p,HEAP32[k>>2]=0,L=h,A=HEAP32[L>>2]|0,O=A+1|0,HEAP32[445]=O,M=h,_=HEAP32[M>>2]|0,HEAP32[Zn>>2]=_,HEAP32[443]=1;for(;;){D=HEAP32[443]|0,P=HEAP32[Zn>>2]|0,H=(D|0)<=(P|0);if(!H)break;j=HEAP32[443]|0,F=fn,I=F+(j<<2)|0,q=HEAP32[I>>2]|0,R=(q|0)<=0,R?(U=p,z=HEAP32[U>>2]|0,W=z+1|0,HEAP32[U>>2]=W,X=HEAP32[443]|0,V=p,J=HEAP32[V>>2]|0,K=yt,Q=K+(J<<2)|0,HEAP32[Q>>2]=X):(G=HEAP32[445]|0,Y=G+ -1|0,HEAP32[445]=Y,Z=HEAP32[443]|0,et=HEAP32[445]|0,tt=yt,nt=tt+(et<<2)|0,HEAP32[nt>>2]=Z),rt=HEAP32[443]|0,st=rt+1|0,HEAP32[443]=st}return ot=d,ut=HEAP32[ot>>2]|0,at=(ut|0)>=99,at?(_s_wsle(1844)|0,ft=p,_do_lio(1660,1236,ft,4)|0,_do_lio(1308,1236,15380,27)|0,lt=N,ct=HEAP32[lt>>2]|0,ht=ct+1|0,HEAP32[Zn>>2]=ht,_do_lio(1660,1236,Zn,4)|0,_e_wsle()|0,STACKTOP=tr,0):(STACKTOP=tr,0)}function _formk_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0;var y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0
,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0,Rd=0,Ud=0,zd=0,Wd=0,Xd=0,Vd=0,$d=0,Jd=0,Kd=0,Qd=0,Gd=0,Yd=0,Zd=0,ev=0,tv=0,nv=0,rv=0,iv=0,sv=0,ov=0,uv=0,av=0,fv=0,lv=0,cv=0,hv=0,pv=0,dv=0,vv=0,mv=0,gv=0,yv=0,bv=0,wv=0,Ev=0,Sv=0,xv=0,Tv=0,Nv=0,Cv=0,kv=0,Lv=0,Av=0,Ov=0,Mv=0,_v=0,Dv=0,Pv=0,Hv=0,Bv=0,jv=0,Fv=0,Iv=0,qv=0,Rv=0,Uv=0,zv=0,Wv=0,Xv=0,Vv=0,$v=0,Jv=0,Kv=0,Qv=0,Gv=0,Yv=0,Zv=0,em=0,tm=0,nm=0,rm=0,im=0,sm=0,om=0,um=0,am=0,fm=0,lm=0,cm=0,hm=0,pm=0,dm=0,vm=0,mm=0,gm=0,ym=0,bm=0,wm=0,Em=0,Sm=0,xm=0,Tm=0,Nm=0,Cm=0,km=0,Lm=0,Am=0,Om=0,Mm=0,_m=0,Dm=0,Pm=0,Hm=0,Bm=0,jm=0,Fm=0,Im=0,qm=0,Rm=0,Um=0,zm=0,Wm=0,Xm=0,Vm=0,$m=0,Jm=0,Km=0,Qm=0,Gm=0,Ym=0,Zm=0,eg=0,tg=0,ng=0,rg=0,ig=0,sg=0,og=0,ug=0,ag=0,fg=0,lg=0,cg=0;cg=STACKTOP,STACKTOP=STACKTOP+128|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Ym=cg+4|0,b=e,xn=t,Ci=n,Oo=r,Da=i,jl=s,qh=o,zd=u,qm=a,w=f,M=l,U=c,Z=h,lt=p,Et=d,_t=v,zt=m,en=g,hn=jl,Tn=hn+ -4|0,jl=Tn,Hn=Ci,Vn=Hn+ -4|0,Ci=Vn,rr=M,dr=HEAP32[rr>>2]|0,eg=dr,Nr=eg,Br=1+Nr|0,tg=Br,$r=tg,ii=lt,vi=0-$r|0,ki=ii+(vi<<3)|0,lt=ki,Fi=b,Ki=HEAP32[Fi>>2]|0,ag=Ki,os=ag,gs=1+os|0,fg=gs,Ls=fg,Is=Z,Qs=0-Ls|0,uo=Is+(Qs<<3)|0,Z=uo,bo=b,Mo=HEAP32[bo>>2]|0,og=Mo,Uo=og,Zo=1+Uo|0,ug=Zo,lu=ug,Eu=U,_u=0-lu|0,zu=Eu+(_u<<3)|0,U=zu,ea=M,ca=HEAP32[ea>>2]|0,Sa=ca<<1,ng=Sa,Pa=ng,Xa=1+Pa|0,rg=Xa,nf=rg,df=w,Nf=0-nf|0,Bf=df+(Nf<<3)|0,w=Bf,$f=M,il=HEAP32[$f>>2]|0,vl=il<<1,ig=vl,Cl=ig,Fl=1+Cl|0,sg=Fl,Kl=sg,oc=qm,gc=0-Kl|0,Lc=oc+(gc<<3)|0,qm=Lc,Ic=zd,Qc=HEAP32[Ic>>2]|0,uh=(Qc|0)!=0;if(uh){yh=qh,Ah=HEAP32[yh>>2]|0,Rh=M,Yh=HEAP32[Rh>>2]|0,fp=(Ah|0)>(Yh|0);e:do if(fp){wp=M,Mp=HEAP32[wp>>2]|0,Up=Mp-1|0,Gm=Up,HEAP32[428]=1;for(;;){Zp=HEAP32[428]|0,ld=Gm,Ed=(Zp|0)<=(ld|0);if(!Ed)break e;_d=M,Wd=HEAP32[_d>>2]|0,tv=HEAP32[428]|0,hv=Wd+tv|0,HEAP32[426]=hv,xv=M,Pv=HEAP32[xv>>2]|0,Xv=HEAP32[428]|0,nm=Pv-Xv|0,HEAP32[Ym>>2]=nm,pm=HEAP32[428]|0,Tm=pm+1|0,Hm=HEAP32[428]|0,Rm=Hm+1|0,Um=ng,zm=Math_imul(Rm,Um)|0,Wm=Tm+zm|0,Xm=w,Vm=Xm+(Wm<<3)|0,$m=HEAP32[428]|0,Jm=HEAP32[428]|0,Km=ng,Qm=Math_imul(Jm,Km)|0,E=$m+Qm|0,S=w,x=S+(E<<3)|0,_dcopy_(Ym,Vm,1236,x,1236)|0,T=M,N=HEAP32[T>>2]|0,C=HEAP32[428]|0,k=N-C|0,HEAP32[Ym>>2]=k,L=HEAP32[426]|0,A=L+1|0,O=HEAP32[426]|0,_=O+1|0,D=ng,P=Math_imul(_,D)|0,H=A+P|0,B=w,j=B+(H<<3)|0,F=HEAP32[426]|0,I=HEAP32[426]|0,q=ng,R=Math_imul(I,q)|0,z=F+R|0,W=w,X=W+(z<<3)|0,_dcopy_(Ym,j,1236,X,1236)|0,V=M,$=HEAP32[V>>2]|0,J=$-1|0,HEAP32[Ym>>2]=J,K=M,Q=HEAP32[K>>2]|0,G=Q+2|0,Y=HEAP32[428]|0,et=Y+1|0,tt=ng,nt=Math_imul(et,tt)|0,rt=G+nt|0,it=w,st=it+(rt<<3)|0,ot=M,ut=HEAP32[ot>>2]|0,at=ut+1|0,ft=HEAP32[428]|0,ct=ng,ht=Math_imul(ft,ct)|0,pt=at+ht|0,dt=w,vt=dt+(pt<<3)|0,_dcopy_(Ym,st,1236,vt,1236)|0,mt=HEAP32[428]|0,gt=mt+1|0,HEAP32[428]=gt}}while(0);HEAP32[438]=1,yt=xn,bt=HEAP32[yt>>2]|0,HEAP32[433]=bt,wt=xn,St=HEAP32[wt>>2]|0,xt=St+1|0,HEAP32[437]=xt,Tt=b,Nt=HEAP32[Tt>>2]|0,HEAP32[432]=Nt,Ct=_t,kt=HEAP32[Ct>>2]|0,HEAP32[427]=kt,Lt=M,At=HEAP32[Lt>>2]|0,Ot=_t,Mt=HEAP32[Ot>>2]|0,Dt=At+Mt|0,HEAP32[425]=Dt,Pt=zt,Ht=HEAP32[Pt>>2]|0,Bt=_t,jt=HEAP32[Bt>>2]|0,Ft=Ht+jt|0,It=Ft-1|0,HEAP32[435]=It,qt=HEAP32[435]|0,Rt=M,Ut=HEAP32[Rt>>2]|0,Wt=(qt|0)>(Ut|0),Wt&&(Xt=M,Vt=HEAP32[Xt>>2]|0,$t=HEAP32[435]|0,Jt=$t-Vt|0,HEAP32[435]=Jt),Kt=zt,Qt=HEAP32[Kt>>2]|0,HEAP32[436]=Qt,Gt=_t,Yt=HEAP32[Gt>>2]|0,Gm=Yt,HEAP32[428]=1;for(;;){Zt=HEAP32[428]|0,tn=Gm,nn=(Zt|0)<=(tn|0);if(!nn)break;rn=M,sn=HEAP32[rn>>2]|0,on=HEAP32[428]|0,un=sn+on|0,HEAP32[426]=un,HEAPF64[46]=0,HEAPF64[47]=0,HEAPF64[48]=0,an=HEAP32[433]|0,HEAP32[Ym>>2]=an,fn=HEAP32[438]|0,HEAP32[422]=fn;for(;;){ln=HEAP32[422]|0,cn=HEAP32[Ym>>2]|0,pn=(ln|0)<=(cn|0);if(!pn)break;dn=HEAP32[422]|0,vn=Ci,mn=vn+(dn<<2)|0,gn=HEAP32[mn>>2]|0,HEAP32[423]=gn,yn=HEAP32[423]|0,bn=HEAP32[435]|0,wn=ag,En=Math_imul(bn,wn)|0,Sn=yn+En|0,Nn=Z,Cn=Nn+(Sn<<3)|0,kn=+HEAPF64[Cn>>3],Ln=HEAP32[423]|0,An=HEAP32[436]|0,On=ag,Mn=Math_imul(An,On)|0,_n=Ln+Mn|0,Dn=Z,Pn=Dn+(_n<<3)|0,Bn=+HEAPF64[Pn>>3],jn=kn*Bn,Fn=+HEAPF64[46],In=Fn+jn,HEAPF64[46]=In,qn=HEAP32[422]|0,Rn=qn+1|0,HEAP32[422]=Rn}Un=HEAP32[432]|0,HEAP32[Ym>>2]=Un,zn=HEAP32[437]|0,HEAP32[422]=zn;for(;;){Wn=HEAP32[422]|0,Xn=HEAP32[Ym>>2]|0,$n=(Wn|0)<=(Xn|0);if(!$n)break;Jn=HEAP32[422]|0,Kn=Ci,Qn=Kn+(Jn<<2)|0,Gn=HEAP32[Qn>>2]|0,HEAP32[423]=Gn,Yn=HEAP32[423]|0,Zn=HEAP32[435]|0,er=og,tr=Math_imul(Zn,er)|0,nr=Yn+tr|0,ir=U,sr=ir+(nr<<3)|0,or=+HEAPF64[sr>>3],ur=HEAP32[423]|0,ar=HEAP32[436]|0,fr=og,lr=Math_imul(ar,fr)|0,cr=ur+lr|0,hr=U,pr=hr+(cr<<3)|0,vr=+HEAPF64[pr>>3],mr=or*vr,gr=+HEAPF64[47],yr=gr+mr,HEAPF64[47]=yr,br=HEAP32[423]|0,wr=HEAP32[435]|0,Er=og,Sr=Math_imul(wr,Er)|0,xr=br+Sr|0,Tr=U,Cr=Tr+(xr<<3)|0,kr=+HEAPF64[Cr>>3],Lr=HEAP32[423]|0,Ar=HEAP32[436]|0,Or=ag,Mr=Math_imul(Ar,Or)|0,_r=Lr+Mr|0,Dr=Z,Pr=Dr+(_r<<3)|0,Hr=+HEAPF64[Pr>>3],jr=kr*Hr,Fr=+HEAPF64[48],Ir=Fr+jr,HEAPF64[48]=Ir,qr=HEAP32[422]|0,Rr=qr+1|0,HEAP32[422]=Rr}Ur=+HEAPF64[46],zr=HEAP32[427]|0,Wr=HEAP32[428]|0,Xr=ng,Vr=Math_imul(Wr,Xr)|0,Jr=zr+Vr|0,Kr=w,Qr=Kr+(Jr<<3)|0,HEAPF64[Qr>>3]=Ur,Gr=+HEAPF64[47],Yr=HEAP32[425]|0,Zr=HEAP32[426]|0,ei=ng,ti=Math_imul(Zr,ei)|0,ni=Yr+ti|0,ri=w,si=ri+(ni<<3)|0,HEAPF64[si>>3]=Gr,oi=+HEAPF64[48],ui=HEAP32[425]|0,ai=HEAP32[428]|0,fi=ng,li=Math_imul(ai,fi)|0,ci=ui+li|0,hi=w,pi=hi+(ci<<3)|0,HEAPF64[pi>>3]=oi,di=HEAP32[436]|0,mi=M,gi=HEAP32[mi>>2]|0,yi=(di|0)%(gi|0)&-1,bi=yi+1|0,HEAP32[436]=bi,wi=HEAP32[428]|0,Ei=wi+1|0,HEAP32[428]=Ei}Si=_t,xi=HEAP32[Si>>2]|0,HEAP32[428]=xi,Ti=zt,Ni=HEAP32[Ti>>2]|0,Li=_t,Ai=HEAP32[Li>>2]|0,Oi=Ni+Ai|0,Mi=Oi-1|0,HEAP32[436]=Mi,_i=HEAP32[436]|0,Di=M,Pi=HEAP32[Di>>2]|0,Hi=(_i|0)>(Pi|0),Hi&&(Bi=M,ji=HEAP32[Bi>>2]|0,Ii=HEAP32[436]|0,qi=Ii-ji|0,HEAP32[436]=qi),Ri=zt,Ui=HEAP32[Ri>>2]|0,HEAP32[435]=Ui,zi=_t,Wi=HEAP32[zi>>2]|0,Gm=Wi,HEAP32[421]=1;for(;;){Xi=HEAP32[421]|0,Vi=Gm,$i=(Xi|0)<=(Vi|0);if(!$i)break;Ji=M,Qi=HEAP32[Ji>>2]|0,Gi=HEAP32[421]|0,Yi=Qi+Gi|0,HEAP32[425]=Yi,HEAPF64[48]=0,Zi=HEAP32[433]|0,HEAP32[Ym>>2]=Zi,es=HEAP32[438]|0,HEAP32[422]=es;for(;;){ts=HEAP32[422]|0,ns=HEAP32[Ym>>2]|0,rs=(ts|0)<=(ns|0);if(!rs)break;is=HEAP32[422]|0,ss=Ci,us=ss+(is<<2)|0,as=HEAP32[us>>2]|0,HEAP32[423]=as,fs=HEAP32[423]|0,ls=HEAP32[435]|0,cs=og,hs=Math_imul(ls,cs)|0,ps=fs+hs|0,ds=U,vs=ds+(ps<<3)|0,ms=+HEAPF64[vs>>3],ys=HEAP32[423]|0,bs=HEAP32[436]|0,ws=ag,Es=Math_imul(bs,ws)|0,Ss=ys+Es|0,xs=Z,Ts=xs+(Ss<<3)|0,Ns=+HEAPF64[Ts>>3],Cs=ms*Ns,ks=+HEAPF64[48],As=ks+Cs,HEAPF64[48]=As,Os=HEAP32[422]|0,Ms=Os+1|0,HEAP32[422]=Ms}_s=HEAP32[435]|0,Ds=M,Ps=HEAP32[Ds>>2]|0,Hs=(_s|0)%(Ps|0)&-1,Bs=Hs+1|0,HEAP32[435]=Bs,js=+HEAPF64[48],Fs=HEAP32[425]|0,qs=HEAP32[428]|0,Rs=ng,Us=Math_imul(qs,Rs)|0,zs=Fs+Us|0,Ws=w,Xs=Ws+(zs<<3)|0,HEAPF64[Xs>>3]=js,Vs=HEAP32[421]|0,$s=Vs+1|0,HEAP32[421]=$s}Js=_t,Ks=HEAP32[Js>>2]|0,Gs=Ks-1|0,HEAP32[434]=Gs}else Ys=_t,Zs=HEAP32[Ys>>2]|0,HEAP32[434]=Zs;eo=zt,to=HEAP32[eo>>2]|0,HEAP32[435]=to,no=HEAP32[434]|0,Gm=no,HEAP32[427]=1;for(;;){ro=HEAP32[427]|0,io=Gm,so=(ro|0)<=(io|0);if(!so)break;oo=M,ao=HEAP32[oo>>2]|0,fo=HEAP32[427]|0,lo=ao+fo|0,HEAP32[425]=lo,co=zt,ho=HEAP32[co>>2]|0,HEAP32[436]=ho,po=HEAP32[427]|0,HEAP32[Ym>>2]=po,HEAP32[428]=1;for(;;){vo=HEAP32[428]|0,mo=HEAP32[Ym>>2]|0,go=(vo|0)<=(mo|0);if(!go)break;yo=M,wo=HEAP32[yo>>2]|0,Eo=HEAP32[428]|0,So=wo+Eo|0,HEAP32[426]=So,HEAPF64[46]=0,HEAPF64[47]=0,HEAPF64[48]=0,HEAPF64[49]=0,xo=Oo,To=HEAP32[xo>>2]|0,Zm=To,HEAP32[422]=1;for(;;){No=HEAP32[422]|0,Co=Zm,ko=(No|0)<=(Co|0);if(!ko)break;Lo=HEAP32[422]|0,Ao=jl,_o=Ao+(Lo<<2)|0,Do=HEAP32[_o>>2]|0,HEAP32[423]=Do,Po=HEAP32[423]|0,Ho=HEAP32[435]|0,Bo=ag,jo=Math_imul(Ho,Bo)|0,Fo=Po+jo|0,Io=Z,qo=Io+(Fo<<3)|0,Ro=+HEAPF64[qo>>3],zo=HEAP32[423]|0,Wo=HEAP32[436]|0,Xo=ag,Vo=Math_imul(Wo,Xo)|0,$o=zo+Vo|0,Jo=Z,Ko=Jo+($o<<3)|0,Qo=+HEAPF64[Ko>>3],Go=Ro*Qo,Yo=+HEAPF64[46],eu=Yo+Go,HEAPF64[46]=eu,tu=HEAP32[423]|0,nu=HEAP32[435]|0,ru=og,iu=Math_imul(nu,ru)|0,su=tu+iu|0,ou=U,uu=ou+(su<<3)|0,au=+HEAPF64[uu>>3],fu=HEAP32[423]|0,cu=HEAP32[436]|0,hu=og,pu=Math_imul(cu,hu)|0,du=fu+pu|0,vu=U,mu=vu+(du<<3)|0,gu=+HEAPF64[mu>>3],yu=au*gu,bu=+HEAPF64[47],wu=bu+yu,HEAPF64[47]=wu,Su=HEAP32[422]|0,xu=Su+1|0,HEAP32[422]=xu}Tu=b,Nu=HEAP32[Tu>>2]|0,Zm=Nu,Cu=Da,ku=HEAP32[Cu>>2]|0,HEAP32[422]=ku;for(;;){Lu=HEAP32[422]|0,Au=Zm,Ou=(Lu|0)<=(Au|0);if(!Ou)break;Mu=HEAP32[422]|0,Du=jl,Pu=Du+(Mu<<2)|0,Hu=HEAP32[Pu>>2]|0,HEAP32[423]=Hu,Bu=HEAP32[423]|0,ju=HEAP32[435]|0,Fu=ag,Iu=Math_imul(ju,Fu)|0,qu=Bu+Iu|0,Ru=Z,Uu=Ru+(qu<<3)|0,Wu=+HEAPF64[Uu>>3],Xu=HEAP32[423]|0,Vu=HEAP32[436]|0,$u=ag,Ju=Math_imul(Vu,$u)|0,Ku=Xu+Ju|0,Qu=Z,Gu=Qu+(Ku<<3)|0,Yu=+HEAPF64[Gu>>3],Zu=Wu*Yu,ta=+HEAPF64[48],na=ta+Zu,HEAPF64[48]=na,ra=HEAP32[423]|0,ia=HEAP32[435]|0,sa=og,oa=Math_imul(ia,sa)|0,ua=ra+oa|0,aa=U,fa=aa+(ua<<3)|0,la=+HEAPF64[fa>>3],ha=HEAP32[423]|0,pa=HEAP32[436]|0,da=og,va=Math_imul(pa,da)|0,ma=ha+va|0,ga=U,ya=ga+(ma<<3)|0,ba=+HEAPF64[ya>>3],wa=la*ba,Ea=+HEAPF64[49],xa=Ea+wa,HEAPF64[49]=xa,Ta=HEAP32[422]|0,Na=Ta+1|0,HEAP32[422]=Na}Ca=HEAP32[427]|0,ka=HEAP32[428]|0,La=ng,Aa=Math_imul(ka,La)|0,Oa=Ca+Aa|0,Ma=w,_a=Ma+(Oa<<3)|0,Ha=+HEAPF64[_a>>3],Ba=+HEAPF64[46],ja=Ha+Ba,Fa=+HEAPF64[48],Ia=ja-Fa,qa=HEAP32[427]|0,Ra=HEAP32[428]|0,Ua=ng,za=Math_imul(Ra,Ua)|0,Wa=qa+za|0,Va=w,$a=Va+(Wa<<3)|0,HEAPF64[$a>>3]=Ia,Ja=HEAP32[425]|0,Ka=HEAP32[426]|0,Qa=ng,Ga=Math_imul(Ka,Qa)|0,Ya=Ja+Ga|0,Za=w,ef=Za+(Ya<<3)|0,tf=+HEAPF64[ef>>3],rf=+HEAPF64[47],sf=tf-rf,of=+HEAPF64[49],uf=sf+of,af=HEAP32[425]|0,ff=HEAP32[426]|0,lf=ng,cf=Math_imul(ff,lf)|0,hf=af+cf|0,pf=w,vf=pf+(hf<<3)|0,HEAPF64[vf>>3]=uf,mf=HEAP32[436]|0,gf=M,yf=HEAP32[gf>>2]|0,bf=(mf|0)%(yf|0)&-1,wf=bf+1|0,HEAP32[436]=wf,Ef=HEAP32[428]|0,Sf=Ef+1|0,HEAP32[428]=Sf}xf=HEAP32[435]|0,Tf=M,Cf=HEAP32[Tf>>2]|0,kf=(xf|0)%(Cf|0)&-1,Lf=kf+1|0,HEAP32[435]=Lf,Af=HEAP32[427]|0,Of=Af+1|0,HEAP32[427]=Of}Mf=zt,_f=HEAP32[Mf>>2]|0,HEAP32[435]=_f,Df=M,Pf=HEAP32[Df>>2]|0,Hf=HEAP32[434]|0,jf=Pf+Hf|0,Gm=jf,Ff=M,If=HEAP32[Ff>>2]|0,qf=If+1|0,HEAP32[425]=qf;for(;;){Rf=HEAP32[425]|0,Uf=Gm,zf=(Rf|0)<=(Uf|0);if(!zf)break;Wf=zt,Xf=HEAP32[Wf>>2]|0,HEAP32[436]=Xf,Vf=HEAP32[434]|0,HEAP32[Ym>>2]=Vf,HEAP32[428]=1;for(;;){Jf=HEAP32[428]|0,Kf=HEAP32[Ym>>2]|0,Qf=(Jf|0)<=(Kf|0);if(!Qf)break;HEAPF64[46]=0,HEAPF64[48]=0,Gf=Oo,Yf=HEAP32[Gf>>2]|0,Zm=Yf,HEAP32[422]=1;for(;;){Zf=HEAP32[422]|0,el=Zm,tl=(Zf|0)<=(el|0);if(!tl)break;nl=HEAP32[422]|0,rl=jl,sl=rl+(nl<<2)|0,ol=HEAP32[sl>>2]|0,HEAP32[423]=ol,ul=HEAP32[423]|0,al=HEAP32[435]|0,fl=og,ll=Math_imul(al,fl)|0,cl=ul+ll|0,hl=U,pl=hl+(cl<<3)|0,dl=+HEAPF64[pl>>3],ml=HEAP32[423]|0,gl=HEAP32[436]|0,yl=ag,bl=Math_imul(gl,yl)|0,wl=ml+bl|0,El=Z,Sl=El+(wl<<3)|0,xl=+HEAPF64[Sl>>3],Tl=dl*xl,Nl=+HEAPF64[46],kl=Nl+Tl,HEAPF64[46]=kl,Ll=HEAP32[422]|0,Al=Ll+1|0,HEAP32[422]=Al}Ol=b,Ml=HEAP32[Ol>>2]|0,Zm=Ml,_l=Da,Dl=HEAP32[_l>>2]|0,HEAP32[422]=Dl;for(;;){Pl=HEAP32[422]|0,Hl=Zm,Bl=(Pl|0)<=(Hl|0);if(!Bl)break;Il=HEAP32[422]|0,ql=jl,Rl=ql+(Il<<2)|0,Ul=HEAP32[Rl>>2]|0,HEAP32[423]=Ul,zl=HEAP32[423]|0,Wl=HEAP32[435]|0,Xl=og,Vl=Math_imul(Wl,Xl)|0,$l=zl+Vl|0,Jl=U,Ql=Jl+($l<<3)|0,Gl=+HEAPF64[Ql>>3],Yl=HEAP32[423]|0,Zl=HEAP32[436]|0,ec=ag,tc=Math_imul(Zl,ec)|0,nc=Yl+tc|0,rc=Z,ic=rc+(nc<<3)|0,sc=+HEAPF64[ic>>3],uc=Gl*sc,ac=+HEAPF64[48],fc=ac+uc,HEAPF64[48]=fc,lc=HEAP32[422]|0,cc=lc+1|0,HEAP32[422]=cc}hc=HEAP32[425]|0,pc=HEAP32[428]|0,dc=M,vc=HEAP32[dc>>2]|0,mc=pc+vc|0,yc=(hc|0)<=(mc|0),bc=HEAP32[425]|0,wc=HEAP32[428]|0,Ec=ng,Sc=Math_imul(wc,Ec)|0,xc=bc+Sc|0,Tc=w,Nc=Tc+(xc<<3)|0,Cc=+HEAPF64[Nc>>3],kc=+HEAPF64[46],yc?(Ac=Cc+kc,Oc=+HEAPF64[48],Mc=Ac-Oc,_c=HEAP32[425]|0,Dc=HEAP32[428]|0,Pc=ng,Hc=Math_imul(Dc,Pc)|0,Bc=_c+Hc|0,jc=w,Fc=jc+(Bc<<3)|0,HEAPF64[Fc>>3]=Mc):(qc=Cc-kc,Rc=+HEAPF64[48],Uc=qc+Rc,zc=HEAP32[425]|0,Wc=HEAP32[428]|0,Xc=ng,Vc=Math_imul(Wc,Xc)|0,$c=zc+Vc|0,Jc=w,Kc=Jc+($c<<3)|0,HEAPF64[Kc>>3]=Uc),Gc=HEAP32[436]|0,Yc=M,Zc=HEAP32[Yc>>2]|0,eh=(Gc|0)%(Zc|0)&-1,th=eh+1|0,HEAP32[436]=th,nh=HEAP32[428]|0,rh=nh+1|0,HEAP32[428]=rh}ih=HEAP32[435]|0,sh=M,oh=HEAP32[sh>>2]|0,ah=(ih|0)%(oh|0)&-1,fh=ah+1|0,HEAP32[435]=fh,lh=HEAP32[425]|0,ch=lh+1|0,HEAP32[425]=ch}hh=M,ph=HEAP32[hh>>2]|0,dh=ph<<1,HEAP32[424]=dh,vh=_t,mh=HEAP32[vh>>2]|0,Gm=mh,HEAP32[427]=1;for(;;){gh=HEAP32[427]|0,bh=Gm,wh=(gh|0)<=(bh|0);if(!wh)break;Eh=_t,Sh=HEAP32[Eh>>2]|0,xh=HEAP32[427]|0,Th=Sh+xh|0,HEAP32[425]=Th,Nh=M,Ch=HEAP32[Nh>>2]|0,kh=HEAP32[427]|0,Lh=Ch+kh|0,HEAP32[429]=Lh,Oh=HEAP32[427]|0,HEAP32[Ym>>2]=Oh,HEAP32[428]=1;for(;;){Mh=HEAP32[428]|0,_h=HEAP32[Ym>>2]|0,Dh=(Mh|0)<=(_h|0);if(!Dh)break;Ph=_t,Hh=HEAP32[Ph>>2]|0,Bh=HEAP32[428]|0,jh=Hh+Bh|0,HEAP32[426]=jh,Fh=M,Ih=HEAP32[Fh>>2]|0,Uh=HEAP32[428]|0,zh=Ih+Uh|0,HEAP32[430]=zh,Wh=HEAP32[427]|0,Xh=HEAP32[428]|0,Vh=ng,$h=Math_imul(Xh,Vh)|0,Jh=Wh+$h|0,Kh=w,Qh=Kh+(Jh<<3)|0,Gh=+HEAPF64[Qh>>3],Zh=Et,ep=+HEAPF64[Zh>>3],tp=Gh/ep,np=HEAP32[428]|0,rp=HEAP32[427]|0,ip=ig,sp=Math_imul(rp,ip)|0,op=np+sp|0,up=qm,ap=up+(op<<3)|0,HEAPF64[ap>>3]=tp,lp=HEAP32[429]|0,cp=HEAP32[430]|0,hp=ng,pp=Math_imul(cp,hp)|0,dp=lp+pp|0,vp=w,mp=vp+(dp<<3)|0,gp=+HEAPF64[mp>>3],yp=Et,bp=+HEAPF64[yp>>3],Ep=gp*bp,Sp=HEAP32[426]|0,xp=HEAP32[425]|0,Tp=ig,Np=Math_imul(xp,Tp)|0,Cp=Sp+Np|0,kp=qm,Lp=kp+(Cp<<3)|0,HEAPF64[Lp>>3]=Ep,Ap=HEAP32[428]|0,Op=Ap+1|0,HEAP32[428]=Op}_p=HEAP32[427]|0,Dp=_p-1|0,HEAP32[Ym>>2]=Dp,HEAP32[428]=1;for(;;){Pp=HEAP32[428]|0,Hp=HEAP32[Ym>>2]|0,Bp=(Pp|0)<=(Hp|0);if(!Bp)break;jp=HEAP32[429]|0,Fp=HEAP32[428]|0,Ip=ng,qp=Math_imul(Fp,Ip)|0,Rp=jp+qp|0,zp=w,Wp=zp+(Rp<<3)|0,Xp=+HEAPF64[Wp>>3],Vp=-Xp,$p=HEAP32[428]|0,Jp=HEAP32[425]|0,Kp=ig,Qp=Math_imul(Jp,Kp)|0,Gp=$p+Qp|0,Yp=qm,ed=Yp+(Gp<<3)|0,HEAPF64[ed>>3]=Vp,td=HEAP32[428]|0,nd=td+1|0,HEAP32[428]=nd}rd=_t,id=HEAP32[rd>>2]|0,HEAP32[Ym>>2]=id,sd=HEAP32[427]|0,HEAP32[428]=sd;for(;;){od=HEAP32[428]|0,ud=HEAP32[Ym>>2]|0,ad=(od|0)<=(ud|0);if(!ad)break;fd=HEAP32[429]|0,cd=HEAP32[428]|0,hd=ng,pd=Math_imul(cd,hd)|0,dd=fd+pd|0,vd=w,md=vd+(dd<<3)|0,gd=+HEAPF64[md>>3],yd=HEAP32[428]|0,bd=HEAP32[425]|0,wd=ig,Sd=Math_imul(bd,wd)|0,xd=yd+Sd|0,Td=qm,Nd=Td+(xd<<3)|0,HEAPF64[Nd>>3]=gd,Cd=HEAP32[428]|0,kd=Cd+1|0,HEAP32[428]=kd}Ld=HEAP32[427]|0,Ad=HEAP32[427]|0,Od=eg,Md=Math_imul(Ad,Od)|0,Dd=Ld+Md|0,Pd=lt,Hd=Pd+(Dd<<3)|0,Bd=+HEAPF64[Hd>>3],jd=HEAP32[427]|0,Fd=HEAP32[427]|0,Id=ig,qd=Math_imul(Fd,Id)|0,Rd=jd+qd|0,Ud=qm,Xd=Ud+(Rd<<3)|0,Vd=+HEAPF64[Xd>>3],$d=Vd+Bd,HEAPF64[Xd>>3]=$d,Jd=HEAP32[427]|0,Kd=Jd+1|0,HEAP32[427]=Kd}Qd=sg,Gd=qm,Yd=Gd+(Qd<<3)|0,Zd=_t,ev=en,_dpofa_(Yd,1696,Zd,ev)|0,nv=en,rv=HEAP32[nv>>2]|0,iv=(rv|0)!=0;if(iv)return sv=en,HEAP32[sv>>2]=-1,y=0,Im=y,STACKTOP=cg,Im|0;ov=_t,uv=HEAP32[ov>>2]|0,av=uv<<1,HEAP32[431]=av,fv=HEAP32[431]|0,Gm=fv,lv=_t,cv=HEAP32[lv>>2]|0,pv=cv+1|0,HEAP32[426]=pv;for(;;){dv=HEAP32[426]|0,vv=Gm,mv=(dv|0)<=(vv|0);if(!mv)break;gv=sg,yv=qm,bv=yv+(gv<<3)|0,wv=_t,Ev=HEAP32[426]|0,Sv=ig,Tv=Math_imul(Ev,Sv)|0,Nv=Tv+1|0,Cv=qm,kv=Cv+(Nv<<3)|0,Lv=en,_dtrsl_(bv,1696,wv,kv,1324,Lv)|0,Av=HEAP32[426]|0,Ov=Av+1|0,HEAP32[426]=Ov}Mv=HEAP32[431]|0,Gm=Mv,_v=_t,Dv=HEAP32[_v>>2]|0,Hv=Dv+1|0,HEAP32[425]=Hv;for(;;){Bv=HEAP32[425]|0,jv=Gm,Fv=(Bv|0)<=(jv|0);if(!Fv)break;Iv=HEAP32[431]|0,HEAP32[Ym>>2]=Iv,qv=HEAP32[425]|0,HEAP32[426]=qv;for(;;){Rv=HEAP32[426]|0,Uv=HEAP32[Ym>>2]|0,zv=(Rv|0)<=(Uv|0);if(!zv)break;Wv=_t,Vv=HEAP32[425]|0,$v=ig,Jv=Math_imul(Vv,$v)|0,Kv=Jv+1|0,Qv=qm,Gv=Qv+(Kv<<3)|0,Yv=HEAP32[426]|0,Zv=ig,em=Math_imul(Yv,Zv)|0,tm=em+1|0,rm=qm,im=rm+(tm<<3)|0,sm=+_ddot_(Wv,Gv,1236,im,1236),om=HEAP32[425]|0,um=HEAP32[426]|0,am=ig,fm=Math_imul(um,am)|0,lm=om+fm|0,cm=qm,hm=cm+(lm<<3)|0,dm=+HEAPF64[hm>>3],vm=dm+sm,HEAPF64[hm>>3]=vm,mm=HEAP32[426]|0,gm=mm+1|0,HEAP32[426]=gm}ym=HEAP32[425]|0,bm=ym+1|0,HEAP32[425]=bm}return wm=_t,Em=HEAP32[wm>>2]|0,Sm=Em+1|0,xm=_t,Nm=HEAP32[xm>>2]|0,Cm=Nm+1|0,km=ig,Lm=Math_imul(Cm,km)|0,Am=Sm+Lm|0,Om=qm,Mm=Om+(Am<<3)|0,_m=_t,Dm=en,_dpofa_(Mm,1696,_m,Dm)|0,Pm=en,Bm=HEAP32[Pm>>2]|0,jm=(Bm|0)!=0,jm?(Fm=en,HEAP32[Fm>>2]=-2,y=0,Im=y,STACKTOP=cg,Im|0):(y=0,Im=y,STACKTOP=cg,Im|0)}function _cmprlb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0;var y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0;wi=STACKTOP,STACKTOP=STACKTOP+128|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),b=e,xn=t,qn=n,Gn=r,ar=i,br=s,Or=o,Rr=u,Yr=a,w=f,M=l,U=c,Z=h,lt=p,Et=d,_t=v,zt=m,en=g,hn=U,Tn=hn+ -4|0,U=Tn,Mn=w,_n=Mn+ -8|0,w=_n,Dn=Yr,Pn=Dn+ -8|0,Yr=Pn,Hn=Gn,Bn=Hn+ -8|0,Gn=Bn,jn=qn,Fn=jn+ -8|0,qn=Fn,In=M,Rn=In+ -8|0,M=Rn,Un=xn,zn=HEAP32[Un>>2]|0,vi=zn,Wn=vi,Xn=1+Wn|0,mi=Xn,Vn=mi,$n=Rr,Jn=0-Vn|0,Kn=$n+(Jn<<3)|0,Rr=Kn,Qn=xn,Yn=HEAP32[Qn>>2]|0,ci=Yn,Zn=ci,er=1+Zn|0,hi=er,tr=hi,nr=Or,rr=0-tr|0,ir=nr+(rr<<3)|0,Or=ir,sr=b,or=HEAP32[sr>>2]|0,gi=or,ur=gi,fr=1+ur|0,yi=fr,lr=yi,cr=br,hr=0-lr|0,pr=cr+(hr<<3)|0,br=pr,dr=b,vr=HEAP32[dr>>2]|0,pi=vr,mr=pi,gr=1+mr|0,di=gr,yr=di,wr=ar,Er=0-yr|0,Sr=wr+(Er<<3)|0,ar=Sr,xr=zt,Tr=HEAP32[xr>>2]|0,Nr=(Tr|0)!=0;e:do if(Nr)bi=6;else{Cr=lt,kr=HEAP32[Cr>>2]|0,Lr=(kr|0)>0;if(Lr){Ar=b,Mr=HEAP32[Ar>>2]|0,fi=Mr,HEAP32[416]=1;for(;;){_r=HEAP32[416]|0,Dr=fi,Pr=(_r|0)<=(Dr|0);if(!Pr)break e;Hr=HEAP32[416]|0,Br=Gn,jr=Br+(Hr<<3)|0,Fr=+HEAPF64[jr>>3],Ir=-Fr,qr=HEAP32[416]|0,Ur=w,zr=Ur+(qr<<3)|0,HEAPF64[zr>>3]=Ir,Wr=HEAP32[416]|0,Xr=Wr+1|0,HEAP32[416]=Xr}}else bi=6}while(0);t:do if((bi|0)==6){Vr=_t,$r=HEAP32[Vr>>2]|0,fi=$r,HEAP32[416]=1;for(;;){Jr=HEAP32[416]|0,Kr=fi,Qr=(Jr|0)<=(Kr|0);if(!Qr)break;Gr=HEAP32[416]|0,Zr=U,ei=Zr+(Gr<<2)|0,ti=HEAP32[ei>>2]|0,HEAP32[418]=ti,ni=Z,ri=+HEAPF64[ni>>3],ii=-ri,si=HEAP32[418]|0,oi=Yr,ui=oi+(si<<3)|0,ai=+HEAPF64[ui>>3],E=HEAP32[418]|0,S=qn,x=S+(E<<3)|0,T=+HEAPF64[x>>3],N=ai-T,C=ii*N,k=HEAP32[418]|0,L=Gn,A=L+(k<<3)|0,O=+HEAPF64[A>>3],_=C-O,D=HEAP32[416]|0,P=w,H=P+(D<<3)|0,HEAPF64[H>>3]=_,B=HEAP32[416]|0,j=B+1|0,HEAP32[416]=j}F=xn,I=hi,q=Or,R=q+(I<<3)|0,z=mi,W=Rr,X=W+(z<<3)|0,V=lt,$=xn,J=HEAP32[$>>2]|0,K=J<<1,Q=K+1|0,G=M,Y=G+(Q<<3)|0,et=M,tt=et+8|0,nt=en,_bmv_(F,R,X,V,Y,tt,nt)|0,rt=en,it=HEAP32[rt>>2]|0,st=(it|0)!=0;if(st)return ot=en,HEAP32[ot>>2]=-8,y=0,On=y,STACKTOP=wi,On|0;ut=Et,at=HEAP32[ut>>2]|0,HEAP32[419]=at,ft=lt,ct=HEAP32[ft>>2]|0,fi=ct,HEAP32[417]=1;for(;;){ht=HEAP32[417]|0,pt=fi,dt=(ht|0)<=(pt|0);if(!dt)break t;vt=HEAP32[417]|0,mt=M,gt=mt+(vt<<3)|0,yt=+HEAPF64[gt>>3],HEAPF64[44]=yt,bt=Z,wt=+HEAPF64[bt>>3],St=lt,xt=HEAP32[St>>2]|0,Tt=HEAP32[417]|0,Nt=xt+Tt|0,Ct=M,kt=Ct+(Nt<<3)|0,Lt=+HEAPF64[kt>>3],At=wt*Lt,HEAPF64[45]=At,Ot=_t,Mt=HEAP32[Ot>>2]|0,li=Mt,HEAP32[416]=1;for(;;){Dt=HEAP32[416]|0,Pt=li,Ht=(Dt|0)<=(Pt|0);if(!Ht)break;Bt=HEAP32[416]|0,jt=U,Ft=jt+(Bt<<2)|0,It=HEAP32[Ft>>2]|0,HEAP32[418]=It,qt=HEAP32[416]|0,Rt=w,Ut=Rt+(qt<<3)|0,Wt=+HEAPF64[Ut>>3],Xt=HEAP32[418]|0,Vt=HEAP32[419]|0,$t=gi,Jt=Math_imul(Vt,$t)|0,Kt=Xt+Jt|0,Qt=br,Gt=Qt+(Kt<<3)|0,Yt=+HEAPF64[Gt>>3],Zt=+HEAPF64[44],tn=Yt*Zt,nn=Wt+tn,rn=HEAP32[418]|0,sn=HEAP32[419]|0,on=pi,un=Math_imul(sn,on)|0,an=rn+un|0,fn=ar,ln=fn+(an<<3)|0,cn=+HEAPF64[ln>>3],pn=+HEAPF64[45],dn=cn*pn,vn=nn+dn,mn=HEAP32[416]|0,gn=w,yn=gn+(mn<<3)|0,HEAPF64[yn>>3]=vn,bn=HEAP32[416]|0,wn=bn+1|0,HEAP32[416]=wn}En=HEAP32[419]|0,Sn=xn,Nn=HEAP32[Sn>>2]|0,Cn=(En|0)%(Nn|0)&-1,kn=Cn+1|0,HEAP32[419]=kn,Ln=HEAP32[417]|0,An=Ln+1|0,HEAP32[417]=An}}while(0);return y=0,On=y,STACKTOP=wi,On|0}function _subsm_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0,E|=0;var S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0;gc=STACKTOP,STACKTOP=STACKTOP+144|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),oc=gc+8|0,x=e,kn=t,Oi=n,Po=r,ja=i,ml=s,kl=o,Fl=u,Kl=a,T=f,H=l,V=c,rt=h,dt=p,Nt=d,Bt=v,$t=m,sn=g,mn=y,Ln=b,In=w,Qn=E,ur=Nt,yr=ur+ -8|0,Nt=yr,Ar=dt,qr=Ar+ -8|0,dt=qr,Gr=T,ai=Gr+ -8|0,T=ai,bi=Kl,Mi=bi+ -8|0,Kl=Mi,Ui=Fl,Zi=Ui+ -8|0,Fl=Zi,ls=kl,Es=ls+ -4|0,kl=Es,_s=ml,zs=_s+ -8|0,ml=zs,eo=ja,co=eo+ -8|0,ja=co,xo=kn,Ho=HEAP32[xo>>2]|0,Vo=Ho<<1,lc=Vo,ru=lc,du=1+ru|0,cc=du,Nu=cc,Bu=Ln,$u=0-Nu|0,ia=Bu+($u<<3)|0,Ln=ia,va=mn,Ca=va+ -8|0,mn=Ca,Fa=x,Ka=HEAP32[Fa>>2]|0,dc=Ka,uf=dc,yf=1+uf|0,vc=yf,Af=vc,qf=V,Gf=0-Af|0,al=qf+(Gf<<3)|0,V=al,dl=x,vl=HEAP32[dl>>2]|0,hc=vl,gl=hc,yl=1+gl|0,pc=yl,bl=pc,wl=H,El=0-bl|0,Sl=wl+(El<<3)|0,H=Sl,xl=Po,Tl=xl+ -4|0,Po=Tl,Nl=Oi,Cl=HEAP32[Nl>>2]|0,Ll=(Cl|0)<=0;if(Ll)return S=0,pl=S,STACKTOP=gc,pl|0;Al=In,Ol=HEAP32[Al>>2]|0,Ml=(Ol|0)>=99,Ml&&(_s_wsfe(2516)|0,_e_wsfe()|0),_l=$t,Dl=HEAP32[_l>>2]|0,HEAP32[628]=Dl,Pl=Bt,Hl=HEAP32[Pl>>2]|0,ac=Hl,HEAP32[620]=1;for(;;){Bl=HEAP32[620]|0,jl=ac,Il=(Bl|0)<=(jl|0);if(!Il)break;HEAPF64[62]=0,HEAPF64[63]=0,ql=Oi,Rl=HEAP32[ql>>2]|0,fc=Rl,HEAP32[621]=1;for(;;){Ul=HEAP32[621]|0,zl=fc,Wl=(Ul|0)<=(zl|0);if(!Wl)break;Xl=HEAP32[621]|0,Vl=Po,$l=Vl+(Xl<<2)|0,Jl=HEAP32[$l>>2]|0,HEAP32[622]=Jl,Ql=HEAP32[622]|0,Gl=HEAP32[628]|0,Yl=dc,Zl=Math_imul(Gl,Yl)|0,ec=Ql+Zl|0,tc=V,nc=tc+(ec<<3)|0,rc=+HEAPF64[nc>>3],ic=HEAP32[621]|0,sc=Kl,N=sc+(ic<<3)|0,C=+HEAPF64[N>>3],k=rc*C,L=+HEAPF64[62],A=L+k,HEAPF64[62]=A,O=HEAP32[622]|0,M=HEAP32[628]|0,_=hc,D=Math_imul(M,_)|0,P=O+D|0,B=H,j=B+(P<<3)|0,F=+HEAPF64[j>>3],I=HEAP32[621]|0,q=Kl,R=q+(I<<3)|0,U=+HEAPF64[R>>3],z=F*U,W=+HEAPF64[63],X=W+z,HEAPF64[63]=X,$=HEAP32[621]|0,J=$+1|0,HEAP32[621]=J}K=+HEAPF64[62],Q=HEAP32[620]|0,G=mn,Y=G+(Q<<3)|0,HEAPF64[Y>>3]=K,Z=rt,et=+HEAPF64[Z>>3],tt=+HEAPF64[63],nt=et*tt,it=Bt,st=HEAP32[it>>2]|0,ot=HEAP32[620]|0,ut=st+ot|0,at=mn,ft=at+(ut<<3)|0,HEAPF64[ft>>3]=nt,lt=HEAP32[628]|0,ct=kn,ht=HEAP32[ct>>2]|0,pt=(lt|0)%(ht|0)&-1,vt=pt+1|0,HEAP32[628]=vt,mt=HEAP32[620]|0,gt=mt+1|0,HEAP32[620]=gt}yt=kn,bt=HEAP32[yt>>2]|0,wt=bt<<1,HEAP32[623]=wt,Et=Bt,St=HEAP32[Et>>2]|0,xt=St<<1,HEAP32[627]=xt,Tt=cc,Ct=Ln,kt=Ct+(Tt<<3)|0,Lt=mn,At=Lt+8|0,Ot=Qn,_dtrsl_(kt,2492,2508,At,1324,Ot)|0,Mt=Qn,_t=HEAP32[Mt>>2]|0,Dt=(_t|0)!=0;if(Dt)return S=0,pl=S,STACKTOP=gc,pl|0;Pt=Bt,Ht=HEAP32[Pt>>2]|0,ac=Ht,HEAP32[620]=1;for(;;){jt=HEAP32[620]|0,Ft=ac,It=(jt|0)<=(Ft|0);if(!It)break;qt=HEAP32[620]|0,Rt=mn,Ut=Rt+(qt<<3)|0,zt=+HEAPF64[Ut>>3],Wt=-zt,Xt=HEAP32[620]|0,Vt=mn,Jt=Vt+(Xt<<3)|0,HEAPF64[Jt>>3]=Wt,Kt=HEAP32[620]|0,Qt=Kt+1|0,HEAP32[620]=Qt}Gt=cc,Yt=Ln,Zt=Yt+(Gt<<3)|0,en=mn,tn=en+8|0,nn=Qn,_dtrsl_(Zt,2492,2508,tn,1236,nn)|0,rn=Qn,on=HEAP32[rn>>2]|0,un=(on|0)!=0;if(un)return S=0,pl=S,STACKTOP=gc,pl|0;an=$t,fn=HEAP32[an>>2]|0,HEAP32[628]=fn,ln=Bt,cn=HEAP32[ln>>2]|0,ac=cn,HEAP32[625]=1;for(;;){hn=HEAP32[625]|0,pn=ac,dn=(hn|0)<=(pn|0);if(!dn)break;vn=Bt,gn=HEAP32[vn>>2]|0,yn=HEAP32[625]|0,bn=gn+yn|0,HEAP32[624]=bn,wn=Oi,En=HEAP32[wn>>2]|0,fc=En,HEAP32[620]=1;for(;;){Sn=HEAP32[620]|0,xn=fc,Tn=(Sn|0)<=(xn|0);if(!Tn)break;Nn=HEAP32[620]|0,Cn=Po,An=Cn+(Nn<<2)|0,On=HEAP32[An>>2]|0,HEAP32[622]=On,Mn=HEAP32[620]|0,_n=Kl,Dn=_n+(Mn<<3)|0,Pn=+HEAPF64[Dn>>3],Hn=HEAP32[622]|0,Bn=HEAP32[628]|0,jn=dc,Fn=Math_imul(Bn,jn)|0,qn=Hn+Fn|0,Rn=V,Un=Rn+(qn<<3)|0,zn=+HEAPF64[Un>>3],Wn=HEAP32[625]|0,Xn=mn,Vn=Xn+(Wn<<3)|0,$n=+HEAPF64[Vn>>3],Jn=zn*$n,Kn=rt,Gn=+HEAPF64[Kn>>3],Yn=Jn/Gn,Zn=Pn+Yn,er=HEAP32[622]|0,tr=HEAP32[628]|0,nr=hc,rr=Math_imul(tr,nr)|0,ir=er+rr|0,sr=H,or=sr+(ir<<3)|0,ar=+HEAPF64[or>>3],fr=HEAP32[624]|0,lr=mn,cr=lr+(fr<<3)|0,hr=+HEAPF64[cr>>3],pr=ar*hr,dr=Zn+pr,vr=HEAP32[620]|0,mr=Kl,gr=mr+(vr<<3)|0,HEAPF64[gr>>3]=dr,br=HEAP32[620]|0,wr=br+1|0,HEAP32[620]=wr}Er=HEAP32[628]|0,Sr=kn,xr=HEAP32[Sr>>2]|0,Tr=(Er|0)%(xr|0)&-1,Nr=Tr+1|0,HEAP32[628]=Nr,Cr=HEAP32[625]|0,kr=Cr+1|0,HEAP32[625]=kr}Lr=rt,Or=+HEAPF64[Lr>>3],Mr=1/Or,HEAPF64[oc>>3]=Mr,_r=Oi,Dr=Kl,Pr=Dr+8|0,_dscal_(_r,oc,Pr,1236)|0,Hr=sn,HEAP32[Hr>>2]=0,Br=x,jr=Fl,Fr=jr+8|0,Ir=T,Rr=Ir+8|0,_dcopy_(Br,Fr,1236,Rr,1236)|0,Ur=Oi,zr=HEAP32[Ur>>2]|0,ac=zr,HEAP32[620]=1;for(;;){Wr=HEAP32[620]|0,Xr=ac,Vr=(Wr|0)<=(Xr|0);if(!Vr)break;$r=HEAP32[620]|0,Jr=Po,Kr=Jr+($r<<2)|0,Qr=HEAP32[Kr>>2]|0,HEAP32[622]=Qr,Yr=HEAP32[620]|0,Zr=Kl,ei=Zr+(Yr<<3)|0,ti=+HEAPF64[ei>>3],HEAPF64[59]=ti,ni=HEAP32[622]|0,ri=Fl,ii=ri+(ni<<3)|0,si=+HEAPF64[ii>>3],HEAPF64[60]=si,oi=HEAP32[622]|0,ui=kl,fi=ui+(oi<<2)|0,li=HEAP32[fi>>2]|0,ci=(li|0)!=0;do{if(ci){hi=HEAP32[622]|0,pi=kl,di=pi+(hi<<2)|0,vi=HEAP32[di>>2]|0,mi=(vi|0)==1,gi=HEAP32[622]|0;if(mi){yi=ja,wi=yi+(gi<<3)|0,Ei=+HEAPF64[wi>>3],HEAPF64[oc>>3]=Ei,Si=+HEAPF64[60],xi=+HEAPF64[59],Ti=Si+xi,uc=Ti,Ni=+HEAPF64[oc>>3],Ci=uc,ki=Ni>=Ci,Li=+HEAPF64[oc>>3],Ai=uc,_i=ki?Li:Ai,Di=HEAP32[622]|0,Pi=Fl,Hi=Pi+(Di<<3)|0,HEAPF64[Hi>>3]=_i,Bi=HEAP32[622]|0,ji=Fl,Fi=ji+(Bi<<3)|0,Ii=+HEAPF64[Fi>>3],qi=HEAP32[622]|0,Ri=ja,zi=Ri+(qi<<3)|0,Wi=+HEAPF64[zi>>3],Xi=Ii==Wi;if(!Xi)break;Vi=sn,HEAP32[Vi>>2]=1;break}$i=kl,Ji=$i+(gi<<2)|0,Ki=HEAP32[Ji>>2]|0,Qi=(Ki|0)==2,Gi=HEAP32[622]|0;if(Qi){Yi=ja,es=Yi+(Gi<<3)|0,ts=+HEAPF64[es>>3],HEAPF64[oc>>3]=ts,ns=+HEAPF64[60],rs=+HEAPF64[59],is=ns+rs,uc=is,ss=+HEAPF64[oc>>3],os=uc,us=ss>=os,as=+HEAPF64[oc>>3],fs=uc,cs=us?as:fs,HEAPF64[60]=cs,hs=HEAP32[622]|0,ps=ml,ds=ps+(hs<<3)|0,vs=+HEAPF64[ds>>3],HEAPF64[oc>>3]=vs,ms=+HEAPF64[oc>>3],gs=+HEAPF64[60],ys=ms<=gs,bs=+HEAPF64[oc>>3],ws=+HEAPF64[60],Ss=ys?bs:ws,xs=HEAP32[622]|0,Ts=Fl,Ns=Ts+(xs<<3)|0,HEAPF64[Ns>>3]=Ss,Cs=HEAP32[622]|0,ks=Fl,Ls=ks+(Cs<<3)|0,As=+HEAPF64[Ls>>3],Os=HEAP32[622]|0,Ms=ja,Ds=Ms+(Os<<3)|0,Ps=+HEAPF64[Ds>>3],Hs=As==Ps;if(!Hs){Bs=HEAP32[622]|0,js=Fl,Fs=js+(Bs<<3)|0,Is=+HEAPF64[Fs>>3],qs=HEAP32[622]|0,Rs=ml,Us=Rs+(qs<<3)|0,Ws=+HEAPF64[Us>>3],Xs=Is==Ws;if(!Xs)break}Vs=sn,HEAP32[Vs>>2]=1;break}$s=kl,Js=$s+(Gi<<2)|0,Ks=HEAP32[Js>>2]|0,Qs=(Ks|0)==3;if(!Qs)break;Gs=HEAP32[622]|0,Ys=ml,Zs=Ys+(Gs<<3)|0,to=+HEAPF64[Zs>>3],HEAPF64[oc>>3]=to,no=+HEAPF64[60],ro=+HEAPF64[59],io=no+ro,uc=io,so=+HEAPF64[oc>>3],oo=uc,uo=so<=oo,ao=+HEAPF64[oc>>3],fo=uc,lo=uo?ao:fo,ho=HEAP32[622]|0,po=Fl,vo=po+(ho<<3)|0,HEAPF64[vo>>3]=lo,mo=HEAP32[622]|0,go=Fl,yo=go+(mo<<3)|0,bo=+HEAPF64[yo>>3],wo=HEAP32[622]|0,Eo=ml,So=Eo+(wo<<3)|0,To=+HEAPF64[So>>3],No=bo==To;if(!No)break;Co=sn,HEAP32[Co>>2]=1;break}ko=+HEAPF64[60],Lo=+HEAPF64[59],Ao=ko+Lo,Oo=HEAP32[622]|0,Mo=Fl,_o=Mo+(Oo<<3)|0,HEAPF64[_o>>3]=Ao}while(0);Do=HEAP32[620]|0,Bo=Do+1|0,HEAP32[620]=Bo}jo=sn,Fo=HEAP32[jo>>2]|0,Io=(Fo|0)==0;e:do if(!Io){HEAPF64[61]=0,qo=x,Ro=HEAP32[qo>>2]|0,ac=Ro,HEAP32[620]=1;for(;;){Uo=HEAP32[620]|0,zo=ac,Wo=(Uo|0)<=(zo|0);if(!Wo)break;Xo=HEAP32[620]|0,$o=Fl,Jo=$o+(Xo<<3)|0,Ko=+HEAPF64[Jo>>3],Qo=HEAP32[620]|0,Go=dt,Yo=Go+(Qo<<3)|0,Zo=+HEAPF64[Yo>>3],eu=Ko-Zo,tu=HEAP32[620
]|0,nu=Nt,iu=nu+(tu<<3)|0,su=+HEAPF64[iu>>3],ou=eu*su,uu=+HEAPF64[61],au=uu+ou,HEAPF64[61]=au,fu=HEAP32[620]|0,lu=fu+1|0,HEAP32[620]=lu}cu=+HEAPF64[61],hu=cu>0;if(hu){pu=x,vu=T,mu=vu+8|0,gu=Fl,yu=gu+8|0,_dcopy_(pu,mu,1236,yu,1236)|0,bu=In,wu=HEAP32[bu>>2]|0,Eu=(wu|0)>=0,Eu&&(_s_wsle(2536)|0,_do_lio(1308,1236,17664,39)|0,_e_wsle()|0,_s_wsle(2556)|0,_do_lio(1308,1236,17704,29)|0,_e_wsle()|0),HEAPF64[64]=1,Su=+HEAPF64[64],HEAPF64[62]=Su,HEAP32[626]=0,xu=Oi,Tu=HEAP32[xu>>2]|0,ac=Tu,HEAP32[620]=1;for(;;){Cu=HEAP32[620]|0,ku=ac,Lu=(Cu|0)<=(ku|0);if(!Lu)break;Au=HEAP32[620]|0,Ou=Po,Mu=Ou+(Au<<2)|0,_u=HEAP32[Mu>>2]|0,HEAP32[622]=_u,Du=HEAP32[620]|0,Pu=Kl,Hu=Pu+(Du<<3)|0,ju=+HEAPF64[Hu>>3],HEAPF64[59]=ju,Fu=HEAP32[622]|0,Iu=kl,qu=Iu+(Fu<<2)|0,Ru=HEAP32[qu>>2]|0,Uu=(Ru|0)!=0;if(Uu){zu=+HEAPF64[59],Wu=zu<0;do if(Wu){Xu=HEAP32[622]|0,Vu=kl,Ju=Vu+(Xu<<2)|0,Ku=HEAP32[Ju>>2]|0,Qu=(Ku|0)<=2;if(Qu){Gu=HEAP32[622]|0,Yu=ja,Zu=Yu+(Gu<<3)|0,ea=+HEAPF64[Zu>>3],ta=HEAP32[622]|0,na=Fl,ra=na+(ta<<3)|0,sa=+HEAPF64[ra>>3],oa=ea-sa,HEAPF64[63]=oa,ua=+HEAPF64[63],aa=ua>=0;if(aa){HEAPF64[62]=0;break}fa=+HEAPF64[59],la=+HEAPF64[64],ca=fa*la,ha=+HEAPF64[63],pa=ca<ha,pa&&(da=+HEAPF64[63],ma=+HEAPF64[59],ga=da/ma,HEAPF64[62]=ga)}else mc=55}else mc=55;while(0);do if((mc|0)==55){mc=0,ya=+HEAPF64[59],ba=ya>0;if(ba){wa=HEAP32[622]|0,Ea=kl,Sa=Ea+(wa<<2)|0,xa=HEAP32[Sa>>2]|0,Ta=(xa|0)>=2;if(Ta){Na=HEAP32[622]|0,ka=ml,La=ka+(Na<<3)|0,Aa=+HEAPF64[La>>3],Oa=HEAP32[622]|0,Ma=Fl,_a=Ma+(Oa<<3)|0,Da=+HEAPF64[_a>>3],Pa=Aa-Da,HEAPF64[63]=Pa,Ha=+HEAPF64[63],Ba=Ha<=0;if(Ba){HEAPF64[62]=0;break}Ia=+HEAPF64[59],qa=+HEAPF64[64],Ra=Ia*qa,Ua=+HEAPF64[63],za=Ra>Ua,za&&(Wa=+HEAPF64[63],Xa=+HEAPF64[59],Va=Wa/Xa,HEAPF64[62]=Va)}}}while(0);$a=+HEAPF64[62],Ja=+HEAPF64[64],Qa=$a<Ja,Qa&&(Ga=+HEAPF64[62],HEAPF64[64]=Ga,Ya=HEAP32[620]|0,HEAP32[626]=Ya)}Za=HEAP32[620]|0,ef=Za+1|0,HEAP32[620]=ef}tf=+HEAPF64[64],nf=tf<1;do if(nf){rf=HEAP32[626]|0,sf=Kl,of=sf+(rf<<3)|0,af=+HEAPF64[of>>3],HEAPF64[59]=af,ff=HEAP32[626]|0,lf=Po,cf=lf+(ff<<2)|0,hf=HEAP32[cf>>2]|0,HEAP32[622]=hf,pf=+HEAPF64[59],df=pf>0;if(df){vf=HEAP32[622]|0,mf=ml,gf=mf+(vf<<3)|0,bf=+HEAPF64[gf>>3],wf=HEAP32[622]|0,Ef=Fl,Sf=Ef+(wf<<3)|0,HEAPF64[Sf>>3]=bf,xf=HEAP32[626]|0,Tf=Kl,Nf=Tf+(xf<<3)|0,HEAPF64[Nf>>3]=0;break}Cf=+HEAPF64[59],kf=Cf<0,kf&&(Lf=HEAP32[622]|0,Of=ja,Mf=Of+(Lf<<3)|0,_f=+HEAPF64[Mf>>3],Df=HEAP32[622]|0,Pf=Fl,Hf=Pf+(Df<<3)|0,HEAPF64[Hf>>3]=_f,Bf=HEAP32[626]|0,jf=Kl,Ff=jf+(Bf<<3)|0,HEAPF64[Ff>>3]=0)}while(0);If=Oi,Rf=HEAP32[If>>2]|0,ac=Rf,HEAP32[620]=1;for(;;){Uf=HEAP32[620]|0,zf=ac,Wf=(Uf|0)<=(zf|0);if(!Wf)break e;Xf=HEAP32[620]|0,Vf=Po,$f=Vf+(Xf<<2)|0,Jf=HEAP32[$f>>2]|0,HEAP32[622]=Jf,Kf=+HEAPF64[64],Qf=HEAP32[620]|0,Yf=Kl,Zf=Yf+(Qf<<3)|0,el=+HEAPF64[Zf>>3],tl=Kf*el,nl=HEAP32[622]|0,rl=Fl,il=rl+(nl<<3)|0,sl=+HEAPF64[il>>3],ol=sl+tl,HEAPF64[il>>3]=ol,ul=HEAP32[620]|0,fl=ul+1|0,HEAP32[620]=fl}}}while(0);return ll=In,cl=HEAP32[ll>>2]|0,hl=(cl|0)>=99,hl&&(_s_wsfe(2576)|0,_e_wsfe()|0),S=0,pl=S,STACKTOP=gc,pl|0}function _lnsrlb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x,T,N,C,k,L,A,O,M,_){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0,g|=0,y|=0,b|=0,w|=0,E|=0,S|=0,x|=0,T|=0,N|=0,C|=0,k|=0,L|=0,A|=0,O|=0,M|=0,_|=0;var D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0;Fs=STACKTOP,STACKTOP=STACKTOP+160|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),P=e,Fn=t,hi=n,xi=r,Pi=i,Xi=s,ns=o,ps=u,Ts=a,H=f,V=l,rt=c,dt=h,Nt=p,Bt=d,$t=v,sn=m,mn=g,kn=y,In=b,Qn=w,ur=E,yr=S,Ar=x,qr=T,Gr=N,ai=C,li=k,ci=L,pi=A,di=O,vi=M,mi=_,gi=Nt,yi=gi+ -8|0,Nt=yi,bi=dt,wi=bi+ -8|0,dt=wi,Ei=rt,Si=Ei+ -8|0,rt=Si,Ti=V,Ni=Ti+ -8|0,V=Ni,Ci=H,ki=Ci+ -8|0,H=ki,Li=Pi,Ai=Li+ -8|0,Pi=Ai,Oi=xi,Mi=Oi+ -4|0,xi=Mi,_i=hi,Di=_i+ -8|0,hi=Di,Hi=Fn,Bi=Hi+ -8|0,Fn=Bi,ji=ci,Fi=ji+ -4|0,ci=Fi,Ii=pi,qi=Ii+ -8|0,pi=qi,Ri=qr,Ui=_s_cmp(Ri,14362,5,5)|0,zi=(Ui|0)==0;if(!zi){Wi=P,Vi=V,$i=Vi+8|0,Ji=V,Ki=Ji+8|0,Qi=+_ddot_(Wi,$i,1236,Ki,1236),Gi=sn,HEAPF64[Gi>>3]=Qi,Yi=sn,Zi=+HEAPF64[Yi>>3],es=+Math_sqrt(+Zi),ts=$t,HEAPF64[ts>>3]=es,rs=kn,HEAPF64[rs>>3]=1e10,is=ai,ss=HEAP32[is>>2]|0,os=(ss|0)!=0;e:do if(os){us=In,as=HEAP32[us>>2]|0,fs=(as|0)==0;if(fs){ls=kn,HEAPF64[ls>>3]=1;break}cs=P,hs=HEAP32[cs>>2]|0,Bs=hs,HEAP32[471]=1;for(;;){ds=HEAP32[471]|0,vs=Bs,ms=(ds|0)<=(vs|0);if(!ms)break e;gs=HEAP32[471]|0,ys=V,bs=ys+(gs<<3)|0,ws=+HEAPF64[bs>>3],HEAPF64[53]=ws,Es=HEAP32[471]|0,Ss=xi,xs=Ss+(Es<<2)|0,Ns=HEAP32[xs>>2]|0,Cs=(Ns|0)!=0;do if(Cs){ks=+HEAPF64[53],Ls=ks<0;if(Ls){As=HEAP32[471]|0,Os=xi,Ms=Os+(As<<2)|0,_s=HEAP32[Ms>>2]|0,Ds=(_s|0)<=2;if(Ds){Ps=HEAP32[471]|0,B=Fn,j=B+(Ps<<3)|0,F=+HEAPF64[j>>3],I=HEAP32[471]|0,q=Pi,R=q+(I<<3)|0,U=+HEAPF64[R>>3],z=F-U,HEAPF64[54]=z,W=+HEAPF64[54],X=W>=0;if(X){$=kn,HEAPF64[$>>3]=0;break}J=+HEAPF64[53],K=kn,Q=+HEAPF64[K>>3],G=J*Q,Y=+HEAPF64[54],Z=G<Y;if(!Z)break;et=+HEAPF64[54],tt=+HEAPF64[53],nt=et/tt,it=kn,HEAPF64[it>>3]=nt;break}}st=+HEAPF64[53],ot=st>0;if(ot){ut=HEAP32[471]|0,at=xi,ft=at+(ut<<2)|0,lt=HEAP32[ft>>2]|0,ct=(lt|0)>=2;if(ct){ht=HEAP32[471]|0,pt=hi,vt=pt+(ht<<3)|0,mt=+HEAPF64[vt>>3],gt=HEAP32[471]|0,yt=Pi,bt=yt+(gt<<3)|0,wt=+HEAPF64[bt>>3],Et=mt-wt,HEAPF64[54]=Et,St=+HEAPF64[54],xt=St<=0;if(xt){Tt=kn,HEAPF64[Tt>>3]=0;break}Ct=+HEAPF64[53],kt=kn,Lt=+HEAPF64[kt>>3],At=Ct*Lt,Ot=+HEAPF64[54],Mt=At>Ot,Mt&&(_t=+HEAPF64[54],Dt=+HEAPF64[53],Pt=_t/Dt,Ht=kn,HEAPF64[Ht>>3]=Pt)}}}while(0);jt=HEAP32[471]|0,Ft=jt+1|0,HEAP32[471]=Ft}}while(0);It=In,qt=HEAP32[It>>2]|0,Rt=(qt|0)==0,Rt?(Ut=Gr,zt=HEAP32[Ut>>2]|0,Wt=(zt|0)!=0,Wt?js=27:(Xt=$t,Vt=+HEAPF64[Xt>>3],Jt=1/Vt,Hs=Jt,Kt=Hs,Qt=kn,Gt=+HEAPF64[Qt>>3],Yt=Kt<=Gt,Yt?(Zt=Hs,rn=Zt):(en=kn,tn=+HEAPF64[en>>3],rn=tn),nn=Bt,HEAPF64[nn>>3]=rn)):js=27,(js|0)==27&&(on=Bt,HEAPF64[on>>3]=1),un=P,an=Pi,fn=an+8|0,ln=dt,cn=ln+8|0,_dcopy_(un,fn,1236,cn,1236)|0,hn=P,pn=H,dn=pn+8|0,vn=rt,gn=vn+8|0,_dcopy_(hn,dn,1236,gn,1236)|0,yn=Xi,bn=+HEAPF64[yn>>3],wn=ns,HEAPF64[wn>>3]=bn,En=Qn,HEAP32[En>>2]=0,Sn=ur,HEAP32[Sn>>2]=0,xn=li,_s_copy(xn,13740,60,5)}Tn=P,Nn=H,Cn=Nn+8|0,Ln=V,An=Ln+8|0,On=+_ddot_(Tn,Cn,1236,An,1236),Mn=ps,HEAPF64[Mn>>3]=On,_n=Qn,Dn=HEAP32[_n>>2]|0,Pn=(Dn|0)==0;if(Pn){Hn=ps,Bn=+HEAPF64[Hn>>3],jn=Ts,HEAPF64[jn>>3]=Bn,qn=ps,Rn=+HEAPF64[qn>>3],Un=Rn>=0;if(Un)return zn=di,Wn=HEAP32[zn>>2]|0,Xn=(Wn|0)>=0,Xn&&(_s_wsle(1888)|0,_do_lio(1308,1236,15408,37)|0,Vn=ps,_do_lio(1908,1236,Vn,8)|0,_e_wsle()|0),$n=Ar,HEAP32[$n>>2]=-4,D=0,fi=D,STACKTOP=Fs,fi|0}Jn=Xi,Kn=ps,Gn=Bt,Yn=kn,Zn=li,er=ci,tr=er+4|0,nr=pi,rr=nr+8|0,_dcsrch_(Jn,Kn,Gn,440,448,456,192,Yn,Zn,tr,rr,60)|0,ir=Bt,sr=+HEAPF64[ir>>3],or=$t,ar=+HEAPF64[or>>3],fr=sr*ar,lr=mn,HEAPF64[lr>>3]=fr,cr=li,hr=_s_cmp(cr,15446,4,4)|0,pr=(hr|0)!=0;t:do if(pr){dr=li,vr=_s_cmp(dr,15451,4,4)|0,mr=(vr|0)!=0;if(mr){gr=qr,_s_copy(gr,15456,60,9),br=Qn,wr=HEAP32[br>>2]|0,Er=wr+1|0,HEAP32[br>>2]=Er,Sr=yr,xr=HEAP32[Sr>>2]|0,Tr=xr+1|0,HEAP32[Sr>>2]=Tr,Nr=Qn,Cr=HEAP32[Nr>>2]|0,kr=Cr-1|0,Lr=ur,HEAP32[Lr>>2]=kr,Or=Bt,Mr=+HEAPF64[Or>>3],_r=Mr==1,Dr=P;if(_r){Pr=Nt,Hr=Pr+8|0,Br=Pi,jr=Br+8|0,_dcopy_(Dr,Hr,1236,jr,1236)|0;break}Fr=HEAP32[Dr>>2]|0,Bs=Fr,HEAP32[471]=1;for(;;){Ir=HEAP32[471]|0,Rr=Bs,Ur=(Ir|0)<=(Rr|0);if(!Ur)break t;zr=Bt,Wr=+HEAPF64[zr>>3],Xr=HEAP32[471]|0,Vr=V,$r=Vr+(Xr<<3)|0,Jr=+HEAPF64[$r>>3],Kr=Wr*Jr,Qr=HEAP32[471]|0,Yr=dt,Zr=Yr+(Qr<<3)|0,ei=+HEAPF64[Zr>>3],ti=Kr+ei,ni=HEAP32[471]|0,ri=Pi,ii=ri+(ni<<3)|0,HEAPF64[ii>>3]=ti,si=HEAP32[471]|0,oi=si+1|0,HEAP32[471]=oi}}else js=41}else js=41;while(0);return(js|0)==41&&(ui=qr,_s_copy(ui,14368,60,5)),D=0,fi=D,STACKTOP=Fs,fi|0}function _prn2lb_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0;var m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0;qt=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),m=e,g=t,L=n,I=r,Q=i,ut=s,yt=o,At=u,Bt=a,jt=f,y=l,b=c,w=h,E=p,S=d,x=v,T=I,N=T+ -8|0,I=N,C=g,k=C+ -8|0,g=k,A=b,O=HEAP32[A>>2]|0,M=(O|0)==0;do{if(!M){D=b,P=HEAP32[D>>2]|0,H=(P|0)==1;if(H){B=y,_s_copy(B,15736,3,3);break}j=b,F=HEAP32[j>>2]|0,q=(F|0)==5,R=y;if(q){_s_copy(R,15740,3,3);break}_s_copy(R,14352,3,3);break}_=y,_s_copy(_,15732,3,3)}while(0);U=Q,z=HEAP32[U>>2]|0,W=(z|0)>=99;if(!W)return St=Q,xt=HEAP32[St>>2]|0,Tt=(xt|0)>0,Tt?(Nt=ut,Ct=HEAP32[Nt>>2]|0,kt=Q,Lt=HEAP32[kt>>2]|0,Ot=(Ct|0)%(Lt|0)&-1,HEAP32[507]=Ot,Mt=HEAP32[507]|0,_t=(Mt|0)==0,_t?(_s_wsfe(2112)|0,Dt=ut,_do_fio(1236,Dt,4)|0,Pt=L,_do_fio(1236,Pt,8)|0,Ht=Bt,_do_fio(1236,Ht,8)|0,_e_wsfe()|0,STACKTOP=qt,0):(STACKTOP=qt,0)):(STACKTOP=qt,0);_s_wsle(2032)|0,_do_lio(1308,1236,15744,11)|0,X=w,_do_lio(1660,1236,X,4)|0,_do_lio(1308,1236,15756,23)|0,V=S,_do_lio(1908,1236,V,8)|0,_e_wsle()|0,_s_wsfe(2052)|0,$=ut,_do_fio(1236,$,4)|0,J=L,_do_fio(1236,J,8)|0,K=Bt,_do_fio(1236,K,8)|0,_e_wsfe()|0,G=Q,Y=HEAP32[G>>2]|0,Z=(Y|0)>100;if(!Z)return STACKTOP=qt,0;_s_wsfe(2072)|0,_do_fio(1236,15780,3)|0,et=m,tt=HEAP32[et>>2]|0,Ft=tt,HEAP32[506]=1;for(;;){nt=HEAP32[506]|0,rt=Ft,it=(nt|0)<=(rt|0);if(!it)break;st=HEAP32[506]|0,ot=g,at=ot+(st<<3)|0,_do_fio(1236,at,8)|0,ft=HEAP32[506]|0,lt=ft+1|0,HEAP32[506]=lt}_e_wsfe()|0,_s_wsfe(2092)|0,_do_fio(1236,15784,3)|0,ct=m,ht=HEAP32[ct>>2]|0,Ft=ht,HEAP32[506]=1;for(;;){pt=HEAP32[506]|0,dt=Ft,vt=(pt|0)<=(dt|0);if(!vt)break;mt=HEAP32[506]|0,gt=I,bt=gt+(mt<<3)|0,_do_fio(1236,bt,8)|0,wt=HEAP32[506]|0,Et=wt+1|0,HEAP32[506]=Et}return _e_wsfe()|0,STACKTOP=qt,0}function _matupd_(e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0,p|=0,d|=0,v|=0,m|=0;var g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0;ls=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),es=ls,g=e,y=t,Sn=n,Or=r,Rr=i,Yr=s,fi=o,wi=u,Mi=a,Ui=f,b=l,O=c,R=h,Y=p,ft=d,wt=v,Mt=m,Ut=wi,Zt=Ut+ -8|0,wi=Zt,cn=fi,xn=cn+ -8|0,fi=xn,Pn=y,Xn=HEAP32[Pn>>2]|0,ts=Xn,nr=ts,pr=1+nr|0,ns=pr,Tr=ns,Cr=Yr,kr=0-Tr|0,Lr=Cr+(kr<<3)|0,Yr=Lr,Ar=y,Mr=HEAP32[Ar>>2]|0,rs=Mr,_r=rs,Dr=1+_r|0,is=Dr,Pr=is,Hr=Rr,Br=0-Pr|0,jr=Hr+(Br<<3)|0,Rr=jr,Fr=g,Ir=HEAP32[Fr>>2]|0,us=Ir,qr=us,Ur=1+qr|0,as=Ur,zr=as,Wr=Or,Xr=0-zr|0,Vr=Wr+(Xr<<3)|0,Or=Vr,$r=g,Jr=HEAP32[$r>>2]|0,ss=Jr,Kr=ss,Qr=1+Kr|0,os=Qr,Gr=os,Zr=Sn,ei=0-Gr|0,ti=Zr+(ei<<3)|0,Sn=ti,ni=Ui,ri=HEAP32[ni>>2]|0,ii=y,si=HEAP32[ii>>2]|0,oi=(ri|0)<=(si|0),oi?(ui=Ui,ai=HEAP32[ui>>2]|0,li=b,HEAP32[li>>2]=ai,ci=O,hi=HEAP32[ci>>2]|0,pi=Ui,di=HEAP32[pi>>2]|0,vi=hi+di|0,mi=vi-2|0,gi=y,yi=HEAP32[gi>>2]|0,bi=(mi|0)%(yi|0)&-1,Ei=bi+1|0,Si=Mi,HEAP32[Si>>2]=Ei):(xi=Mi,Ti=HEAP32[xi>>2]|0,Ni=y,Ci=HEAP32[Ni>>2]|0,ki=(Ti|0)%(Ci|0)&-1,Li=ki+1|0,Ai=Mi,HEAP32[Ai>>2]=Li,Oi=O,_i=HEAP32[Oi>>2]|0,Di=y,Pi=HEAP32[Di>>2]|0,Hi=(_i|0)%(Pi|0)&-1,Bi=Hi+1|0,ji=O,HEAP32[ji>>2]=Bi),Fi=g,Ii=fi,qi=Ii+8|0,Ri=Mi,zi=HEAP32[Ri>>2]|0,Wi=ss,Xi=Math_imul(zi,Wi)|0,Vi=Xi+1|0,$i=Sn,Ji=$i+(Vi<<3)|0,_dcopy_(Fi,qi,1236,Ji,1236)|0,Ki=g,Qi=wi,Gi=Qi+8|0,Yi=Mi,w=HEAP32[Yi>>2]|0,E=us,S=Math_imul(w,E)|0,x=S+1|0,T=Or,N=T+(x<<3)|0,_dcopy_(Ki,Gi,1236,N,1236)|0,C=Y,k=+HEAPF64[C>>3],L=ft,A=+HEAPF64[L>>3],M=k/A,_=R,HEAPF64[_>>3]=M,D=Ui,P=HEAP32[D>>2]|0,H=y,B=HEAP32[H>>2]|0,j=(P|0)>(B|0);e:do if(j){F=b,I=HEAP32[F>>2]|0,q=I-1|0,Zi=q,HEAP32[478]=1;for(;;){U=HEAP32[478]|0,z=Zi,W=(U|0)<=(z|0);if(!W)break e;X=HEAP32[478]|0,V=X+1|0,$=ts,J=Math_imul(V,$)|0,K=J+2|0,Q=Yr,G=Q+(K<<3)|0,Z=HEAP32[478]|0,et=ts,tt=Math_imul(Z,et)|0,nt=tt+1|0,rt=Yr,it=rt+(nt<<3)|0,_dcopy_(1912,G,1236,it,1236)|0,st=b,ot=HEAP32[st>>2]|0,ut=HEAP32[478]|0,at=ot-ut|0,HEAP32[es>>2]=at,lt=HEAP32[478]|0,ct=lt+1|0,ht=HEAP32[478]|0,pt=ht+1|0,dt=rs,vt=Math_imul(pt,dt)|0,mt=ct+vt|0,gt=Rr,yt=gt+(mt<<3)|0,bt=HEAP32[478]|0,Et=HEAP32[478]|0,St=rs,xt=Math_imul(Et,St)|0,Tt=bt+xt|0,Nt=Rr,Ct=Nt+(Tt<<3)|0,_dcopy_(es,yt,1236,Ct,1236)|0,kt=HEAP32[478]|0,Lt=kt+1|0,HEAP32[478]=Lt}}while(0);At=O,Ot=HEAP32[At>>2]|0,HEAP32[479]=Ot,_t=b,Dt=HEAP32[_t>>2]|0,Pt=Dt-1|0,Zi=Pt,HEAP32[478]=1;for(;;){Ht=HEAP32[478]|0,Bt=Zi,jt=(Ht|0)<=(Bt|0);if(!jt)break;Ft=g,It=fi,qt=It+8|0,Rt=HEAP32[479]|0,zt=us,Wt=Math_imul(Rt,zt)|0,Xt=Wt+1|0,Vt=Or,$t=Vt+(Xt<<3)|0,Jt=+_ddot_(Ft,qt,1236,$t,1236),Kt=b,Qt=HEAP32[Kt>>2]|0,Gt=HEAP32[478]|0,Yt=rs,en=Math_imul(Gt,Yt)|0,tn=Qt+en|0,nn=Rr,rn=nn+(tn<<3)|0,HEAPF64[rn>>3]=Jt,sn=g,on=HEAP32[479]|0,un=ss,an=Math_imul(on,un)|0,fn=an+1|0,ln=Sn,hn=ln+(fn<<3)|0,pn=fi,dn=pn+8|0,vn=+_ddot_(sn,hn,1236,dn,1236),mn=HEAP32[478]|0,gn=b,yn=HEAP32[gn>>2]|0,bn=ts,wn=Math_imul(yn,bn)|0,En=mn+wn|0,Tn=Yr,Nn=Tn+(En<<3)|0,HEAPF64[Nn>>3]=vn,Cn=HEAP32[479]|0,kn=y,Ln=HEAP32[kn>>2]|0,An=(Cn|0)%(Ln|0)&-1,On=An+1|0,HEAP32[479]=On,Mn=HEAP32[478]|0,_n=Mn+1|0,HEAP32[478]=_n}return Dn=wt,Hn=+HEAPF64[Dn>>3],Bn=Hn==1,Bn?(jn=Mt,Fn=+HEAPF64[jn>>3],In=b,qn=HEAP32[In>>2]|0,Rn=b,Un=HEAP32[Rn>>2]|0,zn=ts,Wn=Math_imul(Un,zn)|0,Vn=qn+Wn|0,$n=Yr,Jn=$n+(Vn<<3)|0,HEAPF64[Jn>>3]=Fn,dr=ft,vr=+HEAPF64[dr>>3],mr=b,gr=HEAP32[mr>>2]|0,yr=b,br=HEAP32[yr>>2]|0,wr=rs,Er=Math_imul(br,wr)|0,Sr=gr+Er|0,xr=Rr,Nr=xr+(Sr<<3)|0,HEAPF64[Nr>>3]=vr,STACKTOP=ls,0):(Kn=wt,Qn=+HEAPF64[Kn>>3],Gn=wt,Yn=+HEAPF64[Gn>>3],Zn=Qn*Yn,er=Mt,tr=+HEAPF64[er>>3],rr=Zn*tr,ir=b,sr=HEAP32[ir>>2]|0,or=b,ur=HEAP32[or>>2]|0,ar=ts,fr=Math_imul(ur,ar)|0,lr=sr+fr|0,cr=Yr,hr=cr+(lr<<3)|0,HEAPF64[hr>>3]=rr,dr=ft,vr=+HEAPF64[dr>>3],mr=b,gr=HEAP32[mr>>2]|0,yr=b,br=HEAP32[yr>>2]|0,wr=rs,Er=Math_imul(br,wr)|0,Sr=gr+Er|0,xr=Rr,Nr=xr+(Sr<<3)|0,HEAPF64[Nr>>3]=vr,STACKTOP=ls,0)}function _formt_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0;Kn=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),u=e,a=t,it=n,vt=r,Ct=i,jt=s,Jt=o,on=u,gn=HEAP32[on>>2]|0,Un=gn,Ln=Un,f=1+Ln|0,zn=f,w=zn,M=vt,U=0-w|0,Y=M+(U<<3)|0,vt=Y,Z=u,et=HEAP32[Z>>2]|0,Wn=et,tt=Wn,nt=1+tt|0,Xn=nt,rt=Xn,st=it,ot=0-rt|0,ut=st+(ot<<3)|0,it=ut,at=u,ft=HEAP32[at>>2]|0,Vn=ft,lt=Vn,ct=1+lt|0,$n=ct,ht=$n,pt=a,dt=0-ht|0,mt=pt+(dt<<3)|0,a=mt,gt=Ct,yt=HEAP32[gt>>2]|0,In=yt,HEAP32[440]=1;for(;;){bt=HEAP32[440]|0,wt=In,Et=(bt|0)<=(wt|0);if(!Et)break;St=jt,xt=+HEAPF64[St>>3],Tt=HEAP32[440]|0,Nt=Un,kt=Math_imul(Tt,Nt)|0,Lt=kt+1|0,At=vt,Ot=At+(Lt<<3)|0,Mt=+HEAPF64[Ot>>3],_t=xt*Mt,Dt=HEAP32[440]|0,Pt=Vn,Ht=Math_imul(Dt,Pt)|0,Bt=Ht+1|0,Ft=a,It=Ft+(Bt<<3)|0,HEAPF64[It>>3]=_t,qt=HEAP32[440]|0,Rt=qt+1|0,HEAP32[440]=Rt}Ut=Ct,zt=HEAP32[Ut>>2]|0,In=zt,HEAP32[439]=2;for(;;){Wt=HEAP32[439]|0,Xt=In,Vt=(Wt|0)<=(Xt|0);if(!Vt)break;$t=Ct,Kt=HEAP32[$t>>2]|0,qn=Kt,Qt=HEAP32[439]|0,HEAP32[440]=Qt;for(;;){Gt=HEAP32[440]|0,Yt=qn,Zt=(Gt|0)<=(Yt|0),en=HEAP32[439]|0;if(!Zt)break;tn=HEAP32[440]|0,nn=(en|0)<=(tn|0),rn=HEAP32[439]|0,sn=HEAP32[440]|0,un=nn?rn:sn,an=un-1|0,HEAP32[442]=an,HEAPF64[50]=0,fn=HEAP32[442]|0,Rn=fn,HEAP32[441]=1;for(;;){ln=HEAP32[441]|0,cn=Rn,hn=(ln|0)<=(cn|0);if(!hn)break;pn=HEAP32[439]|0,dn=HEAP32[441]|0,vn=Wn,mn=Math_imul(dn,vn)|0,yn=pn+mn|0,bn=it,wn=bn+(yn<<3)|0,En=+HEAPF64[wn>>3],Sn=HEAP32[440]|0,xn=HEAP32[441]|0,Tn=Wn,Nn=Math_imul(xn,Tn)|0,Cn=Sn+Nn|0,kn=it,An=kn+(Cn<<3)|0,On=+HEAPF64[An>>3],Mn=En*On,_n=HEAP32[441]|0,Dn=HEAP32[441]|0,Pn=Wn,Hn=Math_imul(Dn,Pn)|0,Bn=_n+Hn|0,jn=it,Fn=jn+(Bn<<3)|0,l=+HEAPF64[Fn>>3],c=Mn/l,h=+HEAPF64[50],p=h+c,HEAPF64[50]=p,d=HEAP32[441]|0,v=d+1|0,HEAP32[441]=v}m=+HEAPF64[50],g=jt,y=+HEAPF64[g>>3],b=HEAP32[439]|0,E=HEAP32[440]|0,S=Un,x=Math_imul(E,S)|0,T=b+x|0,N=vt,C=N+(T<<3)|0,k=+HEAPF64[C>>3],L=y*k,A=m+L,O=HEAP32[439]|0,_=HEAP32[440]|0,D=Vn,P=Math_imul(_,D)|0,H=O+P|0,B=a,j=B+(H<<3)|0,HEAPF64[j>>3]=A,F=HEAP32[440]|0,I=F+1|0,HEAP32[440]=I}q=en+1|0,HEAP32[439]=q}return R=$n,z=a,W=z+(R<<3)|0,X=u,V=Ct,$=Jt,_dpofa_(W,X,V,$)|0,J=Jt,K=HEAP32[J>>2]|0,Q=(K|0)!=0,Q?(G=Jt,HEAP32[G>>2]=-3,STACKTOP=Kn,0):(STACKTOP=Kn,0)}function _bmv_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0;bi=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),a=e,hn=t,zn=n,er=r,cr=i,Sr=s,Dr=o,Wr=a,ti=HEAP32[Wr>>2]|0,mi=ti,f=mi,w=1+f|0,gi=w,M=gi,U=zn,Z=0-M|0,lt=U+(Z<<3)|0,zn=lt,Et=a,_t=HEAP32[Et>>2]|0,di=_t,zt=di,en=1+zt|0,vi=en,pn=vi,Tn=hn,Hn=0-pn|0,Bn=Tn+(Hn<<3)|0,hn=Bn,jn=Sr,Fn=jn+ -8|0,Sr=Fn,In=cr,qn=In+ -8|0,cr=qn,Rn=er,Un=HEAP32[Rn>>2]|0,Wn=(Un|0)==0;if(Wn)return u=0,Pn=u,STACKTOP=bi,Pn|0;Xn=er,Vn=HEAP32[Xn>>2]|0,$n=Vn+1|0,Jn=cr,Kn=Jn+($n<<3)|0,Qn=+HEAPF64[Kn>>3],Gn=er,Yn=HEAP32[Gn>>2]|0,Zn=Yn+1|0,tr=Sr,nr=tr+(Zn<<3)|0,HEAPF64[nr>>3]=Qn,rr=er,ir=HEAP32[rr>>2]|0,hi=ir,HEAP32[328]=2;for(;;){sr=HEAP32[328]|0,or=hi,ur=(sr|0)<=(or|0);if(!ur)break;ar=er,fr=HEAP32[ar>>2]|0,lr=HEAP32[328]|0,hr=fr+lr|0,HEAP32[330]=hr,HEAPF64[25]=0,pr=HEAP32[328]|0,dr=pr-1|0,pi=dr,HEAP32[329]=1;for(;;){vr=HEAP32[329]|0,mr=pi,gr=(vr|0)<=(mr|0);if(!gr)break;yr=HEAP32[328]|0,br=HEAP32[329]|0,wr=di,Er=Math_imul(br,wr)|0,xr=yr+Er|0,Tr=hn,Nr=Tr+(xr<<3)|0,Cr=+HEAPF64[Nr>>3],kr=HEAP32[329]|0,Lr=cr,Ar=Lr+(kr<<3)|0,Or=+HEAPF64[Ar>>3],Mr=Cr*Or,_r=HEAP32[329]|0,Pr=HEAP32[329]|0,Hr=di,Br=Math_imul(Pr,Hr)|0,jr=_r+Br|0,Fr=hn,Ir=Fr+(jr<<3)|0,qr=+HEAPF64[Ir>>3],Rr=Mr/qr,Ur=+HEAPF64[25],zr=Ur+Rr,HEAPF64[25]=zr,Xr=HEAP32[329]|0,Vr=Xr+1|0,HEAP32[329]=Vr}$r=HEAP32[330]|0,Jr=cr,Kr=Jr+($r<<3)|0,Qr=+HEAPF64[Kr>>3],Gr=+HEAPF64[25],Yr=Qr+Gr,Zr=HEAP32[330]|0,ei=Sr,ni=ei+(Zr<<3)|0,HEAPF64[ni>>3]=Yr,ri=HEAP32[328]|0,ii=ri+1|0,HEAP32[328]=ii}si=gi,oi=zn,ui=oi+(si<<3)|0,ai=a,fi=er,li=er,ci=HEAP32[li>>2]|0,l=ci+1|0,c=Sr,h=c+(l<<3)|0,p=Dr,_dtrsl_(ui,ai,fi,h,1324,p)|0,d=Dr,v=HEAP32[d>>2]|0,m=(v|0)!=0;if(m)return u=0,Pn=u,STACKTOP=bi,Pn|0;g=er,y=HEAP32[g>>2]|0,hi=y,HEAP32[328]=1;for(;;){b=HEAP32[328]|0,E=hi,S=(b|0)<=(E|0);if(!S)break;x=HEAP32[328]|0,T=cr,N=T+(x<<3)|0,C=+HEAPF64[N>>3],k=HEAP32[328]|0,L=HEAP32[328]|0,A=di,O=Math_imul(L,A)|0,_=k+O|0,D=hn,P=D+(_<<3)|0,H=+HEAPF64[P>>3],B=+Math_sqrt(+H),j=C/B,F=HEAP32[328]|0,I=Sr,q=I+(F<<3)|0,HEAPF64[q>>3]=j,R=HEAP32[328]|0,z=R+1|0,HEAP32[328]=z}W=gi,X=zn,V=X+(W<<3)|0,$=a,J=er,K=er,Q=HEAP32[K>>2]|0,G=Q+1|0,Y=Sr,et=Y+(G<<3)|0,tt=Dr,_dtrsl_(V,$,J,et,1236,tt)|0,nt=Dr,rt=HEAP32[nt>>2]|0,it=(rt|0)!=0;if(it)return u=0,Pn=u,STACKTOP=bi,Pn|0;st=er,ot=HEAP32[st>>2]|0,hi=ot,HEAP32[328]=1;for(;;){ut=HEAP32[328]|0,at=hi,ft=(ut|0)<=(at|0);if(!ft)break;ct=HEAP32[328]|0,ht=Sr,pt=ht+(ct<<3)|0,dt=+HEAPF64[pt>>3],vt=-dt,mt=HEAP32[328]|0,gt=HEAP32[328]|0,yt=di,bt=Math_imul(gt,yt)|0,wt=mt+bt|0,St=hn,xt=St+(wt<<3)|0,Tt=+HEAPF64[xt>>3],Nt=+Math_sqrt(+Tt),Ct=vt/Nt,kt=HEAP32[328]|0,Lt=Sr,At=Lt+(kt<<3)|0,HEAPF64[At>>3]=Ct,Ot=HEAP32[328]|0,Mt=Ot+1|0,HEAP32[328]=Mt}Dt=er,Pt=HEAP32[Dt>>2]|0,hi=Pt,HEAP32[328]=1;for(;;){Ht=HEAP32[328]|0,Bt=hi,jt=(Ht|0)<=(Bt|0);if(!jt)break;HEAPF64[25]=0,Ft=er,It=HEAP32[Ft>>2]|0,pi=It,qt=HEAP32[328]|0,Rt=qt+1|0,HEAP32[329]=Rt;for(;;){Ut=HEAP32[329]|0,Wt=pi,Xt=(Ut|0)<=(Wt|0);if(!Xt)break;Vt=HEAP32[329]|0,$t=HEAP32[328]|0,Jt=di,Kt=Math_imul($t,Jt)|0,Qt=Vt+Kt|0,Gt=hn,Yt=Gt+(Qt<<3)|0,Zt=+HEAPF64[Yt>>3],tn=er,nn=HEAP32[tn>>2]|0,rn=HEAP32[329]|0,sn=nn+rn|0,on=Sr,un=on+(sn<<3)|0,an=+HEAPF64[un>>3],fn=Zt*an,ln=HEAP32[328]|0,cn=HEAP32[328]|0,dn=di,vn=Math_imul(cn,dn)|0,mn=ln+vn|0,gn=hn,yn=gn+(mn<<3)|0,bn=+HEAPF64[yn>>3],wn=fn/bn,En=+HEAPF64[25],Sn=En+wn,HEAPF64[25]=Sn,xn=HEAP32[329]|0,Nn=xn+1|0,HEAP32[329]=Nn}Cn=+HEAPF64[25],kn=HEAP32[328]|0,Ln=Sr,An=Ln+(kn<<3)|0,On=+HEAPF64[An>>3],Mn=On+Cn,HEAPF64[An>>3]=Mn,_n=HEAP32[328]|0,Dn=_n+1|0,HEAP32[328]=Dn}return u=0,Pn=u,STACKTOP=bi,Pn|0};

function _hpsolb_(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0;jn=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,s=t,et=n,ct=r,St=et,Dt=St+ -4|0,et=Dt,Wt=s,tn=Wt+ -8|0,s=tn,pn=ct,Tn=HEAP32[pn>>2]|0,o=(Tn|0)==0;e:do if(o){g=i,L=HEAP32[g>>2]|0,Hn=L,HEAP32[468]=2;for(;;){I=HEAP32[468]|0,J=Hn,K=(I|0)<=(J|0);if(!K)break e;Q=HEAP32[468]|0,G=s,Y=G+(Q<<3)|0,Z=+HEAPF64[Y>>3],HEAPF64[52]=Z,tt=HEAP32[468]|0,nt=et,rt=nt+(tt<<2)|0,it=HEAP32[rt>>2]|0,HEAP32[469]=it,st=HEAP32[468]|0,HEAP32[466]=st;for(;;){ot=HEAP32[466]|0,ut=(ot|0)>1;if(!ut)break;at=HEAP32[466]|0,ft=(at|0)/2&-1,HEAP32[467]=ft,lt=+HEAPF64[52],ht=HEAP32[467]|0,pt=s,dt=pt+(ht<<3)|0,vt=+HEAPF64[dt>>3],mt=lt<vt;if(!mt)break;gt=HEAP32[467]|0,yt=s,bt=yt+(gt<<3)|0,wt=+HEAPF64[bt>>3],Et=HEAP32[466]|0,xt=s,Tt=xt+(Et<<3)|0,HEAPF64[Tt>>3]=wt,Nt=HEAP32[467]|0,Ct=et,kt=Ct+(Nt<<2)|0,Lt=HEAP32[kt>>2]|0,At=HEAP32[466]|0,Ot=et,Mt=Ot+(At<<2)|0,HEAP32[Mt>>2]=Lt,_t=HEAP32[467]|0,HEAP32[466]=_t}Pt=+HEAPF64[52],Ht=HEAP32[466]|0,Bt=s,jt=Bt+(Ht<<3)|0,HEAPF64[jt>>3]=Pt,Ft=HEAP32[469]|0,It=HEAP32[466]|0,qt=et,Rt=qt+(It<<2)|0,HEAP32[Rt>>2]=Ft,Ut=HEAP32[468]|0,zt=Ut+1|0,HEAP32[468]=zt}}while(0);Xt=i,Vt=HEAP32[Xt>>2]|0,$t=(Vt|0)>1;if(!$t)return STACKTOP=jn,0;HEAP32[466]=1,Jt=s,Kt=Jt+8|0,Qt=+HEAPF64[Kt>>3],HEAPF64[51]=Qt,Gt=et,Yt=Gt+4|0,Zt=HEAP32[Yt>>2]|0,HEAP32[470]=Zt,en=i,nn=HEAP32[en>>2]|0,rn=s,sn=rn+(nn<<3)|0,on=+HEAPF64[sn>>3],HEAPF64[52]=on,un=i,an=HEAP32[un>>2]|0,fn=et,ln=fn+(an<<2)|0,cn=HEAP32[ln>>2]|0,HEAP32[469]=cn;for(;;){hn=HEAP32[466]|0,dn=HEAP32[466]|0,vn=hn+dn|0,HEAP32[467]=vn,mn=HEAP32[467]|0,gn=i,yn=HEAP32[gn>>2]|0,bn=yn-1|0,wn=(mn|0)<=(bn|0);if(!wn)break;En=HEAP32[467]|0,Sn=En+1|0,xn=s,Nn=xn+(Sn<<3)|0,Cn=+HEAPF64[Nn>>3],kn=HEAP32[467]|0,Ln=s,An=Ln+(kn<<3)|0,On=+HEAPF64[An>>3],Mn=Cn<On,Mn&&(_n=HEAP32[467]|0,Dn=_n+1|0,HEAP32[467]=Dn),Pn=HEAP32[467]|0,u=s,a=u+(Pn<<3)|0,f=+HEAPF64[a>>3],l=+HEAPF64[52],c=f<l;if(!c)break;h=HEAP32[467]|0,p=s,d=p+(h<<3)|0,v=+HEAPF64[d>>3],m=HEAP32[466]|0,y=s,b=y+(m<<3)|0,HEAPF64[b>>3]=v,w=HEAP32[467]|0,E=et,S=E+(w<<2)|0,x=HEAP32[S>>2]|0,T=HEAP32[466]|0,N=et,C=N+(T<<2)|0,HEAP32[C>>2]=x,k=HEAP32[467]|0,HEAP32[466]=k}return A=+HEAPF64[52],O=HEAP32[466]|0,M=s,_=M+(O<<3)|0,HEAPF64[_>>3]=A,D=HEAP32[469]|0,P=HEAP32[466]|0,H=et,B=H+(P<<2)|0,HEAP32[B>>2]=D,j=+HEAPF64[51],F=i,q=HEAP32[F>>2]|0,R=s,U=R+(q<<3)|0,HEAPF64[U>>3]=j,z=HEAP32[470]|0,W=i,X=HEAP32[W>>2]|0,V=et,$=V+(X<<2)|0,HEAP32[$>>2]=z,STACKTOP=jn,0}function _dcsrch_(e,t,n,r,i,s,o,u,a,f,l,c){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0;var h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0;Aa=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),p=e,gn=t,wi=n,To=r,Cu=i,ju=s,Ju=o,sa=u,ma=a,d=f,N=l,B=c,$=N,it=$+ -8|0,N=it,vt=d,Ct=vt+ -4|0,d=Ct,jt=ma,Jt=_s_cmp(jt,13740,5,5)|0,on=(Jt|0)==0;do{if(on){yn=wi,An=+HEAPF64[yn>>3],qn=Ju,Gn=+HEAPF64[qn>>3],ar=An<Gn,ar&&(br=ma,Or=B,_s_copy(br,17734,Or,22)),Rr=wi,Yr=+HEAPF64[Rr>>3],fi=sa,Ei=+HEAPF64[fi>>3],_i=Yr>Ei,_i&&(zi=ma,es=B,_s_copy(zi,17757,es,22)),cs=gn,Ss=+HEAPF64[cs>>3],Ds=Ss>=0,Ds&&(Ws=ma,to=B,_s_copy(Ws,17780,to,26)),ho=To,No=+HEAPF64[ho>>3],Bo=No<0,Bo&&($o=ma,iu=B,_s_copy($o,17807,iu,21)),vu=Cu,Eu=+HEAPF64[vu>>3],Su=Eu<0,Su&&(xu=ma,Tu=B,_s_copy(xu,17829,Tu,21)),Nu=ju,ku=+HEAPF64[Nu>>3],Lu=ku<0,Lu&&(Au=ma,Ou=B,_s_copy(Au,17851,Ou,21)),Mu=Ju,_u=+HEAPF64[Mu>>3],Du=_u<0,Du&&(Pu=ma,Hu=B,_s_copy(Pu,17873,Hu,23)),Bu=sa,Fu=+HEAPF64[Bu>>3],Iu=Ju,qu=+HEAPF64[Iu>>3],Ru=Fu<qu,Ru&&(Uu=ma,zu=B,_s_copy(Uu,17897,zu,25)),Wu=ma,Xu=_s_cmp(Wu,14356,5,5)|0,Vu=(Xu|0)==0;if(!Vu){HEAP32[650]=0,HEAP32[649]=1,$u=p,Ku=+HEAPF64[$u>>3],HEAPF64[77]=Ku,Qu=gn,Gu=+HEAPF64[Qu>>3],HEAPF64[78]=Gu,Yu=To,Zu=+HEAPF64[Yu>>3],ea=+HEAPF64[78],ta=Zu*ea,HEAPF64[81]=ta,na=sa,ra=+HEAPF64[na>>3],ia=Ju,oa=+HEAPF64[ia>>3],ua=ra-oa,HEAPF64[79]=ua,aa=+HEAPF64[79],fa=aa/.5,HEAPF64[84]=fa,HEAPF64[75]=0,la=+HEAPF64[77],HEAPF64[67]=la,ca=+HEAPF64[78],HEAPF64[69]=ca,HEAPF64[76]=0,ha=+HEAPF64[77],HEAPF64[68]=ha,pa=+HEAPF64[78],HEAPF64[70]=pa,HEAPF64[82]=0,da=wi,va=+HEAPF64[da>>3],ga=wi,ya=+HEAPF64[ga>>3],ba=ya*4,wa=va+ba,HEAPF64[83]=wa,Ea=ma,Sa=B,_s_copy(Ea,17923,Sa,2);break}return h=0,wu=h,STACKTOP=Aa,wu|0}xa=d,Ta=xa+4|0,Na=HEAP32[Ta>>2]|0,Ca=(Na|0)==1,Ca?HEAP32[650]=1:HEAP32[650]=0,v=d,m=v+8|0,g=HEAP32[m>>2]|0,HEAP32[649]=g,y=N,b=y+8|0,w=+HEAPF64[b>>3],HEAPF64[78]=w,E=N,S=E+16|0,x=+HEAPF64[S>>3],HEAPF64[81]=x,T=N,C=T+24|0,k=+HEAPF64[C>>3],HEAPF64[69]=k,L=N,A=L+32|0,O=+HEAPF64[A>>3],HEAPF64[70]=O,M=N,_=M+40|0,D=+HEAPF64[_>>3],HEAPF64[77]=D,P=N,H=P+48|0,j=+HEAPF64[H>>3],HEAPF64[67]=j,F=N,I=F+56|0,q=+HEAPF64[I>>3],HEAPF64[68]=q,R=N,U=R+64|0,z=+HEAPF64[U>>3],HEAPF64[75]=z,W=N,X=W+72|0,V=+HEAPF64[X>>3],HEAPF64[76]=V,J=N,K=J+80|0,Q=+HEAPF64[K>>3],HEAPF64[82]=Q,G=N,Y=G+88|0,Z=+HEAPF64[Y>>3],HEAPF64[83]=Z,et=N,tt=et+96|0,nt=+HEAPF64[tt>>3],HEAPF64[79]=nt,rt=N,st=rt+104|0,ot=+HEAPF64[st>>3],HEAPF64[84]=ot,ut=+HEAPF64[77],at=wi,ft=+HEAPF64[at>>3],lt=+HEAPF64[81],ct=ft*lt,ht=ut+ct,HEAPF64[80]=ht,pt=HEAP32[649]|0,dt=(pt|0)==1,dt&&(mt=p,gt=+HEAPF64[mt>>3],yt=+HEAPF64[80],bt=gt<=yt,bt&&(wt=gn,Et=+HEAPF64[wt>>3],St=Et>=0,St&&(HEAP32[649]=2))),xt=HEAP32[650]|0,Tt=(xt|0)!=0;do if(Tt){Nt=wi,kt=+HEAPF64[Nt>>3],Lt=+HEAPF64[82],At=kt<=Lt;if(!At){Ot=wi,Mt=+HEAPF64[Ot>>3],_t=+HEAPF64[83],Dt=Mt>=_t;if(!Dt)break}Pt=ma,Ht=B,_s_copy(Pt,17926,Ht,41)}while(0);Bt=HEAP32[650]|0,Ft=(Bt|0)!=0,Ft&&(It=+HEAPF64[83],qt=+HEAPF64[82],Rt=It-qt,Ut=ju,zt=+HEAPF64[Ut>>3],Wt=+HEAPF64[83],Xt=zt*Wt,Vt=Rt<=Xt,Vt&&($t=ma,Kt=B,_s_copy($t,17968,Kt,28))),Qt=wi,Gt=+HEAPF64[Qt>>3],Yt=sa,Zt=+HEAPF64[Yt>>3],en=Gt==Zt,en&&(tn=p,nn=+HEAPF64[tn>>3],rn=+HEAPF64[80],sn=nn<=rn,sn&&(un=gn,an=+HEAPF64[un>>3],fn=+HEAPF64[81],ln=an<=fn,ln&&(cn=ma,hn=B,_s_copy(cn,17997,hn,21)))),pn=wi,dn=+HEAPF64[pn>>3],vn=Ju,mn=+HEAPF64[vn>>3],bn=dn==mn;do if(bn){wn=p,En=+HEAPF64[wn>>3],Sn=+HEAPF64[80],xn=En>Sn;if(!xn){Tn=gn,Nn=+HEAPF64[Tn>>3],Cn=+HEAPF64[81],kn=Nn>=Cn;if(!kn)break}Ln=ma,On=B,_s_copy(Ln,18019,On,21)}while(0);Mn=p,_n=+HEAPF64[Mn>>3],Dn=+HEAPF64[80],Pn=_n<=Dn,Pn&&(Hn=gn,Bn=+HEAPF64[Hn>>3],jn=Bn>=0,Fn=gn,In=+HEAPF64[Fn>>3],Rn=-In,Un=jn?In:Rn,zn=Cu,Wn=+HEAPF64[zn>>3],Xn=+HEAPF64[78],Vn=-Xn,$n=Wn*Vn,Jn=Un<=$n,Jn&&(Kn=ma,Qn=B,_s_copy(Kn,18041,Qn,11))),Yn=ma,Zn=_s_cmp(Yn,15451,4,4)|0,er=(Zn|0)==0,er||(tr=ma,nr=_s_cmp(tr,15446,4,4)|0,rr=(nr|0)==0,rr||(ir=HEAP32[649]|0,sr=(ir|0)==1,sr?(or=p,ur=+HEAPF64[or>>3],fr=+HEAPF64[67],lr=ur<=fr,lr?(cr=p,hr=+HEAPF64[cr>>3],pr=+HEAPF64[80],dr=hr>pr,dr?(vr=p,mr=+HEAPF64[vr>>3],gr=wi,yr=+HEAPF64[gr>>3],wr=+HEAPF64[81],Er=yr*wr,Sr=mr-Er,HEAPF64[65]=Sr,xr=+HEAPF64[67],Tr=+HEAPF64[75],Nr=+HEAPF64[81],Cr=Tr*Nr,kr=xr-Cr,HEAPF64[71]=kr,Lr=+HEAPF64[68],Ar=+HEAPF64[76],Mr=+HEAPF64[81],_r=Ar*Mr,Dr=Lr-_r,HEAPF64[72]=Dr,Pr=gn,Hr=+HEAPF64[Pr>>3],Br=+HEAPF64[81],jr=Hr-Br,HEAPF64[66]=jr,Fr=+HEAPF64[69],Ir=+HEAPF64[81],qr=Fr-Ir,HEAPF64[73]=qr,Ur=+HEAPF64[70],zr=+HEAPF64[81],Wr=Ur-zr,HEAPF64[74]=Wr,Xr=wi,_dcstep_(600,568,584,608,576,592,Xr,520,528,2600,656,664)|0,Vr=+HEAPF64[71],$r=+HEAPF64[75],Jr=+HEAPF64[81],Kr=$r*Jr,Qr=Vr+Kr,HEAPF64[67]=Qr,Gr=+HEAPF64[72],Zr=+HEAPF64[76],ei=+HEAPF64[81],ti=Zr*ei,ni=Gr+ti,HEAPF64[68]=ni,ri=+HEAPF64[73],ii=+HEAPF64[81],si=ri+ii,HEAPF64[69]=si,oi=+HEAPF64[74],ui=+HEAPF64[81],ai=oi+ui,HEAPF64[70]=ai):La=52):La=52):La=52,(La|0)==52&&(li=wi,ci=p,hi=gn,_dcstep_(600,536,552,608,544,560,li,ci,hi,2600,656,664)|0),pi=HEAP32[650]|0,di=(pi|0)!=0,di&&(vi=+HEAPF64[76],mi=+HEAPF64[75],gi=vi-mi,ka=gi,yi=ka,bi=yi>=0,Si=ka,xi=-Si,Ti=bi?Si:xi,Ni=+HEAPF64[84],Ci=Ni*.66,ki=Ti>=Ci,ki&&(Li=+HEAPF64[75],Ai=+HEAPF64[76],Oi=+HEAPF64[75],Mi=Ai-Oi,Di=Mi*.5,Pi=Li+Di,Hi=wi,HEAPF64[Hi>>3]=Pi),Bi=+HEAPF64[79],HEAPF64[84]=Bi,ji=+HEAPF64[76],Fi=+HEAPF64[75],Ii=ji-Fi,ka=Ii,qi=ka,Ri=qi>=0,Ui=ka,Wi=-Ui,Xi=Ri?Ui:Wi,HEAPF64[79]=Xi),Vi=HEAP32[650]|0,$i=(Vi|0)!=0,$i?(Ji=+HEAPF64[75],Ki=+HEAPF64[76],Qi=Ji<=Ki,Gi=+HEAPF64[75],Yi=+HEAPF64[76],Zi=Qi?Gi:Yi,HEAPF64[82]=Zi,ts=+HEAPF64[75],ns=+HEAPF64[76],rs=ts>=ns,is=+HEAPF64[75],ss=+HEAPF64[76],os=rs?is:ss,HEAPF64[83]=os):(us=wi,as=+HEAPF64[us>>3],fs=wi,ls=+HEAPF64[fs>>3],hs=+HEAPF64[75],ps=ls-hs,ds=ps*1.1,vs=as+ds,HEAPF64[82]=vs,ms=wi,gs=+HEAPF64[ms>>3],ys=wi,bs=+HEAPF64[ys>>3],ws=+HEAPF64[75],Es=bs-ws,xs=Es*4,Ts=gs+xs,HEAPF64[83]=Ts),Ns=wi,Cs=+HEAPF64[Ns>>3],ks=Ju,Ls=+HEAPF64[ks>>3],As=Cs>=Ls,As?(Os=wi,Ms=+HEAPF64[Os>>3],Bs=Ms):(_s=Ju,Ps=+HEAPF64[_s>>3],Bs=Ps),Hs=wi,HEAPF64[Hs>>3]=Bs,js=wi,Fs=+HEAPF64[js>>3],Is=sa,qs=+HEAPF64[Is>>3],Rs=Fs<=qs,Rs?(Us=wi,zs=+HEAPF64[Us>>3],Js=zs):(Xs=sa,Vs=+HEAPF64[Xs>>3],Js=Vs),$s=wi,HEAPF64[$s>>3]=Js,Ks=HEAP32[650]|0,Qs=(Ks|0)!=0,Qs?(Gs=wi,Ys=+HEAPF64[Gs>>3],Zs=+HEAPF64[82],eo=Ys<=Zs,eo?La=71:(no=wi,ro=+HEAPF64[no>>3],io=+HEAPF64[83],so=ro>=io,so?La=71:La=69)):La=69,(La|0)==69&&(oo=HEAP32[650]|0,uo=(oo|0)!=0,uo&&(ao=+HEAPF64[83],fo=+HEAPF64[82],lo=ao-fo,co=ju,po=+HEAPF64[co>>3],vo=+HEAPF64[83],mo=po*vo,go=lo<=mo,go&&(La=71))),(La|0)==71&&(yo=+HEAPF64[75],bo=wi,HEAPF64[bo>>3]=yo),wo=ma,Eo=B,_s_copy(wo,17923,Eo,2)))}while(0);return So=HEAP32[650]|0,xo=(So|0)!=0,Co=d,ko=Co+4|0,xo?HEAP32[ko>>2]=1:HEAP32[ko>>2]=0,Lo=HEAP32[649]|0,Ao=d,Oo=Ao+8|0,HEAP32[Oo>>2]=Lo,Mo=+HEAPF64[78],_o=N,Do=_o+8|0,HEAPF64[Do>>3]=Mo,Po=+HEAPF64[81],Ho=N,jo=Ho+16|0,HEAPF64[jo>>3]=Po,Fo=+HEAPF64[69],Io=N,qo=Io+24|0,HEAPF64[qo>>3]=Fo,Ro=+HEAPF64[70],Uo=N,zo=Uo+32|0,HEAPF64[zo>>3]=Ro,Wo=+HEAPF64[77],Xo=N,Vo=Xo+40|0,HEAPF64[Vo>>3]=Wo,Jo=+HEAPF64[67],Ko=N,Qo=Ko+48|0,HEAPF64[Qo>>3]=Jo,Go=+HEAPF64[68],Yo=N,Zo=Yo+56|0,HEAPF64[Zo>>3]=Go,eu=+HEAPF64[75],tu=N,nu=tu+64|0,HEAPF64[nu>>3]=eu,ru=+HEAPF64[76],su=N,ou=su+72|0,HEAPF64[ou>>3]=ru,uu=+HEAPF64[82],au=N,fu=au+80|0,HEAPF64[fu>>3]=uu,lu=+HEAPF64[83],cu=N,hu=cu+88|0,HEAPF64[hu>>3]=lu,pu=+HEAPF64[79],du=N,mu=du+96|0,HEAPF64[mu>>3]=pu,gu=+HEAPF64[84],yu=N,bu=yu+104|0,HEAPF64[bu>>3]=gu,h=0,wu=h,STACKTOP=Aa,wu|0}function _dcstep_(e,t,n,r,i,s,o,u,a,f,l,c){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0;var h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0;bp=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),h=e,p=t,gn=n,wi=r,To=i,ka=s,Ml=o,Ph=u,Vh=a,rp=f,d=l,N=c,B=Vh,$=+HEAPF64[B>>3],it=gn,vt=+HEAPF64[it>>3],Ct=gn,jt=+HEAPF64[Ct>>3],Jt=jt>=0,on=gn,yn=+HEAPF64[on>>3],An=-yn,qn=Jt?yn:An,Gn=vt/qn,ar=$*Gn,HEAPF64[89]=ar,br=Ph,Or=+HEAPF64[br>>3],Rr=p,Yr=+HEAPF64[Rr>>3],fi=Or>Yr;do if(fi)Ei=p,_i=+HEAPF64[Ei>>3],zi=Ph,es=+HEAPF64[zi>>3],cs=_i-es,Ss=cs*3,Ds=Ml,Ws=+HEAPF64[Ds>>3],to=h,ho=+HEAPF64[to>>3],No=Ws-ho,Bo=Ss/No,$o=gn,iu=+HEAPF64[$o>>3],vu=Bo+iu,Cu=Vh,ju=+HEAPF64[Cu>>3],Ju=vu+ju,HEAPF64[94]=Ju,sa=+HEAPF64[94],ma=sa>=0,La=+HEAPF64[94],Ia=-La,Qa=ma?La:Ia,dp=Qa,af=gn,bf=+HEAPF64[af>>3],Of=bf>=0,Rf=gn,Yf=+HEAPF64[Rf>>3],fl=-Yf,wl=Of?Yf:fl,vp=wl,_l=dp,zl=vp,ec=_l>=zl,cc=dp,Sc=vp,Dc=ec?cc:Sc,dp=Dc,Wc=Vh,th=+HEAPF64[Wc>>3],hh=th>=0,xh=Vh,Hh=+HEAPF64[xh>>3],jh=-Hh,Fh=hh?Hh:jh,vp=Fh,Ih=dp,qh=vp,Rh=Ih>=qh,Uh=dp,zh=vp,Wh=Rh?Uh:zh,HEAPF64[88]=Wh,Xh=+HEAPF64[94],$h=+HEAPF64[88],Jh=Xh/$h,dp=Jh,Kh=+HEAPF64[88],Qh=dp,Gh=dp,Yh=Qh*Gh,Zh=gn,ep=+HEAPF64[Zh>>3],tp=+HEAPF64[88],np=ep/tp,ip=Vh,sp=+HEAPF64[ip>>3],op=+HEAPF64[88],up=sp/op,ap=np*up,fp=Yh-ap,lp=+Math_sqrt(+fp),cp=Kh*lp,HEAPF64[93]=cp,hp=Ml,pp=+HEAPF64[hp>>3],v=h,m=+HEAPF64[v>>3],g=pp<m,g&&(y=+HEAPF64[93],b=-y,HEAPF64[93]=b),w=+HEAPF64[93],E=gn,S=+HEAPF64[E>>3],x=w-S,T=+HEAPF64[94],C=x+T,HEAPF64[85]=C,k=+HEAPF64[93],L=gn,A=+HEAPF64[L>>3],O=k-A,M=+HEAPF64[93],_=O+M,D=Vh,P=+HEAPF64[D>>3],H=_+P,HEAPF64[86]=H,j=+HEAPF64[85],F=+HEAPF64[86],I=j/F,HEAPF64[87]=I,q=h,R=+HEAPF64[q>>3],U=+HEAPF64[87],z=Ml,W=+HEAPF64[z>>3],X=h,V=+HEAPF64[X>>3],J=W-V,K=U*J,Q=R+K,HEAPF64[90]=Q,G=h,Y=+HEAPF64[G>>3],Z=gn,et=+HEAPF64[Z>>3],tt=p,nt=+HEAPF64[tt>>3],rt=Ph,st=+HEAPF64[rt>>3],ot=nt-st,ut=Ml,at=+HEAPF64[ut>>3],ft=h,lt=+HEAPF64[ft>>3],ct=at-lt,ht=ot/ct,pt=gn,dt=+HEAPF64[pt>>3],mt=ht+dt,gt=et/mt,yt=gt/2,bt=Ml,wt=+HEAPF64[bt>>3],Et=h,St=+HEAPF64[Et>>3],xt=wt-St,Tt=yt*xt,Nt=Y+Tt,HEAPF64[92]=Nt,kt=+HEAPF64[90],Lt=h,At=+HEAPF64[Lt>>3],Ot=kt-At,dp=Ot,Mt=dp,_t=Mt>=0,Dt=dp,Pt=-Dt,Ht=_t?Dt:Pt,Bt=+HEAPF64[92],Ft=h,It=+HEAPF64[Ft>>3],qt=Bt-It,vp=qt,Rt=vp,Ut=Rt>=0,zt=vp,Wt=-zt,Xt=Ut?zt:Wt,Vt=Ht<Xt,$t=+HEAPF64[90],Vt?HEAPF64[91]=$t:(Kt=+HEAPF64[92],Qt=+HEAPF64[90],Gt=Kt-Qt,Yt=Gt/2,Zt=$t+Yt,HEAPF64[91]=Zt),en=rp,HEAP32[en>>2]=1;else{tn=+HEAPF64[89],nn=tn<0;if(nn){rn=p,sn=+HEAPF64[rn>>3],un=Ph,an=+HEAPF64[un>>3],fn=sn-an,ln=fn*3,cn=Ml,hn=+HEAPF64[cn>>3],pn=h,dn=+HEAPF64[pn>>3],vn=hn-dn,mn=ln/vn,bn=gn,wn=+HEAPF64[bn>>3],En=mn+wn,Sn=Vh,xn=+HEAPF64[Sn>>3],Tn=En+xn,HEAPF64[94]=Tn,Nn=+HEAPF64[94],Cn=Nn>=0,kn=+HEAPF64[94],Ln=-kn,On=Cn?kn:Ln,dp=On,Mn=gn,_n=+HEAPF64[Mn>>3],Dn=_n>=0,Pn=gn,Hn=+HEAPF64[Pn>>3],Bn=-Hn,jn=Dn?Hn:Bn,vp=jn,Fn=dp,In=vp,Rn=Fn>=In,Un=dp,zn=vp,Wn=Rn?Un:zn,dp=Wn,Xn=Vh,Vn=+HEAPF64[Xn>>3],$n=Vn>=0,Jn=Vh,Kn=+HEAPF64[Jn>>3],Qn=-Kn,Yn=$n?Kn:Qn,vp=Yn,Zn=dp,er=vp,tr=Zn>=er,nr=dp,rr=vp,ir=tr?nr:rr,HEAPF64[88]=ir,sr=+HEAPF64[94],or=+HEAPF64[88],ur=sr/or,dp=ur,fr=+HEAPF64[88],lr=dp,cr=dp,hr=lr*cr,pr=gn,dr=+HEAPF64[pr>>3],vr=+HEAPF64[88],mr=dr/vr,gr=Vh,yr=+HEAPF64[gr>>3],wr=+HEAPF64[88],Er=yr/wr,Sr=mr*Er,xr=hr-Sr,Tr=+Math_sqrt(+xr),Nr=fr*Tr,HEAPF64[93]=Nr,Cr=Ml,kr=+HEAPF64[Cr>>3],Lr=h,Ar=+HEAPF64[Lr>>3],Mr=kr>Ar,Mr&&(_r=+HEAPF64[93],Dr=-_r,HEAPF64[93]=Dr),Pr=+HEAPF64[93],Hr=Vh,Br=+HEAPF64[Hr>>3],jr=Pr-Br,Fr=+HEAPF64[94],Ir=jr+Fr,HEAPF64[85]=Ir,qr=+HEAPF64[93],Ur=Vh,zr=+HEAPF64[Ur>>3],Wr=qr-zr,Xr=+HEAPF64[93],Vr=Wr+Xr,$r=gn,Jr=+HEAPF64[$r>>3],Kr=Vr+Jr,HEAPF64[86]=Kr,Qr=+HEAPF64[85],Gr=+HEAPF64[86],Zr=Qr/Gr,HEAPF64[87]=Zr,ei=Ml,ti=+HEAPF64[ei>>3],ni=+HEAPF64[87],ri=h,ii=+HEAPF64[ri>>3],si=Ml,oi=+HEAPF64[si>>3],ui=ii-oi,ai=ni*ui,li=ti+ai,HEAPF64[90]=li,ci=Ml,hi=+HEAPF64[ci>>3],pi=Vh,di=+HEAPF64[pi>>3],vi=Vh,mi=+HEAPF64[vi>>3],gi=gn,yi=+HEAPF64[gi>>3],bi=mi-yi,Si=di/bi,xi=h,Ti=+HEAPF64[xi>>3],Ni=Ml,Ci=+HEAPF64[Ni>>3],ki=Ti-Ci,Li=Si*ki,Ai=hi+Li,HEAPF64[92]=Ai,Oi=+HEAPF64[90],Mi=Ml,Di=+HEAPF64[Mi>>3],Pi=Oi-Di,dp=Pi,Hi=dp,Bi=Hi>=0,ji=dp,Fi=-ji,Ii=Bi?ji:Fi,qi=+HEAPF64[92],Ri=Ml,Ui=+HEAPF64[Ri>>3],Wi=qi-Ui,vp=Wi,Xi=vp,Vi=Xi>=0,$i=vp,Ji=-$i,Ki=Vi?$i:Ji,Qi=Ii>Ki,Qi?(Gi=+HEAPF64[90],HEAPF64[91]=Gi):(Yi=+HEAPF64[92],HEAPF64[91]=Yi),Zi=rp,HEAP32[Zi>>2]=1;break}ts=Vh,ns=+HEAPF64[ts>>3],rs=ns>=0,is=Vh,ss=+HEAPF64[is>>3],os=-ss,us=rs?ss:os,as=gn,fs=+HEAPF64[as>>3],ls=fs>=0,hs=gn,ps=+HEAPF64[hs>>3],ds=-ps,vs=ls?ps:ds,ms=us<vs;if(!ms){Af=rp,Mf=HEAP32[Af>>2]|0,_f=(Mf|0)!=0;if(_f){Df=Ph,Pf=+HEAPF64[Df>>3],Hf=To,Bf=+HEAPF64[Hf>>3],jf=Pf-Bf,Ff=jf*3,If=wi,qf=+HEAPF64[If>>3],Uf=Ml,zf=+HEAPF64[Uf>>3],Wf=qf-zf,Xf=Ff/Wf,Vf=ka,$f=+HEAPF64[Vf>>3],Jf=Xf+$f,Kf=Vh,Qf=+HEAPF64[Kf>>3],Gf=Jf+Qf,HEAPF64[94]=Gf,Zf=+HEAPF64[94],el=Zf>=0,tl=+HEAPF64[94],nl=-tl,rl=el?tl:nl,dp=rl,il=ka,sl=+HEAPF64[il>>3],ol=sl>=0,ul=ka,al=+HEAPF64[ul>>3],ll=-al,cl=ol?al:ll,vp=cl,hl=dp,pl=vp,dl=hl>=pl,vl=dp,ml=vp,gl=dl?vl:ml,dp=gl,yl=Vh,bl=+HEAPF64[yl>>3],El=bl>=0,Sl=Vh,xl=+HEAPF64[Sl>>3],Tl=-xl,Nl=El?xl:Tl,vp=Nl,Cl=dp,kl=vp,Ll=Cl>=kl,Al=dp,Ol=vp,Dl=Ll?Al:Ol,HEAPF64[88]=Dl,Pl=+HEAPF64[94],Hl=+HEAPF64[88],Bl=Pl/Hl,dp=Bl,jl=+HEAPF64[88],Fl=dp,Il=dp,ql=Fl*Il,Rl=ka,Ul=+HEAPF64[Rl>>3],Wl=+HEAPF64[88],Xl=Ul/Wl,Vl=Vh,$l=+HEAPF64[Vl>>3],Jl=+HEAPF64[88],Kl=$l/Jl,Ql=Xl*Kl,Gl=ql-Ql,Yl=+Math_sqrt(+Gl),Zl=jl*Yl,HEAPF64[93]=Zl,tc=Ml,nc=+HEAPF64[tc>>3],rc=wi,ic=+HEAPF64[rc>>3],sc=nc>ic,sc&&(oc=+HEAPF64[93],uc=-oc,HEAPF64[93]=uc),ac=+HEAPF64[93],fc=Vh,lc=+HEAPF64[fc>>3],hc=ac-lc,pc=+HEAPF64[94],dc=hc+pc,HEAPF64[85]=dc,vc=+HEAPF64[93],mc=Vh,gc=+HEAPF64[mc>>3],yc=vc-gc,bc=+HEAPF64[93],wc=yc+bc,Ec=ka,xc=+HEAPF64[Ec>>3],Tc=wc+xc,HEAPF64[86]=Tc,Nc=+HEAPF64[85],Cc=+HEAPF64[86],kc=Nc/Cc,HEAPF64[87]=kc,Lc=Ml,Ac=+HEAPF64[Lc>>3],Oc=+HEAPF64[87],Mc=wi,_c=+HEAPF64[Mc>>3],Pc=Ml,Hc=+HEAPF64[Pc>>3],Bc=_c-Hc,jc=Oc*Bc,Fc=Ac+jc,HEAPF64[90]=Fc,Ic=+HEAPF64[90],HEAPF64[91]=Ic;break}qc=Ml,Rc=+HEAPF64[qc>>3],Uc=h,zc=+HEAPF64[Uc>>3],Xc=Rc>zc;if(Xc){Vc=N,$c=+HEAPF64[Vc>>3],HEAPF64[91]=$c;break}Jc=d,Kc=+HEAPF64[Jc>>3],HEAPF64[91]=Kc;break}gs=p,ys=+HEAPF64[gs>>3],bs=Ph,ws=+HEAPF64[bs>>3],Es=ys-ws,xs=Es*3,Ts=Ml,Ns=+HEAPF64[Ts>>3],Cs=h,ks=+HEAPF64[Cs>>3],Ls=Ns-ks,As=xs/Ls,Os=gn,Ms=+HEAPF64[Os>>3],_s=As+Ms,Ps=Vh,Hs=+HEAPF64[Ps>>3],Bs=_s+Hs,HEAPF64[94]=Bs,js=+HEAPF64[94],Fs=js>=0,Is=+HEAPF64[94],qs=-Is,Rs=Fs?Is:qs,dp=Rs,Us=gn,zs=+HEAPF64[Us>>3],Xs=zs>=0,Vs=gn,$s=+HEAPF64[Vs>>3],Js=-$s,Ks=Xs?$s:Js,vp=Ks,Qs=dp,Gs=vp,Ys=Qs>=Gs,Zs=dp,eo=vp,no=Ys?Zs:eo,dp=no,ro=Vh,io=+HEAPF64[ro>>3],so=io>=0,oo=Vh,uo=+HEAPF64[oo>>3],ao=-uo,fo=so?uo:ao,vp=fo,lo=dp,co=vp,po=lo>=co,vo=dp,mo=vp,go=po?vo:mo,HEAPF64[88]=go,yo=+HEAPF64[94],bo=+HEAPF64[88],wo=yo/bo,mp=wo,dp=0,Eo=mp,So=mp,xo=Eo*So,Co=gn,ko=+HEAPF64[Co>>3],Lo=+HEAPF64[88],Ao=ko/Lo,Oo=Vh,Mo=+HEAPF64[Oo>>3],_o=+HEAPF64[88],Do=Mo/_o,Po=Ao*Do,Ho=xo-Po,vp=Ho,jo=+HEAPF64[88],Fo=dp,Io=vp,qo=Fo>=Io,Ro=dp,Uo=vp,zo=qo?Ro:Uo,Wo=+Math_sqrt(+zo),Xo=jo*Wo,HEAPF64[93]=Xo,Vo=Ml,Jo=+HEAPF64[Vo>>3],Ko=h,Qo=+HEAPF64[Ko>>3],Go=Jo>Qo,Go&&(Yo=+HEAPF64[93],Zo=-Yo,HEAPF64[93]=Zo),eu=+HEAPF64[93],tu=Vh,nu=+HEAPF64[tu>>3],ru=eu-nu,su=+HEAPF64[94],ou=ru+su,HEAPF64[85]=ou,uu=+HEAPF64[93],au=gn,fu=+HEAPF64[au>>3],lu=Vh,cu=+HEAPF64[lu>>3],hu=fu-cu,pu=uu+hu,du=+HEAPF64[93],mu=pu+du,HEAPF64[86]=mu,gu=+HEAPF64[85],yu=+HEAPF64[86],bu=gu/yu,HEAPF64[87]=bu,wu=+HEAPF64[87],Eu=wu<0,Su=+HEAPF64[93],xu=Su!=0,gp=Eu&xu,Tu=Ml,Nu=+HEAPF64[Tu>>3];do{if(!gp){Hu=h,Bu=+HEAPF64[Hu>>3],Fu=Nu>Bu;if(Fu){Iu=N,qu=+HEAPF64[Iu>>3],HEAPF64[90]=qu;break}Ru=d,Uu=+HEAPF64[Ru>>3],HEAPF64[90]=Uu;break}ku=+HEAPF64[87],Lu=h,Au=+HEAPF64[Lu>>3],Ou=Ml,Mu=+HEAPF64[Ou>>3],_u=Au-Mu,Du=ku*_u,Pu=Nu+Du,HEAPF64[90]=Pu}while(0);zu=Ml,Wu=+HEAPF64[zu>>3],Xu=Vh,Vu=+HEAPF64[Xu>>3],$u=Vh,Ku=+HEAPF64[$u>>3],Qu=gn,Gu=+HEAPF64[Qu>>3],Yu=Ku-Gu,Zu=Vu/Yu,ea=h,ta=+HEAPF64[ea>>3],na=Ml,ra=+HEAPF64[na>>3],ia=ta-ra,oa=Zu*ia,ua=Wu+oa,HEAPF64[92]=ua,aa=rp,fa=HEAP32[aa>>2]|0,la=(fa|0)!=0,ca=+HEAPF64[90],ha=Ml,pa=+HEAPF64[ha>>3],da=ca-pa,dp=da,va=dp,ga=va>=0,ya=dp,ba=-ya,wa=ga?ya:ba,Ea=+HEAPF64[92],Sa=Ml,xa=+HEAPF64[Sa>>3],Ta=Ea-xa,vp=Ta,Na=vp,Ca=Na>=0,Aa=vp,Oa=-Aa,Ma=Ca?Aa:Oa;if(la){_a=wa<Ma,_a?(Da=+HEAPF64[90],HEAPF64[91]=Da):(Pa=+HEAPF64[92],HEAPF64[91]=Pa),Ha=Ml,Ba=+HEAPF64[Ha>>3],ja=h,Fa=+HEAPF64[ja>>3],qa=Ba>Fa,Ra=Ml,Ua=+HEAPF64[Ra>>3],za=wi,Wa=+HEAPF64[za>>3],Xa=Ml,Va=+HEAPF64[Xa>>3],$a=Wa-Va,Ja=$a*.66,Ka=Ua+Ja,dp=Ka,Ga=dp,Ya=+HEAPF64[91];if(qa){Za=Ga<=Ya,ef=dp,tf=+HEAPF64[91],nf=Za?ef:tf,HEAPF64[91]=nf;break}rf=Ga>=Ya,sf=dp,of=+HEAPF64[91],uf=rf?sf:of,HEAPF64[91]=uf;break}ff=wa>Ma,ff?(lf=+HEAPF64[90],HEAPF64[91]=lf):(cf=+HEAPF64[92],HEAPF64[91]=cf),hf=N,pf=+HEAPF64[hf>>3],df=+HEAPF64[91],vf=pf<=df,vf?(mf=N,gf=+HEAPF64[mf>>3],wf=gf):(yf=+HEAPF64[91],wf=yf),HEAPF64[91]=wf,Ef=d,Sf=+HEAPF64[Ef>>3],xf=+HEAPF64[91],Tf=Sf>=xf,Tf?(Nf=d,Cf=+HEAPF64[Nf>>3],Lf=Cf):(kf=+HEAPF64[91],Lf=kf),HEAPF64[91]=Lf}while(0);return Qc=Ph,Gc=+HEAPF64[Qc>>3],Yc=p,Zc=+HEAPF64[Yc>>3],eh=Gc>Zc,eh?(nh=Ml,rh=+HEAPF64[nh>>3],ih=wi,HEAPF64[ih>>3]=rh,sh=Ph,oh=+HEAPF64[sh>>3],uh=To,HEAPF64[uh>>3]=oh,ah=Vh,fh=+HEAPF64[ah>>3],lh=ka,HEAPF64[lh>>3]=fh,Dh=+HEAPF64[91],Bh=Ml,HEAPF64[Bh>>3]=Dh,STACKTOP=bp,0):(ch=+HEAPF64[89],ph=ch<0,ph&&(dh=h,vh=+HEAPF64[dh>>3],mh=wi,HEAPF64[mh>>3]=vh,gh=p,yh=+HEAPF64[gh>>3],bh=To,HEAPF64[bh>>3]=yh,wh=gn,Eh=+HEAPF64[wh>>3],Sh=ka,HEAPF64[Sh>>3]=Eh),Th=Ml,Nh=+HEAPF64[Th>>3],Ch=h,HEAPF64[Ch>>3]=Nh,kh=Ph,Lh=+HEAPF64[kh>>3],Ah=p,HEAPF64[Ah>>3]=Lh,Oh=Vh,Mh=+HEAPF64[Oh>>3],_h=gn,HEAPF64[_h>>3]=Mh,Dh=+HEAPF64[91],Bh=Ml,HEAPF64[Bh>>3]=Dh,STACKTOP=bp,0)}function _dgesv_(e,t,n,r,i,s,o,u){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0;var a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0;Jt=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Vt=Jt,f=e,w=t,M=n,U=r,Z=i,lt=s,Et=o,_t=u,Rt=U,l=HEAP32[Rt>>2]|0,Ut=l,c=Ut,h=1+c|0,zt=h,p=zt,d=M,v=0-p|0,m=d+(v<<3)|0,M=m,g=Z,y=g+ -4|0,Z=y,b=Et,E=HEAP32[b>>2]|0,Wt=E,S=Wt,x=1+S|0,Xt=x,T=Xt,N=lt,C=0-T|0,k=N+(C<<3)|0,lt=k,L=_t,HEAP32[L>>2]=0,A=f,O=HEAP32[A>>2]|0,_=(O|0)<0;do if(_)D=_t,HEAP32[D>>2]=-1;else{P=w,H=HEAP32[P>>2]|0,B=(H|0)<0;if(B){j=_t,HEAP32[j>>2]=-2;break}F=U,I=HEAP32[F>>2]|0,q=f,R=HEAP32[q>>2]|0,z=1>=(R|0),z?$=1:(W=f,X=HEAP32[W>>2]|0,$=X),V=(I|0)<($|0);if(V){J=_t,HEAP32[J>>2]=-4;break}K=Et,Q=HEAP32[K>>2]|0,G=f,Y=HEAP32[G>>2]|0,et=1>=(Y|0),et?it=1:(tt=f,nt=HEAP32[tt>>2]|0,it=nt),rt=(Q|0)<(it|0),rt&&(st=_t,HEAP32[st>>2]=-7)}while(0);return ot=_t,ut=HEAP32[ot>>2]|0,at=(ut|0)!=0,at?(ft=_t,ct=HEAP32[ft>>2]|0,ht=0-ct|0,HEAP32[Vt>>2]=ht,_xerbla_(18053,Vt)|0,a=0,qt=a,STACKTOP=Jt,qt|0):(pt=f,dt=f,vt=zt,mt=M,gt=mt+(vt<<3)|0,yt=U,bt=Z,wt=bt+4|0,St=_t,_dgetrf_(pt,dt,gt,yt,wt,St)|0,xt=_t,Tt=HEAP32[xt>>2]|0,Nt=(Tt|0)==0,Nt&&(Ct=f,kt=w,Lt=zt,At=M,Ot=At+(Lt<<3)|0,Mt=U,Dt=Z,Pt=Dt+4|0,Ht=Xt,Bt=lt,jt=Bt+(Ht<<3)|0,Ft=Et,It=_t,_dgetrs_(18087,Ct,kt,Ot,Mt,Pt,jt,Ft,It)|0),a=0,qt=a,STACKTOP=Jt,qt|0)}function _dgetrf_(e,t,n,r,i,s){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0;var o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0;js=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Cs=js+36|0,Ls=js+28|0,As=js+24|0,_s=js+12|0,Ds=js+8|0,Ms=js,a=e,hn=t,ei=n,ci=r,Si=i,Di=s,Wi=ci,ts=HEAP32[Wi>>2]|0,xs=ts,hs=xs,f=1+hs|0,Ts=f,w=Ts,M=ei,U=0-w|0,Z=M+(U<<3)|0,ei=Z,lt=Si,Et=lt+ -4|0,Si=Et,_t=Di,HEAP32[_t>>2]=0,zt=a,en=HEAP32[zt>>2]|0,pn=(en|0)<0;do if(pn)Tn=Di,HEAP32[Tn>>2]=-1;else{Hn=hn,Vn=HEAP32[Hn>>2]|0,rr=(Vn|0)<0;if(rr){dr=Di,HEAP32[dr>>2]=-2;break}Nr=ci,Br=HEAP32[Nr>>2]|0,$r=a,Zr=HEAP32[$r>>2]|0,ti=1>=(Zr|0),ti?si=1:(ni=a,ri=HEAP32[ni>>2]|0,si=ri),ii=(Br|0)<(si|0),ii&&(oi=Di,HEAP32[oi>>2]=-4)}while(0);ui=Di,ai=HEAP32[ui>>2]|0,fi=(ai|0)!=0;if(fi)return li=Di,hi=HEAP32[li>>2]|0,pi=0-hi|0,HEAP32[Cs>>2]=pi,_xerbla_(18060,Cs)|0,u=0,Yr=u,STACKTOP=js,Yr|0;di=a,vi=HEAP32[di>>2]|0,mi=(vi|0)==0;if(!mi){gi=hn,yi=HEAP32[gi>>2]|0,bi=(yi|0)==0;if(!bi){wi=a,Ei=hn,xi=_ilaenv_(2604,18060,18067,wi,Ei,2608,2608)|0,Ps=xi,Ti=Ps,Ni=(Ti|0)<=1;e:do if(Ni)Bs=19;else{Ci=Ps,ki=a,Li=HEAP32[ki>>2]|0,Ai=hn,Oi=HEAP32[Ai>>2]|0,Mi=(Li|0)<=(Oi|0),Mi?(_i=a,Pi=HEAP32[_i>>2]|0,Fi=Pi):(Hi=hn,Bi=HEAP32[Hi>>2]|0,Fi=Bi),ji=(Ci|0)>=(Fi|0);if(ji)Bs=19;else{Ki=a,Qi=HEAP32[Ki>>2]|0,Gi=hn,Yi=HEAP32[Gi>>2]|0,Zi=(Qi|0)<=(Yi|0),Zi?(es=a,ns=HEAP32[es>>2]|0,ss=ns):(rs=hn,is=HEAP32[rs>>2]|0,ss=is),HEAP32[Cs>>2]=ss,os=Ps,ks=os,HEAP32[_s>>2]=1;for(;;){us=ks,as=(us|0)<0,fs=HEAP32[_s>>2]|0,ls=HEAP32[Cs>>2]|0,cs=(fs|0)>=(ls|0),ps=(fs|0)<=(ls|0),o=as?cs:ps,ds=o&1,vs=(ds|0)!=0;if(!vs)break e;ms=a,gs=HEAP32[ms>>2]|0,ys=hn,bs=HEAP32[ys>>2]|0,ws=(gs|0)<=(bs|0),ws?(Es=a,Ss=HEAP32[Es>>2]|0,d=Ss):(l=hn,c=HEAP32[l>>2]|0,d=c),h=HEAP32[_s>>2]|0,p=d-h|0,v=p+1|0,HEAP32[Ls>>2]=v,m=HEAP32[Ls>>2]|0,g=Ps,y=(m|0)<=(g|0),b=HEAP32[Ls>>2]|0,E=Ps,S=y?b:E,HEAP32[Ds>>2]=S,x=a,T=HEAP32[x>>2]|0,N=HEAP32[_s>>2]|0,C=T-N|0,k=C+1|0,HEAP32[Ls>>2]=k,L=HEAP32[_s>>2]|0,A=HEAP32[_s>>2]|0,O=xs,_=Math_imul(A,O)|0,D=L+_|0,P=ei,H=P+(D<<3)|0,B=ci,j=HEAP32[_s>>2]|0,F=Si,I=F+(j<<2)|0,_dgetf2_(Ls,Ds,H,B,I,Ms)|0,q=Di,R=HEAP32[q>>2]|0,z=(R|0)==0,W=HEAP32[Ms>>2]|0,X=(W|0)>0,Hs=z&X,Hs&&(V=HEAP32[Ms>>2]|0,$=HEAP32[_s>>2]|0,J=V+$|0,K=J-1|0,Q=Di,HEAP32[Q>>2]=K),G=a,Y=HEAP32[G>>2]|0,HEAP32[As>>2]=Y,et=HEAP32[_s>>2]|0,tt=HEAP32[Ds>>2]|0,nt=et+tt|0,rt=nt-1|0,Os=rt,it=HEAP32[As>>2]|0,st=Os,ot=(it|0)<=(st|0),ut=HEAP32[As>>2]|0,at=Os,ft=ot?ut:at,HEAP32[Ls>>2]=ft,ct=HEAP32[_s>>2]|0,Ns=ct;for(;;){ht=Ns,pt=HEAP32[Ls>>2]|0,dt=(ht|0)<=(pt|0),vt=HEAP32[_s>>2]|0,mt=vt-1|0;if(!dt)break;gt=Ns,yt=Si,bt=yt+(gt<<2)|0,wt=HEAP32[bt>>2]|0,St=mt+wt|0,xt=Ns,Tt=Si,Nt=Tt+(xt<<2)|0,HEAP32[Nt>>2]=St,Ct=Ns,kt=Ct+1|0,Ns=kt}HEAP32[Ls>>2]=mt,Lt=HEAP32[_s>>2]|0,At=HEAP32[Ds>>2]|0,Ot=Lt+At|0,Mt=Ot-1|0,HEAP32[As>>2]=Mt,Dt=Ts,Pt=ei,Ht=Pt+(Dt<<3)|0,Bt=ci,jt=Si,Ft=jt+4|0,_dlaswp_(Ls,Ht,Bt,_s,As,Ft,2604)|0,It=HEAP32[_s>>2]|0,qt=HEAP32[Ds>>2]|0,Rt=It+qt|0,Ut=hn,Wt=HEAP32[Ut>>2]|0,Xt=(Rt|0)<=(Wt|0),Xt&&(Vt=hn,$t=HEAP32[Vt>>2]|0,Jt=HEAP32[_s>>2]|0,Kt=$t-Jt|0,Qt=HEAP32[Ds>>2]|0,Gt=Kt-Qt|0,Yt=Gt+1|0,HEAP32[Ls>>2]=Yt,Zt=HEAP32[_s>>2]|0,tn=HEAP32[Ds>>2]|0,nn=Zt+tn|0,rn=nn-1|0,HEAP32[As>>2]=rn,sn=HEAP32[_s>>2]|0,on=HEAP32[Ds>>2]|0,un=sn+on|0,an=xs,fn=Math_imul(un,an)|0,ln=fn+1|0,cn=ei,dn=cn+(ln<<3)|0,vn=ci,mn=Si,gn=mn+4|0,_dlaswp_(Ls,dn,vn,_s,As,gn,2604)|0,yn=hn,bn=HEAP32[yn>>2]|0,wn=HEAP32[_s>>2]|0,En=bn-wn|0,Sn=HEAP32[Ds>>2]|0,xn=En-Sn|0,Nn=xn+1|0,HEAP32[Ls>>2]=Nn,Cn=HEAP32[_s>>2]|0,kn=HEAP32[_s>>2]|0,Ln=xs,An=Math_imul(kn,Ln)|0,On=Cn+An|0,Mn=ei,_n=Mn+(On<<3)|0,Dn=ci,Pn=HEAP32[_s>>2]|0,Bn=HEAP32[_s>>2]|0,jn=HEAP32[Ds>>2]|0,Fn=Bn+jn|0,In=xs,qn=Math_imul(Fn,In)|0,Rn=Pn+qn|0,Un=ei,zn=Un+(Rn<<3)|0,Wn=ci,_dtrsm_(18076,18081,18087,18100,Ds,Ls,760,_n,Dn,zn,Wn)|0,Xn=HEAP32[_s>>2]|0,$n=HEAP32[Ds>>2]|0,Jn=Xn+$n|0,Kn=a,Qn=HEAP32[Kn>>2]|0,Gn=(Jn|0)<=(Qn|0),Gn&&(Yn=a,Zn=HEAP32[Yn>>2]|0,er=HEAP32[_s>>2]|0,tr=Zn-er|0,nr=HEAP32[Ds>>2]|0,ir=tr-nr|0,sr=ir+1|0,HEAP32[Ls>>2]=sr,or=hn,ur=HEAP32[or>>2]|0,ar=HEAP32[_s>>2]|0,fr=ur-ar|0,lr=HEAP32[Ds>>2]|0,cr=fr-lr|0,hr=cr+1|0,HEAP32[As>>2]=hr,pr=HEAP32[_s>>2]|0,vr=HEAP32[Ds>>2]|0,mr=pr+vr|0,gr=HEAP32[_s>>2]|0,yr=xs,br=Math_imul(gr,yr)|0,wr=mr+br|0,Er=ei,Sr=Er+(wr<<3)|0,xr=ci,Tr=HEAP32[_s>>2]|0,Cr=HEAP32[_s>>2]|0,kr=HEAP32[Ds>>2]|0,Lr=Cr+kr|0,Ar=xs,Or=Math_imul(Lr,Ar)|0,Mr=Tr+Or|0,_r=ei,Dr=_r+(Mr<<3)|0,Pr=ci,Hr=HEAP32[_s>>2]|0,jr=HEAP32[Ds>>2]|0,Fr=Hr+jr|0,Ir=HEAP32[_s>>2]|0,qr=HEAP32[Ds>>2]|0,Rr=Ir+qr|0,Ur=xs,zr=
Math_imul(Rr,Ur)|0,Wr=Fr+zr|0,Xr=ei,Vr=Xr+(Wr<<3)|0,Jr=ci,_dgemm_(18087,18087,Ls,As,Ds,768,Sr,xr,Dr,Pr,760,Vr,Jr)|0)),Kr=ks,Qr=HEAP32[_s>>2]|0,Gr=Qr+Kr|0,HEAP32[_s>>2]=Gr}}}while(0);return(Bs|0)==19&&(Ii=a,qi=hn,Ri=Ts,Ui=ei,zi=Ui+(Ri<<3)|0,Xi=ci,Vi=Si,$i=Vi+4|0,Ji=Di,_dgetf2_(Ii,qi,zi,Xi,$i,Ji)|0),u=0,Yr=u,STACKTOP=js,Yr|0}}return u=0,Yr=u,STACKTOP=js,Yr|0}function _dgetrs_(e,t,n,r,i,s,o,u,a){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0;var f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0;Jn=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Xn=Jn+4|0,l=e,ot=t,gt=n,Lt=r,It=i,Qt=s,an=o,bn=u,On=a,c=It,S=HEAP32[c>>2]|0,Rn=S,D=Rn,W=1+D|0,Un=W,et=Un,tt=Lt,nt=0-et|0,rt=tt+(nt<<3)|0,Lt=rt,it=Qt,st=it+ -4|0,Qt=st,ut=bn,at=HEAP32[ut>>2]|0,zn=at,ft=zn,lt=1+ft|0,Wn=lt,ct=Wn,ht=an,pt=0-ct|0,dt=ht+(pt<<3)|0,an=dt,vt=On,HEAP32[vt>>2]=0,mt=l,yt=_lsame_(mt,20460)|0,Vn=yt,bt=Vn,wt=(bt|0)!=0,wt?$n=5:(Et=l,St=_lsame_(Et,20466)|0,xt=(St|0)!=0,xt?$n=5:(Tt=l,Nt=_lsame_(Tt,20468)|0,Ct=(Nt|0)!=0,Ct?$n=5:(kt=On,HEAP32[kt>>2]=-1)));do if(($n|0)==5){At=ot,Ot=HEAP32[At>>2]|0,Mt=(Ot|0)<0;if(Mt){_t=On,HEAP32[_t>>2]=-2;break}Dt=gt,Pt=HEAP32[Dt>>2]|0,Ht=(Pt|0)<0;if(Ht){Bt=On,HEAP32[Bt>>2]=-3;break}jt=It,Ft=HEAP32[jt>>2]|0,qt=ot,Rt=HEAP32[qt>>2]|0,Ut=1>=(Rt|0),Ut?Vt=1:(zt=ot,Wt=HEAP32[zt>>2]|0,Vt=Wt),Xt=(Ft|0)<(Vt|0);if(Xt){$t=On,HEAP32[$t>>2]=-5;break}Jt=bn,Kt=HEAP32[Jt>>2]|0,Gt=ot,Yt=HEAP32[Gt>>2]|0,Zt=1>=(Yt|0),Zt?rn=1:(en=ot,tn=HEAP32[en>>2]|0,rn=tn),nn=(Kt|0)<(rn|0),nn&&(sn=On,HEAP32[sn>>2]=-8)}while(0);on=On,un=HEAP32[on>>2]|0,fn=(un|0)!=0;if(fn)return ln=On,cn=HEAP32[ln>>2]|0,hn=0-cn|0,HEAP32[Xn>>2]=hn,_xerbla_(18069,Xn)|0,f=0,Z=f,STACKTOP=Jn,Z|0;pn=ot,dn=HEAP32[pn>>2]|0,vn=(dn|0)==0;if(!vn){mn=gt,gn=HEAP32[mn>>2]|0,yn=(gn|0)==0;if(!yn)return wn=Vn,En=(wn|0)!=0,En?(Sn=gt,xn=Wn,Tn=an,Nn=Tn+(xn<<3)|0,Cn=bn,kn=ot,Ln=Qt,An=Ln+4|0,_dlaswp_(Sn,Nn,Cn,2612,kn,An,2612)|0,Mn=ot,_n=gt,Dn=Un,Pn=Lt,Hn=Pn+(Dn<<3)|0,Bn=It,jn=Wn,Fn=an,In=Fn+(jn<<3)|0,qn=bn,_dtrsm_(18076,18081,18087,18100,Mn,_n,776,Hn,Bn,In,qn)|0,h=ot,p=gt,d=Un,v=Lt,m=v+(d<<3)|0,g=It,y=Wn,b=an,w=b+(y<<3)|0,E=bn,_dtrsm_(18076,18105,18087,18111,h,p,776,m,g,w,E)|0):(x=ot,T=gt,N=Un,C=Lt,k=C+(N<<3)|0,L=It,A=Wn,O=an,M=O+(A<<3)|0,_=bn,_dtrsm_(18076,18105,18120,18111,x,T,776,k,L,M,_)|0,P=ot,H=gt,B=Un,j=Lt,F=j+(B<<3)|0,I=It,q=Wn,R=an,U=R+(q<<3)|0,z=bn,_dtrsm_(18076,18081,18120,18100,P,H,776,F,I,U,z)|0,X=gt,V=Wn,$=an,J=$+(V<<3)|0,K=bn,Q=ot,G=Qt,Y=G+4|0,_dlaswp_(X,J,K,2612,Q,Y,2616)|0),f=0,Z=f,STACKTOP=Jn,Z|0}return f=0,Z=f,STACKTOP=Jn,Z|0}function _dlaswp_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0;ni=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),l=e,rn=t,vn=n,Cn=r,jn=i,Jn=s,sr=o,mr=vn,kr=HEAP32[mr>>2]|0,Fr=kr,c=Fr,S=1+c|0,Ir=S,D=Ir,W=rn,tt=0-D|0,ht=W+(tt<<3)|0,rn=ht,xt=Jn,Pt=xt+ -4|0,Jn=Pt,Xt=sr,nn=HEAP32[Xt>>2]|0,sn=(nn|0)>0;do{if(!sn){hn=sr,pn=HEAP32[hn>>2]|0,dn=(pn|0)<0;if(dn){mn=jn,gn=HEAP32[mn>>2]|0,yn=1-gn|0,bn=sr,wn=HEAP32[bn>>2]|0,En=Math_imul(yn,wn)|0,Sn=En+1|0,Qr=Sn,xn=jn,Tn=HEAP32[xn>>2]|0,qr=Tn,Nn=Cn,kn=HEAP32[Nn>>2]|0,Rr=kn,$r=-1;break}return f=0,tn=f,STACKTOP=ni,tn|0}on=Cn,un=HEAP32[on>>2]|0,Qr=un,an=Cn,fn=HEAP32[an>>2]|0,qr=fn,ln=jn,cn=HEAP32[ln>>2]|0,Rr=cn,$r=1}while(0);Ln=l,An=HEAP32[Ln>>2]|0,On=(An|0)/32&-1,Mn=On<<5,Zr=Mn,_n=Zr,Dn=(_n|0)!=0;e:do if(Dn){Pn=Zr,zr=Pn,Gr=1;for(;;){Hn=Gr,Bn=zr,Fn=(Hn|0)<=(Bn|0);if(!Fn)break e;In=Qr,Kr=In,qn=Rr,Wr=qn,Rn=$r,Xr=Rn,Un=qr,Ur=Un;for(;;){zn=Xr,Wn=(zn|0)<0,Xn=Ur,Vn=Wr,$n=(Xn|0)>=(Vn|0),Kn=(Xn|0)<=(Vn|0),u=Wn?$n:Kn,Qn=u&1,Gn=(Qn|0)!=0;if(!Gn)break;Yn=Kr,Zn=Jn,er=Zn+(Yn<<2)|0,tr=HEAP32[er>>2]|0,Jr=tr,nr=Jr,rr=Ur,ir=(nr|0)!=(rr|0);t:do if(ir){or=Gr,ur=or+31|0,Vr=ur,ar=Gr,Yr=ar;for(;;){fr=Yr,lr=Vr,cr=(fr|0)<=(lr|0);if(!cr)break t;hr=Ur,pr=Yr,dr=Fr,vr=Math_imul(pr,dr)|0,gr=hr+vr|0,yr=rn,br=yr+(gr<<3)|0,wr=+HEAPF64[br>>3],ei=wr,Er=Jr,Sr=Yr,xr=Fr,Tr=Math_imul(Sr,xr)|0,Nr=Er+Tr|0,Cr=rn,Lr=Cr+(Nr<<3)|0,Ar=+HEAPF64[Lr>>3],Or=Ur,Mr=Yr,_r=Fr,Dr=Math_imul(Mr,_r)|0,Pr=Or+Dr|0,Hr=rn,Br=Hr+(Pr<<3)|0,HEAPF64[Br>>3]=Ar,jr=ei,h=Jr,p=Yr,d=Fr,v=Math_imul(p,d)|0,m=h+v|0,g=rn,y=g+(m<<3)|0,HEAPF64[y>>3]=jr,b=Yr,w=b+1|0,Yr=w}}while(0);E=sr,x=HEAP32[E>>2]|0,T=Kr,N=T+x|0,Kr=N,C=Xr,k=Ur,L=k+C|0,Ur=L}A=Gr,O=A+32|0,Gr=O}}while(0);M=Zr,_=l,P=HEAP32[_>>2]|0,H=(M|0)!=(P|0);n:do if(H){B=Zr,j=B+1|0,Zr=j,F=Qr,Kr=F,I=Rr,zr=I,q=$r,Xr=q,R=qr,Ur=R;for(;;){U=Xr,z=(U|0)<0,X=Ur,V=zr,$=(X|0)>=(V|0),J=(X|0)<=(V|0),a=z?$:J,K=a&1,Q=(K|0)!=0;if(!Q)break n;G=Kr,Y=Jn,Z=Y+(G<<2)|0,et=HEAP32[Z>>2]|0,Jr=et,nt=Jr,rt=Ur,it=(nt|0)!=(rt|0);r:do if(it){st=l,ot=HEAP32[st>>2]|0,Wr=ot,ut=Zr,Yr=ut;for(;;){at=Yr,ft=Wr,lt=(at|0)<=(ft|0);if(!lt)break r;ct=Ur,pt=Yr,dt=Fr,vt=Math_imul(pt,dt)|0,mt=ct+vt|0,gt=rn,yt=gt+(mt<<3)|0,bt=+HEAPF64[yt>>3],ei=bt,wt=Jr,Et=Yr,St=Fr,Tt=Math_imul(Et,St)|0,Nt=wt+Tt|0,Ct=rn,kt=Ct+(Nt<<3)|0,Lt=+HEAPF64[kt>>3],At=Ur,Ot=Yr,Mt=Fr,_t=Math_imul(Ot,Mt)|0,Dt=At+_t|0,Ht=rn,Bt=Ht+(Dt<<3)|0,HEAPF64[Bt>>3]=Lt,jt=ei,Ft=Jr,It=Yr,qt=Fr,Rt=Math_imul(It,qt)|0,Ut=Ft+Rt|0,zt=rn,Wt=zt+(Ut<<3)|0,HEAPF64[Wt>>3]=jt,Vt=Yr,$t=Vt+1|0,Yr=$t}}while(0);Jt=sr,Kt=HEAP32[Jt>>2]|0,Qt=Kr,Gt=Qt+Kt|0,Kr=Gt,Yt=Xr,Zt=Ur,en=Zt+Yt|0,Ur=en}}while(0);return f=0,tn=f,STACKTOP=ni,tn|0}function _dlamch_(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0;_t=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),xt=_t+44|0,Ct=_t+40|0,St=_t+36|0,Nt=_t+32|0,Tt=_t+28|0,kt=_t+24|0,t=e,n=HEAP32[655]|0,p=(n|0)!=0,p&&(_dlamc2_(St,Ct,kt,784,Nt,792,Tt,800)|0,T=HEAP32[St>>2]|0,H=+(T|0),HEAPF64[101]=H,V=HEAP32[Ct>>2]|0,rt=+(V|0),HEAPF64[102]=rt,dt=HEAP32[kt>>2]|0,wt=(dt|0)!=0,wt?(HEAPF64[103]=1,Et=HEAP32[Ct>>2]|0,r=1-Et|0,HEAP32[xt>>2]=r,i=+_pow_di(808,xt),s=i/2,HEAPF64[98]=s):(HEAPF64[103]=0,o=HEAP32[Ct>>2]|0,u=1-o|0,HEAP32[xt>>2]=u,a=+_pow_di(808,xt),HEAPF64[98]=a),f=+HEAPF64[98],l=+HEAPF64[101],c=f*l,HEAPF64[104]=c,h=HEAP32[Nt>>2]|0,d=+(h|0),HEAPF64[105]=d,v=HEAP32[Tt>>2]|0,m=+(v|0),HEAPF64[106]=m,g=+HEAPF64[99],HEAPF64[107]=g,y=+HEAPF64[100],b=1/y,Ot=b,w=Ot,E=+HEAPF64[107],S=w>=E,S&&(x=Ot,N=+HEAPF64[98],C=N+1,k=x*C,HEAPF64[107]=k)),L=t,A=_lsame_(L,18130)|0,O=(A|0)!=0;do if(O)M=+HEAPF64[98],At=M;else{_=t,D=_lsame_(_,18497)|0,P=(D|0)!=0;if(P){B=+HEAPF64[107],At=B;break}j=t,F=_lsame_(j,18132)|0,I=(F|0)!=0;if(I){q=+HEAPF64[101],At=q;break}R=t,U=_lsame_(R,18134)|0,z=(U|0)!=0;if(z){W=+HEAPF64[104],At=W;break}X=t,$=_lsame_(X,20460)|0,J=($|0)!=0;if(J){K=+HEAPF64[102],At=K;break}Q=t,G=_lsame_(Q,20464)|0,Y=(G|0)!=0;if(Y){Z=+HEAPF64[103],At=Z;break}et=t,tt=_lsame_(et,18136)|0,nt=(tt|0)!=0;if(nt){it=+HEAPF64[105],At=it;break}st=t,ot=_lsame_(st,20462)|0,ut=(ot|0)!=0;if(ut){at=+HEAPF64[99],At=at;break}ft=t,lt=_lsame_(ft,20458)|0,ct=(lt|0)!=0;if(ct){ht=+HEAPF64[106],At=ht;break}pt=t,vt=_lsame_(pt,18138)|0,mt=(vt|0)!=0,mt&&(gt=+HEAPF64[100],At=gt)}while(0);return yt=At,Lt=yt,HEAP32[655]=0,bt=Lt,STACKTOP=_t,+bt}function _dlamc2_(e,t,n,r,i,s,o,u){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0;var a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0;$i=STACKTOP,STACKTOP=STACKTOP+208|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Di=$i+160|0,Ti=$i+120|0,Ni=$i+112|0,Ei=$i+80|0,Si=$i+72|0,xi=$i+64|0,Ii=$i+56|0,Pi=$i+152|0,Mi=$i+40|0,Bi=$i+148|0,Xi=$i+32|0,Ai=$i+144|0,Ui=$i+16|0,Oi=$i+140|0,zi=$i+8|0,Ri=$i,Hi=$i+136|0,ji=$i+132|0,Fi=$i+128|0,a=e,f=t,pn=n,Yn=r,fr=i,wr=s,Mr=o,Ur=u,Zr=HEAP32[656]|0,li=(Zr|0)!=0;if(!li)return Ln=HEAP32[657]|0,An=a,HEAP32[An>>2]=Ln,On=HEAP32[658]|0,Mn=f,HEAP32[Mn>>2]=On,_n=HEAP32[Bi>>2]|0,Dn=pn,HEAP32[Dn>>2]=_n,Pn=+HEAPF64[108],Hn=Yn,HEAPF64[Hn>>3]=Pn,jn=HEAP32[659]|0,Fn=fr,HEAP32[Fn>>2]=jn,In=+HEAPF64[109],qn=wr,HEAPF64[qn>>3]=In,Rn=HEAP32[667]|0,Un=Mr,HEAP32[Un>>2]=Rn,zn=+HEAPF64[110],Wn=Ur,HEAPF64[Wn>>3]=zn,STACKTOP=$i,0;HEAPF64[Xi>>3]=0,HEAPF64[Ii>>3]=1,Wi=2,_dlamc1_(2628,2632,Bi,Hi)|0,l=HEAP32[657]|0,E=+(l|0),HEAPF64[Si>>3]=E,_=HEAP32[658]|0,z=0-_|0,HEAP32[Di>>2]=z,et=+_pow_di(Si,Di),HEAPF64[Ei>>3]=et,ct=+HEAPF64[Ei>>3],HEAPF64[108]=ct,St=Wi,Dt=St/3,HEAPF64[Si>>3]=Dt,Wt=+HEAPF64[Ii>>3],tn=Wt/2,HEAPF64[Mi>>3]=tn,dn=+HEAPF64[Mi>>3],Nn=-dn,HEAPF64[Ti>>3]=Nn,Bn=+_dlamc3_(Si,Ti),HEAPF64[Ri>>3]=Bn,Xn=+_dlamc3_(Ri,Ri),HEAPF64[zi>>3]=Xn,Vn=+HEAPF64[Mi>>3],$n=-Vn,HEAPF64[Ti>>3]=$n,Jn=+_dlamc3_(zi,Ti),HEAPF64[Si>>3]=Jn,Kn=+_dlamc3_(Si,Ri),HEAPF64[Si>>3]=Kn,Qn=+HEAPF64[Si>>3],Gn=Qn>=0,Zn=+HEAPF64[Si>>3],er=-Zn,tr=Gn?Zn:er,HEAPF64[Si>>3]=tr,nr=+HEAPF64[Si>>3],rr=+HEAPF64[108],ir=nr<rr,ir&&(sr=+HEAPF64[108],HEAPF64[Si>>3]=sr),HEAPF64[108]=1;for(;;){or=+HEAPF64[108],ur=+HEAPF64[Si>>3],ar=or>ur;if(!ar)break;lr=+HEAPF64[Si>>3],cr=+HEAPF64[Xi>>3],hr=lr>cr;if(!hr)break;pr=+HEAPF64[Si>>3],HEAPF64[108]=pr,dr=+HEAPF64[Mi>>3],vr=+HEAPF64[108],mr=dr*vr,HEAPF64[Ti>>3]=mr,gr=Wi,Ci=gr,yr=Ci,ki=yr,br=Ci,Er=Ci,Sr=Er*br,Ci=Sr,xr=+HEAPF64[108],Li=xr,Tr=ki,Nr=Ci,Cr=Ci,kr=Nr*Cr,Lr=Tr*kr,Ar=Li,Or=Li,_r=Ar*Or,Dr=Lr*_r,HEAPF64[Ni>>3]=Dr,Pr=+_dlamc3_(Ti,Ni),HEAPF64[xi>>3]=Pr,Hr=+HEAPF64[xi>>3],Br=-Hr,HEAPF64[Ti>>3]=Br,jr=+_dlamc3_(Mi,Ti),HEAPF64[xi>>3]=jr,Fr=+_dlamc3_(Mi,xi),HEAPF64[Si>>3]=Fr,Ir=+HEAPF64[Si>>3],qr=-Ir,HEAPF64[Ti>>3]=qr,Rr=+_dlamc3_(Mi,Ti),HEAPF64[xi>>3]=Rr,zr=+_dlamc3_(Mi,xi),HEAPF64[Si>>3]=zr}Wr=+HEAPF64[Ei>>3],Xr=+HEAPF64[108],Vr=Wr<Xr,Vr&&($r=+HEAPF64[Ei>>3],HEAPF64[108]=$r),Jr=+HEAPF64[Ii>>3],Kr=HEAP32[657]|0,Qr=+(Kr|0),Gr=Jr/Qr,qi=Gr,Yr=+HEAPF64[Ii>>3],HEAPF64[Ui>>3]=Yr,_i=1;for(;;){ei=_i,ti=(ei|0)<=3;if(!ti)break;ni=+HEAPF64[Ui>>3],ri=qi,ii=ni*ri,HEAPF64[Ti>>3]=ii,si=+_dlamc3_(Ti,Xi),HEAPF64[Ui>>3]=si,oi=_i,ui=oi+1|0,_i=ui}ai=+_dlamc3_(Ii,Ui),HEAPF64[Ei>>3]=ai,_dlamc4_(Fi,Ii,2628)|0,fi=+HEAPF64[Ii>>3],ci=-fi,HEAPF64[Ti>>3]=ci,_dlamc4_(ji,Ti,2628)|0,_dlamc4_(Oi,Ei,2628)|0,hi=+HEAPF64[Ei>>3],pi=-hi,HEAPF64[Ti>>3]=pi,_dlamc4_(Ai,Ti,2628)|0,HEAP32[Pi>>2]=0,di=HEAP32[Fi>>2]|0,vi=HEAP32[ji>>2]|0,mi=(di|0)==(vi|0);do if(mi){gi=HEAP32[Oi>>2]|0,yi=HEAP32[Ai>>2]|0,bi=(gi|0)==(yi|0);if(bi){wi=HEAP32[Fi>>2]|0,c=HEAP32[Oi>>2]|0,h=(wi|0)==(c|0);if(h){p=HEAP32[Fi>>2]|0,HEAP32[659]=p;break}d=HEAP32[Oi>>2]|0,v=HEAP32[Fi>>2]|0,m=d-v|0,g=(m|0)==3,y=HEAP32[Fi>>2]|0;if(g){b=y-1|0,w=HEAP32[658]|0,S=b+w|0,HEAP32[659]=S,HEAP32[Pi>>2]=1;break}x=HEAP32[Oi>>2]|0,T=(y|0)<=(x|0),N=HEAP32[Fi>>2]|0,C=HEAP32[Oi>>2]|0,k=T?N:C,HEAP32[659]=k,HEAP32[660]=1;break}Vi=20}else Vi=20;while(0);do if((Vi|0)==20){L=HEAP32[Fi>>2]|0,A=HEAP32[Oi>>2]|0,O=(L|0)==(A|0);if(O){M=HEAP32[ji>>2]|0,D=HEAP32[Ai>>2]|0,P=(M|0)==(D|0);if(P){H=HEAP32[Fi>>2]|0,B=HEAP32[ji>>2]|0,j=H-B|0,HEAP32[Di>>2]=j,F=HEAP32[Di>>2]|0,I=(F|0)>=0,q=HEAP32[Di>>2]|0,R=0-q|0,U=I?q:R,W=(U|0)==1,X=HEAP32[Fi>>2]|0,V=HEAP32[ji>>2]|0;if(W){$=(X|0)>=(V|0),J=HEAP32[Fi>>2]|0,K=HEAP32[ji>>2]|0,Q=$?J:K,HEAP32[659]=Q;break}G=(X|0)<=(V|0),Y=HEAP32[Fi>>2]|0,Z=HEAP32[ji>>2]|0,tt=G?Y:Z,HEAP32[659]=tt,HEAP32[660]=1;break}}nt=HEAP32[Fi>>2]|0,rt=HEAP32[ji>>2]|0,it=nt-rt|0,HEAP32[Di>>2]=it,st=HEAP32[Di>>2]|0,ot=(st|0)>=0,ut=HEAP32[Di>>2]|0,at=0-ut|0,ft=ot?ut:at,lt=(ft|0)==1;if(lt){ht=HEAP32[Oi>>2]|0,pt=HEAP32[Ai>>2]|0,dt=(ht|0)==(pt|0);if(dt){vt=HEAP32[Oi>>2]|0,mt=HEAP32[Fi>>2]|0,gt=HEAP32[ji>>2]|0,yt=(mt|0)<=(gt|0),bt=HEAP32[Fi>>2]|0,wt=HEAP32[ji>>2]|0,Et=yt?bt:wt,xt=vt-Et|0,Tt=(xt|0)==3,Nt=HEAP32[Fi>>2]|0,Ct=HEAP32[ji>>2]|0;if(Tt){kt=(Nt|0)>=(Ct|0),Lt=HEAP32[Fi>>2]|0,At=HEAP32[ji>>2]|0,Ot=kt?Lt:At,Mt=Ot-1|0,_t=HEAP32[658]|0,Pt=Mt+_t|0,HEAP32[659]=Pt;break}Ht=(Nt|0)<=(Ct|0),Bt=HEAP32[Fi>>2]|0,jt=HEAP32[ji>>2]|0,Ft=Ht?Bt:jt,HEAP32[659]=Ft,HEAP32[660]=1;break}}It=HEAP32[Fi>>2]|0,qt=HEAP32[ji>>2]|0,Rt=(It|0)<=(qt|0),Ut=HEAP32[Fi>>2]|0,zt=HEAP32[ji>>2]|0,Xt=Rt?Ut:zt,HEAP32[Di>>2]=Xt,Vt=HEAP32[Di>>2]|0,$t=HEAP32[Oi>>2]|0,Jt=(Vt|0)<=($t|0),Kt=HEAP32[Di>>2]|0,Qt=HEAP32[Oi>>2]|0,Gt=Jt?Kt:Qt,HEAP32[Di>>2]=Gt,Yt=HEAP32[Di>>2]|0,Zt=HEAP32[Ai>>2]|0,en=(Yt|0)<=(Zt|0),nn=HEAP32[Di>>2]|0,rn=HEAP32[Ai>>2]|0,sn=en?nn:rn,HEAP32[659]=sn,HEAP32[660]=1}while(0);HEAP32[656]=0,on=HEAP32[660]|0,un=(on|0)!=0,un&&(HEAP32[656]=1,_s_wsfe(2644)|0,_do_fio(2664,2636,4)|0,_e_wsfe()|0),an=HEAP32[Pi>>2]|0,fn=(an|0)!=0,ln=HEAP32[Hi>>2]|0,cn=(ln|0)!=0,hn=fn?1:cn,vn=hn&1,HEAP32[Pi>>2]=vn,HEAPF64[109]=1,mn=HEAP32[659]|0,gn=1-mn|0,HEAP32[Di>>2]=gn,_i=1;for(;;){yn=_i,bn=HEAP32[Di>>2]|0,wn=(yn|0)<=(bn|0);if(!wn)break;En=+HEAPF64[109],Sn=qi,xn=En*Sn,HEAPF64[Ti>>3]=xn,Tn=+_dlamc3_(Ti,Xi),HEAPF64[109]=Tn,Cn=_i,kn=Cn+1|0,_i=kn}return _dlamc5_(2628,2632,2636,Pi,2668,880)|0,Ln=HEAP32[657]|0,An=a,HEAP32[An>>2]=Ln,On=HEAP32[658]|0,Mn=f,HEAP32[Mn>>2]=On,_n=HEAP32[Bi>>2]|0,Dn=pn,HEAP32[Dn>>2]=_n,Pn=+HEAPF64[108],Hn=Yn,HEAPF64[Hn>>3]=Pn,jn=HEAP32[659]|0,Fn=fr,HEAP32[Fn>>2]=jn,In=+HEAPF64[109],qn=wr,HEAPF64[qn>>3]=In,Rn=HEAP32[667]|0,Un=Mr,HEAP32[Un>>2]=Rn,zn=+HEAPF64[110],Wn=Ur,HEAPF64[Wn>>3]=zn,STACKTOP=$i,0}function _dlamc1_(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0;en=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Xt=en+80|0,Vt=en+72|0,Ut=en+64|0,zt=en+56|0,Wt=en+48|0,$t=en+40|0,Jt=en+16|0,Qt=en,i=e,s=t,m=n,k=r,F=HEAP32[668]|0,K=(F|0)!=0;if(!K)return Dt=HEAP32[669]|0,Pt=i,HEAP32[Pt>>2]=Dt,Ht=HEAP32[672]|0,Bt=s,HEAP32[Bt>>2]=Ht,jt=HEAP32[670]|0,Ft=m,HEAP32[Ft>>2]=jt,qt=HEAP32[671]|0,Rt=k,HEAP32[Rt>>2]=qt,HEAP32[668]=0,STACKTOP=en,0;HEAPF64[Jt>>3]=1,HEAPF64[Ut>>3]=1,HEAPF64[Wt>>3]=1;for(;;){ot=+HEAPF64[Wt>>3],gt=+HEAPF64[Jt>>3],Lt=ot==gt;if(!Lt)break;It=+HEAPF64[Ut>>3],o=It*2,HEAPF64[Ut>>3]=o,u=+_dlamc3_(Ut,Jt),HEAPF64[Wt>>3]=u,a=+HEAPF64[Ut>>3],f=-a,HEAPF64[Xt>>3]=f,l=+_dlamc3_(Wt,Xt),HEAPF64[Wt>>3]=l}HEAPF64[zt>>3]=1,c=+_dlamc3_(Ut,zt),HEAPF64[Wt>>3]=c;for(;;){h=+HEAPF64[Wt>>3],p=+HEAPF64[Ut>>3],d=h==p;if(!d)break;v=+HEAPF64[zt>>3],g=v*2,HEAPF64[zt>>3]=g,y=+_dlamc3_(Ut,zt),HEAPF64[Wt>>3]=y}b=+HEAPF64[Jt>>3],w=b/4,Kt=w,E=+HEAPF64[Wt>>3],HEAPF64[Qt>>3]=E,S=+HEAPF64[Ut>>3],x=-S,HEAPF64[Xt>>3]=x,T=+_dlamc3_(Wt,Xt),HEAPF64[Wt>>3]=T,N=+HEAPF64[Wt>>3],C=Kt,L=N+C,A=~~L,HEAP32[669]=A,O=HEAP32[669]|0,M=+(O|0),HEAPF64[zt>>3]=M,_=+HEAPF64[zt>>3],D=_/2,HEAPF64[Xt>>3]=D,P=+HEAPF64[zt>>3],H=-P,B=H/100,HEAPF64[Vt>>3]=B,j=+_dlamc3_(Xt,Vt),HEAPF64[$t>>3]=j,I=+_dlamc3_($t,Ut),HEAPF64[Wt>>3]=I,q=+HEAPF64[Wt>>3],R=+HEAPF64[Ut>>3],U=q==R,U?HEAP32[670]=1:HEAP32[670]=0,z=+HEAPF64[zt>>3],W=z/2,HEAPF64[Xt>>3]=W,X=+HEAPF64[zt>>3],V=X/100,HEAPF64[Vt>>3]=V,$=+_dlamc3_(Xt,Vt),HEAPF64[$t>>3]=$,J=+_dlamc3_($t,Ut),HEAPF64[Wt>>3]=J,Q=HEAP32[670]|0,G=(Q|0)!=0,G&&(Y=+HEAPF64[Wt>>3],Z=+HEAPF64[Ut>>3],et=Y==Z,et&&(HEAP32[670]=0)),tt=+HEAPF64[zt>>3],nt=tt/2,HEAPF64[Xt>>3]=nt,rt=+_dlamc3_(Xt,Ut),Gt=rt,it=+HEAPF64[zt>>3],st=it/2,HEAPF64[Xt>>3]=st,ut=+_dlamc3_(Xt,Qt),Yt=ut,at=Gt,ft=+HEAPF64[Ut>>3],lt=at==ft,lt?(ct=Yt,ht=+HEAPF64[Qt>>3],pt=ct>ht,pt?(dt=HEAP32[670]|0,vt=(dt|0)!=0,yt=vt):yt=0):yt=0,mt=yt&1,HEAP32[671]=mt,HEAP32[672]=0,HEAPF64[Ut>>3]=1,HEAPF64[Wt>>3]=1;for(;;){bt=+HEAPF64[Wt>>3],wt=+HEAPF64[Jt>>3],Et=bt==wt;if(!Et)break;St=HEAP32[672]|0,xt=St+1|0,HEAP32[672]=xt,Tt=HEAP32[669]|0,Nt=+(Tt|0),Ct=+HEAPF64[Ut>>3],kt=Ct*Nt,HEAPF64[Ut>>3]=kt,At=+_dlamc3_(Ut,Jt),HEAPF64[Wt>>3]=At,Ot=+HEAPF64[Ut>>3],Mt=-Ot,HEAPF64[Xt>>3]=Mt,_t=+_dlamc3_(Wt,Xt),HEAPF64[Wt>>3]=_t}return Dt=HEAP32[669]|0,Pt=i,HEAP32[Pt>>2]=Dt,Ht=HEAP32[672]|0,Bt=s,HEAP32[Bt>>2]=Ht,jt=HEAP32[670]|0,Ft=m,HEAP32[Ft>>2]=jt,qt=HEAP32[671]|0,Rt=k,HEAP32[Rt>>2]=qt,HEAP32[668]=0,STACKTOP=en,0}function _dlamc3_(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0;return h=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=t,i=n,s=+HEAPF64[i>>3],o=r,u=+HEAPF64[o>>3],a=s+u,l=a,f=l,STACKTOP=h,+f}function _dlamc4_(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0;qt=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Dt=qt+80|0,Ft=qt+8|0,r=e,i=t,v=n,C=i,j=+HEAPF64[C>>3],Ct=j,Bt=1,J=Bt,st=v,mt=HEAP32[st>>2]|0,Tt=+(mt|0),Nt=J/Tt,jt=Nt,HEAPF64[Ft>>3]=0,s=r,HEAP32[s>>2]=1,o=Ct,u=jt,a=o*u,HEAPF64[Dt>>3]=a,f=+_dlamc3_(Dt,Ft),kt=f,l=Ct,At=l,c=Ct,Ot=c,h=Ct,Mt=h,p=Ct,_t=p;e:for(;;){d=At,m=Ct,g=d==m;if(!g){It=12;break}y=Ot,b=Ct,w=y==b;if(!w){It=12;break}E=Mt,S=Ct,x=E==S;if(!x){It=12;break}T=_t,N=Ct,k=T==N;if(!k){It=12;break}L=r,A=HEAP32[L>>2]|0,O=A+ -1|0,HEAP32[L>>2]=O,M=kt,Ct=M,_=Ct,D=v,P=HEAP32[D>>2]|0,H=+(P|0),B=_/H,HEAPF64[Dt>>3]=B,F=+_dlamc3_(Dt,Ft),kt=F,I=kt,q=v,R=HEAP32[q>>2]|0,U=+(R|0),z=I*U,HEAPF64[Dt>>3]=z,W=+_dlamc3_(Dt,Ft),At=W,X=+HEAPF64[Ft>>3],Mt=X,V=v,$=HEAP32[V>>2]|0,Ht=$,Pt=1;for(;;){K=Pt,Q=Ht,G=(K|0)<=(Q|0);if(!G)break;Y=kt,Z=Mt,et=Z+Y,Mt=et,tt=Pt,nt=tt+1|0,Pt=nt}rt=Ct,it=jt,ot=rt*it,HEAPF64[Dt>>3]=ot,ut=+_dlamc3_(Dt,Ft),Lt=ut,at=Lt,ft=jt,lt=at/ft,HEAPF64[Dt>>3]=lt,ct=+_dlamc3_(Dt,Ft),Ot=ct,ht=+HEAPF64[Ft>>3],_t=ht,pt=v,dt=HEAP32[pt>>2]|0,Ht=dt,Pt=1;for(;;){vt=Pt,gt=Ht,yt=(vt|0)<=(gt|0);if(!yt)continue e;bt=Lt,wt=_t,Et=wt+bt,_t=Et,St=Pt,xt=St+1|0,Pt=xt}}return(It|0)==12?(STACKTOP=qt,0):0}function _dlamc5_(e,t,n,r,i,s){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0;var o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0;gn=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),nn=gn+32|0,dn=gn+24|0,vn=gn+16|0,o=e,u=t,S=n,D=r,W=i,tt=s,an=1,rn=1;for(;;){ht=an,xt=ht<<1,hn=xt,Pt=hn,Xt=S,a=HEAP32[Xt>>2]|0,p=0-a|0,d=(Pt|0)<=(p|0);if(!d)break;v=hn,an=v,m=rn,g=m+1|0,rn=g}y=an,b=S,w=HEAP32[b>>2]|0,E=0-w|0,x=(y|0)==(E|0),x?(T=an,pn=T):(N=hn,pn=N,C=rn,k=C+1|0,rn=k),L=pn,A=S,O=HEAP32[A>>2]|0,M=L+O|0,_=an,P=0-_|0,H=S,B=HEAP32[H>>2]|0,j=P-B|0,F=(M|0)>(j|0),F?(I=an,q=I<<1,sn=q):(R=pn,U=R<<1,sn=U),z=sn,X=S,V=HEAP32[X>>2]|0,$=z+V|0,J=$-1|0,K=W,HEAP32[K>>2]=J,Q=rn,G=Q+1|0,Y=u,Z=HEAP32[Y>>2]|0,et=G+Z|0,fn=et,nt=fn,rt=(nt|0)%2&-1,it=(rt|0)==1,it&&(st=o,ot=HEAP32[st>>2]|0,ut=(ot|0)==2,ut&&(at=W,ft=HEAP32[at>>2]|0,lt=ft+ -1|0,HEAP32[at>>2]=lt)),ct=D,pt=HEAP32[ct>>2]|0,dt=(pt|0)!=0,dt&&(vt=W,mt=HEAP32[vt>>2]|0,gt=mt+ -1|0,HEAP32[vt>>2]=gt),yt=o,bt=HEAP32[yt>>2]|0,wt=+(bt|0),Et=1/wt,cn=Et,St=o,Tt=HEAP32[St>>2]|0,Nt=+(Tt|0),Ct=Nt-1,HEAPF64[vn>>3]=Ct,HEAPF64[dn>>3]=0,kt=u,Lt=HEAP32[kt>>2]|0,un=Lt,on=1;for(;;){At=on,Ot=un,Mt=(At|0)<=(Ot|0);if(!Mt)break;_t=cn,Dt=+HEAPF64[vn>>3],Ht=Dt*_t,HEAPF64[vn>>3]=Ht,Bt=+HEAPF64[dn>>3],jt=Bt<1,jt&&(Ft=+HEAPF64[dn>>3],ln=Ft),It=+_dlamc3_(dn,vn),HEAPF64[dn>>3]=It,qt=on,Rt=qt+1|0,on=Rt}Ut=+HEAPF64[dn>>3],zt=Ut>=1,zt&&(Wt=ln,HEAPF64[dn>>3]=Wt),Vt=W,$t=HEAP32[Vt>>2]|0,un=$t,on=1;for(;;){Jt=on,Kt=un,Qt=(Jt|0)<=(Kt|0),Gt=+HEAPF64[dn>>3];if(!Qt)break;Yt=o,Zt=HEAP32[Yt>>2]|0,en=+(Zt|0),tn=Gt*en,HEAPF64[nn>>3]=tn,f=+_dlamc3_(nn,888),HEAPF64[dn>>3]=f,l=on,c=l+1|0,on=c}return h=tt,HEAPF64[h>>3]=Gt,STACKTOP=gn,0}function _ilaenv_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0;Sl=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Xf=Sl+90|0,Vf=Sl+88|0,$f=Sl+84|0,Jf=Sl+82|0,wl=Sl+76|0,a=e,hn=t,vi=n,bo=r,Sa=i,ef=s,hf=o,xf=hn,Pf=_strlen(xf)|0,Zf=Pf,f=vi,w=_strlen(f)|0,rl=w,M=a,U=HEAP32[M>>2]|0;do switch(U|0){case 3:case 2:case 1:yl=1,lt=hn,Et=Zf,_s_copy(wl,lt,1,Et),_t=HEAP8[wl>>0]|0,zt=_t&255,Gf=zt,Yf=90,en=Yf,pn=(en|0)==90,Tn=Yf,Hn=(Tn|0)==122,il=pn|Hn;e:do if(il){Vn=Gf,rr=(Vn|0)>=97,dr=Gf,Nr=(dr|0)<=122,dl=rr&Nr;if(dl){Br=Gf,$r=Br-32|0,ii=$r&255,HEAP8[wl>>0]=ii,Qf=2;for(;;){mi=Qf,ki=(mi|0)<=6;if(!ki)break e;Fi=Qf,Ki=Fi-1|0,os=wl+Ki|0,gs=HEAP8[os>>0]|0,Ls=gs&255,Gf=Ls,Is=Gf,Qs=(Is|0)>=97,uo=Gf,wo=(uo|0)<=122,vl=Qs&wo,vl&&(Mo=Gf,Uo=Mo-32|0,Zo=Uo&255,lu=Qf,Eu=lu-1|0,_u=wl+Eu|0,HEAP8[_u>>0]=Zo),zu=Qf,ea=zu+1|0,Qf=ea}}}else{ca=Yf,xa=(ca|0)==233,Pa=Yf,Xa=(Pa|0)==169,ml=xa|Xa;if(!ml){qf=Yf,Rf=(qf|0)==218,Uf=Yf,zf=(Uf|0)==250,ll=Rf|zf;if(!ll)break;Wf=Gf,l=(Wf|0)>=225,c=Gf,h=(c|0)<=250,cl=l&h;if(!cl)break;p=Gf,d=p-32|0,v=d&255,HEAP8[wl>>0]=v,Qf=2;for(;;){m=Qf,g=(m|0)<=6;if(!g)break e;y=Qf,b=y-1|0,E=wl+b|0,S=HEAP8[E>>0]|0,x=S&255,Gf=x,T=Gf,N=(T|0)>=225,C=Gf,k=(C|0)<=250,hl=N&k,hl&&(L=Gf,A=L-32|0,O=A&255,_=Qf,D=_-1|0,P=wl+D|0,HEAP8[P>>0]=O),H=Qf,B=H+1|0,Qf=B}}$a=Gf,Ja=($a|0)>=129,Ka=Gf,Qa=(Ka|0)<=137,gl=Ja&Qa;if(!gl){Ga=Gf,Ya=(Ga|0)>=145,Za=Gf,tf=(Za|0)<=153,sl=Ya&tf;if(!sl){nf=Gf,rf=(nf|0)>=162,sf=Gf,of=(sf|0)<=169,ol=rf&of;if(!ol)break}}uf=Gf,af=uf+64|0,ff=af&255,HEAP8[wl>>0]=ff,Qf=2;for(;;){lf=Qf,cf=(lf|0)<=6;if(!cf)break e;pf=Qf,df=pf-1|0,vf=wl+df|0,mf=HEAP8[vf>>0]|0,gf=mf&255,Gf=gf,yf=Gf,bf=(yf|0)>=129,wf=Gf,Ef=(wf|0)<=137,ul=bf&Ef,ul?El=19:(Sf=Gf,Tf=(Sf|0)>=145,Nf=Gf,Cf=(Nf|0)<=153,al=Tf&Cf,al?El=19:(kf=Gf,Lf=(kf|0)>=162,Af=Gf,Of=(Af|0)<=169,fl=Lf&Of,fl&&(El=19))),(El|0)==19&&(El=0,Mf=Gf,_f=Mf+64|0,Df=_f&255,Hf=Qf,Bf=Hf-1|0,jf=wl+Bf|0,HEAP8[jf>>0]=Df),Ff=Qf,If=Ff+1|0,Qf=If}}while(0);j=HEAP8[wl>>0]|0,HEAP8[Xf>>0]=j,F=HEAP8[Xf>>0]|0,I=F&255,q=(I|0)==83,q?V=1:(R=HEAP8[Xf>>0]|0,z=R&255,W=(z|0)==68,V=W),X=V&1,bl=X,$=HEAP8[Xf>>0]|0,J=$&255,K=(J|0)==67,K?tt=1:(Q=HEAP8[Xf>>0]|0,G=Q&255,Y=(G|0)==90,tt=Y),et=tt&1,Kf=et,nt=Kf,rt=(nt|0)!=0,it=bl,st=(it|0)!=0,pl=rt|st;if(!pl)return ot=yl,u=ot,Va=u,STACKTOP=Sl,Va|0;ut=wl+1|0,_s_copy(Vf,ut,1,2),at=wl+3|0,_s_copy($f,at,1,3),ft=$f+1|0,_s_copy(Jf,ft,1,2),ct=a,ht=HEAP32[ct>>2]|0;switch(ht|0){case 3:nl=0,Bo=_s_cmp(Vf,18391,1,2)|0,jo=(Bo|0)==0;t:do if(jo){Fo=_s_cmp($f,18398,1,3)|0,Io=(Fo|0)==0;if(!Io){qo=_s_cmp($f,18402,1,3)|0,Ro=(qo|0)==0;if(!Ro){zo=_s_cmp($f,18406,1,3)|0,Wo=(zo|0)==0;if(!Wo){Xo=_s_cmp($f,18410,1,3)|0,Vo=(Xo|0)==0;if(!Vo){$o=_s_cmp($f,18414,1,3)|0,Jo=($o|0)==0;if(Jo){nl=128;break}Ko=_s_cmp($f,18418,1,3)|0,Qo=(Ko|0)==0;if(!Qo)break;nl=128;break}}}}nl=128}else{Go=_s_cmp(Vf,18429,1,2)|0,Yo=(Go|0)==0;if(Yo){eu=bl,tu=(eu|0)!=0;if(!tu)break;nu=_s_cmp($f,18432,1,3)|0,ru=(nu|0)==0;if(!ru)break;nl=32;break}iu=Kf,su=(iu|0)!=0;if(su){ou=_s_cmp(Vf,18440,1,2)|0,uu=(ou|0)==0;if(uu){au=_s_cmp($f,18432,1,3)|0,fu=(au|0)==0;if(!fu)break;nl=32;break}}cu=bl,hu=(cu|0)!=0;if(hu){pu=_s_cmp(Vf,18443,1,2)|0,du=(pu|0)==0;if(du){vu=HEAP8[$f>>0]|0,mu=vu&255,gu=(mu|0)==71;if(!gu)break;yu=_s_cmp(Jf,18446,1,2)|0,bu=(yu|0)==0;do if(!bu){wu=_s_cmp(Jf,18449,1,2)|0,Su=(wu|0)==0;if(!Su){xu=_s_cmp(Jf,18452,1,2)|0,Tu=(xu|0)==0;if(!Tu){Nu=_s_cmp(Jf,18455,1,2)|0,Cu=(Nu|0)==0;if(!Cu){ku=_s_cmp(Jf,18458,1,2)|0,Lu=(ku|0)==0;if(!Lu){Au=_s_cmp(Jf,18461,1,2)|0,Ou=(Au|0)==0;if(Ou)break;Mu=_s_cmp(Jf,18464,1,2)|0,Du=(Mu|0)==0;if(!Du)break t}}}}}while(0);nl=128;break}}Pu=Kf,Hu=(Pu|0)!=0;if(Hu){Bu=_s_cmp(Vf,18467,1,2)|0,ju=(Bu|0)==0;if(ju){Fu=HEAP8[$f>>0]|0,Iu=Fu&255,qu=(Iu|0)==71;if(qu){Ru=_s_cmp(Jf,18446,1,2)|0,Uu=(Ru|0)==0;do if(!Uu){Wu=_s_cmp(Jf,18449,1,2)|0,Xu=(Wu|0)==0;if(!Xu){Vu=_s_cmp(Jf,18452,1,2)|0,$u=(Vu|0)==0;if(!$u){Ju=_s_cmp(Jf,18455,1,2)|0,Ku=(Ju|0)==0;if(!Ku){Qu=_s_cmp(Jf,18458,1,2)|0,Gu=(Qu|0)==0;if(!Gu){Yu=_s_cmp(Jf,18461,1,2)|0,Zu=(Yu|0)==0;if(Zu)break;ta=_s_cmp(Jf,18464,1,2)|0,na=(ta|0)==0;if(!na)break t}}}}}while(0);nl=128}}}}while(0);return ra=nl,yl=ra,ia=yl,u=ia,Va=u,STACKTOP=Sl,Va|0;case 2:tl=2,xi=_s_cmp(Vf,18391,1,2)|0,Ti=(xi|0)==0;n:do if(Ti){Ni=_s_cmp($f,18398,1,3)|0,Ci=(Ni|0)==0;if(!Ci){Li=_s_cmp($f,18402,1,3)|0,Ai=(Li|0)==0;if(!Ai){Oi=_s_cmp($f,18406,1,3)|0,Mi=(Oi|0)==0;if(!Mi){_i=_s_cmp($f,18410,1,3)|0,Di=(_i|0)==0;if(!Di){Pi=_s_cmp($f,18414,1,3)|0,Hi=(Pi|0)==0;if(Hi){tl=2;break}Bi=_s_cmp($f,18418,1,3)|0,ji=(Bi|0)==0;if(ji){tl=2;break}Ii=_s_cmp($f,18422,1,3)|0,qi=(Ii|0)==0;if(!qi)break;tl=2;break}}}}tl=2}else{Ri=_s_cmp(Vf,18429,1,2)|0,Ui=(Ri|0)==0;if(Ui){zi=_s_cmp($f,18394,1,3)|0,Wi=(zi|0)==0,Xi=bl,Vi=(Xi|0)!=0;if(Wi){tl=8;break}if(!Vi)break;$i=_s_cmp($f,18432,1,3)|0,Ji=($i|0)==0;if(!Ji)break;tl=2;break}Qi=Kf,Gi=(Qi|0)!=0;if(Gi){Yi=_s_cmp(Vf,18440,1,2)|0,Zi=(Yi|0)==0;if(Zi){es=_s_cmp($f,18432,1,3)|0,ts=(es|0)==0;if(!ts)break;tl=2;break}}ns=bl,rs=(ns|0)!=0;if(rs){is=_s_cmp(Vf,18443,1,2)|0,ss=(is|0)==0;if(ss){us=HEAP8[$f>>0]|0,as=us&255,fs=(as|0)==71;if(fs){ls=_s_cmp(Jf,18446,1,2)|0,cs=(ls|0)==0;do if(!cs){hs=_s_cmp(Jf,18449,1,2)|0,ps=(hs|0)==0;if(!ps){ds=_s_cmp(Jf,18452,1,2)|0,vs=(ds|0)==0;if(!vs){ms=_s_cmp(Jf,18455,1,2)|0,ys=(ms|0)==0;if(!ys){bs=_s_cmp(Jf,18458,1,2)|0,ws=(bs|0)==0;if(ws)break;Es=_s_cmp(Jf,18461,1,2)|0,Ss=(Es|0)==0;if(Ss)break;xs=_s_cmp(Jf,18464,1,2)|0,Ts=(xs|0)==0;if(!Ts)break n}}}}while(0);tl=2;break}Ns=HEAP8[$f>>0]|0,Cs=Ns&255,ks=(Cs|0)==77;if(!ks)break;As=_s_cmp(Jf,18446,1,2)|0,Os=(As|0)==0;do if(!Os){Ms=_s_cmp(Jf,18449,1,2)|0,_s=(Ms|0)==0;if(!_s){Ds=_s_cmp(Jf,18452,1,2)|0,Ps=(Ds|0)==0;if(!Ps){Hs=_s_cmp(Jf,18455,1,2)|0,Bs=(Hs|0)==0;if(!Bs){js=_s_cmp(Jf,18458,1,2)|0,Fs=(js|0)==0;if(Fs)break;qs=_s_cmp(Jf,18461,1,2)|0,Rs=(qs|0)==0;if(Rs)break;Us=_s_cmp(Jf,18464,1,2)|0,zs=(Us|0)==0;if(!zs)break n}}}}while(0);tl=2;break}}Ws=Kf,Xs=(Ws|0)!=0;if(Xs){Vs=_s_cmp(Vf,18467,1,2)|0,$s=(Vs|0)==0;if($s){Js=HEAP8[$f>>0]|0,Ks=Js&255,Gs=(Ks|0)==71;if(Gs){Ys=_s_cmp(Jf,18446,1,2)|0,Zs=(Ys|0)==0;do if(!Zs){eo=_s_cmp(Jf,18449,1,2)|0,to=(eo|0)==0;if(!to){no=_s_cmp(Jf,18452,1,2)|0,ro=(no|0)==0;if(!ro){io=_s_cmp(Jf,18455,1,2)|0,so=(io|0)==0;if(!so){oo=_s_cmp(Jf,18458,1,2)|0,ao=(oo|0)==0;if(ao)break;fo=_s_cmp(Jf,18461,1,2)|0,lo=(fo|0)==0;if(lo)break;co=_s_cmp(Jf,18464,1,2)|0,ho=(co|0)==0;if(!ho)break n}}}}while(0);tl=2;break}po=HEAP8[$f>>0]|0,vo=po&255,mo=(vo|0)==77;if(mo){go=_s_cmp(Jf,18446,1,2)|0,yo=(go|0)==0;do if(!yo){Eo=_s_cmp(Jf,18449,1,2)|0,So=(Eo|0)==0;if(!So){xo=_s_cmp(Jf,18452,1,2)|0,To=(xo|0)==0;if(!To){No=_s_cmp(Jf,18455,1,2)|0,Co=(No|0)==0;if(!Co){ko=_s_cmp(Jf,18458,1,2)|0,Lo=(ko|0)==0;if(Lo)break;Ao=_s_cmp(Jf,18461,1,2)|0,Oo=(Ao|0)==0;if(Oo)break;_o=_s_cmp(Jf,18464,1,2)|0,Do=(_o|0)==0;if(!Do)break n}}}}while(0);tl=2}}}}while(0);return Po=tl,yl=Po,Ho=yl,u=Ho,Va=u,STACKTOP=Sl,Va|0;default:el=1,pt=_s_cmp(Vf,18391,1,2)|0,dt=(pt|0)==0;r:do if(dt){vt=_s_cmp($f,18394,1,3)|0,mt=(vt|0)==0;if(mt){el=64;break}gt=_s_cmp($f,18398,1,3)|0,yt=(gt|0)==0;if(!yt){bt=_s_cmp($f,18402,1,3)|0,wt=(bt|0)==0;if(!wt){St=_s_cmp($f,18406,1,3)|0,xt=(St|0)==0;if(!xt){Tt=_s_cmp($f,18410,1,3)|0,Nt=(Tt|0)==0;if(!Nt){Ct=_s_cmp($f,18414,1,3)|0,kt=(Ct|0)==0;if(kt){el=32;break}Lt=_s_cmp($f,18418,1,3)|0,At=(Lt|0)==0;if(At){el=32;break}Ot=_s_cmp($f,18422,1,3)|0,Mt=(Ot|0)==0;if(!Mt)break;el=64;break}}}}el=32}else{Dt=_s_cmp(Vf,18426,1,2)|0,Pt=(Dt|0)==0;if(Pt){Ht=_s_cmp($f,18394,1,3)|0,Bt=(Ht|0)==0;if(!Bt)break;el=64;break}jt=_s_cmp(Vf,18429,1,2)|0,Ft=(jt|0)==0;if(Ft){It=_s_cmp($f,18394,1,3)|0,qt=(It|0)==0,Rt=bl,Ut=(Rt|0)!=0;if(qt){el=64;break}if(Ut){Wt=_s_cmp($f,18432
,1,3)|0,Xt=(Wt|0)==0;if(Xt){el=32;break}}Vt=bl,$t=(Vt|0)!=0;if(!$t)break;Jt=_s_cmp($f,18436,1,3)|0,Kt=(Jt|0)==0;if(!Kt)break;el=64;break}Qt=Kf,Gt=(Qt|0)!=0;if(Gt){Yt=_s_cmp(Vf,18440,1,2)|0,Zt=(Yt|0)==0;if(Zt){tn=_s_cmp($f,18394,1,3)|0,nn=(tn|0)==0;if(nn){el=64;break}rn=_s_cmp($f,18432,1,3)|0,sn=(rn|0)==0;if(sn){el=32;break}on=_s_cmp($f,18436,1,3)|0,un=(on|0)==0;if(!un)break;el=64;break}}an=bl,fn=(an|0)!=0;if(fn){ln=_s_cmp(Vf,18443,1,2)|0,cn=(ln|0)==0;if(cn){dn=HEAP8[$f>>0]|0,vn=dn&255,mn=(vn|0)==71;if(mn){gn=_s_cmp(Jf,18446,1,2)|0,yn=(gn|0)==0;do if(!yn){bn=_s_cmp(Jf,18449,1,2)|0,wn=(bn|0)==0;if(!wn){En=_s_cmp(Jf,18452,1,2)|0,Sn=(En|0)==0;if(!Sn){xn=_s_cmp(Jf,18455,1,2)|0,Nn=(xn|0)==0;if(Nn)break;Cn=_s_cmp(Jf,18458,1,2)|0,kn=(Cn|0)==0;if(kn)break;Ln=_s_cmp(Jf,18461,1,2)|0,An=(Ln|0)==0;if(An)break;On=_s_cmp(Jf,18464,1,2)|0,Mn=(On|0)==0;if(!Mn)break r}}}while(0);el=32;break}_n=HEAP8[$f>>0]|0,Dn=_n&255,Pn=(Dn|0)==77;if(!Pn)break;Bn=_s_cmp(Jf,18446,1,2)|0,jn=(Bn|0)==0;do if(!jn){Fn=_s_cmp(Jf,18449,1,2)|0,In=(Fn|0)==0;if(!In){qn=_s_cmp(Jf,18452,1,2)|0,Rn=(qn|0)==0;if(!Rn){Un=_s_cmp(Jf,18455,1,2)|0,zn=(Un|0)==0;if(zn)break;Wn=_s_cmp(Jf,18458,1,2)|0,Xn=(Wn|0)==0;if(Xn)break;$n=_s_cmp(Jf,18461,1,2)|0,Jn=($n|0)==0;if(Jn)break;Kn=_s_cmp(Jf,18464,1,2)|0,Qn=(Kn|0)==0;if(!Qn)break r}}}while(0);el=32;break}}Gn=Kf,Yn=(Gn|0)!=0;if(Yn){Zn=_s_cmp(Vf,18467,1,2)|0,er=(Zn|0)==0;if(er){tr=HEAP8[$f>>0]|0,nr=tr&255,ir=(nr|0)==71;if(ir){sr=_s_cmp(Jf,18446,1,2)|0,or=(sr|0)==0;do if(!or){ur=_s_cmp(Jf,18449,1,2)|0,ar=(ur|0)==0;if(!ar){fr=_s_cmp(Jf,18452,1,2)|0,lr=(fr|0)==0;if(lr)break;cr=_s_cmp(Jf,18455,1,2)|0,hr=(cr|0)==0;if(hr)break;pr=_s_cmp(Jf,18458,1,2)|0,vr=(pr|0)==0;if(vr)break;mr=_s_cmp(Jf,18461,1,2)|0,gr=(mr|0)==0;if(gr)break;yr=_s_cmp(Jf,18464,1,2)|0,br=(yr|0)==0;if(!br)break r}}while(0);el=32;break}wr=HEAP8[$f>>0]|0,Er=wr&255,Sr=(Er|0)==77;if(!Sr)break;xr=_s_cmp(Jf,18446,1,2)|0,Tr=(xr|0)==0;do if(!Tr){Cr=_s_cmp(Jf,18449,1,2)|0,kr=(Cr|0)==0;if(!kr){Lr=_s_cmp(Jf,18452,1,2)|0,Ar=(Lr|0)==0;if(Ar)break;Or=_s_cmp(Jf,18455,1,2)|0,Mr=(Or|0)==0;if(Mr)break;_r=_s_cmp(Jf,18458,1,2)|0,Dr=(_r|0)==0;if(Dr)break;Pr=_s_cmp(Jf,18461,1,2)|0,Hr=(Pr|0)==0;if(Hr)break;jr=_s_cmp(Jf,18464,1,2)|0,Fr=(jr|0)==0;if(!Fr)break r}}while(0);el=32;break}}Ir=_s_cmp(Vf,18470,1,2)|0,qr=(Ir|0)==0;if(qr){Rr=_s_cmp($f,18394,1,3)|0,Ur=(Rr|0)==0;if(!Ur)break;zr=bl,Wr=(zr|0)!=0,Xr=hf,Vr=HEAP32[Xr>>2]|0,Jr=(Vr|0)<=64;if(Wr){if(Jr){el=1;break}el=32;break}if(Jr){el=1;break}el=32;break}Kr=_s_cmp(Vf,18473,1,2)|0,Qr=(Kr|0)==0;if(Qr){Gr=_s_cmp($f,18394,1,3)|0,Yr=(Gr|0)==0;if(!Yr)break;Zr=bl,ei=(Zr|0)!=0,ti=Sa,ni=HEAP32[ti>>2]|0,ri=(ni|0)<=64;if(ei){if(ri){el=1;break}el=32;break}if(ri){el=1;break}el=32;break}si=_s_cmp(Vf,18461,1,2)|0,oi=(si|0)==0;if(oi){ui=_s_cmp($f,18422,1,3)|0,ai=(ui|0)==0;if(!ai)break;el=64;break}fi=_s_cmp(Vf,18476,1,2)|0,li=(fi|0)==0;if(li){ci=_s_cmp($f,18479,1,3)|0,hi=(ci|0)==0;if(!hi)break;el=64;break}pi=bl,di=(pi|0)!=0,di&&(gi=_s_cmp(Vf,18483,1,2)|0,yi=(gi|0)==0,yi&&(bi=_s_cmp($f,18486,1,3)|0,wi=(bi|0)==0,wi&&(el=1)))}while(0);return Ei=el,yl=Ei,Si=yl,u=Si,Va=u,STACKTOP=Sl,Va|0}break;case 4:return yl=6,sa=yl,u=sa,Va=u,STACKTOP=Sl,Va|0;case 5:return yl=2,oa=yl,u=oa,Va=u,STACKTOP=Sl,Va|0;case 6:return ua=bo,aa=HEAP32[ua>>2]|0,fa=Sa,la=HEAP32[fa>>2]|0,ha=(aa|0)<=(la|0),ha?(pa=bo,da=HEAP32[pa>>2]|0,ya=da):(va=Sa,ma=HEAP32[va>>2]|0,ya=ma),ga=+(ya|0),ba=ga*1.600000023841858,wa=~~ba,yl=wa,Ea=yl,u=Ea,Va=u,STACKTOP=Sl,Va|0;case 7:return yl=1,Ta=yl,u=Ta,Va=u,STACKTOP=Sl,Va|0;case 8:return yl=50,Na=yl,u=Na,Va=u,STACKTOP=Sl,Va|0;case 9:return yl=25,Ca=yl,u=Ca,Va=u,STACKTOP=Sl,Va|0;case 10:return yl=1,ka=yl,La=(ka|0)==1,La&&(Aa=_ieeeck_(2692,2696,2700)|0,yl=Aa),Oa=yl,u=Oa,Va=u,STACKTOP=Sl,Va|0;case 11:return yl=1,Ma=yl,_a=(Ma|0)==1,_a&&(Da=_ieeeck_(2704,2696,2700)|0,yl=Da),Ha=yl,u=Ha,Va=u,STACKTOP=Sl,Va|0;case 16:case 15:case 14:case 13:case 12:return Ba=a,ja=hn,Fa=vi,Ia=bo,qa=Sa,Ra=ef,Ua=hf,za=_iparmq_(Ba,ja,Fa,Ia,qa,Ra,Ua)|0,yl=za,Wa=yl,u=Wa,Va=u,STACKTOP=Sl,Va|0;default:return yl=-1,Z=yl,u=Z,Va=u,STACKTOP=Sl,Va|0}while(0);return 0}function _ieeeck_(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0;Bn=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,z=t,et=n,Pn=1,ct=et,St=+HEAPF32[ct>>2],Dt=z,Wt=+HEAPF32[Dt>>2],tn=St/Wt,Dn=tn,pn=Dn,s=et,m=+HEAPF32[s>>2],k=pn<=m;do{if(!k){j=et,F=+HEAPF32[j>>2],I=-F,q=z,R=+HEAPF32[q>>2],U=I/R,On=U,W=On,X=z,V=+HEAPF32[X>>2],$=W>=V;if($){Pn=0,J=Pn,r=J;break}K=et,Q=+HEAPF32[K>>2],G=On,Y=et,Z=+HEAPF32[Y>>2],tt=G+Z,nt=Q/tt,Mn=nt,rt=Mn,it=z,st=+HEAPF32[it>>2],ot=rt!=st;if(ot){Pn=0,ut=Pn,r=ut;break}at=et,ft=+HEAPF32[at>>2],lt=Mn,ht=ft/lt,On=ht,pt=On,dt=z,vt=+HEAPF32[dt>>2],mt=pt>=vt;if(mt){Pn=0,gt=Pn,r=gt;break}yt=Mn,bt=z,wt=+HEAPF32[bt>>2],Et=yt+wt,_n=Et,xt=_n,Tt=z,Nt=+HEAPF32[Tt>>2],Ct=xt!=Nt;if(Ct){Pn=0,kt=Pn,r=kt;break}Lt=et,At=+HEAPF32[Lt>>2],Ot=_n,Mt=At/Ot,Dn=Mt,_t=Dn,Pt=et,Ht=+HEAPF32[Pt>>2],Bt=_t<=Ht;if(Bt){Pn=0,jt=Pn,r=jt;break}Ft=Dn,It=On,qt=It*Ft,On=qt,Rt=On,Ut=z,zt=+HEAPF32[Ut>>2],Xt=Rt>=zt;if(Xt){Pn=0,Vt=Pn,r=Vt;break}$t=Dn,Jt=Dn,Kt=Jt*$t,Dn=Kt,Qt=Dn,Gt=et,Yt=+HEAPF32[Gt>>2],Zt=Qt<=Yt;if(Zt){Pn=0,en=Pn,r=en;break}nn=i,rn=HEAP32[nn>>2]|0,sn=(rn|0)==0;if(sn){on=Pn,r=on;break}un=Dn,an=On,fn=un+an,Tn=fn,ln=Dn,cn=On,hn=ln/cn,Nn=hn,dn=Dn,vn=Dn,mn=dn/vn,Cn=mn,gn=Dn,yn=z,bn=+HEAPF32[yn>>2],wn=gn*bn,kn=wn,En=On,Sn=Mn,xn=En*Sn,Ln=xn,o=Ln,u=o*0,An=u,a=Tn,f=Tn,l=a==f;if(l){Pn=0,c=Pn,r=c;break}h=Nn,p=Nn,d=h==p;if(d){Pn=0,v=Pn,r=v;break}g=Cn,y=Cn,b=g==y;if(b){Pn=0,w=Pn,r=w;break}E=kn,S=kn,x=E==S;if(x){Pn=0,T=Pn,r=T;break}N=Ln,C=Ln,L=N==C;if(L){Pn=0,A=Pn,r=A;break}O=An,M=An,_=O==M;if(_){Pn=0,D=Pn,r=D;break}P=Pn,r=P;break}Pn=0,B=Pn,r=B}while(0);return H=r,STACKTOP=Bn,H|0}function _iparmq_(e,t,n,r,i,s,o){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0;var u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0;return Qt=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),$t=Qt+8|0,c=e,h=t,x=n,P=r,X=i,nt=s,pt=o,Tt=c,Ht=HEAP32[Tt>>2]|0,Ut=(Ht|0)==15,Ut?Kt=4:(p=c,d=HEAP32[p>>2]|0,v=(d|0)==13,v?Kt=4:(m=c,g=HEAP32[m>>2]|0,y=(g|0)==16,y&&(Kt=4))),(Kt|0)==4&&(b=nt,w=HEAP32[b>>2]|0,E=X,S=HEAP32[E>>2]|0,T=w-S|0,N=T+1|0,Xt=N,Vt=2,C=Xt,k=(C|0)>=30,u=k?4:2,Vt=u,L=Xt,A=(L|0)>=60,a=A?10:u,Vt=a,O=Xt,M=(O|0)>=150,M&&(_=Xt,D=+(_|0),H=D,B=+Math_log(+H),j=+Math_log(2),F=B/j,I=F,HEAPF32[$t>>2]=I,zt=10,q=Xt,R=_i_nint($t)|0,U=(q|0)/(R|0)&-1,Wt=U,z=zt,W=Wt,V=(z|0)>=(W|0),$=zt,J=Wt,K=V?$:J,Vt=K),Q=Xt,G=(Q|0)>=590,G&&(Vt=64),Y=Xt,Z=(Y|0)>=3e3,Z&&(Vt=128),et=Xt,tt=(et|0)>=6e3,tt&&(Vt=256),zt=2,rt=Vt,it=Vt,st=(it|0)%2&-1,ot=rt-st|0,Wt=ot,ut=zt,at=Wt,ft=(ut|0)>=(at|0),lt=zt,ct=Wt,ht=ft?lt:ct,Vt=ht),dt=c,vt=HEAP32[dt>>2]|0,mt=(vt|0)==12,mt?(Jt=75,Rt=Jt,STACKTOP=Qt,Rt|0):(gt=c,yt=HEAP32[gt>>2]|0,bt=(yt|0)==14,bt?(Jt=14,Rt=Jt,STACKTOP=Qt,Rt|0):(wt=c,Et=HEAP32[wt>>2]|0,St=(Et|0)==15,St?(xt=Vt,Jt=xt,Rt=Jt,STACKTOP=Qt,Rt|0):(Nt=c,Ct=HEAP32[Nt>>2]|0,kt=(Ct|0)==13,kt?(Lt=Xt,At=(Lt|0)<=500,Ot=Vt,At?(Jt=Ot,Rt=Jt,STACKTOP=Qt,Rt|0):(Mt=Ot*3|0,_t=(Mt|0)/2&-1,Jt=_t,Rt=Jt,STACKTOP=Qt,Rt|0)):(Dt=c,Pt=HEAP32[Dt>>2]|0,Bt=(Pt|0)==16,Bt?(Jt=0,jt=Vt,Ft=(jt|0)>=14,l=Ft?1:0,Jt=l,It=Vt,qt=(It|0)>=14,f=qt?2:l,Jt=f,Rt=Jt,STACKTOP=Qt,Rt|0):(Jt=-1,Rt=Jt,STACKTOP=Qt,Rt|0)))))}function _lsame_(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0;nn=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,d=t,N=r,B=HEAP8[N>>0]|0,$=B&255,it=d,vt=HEAP8[it>>0]|0,Ct=vt&255,jt=($|0)==(Ct|0),i=jt&1,Zt=i,s=Zt,o=(s|0)!=0;if(o)return u=Zt,n=u,Bt=n,STACKTOP=nn,Bt|0;en=90,a=r,f=HEAP8[a>>0]|0,l=f&255,Ft=l,c=d,h=HEAP8[c>>0]|0,p=h&255,It=p,v=en,m=(v|0)==90,g=en,y=(g|0)==122,qt=m|y;do if(qt)b=Ft,w=(b|0)>=97,E=Ft,S=(E|0)<=122,Kt=w&S,Kt&&(x=Ft,T=x+ -32|0,Ft=T),C=It,k=(C|0)>=97,L=It,A=(L|0)<=122,Qt=k&A,Qt&&(O=It,M=O+ -32|0,It=M);else{_=en,D=(_|0)==233,P=en,H=(P|0)==169,Gt=D|H;if(!Gt){pt=en,dt=(pt|0)==218,mt=en,gt=(mt|0)==250,Vt=dt|gt;if(!Vt)break;yt=Ft,bt=(yt|0)>=225,wt=Ft,Et=(wt|0)<=250,$t=bt&Et,$t&&(St=Ft,xt=St+ -32|0,Ft=xt),Tt=It,Nt=(Tt|0)>=225,kt=It,Lt=(kt|0)<=250,Jt=Nt&Lt;if(!Jt)break;At=It,Ot=At+ -32|0,It=Ot;break}j=Ft,F=(j|0)>=129,I=Ft,q=(I|0)<=137,Yt=F&q,Yt?tn=12:(R=Ft,U=(R|0)>=145,z=Ft,W=(z|0)<=153,Rt=U&W,Rt?tn=12:(X=Ft,V=(X|0)>=162,J=Ft,K=(J|0)<=169,Ut=V&K,Ut&&(tn=12))),(tn|0)==12&&(Q=Ft,G=Q+64|0,Ft=G),Y=It,Z=(Y|0)>=129,et=It,tt=(et|0)<=137,zt=Z&tt;if(!zt){nt=It,rt=(nt|0)>=145,st=It,ot=(st|0)<=153,Wt=rt&ot;if(!Wt){ut=It,at=(ut|0)>=162,ft=It,lt=(ft|0)<=169,Xt=at&lt;if(!Xt)break}}ct=It,ht=ct+64|0,It=ht}while(0);return Mt=Ft,_t=It,Dt=(Mt|0)==(_t|0),Pt=Dt&1,Zt=Pt,Ht=Zt,n=Ht,Bt=n,STACKTOP=nn,Bt|0}function _dgetf2_(e,t,n,r,i,s){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0;var o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0;Li=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Ei=Li+36|0,Si=Li+32|0,xi=Li+28|0,bi=Li+8|0,u=e,cn=t,Jn=n,sr=r,mr=i,kr=s,Fr=sr,Kr=HEAP32[Fr>>2]|0,gi=Kr,oi=gi,a=1+oi|0,yi=a,b=yi,O=Jn,R=0-b|0,Y=O+(R<<3)|0,Jn=Y,ft=mr,wt=ft+ -4|0,mr=wt,Mt=kr,HEAP32[Mt>>2]=0,Ut=u,Zt=HEAP32[Ut>>2]|0,hn=(Zt|0)<0;do if(hn)xn=kr,HEAP32[xn>>2]=-1;else{Pn=cn,Rn=HEAP32[Pn>>2]|0,Un=(Rn|0)<0;if(Un){zn=kr,HEAP32[zn>>2]=-2;break}Wn=sr,Xn=HEAP32[Wn>>2]|0,Vn=u,$n=HEAP32[Vn>>2]|0,Kn=1>=($n|0),Kn?Zn=1:(Qn=u,Gn=HEAP32[Qn>>2]|0,Zn=Gn),Yn=(Xn|0)<(Zn|0),Yn&&(er=kr,HEAP32[er>>2]=-4)}while(0);tr=kr,nr=HEAP32[tr>>2]|0,rr=(nr|0)!=0;if(rr)return ir=kr,or=HEAP32[ir>>2]|0,ur=0-or|0,HEAP32[Ei>>2]=ur,_xerbla_(18490,Ei)|0,o=0,qn=o,STACKTOP=Li,qn|0;ar=u,fr=HEAP32[ar>>2]|0,lr=(fr|0)==0;if(!lr){cr=cn,hr=HEAP32[cr>>2]|0,pr=(hr|0)==0;if(!pr){dr=+_dlamch_(18497),Ci=dr,vr=u,gr=HEAP32[vr>>2]|0,yr=cn,br=HEAP32[yr>>2]|0,wr=(gr|0)<=(br|0),wr?(Er=u,Sr=HEAP32[Er>>2]|0,Nr=Sr):(xr=cn,Tr=HEAP32[xr>>2]|0,Nr=Tr),HEAP32[Ei>>2]=Nr,Ti=1;for(;;){Cr=Ti,Lr=HEAP32[Ei>>2]|0,Ar=(Cr|0)<=(Lr|0);if(!Ar)break;Or=u,Mr=HEAP32[Or>>2]|0,_r=Ti,Dr=Mr-_r|0,Pr=Dr+1|0,HEAP32[Si>>2]=Pr,Hr=Ti,Br=Hr-1|0,jr=Ti,Ir=Ti,qr=gi,Rr=Math_imul(Ir,qr)|0,Ur=jr+Rr|0,zr=Jn,Wr=zr+(Ur<<3)|0,Xr=_idamax_(Si,Wr,2708)|0,Vr=Br+Xr|0,Ni=Vr,$r=Ni,Jr=Ti,Qr=mr,Gr=Qr+(Jr<<2)|0,HEAP32[Gr>>2]=$r,Yr=Ni,Zr=Ti,ei=gi,ti=Math_imul(Zr,ei)|0,ni=Yr+ti|0,ri=Jn,ii=ri+(ni<<3)|0,si=+HEAPF64[ii>>3],ui=si!=0;e:do if(ui){ai=Ni,fi=Ti,li=(ai|0)!=(fi|0),li&&(ci=cn,hi=Ti,pi=gi,di=hi+pi|0,vi=Jn,mi=vi+(di<<3)|0,f=sr,l=Ni,c=gi,h=l+c|0,p=Jn,d=p+(h<<3)|0,v=sr,_dswap_(ci,mi,f,d,v)|0),m=Ti,g=u,y=HEAP32[g>>2]|0,w=(m|0)<(y|0);if(w){E=Ti,S=Ti,x=gi,T=Math_imul(S,x)|0,N=E+T|0,C=Jn,k=C+(N<<3)|0,L=+HEAPF64[k>>3],HEAPF64[bi>>3]=L,A=+HEAPF64[bi>>3],M=A>=0,_=+HEAPF64[bi>>3],D=-_,P=M?_:D,H=Ci,B=P>=H,j=u,F=HEAP32[j>>2]|0,I=Ti,q=F-I|0,HEAP32[Si>>2]=q;if(B){U=Ti,z=Ti,W=gi,X=Math_imul(z,W)|0,V=U+X|0,$=Jn,J=$+(V<<3)|0,K=+HEAPF64[J>>3],Q=1/K,HEAPF64[bi>>3]=Q,G=Ti,Z=G+1|0,et=Ti,tt=gi,nt=Math_imul(et,tt)|0,rt=Z+nt|0,it=Jn,st=it+(rt<<3)|0,_dscal_(Si,bi,st,2708)|0;break}wi=1;for(;;){ot=wi,ut=HEAP32[Si>>2]|0,at=(ot|0)<=(ut|0);if(!at)break e;lt=Ti,ct=Ti,ht=gi,pt=Math_imul(ct,ht)|0,dt=lt+pt|0,vt=Jn,mt=vt+(dt<<3)|0,gt=+HEAPF64[mt>>3],yt=Ti,bt=wi,Et=yt+bt|0,St=Ti,xt=gi,Tt=Math_imul(St,xt)|0,Nt=Et+Tt|0,Ct=Jn,kt=Ct+(Nt<<3)|0,Lt=+HEAPF64[kt>>3],At=Lt/gt,HEAPF64[kt>>3]=At,Ot=wi,_t=Ot+1|0,wi=_t}}}else Dt=kr,Pt=HEAP32[Dt>>2]|0,Ht=(Pt|0)==0,Ht&&(Bt=Ti,jt=kr,HEAP32[jt>>2]=Bt);while(0);Ft=Ti,It=u,qt=HEAP32[It>>2]|0,Rt=cn,zt=HEAP32[Rt>>2]|0,Wt=(qt|0)<=(zt|0),Wt?(Xt=u,Vt=HEAP32[Xt>>2]|0,Qt=Vt):($t=cn,Jt=HEAP32[$t>>2]|0,Qt=Jt),Kt=(Ft|0)<(Qt|0),Kt&&(Gt=u,Yt=HEAP32[Gt>>2]|0,en=Ti,tn=Yt-en|0,HEAP32[Si>>2]=tn,nn=cn,rn=HEAP32[nn>>2]|0,sn=Ti,on=rn-sn|0,HEAP32[xi>>2]=on,un=Ti,an=un+1|0,fn=Ti,ln=gi,pn=Math_imul(fn,ln)|0,dn=an+pn|0,vn=Jn,mn=vn+(dn<<3)|0,gn=Ti,yn=Ti,bn=yn+1|0,wn=gi,En=Math_imul(bn,wn)|0,Sn=gn+En|0,Tn=Jn,Nn=Tn+(Sn<<3)|0,Cn=sr,kn=Ti,Ln=kn+1|0,An=Ti,On=An+1|0,Mn=gi,_n=Math_imul(On,Mn)|0,Dn=Ln+_n|0,Hn=Jn,Bn=Hn+(Dn<<3)|0,jn=sr,_dger_(Si,xi,896,mn,2708,Nn,Cn,Bn,jn)|0),Fn=Ti,In=Fn+1|0,Ti=In}return o=0,qn=o,STACKTOP=Li,qn|0}}return o=0,qn=o,STACKTOP=Li,qn|0}function _pow_di(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0;H=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=t,M=1,d=n,x=+HEAPF64[d>>3],D=x,T=r,N=HEAP32[T>>2]|0,O=N,C=O,k=(C|0)!=0;if(!k)return S=M,STACKTOP=H,+S;L=O,A=(L|0)<0,A&&(i=O,s=0-i|0,O=s,o=D,u=1/o,D=u),a=O,_=a;for(;;){f=_,l=f&1,c=(l|0)!=0,c&&(h=D,p=M,v=p*h,M=v),m=_,g=m>>>1,_=g,y=(g|0)!=0;if(!y)break;b=D,w=D,E=w*b,D=E}return S=M,STACKTOP=H,+S}function _i_nint(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0;return m=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=e,n=t,u=+HEAPF32[n>>2],a=u>=0,f=t,l=+HEAPF32[f>>2],c=l,a?(h=c+.5,p=+Math_floor(+h),o=p):(d=.5-c,r=+Math_floor(+d),i=-r,o=i),s=~~o,STACKTOP=m,s|0}function _s_cmp(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0;Bt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=e,m=t,k=n,F=r,K=s,Mt=K,ot=m,Dt=ot,gt=Mt,Lt=k,Ot=gt+Lt|0,_t=Ot,o=Dt,u=F,a=o+u|0,Pt=a,f=k,l=F,c=(f|0)<=(l|0);e:do{if(c){for(;;){h=Mt,p=_t,d=h>>>0<p>>>0;if(!d)break;v=Mt,g=HEAP8[v>>0]|0,y=g&255,b=Dt,w=HEAP8[b>>0]|0,E=w&255,S=(y|0)!=(E|0),x=Mt;if(S){Ht=4;break}M=x+1|0,Mt=M,_=Dt,D=_+1|0,Dt=D}if((Ht|0)==4)return T=HEAP8[x>>0]|0,N=T&255,C=Dt,L=HEAP8[C>>0]|0,A=L&255,O=N-A|0,i=O,At=i,STACKTOP=Bt,At|0;for(;;){P=Dt,H=Pt,B=P>>>0<H>>>0;if(!B)break e;j=Dt,I=HEAP8[j>>0]|0,q=I&255,R=(q|0)!=32,U=Dt;if(R)break;V=U+1|0,Dt=V}return z=HEAP8[U>>0]|0,W=z&255,X=32-W|0,i=X,At=i,STACKTOP=Bt,At|0}for(;;){$=Dt,J=Pt,Q=$>>>0<J>>>0;if(!Q)break;G=Mt,Y=HEAP8[G>>0]|0,Z=Y&255,et=Dt,tt=HEAP8[et>>0]|0,nt=tt&255,rt=(Z|0)==(nt|0),it=Mt;if(!rt){Ht=13;break}st=it+1|0,Mt=st,ut=Dt,at=ut+1|0,Dt=at}if((Ht|0)==13)return ft=HEAP8[it>>0]|0,lt=ft&255,ct=Dt,ht=HEAP8[ct>>0]|0,pt=ht&255,dt=lt-pt|0,i=dt,At=i,STACKTOP=Bt,At|0;for(;;){vt=Mt,mt=_t,yt=vt>>>0<mt>>>0;if(!yt)break e;bt=Mt,wt=HEAP8[bt>>0]|0,Et=wt&255,St=(Et|0)!=32,xt=Mt;if(St)break;kt=xt+1|0,Mt=kt}return Tt=HEAP8[xt>>0]|0,Nt=Tt&255,Ct=Nt-32|0,i=Ct,At=i,STACKTOP=Bt,0}while(0);return i=0,At=i,STACKTOP=Bt,At|0}function _s_copy(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0;Ct=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,s=t,m=n,k=r,F=i,K=m,ot=F+K|0,xt=ot,gt=m,Et=k,St=(gt|0)<=(Et|0);if(St){o=i,u=s,a=o>>>0<=u>>>0;if(!a){f=i,l=s,c=m,h=l+c|0,p=f>>>0>=h>>>0;if(!p){x=m,T=s,N=T+x|0,s=N;for(;;){C=i,L=xt,A=C>>>0<L>>>0;if(!A)break;O=s,M=O+ -1|0,s=M,_=HEAP8[M>>0]|0,D=xt,P=D+ -1|0,xt=P,HEAP8[P>>0]=_}STACKTOP=Ct;return}}for(;;){d=i,v=xt,g=d>>>0<v>>>0;if(!g)break;y=s,b=y+1|0,s=b,w=HEAP8[y>>0]|0,E=i,S=E+1|0,i=S,HEAP8[E>>0]=w}STACKTOP=Ct;return}H=s,B=k,j=H+B|0,Tt=j,I=i,q=s,R=I>>>0<=q>>>0;if(R)Nt=11;else{U=i,z=Tt,W=U>>>0>=z>>>0;if(W)Nt=11;else{et=k,tt=i,nt=tt+et|0,i=nt;for(;;){rt=s,it=Tt,st=rt>>>0<it>>>0;if(!st)break;ut=Tt,at=ut+ -1|0,Tt=at,ft=HEAP8[at>>0]|0,lt=i,ct=lt+ -1|0,i=ct,HEAP8[ct>>0]=ft}ht=k,pt=i,dt=pt+ht|0,i=dt}}e:do if((Nt|0)==11)for(;;){Nt=0,X=s,V=Tt,$=X>>>0<V>>>0;if(!$)break e;J=s,Q=J+1|0,s=Q,G=HEAP8[J>>0]|0,Y=i,Z=Y+1|0,i=Z,HEAP8[Y>>0]=G,Nt=11}while(0);for(;;){vt=i,mt=xt,yt=vt>>>0<mt>>>0;if(!yt)break;bt=i,wt=bt+1|0,i=wt,HEAP8[bt>>0]=32}STACKTOP=Ct;return}function _do_lio(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0;return m=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,s=t,u=n,a=r,f=HEAP32[3163]|0,l=s,c=u,h=a,p=i,d=HEAP32[p>>2]|0,o=FUNCTION_TABLE_iiiii[f&15](l,c,h,d)|0,STACKTOP=m,o|0}function _pars_f(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0;return a=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,HEAP32[678]=0,HEAP32[679]=0,HEAP32[680]=0,r=n,i=_f_s(r,0)|0,s=(i|0)==(0|0),s?t=-1:t=0,o=t,STACKTOP=a,o|0}function _do_fio(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0;Pn=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Mn=Pn,i=e,$=t,it=n,Ln=0;e:for(;;){vt=Ln,Ct=i,jt=HEAP32[Ct>>2]|0,Jt=(vt|0)<(jt|0);if(!Jt){Dn=41;break}t:for(;;){on=HEAP32[678]|0,gn=2820+(on<<4)|0,On=gn,s=HEAP32[gn>>2]|0,m=_type_f(s)|0;do switch(m|0){case 9:J=HEAP32[3155]|0,K=On,Q=FUNCTION_TABLE_ii[J&15](K)|0,G=(Q|0)!=0,Y=HEAP32[678]|0,Z=Y+1|0,HEAP32[678]=Z;if(G)continue t;break t;case 8:et=HEAP32[681]|0,tt=2728+(et<<2)|0,nt=HEAP32[tt>>2]|0,rt=(nt|0)<=0;if(!rt){Dn=11;break t}st=HEAP32[681]|0,ot=st+ -1|0,HEAP32[681]=ot,ut=HEAP32[678]|0,at=ut+1|0,HEAP32[678]=at;continue t;case 6:Ft=On,It=Ft+4|0,qt=HEAP32[It>>2]|0,Rt=HEAP32[681]|0,Ut=Rt+1|0,HEAP32[681]=Ut,zt=2728+(Ut<<2)|0,HEAP32[zt>>2]=qt,Wt=HEAP32[678]|0,Xt=Wt+1|0,HEAP32[678]=Xt;continue t;case 1:Vt=On,$t=Vt+4|0,Kt=HEAP32[$t>>2]|0,Qt=HEAP32[693]|0,Gt=Qt+1|0,HEAP32[693]=Gt,Yt=2776+(Gt<<2)|0,HEAP32[Yt>>2]=Kt,Zt=HEAP32[678]|0,en=Zt+1|0,HEAP32[678]=en;continue t;case 3:tn=HEAP32[681]|0,nn=2728+(tn<<2)|0,rn=HEAP32[nn>>2]|0,sn=rn+ -1|0,HEAP32[nn>>2]=sn,un=(sn|0)<=0;if(un){an=HEAP32[681]|0,fn=an+ -1|0,HEAP32[681]=fn,ln=HEAP32[693]|0,cn=ln+ -1|0,HEAP32[693]=cn,hn=HEAP32[678]|0,pn=hn+1|0,HEAP32[678]=pn;continue t}dn=HEAP32[693]|0,vn=dn+ -1|0,HEAP32[693]=vn,mn=2776+(dn<<2)|0,yn=HEAP32[mn>>2]|0,bn=1+yn|0,HEAP32[678]=bn;continue t;case 2:HEAP32[681]=0,HEAP32[693]=0,wn=On,En=wn+4|0,Sn=HEAP32[En>>2]|0,HEAP32[678]=Sn,xn=$,Tn=(xn|0)==(0|0);if(Tn){Dn=26;break e}kn=HEAP32[692]|0,o=(kn|0)!=0;if(!o){Dn=28;break e}u=HEAP32[3156]|0,a=FUNCTION_TABLE_i[u&15]()|0,An=a,f=(a|0)!=0;if(f){Dn=30;break e}continue t;case 16:c=$,h=(c|0)==(0|0);if(h){Dn=32;break e}v=HEAP32[678]|0,g=v+1|0,HEAP32[678]=g;continue t;case 33:HEAP32[704]=1,y=HEAP32[678]|0,b=y+1|0,HEAP32[678]=b;continue t;case 19:case 17:HEAP32[3151]=0,w=HEAP32[678]|0,E=w+1|0,HEAP32[678]=E;continue t;case 18:HEAP32[3151]=1,S=HEAP32[678]|0,x=S+1|0,HEAP32[678]=x;continue t;case 20:T=On,N=T+4|0,C=HEAP32[N>>2]|0,HEAP32[3161]=C,L=HEAP32[678]|0,A=L+1|0,HEAP32[678]=A;continue t;case 21:HEAP32[3152]=0,O=HEAP32[678]|0,M=O+1|0,HEAP32[678]=M;continue t;case 22:HEAP32[3152]=1,_=HEAP32[678]|0,D=_+1|0,HEAP32[678]=D;continue t;default:Dn=4;break e}while(0)}if((Dn|0)==11){Dn=0,ft=$,lt=(ft|0)==(0|0);if(lt){Dn=12;break}pt=HEAP32[681]|0,dt=2728+(pt<<2)|0,mt=HEAP32[dt>>2]|0,gt=mt+ -1|0,HEAP32[dt>>2]=gt,HEAP32[692]=1,yt=HEAP32[3154]|0,bt=On,wt=$,Et=it,St=FUNCTION_TABLE_iiii[yt&15](bt,wt,Et)|0,An=St,xt=(St|0)>0;if(xt){Dn=14;break}Ot=An,Mt=(Ot|0)<0;if(Mt){Dn=16;break}}P=Ln,H=P+1|0,Ln=H,B=it,j=$,I=j+B|0,$=I}return(Dn|0)==4?(k=HEAP32[3184]|0,F=On,R=HEAP32[F>>2]|0,U=HEAP32[3141]|0,HEAP32[Mn>>2]=R,_n=Mn+4|0,HEAP32[_n>>2]=U,_fprintf(k,18499,Mn)|0,z=HEAP32[3149]|0,W=HEAP32[z>>2]|0,X=(W|0)!=0,X?(V=___errno_location()|0,HEAP32[V>>2]=100):_f__fatal(100,18530),r=100,q=r,STACKTOP=Pn,q|0):(Dn|0)==12?(ct=HEAP32[3148]|0,ht=FUNCTION_TABLE_i[ct&15]()|0,r=ht,q=r,STACKTOP=Pn,q|0):(Dn|0)==14?(Tt=HEAP32[3149]|0,Nt=HEAP32[Tt>>2]|0,kt=___errno_location()|0,Lt=HEAP32[kt>>2]|0,At=_err__fl(Nt,Lt,18537)|0,r=At,q=r,STACKTOP=Pn,q|0):(Dn|0)==16?(_t=HEAP32[3149]|0,Dt=_t+8|0,Pt=HEAP32[Dt>>2]|0,Ht=(Pt|0)!=0,Ht?(Bt=___errno_location()|0,HEAP32[Bt>>2]=-1):_f__fatal(-1,18537),r=-1,q=r,STACKTOP=Pn,q|0):(Dn|0)==26?(Nn=HEAP32[3148]|0,Cn=FUNCTION_TABLE_i[Nn&15]()|0,r=Cn,q=r,STACKTOP=Pn,q|0):(Dn|0)==28?(r=0,q=r,STACKTOP=Pn,q|0):(Dn|0)==30?(l=An,r=l,q=r,STACKTOP=Pn,q|0):(Dn|0)==32?(p=HEAP32[3148]|0,d=FUNCTION_TABLE_i[p&15]()|0,r=d,q=r,STACKTOP=Pn,q|0):(Dn|0)==41?(r=0,q=r,STACKTOP=Pn,q|0):0}function _en_fio(){var e=0,t=0,n=0,r=0;return r=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=r,HEAP32[t>>2]=1,e=_do_fio(t,0,0)|0,STACKTOP=r,e|0}function _fmt_bg(){var e=0,t=0;t=STACKTOP,HEAP32[3159]=0,HEAP32[678]=0,HEAP32[693]=0,HEAP32[681]=0,HEAP32[692]=0,HEAP32[694]=0,HEAP32[682]=0;return}function _f_s(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0;_=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,d=t;for(;;){T=r,N=HEAP8[T>>0]|0,C=N<<24>>24,k=(C|0)==32,L=r,A=L+1|0,r=A;if(!k)break}O=HEAP8[L>>0]|0,i=O<<24>>24,s=(i|0)!=40;if(s)return n=0,x=n,STACKTOP=_,x|0;o=HEAP32[680]|0,u=o+1|0,HEAP32[680]=u,a=(o|0)==1,a&&(f=d,HEAP32[679]=f),l=d,c=_op_gen(1,l,0,0)|0,h=(c|0)<0;if(!h){p=r,v=_f_list(p)|0,r=v,m=(v|0)==(0|0);if(!m){for(;;){g=r,y=HEAP8[g>>0]|0,b=y<<24>>24,w=(b|0)==32,E=r;if(!w)break;S=E+1|0,r=S}return n=E,x=n,STACKTOP=_,x|0}}return n=0,x=n,STACKTOP=_,x|0}function _type_f(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0;u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=n;switch(r|0){case 1:t=1;break;case 2:t=2;break;case 3:t=3;break;case 6:t=6;break;case 14:case 13:case 15:case 12:case 11:case 5:case 4:t=9;break;case 36:case 35:case 28:case 27:case 26:case 25:case 24:case 29:case 34:case 32:case 31:case 30:case 10:case 7:case 23:t=8;break;default:i=n,t=i}return s=t,STACKTOP=u,s|0}function _op_gen(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0;return M=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),A=M,i=e,s=t,m=n,E=r,S=HEAP32[678]|0,x=2820+(S<<4)|0,L=x,T=HEAP32[678]|0,N=(T|0)>=300,N&&(C=HEAP32[3184]|0,_fprintf(C,18552,A)|0,k=HEAP32[3141]|0,_sig_die(k,1)),o=i,u=L,HEAP32[u>>2]=o,a=s,f=L,l=f+4|0,HEAP32[l>>2]=a,c=m,h=L,p=h+8|0,HEAP32[p>>2]=c,d=E,v=L,g=v+8|0,y=g+4|0,HEAP32[y>>2]=d,b=HEAP32[678]|0,w=b+1|0,HEAP32[678]=w,STACKTOP=M,b|0}function _f_list(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0;I=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e;for(;;){p=n,T=HEAP8[p>>0]|0,_=T<<24>>24,D=(_|0)!=0;if(!D){F=15;break}for(;;){P=n,H=HEAP8[P>>0]|0,B=H<<24>>24,j=(B|0)==32,r=n;if(!j)break;i=r+1|0,n=i}s=_i_tem(r)|0,n=s,o=(s|0)==(0|0);if(o){F=6;break}for(;;){u=n,a=HEAP8[u>>0]|0,f=a<<24>>24,l=(f|0)==32,c=n;if(!l)break;h=c+1|0,n=h}d=HEAP8[c>>0]|0,v=d<<24>>24,m=(v|0)==44,g=n;if(m){y=g+1|0,n=y;continue}b=HEAP8[g>>0]|0,w=b<<24>>24,E=(w|0)==41;if(E){F=12;break}continue}return(F|0)==6?(t=0,M=t,STACKTOP=I,M|0):(F|0)==12?(S=HEAP32[680]|0,x=S+ -1|0,HEAP32[680]=x,N=(x|0)==0,N?(C=HEAP32[679]|0,_op_gen(2,C,0,0)|0,k=n,L=k+1|0,n=L,t=L,M=t,STACKTOP=I,M|0):(_op_gen(3,0,0,0)|0,A=n,O=A+1|0,n=O,t=O,M=t,STACKTOP=I,M|0)):(F|0)==15?(t=0,M=t,STACKTOP=I,M|0):0}function _i_tem(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0;L=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),C=L+8|0,N=L+4|0,n=e,p=n,g=HEAP8[p>>0]|0,y=g<<24>>24,b=(y|0)==41,w=n;do{if(!b){E=_ne_d(w,C)|0,S=(E|0)!=0;if(S){x=HEAP32[C>>2]|0,t=x;break}r=n,i=_e_d(r,C)|0,s=(i|0)!=0;if(s){o=HEAP32[C>>2]|0,t=o;break}u=n,a=_gt_num(u,N,1)|0,n=a,f=HEAP32[N>>2]|0,l=_op_gen(6,f,0,0)|0,T=l,c=(l|0)<0;if(c){t=0;break}h=n,d=T,v=_f_s(h,d)|0,t=v;break}t=w}while(0);return m=t,STACKTOP=L,m|0}function _ne_d(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0;cn=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),on=cn+12|0,r=e,N=t,un=0,B=r,$=HEAP8[B>>0]|0,it=$<<24>>24;e:do switch(it|0){case 58:_op_gen(16,0,0,0)|0,ln=43;break;case 36:_op_gen(33,0,0,0)|0,ln=43;break;case 98:case 66:vt=r,Ct=vt+1|0,r=Ct,jt=HEAP8[Ct>>0]|0,Jt=jt<<24>>24,i=(Jt|0)==122;if(!i){v=r,g=HEAP8[v>>0]|0,y=g<<24>>24,b=(y|0)==90;if(!b){_op_gen(21,0,0,0)|0,ln=43;break e}}_op_gen(22,0,0,0)|0,ln=43;break;case 115:case 83:w=r,E=w+1|0,S=HEAP8[E>>0]|0,x=S<<24>>24,T=(x|0)==115;do if(T)ln=11;else{C=r,k=C+1|0,L=HEAP8[k>>0]|0,A=L<<24>>24,O=(A|0)==83;if(O)ln=11;else{D=r,P=D+1|0,H=HEAP8[P>>0]|0,j=H<<24>>24,F=(j|0)==112;if(!F){I=r,q=I+1|0,R=HEAP8[q>>0]|0,U=R<<24>>24,z=(U|0)==80;if(!z){fn=17;break}}fn=18,W=r,X=W+1|0,r=X}}while(0);(ln|0)==11&&(fn=19,M=r,_=M+1|0,r=_),V=fn,_op_gen(V,0,0,0)|0,ln=43;break;case 47:_op_gen(5,0,0,0)|0,ln=43;break;case 45:un=1,ln=19;break;case 43:ln=19;break;case 57:case 56:case 55:case 54:case 53:case 52:case 51:case 50:case 49:case 48:ln=20;break;case 39:case 34:case 2:return Et=_op_gen(11,0,0,0)|0,St=2820+(Et<<4)|0,an=St,xt=r,Tt=an,Nt=Tt+8|0,HEAP32[Nt>>2]=xt,kt=r,Lt=_ap_end(kt)|0,At=N,HEAP32[At>>2]=Lt,Ot=(Lt|0)==(0|0),Ot?(n=0,m=n,STACKTOP=cn,m|0):(n=1,m=n,STACKTOP=cn,m|0);case 116:case 84:Mt=r,_t=Mt+1|0,Dt=HEAP8[_t>>0]|0,Pt=Dt<<24>>24,Ht=(Pt|0)==108;do if(Ht)ln=34;else{Bt=r,Ft=Bt+1|0,It=HEAP8[Ft>>0]|0,qt=It<<24>>24,Rt=(qt|0)==76;if(Rt)ln=34;else{Wt=r,Xt=Wt+1|0,Vt=HEAP8[Xt>>0]|0,$t=Vt<<24>>24,Kt=($t|0)==114;if(!Kt){Qt=r,Gt=Qt+1|0,Yt=HEAP8[Gt>>0]|0,Zt=Yt<<24>>24,en=(Zt|0)==82;if(!en){fn=15;break}}fn=14,tn=r,nn=tn+1|0,r=nn}}while(0);(ln|0)==34&&(fn=13,Ut=r,zt=Ut+1|0,r=zt),rn=r,sn=rn+1|0,s=_gt_num(sn,on,0)|0,r=s,o=(s|0)!=(0|0),o?(u=r,a=u+ -1|0,r=a,f=fn,l=HEAP32[on>>2]|0,_op_gen(f,l,0,0)|0,ln=43):ln=21;break;case 120:case 88:_op_gen(4,1,0,0)|0,ln=43;break;case 112:case 80:_op_gen(20,1,0,0)|0,ln=43;break;default:return n=0,m=n,STACKTOP=cn,m|0}while(0);(ln|0)==19&&(J=r,K=J+1|0,r=K,ln=20);t:do if((ln|0)==20){Q=r,G=_gt_num(Q,on,0)|0,r=G,Y=(G|0)!=(0|0);if(Y){et=r,tt=HEAP8[et>>0]|0,nt=tt<<24>>24;switch(nt|0){case 112:case 80:rt=un,st=(rt|0)!=0,st&&(ot=HEAP32[on>>2]|0,ut=0-ot|0,HEAP32[on>>2]=ut),at=HEAP32[on>>2]|0,_op_gen(20,at,0,0)|0,ln=43;break t;case 120:case 88:ft=HEAP32[on>>2]|0,_op_gen(4,ft,0,0)|0,ln=43;break t;case 104:case 72:lt=HEAP32[on>>2]|0,ct=_op_gen(12,lt,0,0)|0,ht=2820+(ct<<4)|0,an=ht,pt=r,dt=pt+1|0,mt=an,gt=mt+8|0,HEAP32[gt>>2]=dt,yt=HEAP32[on>>2]|0,bt=r,wt=bt+yt|0,r=wt,ln=43;break t;default:return n=0,m=n,STACKTOP=cn,m|0}}else ln=21}while(0);return(ln|0)==21?(Z=N,HEAP32[Z>>2]=0,n=1,m=n,STACKTOP=cn,m|0):(ln|0)==43?(c=r,h=c+1|0,r=h,p=r,d=N,HEAP32[d>>2]=p,n=1,m=n,STACKTOP=cn,m|0):0}function _e_d(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0;Pn=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),An=Pn+24|0,Mn=Pn+20|0,Tn=Pn+16|0,Nn=Pn+12|0,r=e,z=t,Cn=0,_n=0,et=r,On=et,ct=r,St=_gt_num(ct,An,1)|0,r=St,Dt=HEAP32[An>>2]|0,_op_gen(6,Dt,0,0)|0,Wt=r,tn=Wt+1|0,r=tn,pn=HEAP8[Wt>>0]|0,i=pn<<24>>24;e:do switch(i|0){case 101:case 69:_n=1,Dn=3;break;case 103:case 71:Dn=3;break;case 111:case 79:kn=32,Ln=34,Dn=40;break;case 122:case 90:kn=35,Ln=36,Dn=40;break;case 108:case 76:Cn=1,wt=r,Et=_gt_num(wt,Mn,0)|0,r=Et,xt=(Et|0)!=(0|0),xt?(Tt=HEAP32[Mn>>2]|0,Nt=(Tt|0)==0,Nt||(Ct=HEAP32[Mn>>2]|0,_op_gen(29,Ct,0,0)|0)):Dn=4;break;case 97:case 65:Cn=1;for(;;){kt=r,Lt=HEAP8[kt>>0]|0,At=Lt<<24>>24,Ot=(At|0)==32,Mt=r;if(!Ot)break;_t=Mt+1|0,r=_t}Pt=HEAP8[Mt>>0]|0,Ht=Pt<<24>>24,Bt=(Ht|0)>=48;if(Bt){jt=r,Ft=HEAP8[jt>>0]|0,It=Ft<<24>>24,qt=(It|0)<=57;if(qt){Rt=r,Ut=_gt_num(Rt,Mn,1)|0,r=Ut,zt=HEAP32[Mn>>2]|0,Xt=(zt|0)==0;if(Xt)break e;Vt=HEAP32[Mn>>2]|0,_op_gen(31,Vt,0,0)|0;break e}}_op_gen(30,0,0,0)|0;break;case 102:case 70:$t=r,Jt=_gt_num($t,Mn,0)|0,r=Jt,Kt=(Jt|0)!=(0|0);if(Kt){Cn=1,Qt=HEAP32[Mn>>2]|0,Gt=(Qt|0)==0;if(!Gt){Yt=r,Zt=HEAP8[Yt>>0]|0,en=Zt<<24>>24,nn=(en|0)==46;if(nn){rn=r,sn=rn+1|0,on=_gt_num(sn,Tn,0)|0,r=on,un=(on|0)!=(0|0);if(!un){Dn=4;break e}}else HEAP32[Tn>>2]=0;an=HEAP32[Mn>>2]|0,fn=HEAP32[Tn>>2]|0,_op_gen(23,an,fn,0)|0}}else Dn=4;break;case 100:case 68:Cn=1,ln=r,cn=_gt_num(ln,Mn,0)|0,r=cn,hn=(cn|0)!=(0|0);if(hn){dn=HEAP32[Mn>>2]|0,vn=(dn|0)==0;if(!vn){mn=r,gn=HEAP8[mn>>0]|0,yn=gn<<24>>24,bn=(yn|0)==46;if(bn){wn=r,En=wn+1|0,Sn=_gt_num(En,Tn,0)|0,r=Sn,xn=(Sn|0)!=(0|0);if(!xn){Dn=4;break e}}else HEAP32[Tn>>2]=0;s=HEAP32[Mn>>2]|0,o=HEAP32[Tn>>2]|0,_op_gen(26,s,o,0)|0}}else Dn=4;break;case 105:case 73:kn=7,Ln=10,Dn=40;break;default:}while(0);do if((Dn|0)==3){Cn=1,v=r,C=_gt_num(v,Mn,0)|0,r=C,B=(C|0)!=(0|0);if(B){F=HEAP32[Mn>>2]|0,I=(F|0)==0;if(!I){q=r,R=HEAP8[q>>0]|0,U=R<<24>>24,W=(U|0)==46;if(W){X=r,V=X+1|0,$=_gt_num(V,Tn,0)|0,r=$,J=($|0)!=(0|0);if(!J){Dn=4;break}}else HEAP32[Tn>>2]=0;K=r,Q=HEAP8[K>>0]|0,G=Q<<24>>24,Y=(G|0)!=69;if(Y){Z=r,tt=HEAP8[Z>>0]|0,nt=tt<<24>>24,rt=(nt|0)!=101;if(rt){it=_n,st=(it|0)==1,ot=st?24:27,ut=HEAP32[Mn>>2]|0,at=HEAP32[Tn>>2]|0,_op_gen(ot,ut,at,0)|0;break}}ft=r,lt=ft+1|0,ht=_gt_num(lt,Nn,0)|0,r=ht,pt=(ht|0)!=(0|0),pt?(dt=_n,vt=(dt|0)==1,mt=vt?25:28,gt=HEAP32[Mn>>2]|0,yt=HEAP32[Tn>>2
]|0,bt=HEAP32[Nn>>2]|0,_op_gen(mt,gt,yt,bt)|0):Dn=4}}else Dn=4}else if((Dn|0)==40){u=r,a=_gt_num(u,Mn,0)|0,r=a,f=(a|0)!=(0|0);if(f){Cn=1,l=HEAP32[Mn>>2]|0,c=(l|0)==0;if(!c){h=r,p=HEAP8[h>>0]|0,d=p<<24>>24,m=(d|0)!=46;if(m){g=kn,y=HEAP32[Mn>>2]|0,_op_gen(g,y,0,0)|0;break}b=r,w=b+1|0,E=_gt_num(w,Tn,0)|0,r=E,S=(E|0)!=(0|0),S?(x=Ln,T=HEAP32[Mn>>2]|0,N=HEAP32[Tn>>2]|0,_op_gen(x,T,N,0)|0):Dn=4}}else Dn=4}while(0);return(Dn|0)==4?(j=z,HEAP32[j>>2]=0,n=1,H=n,STACKTOP=Pn,H|0):(k=Cn,L=(k|0)==0,L?(A=HEAP32[678]|0,O=A+ -1|0,HEAP32[678]=O,M=On,_=z,HEAP32[_>>2]=M,n=0,H=n,STACKTOP=Pn,H|0):(D=r,P=z,HEAP32[P>>2]=D,n=1,H=n,STACKTOP=Pn,H|0))}function _gt_num(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0;W=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,v=n,U=0,R=0,C=r,P=HEAP8[C>>0]|0,q=P;for(;;){H=q,B=H<<24>>24,j=(B|0)==32;if(j)F=r,I=F+1|0,r=I;else{s=q,o=s<<24>>24,u=(o|0)>57;if(u)break;a=q,f=a<<24>>24,l=(f|0)<48;if(l)break;c=U,h=c*10|0,p=q,d=p<<24>>24,m=h+d|0,g=m-48|0,U=g,y=R,b=y+1|0,R=b,w=r,E=w+1|0,r=E}S=r,x=HEAP8[S>>0]|0,q=x}return T=R,N=(T|0)==0,N?(k=v,L=(k|0)!=0,L||(r=0),A=v,O=i,HEAP32[O>>2]=A,D=r,STACKTOP=W,D|0):(M=U,_=i,HEAP32[_>>2]=M,D=r,STACKTOP=W,D|0)}function _ap_end(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0;_=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,x=p+1|0,n=x,T=HEAP8[p>>0]|0,O=T;for(;;){N=n,C=HEAP8[N>>0]|0,k=C<<24>>24!=0;if(!k)break;L=n,A=HEAP8[L>>0]|0,r=A<<24>>24,i=O,s=i<<24>>24,o=(r|0)!=(s|0);if(!o){u=n,a=u+1|0,n=a,f=HEAP8[a>>0]|0,l=f<<24>>24,c=O,h=c<<24>>24,d=(l|0)!=(h|0);if(d){M=5;break}}m=n,g=m+1|0,n=g}return(M|0)==5?(v=n,t=v,S=t,STACKTOP=_,S|0):(y=HEAP32[3149]|0,b=HEAP32[y>>2]|0,w=(b|0)!=0,w?(E=___errno_location()|0,HEAP32[E>>2]=100,t=0,S=t,STACKTOP=_,S|0):(_f__fatal(100,18541),t=0,S=t,STACKTOP=_,S|0))}function _c_sfe(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0;U=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,T=p+4|0,P=HEAP32[T>>2]|0,H=7764+(P*48|0)|0,q=H,HEAP32[1940]=H,B=n,j=B+4|0,F=HEAP32[j>>2]|0,I=(F|0)>=100;if(!I){r=n,i=r+4|0,s=HEAP32[i>>2]|0,o=(s|0)<0;if(!o){c=q,h=HEAP32[c>>2]|0,d=(h|0)==(0|0);if(d){v=n,m=v+4|0,g=HEAP32[m>>2]|0,y=_fk_open(3,5,g)|0,b=(y|0)!=0;if(b)return w=n,E=HEAP32[w>>2]|0,S=(E|0)!=0,S?(x=___errno_location()|0,HEAP32[x>>2]=114):_f__fatal(114,18577),t=114,D=t,STACKTOP=U,D|0}return N=q,C=N+24|0,k=HEAP32[C>>2]|0,L=(k|0)!=0,L?(t=0,D=t,STACKTOP=U,D|0):(A=n,O=HEAP32[A>>2]|0,M=(O|0)!=0,M?(_=___errno_location()|0,HEAP32[_>>2]=102):_f__fatal(102,18577),t=102,D=t,STACKTOP=U,D|0)}}return u=n,a=HEAP32[u>>2]|0,f=(a|0)!=0,f?(l=___errno_location()|0,HEAP32[l>>2]=101):_f__fatal(101,18581),t=101,D=t,STACKTOP=U,D|0}function _e_wsfe(){var e=0,t=0,n=0,r=0,i=0;return i=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),e=_en_fio()|0,n=e,HEAP32[3141]=0,t=n,STACKTOP=i,t|0}function _x_wSL(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),e=_f__putbuf(10)|0,i=e,HEAP32[3159]=0,HEAP32[3158]=0,HEAP32[3160]=0,t=i,n=(t|0)==0,r=n&1,STACKTOP=o,r|0}function _s_wsfe(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0;z=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=HEAP32[3146]|0,T=(p|0)!=0,T||_f_init(),HEAP32[3142]=0,HEAP32[3143]=1,HEAP32[3144]=1,HEAP32[3145]=1,H=n,B=_c_sfe(H)|0,R=B,j=(B|0)!=0;if(j)return F=R,t=F,P=t,STACKTOP=z,P|0;I=n,HEAP32[3149]=I,HEAP32[3158]=0,HEAP32[3159]=0,HEAP32[3160]=0,HEAP32[704]=0,HEAP32[3161]=0,q=n,r=q+12|0,i=HEAP32[r>>2]|0,HEAP32[3141]=i,s=HEAP32[1940]|0,o=HEAP32[s>>2]|0,HEAP32[3147]=o,u=HEAP32[3141]|0,a=_pars_f(u)|0,f=(a|0)<0;if(f)return l=n,c=HEAP32[l>>2]|0,h=(c|0)!=0,h?(d=___errno_location()|0,HEAP32[d>>2]=100):_f__fatal(100,18581),t=100,P=t,STACKTOP=z,P|0;HEAP32[3153]=7,HEAP32[3154]=8,HEAP32[3155]=9,HEAP32[3148]=10,HEAP32[3156]=11,HEAP32[3157]=12,_fmt_bg(),HEAP32[3151]=0,v=HEAP32[1940]|0,m=v+32|0,g=HEAP32[m>>2]|0,HEAP32[3152]=g,y=HEAP32[1940]|0,b=y+40|0,w=HEAP32[b>>2]|0,E=(w|0)!=1;if(E){S=HEAP32[1940]|0,x=_f__nowwriting(S)|0,N=(x|0)!=0;if(N)return C=n,k=HEAP32[C>>2]|0,L=(k|0)!=0,A=___errno_location()|0,O=HEAP32[A>>2]|0,L?(M=___errno_location()|0,HEAP32[M>>2]=O):_f__fatal(O,18589),_=___errno_location()|0,D=HEAP32[_>>2]|0,t=D,P=t,STACKTOP=z,P|0}return t=0,P=t,STACKTOP=z,P|0}function _xw_end(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),e=HEAP32[704]|0,t=(e|0)!=0,t?(s=0,_f__putbuf(0)|0,n=HEAP32[3147]|0,_fflush(n)|0):(r=_f__putbuf(10)|0,s=r),HEAP32[3159]=0,HEAP32[3158]=0,HEAP32[3160]=0,i=s,STACKTOP=u,i|0}function _xw_rev(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=0,e=HEAP32[692]|0,t=(e|0)!=0,t&&(n=_f__putbuf(10)|0,i=n,HEAP32[692]=0),HEAP32[3159]=0,HEAP32[3158]=0,HEAP32[3160]=0,r=i,STACKTOP=o,r|0}function _s_wsle(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0;N=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,m=_c_le(p)|0,x=m,g=(m|0)!=0;if(g)return y=x,t=y,v=t,STACKTOP=N,v|0;HEAP32[3142]=0,HEAP32[3145]=1,HEAP32[3144]=1,HEAP32[3153]=7,HEAP32[3163]=13,HEAP32[3164]=80,HEAP32[3157]=12,b=HEAP32[1940]|0,w=b+40|0,E=HEAP32[w>>2]|0,S=(E|0)!=1;if(S){r=HEAP32[1940]|0,i=_f__nowwriting(r)|0,s=(i|0)!=0;if(s)return o=n,u=HEAP32[o>>2]|0,a=(u|0)!=0,f=___errno_location()|0,l=HEAP32[f>>2]|0,a?(c=___errno_location()|0,HEAP32[c>>2]=l):_f__fatal(l,18601),h=___errno_location()|0,d=HEAP32[h>>2]|0,t=d,v=t,STACKTOP=N,v|0}return t=0,v=t,STACKTOP=N,v|0}function _e_wsle(){var e=0,t=0,n=0,r=0,i=0;return i=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),e=_f__putbuf(10)|0,n=e,HEAP32[3158]=0,t=n,STACKTOP=i,t|0}function _sig_die(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0;h=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),l=h,n=e,r=t,i=HEAP32[3184]|0,s=n,HEAP32[l>>2]=s,_fprintf(i,18619,l)|0,o=r,u=(o|0)!=0,u?(a=HEAP32[3184]|0,_fflush(a)|0,_f_exit(),f=HEAP32[3184]|0,_fflush(f)|0,_signal(6,0)|0,_abort()):_exit(1)}function _f_clos(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0;ut=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,T=p+4|0,H=HEAP32[T>>2]|0,V=(H|0)>=100;if(V)return t=0,et=t,STACKTOP=ut,et|0;tt=n,nt=tt+4|0,rt=HEAP32[nt>>2]|0,it=7764+(rt*48|0)|0,st=it,r=st,i=HEAP32[r>>2]|0,s=(i|0)==(0|0);if(!s){o=st,u=o+44|0,a=HEAP32[u>>2]|0,f=(a|0)==1;e:do if(f)ot=11;else{l=n,c=l+8|0,h=HEAP32[c>>2]|0,d=(h|0)!=(0|0);if(d){v=n,m=v+8|0,g=HEAP32[m>>2]|0,y=HEAP8[g>>0]|0,b=y<<24>>24;switch(b|0){case 100:case 68:ot=11;break e;default:}}w=st,E=w+40|0,S=HEAP32[E>>2]|0,x=(S|0)==1,x&&(N=n,_t_runc(N)|0),C=st,k=C+4|0,L=HEAP32[k>>2]|0,A=(L|0)!=(0|0),A&&(O=st,M=HEAP32[O>>2]|0,_fclose(M)|0,_=st,D=_+4|0,P=HEAP32[D>>2]|0,_free(P))}while(0);(ot|0)==11&&(B=st,j=HEAP32[B>>2]|0,_fclose(j)|0,F=st,I=F+4|0,q=HEAP32[I>>2]|0,R=(q|0)!=(0|0),R&&(U=st,z=U+4|0,W=HEAP32[z>>2]|0,_unlink(W)|0,X=st,$=X+4|0,J=HEAP32[$>>2]|0,_free(J))),K=st,HEAP32[K>>2]=0}return Q=st,G=Q+36|0,HEAP32[G>>2]=0,Y=st,Z=Y+4|0,HEAP32[Z>>2]=0,t=0,et=t,STACKTOP=ut,et|0}function _f_exit(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0;f=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),e=HEAP32[1905]|0,t=(e|0)!=0;if(t){STACKTOP=f;return}HEAP32[1905]=1,HEAP32[1907]=0,u=0;for(;;){n=u,r=(n|0)<100;if(!r)break;i=u,HEAP32[1906]=i,_f_clos(7620)|0,s=u,o=s+1|0,u=o}STACKTOP=f;return}function _t_runc(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0;G=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,T=p+4|0,H=HEAP32[T>>2]|0,q=7764+(H*48|0)|0,X=q,R=X,U=R+16|0,z=HEAP32[U>>2]|0,W=(z|0)!=0;if(W)return t=0,I=t,STACKTOP=G,I|0;r=X,i=HEAP32[r>>2]|0,V=i,s=_ftell(i)|0,J=s,o=V,_fseek(o,0,2)|0,u=V,a=_ftell(u)|0,$=a,f=J,l=$,c=(f|0)>=(l|0);if(!c){h=X,d=h+20|0,v=HEAP32[d>>2]|0,m=(v|0)==0;if(!m)return g=X,y=g+28|0,b=HEAP32[y>>2]|0,w=b&2,E=(w|0)!=0,E&&(S=X,x=HEAP32[S>>2]|0,_fflush(x)|0),N=X,C=HEAP32[N>>2]|0,k=_fileno(C)|0,L=J,A=_ftruncate(k,L)|0,K=A,O=X,M=HEAP32[O>>2]|0,_fseek(M,0,2)|0,_=K,D=(_|0)!=0,D?(P=n,B=HEAP32[P>>2]|0,j=(B|0)!=0,j?(F=___errno_location()|0,HEAP32[F>>2]=111):_f__fatal(111,18623),t=111,I=t,STACKTOP=G,I|0):(t=0,I=t,STACKTOP=G,I|0)}return t=0,I=t,STACKTOP=G,I|0}function _f__canseek(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0;x=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),E=x,n=e,l=n,c=_fileno(l)|0,h=_fstat(c,E)|0,p=(h|0)<0;e:do{if(!p){d=E+12|0,v=HEAP32[d>>2]|0,m=v&61440,b=(m|0)<24576;t:do{if(b){y=(m|0)<16384;if(!y)switch(m|0){case 16384:break t;default:S=11;break t}switch(m|0){case 8192:break;default:S=11;break t}s=n,o=_fileno(s)|0,u=_isatty(o)|0,a=(u|0)!=0;if(a){t=0;break e}t=1;break e}w=(m|0)<32768;if(!w)switch(m|0){case 32768:break t;default:S=11;break t}switch(m|0){case 24576:break;default:S=11;break t}t=1;break e}while(0);if((S|0)==11){t=0;break}g=E+16|0,r=HEAP32[g>>2]|0,i=r>>>0>0;if(i){t=1;break}t=0;break}t=0}while(0);return f=t,STACKTOP=x,f|0}function _f__fatal(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0;bt=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),lt=bt+56|0,ft=bt+48|0,at=bt+40|0,ut=bt+32|0,ot=bt+24|0,ht=bt+16|0,ct=bt+8|0,st=bt,n=e,r=t,d=n,N=(d|0)<100,B=n,$=(B|0)>=0,rt=N&$;do{if(!rt){et=n,tt=(et|0)>=132,nt=n,i=(nt|0)<-1,it=tt|i;if(it){s=HEAP32[3184]|0,o=r,u=n,HEAP32[st>>2]=o,gt=st+4|0,HEAP32[gt>>2]=u,_fprintf(s,18631,st)|0;break}a=n,f=(a|0)==-1,l=HEAP32[3184]|0,c=r;if(f){HEAP32[ct>>2]=c,_fprintf(l,18660,ct)|0;break}h=n,p=h-100|0,v=7632+(p<<2)|0,m=HEAP32[v>>2]|0,HEAP32[ht>>2]=c,pt=ht+4|0,HEAP32[pt>>2]=m,_fprintf(l,18677,ht)|0;break}Z=r,_perror(Z)}while(0);g=HEAP32[1940]|0,y=(g|0)!=(0|0),b=HEAP32[3184]|0,y?(w=HEAP32[1940]|0,E=w,S=E-7764|0,x=(S|0)/48&-1,HEAP32[ot>>2]=x,_fprintf(b,18685,ot)|0,T=HEAP32[3184]|0,C=HEAP32[1940]|0,k=C+4|0,L=HEAP32[k>>2]|0,A=(L|0)!=(0|0),O=A?18710:18720,M=HEAP32[1940]|0,_=M+4|0,D=HEAP32[_>>2]|0,HEAP32[ut>>2]=D,_fprintf(T,O,ut)|0):_fprintf(b,18731,at)|0,P=HEAP32[3141]|0,H=(P|0)!=(0|0),H&&(j=HEAP32[3184]|0,F=HEAP32[3141]|0,HEAP32[ft>>2]=F,_fprintf(j,18761,ft)|0),I=HEAP32[3184]|0,q=HEAP32[3142]|0,R=(q|0)!=0,U=R?18778:18786,z=HEAP32[3143]|0,W=(z|0)!=0,X=W?18794:18805,V=HEAP32[3144]|0,J=(V|0)!=0,K=J?18812:18822,Q=HEAP32[3145]|0,G=(Q|0)!=0,Y=G?18834:18843,HEAP32[lt>>2]=U,dt=lt+4|0,HEAP32[dt>>2]=X,vt=lt+8|0,HEAP32[vt>>2]=K,mt=lt+12|0,HEAP32[mt>>2]=Y,_fprintf(I,18852,lt)|0,_sig_die(18871,1),STACKTOP=bt;return}function _f_init(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0;_=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),HEAP32[3146]=1,O=7764,e=HEAP32[3184]|0,t=O,HEAP32[t>>2]=e,h=HEAP32[3184]|0,x=_f__canseek(h)|0,T=O,N=T+20|0,HEAP32[N>>2]=x,C=O,k=C+24|0,HEAP32[k>>2]=1,L=O,A=L+40|0,HEAP32[A>>2]=1,O=8004,n=HEAP32[3185]|0,r=O,HEAP32[r>>2]=n,i=HEAP32[3185]|0,s=_f__canseek(i)|0,o=O,u=o+20|0,HEAP32[u>>2]=s,a=O,f=a+24|0,HEAP32[f>>2]=1,l=O,c=l+40|0,HEAP32[c>>2]=0,O=8052,p=HEAP32[3182]|0,d=O,HEAP32[d>>2]=p,v=HEAP32[3182]|0,m=_f__canseek(v)|0,g=O,y=g+20|0,HEAP32[y>>2]=m,b=O,w=b+24|0,HEAP32[w>>2]=1,E=O,S=E+40|0,HEAP32[S>>2]=1,STACKTOP=_;return}function _f__nowwriting(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0;vt=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=n,T=p+28|0,H=HEAP32[T>>2]|0,V=H&2,rt=(V|0)!=0,ft=n;e:do{if(!rt){u=ft+4|0,a=HEAP32[u>>2]|0,f=(a|0)!=(0|0);do if(f){l=n,c=l+16|0,h=HEAP32[c>>2]|0,d=(h|0)!=0,d?y=0:(v=n,m=v+24|0,g=HEAP32[m>>2]|0,y=g),pt=y,b=n,w=b+40|0,E=HEAP32[w>>2]|0,S=(E|0)==3,x=n;if(S){N=x+4|0,C=HEAP32[N>>2]|0,k=pt,L=12672+(k<<2)|0,A=HEAP32[L>>2]|0,O=n,M=HEAP32[O>>2]|0,_=_freopen(C,A,M)|0,D=n,HEAP32[D>>2]=_,HEAP32[3147]=_,P=(_|0)!=(0|0);if(!P)break;B=n,j=B+28|0,HEAP32[j>>2]=2;break e}F=HEAP32[x>>2]|0,I=_ftell(F)|0,ht=I,q=n,R=q+4|0,U=HEAP32[R>>2]|0,z=pt,W=z|2,X=12672+(W<<2)|0,$=HEAP32[X>>2]|0,J=n,K=HEAP32[J>>2]|0,Q=_freopen(U,$,K)|0,G=n,HEAP32[G>>2]=Q,HEAP32[3147]=Q,Y=(Q|0)!=(0|0),Z=n;if(Y){tt=Z+28|0,HEAP32[tt>>2]=3,nt=n,it=HEAP32[nt>>2]|0,st=ht,_fseek(it,st,0)|0;break e}HEAP32[Z>>2]=0;break}while(0);return et=___errno_location()|0,HEAP32[et>>2]=127,t=1,at=t,STACKTOP=vt,at|0}lt=ft+28|0,ct=HEAP32[lt>>2]|0,r=ct&1,i=(r|0)!=0,i&&(s=n,o=HEAP32[s>>2]|0,_fseek(o,0,1)|0)}while(0);return ot=n,ut=ot+40|0,HEAP32[ut>>2]=1,t=0,at=t,STACKTOP=vt,at|0}function _err__fl(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0;return m=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,u=n,a=r,f=(a|0)!=0,f||(l=i,c=u,_f__fatal(l,c)),h=HEAP32[3148]|0,p=(h|0)!=(0|0),p&&(d=HEAP32[3148]|0,FUNCTION_TABLE_i[d&15]()|0),s=i,o=___errno_location()|0,HEAP32[o>>2]=s,STACKTOP=m,s|0}function _c_le(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0;V=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=HEAP32[3146]|0,T=(p|0)!=0,T||_f_init(),HEAP32[3141]=19540,H=n,q=H+4|0,R=HEAP32[q>>2]|0,U=7764+(R*48|0)|0,HEAP32[1940]=U,z=n,W=z+4|0,r=HEAP32[W>>2]|0,i=(r|0)>=100;if(!i){s=n,o=s+4|0,u=HEAP32[o>>2]|0,a=(u|0)<0;if(!a){HEAP32[3158]=0,HEAP32[3161]=0,d=n,HEAP32[3149]=d,v=HEAP32[1940]|0,m=HEAP32[v>>2]|0,g=(m|0)==(0|0);if(g){y=n,b=y+4|0,w=HEAP32[b>>2]|0,E=_fk_open(3,5,w)|0,S=(E|0)!=0;if(S)return x=n,N=HEAP32[x>>2]|0,C=(N|0)!=0,C?(k=___errno_location()|0,HEAP32[k>>2]=102):_f__fatal(102,19554),t=102,I=t,STACKTOP=V,I|0}return L=HEAP32[1940]|0,A=HEAP32[L>>2]|0,HEAP32[3147]=A,O=HEAP32[1940]|0,M=O+24|0,_=HEAP32[M>>2]|0,D=(_|0)!=0,D?(t=0,I=t,STACKTOP=V,I|0):(P=n,B=HEAP32[P>>2]|0,j=(B|0)!=0,j?(F=___errno_location()|0,HEAP32[F>>2]=103):_f__fatal(103,19554),t=103,I=t,STACKTOP=V,I|0)}}return f=n,l=HEAP32[f>>2]|0,c=(l|0)!=0,c?(h=___errno_location()|0,HEAP32[h>>2]=101):_f__fatal(101,19548),t=101,I=t,STACKTOP=V,I|0}function _l_write(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0;vt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,s=t,m=n,k=r,at=0;for(;;){F=at,K=i,it=HEAP32[K>>2]|0,st=(F|0)<(it|0);if(!st)break;ot=k;do switch(ot|0){case 11:dt=5;break;case 2:a=s,f=HEAP16[a>>1]|0,l=f<<16>>16,ft=l,dt=8;break;case 3:c=s,h=HEAP32[c>>2]|0,ft=h,dt=8;break;case 4:d=s,v=+HEAPF32[d>>2],g=v,ct=g,dt=11;break;case 5:y=s,b=+HEAPF64[y>>3],ct=b,dt=11;break;case 6:E=s,lt=E,S=lt,x=S+4|0,lt=x,T=+HEAPF32[S>>2],N=T,ct=N,C=lt,L=+HEAPF32[C>>2],A=L,pt=A,dt=14;break;case 7:O=s,ht=O,M=ht,_=M+8|0,ht=_,D=+HEAPF64[M>>3],ct=D,P=ht,H=+HEAPF64[P>>3],pt=H,dt=14;break;case 12:I=s,q=HEAP8[I>>0]|0,R=q<<24>>24,ft=R,dt=18;break;case 13:U=s,z=HEAP16[U>>1]|0,W=z<<16>>16,ft=W,dt=18;break;case 8:X=s,V=HEAP32[X>>2]|0,ft=V,dt=18;break;case 9:G=s,Y=m,_lwrt_A(G,Y);break;default:_f__fatal(117,19558),dt=5}while(0);(dt|0)==5?(dt=0,ut=s,o=HEAP8[ut>>0]|0,u=o<<24>>24,ft=u,dt=8):(dt|0)==11?(dt=0,w=ct,_lwrt_F(w)):(dt|0)==14?(dt=0,B=ct,j=pt,_lwrt_C(B,j)):(dt|0)==18&&(dt=0,$=s,J=HEAP32[$>>2]|0,Q=m,_lwrt_L(J,Q)),(dt|0)==8&&(dt=0,p=ft,_lwrt_I(p)),Z=m,et=s,tt=et+Z|0,s=tt,nt=at,rt=nt+1|0,at=rt}return STACKTOP=vt,0}function _lwrt_I(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0;N=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),E=N+4|0,x=N,t=e,n=t,p=_f__icvt(n,E,x,10)|0,S=p,d=HEAP32[3158]|0,v=HEAP32[E>>2]|0,m=d+v|0,g=HEAP32[3164]|0,y=(m|0)>=(g|0),y&&_donewrec(),b=HEAP32[3153]|0,FUNCTION_TABLE_vi[b&15](32),w=HEAP32[x>>2]|0,r=(w|0)!=0,r&&(i=HEAP32[3153]|0,FUNCTION_TABLE_vi[i&15](45));for(;;){s=S,o=HEAP8[s>>0]|0,u=o<<24>>24!=0;if(!u)break;a=HEAP32[3153]|0,f=S,l=f+1|0,S=l,c=HEAP8[f>>0]|0,h=c<<24>>24,FUNCTION_TABLE_vi[a&15](h)}STACKTOP=N;return}function _lwrt_F(e){e=+e;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0;l=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),a=l+8|0,t=e,n=HEAP32[3158]|0,r=t,i=_l_g(a,r)|0,s=n+i|0,o=HEAP32[3164]|0,u=(s|0)>=(o|0);if(!u){_l_put(a),STACKTOP=l;return}_donewrec(),_l_put(a),STACKTOP=l;return}function _lwrt_C(e,t){e=+e,t=+t;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0;Z=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Q=Z+56|0,G=Z+32|0,n=e,r=t,d=n,N=_l_g(Q,d)|0,V=N,$=Q;for(;;){B=$,R=HEAP8[B>>0]|0,U=R<<24>>24,z=(U|0)==32;if(!z)break;W=V,X=W+ -1|0,V=X,i=$,s=i+1|0,$=s}o=r,u=_l_g(G,o)|0,a=u+1|0,K=a,J=G;for(;;){f=J,l=HEAP8[f>>0]|0,c=l<<24>>24,h=(c|0)==32;if(!h)break;p=K,v=p+ -1|0,K=v,m=J,g=m+1|0,J=g}y=HEAP32[3158]|0,b=V,w=y+b|0,E=K,S=w+E|0,x=S+3|0,T=HEAP32[3164]|0,C=(x|0)>=(T|0),C&&_donewrec(),k=HEAP32[3153]|0,FUNCTION_TABLE_vi[k&15](32),L=HEAP32[3153]|0,FUNCTION_TABLE_vi[L&15](40),A=$,_l_put(A),O=HEAP32[3153]|0,FUNCTION_TABLE_vi[O&15](44),M=HEAP32[3158]|0,_=K,D=M+_|0,P=HEAP32[3164]|0,H=(D|0)>=(P|0);if(!H){I=J,_l_put(I),q=HEAP32[3153]|0,FUNCTION_TABLE_vi[q&15](41),STACKTOP=Z;return}j=HEAP32[3157]|0,FUNCTION_TABLE_i[j&15]()|0,F=HEAP32[3153]|0,FUNCTION_TABLE_vi[F&15](32),I=J,_l_put(I),q=HEAP32[3153]|0,FUNCTION_TABLE_vi[q&15](41),STACKTOP=Z;return}function _lwrt_L(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0;l=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=l+4|0,HEAP32[n>>2]=e,r=t,i=HEAP32[3158]|0,s=i+2|0,o=HEAP32[3164]|0,u=(s|0)>=(o|0),u&&_donewrec(),a=r,_wrt_L(n,2,a)|0,STACKTOP=l;return}function _lwrt_A(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0;Ot=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=t,Nt=0,d=n,N=r,B=d+N|0,Lt=B,$=HEAP32[3165]|0,it=($|0)!=0;e:do if(it){Nt=3,vt=r,xt=(vt|0)>1;if(xt){Tt=r,i=Tt-1|0,s=n,o=s+i|0,u=HEAP8[o>>0]|0,a=u<<24>>24,f=(a|0)==32;if(f){for(;;){l=r,c=l+ -1|0,r=c,h=(c|0)>1;if(!h)break;p=r,v=p-1|0,m=n,g=m+v|0,y=HEAP8[g>>0]|0,b=y<<24>>24,w=(b|0)==32;if(!w)break}E=n,S=r,x=E+S|0,Lt=x}}T=n,kt=T;for(;;){C=kt,k=Lt,L=C>>>0<k>>>0;if(!L)break e;A=kt,O=A+1|0,kt=O,M=HEAP8[A>>0]|0,_=M<<24>>24,D=(_|0)==39;if(!D)continue;P=Nt,H=P+1|0,Nt=H}}while(0);j=HEAP32[3158]|0,F=r,I=j+F|0,q=Nt,R=I+q|0,U=HEAP32[3164]|0,z=(R|0)>=(U|0),z&&_donewrec(),W=Nt,X=(W|0)==0,V=HEAP32[3158]|0,J=(V|0)!=0,Ct=X&J,Ct||(K=HEAP32[3153]|0,FUNCTION_TABLE_vi[K&15](32)),Q=Nt,G=(Q|0)!=0;if(!G){for(;;){dt=n,mt=Lt,gt=dt>>>0<mt>>>0;if(!gt)break;yt=HEAP32[3153]|0,bt=n,wt=bt+1|0,n=wt,Et=HEAP8[bt>>0]|0,St=Et<<24>>24,FUNCTION_TABLE_vi[yt&15](St)}STACKTOP=Ot;return}Y=HEAP32[3153]|0,FUNCTION_TABLE_vi[Y&15](39);for(;;){Z=n,et=Lt,tt=Z>>>0<et>>>0;if(!tt)break;nt=n,rt=HEAP8[nt>>0]|0,st=rt<<24>>24,ot=(st|0)==39,ot&&(ut=HEAP32[3153]|0,FUNCTION_TABLE_vi[ut&15](39)),at=HEAP32[3153]|0,ft=n,lt=ft+1|0,n=lt,ct=HEAP8[ft>>0]|0,ht=ct<<24>>24,FUNCTION_TABLE_vi[at&15](ht)}pt=HEAP32[3153]|0,FUNCTION_TABLE_vi[pt&15](39),STACKTOP=Ot;return}function _donewrec(){var e=0,t=0,n=0,r=0,i=0;i=STACKTOP,e=HEAP32[3158]|0,t=(e|0)!=0;if(!t)return;n=HEAP32[3157]|0,FUNCTION_TABLE_i[n&15]()|0;return}function _l_g(e,t){e|=0,t=+t;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0;lt=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),at=lt+8|0,n=e,r=t,d=n,st=d,N=st,B=N+1|0,st=B,HEAP8[N>>0]=32,$=r,tt=$<0,nt=st,rt=nt+1|0,st=rt,tt?(HEAP8[nt>>0]=45,it=r,i=-it,r=i):HEAP8[nt>>0]=32,s=r,o=s==0,u=st;e:do if(o)a=u+1|0,st=a,HEAP8[u>>0]=48,f=st,l=f+1|0,st=l,HEAP8[f>>0]=46,c=st,HEAP8[c>>0]=0;else{h=r,HEAPF64[at>>3]=h,_sprintf(u,19578,at)|0,p=st,v=HEAP8[p>>0]|0,m=v<<24>>24;switch(m|0){case 48:for(;;){g=st,y=g+1|0,b=HEAP8[y>>0]|0,w=st,HEAP8[w>>0]=b,E=b<<24>>24!=0;if(!E)break e;S=st,x=S+1|0,st=x}break;case 78:case 110:case 73:case 105:for(;;){T=st,C=T+1|0,st=C,k=HEAP8[C>>0]|0,L=k<<24>>24!=0;if(!L)break}break;default:t:for(;;){A=st,O=HEAP8[A>>0]|0,M=O<<24>>24;switch(M|0){case 0:ft=11;break t;case 46:ft=12;break t;case 69:ft=13;break t;default:}J=st,K=J+1|0,st=K}if((ft|0)==11){_=st,D=_+1|0,st=D,HEAP8[_>>0]=46,P=st,HEAP8[P>>0]=0;break e}if((ft|0)==12)for(;;){ft=0,H=st,j=H+1|0,st=j,F=HEAP8[j>>0]|0,I=F<<24>>24!=0;if(!I)break;ft=12}else if((ft|0)==13){ut=46,ot=69;for(;;){q=ut,R=st,HEAP8[R>>0]=q,U=q<<24>>24!=0;if(!U)break e;z=ot,ut=z,W=st,X=W+1|0,st=X,V=HEAP8[X>>0]|0,ot=V}}}}while(0);return Q=st,G=n,Y=Q,Z=G,et=Y-Z|0,STACKTOP=lt,et|0}function _l_put(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0;p=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=e,n=HEAP32[3153]|0,c=n;for(;;){r=t,i=r+1|0,t=i,s=HEAP8[r>>0]|0,o=s<<24>>24,l=o,u=(o|0)!=0;if(!u)break;a=c,f=l,FUNCTION_TABLE_vi[a&15](f)}STACKTOP=p;return}function _f__putbuf(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0;U=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=e,n=HEAP32[3160]|0,p=HEAP32[3158]|0,T=(n|0)>(p|0),T&&(_=HEAP32[3160]|0,HEAP32[3158]=_),D=HEAP32[3158]|0,P=D+1|0,F=P,H=F,B=HEAP32[3172]|0,j=(H|0)>=(B|0),j&&(r=F,i=HEAP32[3158]|0,_f__bufadj(r,i)),s=HEAP32[3173]|0,I=s,o=I,u=HEAP32[3158]|0,a=o+u|0,q=a,f=t,l=(f|0)!=0,l&&(c=t,h=c&255,d=q,v=d+1|0,q=v,HEAP8[d>>0]=h),m=q,HEAP8[m>>0]=0;for(;;){g=I,y=HEAP32[3147]|0,_fputs(g,y)|0,b=I,w=_strlen(b)|0,E=I,S=E+w|0,I=S,x=I,N=q,C=x>>>0>=N>>>0;if(C)break;k=I,L=k+1|0,I=L,A=HEAP8[k>>0]|0,O=A<<24>>24,M=HEAP32[3147]|0,_putc(O,M)|0}return STACKTOP=U,0}function _x_putc(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0;d=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=e,n=HEAP32[3158]|0,s=HEAP32[3172]|0,o=(n|0)>=(s|0),o&&(u=HEAP32[3158]|0,a=HEAP32[3172]|0,_f__bufadj(u,a)),f=t,l=f&255,c=HEAP32[3158]|0,h=c+1|0,HEAP32[3158]=h,r=HEAP32[3173]|0,i=r+c|0,HEAP8[i>>0]=l,STACKTOP=d;return}function _f_open(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0;to=STACKTOP,STACKTOP=STACKTOP+304|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Ys=to,Vs=to+48|0,Zs=to+16|0,$s=to+4|0,n=e,HEAP32[3145]=1,on=n,fi=on+4|0,Di=HEAP32[fi>>2]|0,Wi=(Di|0)>=100;if(!Wi){ts=n,hs=ts+4|0,xs=HEAP32[hs>>2]|0,Ps=(xs|0)<0;if(!Ps){$=HEAP32[3146]|0,it=($|0)!=0,it||_f_init(),vt=n,Ct=vt+4|0,jt=HEAP32[Ct>>2]|0,Jt=7764+(jt*48|0)|0,Xs=Jt,HEAP32[1940]=Jt,un=Xs,yn=HEAP32[un>>2]|0,An=(yn|0)!=(0|0);e:do if(An){qn=n,Gn=qn+8|0,ar=HEAP32[Gn>>2]|0,br=(ar|0)==(0|0);do if(!br){Ri=n,Ui=Ri+8|0,zi=HEAP32[Ui>>2]|0,Xi=n,Vi=Xi+12|0,$i=HEAP32[Vi>>2]|0,_g_char(zi,$i,Vs),Ji=_f__inode(Vs,$s)|0,Ki=Xs,Qi=Ki+8|0,Gi=HEAP32[Qi>>2]|0,Yi=(Ji|0)==(Gi|0);if(Yi){Zi=HEAP32[$s>>2]|0,es=Xs,ns=es+12|0,rs=HEAP32[ns>>2]|0,is=(Zi|0)==(rs|0);if(is)break}ss=n,os=ss+4|0,us=HEAP32[os>>2]|0,as=Zs+4|0,HEAP32[as>>2]=us,fs=Zs+8|0,HEAP32[fs>>2]=0,ls=n,cs=HEAP32[ls>>2]|0,HEAP32[Zs>>2]=cs,ps=_f_clos(Zs)|0,Js=ps,ds=(ps|0)!=0;if(!ds)break e;return vs=Js,t=vs,Ti=t,STACKTOP=to,Ti|0}while(0);return Or=n,Rr=Or+32|0,Yr=HEAP32[Rr>>2]|0,li=(Yr|0)!=(0|0),li&&(Ei=n,Ni=Ei+32|0,Ci=HEAP32[Ni>>2]|0,ki=HEAP8[Ci>>0]|0,Li=ki<<24>>24,Ai=(Li|0)==122,Ai?Fi=1:(Oi=n,Mi=Oi+32|0,_i=HEAP32[Mi>>2]|0,Pi=HEAP8[_i>>0]|0,Hi=Pi<<24>>24,Bi=(Hi|0)==90,Fi=Bi),ji=Fi&1,Ii=Xs,qi=Ii+32|0,HEAP32[qi>>2]=ji),t=0,Ti=t,STACKTOP=to,Ti|0}while(0);ms=n,gs=ms+28|0,ys=HEAP32[gs>>2]|0,bs=Xs,ws=bs+16|0,HEAP32[ws>>2]=ys,Es=n,Ss=Es+32|0,Ts=HEAP32[Ss>>2]|0,Ns=(Ts|0)!=(0|0),Ns?(Cs=n,ks=Cs+32|0,Ls=HEAP32[ks>>2]|0,As=HEAP8[Ls>>0]|0,Os=As<<24>>24,Ms=(Os|0)==122,Ms?qs=1:(_s=n,Ds=_s+32|0,Hs=HEAP32[Ds>>2]|0,Bs=HEAP8[Hs>>0]|0,js=Bs<<24>>24,Fs=(js|0)==90,qs=Fs)):qs=0,Is=qs&1,Rs=Xs,Us=Rs+32|0,HEAP32[Us>>2]=Is,zs=n,Ws=zs+24|0,i=HEAP32[Ws>>2]|0,s=(i|0)==(0|0);do{if(s){o=Xs,u=o+16|0,a=HEAP32[u>>2]|0,f=(a|0)>0,l=Xs,c=l+24|0;if(f){HEAP32[c>>2]=0;break}HEAP32[c>>2]=1;break}h=n,p=h+24|0,v=HEAP32[p>>2]|0,m=HEAP8[v>>0]|0,g=m<<24>>24,y=(g|0)==102;if(!y){b=n,w=b+24|0,E=HEAP32[w>>2]|0,S=HEAP8[E>>0]|0,x=S<<24>>24,T=(x|0)==70;if(!T){L=Xs,A=L+24|0,HEAP32[A>>2]=0;break}}C=Xs,k=C+24|0,HEAP32[k>>2]=1}while(0);O=Xs,M=O+24|0,_=HEAP32[M>>2]|0,Gs=_,D=n,P=D+8|0,H=HEAP32[P>>2]|0,j=(H|0)!=(0|0);if(j){F=n,I=F+8|0,q=HEAP32[I>>2]|0,R=n,U=R+12|0,z=HEAP32[U>>2]|0,_g_char(q,z,Vs),W=HEAP8[Vs>>0]|0,X=W<<24>>24!=0;if(!X)return V=n,J=HEAP32[V>>2]|0,K=(J|0)!=0,K?(Q=___errno_location()|0,HEAP32[Q>>2]=107):(G=n,_opn_err(107,19583,G)),t=107,Ti=t,STACKTOP=to,Ti|0}else Y=n,Z=Y+4|0,et=HEAP32[Z>>2]|0,HEAP32[Ys>>2]=et,_sprintf(Vs,19588,Ys)|0;tt=Xs,nt=tt+44|0,HEAP32[nt>>2]=0,rt=Xs,st=rt+36|0,HEAP32[st>>2]=0,ot=Xs,ut=ot+40|0,HEAP32[ut>>2]=0,at=Xs,HEAP32[at>>2]=0,ft=Xs,lt=ft+28|0,HEAP32[lt>>2]=3,ct=n,ht=ct+16|0,pt=HEAP32[ht>>2]|0,dt=(pt|0)!=(0|0),dt?(mt=n,gt=mt+16|0,yt=HEAP32[gt>>2]|0,bt=HEAP8[yt>>0]|0,wt=bt<<24>>24,Et=wt):Et=117;switch(Et|0){case 79:case 111:St=_access(Vs,0)|0,xt=(St|0)!=0;if(xt)return Tt=n,Nt=HEAP32[Tt>>2]|0,kt=(Nt|0)!=0,Lt=___errno_location()|0,At=HEAP32[Lt>>2]|0,kt?(Ot=___errno_location()|0,HEAP32[Ot>>2]=At):(Mt=n,_opn_err(At,19583,Mt)),_t=___errno_location()|0,Dt=HEAP32[_t>>2]|0,t=Dt,Ti=t,STACKTOP=to,Ti|0;break;case 83:case 115:return Pt=Xs,Ht=Pt+44|0,HEAP32[Ht>>2]=1,Bt=_tmpfile()|0,Ft=Xs,HEAP32[Ft>>2]=Bt,It=(Bt|0)!=(0|0),It?(Qt=Xs,Gt=Qt+4|0,HEAP32[Gt>>2]=0,Yt=Xs,Zt=Yt+12|0,HEAP32[Zt>>2]=-1,en=Xs,tn=en+8|0,HEAP32[tn>>2]=-1,nn=Xs,rn=nn+20|0,HEAP32[rn>>2]=1,t=0,Ti=t,STACKTOP=to,Ti|0):(qt=n,Rt=HEAP32[qt>>2]|0,Ut=(Rt|0)!=0,zt=___errno_location()|0,Wt=HEAP32[zt>>2]|0,Ut?(Xt=___errno_location()|0,HEAP32[Xt>>2]=Wt):(Vt=n,_opn_err(Wt,19583,Vt)),$t=___errno_location()|0,Kt=HEAP32[$t>>2]|0,t=Kt,Ti=t,STACKTOP=to,Ti|0);case 78:case 110:sn=_access(Vs,0)|0,an=(sn|0)!=0;if(!an)return fn=n,ln=HEAP32[fn>>2]|0,cn=(ln|0)!=0,cn?(hn=___errno_location()|0,HEAP32[hn>>2]=128):(pn=n,_opn_err(128,19583,pn)),t=128,Ti=t,STACKTOP=to,Ti|0;eo=57;break;case 82:case 114:eo=57;break;default:}(eo|0)==57&&(dn=HEAP32[3168]|0,vn=_fopen(Vs,dn)|0,Qs=vn,mn=(vn|0)!=(0|0),mn&&(gn=Qs,_fclose(gn)|0)),bn=_strlen(Vs)|0,wn=bn+1|0,En=_malloc(wn)|0,Sn=Xs,xn=Sn+4|0,HEAP32[xn>>2]=En,Tn=Xs,Nn=Tn+4|0,Cn=HEAP32[Nn>>2]|0,kn=(Cn|0)==(0|0);if(kn)return Ln=n,On=HEAP32[Ln>>2]|0,Mn=(On|0)!=0,Mn?(_n=___errno_location()|0,HEAP32[_n>>2]=113):(Dn=n,_opn_err(113,19597,Dn)),t=113,Ti=t,STACKTOP=to,Ti|0;Pn=Xs,Hn=Pn+4|0,Bn=HEAP32[Hn>>2]|0,_strcpy(Bn,Vs)|0,jn=n,Fn=jn+20|0,In=HEAP32[Fn>>2]|0,Ks=In,Rn=(In|0)!=(0|0),Rn&&(Un=Xs,zn=Un+16|0,Wn=HEAP32[zn>>2]|0,Xn=(Wn|0)!=0,Xn&&(Gs=0)),Vn=Gs,$n=Vn|2,Jn=12672+($n<<2)|0,Kn=HEAP32[Jn>>2]|0,Qn=_fopen(Vs,Kn)|0,Qs=Qn,Yn=(Qn|0)!=(0|0);do if(!Yn){Zn=Gs,er=12664+(Zn<<2)|0,tr=HEAP32[er>>2]|0,nr=_fopen(Vs,tr)|0,Qs=nr,rr=(nr|0)!=(0|0);if(rr){ir=Xs,sr=ir+28|0,HEAP32[sr>>2]=1;break}or=Gs,ur=12672+(or<<2)|0,fr=HEAP32[ur>>2]|0,lr=_fopen(Vs,fr)|0,Qs=lr,cr=(lr|0)!=(0|0);if(cr){hr=Xs,pr=hr+40|0,HEAP32[pr>>2]=1,dr=Xs,vr=dr+28|0,HEAP32[vr>>2]=2;break}return mr=n,gr=HEAP32[mr>>2]|0,yr=(gr|0)!=0,wr=___errno_location()|0,Er=HEAP32[wr>>2]|0,yr?(Sr=___errno_location()|0,HEAP32[Sr>>2]=Er):_f__fatal(Er,19583),xr=___errno_location()|0,Tr=HEAP32[xr>>2]|0,t=Tr,Ti=t,STACKTOP=to,Ti|0}while(0);Nr=Qs,Cr=Xs,HEAP32[Cr>>2]=Nr,kr=_f__canseek(Nr)|0,Lr=Xs,Ar=Lr+20|0,HEAP32[Ar>>2]=kr,Mr=Xs,_r=Mr+12|0,Dr=_f__inode(Vs,_r)|0,Pr=Xs,Hr=Pr+8|0,HEAP32[Hr>>2]=Dr,Br=(Dr|0)==-1;if(Br)return jr=n,Fr=HEAP32[jr>>2]|0,Ir=(Fr|0)!=0,Ir?(qr=___errno_location()|0,HEAP32[qr>>2]=108):(Ur=n,_opn_err(108,19583,Ur)),t=108,Ti=t,STACKTOP=to,Ti|0;zr=Xs,Wr=zr+20|0,Xr=HEAP32[Wr>>2]|0,Vr=(Xr|0)!=0;do if(Vr){$r=n,Jr=$r+28|0,Kr=HEAP32[Jr>>2]|0,Qr=(Kr|0)!=0;if(Qr){Gr=Xs,Zr=HEAP32[Gr>>2]|0,_rewind(Zr);break}ei=n,ti=ei+20|0,ni=HEAP32[ti>>2]|0,Ks=ni,ri=(ni|0)!=(0|0);if(ri){ii=Ks,si=HEAP8[ii>>0]|0,oi=si<<24>>24,ui=(oi|0)==97;if(!ui){ai=Ks,ci=HEAP8[ai>>0]|0,hi=ci<<24>>24,pi=(hi|0)==65;if(!pi)break}di=Xs,vi=HEAP32[di>>2]|0,mi=_fseek(vi,0,2)|0,gi=(mi|0)!=0;if(gi)return yi=n,bi=HEAP32[yi>>2]|0,wi=(bi|0)!=0,wi?(Si=___errno_location()|0,HEAP32[Si>>2]=129):(xi=n,_opn_err(129,19583,xi)),t=129,Ti=t,STACKTOP=to,Ti|0}}while(0);return t=0,Ti=t,STACKTOP=to,Ti|0}}return r=n,d=HEAP32[r>>2]|0,N=(d|0)!=0,N?(B=___errno_location()|0,HEAP32[B>>2]=101):_f__fatal(101,19583),t=101,Ti=t,STACKTOP=to,Ti|0}function _fk_open(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0;return M=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),A=M,L=M+52|0,k=M+4|0,r=e,i=t,v=n,w=v,HEAP32[A>>2]=w,_sprintf(L,19588,A)|0,HEAP32[k>>2]=1,E=v,S=k+4|0,HEAP32[S>>2]=E,x=k+8|0,HEAP32[x>>2]=L,T=_strlen(L)|0,N=k+12|0,HEAP32[N>>2]=T,C=k+16|0,HEAP32[C>>2]=0,s=r,o=(s|0)==3,u=o?19606:19608,a=k+20|0,HEAP32[a>>2]=u,f=i,l=(f|0)==5,c=l?19610:19612,h=k+24|0,HEAP32[h>>2]=c,p=r,d=(p|0)==4,m=d?1:0,g=k+28|0,HEAP32[g>>2]=m,y=k+32|0,HEAP32[y>>2]=0,b=_f_open(k)|0,STACKTOP=M,b|0}function _f__bufadj(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0;q=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=t,d=HEAP32[3173]|0,N=(d|0)==(19631|0),N&&(HEAP32[3172]=1024);for(;;){L=HEAP32[3172]|0,A=n,O=(L|0)<=(A|0),M=HEAP32[3172]|0;if(!O)break;_=M<<1,HEAP32[3172]=_}P=M,D=P,i=HEAP32
[3172]|0,s=(D|0)!=(i|0),s?I=7:(o=P,u=_malloc(o)|0,H=u,a=(u|0)!=(0|0),a||(I=7)),(I|0)==7&&_f__fatal(113,20031),f=H,B=f,l=HEAP32[3173]|0,j=l,c=j,h=r,p=c+h|0,F=p;for(;;){v=j,m=F,g=v>>>0<m>>>0;if(!g)break;y=j,b=y+1|0,j=b,w=HEAP8[y>>0]|0,E=B,S=E+1|0,B=S,HEAP8[E>>0]=w}x=HEAP32[3173]|0,T=(x|0)!=(19631|0);if(!T){k=H,HEAP32[3173]=k,STACKTOP=q;return}C=HEAP32[3173]|0,_free(C),k=H,HEAP32[3173]=k,STACKTOP=q;return}function _opn_err(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0;O=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,v=n,S=v,x=S+8|0,T=HEAP32[x>>2]|0,N=(T|0)!=(0|0);if(!N){w=r,E=i,_f__fatal(w,E),STACKTOP=O;return}C=v,k=C+12|0,L=HEAP32[k>>2]|0,s=HEAP32[3172]|0,o=(L|0)>=(s|0),o&&(u=v,a=u+12|0,f=HEAP32[a>>2]|0,_f__bufadj(f,0)),l=v,c=l+8|0,h=HEAP32[c>>2]|0,p=v,d=p+12|0,m=HEAP32[d>>2]|0,g=HEAP32[3173]|0,y=HEAP32[1940]|0,b=y+4|0,HEAP32[b>>2]=g,_g_char(h,m,g),w=r,E=i,_f__fatal(w,E),STACKTOP=O;return}function _g_char(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0;P=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,v=n,T=r,N=i,C=T+N|0,M=C,k=v,L=i,A=k+L|0,_=A;for(;;){O=M,s=r,o=O>>>0<=s>>>0;if(o){D=3;break}a=M,f=a+ -1|0,M=f,l=HEAP8[f>>0]|0,c=l<<24>>24,h=(c|0)!=32,p=_,d=p+ -1|0,_=d;if(h)break}if((D|0)==3){u=v,HEAP8[u>>0]=0,STACKTOP=P;return}HEAP8[p>>0]=0;for(;;){m=M,g=HEAP8[m>>0]|0,y=_,b=y+ -1|0,_=b,HEAP8[y>>0]=g,w=M,E=w+ -1|0,M=E,S=r,x=w>>>0>S>>>0;if(!x)break}STACKTOP=P;return}function _f__inode(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0;return v=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),p=v,r=e,s=t,o=r,u=_stat(o,p)|0,a=(u|0)<0,a?(n=-1,i=n,STACKTOP=v,i|0):(f=HEAP32[p>>2]|0,l=s,HEAP32[l>>2]=f,c=p+72|0,h=HEAP32[c>>2]|0,n=h,i=n,STACKTOP=v,i|0)}function _wrt_L(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0;O=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,v=n,w=v,E=4==(w|0);do{if(!E){T=v,N=(T|0)==1,C=r;if(N){s=HEAP8[C>>0]|0,o=s<<24>>24,L=o;break}u=HEAP16[C>>1]|0,a=u<<16>>16,L=a;break}S=r,x=HEAP32[S>>2]|0,L=x}while(0);k=0;for(;;){f=k,l=i,c=l-1|0,h=(f|0)<(c|0);if(!h)break;p=HEAP32[3153]|0,FUNCTION_TABLE_vi[p&15](32),d=k,m=d+1|0,k=m}return g=L,y=(g|0)!=0,b=HEAP32[3153]|0,y?(FUNCTION_TABLE_vi[b&15](84),STACKTOP=O,0):(FUNCTION_TABLE_vi[b&15](70),STACKTOP=O,0)}function _w_ed(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0;ln=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),an=ln,i=e,C=t,j=n,J=HEAP32[3159]|0,st=(J|0)!=0,st?(mt=_mv_cur()|0,un=mt,kt=(mt|0)!=0,kt?(Ft=un,r=Ft):fn=4):fn=4;e:do if((fn|0)==4){Kt=i,s=HEAP32[Kt>>2]|0;do switch(s|0){case 7:break;case 10:L=C,A=i,O=A+4|0,M=HEAP32[O>>2]|0,_=i,D=_+8|0,P=HEAP32[D>>2]|0,H=j,B=_wrt_IM(L,M,P,H,10)|0,r=B;break e;case 32:F=C,I=i,q=I+4|0,R=HEAP32[q>>2]|0,U=j,z=_wrt_I(F,R,U,8)|0,r=z;break e;case 34:W=C,X=i,V=X+4|0,$=HEAP32[V>>2]|0,K=i,Q=K+8|0,G=HEAP32[Q>>2]|0,Y=j,Z=_wrt_IM(W,$,G,Y,8)|0,r=Z;break e;case 29:et=C,tt=i,nt=tt+4|0,rt=HEAP32[nt>>2]|0,it=j,ot=_wrt_L(et,rt,it)|0,r=ot;break e;case 30:ut=C,at=j,ft=_wrt_A(ut,at)|0,r=ft;break e;case 31:lt=C,ct=i,ht=ct+4|0,pt=HEAP32[ht>>2]|0,dt=j,vt=_wrt_AW(lt,pt,dt)|0,r=vt;break e;case 25:case 24:case 26:gt=C,yt=i,bt=yt+4|0,wt=HEAP32[bt>>2]|0,Et=i,St=Et+8|0,xt=HEAP32[St>>2]|0,Tt=i,Nt=Tt+8|0,Ct=Nt+4|0,Lt=HEAP32[Ct>>2]|0,At=j,Ot=_wrt_E(gt,wt,xt,Lt,At)|0,r=Ot;break e;case 28:case 27:Mt=C,_t=i,Dt=_t+4|0,Pt=HEAP32[Dt>>2]|0,Ht=i,Bt=Ht+8|0,jt=HEAP32[Bt>>2]|0,It=i,qt=It+8|0,Rt=qt+4|0,Ut=HEAP32[Rt>>2]|0,zt=j,Wt=_wrt_G(Mt,Pt,jt,Ut,zt)|0,r=Wt;break e;case 23:Xt=C,Vt=i,$t=Vt+4|0,Jt=HEAP32[$t>>2]|0,Qt=i,Gt=Qt+8|0,Yt=HEAP32[Gt>>2]|0,Zt=j,en=_wrt_F(Xt,Jt,Yt,Zt)|0,r=en;break e;case 35:tn=C,nn=i,rn=nn+4|0,sn=HEAP32[rn>>2]|0,on=j,o=_wrt_Z(tn,sn,0,on)|0,r=o;break e;case 36:u=C,a=i,f=a+4|0,l=HEAP32[f>>2]|0,c=i,h=c+8|0,p=HEAP32[h>>2]|0,d=j,v=_wrt_Z(u,l,p,d)|0,r=v;break e;default:m=HEAP32[3184]|0,y=i,b=HEAP32[y>>2]|0,HEAP32[an>>2]=b,_fprintf(m,20046,an)|0,w=HEAP32[3141]|0,_sig_die(w,1)}while(0);E=C,S=i,x=S+4|0,T=HEAP32[x>>2]|0,N=j,k=_wrt_I(E,T,N,10)|0,r=k}while(0);return g=r,STACKTOP=ln,g|0}function _w_ned(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0;$=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),X=$,n=e,p=n,T=HEAP32[p>>2]|0;switch(T|0){case 5:break;case 15:return r=n,i=r+4|0,s=HEAP32[i>>2]|0,o=HEAP32[3158]|0,u=s-o|0,a=u-1|0,HEAP32[3159]=a,t=1,I=t,STACKTOP=$,I|0;case 13:return f=n,l=f+4|0,c=HEAP32[l>>2]|0,h=HEAP32[3159]|0,d=h-c|0,HEAP32[3159]=d,v=HEAP32[3159]|0,m=HEAP32[3158]|0,g=0-m|0,y=(v|0)<(g|0),y&&(b=HEAP32[3158]|0,w=0-b|0,HEAP32[3159]=w),t=1,I=t,STACKTOP=$,I|0;case 4:case 14:return E=n,S=E+4|0,x=HEAP32[S>>2]|0,N=HEAP32[3159]|0,C=N+x|0,HEAP32[3159]=C,t=1,I=t,STACKTOP=$,I|0;case 11:return k=n,L=k+8|0,A=HEAP32[L>>2]|0,O=_wrt_AP(A)|0,t=O,I=t,STACKTOP=$,I|0;case 12:return M=n,_=M+4|0,D=HEAP32[_>>2]|0,P=n,B=P+8|0,j=HEAP32[B>>2]|0,F=_wrt_H(D,j)|0,t=F,I=t,STACKTOP=$,I|0;default:H=HEAP32[3184]|0,q=n,R=HEAP32[q>>2]|0,HEAP32[X>>2]=R,_fprintf(H,20073,X)|0,U=HEAP32[3141]|0,_sig_die(U,1)}return z=HEAP32[3157]|0,W=FUNCTION_TABLE_i[z&15]()|0,t=W,I=t,STACKTOP=$,I|0}function _mv_cur(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0;cn=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=HEAP32[3159]|0,fn=t,HEAP32[3159]=0,L=HEAP32[3145]|0,I=(L|0)==0,Q=fn;if(!I){Ot=(Q|0)>0;e:do if(Ot){Mt=HEAP32[3160]|0,_t=HEAP32[3158]|0,Dt=(Mt|0)<=(_t|0);if(Dt)for(;;){Pt=fn,Ht=(Pt|0)>0;if(!Ht)break e;Bt=HEAP32[3153]|0,FUNCTION_TABLE_vi[Bt&15](32),jt=fn,Ft=jt+ -1|0,fn=Ft}It=HEAP32[3160]|0,Rt=HEAP32[3158]|0,Ut=fn,zt=Rt+Ut|0,Wt=(It|0)<=(zt|0);if(!Wt){rn=fn,sn=HEAP32[3158]|0,on=sn+rn|0,HEAP32[3158]=on;break}Xt=HEAP32[3160]|0,Vt=HEAP32[3158]|0,$t=Xt-Vt|0,Jt=fn,Kt=Jt-$t|0,fn=Kt,Qt=HEAP32[3160]|0,HEAP32[3158]=Qt;for(;;){Yt=fn,Zt=(Yt|0)>0;if(!Zt)break e;en=HEAP32[3153]|0,FUNCTION_TABLE_vi[en&15](32),tn=fn,nn=tn+ -1|0,fn=nn}}else{un=fn,an=(un|0)<0;if(an){r=fn,i=HEAP32[3158]|0,s=r+i|0,o=(s|0)<0;if(!o){c=HEAP32[3160]|0,h=HEAP32[3158]|0,d=(c|0)<(h|0),d&&(v=HEAP32[3158]|0,HEAP32[3160]=v),m=fn,g=HEAP32[3158]|0,y=g+m|0,HEAP32[3158]=y;break}return u=HEAP32[3149]|0,a=HEAP32[u>>2]|0,f=(a|0)!=0,f?(l=___errno_location()|0,HEAP32[l>>2]=110):_f__fatal(110,20118),e=110,b=e,STACKTOP=cn,b|0}}while(0);return e=0,b=e,STACKTOP=cn,b|0}ut=(Q|0)<0;t:do if(ut){yt=HEAP32[3160]|0,At=HEAP32[3158]|0,qt=(yt|0)<(At|0),qt&&(Gt=HEAP32[3158]|0,HEAP32[3160]=Gt),n=fn,p=HEAP32[3158]|0,w=p+n|0,HEAP32[3158]=w,E=fn,S=HEAP32[3162]|0,x=S+E|0,HEAP32[3162]=x,T=HEAP32[3158]|0,N=(T|0)<0;if(N)return C=HEAP32[3149]|0,k=HEAP32[C>>2]|0,A=(k|0)!=0,A?(O=___errno_location()|0,HEAP32[O>>2]=110):_f__fatal(110,20118),e=110,b=e,STACKTOP=cn,b|0}else{M=fn,_=(M|0)>0;if(_){D=HEAP32[3158]|0,P=fn,H=D+P|0,B=HEAP32[3150]|0,j=B+16|0,F=HEAP32[j>>2]|0,q=(H|0)>=(F|0);if(q)return R=HEAP32[3149]|0,U=HEAP32[R>>2]|0,z=(U|0)!=0,z?(W=___errno_location()|0,HEAP32[W>>2]=110):_f__fatal(110,20127),e=110,b=e,STACKTOP=cn,b|0;X=HEAP32[3160]|0,V=HEAP32[3158]|0,$=(X|0)<=(V|0);if($)for(;;){J=fn,K=(J|0)>0;if(!K)break t;G=HEAP32[3153]|0,FUNCTION_TABLE_vi[G&15](32),Y=fn,Z=Y+ -1|0,fn=Z}et=HEAP32[3160]|0,tt=HEAP32[3158]|0,nt=fn,rt=tt+nt|0,it=(et|0)<=(rt|0);if(!it){xt=fn,Tt=HEAP32[3162]|0,Nt=Tt+xt|0,HEAP32[3162]=Nt,Ct=fn,kt=HEAP32[3158]|0,Lt=kt+Ct|0,HEAP32[3158]=Lt;break}st=HEAP32[3160]|0,ot=HEAP32[3158]|0,at=st-ot|0,ft=fn,lt=ft-at|0,fn=lt,ct=HEAP32[3160]|0,ht=HEAP32[3158]|0,pt=ct-ht|0,dt=HEAP32[3162]|0,vt=dt+pt|0,HEAP32[3162]=vt,mt=HEAP32[3160]|0,HEAP32[3158]=mt;for(;;){gt=fn,bt=(gt|0)>0;if(!bt)break t;wt=HEAP32[3153]|0,FUNCTION_TABLE_vi[wt&15](32),Et=fn,St=Et+ -1|0,fn=St}}}while(0);return e=0,b=e,STACKTOP=cn,b|0}function _wrt_I(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0;vt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),ft=vt+20|0,ct=vt+16|0,i=e,s=t,m=n,k=r,F=m,K=(F|0)==4;do{if(!K){st=m,ot=(st|0)==1,o=i;if(ot){u=HEAP8[o>>0]|0,a=u<<24>>24,pt=a;break}f=HEAP16[o>>1]|0,l=f<<16>>16,pt=l;break}rt=i,it=HEAP32[rt>>2]|0,pt=it}while(0);c=pt,h=k,p=_f__icvt(c,ft,ct,h)|0,ut=p,d=s,v=HEAP32[ft>>2]|0,g=d-v|0,ht=g,y=HEAP32[ct>>2]|0,b=(y|0)!=0,w=HEAP32[3151]|0,E=(w|0)!=0,lt=b|E,lt&&(S=ht,x=S+ -1|0,ht=x),T=ht,N=(T|0)<0,at=0;if(N){for(;;){C=at,L=s,A=(C|0)<(L|0);if(!A)break;O=HEAP32[3153]|0,FUNCTION_TABLE_vi[O&15](42),M=at,_=M+1|0,at=_}return STACKTOP=vt,0}for(;;){D=at,P=ht,H=(D|0)<(P|0);if(!H)break;B=HEAP32[3153]|0,FUNCTION_TABLE_vi[B&15](32),j=at,I=j+1|0,at=I}q=HEAP32[ct>>2]|0,R=(q|0)!=0,R?(U=HEAP32[3153]|0,FUNCTION_TABLE_vi[U&15](45)):(z=HEAP32[3151]|0,W=(z|0)!=0,W&&(X=HEAP32[3153]|0,FUNCTION_TABLE_vi[X&15](43))),at=0;for(;;){V=at,$=HEAP32[ft>>2]|0,J=(V|0)<($|0);if(!J)break;Q=HEAP32[3153]|0,G=ut,Y=G+1|0,ut=Y,Z=HEAP8[G>>0]|0,et=Z<<24>>24,FUNCTION_TABLE_vi[Q&15](et),tt=at,nt=tt+1|0,at=nt}return STACKTOP=vt,0}function _wrt_IM(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0;en=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Vt=en+24|0,Kt=en+20|0,o=e,g=t,L=n,I=r,Q=i,ut=I,yt=4==(ut|0);do{if(!yt){u=I,a=(u|0)==1,f=o;if(a){l=HEAP8[f>>0]|0,c=l<<24>>24,Gt=c;break}h=HEAP16[f>>1]|0,p=h<<16>>16,Gt=p;break}At=o,qt=HEAP32[At>>2]|0,Gt=qt}while(0);d=Gt,v=Q,m=_f__icvt(d,Vt,Kt,v)|0,Wt=m,y=HEAP32[Kt>>2]|0,b=(y|0)!=0,w=HEAP32[3151]|0,E=(w|0)!=0,$t=b|E,$t?Yt=1:Yt=0,S=HEAP32[Vt>>2]|0,x=Yt,T=S+x|0,N=g,C=(T|0)>(N|0);if(!C){k=L,A=Yt,O=k+A|0,M=g,_=(O|0)>(M|0);if(!_){q=Gt,R=(q|0)==0,U=L,z=(U|0)==0,Jt=R&z;if(Jt){Xt=0;for(;;){W=Xt,X=g,V=(W|0)<(X|0);if(!V)break;$=HEAP32[3153]|0,FUNCTION_TABLE_vi[$&15](32),J=Xt,K=J+1|0,Xt=K}return s=0,zt=s,STACKTOP=en,zt|0}G=HEAP32[Vt>>2]|0,Y=L,Z=(G|0)>=(Y|0),et=g,Z?(tt=HEAP32[Vt>>2]|0,nt=et-tt|0,rt=Yt,it=nt-rt|0,Qt=it):(st=L,ot=et-st|0,at=Yt,ft=ot-at|0,Qt=ft),Xt=0;for(;;){lt=Xt,ct=Qt,ht=(lt|0)<(ct|0);if(!ht)break;pt=HEAP32[3153]|0,FUNCTION_TABLE_vi[pt&15](32),dt=Xt,vt=dt+1|0,Xt=vt}mt=HEAP32[Kt>>2]|0,gt=(mt|0)!=0,gt?(bt=HEAP32[3153]|0,FUNCTION_TABLE_vi[bt&15](45)):(wt=HEAP32[3151]|0,Et=(wt|0)!=0,Et&&(St=HEAP32[3153]|0,FUNCTION_TABLE_vi[St&15](43))),Xt=0;for(;;){xt=Xt,Tt=L,Nt=HEAP32[Vt>>2]|0,Ct=Tt-Nt|0,kt=(xt|0)<(Ct|0);if(!kt)break;Lt=HEAP32[3153]|0,FUNCTION_TABLE_vi[Lt&15](48),Ot=Xt,Mt=Ot+1|0,Xt=Mt}Xt=0;for(;;){_t=Xt,Dt=HEAP32[Vt>>2]|0,Pt=(_t|0)<(Dt|0);if(!Pt)break;Ht=HEAP32[3153]|0,Bt=Wt,jt=Bt+1|0,Wt=jt,Ft=HEAP8[Bt>>0]|0,It=Ft<<24>>24,FUNCTION_TABLE_vi[Ht&15](It),Rt=Xt,Ut=Rt+1|0,Xt=Ut}return s=0,zt=s,STACKTOP=en,zt|0}}Xt=0;for(;;){D=Xt,P=g,H=(D|0)<(P|0);if(!H)break;B=HEAP32[3153]|0,FUNCTION_TABLE_vi[B&15](42),j=Xt,F=j+1|0,Xt=F}return s=0,zt=s,STACKTOP=en,zt|0}function _wrt_A(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0;p=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,r=t;for(;;){i=r,s=i+ -1|0,r=s,o=(i|0)>0;if(!o)break;u=HEAP32[3153]|0,a=n,f=a+1|0,n=f,l=HEAP8[a>>0]|0,c=l<<24>>24,FUNCTION_TABLE_vi[u&15](c)}return STACKTOP=p,0}function _wrt_AW(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0;E=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,i=t,h=n;for(;;){p=i,d=h,v=(p|0)>(d|0);if(!v)break;m=i,g=m+ -1|0,i=g,y=HEAP32[3153]|0,FUNCTION_TABLE_vi[y&15](32)}for(;;){b=i,s=b+ -1|0,i=s,o=(b|0)>0;if(!o)break;u=HEAP32[3153]|0,a=r,f=a+1|0,r=f,l=HEAP8[a>>0]|0,c=l<<24>>24,FUNCTION_TABLE_vi[u&15](c)}return STACKTOP=E,0}function _wrt_G(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0;Tt=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=e,g=t,L=n,I=r,Q=i,Et=1,gt=0,ut=Q,dt=(ut|0)==4,vt=o,dt?(mt=+HEAPF32[vt>>2],u=mt,f=u):(a=+HEAPF64[vt>>3],f=a),St=f,l=St,c=l<0,c&&(h=St,p=-h,St=p),d=St,v=d<.1;e:do{if(v){m=St,y=m!=0;if(!y){gt=1;break}return b=o,w=g,E=L,S=I,x=Q,T=_wrt_E(b,w,E,S,x)|0,s=T,pt=s,STACKTOP=Tt,pt|0}for(;;){N=gt,C=L,k=(N|0)<=(C|0);if(!k)break;A=St,O=Et,M=A>=O;if(!M)break e;nt=gt,rt=nt+1|0,gt=rt,it=Et,st=it*10,Et=st}return ot=o,at=g,ft=L,lt=I,ct=Q,ht=_wrt_E(ot,at,ft,lt,ct)|0,s=ht,pt=s,STACKTOP=Tt,pt|0}while(0);_=HEAP32[3161]|0,wt=_,HEAP32[3161]=0,D=I,P=(D|0)==0,P?bt=4:(H=I,B=H+2|0,bt=B),j=o,F=g,q=bt,R=F-q|0,U=L,z=gt,W=U-z|0,X=Q,V=_wrt_F(j,R,W,X)|0,gt=V,yt=0;for(;;){$=yt,J=bt,K=($|0)<(J|0);if(!K)break;G=HEAP32[3153]|0,FUNCTION_TABLE_vi[G&15](32),Y=yt,Z=Y+1|0,yt=Z}return et=wt,HEAP32[3161]=et,tt=gt,s=tt,pt=s,STACKTOP=Tt,pt|0}function _wrt_Z(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0;hn=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=e,C=t,j=n,J=r,st=s,an=st,mt=J,kt=mt+ -1|0,J=kt,Ft=HEAP8[12696]|0,Kt=Ft<<24>>24!=0,o=an,Kt?(fn=o,g=J,y=an,b=y+g|0,an=b,un=-1):(w=J,E=o+w|0,fn=E,un=1);for(;;){S=an,x=fn,T=(S|0)==(x|0);if(T)break;N=an,k=HEAP8[N>>0]|0,L=k<<24>>24,A=(L|0)!=0;if(A)break;O=un,M=an,_=M+O|0,an=_}D=un,P=fn,H=an,B=P,F=H,I=B-F|0,q=Math_imul(D,I)|0,R=q<<1,U=R+1|0,ln=U,z=an,W=HEAP8[z>>0]|0,X=W<<24>>24,V=X&240,$=(V|0)!=0,$&&(K=ln,Q=K+1|0,ln=Q),G=ln,Y=C,Z=(G|0)>(Y|0);e:do if(Z){un=0;for(;;){et=un,tt=C,nt=(et|0)<(tt|0);if(!nt)break e;rt=HEAP32[3153]|0,FUNCTION_TABLE_vi[rt&15](42),it=un,ot=it+1|0,un=ot}}else{ut=ln,at=j,ft=at-ut|0,j=ft,lt=(ft|0)>0,lt&&(ct=j,ht=ln,pt=ht+ct|0,ln=pt);for(;;){dt=C,vt=dt+ -1|0,C=vt,gt=ln,yt=(vt|0)>=(gt|0);if(!yt)break;bt=HEAP32[3153]|0,FUNCTION_TABLE_vi[bt&15](32)}for(;;){wt=j,Et=wt+ -1|0,j=Et,St=(Et|0)>=0;if(!St)break;xt=HEAP32[3153]|0,FUNCTION_TABLE_vi[xt&15](48)}Tt=an,Nt=HEAP8[Tt>>0]|0,Ct=Nt<<24>>24,Lt=Ct&240,At=(Lt|0)!=0;do if(!At){Ot=HEAP32[3153]|0,Mt=an,_t=HEAP8[Mt>>0]|0,Dt=_t<<24>>24,Pt=Dt&15,Ht=20101+Pt|0,Bt=HEAP8[Ht>>0]|0,jt=Bt<<24>>24,FUNCTION_TABLE_vi[Ot&15](jt),It=an,qt=fn,Rt=(It|0)==(qt|0);if(!Rt){Ut=un,zt=an,Wt=zt+Ut|0,an=Wt;break}return i=0,m=i,STACKTOP=hn,m|0}while(0);for(;;){Xt=HEAP32[3153]|0,Vt=an,$t=HEAP8[Vt>>0]|0,Jt=$t<<24>>24,Qt=Jt>>4,Gt=Qt&15,Yt=20101+Gt|0,Zt=HEAP8[Yt>>0]|0,en=Zt<<24>>24,FUNCTION_TABLE_vi[Xt&15](en),tn=HEAP32[3153]|0,nn=an,rn=HEAP8[nn>>0]|0,sn=rn<<24>>24,on=sn&15,u=20101+on|0,a=HEAP8[u>>0]|0,f=a<<24>>24,FUNCTION_TABLE_vi[tn&15](f),l=an,c=fn,h=(l|0)==(c|0);if(h)break e;p=un,d=an,v=d+p|0,an=v}}while(0);return i=0,m=i,STACKTOP=hn,m|0}function _wrt_AP(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0;R=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),n=e,p=HEAP32[3159]|0,T=(p|0)!=0;if(T){_=_mv_cur()|0,F=_,D=(_|0)!=0;if(D)return P=F,t=P,M=t,STACKTOP=R,M|0}H=n,B=H+1|0,n=B,j=HEAP8[H>>0]|0,I=j;for(;;){r=n,i=HEAP8[r>>0]|0,s=i<<24>>24!=0;if(!s){q=12;break}o=n,u=HEAP8[o>>0]|0,a=u<<24>>24,f=I,l=f<<24>>24,c=(a|0)!=(l|0);if(c)h=HEAP32[3153]|0,d=n,v=HEAP8[d>>0]|0,m=v<<24>>24,FUNCTION_TABLE_vi[h&15](m);else{g=n,y=g+1|0,n=y,b=HEAP8[y>>0]|0,w=b<<24>>24,E=I,S=E<<24>>24,x=(w|0)==(S|0);if(!x){q=10;break}N=HEAP32[3153]|0,C=n,k=HEAP8[C>>0]|0,L=k<<24>>24,FUNCTION_TABLE_vi[N&15](L)}A=n,O=A+1|0,n=O}return(q|0)==10?(t=1,M=t,STACKTOP=R,M|0):(q|0)==12?(t=1,M=t,STACKTOP=R,M|0):0}function _wrt_H(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0;E=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=e,c=t,h=HEAP32[3159]|0,p=(h|0)!=0;if(p){d=_mv_cur()|0,b=d,v=(d|0)!=0;if(v)return m=b,n=m,l=n,STACKTOP=E,l|0}for(;;){g=r,y=g+ -1|0,r=y,i=(g|0)!=0;if(!i)break;s=HEAP32[3153]|0,o=c,u=o+1|0,c=u,a=HEAP8[o>>0]|0,f=a<<24>>24,FUNCTION_TABLE_vi[s&15](f)}return n=1,l=n,STACKTOP=E,l|0}function _f__icvt(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0;R=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=e,m=t,k=n,_=r,D=s,P=(D|0)>0,H=s;do{if(!P){j=(H|0)<0;if(j){o=s,u=0-o|0,I=u,a=k,HEAP32[a>>2]=1;break}return f=k,HEAP32[f>>2]=0,l=m,HEAP32[l>>2]=1,HEAP8[20420]=48,i=20420,M=i,STACKTOP=R,M|0}I=H,B=k,HEAP32[B>>2]=0}while(0);F=23;for(;;){c=I,h=_,p=(c>>>0)%(h>>>0)&-1,d=p+48|0,v=d&255,g=F,y=g+ -1|0,F=y,b=20398+y|0,HEAP8[b>>0]=v,w=_,E=I,S=(E>>>0)/(w>>>0)&-1,I=S,x=I,T=x>>>0>0;if(!T)break}return N=F,C=23-N|0,L=m,HEAP32[L>>2]=C,A=F,O=20398+A|0,i=O,M=i,STACKTOP=R,M|0}function _wrt_E(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0;Fi=STACKTOP,STACKTOP=STACKTOP+144|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Hi=Fi+24|0,Pi=Fi+8|0,xi=Fi+88|0,o=e,ln=t,er=n,cr=r,Sr=i,Dr=cr,ki=Dr,Wr=cr,ti=(Wr|0)<=0,ti&&(cr=2),hi=HEAP32[3161]|0,u=(hi|0)!=0,u?(y=HEAP32[3161]|0,A=er,q=A+2|0,G=(y|0)>=(q|0),G||(at=HEAP32[3161]|0,bt=er,Ot=0-bt|0,Rt=(at|0)<=(Ot|0),Rt||(ji=6))):ji=6;e:do if((ji|0)==6){Yt=HEAP32[3161]|0,cn=(Yt|0)<=0,cn&&(Sn=er,Dn=Sn+ -1|0,er=Dn),Wn=Sr,Jn=(Wn|0)==4,Kn=o,Jn?(Qn=+HEAPF32[Kn>>2],Gn=Qn,Ni=Gn):(Yn=+HEAPF64[Kn>>3],Ni=Yn),Zn=Ni,tr=Zn<0,tr?(_i=1,Di=1,nr=Ni,rr=-nr,Ni=rr):(_i=0,ir=HEAP32[3151]|0,Di=ir,sr=Ni,or=sr!=0,or||(Ni=0)),ur=ln,ar=Di,fr=4+ar|0,lr=er,hr=fr+lr|0,pr=cr,dr=hr+pr|0,vr=ur-dr|0,Ci=vr,mr=Ci,gr=(mr|0)<0;if(!gr){xr=HEAP32[3161]|0,Tr=(xr|0)<0,Tr&&(Nr=HEAP32[3161]|0,Cr=er,kr=Cr+Nr|0,er=kr),Lr=er,Ar=(Lr|0)>40,Ar?(Or=er,Mr=Or-40|0,Ti=Mr,er=40):Ti=0,_r=er,Pr=Ni,HEAP32[Pi>>2]=_r,Bi=Pi+8|0,HEAPF64[Bi>>3]=Pr,_sprintf(xi,20422,Pi)|0,Hr=HEAP8[xi>>0]|0,Br=Hr<<24>>24,jr=Br+8|0,Fr=20134+jr|0,Ir=HEAP8[Fr>>0]|0,qr=Ir<<24>>24,Rr=qr&1,Ur=(Rr|0)!=0;if(!Ur){zr=HEAP8[xi>>0]|0,Xr=zr<<24>>24;switch(Xr|0){case 78:case 110:Di=0;break;default:}Vr=ln,$r=_strlen(xi)|0,Jr=Vr-$r|0,Kr=Di,Qr=Jr-Kr|0,Ci=Qr,Gr=Ci,Yr=(Gr|0)<0;if(Yr)break;for(;;){Zr=Ci,ei=Zr+ -1|0,Ci=ei,ni=(ei|0)>=0;if(!ni)break;ri=HEAP32[3153]|0,FUNCTION_TABLE_vi[ri&15](32)}ii=Di,si=(ii|0)!=0,si&&(oi=HEAP32[3153]|0,ui=_i,ai=(ui|0)!=0,fi=ai?45:43,FUNCTION_TABLE_vi[oi&15](fi)),Oi=xi;for(;;){li=Oi,ci=HEAP8[li>>0]|0,pi=ci<<24>>24!=0;if(!pi)break;di=HEAP32[3153]|0,vi=Oi,mi=HEAP8[vi>>0]|0,gi=mi<<24>>24,FUNCTION_TABLE_vi[di&15](gi),yi=Oi,bi=yi+1|0,Oi=bi}return s=0,$n=s,STACKTOP=Fi,$n|0}wi=er,Ei=xi+wi|0,Si=Ei+3|0,Mi=Si,a=Ni,f=a!=0,l=Mi,f?(c=Mi,h=_atoi(c)|0,p=h+1|0,d=HEAP32[3161]|0,v=p-d|0,HEAP32[Hi>>2]=v,_sprintf(l,20428,Hi)|0):_strcpy(l,20434)|0,m=Mi,g=m+1|0,Mi=g,Oi=g,b=cr,w=(b|0)<2;if(w){E=Oi,S=HEAP8[E>>0]|0,x=S<<24>>24,T=(x|0)!=48;if(T)break}N=Oi,C=N+2|0,k=HEAP8[C>>0]|0,L=k<<24>>24!=0;t:do if(L){O=ki,M=(O|0)!=0;if(M){U=ki,z=(U|0)>=0;if(z){ji=48;break}W=cr,Li=W;break}_=Oi,D=_+ -2|0,Oi=D,Li=2;for(;;){P=Oi,H=P+1|0,B=HEAP8[H>>0]|0,j=Oi,HEAP8[j>>0]=B,F=B<<24>>24!=0;if(!F)break t;I=Oi,R=I+1|0,Oi=R}}else ji=48;while(0);n:do if((ji|0)==48){X=Oi,V=X+2|0,Oi=V,Li=2;for(;;){$=Oi,J=HEAP8[$>>0]|0,K=J<<24>>24!=0;if(!K)break n;Q=Li,Y=cr,Z=(Q|0)>=(Y|0);if(Z)break e;et=Li,tt=et+1|0,Li=tt,nt=Oi,rt=nt+1|0,Oi=rt}}while(0);for(;;){it=Ci,st=it+ -1|0,Ci=st,ot=(st|0)>=0;if(!ot)break;ut=HEAP32[3153]|0,FUNCTION_TABLE_vi[ut&15](32)}ft=Di,lt=(ft|0)!=0,lt&&(ct=HEAP32[3153]|0,ht=_i,pt=(ht|0)!=0,dt=pt?45:43,FUNCTION_TABLE_vi[ct&15](dt)),Oi=xi,vt=HEAP32[3161]|0,Ai=vt,mt=HEAP32[3161]|0,gt=(mt|0)<=0;if(gt){yt=HEAP32[3153]|0,FUNCTION_TABLE_vi[yt&15](46);for(;;){wt=Ai,Et=(wt|0)<0,St=HEAP32[3153]|0;if(!Et)break;FUNCTION_TABLE_vi[St&15](48),xt=Ai,Tt=xt+1|0,Ai=Tt}Nt=Oi,Ct=HEAP8[Nt>>0]|0,kt=Ct<<24>>24,FUNCTION_TABLE_vi[St&15](kt),Lt=Oi,At=Lt+2|0,Oi=At}else{Mt=HEAP32[3161]|0,_t=(Mt|0)>1;if(_t){Dt=HEAP32[3153]|0,Pt=Oi,Ht=HEAP8[Pt>>0]|0,Bt=Ht<<24>>24,FUNCTION_TABLE_vi[Dt&15](Bt),jt=Oi,Ft=jt+2|0,Oi=Ft;for(;;){It=Ai,qt=It+ -1|0,Ai=qt,Ut=(qt|0)>0,zt=HEAP32[3153]|0;if(!Ut)break;Wt=Oi,Xt=Wt+1|0,Oi=Xt,Vt=HEAP8[Wt>>0]|0,$t=Vt<<24>>24,FUNCTION_TABLE_vi[zt&15]($t)}FUNCTION_TABLE_vi[zt&15](46)}}Jt=Ti,Kt=(Jt|0)!=0;if(Kt){Qt=Mi,Gt=Qt+ -2|0,Mi=Gt;for(;;){Zt=Oi,en=Mi,tn=Zt>>>0<en>>>0;if(!tn)break;nn=HEAP32[3153]|0,rn=Oi,sn=rn+1|0,Oi=sn,on=HEAP8[rn>>0]|0,un=on<<24>>24,FUNCTION_TABLE_vi[nn&15](un)}an=Mi,fn=an+2|0,Mi=fn;for(;;){hn=HEAP32[3153]|0,FUNCTION_TABLE_vi[hn&15](48),pn=Ti,dn=pn+ -1|0,Ti=dn,vn=(dn|0)>0;if(!vn)break}}for(;;){mn=Oi,gn=Mi,yn=mn>>>0<gn>>>0;if(!yn)break;bn=HEAP32[3153]|0,wn=Oi,En=wn+1|0,Oi=En,xn=HEAP8[wn>>0]|0,Tn=xn<<24>>24,FUNCTION_TABLE_vi[bn&15](Tn)}Nn=cr,Cn=(Nn|0)<2;r:do if(Cn)kn=HEAP32[3153]|0,Ln=Oi,An=Ln+1|0,On=HEAP8[An>>0]|0,Mn=On<<24>>24,FUNCTION_TABLE_vi[kn&15](Mn);else{for(;;){_n=Li,Pn=_n+1|0,Li=Pn,Hn=cr,Bn=(Pn|0)<=(Hn|0);if(!Bn)break;jn=HEAP32[3153]|0,FUNCTION_TABLE_vi[jn&15](48)}for(;;){Fn=Oi,In=HEAP8[Fn>>0]|0,qn=In<<24>>24!=0;if(!qn)break r;Rn=HEAP32[3153]|0,Un=Oi,zn=Un+1|0,Oi=zn,Xn=HEAP8[Un>>0]|0,Vn=Xn<<24>>24,FUNCTION_TABLE_vi[Rn&15](Vn)}}while(0);return s=0,$n=s,STACKTOP=Fi,$n|0}}while(0);for(;;){yr=ln,br=yr+ -1|0,ln=br,wr=(br|0)>=0;if(!wr)break;Er=HEAP32[3153]|0,FUNCTION_TABLE_vi[Er&15](42)}return s=0,$n=s,STACKTOP=Fi,$n|0}function _wrt_F(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0;ln=STACKTOP,STACKTOP=STACKTOP+720|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),on=ln+8|0,Yt=ln+64|0,s=e,g=t,L=n,I=r,Q=I,ut=(Q|0)==4,yt=s,ut?(At=+HEAPF32[yt>>2],qt=At,a=qt):(o=+HEAPF64[yt>>3],a=o),an=a,f=L,l=(f|0)<344,l?Zt=0:(c=L,h=c-344|0,Zt=h,L=344),p=an,d=p<0,d?(v=an,m=-v,an=m,sn=1):(sn=0,y=an,b=y!=0,b||(an=0)),w=HEAP32[3161]|0,en=w,E=(w|0)!=0;if(E){S=en,x=(S|0)>0;if(x)for(;;){T=an,N=T*10,an=N,C=en,k=C+ -1|0,en=k,A=(k|0)>0;if(!A)break}else for(;;){O=an,M=O*.1,an=M,_=en,D=_+1|0,en=D,P=(D|0)<0;if(!P)break}}Gt=Yt,H=L,B=an,HEAP32[on>>2]=H,un=on+8|0,HEAPF64[un>>3]=B,j=_sprintf(Yt,20438,on)|0,F=Zt,q=j+F|0,en=q,R=HEAP8[Yt>>0]|0,U=R<<24>>24,z=(U|0)==48,W=L,X=(W|0)!=0,tn=z&X,tn&&(V=Gt,$=V+1|0,Gt=$,J=en,K=J+ -1|0,en=K),G=sn,Y=(G|0)==1;e:do if(Y){Z=Gt,rn=Z;t:for(;;){et=rn,tt=HEAP8[et>>0]|0,nt=tt<<24>>24,rt=(nt|0)==48,it=rn;if(rt){st=it+1|0,rn=st;continue}ot=HEAP8[it>>0]|0,at=ot<<24>>24;switch(at|0){case 0:break t;case 46:break;default:break e}ft=rn,lt=ft+1|0,rn=lt}sn=0}while(0);ct=sn,ht=(ct|0)!=0,pt=HEAP32[3151]|0,dt=(pt|0)!=0,nn=ht|dt,nn&&(vt=en,mt=vt+1|0,en=mt),gt=en,bt=g,wt=(gt|0)>(bt|0);if(wt){for(;;){Et=g,St=Et+ -1|0,g=St,xt=(St|0)>=0;if(!xt)break;Tt=HEAP32[3153]|0,FUNCTION_TABLE_vi[Tt&15](42)}return i=0,u=i,STACKTOP=ln,u|0}Nt=en,Ct=g,kt=Ct-Nt|0,g=kt;for(;;){Lt=g,Ot=Lt+ -1|0,g=Ot,Mt=(Ot|0)>=0;if(!Mt)break;_t=HEAP32[3153]|0,FUNCTION_TABLE_vi[_t&15](32)}Dt=sn,Pt=(Dt|0)!=0,Pt?(Ht=HEAP32[3153]|0,FUNCTION_TABLE_vi[Ht&15](45)):(Bt=HEAP32[3151]|0,jt=(Bt|0)!=0,jt&&(Ft=HEAP32[3153]|0,FUNCTION_TABLE_vi[Ft&15](43)));for(;;){It=Gt,Rt=It+1|0,Gt=Rt,Ut=HEAP8[It>>0]|0,zt=Ut<<24>>24,en=zt,Wt=(zt|0)!=0;if(!Wt)break;Xt=HEAP32[3153]|0,Vt=en,FUNCTION_TABLE_vi[Xt&15](Vt)}for(;;){$t=Zt,Jt=$t+ -1|0,Zt=Jt,Kt=(Jt|0)>=0;if(!Kt)break;Qt=HEAP32[3153]|0,FUNCTION_TABLE_vi[Qt&15](48)}return i=0,u=i,STACKTOP=ln,u|0}function _idamax_(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0;an=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=e,w=t,M=n,U=w,Z=U+ -8|0,w=Z,on=0,lt=i,Et=HEAP32[lt>>2]|0,_t=(Et|0)<1;if(!_t){zt=M,s=HEAP32[zt>>2]|0,c=(s|0)<=0;if(!c){on=1,p=i,d=HEAP32[p>>2]|0,v=(d|0)==1;if(v)return m=on,r=m,l=r,STACKTOP=an,l|0;g=M,y=HEAP32[g>>2]|0,b=(y|0)==1;if(b){gt=w,yt=gt+8|0,bt=+HEAPF64[yt>>3],wt=bt>=0,St=w,xt=St+8|0,Tt=+HEAPF64[xt>>3],Nt=-Tt,Ct=wt?Tt:Nt,tn=Ct,kt=i,Lt=HEAP32[kt>>2]|0,rn=Lt,nn=2;for(;;){At=nn,Ot=rn,Mt=(At|0)<=(Ot|0);if(!Mt)break;Dt=nn,Pt=w,Ht=Pt+(Dt<<3)|0,Bt=+HEAPF64[Ht>>3],en=Bt,jt=en,Ft=jt>=0,It=en,qt=-It,Rt=Ft?It:qt,Ut=tn,Wt=Rt<=Ut,Wt||(Xt=nn,on=Xt,Vt=nn,$t=w,Jt=$t+(Vt<<3)|0,Kt=+HEAPF64[Jt>>3],en=Kt,Qt=en,Gt=Qt>=0,Yt=en,Zt=-Yt,o=Gt?Yt:Zt,tn=o),u=nn,a=u+1|0,nn=a}return f=on,r=f,l=r,STACKTOP=an,l|0}sn=1,E=w,S=E+8|0,x=+HEAPF64[S>>3],T=x>=0,N=w,C=N+8|0,k=+HEAPF64[C>>3],L=-k,A=T?k:L,tn=A,O=M,_=HEAP32[O>>2]|0,D=sn,P=D+_|0,sn=P,H=i,B=HEAP32[H>>2]|0,rn=B,nn=2;for(;;){j=nn,F=rn,I=(j|0)<=(F|0);if(!I)break;q=sn,R=w,z=R+(q<<3)|0,W=+HEAPF64[z>>3],en=W,X=en,V=X>=0,$=en,J=-$,K=V?$:J,Q=tn,G=K<=Q,G||(Y=nn,on=Y,et=sn,tt=w,nt=tt+(et<<3)|0,rt=+HEAPF64[nt>>3],en=rt,it=en,st=it>=0,ot=en,ut=-ot,at=st?ot:ut,tn=at),ft=M,ct=HEAP32[ft>>2]|0,ht=sn,pt=ht+ct|0,sn=pt,dt=nn,vt=dt+1|0,nn=vt}return mt=on,r=mt,l=r,STACKTOP=an,l|0}}return h=on,r=h,l=r,STACKTOP=an,l|0}function _daxpy_(e,t,n,r,i,s){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0;var o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0;vr=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),u=e,kt=t,Ft=n,Kt=r,un=i,yn=s,An=un,qn=An+ -8|0,un=qn,Gn=Ft,a=Gn+ -8|0,Ft=a,b=u,O=HEAP32[b>>2]|0,R=(O|0)<=0;if(R)return o=0,xt=o,STACKTOP=vr,xt|0;Y=kt,ft=+HEAPF64[Y>>3],wt=ft==0;if(wt)return o=0,xt=o,STACKTOP=vr,xt|0;Tt=Kt,Nt=HEAP32[Tt>>2]|0,Ct=(Nt|0)==1;if(Ct){Lt=yn,At=HEAP32[Lt>>2]|0,Ot=(At|0)==1;if(Ot){_n=u,Dn=HEAP32[_n>>2]|0,Pn=(Dn|0)%4&-1,hr=Pn,Hn=hr,Bn=(Hn|0)==0;if(!Bn){jn=hr,fr=jn,ar=1;for(;;){Fn=ar,In=fr,Rn=(Fn|0)<=(In|0);if(!Rn)break;Un=kt,zn=+HEAPF64[Un>>3],Wn=ar,Xn=Ft,Vn=Xn+(Wn<<3)|0,$n=+HEAPF64[Vn>>3],Jn=zn*$n,Kn=ar,Qn=un,Yn=Qn+(Kn<<3)|0,Zn=+HEAPF64[Yn>>3],er=Zn+Jn,HEAPF64[Yn>>3]=er,tr=ar,nr=tr+1|0,ar=nr}rr=u,ir=HEAP32[rr>>2]|0,sr=(ir|0)<4;if(sr)return o=0,xt=o,STACKTOP=vr,xt|0}or=hr,ur=or+1|0,pr=ur,f=u,l=HEAP32[f>>2]|0,fr=l,c=pr,ar=c;for(;;){h=ar,p=fr,d=(h|0)<=(p|0);if(!d)break;v=kt,m=+HEAPF64[v>>3],g=ar,y=Ft,w=y+(g<<3)|0,E=+HEAPF64[w>>3],S=m*E,x=ar,T=un,N=T+(x<<3)|0,C=+HEAPF64[N>>3],k=C+S,HEAPF64[N>>3]=k,L=kt,A=+HEAPF64[L>>3],M=ar,_=M+1|0,D=Ft,P=D+(_<<3)|0,H=+HEAPF64[P>>3],B=A*H,j=ar,F=j+1|0,I=un,q=I+(F<<3)|0,U=+HEAPF64[q>>3],z=U+B,HEAPF64[q>>3]=z,W=kt,X=+HEAPF64[W>>3],V=ar,$=V+2|0,J=Ft,K=J+($<<3)|0,Q=+HEAPF64[K>>3],G=X*Q,Z=ar,et=Z+2|0,tt=un,nt=tt+(et<<3)|0,rt=+HEAPF64[nt>>3],it=rt+G,HEAPF64[nt>>3]=it,st=kt,ot=+HEAPF64[st>>3],ut=ar,at=ut+3|0,lt=Ft,ct=lt+(at<<3)|0,ht=+HEAPF64[ct>>3],pt=ot*ht,dt=ar,vt=dt+3|0,mt=un,gt=mt+(vt<<3)|0,yt=+HEAPF64[gt>>3],bt=yt+pt,HEAPF64[gt>>3]=bt,Et=ar,St=Et+4|0,ar=St}return o=0,xt=o,STACKTOP=vr,xt|0}}lr=1,cr=1,Mt=Kt,_t=HEAP32[Mt>>2]|0,Dt=(_t|0)<0,Dt&&(Pt=u,Ht=HEAP32[Pt>>2]|0,Bt=0-Ht|0,jt=Bt+1|0,It=Kt,qt=HEAP32[It>>2]|0,Rt=Math_imul(jt,qt)|0,Ut=Rt+1|0,lr=Ut),zt=yn,Wt=HEAP32[zt>>2]|0,Xt=(Wt|0)<0,Xt&&(Vt=u,$t=HEAP32[Vt>>2]|0,Jt=0-$t|0,Qt=Jt+1|0,Gt=yn,Yt=HEAP32[Gt>>2]|0,Zt=Math_imul(Qt,Yt)|0,en=Zt+1|0,cr=en),tn=u,nn=HEAP32[tn>>2]|0,fr=nn,ar=1;for(;;){rn=ar,sn=fr,on=(rn|0)<=(sn|0);if(!on)break;an=kt,fn=+HEAPF64[an>>3],ln=lr,cn=Ft,hn=cn+(ln<<3)|0,pn=+HEAPF64[hn>>3],dn=fn*pn,vn=cr,mn=un,gn=mn+(vn<<3)|0,bn=+HEAPF64[gn>>3],wn=bn+dn,HEAPF64[gn>>3]=wn,En=Kt,Sn=HEAP32[En>>2]|0,xn=lr,Tn=xn+Sn|0,lr=Tn,Nn=yn,Cn=HEAP32[Nn>>2]|0,kn=cr,Ln=kn+Cn|0,cr=Ln,On=ar,Mn=On+1|0,ar=Mn}return o=0,xt=o,STACKTOP=vr,xt|0}function _dcopy_(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0;ur=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=e,bt=t,Ot=n,Rt=r,Yt=i,ln=Rt,En=ln+ -8|0,Rt=En,_n=
bt,zn=_n+ -8|0,bt=zn,u=o,y=HEAP32[u>>2]|0,A=(y|0)<=0;if(A)return s=0,dt=s,STACKTOP=ur,dt|0;q=Ot,G=HEAP32[q>>2]|0,at=(G|0)==1;if(at){vt=Yt,mt=HEAP32[vt>>2]|0,gt=(mt|0)==1;if(gt){dn=o,vn=HEAP32[dn>>2]|0,mn=(vn|0)%7&-1,ir=mn,gn=ir,yn=(gn|0)==0;if(!yn){bn=ir,tr=bn,er=1;for(;;){wn=er,Sn=tr,xn=(wn|0)<=(Sn|0);if(!xn)break;Tn=er,Nn=bt,Cn=Nn+(Tn<<3)|0,kn=+HEAPF64[Cn>>3],Ln=er,An=Rt,On=An+(Ln<<3)|0,HEAPF64[On>>3]=kn,Mn=er,Dn=Mn+1|0,er=Dn}Pn=o,Hn=HEAP32[Pn>>2]|0,Bn=(Hn|0)<7;if(Bn)return s=0,dt=s,STACKTOP=ur,dt|0}jn=ir,Fn=jn+1|0,sr=Fn,In=o,qn=HEAP32[In>>2]|0,tr=qn,Rn=sr,er=Rn;for(;;){Un=er,Wn=tr,Xn=(Un|0)<=(Wn|0);if(!Xn)break;Vn=er,$n=bt,Jn=$n+(Vn<<3)|0,Kn=+HEAPF64[Jn>>3],Qn=er,Gn=Rt,Yn=Gn+(Qn<<3)|0,HEAPF64[Yn>>3]=Kn,Zn=er,a=Zn+1|0,f=bt,l=f+(a<<3)|0,c=+HEAPF64[l>>3],h=er,p=h+1|0,d=Rt,v=d+(p<<3)|0,HEAPF64[v>>3]=c,m=er,g=m+2|0,b=bt,w=b+(g<<3)|0,E=+HEAPF64[w>>3],S=er,x=S+2|0,T=Rt,N=T+(x<<3)|0,HEAPF64[N>>3]=E,C=er,k=C+3|0,L=bt,O=L+(k<<3)|0,M=+HEAPF64[O>>3],_=er,D=_+3|0,P=Rt,H=P+(D<<3)|0,HEAPF64[H>>3]=M,B=er,j=B+4|0,F=bt,I=F+(j<<3)|0,R=+HEAPF64[I>>3],U=er,z=U+4|0,W=Rt,X=W+(z<<3)|0,HEAPF64[X>>3]=R,V=er,$=V+5|0,J=bt,K=J+($<<3)|0,Q=+HEAPF64[K>>3],Y=er,Z=Y+5|0,et=Rt,tt=et+(Z<<3)|0,HEAPF64[tt>>3]=Q,nt=er,rt=nt+6|0,it=bt,st=it+(rt<<3)|0,ot=+HEAPF64[st>>3],ut=er,ft=ut+6|0,lt=Rt,ct=lt+(ft<<3)|0,HEAPF64[ct>>3]=ot,ht=er,pt=ht+7|0,er=pt}return s=0,dt=s,STACKTOP=ur,dt|0}}nr=1,rr=1,yt=Ot,wt=HEAP32[yt>>2]|0,Et=(wt|0)<0,Et&&(St=o,xt=HEAP32[St>>2]|0,Tt=0-xt|0,Nt=Tt+1|0,Ct=Ot,kt=HEAP32[Ct>>2]|0,Lt=Math_imul(Nt,kt)|0,At=Lt+1|0,nr=At),Mt=Yt,_t=HEAP32[Mt>>2]|0,Dt=(_t|0)<0,Dt&&(Pt=o,Ht=HEAP32[Pt>>2]|0,Bt=0-Ht|0,jt=Bt+1|0,Ft=Yt,It=HEAP32[Ft>>2]|0,qt=Math_imul(jt,It)|0,Ut=qt+1|0,rr=Ut),zt=o,Wt=HEAP32[zt>>2]|0,tr=Wt,er=1;for(;;){Xt=er,Vt=tr,$t=(Xt|0)<=(Vt|0);if(!$t)break;Jt=nr,Kt=bt,Qt=Kt+(Jt<<3)|0,Gt=+HEAPF64[Qt>>3],Zt=rr,en=Rt,tn=en+(Zt<<3)|0,HEAPF64[tn>>3]=Gt,nn=Ot,rn=HEAP32[nn>>2]|0,sn=nr,on=sn+rn|0,nr=on,un=Yt,an=HEAP32[un>>2]|0,fn=rr,cn=fn+an|0,rr=cn,hn=er,pn=hn+1|0,er=pn}return s=0,dt=s,STACKTOP=ur,dt|0}function _ddot_(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0;wr=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=e,Ot=t,Rt=n,Yt=r,ln=i,En=Yt,_n=En+ -8|0,Yt=_n,zn=Ot,er=zn+ -8|0,Ot=er,yr=0,cr=0,u=o,y=HEAP32[u>>2]|0,A=(y|0)<=0;if(A)return q=yr,s=q,Ct=s,STACKTOP=wr,+Ct;G=Rt,at=HEAP32[G>>2]|0,bt=(at|0)==1;if(bt){kt=ln,Lt=HEAP32[kt>>2]|0,At=(Lt|0)==1;if(At){Dn=o,Pn=HEAP32[Dn>>2]|0,Hn=(Pn|0)%5&-1,mr=Hn,Bn=mr,jn=(Bn|0)==0;if(jn)br=18;else{Fn=mr,pr=Fn,hr=1;for(;;){In=hr,qn=pr,Rn=(In|0)<=(qn|0);if(!Rn)break;Un=hr,Wn=Ot,Xn=Wn+(Un<<3)|0,Vn=+HEAPF64[Xn>>3],$n=hr,Jn=Yt,Kn=Jn+($n<<3)|0,Qn=+HEAPF64[Kn>>3],Gn=Vn*Qn,Yn=cr,Zn=Yn+Gn,cr=Zn,tr=hr,nr=tr+1|0,hr=nr}rr=o,ir=HEAP32[rr>>2]|0,sr=(ir|0)<5,sr||(br=18)}e:do if((br|0)==18){or=mr,ur=or+1|0,gr=ur,ar=o,fr=HEAP32[ar>>2]|0,pr=fr,lr=gr,hr=lr;for(;;){a=hr,f=pr,l=(a|0)<=(f|0);if(!l)break e;c=cr,h=hr,p=Ot,d=p+(h<<3)|0,v=+HEAPF64[d>>3],m=hr,g=Yt,b=g+(m<<3)|0,w=+HEAPF64[b>>3],E=v*w,S=c+E,x=hr,T=x+1|0,N=Ot,C=N+(T<<3)|0,k=+HEAPF64[C>>3],L=hr,O=L+1|0,M=Yt,_=M+(O<<3)|0,D=+HEAPF64[_>>3],P=k*D,H=S+P,B=hr,j=B+2|0,F=Ot,I=F+(j<<3)|0,R=+HEAPF64[I>>3],U=hr,z=U+2|0,W=Yt,X=W+(z<<3)|0,V=+HEAPF64[X>>3],$=R*V,J=H+$,K=hr,Q=K+3|0,Y=Ot,Z=Y+(Q<<3)|0,et=+HEAPF64[Z>>3],tt=hr,nt=tt+3|0,rt=Yt,it=rt+(nt<<3)|0,st=+HEAPF64[it>>3],ot=et*st,ut=J+ot,ft=hr,lt=ft+4|0,ct=Ot,ht=ct+(lt<<3)|0,pt=+HEAPF64[ht>>3],dt=hr,vt=dt+4|0,mt=Yt,gt=mt+(vt<<3)|0,yt=+HEAPF64[gt>>3],wt=pt*yt,Et=ut+wt,cr=Et,St=hr,xt=St+5|0,hr=xt}}while(0);return Tt=cr,yr=Tt,Nt=yr,s=Nt,Ct=s,STACKTOP=wr,+Ct}}dr=1,vr=1,Mt=Rt,_t=HEAP32[Mt>>2]|0,Dt=(_t|0)<0,Dt&&(Pt=o,Ht=HEAP32[Pt>>2]|0,Bt=0-Ht|0,jt=Bt+1|0,Ft=Rt,It=HEAP32[Ft>>2]|0,qt=Math_imul(jt,It)|0,Ut=qt+1|0,dr=Ut),zt=ln,Wt=HEAP32[zt>>2]|0,Xt=(Wt|0)<0,Xt&&(Vt=o,$t=HEAP32[Vt>>2]|0,Jt=0-$t|0,Kt=Jt+1|0,Qt=ln,Gt=HEAP32[Qt>>2]|0,Zt=Math_imul(Kt,Gt)|0,en=Zt+1|0,vr=en),tn=o,nn=HEAP32[tn>>2]|0,pr=nn,hr=1;for(;;){rn=hr,sn=pr,on=(rn|0)<=(sn|0);if(!on)break;un=dr,an=Ot,fn=an+(un<<3)|0,cn=+HEAPF64[fn>>3],hn=vr,pn=Yt,dn=pn+(hn<<3)|0,vn=+HEAPF64[dn>>3],mn=cn*vn,gn=cr,yn=gn+mn,cr=yn,bn=Rt,wn=HEAP32[bn>>2]|0,Sn=dr,xn=Sn+wn|0,dr=xn,Tn=ln,Nn=HEAP32[Tn>>2]|0,Cn=vr,kn=Cn+Nn|0,vr=kn,Ln=hr,An=Ln+1|0,hr=An}return On=cr,yr=On,Mn=yr,s=Mn,Ct=s,STACKTOP=wr,+Ct}function _dscal_(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0;Rn=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=e,Z=t,lt=n,Et=r,_t=lt,zt=_t+ -8|0,lt=zt,en=o,hn=HEAP32[en>>2]|0,xn=(hn|0)<=0;if(!xn){u=Et,y=HEAP32[u>>2]|0,A=(y|0)<=0;if(!A){q=Et,$=HEAP32[q>>2]|0,J=($|0)==1,K=o,Q=HEAP32[K>>2]|0;if(!J){G=Et,Y=HEAP32[G>>2]|0,et=Math_imul(Q,Y)|0,In=et,tt=In,Hn=tt,nt=Et,rt=HEAP32[nt>>2]|0,Bn=rt,Pn=1;for(;;){it=Bn,st=(it|0)<0,ot=Pn,ut=Hn,at=(ot|0)>=(ut|0),ft=(ot|0)<=(ut|0),i=st?at:ft,ct=i&1,ht=(ct|0)!=0;if(!ht)break;pt=Z,dt=+HEAPF64[pt>>3],vt=Pn,mt=lt,gt=mt+(vt<<3)|0,yt=+HEAPF64[gt>>3],bt=dt*yt,wt=Pn,St=lt,xt=St+(wt<<3)|0,HEAPF64[xt>>3]=bt,Tt=Bn,Nt=Pn,Ct=Nt+Tt|0,Pn=Ct}return s=0,V=s,STACKTOP=Rn,V|0}kt=(Q|0)%5&-1,jn=kt,Lt=jn,At=(Lt|0)==0;if(!At){Ot=jn,Bn=Ot,Pn=1;for(;;){Mt=Pn,Dt=Bn,Pt=(Mt|0)<=(Dt|0);if(!Pt)break;Ht=Z,Bt=+HEAPF64[Ht>>3],jt=Pn,Ft=lt,It=Ft+(jt<<3)|0,qt=+HEAPF64[It>>3],Rt=Bt*qt,Ut=Pn,Wt=lt,Xt=Wt+(Ut<<3)|0,HEAPF64[Xt>>3]=Rt,Vt=Pn,$t=Vt+1|0,Pn=$t}Jt=o,Kt=HEAP32[Jt>>2]|0,Qt=(Kt|0)<5;if(Qt)return s=0,V=s,STACKTOP=Rn,V|0}Gt=jn,Yt=Gt+1|0,Fn=Yt,Zt=o,tn=HEAP32[Zt>>2]|0,Bn=tn,nn=Fn,Pn=nn;for(;;){rn=Pn,sn=Bn,on=(rn|0)<=(sn|0);if(!on)break;un=Z,an=+HEAPF64[un>>3],fn=Pn,ln=lt,cn=ln+(fn<<3)|0,pn=+HEAPF64[cn>>3],dn=an*pn,vn=Pn,mn=lt,gn=mn+(vn<<3)|0,HEAPF64[gn>>3]=dn,yn=Z,bn=+HEAPF64[yn>>3],wn=Pn,En=wn+1|0,Sn=lt,Tn=Sn+(En<<3)|0,Nn=+HEAPF64[Tn>>3],Cn=bn*Nn,kn=Pn,Ln=kn+1|0,An=lt,On=An+(Ln<<3)|0,HEAPF64[On>>3]=Cn,Mn=Z,_n=+HEAPF64[Mn>>3],Dn=Pn,a=Dn+2|0,f=lt,l=f+(a<<3)|0,c=+HEAPF64[l>>3],h=_n*c,p=Pn,d=p+2|0,v=lt,m=v+(d<<3)|0,HEAPF64[m>>3]=h,g=Z,b=+HEAPF64[g>>3],w=Pn,E=w+3|0,S=lt,x=S+(E<<3)|0,T=+HEAPF64[x>>3],N=b*T,C=Pn,k=C+3|0,L=lt,O=L+(k<<3)|0,HEAPF64[O>>3]=N,M=Z,_=+HEAPF64[M>>3],D=Pn,P=D+4|0,H=lt,B=H+(P<<3)|0,j=+HEAPF64[B>>3],F=_*j,I=Pn,R=I+4|0,U=lt,z=U+(R<<3)|0,HEAPF64[z>>3]=F,W=Pn,X=W+5|0,Pn=X}return s=0,V=s,STACKTOP=Rn,V|0}}return s=0,V=s,STACKTOP=Rn,V|0}function _dswap_(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0;mr=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=e,kt=t,Ft=n,Kt=r,un=i,yn=Kt,An=yn+ -8|0,Kt=An,qn=kt,Gn=qn+ -8|0,kt=Gn,u=o,y=HEAP32[u>>2]|0,A=(y|0)<=0;if(A)return s=0,xt=s,STACKTOP=mr,xt|0;q=Ft,G=HEAP32[q>>2]|0,at=(G|0)==1;if(at){bt=un,Tt=HEAP32[bt>>2]|0,Nt=(Tt|0)==1;if(Nt){Mn=o,_n=HEAP32[Mn>>2]|0,Dn=(_n|0)%3&-1,pr=Dn,Pn=pr,Hn=(Pn|0)==0;if(!Hn){Bn=pr,lr=Bn,fr=1;for(;;){jn=fr,Fn=lr,In=(jn|0)<=(Fn|0);if(!In)break;Rn=fr,Un=kt,zn=Un+(Rn<<3)|0,Wn=+HEAPF64[zn>>3],ar=Wn,Xn=fr,Vn=Kt,$n=Vn+(Xn<<3)|0,Jn=+HEAPF64[$n>>3],Kn=fr,Qn=kt,Yn=Qn+(Kn<<3)|0,HEAPF64[Yn>>3]=Jn,Zn=ar,er=fr,tr=Kt,nr=tr+(er<<3)|0,HEAPF64[nr>>3]=Zn,rr=fr,ir=rr+1|0,fr=ir}sr=o,or=HEAP32[sr>>2]|0,ur=(or|0)<3;if(ur)return s=0,xt=s,STACKTOP=mr,xt|0}a=pr,f=a+1|0,dr=f,l=o,c=HEAP32[l>>2]|0,lr=c,h=dr,fr=h;for(;;){p=fr,d=lr,v=(p|0)<=(d|0);if(!v)break;m=fr,g=kt,b=g+(m<<3)|0,w=+HEAPF64[b>>3],ar=w,E=fr,S=Kt,x=S+(E<<3)|0,T=+HEAPF64[x>>3],N=fr,C=kt,k=C+(N<<3)|0,HEAPF64[k>>3]=T,L=ar,O=fr,M=Kt,_=M+(O<<3)|0,HEAPF64[_>>3]=L,D=fr,P=D+1|0,H=kt,B=H+(P<<3)|0,j=+HEAPF64[B>>3],ar=j,F=fr,I=F+1|0,R=Kt,U=R+(I<<3)|0,z=+HEAPF64[U>>3],W=fr,X=W+1|0,V=kt,$=V+(X<<3)|0,HEAPF64[$>>3]=z,J=ar,K=fr,Q=K+1|0,Y=Kt,Z=Y+(Q<<3)|0,HEAPF64[Z>>3]=J,et=fr,tt=et+2|0,nt=kt,rt=nt+(tt<<3)|0,it=+HEAPF64[rt>>3],ar=it,st=fr,ot=st+2|0,ut=Kt,ft=ut+(ot<<3)|0,lt=+HEAPF64[ft>>3],ct=fr,ht=ct+2|0,pt=kt,dt=pt+(ht<<3)|0,HEAPF64[dt>>3]=lt,vt=ar,mt=fr,gt=mt+2|0,yt=Kt,wt=yt+(gt<<3)|0,HEAPF64[wt>>3]=vt,Et=fr,St=Et+3|0,fr=St}return s=0,xt=s,STACKTOP=mr,xt|0}}cr=1,hr=1,Ct=Ft,Lt=HEAP32[Ct>>2]|0,At=(Lt|0)<0,At&&(Ot=o,Mt=HEAP32[Ot>>2]|0,_t=0-Mt|0,Dt=_t+1|0,Pt=Ft,Ht=HEAP32[Pt>>2]|0,Bt=Math_imul(Dt,Ht)|0,jt=Bt+1|0,cr=jt),It=un,qt=HEAP32[It>>2]|0,Rt=(qt|0)<0,Rt&&(Ut=o,zt=HEAP32[Ut>>2]|0,Wt=0-zt|0,Xt=Wt+1|0,Vt=un,$t=HEAP32[Vt>>2]|0,Jt=Math_imul(Xt,$t)|0,Qt=Jt+1|0,hr=Qt),Gt=o,Yt=HEAP32[Gt>>2]|0,lr=Yt,fr=1;for(;;){Zt=fr,en=lr,tn=(Zt|0)<=(en|0);if(!tn)break;nn=cr,rn=kt,sn=rn+(nn<<3)|0,on=+HEAPF64[sn>>3],ar=on,an=hr,fn=Kt,ln=fn+(an<<3)|0,cn=+HEAPF64[ln>>3],hn=cr,pn=kt,dn=pn+(hn<<3)|0,HEAPF64[dn>>3]=cn,vn=ar,mn=hr,gn=Kt,bn=gn+(mn<<3)|0,HEAPF64[bn>>3]=vn,wn=Ft,En=HEAP32[wn>>2]|0,Sn=cr,xn=Sn+En|0,cr=xn,Tn=un,Nn=HEAP32[Tn>>2]|0,Cn=hr,kn=Cn+Nn|0,hr=kn,Ln=fr,On=Ln+1|0,fr=On}return s=0,xt=s,STACKTOP=mr,xt|0}function _dger_(e,t,n,r,i,s,o,u,a){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0;var f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0;Dr=STACKTOP,STACKTOP=STACKTOP+96|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Cr=Dr+8|0,l=e,qt=t,Gt=n,fn=r,wn=i,Mn=s,Un=o,Zn=u,lr=a,c=fn,S=c+ -8|0,fn=S,D=Mn,W=D+ -8|0,Mn=W,tt=lr,ht=HEAP32[tt>>2]|0,Er=ht,xt=Er,Pt=1+xt|0,Sr=Pt,Ft=Sr,It=Zn,Rt=0-Ft|0,Ut=It+(Rt<<3)|0,Zn=Ut,HEAP32[Cr>>2]=0,zt=l,Wt=HEAP32[zt>>2]|0,Xt=(Wt|0)<0;do if(Xt)HEAP32[Cr>>2]=1;else{Vt=qt,$t=HEAP32[Vt>>2]|0,Jt=($t|0)<0;if(Jt){HEAP32[Cr>>2]=2;break}Kt=wn,Qt=HEAP32[Kt>>2]|0,Yt=(Qt|0)==0;if(Yt){HEAP32[Cr>>2]=5;break}Zt=Un,en=HEAP32[Zt>>2]|0,tn=(en|0)==0;if(tn){HEAP32[Cr>>2]=7;break}nn=lr,rn=HEAP32[nn>>2]|0,sn=l,on=HEAP32[sn>>2]|0,un=1>=(on|0),un?hn=1:(an=l,ln=HEAP32[an>>2]|0,hn=ln),cn=(rn|0)<(hn|0),cn&&(HEAP32[Cr>>2]=9)}while(0);pn=HEAP32[Cr>>2]|0,dn=(pn|0)!=0;if(dn)return _xerbla_(20444,Cr)|0,f=0,jt=f,STACKTOP=Dr,jt|0;vn=l,mn=HEAP32[vn>>2]|0,gn=(mn|0)==0;if(!gn){yn=qt,bn=HEAP32[yn>>2]|0,En=(bn|0)==0;if(!En){Sn=Gt,xn=+HEAPF64[Sn>>3],Tn=xn==0;if(!Tn){Nn=Un,Cn=HEAP32[Nn>>2]|0,kn=(Cn|0)>0,kn?Ar=1:(Ln=qt,An=HEAP32[Ln>>2]|0,On=An-1|0,_n=Un,Dn=HEAP32[_n>>2]|0,Pn=Math_imul(On,Dn)|0,Hn=1-Pn|0,Ar=Hn),Bn=wn,jn=HEAP32[Bn>>2]|0,Fn=(jn|0)==1;e:do if(Fn){In=qt,qn=HEAP32[In>>2]|0,Tr=qn,Lr=1;for(;;){Rn=Lr,zn=Tr,Wn=(Rn|0)<=(zn|0);if(!Wn)break e;Xn=Ar,Vn=Mn,$n=Vn+(Xn<<3)|0,Jn=+HEAPF64[$n>>3],Kn=Jn!=0;t:do if(Kn){Qn=Gt,Gn=+HEAPF64[Qn>>3],Yn=Ar,er=Mn,tr=er+(Yn<<3)|0,nr=+HEAPF64[tr>>3],rr=Gn*nr,Mr=rr,ir=l,sr=HEAP32[ir>>2]|0,Nr=sr,xr=1;for(;;){or=xr,ur=Nr,ar=(or|0)<=(ur|0);if(!ar)break t;fr=xr,cr=fn,hr=cr+(fr<<3)|0,pr=+HEAPF64[hr>>3],dr=Mr,vr=pr*dr,mr=xr,gr=Lr,yr=Er,br=Math_imul(gr,yr)|0,wr=mr+br|0,h=Zn,p=h+(wr<<3)|0,d=+HEAPF64[p>>3],v=d+vr,HEAPF64[p>>3]=v,m=xr,g=m+1|0,xr=g}}while(0);y=Un,b=HEAP32[y>>2]|0,w=Ar,E=w+b|0,Ar=E,x=Lr,T=x+1|0,Lr=T}}else{N=wn,C=HEAP32[N>>2]|0,k=(C|0)>0,k?Or=1:(L=l,A=HEAP32[L>>2]|0,O=A-1|0,M=wn,_=HEAP32[M>>2]|0,P=Math_imul(O,_)|0,H=1-P|0,Or=H),B=qt,j=HEAP32[B>>2]|0,Tr=j,Lr=1;for(;;){F=Lr,I=Tr,q=(F|0)<=(I|0);if(!q)break e;R=Ar,U=Mn,z=U+(R<<3)|0,X=+HEAPF64[z>>3],V=X!=0;n:do if(V){$=Gt,J=+HEAPF64[$>>3],K=Ar,Q=Mn,G=Q+(K<<3)|0,Y=+HEAPF64[G>>3],Z=J*Y,Mr=Z,et=Or,kr=et,nt=l,rt=HEAP32[nt>>2]|0,Nr=rt,xr=1;for(;;){it=xr,st=Nr,ot=(it|0)<=(st|0);if(!ot)break n;ut=kr,at=fn,ft=at+(ut<<3)|0,lt=+HEAPF64[ft>>3],ct=Mr,pt=lt*ct,dt=xr,vt=Lr,mt=Er,gt=Math_imul(vt,mt)|0,yt=dt+gt|0,bt=Zn,wt=bt+(yt<<3)|0,Et=+HEAPF64[wt>>3],St=Et+pt,HEAPF64[wt>>3]=St,Tt=wn,Nt=HEAP32[Tt>>2]|0,Ct=kr,kt=Ct+Nt|0,kr=kt,Lt=xr,At=Lt+1|0,xr=At}}while(0);Ot=Un,Mt=HEAP32[Ot>>2]|0,_t=Ar,Dt=_t+Mt|0,Ar=Dt,Ht=Lr,Bt=Ht+1|0,Lr=Bt}}while(0);return f=0,jt=f,STACKTOP=Dr,jt|0}}}return f=0,jt=f,STACKTOP=Dr,jt|0}function _dgemm_(e,t,n,r,i,s,o,u,a,f,l,c,h){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0,c|=0,h|=0;var p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0;Ol=STACKTOP,STACKTOP=STACKTOP+144|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),wl=Ol+28|0,d=e,yn=t,Ei=n,No=r,La=i,wf=s,Mf=o,Uf=u,Zf=a,v=f,C=l,j=c,J=h,st=Uf,mt=HEAP32[st>>2]|0,ll=mt,kt=ll,Ft=1+kt|0,cl=Ft,Kt=cl,un=Mf,bn=0-Kt|0,On=un+(bn<<3)|0,Mf=On,Rn=v,Yn=HEAP32[Rn>>2]|0,hl=Yn,fr=hl,wr=1+fr|0,pl=wr,Mr=pl,Ur=Zf,Zr=0-Mr|0,li=Ur+(Zr<<3)|0,Zf=li,Si=J,Di=HEAP32[Si>>2]|0,dl=Di,Wi=dl,ts=1+Wi|0,vl=ts,hs=vl,xs=j,Ps=0-hs|0,Xs=xs+(Ps<<3)|0,j=Xs,no=d,po=_lsame_(no,20460)|0,Tl=po,Co=yn,jo=_lsame_(Co,20460)|0,Nl=jo,Jo=Tl,su=(Jo|0)!=0,su?(mu=Ei,ku=HEAP32[mu>>2]|0,Cl=ku,Fu=La,Ku=HEAP32[Fu>>2]|0,xl=Ku):(oa=La,ga=HEAP32[oa>>2]|0,Cl=ga,Aa=Ei,qa=HEAP32[Aa>>2]|0,xl=qa),Ga=Nl,ff=(Ga|0)!=0,ff?(df=La,vf=HEAP32[df>>2]|0,kl=vf):(mf=No,gf=HEAP32[mf>>2]|0,kl=gf),HEAP32[wl>>2]=0,yf=Tl,bf=(yf|0)!=0,bf?Al=11:(Ef=d,Sf=_lsame_(Ef,20468)|0,xf=(Sf|0)!=0,xf?Al=11:(Tf=d,Nf=_lsame_(Tf,20466)|0,Cf=(Nf|0)!=0,Cf?Al=11:HEAP32[wl>>2]=1));do if((Al|0)==11){kf=Nl,Lf=(kf|0)!=0;if(!Lf){Af=yn,Of=_lsame_(Af,20468)|0,_f=(Of|0)!=0;if(!_f){Df=yn,Pf=_lsame_(Df,20466)|0,Hf=(Pf|0)!=0;if(!Hf){HEAP32[wl>>2]=2;break}}}Bf=Ei,jf=HEAP32[Bf>>2]|0,Ff=(jf|0)<0;if(Ff){HEAP32[wl>>2]=3;break}If=No,qf=HEAP32[If>>2]|0,Rf=(qf|0)<0;if(Rf){HEAP32[wl>>2]=4;break}zf=La,Wf=HEAP32[zf>>2]|0,Xf=(Wf|0)<0;if(Xf){HEAP32[wl>>2]=5;break}Vf=Uf,$f=HEAP32[Vf>>2]|0,Jf=Cl,Kf=1>=(Jf|0),Qf=Cl,Gf=Kf?1:Qf,Yf=($f|0)<(Gf|0);if(Yf){HEAP32[wl>>2]=8;break}el=v,tl=HEAP32[el>>2]|0,nl=kl,rl=1>=(nl|0),il=kl,sl=rl?1:il,ol=(tl|0)<(sl|0);if(ol){HEAP32[wl>>2]=10;break}ul=J,al=HEAP32[ul>>2]|0,fl=Ei,m=HEAP32[fl>>2]|0,g=1>=(m|0),g?E=1:(y=Ei,b=HEAP32[y>>2]|0,E=b),w=(al|0)<(E|0),w&&(HEAP32[wl>>2]=13)}while(0);S=HEAP32[wl>>2]|0,x=(S|0)!=0;if(x)return _xerbla_(20451,wl)|0,p=0,pf=p,STACKTOP=Ol,pf|0;T=Ei,N=HEAP32[T>>2]|0,k=(N|0)==0;do if(!k){L=No,A=HEAP32[L>>2]|0,O=(A|0)==0;if(!O){M=wf,_=+HEAPF64[M>>3],D=_==0,D?Al=35:(P=La,H=HEAP32[P>>2]|0,B=(H|0)==0,B&&(Al=35));if((Al|0)==35){F=C,I=+HEAPF64[F>>3],q=I==1;if(q)break}R=wf,U=+HEAPF64[R>>3],z=U==0;if(z){W=C,X=+HEAPF64[W>>3],V=X==0,$=No,K=HEAP32[$>>2]|0,gl=K,El=1;e:do if(V)for(;;){Q=El,G=gl,Y=(Q|0)<=(G|0);if(!Y)break e;Z=Ei,et=HEAP32[Z>>2]|0,yl=et,ml=1;for(;;){tt=ml,nt=yl,rt=(tt|0)<=(nt|0);if(!rt)break;it=ml,ot=El,ut=dl,at=Math_imul(ot,ut)|0,ft=it+at|0,lt=j,ct=lt+(ft<<3)|0,HEAPF64[ct>>3]=0,ht=ml,pt=ht+1|0,ml=pt}dt=El,vt=dt+1|0,El=vt}else for(;;){gt=El,yt=gl,bt=(gt|0)<=(yt|0);if(!bt)break e;wt=Ei,Et=HEAP32[wt>>2]|0,yl=Et,ml=1;for(;;){St=ml,xt=yl,Tt=(St|0)<=(xt|0);if(!Tt)break;Nt=C,Ct=+HEAPF64[Nt>>3],Lt=ml,At=El,Ot=dl,Mt=Math_imul(At,Ot)|0,_t=Lt+Mt|0,Dt=j,Pt=Dt+(_t<<3)|0,Ht=+HEAPF64[Pt>>3],Bt=Ct*Ht,jt=ml,It=El,qt=dl,Rt=Math_imul(It,qt)|0,Ut=jt+Rt|0,zt=j,Wt=zt+(Ut<<3)|0,HEAPF64[Wt>>3]=Bt,Xt=ml,Vt=Xt+1|0,ml=Vt}$t=El,Jt=$t+1|0,El=Jt}while(0);return p=0,pf=p,STACKTOP=Ol,pf|0}Qt=Nl,Gt=(Qt|0)!=0,Yt=Tl,Zt=(Yt|0)!=0,en=No,tn=HEAP32[en>>2]|0,gl=tn,El=1;t:do if(Gt){if(!Zt)for(;;){fi=El,ci=gl,hi=(fi|0)<=(ci|0);if(!hi)break t;pi=Ei,di=HEAP32[pi>>2]|0,yl=di,ml=1;for(;;){vi=ml,mi=yl,gi=(vi|0)<=(mi|0);if(!gi)break;Ll=0,yi=La,bi=HEAP32[yi>>2]|0,bl=bi,Sl=1;for(;;){wi=Sl,xi=bl,Ti=(wi|0)<=(xi|0);if(!Ti)break;Ni=Sl,Ci=ml,ki=ll,Li=Math_imul(Ci,ki)|0,Ai=Ni+Li|0,Oi=Mf,Mi=Oi+(Ai<<3)|0,_i=+HEAPF64[Mi>>3],Pi=Sl,Hi=El,Bi=hl,ji=Math_imul(Hi,Bi)|0,Fi=Pi+ji|0,Ii=Zf,qi=Ii+(Fi<<3)|0,Ri=+HEAPF64[qi>>3],Ui=_i*Ri,zi=Ll,Xi=zi+Ui,Ll=Xi,Vi=Sl,$i=Vi+1|0,Sl=$i}Ji=C,Ki=+HEAPF64[Ji>>3],Qi=Ki==0,Gi=wf,Yi=+HEAPF64[Gi>>3],Zi=Ll,es=Yi*Zi,Qi?(ns=ml,rs=El,is=dl,ss=Math_imul(rs,is)|0,os=ns+ss|0,us=j,as=us+(os<<3)|0,HEAPF64[as>>3]=es):(fs=C,ls=+HEAPF64[fs>>3],cs=ml,ps=El,ds=dl,vs=Math_imul(ps,ds)|0,ms=cs+vs|0,gs=j,ys=gs+(ms<<3)|0,bs=+HEAPF64[ys>>3],ws=ls*bs,Es=es+ws,Ss=ml,Ts=El,Ns=dl,Cs=Math_imul(Ts,Ns)|0,ks=Ss+Cs|0,Ls=j,As=Ls+(ks<<3)|0,HEAPF64[As>>3]=Es),Os=ml,Ms=Os+1|0,ml=Ms}_s=El,Ds=_s+1|0,El=Ds}for(;;){nn=El,rn=gl,sn=(nn|0)<=(rn|0);if(!sn)break t;on=C,an=+HEAPF64[on>>3],fn=an==0;n:do if(fn){ln=Ei,cn=HEAP32[ln>>2]|0,yl=cn,ml=1;for(;;){hn=ml,pn=yl,dn=(hn|0)<=(pn|0);if(!dn)break n;vn=ml,mn=El,gn=dl,wn=Math_imul(mn,gn)|0,En=vn+wn|0,Sn=j,xn=Sn+(En<<3)|0,HEAPF64[xn>>3]=0,Tn=ml,Nn=Tn+1|0,ml=Nn}}else{Cn=C,kn=+HEAPF64[Cn>>3],Ln=kn!=1;if(Ln){An=Ei,Mn=HEAP32[An>>2]|0,yl=Mn,ml=1;for(;;){_n=ml,Dn=yl,Pn=(_n|0)<=(Dn|0);if(!Pn)break n;Hn=C,Bn=+HEAPF64[Hn>>3],jn=ml,Fn=El,In=dl,qn=Math_imul(Fn,In)|0,Un=jn+qn|0,zn=j,Wn=zn+(Un<<3)|0,Xn=+HEAPF64[Wn>>3],Vn=Bn*Xn,$n=ml,Jn=El,Kn=dl,Qn=Math_imul(Jn,Kn)|0,Gn=$n+Qn|0,Zn=j,er=Zn+(Gn<<3)|0,HEAPF64[er>>3]=Vn,tr=ml,nr=tr+1|0,ml=nr}}}while(0);rr=La,ir=HEAP32[rr>>2]|0,yl=ir,Sl=1;for(;;){sr=Sl,or=yl,ur=(sr|0)<=(or|0);if(!ur)break;ar=Sl,lr=El,cr=hl,hr=Math_imul(lr,cr)|0,pr=ar+hr|0,dr=Zf,vr=dr+(pr<<3)|0,mr=+HEAPF64[vr>>3],gr=mr!=0;r:do if(gr){yr=wf,br=+HEAPF64[yr>>3],Er=Sl,Sr=El,xr=hl,Tr=Math_imul(Sr,xr)|0,Nr=Er+Tr|0,Cr=Zf,kr=Cr+(Nr<<3)|0,Lr=+HEAPF64[kr>>3],Ar=br*Lr,Ll=Ar,Or=Ei,_r=HEAP32[Or>>2]|0,bl=_r,ml=1;for(;;){Dr=ml,Pr=bl,Hr=(Dr|0)<=(Pr|0);if(!Hr)break r;Br=Ll,jr=ml,Fr=Sl,Ir=ll,qr=Math_imul(Fr,Ir)|0,Rr=jr+qr|0,zr=Mf,Wr=zr+(Rr<<3)|0,Xr=+HEAPF64[Wr>>3],Vr=Br*Xr,$r=ml,Jr=El,Kr=dl,Qr=Math_imul(Jr,Kr)|0,Gr=$r+Qr|0,Yr=j,ei=Yr+(Gr<<3)|0,ti=+HEAPF64[ei>>3],ni=ti+Vr,HEAPF64[ei>>3]=ni,ri=ml,ii=ri+1|0,ml=ii}}while(0);si=Sl,oi=si+1|0,Sl=oi}ui=El,ai=ui+1|0,El=ai}}else{if(!Zt)for(;;){Ru=El,Uu=gl,zu=(Ru|0)<=(Uu|0);if(!zu)break t;Wu=Ei,Xu=HEAP32[Wu>>2]|0,yl=Xu,ml=1;for(;;){Vu=ml,$u=yl,Ju=(Vu|0)<=($u|0);if(!Ju)break;Ll=0,Qu=La,Gu=HEAP32[Qu>>2]|0,bl=Gu,Sl=1;for(;;){Yu=Sl,Zu=bl,ea=(Yu|0)<=(Zu|0);if(!ea)break;ta=Sl,na=ml,ra=ll,ia=Math_imul(na,ra)|0,sa=ta+ia|0,ua=Mf,aa=ua+(sa<<3)|0,fa=+HEAPF64[aa>>3],la=El,ca=Sl,ha=hl,pa=Math_imul(ca,ha)|0,da=la+pa|0,va=Zf,ma=va+(da<<3)|0,ya=+HEAPF64[ma>>3],ba=fa*ya,wa=Ll,Ea=wa+ba,Ll=Ea,Sa=Sl,xa=Sa+1|0,Sl=xa}Ta=C,Na=+HEAPF64[Ta>>3],Ca=Na==0,ka=wf,Oa=+HEAPF64[ka>>3],Ma=Ll,_a=Oa*Ma,Ca?(Da=ml,Pa=El,Ha=dl,Ba=Math_imul(Pa,Ha)|0,ja=Da+Ba|0,Fa=j,Ia=Fa+(ja<<3)|0,HEAPF64[Ia>>3]=_a):(Ra=C,Ua=+HEAPF64[Ra>>3],za=ml,Wa=El,Xa=dl,Va=Math_imul(Wa,Xa)|0,$a=za+Va|0,Ja=j,Ka=Ja+($a<<3)|0,Qa=+HEAPF64[Ka>>3],Ya=Ua*Qa,Za=_a+Ya,ef=ml,tf=El,nf=dl,rf=Math_imul(tf,nf)|0,sf=ef+rf|0,of=j,uf=of+(sf<<3)|0,HEAPF64[uf>>3]=Za),af=ml,lf=af+1|0,ml=lf}cf=El,hf=cf+1|0,El=hf}for(;;){Hs=El,Bs=gl,js=(Hs|0)<=(Bs|0);if(!js)break t;Fs=C,Is=+HEAPF64[Fs>>3],qs=Is==0;i:do if(qs){Rs=Ei,Us=HEAP32[Rs>>2]|0,yl=Us,ml=1;for(;;){zs=ml,Ws=yl,Vs=(zs|0)<=(Ws|0);if(!Vs)break i;$s=ml,Js=El,Ks=dl,Qs=Math_imul(Js,Ks)|0,Gs=$s+Qs|0,Ys=j,Zs=Ys+(Gs<<3)|0,HEAPF64[Zs>>3]=0,eo=ml,to=eo+1|0,ml=to}}else{ro=C,io=+HEAPF64[ro>>3],so=io!=1;if(so){oo=Ei,uo=HEAP32[oo>>2]|0,yl=uo,ml=1;for(;;){ao=ml,fo=yl,lo=(ao|0)<=(fo|0);if(!lo)break i;co=C,ho=+HEAPF64[co>>3],vo=ml,mo=El,go=dl,yo=Math_imul(mo,go)|0,bo=vo+yo|0,wo=j,Eo=wo+(bo<<3)|0,So=+HEAPF64[Eo>>3],xo=ho*So,To=ml,ko=El,Lo=dl,Ao=Math_imul(ko,Lo)|0,Oo=To+Ao|0,Mo=j,_o=Mo+(Oo<<3)|0,HEAPF64[_o>>3]=xo,Do=ml,Po=Do+1|0,ml=Po}}}while(0);Ho=La,Bo=HEAP32[Ho>>2]|0,yl=Bo,Sl=1;for(;;){Fo=Sl,Io=yl,qo=(Fo|0)<=(Io|0),Ro=El;if(!qo)break;Uo=Sl,zo=hl,Wo=Math_imul(Uo,zo)|0,Xo=Ro+Wo|0,Vo=Zf,$o=Vo+(Xo<<3)|0,Ko=+HEAPF64[$o>>3],Qo=Ko!=0;s:do if(Qo){Go=wf,Yo=+HEAPF64[Go>>3],Zo=El,eu=Sl,tu=hl,nu=Math_imul(eu,tu)|0,ru=Zo+nu|0,iu=Zf,ou=iu+(ru<<3)|0,uu=+HEAPF64[ou>>3],au=Yo*uu,Ll=au,fu=Ei,lu=HEAP32[fu>>2]|0,bl=lu,ml=1;for(;;){cu=ml,hu=bl,pu=(cu|0)<=(hu|0);if(!pu)break s;du=Ll,vu=ml,gu=Sl,yu=ll,bu=Math_imul(gu,yu)|0,wu=vu+bu|0,Eu=Mf,Su=Eu+(wu<<3)|0,xu=+HEAPF64[Su>>3],Tu=du*xu,Nu=ml,Cu=El,Lu=dl,Au=Math_imul(Cu,Lu)|0,Ou=Nu+Au|0,Mu=j,_u=Mu+(Ou<<3)|0,Du=+HEAPF64[_u>>3],Pu=Du+Tu,HEAPF64[_u>>3]=Pu,Hu=ml,Bu=Hu+1|0,ml=Bu}}while(0);ju=Sl,Iu=ju+1|0,Sl=Iu}qu=Ro+1|0,El=qu}}while(0);return p=0,pf=p,STACKTOP=Ol,pf|0}}while(0);return p=0,pf=p,STACKTOP=Ol,pf|0}function _dtrsm_(e,t,n,r,i,s,o,u,a,f,l){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0,o|=0,u|=0,a|=0,f|=0,l|=0;var c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0,Rd=0,Ud=0,zd=0,Wd=0,Xd=0,Vd=0,$d=0,Jd=0,Kd=0,Qd=0,Gd=0,Yd=0,Zd=0,ev=0,tv=0,nv=0,rv=0,iv=0,sv=0,ov=0,uv=0,av=0,fv=0,lv=0,cv=0,hv=0,pv=0,dv=0,vv=0,mv=0,gv=0,yv=0,bv=0,wv=0,Ev=0,Sv=0,xv=0,Tv=0,Nv=0,Cv=0,kv=0,Lv=0,Av=0,Ov=0,Mv=0,_v=0,Dv=0,Pv=0,Hv=0,Bv=0,jv=0,Fv=0,Iv=0,qv=0,Rv=0,Uv=0,zv=0,Wv=0,Xv=0,Vv=0,$v=0,Jv=0,Kv=0,Qv=0,Gv=0,Yv=0,Zv=0,em=0,tm=0,nm=0,rm=0,im=0,sm=0,om=0,um=0,am=0,fm=0,lm=0,cm=0,hm=0,pm=0,dm=0,vm=0,mm=0,gm=0,ym=0,bm=0,wm=0,Em=0,Sm=0,xm=0,Tm=0,Nm=0,Cm=0,km=0,Lm=0,Am=0,Om=0,Mm=0,_m=0,Dm=0,Pm=0,Hm=0,Bm=0,jm=0,Fm=0,Im=0,qm=0,Rm=0,Um=0,zm=0,Wm=0,Xm=0,Vm=0,$m=0,Jm=0,Km=0,Qm=0,Gm=0,Ym=0,Zm=0,eg=0,tg=0,ng=0,rg=0,ig=0,sg=0,og=0,ug=0,ag=0,fg=0,lg=0,cg=0,hg=0,pg=0,dg=0,vg=0,mg=0,gg=0,yg=0,bg=0,wg=0,Eg=0,Sg=0,xg=0,Tg=0,Ng=0,Cg=0,kg=0,Lg=0,Ag=0,Og=0,Mg=0,_g=0,Dg=0,Pg=0,Hg=0,Bg=0,jg=0,Fg=0,Ig=0,qg=0,Rg=0,Ug=0,zg=0,Wg=0,Xg=0,Vg=0,$g=0,Jg=0,Kg=0,Qg=0,Gg=0,Yg=0,Zg=0,ey=0,ty=0,ny=0;ny=STACKTOP,STACKTOP=STACKTOP+128|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),$g=ny+24|0,h=e,mn=t,bi=n,xo=r,Ca=i,Ol=s,Dh=o,Bd=u,Im=a,p=f,T=l,H=Im,V=HEAP32[H>>2]|0,Ig=V,rt=Ig,dt=1+rt|0,qg=dt,Nt=qg,Bt=Bd,$t=0-Nt|0,sn=Bt+($t<<3)|0,Bd=sn,gn=T,Ln=HEAP32[gn>>2]|0,Rg=Ln,In=Rg,Qn=1+In|0,Ug=Qn,ur=Ug,yr=p,Ar=0-ur|0,qr=yr+(Ar<<3)|0,p=qr,Gr=h,ai=_lsame_(Gr,20458)|0,Qg=ai,wi=Qg,Mi=(wi|0)!=0,Mi?(Ui=Ca,Zi=HEAP32[Ui>>2]|0,Yg=Zi):(ls=Ol,Es=HEAP32[ls>>2]|0,Yg=Es),_s=xo,zs=_lsame_(_s,20460)|0,Gg=zs,eo=mn,co=_lsame_(eo,20462)|0,ey=co,HEAP32[$g>>2]=0,To=Qg,Ho=(To|0)!=0,Ho?ty=7:(Vo=h,ru=_lsame_(Vo,20464)|0,du=(ru|0)!=0,du?ty=7:HEAP32[$g>>2]=1);do if((ty|0)==7){Nu=ey,Bu=(Nu|0)!=0;if(!Bu){$u=mn,ia=_lsame_($u,20458)|0,va=(ia|0)!=0;if(!va){HEAP32[$g>>2]=2;break}}ka=bi,Fa=_lsame_(ka,20460)|0,Ka=(Fa|0)!=0;if(!Ka){uf=bi,yf=_lsame_(uf,20466)|0,Af=(yf|0)!=0;if(!Af){qf=bi,Gf=_lsame_(qf,20468)|0,al=(Gf|0)!=0;if(!al){HEAP32[$g>>2]=3;break}}}bl=xo,Ml=_lsame_(bl,20462)|0,Ul=(Ml|0)!=0;if(!Ul){Zl=xo,lc=_lsame_(Zl,20460)|0,Ec=(lc|0)!=0;if(!Ec){HEAP32[$g>>2]=4;break}}_c=Ca,zc=HEAP32[_c>>2]|0,eh=(zc|0)<0;if(eh){HEAP32[$g>>2]=5;break}ch=Ol,Sh=HEAP32[ch>>2]|0,Ph=(Sh|0)<0;if(Ph){HEAP32[$g>>2]=6;break}Xh=Im,np=HEAP32[Xh>>2]|0,pp=Yg,Tp=1>=(pp|0),Hp=Yg,Vp=Tp?1:Hp,rd=(np|0)<(Vp|0);if(rd){HEAP32[$g>>2]=9;break}dd=T,Nd=HEAP32[dd>>2]|0,jd=Ca,Jd=HEAP32[jd>>2]|0,sv=1>=(Jd|0),sv?Kv=1:(mv=Ca,kv=HEAP32[mv>>2]|0,Kv=kv),Fv=(Nd|0)<(Kv|0),Fv&&(HEAP32[$g>>2]=11)}while(0);om=HEAP32[$g>>2]|0,gm=(om|0)!=0;if(gm)return _xerbla_(20470,$g)|0,c=0,Dg=c,STACKTOP=ny,Dg|0;Lm=Ca,qm=HEAP32[Lm>>2]|0,Gm=(qm|0)==0;if(!Gm){ag=Ol,bg=HEAP32[ag>>2]|0,Og=(bg|0)==0;if(!Og){Pg=Dh,Hg=+HEAPF64[Pg>>3],Bg=Hg==0;if(Bg){jg=Ol,Fg=HEAP32[jg>>2]|0,Wg=Fg,Jg=1;for(;;){d=Jg,v=Wg,m=(d|0)<=(v|0);if(!m)break;g=Ca,y=HEAP32[g>>2]|0,Xg=y,zg=1;for(;;){b=zg,w=Xg,E=(b|0)<=(w|0);if(!E)break;S=zg,x=Jg,N=Rg,C=Math_imul(x,N)|0,k=S+C|0,L=p,A=L+(k<<3)|0,HEAPF64[A>>3]=0,O=zg,M=O+1|0,zg=M}_=Jg,D=_+1|0,Jg=D}return c=0,Dg=c,STACKTOP=ny,Dg|0}P=Qg,B=(P|0)!=0,j=bi,F=_lsame_(j,20460)|0,I=(F|0)!=0,q=ey,R=(q|0)!=0,U=Ol,z=HEAP32[U>>2]|0;e:do if(B){Wg=z,Jg=1;if(!I)if(R)for(;;){ts=Jg,ns=Wg,rs=(ts|0)<=(ns|0);if(!rs)break e;is=Ca,ss=HEAP32[is>>2]|0,Xg=ss,zg=1;for(;;){os=zg,us=Xg,as=(os|0)<=(us|0);if(!as)break;fs=Dh,cs=+HEAPF64[fs>>3],hs=zg,ps=Jg,ds=Rg,vs=Math_imul(ps,ds)|0,ms=hs+vs|0,gs=p,ys=gs+(ms<<3)|0,bs=+HEAPF64[ys>>3],ws=cs*bs,Zg=ws,Ss=zg,xs=Ss-1|0,Vg=xs,Kg=1;for(;;){Ts=Kg,Ns=Vg,Cs=(Ts|0)<=(Ns|0);if(!Cs)break;ks=Kg,Ls=zg,As=Ig,Os=Math_imul(Ls,As)|0,Ms=ks+Os|0,Ds=Bd,Ps=Ds+(Ms<<3)|0,Hs=+HEAPF64[Ps>>3],Bs=Kg,js=Jg,Fs=Rg,Is=Math_imul(js,Fs)|0,qs=Bs+Is|0,Rs=p,Us=Rs+(qs<<3)|0,Ws=+HEAPF64[Us>>3],Xs=Hs*Ws,Vs=Zg,$s=Vs-Xs,Zg=$s,Js=Kg,Ks=Js+1|0,Kg=Ks}Qs=Gg,Gs=(Qs|0)!=0,Gs&&(Ys=zg,Zs=zg,to=Ig,no=Math_imul(Zs,to)|0,ro=Ys+no|0,io=Bd,so=io+(ro<<3)|0,oo=+HEAPF64[so>>3],uo=Zg,ao=uo/oo,Zg=ao),fo=Zg,lo=zg,ho=Jg,po=Rg,vo=Math_imul(ho,po)|0,mo=lo+vo|0,go=p,yo=go+(mo<<3)|0,HEAPF64[yo>>3]=fo,bo=zg,wo=bo+1|0,zg=wo}Eo=Jg,So=Eo+1|0,Jg=So}else for(;;){No=Jg,Co=Wg,ko=(No|0)<=(Co|0);if(!ko)break e;Lo=Ca,Ao=HEAP32[Lo>>2]|0,zg=Ao;for(;;){Oo=zg,Mo=(Oo|0)>=1;if(!Mo)break;_o=Dh,Do=+HEAPF64[_o>>3],Po=zg,Bo=Jg,jo=Rg,Fo=Math_imul(Bo,jo)|0,Io=Po+Fo|0,qo=p,Ro=qo+(Io<<3)|0,Uo=+HEAPF64[Ro>>3],zo=Do*Uo,Zg=zo,Wo=Ca,Xo=HEAP32[Wo>>2]|0,Xg=Xo,$o=zg,Jo=$o+1|0,Kg=Jo;for(;;){Ko=Kg,Qo=Xg,Go=(Ko|0)<=
(Qo|0);if(!Go)break;Yo=Kg,Zo=zg,eu=Ig,tu=Math_imul(Zo,eu)|0,nu=Yo+tu|0,iu=Bd,su=iu+(nu<<3)|0,ou=+HEAPF64[su>>3],uu=Kg,au=Jg,fu=Rg,lu=Math_imul(au,fu)|0,cu=uu+lu|0,hu=p,pu=hu+(cu<<3)|0,vu=+HEAPF64[pu>>3],mu=ou*vu,gu=Zg,yu=gu-mu,Zg=yu,bu=Kg,wu=bu+1|0,Kg=wu}Eu=Gg,Su=(Eu|0)!=0,Su&&(xu=zg,Tu=zg,Cu=Ig,ku=Math_imul(Tu,Cu)|0,Lu=xu+ku|0,Au=Bd,Ou=Au+(Lu<<3)|0,Mu=+HEAPF64[Ou>>3],_u=Zg,Du=_u/Mu,Zg=Du),Pu=Zg,Hu=zg,ju=Jg,Fu=Rg,Iu=Math_imul(ju,Fu)|0,qu=Hu+Iu|0,Ru=p,Uu=Ru+(qu<<3)|0,HEAPF64[Uu>>3]=Pu,zu=zg,Wu=zu+ -1|0,zg=Wu}Xu=Jg,Vu=Xu+1|0,Jg=Vu}if(R)for(;;){W=Jg,X=Wg,$=(W|0)<=(X|0);if(!$)break e;J=Dh,K=+HEAPF64[J>>3],Q=K!=1;t:do if(Q){G=Ca,Y=HEAP32[G>>2]|0,Xg=Y,zg=1;for(;;){Z=zg,et=Xg,tt=(Z|0)<=(et|0);if(!tt)break t;nt=Dh,it=+HEAPF64[nt>>3],st=zg,ot=Jg,ut=Rg,at=Math_imul(ot,ut)|0,ft=st+at|0,lt=p,ct=lt+(ft<<3)|0,ht=+HEAPF64[ct>>3],pt=it*ht,vt=zg,mt=Jg,gt=Rg,yt=Math_imul(mt,gt)|0,bt=vt+yt|0,wt=p,Et=wt+(bt<<3)|0,HEAPF64[Et>>3]=pt,St=zg,xt=St+1|0,zg=xt}}while(0);Tt=Ca,Ct=HEAP32[Tt>>2]|0,Kg=Ct;for(;;){kt=Kg,Lt=(kt|0)>=1;if(!Lt)break;At=Kg,Ot=Jg,Mt=Rg,_t=Math_imul(Ot,Mt)|0,Dt=At+_t|0,Pt=p,Ht=Pt+(Dt<<3)|0,jt=+HEAPF64[Ht>>3],Ft=jt!=0;n:do if(Ft){It=Gg,qt=(It|0)!=0,qt&&(Rt=Kg,Ut=Kg,zt=Ig,Wt=Math_imul(Ut,zt)|0,Xt=Rt+Wt|0,Vt=Bd,Jt=Vt+(Xt<<3)|0,Kt=+HEAPF64[Jt>>3],Qt=Kg,Gt=Jg,Yt=Rg,Zt=Math_imul(Gt,Yt)|0,en=Qt+Zt|0,tn=p,nn=tn+(en<<3)|0,rn=+HEAPF64[nn>>3],on=rn/Kt,HEAPF64[nn>>3]=on),un=Kg,an=un-1|0,Xg=an,zg=1;for(;;){fn=zg,ln=Xg,cn=(fn|0)<=(ln|0);if(!cn)break n;hn=Kg,pn=Jg,dn=Rg,vn=Math_imul(pn,dn)|0,yn=hn+vn|0,bn=p,wn=bn+(yn<<3)|0,En=+HEAPF64[wn>>3],Sn=zg,xn=Kg,Tn=Ig,Nn=Math_imul(xn,Tn)|0,Cn=Sn+Nn|0,kn=Bd,An=kn+(Cn<<3)|0,On=+HEAPF64[An>>3],Mn=En*On,_n=zg,Dn=Jg,Pn=Rg,Hn=Math_imul(Dn,Pn)|0,Bn=_n+Hn|0,jn=p,Fn=jn+(Bn<<3)|0,qn=+HEAPF64[Fn>>3],Rn=qn-Mn,HEAPF64[Fn>>3]=Rn,Un=zg,zn=Un+1|0,zg=zn}}while(0);Wn=Kg,Xn=Wn+ -1|0,Kg=Xn}Vn=Jg,$n=Vn+1|0,Jg=$n}else for(;;){Jn=Jg,Kn=Wg,Gn=(Jn|0)<=(Kn|0);if(!Gn)break e;Yn=Dh,Zn=+HEAPF64[Yn>>3],er=Zn!=1;r:do if(er){tr=Ca,nr=HEAP32[tr>>2]|0,Xg=nr,zg=1;for(;;){rr=zg,ir=Xg,sr=(rr|0)<=(ir|0);if(!sr)break r;or=Dh,ar=+HEAPF64[or>>3],fr=zg,lr=Jg,cr=Rg,hr=Math_imul(lr,cr)|0,pr=fr+hr|0,dr=p,vr=dr+(pr<<3)|0,mr=+HEAPF64[vr>>3],gr=ar*mr,br=zg,wr=Jg,Er=Rg,Sr=Math_imul(wr,Er)|0,xr=br+Sr|0,Tr=p,Nr=Tr+(xr<<3)|0,HEAPF64[Nr>>3]=gr,Cr=zg,kr=Cr+1|0,zg=kr}}while(0);Lr=Ca,Or=HEAP32[Lr>>2]|0,Xg=Or,Kg=1;for(;;){Mr=Kg,_r=Xg,Dr=(Mr|0)<=(_r|0);if(!Dr)break;Pr=Kg,Hr=Jg,Br=Rg,jr=Math_imul(Hr,Br)|0,Fr=Pr+jr|0,Ir=p,Rr=Ir+(Fr<<3)|0,Ur=+HEAPF64[Rr>>3],zr=Ur!=0;i:do if(zr){Wr=Gg,Xr=(Wr|0)!=0,Xr&&(Vr=Kg,$r=Kg,Jr=Ig,Kr=Math_imul($r,Jr)|0,Qr=Vr+Kr|0,Yr=Bd,Zr=Yr+(Qr<<3)|0,ei=+HEAPF64[Zr>>3],ti=Kg,ni=Jg,ri=Rg,ii=Math_imul(ni,ri)|0,si=ti+ii|0,oi=p,ui=oi+(si<<3)|0,fi=+HEAPF64[ui>>3],li=fi/ei,HEAPF64[ui>>3]=li),ci=Ca,hi=HEAP32[ci>>2]|0,Vg=hi,pi=Kg,di=pi+1|0,zg=di;for(;;){vi=zg,mi=Vg,gi=(vi|0)<=(mi|0);if(!gi)break i;yi=Kg,Ei=Jg,Si=Rg,xi=Math_imul(Ei,Si)|0,Ti=yi+xi|0,Ni=p,Ci=Ni+(Ti<<3)|0,ki=+HEAPF64[Ci>>3],Li=zg,Ai=Kg,Oi=Ig,_i=Math_imul(Ai,Oi)|0,Di=Li+_i|0,Pi=Bd,Hi=Pi+(Di<<3)|0,Bi=+HEAPF64[Hi>>3],ji=ki*Bi,Fi=zg,Ii=Jg,qi=Rg,Ri=Math_imul(Ii,qi)|0,zi=Fi+Ri|0,Wi=p,Xi=Wi+(zi<<3)|0,Vi=+HEAPF64[Xi>>3],$i=Vi-ji,HEAPF64[Xi>>3]=$i,Ji=zg,Ki=Ji+1|0,zg=Ki}}while(0);Qi=Kg,Gi=Qi+1|0,Kg=Gi}Yi=Jg,es=Yi+1|0,Jg=es}}else if(I)if(R){Wg=z,Jg=1;for(;;){Ju=Jg,Ku=Wg,Qu=(Ju|0)<=(Ku|0);if(!Qu)break e;Gu=Dh,Yu=+HEAPF64[Gu>>3],Zu=Yu!=1;s:do if(Zu){ea=Ca,ta=HEAP32[ea>>2]|0,Xg=ta,zg=1;for(;;){na=zg,ra=Xg,sa=(na|0)<=(ra|0);if(!sa)break s;oa=Dh,ua=+HEAPF64[oa>>3],aa=zg,fa=Jg,la=Rg,ca=Math_imul(fa,la)|0,ha=aa+ca|0,pa=p,da=pa+(ha<<3)|0,ma=+HEAPF64[da>>3],ga=ua*ma,ya=zg,ba=Jg,wa=Rg,Ea=Math_imul(ba,wa)|0,Sa=ya+Ea|0,xa=p,Ta=xa+(Sa<<3)|0,HEAPF64[Ta>>3]=ga,Na=zg,La=Na+1|0,zg=La}}while(0);Aa=Jg,Oa=Aa-1|0,Xg=Oa,Kg=1;for(;;){Ma=Kg,_a=Xg,Da=(Ma|0)<=(_a|0);if(!Da)break;Pa=Kg,Ha=Jg,Ba=Ig,ja=Math_imul(Ha,Ba)|0,Ia=Pa+ja|0,qa=Bd,Ra=qa+(Ia<<3)|0,Ua=+HEAPF64[Ra>>3],za=Ua!=0;o:do if(za){Wa=Ca,Xa=HEAP32[Wa>>2]|0,Vg=Xa,zg=1;for(;;){Va=zg,$a=Vg,Ja=(Va|0)<=($a|0);if(!Ja)break o;Qa=Kg,Ga=Jg,Ya=Ig,Za=Math_imul(Ga,Ya)|0,ef=Qa+Za|0,tf=Bd,nf=tf+(ef<<3)|0,rf=+HEAPF64[nf>>3],sf=zg,of=Kg,af=Rg,ff=Math_imul(of,af)|0,lf=sf+ff|0,cf=p,hf=cf+(lf<<3)|0,pf=+HEAPF64[hf>>3],df=rf*pf,vf=zg,mf=Jg,gf=Rg,bf=Math_imul(mf,gf)|0,wf=vf+bf|0,Ef=p,Sf=Ef+(wf<<3)|0,xf=+HEAPF64[Sf>>3],Tf=xf-df,HEAPF64[Sf>>3]=Tf,Nf=zg,Cf=Nf+1|0,zg=Cf}}while(0);kf=Kg,Lf=kf+1|0,Kg=Lf}Of=Gg,Mf=(Of|0)!=0;u:do if(Mf){_f=Jg,Df=Jg,Pf=Ig,Hf=Math_imul(Df,Pf)|0,Bf=_f+Hf|0,jf=Bd,Ff=jf+(Bf<<3)|0,If=+HEAPF64[Ff>>3],Rf=1/If,Zg=Rf,Uf=Ca,zf=HEAP32[Uf>>2]|0,Xg=zf,zg=1;for(;;){Wf=zg,Xf=Xg,Vf=(Wf|0)<=(Xf|0);if(!Vf)break u;$f=Zg,Jf=zg,Kf=Jg,Qf=Rg,Yf=Math_imul(Kf,Qf)|0,Zf=Jf+Yf|0,el=p,tl=el+(Zf<<3)|0,nl=+HEAPF64[tl>>3],rl=$f*nl,il=zg,sl=Jg,ol=Rg,ul=Math_imul(sl,ol)|0,fl=il+ul|0,ll=p,cl=ll+(fl<<3)|0,HEAPF64[cl>>3]=rl,hl=zg,pl=hl+1|0,zg=pl}}while(0);dl=Jg,vl=dl+1|0,Jg=vl}}else{Jg=z;for(;;){ml=Jg,gl=(ml|0)>=1;if(!gl)break e;yl=Dh,wl=+HEAPF64[yl>>3],El=wl!=1;a:do if(El){Sl=Ca,xl=HEAP32[Sl>>2]|0,Wg=xl,zg=1;for(;;){Tl=zg,Nl=Wg,Cl=(Tl|0)<=(Nl|0);if(!Cl)break a;kl=Dh,Ll=+HEAPF64[kl>>3],Al=zg,_l=Jg,Dl=Rg,Pl=Math_imul(_l,Dl)|0,Hl=Al+Pl|0,Bl=p,jl=Bl+(Hl<<3)|0,Fl=+HEAPF64[jl>>3],Il=Ll*Fl,ql=zg,Rl=Jg,zl=Rg,Wl=Math_imul(Rl,zl)|0,Xl=ql+Wl|0,Vl=p,$l=Vl+(Xl<<3)|0,HEAPF64[$l>>3]=Il,Jl=zg,Kl=Jl+1|0,zg=Kl}}while(0);Ql=Ol,Gl=HEAP32[Ql>>2]|0,Wg=Gl,Yl=Jg,ec=Yl+1|0,Kg=ec;for(;;){tc=Kg,nc=Wg,rc=(tc|0)<=(nc|0);if(!rc)break;ic=Kg,sc=Jg,oc=Ig,uc=Math_imul(sc,oc)|0,ac=ic+uc|0,fc=Bd,cc=fc+(ac<<3)|0,hc=+HEAPF64[cc>>3],pc=hc!=0;f:do if(pc){dc=Ca,vc=HEAP32[dc>>2]|0,Xg=vc,zg=1;for(;;){mc=zg,gc=Xg,yc=(mc|0)<=(gc|0);if(!yc)break f;bc=Kg,wc=Jg,Sc=Ig,xc=Math_imul(wc,Sc)|0,Tc=bc+xc|0,Nc=Bd,Cc=Nc+(Tc<<3)|0,kc=+HEAPF64[Cc>>3],Lc=zg,Ac=Kg,Oc=Rg,Mc=Math_imul(Ac,Oc)|0,Dc=Lc+Mc|0,Pc=p,Hc=Pc+(Dc<<3)|0,Bc=+HEAPF64[Hc>>3],jc=kc*Bc,Fc=zg,Ic=Jg,qc=Rg,Rc=Math_imul(Ic,qc)|0,Uc=Fc+Rc|0,Wc=p,Xc=Wc+(Uc<<3)|0,Vc=+HEAPF64[Xc>>3],$c=Vc-jc,HEAPF64[Xc>>3]=$c,Jc=zg,Kc=Jc+1|0,zg=Kc}}while(0);Qc=Kg,Gc=Qc+1|0,Kg=Gc}Yc=Gg,Zc=(Yc|0)!=0;l:do if(Zc){th=Jg,nh=Jg,rh=Ig,ih=Math_imul(nh,rh)|0,sh=th+ih|0,oh=Bd,uh=oh+(sh<<3)|0,ah=+HEAPF64[uh>>3],fh=1/ah,Zg=fh,lh=Ca,hh=HEAP32[lh>>2]|0,Wg=hh,zg=1;for(;;){ph=zg,dh=Wg,vh=(ph|0)<=(dh|0);if(!vh)break l;mh=Zg,gh=zg,yh=Jg,bh=Rg,wh=Math_imul(yh,bh)|0,Eh=gh+wh|0,xh=p,Th=xh+(Eh<<3)|0,Nh=+HEAPF64[Th>>3],Ch=mh*Nh,kh=zg,Lh=Jg,Ah=Rg,Oh=Math_imul(Lh,Ah)|0,Mh=kh+Oh|0,_h=p,Hh=_h+(Mh<<3)|0,HEAPF64[Hh>>3]=Ch,Bh=zg,jh=Bh+1|0,zg=jh}}while(0);Fh=Jg,Ih=Fh+ -1|0,Jg=Ih}}else if(R){Kg=z;for(;;){qh=Kg,Rh=(qh|0)>=1;if(!Rh)break e;Uh=Gg,zh=(Uh|0)!=0;c:do if(zh){Wh=Kg,Vh=Kg,$h=Ig,Jh=Math_imul(Vh,$h)|0,Kh=Wh+Jh|0,Qh=Bd,Gh=Qh+(Kh<<3)|0,Yh=+HEAPF64[Gh>>3],Zh=1/Yh,Zg=Zh,ep=Ca,tp=HEAP32[ep>>2]|0,Wg=tp,zg=1;for(;;){rp=zg,ip=Wg,sp=(rp|0)<=(ip|0);if(!sp)break c;op=Zg,up=zg,ap=Kg,fp=Rg,lp=Math_imul(ap,fp)|0,cp=up+lp|0,hp=p,dp=hp+(cp<<3)|0,vp=+HEAPF64[dp>>3],mp=op*vp,gp=zg,yp=Kg,bp=Rg,wp=Math_imul(yp,bp)|0,Ep=gp+wp|0,Sp=p,xp=Sp+(Ep<<3)|0,HEAPF64[xp>>3]=mp,Np=zg,Cp=Np+1|0,zg=Cp}}while(0);kp=Kg,Lp=kp-1|0,Wg=Lp,Jg=1;for(;;){Ap=Jg,Op=Wg,Mp=(Ap|0)<=(Op|0);if(!Mp)break;_p=Jg,Dp=Kg,Pp=Ig,Bp=Math_imul(Dp,Pp)|0,jp=_p+Bp|0,Fp=Bd,Ip=Fp+(jp<<3)|0,qp=+HEAPF64[Ip>>3],Rp=qp!=0;h:do if(Rp){Up=Jg,zp=Kg,Wp=Ig,Xp=Math_imul(zp,Wp)|0,$p=Up+Xp|0,Jp=Bd,Kp=Jp+($p<<3)|0,Qp=+HEAPF64[Kp>>3],Zg=Qp,Gp=Ca,Yp=HEAP32[Gp>>2]|0,Xg=Yp,zg=1;for(;;){Zp=zg,ed=Xg,td=(Zp|0)<=(ed|0);if(!td)break h;nd=Zg,id=zg,sd=Kg,od=Rg,ud=Math_imul(sd,od)|0,ad=id+ud|0,fd=p,ld=fd+(ad<<3)|0,cd=+HEAPF64[ld>>3],hd=nd*cd,pd=zg,vd=Jg,md=Rg,gd=Math_imul(vd,md)|0,yd=pd+gd|0,bd=p,wd=bd+(yd<<3)|0,Ed=+HEAPF64[wd>>3],Sd=Ed-hd,HEAPF64[wd>>3]=Sd,xd=zg,Td=xd+1|0,zg=Td}}while(0);Cd=Jg,kd=Cd+1|0,Jg=kd}Ld=Dh,Ad=+HEAPF64[Ld>>3],Od=Ad!=1;p:do if(Od){Md=Ca,_d=HEAP32[Md>>2]|0,Wg=_d,zg=1;for(;;){Dd=zg,Pd=Wg,Hd=(Dd|0)<=(Pd|0);if(!Hd)break p;Fd=Dh,Id=+HEAPF64[Fd>>3],qd=zg,Rd=Kg,Ud=Rg,zd=Math_imul(Rd,Ud)|0,Wd=qd+zd|0,Xd=p,Vd=Xd+(Wd<<3)|0,$d=+HEAPF64[Vd>>3],Kd=Id*$d,Qd=zg,Gd=Kg,Yd=Rg,Zd=Math_imul(Gd,Yd)|0,ev=Qd+Zd|0,tv=p,nv=tv+(ev<<3)|0,HEAPF64[nv>>3]=Kd,rv=zg,iv=rv+1|0,zg=iv}}while(0);ov=Kg,uv=ov+ -1|0,Kg=uv}}else{Wg=z,Kg=1;for(;;){av=Kg,fv=Wg,lv=(av|0)<=(fv|0);if(!lv)break e;cv=Gg,hv=(cv|0)!=0;d:do if(hv){pv=Kg,dv=Kg,vv=Ig,gv=Math_imul(dv,vv)|0,yv=pv+gv|0,bv=Bd,wv=bv+(yv<<3)|0,Ev=+HEAPF64[wv>>3],Sv=1/Ev,Zg=Sv,xv=Ca,Tv=HEAP32[xv>>2]|0,Xg=Tv,zg=1;for(;;){Nv=zg,Cv=Xg,Lv=(Nv|0)<=(Cv|0);if(!Lv)break d;Av=Zg,Ov=zg,Mv=Kg,_v=Rg,Dv=Math_imul(Mv,_v)|0,Pv=Ov+Dv|0,Hv=p,Bv=Hv+(Pv<<3)|0,jv=+HEAPF64[Bv>>3],Iv=Av*jv,qv=zg,Rv=Kg,Uv=Rg,zv=Math_imul(Rv,Uv)|0,Wv=qv+zv|0,Xv=p,Vv=Xv+(Wv<<3)|0,HEAPF64[Vv>>3]=Iv,$v=zg,Jv=$v+1|0,zg=Jv}}while(0);Qv=Ol,Gv=HEAP32[Qv>>2]|0,Xg=Gv,Yv=Kg,Zv=Yv+1|0,Jg=Zv;for(;;){em=Jg,tm=Xg,nm=(em|0)<=(tm|0);if(!nm)break;rm=Jg,im=Kg,sm=Ig,um=Math_imul(im,sm)|0,am=rm+um|0,fm=Bd,lm=fm+(am<<3)|0,cm=+HEAPF64[lm>>3],hm=cm!=0;v:do if(hm){pm=Jg,dm=Kg,vm=Ig,mm=Math_imul(dm,vm)|0,ym=pm+mm|0,bm=Bd,wm=bm+(ym<<3)|0,Em=+HEAPF64[wm>>3],Zg=Em,Sm=Ca,xm=HEAP32[Sm>>2]|0,Vg=xm,zg=1;for(;;){Tm=zg,Nm=Vg,Cm=(Tm|0)<=(Nm|0);if(!Cm)break v;km=Zg,Am=zg,Om=Kg,Mm=Rg,_m=Math_imul(Om,Mm)|0,Dm=Am+_m|0,Pm=p,Hm=Pm+(Dm<<3)|0,Bm=+HEAPF64[Hm>>3],jm=km*Bm,Fm=zg,Rm=Jg,Um=Rg,zm=Math_imul(Rm,Um)|0,Wm=Fm+zm|0,Xm=p,Vm=Xm+(Wm<<3)|0,$m=+HEAPF64[Vm>>3],Jm=$m-jm,HEAPF64[Vm>>3]=Jm,Km=zg,Qm=Km+1|0,zg=Qm}}while(0);Ym=Jg,Zm=Ym+1|0,Jg=Zm}eg=Dh,tg=+HEAPF64[eg>>3],ng=tg!=1;m:do if(ng){rg=Ca,ig=HEAP32[rg>>2]|0,Xg=ig,zg=1;for(;;){sg=zg,og=Xg,ug=(sg|0)<=(og|0);if(!ug)break m;fg=Dh,lg=+HEAPF64[fg>>3],cg=zg,hg=Kg,pg=Rg,dg=Math_imul(hg,pg)|0,vg=cg+dg|0,mg=p,gg=mg+(vg<<3)|0,yg=+HEAPF64[gg>>3],wg=lg*yg,Eg=zg,Sg=Kg,xg=Rg,Tg=Math_imul(Sg,xg)|0,Ng=Eg+Tg|0,Cg=p,kg=Cg+(Ng<<3)|0,HEAPF64[kg>>3]=wg,Lg=zg,Ag=Lg+1|0,zg=Ag}}while(0);Mg=Kg,_g=Mg+1|0,Kg=_g}}while(0);return c=0,Dg=c,STACKTOP=ny,Dg|0}}return c=0,Dg=c,STACKTOP=ny,Dg|0}function _xerbla_(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0;return l=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),u=l,n=e,r=t,i=n,s=r,o=HEAP32[s>>2]|0,HEAP32[u>>2]=i,a=u+4|0,HEAP32[a>>2]=o,_printf(20477,u)|0,STACKTOP=l,0}function _timer_(e){e|=0;var t=0,n=0,r=0,i=0;return i=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),t=e,n=t,HEAPF64[n>>3]=0,STACKTOP=i,0}function _dpofa_(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0;nn=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),en=nn,i=e,s=t,m=n,k=r,F=s,K=HEAP32[F>>2]|0,Qt=K,ot=Qt,gt=1+ot|0,Gt=gt,Lt=Gt,It=i,o=0-Lt|0,u=It+(o<<3)|0,i=u,a=m,f=HEAP32[a>>2]|0,Yt=f,HEAP32[3175]=1;for(;;){l=HEAP32[3175]|0,c=Yt,h=(l|0)<=(c|0);if(!h)break;p=HEAP32[3175]|0,d=k,HEAP32[d>>2]=p,HEAPF64[113]=0,v=HEAP32[3175]|0,g=v-1|0,HEAP32[3176]=g,y=HEAP32[3176]|0,b=(y|0)<1;e:do if(!b){w=HEAP32[3176]|0,Zt=w,HEAP32[3177]=1;for(;;){E=HEAP32[3177]|0,S=Zt,x=(E|0)<=(S|0);if(!x)break e;T=HEAP32[3177]|0,N=T-1|0,HEAP32[en>>2]=N,C=HEAP32[3177]|0,L=HEAP32[3175]|0,A=Qt,O=Math_imul(L,A)|0,M=C+O|0,_=i,D=_+(M<<3)|0,P=+HEAPF64[D>>3],H=HEAP32[3177]|0,B=Qt,j=Math_imul(H,B)|0,I=j+1|0,q=i,R=q+(I<<3)|0,U=HEAP32[3175]|0,z=Qt,W=Math_imul(U,z)|0,X=W+1|0,V=i,$=V+(X<<3)|0,J=+_ddot_(en,R,12712,$,12712),Q=P-J,HEAPF64[114]=Q,G=HEAP32[3177]|0,Y=HEAP32[3177]|0,Z=Qt,et=Math_imul(Y,Z)|0,tt=G+et|0,nt=i,rt=nt+(tt<<3)|0,it=+HEAPF64[rt>>3],st=+HEAPF64[114],ut=st/it,HEAPF64[114]=ut,at=+HEAPF64[114],ft=HEAP32[3177]|0,lt=HEAP32[3175]|0,ct=Qt,ht=Math_imul(lt,ct)|0,pt=ft+ht|0,dt=i,vt=dt+(pt<<3)|0,HEAPF64[vt>>3]=at,mt=+HEAPF64[114],yt=+HEAPF64[114],bt=mt*yt,wt=+HEAPF64[113],Et=wt+bt,HEAPF64[113]=Et,St=HEAP32[3177]|0,xt=St+1|0,HEAP32[3177]=xt}}while(0);Tt=HEAP32[3175]|0,Nt=HEAP32[3175]|0,Ct=Qt,kt=Math_imul(Nt,Ct)|0,At=Tt+kt|0,Ot=i,Mt=Ot+(At<<3)|0,_t=+HEAPF64[Mt>>3],Dt=+HEAPF64[113],Pt=_t-Dt,HEAPF64[113]=Pt,Ht=+HEAPF64[113],Bt=Ht<=0;if(Bt){tn=10;break}jt=+HEAPF64[113],Ft=+Math_sqrt(+jt),qt=HEAP32[3175]|0,Rt=HEAP32[3175]|0,Ut=Qt,zt=Math_imul(Rt,Ut)|0,Wt=qt+zt|0,Xt=i,Vt=Xt+(Wt<<3)|0,HEAPF64[Vt>>3]=Ft,$t=HEAP32[3175]|0,Jt=$t+1|0,HEAP32[3175]=Jt}return(tn|0)==10?(STACKTOP=nn,0):(Kt=k,HEAP32[Kt>>2]=0,STACKTOP=nn,0)}function _dtrsl_(e,t,n,r,i,s){e|=0,t|=0,n|=0,r|=0,i|=0,s|=0;var o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0;gs=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),ps=gs,u=e,a=t,hn=n,zr=r,ei=i,ci=s,Si=a,Di=HEAP32[Si>>2]|0,ds=Di,Wi=ds,ts=1+Wi|0,vs=ts,f=vs,w=u,M=0-f|0,U=w+(M<<3)|0,u=U,Z=zr,lt=Z+ -8|0,zr=lt,Et=hn,_t=HEAP32[Et>>2]|0,hs=_t,zt=ci,HEAP32[zt>>2]=1;for(;;){en=ci,pn=HEAP32[en>>2]|0,Tn=hs,Hn=(pn|0)<=(Tn|0),Vn=ci;if(!Hn)break;rr=HEAP32[Vn>>2]|0,dr=ci,Nr=HEAP32[dr>>2]|0,Br=ds,Rr=Math_imul(Nr,Br)|0,Ur=rr+Rr|0,Wr=u,Xr=Wr+(Ur<<3)|0,Vr=+HEAPF64[Xr>>3],$r=Vr==0;if($r){ms=24;break}Jr=ci,Kr=HEAP32[Jr>>2]|0,Qr=Kr+1|0,HEAP32[Jr>>2]=Qr}if((ms|0)==24)return STACKTOP=gs,0;HEAP32[Vn>>2]=0,HEAP32[3179]=1,Gr=ei,Yr=HEAP32[Gr>>2]|0,Zr=(Yr|0)%10&-1,ti=(Zr|0)!=0,o=ti?2:1,HEAP32[3179]=o,ni=ei,ri=HEAP32[ni>>2]|0,ii=(ri|0)%100&-1,si=(ii|0)/10&-1,oi=(si|0)!=0,oi&&(ui=HEAP32[3179]|0,ai=ui+2|0,HEAP32[3179]=ai),fi=HEAP32[3179]|0;switch(fi|0){case 4:Un=ds,zn=Un+1|0,Wn=u,Xn=Wn+(zn<<3)|0,$n=+HEAPF64[Xn>>3],Jn=zr,Kn=Jn+8|0,Qn=+HEAPF64[Kn>>3],Gn=Qn/$n,HEAPF64[Kn>>3]=Gn,Yn=hn,Zn=HEAP32[Yn>>2]|0,er=(Zn|0)<2;if(er)return STACKTOP=gs,0;tr=hn,nr=HEAP32[tr>>2]|0,hs=nr,HEAP32[3180]=2;for(;;){ir=HEAP32[3180]|0,sr=hs,or=(ir|0)<=(sr|0);if(!or)break;ur=HEAP32[3180]|0,ar=ur-1|0,HEAP32[ps>>2]=ar,fr=HEAP32[3180]|0,lr=ds,cr=Math_imul(fr,lr)|0,hr=cr+1|0,pr=u,vr=pr+(hr<<3)|0,mr=zr,gr=mr+8|0,yr=+_ddot_(ps,vr,12712,gr,12712),br=HEAP32[3180]|0,wr=zr,Er=wr+(br<<3)|0,Sr=+HEAPF64[Er>>3],xr=Sr-yr,HEAPF64[Er>>3]=xr,Tr=HEAP32[3180]|0,Cr=HEAP32[3180]|0,kr=ds,Lr=Math_imul(Cr,kr)|0,Ar=Tr+Lr|0,Or=u,Mr=Or+(Ar<<3)|0,_r=+HEAPF64[Mr>>3],Dr=HEAP32[3180]|0,Pr=zr,Hr=Pr+(Dr<<3)|0,jr=+HEAPF64[Hr>>3],Fr=jr/_r,HEAPF64[Hr>>3]=Fr,Ir=HEAP32[3180]|0,qr=Ir+1|0,HEAP32[3180]=qr}return STACKTOP=gs,0;case 2:p=hn,d=HEAP32[p>>2]|0,v=hn,m=HEAP32[v>>2]|0,g=ds,y=Math_imul(m,g)|0,b=d+y|0,E=u,S=E+(b<<3)|0,x=+HEAPF64[S>>3],T=hn,N=HEAP32[T>>2]|0,C=zr,k=C+(N<<3)|0,L=+HEAPF64[k>>3],A=L/x,HEAPF64[k>>3]=A,O=hn,_=HEAP32[O>>2]|0,D=(_|0)<2;if(D)return STACKTOP=gs,0;P=hn,H=HEAP32[P>>2]|0,hs=H,HEAP32[3181]=2;for(;;){B=HEAP32[3181]|0,j=hs,F=(B|0)<=(j|0);if(!F)break;I=hn,q=HEAP32[I>>2]|0,R=HEAP32[3181]|0,z=q-R|0,W=z+1|0,HEAP32[3180]=W,X=HEAP32[3180]|0,V=X+1|0,$=zr,J=$+(V<<3)|0,K=+HEAPF64[J>>3],Q=-K,HEAPF64[115]=Q,G=HEAP32[3180]|0,Y=G+1|0,et=ds,tt=Math_imul(Y,et)|0,nt=tt+1|0,rt=u,it=rt+(nt<<3)|0,st=zr,ot=st+8|0,_daxpy_(12720,920,it,12712,ot,12712)|0,ut=HEAP32[3180]|0,at=HEAP32[3180]|0,ft=ds,ct=Math_imul(at,ft)|0,ht=ut+ct|0,pt=u,dt=pt+(ht<<3)|0,vt=+HEAPF64[dt>>3],mt=HEAP32[3180]|0,gt=zr,yt=gt+(mt<<3)|0,bt=+HEAPF64[yt>>3],wt=bt/vt,HEAPF64[yt>>3]=wt,St=HEAP32[3181]|0,xt=St+1|0,HEAP32[3181]=xt}return STACKTOP=gs,0;case 3:Tt=hn,Nt=HEAP32[Tt>>2]|0,Ct=hn,kt=HEAP32[Ct>>2]|0,Lt=ds,At=Math_imul(kt,Lt)|0,Ot=Nt+At|0,Mt=u,Dt=Mt+(Ot<<3)|0,Pt=+HEAPF64[Dt>>3],Ht=hn,Bt=HEAP32[Ht>>2]|0,jt=zr,Ft=jt+(Bt<<3)|0,It=+HEAPF64[Ft>>3],qt=It/Pt,HEAPF64[Ft>>3]=qt,Rt=hn,Ut=HEAP32[Rt>>2]|0,Wt=(Ut|0)<2;if(Wt)return STACKTOP=gs,0;Xt=hn,Vt=HEAP32[Xt>>2]|0,hs=Vt,HEAP32[3181]=2;for(;;){$t=HEAP32[3181]|0,Jt=hs,Kt=($t|0)<=(Jt|0);if(!Kt)break;Qt=hn,Gt=HEAP32[Qt>>2]|0,Yt=HEAP32[3181]|0,Zt=Gt-Yt|0,tn=Zt+1|0,HEAP32[3180]=tn,nn=HEAP32[3181]|0,rn=nn-1|0,HEAP32[ps>>2]=rn,sn=HEAP32[3180]|0,on=sn+1|0,un=HEAP32[3180]|0,an=ds,fn=Math_imul(un,an)|0,ln=on+fn|0,cn=u,dn=cn+(ln<<3)|0,vn=HEAP32[3180]|0,mn=vn+1|0,gn=zr,yn=gn+(mn<<3)|0,bn=+_ddot_(ps,dn,12712,yn,12712),wn=HEAP32[3180]|0,En=zr,Sn=En+(wn<<3)|0,xn=+HEAPF64[Sn>>3],Nn=xn-bn,HEAPF64[Sn>>3]=Nn,Cn=HEAP32[3180]|0,kn=HEAP32[3180]|0,Ln=ds,An=Math_imul(kn,Ln)|0,On=Cn+An|0,Mn=u,_n=Mn+(On<<3)|0,Dn=+HEAPF64[_n>>3],Pn=HEAP32[3180]|0,Bn=zr,jn=Bn+(Pn<<3)|0,Fn=+HEAPF64[jn>>3],In=Fn/Dn,HEAPF64[jn>>3]=In,qn=HEAP32[3181]|0,Rn=qn+1|0,HEAP32[3181]=Rn}return STACKTOP=gs,0;default:li=ds,hi=li+1|0,pi=u,di=pi+(hi<<3)|0,vi=+HEAPF64[di>>3],mi=zr,gi=mi+8|0,yi=+HEAPF64[gi>>3],bi=yi/vi,HEAPF64[gi>>3]=bi,wi=hn,Ei=HEAP32[wi>>2]|0,xi=(Ei|0)<2;if(xi)return STACKTOP=gs,0;Ti=hn,Ni=HEAP32[Ti>>2]|0,hs=Ni,HEAP32[3180]=2;for(;;){Ci=HEAP32[3180]|0,ki=hs,Li=(Ci|0)<=(ki|0);if(!Li)break;Ai=HEAP32[3180]|0,Oi=Ai-1|0,Mi=zr,_i=Mi+(Oi<<3)|0,Pi=+HEAPF64[_i>>3],Hi=-Pi,HEAPF64[115]=Hi,Bi=hn,ji=HEAP32[Bi>>2]|0,Fi=HEAP32[3180]|0,Ii=ji-Fi|0,qi=Ii+1|0,HEAP32[ps>>2]=qi,Ri=HEAP32[3180]|0,Ui=HEAP32[3180]|0,zi=Ui-1|0,Xi=ds,Vi=Math_imul(zi,Xi)|0,$i=Ri+Vi|0,Ji=u,Ki=Ji+($i<<3)|0,Qi=HEAP32[3180]|0,Gi=zr,Yi=Gi+(Qi<<3)|0,_daxpy_(ps,920,Ki,12712,Yi,12712)|0,Zi=HEAP32[3180]|0,es=HEAP32[3180]|0,ns=ds,rs=Math_imul(es,ns)|0,is=Zi+rs|0,ss=u,os=ss+(is<<3)|0,us=+HEAPF64[os>>3],as=HEAP32[3180]|0,fs=zr,ls=fs+(as<<3)|0,cs=+HEAPF64[ls>>3],l=cs/us,HEAPF64[ls>>3]=l,c=HEAP32[3180]|0,h=c+1|0,HEAP32[3180]=h}return STACKTOP=gs,0}return 0}function _unlink(e){e|=0;var t=0,n=0,r=0,i=0,s=0;return s=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),r=s,HEAP32[r>>2]=e,t=___syscall10(10,r|0)|0,n=___syscall_ret(t)|0,STACKTOP=s,n|0}function ___dup3(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0;T=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),g=T+24|0,m=T+16|0,v=T,o=(e|0)==(t|0);e:do if(o)u=___syscall_ret(-22)|0,r=u;else{a=n&524288,f=(a|0)!=0;t:do if(f){n:for(;;){HEAP32[v>>2]=e,y=v+4|0,HEAP32[y>>2]=t,E=v+8|0,HEAP32[E>>2]=n,l=___syscall330(330,v|0)|0;switch(l|0){case-38:break t;case-16:break;default:s=l;break n}}c=___syscall_ret(s)|0,r=c;break e}while(0);for(;;){HEAP32[m>>2]=e,S=m+4|0,HEAP32[S>>2]=t,h=___syscall63(63,m|0)|0,p=(h|0)==-16;if(!p){i=h;break}}f&&(HEAP32[g>>2]=t,b=g+4|0,HEAP32[b>>2]=2,w=g+8|0,HEAP32[w>>2]=1,___syscall221(221,g|0)|0),d=___syscall_ret(i)|0,r=d}while(0);return STACKTOP=T,r|0}function _ftruncate(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0;return c=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),o=c,n=(t|0)<0,r=n<<31>>31,HEAP32[o>>2]=e,u=o+4|0,HEAP32[u>>2]=0,a=o+8|0,HEAP32[a>>2]=t,f=o+12|0,HEAP32[f>>2]=r,i=___syscall194(194,o|0)|0,s=___syscall_ret(i)|0,STACKTOP=c,s|0}function _isatty(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=o,t=_tcgetattr(e,i)|0,n=(t|0)==0,r=n&1,STACKTOP=o,r|0}function _access(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=u,HEAP32[i>>2]=e,s=i+4|0,HEAP32[s>>2]=t,n=___syscall33(33,i|0)|0,r=___syscall_ret(n)|0,STACKTOP=u,r|0}function _printf(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=o,HEAP32[i>>2]=t,n=HEAP32[3182]|0,r=_vfprintf(n,e,i)|0,STACKTOP=o,r|0}function _vsnprintf(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0;_=STACKTOP,STACKTOP=STACKTOP+128|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),L=_+112|0,A=_,O=A,D=12788,P=O+112|0;do HEAP32[O>>2]=HEAP32[D>>2]|0,O=O+4|0,D=D+4|0;while((O|0)<(P|0));return a=t+ -1|0,f=a>>>0>2147483646,f?(w=(t|0)==0,w?(o=L,u=1,M=4):(E=___errno_location()|0,HEAP32[E>>2]=75,s=-1)):(o=e,u=t,M=4),(M|0)==4&&(S=o,x=-2-S|0,T=u>>>0>x>>>0,i=T?x:u,N=A+48|0,HEAP32[N>>2]=i,C=A+20|0,HEAP32[C>>2]=o,k=A+44|0,HEAP32[k>>2]=o,l=o+i|0,c=A+16|0,HEAP32[c>>2]=l,h=A+28|0,HEAP32[h>>2]=l,p=_vfprintf(A,n,r)|0,d=(i|0)==0,d?s=p:(v=HEAP32[C>>2]|0,m=HEAP32[c>>2]|0,g=(v|0)==(m|0),y=g<<31>>31,b=v+y|0,HEAP8[b>>0]=0,s=p)),STACKTOP=_,s|0}function _fflush(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0;F=STACKTOP,s=(e|0)==(0|0);do if(s){O=HEAP32[3183]|0,M=(O|0)==(0|0),M?N=0:(_=HEAP32[3183]|0,u=_fflush(_)|0,N=u),___lock(12772),r=HEAP32[3192]|0,a=(r|0)==(0|0);if(a)P=N;else{i=r,H=N;for(;;){f=i+76|0,l=HEAP32[f>>2]|0,c=(l|0)>-1,c?(h=___lockfile(i)|0,E=h):E=0,p=i+20|0,d=HEAP32[p>>2]|0,v=i+28|0,m=HEAP32[v>>2]|0,y=d>>>0>m>>>0,y?(b=___fflush_unlocked(i)|0,w=b|H,B=w):B=H,S=(E|0)==0,S||___unlockfile(i),x=i+56|0,n=HEAP32[x>>2]|0,T=(n|0)==(0|0);if(T){P=B;break}i=n,H=B}}___unlock(12772),t=P}else{o=e+76|0,g=HEAP32[o>>2]|0,C=(g|0)>-1;if(!C){k=___fflush_unlocked(e)|0,t=k;break}L=___lockfile(e)|0,D=(L|0)==0,A=___fflush_unlocked(e)|0,D?t=A:(___unlockfile(e),t=A)}while(0);return t|0}function ___towrite(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0;return T=STACKTOP,n=e+74|0,r=HEAP8[n>>0]|0,d=r<<24>>24,m=d+255|0,g=m|d,y=g&255,HEAP8[n>>0]=y,b=HEAP32[e>>2]|0,w=b&8,E=(w|0)==0,E?(i=e+8|0,HEAP32[i>>2]=0,s=e+4|0,HEAP32[s>>2]=0,o=e+44|0,u=HEAP32[o>>2]|0,a=e+28|0,HEAP32[a>>2]=u,f=e+20|0,HEAP32[f>>2]=u,l=u,c=e+48|0,h=HEAP32[c>>2]|0,p=l+h|0,v=e+16|0,HEAP32[v>>2]=p,t=0):(S=b|32,HEAP32[e>>2]=S,t=-1),t|0}function ___ftello_unlocked(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0;return D=STACKTOP,n=e+40|0,r=HEAP32[n>>2]|0,d=HEAP32[e>>2]|0,T=d&128,N=(T|0)==0,N?i=1:(C=e+20|0,k=HEAP32[C>>2]|0,L=e+28|0,A=HEAP32[L>>2]|0,O=k>>>0>A>>>0,M=O?2:1,i=M),s=FUNCTION_TABLE_iiii[r&15](e,0,i)|0,o=(s|0)<0,o?t=s:(u=e+8|0,a=HEAP32[u>>2]|0,f=e+4|0,l=HEAP32[f>>2]|0,c=a,h=l,p=e+20|0,v=HEAP32[p>>2]|0,m=e+28|0,g=HEAP32[m>>2]|0,y=v,b=g,w=s-c|0,E=w+h|0,S=E+y|0,x=S-b|0,t=x),t|0}function ___ftello(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0;return l=STACKTOP,t=e+76|0,n=HEAP32[t>>2]|0,r=(n|0)>-1,r?(s=___lockfile(e)|0,a=(s|0)==0,o=___ftello_unlocked(e)|0,a?u=o:(___unlockfile(e),u=o)):(i=___ftello_unlocked(e)|0,u=i),u|0}function _ftell(e){e|=0;var t=0,n=0,r=0;return r=STACKTOP,t=___ftello(e)|0,t|0}function ___stdout_write(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0;return y=STACKTOP,STACKTOP=STACKTOP+80|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),d=y,p=y+12|0,r=e+36|0,HEAP32[r>>2]=5,i=HEAP32[e>>2]|0,s=i&64,o=(s|0)==0,o&&(u=e+60|0,a=HEAP32[u>>2]|0,HEAP32[d>>2]=a,v=d+4|0,HEAP32[v>>2]=21505,m=d+8|0,HEAP32[m>>2]=p,f=___syscall54(54,d|0)|0,l=(f|0)==0,l||(c=e+75|0,HEAP8[c>>0]=-1)),h=___stdio_write(e,t,n)|0,STACKTOP=y,h|0}function ___fdopen(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0;at=STACKTOP,STACKTOP=STACKTOP+112|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Q=at+40|0,Y=at+24|0,G=at+16|0,K=at,J=at+52|0,s=HEAP8[t>>0]|0,o=s<<24>>24,$=_memchr(20543,o,4)|0,g=($|0)==(0|0);if(g)L=___errno_location()|0,HEAP32[L>>2]=22,n=0;else{I=_malloc(1144)|0,U=(I|0)==(0|0);if(U)n=0;else{ot=I,ft=ot+112|0;do HEAP32[ot>>2]=0,ot=ot+4|0;while((ot|0)<(ft|0));z=_strchr(t,43)|0,W=(z|0)==(0|0),W&&(X=s<<24>>24==114,V=X?8:4,HEAP32[I>>2]=V),u=_strchr(t,101)|0,a=(u|0)==(0|0),a?f=s:(HEAP32[K>>2]=e,Z=K+4|0,HEAP32[Z>>2]=2,it=K+8|0,HEAP32[it>>2]=1,___syscall221(221,K|0)|0,r=HEAP8[t>>0]|0,f=r),l=f<<24>>24==97,l?(HEAP32[G>>2]=e,st=G+4|0,HEAP32[st>>2]=3,c=___syscall221(221,G|0)|0,h=c&1024,p=(h|0)==0,p&&(d=c|1024,HEAP32[Y>>2]=e,et=Y+4|0,HEAP32[et>>2]=4,tt=Y+8|0,HEAP32[tt>>2]=d,___syscall221(221,Y|0)|0),v=HEAP32[I>>2]|0,m=v|128,HEAP32[I>>2]=m,T=m):(i=HEAP32[I>>2]|0,T=i),y=I+60|0,HEAP32[y>>2]=e,b=I+120|0,w=I+44|0,HEAP32[w>>2]=b,E=I+48|0,HEAP32[E>>2]=1024,S=I+75|0,HEAP8[S>>0]=-1,x=T&8,N=(x|0)==0,N&&(HEAP32[Q>>2]=e,nt=Q+4|0,HEAP32[nt>>2]=21505,rt=Q+8|0,HEAP32[rt>>2]=J,C=___syscall54(54,Q|0)|0,k=(C|0)==0,k&&(HEAP8[S>>0]=10)),A=I+32|0,HEAP32[A>>2]=6,O=I+36|0,HEAP32[O>>2]=5,M=I+40|0,HEAP32[M>>2]=4,_=I+12|0,HEAP32[_>>2]=2,D=HEAP32[3187]|0,P=(D|0)==0,P&&(H=I+76|0,HEAP32[H>>2]=-1),___lock(12772),B=HEAP32[3192]|0,j=I+56|0,HEAP32[j>>2]=B,F=(B|0)==0,F||(q=B,R=q+52|0,HEAP32[R>>2]=I),HEAP32[3192]=I,___unlock(12772),n=I}}return STACKTOP=at,n|0}function _sprintf(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=o,HEAP32[i>>2]=n,r=_vsprintf(e,t,i)|0,STACKTOP=o,r|0}function _fputc(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0;B=STACKTOP,r=t+76|0,i=HEAP32[r>>2]|0,v=(i|0)<0,v?H=3:(C=___lockfile(t)|0,A=(C|0)==0,A?H=3:(p=t+75|0,d=HEAP8[p>>0]|0,m=d<<24>>24,g=(m|0)==(e|0),g?H=10:(y=t+20|0,b=HEAP32[y>>2]|0,w=t+16|0,E=HEAP32[w>>2]|0,S=b>>>0<E>>>0,S?(x=e&255,T=b+1|0,HEAP32[y>>2]=T,HEAP8[b>>0]=x,N=e&255,L=N):H=10),(H|0)==10&&(k=___overflow(t,e)|0,L=k),___unlockfile(t),n=L));do if((H|0)==3){O=t+75|0,M=HEAP8[O>>0]|0,_=M<<24>>24,D=(_|0)==(e|0);if(!D){P=t+20|0,s=HEAP32[P>>2]|0,o=t+16|0,u=HEAP32[o>>2]|0,a=s>>>0<u>>>0;if(a){f=e&255,l=s+1|0,HEAP32[P>>2]=l,HEAP8[s>>0]=f,c=e&255,n=c;break}}h=___overflow(t,e)|0,n=h}while(0);return n|0}function _fileno(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0;return f=STACKTOP,t=e+76|0,n=HEAP32[t>>2]|0,r=(n|0)>-1,r&&(i=___lockfile(e)|0,u=(i|0)==0,u||___unlockfile(e)),s=e+60|0,o=HEAP32[s>>2]|0,o|0}function ___fseeko_unlocked(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0;return D=STACKTOP,s=(n|0)==1,s?(o=e+8|0,g=HEAP32[o>>2]|0,N=e+4|0,C=HEAP32[N>>2]|0,k=g,L=C,A=t-k|0,O=A+L|0,i=O):i=t,M=e+20|0,u=HEAP32[M>>2]|0,a=e+28|0,f=HEAP32[a>>2]|0,l=u>>>0>f>>>0,l?(c=e+36|0,h=HEAP32[c>>2]|0,FUNCTION_TABLE_iiii[h&15](e,0,0)|0,p=HEAP32[M>>2]|0,d=(p|0)==(0|0),d?r=-1:_=5):_=5,(_|0)==5&&(v=e+16|0,HEAP32[v>>2]=0,HEAP32[a>>2]=0,HEAP32[M>>2]=0,m=e+40|0,y=HEAP32[m>>2]|0,b=FUNCTION_TABLE_iiii[y&15](e,i,n)|0,w=(b|0)<0,w?r=-1:(E=e+8|0,HEAP32[E>>2]=0,S=e+4|0,HEAP32[S>>2]=0,x=HEAP32[e>>2]|0,T=x&-17,HEAP32[e>>2]=T,r=0)),r|0}function ___fseeko(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0;return h=STACKTOP,r=e+76|0,i=HEAP32[r>>2]|0,s=(i|0)>-1,s?(u=___lockfile(e)|0,l=(u|0)==0,a=___fseeko_unlocked(e,t,n)|0,l?f=a:(___unlockfile(e),f=a)):(o=___fseeko_unlocked(e,t,n)|0,f=o),f|0}function _fseek(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0;return s=STACKTOP,r=___fseeko(e,t,n)|0,r|0}function _tmpfile(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0;y=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),d=y+16|0,p=y,c=y+20|0,h=0;for(;;){s=_tmpnam(c)|0,o=(s|0)==(0|0);if(o){e=0;break}HEAP32[p>>2]=s,v=p+4|0,HEAP32[v>>2]=32962,m=p+8|0,HEAP32[m>>2]=384,u=___syscall5(5,p|0)|0,a=___syscall_ret(u)|0,f=(a|0)>-1,r=h+1|0;if(f){t=a,n=s,g=5;break}i=(r|0)<100;if(!i){e=0;break}h=r}return(g|0)==5&&(l=___fdopen(t,20540)|0,HEAP32[d>>2]=n,___syscall10(10,d|0)|0,e=l),STACKTOP=y,e|0}function ___overflow(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0;L=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),C=L,i=t&255,HEAP8[C>>0]=i,s=e+16|0,m=HEAP32[s>>2]|0,b=(m|0)==(0|0),b?(w=___towrite(e)|0,E=(w|0)==0,E?(r=HEAP32[s>>2]|0,N=r,k=4):n=-1):(N=m,k=4);do if((k|0)==4){S=e+20|0,x=HEAP32[S>>2]|0,T=x>>>0<N>>>0;if(T){o=t&255,u=e+75|0,a=HEAP8[u>>0]|0,f=a<<24>>24,l=(o|0)==(f|0);if(!l){c=x+1|0,HEAP32[S>>2]=c,HEAP8[x>>0]=i,n=o;break}}h=e+36|0,p=HEAP32[h>>2]|0,d=FUNCTION_TABLE_iiii[p&15](e,C,1)|0,v=(d|0)==1,v?(g=HEAP8[C>>0]|0,y=g&255,n=y):n=-1}while(0);return STACKTOP=L,n|0}function _perror(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0;g=STACKTOP,t=HEAP32[3184]|0,n=___errno_location()|0,a=HEAP32[n>>2]|0,f=_strerror(a)|0,l=t+76|0,c=HEAP32[l>>2]|0,h=(c|0)>-1,h?(p=___lockfile(t)|0,o=p):o=0,d=(e|0)==(0|0),d||(v=HEAP8[e>>0]|0,r=v<<24>>24==0,r||(i=_strlen(e)|0,_fwrite(e,i,1,t)|0,_fputc(58,t)|0,_fputc(32,t)|0)),s=_strlen(f)|0,_fwrite(f,s,1,t)|0,_fputc(10,t)|0,u=(o|0)==0,u||___unlockfile(t);return}function _snprintf(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=u,HEAP32[s>>2]=r,i=_vsnprintf(e,t,n,s)|0,STACKTOP=u,i|0}function ___stdio_write(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0;wt=STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),dt=wt+16|0,pt=wt,ct=wt+32|0,o=e+28|0,u=HEAP32[o>>2]|0,HEAP32[ct>>2]=u,y=ct+4|0,A=e+20|0,q=HEAP32[A>>2]|0,G=q,Z=G-u|0,HEAP32[y>>2]=Z,et=ct+8|0,HEAP32[et>>2]=t,tt=ct+12|0,HEAP32[tt>>2]=n,nt=Z+n|0,a=e+60|0,f=e+44|0,st=ct,at=2,ht=nt;for(;;){l=HEAP32[3186]|0,c=(l|0)==(0|0),c?(v=HEAP32[a>>2]|0,HEAP32[dt>>2]=v,gt=dt+4|0,HEAP32[gt>>2]=st,yt=dt+8|0,HEAP32[yt>>2]=at,m=___syscall146(146,dt|0)|0,g=___syscall_ret(m)|0,rt=g):(_pthread_cleanup_push(14,e|0),h=HEAP32[a>>2]|0,HEAP32[pt>>2]=h,vt=pt+4|0,HEAP32[vt>>2]=st,mt=pt+8|0,HEAP32[mt>>2]=at,p=___syscall146(146,pt|0)|0,d=___syscall_ret(p)|0,_pthread_cleanup_pop(0),rt=d),b=(ht|0)==(rt|0);if(b){bt=6;break}C=(rt|0)<0;if(C){ot=st,ft=at,bt=8;break}H=ht-rt|0,B=st+4|0,j=HEAP32[B>>2]|0,F=rt>>>0>j>>>0,F?(I=HEAP32[f>>2]|0,HEAP32[o>>2]=I,HEAP32[A>>2]=I,R=rt-j|0,U=st+8|0,z=at+ -1|0,i=st+12|0,s=HEAP32[i>>2]|0,Y=s,it=R,ut=U,lt=z):(W=(at|0)==2,W?(X=HEAP32[o>>2]|0,V=X+rt|0,HEAP32[o>>2]=V,Y=j,it=rt,ut=st,lt=2):(Y=j,it=rt,ut=st,lt=at)),$=HEAP32[ut>>2]|0,J=$+it|0,HEAP32[ut>>2]=J,K=ut+4|0,Q=Y-it|0,HEAP32[K>>2]=Q,st=ut,at=lt,ht=H}return(bt|0)==6?(w=HEAP32[f>>2]|0,E=e+48|0,S=HEAP32[E>>2]|0,x=w+S|0,T=e+16|0,HEAP32[T>>2]=x,N=w,HEAP32[o>>2]=N,HEAP32[A>>2]=N,r=n):(bt|0)==8&&(k=e+16|0,HEAP32[k>>2]=0,HEAP32[o>>2]=0,HEAP32[A>>2]=0,L=HEAP32[e>>2]|0,O=L|32,HEAP32[e>>2]=O,M=(ft|0)==2,M?r=0:(_=ot+4|0,D=HEAP32[_>>2]|0,P=n-D|0,r=P)),STACKTOP=wt,r|0}function _fopen(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0;return w=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),m=w+16|0,v=w,r=HEAP8[t>>0]|0,i=r<<24>>24,d=_memchr(20543,i,4)|0,o=(d|0)==(0|0),o?(u=___errno_location()|0,HEAP32[u>>2]=22,n=0):(a=___fmodeflags(t)|0,f=a|32768,HEAP32[v>>2]=e,g=v+4|0,HEAP32[g>>2]=f,y=v+8|0,HEAP32[y>>2]=438,l=___syscall5(5,v|0)|0,c=___syscall_ret(l)|0,h=(c|0)<0,h?n=0:(p=___fdopen(c,t)|0,s=(p|0)==(0|0),s?(HEAP32[m>>2]=c,___syscall6(6,m|0)|0,n=0):n=p)),STACKTOP=w,n|0}function ___fmodeflags(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0;return N=STACKTOP,r=_strchr(e,43)|0,i=(r|0)==(0|0),l=HEAP8[e>>0]|0,x=l<<24>>24!=114,t=x&1,y=i?t:2,c=_strchr(e,120)|0,h=(c|0)==(0|0),p=y|128,b=h?y:p,d=_strchr(e,101)|0,v=(d|0)==(0|0),m=b|524288,w=v?b:m,g=l<<24>>24==114,s=w|64,E=g?w:s,o=l<<24>>24==119,u=E|512,S=o?u:E,a=l<<24>>24==97,f=S|1024,n=a?f:S,n|0}function ___lockfile(e){e|=0;var t=0,n=0;return n=STACKTOP,0}function ___unlockfile(e){e|=0;var t=0,n=0;n=STACKTOP;return}function ___stdio_read(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0;return et=
STACKTOP,STACKTOP=STACKTOP+48|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),J=et+16|0,$=et,V=et+32|0,HEAP32[V>>2]=t,i=V+4|0,s=e+48|0,m=HEAP32[s>>2]|0,k=(m|0)!=0,F=k&1,q=n-F|0,HEAP32[i>>2]=q,R=V+8|0,U=e+44|0,z=HEAP32[U>>2]|0,HEAP32[R>>2]=z,W=V+12|0,HEAP32[W>>2]=m,o=HEAP32[3186]|0,u=(o|0)==(0|0),u?(h=e+60|0,p=HEAP32[h>>2]|0,HEAP32[J>>2]=p,G=J+4|0,HEAP32[G>>2]=V,Y=J+8|0,HEAP32[Y>>2]=2,d=___syscall145(145,J|0)|0,v=___syscall_ret(d)|0,X=v):(_pthread_cleanup_push(15,e|0),a=e+60|0,f=HEAP32[a>>2]|0,HEAP32[$>>2]=f,K=$+4|0,HEAP32[K>>2]=V,Q=$+8|0,HEAP32[Q>>2]=2,l=___syscall145(145,$|0)|0,c=___syscall_ret(l)|0,_pthread_cleanup_pop(0),X=c),g=(X|0)<1,g?(y=X&48,b=y^16,w=HEAP32[e>>2]|0,E=w|b,HEAP32[e>>2]=E,S=e+8|0,HEAP32[S>>2]=0,x=e+4|0,HEAP32[x>>2]=0,r=X):(T=HEAP32[i>>2]|0,N=X>>>0>T>>>0,N?(C=X-T|0,L=HEAP32[U>>2]|0,A=e+4|0,HEAP32[A>>2]=L,O=L,M=O+C|0,_=e+8|0,HEAP32[_>>2]=M,D=HEAP32[s>>2]|0,P=(D|0)==0,P?r=n:(H=O+1|0,HEAP32[A>>2]=H,B=HEAP8[O>>0]|0,j=n+ -1|0,I=t+j|0,HEAP8[I>>0]=B,r=n)):r=X),STACKTOP=et,r|0}function _vsprintf(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0;return s=STACKTOP,r=_vsnprintf(e,2147483647,t,n)|0,r|0}function _putc(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0;B=STACKTOP,r=t+76|0,i=HEAP32[r>>2]|0,v=(i|0)<0,v?H=3:(C=___lockfile(t)|0,A=(C|0)==0,A?H=3:(p=t+75|0,d=HEAP8[p>>0]|0,m=d<<24>>24,g=(m|0)==(e|0),g?H=10:(y=t+20|0,b=HEAP32[y>>2]|0,w=t+16|0,E=HEAP32[w>>2]|0,S=b>>>0<E>>>0,S?(x=e&255,T=b+1|0,HEAP32[y>>2]=T,HEAP8[b>>0]=x,N=e&255,L=N):H=10),(H|0)==10&&(k=___overflow(t,e)|0,L=k),___unlockfile(t),n=L));do if((H|0)==3){O=t+75|0,M=HEAP8[O>>0]|0,_=M<<24>>24,D=(_|0)==(e|0);if(!D){P=t+20|0,s=HEAP32[P>>2]|0,o=t+16|0,u=HEAP32[o>>2]|0,a=s>>>0<u>>>0;if(a){f=e&255,l=s+1|0,HEAP32[P>>2]=l,HEAP8[s>>0]=f,c=e&255,n=c;break}}h=___overflow(t,e)|0,n=h}while(0);return n|0}function _tmpnam(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0;B=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),k=B+32|0,A=B+24|0,L=B+8|0,C=B,r=B+48|0,N=B+40|0,i=(e|0)==(0|0),n=i?20547:e,HEAP32[r>>2]=n,HEAP32[C>>2]=20567,O=C+4|0,HEAP32[O>>2]=7,v=___syscall33(33,C|0)|0,m=(v|0)==0;if(m){g=N+4|0,y=r,x=0;for(;;){HEAP32[L>>2]=0,D=L+4|0,HEAP32[D>>2]=N,P=L+8|0,HEAP32[P>>2]=0,___syscall265(265,L|0)|0,b=HEAP32[g>>2]|0,w=b^y,E=HEAP32[r>>2]|0,S=E,s=w^S,o=HEAP32[3225]|0,HEAP32[3225]=o+1|0,HEAP32[A>>2]=o,M=A+4|0,HEAP32[M>>2]=s,_snprintf(E,20,20572,A)|0,u=HEAP32[r>>2]|0,HEAP32[k>>2]=u,_=k+4|0,HEAP32[_>>2]=0,a=___syscall33(33,k|0)|0,f=(a|0)==0;if(!f){T=x;break}l=x+1|0,c=(x|0)<100;if(!c){T=l;break}x=l}h=(T|0)>99,p=HEAP32[r>>2]|0,d=h?0:p,t=d}else t=0;return STACKTOP=B,t|0}function _fclose(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0;return A=STACKTOP,n=e+76|0,r=HEAP32[n>>2]|0,d=(r|0)>-1,d,E=HEAP32[e>>2]|0,S=E&1,x=(S|0)!=0,x||(___lock(12772),T=e+52|0,N=HEAP32[T>>2]|0,C=(N|0)==(0|0),k=N,t=e+56|0,C||(i=HEAP32[t>>2]|0,s=N+56|0,HEAP32[s>>2]=i),o=HEAP32[t>>2]|0,u=(o|0)==(0|0),a=o,u||(f=o+52|0,HEAP32[f>>2]=k),l=HEAP32[3192]|0,c=(l|0)==(e|0),c&&(HEAP32[3192]=a),___unlock(12772)),h=_fflush(e)|0,p=e+12|0,v=HEAP32[p>>2]|0,m=FUNCTION_TABLE_ii[v&15](e)|0,g=m|h,y=e+92|0,b=HEAP32[y>>2]|0,w=(b|0)==(0|0),w||_free(b),x||_free(e),g|0}function _freopen(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0;et=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),J=et+16|0,$=et,s=___fmodeflags(t)|0,o=n+76|0,g=HEAP32[o>>2]|0,L=(g|0)>-1,L?(I=___lockfile(n)|0,q=I):q=0,_fflush(n)|0,U=(e|0)==(0|0);do if(U)z=s&524288,W=(z|0)==0,i=n+60|0,W||(X=HEAP32[i>>2]|0,HEAP32[$>>2]=X,K=$+4|0,HEAP32[K>>2]=2,Q=$+8|0,HEAP32[Q>>2]=1,___syscall221(221,$|0)|0),V=s&-524481,u=HEAP32[i>>2]|0,HEAP32[J>>2]=u,G=J+4|0,HEAP32[G>>2]=4,Y=J+8|0,HEAP32[Y>>2]=V,a=___syscall221(221,J|0)|0,f=___syscall_ret(a)|0,l=(f|0)<0,l?Z=15:Z=12;else{c=_fopen(e,t)|0,h=(c|0)==(0|0);if(h)Z=15;else{p=c+60|0,d=HEAP32[p>>2]|0,v=n+60|0,m=HEAP32[v>>2]|0,y=(d|0)==(m|0);if(y)HEAP32[p>>2]=-1;else{b=s&524288,w=___dup3(d,m,b)|0,E=(w|0)<0;if(E){_fclose(c)|0,Z=15;break}}S=HEAP32[n>>2]|0,x=S&1,T=HEAP32[c>>2]|0,N=x|T,HEAP32[n>>2]=N,C=c+32|0,k=HEAP32[C>>2]|0,A=n+32|0,HEAP32[A>>2]=k,O=c+36|0,M=HEAP32[O>>2]|0,_=n+36|0,HEAP32[_>>2]=M,D=c+40|0,P=HEAP32[D>>2]|0,H=n+40|0,HEAP32[H>>2]=P,B=c+12|0,j=HEAP32[B>>2]|0,F=n+12|0,HEAP32[F>>2]=j,_fclose(c)|0,Z=12}}while(0);return(Z|0)==12?(R=(q|0)==0,R?r=n:(___unlockfile(n),r=n)):(Z|0)==15&&(_fclose(n)|0,r=0),STACKTOP=et,r|0}function _rewind(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0;c=STACKTOP,t=e+76|0,n=HEAP32[t>>2]|0,r=(n|0)>-1,r?(i=___lockfile(e)|0,f=(i|0)==0,___fseeko_unlocked(e,0,0)|0,s=HEAP32[e>>2]|0,o=s&-33,HEAP32[e>>2]=o,f||___unlockfile(e)):(___fseeko_unlocked(e,0,0)|0,u=HEAP32[e>>2]|0,a=u&-33,HEAP32[e>>2]=a);return}function _vfprintf(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0;J=STACKTOP,STACKTOP=STACKTOP+224|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),I=J+120|0,U=J+80|0,R=J,q=J+136|0,V=U,K=V+40|0;do HEAP32[V>>2]=0,V=V+4|0;while((V|0)<(K|0));return X=HEAP32[n>>2]|0,HEAP32[I>>2]=X,s=_printf_core(0,t,I,R,U)|0,o=(s|0)<0,o?i=-1:(g=e+76|0,L=HEAP32[g>>2]|0,D=(L|0)>-1,D?(P=___lockfile(e)|0,M=P):M=0,H=HEAP32[e>>2]|0,B=H&32,j=e+74|0,F=HEAP8[j>>0]|0,u=F<<24>>24<1,u&&(a=H&-33,HEAP32[e>>2]=a),f=e+48|0,l=HEAP32[f>>2]|0,c=(l|0)==0,c?(p=e+44|0,d=HEAP32[p>>2]|0,HEAP32[p>>2]=q,v=e+28|0,HEAP32[v>>2]=q,m=e+20|0,HEAP32[m>>2]=q,HEAP32[f>>2]=80,y=q+80|0,b=e+16|0,HEAP32[b>>2]=y,w=_printf_core(e,t,I,R,U)|0,E=(d|0)==(0|0),E?z=w:(S=e+36|0,x=HEAP32[S>>2]|0,FUNCTION_TABLE_iiii[x&15](e,0,0)|0,T=HEAP32[m>>2]|0,N=(T|0)==(0|0),r=N?-1:w,HEAP32[p>>2]=d,HEAP32[f>>2]=0,HEAP32[b>>2]=0,HEAP32[v>>2]=0,HEAP32[m>>2]=0,z=r)):(h=_printf_core(e,t,I,R,U)|0,z=h),C=HEAP32[e>>2]|0,k=C&32,A=(k|0)==0,W=A?z:-1,O=C|B,HEAP32[e>>2]=O,_=(M|0)==0,_||___unlockfile(e),i=W),STACKTOP=J,i|0}function ___stdio_seek(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0;return g=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),c=g,l=g+20|0,i=e+60|0,s=HEAP32[i>>2]|0,HEAP32[c>>2]=s,h=c+4|0,HEAP32[h>>2]=0,p=c+8|0,HEAP32[p>>2]=t,d=c+12|0,HEAP32[d>>2]=l,v=c+16|0,HEAP32[v>>2]=n,o=___syscall140(140,c|0)|0,u=___syscall_ret(o)|0,a=(u|0)<0,a?(HEAP32[l>>2]=-1,f=-1):(r=HEAP32[l>>2]|0,f=r),STACKTOP=g,f|0}function _fputs(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,n=_strlen(e)|0,r=_fwrite(e,n,1,t)|0,i=r+ -1|0,i|0}function ___fwritex(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0;X=STACKTOP,a=n+16|0,f=HEAP32[a>>2]|0,w=(f|0)==(0|0),w?(M=___towrite(n)|0,H=(M|0)==0,H?(o=HEAP32[a>>2]|0,F=o,W=4):r=0):(F=f,W=4);e:do if((W|0)==4){B=n+20|0,j=HEAP32[B>>2]|0,I=F,q=j,l=I-q|0,c=l>>>0<t>>>0;if(c){h=n+36|0,p=HEAP32[h>>2]|0,d=FUNCTION_TABLE_iiii[p&15](n,e,t)|0,r=d;break}v=n+75|0,m=HEAP8[v>>0]|0,g=m<<24>>24>-1;t:do if(g){R=t;for(;;){y=(R|0)==0;if(y){i=t,s=e,O=j,z=0;break t}b=R+ -1|0,E=e+b|0,S=HEAP8[E>>0]|0,x=S<<24>>24==10;if(x){U=R;break}R=b}T=n+36|0,N=HEAP32[T>>2]|0,C=FUNCTION_TABLE_iiii[N&15](n,e,U)|0,k=C>>>0<U>>>0;if(k){r=U;break e}L=e+U|0,A=t-U|0,u=HEAP32[B>>2]|0,i=A,s=L,O=u,z=U}else i=t,s=e,O=j,z=0;while(0);_memcpy(O|0,s|0,i|0)|0,_=HEAP32[B>>2]|0,D=_+i|0,HEAP32[B>>2]=D,P=z+i|0,r=P}while(0);return r|0}function _fwrite(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0;return g=STACKTOP,i=Math_imul(n,t)|0,s=r+76|0,u=HEAP32[s>>2]|0,a=(u|0)>-1,a?(l=___lockfile(r)|0,v=(l|0)==0,c=___fwritex(e,i,r)|0,v?h=c:(___unlockfile(r),h=c)):(f=___fwritex(e,i,r)|0,h=f),p=(h|0)==(i|0),p?o=n:(d=(h>>>0)/(t>>>0)&-1,o=d),o|0}function _fprintf(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0;return o=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=o,HEAP32[i>>2]=n,r=_vfprintf(e,t,i)|0,STACKTOP=o,r|0}function ___stdio_close(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=u,t=e+60|0,n=HEAP32[t>>2]|0,HEAP32[s>>2]=n,r=___syscall6(6,s|0)|0,i=___syscall_ret(r)|0,STACKTOP=u,i|0}function ___errno_location(){var e=0,t=0,n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,t=HEAP32[3186]|0,n=(t|0)==(0|0),n?e=12904:(r=_pthread_self()|0,i=r+60|0,s=HEAP32[i>>2]|0,e=s),e|0}function _strerror(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0;E=STACKTOP,d=0;for(;;){r=20584+d|0,o=HEAP8[r>>0]|0,u=o&255,a=(u|0)==(e|0);if(a){v=d,w=2;break}f=d+1|0,l=(f|0)==87;if(l){m=87,y=20672,w=5;break}d=f}(w|0)==2&&(n=(v|0)==0,n?g=20672:(m=v,y=20672,w=5));if((w|0)==5)for(;;){w=0,b=y;for(;;){c=HEAP8[b>>0]|0,h=c<<24>>24==0,p=b+1|0;if(h){t=p;break}b=p}i=m+ -1|0,s=(i|0)==0;if(s){g=t;break}m=i,y=t,w=5}return g|0}function _stat(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),i=u,HEAP32[i>>2]=e,s=i+4|0,HEAP32[s>>2]=t,n=___syscall195(195,i|0)|0,r=___syscall_ret(n)|0,STACKTOP=u,r|0}function _fstat(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0;return y=STACKTOP,STACKTOP=STACKTOP+64|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),p=y+16|0,h=y+8|0,c=y,l=y+24|0,HEAP32[c>>2]=e,d=c+4|0,HEAP32[d>>2]=t,r=___syscall197(197,c|0)|0,i=(r|0)==-9,i?(HEAP32[h>>2]=e,v=h+4|0,HEAP32[v>>2]=1,s=___syscall221(221,h|0)|0,o=(s|0)<0,o?g=3:(___procfdname(l,e),HEAP32[p>>2]=l,m=p+4|0,HEAP32[m>>2]=t,a=___syscall195(195,p|0)|0,f=___syscall_ret(a)|0,n=f)):g=3,(g|0)==3&&(u=___syscall_ret(r)|0,n=u),STACKTOP=y,n|0}function _atoi(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0;P=STACKTOP,t=e;for(;;){u=HEAP8[t>>0]|0,a=u<<24>>24,m=_isspace(a)|0,g=(m|0)==0,y=t+1|0;if(g){n=t,s=y;break}t=y}b=HEAP8[n>>0]|0,w=b<<24>>24;switch(w|0){case 45:M=1,D=5;break;case 43:M=0,D=5;break;default:r=n,S=b,_=0}(D|0)==5&&(o=HEAP8[s>>0]|0,r=s,S=o,_=M),E=S<<24>>24,k=E+ -48|0,N=k>>>0<10;if(N){i=r,L=k,O=0;for(;;){x=O*10|0,f=i+1|0,l=x-L|0,c=HEAP8[f>>0]|0,h=c<<24>>24,C=h+ -48|0,T=C>>>0<10;if(!T){A=l;break}i=f,L=C,O=l}}else A=0;return p=(_|0)!=0,d=0-A|0,v=p?A:d,v|0}function _frexp(e,t){e=+e,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0;w=STACKTOP,HEAPF64[tempDoublePtr>>3]=e,i=HEAP32[tempDoublePtr>>2]|0,s=HEAP32[tempDoublePtr+4>>2]|0,l=_bitshift64Lshr(i|0,s|0,52)|0,c=tempRet0,h=l&2047;switch(h|0){case 0:p=e!=0,p?(d=e*0x10000000000000000,v=+_frexp(d,t),m=HEAP32[t>>2]|0,g=m+ -64|0,r=v,y=g):(r=e,y=0),HEAP32[t>>2]=y,n=r;break;case 2047:n=e;break;default:o=h+ -1022|0,HEAP32[t>>2]=o,u=s&-2146435073,a=u|1071644672,HEAP32[tempDoublePtr>>2]=i,HEAP32[tempDoublePtr+4>>2]=a,f=+HEAPF64[tempDoublePtr>>3],n=f}return+n}function _frexpl(e,t){e=+e,t|=0;var n=0,r=0,i=0;return i=STACKTOP,n=+_frexp(e,t),+n}function ___syscall_ret(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,n=e>>>0>4294963200,n?(r=0-e|0,i=___errno_location()|0,HEAP32[i>>2]=r,t=-1):t=e,t|0}function ___procfdname(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0;L=STACKTOP,S=0;for(;;){i=22476+S|0,s=HEAP8[i>>0]|0,d=e+S|0,HEAP8[d>>0]=s,v=(S|0)==14,m=S+1|0;if(v)break;S=m}g=(t|0)==0;if(g)C=e+14|0,HEAP8[C>>0]=48,y=e+15|0,HEAP8[y>>0]=0;else{x=14,N=t;for(;;){b=(N>>>0)/10&-1,w=x+1|0,E=N>>>0<10;if(E){r=w;break}x=w,N=b}o=e+r|0,HEAP8[o>>0]=0,n=t,T=r;for(;;){u=(n>>>0)%10&-1,a=u|48,f=a&255,l=T+ -1|0,c=e+l|0,HEAP8[c>>0]=f,h=(n>>>0)/10&-1,p=n>>>0<10;if(p)break;n=h,T=l}}return}function _wctomb(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0;return o=STACKTOP,r=(e|0)==(0|0),r?n=0:(i=_wcrtomb(e,t,0)|0,n=i),n|0}function _wcrtomb(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0;Y=STACKTOP,i=(e|0)==(0|0);do{if(!i){s=t>>>0<128;if(s){m=t&255,HEAP8[e>>0]=m,r=1;break}k=t>>>0<2048;if(k){F=t>>>6,X=F|192,V=X&255,$=e+1|0,HEAP8[e>>0]=V,J=t&63,K=J|128,o=K&255,HEAP8[$>>0]=o,r=2;break}u=t>>>0<55296,a=t&-8192,f=(a|0)==57344,Q=u|f;if(Q){l=t>>>12,c=l|224,h=c&255,p=e+1|0,HEAP8[e>>0]=h,d=t>>>6,v=d&63,g=v|128,y=g&255,b=e+2|0,HEAP8[p>>0]=y,w=t&63,E=w|128,S=E&255,HEAP8[b>>0]=S,r=3;break}x=t+ -65536|0,T=x>>>0<1048576;if(T){N=t>>>18,C=N|240,L=C&255,A=e+1|0,HEAP8[e>>0]=L,O=t>>>12,M=O&63,_=M|128,D=_&255,P=e+2|0,HEAP8[A>>0]=D,H=t>>>6,B=H&63,j=B|128,I=j&255,q=e+3|0,HEAP8[P>>0]=I,R=t&63,U=R|128,z=U&255,HEAP8[q>>0]=z,r=4;break}W=___errno_location()|0,HEAP32[W>>2]=84,r=-1;break}r=1}while(0);return r|0}function _ioctl(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0;return T=STACKTOP,STACKTOP=STACKTOP+32|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),w=T,h=T+16|0,HEAP32[h>>2]=n,p=HEAP32[h>>2]|0,r=p,i=4,m=i,v=m-1|0,s=r+v|0,o=4,b=o,y=b-1|0,g=y^-1,u=s&g,a=u,f=HEAP32[a>>2]|0,d=a+4|0,HEAP32[h>>2]=d,HEAP32[w>>2]=e,E=w+4|0,HEAP32[E>>2]=t,S=w+8|0,HEAP32[S>>2]=f,l=___syscall54(54,w|0)|0,c=___syscall_ret(l)|0,STACKTOP=T,c|0}function _tcgetattr(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,STACKTOP=STACKTOP+16|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),s=u,HEAP32[s>>2]=t,r=_ioctl(e,21505,s)|0,i=(r|0)!=0,n=i<<31>>31,STACKTOP=u,n|0}function _isspace(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0;return u=STACKTOP,t=(e|0)==32,n=e+ -9|0,r=n>>>0<5,i=t|r,s=i&1,s|0}function _strlen(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0;P=STACKTOP,f=e,l=f&3,E=(l|0)==0;e:do if(E)n=e,D=4;else{r=e,x=f;for(;;){T=HEAP8[r>>0]|0,N=T<<24>>24==0;if(N){o=x;break e}C=r+1|0,k=C,L=k&3,A=(L|0)==0;if(A){n=C,D=4;break}r=C,x=k}}while(0);if((D|0)==4){M=n;for(;;){O=HEAP32[M>>2]|0,c=O+ -16843009|0,h=O&-2139062144,p=h^-2139062144,d=p&c,v=(d|0)==0,m=M+4|0;if(!v){s=O,_=M;break}M=m}g=s&255,y=g<<24>>24==0;if(y)i=_;else{u=_;for(;;){b=u+1|0,a=HEAP8[b>>0]|0,w=a<<24>>24==0;if(w){i=b;break}u=b}}S=i,o=S}return t=o-f|0,t|0}function ___strchrnul(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0;Y=STACKTOP,o=t&255,u=(o|0)==0;e:do if(u)z=_strlen(e)|0,W=e+z|0,n=W;else{y=e,A=y&3,q=(A|0)==0;if(q)r=e;else{U=t&255,i=e;for(;;){X=HEAP8[i>>0]|0,V=X<<24>>24==0,a=X<<24>>24==U<<24>>24,$=V|a;if($){n=i;break e}f=i+1|0,l=f,c=l&3,h=(c|0)==0;if(h){r=f;break}i=f}}p=Math_imul(o,16843009)|0,d=HEAP32[r>>2]|0,v=d+ -16843009|0,m=d&-2139062144,g=m^-2139062144,b=g&v,w=(b|0)==0;t:do if(w){S=d,Q=r;for(;;){E=S^p,x=E+ -16843009|0,T=E&-2139062144,N=T^-2139062144,C=N&x,k=(C|0)==0;if(!k){K=Q;break t}L=Q+4|0,O=HEAP32[L>>2]|0,M=O+ -16843009|0,_=O&-2139062144,D=_^-2139062144,P=D&M,H=(P|0)==0;if(!H){K=L;break}S=O,Q=L}}else K=r;while(0);B=t&255,s=K;for(;;){j=HEAP8[s>>0]|0,F=j<<24>>24==0,I=j<<24>>24==B<<24>>24,J=F|I,R=s+1|0;if(J){n=s;break}s=R}}while(0);return n|0}function _memchr(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0;ot=STACKTOP,h=t&255,p=e,T=p&3,H=(T|0)!=0,z=(n|0)!=0,Q=z&H;e:do if(Q){W=t&255,s=n,Z=e;for(;;){X=HEAP8[Z>>0]|0,V=X<<24>>24==W<<24>>24;if(V){i=s,Y=Z,st=6;break e}$=Z+1|0,J=s+ -1|0,d=$,v=d&3,m=(v|0)!=0,g=(J|0)!=0,K=g&m;if(!K){r=J,c=g,G=$,st=5;break}s=J,Z=$}}else r=n,c=z,G=e,st=5;while(0);(st|0)==5&&(c?(i=r,Y=G,st=6):(l=0,tt=G));t:do if((st|0)==6){y=HEAP8[Y>>0]|0,b=t&255,w=y<<24>>24==b<<24>>24;if(w)l=i,tt=Y;else{E=Math_imul(h,16843009)|0,S=i>>>0>3;n:do if(S){u=i,rt=Y;for(;;){x=HEAP32[rt>>2]|0,N=x^E,C=N+ -16843009|0,k=N&-2139062144,L=k^-2139062144,A=L&C,O=(A|0)==0;if(!O){a=u,it=rt;break}M=rt+4|0,_=u+ -4|0,D=_>>>0>3;if(!D){o=_,nt=M,st=11;break n}u=_,rt=M}f=a,et=it}else o=i,nt=Y,st=11;while(0);if((st|0)==11){P=(o|0)==0;if(P){l=0,tt=nt;break}f=o,et=nt}for(;;){B=HEAP8[et>>0]|0,j=B<<24>>24==b<<24>>24;if(j){l=f,tt=et;break t}F=et+1|0,I=f+ -1|0,q=(I|0)==0;if(q){l=0,tt=F;break}f=I,et=F}}}while(0);return R=(l|0)!=0,U=R?tt:0,U|0}function _strcpy(e,t){e|=0,t|=0;var n=0,r=0;return r=STACKTOP,___stpcpy(e,t)|0,e|0}function _strchr(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0;return a=STACKTOP,n=___strchrnul(e,t)|0,r=HEAP8[n>>0]|0,i=t&255,s=r<<24>>24==i<<24>>24,o=s?n:0,o|0}function ___stpcpy(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0;G=STACKTOP,c=t,h=e,x=c^h,P=x&3,q=(P|0)==0;e:do if(q){R=c&3,U=(R|0)==0;if(U)n=t,r=e;else{i=e,s=t;for(;;){z=HEAP8[s>>0]|0,HEAP8[i>>0]=z,W=z<<24>>24==0;if(W){o=i;break e}X=s+1|0,p=i+1|0,d=X,v=d&3,m=(v|0)==0;if(m){n=X,r=p;break}i=p,s=X}}g=HEAP32[n>>2]|0,y=g+ -16843009|0,b=g&-2139062144,w=b^-2139062144,E=w&y,S=(E|0)==0;if(S){C=g,$=r,K=n;for(;;){T=K+4|0,N=$+4|0,HEAP32[$>>2]=C,k=HEAP32[T>>2]|0,L=k+ -16843009|0,A=k&-2139062144,O=A^-2139062144,M=O&L,_=(M|0)==0;if(!_){V=N,J=T;break}C=k,$=N,K=T}}else V=r,J=n;u=J,a=V,Q=8}else u=t,a=e,Q=8;while(0);if((Q|0)==8){D=HEAP8[u>>0]|0,HEAP8[a>>0]=D,H=D<<24>>24==0;if(H)o=a;else{f=a,l=u;for(;;){B=l+1|0,j=f+1|0,F=HEAP8[B>>0]|0,HEAP8[j>>0]=F,I=F<<24>>24==0;if(I){o=j;break}f=j,l=B}}}return o|0}function ___fflush_unlocked(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0;return x=STACKTOP,n=e+20|0,r=HEAP32[n>>2]|0,d=e+28|0,v=HEAP32[d>>2]|0,m=r>>>0>v>>>0,m?(g=e+36|0,y=HEAP32[g>>2]|0,FUNCTION_TABLE_iiii[y&15](e,0,0)|0,b=HEAP32[n>>2]|0,w=(b|0)==(0|0),w?t=-1:S=3):S=3,(S|0)==3&&(E=e+4|0,i=HEAP32[E>>2]|0,s=e+8|0,o=HEAP32[s>>2]|0,u=i>>>0<o>>>0,u&&(a=e+40|0,f=HEAP32[a>>2]|0,l=i,c=o,h=l-c|0,FUNCTION_TABLE_iiii[f&15](e,h,1)|0),p=e+16|0,HEAP32[p>>2]=0,HEAP32[d>>2]=0,HEAP32[n>>2]=0,HEAP32[s>>2]=0,HEAP32[E>>2]=0,t=0),t|0}function _cleanup(e){e|=0;var t=0,n=0,r=0,i=0,s=0;s=STACKTOP,t=e+68|0,n=HEAP32[t>>2]|0,r=(n|0)==0,r&&___unlockfile(e);return}function _cleanup52(e){e|=0;var t=0,n=0,r=0,i=0,s=0;s=STACKTOP,t=e+68|0,n=HEAP32[t>>2]|0,r=(n|0)==0,r&&___unlockfile(e);return}function _printf_core(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0,Rd=0,Ud=0,zd=0,Wd=0,Xd=0,Vd=0,$d=0,Jd=0,Kd=0,Qd=0,Gd=0,Yd=0,Zd=0,ev=0,tv=0,nv=0,rv=0,iv=0,sv=0,ov=0,uv=0,av=0,fv=0,lv=0,cv=0,hv=0,pv=0,dv=0,vv=0,mv=0,gv=0,yv=0,bv=0,wv=0,Ev=0,Sv=0,xv=0,Tv=0,Nv=0,Cv=0,kv=0,Lv=0,Av=0,Ov=0,Mv=0,_v=0,Dv=0,Pv=0,Hv=0,Bv=0,jv=0,Fv=0,Iv=0,qv=0,Rv=0,Uv=0,zv=0,Wv=0,Xv=0,Vv=0,$v=0,Jv=0,Kv=0,Qv=0,Gv=0,Yv=0,Zv=0,em=0,tm=0,nm=0,rm=0,im=0,sm=0,om=0,um=0,am=0,fm=0,lm=0,cm=0,hm=0,pm=0,dm=0,vm=0,mm=0,gm=0,ym=0,bm=0,wm=0,Em=0,Sm=0,xm=0,Tm=0,Nm=0,Cm=0,km=0,Lm=0,Am=0,Om=0,Mm=0,_m=0,Dm=0,Pm=0,Hm=0,Bm=0,jm=0,Fm=0,Im=0,qm=0,Rm=0,Um=0,zm=0,Wm=0,Xm=0,Vm=0,$m=0,Jm=0,Km=0,Qm=0,Gm=0,Ym=0,Zm=0,eg=0,tg=0,ng=0,rg=0,ig=0,sg=0,og=0,ug=0,ag=0,fg=0,lg=0,cg=0,hg=0,pg=0,dg=0,vg=0,mg=0,gg=0,yg=0,bg=0,wg=0,Eg=0,Sg=0,xg=0,Tg=0,Ng=0,Cg=0,kg=0,Lg=0,Ag=0,Og=0,Mg=0,_g=0,Dg=0,Pg=0,Hg=0,Bg=0,jg=0,Fg=0,Ig=0,qg=0,Rg=0,Ug=0,zg=0,Wg=0,Xg=0,Vg=0,$g=0,Jg=0,Kg=0,Qg=0,Gg=0,Yg=0,Zg=0,ey=0,ty=0,ny=0,ry=0,iy=0,sy=0,oy=0,uy=0,ay=0,fy=0,ly=0,cy=0,hy=0,py=0,dy=0,vy=0,my=0,gy=0,yy=0,by=0,wy=0,Ey=0,Sy=0,xy=0,Ty=0,Ny=0,Cy=0,ky=0,Ly=0,Ay=0,Oy=0,My=0,_y=0,Dy=0,Py=0,Hy=0,By=0,jy=0,Fy=0,Iy=0,qy=0,Ry=0,Uy=0,zy=0,Wy=0,Xy=0,Vy=0,$y=0,Jy=0,Ky=0,Qy=0,Gy=0,Yy=0,Zy=0,eb=0,tb=0,nb=0,rb=0,ib=0,sb=0,ob=0,ub=0,ab=0,fb=0,lb=0,cb=0,hb=0,pb=0,db=0,vb=0,mb=0,gb=0,yb=0,bb=0,wb=0,Eb=0,Sb=0,xb=0,Tb=0,Nb=0,Cb=0,kb=0,Lb=0,Ab=0,Ob=0,Mb=0,_b=0,Db=0,Pb=0,Hb=0,Bb=0,jb=0,Fb=0,Ib=0,qb=0,Rb=0,Ub=0,zb=0,Wb=0,Xb=0;Xb=STACKTOP,STACKTOP=STACKTOP+624|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),Dm=Xb+24|0,ng=Xb+16|0,Hm=Xb+588|0,rg=Xb+576|0,km=Xb,Pm=Xb+536|0,Nb=Xb+8|0,by=Xb+528|0,yt=(e|0)!=(0|0),bt=Pm+40|0,xr=bt,Cs=Pm+39|0,Ou=Nb+4|0,Pf=rg+12|0,jc=rg+11|0,qp=Hm,zv=Pf,rm=zv-qp|0,wt=-2-qp|0,Mt=zv+2|0,Ut=Dm+288|0,Zt=Hm+9|0,cn=Zt,Sn=Hm+8|0,Vr=t,Fm=0,ay=0,py=0;e:for(;;){Dn=(Fm|0)>-1;do{if(Dn){Wn=2147483647-Fm|0,tr=(ay|0)>(Wn|0);if(tr){hr=___errno_location()|0,HEAP32[hr>>2]=75,Im=-1;break}Tr=ay+Fm|0,Im=Tr;break}Im=Fm}while(0);Hr=HEAP8[Vr>>0]|0,ri=Hr<<24>>24==0;if(ri){qm=Im,dy=py,Wb=245;break}di=Hr,Bi=Vr;t:for(;;){switch(di<<24>>24){case 37:is=Bi,Ob=Bi,Wb=9;break t;case 0:Q=Bi,Ab=Bi;break t;default:}Ni=Bi+1|0,ot=HEAP8[Ni>>0]|0,di=ot,Bi=Ni}n:do if((Wb|0)==9)for(;;){Wb=0,$i=is+1|0,vs=HEAP8[$i>>0]|0,ks=vs<<24>>24==37;if(!ks){Q=is,Ab=Ob;break n}Fs=Ob+1|0,Ks=is+2|0,oo=HEAP8[Ks>>0]|0,yo=oo<<24>>24==37;if(!yo){Q=Ks,Ab=Fs;break}is=Ks,Ob=Fs,Wb=9}while(0);Ao=Ab,qo=Vr,Go=Ao-qo|0,yt&&(au=HEAP32[e>>2]|0,bu=au&32,Mu=(bu|0)==0,Mu&&___fwritex(Vr,Go,e)|0),Uu=(Ab|0)==(Vr|0);if(!Uu){vy=py,Vr=Q,Fm=Im,ay=Go,py=vy;continue}Zu=Q+1|0,la=HEAP8[Zu>>0]|0,Ea=la<<24>>24,$g=Ea+ -48|0,qg=$g>>>0<10,qg?(_a=Q+2|0,za=HEAP8[_a>>0]|0,ef=za<<24>>24==36,hf=Q+3|0,A=ef?hf:Zu,B=ef?1:py,Jg=ef?$g:-1,ht=HEAP8[A>>0]|0,Hf=ht,_m=Jg,my=B,pb=A):(Hf=la,_m=-1,my=py,pb=Zu),xf=Hf<<24>>24,Vf=xf&-32,rl=(Vf|0)==32;r:do if(rl){Nl=xf,Cc=Hf,bg=0,vb=pb;for(;;){dl=Nl+ -32|0,Bl=1<<dl,$l=Bl&75913,ic=($l|0)==0;if(ic){yp=Cc,wg=bg,mb=vb;break r}vc=Cc<<24>>24,Fc=vc+ -32|0,Kc=1<<Fc,oh=Kc|bg,gh=vb+1|0,Lh=HEAP8[gh>>0]|0,Ih=Lh<<24>>24,Qh=Ih&-32,up=(Qh|0)==32;if(!up){yp=Lh,wg=oh,mb=gh;break}Nl=Ih,Cc=Lh,bg=oh,vb=gh}}else yp=Hf,wg=0,mb=pb;while(0);Ap=yp<<24>>24==42;do if(Ap){Rp=mb+1|0,Yp=HEAP8[Rp>>0]|0,fd=Yp<<24>>24,Zg=fd+ -48|0,Wg=Zg>>>0<10,Wg?(wd=mb+2|0,Md=HEAP8[wd>>0]|0,Ud=Md<<24>>24==36,Ud?(Zd=i+(Zg<<2)|0,HEAP32[Zd>>2]=10,lv=HEAP8[Rp>>0]|0,Ev=lv<<24>>24,_v=Ev+ -48|0,Wv=r+(_v<<3)|0,Jv=Wv,Kv=Jv,Qv=HEAP32[Kv>>2]|0,Gv=Jv+4|0,Yv=Gv,Zv=HEAP32[Yv>>2]|0,em=mb+3|0,gy=1,db=em,Eb=Qv):Wb=24):Wb=24;if((Wb|0)==24){Wb=0,tm=(my|0)==0;if(!tm){u=-1;break e}if(!yt){At=Rp,Eg=wg,yy=0,Sb=0;break}Lm=HEAP32[n>>2]|0,nm=Lm,im=4,vg=im,fg=vg-1|0,sm=nm+fg|0,om=4,yg=om,gg=yg-1|0,mg=gg^-1,um=sm&mg,am=um,fm=HEAP32[am>>2]|0,Om=am+4|0,HEAP32[n>>2]=Om,gy=0,db=Rp,Eb=fm}lm=(Eb|0)<0,lm?(cm=wg|8192,hm=0-Eb|0,At=db,Eg=cm,yy=gy,Sb=hm):(At=db,Eg=wg,yy=gy,Sb=Eb)}else{pm=yp<<24>>24,Gg=pm+ -48|0,Xg=Gg>>>0<10;if(Xg){Tt=mb,Og=0,ey=Gg;for(;;){Et=Og*10|0,St=Et+ey|0,xt=Tt+1|0,Nt=HEAP8[xt>>0]|0,Ct=Nt<<24>>24,Kg=Ct+ -48|0,Rg=Kg>>>0<10;if(!Rg){F=St,I=xt;break}Tt=xt,Og=St,ey=Kg}kt=(F|0)<0;if(kt){u=-1;break e}At=I,Eg=wg,yy=my,Sb=F}else At=mb,Eg=wg,yy=my,Sb=0}while(0);Lt=HEAP8[At>>0]|0,Ot=Lt<<24>>24==46;i:do if(Ot){_t=At+1|0,Dt=HEAP8[_t>>0]|0,Pt=Dt<<24>>24==42;if(!Pt){ln=Dt<<24>>24,Yg=ln+ -48|0,Vg=Yg>>>0<10;if(!Vg){$v=_t,My=0;break}vn=_t,Mg=0,ty=Yg;for(;;){hn=Mg*10|0,pn=hn+ty|0,dn=vn+1|0,mn=HEAP8[dn>>0]|0,gn=mn<<24>>24,Qg=gn+ -48|0,Ug=Qg>>>0<10;if(!Ug){$v=dn,My=pn;break i}vn=dn,Mg=pn,ty=Qg}}Ht=At+2|0,Bt=HEAP8[Ht>>0]|0,jt=Bt<<24>>24,ny=jt+ -48|0,zg=ny>>>0<10;if(zg){Ft=At+3|0,It=HEAP8[Ft>>0]|0,qt=It<<24>>24==36;if(qt){Rt=i+(ny<<2)|0,HEAP32[Rt>>2]=10,zt=HEAP8[Ht>>0]|0,Wt=zt<<24>>24,Xt=Wt+ -48|0,Vt=r+(Xt<<3)|0,$t=Vt,Jt=$t,Kt=HEAP32[Jt>>2]|0,Qt=$t+4|0,Gt=Qt,Yt=HEAP32[Gt>>2]|0,en=At+4|0,$v=en,My=Kt;break}}tn=(yy|0)==0;if(!tn){u=-1;break e}yt?(Am=HEAP32[n>>2]|0,nn=Am,rn=4,cg=rn,lg=cg-1|0,sn=nn+lg|0,on=4,dg=on,pg=dg-1|0,hg=pg^-1,un=sn&hg,an=un,fn=HEAP32[an>>2]|0,Mm=an+4|0,HEAP32[n>>2]=Mm,$v=Ht,My=fn):($v=Ht,My=0)}else $v=At,My=-1;while(0);bn=$v,cb=0;for(;;){yn=HEAP8[bn>>0]|0,wn=yn<<24>>24,En=wn+ -65|0,xn=En>>>0>57;if(xn){u=-1;break e}Tn=bn+1|0,Nn=(24563+(cb*58|0)|0)+En|0,Cn=HEAP8[Nn>>0]|0,kn=Cn&255,Ln=kn+ -1|0,An=Ln>>>0<8;if(!An){q=bn,R=Tn,U=Cn,z=kn,hb=cb;break}bn=Tn,cb=kn}On=U<<24>>24==0;if(On){u=-1;break}Mn=U<<24>>24==19,_n=(_m|0)>-1;do if(Mn){if(_n){u=-1;break e}Wb=52}else{if(_n){Pn=i+(_m<<2)|0,HEAP32[Pn>>2]=z,Hn=r+(_m<<3)|0,Bn=Hn,jn=Bn,Fn=HEAP32[jn>>2]|0,In=Bn+4|0,qn=In,Rn=HEAP32[qn>>2]|0,Un=km,zn=Un,HEAP32[zn>>2]=Fn,Xn=Un+4|0,Vn=Xn,HEAP32[Vn>>2]=Rn,Wb=52;break}if(!yt){u=0;break e}_pop_arg80(km,z,n)}while(0);if((Wb|0)==52){Wb=0;if(!yt){Vr=R,Fm=Im,ay=Go,py=yy;continue}}$n=HEAP8[q>>0]|0,Jn=$n<<24>>24,Kn=(hb|0)!=0,Qn=Jn&15,Gn=(Qn|0)==3,Ty=Kn&Gn,Yn=Jn&-33,yb=Ty?Yn:Jn,Zn=Eg&8192,er=(Zn|0)==0,nr=Eg&-65537,Sg=er?Eg:nr;s:do switch(yb|0){case 110:switch(hb|0){case 0:fr=HEAP32[km>>2]|0,HEAP32[fr>>2]=Im,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 1:lr=HEAP32[km>>2]|0,HEAP32[lr>>2]=Im,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 2:cr=(Im|0)<0,pr=cr<<31>>31,dr=HEAP32[km>>2]|0,vr=dr,mr=vr,HEAP32[mr>>2]=Im,gr=vr+4|0,yr=gr,HEAP32[yr>>2]=pr,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 3:br=Im&65535,wr=HEAP32[km>>2]|0,HEAP16[wr>>1]=br,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 4:Er=Im&255,Sr=HEAP32[km>>2]|0,HEAP8[Sr>>0]=Er,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 6:Nr=HEAP32[km>>2]|0,HEAP32[Nr>>2]=Im,Vr=R,Fm=Im,ay=Go,py=yy;continue e;case 7:Cr=(Im|0)<0,kr=Cr<<31>>31,Lr=HEAP32[km>>2]|0,Ar=Lr,Or=Ar,HEAP32[Or>>2]=Im,Mr=Ar+4|0,_r=Mr,HEAP32[_r>>2]=kr,Vr=R,Fm=Im,ay=Go,py=yy;continue e;default:Vr=R,Fm=Im,ay=Go,py=yy;continue e}break;case 112:Dr=My>>>0>8,Pr=Dr?My:8,Br=Sg|8,xg=Br,_y=Pr,bb=120,Wb=64;break;case 88:case 120:xg=Sg,_y=My,bb=yb,Wb=64;break;case 111:xi=km,Ti=xi,Ci=HEAP32[Ti>>2]|0,ki=xi+4|0,Li=ki,Ai=HEAP32[Li>>2]|0,Oi=(Ci|0)==0,Mi=(Ai|0)==0,_i=Oi&Mi;if(_i)f=bt;else{h=bt,Pi=Ci,Ii=Ai;for(;;){Di=Pi&7,Hi=Di|48,ji=Hi&255,Fi=h+ -1|0,HEAP8[Fi>>0]=ji,qi=_bitshift64Lshr(Pi|0,Ii|0,3)|0,Ri=tempRet0,Ui=(qi|0)==0,zi=(Ri|0)==0,Wi=Ui&zi;if(Wi){f=Fi;break}h=Fi,Pi=qi,Ii=Ri}}Xi=Sg&8,Vi=(Xi|0)==0,Vi?(dm=f,Tg=Sg,Dy=My,qy=0,Vy=25043,Wb=77):(Ji=f,Ki=xr-Ji|0,Qi=Ki+1|0,Gi=(My|0)<(Qi|0),et=Gi?Qi:My,dm=f,Tg=Sg,Dy=et,qy=0,Vy=25043,Wb=77);break;case 105:case 100:Yi=km,Zi=Yi,es=HEAP32[Zi>>2]|0,ts=Yi+4|0,ns=ts,rs=HEAP32[ns>>2]|0,ss=(rs|0)<0;if(ss){os=_i64Subtract(0,0,es|0,rs|0)|0,us=tempRet0,as=km,fs=as,HEAP32[fs>>2]=os,ls=as+4|0,cs=ls,HEAP32[cs>>2]=us,gs=os,ys=us,Fy=1,zy=25043,Wb=76;break s}hs=Sg&2048,ps=(hs|0)==0,ps?(ds=Sg&1,ms=(ds|0)==0,s=ms?25043:25045,gs=es,ys=rs,Fy=ds,zy=s,Wb=76):(gs=es,ys=rs,Fy=1,zy=25044,Wb=76);break;case 117:rr=km,ir=rr,sr=HEAP32[ir>>2]|0,or=rr+4|0,ur=or,ar=HEAP32[ur>>2]|0,gs=sr,ys=ar,Fy=0,zy=25043,Wb=76;break;case 99:Rs=km,Us=Rs,zs=HEAP32[Us>>2]|0,Ws=Rs+4|0,Xs=Ws,Vs=HEAP32[Xs>>2]|0,$s=zs&255,HEAP8[Cs>>0]=$s,ym=Cs,Ng=nr,jy=1,Uy=0,$y=25043,Pb=bt;break;case 109:Js=___errno_location()|0,Qs=HEAP32[Js>>2]|0,Gs=_strerror(Qs)|0,vm=Gs,Wb=82;break;case 115:Ys=HEAP32[km>>2]|0,Zs=(Ys|0)!=(0|0),eo=Zs?Ys:25053,vm=eo,Wb=82;break;case 67:ao=km,fo=ao,lo=HEAP32[fo>>2]|0,co=ao+4|0,ho=co,po=HEAP32[ho>>2]|0,HEAP32[Nb>>2]=lo,HEAP32[Ou>>2]=0,HEAP32[km>>2]=Nb,By=-1,Wb=86;break;case 83:vo=(My|0)==0,vo?(_pad(e,32,Sb,0,Sg),kg=0,Wb=98):(By=My,Wb=86);break;case 65:case 71:case 70:case 69:case 97:case 103:case 102:case 101:Xo=+HEAPF64[km>>3],HEAP32[ng>>2]=0,HEAPF64[tempDoublePtr>>3]=Xo,Vo=HEAP32[tempDoublePtr>>2]|0,$o=HEAP32[tempDoublePtr+4>>2]|0,Jo=($o|0)<0,Jo?(Ko=-Xo,p=Ko,Iy=1,Xy=25060):(Qo=Sg&2048,Yo=(Qo|0)==0,Yo?(Zo=Sg&1,eu=(Zo|0)==0,o=eu?25061:25066,p=Xo,Iy=Zo,Xy=o):(p=Xo,Iy=1,Xy=25063)),HEAPF64[tempDoublePtr>>3]=p,tu=HEAP32[tempDoublePtr>>2]|0,nu=HEAP32[tempDoublePtr+4>>2]|0,ru=nu&2146435072,iu=ru>>>0<2146435072,su=!1,ou=(ru|0)==2146435072,uu=ou&su,fu=iu|uu;do if(fu){Cu=+_frexpl(p,ng),ku=Cu*2,Lu=ku!=0,Lu&&(Au=HEAP32[ng>>2]|0,_u=Au+ -1|0,HEAP32[ng>>2]=_u),Du=yb|32,Pu=(Du|0)==97;if(Pu){Hu=yb&32,Bu=(Hu|0)==0,ju=Xy+9|0,Wy=Bu?Xy:ju,Fu=Iy|2,Iu=My>>>0>11,qu=12-My|0,Ru=(qu|0)==0,zu=Iu|Ru;do{if(!zu){Ky=qu,Qy=8;for(;;){Wu=Ky+ -1|0,Xu=Qy*16,Vu=(Wu|0)==0;if(Vu){J=Xu;break}Ky=Wu,Qy=Xu}$u=HEAP8[Wy>>0]|0,Ju=$u<<24>>24==45;if(Ju){Ku=-ku,Qu=Ku-J,Gu=J+Qu,Yu=-Gu,d=Yu;break}ea=ku+J,ta=ea-J,d=ta;break}d=ku}while(0);na=HEAP32[ng>>2]|0,ra=(na|0)<0,ia=0-na|0,sa=ra?ia:na,oa=(sa|0)<0,ua=oa<<31>>31,aa=_fmt_u(sa,ua,Pf)|0,fa=(aa|0)==(Pf|0),fa?(HEAP8[jc>>0]=48,ig=jc):ig=aa,ca=na>>31,ha=ca&2,pa=ha+43|0,da=pa&255,va=ig+ -1|0,HEAP8[va>>0]=da,ma=yb+15|0,ga=ma&255,ya=ig+ -2|0,HEAP8[ya>>0]=ga,Ey=(My|0)<1,ba=Sg&8,wa=(ba|0)==0,m=d,Yy=Hm;for(;;){Sa=~~m,xa=25027+Sa|0,Ta=HEAP8[xa>>0]|0,Na=Ta&255,Ca=Na|Hu,ka=Ca&255,La=Yy+1|0,HEAP8[Yy>>0]=ka,Aa=+(Sa|0),Oa=m-Aa,Ma=Oa*16,Da=La,Pa=Da-qp|0,Ha=(Pa|0)==1;do if(Ha){wy=Ma==0,Ay=Ey&wy,xy=wa&Ay;if(xy){Zy=La;break}Ba=Yy+2|0,HEAP8[La>>0]=46,Zy=Ba}else Zy=La;while(0);ja=Ma!=0;if(!ja){eb=Zy;break}m=Ma,Yy=Zy}Fa=(My|0)!=0,lt=eb,Ia=wt+lt|0,qa=(Ia|0)<(My|0),ky=Fa&qa,Ra=ya,Ua=Mt+My|0,Wa=Ua-Ra|0,Xa=ya,Va=rm-Xa|0,$a=Va+lt|0,fy=ky?Wa:$a,Ja=fy+Fu|0,_pad(e,32,Sb,Ja,Sg),Ka=HEAP32[e>>2]|0,Qa=Ka&32,Ga=(Qa|0)==0,Ga&&___fwritex(Wy,Fu,e)|0,Ya=Sg^65536,_pad(e,48,Sb,Ja,Ya),Za=lt-qp|0
,tf=HEAP32[e>>2]|0,nf=tf&32,rf=(nf|0)==0,rf&&___fwritex(Hm,Za,e)|0,sf=ya,of=zv-sf|0,gb=Za+of|0,uf=fy-gb|0,_pad(e,48,uf,0,0),af=HEAP32[e>>2]|0,ff=af&32,lf=(ff|0)==0,lf&&___fwritex(ya,of,e)|0,cf=Sg^8192,_pad(e,32,Sb,Ja,cf),pf=(Ja|0)<(Sb|0),wb=pf?Sb:Ja,a=wb;break}df=(My|0)<0,nt=df?6:My,Lu?(vf=ku*268435456,mf=HEAP32[ng>>2]|0,gf=mf+ -28|0,HEAP32[ng>>2]=gf,x=vf,yf=gf):(ft=HEAP32[ng>>2]|0,x=ku,yf=ft),bf=(yf|0)<0,T=bf?Dm:Ut,wf=T,C=x,Lb=T;for(;;){Ef=~~C>>>0,HEAP32[Lb>>2]=Ef,Sf=Lb+4|0,Tf=+(Ef>>>0),Nf=C-Tf,Cf=Nf*1e9,kf=Cf!=0;if(!kf){W=Sf;break}C=Cf,Lb=Sf}it=HEAP32[ng>>2]|0,Lf=(it|0)>0;if(Lf){Af=it,gm=T,Db=W;for(;;){Of=(Af|0)>29,Mf=Of?29:Af,Um=Db+ -4|0,_f=Um>>>0<gm>>>0;do if(_f)bm=gm;else{Bm=0,zm=Um;for(;;){Df=HEAP32[zm>>2]|0,Bf=_bitshift64Shl(Df|0,0,Mf|0)|0,jf=tempRet0,Ff=_i64Add(Bf|0,jf|0,Bm|0,0)|0,If=tempRet0,qf=___uremdi3(Ff|0,If|0,1e9,0)|0,Rf=tempRet0,HEAP32[zm>>2]=qf,Uf=___udivdi3(Ff|0,If|0,1e9,0)|0,zf=tempRet0,Rm=zm+ -4|0,Wf=Rm>>>0<gm>>>0;if(Wf){X=Uf;break}Bm=Uf,zm=Rm}Xf=(X|0)==0;if(Xf){bm=gm;break}$f=gm+ -4|0,HEAP32[$f>>2]=X,bm=$f}while(0);Hb=Db;for(;;){Jf=Hb>>>0>bm>>>0;if(!Jf){Bb=Hb;break}Kf=Hb+ -4|0,Qf=HEAP32[Kf>>2]|0,Gf=(Qf|0)==0;if(!Gf){Bb=Hb;break}Hb=Kf}Yf=HEAP32[ng>>2]|0,Zf=Yf-Mf|0,HEAP32[ng>>2]=Zf,el=(Zf|0)>0;if(!el){st=Zf,mm=bm,_b=Bb;break}Af=Zf,gm=bm,Db=Bb}}else st=it,mm=T,_b=W;tl=(st|0)<0;if(tl){nl=nt+25|0,il=(nl|0)/9&-1,sl=il+1|0,ol=(Du|0)==102,al=st,Em=mm,Fb=_b;for(;;){ul=0-al|0,fl=(ul|0)>9,ll=fl?9:ul,cl=Em>>>0<Fb>>>0;do if(cl){ml=1<<ll,gl=ml+ -1|0,yl=1e9>>>ll,jm=0,Wm=Em;for(;;){bl=HEAP32[Wm>>2]|0,wl=bl&gl,El=bl>>>ll,Sl=El+jm|0,HEAP32[Wm>>2]=Sl,xl=Math_imul(wl,yl)|0,Tl=Wm+4|0,Cl=Tl>>>0<Fb>>>0;if(!Cl){V=xl;break}jm=xl,Wm=Tl}kl=HEAP32[Em>>2]|0,Ll=(kl|0)==0,Al=Em+4|0,_=Ll?Al:Em,Ol=(V|0)==0;if(Ol){P=_,Ib=Fb;break}Ml=Fb+4|0,HEAP32[Fb>>2]=V,P=_,Ib=Ml}else hl=HEAP32[Em>>2]|0,pl=(hl|0)==0,vl=Em+4|0,D=pl?vl:Em,P=D,Ib=Fb;while(0);_l=ol?T:P,Dl=Ib,Pl=_l,Hl=Dl-Pl|0,jl=Hl>>2,Fl=(jl|0)>(sl|0),Il=_l+(sl<<2)|0,gt=Fl?Il:Ib,ql=HEAP32[ng>>2]|0,Rl=ql+ll|0,HEAP32[ng>>2]=Rl,Ul=(Rl|0)<0;if(!Ul){wm=P,jb=gt;break}al=Rl,Em=P,Fb=gt}}else wm=mm,jb=_b;zl=wm>>>0<jb>>>0;do if(zl){Wl=wm,Xl=wf-Wl|0,Vl=Xl>>2,Jl=Vl*9|0,Kl=HEAP32[wm>>2]|0,Ql=Kl>>>0<10;if(Ql){Ym=Jl;break}Gm=Jl,Ag=10;for(;;){Gl=Ag*10|0,Yl=Gm+1|0,Zl=Kl>>>0<Gl>>>0;if(Zl){Ym=Yl;break}Gm=Yl,Ag=Gl}}else Ym=0;while(0);ec=(Du|0)!=102,tc=ec?Ym:0,nc=nt-tc|0,rc=(Du|0)==103,sc=(nt|0)!=0,oc=sc&rc,G=oc<<31>>31,uc=nc+G|0,ac=jb,fc=ac-wf|0,lc=fc>>2,cc=lc*9|0,hc=cc+ -9|0,pc=(uc|0)<(hc|0);if(pc){dc=uc+9216|0,mc=(dc|0)/9&-1,pt=mc+ -1023|0,gc=T+(pt<<2)|0,yc=(dc|0)%9&-1,iy=yc+1|0,bc=(iy|0)<9;if(bc){Dg=10,sy=iy;for(;;){wc=Dg*10|0,ry=sy+1|0,ag=(ry|0)==9;if(ag){_g=wc;break}Dg=wc,sy=ry}}else _g=10;Ec=HEAP32[gc>>2]|0,Sc=(Ec>>>0)%(_g>>>0)&-1,xc=(Sc|0)==0,xc?(dt=mc+ -1022|0,Tc=T+(dt<<2)|0,Nc=(Tc|0)==(jb|0),Nc?(Nm=wm,$m=gc,eg=Ym):Wb=163):Wb=163;do if((Wb|0)==163){Wb=0,kc=(Ec>>>0)/(_g>>>0)&-1,Lc=kc&1,Ac=(Lc|0)==0,g=Ac?9007199254740992:9007199254740994,Oc=(_g|0)/2&-1,Mc=Sc>>>0<Oc>>>0;do if(Mc)fb=.5;else{_c=(Sc|0)==(Oc|0);if(_c){vt=mc+ -1022|0,Dc=T+(vt<<2)|0,Pc=(Dc|0)==(jb|0);if(Pc){fb=1;break}}fb=1.5}while(0);Hc=(Iy|0)==0;do if(Hc)Gy=g,lb=fb;else{Bc=HEAP8[Xy>>0]|0,Ic=Bc<<24>>24==45;if(!Ic){Gy=g,lb=fb;break}qc=-g,Rc=-fb,Gy=qc,lb=Rc}while(0);Uc=Ec-Sc|0,HEAP32[gc>>2]=Uc,zc=Gy+lb,Wc=zc!=Gy;if(!Wc){Nm=wm,$m=gc,eg=Ym;break}Xc=Uc+_g|0,HEAP32[gc>>2]=Xc,Vc=Xc>>>0>999999999;if(Vc){xm=wm,Vm=gc;for(;;){$c=Vm+ -4|0,HEAP32[Vm>>2]=0,Jc=$c>>>0<xm>>>0,Jc?(Qc=xm+ -4|0,HEAP32[Qc>>2]=0,Tm=Qc):Tm=xm,Gc=HEAP32[$c>>2]|0,Yc=Gc+1|0,HEAP32[$c>>2]=Yc,Zc=Yc>>>0>999999999;if(!Zc){Sm=Tm,Xm=$c;break}xm=Tm,Vm=$c}}else Sm=wm,Xm=gc;eh=Sm,th=wf-eh|0,nh=th>>2,rh=nh*9|0,ih=HEAP32[Sm>>2]|0,sh=ih>>>0<10;if(sh){Nm=Sm,$m=Xm,eg=rh;break}Zm=rh,Hg=10;for(;;){uh=Hg*10|0,ah=Zm+1|0,fh=ih>>>0<uh>>>0;if(fh){Nm=Sm,$m=Xm,eg=ah;break}Zm=ah,Hg=uh}}while(0);lh=$m+4|0,ch=jb>>>0>lh>>>0,mt=ch?lh:jb,Cm=Nm,tg=eg,zb=mt}else Cm=wm,tg=Ym,zb=jb;hh=0-tg|0,Rb=zb;for(;;){ph=Rb>>>0>Cm>>>0;if(!ph){j=0,Ub=Rb;break}dh=Rb+ -4|0,vh=HEAP32[dh>>2]|0,mh=(vh|0)==0;if(!mh){j=1,Ub=Rb;break}Rb=dh}do{if(rc){yh=sc&1,bh=yh^1,Z=bh+nt|0,wh=(Z|0)>(tg|0),Eh=(tg|0)>-5,Oy=wh&Eh,Oy?(Sh=yb+ -1|0,Y=Z+ -1|0,xh=Y-tg|0,c=Sh,E=xh):(Th=yb+ -2|0,Nh=Z+ -1|0,c=Th,E=Nh),Ch=Sg&8,kh=(Ch|0)==0;if(!kh){v=c,N=E,at=Ch;break}do if(j){Ah=Ub+ -4|0,Oh=HEAP32[Ah>>2]|0,Mh=(Oh|0)==0;if(Mh){uy=9;break}_h=(Oh>>>0)%10&-1,Dh=(_h|0)==0;if(!Dh){uy=0;break}Ig=10,oy=0;for(;;){Ph=Ig*10|0,Hh=oy+1|0,Bh=(Oh>>>0)%(Ph>>>0)&-1,jh=(Bh|0)==0;if(!jh){uy=Hh;break}Ig=Ph,oy=Hh}}else uy=9;while(0);Fh=c|32,qh=(Fh|0)==102,Rh=Ub,Uh=Rh-wf|0,zh=Uh>>2,Wh=zh*9|0,Xh=Wh+ -9|0;if(qh){Vh=Xh-uy|0,$h=(Vh|0)<0,y=$h?0:Vh,Jh=(E|0)<(y|0),b=Jh?E:y,v=c,N=b,at=0;break}Kh=Xh+tg|0,Gh=Kh-uy|0,Yh=(Gh|0)<0,S=Yh?0:Gh,Zh=(E|0)<(S|0),w=Zh?E:S,v=c,N=w,at=0;break}ct=Sg&8,v=yb,N=nt,at=ct}while(0);ep=N|at,tp=(ep|0)!=0,np=tp&1,rp=v|32,ip=(rp|0)==102;if(ip)sp=(tg|0)>0,op=sp?tg:0,rt=op,ug=0;else{ap=(tg|0)<0,fp=ap?hh:tg,lp=(fp|0)<0,cp=lp<<31>>31,hp=_fmt_u(fp,cp,Pf)|0,pp=hp,dp=zv-pp|0,vp=(dp|0)<2;if(vp){og=hp;for(;;){mp=og+ -1|0,HEAP8[mp>>0]=48,gp=mp,bp=zv-gp|0,wp=(bp|0)<2;if(!wp){sg=mp;break}og=mp}}else sg=hp;Ep=tg>>31,Sp=Ep&2,xp=Sp+43|0,Tp=xp&255,Np=sg+ -1|0,HEAP8[Np>>0]=Tp,Cp=v&255,kp=sg+ -2|0,HEAP8[kp>>0]=Cp,Lp=kp,Op=zv-Lp|0,rt=Op,ug=kp}Mp=Iy+1|0,_p=Mp+N|0,ly=_p+np|0,Dp=ly+rt|0,_pad(e,32,Sb,Dp,Sg),Pp=HEAP32[e>>2]|0,Hp=Pp&32,Bp=(Hp|0)==0,Bp&&___fwritex(Xy,Iy,e)|0,jp=Sg^65536,_pad(e,48,Sb,Dp,jp);do if(ip){Fp=Cm>>>0>T>>>0,Jy=Fp?T:Cm,Jm=Jy;for(;;){Ip=HEAP32[Jm>>2]|0,Up=_fmt_u(Ip,0,Zt)|0,zp=(Jm|0)==(Jy|0);do if(zp){$p=(Up|0)==(Zt|0);if(!$p){rb=Up;break}HEAP8[Sn>>0]=48,rb=Sn}else{Wp=Up>>>0>Hm>>>0;if(!Wp){rb=Up;break}nb=Up;for(;;){Xp=nb+ -1|0,HEAP8[Xp>>0]=48,Vp=Xp>>>0>Hm>>>0;if(!Vp){rb=Xp;break}nb=Xp}}while(0);Jp=HEAP32[e>>2]|0,Kp=Jp&32,Qp=(Kp|0)==0,Qp&&(Gp=rb,Zp=cn-Gp|0,___fwritex(rb,Zp,e)|0),ed=Jm+4|0,td=ed>>>0>T>>>0;if(td){$=ed;break}Jm=ed}nd=(ep|0)==0;do if(!nd){rd=HEAP32[e>>2]|0,id=rd&32,sd=(id|0)==0;if(!sd)break;___fwritex(25095,1,e)|0}while(0);od=$>>>0<Ub>>>0,ud=(N|0)>0,ad=ud&od;if(ad){L=N,Km=$;for(;;){ld=HEAP32[Km>>2]|0,cd=_fmt_u(ld,0,Zt)|0,hd=cd>>>0>Hm>>>0;if(hd){sb=cd;for(;;){pd=sb+ -1|0,HEAP8[pd>>0]=48,dd=pd>>>0>Hm>>>0;if(!dd){ib=pd;break}sb=pd}}else ib=cd;vd=HEAP32[e>>2]|0,md=vd&32,gd=(md|0)==0,gd&&(yd=(L|0)>9,bd=yd?9:L,___fwritex(ib,bd,e)|0),Ed=Km+4|0,Sd=L+ -9|0,xd=Ed>>>0<Ub>>>0,Td=(L|0)>9,Nd=Td&xd;if(!Nd){k=Sd;break}L=Sd,Km=Ed}}else k=N;Cd=k+9|0,_pad(e,48,Cd,9,0)}else{kd=Cm+4|0,qb=j?Ub:kd,Ld=(N|0)>-1;if(Ld){Ad=(at|0)==0,M=N,Qm=Cm;for(;;){Od=HEAP32[Qm>>2]|0,_d=_fmt_u(Od,0,Zt)|0,Dd=(_d|0)==(Zt|0),Dd?(HEAP8[Sn>>0]=48,ob=Sn):ob=_d,Pd=(Qm|0)==(Cm|0);do if(Pd){Fd=ob+1|0,Id=HEAP32[e>>2]|0,qd=Id&32,Rd=(qd|0)==0,Rd&&___fwritex(ob,1,e)|0,zd=(M|0)<1,Ly=Ad&zd;if(Ly){ab=Fd;break}Wd=HEAP32[e>>2]|0,Xd=Wd&32,Vd=(Xd|0)==0;if(!Vd){ab=Fd;break}___fwritex(25095,1,e)|0,ab=Fd}else{Hd=ob>>>0>Hm>>>0;if(!Hd){ab=ob;break}ub=ob;for(;;){Bd=ub+ -1|0,HEAP8[Bd>>0]=48,jd=Bd>>>0>Hm>>>0;if(!jd){ab=Bd;break}ub=Bd}}while(0);$d=ab,Jd=cn-$d|0,Kd=HEAP32[e>>2]|0,Qd=Kd&32,Gd=(Qd|0)==0,Gd&&(Yd=(M|0)>(Jd|0),ev=Yd?Jd:M,___fwritex(ab,ev,e)|0),tv=M-Jd|0,nv=Qm+4|0,rv=nv>>>0<qb>>>0,iv=(tv|0)>-1,sv=rv&iv;if(!sv){O=tv;break}M=tv,Qm=nv}}else O=N;ov=O+18|0,_pad(e,48,ov,18,0),uv=HEAP32[e>>2]|0,av=uv&32,fv=(av|0)==0;if(!fv)break;cv=ug,hv=zv-cv|0,___fwritex(ug,hv,e)|0}while(0);pv=Sg^8192,_pad(e,32,Sb,Dp,pv),dv=(Dp|0)<(Sb|0),Tb=dv?Sb:Dp,a=Tb}else lu=yb&32,cu=(lu|0)!=0,hu=cu?25079:25083,pu=p!=p|!1,du=cu?25087:25091,Ry=pu?0:Iy,tb=pu?du:hu,vu=Ry+3|0,_pad(e,32,Sb,vu,nr),mu=HEAP32[e>>2]|0,gu=mu&32,yu=(gu|0)==0,yu?(___fwritex(Xy,Ry,e)|0,ut=HEAP32[e>>2]|0,Eu=ut):Eu=mu,wu=Eu&32,Su=(wu|0)==0,Su&&___fwritex(tb,3,e)|0,xu=Sg^8192,_pad(e,32,Sb,vu,xu),Tu=(vu|0)<(Sb|0),Nu=Tu?Sb:vu,a=Nu;while(0);Vr=R,Fm=Im,ay=a,py=yy;continue e;default:ym=Vr,Ng=Sg,jy=My,Uy=0,$y=25043,Pb=bt}while(0);o:do if((Wb|0)==64){Wb=0,jr=km,Fr=jr,Ir=HEAP32[Fr>>2]|0,qr=jr+4|0,Rr=qr,Ur=HEAP32[Rr>>2]|0,zr=bb&32,Wr=(Ir|0)==0,Xr=(Ur|0)==0,$r=Wr&Xr;if($r)dm=bt,Tg=xg,Dy=_y,qy=0,Vy=25043,Wb=77;else{l=bt,Kr=Ir,ni=Ur;for(;;){Jr=Kr&15,Qr=25027+Jr|0,Gr=HEAP8[Qr>>0]|0,Yr=Gr&255,Zr=Yr|zr,ei=Zr&255,ti=l+ -1|0,HEAP8[ti>>0]=ei,ii=_bitshift64Lshr(Kr|0,ni|0,4)|0,si=tempRet0,oi=(ii|0)==0,ui=(si|0)==0,ai=oi&ui;if(ai){K=ti;break}l=ti,Kr=ii,ni=si}fi=km,li=fi,ci=HEAP32[li>>2]|0,hi=fi+4|0,pi=hi,vi=HEAP32[pi>>2]|0,mi=(ci|0)==0,gi=(vi|0)==0,yi=mi&gi,bi=xg&8,wi=(bi|0)==0,Ny=wi|yi,Ny?(dm=K,Tg=xg,Dy=_y,qy=0,Vy=25043,Wb=77):(Ei=bb>>4,Si=25043+Ei|0,dm=K,Tg=xg,Dy=_y,qy=2,Vy=Si,Wb=77)}}else if((Wb|0)==76)Wb=0,bs=_fmt_u(gs,ys,bt)|0,dm=bs,Tg=Sg,Dy=My,qy=Fy,Vy=zy,Wb=77;else if((Wb|0)==82)Wb=0,to=_memchr(vm,0,My)|0,no=(to|0)==(0|0),ro=to,io=vm,so=ro-io|0,uo=vm+My|0,Mb=no?uo:to,Hy=no?My:so,ym=vm,Ng=nr,jy=Hy,Uy=0,$y=25043,Pb=Mb;else if((Wb|0)==86){Wb=0,mo=HEAP32[km>>2]|0,Lg=0,cy=0,Cb=mo;for(;;){go=HEAP32[Cb>>2]|0,bo=(go|0)==0;if(bo){Cg=Lg,hy=cy;break}wo=_wctomb(by,go)|0,Eo=(wo|0)<0,So=By-Lg|0,xo=wo>>>0>So>>>0,Cy=Eo|xo;if(Cy){Cg=Lg,hy=wo;break}To=Cb+4|0,No=wo+Lg|0,Co=By>>>0>No>>>0;if(!Co){Cg=No,hy=wo;break}Lg=No,cy=wo,Cb=To}ko=(hy|0)<0;if(ko){u=-1;break e}_pad(e,32,Sb,Cg,Sg),Lo=(Cg|0)==0;if(Lo)kg=0,Wb=98;else{Oo=HEAP32[km>>2]|0,Pg=0,kb=Oo;for(;;){Mo=HEAP32[kb>>2]|0,_o=(Mo|0)==0;if(_o){kg=Cg,Wb=98;break o}Do=kb+4|0,Po=_wctomb(by,Mo)|0,Ho=Po+Pg|0,Bo=(Ho|0)>(Cg|0);if(Bo){kg=Cg,Wb=98;break o}jo=HEAP32[e>>2]|0,Fo=jo&32,Io=(Fo|0)==0,Io&&___fwritex(by,Po,e)|0,Ro=Ho>>>0<Cg>>>0;if(!Ro){kg=Cg,Wb=98;break}Pg=Ho,kb=Do}}}while(0);if((Wb|0)==98){Wb=0,Uo=Sg^8192,_pad(e,32,Sb,kg,Uo),zo=(Sb|0)>(kg|0),Wo=zo?Sb:kg,Vr=R,Fm=Im,ay=Wo,py=yy;continue}(Wb|0)==77&&(Wb=0,ws=(Dy|0)>-1,Es=Tg&-65537,H=ws?Es:Tg,Ss=km,xs=Ss,Ts=HEAP32[xs>>2]|0,Ns=Ss+4|0,Ls=Ns,As=HEAP32[Ls>>2]|0,Os=(Ts|0)!=0,Ms=(As|0)!=0,_s=Os|Ms,Ds=(Dy|0)!=0,Sy=Ds|_s,Sy?(Ps=dm,Hs=xr-Ps|0,Bs=_s&1,js=Bs^1,Is=js+Hs|0,qs=(Dy|0)>(Is|0),Py=qs?Dy:Is,ym=dm,Ng=H,jy=Py,Uy=qy,$y=Vy,Pb=bt):(ym=bt,Ng=H,jy=0,Uy=qy,$y=Vy,Pb=bt)),vv=Pb,mv=ym,gv=vv-mv|0,yv=(jy|0)<(gv|0),tt=yv?gv:jy,bv=Uy+tt|0,wv=(Sb|0)<(bv|0),xb=wv?bv:Sb,_pad(e,32,xb,bv,Ng),Sv=HEAP32[e>>2]|0,xv=Sv&32,Tv=(xv|0)==0,Tv&&___fwritex($y,Uy,e)|0,Nv=Ng^65536,_pad(e,48,xb,bv,Nv),_pad(e,48,tt,gv,0),Cv=HEAP32[e>>2]|0,kv=Cv&32,Lv=(kv|0)==0,Lv&&___fwritex(ym,gv,e)|0,Av=Ng^8192,_pad(e,32,xb,bv,Av),Vr=R,Fm=Im,ay=xb,py=yy}u:do if((Wb|0)==245){Ov=(e|0)==(0|0);if(Ov){Mv=(dy|0)==0;if(Mv)u=0;else{Bg=1;for(;;){Dv=i+(Bg<<2)|0,Pv=HEAP32[Dv>>2]|0,Hv=(Pv|0)==0;if(Hv){jg=Bg;break}jv=r+(Bg<<3)|0,_pop_arg80(jv,Pv,n),Fv=Bg+1|0,Iv=(Fv|0)<10;if(!Iv){u=1;break u}Bg=Fv}Bv=(jg|0)<10;if(Bv){Fg=jg;for(;;){Uv=i+(Fg<<2)|0,Xv=HEAP32[Uv>>2]|0,Vv=(Xv|0)==0,qv=Fg+1|0;if(!Vv){u=-1;break u}Rv=(qv|0)<10;if(!Rv){u=1;break}Fg=qv}}else u=1}}else u=qm}while(0);return STACKTOP=Xb,u|0}function _pop_arg80(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0;Ar=STACKTOP,s=t>>>0>20;e:do if(!s)do switch(t|0){case 9:hn=HEAP32[n>>2]|0,o=hn,M=4,Dn=M,_n=Dn-1|0,U=o+_n|0,Z=4,Bn=Z,Hn=Bn-1|0,Pn=Hn^-1,lt=U&Pn,Et=lt,_t=HEAP32[Et>>2]|0,Sn=Et+4|0,HEAP32[n>>2]=Sn,HEAP32[e>>2]=_t;break e;case 10:mn=HEAP32[n>>2]|0,zt=mn,en=4,Fn=en,jn=Fn-1|0,u=zt+jn|0,y=4,Rn=y,qn=Rn-1|0,In=qn^-1,x=u&In,T=x,N=HEAP32[T>>2]|0,An=T+4|0,HEAP32[n>>2]=An,C=(N|0)<0,k=C<<31>>31,L=e,A=L,HEAP32[A>>2]=N,O=L+4|0,_=O,HEAP32[_>>2]=k;break e;case 11:wn=HEAP32[n>>2]|0,D=wn,P=4,zn=P,Un=zn-1|0,H=D+Un|0,B=4,Vn=B,Xn=Vn-1|0,Wn=Xn^-1,j=H&Wn,F=j,I=HEAP32[F>>2]|0,On=F+4|0,HEAP32[n>>2]=On,q=e,R=q,HEAP32[R>>2]=I,z=q+4|0,W=z,HEAP32[W>>2]=0;break e;case 12:En=HEAP32[n>>2]|0,X=En,V=8,Jn=V,$n=Jn-1|0,$=X+$n|0,J=8,Gn=J,Qn=Gn-1|0,Kn=Qn^-1,K=$&Kn,Q=K,G=Q,Y=G,et=HEAP32[Y>>2]|0,tt=G+4|0,nt=tt,rt=HEAP32[nt>>2]|0,Mn=Q+8|0,HEAP32[n>>2]=Mn,it=e,st=it,HEAP32[st>>2]=et,ot=it+4|0,ut=ot,HEAP32[ut>>2]=rt;break e;case 13:pn=HEAP32[n>>2]|0,at=pn,ft=4,Zn=ft,Yn=Zn-1|0,ct=at+Yn|0,ht=4,nr=ht,tr=nr-1|0,er=tr^-1,pt=ct&er,dt=pt,vt=HEAP32[dt>>2]|0,xn=dt+4|0,HEAP32[n>>2]=xn,mt=vt&65535,gt=mt<<16>>16,yt=(gt|0)<0,bt=yt<<31>>31,wt=e,St=wt,HEAP32[St>>2]=gt,xt=wt+4|0,Tt=xt,HEAP32[Tt>>2]=bt;break e;case 14:dn=HEAP32[n>>2]|0,Nt=dn,Ct=4,ir=Ct,rr=ir-1|0,kt=Nt+rr|0,Lt=4,ur=Lt,or=ur-1|0,sr=or^-1,At=kt&sr,Ot=At,Mt=HEAP32[Ot>>2]|0,Tn=Ot+4|0,HEAP32[n>>2]=Tn,i=Mt&65535,Dt=e,Pt=Dt,HEAP32[Pt>>2]=i,Ht=Dt+4|0,Bt=Ht,HEAP32[Bt>>2]=0;break e;case 15:vn=HEAP32[n>>2]|0,jt=vn,Ft=4,fr=Ft,ar=fr-1|0,It=jt+ar|0,qt=4,hr=qt,cr=hr-1|0,lr=cr^-1,Rt=It&lr,Ut=Rt,Wt=HEAP32[Ut>>2]|0,Nn=Ut+4|0,HEAP32[n>>2]=Nn,Xt=Wt&255,Vt=Xt<<24>>24,$t=(Vt|0)<0,Jt=$t<<31>>31,Kt=e,Qt=Kt,HEAP32[Qt>>2]=Vt,Gt=Kt+4|0,Yt=Gt,HEAP32[Yt>>2]=Jt;break e;case 16:gn=HEAP32[n>>2]|0,Zt=gn,tn=4,dr=tn,pr=dr-1|0,nn=Zt+pr|0,rn=4,gr=rn,mr=gr-1|0,vr=mr^-1,sn=nn&vr,on=sn,un=HEAP32[on>>2]|0,Cn=on+4|0,HEAP32[n>>2]=Cn,r=un&255,an=e,fn=an,HEAP32[fn>>2]=r,ln=an+4|0,cn=ln,HEAP32[cn>>2]=0;break e;case 17:yn=HEAP32[n>>2]|0,a=yn,f=8,br=f,yr=br-1|0,l=a+yr|0,c=8,Sr=c,Er=Sr-1|0,wr=Er^-1,h=l&wr,p=h,d=+HEAPF64[p>>3],kn=p+8|0,HEAP32[n>>2]=kn,HEAPF64[e>>3]=d;break e;case 18:bn=HEAP32[n>>2]|0,v=bn,m=8,Tr=m,xr=Tr-1|0,g=v+xr|0,b=8,kr=b,Cr=kr-1|0,Nr=Cr^-1,w=g&Nr,E=w,S=+HEAPF64[E>>3],Ln=E+8|0,HEAP32[n>>2]=Ln,HEAPF64[e>>3]=S;break e;default:break e}while(0);while(0);return}function _fmt_u(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0;I=STACKTOP,b=t>>>0>0,O=e>>>0>4294967295,M=(t|0)==0,_=M&O,D=b|_;if(D){s=n,P=e,H=t;for(;;){B=___uremdi3(P|0,H|0,10,0)|0,f=tempRet0,l=B|48,c=l&255,h=s+ -1|0,HEAP8[h>>0]=c,p=___udivdi3(P|0,H|0,10,0)|0,d=tempRet0,v=H>>>0>9,m=P>>>0>4294967295,g=(H|0)==9,y=g&m,w=v|y;if(!w){a=h,L=p,A=d;break}s=h,P=p,H=d}r=a,i=L}else r=n,i=e;E=(i|0)==0;if(E)o=r;else{u=r,j=i;for(;;){S=(j>>>0)%10&-1,x=S|48,T=x&255,N=u+ -1|0,HEAP8[N>>0]=T,C=(j>>>0)/10&-1,k=j>>>0<10;if(k){o=N;break}u=N,j=C}}return o|0}function _pad(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0;O=STACKTOP,STACKTOP=STACKTOP+256|0,(STACKTOP|0)>=(STACK_MAX|0)&&abort(),L=O,a=i&73728,f=(a|0)==0,b=(n|0)>(r|0),k=b&f;do if(k){w=n-r|0,E=w>>>0>256,S=E?256:w,_memset(L|0,t|0,S|0)|0,x=w>>>0>255,T=HEAP32[e>>2]|0,N=T&32,C=(N|0)==0;if(x){l=n-r|0,o=w,g=T,y=C;for(;;){y?(___fwritex(L,256,e)|0,u=HEAP32[e>>2]|0,d=u):d=g,c=o+ -256|0,h=c>>>0>255,p=d&32,v=(p|0)==0;if(!h)break;o=c,g=d,y=v}m=l&255;if(!v)break;s=m}else{if(!C)break;s=w}___fwritex(L,s,e)|0}while(0);STACKTOP=O;return}function _sn_write(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0;return v=STACKTOP,r=e+16|0,i=HEAP32[r>>2]|0,s=e+20|0,o=HEAP32[s>>2]|0,u=i,a=o,f=u-a|0,l=f>>>0>n>>>0,p=l?n:f,_memcpy(o|0,t|0,p|0)|0,c=HEAP32[s>>2]|0,h=c+p|0,HEAP32[s>>2]=h,n|0}function _malloc(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0,su=0,ou=0,uu=0,au=0,fu=0,lu=0,cu=0,hu=0,pu=0,du=0,vu=0,mu=0,gu=0,yu=0,bu=0,wu=0,Eu=0,Su=0,xu=0,Tu=0,Nu=0,Cu=0,ku=0,Lu=0,Au=0,Ou=0,Mu=0,_u=0,Du=0,Pu=0,Hu=0,Bu=0,ju=0,Fu=0,Iu=0,qu=0,Ru=0,Uu=0,zu=0,Wu=0,Xu=0,Vu=0,$u=0,Ju=0,Ku=0,Qu=0,Gu=0,Yu=0,Zu=0,ea=0,ta=0,na=0,ra=0,ia=0,sa=0,oa=0,ua=0,aa=0,fa=0,la=0,ca=0,ha=0,pa=0,da=0,va=0,ma=0,ga=0,ya=0,ba=0,wa=0,Ea=0,Sa=0,xa=0,Ta=0,Na=0,Ca=0,ka=0,La=0,Aa=0,Oa=0,Ma=0,_a=0,Da=0,Pa=0,Ha=0,Ba=0,ja=0,Fa=0,Ia=0,qa=0,Ra=0,Ua=0,za=0,Wa=0,Xa=0,Va=0,$a=0,Ja=0,Ka=0,Qa=0,Ga=0,Ya=0,Za=0,ef=0,tf=0,nf=0,rf=0,sf=0,of=0,uf=0,af=0,ff=0,lf=0,cf=0,hf=0,pf=0,df=0,vf=0,mf=0,gf=0,yf=0,bf=0,wf=0,Ef=0,Sf=0,xf=0,Tf=0,Nf=0,Cf=0,kf=0,Lf=0,Af=0,Of=0,Mf=0,_f=0,Df=0,Pf=0,Hf=0,Bf=0,jf=0,Ff=0,If=0,qf=0,Rf=0,Uf=0,zf=0,Wf=0,Xf=0,Vf=0,$f=0,Jf=0,Kf=0,Qf=0,Gf=0,Yf=0,Zf=0,el=0,tl=0,nl=0,rl=0,il=0,sl=0,ol=0,ul=0,al=0,fl=0,ll=0,cl=0,hl=0,pl=0,dl=0,vl=0,ml=0,gl=0,yl=0,bl=0,wl=0,El=0,Sl=0,xl=0,Tl=0,Nl=0,Cl=0,kl=0,Ll=0,Al=0,Ol=0,Ml=0,_l=0,Dl=0,Pl=0,Hl=0,Bl=0,jl=0,Fl=0,Il=0,ql=0,Rl=0,Ul=0,zl=0,Wl=0,Xl=0,Vl=0,$l=0,Jl=0,Kl=0,Ql=0,Gl=0,Yl=0,Zl=0,ec=0,tc=0,nc=0,rc=0,ic=0,sc=0,oc=0,uc=0,ac=0,fc=0,lc=0,cc=0,hc=0,pc=0,dc=0,vc=0,mc=0,gc=0,yc=0,bc=0,wc=0,Ec=0,Sc=0,xc=0,Tc=0,Nc=0,Cc=0,kc=0,Lc=0,Ac=0,Oc=0,Mc=0,_c=0,Dc=0,Pc=0,Hc=0,Bc=0,jc=0,Fc=0,Ic=0,qc=0,Rc=0,Uc=0,zc=0,Wc=0,Xc=0,Vc=0,$c=0,Jc=0,Kc=0,Qc=0,Gc=0,Yc=0,Zc=0,eh=0,th=0,nh=0,rh=0,ih=0,sh=0,oh=0,uh=0,ah=0,fh=0,lh=0,ch=0,hh=0,ph=0,dh=0,vh=0,mh=0,gh=0,yh=0,bh=0,wh=0,Eh=0,Sh=0,xh=0,Th=0,Nh=0,Ch=0,kh=0,Lh=0,Ah=0,Oh=0,Mh=0,_h=0,Dh=0,Ph=0,Hh=0,Bh=0,jh=0,Fh=0,Ih=0,qh=0,Rh=0,Uh=0,zh=0,Wh=0,Xh=0,Vh=0,$h=0,Jh=0,Kh=0,Qh=0,Gh=0,Yh=0,Zh=0,ep=0,tp=0,np=0,rp=0,ip=0,sp=0,op=0,up=0,ap=0,fp=0,lp=0,cp=0,hp=0,pp=0,dp=0,vp=0,mp=0,gp=0,yp=0,bp=0,wp=0,Ep=0,Sp=0,xp=0,Tp=0,Np=0,Cp=0,kp=0,Lp=0,Ap=0,Op=0,Mp=0,_p=0,Dp=0,Pp=0,Hp=0,Bp=0,jp=0,Fp=0,Ip=0,qp=0,Rp=0,Up=0,zp=0,Wp=0,Xp=0,Vp=0,$p=0,Jp=0,Kp=0,Qp=0,Gp=0,Yp=0,Zp=0,ed=0,td=0,nd=0,rd=0,id=0,sd=0,od=0,ud=0,ad=0,fd=0,ld=0,cd=0,hd=0,pd=0,dd=0,vd=0,md=0,gd=0,yd=0,bd=0,wd=0,Ed=0,Sd=0,xd=0,Td=0,Nd=0,Cd=0,kd=0,Ld=0,Ad=0,Od=0,Md=0,_d=0,Dd=0,Pd=0,Hd=0,Bd=0,jd=0,Fd=0,Id=0,qd=0,Rd=0,Ud=0,zd=0,Wd=0,Xd=0,Vd=0,$d=0,Jd=0,Kd=0,Qd=0,Gd=0,Yd=0,Zd=0,ev=0,tv=0,nv=0,rv=0,iv=0,sv=0,ov=0,uv=0,av=0,fv=0,lv=0,cv=0,hv=0,pv=0,dv=0,vv=0,mv=0,gv=0,yv=0,bv=0,wv=0,Ev=0,Sv=0,xv=0,Tv=0,Nv=0,Cv=0,kv=0,Lv=0,Av=0,Ov=0,Mv=0,_v=0,Dv=0,Pv=0,Hv=0,Bv=0,jv=0,Fv=0,Iv=0,qv=0,Rv=0,Uv=0,zv=0,Wv=0,Xv=0,Vv=0,$v=0,Jv=0,Kv=0,Qv=0,Gv=0,Yv=0,Zv=0,em=0,tm=0,nm=0,rm=0,im=0,sm=0,om=0,um=0,am=0,fm=0,lm=0,cm=0,hm=0,pm=0,dm=0,vm=0,mm=0,gm=0,ym=0,bm=0,wm=0,Em=0,Sm=0,xm=0,Tm=0,Nm=0,Cm=0,km=0,Lm=0,Am=0,Om=0,Mm=0,_m=0,Dm=0,Pm=0,Hm=0,Bm=0,jm=0,Fm=0,Im=0,qm=0,Rm=0,Um=0,zm=0,Wm=0,Xm=0,Vm=0,$m=0,Jm=0,Km=0,Qm=0,Gm=0,Ym=0,Zm=0,eg=0,tg=0,ng=0,rg=0,ig=0,sg=0,og=0,ug=0,ag=0,fg=0,lg=0,cg=0,hg=0,pg=0,dg=0,vg=0,mg=0,gg=0,yg=0,bg=0,wg=0,Eg=0,Sg=0,xg=0,Tg=0,Ng=0,Cg=0,kg=0,Lg=0,Ag=0,Og=0,Mg=0,_g=0,Dg=0,Pg=0,Hg=0,Bg=0,jg=0,Fg=0,Ig=0,qg=0,Rg=0,Ug=0,zg=0,Wg=0,Xg=0,Vg=0,$g=0,Jg=0,Kg=0,Qg=0,Gg=0,Yg=0,Zg=0,ey=0,ty=0,ny=0,ry=0,iy=0,sy=0,oy=0,uy=0,ay=0,fy=0,ly=0,cy=0,hy=0,py=0,dy=0,vy=0,my=0,gy=0,yy=0,by=0,wy=0,Ey=0,Sy=0,xy=0,Ty=0,Ny=0,Cy=0,ky=0,Ly=0,Ay=0,Oy=0,My=0,_y=0,Dy=0,Py=0,Hy=0,By=0,jy=0,Fy=0,Iy=0,qy=0,Ry=0,Uy=0,zy=0,Wy=0,Xy=0,Vy=0,$y=0,Jy=0,Ky=0,Qy=0,Gy=0,Yy=0,Zy=0,eb=0,tb=0,nb=0,rb=0,ib=0,sb=0,ob=0,ub=0,ab=0,fb=0,lb=0,cb=0,hb=0,pb=0,db=0,vb=0,mb=0,gb=0,yb=0,bb=0,wb=0,Eb=0,Sb=0,xb=0,Tb=0,Nb=0,Cb=0,kb=0,Lb=0,Ab=0,Ob=0,Mb=0,_b=0,Db=0,Pb=0,Hb=0,Bb=0,jb=0,Fb=0,Ib=0,qb=0,Rb=0,Ub=0,zb=0,Wb=0,Xb=0,Vb=0,$b=0,Jb=0,Kb=0,Qb=0,Gb=0,Yb=0,Zb=0,ew=0,tw=0,nw=0,rw=0,iw=0,sw=0,ow=0,uw=0,aw=0,fw=0,lw=0,cw=0,hw=0,pw=0,dw=0,vw=0,mw=0,gw=0,yw=0,bw=0,ww=0,Ew=0,Sw=0,xw=0,Tw=0,Nw=0,Cw=0,kw=0,Lw=0,Aw=0,Ow=0,Mw=0,_w=0,Dw=0,Pw=0,Hw=0,Bw=0,jw=0,Fw=0,Iw=0,qw=0,Rw=0,Uw=0,zw=0,Ww=0,Xw=0,Vw=0,$w=0,Jw=0,Kw=0,Qw=0,Gw=0,Yw=0,Zw=0,eE=0,tE=0,nE=0,rE=0,iE=0,sE=0,oE=0,uE=0,aE=0,fE=0,lE=0,cE=0,hE=0,pE=0,dE=0,vE=0,mE=0,gE=0,yE=0,bE=0,wE=0,EE=0,SE=0,xE=0,TE=0,NE=0,CE=0,kE=0,LE=0,AE=0,OE=0,ME=0,_E=0,DE=0,PE=0,HE=0,BE=0,jE=0,FE=0,IE=0,qE=0,RE=0,UE=0,zE=0,WE=0,XE=0,VE=0,$E=0,JE=0,KE=0,QE=0,GE=0,YE=0,ZE=0,eS=0,tS=0,nS=0,rS=0,iS=0,sS=0,oS=0,uS=0,aS=0,fS=0,lS=0,cS=0,hS=0,pS=0,dS=0,vS=0,mS=0,gS=0,yS=0,bS=0,wS=0,ES=0,SS=0,xS=0,TS=0,NS=0,CS=0,kS=0,LS=0,AS=0,OS=0,MS=0,_S=0,DS=0,PS=0,HS=0,BS=0,jS=0,FS=0,IS=0,qS=0,RS=0,US=0,zS=0,WS=0,XS=0,VS=0,$S=0,JS=0,KS=0,QS=0,GS=0,YS=0,ZS=0,ex=0,tx=0,nx=0,rx=0,ix=0,sx=0,ox=0,ux=0,ax=0,fx=0,lx=0,cx=0,hx=0,px=0,dx=0,vx=0,mx=0,gx=0,yx=0,bx=0,wx=0,Ex=0,Sx=0,xx=0,Tx=0,Nx=0,Cx=0,kx=0,Lx=0,Ax=0,Ox=0,Mx=0,_x=0,Dx=0,Px=0,Hx=0,Bx=0,jx=0,Fx=0,Ix=0,qx=0,Rx=0,Ux=0,zx=0,Wx=0,Xx=0,Vx=0,$x=0,Jx=0,Kx=0,Qx=0,Gx=0,Yx=0,Zx=0,eT=0,tT=0,nT=0,rT=0,iT=0;iT=STACKTOP,bn=e>>>0<245;do if(bn){wn=e>>>0<11,qs=e+11|0,Wu=qs&-8,Jf=wn?16:Wu,Gc=Jf>>>3,ed=HEAP32[3311]|0,rm=ed>>>Gc,oy=rm&3,fw=(oy|0)==0;if(!fw){En=rm&1,Kr=En^1,oi=Kr+Gc|0,gi=oi<<1,Li=13284+(gi<<2)|0,Q=gi+2|0,Ii=13284+(Q<<2)|0,Qi=HEAP32[Ii>>2]|0,us=Qi+8|0,ys=HEAP32[us>>2]|0,As=(Li|0)==(ys|0);do if(As)Rs=1<<oi,Ys=Rs^-1,fo=ed&Ys,HEAP32[3311]=fo;else{Eo=HEAP32[3315]|0,_o=ys>>>0<Eo>>>0,_o&&_abort(),zo=ys+12|0,eu=HEAP32[zo>>2]|0,cu=(eu|0)==(Qi|0);if(cu){HEAP32[zo>>2]=Li,HEAP32[Ii>>2]=ys;break}_abort()}while(0);return Su=oi<<3,Du=Su|3,Xu=Qi+4|0,HEAP32[Xu>>2]=Du,tt=Su|4,na=Qi+tt|0,pa=HEAP32[na>>2]|0,Ta=pa|1,HEAP32[na>>2]=Ta,ex=us,ex|0}Ha=HEAP32[3313]|0,Va=Jf>>>0>Ha>>>0;if(Va){rf=(rm|0)==0;if(!rf){vf=rm<<Gc,Cf=2<<Gc,jf=0-Cf|0,Kf=Cf|jf,ol=vf&Kf,gl=0-ol|0,Ll=ol&gl,Il=Ll+ -1|0,Ql=Il>>>12,uc=Ql&16,yc=Il>>>uc,Ac=yc>>>5,qc=Ac&8,Yc=qc|uc,fh=yc>>>qc,wh=fh>>>2,Mh=wh&4,Uh=Yc|Mh,Zh=fh>>>Mh,lp=Zh>>>1,Ep=lp&2,_p=Uh|Ep,zp=Zh>>>Ep,td=zp>>>1,hd=td&1,xd=_p|hd,Pd=zp>>>hd,Xd=xd+Pd|0,nv=Xd<<1,pv=13284+(nv<<2)|0,Zt=nv+2|0,Tv=13284+(Zt<<2)|0,Hv=HEAP32[Tv>>2]|0,Vv=Hv+8|0,im=HEAP32[Vv>>2]|0,vm=(pv|0)==(im|0);do if(vm)Cm=1<<Xd,jm=Cm^-1,Jm=ed&jm,HEAP32[3311]=Jm,Rb=Ha;else{sg=HEAP32[3315]|0,mg=im>>>0<sg>>>0,mg&&_abort(),kg=im+12|0,Fg=HEAP32[kg>>2]|0,Kg=(Fg|0)==(Hv|0);if(Kg){HEAP32[kg>>2]=pv,HEAP32[Tv>>2]=im,d=HEAP32[3313]|0,Rb=d;break}_abort()}while(0);return uy=Xd<<3,yy=uy-Jf|0,Ay=Jf|3,qy=Hv+4|0,HEAP32[qy>>2]=Ay,Gy=Hv+Jf|0,ab=yy|1,fn=Jf|4,bb=Hv+fn|0,HEAP32[bb>>2]=ab,Ob=Hv+uy|0,HEAP32[Ob>>2]=yy,Yb=(Rb|0)==0,Yb||(lw=HEAP32[3316]|0,Ew=Rb>>>3,_w=Ew<<1,zw=13284+(_w<<2)|0,eE=HEAP32[3311]|0,cE=1<<Ew,SE=eE&cE,DE=(SE|0)==0,DE?(WE=eE|cE,HEAP32[3311]=WE,N=_w+2|0,C=13284+(N<<2)|0,T=C,dS=zw):(dn=_w+2|0,tS=13284+(dn<<2)|0,Sn=HEAP32[tS>>2]|0,Dn=HEAP32[3315]|0,Wn=Sn>>>0<Dn>>>0,Wn?_abort():(T=tS,dS=Sn)),HEAP32[T>>2]=lw,tr=dS+12|0,HEAP32[tr>>2]=lw,hr=lw+8|0,HEAP32[hr>>2]=dS,xr=lw+12|0,HEAP32[xr>>2]=zw),HEAP32[3313]=yy,HEAP32[3316]=Gy,ex=Vv,ex|0}Pr=HEAP32[3312]|0,Xr=(Pr|0)==0;if(!Xr){$r=0-Pr|0,Jr=Pr&$r,Qr=Jr+ -1|0,Gr=Qr>>>12,Yr=Gr&16,Zr=Qr>>>Yr,ei=Zr>>>5,ti=ei&8,ni=ti|Yr,ri=Zr>>>ti,ii=ri>>>2,si=ii&4,ui=ni|si,ai=ri>>>si,fi=ai>>>1,li=fi&2,ci=ui|li,hi=ai>>>li,pi=hi>>>1,di=pi&1,vi=ci|di,mi=hi>>>di,yi=vi+mi|0,bi=13548+(yi<<2)|0,wi=HEAP32[bi>>2]|0,Ei=wi+4|0,Si=HEAP32[Ei>>2]|0,xi=Si&-8,Ti=xi-Jf|0,yx=Ti,Fx=wi,Kx=wi;for(;;){Ni=Fx+16|0,Ci=HEAP32[Ni>>2]|0,ki=(Ci|0)==(0|0);if(ki){Ai=Fx+20|0,Oi=HEAP32[Ai>>2]|0,Mi=(Oi|0)==(0|0);if(Mi){bx=yx,Qx=Kx;break}Di=Oi}else Di=Ci;_i=Di+4|0,Pi=HEAP32[_i>>2]|0,Hi=Pi&-8,Bi=Hi-Jf|0,ji=Bi>>>0<yx>>>0,_=ji?Bi:yx,yn=ji?Di:Kx,yx=_,Fx=Di,Kx=yn}Fi=HEAP32[3315]|0,qi=Qx>>>0<Fi>>>0,qi&&_abort(),Ri=Qx+Jf|0,Ui=Qx>>>0<Ri>>>0,Ui||_abort(),zi=Qx+24|0,Wi=HEAP32[zi>>2]|0,Xi=Qx+12|0,Vi=HEAP32[Xi>>2]|0,$i=(Vi|0)==(Qx|0);do if($i){is=Qx+20|0,ss=HEAP32[is>>2]|0,os=(ss|0)==(0|0);if(os){as=Qx+16|0,fs=HEAP32[as>>2]|0,ls=(fs|0)==(0|0);if(ls){AS=0;break}xS=fs,_S=as}else xS=ss,_S=is;for(;;){cs=xS+20|0,hs=HEAP32[cs>>2]|0,ps=(hs|0)==(0|0);if(!ps){xS=hs,_S=cs;continue}ds=xS+16|0,vs=HEAP32[ds>>2]|0,ms=(vs|0)==(0|0);if(ms){CS=xS,HS=_S;break}xS=vs,_S=ds}gs=HS>>>0<Fi>>>0;if(!gs){HEAP32[HS>>2]=0,AS=CS;break}_abort()}else{Ji=Qx+8|0,Ki=HEAP32[Ji>>2]|0,Gi=Ki>>>0<Fi>>>0,Gi&&_abort(),Yi=Ki+12|0,Zi=HEAP32[Yi>>2]|0,es=(Zi|0)==(Qx|0),es||_abort(),ts=Vi+8|0,ns=HEAP32[ts>>2]|0,rs=(ns|0)==(Qx|0);if(rs){HEAP32[Yi>>2]=Vi,HEAP32[ts>>2]=Ki,AS=Vi;break}_abort()}while(0);bs=(Wi|0)==(0|0);do if(!bs){ws=Qx+28|0,Es=HEAP32[ws>>2]|0,Ss=13548+(Es<<2)|0,xs=HEAP32[Ss>>2]|0,Ts=(Qx|0)==(xs|0);if(Ts){HEAP32[Ss>>2]=AS,JS=(AS|0)==(0|0);if(JS){Ns=1<<Es,Cs=Ns^-1,ks=HEAP32[3312]|0,Ls=ks&Cs,HEAP32[3312]=Ls;break}}else{Os=HEAP32[3315]|0,Ms=Wi>>>0<Os>>>0,Ms&&_abort(),_s=Wi+16|0,Ds=HEAP32[_s>>2]|0,Ps=(Ds|0)==(Qx|0),Ps?HEAP32[_s>>2]=AS:(Hs=Wi+20|0,HEAP32[Hs>>2]=AS),Bs=(AS|0)==(0|0);if(Bs)break}js=HEAP32[3315]|0,Fs=AS>>>0<js>>>0,Fs&&_abort(),Is=AS+24|0,HEAP32[Is>>2]=Wi,Us=Qx+16|0,zs=HEAP32[Us>>2]|0,Ws=(zs|0)==(0|0);do if(!Ws){Xs=zs>>>0<js>>>0;if(!Xs){Vs=AS+16|0,HEAP32[Vs>>2]=zs,$s=zs+24|0,HEAP32[$s>>2]=AS;break}_abort()}while(0);Js=Qx+20|0,Ks=HEAP32[Js>>2]|0,Qs=(Ks|0)==(0|0);if(!Qs){Gs=HEAP32[3315]|0,Zs=Ks>>>0<Gs>>>0;if(!Zs){eo=AS+20|0,HEAP32[eo>>2]=Ks,to=Ks+24|0,HEAP32[to>>2]=AS;break}_abort()}}while(0);return no=bx>>>0<16,no?(ro=bx+Jf|0,io=ro|3,so=Qx+4|0,HEAP32[so>>2]=io,en=ro+4|0,oo=Qx+en|0,uo=HEAP32[oo>>2]|0,ao=uo|1,HEAP32[oo>>2]=ao):(lo=Jf|3,co=Qx+4|0,HEAP32[co>>2]=lo,ho=bx|1,z=Jf|4,po=Qx+z|0,HEAP32[po>>2]=ho,X=bx+Jf|0,vo=Qx+X|0,HEAP32[vo>>2]=bx,mo=HEAP32[3313]|0,go=(mo|0)==0,go||(yo=HEAP32[3316]|0,bo=mo>>>3,wo=bo<<1,So=13284+(wo<<2)|0,xo=HEAP32[3311]|0,To=1<<bo,No=xo&To,Co=(No|0)==0,Co?(ko=xo|To,HEAP32[3311]=ko,v=wo+2|0,M=13284+(v<<2)|0,S=M,pS=So):(Xt=wo+2|0,Lo=13284+(Xt<<2)|0,Ao=HEAP32[Lo>>2]|0,Oo=HEAP32[3315]|0,Mo=Ao>>>0<Oo>>>0,Mo?_abort():(S=Lo,pS=Ao)),HEAP32[S>>2]=yo,Do=pS+12|0,HEAP32[Do>>2]=yo,Po=yo+8|0,HEAP32[Po>>2]=pS,Ho=yo+12|0,HEAP32[Ho>>2]=So),HEAP32[3313]=bx,HEAP32[3316]=Ri),Bo=Qx+8|0,ex=Bo,ex|0}tx=Jf}else tx=Jf}else{jo=e>>>0>4294967231;if(jo)tx=-1;else{Fo=e+11|0,Io=Fo&-8,qo=HEAP32[3312]|0,Ro=(qo|0)==0;if(Ro)tx=Io;else{Uo=0-Io|0,Wo=Fo>>>8,Xo=(Wo|0)==0,Xo?ZS=0:(Vo=Io>>>0>16777215,Vo?ZS=31:($o=Wo+1048320|0,Jo=$o>>>16,Ko=Jo&8,Qo=Wo<<Ko,Go=Qo+520192|0,Yo=Go>>>16,Zo=Yo&4,tu=Zo|Ko,nu=Qo<<Zo,ru=nu+245760|0,iu=ru>>>16,su=iu&2,ou=tu|su,uu=14-ou|0,au=nu<<su,fu=au>>>15,lu=uu+fu|0,hu=lu<<1,pu=lu+7|0,du=Io>>>pu,vu=du&1,mu=vu|hu,ZS=mu)),gu=13548+(ZS<<2)|0,yu=HEAP32[gu>>2]|0,bu=(yu|0)==(0|0);e:do if(bu)Sx=Uo,qx=0,Zx=0,rT=86;else{wu=(ZS|0)==31,Eu=ZS>>>1,xu=25-Eu|0,Tu=wu?0:xu,Nu=Io<<Tu,wx=Uo,Nx=0,kx=Nu,Ix=yu,Gx=0;for(;;){Cu=Ix+4|0,ku=HEAP32[Cu>>2]|0,Lu=ku&-8,Au=Lu-Io|0,Ou=Au>>>0<wx>>>0;if(Ou){Mu=(Lu|0)==(Io|0);if(Mu){Tx=Au,zx=Ix,nT=Ix,rT=90;break e}Ex=Au,Yx=Ix}else Ex=wx,Yx=Gx;_u=Ix+20|0,Pu=HEAP32[_u>>2]|0,Hu=kx>>>31,Bu=(Ix+16|0)+(Hu<<2)|0,ju=HEAP32[Bu>>2]|0,Fu=(Pu|0)==(0|0),Iu=(Pu|0)==(ju|0),fx=Fu|Iu,Cx=fx?Nx:Pu,qu=(ju|0)==(0|0),Ru=kx<<1;if(qu){Sx=Ex,qx=Cx,Zx=Yx,rT=86;break}wx=Ex,Nx=Cx,kx=Ru,Ix=ju,Gx=Yx}}while(0);if((rT|0)==86){Uu=(qx|0)==(0|0),zu=(Zx|0)==(0|0),ox=Uu&zu;if(ox){Vu=2<<ZS,$u=0-Vu|0,Ju=Vu|$u,Ku=qo&Ju,Qu=(Ku|0)==0;if(Qu){tx=Io;break}Gu=0-Ku|0,Yu=Ku&Gu,Zu=Yu+ -1|0,ea=Zu>>>12,ta=ea&16,ra=Zu>>>ta,ia=ra>>>5,sa=ia&8,oa=sa|ta,ua=ra>>>sa,aa=ua>>>2,fa=aa&4,la=oa|fa,ca=ua>>>fa,ha=ca>>>1,da=ha&2,va=la|da,ma=ca>>>da,ga=ma>>>1,ya=ga&1,ba=va|ya,wa=ma>>>ya,Ea=ba+wa|0,Sa=13548+(Ea<<2)|0,xa=HEAP32[Sa>>2]|0,Rx=xa,tT=0}else Rx=qx,tT=Zx;Na=(Rx|0)==(0|0),Na?(xx=Sx,eT=tT):(Tx=Sx,zx=Rx,nT=tT,rT=90)}if((rT|0)==90)for(;;){rT=0,Ca=zx+4|0,ka=HEAP32[Ca>>2]|0,La=ka&-8,Aa=La-Io|0,Oa=Aa>>>0<Tx>>>0,D=Oa?Aa:Tx,Ux=Oa?zx:nT,Ma=zx+16|0,_a=HEAP32[Ma>>2]|0,Da=(_a|0)==(0|0);if(!Da){Tx=D,zx=_a,nT=Ux,rT=90;continue}Pa=zx+20|0,Ba=HEAP32[Pa>>2]|0,ja=(Ba|0)==(0|0);if(ja){xx=D,eT=Ux;break}Tx=D,zx=Ba,nT=Ux,rT=90}Fa=(eT|0)==(0|0);if(Fa)tx=Io;else{Ia=HEAP32[3313]|0,qa=Ia-Io|0,Ra=xx>>>0<qa>>>0;if(Ra){Ua=HEAP32[3315]|0,za=eT>>>0<Ua>>>0,za&&_abort(),Wa=eT+Io|0,Xa=eT>>>0<Wa>>>0,Xa||_abort(),$a=eT+24|0,Ja=HEAP32[$a>>2]|0,Ka=eT+12|0,Qa=HEAP32[Ka>>2]|0,Ga=(Qa|0)==(eT|0);do if(Ga){ff=eT+20|0,lf=HEAP32[ff>>2]|0,cf=(lf|0)==(0|0);if(cf){hf=eT+16|0,pf=HEAP32[hf>>2]|0,df=(pf|0)==(0|0);if(df){MS=0;break}kS=pf,BS=hf}else kS=lf,BS=ff;for(;;){mf=kS+20|0,gf=HEAP32[mf>>2]|0,yf=(gf|0)==(0|0);if(!yf){kS=gf,BS=mf;continue}bf=kS+16|0,wf=HEAP32[bf>>2]|0,Ef=(wf|0)==(0|0);if(Ef){LS=kS,jS=BS;break}kS=wf,BS=bf}Sf=jS>>>0<Ua>>>0;if(!Sf){HEAP32[jS>>2]=0,MS=LS;break}_abort()}else{Ya=eT+8|0,Za=HEAP32[Ya>>2]|0,ef=Za>>>0<Ua>>>0,ef&&_abort(),tf=Za+12|0,nf=HEAP32[tf>>2]|0,sf=(nf|0)==(eT|0),sf||_abort(),of=Qa+8|0,uf=HEAP32[of>>2]|0,af=(uf|0)==(eT|0);if(af){HEAP32[tf>>2]=Qa,HEAP32[of>>2]=Za,MS=Qa;break}_abort()}while(0);xf=(Ja|0)==(0|0);do if(!xf){Tf=eT+28|0,Nf=HEAP32[Tf>>2]|0,kf=13548+(Nf<<2)|0,Lf=HEAP32[kf>>2]|0,Af=(eT|0)==(Lf|0);if(Af){HEAP32[kf>>2]=MS,QS=(MS|0)==(0|0);if(QS){Of=1<<Nf,Mf=Of^-1,_f=HEAP32[3312]|0,Df=_f&Mf,HEAP32[3312]=Df;break}}else{Pf=HEAP32[3315]|0,Hf=Ja>>>0<Pf>>>0,Hf&&_abort(),Bf=Ja+16|0,Ff=HEAP32[Bf>>2]|0,If=(Ff|0)==(eT|0),If?HEAP32[Bf>>2]=MS:(qf=Ja+20|0,HEAP32[qf>>2]=MS),Rf=(MS|0)==(0|0);if(Rf)break}Uf=HEAP32[3315]|0,zf=MS>>>0<Uf>>>0,zf&&_abort(),Wf=MS+24|0,HEAP32[Wf>>2]=Ja,Xf=eT+16|0,Vf=HEAP32[Xf>>2]|0,$f=(Vf|0)==(0|0);do if(!$f){Qf=Vf>>>0<Uf>>>0;if(!Qf){Gf=MS+16|0,HEAP32[Gf>>2]=Vf,Yf=Vf+24|0,HEAP32[Yf>>2]=MS;break}_abort()}while(0);Zf=eT+20|0,el=HEAP32[Zf>>2]|0,tl=(el|0)==(0|0);if(!tl){nl=HEAP32[3315]|0,rl=el>>>0<nl>>>0;if(!rl){il=MS+20|0,HEAP32[il>>2]=el,sl=el+24|0,HEAP32[sl>>2]=MS;break}_abort()}}while(0);ul=xx>>>0<16;t:do if(ul)al=xx+Io|0,fl=al|3,ll=eT+4|0,HEAP32[ll>>2]=fl,kt=al+4|0,cl=eT+kt|0,hl=HEAP32[cl>>2]|0,pl=hl|1,HEAP32[cl>>2]=pl;else{dl=Io|3,vl=eT+4|0,HEAP32[vl>>2]=dl,ml=xx|1,R=Io|4,yl=eT+R|0,HEAP32[yl>>2]=ml,K=xx+Io|0,bl=eT+K|0,HEAP32[bl>>2]=xx,wl=xx>>>3,El=xx>>>0<256;if(El){Sl=wl<<1,xl=13284+(Sl<<2)|0,Tl=HEAP32[3311]|0,Nl=1<<wl,Cl=Tl&Nl,kl=(Cl|0)==0,kl?(Al=Tl|Nl,HEAP32[3311]=Al,y=Sl+2|0,L=13284+(y<<2)|0,E=L,mS=xl):(Nt=Sl+2|0,Ol=13284+(Nt<<2)|0,Ml=HEAP32[Ol>>2]|0,_l=HEAP32[3315]|0,Dl=Ml>>>0<_l>>>0,Dl?_abort():(E=Ol,mS=Ml)),HEAP32[E>>2]=Wa,Pl=mS+12|0,HEAP32[Pl>>2]=Wa,Et=Io+8|0,Hl=eT+Et|0,HEAP32[Hl>>2]=mS,xt=Io+12|0,Bl=eT+xt|0,HEAP32[Bl>>2]=xl;break}jl=xx>>>8,Fl=(jl|0)==0,Fl?yS=0:(ql=xx>>>0>16777215,ql?yS=31:(Rl=jl+1048320|0,Ul=Rl>>>16,zl=Ul&8,Wl=jl<<zl,Xl=Wl+520192|0,Vl=Xl>>>16,$l=Vl&4,Jl=$l|zl,Kl=Wl<<$l,Gl=Kl+245760|0,Yl=Gl>>>16,Zl=Yl&2,ec=Jl|Zl,tc=14-ec|0,nc=Kl<<Zl,rc=nc>>>15,ic=tc+rc|0,sc=ic<<1,oc=ic+7|0,ac=xx>>>oc,fc=ac&1,lc=fc|sc,yS=lc)),cc=13548+(yS<<2)|0,Ot=Io+28|0,hc=eT+Ot|0,HEAP32[hc>>2]=yS,Vt=Io+16|0,pc=eT+Vt|0,nn=Io+20|0,dc=eT+nn|0,HEAP32[dc>>2]=0,HEAP32[pc>>2]=0,vc=HEAP32[3312]|0,mc=1<<yS,gc=vc&mc,bc=(gc|0)==0;if(bc){wc=vc|mc,HEAP32[3312]=wc,HEAP32[cc>>2]=Wa,un=Io+24|0,Ec=eT+un|0,HEAP32[Ec>>2]=cc,ln=Io+12|0,Sc=eT+ln|0,HEAP32[Sc>>2]=Wa,hn=Io+8|0,xc=eT+hn|0,HEAP32[xc>>2]=Wa;break}Tc=HEAP32[cc>>2]|0,Nc=Tc+4|0,Cc=HEAP32[Nc>>2]|0,kc=Cc&-8,Lc=(kc|0)==(xx|0);n:do if(Lc)FS=Tc;else{Oc=(yS|0)==31,Mc=yS>>>1,_c=25-Mc|0,Dc=Oc?0:_c,Pc=xx<<Dc,wS=Pc,RS=Tc;for(;;){Uc=wS>>>31,zc=(RS+16|0)+(Uc<<2)|0,jc=HEAP32[zc>>2]|0,Wc=(jc|0)==(0|0);if(Wc){p=zc,US=RS;break}Hc=wS<<1,Bc=jc+4|0,Fc=HEAP32[Bc>>2]|0,Ic=Fc&-8,Rc=(Ic|0)==(xx|0);if(Rc){FS=jc;break n}wS=Hc,RS=jc}Xc=HEAP32[3315]|0,Vc=p>>>0<Xc>>>0;if(!Vc){HEAP32[p>>2]=Wa,Z=Io+24|0,$c=eT+Z|0,HEAP32[$c>>2]=US,lt=Io+12|0,Jc=eT+lt|0,HEAP32[Jc>>2]=Wa,yt=Io+8|0,Kc=eT+yt|0,HEAP32[Kc>>2]=Wa;break t}_abort()}while(0);Qc=FS+8|0,Zc=HEAP32[Qc>>2]|0,eh=HEAP32[3315]|0,th=Zc>>>0>=eh>>>0,nx=FS>>>0>=eh>>>0,nh=th&nx;if(nh){rh=Zc+12|0,HEAP32[rh>>2]=Wa,HEAP32[Qc>>2]=Wa,pn=Io+8|0,ih=eT+pn|0,HEAP32[ih>>2]=Zc,vn=Io+12|0,sh=eT+vn|0,HEAP32[sh>>2]=FS,G=Io+24|0,oh=eT+G|0,HEAP32[oh>>2]=0;break}_abort()}while(0);return uh=eT+8|0,ex=uh,ex|0}tx=Io}}}}while(0);ah=HEAP32[3313]|0,lh=ah>>>0<tx>>>0;if(!lh)return ch=ah-tx|0,hh=HEAP32[3316]|0,ph=ch>>>0>15,ph?(dh=hh+tx|0,HEAP32[3316]=dh,HEAP32[3313]=ch,vh=ch|1,At=tx+4|0,mh=hh+At|0,HEAP32[mh>>2]=vh,gh=hh+ah|0,HEAP32[gh>>2]=ch,yh=tx|3,bh=hh+4|0,HEAP32[bh>>2]=yh):(HEAP32[3313]=0,HEAP32[3316]=0,Eh=ah|3,Sh=hh+4|0,HEAP32[Sh>>2]=Eh,W=ah+4|0,xh=hh+W|0,Th=HEAP32[xh>>2]|0,Nh=Th|1,HEAP32[xh>>2]=Nh),
Ch=hh+8|0,ex=Ch,ex|0;kh=HEAP32[3314]|0,Lh=kh>>>0>tx>>>0;if(Lh)return Ah=kh-tx|0,HEAP32[3314]=Ah,Oh=HEAP32[3317]|0,_h=Oh+tx|0,HEAP32[3317]=_h,Dh=Ah|1,P=tx+4|0,Ph=Oh+P|0,HEAP32[Ph>>2]=Dh,Hh=tx|3,Bh=Oh+4|0,HEAP32[Bh>>2]=Hh,jh=Oh+8|0,ex=jh,ex|0;Fh=HEAP32[3429]|0,Ih=(Fh|0)==0;do if(Ih){qh=_sysconf(30)|0,Rh=qh+ -1|0,zh=Rh&qh,Wh=(zh|0)==0;if(Wh){HEAP32[3431]=qh,HEAP32[3430]=qh,HEAP32[3432]=-1,HEAP32[3433]=-1,HEAP32[3434]=0,HEAP32[3422]=0,Xh=_time(0)|0,Vh=Xh&-16,$h=Vh^1431655768,HEAP32[3429]=$h;break}_abort()}while(0);Jh=tx+48|0,Kh=HEAP32[3431]|0,Qh=tx+47|0,Gh=Kh+Qh|0,Yh=0-Kh|0,ep=Gh&Yh,tp=ep>>>0>tx>>>0;if(!tp)return ex=0,ex|0;np=HEAP32[3421]|0,rp=(np|0)==0;if(!rp){ip=HEAP32[3419]|0,sp=ip+ep|0,op=sp>>>0<=ip>>>0,up=sp>>>0>np>>>0,ax=op|up;if(ax)return ex=0,ex|0}ap=HEAP32[3422]|0,fp=ap&4,cp=(fp|0)==0;r:do if(cp){hp=HEAP32[3317]|0,pp=(hp|0)==(0|0);i:do if(pp)rT=174;else{Lx=13692;for(;;){dp=HEAP32[Lx>>2]|0,vp=dp>>>0>hp>>>0;if(!vp){mp=Lx+4|0,gp=HEAP32[mp>>2]|0,yp=dp+gp|0,bp=yp>>>0>hp>>>0;if(bp){c=Lx,h=mp;break}}wp=Lx+8|0,Sp=HEAP32[wp>>2]|0,xp=(Sp|0)==(0|0);if(xp){rT=174;break i}Lx=Sp}Jp=HEAP32[3314]|0,Kp=Gh-Jp|0,Qp=Kp&Yh,Gp=Qp>>>0<2147483647;if(Gp){Yp=_sbrk(Qp|0)|0,Zp=HEAP32[c>>2]|0,nd=HEAP32[h>>2]|0,rd=Zp+nd|0,id=(Yp|0)==(rd|0),t=id?Qp:0;if(id){sd=(Yp|0)==(-1|0);if(!sd){Wx=Yp,Jx=t,rT=194;break r}Vx=t}else $S=Yp,Bx=Qp,Xx=t,rT=184}else Vx=0}while(0);do if((rT|0)==174){Tp=_sbrk(0)|0,Np=(Tp|0)==(-1|0);if(Np)Vx=0;else{Cp=Tp,kp=HEAP32[3430]|0,Lp=kp+ -1|0,Ap=Lp&Cp,Op=(Ap|0)==0,Op?Hx=ep:(Mp=Lp+Cp|0,Dp=0-kp|0,Pp=Mp&Dp,Hp=ep-Cp|0,Bp=Hp+Pp|0,Hx=Bp),jp=HEAP32[3419]|0,Fp=jp+Hx|0,Ip=Hx>>>0>tx>>>0,qp=Hx>>>0<2147483647,ux=Ip&qp;if(ux){Rp=HEAP32[3421]|0,Up=(Rp|0)==0;if(!Up){Wp=Fp>>>0<=jp>>>0,Xp=Fp>>>0>Rp>>>0,lx=Wp|Xp;if(lx){Vx=0;break}}Vp=_sbrk(Hx|0)|0,$p=(Vp|0)==(Tp|0),Px=$p?Hx:0;if($p){Wx=Tp,Jx=Px,rT=194;break r}$S=Vp,Bx=Hx,Xx=Px,rT=184}else Vx=0}}while(0);s:do if((rT|0)==184){od=0-Bx|0,ud=($S|0)!=(-1|0),ad=Bx>>>0<2147483647,hx=ad&ud,fd=Jh>>>0>Bx>>>0,dx=fd&hx;do if(dx){ld=HEAP32[3431]|0,cd=Qh-Bx|0,pd=cd+ld|0,dd=0-ld|0,vd=pd&dd,md=vd>>>0<2147483647;if(md){gd=_sbrk(vd|0)|0,yd=(gd|0)==(-1|0);if(yd){_sbrk(od|0)|0,Vx=Xx;break s}bd=vd+Bx|0,jx=bd;break}jx=Bx}else jx=Bx;while(0);wd=($S|0)==(-1|0);if(!wd){Wx=$S,Jx=jx,rT=194;break r}Vx=Xx}while(0);Ed=HEAP32[3422]|0,Sd=Ed|4,HEAP32[3422]=Sd,$x=Vx,rT=191}else $x=0,rT=191;while(0);(rT|0)==191&&(Td=ep>>>0<2147483647,Td&&(Nd=_sbrk(ep|0)|0,Cd=_sbrk(0)|0,kd=(Nd|0)!=(-1|0),Ld=(Cd|0)!=(-1|0),cx=kd&Ld,Ad=Nd>>>0<Cd>>>0,vx=Ad&cx,vx&&(Od=Cd,Md=Nd,_d=Od-Md|0,Dd=tx+40|0,Hd=_d>>>0>Dd>>>0,gn=Hd?_d:$x,Hd&&(Wx=Nd,Jx=gn,rT=194))));if((rT|0)==194){Bd=HEAP32[3419]|0,jd=Bd+Jx|0,HEAP32[3419]=jd,Fd=HEAP32[3420]|0,Id=jd>>>0>Fd>>>0,Id&&(HEAP32[3420]=jd),qd=HEAP32[3317]|0,Rd=(qd|0)==(0|0);o:do if(Rd){Ud=HEAP32[3315]|0,zd=(Ud|0)==(0|0),Wd=Wx>>>0<Ud>>>0,mx=zd|Wd,mx&&(HEAP32[3315]=Wx),HEAP32[3423]=Wx,HEAP32[3424]=Jx,HEAP32[3426]=0,Vd=HEAP32[3429]|0,HEAP32[3320]=Vd,HEAP32[3319]=-1,YS=0;for(;;){$d=YS<<1,Jd=13284+($d<<2)|0,H=$d+3|0,Kd=13284+(H<<2)|0,HEAP32[Kd>>2]=Jd,V=$d+2|0,Qd=13284+(V<<2)|0,HEAP32[Qd>>2]=Jd,Gd=YS+1|0,GS=(Gd|0)==32;if(GS)break;YS=Gd}Yd=Jx+ -40|0,Zd=Wx+8|0,ev=Zd,tv=ev&7,rv=(tv|0)==0,iv=0-ev|0,sv=iv&7,ov=rv?0:sv,uv=Wx+ov|0,av=Yd-ov|0,HEAP32[3317]=uv,HEAP32[3314]=av,fv=av|1,j=ov+4|0,lv=Wx+j|0,HEAP32[lv>>2]=fv,Mt=Jx+ -36|0,cv=Wx+Mt|0,HEAP32[cv>>2]=40,hv=HEAP32[3433]|0,HEAP32[3318]=hv}else{Ox=13692;for(;;){dv=HEAP32[Ox>>2]|0,vv=Ox+4|0,mv=HEAP32[vv>>2]|0,gv=dv+mv|0,yv=(Wx|0)==(gv|0);if(yv){a=dv,f=vv,l=mv,Mx=Ox,rT=204;break}bv=Ox+8|0,wv=HEAP32[bv>>2]|0,Ev=(wv|0)==(0|0);if(Ev)break;Ox=wv}if((rT|0)==204){Sv=Mx+12|0,xv=HEAP32[Sv>>2]|0,Nv=xv&8,Cv=(Nv|0)==0;if(Cv){kv=qd>>>0>=a>>>0,Lv=qd>>>0<Wx>>>0,px=Lv&kv;if(px){Av=l+Jx|0,HEAP32[f>>2]=Av,Ov=HEAP32[3314]|0,Mv=Ov+Jx|0,_v=qd+8|0,Dv=_v,Pv=Dv&7,Bv=(Pv|0)==0,jv=0-Dv|0,Fv=jv&7,Iv=Bv?0:Fv,qv=qd+Iv|0,Rv=Mv-Iv|0,HEAP32[3317]=qv,HEAP32[3314]=Rv,Uv=Rv|1,I=Iv+4|0,zv=qd+I|0,HEAP32[zv>>2]=Uv,Pt=Mv+4|0,Wv=qd+Pt|0,HEAP32[Wv>>2]=40,Xv=HEAP32[3433]|0,HEAP32[3318]=Xv;break}}}$v=HEAP32[3315]|0,Jv=Wx>>>0<$v>>>0,Jv?(HEAP32[3315]=Wx,cg=Wx):cg=$v,Kv=Wx+Jx|0,_x=13692;for(;;){Qv=HEAP32[_x>>2]|0,Gv=(Qv|0)==(Kv|0);if(Gv){u=_x,Dx=_x,rT=212;break}Yv=_x+8|0,Zv=HEAP32[Yv>>2]|0,em=(Zv|0)==(0|0);if(em){Ax=13692;break}_x=Zv}if((rT|0)==212){tm=Dx+12|0,nm=HEAP32[tm>>2]|0,sm=nm&8,om=(sm|0)==0;if(om){HEAP32[u>>2]=Wx,um=Dx+4|0,am=HEAP32[um>>2]|0,fm=am+Jx|0,HEAP32[um>>2]=fm,lm=Wx+8|0,cm=lm,hm=cm&7,pm=(hm|0)==0,dm=0-cm|0,mm=dm&7,gm=pm?0:mm,ym=Wx+gm|0,nt=Jx+8|0,bm=Wx+nt|0,wm=bm,Em=wm&7,Sm=(Em|0)==0,xm=0-wm|0,Tm=xm&7,Nm=Sm?0:Tm,rt=Nm+Jx|0,km=Wx+rt|0,Lm=km,Am=ym,Om=Lm-Am|0,q=gm+tx|0,Mm=Wx+q|0,_m=Om-tx|0,Dm=tx|3,J=gm+4|0,Pm=Wx+J|0,HEAP32[Pm>>2]=Dm,Hm=(km|0)==(qd|0);u:do if(Hm)Bm=HEAP32[3314]|0,Fm=Bm+_m|0,HEAP32[3314]=Fm,HEAP32[3317]=Mm,Im=Fm|1,on=q+4|0,qm=Wx+on|0,HEAP32[qm>>2]=Im;else{Rm=HEAP32[3316]|0,Um=(km|0)==(Rm|0);if(Um){zm=HEAP32[3313]|0,Wm=zm+_m|0,HEAP32[3313]=Wm,HEAP32[3316]=Mm,Xm=Wm|1,rn=q+4|0,Vm=Wx+rn|0,HEAP32[Vm>>2]=Xm,sn=Wm+q|0,$m=Wx+sn|0,HEAP32[$m>>2]=Wm;break}Ht=Jx+4|0,it=Ht+Nm|0,Km=Wx+it|0,Qm=HEAP32[Km>>2]|0,Gm=Qm&3,Ym=(Gm|0)==1;if(Ym){Zm=Qm&-8,eg=Qm>>>3,tg=Qm>>>0<256;a:do if(tg){Gt=Nm|8,mt=Gt+Jx|0,ng=Wx+mt|0,rg=HEAP32[ng>>2]|0,Yt=Jx+12|0,gt=Yt+Nm|0,ig=Wx+gt|0,og=HEAP32[ig>>2]|0,ug=eg<<1,ag=13284+(ug<<2)|0,fg=(rg|0)==(ag|0);do if(!fg){lg=rg>>>0<cg>>>0,lg&&_abort(),hg=rg+12|0,pg=HEAP32[hg>>2]|0,dg=(pg|0)==(km|0);if(dg)break;_abort()}while(0);vg=(og|0)==(rg|0);if(vg){gg=1<<eg,yg=gg^-1,bg=HEAP32[3311]|0,wg=bg&yg,HEAP32[3311]=wg;break}Eg=(og|0)==(ag|0);do if(Eg)O=og+8|0,x=O;else{Sg=og>>>0<cg>>>0,Sg&&_abort(),xg=og+8|0,Tg=HEAP32[xg>>2]|0,Ng=(Tg|0)==(km|0);if(Ng){x=xg;break}_abort()}while(0);Cg=rg+12|0,HEAP32[Cg>>2]=og,HEAP32[x>>2]=rg}else{Kt=Nm|24,st=Kt+Jx|0,Lg=Wx+st|0,Ag=HEAP32[Lg>>2]|0,an=Jx+12|0,ot=an+Nm|0,Og=Wx+ot|0,Mg=HEAP32[Og>>2]|0,_g=(Mg|0)==(km|0);do if(_g){cn=Nm|16,dt=Ht+cn|0,zg=Wx+dt|0,Wg=HEAP32[zg>>2]|0,Xg=(Wg|0)==(0|0);if(Xg){vt=cn+Jx|0,Vg=Wx+vt|0,$g=HEAP32[Vg>>2]|0,Jg=($g|0)==(0|0);if(Jg){OS=0;break}TS=$g,DS=Vg}else TS=Wg,DS=zg;for(;;){Qg=TS+20|0,Gg=HEAP32[Qg>>2]|0,Yg=(Gg|0)==(0|0);if(!Yg){TS=Gg,DS=Qg;continue}Zg=TS+16|0,ey=HEAP32[Zg>>2]|0,ty=(ey|0)==(0|0);if(ty){NS=TS,PS=DS;break}TS=ey,DS=Zg}ny=PS>>>0<cg>>>0;if(!ny){HEAP32[PS>>2]=0,OS=NS;break}_abort()}else{Qt=Nm|8,ut=Qt+Jx|0,Dg=Wx+ut|0,Pg=HEAP32[Dg>>2]|0,Hg=Pg>>>0<cg>>>0,Hg&&_abort(),Bg=Pg+12|0,jg=HEAP32[Bg>>2]|0,Ig=(jg|0)==(km|0),Ig||_abort(),qg=Mg+8|0,Rg=HEAP32[qg>>2]|0,Ug=(Rg|0)==(km|0);if(Ug){HEAP32[Bg>>2]=Mg,HEAP32[qg>>2]=Pg,OS=Mg;break}_abort()}while(0);ry=(Ag|0)==(0|0);if(ry)break;$t=Jx+28|0,at=$t+Nm|0,iy=Wx+at|0,sy=HEAP32[iy>>2]|0,ay=13548+(sy<<2)|0,fy=HEAP32[ay>>2]|0,ly=(km|0)==(fy|0);do{if(ly){HEAP32[ay>>2]=OS,KS=(OS|0)==(0|0);if(!KS)break;cy=1<<sy,hy=cy^-1,py=HEAP32[3312]|0,dy=py&hy,HEAP32[3312]=dy;break a}vy=HEAP32[3315]|0,my=Ag>>>0<vy>>>0,my&&_abort(),gy=Ag+16|0,by=HEAP32[gy>>2]|0,wy=(by|0)==(km|0),wy?HEAP32[gy>>2]=OS:(Ey=Ag+20|0,HEAP32[Ey>>2]=OS),Sy=(OS|0)==(0|0);if(Sy)break a}while(0);xy=HEAP32[3315]|0,Ty=OS>>>0<xy>>>0,Ty&&_abort(),Ny=OS+24|0,HEAP32[Ny>>2]=Ag,Jt=Nm|16,ft=Jt+Jx|0,Cy=Wx+ft|0,ky=HEAP32[Cy>>2]|0,Ly=(ky|0)==(0|0);do if(!Ly){Oy=ky>>>0<xy>>>0;if(!Oy){My=OS+16|0,HEAP32[My>>2]=ky,_y=ky+24|0,HEAP32[_y>>2]=OS;break}_abort()}while(0);ht=Ht+Jt|0,Dy=Wx+ht|0,Py=HEAP32[Dy>>2]|0,Hy=(Py|0)==(0|0);if(Hy)break;By=HEAP32[3315]|0,jy=Py>>>0<By>>>0;if(!jy){Fy=OS+20|0,HEAP32[Fy>>2]=Py,Iy=Py+24|0,HEAP32[Iy>>2]=OS;break}_abort()}while(0);mn=Zm|Nm,pt=mn+Jx|0,Ry=Wx+pt|0,Uy=Zm+_m|0,sx=Ry,gx=Uy}else sx=km,gx=_m;zy=sx+4|0,Wy=HEAP32[zy>>2]|0,Xy=Wy&-2,HEAP32[zy>>2]=Xy,Vy=gx|1,Y=q+4|0,$y=Wx+Y|0,HEAP32[$y>>2]=Vy,et=gx+q|0,Jy=Wx+et|0,HEAP32[Jy>>2]=gx,Ky=gx>>>3,Qy=gx>>>0<256;if(Qy){Yy=Ky<<1,Zy=13284+(Yy<<2)|0,eb=HEAP32[3311]|0,tb=1<<Ky,nb=eb&tb,rb=(nb|0)==0;do if(rb)ib=eb|tb,HEAP32[3311]=ib,g=Yy+2|0,A=13284+(g<<2)|0,w=A,vS=Zy;else{Wt=Yy+2|0,sb=13284+(Wt<<2)|0,ob=HEAP32[sb>>2]|0,ub=HEAP32[3315]|0,fb=ob>>>0<ub>>>0;if(!fb){w=sb,vS=ob;break}_abort()}while(0);HEAP32[w>>2]=Mm,lb=vS+12|0,HEAP32[lb>>2]=Mm,Ut=q+8|0,cb=Wx+Ut|0,HEAP32[cb>>2]=vS,zt=q+12|0,hb=Wx+zt|0,HEAP32[hb>>2]=Zy;break}pb=gx>>>8,db=(pb|0)==0;do if(db)bS=0;else{vb=gx>>>0>16777215;if(vb){bS=31;break}mb=pb+1048320|0,gb=mb>>>16,yb=gb&8,wb=pb<<yb,Eb=wb+520192|0,Sb=Eb>>>16,xb=Sb&4,Tb=xb|yb,Nb=wb<<xb,Cb=Nb+245760|0,kb=Cb>>>16,Lb=kb&2,Ab=Tb|Lb,Mb=14-Ab|0,_b=Nb<<Lb,Db=_b>>>15,Pb=Mb+Db|0,Hb=Pb<<1,Bb=Pb+7|0,jb=gx>>>Bb,Fb=jb&1,Ib=Fb|Hb,bS=Ib}while(0);qb=13548+(bS<<2)|0,ct=q+28|0,Ub=Wx+ct|0,HEAP32[Ub>>2]=bS,bt=q+16|0,zb=Wx+bt|0,wt=q+20|0,Wb=Wx+wt|0,HEAP32[Wb>>2]=0,HEAP32[zb>>2]=0,Xb=HEAP32[3312]|0,Vb=1<<bS,$b=Xb&Vb,Jb=($b|0)==0;if(Jb){Kb=Xb|Vb,HEAP32[3312]=Kb,HEAP32[qb>>2]=Mm,St=q+24|0,Qb=Wx+St|0,HEAP32[Qb>>2]=qb,Tt=q+12|0,Gb=Wx+Tt|0,HEAP32[Gb>>2]=Mm,Ct=q+8|0,Zb=Wx+Ct|0,HEAP32[Zb>>2]=Mm;break}ew=HEAP32[qb>>2]|0,tw=ew+4|0,nw=HEAP32[tw>>2]|0,rw=nw&-8,iw=(rw|0)==(gx|0);f:do if(iw)qS=ew;else{sw=(bS|0)==31,ow=bS>>>1,uw=25-ow|0,aw=sw?0:uw,cw=gx<<aw,SS=cw,zS=ew;for(;;){yw=SS>>>31,bw=(zS+16|0)+(yw<<2)|0,dw=HEAP32[bw>>2]|0,ww=(dw|0)==(0|0);if(ww){n=bw,WS=zS;break}hw=SS<<1,pw=dw+4|0,vw=HEAP32[pw>>2]|0,mw=vw&-8,gw=(mw|0)==(gx|0);if(gw){qS=dw;break f}SS=hw,zS=dw}Sw=HEAP32[3315]|0,xw=n>>>0<Sw>>>0;if(!xw){HEAP32[n>>2]=Mm,It=q+24|0,Tw=Wx+It|0,HEAP32[Tw>>2]=WS,qt=q+12|0,Nw=Wx+qt|0,HEAP32[Nw>>2]=Mm,Rt=q+8|0,Cw=Wx+Rt|0,HEAP32[Cw>>2]=Mm;break u}_abort()}while(0);kw=qS+8|0,Lw=HEAP32[kw>>2]|0,Aw=HEAP32[3315]|0,Ow=Lw>>>0>=Aw>>>0,ix=qS>>>0>=Aw>>>0,Mw=Ow&ix;if(Mw){Dw=Lw+12|0,HEAP32[Dw>>2]=Mm,HEAP32[kw>>2]=Mm,Bt=q+8|0,Pw=Wx+Bt|0,HEAP32[Pw>>2]=Lw,jt=q+12|0,Hw=Wx+jt|0,HEAP32[Hw>>2]=qS,Ft=q+24|0,Bw=Wx+Ft|0,HEAP32[Bw>>2]=0;break}_abort()}while(0);return Lt=gm|8,jw=Wx+Lt|0,ex=jw,ex|0}Ax=13692}for(;;){Fw=HEAP32[Ax>>2]|0,Iw=Fw>>>0>qd>>>0;if(!Iw){qw=Ax+4|0,Rw=HEAP32[qw>>2]|0,Uw=Fw+Rw|0,Ww=Uw>>>0>qd>>>0;if(Ww){i=Fw,s=Rw,o=Uw;break}}Xw=Ax+8|0,Vw=HEAP32[Xw>>2]|0,Ax=Vw}F=s+ -47|0,$=s+ -39|0,$w=i+$|0,Jw=$w,Kw=Jw&7,Qw=(Kw|0)==0,Gw=0-Jw|0,Yw=Gw&7,Zw=Qw?0:Yw,Dt=F+Zw|0,tE=i+Dt|0,nE=qd+16|0,rE=tE>>>0<nE>>>0,iE=rE?qd:tE,sE=iE+8|0,oE=Jx+ -40|0,uE=Wx+8|0,aE=uE,fE=aE&7,lE=(fE|0)==0,hE=0-aE|0,pE=hE&7,dE=lE?0:pE,vE=Wx+dE|0,mE=oE-dE|0,HEAP32[3317]=vE,HEAP32[3314]=mE,gE=mE|1,B=dE+4|0,yE=Wx+B|0,HEAP32[yE>>2]=gE,_t=Jx+ -36|0,bE=Wx+_t|0,HEAP32[bE>>2]=40,wE=HEAP32[3433]|0,HEAP32[3318]=wE,EE=iE+4|0,HEAP32[EE>>2]=27,HEAP32[sE>>2]=HEAP32[3423]|0,HEAP32[sE+4>>2]=HEAP32[3424]|0,HEAP32[sE+8>>2]=HEAP32[3425]|0,HEAP32[sE+12>>2]=HEAP32[3426]|0,HEAP32[3423]=Wx,HEAP32[3424]=Jx,HEAP32[3426]=0,HEAP32[3425]=sE,xE=iE+28|0,HEAP32[xE>>2]=7,TE=iE+32|0,NE=TE>>>0<o>>>0;if(NE){kE=xE;for(;;){CE=kE+4|0,HEAP32[CE>>2]=7,LE=kE+8|0,AE=LE>>>0<o>>>0;if(!AE)break;kE=CE}}OE=(iE|0)==(qd|0);if(!OE){ME=iE,_E=qd,PE=ME-_E|0,HE=HEAP32[EE>>2]|0,BE=HE&-2,HEAP32[EE>>2]=BE,jE=PE|1,FE=qd+4|0,HEAP32[FE>>2]=jE,HEAP32[iE>>2]=PE,IE=PE>>>3,qE=PE>>>0<256;if(qE){RE=IE<<1,UE=13284+(RE<<2)|0,zE=HEAP32[3311]|0,XE=1<<IE,VE=zE&XE,$E=(VE|0)==0,$E?(JE=zE|XE,HEAP32[3311]=JE,m=RE+2|0,k=13284+(m<<2)|0,b=k,hS=UE):(tn=RE+2|0,KE=13284+(tn<<2)|0,QE=HEAP32[KE>>2]|0,GE=HEAP32[3315]|0,YE=QE>>>0<GE>>>0,YE?_abort():(b=KE,hS=QE)),HEAP32[b>>2]=qd,ZE=hS+12|0,HEAP32[ZE>>2]=qd,eS=qd+8|0,HEAP32[eS>>2]=hS,nS=qd+12|0,HEAP32[nS>>2]=UE;break}rS=PE>>>8,iS=(rS|0)==0,iS?gS=0:(sS=PE>>>0>16777215,sS?gS=31:(oS=rS+1048320|0,uS=oS>>>16,aS=uS&8,fS=rS<<aS,lS=fS+520192|0,cS=lS>>>16,xn=cS&4,Tn=xn|aS,Nn=fS<<xn,Cn=Nn+245760|0,kn=Cn>>>16,Ln=kn&2,An=Tn|Ln,On=14-An|0,Mn=Nn<<Ln,_n=Mn>>>15,Pn=On+_n|0,Hn=Pn<<1,Bn=Pn+7|0,jn=PE>>>Bn,Fn=jn&1,In=Fn|Hn,gS=In)),qn=13548+(gS<<2)|0,Rn=qd+28|0,HEAP32[Rn>>2]=gS,Un=qd+20|0,HEAP32[Un>>2]=0,HEAP32[nE>>2]=0,zn=HEAP32[3312]|0,Xn=1<<gS,Vn=zn&Xn,$n=(Vn|0)==0;if($n){Jn=zn|Xn,HEAP32[3312]=Jn,HEAP32[qn>>2]=qd,Kn=qd+24|0,HEAP32[Kn>>2]=qn,Qn=qd+12|0,HEAP32[Qn>>2]=qd,Gn=qd+8|0,HEAP32[Gn>>2]=qd;break}Yn=HEAP32[qn>>2]|0,Zn=Yn+4|0,er=HEAP32[Zn>>2]|0,nr=er&-8,rr=(nr|0)==(PE|0);l:do if(rr)IS=Yn;else{ir=(gS|0)==31,sr=gS>>>1,or=25-sr|0,ur=ir?0:or,ar=PE<<ur,ES=ar,XS=Yn;for(;;){mr=ES>>>31,gr=(XS+16|0)+(mr<<2)|0,cr=HEAP32[gr>>2]|0,yr=(cr|0)==(0|0);if(yr){r=gr,VS=XS;break}fr=ES<<1,lr=cr+4|0,pr=HEAP32[lr>>2]|0,dr=pr&-8,vr=(dr|0)==(PE|0);if(vr){IS=cr;break l}ES=fr,XS=cr}br=HEAP32[3315]|0,wr=r>>>0<br>>>0;if(!wr){HEAP32[r>>2]=qd,Er=qd+24|0,HEAP32[Er>>2]=VS,Sr=qd+12|0,HEAP32[Sr>>2]=qd,Tr=qd+8|0,HEAP32[Tr>>2]=qd;break o}_abort()}while(0);Nr=IS+8|0,Cr=HEAP32[Nr>>2]|0,kr=HEAP32[3315]|0,Lr=Cr>>>0>=kr>>>0,rx=IS>>>0>=kr>>>0,Ar=Lr&rx;if(Ar){Or=Cr+12|0,HEAP32[Or>>2]=qd,HEAP32[Nr>>2]=qd,Mr=qd+8|0,HEAP32[Mr>>2]=Cr,_r=qd+12|0,HEAP32[_r>>2]=IS,Dr=qd+24|0,HEAP32[Dr>>2]=0;break}_abort()}}while(0);Hr=HEAP32[3314]|0,Br=Hr>>>0>tx>>>0;if(Br)return jr=Hr-tx|0,HEAP32[3314]=jr,Fr=HEAP32[3317]|0,Ir=Fr+tx|0,HEAP32[3317]=Ir,qr=jr|1,U=tx+4|0,Rr=Fr+U|0,HEAP32[Rr>>2]=qr,Ur=tx|3,zr=Fr+4|0,HEAP32[zr>>2]=Ur,Wr=Fr+8|0,ex=Wr,ex|0}return Vr=___errno_location()|0,HEAP32[Vr>>2]=12,ex=0,ex|0}function _free(e){e|=0;var t=0,n=0,r=0,i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0,vt=0,mt=0,gt=0,yt=0,bt=0,wt=0,Et=0,St=0,xt=0,Tt=0,Nt=0,Ct=0,kt=0,Lt=0,At=0,Ot=0,Mt=0,_t=0,Dt=0,Pt=0,Ht=0,Bt=0,jt=0,Ft=0,It=0,qt=0,Rt=0,Ut=0,zt=0,Wt=0,Xt=0,Vt=0,$t=0,Jt=0,Kt=0,Qt=0,Gt=0,Yt=0,Zt=0,en=0,tn=0,nn=0,rn=0,sn=0,on=0,un=0,an=0,fn=0,ln=0,cn=0,hn=0,pn=0,dn=0,vn=0,mn=0,gn=0,yn=0,bn=0,wn=0,En=0,Sn=0,xn=0,Tn=0,Nn=0,Cn=0,kn=0,Ln=0,An=0,On=0,Mn=0,_n=0,Dn=0,Pn=0,Hn=0,Bn=0,jn=0,Fn=0,In=0,qn=0,Rn=0,Un=0,zn=0,Wn=0,Xn=0,Vn=0,$n=0,Jn=0,Kn=0,Qn=0,Gn=0,Yn=0,Zn=0,er=0,tr=0,nr=0,rr=0,ir=0,sr=0,or=0,ur=0,ar=0,fr=0,lr=0,cr=0,hr=0,pr=0,dr=0,vr=0,mr=0,gr=0,yr=0,br=0,wr=0,Er=0,Sr=0,xr=0,Tr=0,Nr=0,Cr=0,kr=0,Lr=0,Ar=0,Or=0,Mr=0,_r=0,Dr=0,Pr=0,Hr=0,Br=0,jr=0,Fr=0,Ir=0,qr=0,Rr=0,Ur=0,zr=0,Wr=0,Xr=0,Vr=0,$r=0,Jr=0,Kr=0,Qr=0,Gr=0,Yr=0,Zr=0,ei=0,ti=0,ni=0,ri=0,ii=0,si=0,oi=0,ui=0,ai=0,fi=0,li=0,ci=0,hi=0,pi=0,di=0,vi=0,mi=0,gi=0,yi=0,bi=0,wi=0,Ei=0,Si=0,xi=0,Ti=0,Ni=0,Ci=0,ki=0,Li=0,Ai=0,Oi=0,Mi=0,_i=0,Di=0,Pi=0,Hi=0,Bi=0,ji=0,Fi=0,Ii=0,qi=0,Ri=0,Ui=0,zi=0,Wi=0,Xi=0,Vi=0,$i=0,Ji=0,Ki=0,Qi=0,Gi=0,Yi=0,Zi=0,es=0,ts=0,ns=0,rs=0,is=0,ss=0,os=0,us=0,as=0,fs=0,ls=0,cs=0,hs=0,ps=0,ds=0,vs=0,ms=0,gs=0,ys=0,bs=0,ws=0,Es=0,Ss=0,xs=0,Ts=0,Ns=0,Cs=0,ks=0,Ls=0,As=0,Os=0,Ms=0,_s=0,Ds=0,Ps=0,Hs=0,Bs=0,js=0,Fs=0,Is=0,qs=0,Rs=0,Us=0,zs=0,Ws=0,Xs=0,Vs=0,$s=0,Js=0,Ks=0,Qs=0,Gs=0,Ys=0,Zs=0,eo=0,to=0,no=0,ro=0,io=0,so=0,oo=0,uo=0,ao=0,fo=0,lo=0,co=0,ho=0,po=0,vo=0,mo=0,go=0,yo=0,bo=0,wo=0,Eo=0,So=0,xo=0,To=0,No=0,Co=0,ko=0,Lo=0,Ao=0,Oo=0,Mo=0,_o=0,Do=0,Po=0,Ho=0,Bo=0,jo=0,Fo=0,Io=0,qo=0,Ro=0,Uo=0,zo=0,Wo=0,Xo=0,Vo=0,$o=0,Jo=0,Ko=0,Qo=0,Go=0,Yo=0,Zo=0,eu=0,tu=0,nu=0,ru=0,iu=0;iu=STACKTOP,D=(e|0)==(0|0);if(D)return;P=e+ -8|0,Fn=HEAP32[3315]|0,Ri=P>>>0<Fn>>>0,Ri&&_abort(),ws=e+ -4|0,Ms=HEAP32[ws>>2]|0,Us=Ms&3,Zs=(Us|0)==1,Zs&&_abort(),lo=Ms&-8,f=lo+ -8|0,So=e+f|0,H=Ms&1,V=(H|0)==0;do if(V){rt=HEAP32[P>>2]|0,dt=(Us|0)==0;if(dt)return;m=-8-rt|0,Nt=e+m|0,Bt=rt+lo|0,$t=Nt>>>0<Fn>>>0,$t&&_abort(),sn=HEAP32[3316]|0,mn=(Nt|0)==(sn|0);if(mn){C=lo+ -4|0,I=e+C|0,q=HEAP32[I>>2]|0,R=q&3,U=(R|0)==3;if(!U){Yo=Nt,Zo=Bt;break}HEAP32[3313]=Bt,z=q&-2,HEAP32[I>>2]=z,W=Bt|1,g=m+4|0,X=e+g|0,HEAP32[X>>2]=W,HEAP32[So>>2]=Bt;return}kn=rt>>>3,In=rt>>>0<256;if(In){k=m+8|0,Qn=e+k|0,ur=HEAP32[Qn>>2]|0,L=m+12|0,yr=e+L|0,Ar=HEAP32[yr>>2]|0,qr=kn<<1,Gr=13284+(qr<<2)|0,ai=(ur|0)==(Gr|0),ai||(bi=ur>>>0<Fn>>>0,bi&&_abort(),Oi=ur+12|0,Ui=HEAP32[Oi>>2]|0,Zi=(Ui|0)==(Nt|0),Zi||_abort()),ls=(Ar|0)==(ur|0);if(ls){ps=1<<kn,ds=ps^-1,vs=HEAP32[3311]|0,ms=vs&ds,HEAP32[3311]=ms,Yo=Nt,Zo=Bt;break}gs=(Ar|0)==(Gr|0),gs?(a=Ar+8|0,i=a):(ys=Ar>>>0<Fn>>>0,ys&&_abort(),bs=Ar+8|0,Es=HEAP32[bs>>2]|0,Ss=(Es|0)==(Nt|0),Ss?i=bs:_abort()),xs=ur+12|0,HEAP32[xs>>2]=Ar,HEAP32[i>>2]=ur,Yo=Nt,Zo=Bt;break}y=m+24|0,Ts=e+y|0,Ns=HEAP32[Ts>>2]|0,b=m+12|0,Cs=e+b|0,ks=HEAP32[Cs>>2]|0,Ls=(ks|0)==(Nt|0);do if(Ls){E=m+20|0,Is=e+E|0,qs=HEAP32[Is>>2]|0,Rs=(qs|0)==(0|0);if(Rs){w=m+16|0,zs=e+w|0,Ws=HEAP32[zs>>2]|0,Xs=(Ws|0)==(0|0);if(Xs){Fo=0;break}Bo=Ws,Uo=zs}else Bo=qs,Uo=Is;for(;;){Vs=Bo+20|0,$s=HEAP32[Vs>>2]|0,Js=($s|0)==(0|0);if(!Js){Bo=$s,Uo=Vs;continue}Ks=Bo+16|0,Qs=HEAP32[Ks>>2]|0,Gs=(Qs|0)==(0|0);if(Gs){jo=Bo,zo=Uo;break}Bo=Qs,Uo=Ks}Ys=zo>>>0<Fn>>>0;if(!Ys){HEAP32[zo>>2]=0,Fo=jo;break}_abort()}else{N=m+8|0,As=e+N|0,Os=HEAP32[As>>2]|0,_s=Os>>>0<Fn>>>0,_s&&_abort(),Ds=Os+12|0,Ps=HEAP32[Ds>>2]|0,Hs=(Ps|0)==(Nt|0),Hs||_abort(),Bs=ks+8|0,js=HEAP32[Bs>>2]|0,Fs=(js|0)==(Nt|0);if(Fs){HEAP32[Ds>>2]=ks,HEAP32[Bs>>2]=Os,Fo=ks;break}_abort()}while(0);eo=(Ns|0)==(0|0);if(eo)Yo=Nt,Zo=Bt;else{S=m+28|0,to=e+S|0,no=HEAP32[to>>2]|0,ro=13548+(no<<2)|0,io=HEAP32[ro>>2]|0,so=(Nt|0)==(io|0);if(so){HEAP32[ro>>2]=Fo,Ko=(Fo|0)==(0|0);if(Ko){oo=1<<no,uo=oo^-1,ao=HEAP32[3312]|0,fo=ao&uo,HEAP32[3312]=fo,Yo=Nt,Zo=Bt;break}}else{co=HEAP32[3315]|0,ho=Ns>>>0<co>>>0,ho&&_abort(),po=Ns+16|0,vo=HEAP32[po>>2]|0,mo=(vo|0)==(Nt|0),mo?HEAP32[po>>2]=Fo:(go=Ns+20|0,HEAP32[go>>2]=Fo),yo=(Fo|0)==(0|0);if(yo){Yo=Nt,Zo=Bt;break}}bo=HEAP32[3315]|0,wo=Fo>>>0<bo>>>0,wo&&_abort(),Eo=Fo+24|0,HEAP32[Eo>>2]=Ns,x=m+16|0,xo=e+x|0,To=HEAP32[xo>>2]|0,No=(To|0)==(0|0);do if(!No){Co=To>>>0<bo>>>0;if(!Co){ko=Fo+16|0,HEAP32[ko>>2]=To,Lo=To+24|0,HEAP32[Lo>>2]=Fo;break}_abort()}while(0);T=m+20|0,Ao=e+T|0,Oo=HEAP32[Ao>>2]|0,Mo=(Oo|0)==(0|0);if(Mo)Yo=Nt,Zo=Bt;else{_o=HEAP32[3315]|0,B=Oo>>>0<_o>>>0;if(!B){j=Fo+20|0,HEAP32[j>>2]=Oo,F=Oo+24|0,HEAP32[F>>2]=Fo,Yo=Nt,Zo=Bt;break}_abort()}}}else Yo=P,Zo=lo;while(0);$=Yo>>>0<So>>>0,$||_abort(),v=lo+ -4|0,J=e+v|0,K=HEAP32[J>>2]|0,Q=K&1,G=(Q|0)==0,G&&_abort(),Y=K&2,Z=(Y|0)==0;if(Z){et=HEAP32[3317]|0,tt=(So|0)==(et|0);if(tt){nt=HEAP32[3314]|0,it=nt+Zo|0,HEAP32[3314]=it,HEAP32[3317]=Yo,st=it|1,ot=Yo+4|0,HEAP32[ot>>2]=st,ut=HEAP32[3316]|0,at=(Yo|0)==(ut|0);if(!at)return;HEAP32[3316]=0,HEAP32[3313]=0;return}ft=HEAP32[3316]|0,lt=(So|0)==(ft|0);if(lt){ct=HEAP32[3313]|0,ht=ct+Zo|0,HEAP32[3313]=ht,HEAP32[3316]=Yo,pt=ht|1,vt=Yo+4|0,HEAP32[vt>>2]=pt,mt=Yo+ht|0,HEAP32[mt>>2]=ht;return}gt=K&-8,yt=gt+Zo|0,bt=K>>>3,wt=K>>>0<256;do if(wt){Et=e+lo|0,St=HEAP32[Et>>2]|0,d=lo|4,xt=e+d|0,Tt=HEAP32[xt>>2]|0,Ct=bt<<1,kt=13284+(Ct<<2)|0,Lt=(St|0)==(kt|0),Lt||(At=HEAP32[3315]|0,Ot=St>>>0<At>>>0,Ot&&_abort(),Mt=St+12|0,_t=HEAP32[Mt>>2]|0,Dt=(_t|0)==(So|0),Dt||_abort()),Pt=(Tt|0)==(St|0);if(Pt){Ht=1<<bt,jt=Ht^-1,Ft=HEAP32[3311]|0,It=Ft&jt,HEAP32[3311]=It;break}qt=(Tt|0)==(kt|0),qt?(u=Tt+8|0,r=u):(Rt=HEAP32[3315]|0,Ut=Tt>>>0<Rt>>>0,Ut&&_abort(),zt=Tt+8|0,Wt=HEAP32[zt>>2]|0,Xt=(Wt|0)==(So|0),Xt?r=zt:_abort()),Vt=St+12|0,HEAP32[Vt>>2]=Tt,HEAP32[r>>2]=St}else{A=lo+16|0,Jt=e+A|0,Kt=HEAP32[Jt>>2]|0,O=lo|4,Qt=e+O|0,Gt=HEAP32[Qt>>2]|0,Yt=(Gt|0)==(So|0);do if(Yt){_=lo+12|0,cn=e+_|0,hn=HEAP32[cn>>2]|0,pn=(hn|0)==(0|0);if(pn){M=lo+8|0,dn=e+M|0,vn=HEAP32[dn>>2]|0,gn=(vn|0)==(0|0);if(gn){Ro=0;break}Io=vn,Wo=dn}else Io=hn,Wo=cn;for(;;){yn=Io+20|0,bn=HEAP32[yn>>2]|0,wn=(bn|0)==(0|0);if(!wn){Io=bn,Wo=yn;continue}En=Io+16|0,Sn=HEAP32[En>>2]|0,xn=(Sn|0)==(0|0);if(xn){qo=Io,Xo=Wo;break}Io=Sn,Wo=En}Tn=HEAP32[3315]|0,Nn=Xo>>>0<Tn>>>0;if(!Nn){HEAP32[Xo>>2]=0,Ro=qo;break}_abort()}else{Zt=e+lo|0,en=HEAP32[Zt>>2]|0,tn=HEAP32[3315]|0,nn=en>>>0<tn>>>0,nn&&_abort(),rn=en+12|0,on=HEAP32[rn>>2]|0,un=(on|0)==(So|0),un||_abort(),an=Gt+8|0,fn=HEAP32[an>>2]|0,ln=(fn|0)==(So|0);if(ln){HEAP32[rn>>2]=Gt,HEAP32[an>>2]=en,Ro=Gt;break}_abort()}while(0);Cn=(Kt|0)==(0|0);if(!Cn){c=lo+20|0,Ln=e+c|0,An=HEAP32[Ln>>2]|0,On=13548+(An<<2)|0,Mn=HEAP32[On>>2]|0,_n=(So|0)==(Mn|0);if(_n){HEAP32[On>>2]=Ro,Qo=(Ro|0)==(0|0);if(Qo){Dn=1<<An,Pn=Dn^-1,Hn=HEAP32[3312]|0,Bn=Hn&Pn,HEAP32[3312]=Bn;break}}else{jn=HEAP32[3315]|0,qn=Kt>>>0<jn>>>0,qn&&_abort(),Rn=Kt+16|0,Un=HEAP32[Rn>>2]|0,zn=(Un|0)==(So|0),zn?HEAP32[Rn>>2]=Ro:(Wn=Kt+20|0,HEAP32[Wn>>2]=Ro),Xn=(Ro|0)==(0|0);if(Xn)break}Vn=HEAP32[3315]|0,$n=Ro>>>0<Vn>>>0,$n&&_abort(),Jn=Ro+24|0,HEAP32[Jn>>2]=Kt,h=lo+8|0,Kn=e+h|0,Gn=HEAP32[Kn>>2]|0,Yn=(Gn|0)==(0|0);do if(!Yn){Zn=Gn>>>0<Vn>>>0;if(!Zn){er=Ro+16|0,HEAP32[er>>2]=Gn,tr=Gn+24|0,HEAP32[tr>>2]=Ro;break}_abort()}while(0);p=lo+12|0,nr=e+p|0,rr=HEAP32[nr>>2]|0,ir=(rr|0)==(0|0);if(!ir){sr=HEAP32[3315]|0,or=rr>>>0<sr>>>0;if(!or){ar=Ro+20|0,HEAP32[ar>>2]=rr,fr=rr+24|0,HEAP32[fr>>2]=Ro;break}_abort()}}}while(0);lr=yt|1,cr=Yo+4|0,HEAP32[cr>>2]=lr,hr=Yo+yt|0,HEAP32[hr>>2]=yt,pr=HEAP32[3316]|0,dr=(Yo|0)==(pr|0);if(dr){HEAP32[3313]=yt;return}eu=yt}else vr=K&-2,HEAP32[J>>2]=vr,mr=Zo|1,gr=Yo+4|0,HEAP32[gr>>2]=mr,br=Yo+Zo|0,HEAP32[br>>2]=Zo,eu=Zo;wr=eu>>>3,Er=eu>>>0<256;if(Er){Sr=wr<<1,xr=13284+(Sr<<2)|0,Tr=HEAP32[3311]|0,Nr=1<<wr,Cr=Tr&Nr,kr=(Cr|0)==0,kr?(Lr=Tr|Nr,HEAP32[3311]=Lr,n=Sr+2|0,o=13284+(n<<2)|0,s=o,Do=xr):(l=Sr+2|0,Or=13284+(l<<2)|0,Mr=HEAP32[Or>>2]|0,_r=HEAP32[3315]|0,Dr=Mr>>>0<_r>>>0,Dr?_abort():(s=Or,Do=Mr)),HEAP32[s>>2]=Yo,Pr=Do+12|0,HEAP32[Pr>>2]=Yo,Hr=Yo+8|0,HEAP32[Hr>>2]=Do,Br=Yo+12|0,HEAP32[Br>>2]=xr;return}jr=eu>>>8,Fr=(jr|0)==0,Fr?Po=0:(Ir=eu>>>0>16777215,Ir?Po=31:(Rr=jr+1048320|0,Ur=Rr>>>16,zr=Ur&8,Wr=jr<<zr,Xr=Wr+520192|0,Vr=Xr>>>16,$r=Vr&4,Jr=$r|zr,Kr=Wr<<$r,Qr=Kr+245760|0,Yr=Qr>>>16,Zr=Yr&2,ei=Jr|Zr,ti=14-ei|0,ni=Kr<<Zr,ri=ni>>>15,ii=ti+ri|0,si=ii<<1,oi=ii+7|0,ui=eu>>>oi,fi=ui&1,li=fi|si,Po=li)),ci=13548+(Po<<2)|0,hi=Yo+28|0,HEAP32[hi>>2]=Po,pi=Yo+16|0,di=Yo+20|0,HEAP32[di>>2]=0,HEAP32[pi>>2]=0,vi=HEAP32[3312]|0,mi=1<<Po,gi=vi&mi,yi=(gi|0)==0;e:do if(yi)wi=vi|mi,HEAP32[3312]=wi,HEAP32[ci>>2]=Yo,Ei=Yo+24|0,HEAP32[Ei>>2]=ci,Si=Yo+12|0,HEAP32[Si>>2]=Yo,xi=Yo+8|0,HEAP32[xi>>2]=Yo;else{Ti=HEAP32[ci>>2]|0,Ni=Ti+4|0,Ci=HEAP32[Ni>>2]|0,ki=Ci&-8,Li=(ki|0)==(eu|0);t:do if(Li)Vo=Ti;else{Ai=(Po|0)==31,Mi=Po>>>1,_i=25-Mi|0,Di=Ai?0:_i,Pi=eu<<Di,Ho=Pi,$o=Ti;for(;;){zi=Ho>>>31,Wi=($o+16|0)+(zi<<2)|0,ji=HEAP32[Wi>>2]|0,Xi=(ji|0)==(0|0);if(Xi){t=Wi,Jo=$o;break}Hi=Ho<<1,Bi=ji+4|0,Fi=HEAP32[Bi>>2]|0,Ii=Fi&-8,qi=(Ii|0)==(eu|0);if(qi){Vo=ji;break t}Ho=Hi,$o=ji}Vi=HEAP32[3315]|0,$i=t>>>0<Vi>>>0;if(!$i){HEAP32[t>>2]=Yo,Ji=Yo+24|0,HEAP32[Ji>>2]=Jo,Ki=Yo+12|0,HEAP32[Ki>>2]=Yo,Qi=Yo+8|0,HEAP32[Qi>>2]=Yo;break e}_abort()}while(0);Gi=Vo+8|0,Yi=HEAP32[Gi>>2]|0,es=HEAP32[3315]|0,ts=Yi>>>0>=es>>>0,Go=Vo>>>0>=es>>>0,ns=ts&Go;if(ns){rs=Yi+12|0,HEAP32[rs>>2]=Yo,HEAP32[Gi>>2]=Yo,is=Yo+8|0,HEAP32[is>>2]=Yi,ss=Yo+12|0,HEAP32[ss>>2]=Vo,os=Yo+24|0,HEAP32[os>>2]=0;break}_abort()}while(0);us=HEAP32[3319]|0,as=us+ -1|0,HEAP32[3319]=as,fs=(as|0)==0;if(!fs)return;nu=13700;for(;;){tu=HEAP32[nu>>2]|0,cs=(tu|0)==(0|0),hs=tu+8|0;if(cs)break;nu=hs}HEAP32[3319]=-1;return}function runPostSets(){}function _i64Subtract(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0;return i=e-n>>>0,s=t-r>>>0,s=t-r-(n>>>0>e>>>0|0)>>>0,(tempRet0=s,i|0)|0}function _memset(e,t,n){e|=0,t|=0,n|=0;var r=0,i=0,s=0,o=0;r=e+n|0;if((n|0)>=20){t&=255,o=e&3,i=t|t<<8|t<<16|t<<24,s=r&-4;if(o){o=e+4-o|0;while((e|0)<(o|0))HEAP8[e>>0]=t,e=e+1|0}while((e|0)<(s|0))HEAP32[e>>2]=i,e=e+4|0}while((e|0)<(r|0))HEAP8[e>>0]=t,e=e+1|0;return e-n|0}function _bitshift64Shl(e,t,n){e|=0,t|=0,n|=0;var r=0;return(n|0)<32?(r=(1<<n)-1|0,tempRet0=t<<n|(e&r<<32-n)>>>32-n,e<<n):(tempRet0=e<<n-32,0)}function _i64Add(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0;return i=e+n>>>0,s=t+r+(i>>>0<e>>>0|0)>>>0,(tempRet0=s,i|0)|0}function _bitshift64Lshr(e,t,n){e|=0,t|=0,n|=0;var r=0;return(n|0)<32?(r=(1<<n)-1|0,tempRet0=t>>>n,e>>>n|(t&r)<<32-n):(tempRet0=0,t>>>n-32|0)}function _memcpy(e,t,n){e|=0,t|=0,n|=0;var r=0;if((n|0)>=4096)return _emscripten_memcpy_big(e|0,t|0,n|0)|0;r=e|0;if((e&3)==(t&3)){while(e&3){if((n|0)==0)return r|0;HEAP8[e>>0]=HEAP8[t>>0]|0,e=e+1|0,t=t+1|0,n=n-1|0}while((n|0)>=4)HEAP32[e>>2]=HEAP32[t>>2]|0,e=e+4|0,t=t+4|0,n=n-4|0}while((n|0)>0)HEAP8[e>>0]=HEAP8[t>>0]|0,e=e+1|0,t=t+1|0,n=n-1|0;return r|0}function _bitshift64Ashr(e,t,n){e|=0,t|=0,n|=0;var r=0;return(n|0)<32?(r=(1<<n)-1|0,tempRet0=t>>n,e>>>n|(t&r)<<32-n):(tempRet0=(t|0)<0?-1:0,t>>n-32|0)}function _llvm_cttz_i32(e){e|=0;var t=0;return t=HEAP8[cttz_i8+(e&255)>>0]|0,(t|0)<8?t|0:(t=HEAP8[cttz_i8+(e>>8&255)>>0]|0,(t|0)<8?t+8|0:(t=HEAP8[cttz_i8+(e>>16&255)>>0]|0,(t|0)<8?t+16|0:(HEAP8[cttz_i8+(e>>>24)>>0]|0)+24|0))}function ___muldsi3(e,t){e|=0,t|=0;var n=0,r=0,i=0,s=0,o=0,u=0,a=0;return n=e&65535,r=t&65535,i=Math_imul(r,n)|0,s=e>>>16,o=(i>>>16)+(Math_imul(r,s)|0)|0,u=t>>>16,a=Math_imul(u,n)|0,(tempRet0=((o>>>16)+(Math_imul(u,s)|0)|0)+(((o&65535)+a|0)>>>16)|0,0|(o+a<<16|i&65535))|0}function ___divdi3(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0;return i=t>>31|((t|0)<0?-1:0)<<1,s=((t|0)<0?-1:0)>>31|((t|0)<0?-1:0)<<1,o=r>>31|((r|0)<0?-1:0)<<1,u=((r|0)<0?-1:0)>>31|((r|0)<0?-1:0)<<1,a=_i64Subtract(i^e,s^t,i,s)|0,f=tempRet0,l=_i64Subtract(o^n,u^r,o,u)|0,c=o^i,h=u^s,p=___udivmoddi4(a,f,l,tempRet0,0)|0,d=_i64Subtract(p^c,tempRet0^h,c,h)|0,d|0}function ___remdi3(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0;return d=STACKTOP,STACKTOP=STACKTOP+16|0,i=d|0,s=t>>31|((t|0)<0?-1:0)<<1,o=((t|0)<0?-1:0)>>31|((t|0)<0?-1:0)<<1,u=r>>31|((r|0)<0?-1:0)<<1,a=((r|0)<0?-1:0)>>31|((r|0)<0?-1:0)<<1,f=_i64Subtract(s^e,o^t,s,o)|0,l=tempRet0,c=_i64Subtract(u^n,a^r,u,a)|0,___udivmoddi4(f,l,c,tempRet0,i)|0,h=_i64Subtract(HEAP32[i>>2]^s,HEAP32[i+4>>2]^o,s,o)|0,p=tempRet0,STACKTOP=d,(tempRet0=p,h)|0};

function ___muldi3(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0,o=0,u=0,a=0;return i=e,s=n,o=___muldsi3(i,s)|0,u=tempRet0,a=Math_imul(t,s)|0,(tempRet0=((Math_imul(r,i)|0)+a|0)+u|u&0,0|o&-1)|0}function ___udivdi3(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0;return i=___udivmoddi4(e,t,n,r,0)|0,i|0}function ___uremdi3(e,t,n,r){e|=0,t|=0,n|=0,r|=0;var i=0,s=0;return s=STACKTOP,STACKTOP=STACKTOP+16|0,i=s|0,___udivmoddi4(e,t,n,r,i)|0,STACKTOP=s,(tempRet0=HEAP32[i+4>>2]|0,HEAP32[i>>2]|0)|0}function ___udivmoddi4(e,t,n,r,i){e|=0,t|=0,n|=0,r|=0,i|=0;var s=0,o=0,u=0,a=0,f=0,l=0,c=0,h=0,p=0,d=0,v=0,m=0,g=0,y=0,b=0,w=0,E=0,S=0,x=0,T=0,N=0,C=0,k=0,L=0,A=0,O=0,M=0,_=0,D=0,P=0,H=0,B=0,j=0,F=0,I=0,q=0,R=0,U=0,z=0,W=0,X=0,V=0,$=0,J=0,K=0,Q=0,G=0,Y=0,Z=0,et=0,tt=0,nt=0,rt=0,it=0,st=0,ot=0,ut=0,at=0,ft=0,lt=0,ct=0,ht=0,pt=0,dt=0;s=e,o=t,u=o,a=n,f=r,l=f;if((u|0)==0)return c=(i|0)!=0,(l|0)==0?(c&&(HEAP32[i>>2]=(s>>>0)%(a>>>0),HEAP32[i+4>>2]=0),dt=0,pt=(s>>>0)/(a>>>0)>>>0,(tempRet0=dt,pt)|0):c?(HEAP32[i>>2]=e&-1,HEAP32[i+4>>2]=t&0,dt=0,pt=0,(tempRet0=dt,pt)|0):(dt=0,pt=0,(tempRet0=dt,pt)|0);h=(l|0)==0;do{if((a|0)==0){if(h)return(i|0)!=0&&(HEAP32[i>>2]=(u>>>0)%(a>>>0),HEAP32[i+4>>2]=0),dt=0,pt=(u>>>0)/(a>>>0)>>>0,(tempRet0=dt,pt)|0;if((s|0)==0)return(i|0)!=0&&(HEAP32[i>>2]=0,HEAP32[i+4>>2]=(u>>>0)%(l>>>0)),dt=0,pt=(u>>>0)/(l>>>0)>>>0,(tempRet0=dt,pt)|0;p=l-1|0;if((p&l|0)==0)return(i|0)!=0&&(HEAP32[i>>2]=0|e&-1,HEAP32[i+4>>2]=p&u|t&0),dt=0,pt=u>>>((_llvm_cttz_i32(l|0)|0)>>>0),(tempRet0=dt,pt)|0;d=Math_clz32(l|0)|0,v=d-(Math_clz32(u|0)|0)|0;if(v>>>0<=30){m=v+1|0,g=31-v|0,B=m,H=u<<g|s>>>(m>>>0),P=u>>>(m>>>0),D=0,_=s<<g;break}return(i|0)==0?(dt=0,pt=0,(tempRet0=dt,pt)|0):(HEAP32[i>>2]=0|e&-1,HEAP32[i+4>>2]=o|t&0,dt=0,pt=0,(tempRet0=dt,pt)|0)}if(!h){k=Math_clz32(l|0)|0,L=k-(Math_clz32(u|0)|0)|0;if(L>>>0<=31){A=L+1|0,O=31-L|0,M=L-31>>31,B=A,H=s>>>(A>>>0)&M|u<<O,P=u>>>(A>>>0)&M,D=0,_=s<<O;break}return(i|0)==0?(dt=0,pt=0,(tempRet0=dt,pt)|0):(HEAP32[i>>2]=0|e&-1,HEAP32[i+4>>2]=o|t&0,dt=0,pt=0,(tempRet0=dt,pt)|0)}y=a-1|0;if((y&a|0)!=0){w=(Math_clz32(a|0)|0)+33|0,E=w-(Math_clz32(u|0)|0)|0,S=64-E|0,x=32-E|0,T=x>>31,N=E-32|0,C=N>>31,B=E,H=x-1>>31&u>>>(N>>>0)|(u<<x|s>>>(E>>>0))&C,P=C&u>>>(E>>>0),D=s<<S&T,_=(u<<S|s>>>(N>>>0))&T|s<<x&E-33>>31;break}return(i|0)!=0&&(HEAP32[i>>2]=y&s,HEAP32[i+4>>2]=0),(a|0)==1?(dt=o|t&0,pt=0|e&-1,(tempRet0=dt,pt)|0):(b=_llvm_cttz_i32(a|0)|0,dt=0|u>>>(b>>>0),pt=u<<32-b|s>>>(b>>>0)|0,(tempRet0=dt,pt)|0)}while(0);if((B|0)==0)ft=_,at=D,ut=P,ot=H,st=0,it=0;else{j=0|n&-1,F=f|r&0,I=_i64Add(j|0,F|0,-1,-1)|0,q=tempRet0,V=_,X=D,W=P,z=H,U=B,R=0;for(;;){$=X>>>31|V<<1,J=R|X<<1,K=0|(z<<1|V>>>31),Q=z>>>31|W<<1|0,_i64Subtract(I,q,K,Q)|0,G=tempRet0,Y=G>>31|((G|0)<0?-1:0)<<1,Z=Y&1,et=_i64Subtract(K,Q,Y&j,(((G|0)<0?-1:0)>>31|((G|0)<0?-1:0)<<1)&F)|0,tt=et,nt=tempRet0,rt=U-1|0;if((rt|0)==0)break;V=$,X=J,W=nt,z=tt,U=rt,R=Z}ft=$,at=J,ut=nt,ot=tt,st=0,it=Z}return lt=at,ct=0,ht=ft|ct,(i|0)!=0&&(HEAP32[i>>2]=0|ot,HEAP32[i+4>>2]=ut|0),dt=(0|lt)>>>31|ht<<1|(ct<<1|lt>>>31)&0|st,pt=(lt<<1|0)&-2|it,(tempRet0=dt,pt)|0};
// =======================================================================




function dynCall_i(index) {
index = index|0;

return FUNCTION_TABLE_i[index&15]()|0;
}


function dynCall_ii(index,a1) {
index = index|0;
a1=a1|0;
return FUNCTION_TABLE_ii[index&15](a1|0)|0;
}


function dynCall_iiii(index,a1,a2,a3) {
index = index|0;
a1=a1|0; a2=a2|0; a3=a3|0;
return FUNCTION_TABLE_iiii[index&15](a1|0,a2|0,a3|0)|0;
}


function dynCall_iiiii(index,a1,a2,a3,a4) {
index = index|0;
a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
return FUNCTION_TABLE_iiiii[index&15](a1|0,a2|0,a3|0,a4|0)|0;
}


function dynCall_vi(index,a1) {
index = index|0;
a1=a1|0;
FUNCTION_TABLE_vi[index&15](a1|0);
}

function b0() {
; nullFunc_i(0);return 0;
}
function b1(p0) {
p0 = p0|0; nullFunc_ii(1);return 0;
}
function b2(p0,p1,p2) {
p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(2);return 0;
}
function b3(p0,p1,p2,p3) {
p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_iiiii(3);return 0;
}
function b4(p0) {
p0 = p0|0; nullFunc_vi(4);
}

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_i = [b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_xw_end,_xw_rev,_x_wSL,b0,b0,b0];
var FUNCTION_TABLE_ii = [b1,b1,___stdio_close,b1,b1,b1,b1,b1,b1,_w_ned,b1,b1,b1,b1,b1,b1];
var FUNCTION_TABLE_iiii = [b2,_sn_write,b2,___stdout_write,___stdio_seek,___stdio_write,___stdio_read,b2,_w_ed,b2,b2,b2,b2,b2,b2,b2];
var FUNCTION_TABLE_iiiii = [b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,_l_write,b3,b3];
var FUNCTION_TABLE_vi = [b4,b4,b4,b4,b4,b4,b4,_x_putc,b4,b4,b4,b4,b4,b4,_cleanup,_cleanup52];

return { _i64Subtract: _i64Subtract, _free: _free, _i64Add: _i64Add, _dgesv_: _dgesv_, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, _setulb_: _setulb_, _bitshift64Lshr: _bitshift64Lshr, _fflush: _fflush, ___errno_location: ___errno_location, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_i: dynCall_i, dynCall_ii: dynCall_ii, dynCall_iiii: dynCall_iiii, dynCall_iiiii: dynCall_iiiii, dynCall_vi: dynCall_vi };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var real__i64Subtract = asm["_i64Subtract"]; asm["_i64Subtract"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Subtract.apply(null, arguments);
};

var real__free = asm["_free"]; asm["_free"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__free.apply(null, arguments);
};

var real__i64Add = asm["_i64Add"]; asm["_i64Add"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Add.apply(null, arguments);
};

var real__dgesv_ = asm["_dgesv_"]; asm["_dgesv_"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__dgesv_.apply(null, arguments);
};

var real__malloc = asm["_malloc"]; asm["_malloc"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__malloc.apply(null, arguments);
};

var real__setulb_ = asm["_setulb_"]; asm["_setulb_"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__setulb_.apply(null, arguments);
};

var real__bitshift64Lshr = asm["_bitshift64Lshr"]; asm["_bitshift64Lshr"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Lshr.apply(null, arguments);
};

var real__fflush = asm["_fflush"]; asm["_fflush"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__fflush.apply(null, arguments);
};

var real____errno_location = asm["___errno_location"]; asm["___errno_location"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real____errno_location.apply(null, arguments);
};

var real__bitshift64Shl = asm["_bitshift64Shl"]; asm["_bitshift64Shl"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Shl.apply(null, arguments);
};
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _free = Module["_free"] = asm["_free"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _dgesv_ = Module["_dgesv_"] = asm["_dgesv_"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _setulb_ = Module["_setulb_"] = asm["_setulb_"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _fflush = Module["_fflush"] = asm["_fflush"];
var ___errno_location = Module["___errno_location"] = asm["___errno_location"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
;

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];

Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];



// === Auto-generated postamble setup entry stuff ===


function ExitStatus(status) {
this.name = "ExitStatus";
this.message = "Program terminated with exit(" + status + ")";
this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
// If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
if (!Module['calledRun']) run();
if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

args = args || [];

ensureInitRuntime();

var argc = args.length+1;
function pad() {
for (var i = 0; i < 4-1; i++) {
argv.push(0);
}
}
var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
pad();
for (var i = 0; i < argc-1; i = i + 1) {
argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
pad();
}
argv.push(0);
argv = allocate(argv, 'i32', ALLOC_NORMAL);


try {

var ret = Module['_main'](argc, argv, 0);


// if we're not running an evented main loop, it's time to exit
exit(ret, /* implicit = */ true);
}
catch(e) {
if (e instanceof ExitStatus) {
// exit() throws this once it's done to make sure execution
// has been stopped completely
return;
} else if (e == 'SimulateInfiniteLoop') {
// running an evented main loop, don't immediately exit
Module['noExitRuntime'] = true;
return;
} else {
if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
throw e;
}
} finally {
calledMain = true;
}
}




function run(args) {
args = args || Module['arguments'];

if (preloadStartTime === null) preloadStartTime = Date.now();

if (runDependencies > 0) {
Module.printErr('run() called, but dependencies remain, so not running');
return;
}

preRun();

if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

function doRun() {
if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
Module['calledRun'] = true;

if (ABORT) return; 

ensureInitRuntime();

preMain();

if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
}

if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

if (Module['_main'] && shouldRunNow) Module['callMain'](args);

postRun();
}

if (Module['setStatus']) {
Module['setStatus']('Running...');
setTimeout(function() {
setTimeout(function() {
Module['setStatus']('');
}, 1);
doRun();
}, 1);
} else {
doRun();
}
}
Module['run'] = Module.run = run;

function exit(status, implicit) {
if (implicit && Module['noExitRuntime']) {
Module.printErr('exit(' + status + ') implicitly called by end of main(), but noExitRuntime, so not exiting the runtime (you can use emscripten_force_exit, if you want to force a true shutdown)');
return;
}

if (Module['noExitRuntime']) {
Module.printErr('exit(' + status + ') called, but noExitRuntime, so halting execution but not exiting the runtime or preventing further async execution (you can use emscripten_force_exit, if you want to force a true shutdown)');
} else {

ABORT = true;
EXITSTATUS = status;
STACKTOP = initialStackTop;

exitRuntime();

if (Module['onExit']) Module['onExit'](status);
}

if (ENVIRONMENT_IS_NODE) {
// Work around a node.js bug where stdout buffer is not flushed at process exit:
// Instead of process.exit() directly, wait for stdout flush event.
// See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
// Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
process['stdout']['once']('drain', function () {
process['exit'](status);
});
console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
// Work around another node bug where sometimes 'drain' is never fired - make another effort
// to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
setTimeout(function() {
process['exit'](status);
}, 500);
} else
if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
quit(status);
}
// if we reach here, we must throw an exception to halt the current execution
throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
if (what !== undefined) {
Module.print(what);
Module.printErr(what);
what = JSON.stringify(what)
} else {
what = '';
}

ABORT = true;
EXITSTATUS = 1;

var extra = '';

var output = 'abort(' + what + ') at ' + stackTrace() + extra;
if (abortDecorators) {
abortDecorators.forEach(function(decorator) {
output = decorator(output, what);
});
}
throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
while (Module['preInit'].length > 0) {
Module['preInit'].pop()();
}
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



