// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_WEB = typeof window === 'object';
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function' && !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_WORKER;
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) {
    var ret = Module['read'](filename, true);
    if (!ret.buffer) {
      ret = new Uint8Array(ret);
    }
    assert(ret.buffer);
    return ret;
  };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (!Module['thisProgram']) {
    if (process['argv'].length > 1) {
      Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
    } else {
      Module['thisProgram'] = 'unknown-program';
    }
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });

  Module['inspect'] = function () { return '[Emscripten Module object]'; };
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WORKER) {
    Module['load'] = importScripts;
  }

  if (typeof Module['setWindowTitle'] === 'undefined') {
    Module['setWindowTitle'] = function(title) { document.title = title };
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  prepVararg: function (ptr, type) {
    if (type === 'double' || type === 'i64') {
      // move so the load is aligned
      if (ptr & 7) {
        assert((ptr & 7) === 4);
        ptr += 4;
      }
    } else {
      assert((ptr & 3) === 0);
    }
    return ptr;
  },
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret;  return 0; } }; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}



Module["Runtime"] = Runtime;



//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args, opts) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if ((!opts || !opts.async) && typeof EmterpreterAsync === 'object') {
      assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling ccall');
    }
    if (opts && opts.async) assert(!returnType, 'async ccalls cannot return values');
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) {
      if (opts && opts.async) {
        EmterpreterAsync.asyncFinalizers.push(function() {
          Runtime.stackRestore(stack);
        });
        return;
      }
      Runtime.stackRestore(stack);
    }
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    funcstr += "if (typeof EmterpreterAsync === 'object') { assert(!EmterpreterAsync.state, 'cannot start async op with normal JS calling cwrap') }";
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["ccall"] = ccall;
Module["cwrap"] = cwrap;

function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module["setValue"] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module["getValue"] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module["ALLOC_NORMAL"] = ALLOC_NORMAL;
Module["ALLOC_STACK"] = ALLOC_STACK;
Module["ALLOC_STATIC"] = ALLOC_STATIC;
Module["ALLOC_DYNAMIC"] = ALLOC_DYNAMIC;
Module["ALLOC_NONE"] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module["allocate"] = allocate;

// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
function getMemory(size) {
  if (!staticSealed) return Runtime.staticAlloc(size);
  if ((typeof _sbrk !== 'undefined' && !_sbrk.called) || !runtimeInitialized) return Runtime.dynamicAlloc(size);
  return _malloc(size);
}
Module["getMemory"] = getMemory;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = 0;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    hasUtf |= t;
    if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (hasUtf < 128) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  return Module['UTF8ToString'](ptr);
}
Module["Pointer_stringify"] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
  var str = '';
  while (1) {
    var ch = HEAP8[((ptr++)>>0)];
    if (!ch) return str;
    str += String.fromCharCode(ch);
  }
}
Module["AsciiToString"] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
  return writeAsciiToMemory(str, outPtr, false);
}
Module["stringToAscii"] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
  var u0, u1, u2, u3, u4, u5;

  var str = '';
  while (1) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    u0 = u8Array[idx++];
    if (!u0) return str;
    if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
    u1 = u8Array[idx++] & 63;
    if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
    u2 = u8Array[idx++] & 63;
    if ((u0 & 0xF0) == 0xE0) {
      u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
    } else {
      u3 = u8Array[idx++] & 63;
      if ((u0 & 0xF8) == 0xF0) {
        u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
      } else {
        u4 = u8Array[idx++] & 63;
        if ((u0 & 0xFC) == 0xF8) {
          u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
        } else {
          u5 = u8Array[idx++] & 63;
          u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
        }
      }
    }
    if (u0 < 0x10000) {
      str += String.fromCharCode(u0);
    } else {
      var ch = u0 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    }
  }
}
Module["UTF8ArrayToString"] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
  return UTF8ArrayToString(HEAPU8,ptr);
}
Module["UTF8ToString"] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(str, outU8Array, outIdx, maxBytesToWrite) {
  if (!(maxBytesToWrite > 0)) // Parameter maxBytesToWrite is not optional. Negative values, 0, null, undefined and false each don't write out any bytes.
    return 0;

  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1; // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      if (outIdx >= endIdx) break;
      outU8Array[outIdx++] = u;
    } else if (u <= 0x7FF) {
      if (outIdx + 1 >= endIdx) break;
      outU8Array[outIdx++] = 0xC0 | (u >> 6);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0xFFFF) {
      if (outIdx + 2 >= endIdx) break;
      outU8Array[outIdx++] = 0xE0 | (u >> 12);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x1FFFFF) {
      if (outIdx + 3 >= endIdx) break;
      outU8Array[outIdx++] = 0xF0 | (u >> 18);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x3FFFFFF) {
      if (outIdx + 4 >= endIdx) break;
      outU8Array[outIdx++] = 0xF8 | (u >> 24);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else {
      if (outIdx + 5 >= endIdx) break;
      outU8Array[outIdx++] = 0xFC | (u >> 30);
      outU8Array[outIdx++] = 0x80 | ((u >> 24) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    }
  }
  // Null-terminate the pointer to the buffer.
  outU8Array[outIdx] = 0;
  return outIdx - startIdx;
}
Module["stringToUTF8Array"] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF8(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  return stringToUTF8Array(str, HEAPU8,outPtr, maxBytesToWrite);
}
Module["stringToUTF8"] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      ++len;
    } else if (u <= 0x7FF) {
      len += 2;
    } else if (u <= 0xFFFF) {
      len += 3;
    } else if (u <= 0x1FFFFF) {
      len += 4;
    } else if (u <= 0x3FFFFFF) {
      len += 5;
    } else {
      len += 6;
    }
  }
  return len;
}
Module["lengthBytesUTF8"] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module["UTF16ToString"] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF16(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 2) return 0;
  maxBytesToWrite -= 2; // Null terminator.
  var startPtr = outPtr;
  var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
  for (var i = 0; i < numCharsToWrite; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[((outPtr)>>1)]=codeUnit;
    outPtr += 2;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[((outPtr)>>1)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF16"] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
  return str.length*2;
}
Module["lengthBytesUTF16"] = lengthBytesUTF16;

function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module["UTF32ToString"] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
  assert(typeof maxBytesToWrite == 'number', 'stringToUTF32(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 4) return 0;
  var startPtr = outPtr;
  var endPtr = startPtr + maxBytesToWrite - 4;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++i);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[((outPtr)>>2)]=codeUnit;
    outPtr += 4;
    if (outPtr + 4 > endPtr) break;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[((outPtr)>>2)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF32"] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i);
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
    len += 4;
  }

  return len;
}
Module["lengthBytesUTF32"] = lengthBytesUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module["stackTrace"] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;

function alignMemoryPage(x) {
  if (x % 4096 > 0) {
    x += (4096 - (x % 4096));
  }
  return x;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk


function abortOnCannotGrowMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with  -s TOTAL_MEMORY=X  with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with  -s ALLOW_MEMORY_GROWTH=1  which adjusts the size at runtime but prevents some optimizations, (3) set Module.TOTAL_MEMORY to a higher value before the program runs, or if you want malloc to return NULL (0) instead of this abort, compile with  -s ABORTING_MALLOC=0 ');
}

function enlargeMemory() {
  abortOnCannotGrowMemory();
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be compliant with the asm.js spec (and given that TOTAL_STACK=' + TOTAL_STACK + ')');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer;



buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);


// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;


function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module["addOnPreRun"] = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module["addOnInit"] = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module["addOnPreMain"] = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module["addOnExit"] = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module["addOnPostRun"] = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
}
Module["intArrayFromString"] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module["intArrayToString"] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module["writeStringToMemory"] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[((buffer++)>>0)]=array[i];
  }
}
Module["writeArrayToMemory"] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; ++i) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
  }
  // Null-terminate the pointer to the HEAP.
  if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module["writeAsciiToMemory"] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}


// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
  x = x >>> 0;
  for (var i = 0; i < 32; i++) {
    if (x & (1 << (31 - i))) return i;
  }
  return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function getUniqueRunDependency(id) {
  var orig = id;
  while (1) {
    if (!runDependencyTracking[id]) return id;
    id = orig + Math.random();
  }
  return id;
}

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module["addRunDependency"] = addRunDependency;

function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module["removeRunDependency"] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data



var memoryInitializer = null;



// === Body ===

var ASM_CONSTS = [];




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 25104;
  /* global initializers */  __ATINIT__.push();
  

/* memory initializer */ allocate([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,169,241,210,77,98,80,63,205,204,204,204,204,204,236,63,154,153,153,153,153,153,185,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,63,0,0,0,0,0,0,240,191,0,0,0,0,0,0,240,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,240,191,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,178,53,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,241,53,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,54,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,236,54,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,80,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,140,55,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,210,56,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,99,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,156,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,195,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,1,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,51,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,101,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,51,58,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,156,57,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,161,58,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,106,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,194,60,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,9,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,72,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,72,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,9,61,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,172,61,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,66,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,144,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,197,63,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,241,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,249,63,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,63,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,133,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,203,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,173,65,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,66,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,125,66,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,128,67,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,31,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,145,68,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,200,68,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,255,255,255,255,1,0,0,0,255,255,255,255,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,220,70,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,128,63,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,187,73,0,0,203,73,0,0,223,73,0,0,248,73,0,0,19,74,0,0,41,74,0,0,67,74,0,0,88,74,0,0,103,74,0,0,119,74,0,0,138,74,0,0,156,74,0,0,185,74,0,0,213,74,0,0,119,74,0,0,231,74,0,0,1,75,0,0,25,75,0,0,43,75,0,0,61,75,0,0,86,75,0,0,100,75,0,0,125,75,0,0,155,75,0,0,177,75,0,0,201,75,0,0,225,75,0,0,241,75,0,0,2,76,0,0,20,76,0,0,41,76,0,0,68,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,158,76,0,0,161,76,0,0,163,76,0,0,166,76,0,0,168,76,0,0,172,76,0,0,144,1,0,0,175,76,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,108,50,0,0,108,50,0,0,220,50,0,0,76,51,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,4,0,0,0,243,91,0,0,0,4,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,4,0,0,0,235,91,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,4,0,0,0,227,87,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,84,65,82,84,0,40,47,44,2,65,116,32,105,116,101,114,97,116,101,2,44,105,53,44,52,120,44,2,102,61,32,2,44,49,112,44,100,49,50,46,53,44,52,120,44,2,124,112,114,111,106,32,103,124,61,32,2,44,49,112,44,100,49,50,46,53,41,0,40,47,47,44,2,73,84,69,82,65,84,73,79,78,32,2,44,105,53,41,0,40,47,44,2,32,83,105,110,103,117,108,97,114,32,116,114,105,97,110,103,117,108,97,114,32,115,121,115,116,101,109,32,100,101,116,101,99,116,101,100,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,47,44,2,32,78,111,110,112,111,115,105,116,105,118,101,32,100,101,102,105,110,105,116,101,110,101,115,115,32,105,110,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,47,44,2,32,66,97,100,32,100,105,114,101,99,116,105,111,110,32,105,110,32,116,104,101,32,108,105,110,101,32,115,101,97,114,99,104,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,40,2,32,32,121,115,61,2,44,49,112,44,101,49,48,46,51,44,2,32,32,45,103,115,61,2,44,49,112,44,101,49,48,46,51,44,2,32,66,70,71,83,32,117,112,100,97,116,101,32,83,75,73,80,80,69,68,2,41,0,40,47,44,2,32,78,111,110,112,111,115,105,116,105,118,101,32,100,101,102,105,110,105,116,101,110,101,115,115,32,105,110,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,116,59,2,44,47,44,2,32,32,32,114,101,102,114,101,115,104,32,116,104,101,32,108,98,102,103,115,32,109,101,109,111,114,121,32,97,110,100,32,114,101,115,116,97,114,116,32,116,104,101,32,105,116,101,114,97,116,105,111,110,46,2,41,0,0,0,0,101,0,45,45,45,0,69,82,82,79,82,0,70,71,95,76,78,0,78,69,87,95,88,0,70,71,95,83,84,0,83,84,79,80,0,67,80,85,0,70,71,95,83,84,65,82,84,0,67,79,78,86,69,82,71,69,78,67,69,58,32,78,79,82,77,95,79,70,95,80,82,79,74,69,67,84,69,68,95,71,82,65,68,73,69,78,84,95,60,61,95,80,71,84,79,76,0,65,66,78,79,82,77,65,76,95,84,69,82,77,73,78,65,84,73,79,78,95,73,78,95,76,78,83,82,67,72,0,82,69,83,84,65,82,84,95,70,82,79,77,95,76,78,83,82,67,72,0,67,79,78,86,69,82,71,69,78,67,69,58,32,82,69,76,95,82,69,68,85,67,84,73,79,78,95,79,70,95,70,95,60,61,95,70,65,67,84,82,42,69,80,83,77,67,72,0,40,47,44,2,65,116,32,88,48,32,2,44,105,57,44,2,32,118,97,114,105,97,98,108,101,115,32,97,114,101,32,101,120,97,99,116,108,121,32,97,116,32,116,104,101,32,98,111,117,110,100,115,2,41,0,84,104,101,32,105,110,105,116,105,97,108,32,88,32,105,115,32,105,110,102,101,97,115,105,98,108,101,46,32,32,82,101,115,116,97,114,116,32,119,105,116,104,32,105,116,115,32,112,114,111,106,101,99,116,105,111,110,46,0,84,104,105,115,32,112,114,111,98,108,101,109,32,105,115,32,117,110,99,111,110,115,116,114,97,105,110,101,100,46,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,32,67,65,85,67,72,89,32,101,110,116,101,114,101,100,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,41,0,40,2,67,97,117,99,104,121,32,88,32,61,32,32,2,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,40,47,44,2,80,105,101,99,101,32,32,32,32,2,44,105,51,44,2,32,45,45,102,49,44,32,102,50,32,97,116,32,115,116,97,114,116,32,112,111,105,110,116,32,2,44,49,112,44,50,40,49,120,44,100,49,49,46,52,41,41,0,40,2,68,105,115,116,97,110,99,101,32,116,111,32,116,104,101,32,110,101,120,116,32,98,114,101,97,107,32,112,111,105,110,116,32,61,32,32,2,44,49,112,44,100,49,49,46,52,41,0,40,2,68,105,115,116,97,110,99,101,32,116,111,32,116,104,101,32,115,116,97,116,105,111,110,97,114,121,32,112,111,105,110,116,32,61,32,32,2,44,49,112,44,100,49,49,46,52,41,0,40,2,80,105,101,99,101,32,32,32,32,2,44,105,51,44,2,32,45,45,102,49,44,32,102,50,32,97,116,32,115,116,97,114,116,32,112,111,105,110,116,32,2,44,49,112,44,50,40,49,120,44,100,49,49,46,52,41,41,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,32,101,120,105,116,32,67,65,85,67,72,89,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,83,117,98,103,110,111,114,109,32,61,32,48,46,32,32,71,67,80,32,61,32,88,46,0,84,104,101,114,101,32,97,114,101,32,0,32,32,98,114,101,97,107,112,111,105,110,116,115,32,0,86,97,114,105,97,98,108,101,32,32,0,32,32,105,115,32,102,105,120,101,100,46,0,71,67,80,32,102,111,117,110,100,32,105,110,32,116,104,105,115,32,115,101,103,109,101,110,116,0,69,82,82,79,82,58,32,78,32,46,76,69,46,32,48,0,69,82,82,79,82,58,32,77,32,46,76,69,46,32,48,0,69,82,82,79,82,58,32,70,65,67,84,82,32,46,76,84,46,32,48,0,69,82,82,79,82,58,32,73,78,86,65,76,73,68,32,78,66,68,0,69,82,82,79,82,58,32,78,79,32,70,69,65,83,73,66,76,69,32,83,79,76,85,84,73,79,78,0,86,97,114,105,97,98,108,101,32,0,32,108,101,97,118,101,115,32,116,104,101,32,115,101,116,32,111,102,32,102,114,101,101,32,118,97,114,105,97,98,108,101,115,0,32,101,110,116,101,114,115,32,116,104,101,32,115,101,116,32,111,102,32,102,114,101,101,32,118,97,114,105,97,98,108,101,115,0,32,118,97,114,105,97,98,108,101,115,32,108,101,97,118,101,59,32,0,32,118,97,114,105,97,98,108,101,115,32,101,110,116,101,114,0,32,118,97,114,105,97,98,108,101,115,32,97,114,101,32,102,114,101,101,32,97,116,32,71,67,80,32,0,32,97,115,99,101,110,116,32,100,105,114,101,99,116,105,111,110,32,105,110,32,112,114,111,106,101,99,116,105,111,110,32,103,100,32,61,32,0,67,79,78,86,0,87,65,82,78,0,70,71,95,76,78,83,82,67,72,0,40,2,82,85,78,78,73,78,71,32,84,72,69,32,76,45,66,70,71,83,45,66,32,67,79,68,69,2,44,47,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,44,47,44,47,44,2,77,97,99,104,105,110,101,32,112,114,101,99,105,115,105,111,110,32,61,2,44,49,112,44,100,49,48,46,51,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,78,32,61,32,0,32,32,32,32,77,32,61,32,0,76,32,61,0,88,48,32,61,0,85,32,61,0,40,47,44,2,65,116,32,105,116,101,114,97,116,101,2,44,105,53,44,52,120,44,2,102,61,32,2,44,49,112,44,100,49,50,46,53,44,52,120,44,2,124,112,114,111,106,32,103,124,61,32,2,44,49,112,44,100,49,50,46,53,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,99,111,110,0,98,110,100,0,84,78,84,0,76,73,78,69,32,83,69,65,82,67,72,0,32,116,105,109,101,115,59,32,110,111,114,109,32,111,102,32,115,116,101,112,32,61,32,0,88,32,61,0,71,32,61,0,40,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,44,47,44,47,44,2,84,105,116,32,32,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,105,116,101,114,97,116,105,111,110,115,2,44,47,44,2,84,110,102,32,32,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,102,117,110,99,116,105,111,110,32,101,118,97,108,117,97,116,105,111,110,115,2,44,47,44,2,84,110,105,110,116,32,61,32,116,111,116,97,108,32,110,117,109,98,101,114,32,111,102,32,115,101,103,109,101,110,116,115,32,101,120,112,108,111,114,101,100,32,100,117,114,105,110,103,2,44,2,32,67,97,117,99,104,121,32,115,101,97,114,99,104,101,115,2,44,47,44,2,83,107,105,112,32,32,61,32,110,117,109,98,101,114,32,111,102,32,66,70,71,83,32,117,112,100,97,116,101,115,32,115,107,105,112,112,101,100,2,44,47,44,2,78,97,99,116,32,32,61,32,110,117,109,98,101,114,32,111,102,32,97,99,116,105,118,101,32,98,111,117,110,100,115,32,97,116,32,102,105,110,97,108,32,103,101,110,101,114,97,108,105,122,101,100,2,44,2,32,67,97,117,99,104,121,32,112,111,105,110,116,2,44,47,44,2,80,114,111,106,103,32,61,32,110,111,114,109,32,111,102,32,116,104,101,32,102,105,110,97,108,32,112,114,111,106,101,99,116,101,100,32,103,114,97,100,105,101,110,116,2,44,47,44,2,70,32,32,32,32,32,61,32,102,105,110,97,108,32,102,117,110,99,116,105,111,110,32,118,97,108,117,101,2,44,47,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,42,32,42,32,42,2,41,0,40,47,44,51,120,44,2,78,2,44,52,120,44,2,84,105,116,2,44,53,120,44,2,84,110,102,2,44,50,120,44,2,84,110,105,110,116,2,44,50,120,44,2,83,107,105,112,2,44,50,120,44,2,78,97,99,116,2,44,53,120,44,2,80,114,111,106,103,2,44,56,120,44,2,70,2,41,0,40,105,53,44,50,40,49,120,44,105,54,41,44,40,49,120,44,105,54,41,44,40,50,120,44,105,52,41,44,40,49,120,44,105,53,41,44,49,112,44,50,40,50,120,44,100,49,48,46,51,41,41,0,40,47,44,97,52,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,44,47,44,40,52,120,44,49,112,44,54,40,49,120,44,100,49,49,46,52,41,41,41,0,40,47,44,97,54,48,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,49,115,116,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,50,115,116,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,107,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,77,97,116,114,105,120,32,105,110,32,116,104,101,32,67,104,111,108,101,115,107,121,32,102,97,99,116,111,114,105,122,97,116,105,111,110,32,105,110,32,102,111,114,109,116,32,105,115,32,110,111,116,32,80,111,115,46,32,68,101,102,46,2,41,0,40,47,44,2,32,68,101,114,105,118,97,116,105,118,101,32,62,61,32,48,44,32,98,97,99,107,116,114,97,99,107,105,110,103,32,108,105,110,101,32,115,101,97,114,99,104,32,105,109,112,111,115,115,105,98,108,101,46,2,44,47,44,2,32,32,32,80,114,101,118,105,111,117,115,32,120,44,32,102,32,97,110,100,32,103,32,114,101,115,116,111,114,101,100,46,2,44,47,44,2,32,80,111,115,115,105,98,108,101,32,99,97,117,115,101,115,58,32,49,32,101,114,114,111,114,32,105,110,32,102,117,110,99,116,105,111,110,32,111,114,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,59,2,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,50,32,114,111,117,110,100,105,110,103,32,101,114,114,111,114,115,32,100,111,109,105,110,97,116,101,32,99,111,109,112,117,116,97,116,105,111,110,46,2,41,0,40,47,44,2,32,87,97,114,110,105,110,103,58,32,32,109,111,114,101,32,116,104,97,110,32,49,48,32,102,117,110,99,116,105,111,110,32,97,110,100,32,103,114,97,100,105,101,110,116,2,44,47,44,2,32,32,32,101,118,97,108,117,97,116,105,111,110,115,32,105,110,32,116,104,101,32,108,97,115,116,32,108,105,110,101,32,115,101,97,114,99,104,46,32,32,84,101,114,109,105,110,97,116,105,111,110,2,44,47,44,2,32,32,32,109,97,121,32,112,111,115,115,105,98,108,121,32,98,101,32,99,97,117,115,101,100,32,98,121,32,97,32,98,97,100,32,115,101,97,114,99,104,32,100,105,114,101,99,116,105,111,110,46,2,41,0,40,47,44,2,32,84,104,101,32,116,114,105,97,110,103,117,108,97,114,32,115,121,115,116,101,109,32,105,115,32,115,105,110,103,117,108,97,114,46,2,41,0,40,47,44,2,32,76,105,110,101,32,115,101,97,114,99,104,32,99,97,110,110,111,116,32,108,111,99,97,116,101,32,97,110,32,97,100,101,113,117,97,116,101,32,112,111,105,110,116,32,97,102,116,101,114,32,50,48,32,102,117,110,99,116,105,111,110,2,44,47,44,2,32,32,97,110,100,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,115,46,32,32,80,114,101,118,105,111,117,115,32,120,44,32,102,32,97,110,100,32,103,32,114,101,115,116,111,114,101,100,46,2,44,47,44,2,32,80,111,115,115,105,98,108,101,32,99,97,117,115,101,115,58,32,49,32,101,114,114,111,114,32,105,110,32,102,117,110,99,116,105,111,110,32,111,114,32,103,114,97,100,105,101,110,116,32,101,118,97,108,117,97,116,105,111,110,59,2,44,47,44,2,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,50,32,114,111,117,110,100,105,110,103,32,101,114,114,111,114,32,100,111,109,105,110,97,116,101,32,99,111,109,112,117,116,97,116,105,111,110,46,2,41,0,40,47,44,2,32,67,97,117,99,104,121,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,2,32,83,117,98,115,112,97,99,101,32,109,105,110,105,109,105,122,97,116,105,111,110,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,2,32,76,105,110,101,32,115,101,97,114,99,104,32,32,32,32,32,32,32,32,32,32,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,41,0,40,47,44,2,32,84,111,116,97,108,32,85,115,101,114,32,116,105,109,101,2,44,49,112,44,101,49,48,46,51,44,2,32,115,101,99,111,110,100,115,46,2,44,47,41,0,32,70,32,61,0,32,73,110,112,117,116,32,110,98,100,40,0,41,32,105,115,32,105,110,118,97,108,105,100,46,0,32,108,40,0,41,32,62,32,117,40,0,41,46,32,32,78,111,32,102,101,97,115,105,98,108,101,32,115,111,108,117,116,105,111,110,46,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,83,85,66,83,77,32,101,110,116,101,114,101,100,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,40,47,44,2,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,101,120,105,116,32,83,85,66,83,77,32,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,2,44,47,41,0,32,80,111,115,105,116,105,118,101,32,100,105,114,32,100,101,114,105,118,97,116,105,118,101,32,105,110,32,112,114,111,106,101,99,116,105,111,110,32,0,32,85,115,105,110,103,32,116,104,101,32,98,97,99,107,116,114,97,99,107,105,110,103,32,115,116,101,112,32,0,69,82,82,79,82,58,32,83,84,80,32,46,76,84,46,32,83,84,80,77,73,78,0,69,82,82,79,82,58,32,83,84,80,32,46,71,84,46,32,83,84,80,77,65,88,0,69,82,82,79,82,58,32,73,78,73,84,73,65,76,32,71,32,46,71,69,46,32,90,69,82,79,0,69,82,82,79,82,58,32,70,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,71,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,88,84,79,76,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,83,84,80,77,73,78,32,46,76,84,46,32,90,69,82,79,0,69,82,82,79,82,58,32,83,84,80,77,65,88,32,46,76,84,46,32,83,84,80,77,73,78,0,70,71,0,87,65,82,78,73,78,71,58,32,82,79,85,78,68,73,78,71,32,69,82,82,79,82,83,32,80,82,69,86,69,78,84,32,80,82,79,71,82,69,83,83,0,87,65,82,78,73,78,71,58,32,88,84,79,76,32,84,69,83,84,32,83,65,84,73,83,70,73,69,68,0,87,65,82,78,73,78,71,58,32,83,84,80,32,61,32,83,84,80,77,65,88,0,87,65,82,78,73,78,71,58,32,83,84,80,32,61,32,83,84,80,77,73,78,0,67,79,78,86,69,82,71,69,78,67,69,0,68,71,69,83,86,32,0,68,71,69,84,82,70,0,32,0,68,71,69,84,82,83,0,76,101,102,116,0,76,111,119,101,114,0,78,111,32,116,114,97,110,115,112,111,115,101,0,85,110,105,116,0,85,112,112,101,114,0,78,111,110,45,117,110,105,116,0,84,114,97,110,115,112,111,115,101,0,69,0,66,0,80,0,77,0,79,0,40,47,47,2,32,87,65,82,78,73,78,71,46,32,84,104,101,32,118,97,108,117,101,32,69,77,73,78,32,109,97,121,32,98,101,32,105,110,99,111,114,114,101,99,116,58,45,2,44,2,32,32,69,77,73,78,32,61,32,2,44,105,56,44,47,2,32,73,102,44,32,97,102,116,101,114,32,105,110,115,112,101,99,116,105,111,110,44,32,116,104,101,32,118,97,108,117,101,32,69,77,73,78,32,108,111,111,107,115,2,44,2,32,97,99,99,101,112,116,97,98,108,101,32,112,108,101,97,115,101,32,99,111,109,109,101,110,116,32,111,117,116,32,2,44,47,2,32,116,104,101,32,73,70,32,98,108,111,99,107,32,97,115,32,109,97,114,107,101,100,32,119,105,116,104,105,110,32,116,104,101,32,99,111,100,101,32,111,102,32,114,111,117,116,105,110,101,2,44,2,32,68,76,65,77,67,50,44,2,44,47,2,32,111,116,104,101,114,119,105,115,101,32,115,117,112,112,108,121,32,69,77,73,78,32,101,120,112,108,105,99,105,116,108,121,46,2,44,47,41,0,71,69,0,84,82,70,0,81,82,70,0,82,81,70,0,76,81,70,0,81,76,70,0,72,82,68,0,66,82,68,0,84,82,73,0,80,79,0,83,89,0,84,82,68,0,71,83,84,0,72,69,0,79,82,0,81,82,0,82,81,0,76,81,0,81,76,0,72,82,0,84,82,0,66,82,0,85,78,0,71,66,0,80,66,0,76,65,0,85,85,77,0,83,84,0,69,66,90,0,68,71,69,84,70,50,0,83,0,117,110,107,110,111,119,110,32,99,111,100,101,32,105,110,32,100,111,95,102,105,111,58,32,37,100,10,37,115,10,0,100,111,95,102,105,111,0,102,109,116,0,98,97,100,32,115,116,114,105,110,103,0,102,111,114,109,97,116,32,116,111,111,32,99,111,109,112,108,105,99,97,116,101,100,58,10,0,115,102,101,0,115,116,97,114,116,105,111,0,119,114,105,116,101,32,115,116,97,114,116,0,108,105,115,116,32,111,117,116,112,117,116,32,115,116,97,114,116,0,37,115,10,0,101,110,100,102,105,108,101,0,37,115,58,32,105,108,108,101,103,97,108,32,101,114,114,111,114,32,110,117,109,98,101,114,32,37,100,10,0,37,115,58,32,101,110,100,32,111,102,32,102,105,108,101,10,0,37,115,58,32,37,115,10,0,97,112,112,97,114,101,110,116,32,115,116,97,116,101,58,32,117,110,105,116,32,37,100,32,0,110,97,109,101,100,32,37,115,10,0,40,117,110,110,97,109,101,100,41,10,0,97,112,112,97,114,101,110,116,32,115,116,97,116,101,58,32,105,110,116,101,114,110,97,108,32,73,47,79,10,0,108,97,115,116,32,102,111,114,109,97,116,58,32,37,115,10,0,114,101,97,100,105,110,103,0,119,114,105,116,105,110,103,0,115,101,113,117,101,110,116,105,97,108,0,100,105,114,101,99,116,0,102,111,114,109,97,116,116,101,100,0,117,110,102,111,114,109,97,116,116,101,100,0,101,120,116,101,114,110,97,108,0,105,110,116,101,114,110,97,108,0,108,97,116,101,108,121,32,37,115,32,37,115,32,37,115,32,37,115,0,32,73,79,0,101,114,114,111,114,32,105,110,32,102,111,114,109,97,116,0,105,108,108,101,103,97,108,32,117,110,105,116,32,110,117,109,98,101,114,0,102,111,114,109,97,116,116,101,100,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,117,110,102,111,114,109,97,116,116,101,100,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,100,105,114,101,99,116,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,115,101,113,117,101,110,116,105,97,108,32,105,111,32,110,111,116,32,97,108,108,111,119,101,100,0,99,97,110,39,116,32,98,97,99,107,115,112,97,99,101,32,102,105,108,101,0,110,117,108,108,32,102,105,108,101,32,110,97,109,101,0,99,97,110,39,116,32,115,116,97,116,32,102,105,108,101,0,117,110,105,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,111,102,102,32,101,110,100,32,111,102,32,114,101,99,111,114,100,0,116,114,117,110,99,97,116,105,111,110,32,102,97,105,108,101,100,32,105,110,32,101,110,100,102,105,108,101,0,105,110,99,111,109,112,114,101,104,101,110,115,105,98,108,101,32,108,105,115,116,32,105,110,112,117,116,0,111,117,116,32,111,102,32,102,114,101,101,32,115,112,97,99,101,0,114,101,97,100,32,117,110,101,120,112,101,99,116,101,100,32,99,104,97,114,97,99,116,101,114,0,98,97,100,32,108,111,103,105,99,97,108,32,105,110,112,117,116,32,102,105,101,108,100,0,98,97,100,32,118,97,114,105,97,98,108,101,32,116,121,112,101,0,98,97,100,32,110,97,109,101,108,105,115,116,32,110,97,109,101,0,118,97,114,105,97,98,108,101,32,110,111,116,32,105,110,32,110,97,109,101,108,105,115,116,0,110,111,32,101,110,100,32,114,101,99,111,114,100,0,118,97,114,105,97,98,108,101,32,99,111,117,110,116,32,105,110,99,111,114,114,101,99,116,0,115,117,98,115,99,114,105,112,116,32,102,111,114,32,115,99,97,108,97,114,32,118,97,114,105,97,98,108,101,0,105,110,118,97,108,105,100,32,97,114,114,97,121,32,115,101,99,116,105,111,110,0,115,117,98,115,116,114,105,110,103,32,111,117,116,32,111,102,32,98,111,117,110,100,115,0,115,117,98,115,99,114,105,112,116,32,111,117,116,32,111,102,32,98,111,117,110,100,115,0,99,97,110,39,116,32,114,101,97,100,32,102,105,108,101,0,99,97,110,39,116,32,119,114,105,116,101,32,102,105,108,101,0,39,110,101,119,39,32,102,105,108,101,32,101,120,105,115,116,115,0,99,97,110,39,116,32,97,112,112,101,110,100,32,116,111,32,102,105,108,101,0,110,111,110,45,112,111,115,105,116,105,118,101,32,114,101,99,111,114,100,32,110,117,109,98,101,114,0,110,109,76,98,117,102,32,111,118,101,114,102,108,111,119,0,108,105,115,116,32,105,111,0,115,116,108,101,114,0,108,105,111,0,117,110,107,110,111,119,110,32,116,121,112,101,32,105,110,32,108,105,111,0,37,46,57,71,0,111,112,101,110,0,102,111,114,116,46,37,108,100,0,110,111,32,115,112,97,99,101,0,115,0,100,0,102,0,117,0,114,98,0,114,0,119,98,0,119,0,114,43,98,0,114,43,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,109,97,108,108,111,99,32,102,97,105,108,117,114,101,0,119,95,101,100,44,32,117,110,101,120,112,101,99,116,101,100,32,99,111,100,101,58,32,37,100,10,0,119,95,110,101,100,44,32,117,110,101,120,112,101,99,116,101,100,32,99,111,100,101,58,32,37,100,10,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,0,108,101,102,116,32,111,102,102,0,114,101,99,101,110,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,37,35,46,42,69,0,37,43,46,50,100,0,43,48,48,0,37,35,46,42,102,0,68,71,69,82,32,32,0,68,71,69,77,77,32,0,76,0,78,0,85,0,82,0,84,0,67,0,68,84,82,83,77,32,0,42,42,32,79,110,32,101,110,116,114,121,32,116,111,32,37,54,115,44,32,112,97,114,97,109,101,116,101,114,32,110,117,109,98,101,114,32,37,50,105,32,104,97,100,32,97,110,32,105,108,108,101,103,97,108,32,118,97,108,117,101,10,0,119,43,0,114,119,97,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,47,116,109,112,0,47,116,109,112,47,116,37,120,45,37,120,0,84,33,34,25,13,1,2,3,17,75,28,12,16,4,11,29,18,30,39,104,110,111,112,113,98,32,5,6,15,19,20,21,26,8,22,7,40,36,23,24,9,10,14,27,31,37,35,131,130,125,38,42,43,60,61,62,63,67,71,74,77,88,89,90,91,92,93,94,95,96,97,99,100,101,102,103,105,106,107,108,114,115,116,121,122,123,124,0,73,108,108,101,103,97,108,32,98,121,116,101,32,115,101,113,117,101,110,99,101,0,68,111,109,97,105,110,32,101,114,114,111,114,0,82,101,115,117,108,116,32,110,111,116,32,114,101,112,114,101,115,101,110,116,97,98,108,101,0,78,111,116,32,97,32,116,116,121,0,80,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,79,112,101,114,97,116,105,111,110,32,110,111,116,32,112,101,114,109,105,116,116,101,100,0,78,111,32,115,117,99,104,32,102,105,108,101,32,111,114,32,100,105,114,101,99,116,111,114,121,0,78,111,32,115,117,99,104,32,112,114,111,99,101,115,115,0,70,105,108,101,32,101,120,105,115,116,115,0,86,97,108,117,101,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,100,97,116,97,32,116,121,112,101,0,78,111,32,115,112,97,99,101,32,108,101,102,116,32,111,110,32,100,101,118,105,99,101,0,79,117,116,32,111,102,32,109,101,109,111,114,121,0,82,101,115,111,117,114,99,101,32,98,117,115,121,0,73,110,116,101,114,114,117,112,116,101,100,32,115,121,115,116,101,109,32,99,97,108,108,0,82,101,115,111,117,114,99,101,32,116,101,109,112,111,114,97,114,105,108,121,32,117,110,97,118,97,105,108,97,98,108,101,0,73,110,118,97,108,105,100,32,115,101,101,107,0,67,114,111,115,115,45,100,101,118,105,99,101,32,108,105,110,107,0,82,101,97,100,45,111,110,108,121,32,102,105,108,101,32,115,121,115,116,101,109,0,68,105,114,101,99,116,111,114,121,32,110,111,116,32,101,109,112,116,121,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,112,101,101,114,0,79,112,101,114,97,116,105,111,110,32,116,105,109,101,100,32,111,117,116,0,67,111,110,110,101,99,116,105,111,110,32,114,101,102,117,115,101,100,0,72,111,115,116,32,105,115,32,100,111,119,110,0,72,111,115,116,32,105,115,32,117,110,114,101,97,99,104,97,98,108,101,0,65,100,100,114,101,115,115,32,105,110,32,117,115,101,0,66,114,111,107,101,110,32,112,105,112,101,0,73,47,79,32,101,114,114,111,114,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,32,111,114,32,97,100,100,114,101,115,115,0,66,108,111,99,107,32,100,101,118,105,99,101,32,114,101,113,117,105,114,101,100,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,0,78,111,116,32,97,32,100,105,114,101,99,116,111,114,121,0,73,115,32,97,32,100,105,114,101,99,116,111,114,121,0,84,101,120,116,32,102,105,108,101,32,98,117,115,121,0,69,120,101,99,32,102,111,114,109,97,116,32,101,114,114,111,114,0,73,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,0,65,114,103,117,109,101,110,116,32,108,105,115,116,32,116,111,111,32,108,111,110,103,0,83,121,109,98,111,108,105,99,32,108,105,110,107,32,108,111,111,112,0,70,105,108,101,110,97,109,101,32,116,111,111,32,108,111,110,103,0,84,111,111,32,109,97,110,121,32,111,112,101,110,32,102,105,108,101,115,32,105,110,32,115,121,115,116,101,109,0,78,111,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,115,32,97,118,97,105,108,97,98,108,101,0,66,97,100,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,0,78,111,32,99,104,105,108,100,32,112,114,111,99,101,115,115,0,66,97,100,32,97,100,100,114,101,115,115,0,70,105,108,101,32,116,111,111,32,108,97,114,103,101,0,84,111,111,32,109,97,110,121,32,108,105,110,107,115,0,78,111,32,108,111,99,107,115,32,97,118,97,105,108,97,98,108,101,0,82,101,115,111,117,114,99,101,32,100,101,97,100,108,111,99,107,32,119,111,117,108,100,32,111,99,99,117,114,0,83,116,97,116,101,32,110,111,116,32,114,101,99,111,118,101,114,97,98,108,101,0,80,114,101,118,105,111,117,115,32,111,119,110,101,114,32,100,105,101,100,0,79,112,101,114,97,116,105,111,110,32,99,97,110,99,101,108,101,100,0,70,117,110,99,116,105,111,110,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,0,78,111,32,109,101,115,115,97,103,101,32,111,102,32,100,101,115,105,114,101,100,32,116,121,112,101,0,73,100,101,110,116,105,102,105,101,114,32,114,101,109,111,118,101,100,0,68,101,118,105,99,101,32,110,111,116,32,97,32,115,116,114,101,97,109,0,78,111,32,100,97,116,97,32,97,118,97,105,108,97,98,108,101,0,68,101,118,105,99,101,32,116,105,109,101,111,117,116,0,79,117,116,32,111,102,32,115,116,114,101,97,109,115,32,114,101,115,111,117,114,99,101,115,0,76,105,110,107,32,104,97,115,32,98,101,101,110,32,115,101,118,101,114,101,100,0,80,114,111,116,111,99,111,108,32,101,114,114,111,114,0,66,97,100,32,109,101,115,115,97,103,101,0,70,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,105,110,32,98,97,100,32,115,116,97,116,101,0,78,111,116,32,97,32,115,111,99,107,101,116,0,68,101,115,116,105,110,97,116,105,111,110,32,97,100,100,114,101,115,115,32,114,101,113,117,105,114,101,100,0,77,101,115,115,97,103,101,32,116,111,111,32,108,97,114,103,101,0,80,114,111,116,111,99,111,108,32,119,114,111,110,103,32,116,121,112,101,32,102,111,114,32,115,111,99,107,101,116,0,80,114,111,116,111,99,111,108,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,80,114,111,116,111,99,111,108,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,83,111,99,107,101,116,32,116,121,112,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,78,111,116,32,115,117,112,112,111,114,116,101,100,0,80,114,111,116,111,99,111,108,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,65,100,100,114,101,115,115,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,112,114,111,116,111,99,111,108,0,65,100,100,114,101,115,115,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,78,101,116,119,111,114,107,32,105,115,32,100,111,119,110,0,78,101,116,119,111,114,107,32,117,110,114,101,97,99,104,97,98,108,101,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,110,101,116,119,111,114,107,0,67,111,110,110,101,99,116,105,111,110,32,97,98,111,114,116,101,100,0,78,111,32,98,117,102,102,101,114,32,115,112,97,99,101,32,97,118,97,105,108,97,98,108,101,0,83,111,99,107,101,116,32,105,115,32,99,111,110,110,101,99,116,101,100,0,83,111,99,107,101,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,67,97,110,110,111,116,32,115,101,110,100,32,97,102,116,101,114,32,115,111,99,107,101,116,32,115,104,117,116,100,111,119,110,0,79,112,101,114,97,116,105,111,110,32,97,108,114,101,97,100,121,32,105,110,32,112,114,111,103,114,101,115,115,0,79,112,101,114,97,116,105,111,110,32,105,110,32,112,114,111,103,114,101,115,115,0,83,116,97,108,101,32,102,105,108,101,32,104,97,110,100,108,101,0,82,101,109,111,116,101,32,73,47,79,32,101,114,114,111,114,0,81,117,111,116,97,32,101,120,99,101,101,100,101,100,0,78,111,32,109,101,100,105,117,109,32,102,111,117,110,100,0,87,114,111,110,103,32,109,101,100,105,117,109,32,116,121,112,101,0,78,111,32,101,114,114,111,114,32,105,110,102,111,114,109,97,116,105,111,110,0,0,47,112,114,111,99,47,115,101,108,102,47,102,100,47,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,45,43,32,32,32,48,88,48,120,0,40,110,117,108,108,41,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,105,110,102,0,73,78,70,0,110,97,110,0,78,65,78,0,46,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);





/* no memory initializer */
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}

// {{PRE_LIBRARY}}


   
  Module["_i64Subtract"] = _i64Subtract;

  
  function ___setErrNo(value) {
      if (Module['___errno_location']) HEAP32[((Module['___errno_location']())>>2)]=value;
      else Module.printErr('failed to set errno from JS');
      return value;
    }
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 85: return totalMemory / PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 79:
          return 0;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              // we will read data by chunks of BUFSIZE
              var BUFSIZE = 256;
              var buf = new Buffer(BUFSIZE);
              var bytesRead = 0;
  
              var fd = process.stdin.fd;
              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
              var usingDevice = false;
              try {
                fd = fs.openSync('/dev/stdin', 'r');
                usingDevice = true;
              } catch (e) {}
  
              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
  
              if (usingDevice) { fs.closeSync(fd); }
              if (bytesRead > 0) {
                result = buf.slice(0, bytesRead).toString('utf-8');
              } else {
                result = null;
              }
  
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val);
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap,
                msync: MEMFS.stream_ops.msync
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
  
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else {
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        },msync:function (stream, buffer, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          if (mmapFlags & 2) {
            // MAP_PRIVATE calls need not to be synced back to underlying fs
            return 0;
          }
  
          var bytesWritten = MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
          // should we check if bytesWritten and length are the same?
          return 0;
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          if (!fileStore.indexNames.contains('timestamp')) {
            fileStore.createIndex('timestamp', 'timestamp', { unique: false });
          }
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function(e) {
            callback(this.error);
            e.preventDefault();
          };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function(e) {
          done(this.error);
          e.preventDefault();
        };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        flags &= ~0100000 /*O_LARGEFILE*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            path = fs.readlinkSync(path);
            path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
            return path;
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var WORKERFS={DIR_MODE:16895,FILE_MODE:33279,reader:null,mount:function (mount) {
        assert(ENVIRONMENT_IS_WORKER);
        if (!WORKERFS.reader) WORKERFS.reader = new FileReaderSync();
        var root = WORKERFS.createNode(null, '/', WORKERFS.DIR_MODE, 0);
        var createdParents = {};
        function ensureParent(path) {
          // return the parent node, creating subdirs as necessary
          var parts = path.split('/');
          var parent = root;
          for (var i = 0; i < parts.length-1; i++) {
            var curr = parts.slice(0, i+1).join('/');
            if (!createdParents[curr]) {
              createdParents[curr] = WORKERFS.createNode(parent, curr, WORKERFS.DIR_MODE, 0);
            }
            parent = createdParents[curr];
          }
          return parent;
        }
        function base(path) {
          var parts = path.split('/');
          return parts[parts.length-1];
        }
        // We also accept FileList here, by using Array.prototype
        Array.prototype.forEach.call(mount.opts["files"] || [], function(file) {
          WORKERFS.createNode(ensureParent(file.name), base(file.name), WORKERFS.FILE_MODE, 0, file, file.lastModifiedDate);
        });
        (mount.opts["blobs"] || []).forEach(function(obj) {
          WORKERFS.createNode(ensureParent(obj["name"]), base(obj["name"]), WORKERFS.FILE_MODE, 0, obj["data"]);
        });
        (mount.opts["packages"] || []).forEach(function(pack) {
          pack['metadata'].files.forEach(function(file) {
            var name = file.filename.substr(1); // remove initial slash
            WORKERFS.createNode(ensureParent(name), base(name), WORKERFS.FILE_MODE, 0, pack['blob'].slice(file.start, file.end));
          });
        });
        return root;
      },createNode:function (parent, name, mode, dev, contents, mtime) {
        var node = FS.createNode(parent, name, mode);
        node.mode = mode;
        node.node_ops = WORKERFS.node_ops;
        node.stream_ops = WORKERFS.stream_ops;
        node.timestamp = (mtime || new Date).getTime();
        assert(WORKERFS.FILE_MODE !== WORKERFS.DIR_MODE);
        if (mode === WORKERFS.FILE_MODE) {
          node.size = contents.size;
          node.contents = contents;
        } else {
          node.size = 4096;
          node.contents = {};
        }
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },node_ops:{getattr:function (node) {
          return {
            dev: 1,
            ino: undefined,
            mode: node.mode,
            nlink: 1,
            uid: 0,
            gid: 0,
            rdev: undefined,
            size: node.size,
            atime: new Date(node.timestamp),
            mtime: new Date(node.timestamp),
            ctime: new Date(node.timestamp),
            blksize: 4096,
            blocks: Math.ceil(node.size / 4096),
          };
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
        },lookup:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        },mknod:function (parent, name, mode, dev) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rename:function (oldNode, newDir, newName) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },unlink:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rmdir:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readdir:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },symlink:function (parent, newName, oldPath) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readlink:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          if (position >= stream.node.size) return 0;
          var chunk = stream.node.contents.slice(position, position + length);
          var ab = WORKERFS.reader.readAsArrayBuffer(chunk);
          buffer.set(new Uint8Array(ab), offset);
          return chunk.size;
        },write:function (stream, buffer, offset, length, position) {
          throw new FS.ErrnoError(ERRNO_CODES.EIO);
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.size;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},filesystems:null,handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
  
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var perms = ['r', 'w', 'rw'][flag & 3];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return PATH.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // if asked only for a directory, then this must be one
        if ((flags & 65536) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        if (stream.getdents) stream.getdents = null; // free readdir state
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },msync:function (stream, buffer, offset, length, mmapFlags) {
        if (!stream || !stream.stream_ops.msync) {
          return 0;
        }
        return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
      },munmap:function (stream) {
        return 0;
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var buf = new Uint8Array(lengthBytesUTF8(data)+1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function(stream, buffer, offset, length, pos) { return length; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createSpecialDirectories:function () {
        // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the name of the stream for fd 6 (see test_unistd_ttyname)
        FS.mkdir('/proc');
        FS.mkdir('/proc/self');
        FS.mkdir('/proc/self/fd');
        FS.mount({
          mount: function() {
            var node = FS.createNode('/proc/self', 'fd', 16384 | 0777, 73);
            node.node_ops = {
              lookup: function(parent, name) {
                var fd = +name;
                var stream = FS.getStream(fd);
                if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
                var ret = {
                  parent: null,
                  mount: { mountpoint: 'fake' },
                  node_ops: { readlink: function() { return stream.path } }
                };
                ret.parent = ret; // make it look like a simple root node
                return ret;
              }
            };
            return node;
          }
        }, {}, '/proc/self/fd');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          //Module.printErr(stackTrace()); // useful for debugging
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
        FS.createSpecialDirectories();
  
        FS.filesystems = {
          'MEMFS': MEMFS,
          'IDBFS': IDBFS,
          'NODEFS': NODEFS,
          'WORKERFS': WORKERFS,
        };
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        // force-flush all streams, so we get musl std streams printed out
        var fflush = Module['_fflush'];
        if (fflush) fflush(0);
        // close all of our streams
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
        function processData(byteArray) {
          function finish(byteArray) {
            if (preFinish) preFinish();
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency(dep);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency(dep);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency(dep);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};var SYSCALLS={DEFAULT_POLLMASK:5,mappings:{},umask:511,calculateAt:function (dirfd, path) {
        if (path[0] !== '/') {
          // relative path
          var dir;
          if (dirfd === -100) {
            dir = FS.cwd();
          } else {
            var dirstream = FS.getStream(dirfd);
            if (!dirstream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
            dir = dirstream.path;
          }
          path = PATH.join2(dir, path);
        }
        return path;
      },doStat:function (func, path, buf) {
        try {
          var stat = func(path);
        } catch (e) {
          if (e && e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
            // an error occurred while trying to look up the path; we should just report ENOTDIR
            return -ERRNO_CODES.ENOTDIR;
          }
          throw e;
        }
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      },doMsync:function (addr, stream, len, flags) {
        var buffer = new Uint8Array(HEAPU8.subarray(addr, addr + len));
        FS.msync(stream, buffer, 0, len, flags);
      },doMkdir:function (path, mode) {
        // remove a trailing slash, if one - /a/b/ has basename of '', but
        // we want to create b in the context of this function
        path = PATH.normalize(path);
        if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
        FS.mkdir(path, mode, 0);
        return 0;
      },doMknod:function (path, mode, dev) {
        // we don't want this in the JS API as it uses mknod to create all nodes.
        switch (mode & 61440) {
          case 32768:
          case 8192:
          case 24576:
          case 4096:
          case 49152:
            break;
          default: return -ERRNO_CODES.EINVAL;
        }
        FS.mknod(path, mode, dev);
        return 0;
      },doReadlink:function (path, buf, bufsize) {
        if (bufsize <= 0) return -ERRNO_CODES.EINVAL;
        var ret = FS.readlink(path);
        ret = ret.slice(0, Math.max(0, bufsize));
        writeStringToMemory(ret, buf, true);
        return ret.length;
      },doAccess:function (path, amode) {
        if (amode & ~7) {
          // need a valid mode
          return -ERRNO_CODES.EINVAL;
        }
        var node;
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
        var perms = '';
        if (amode & 4) perms += 'r';
        if (amode & 2) perms += 'w';
        if (amode & 1) perms += 'x';
        if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
          return -ERRNO_CODES.EACCES;
        }
        return 0;
      },doDup:function (path, flags, suggestFD) {
        var suggest = FS.getStream(suggestFD);
        if (suggest) FS.close(suggest);
        return FS.open(path, flags, 0, suggestFD, suggestFD).fd;
      },doReadv:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.read(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
          if (curr < len) break; // nothing more to read
        }
        return ret;
      },doWritev:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.write(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
        }
        return ret;
      },varargs:0,get:function (varargs) {
        SYSCALLS.varargs += 4;
        var ret = HEAP32[(((SYSCALLS.varargs)-(4))>>2)];
        return ret;
      },getStr:function () {
        var ret = Pointer_stringify(SYSCALLS.get());
        return ret;
      },getStreamFromFD:function () {
        var stream = FS.getStream(SYSCALLS.get());
        if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return stream;
      },getSocketFromFD:function () {
        var socket = SOCKFS.getSocket(SYSCALLS.get());
        if (!socket) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return socket;
      },getSocketAddress:function (allowNull) {
        var addrp = SYSCALLS.get(), addrlen = SYSCALLS.get();
        if (allowNull && addrp === 0) return null;
        var info = __read_sockaddr(addrp, addrlen);
        if (info.errno) throw new FS.ErrnoError(info.errno);
        info.addr = DNS.lookup_addr(info.addr) || info.addr;
        return info;
      },get64:function () {
        var low = SYSCALLS.get(), high = SYSCALLS.get();
        if (low >= 0) assert(high === 0);
        else assert(high === -1);
        return low;
      },getZero:function () {
        assert(SYSCALLS.get() === 0);
      }};function ___syscall63(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // dup2
      var old = SYSCALLS.getStreamFromFD(), suggestFD = SYSCALLS.get();
      if (old.fd === suggestFD) return suggestFD;
      return SYSCALLS.doDup(old.path, old.flags, suggestFD);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

   
  Module["_memset"] = _memset;

  var _BDtoILow=true;

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  function ___syscall195(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_stat64
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall194(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ftruncate64
      var fd = SYSCALLS.get(), zero = SYSCALLS.getZero(), length = SYSCALLS.get64();
      FS.ftruncate(fd, length);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall197(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_fstat64
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, stream.path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___lock() {}

  function ___unlock() {}

   
  Module["_i64Add"] = _i64Add;

  var _floor=Math_floor;

  var _sqrt=Math_sqrt;

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        console.error('emscripten_set_main_loop_timing: Cannot set timing mode for main loop since a main loop does not exist! Call emscripten_set_main_loop first to set one up.');
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setTimeout() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_rAF() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      } else if (mode == 2 /*EM_TIMING_SETIMMEDIATE*/) {
        if (!window['setImmediate']) {
          // Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
          var setImmediates = [];
          var emscriptenMainLoopMessageId = '__emcc';
          function Browser_setImmediate_messageHandler(event) {
            if (event.source === window && event.data === emscriptenMainLoopMessageId) {
              event.stopPropagation();
              setImmediates.shift()();
            }
          }
          window.addEventListener("message", Browser_setImmediate_messageHandler, true);
          window['setImmediate'] = function Browser_emulated_setImmediate(func) {
            setImmediates.push(func);
            window.postMessage(emscriptenMainLoopMessageId, "*");
          }
        }
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setImmediate() {
          window['setImmediate'](Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'immediate';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg, noSetTiming) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (!noSetTiming) {
        if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
        else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
        Browser.mainLoop.scheduler();
      }
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg, true /* do not set timing and call scheduler, we will do it on the next lines */);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
          Browser.mainLoop.scheduler();
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        Browser.vrDevice = vrDevice;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
  
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
  
        if (vrDevice) {
          canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
        } else {
          canvasContainer.requestFullScreen();
        }
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },allowAsyncCallbacks:true,queuedAsyncCallbacks:[],pauseAsyncCallbacks:function () {
        Browser.allowAsyncCallbacks = false;
      },resumeAsyncCallbacks:function () { // marks future callbacks as ok to execute, and synchronously runs any remaining ones right now
        Browser.allowAsyncCallbacks = true;
        if (Browser.queuedAsyncCallbacks.length > 0) {
          var callbacks = Browser.queuedAsyncCallbacks;
          Browser.queuedAsyncCallbacks = [];
          callbacks.forEach(function(func) {
            func();
          });
        }
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } // drop it on the floor otherwise, next interval will kick in
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              var last = Browser.touches[touch.identifier];
              if (!last) last = coords;
              Browser.lastTouches[touch.identifier] = last;
              Browser.touches[touch.identifier] = coords;
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};

  function ___syscall330(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // dup3
      var old = SYSCALLS.getStreamFromFD(), suggestFD = SYSCALLS.get(), flags = SYSCALLS.get();
      assert(!flags);
      if (old.fd === suggestFD) return -ERRNO_CODES.EINVAL;
      return SYSCALLS.doDup(old.path, old.flags, suggestFD);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  function ___syscall6(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // close
      var stream = SYSCALLS.getStreamFromFD();
      FS.close(stream);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall54(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ioctl
      var stream = SYSCALLS.getStreamFromFD(), op = SYSCALLS.get();
      switch (op) {
        case 21505: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0;
        }
        case 21506: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0; // no-op, not actually adjusting terminal settings
        }
        case 21519: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          var argp = SYSCALLS.get();
          HEAP32[((argp)>>2)]=0;
          return 0;
        }
        case 21520: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return -ERRNO_CODES.EINVAL; // not supported
        }
        case 21531: {
          var argp = SYSCALLS.get();
          return FS.ioctl(stream, op, argp);
        }
        default: abort('bad ioctl syscall ' + op);
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

  function ___syscall33(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // access
      var path = SYSCALLS.getStr(), amode = SYSCALLS.get();
      return SYSCALLS.doAccess(path, amode);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _BDtoIHigh=true;

  function _pthread_cleanup_push(routine, arg) {
      __ATEXIT__.push(function() { Runtime.dynCall('vi', routine, [arg]) })
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  function ___syscall10(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // unlink
      var path = SYSCALLS.getStr();
      FS.unlink(path);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _pthread_cleanup_pop() {
      assert(_pthread_cleanup_push.level == __ATEXIT__.length, 'cannot pop if something else added meanwhile!');
      __ATEXIT__.pop();
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  function ___syscall5(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // open
      var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get() // optional TODO
      var stream = FS.open(pathname, flags, mode);
      return stream.fd;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  var _log=Math_log;

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) {
        var success = self.alloc(bytes);
        if (!success) return -1 >>> 0; // sbrk failure code
      }
      return ret;  // Previous break location.
    }

  
  var __sigalrm_handler=0;function _signal(sig, func) {
      if (sig == 14 /*SIGALRM*/) {
        __sigalrm_handler = func;
      } else {
        Module.printErr('Calling stub instead of signal()');
      }
      return 0;
    }

  var _BItoD=true;

  function ___syscall265(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // clock_nanosleep
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  function _pthread_self() {
      //FIXME: assumes only a single thread
      return 0;
    }

  function ___syscall140(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // llseek
      var stream = SYSCALLS.getStreamFromFD(), offset_high = SYSCALLS.get(), offset_low = SYSCALLS.get(), result = SYSCALLS.get(), whence = SYSCALLS.get();
      var offset = offset_low;
      assert(offset_high === 0);
      FS.llseek(stream, offset, whence);
      HEAP32[((result)>>2)]=stream.position;
      if (stream.getdents && offset === 0 && whence === 0) stream.getdents = null; // reset readdir state
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall146(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // writev
      var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
      return SYSCALLS.doWritev(stream, iov, iovcnt);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall221(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fcntl64
      var stream = SYSCALLS.getStreamFromFD(), cmd = SYSCALLS.get();
      switch (cmd) {
        case 0: {
          var arg = SYSCALLS.get();
          if (arg < 0) {
            return -ERRNO_CODES.EINVAL;
          }
          var newStream;
          newStream = FS.open(stream.path, stream.flags, 0, arg);
          return newStream.fd;
        }
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          return stream.flags;
        case 4: {
          var arg = SYSCALLS.get();
          stream.flags |= arg;
          return 0;
        }
        case 12:
        case 12: {
          var arg = SYSCALLS.get();
          var offset = 0;
          // We're always unlocked.
          HEAP16[(((arg)+(offset))>>1)]=2;
          return 0;
        }
        case 13:
        case 14:
        case 13:
        case 14:
          return 0; // Pretend that the locking is successful.
        case 16:
        case 8:
          return -ERRNO_CODES.EINVAL; // These are for sockets. We don't have them fully implemented yet.
        case 9:
          // musl trusts getown return values, due to a bug where they must be, as they overlap with errors. just return -1 here, so fnctl() returns that, and we set errno ourselves.
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default: {
          return -ERRNO_CODES.EINVAL;
        }
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall145(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // readv
      var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
      return SYSCALLS.doReadv(stream, iov, iovcnt);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }
FS.staticInit();__ATINIT__.unshift(function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() });__ATMAIN__.push(function() { FS.ignorePermissions = false });__ATEXIT__.push(function() { FS.quit() });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;Module["FS_unlink"] = FS.unlink;
__ATINIT__.unshift(function() { TTY.init() });__ATEXIT__.push(function() { TTY.shutdown() });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
  Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function nullFunc_i(x) { Module["printErr"]("Invalid function pointer called with signature 'i'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_ii(x) { Module["printErr"]("Invalid function pointer called with signature 'ii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_iiiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function nullFunc_vi(x) { Module["printErr"]("Invalid function pointer called with signature 'vi'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info.");abort(x) }

function invoke_i(index) {
  try {
    return Module["dynCall_i"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };

Module.asmLibraryArg = { "abort": abort, "assert": assert, "nullFunc_i": nullFunc_i, "nullFunc_ii": nullFunc_ii, "nullFunc_iiii": nullFunc_iiii, "nullFunc_iiiii": nullFunc_iiiii, "nullFunc_vi": nullFunc_vi, "invoke_i": invoke_i, "invoke_ii": invoke_ii, "invoke_iiii": invoke_iiii, "invoke_iiiii": invoke_iiiii, "invoke_vi": invoke_vi, "_pthread_cleanup_pop": _pthread_cleanup_pop, "___syscall221": ___syscall221, "___syscall265": ___syscall265, "_log": _log, "___syscall63": ___syscall63, "___syscall6": ___syscall6, "___setErrNo": ___setErrNo, "_floor": _floor, "_signal": _signal, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "___syscall330": ___syscall330, "___syscall197": ___syscall197, "_emscripten_memcpy_big": _emscripten_memcpy_big, "___syscall194": ___syscall194, "__exit": __exit, "___syscall195": ___syscall195, "_pthread_self": _pthread_self, "_sqrt": _sqrt, "___syscall33": ___syscall33, "___syscall54": ___syscall54, "___unlock": ___unlock, "_emscripten_set_main_loop": _emscripten_set_main_loop, "___syscall10": ___syscall10, "_sysconf": _sysconf, "___lock": ___lock, "_abort": _abort, "___syscall5": ___syscall5, "_time": _time, "___syscall140": ___syscall140, "_exit": _exit, "_pthread_cleanup_push": _pthread_cleanup_push, "___syscall145": ___syscall145, "___syscall146": ___syscall146, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8 };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'almost asm';
  
  
  var HEAP8 = new global.Int8Array(buffer);
  var HEAP16 = new global.Int16Array(buffer);
  var HEAP32 = new global.Int32Array(buffer);
  var HEAPU8 = new global.Uint8Array(buffer);
  var HEAPU16 = new global.Uint16Array(buffer);
  var HEAPU32 = new global.Uint32Array(buffer);
  var HEAPF32 = new global.Float32Array(buffer);
  var HEAPF64 = new global.Float64Array(buffer);


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = global.NaN, inf = global.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var Math_min=global.Math.min;
  var Math_clz32=global.Math.clz32;
  var abort=env.abort;
  var assert=env.assert;
  var nullFunc_i=env.nullFunc_i;
  var nullFunc_ii=env.nullFunc_ii;
  var nullFunc_iiii=env.nullFunc_iiii;
  var nullFunc_iiiii=env.nullFunc_iiiii;
  var nullFunc_vi=env.nullFunc_vi;
  var invoke_i=env.invoke_i;
  var invoke_ii=env.invoke_ii;
  var invoke_iiii=env.invoke_iiii;
  var invoke_iiiii=env.invoke_iiiii;
  var invoke_vi=env.invoke_vi;
  var _pthread_cleanup_pop=env._pthread_cleanup_pop;
  var ___syscall221=env.___syscall221;
  var ___syscall265=env.___syscall265;
  var _log=env._log;
  var ___syscall63=env.___syscall63;
  var ___syscall6=env.___syscall6;
  var ___setErrNo=env.___setErrNo;
  var _floor=env._floor;
  var _signal=env._signal;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _sbrk=env._sbrk;
  var ___syscall330=env.___syscall330;
  var ___syscall197=env.___syscall197;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var ___syscall194=env.___syscall194;
  var __exit=env.__exit;
  var ___syscall195=env.___syscall195;
  var _pthread_self=env._pthread_self;
  var _sqrt=env._sqrt;
  var ___syscall33=env.___syscall33;
  var ___syscall54=env.___syscall54;
  var ___unlock=env.___unlock;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var ___syscall10=env.___syscall10;
  var _sysconf=env._sysconf;
  var ___lock=env.___lock;
  var _abort=env._abort;
  var ___syscall5=env.___syscall5;
  var _time=env._time;
  var ___syscall140=env.___syscall140;
  var _exit=env._exit;
  var _pthread_cleanup_push=env._pthread_cleanup_push;
  var ___syscall145=env.___syscall145;
  var ___syscall146=env.___syscall146;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 15)&-16;
if ((STACKTOP|0) >= (STACK_MAX|0)) abort();

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
  stackBase = stackBase|0;
  stackMax = stackMax|0;
  STACKTOP = stackBase;
  STACK_MAX = stackMax;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}

function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function _setulb_($n,$m,$x,$l,$u,$nbd,$f,$g,$factr,$pgtol,$wa,$iwa,$task,$iprint,$csave,$lsave,$isave,$dsave,$maxls,$task_len,$csave_len) {
 $n = $n|0;
 $m = $m|0;
 $x = $x|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $f = $f|0;
 $g = $g|0;
 $factr = $factr|0;
 $pgtol = $pgtol|0;
 $wa = $wa|0;
 $iwa = $iwa|0;
 $task = $task|0;
 $iprint = $iprint|0;
 $csave = $csave|0;
 $lsave = $lsave|0;
 $isave = $isave|0;
 $dsave = $dsave|0;
 $maxls = $maxls|0;
 $task_len = $task_len|0;
 $csave_len = $csave_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $m;
 $2 = $x;
 $3 = $l;
 $4 = $u;
 $5 = $nbd;
 $6 = $f;
 $7 = $g;
 $8 = $factr;
 $9 = $pgtol;
 $10 = $wa;
 $11 = $iwa;
 $12 = $task;
 $13 = $iprint;
 $14 = $csave;
 $15 = $lsave;
 $16 = $isave;
 $17 = $dsave;
 $18 = $maxls;
 $19 = $task_len;
 $20 = $csave_len;
 $21 = $11; //@line 275 "lbfgsb.c"
 $22 = ((($21)) + -4|0); //@line 275 "lbfgsb.c"
 $11 = $22; //@line 275 "lbfgsb.c"
 $23 = $7; //@line 276 "lbfgsb.c"
 $24 = ((($23)) + -8|0); //@line 276 "lbfgsb.c"
 $7 = $24; //@line 276 "lbfgsb.c"
 $25 = $5; //@line 277 "lbfgsb.c"
 $26 = ((($25)) + -4|0); //@line 277 "lbfgsb.c"
 $5 = $26; //@line 277 "lbfgsb.c"
 $27 = $4; //@line 278 "lbfgsb.c"
 $28 = ((($27)) + -8|0); //@line 278 "lbfgsb.c"
 $4 = $28; //@line 278 "lbfgsb.c"
 $29 = $3; //@line 279 "lbfgsb.c"
 $30 = ((($29)) + -8|0); //@line 279 "lbfgsb.c"
 $3 = $30; //@line 279 "lbfgsb.c"
 $31 = $2; //@line 280 "lbfgsb.c"
 $32 = ((($31)) + -8|0); //@line 280 "lbfgsb.c"
 $2 = $32; //@line 280 "lbfgsb.c"
 $33 = $10; //@line 281 "lbfgsb.c"
 $34 = ((($33)) + -8|0); //@line 281 "lbfgsb.c"
 $10 = $34; //@line 281 "lbfgsb.c"
 $35 = $15; //@line 282 "lbfgsb.c"
 $36 = ((($35)) + -4|0); //@line 282 "lbfgsb.c"
 $15 = $36; //@line 282 "lbfgsb.c"
 $37 = $16; //@line 283 "lbfgsb.c"
 $38 = ((($37)) + -4|0); //@line 283 "lbfgsb.c"
 $16 = $38; //@line 283 "lbfgsb.c"
 $39 = $17; //@line 284 "lbfgsb.c"
 $40 = ((($39)) + -8|0); //@line 284 "lbfgsb.c"
 $17 = $40; //@line 284 "lbfgsb.c"
 $41 = $12; //@line 287 "lbfgsb.c"
 $42 = (_s_cmp($41,13740,60,5)|0); //@line 287 "lbfgsb.c"
 $43 = ($42|0)==(0); //@line 287 "lbfgsb.c"
 if ($43) {
  $44 = $1; //@line 288 "lbfgsb.c"
  $45 = HEAP32[$44>>2]|0; //@line 288 "lbfgsb.c"
  $46 = $0; //@line 288 "lbfgsb.c"
  $47 = HEAP32[$46>>2]|0; //@line 288 "lbfgsb.c"
  $48 = Math_imul($45, $47)|0; //@line 288 "lbfgsb.c"
  $49 = $16; //@line 288 "lbfgsb.c"
  $50 = ((($49)) + 4|0); //@line 288 "lbfgsb.c"
  HEAP32[$50>>2] = $48; //@line 288 "lbfgsb.c"
  $51 = $1; //@line 290 "lbfgsb.c"
  $52 = HEAP32[$51>>2]|0; //@line 290 "lbfgsb.c"
  $i__1 = $52; //@line 290 "lbfgsb.c"
  $53 = $i__1; //@line 291 "lbfgsb.c"
  $54 = $i__1; //@line 291 "lbfgsb.c"
  $55 = Math_imul($53, $54)|0; //@line 291 "lbfgsb.c"
  $56 = $16; //@line 291 "lbfgsb.c"
  $57 = ((($56)) + 8|0); //@line 291 "lbfgsb.c"
  HEAP32[$57>>2] = $55; //@line 291 "lbfgsb.c"
  $58 = $1; //@line 293 "lbfgsb.c"
  $59 = HEAP32[$58>>2]|0; //@line 293 "lbfgsb.c"
  $i__1 = $59; //@line 293 "lbfgsb.c"
  $60 = $i__1; //@line 294 "lbfgsb.c"
  $61 = $i__1; //@line 294 "lbfgsb.c"
  $62 = Math_imul($60, $61)|0; //@line 294 "lbfgsb.c"
  $63 = $62 << 2; //@line 294 "lbfgsb.c"
  $64 = $16; //@line 294 "lbfgsb.c"
  $65 = ((($64)) + 12|0); //@line 294 "lbfgsb.c"
  HEAP32[$65>>2] = $63; //@line 294 "lbfgsb.c"
  $66 = $16; //@line 295 "lbfgsb.c"
  $67 = ((($66)) + 16|0); //@line 295 "lbfgsb.c"
  HEAP32[$67>>2] = 1; //@line 295 "lbfgsb.c"
  $68 = $16; //@line 297 "lbfgsb.c"
  $69 = ((($68)) + 16|0); //@line 297 "lbfgsb.c"
  $70 = HEAP32[$69>>2]|0; //@line 297 "lbfgsb.c"
  $71 = $16; //@line 297 "lbfgsb.c"
  $72 = ((($71)) + 4|0); //@line 297 "lbfgsb.c"
  $73 = HEAP32[$72>>2]|0; //@line 297 "lbfgsb.c"
  $74 = (($70) + ($73))|0; //@line 297 "lbfgsb.c"
  $75 = $16; //@line 297 "lbfgsb.c"
  $76 = ((($75)) + 20|0); //@line 297 "lbfgsb.c"
  HEAP32[$76>>2] = $74; //@line 297 "lbfgsb.c"
  $77 = $16; //@line 299 "lbfgsb.c"
  $78 = ((($77)) + 20|0); //@line 299 "lbfgsb.c"
  $79 = HEAP32[$78>>2]|0; //@line 299 "lbfgsb.c"
  $80 = $16; //@line 299 "lbfgsb.c"
  $81 = ((($80)) + 4|0); //@line 299 "lbfgsb.c"
  $82 = HEAP32[$81>>2]|0; //@line 299 "lbfgsb.c"
  $83 = (($79) + ($82))|0; //@line 299 "lbfgsb.c"
  $84 = $16; //@line 299 "lbfgsb.c"
  $85 = ((($84)) + 24|0); //@line 299 "lbfgsb.c"
  HEAP32[$85>>2] = $83; //@line 299 "lbfgsb.c"
  $86 = $16; //@line 301 "lbfgsb.c"
  $87 = ((($86)) + 24|0); //@line 301 "lbfgsb.c"
  $88 = HEAP32[$87>>2]|0; //@line 301 "lbfgsb.c"
  $89 = $16; //@line 301 "lbfgsb.c"
  $90 = ((($89)) + 8|0); //@line 301 "lbfgsb.c"
  $91 = HEAP32[$90>>2]|0; //@line 301 "lbfgsb.c"
  $92 = (($88) + ($91))|0; //@line 301 "lbfgsb.c"
  $93 = $16; //@line 301 "lbfgsb.c"
  $94 = ((($93)) + 28|0); //@line 301 "lbfgsb.c"
  HEAP32[$94>>2] = $92; //@line 301 "lbfgsb.c"
  $95 = $16; //@line 303 "lbfgsb.c"
  $96 = ((($95)) + 28|0); //@line 303 "lbfgsb.c"
  $97 = HEAP32[$96>>2]|0; //@line 303 "lbfgsb.c"
  $98 = $16; //@line 303 "lbfgsb.c"
  $99 = ((($98)) + 8|0); //@line 303 "lbfgsb.c"
  $100 = HEAP32[$99>>2]|0; //@line 303 "lbfgsb.c"
  $101 = (($97) + ($100))|0; //@line 303 "lbfgsb.c"
  $102 = $16; //@line 303 "lbfgsb.c"
  $103 = ((($102)) + 32|0); //@line 303 "lbfgsb.c"
  HEAP32[$103>>2] = $101; //@line 303 "lbfgsb.c"
  $104 = $16; //@line 305 "lbfgsb.c"
  $105 = ((($104)) + 32|0); //@line 305 "lbfgsb.c"
  $106 = HEAP32[$105>>2]|0; //@line 305 "lbfgsb.c"
  $107 = $16; //@line 305 "lbfgsb.c"
  $108 = ((($107)) + 8|0); //@line 305 "lbfgsb.c"
  $109 = HEAP32[$108>>2]|0; //@line 305 "lbfgsb.c"
  $110 = (($106) + ($109))|0; //@line 305 "lbfgsb.c"
  $111 = $16; //@line 305 "lbfgsb.c"
  $112 = ((($111)) + 36|0); //@line 305 "lbfgsb.c"
  HEAP32[$112>>2] = $110; //@line 305 "lbfgsb.c"
  $113 = $16; //@line 307 "lbfgsb.c"
  $114 = ((($113)) + 36|0); //@line 307 "lbfgsb.c"
  $115 = HEAP32[$114>>2]|0; //@line 307 "lbfgsb.c"
  $116 = $16; //@line 307 "lbfgsb.c"
  $117 = ((($116)) + 12|0); //@line 307 "lbfgsb.c"
  $118 = HEAP32[$117>>2]|0; //@line 307 "lbfgsb.c"
  $119 = (($115) + ($118))|0; //@line 307 "lbfgsb.c"
  $120 = $16; //@line 307 "lbfgsb.c"
  $121 = ((($120)) + 40|0); //@line 307 "lbfgsb.c"
  HEAP32[$121>>2] = $119; //@line 307 "lbfgsb.c"
  $122 = $16; //@line 309 "lbfgsb.c"
  $123 = ((($122)) + 40|0); //@line 309 "lbfgsb.c"
  $124 = HEAP32[$123>>2]|0; //@line 309 "lbfgsb.c"
  $125 = $16; //@line 309 "lbfgsb.c"
  $126 = ((($125)) + 12|0); //@line 309 "lbfgsb.c"
  $127 = HEAP32[$126>>2]|0; //@line 309 "lbfgsb.c"
  $128 = (($124) + ($127))|0; //@line 309 "lbfgsb.c"
  $129 = $16; //@line 309 "lbfgsb.c"
  $130 = ((($129)) + 44|0); //@line 309 "lbfgsb.c"
  HEAP32[$130>>2] = $128; //@line 309 "lbfgsb.c"
  $131 = $16; //@line 311 "lbfgsb.c"
  $132 = ((($131)) + 44|0); //@line 311 "lbfgsb.c"
  $133 = HEAP32[$132>>2]|0; //@line 311 "lbfgsb.c"
  $134 = $0; //@line 311 "lbfgsb.c"
  $135 = HEAP32[$134>>2]|0; //@line 311 "lbfgsb.c"
  $136 = (($133) + ($135))|0; //@line 311 "lbfgsb.c"
  $137 = $16; //@line 311 "lbfgsb.c"
  $138 = ((($137)) + 48|0); //@line 311 "lbfgsb.c"
  HEAP32[$138>>2] = $136; //@line 311 "lbfgsb.c"
  $139 = $16; //@line 313 "lbfgsb.c"
  $140 = ((($139)) + 48|0); //@line 313 "lbfgsb.c"
  $141 = HEAP32[$140>>2]|0; //@line 313 "lbfgsb.c"
  $142 = $0; //@line 313 "lbfgsb.c"
  $143 = HEAP32[$142>>2]|0; //@line 313 "lbfgsb.c"
  $144 = (($141) + ($143))|0; //@line 313 "lbfgsb.c"
  $145 = $16; //@line 313 "lbfgsb.c"
  $146 = ((($145)) + 52|0); //@line 313 "lbfgsb.c"
  HEAP32[$146>>2] = $144; //@line 313 "lbfgsb.c"
  $147 = $16; //@line 315 "lbfgsb.c"
  $148 = ((($147)) + 52|0); //@line 315 "lbfgsb.c"
  $149 = HEAP32[$148>>2]|0; //@line 315 "lbfgsb.c"
  $150 = $0; //@line 315 "lbfgsb.c"
  $151 = HEAP32[$150>>2]|0; //@line 315 "lbfgsb.c"
  $152 = (($149) + ($151))|0; //@line 315 "lbfgsb.c"
  $153 = $16; //@line 315 "lbfgsb.c"
  $154 = ((($153)) + 56|0); //@line 315 "lbfgsb.c"
  HEAP32[$154>>2] = $152; //@line 315 "lbfgsb.c"
  $155 = $16; //@line 317 "lbfgsb.c"
  $156 = ((($155)) + 56|0); //@line 317 "lbfgsb.c"
  $157 = HEAP32[$156>>2]|0; //@line 317 "lbfgsb.c"
  $158 = $0; //@line 317 "lbfgsb.c"
  $159 = HEAP32[$158>>2]|0; //@line 317 "lbfgsb.c"
  $160 = (($157) + ($159))|0; //@line 317 "lbfgsb.c"
  $161 = $16; //@line 317 "lbfgsb.c"
  $162 = ((($161)) + 60|0); //@line 317 "lbfgsb.c"
  HEAP32[$162>>2] = $160; //@line 317 "lbfgsb.c"
  $163 = $16; //@line 319 "lbfgsb.c"
  $164 = ((($163)) + 60|0); //@line 319 "lbfgsb.c"
  $165 = HEAP32[$164>>2]|0; //@line 319 "lbfgsb.c"
  $166 = $0; //@line 319 "lbfgsb.c"
  $167 = HEAP32[$166>>2]|0; //@line 319 "lbfgsb.c"
  $168 = (($165) + ($167))|0; //@line 319 "lbfgsb.c"
  $169 = $16; //@line 319 "lbfgsb.c"
  $170 = ((($169)) + 64|0); //@line 319 "lbfgsb.c"
  HEAP32[$170>>2] = $168; //@line 319 "lbfgsb.c"
 }
 $171 = $16; //@line 322 "lbfgsb.c"
 $172 = ((($171)) + 16|0); //@line 322 "lbfgsb.c"
 $173 = HEAP32[$172>>2]|0; //@line 322 "lbfgsb.c"
 HEAP32[960>>2] = $173; //@line 322 "lbfgsb.c"
 $174 = $16; //@line 323 "lbfgsb.c"
 $175 = ((($174)) + 20|0); //@line 323 "lbfgsb.c"
 $176 = HEAP32[$175>>2]|0; //@line 323 "lbfgsb.c"
 HEAP32[972>>2] = $176; //@line 323 "lbfgsb.c"
 $177 = $16; //@line 324 "lbfgsb.c"
 $178 = ((($177)) + 24|0); //@line 324 "lbfgsb.c"
 $179 = HEAP32[$178>>2]|0; //@line 324 "lbfgsb.c"
 HEAP32[968>>2] = $179; //@line 324 "lbfgsb.c"
 $180 = $16; //@line 325 "lbfgsb.c"
 $181 = ((($180)) + 28|0); //@line 325 "lbfgsb.c"
 $182 = HEAP32[$181>>2]|0; //@line 325 "lbfgsb.c"
 HEAP32[952>>2] = $182; //@line 325 "lbfgsb.c"
 $183 = $16; //@line 326 "lbfgsb.c"
 $184 = ((($183)) + 32|0); //@line 326 "lbfgsb.c"
 $185 = HEAP32[$184>>2]|0; //@line 326 "lbfgsb.c"
 HEAP32[964>>2] = $185; //@line 326 "lbfgsb.c"
 $186 = $16; //@line 327 "lbfgsb.c"
 $187 = ((($186)) + 36|0); //@line 327 "lbfgsb.c"
 $188 = HEAP32[$187>>2]|0; //@line 327 "lbfgsb.c"
 HEAP32[948>>2] = $188; //@line 327 "lbfgsb.c"
 $189 = $16; //@line 328 "lbfgsb.c"
 $190 = ((($189)) + 40|0); //@line 328 "lbfgsb.c"
 $191 = HEAP32[$190>>2]|0; //@line 328 "lbfgsb.c"
 HEAP32[976>>2] = $191; //@line 328 "lbfgsb.c"
 $192 = $16; //@line 329 "lbfgsb.c"
 $193 = ((($192)) + 44|0); //@line 329 "lbfgsb.c"
 $194 = HEAP32[$193>>2]|0; //@line 329 "lbfgsb.c"
 HEAP32[940>>2] = $194; //@line 329 "lbfgsb.c"
 $195 = $16; //@line 330 "lbfgsb.c"
 $196 = ((($195)) + 48|0); //@line 330 "lbfgsb.c"
 $197 = HEAP32[$196>>2]|0; //@line 330 "lbfgsb.c"
 HEAP32[932>>2] = $197; //@line 330 "lbfgsb.c"
 $198 = $16; //@line 331 "lbfgsb.c"
 $199 = ((($198)) + 52|0); //@line 331 "lbfgsb.c"
 $200 = HEAP32[$199>>2]|0; //@line 331 "lbfgsb.c"
 HEAP32[928>>2] = $200; //@line 331 "lbfgsb.c"
 $201 = $16; //@line 332 "lbfgsb.c"
 $202 = ((($201)) + 56|0); //@line 332 "lbfgsb.c"
 $203 = HEAP32[$202>>2]|0; //@line 332 "lbfgsb.c"
 HEAP32[936>>2] = $203; //@line 332 "lbfgsb.c"
 $204 = $16; //@line 333 "lbfgsb.c"
 $205 = ((($204)) + 60|0); //@line 333 "lbfgsb.c"
 $206 = HEAP32[$205>>2]|0; //@line 333 "lbfgsb.c"
 HEAP32[956>>2] = $206; //@line 333 "lbfgsb.c"
 $207 = $16; //@line 334 "lbfgsb.c"
 $208 = ((($207)) + 64|0); //@line 334 "lbfgsb.c"
 $209 = HEAP32[$208>>2]|0; //@line 334 "lbfgsb.c"
 HEAP32[944>>2] = $209; //@line 334 "lbfgsb.c"
 $210 = $0; //@line 335 "lbfgsb.c"
 $211 = $1; //@line 335 "lbfgsb.c"
 $212 = $2; //@line 335 "lbfgsb.c"
 $213 = ((($212)) + 8|0); //@line 335 "lbfgsb.c"
 $214 = $3; //@line 335 "lbfgsb.c"
 $215 = ((($214)) + 8|0); //@line 335 "lbfgsb.c"
 $216 = $4; //@line 335 "lbfgsb.c"
 $217 = ((($216)) + 8|0); //@line 335 "lbfgsb.c"
 $218 = $5; //@line 335 "lbfgsb.c"
 $219 = ((($218)) + 4|0); //@line 335 "lbfgsb.c"
 $220 = $6; //@line 335 "lbfgsb.c"
 $221 = $7; //@line 335 "lbfgsb.c"
 $222 = ((($221)) + 8|0); //@line 335 "lbfgsb.c"
 $223 = $8; //@line 335 "lbfgsb.c"
 $224 = $9; //@line 335 "lbfgsb.c"
 $225 = HEAP32[960>>2]|0; //@line 336 "lbfgsb.c"
 $226 = $10; //@line 335 "lbfgsb.c"
 $227 = (($226) + ($225<<3)|0); //@line 335 "lbfgsb.c"
 $228 = HEAP32[972>>2]|0; //@line 336 "lbfgsb.c"
 $229 = $10; //@line 336 "lbfgsb.c"
 $230 = (($229) + ($228<<3)|0); //@line 336 "lbfgsb.c"
 $231 = HEAP32[968>>2]|0; //@line 336 "lbfgsb.c"
 $232 = $10; //@line 336 "lbfgsb.c"
 $233 = (($232) + ($231<<3)|0); //@line 336 "lbfgsb.c"
 $234 = HEAP32[952>>2]|0; //@line 336 "lbfgsb.c"
 $235 = $10; //@line 336 "lbfgsb.c"
 $236 = (($235) + ($234<<3)|0); //@line 336 "lbfgsb.c"
 $237 = HEAP32[964>>2]|0; //@line 336 "lbfgsb.c"
 $238 = $10; //@line 336 "lbfgsb.c"
 $239 = (($238) + ($237<<3)|0); //@line 336 "lbfgsb.c"
 $240 = HEAP32[948>>2]|0; //@line 336 "lbfgsb.c"
 $241 = $10; //@line 336 "lbfgsb.c"
 $242 = (($241) + ($240<<3)|0); //@line 336 "lbfgsb.c"
 $243 = HEAP32[976>>2]|0; //@line 336 "lbfgsb.c"
 $244 = $10; //@line 336 "lbfgsb.c"
 $245 = (($244) + ($243<<3)|0); //@line 336 "lbfgsb.c"
 $246 = HEAP32[940>>2]|0; //@line 337 "lbfgsb.c"
 $247 = $10; //@line 337 "lbfgsb.c"
 $248 = (($247) + ($246<<3)|0); //@line 337 "lbfgsb.c"
 $249 = HEAP32[932>>2]|0; //@line 337 "lbfgsb.c"
 $250 = $10; //@line 337 "lbfgsb.c"
 $251 = (($250) + ($249<<3)|0); //@line 337 "lbfgsb.c"
 $252 = HEAP32[928>>2]|0; //@line 337 "lbfgsb.c"
 $253 = $10; //@line 337 "lbfgsb.c"
 $254 = (($253) + ($252<<3)|0); //@line 337 "lbfgsb.c"
 $255 = HEAP32[936>>2]|0; //@line 337 "lbfgsb.c"
 $256 = $10; //@line 337 "lbfgsb.c"
 $257 = (($256) + ($255<<3)|0); //@line 337 "lbfgsb.c"
 $258 = HEAP32[956>>2]|0; //@line 337 "lbfgsb.c"
 $259 = $10; //@line 337 "lbfgsb.c"
 $260 = (($259) + ($258<<3)|0); //@line 337 "lbfgsb.c"
 $261 = HEAP32[944>>2]|0; //@line 337 "lbfgsb.c"
 $262 = $10; //@line 337 "lbfgsb.c"
 $263 = (($262) + ($261<<3)|0); //@line 337 "lbfgsb.c"
 $264 = $11; //@line 337 "lbfgsb.c"
 $265 = ((($264)) + 4|0); //@line 337 "lbfgsb.c"
 $266 = $0; //@line 338 "lbfgsb.c"
 $267 = HEAP32[$266>>2]|0; //@line 338 "lbfgsb.c"
 $268 = (($267) + 1)|0; //@line 338 "lbfgsb.c"
 $269 = $11; //@line 338 "lbfgsb.c"
 $270 = (($269) + ($268<<2)|0); //@line 338 "lbfgsb.c"
 $271 = $0; //@line 338 "lbfgsb.c"
 $272 = HEAP32[$271>>2]|0; //@line 338 "lbfgsb.c"
 $273 = $272 << 1; //@line 338 "lbfgsb.c"
 $274 = (($273) + 1)|0; //@line 338 "lbfgsb.c"
 $275 = $11; //@line 338 "lbfgsb.c"
 $276 = (($275) + ($274<<2)|0); //@line 338 "lbfgsb.c"
 $277 = $12; //@line 338 "lbfgsb.c"
 $278 = $13; //@line 338 "lbfgsb.c"
 $279 = $14; //@line 338 "lbfgsb.c"
 $280 = $15; //@line 338 "lbfgsb.c"
 $281 = ((($280)) + 4|0); //@line 338 "lbfgsb.c"
 $282 = $16; //@line 339 "lbfgsb.c"
 $283 = ((($282)) + 88|0); //@line 339 "lbfgsb.c"
 $284 = $17; //@line 339 "lbfgsb.c"
 $285 = ((($284)) + 8|0); //@line 339 "lbfgsb.c"
 $286 = $18; //@line 339 "lbfgsb.c"
 (_mainlb_($210,$211,$213,$215,$217,$219,$220,$222,$223,$224,$227,$230,$233,$236,$239,$242,$245,$248,$251,$254,$257,$260,$263,$265,$270,$276,$277,$278,$279,$281,$283,$285,$286,60,60)|0); //@line 335 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 340 "lbfgsb.c"
}
function _mainlb_($n,$m,$x,$l,$u,$nbd,$f,$g,$factr,$pgtol,$ws,$wy,$sy,$ss,$wt,$wn,$snd,$z__,$r__,$d__,$t,$xp,$wa,$index,$iwhere,$indx2,$task,$iprint,$csave,$lsave,$isave,$dsave,$maxls,$task_len,$csave_len) {
 $n = $n|0;
 $m = $m|0;
 $x = $x|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $f = $f|0;
 $g = $g|0;
 $factr = $factr|0;
 $pgtol = $pgtol|0;
 $ws = $ws|0;
 $wy = $wy|0;
 $sy = $sy|0;
 $ss = $ss|0;
 $wt = $wt|0;
 $wn = $wn|0;
 $snd = $snd|0;
 $z__ = $z__|0;
 $r__ = $r__|0;
 $d__ = $d__|0;
 $t = $t|0;
 $xp = $xp|0;
 $wa = $wa|0;
 $index = $index|0;
 $iwhere = $iwhere|0;
 $indx2 = $indx2|0;
 $task = $task|0;
 $iprint = $iprint|0;
 $csave = $csave|0;
 $lsave = $lsave|0;
 $isave = $isave|0;
 $dsave = $dsave|0;
 $maxls = $maxls|0;
 $task_len = $task_len|0;
 $csave_len = $csave_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0, $245 = 0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0, $251 = 0, $252 = 0.0, $253 = 0, $254 = 0, $255 = 0.0, $256 = 0, $257 = 0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0.0, $262 = 0, $263 = 0, $264 = 0.0, $265 = 0, $266 = 0, $267 = 0.0, $268 = 0, $269 = 0, $27 = 0, $270 = 0.0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0, $275 = 0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0.0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0.0, $286 = 0, $287 = 0, $288 = 0.0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0.0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0.0, $334 = 0, $335 = 0.0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0.0, $414 = 0.0, $415 = 0.0, $416 = 0.0, $417 = 0.0, $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0.0;
 var $422 = 0.0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0.0, $471 = 0.0, $472 = 0.0, $473 = 0.0, $474 = 0.0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0.0, $541 = 0.0, $542 = 0.0, $543 = 0.0, $544 = 0.0, $545 = 0.0, $546 = 0.0, $547 = 0.0, $548 = 0.0;
 var $549 = 0.0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0.0, $563 = 0.0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0.0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0.0, $636 = 0.0, $637 = 0.0, $638 = 0.0;
 var $639 = 0.0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0.0, $644 = 0.0, $645 = 0.0, $646 = 0.0, $647 = 0.0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0.0, $669 = 0, $67 = 0, $670 = 0.0, $671 = 0, $672 = 0, $673 = 0.0, $674 = 0;
 var $675 = 0.0, $676 = 0.0, $677 = 0.0, $678 = 0, $679 = 0.0, $68 = 0, $680 = 0, $681 = 0, $682 = 0.0, $683 = 0.0, $684 = 0.0, $685 = 0.0, $686 = 0.0, $687 = 0, $688 = 0.0, $689 = 0.0, $69 = 0, $690 = 0.0, $691 = 0.0, $692 = 0;
 var $693 = 0.0, $694 = 0.0, $695 = 0.0, $696 = 0, $697 = 0.0, $698 = 0.0, $699 = 0.0, $7 = 0, $70 = 0, $700 = 0.0, $701 = 0.0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0.0, $715 = 0, $716 = 0, $717 = 0, $718 = 0.0, $719 = 0.0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0.0, $731 = 0.0, $732 = 0, $733 = 0.0, $734 = 0.0, $735 = 0.0, $736 = 0.0, $737 = 0.0, $738 = 0.0, $739 = 0.0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0.0, $744 = 0.0, $745 = 0.0, $746 = 0.0;
 var $747 = 0.0, $748 = 0.0, $749 = 0.0, $75 = 0, $750 = 0.0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0.0, $793 = 0.0, $794 = 0.0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0.0, $865 = 0, $866 = 0, $867 = 0.0, $868 = 0, $869 = 0, $87 = 0, $870 = 0.0, $871 = 0, $872 = 0;
 var $873 = 0.0, $874 = 0, $875 = 0, $876 = 0.0, $877 = 0, $878 = 0, $879 = 0.0, $88 = 0, $880 = 0, $881 = 0, $882 = 0.0, $883 = 0, $884 = 0, $885 = 0.0, $886 = 0, $887 = 0, $888 = 0.0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0.0, $892 = 0, $893 = 0, $894 = 0.0, $895 = 0, $896 = 0, $897 = 0.0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0.0, $901 = 0, $902 = 0, $903 = 0.0, $904 = 0, $905 = 0, $906 = 0.0, $907 = 0, $908 = 0;
 var $909 = 0.0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d__1 = 0.0, $d__2 = 0.0, $i__1 = 0, $or$cond = 0, $or$cond3 = 0, $snd_dim1 = 0, $snd_offset = 0;
 var $ss_dim1 = 0, $ss_offset = 0, $sy_dim1 = 0, $sy_offset = 0, $wn_dim1 = 0, $wn_offset = 0, $ws_dim1 = 0, $ws_offset = 0, $wt_dim1 = 0, $wt_offset = 0, $wy_dim1 = 0, $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 16|0;
 $1 = $n;
 $2 = $m;
 $3 = $x;
 $4 = $l;
 $5 = $u;
 $6 = $nbd;
 $7 = $f;
 $8 = $g;
 $9 = $factr;
 $10 = $pgtol;
 $11 = $ws;
 $12 = $wy;
 $13 = $sy;
 $14 = $ss;
 $15 = $wt;
 $16 = $wn;
 $17 = $snd;
 $18 = $z__;
 $19 = $r__;
 $20 = $d__;
 $21 = $t;
 $22 = $xp;
 $23 = $wa;
 $24 = $index;
 $25 = $iwhere;
 $26 = $indx2;
 $27 = $task;
 $28 = $iprint;
 $29 = $csave;
 $30 = $lsave;
 $31 = $isave;
 $32 = $dsave;
 $33 = $maxls;
 $34 = $task_len;
 $35 = $csave_len;
 $36 = $26; //@line 675 "lbfgsb.c"
 $37 = ((($36)) + -4|0); //@line 675 "lbfgsb.c"
 $26 = $37; //@line 675 "lbfgsb.c"
 $38 = $25; //@line 676 "lbfgsb.c"
 $39 = ((($38)) + -4|0); //@line 676 "lbfgsb.c"
 $25 = $39; //@line 676 "lbfgsb.c"
 $40 = $24; //@line 677 "lbfgsb.c"
 $41 = ((($40)) + -4|0); //@line 677 "lbfgsb.c"
 $24 = $41; //@line 677 "lbfgsb.c"
 $42 = $22; //@line 678 "lbfgsb.c"
 $43 = ((($42)) + -8|0); //@line 678 "lbfgsb.c"
 $22 = $43; //@line 678 "lbfgsb.c"
 $44 = $21; //@line 679 "lbfgsb.c"
 $45 = ((($44)) + -8|0); //@line 679 "lbfgsb.c"
 $21 = $45; //@line 679 "lbfgsb.c"
 $46 = $20; //@line 680 "lbfgsb.c"
 $47 = ((($46)) + -8|0); //@line 680 "lbfgsb.c"
 $20 = $47; //@line 680 "lbfgsb.c"
 $48 = $19; //@line 681 "lbfgsb.c"
 $49 = ((($48)) + -8|0); //@line 681 "lbfgsb.c"
 $19 = $49; //@line 681 "lbfgsb.c"
 $50 = $18; //@line 682 "lbfgsb.c"
 $51 = ((($50)) + -8|0); //@line 682 "lbfgsb.c"
 $18 = $51; //@line 682 "lbfgsb.c"
 $52 = $8; //@line 683 "lbfgsb.c"
 $53 = ((($52)) + -8|0); //@line 683 "lbfgsb.c"
 $8 = $53; //@line 683 "lbfgsb.c"
 $54 = $6; //@line 684 "lbfgsb.c"
 $55 = ((($54)) + -4|0); //@line 684 "lbfgsb.c"
 $6 = $55; //@line 684 "lbfgsb.c"
 $56 = $5; //@line 685 "lbfgsb.c"
 $57 = ((($56)) + -8|0); //@line 685 "lbfgsb.c"
 $5 = $57; //@line 685 "lbfgsb.c"
 $58 = $4; //@line 686 "lbfgsb.c"
 $59 = ((($58)) + -8|0); //@line 686 "lbfgsb.c"
 $4 = $59; //@line 686 "lbfgsb.c"
 $60 = $3; //@line 687 "lbfgsb.c"
 $61 = ((($60)) + -8|0); //@line 687 "lbfgsb.c"
 $3 = $61; //@line 687 "lbfgsb.c"
 $62 = $23; //@line 688 "lbfgsb.c"
 $63 = ((($62)) + -8|0); //@line 688 "lbfgsb.c"
 $23 = $63; //@line 688 "lbfgsb.c"
 $64 = $2; //@line 689 "lbfgsb.c"
 $65 = HEAP32[$64>>2]|0; //@line 689 "lbfgsb.c"
 $66 = $65<<1; //@line 689 "lbfgsb.c"
 $snd_dim1 = $66; //@line 689 "lbfgsb.c"
 $67 = $snd_dim1; //@line 690 "lbfgsb.c"
 $68 = (1 + ($67))|0; //@line 690 "lbfgsb.c"
 $snd_offset = $68; //@line 690 "lbfgsb.c"
 $69 = $snd_offset; //@line 691 "lbfgsb.c"
 $70 = $17; //@line 691 "lbfgsb.c"
 $71 = (0 - ($69))|0; //@line 691 "lbfgsb.c"
 $72 = (($70) + ($71<<3)|0); //@line 691 "lbfgsb.c"
 $17 = $72; //@line 691 "lbfgsb.c"
 $73 = $2; //@line 692 "lbfgsb.c"
 $74 = HEAP32[$73>>2]|0; //@line 692 "lbfgsb.c"
 $75 = $74<<1; //@line 692 "lbfgsb.c"
 $wn_dim1 = $75; //@line 692 "lbfgsb.c"
 $76 = $wn_dim1; //@line 693 "lbfgsb.c"
 $77 = (1 + ($76))|0; //@line 693 "lbfgsb.c"
 $wn_offset = $77; //@line 693 "lbfgsb.c"
 $78 = $wn_offset; //@line 694 "lbfgsb.c"
 $79 = $16; //@line 694 "lbfgsb.c"
 $80 = (0 - ($78))|0; //@line 694 "lbfgsb.c"
 $81 = (($79) + ($80<<3)|0); //@line 694 "lbfgsb.c"
 $16 = $81; //@line 694 "lbfgsb.c"
 $82 = $2; //@line 695 "lbfgsb.c"
 $83 = HEAP32[$82>>2]|0; //@line 695 "lbfgsb.c"
 $wt_dim1 = $83; //@line 695 "lbfgsb.c"
 $84 = $wt_dim1; //@line 696 "lbfgsb.c"
 $85 = (1 + ($84))|0; //@line 696 "lbfgsb.c"
 $wt_offset = $85; //@line 696 "lbfgsb.c"
 $86 = $wt_offset; //@line 697 "lbfgsb.c"
 $87 = $15; //@line 697 "lbfgsb.c"
 $88 = (0 - ($86))|0; //@line 697 "lbfgsb.c"
 $89 = (($87) + ($88<<3)|0); //@line 697 "lbfgsb.c"
 $15 = $89; //@line 697 "lbfgsb.c"
 $90 = $2; //@line 698 "lbfgsb.c"
 $91 = HEAP32[$90>>2]|0; //@line 698 "lbfgsb.c"
 $ss_dim1 = $91; //@line 698 "lbfgsb.c"
 $92 = $ss_dim1; //@line 699 "lbfgsb.c"
 $93 = (1 + ($92))|0; //@line 699 "lbfgsb.c"
 $ss_offset = $93; //@line 699 "lbfgsb.c"
 $94 = $ss_offset; //@line 700 "lbfgsb.c"
 $95 = $14; //@line 700 "lbfgsb.c"
 $96 = (0 - ($94))|0; //@line 700 "lbfgsb.c"
 $97 = (($95) + ($96<<3)|0); //@line 700 "lbfgsb.c"
 $14 = $97; //@line 700 "lbfgsb.c"
 $98 = $2; //@line 701 "lbfgsb.c"
 $99 = HEAP32[$98>>2]|0; //@line 701 "lbfgsb.c"
 $sy_dim1 = $99; //@line 701 "lbfgsb.c"
 $100 = $sy_dim1; //@line 702 "lbfgsb.c"
 $101 = (1 + ($100))|0; //@line 702 "lbfgsb.c"
 $sy_offset = $101; //@line 702 "lbfgsb.c"
 $102 = $sy_offset; //@line 703 "lbfgsb.c"
 $103 = $13; //@line 703 "lbfgsb.c"
 $104 = (0 - ($102))|0; //@line 703 "lbfgsb.c"
 $105 = (($103) + ($104<<3)|0); //@line 703 "lbfgsb.c"
 $13 = $105; //@line 703 "lbfgsb.c"
 $106 = $1; //@line 704 "lbfgsb.c"
 $107 = HEAP32[$106>>2]|0; //@line 704 "lbfgsb.c"
 $wy_dim1 = $107; //@line 704 "lbfgsb.c"
 $108 = $wy_dim1; //@line 705 "lbfgsb.c"
 $109 = (1 + ($108))|0; //@line 705 "lbfgsb.c"
 $wy_offset = $109; //@line 705 "lbfgsb.c"
 $110 = $wy_offset; //@line 706 "lbfgsb.c"
 $111 = $12; //@line 706 "lbfgsb.c"
 $112 = (0 - ($110))|0; //@line 706 "lbfgsb.c"
 $113 = (($111) + ($112<<3)|0); //@line 706 "lbfgsb.c"
 $12 = $113; //@line 706 "lbfgsb.c"
 $114 = $1; //@line 707 "lbfgsb.c"
 $115 = HEAP32[$114>>2]|0; //@line 707 "lbfgsb.c"
 $ws_dim1 = $115; //@line 707 "lbfgsb.c"
 $116 = $ws_dim1; //@line 708 "lbfgsb.c"
 $117 = (1 + ($116))|0; //@line 708 "lbfgsb.c"
 $ws_offset = $117; //@line 708 "lbfgsb.c"
 $118 = $ws_offset; //@line 709 "lbfgsb.c"
 $119 = $11; //@line 709 "lbfgsb.c"
 $120 = (0 - ($118))|0; //@line 709 "lbfgsb.c"
 $121 = (($119) + ($120<<3)|0); //@line 709 "lbfgsb.c"
 $11 = $121; //@line 709 "lbfgsb.c"
 $122 = $30; //@line 710 "lbfgsb.c"
 $123 = ((($122)) + -4|0); //@line 710 "lbfgsb.c"
 $30 = $123; //@line 710 "lbfgsb.c"
 $124 = $31; //@line 711 "lbfgsb.c"
 $125 = ((($124)) + -4|0); //@line 711 "lbfgsb.c"
 $31 = $125; //@line 711 "lbfgsb.c"
 $126 = $32; //@line 712 "lbfgsb.c"
 $127 = ((($126)) + -8|0); //@line 712 "lbfgsb.c"
 $32 = $127; //@line 712 "lbfgsb.c"
 $128 = $27; //@line 715 "lbfgsb.c"
 $129 = (_s_cmp($128,13740,60,5)|0); //@line 715 "lbfgsb.c"
 $130 = ($129|0)==(0); //@line 715 "lbfgsb.c"
 //console.log("line 715 " + $130);
 L1: do {
  if ($130) {
   //console.log("line 716");
   $131 = (+_dlamch_(14350)); //@line 716 "lbfgsb.c"
   $132 = $131 * 2.0; //@line 716 "lbfgsb.c"
   HEAPF64[160>>3] = $132; //@line 716 "lbfgsb.c"
   (_timer_(96)|0); //@line 717 "lbfgsb.c"
   HEAP32[988>>2] = 0; //@line 720 "lbfgsb.c"
   HEAP32[996>>2] = 1; //@line 721 "lbfgsb.c"
   HEAPF64[120>>3] = 1.0; //@line 722 "lbfgsb.c"
   HEAP32[1060>>2] = 0; //@line 723 "lbfgsb.c"
   HEAP32[1052>>2] = 0; //@line 724 "lbfgsb.c"
   HEAP32[1024>>2] = 0; //@line 725 "lbfgsb.c"
   HEAP32[1036>>2] = 0; //@line 726 "lbfgsb.c"
   HEAP32[1044>>2] = 0; //@line 727 "lbfgsb.c"
   HEAP32[1000>>2] = 0; //@line 728 "lbfgsb.c"
   HEAP32[1048>>2] = 0; //@line 729 "lbfgsb.c"
   HEAP32[1068>>2] = 0; //@line 730 "lbfgsb.c"
   HEAPF64[72>>3] = 0.0; //@line 731 "lbfgsb.c"
   HEAPF64[128>>3] = 0.0; //@line 732 "lbfgsb.c"
   HEAPF64[56>>3] = 0.0; //@line 733 "lbfgsb.c"
   HEAPF64[8>>3] = 0.0; //@line 734 "lbfgsb.c"
   HEAPF64[144>>3] = 0.0; //@line 735 "lbfgsb.c"
   HEAPF64[176>>3] = 0.0; //@line 736 "lbfgsb.c"
   HEAPF64[48>>3] = 0.0; //@line 737 "lbfgsb.c"
   HEAPF64[112>>3] = 0.0; //@line 738 "lbfgsb.c"
   HEAPF64[32>>3] = 0.0; //@line 739 "lbfgsb.c"
   HEAP32[1020>>2] = 0; //@line 741 "lbfgsb.c"
   HEAP32[1012>>2] = 0; //@line 742 "lbfgsb.c"
   HEAP32[1008>>2] = 0; //@line 743 "lbfgsb.c"
   HEAP32[1072>>2] = 0; //@line 744 "lbfgsb.c"
   HEAP32[1040>>2] = 0; //@line 745 "lbfgsb.c"
   $133 = $1; //@line 746 "lbfgsb.c"
   $134 = HEAP32[$133>>2]|0; //@line 746 "lbfgsb.c"
   HEAP32[1028>>2] = $134; //@line 746 "lbfgsb.c"
   HEAP32[1016>>2] = 0; //@line 747 "lbfgsb.c"
   $135 = $9; //@line 749 "lbfgsb.c"
   $136 = +HEAPF64[$135>>3]; //@line 749 "lbfgsb.c"
   $137 = +HEAPF64[160>>3]; //@line 749 "lbfgsb.c"
   $138 = $136 * $137; //@line 749 "lbfgsb.c"
   HEAPF64[40>>3] = $138; //@line 749 "lbfgsb.c"
   HEAPF64[152>>3] = 0.0; //@line 751 "lbfgsb.c"
   HEAPF64[168>>3] = 0.0; //@line 752 "lbfgsb.c"
   HEAPF64[184>>3] = 0.0; //@line 753 "lbfgsb.c"
   _s_copy(14347,14352,3,3); //@line 755 "lbfgsb.c"
   HEAP32[1004>>2] = 0; //@line 757 "lbfgsb.c"
   $139 = $1; //@line 759 "lbfgsb.c"
   $140 = $2; //@line 759 "lbfgsb.c"
   $141 = $9; //@line 759 "lbfgsb.c"
   $142 = $4; //@line 759 "lbfgsb.c"
   $143 = ((($142)) + 8|0); //@line 759 "lbfgsb.c"
   $144 = $5; //@line 759 "lbfgsb.c"
   $145 = ((($144)) + 8|0); //@line 759 "lbfgsb.c"
   $146 = $6; //@line 759 "lbfgsb.c"
   $147 = ((($146)) + 4|0); //@line 759 "lbfgsb.c"
   $148 = $27; //@line 759 "lbfgsb.c"
   (_errclb_($139,$140,$141,$143,$145,$147,$148,1004,984,60)|0); //@line 759 "lbfgsb.c"
   $149 = $27; //@line 761 "lbfgsb.c"
   $150 = (_s_cmp($149,14356,5,5)|0); //@line 761 "lbfgsb.c"
   $151 = ($150|0)==(0); //@line 761 "lbfgsb.c"
   $152 = $1; //@line 762 "lbfgsb.c"
   if (!($151)) {
    $158 = $2; //@line 768 "lbfgsb.c"
    $159 = $4; //@line 768 "lbfgsb.c"
    $160 = ((($159)) + 8|0); //@line 768 "lbfgsb.c"
    $161 = $5; //@line 768 "lbfgsb.c"
    $162 = ((($161)) + 8|0); //@line 768 "lbfgsb.c"
    $163 = $3; //@line 768 "lbfgsb.c"
    $164 = ((($163)) + 8|0); //@line 768 "lbfgsb.c"
    $165 = $28; //@line 768 "lbfgsb.c"
    (_prn1lb_($152,$158,$160,$162,$164,$165,160)|0); //@line 768 "lbfgsb.c"
    $166 = $1; //@line 770 "lbfgsb.c"
    $167 = $4; //@line 770 "lbfgsb.c"
    $168 = ((($167)) + 8|0); //@line 770 "lbfgsb.c"
    $169 = $5; //@line 770 "lbfgsb.c"
    $170 = ((($169)) + 8|0); //@line 770 "lbfgsb.c"
    $171 = $6; //@line 770 "lbfgsb.c"
    $172 = ((($171)) + 4|0); //@line 770 "lbfgsb.c"
    $173 = $3; //@line 770 "lbfgsb.c"
    $174 = ((($173)) + 8|0); //@line 770 "lbfgsb.c"
    $175 = $25; //@line 770 "lbfgsb.c"
    $176 = ((($175)) + 4|0); //@line 770 "lbfgsb.c"
    $177 = $28; //@line 770 "lbfgsb.c"
    (_active_($166,$168,$170,$172,$174,$176,$177,1056,1064,1032)|0); //@line 770 "lbfgsb.c"
    label = 11;
    break;
   }
   $153 = $3; //@line 762 "lbfgsb.c"
   $154 = ((($153)) + 8|0); //@line 762 "lbfgsb.c"
   $155 = $7; //@line 762 "lbfgsb.c"
   $156 = $27; //@line 762 "lbfgsb.c"
   $157 = $28; //@line 762 "lbfgsb.c"
   (_prn3lb_($152,$154,$155,$156,$157,1004,1020,1012,1072,1040,1000,176,192,1008,14347,1024,48,136,984,152,168,184,60,3)|0); //@line 762 "lbfgsb.c"
   $0 = 0; //@line 766 "lbfgsb.c"
   $912 = $0; //@line 1181 "lbfgsb.c"
   STACKTOP = sp;return ($912|0); //@line 1181 "lbfgsb.c"
  } else {
   $178 = $30; //@line 775 "lbfgsb.c"
   $179 = ((($178)) + 4|0); //@line 775 "lbfgsb.c"
   $180 = HEAP32[$179>>2]|0; //@line 775 "lbfgsb.c"
   HEAP32[1056>>2] = $180; //@line 775 "lbfgsb.c"
   $181 = $30; //@line 776 "lbfgsb.c"
   $182 = ((($181)) + 8|0); //@line 776 "lbfgsb.c"
   $183 = HEAP32[$182>>2]|0; //@line 776 "lbfgsb.c"
   HEAP32[1064>>2] = $183; //@line 776 "lbfgsb.c"
   $184 = $30; //@line 777 "lbfgsb.c"
   $185 = ((($184)) + 12|0); //@line 777 "lbfgsb.c"
   $186 = HEAP32[$185>>2]|0; //@line 777 "lbfgsb.c"
   HEAP32[1032>>2] = $186; //@line 777 "lbfgsb.c"
   $187 = $30; //@line 778 "lbfgsb.c"
   $188 = ((($187)) + 16|0); //@line 778 "lbfgsb.c"
   $189 = HEAP32[$188>>2]|0; //@line 778 "lbfgsb.c"
   HEAP32[1052>>2] = $189; //@line 778 "lbfgsb.c"
   $190 = $31; //@line 779 "lbfgsb.c"
   $191 = ((($190)) + 4|0); //@line 779 "lbfgsb.c"
   $192 = HEAP32[$191>>2]|0; //@line 779 "lbfgsb.c"
   HEAP32[1072>>2] = $192; //@line 779 "lbfgsb.c"
   $193 = $31; //@line 780 "lbfgsb.c"
   $194 = ((($193)) + 16|0); //@line 780 "lbfgsb.c"
   $195 = HEAP32[$194>>2]|0; //@line 780 "lbfgsb.c"
   HEAP32[1024>>2] = $195; //@line 780 "lbfgsb.c"
   $196 = $31; //@line 781 "lbfgsb.c"
   $197 = ((($196)) + 20|0); //@line 781 "lbfgsb.c"
   $198 = HEAP32[$197>>2]|0; //@line 781 "lbfgsb.c"
   HEAP32[1040>>2] = $198; //@line 781 "lbfgsb.c"
   $199 = $31; //@line 782 "lbfgsb.c"
   $200 = ((($199)) + 24|0); //@line 782 "lbfgsb.c"
   $201 = HEAP32[$200>>2]|0; //@line 782 "lbfgsb.c"
   HEAP32[996>>2] = $201; //@line 782 "lbfgsb.c"
   $202 = $31; //@line 783 "lbfgsb.c"
   $203 = ((($202)) + 28|0); //@line 783 "lbfgsb.c"
   $204 = HEAP32[$203>>2]|0; //@line 783 "lbfgsb.c"
   HEAP32[988>>2] = $204; //@line 783 "lbfgsb.c"
   $205 = $31; //@line 784 "lbfgsb.c"
   $206 = ((($205)) + 32|0); //@line 784 "lbfgsb.c"
   $207 = HEAP32[$206>>2]|0; //@line 784 "lbfgsb.c"
   HEAP32[1036>>2] = $207; //@line 784 "lbfgsb.c"
   $208 = $31; //@line 785 "lbfgsb.c"
   $209 = ((($208)) + 36|0); //@line 785 "lbfgsb.c"
   $210 = HEAP32[$209>>2]|0; //@line 785 "lbfgsb.c"
   HEAP32[1020>>2] = $210; //@line 785 "lbfgsb.c"
   $211 = $31; //@line 786 "lbfgsb.c"
   $212 = ((($211)) + 40|0); //@line 786 "lbfgsb.c"
   $213 = HEAP32[$212>>2]|0; //@line 786 "lbfgsb.c"
   HEAP32[1060>>2] = $213; //@line 786 "lbfgsb.c"
   $214 = $31; //@line 787 "lbfgsb.c"
   $215 = ((($214)) + 48|0); //@line 787 "lbfgsb.c"
   $216 = HEAP32[$215>>2]|0; //@line 787 "lbfgsb.c"
   HEAP32[1008>>2] = $216; //@line 787 "lbfgsb.c"
   $217 = $31; //@line 788 "lbfgsb.c"
   $218 = ((($217)) + 52|0); //@line 788 "lbfgsb.c"
   $219 = HEAP32[$218>>2]|0; //@line 788 "lbfgsb.c"
   HEAP32[1012>>2] = $219; //@line 788 "lbfgsb.c"
   $220 = $31; //@line 789 "lbfgsb.c"
   $221 = ((($220)) + 56|0); //@line 789 "lbfgsb.c"
   $222 = HEAP32[$221>>2]|0; //@line 789 "lbfgsb.c"
   HEAP32[1004>>2] = $222; //@line 789 "lbfgsb.c"
   $223 = $31; //@line 790 "lbfgsb.c"
   $224 = ((($223)) + 60|0); //@line 790 "lbfgsb.c"
   $225 = HEAP32[$224>>2]|0; //@line 790 "lbfgsb.c"
   HEAP32[1016>>2] = $225; //@line 790 "lbfgsb.c"
   $226 = $31; //@line 791 "lbfgsb.c"
   $227 = ((($226)) + 64|0); //@line 791 "lbfgsb.c"
   $228 = HEAP32[$227>>2]|0; //@line 791 "lbfgsb.c"
   HEAP32[1044>>2] = $228; //@line 791 "lbfgsb.c"
   $229 = $31; //@line 792 "lbfgsb.c"
   $230 = ((($229)) + 68|0); //@line 792 "lbfgsb.c"
   $231 = HEAP32[$230>>2]|0; //@line 792 "lbfgsb.c"
   HEAP32[1028>>2] = $231; //@line 792 "lbfgsb.c"
   $232 = $31; //@line 793 "lbfgsb.c"
   $233 = ((($232)) + 72|0); //@line 793 "lbfgsb.c"
   $234 = HEAP32[$233>>2]|0; //@line 793 "lbfgsb.c"
   HEAP32[1000>>2] = $234; //@line 793 "lbfgsb.c"
   $235 = $31; //@line 794 "lbfgsb.c"
   $236 = ((($235)) + 76|0); //@line 794 "lbfgsb.c"
   $237 = HEAP32[$236>>2]|0; //@line 794 "lbfgsb.c"
   HEAP32[1048>>2] = $237; //@line 794 "lbfgsb.c"
   $238 = $31; //@line 795 "lbfgsb.c"
   $239 = ((($238)) + 80|0); //@line 795 "lbfgsb.c"
   $240 = HEAP32[$239>>2]|0; //@line 795 "lbfgsb.c"
   HEAP32[1068>>2] = $240; //@line 795 "lbfgsb.c"
   $241 = $32; //@line 796 "lbfgsb.c"
   $242 = ((($241)) + 8|0); //@line 796 "lbfgsb.c"
   $243 = +HEAPF64[$242>>3]; //@line 796 "lbfgsb.c"
   HEAPF64[120>>3] = $243; //@line 796 "lbfgsb.c"
   $244 = $32; //@line 797 "lbfgsb.c"
   $245 = ((($244)) + 16|0); //@line 797 "lbfgsb.c"
   $246 = +HEAPF64[$245>>3]; //@line 797 "lbfgsb.c"
   HEAPF64[72>>3] = $246; //@line 797 "lbfgsb.c"
   $247 = $32; //@line 798 "lbfgsb.c"
   $248 = ((($247)) + 24|0); //@line 798 "lbfgsb.c"
   $249 = +HEAPF64[$248>>3]; //@line 798 "lbfgsb.c"
   HEAPF64[40>>3] = $249; //@line 798 "lbfgsb.c"
   $250 = $32; //@line 799 "lbfgsb.c"
   $251 = ((($250)) + 32|0); //@line 799 "lbfgsb.c"
   $252 = +HEAPF64[$251>>3]; //@line 799 "lbfgsb.c"
   HEAPF64[128>>3] = $252; //@line 799 "lbfgsb.c"
   $253 = $32; //@line 800 "lbfgsb.c"
   $254 = ((($253)) + 40|0); //@line 800 "lbfgsb.c"
   $255 = +HEAPF64[$254>>3]; //@line 800 "lbfgsb.c"
   HEAPF64[160>>3] = $255; //@line 800 "lbfgsb.c"
   $256 = $32; //@line 801 "lbfgsb.c"
   $257 = ((($256)) + 48|0); //@line 801 "lbfgsb.c"
   $258 = +HEAPF64[$257>>3]; //@line 801 "lbfgsb.c"
   HEAPF64[56>>3] = $258; //@line 801 "lbfgsb.c"
   $259 = $32; //@line 802 "lbfgsb.c"
   $260 = ((($259)) + 56|0); //@line 802 "lbfgsb.c"
   $261 = +HEAPF64[$260>>3]; //@line 802 "lbfgsb.c"
   HEAPF64[152>>3] = $261; //@line 802 "lbfgsb.c"
   $262 = $32; //@line 803 "lbfgsb.c"
   $263 = ((($262)) + 64|0); //@line 803 "lbfgsb.c"
   $264 = +HEAPF64[$263>>3]; //@line 803 "lbfgsb.c"
   HEAPF64[168>>3] = $264; //@line 803 "lbfgsb.c"
   $265 = $32; //@line 804 "lbfgsb.c"
   $266 = ((($265)) + 72|0); //@line 804 "lbfgsb.c"
   $267 = +HEAPF64[$266>>3]; //@line 804 "lbfgsb.c"
   HEAPF64[184>>3] = $267; //@line 804 "lbfgsb.c"
   $268 = $32; //@line 805 "lbfgsb.c"
   $269 = ((($268)) + 80|0); //@line 805 "lbfgsb.c"
   $270 = +HEAPF64[$269>>3]; //@line 805 "lbfgsb.c"
   HEAPF64[96>>3] = $270; //@line 805 "lbfgsb.c"
   $271 = $32; //@line 806 "lbfgsb.c"
   $272 = ((($271)) + 88|0); //@line 806 "lbfgsb.c"
   $273 = +HEAPF64[$272>>3]; //@line 806 "lbfgsb.c"
   HEAPF64[8>>3] = $273; //@line 806 "lbfgsb.c"
   $274 = $32; //@line 807 "lbfgsb.c"
   $275 = ((($274)) + 96|0); //@line 807 "lbfgsb.c"
   $276 = +HEAPF64[$275>>3]; //@line 807 "lbfgsb.c"
   HEAPF64[144>>3] = $276; //@line 807 "lbfgsb.c"
   $277 = $32; //@line 808 "lbfgsb.c"
   $278 = ((($277)) + 104|0); //@line 808 "lbfgsb.c"
   $279 = +HEAPF64[$278>>3]; //@line 808 "lbfgsb.c"
   HEAPF64[176>>3] = $279; //@line 808 "lbfgsb.c"
   $280 = $32; //@line 809 "lbfgsb.c"
   $281 = ((($280)) + 112|0); //@line 809 "lbfgsb.c"
   $282 = +HEAPF64[$281>>3]; //@line 809 "lbfgsb.c"
   HEAPF64[48>>3] = $282; //@line 809 "lbfgsb.c"
   $283 = $32; //@line 810 "lbfgsb.c"
   $284 = ((($283)) + 120|0); //@line 810 "lbfgsb.c"
   $285 = +HEAPF64[$284>>3]; //@line 810 "lbfgsb.c"
   HEAPF64[112>>3] = $285; //@line 810 "lbfgsb.c"
   $286 = $32; //@line 811 "lbfgsb.c"
   $287 = ((($286)) + 128|0); //@line 811 "lbfgsb.c"
   $288 = +HEAPF64[$287>>3]; //@line 811 "lbfgsb.c"
   HEAPF64[32>>3] = $288; //@line 811 "lbfgsb.c"
   $289 = $27; //@line 814 "lbfgsb.c"
   $290 = (_s_cmp($289,14362,5,5)|0); //@line 814 "lbfgsb.c"
   $291 = ($290|0)==(0); //@line 814 "lbfgsb.c"
   do {
    if ($291) {
     label = 43;
    } else {
     $292 = $27; //@line 817 "lbfgsb.c"
     $293 = (_s_cmp($292,14368,5,5)|0); //@line 817 "lbfgsb.c"
     $294 = ($293|0)==(0); //@line 817 "lbfgsb.c"
     if (!($294)) {
      $295 = $27; //@line 820 "lbfgsb.c"
      $296 = (_s_cmp($295,14374,5,5)|0); //@line 820 "lbfgsb.c"
      $297 = ($296|0)==(0); //@line 820 "lbfgsb.c"
      if (!($297)) {
       $298 = $27; //@line 823 "lbfgsb.c"
       $299 = (_s_cmp($298,14380,4,4)|0); //@line 823 "lbfgsb.c"
       $300 = ($299|0)==(0); //@line 823 "lbfgsb.c"
       if (!($300)) {
        label = 11;
        break L1;
       }
       $301 = $27; //@line 824 "lbfgsb.c"
       $302 = ((($301)) + 6|0); //@line 824 "lbfgsb.c"
       $303 = (_s_cmp($302,14385,3,3)|0); //@line 824 "lbfgsb.c"
       $304 = ($303|0)==(0); //@line 824 "lbfgsb.c"
       if (!($304)) {
        label = 74;
        break;
       }
       $305 = $1; //@line 826 "lbfgsb.c"
       $306 = $21; //@line 826 "lbfgsb.c"
       $307 = ((($306)) + 8|0); //@line 826 "lbfgsb.c"
       $308 = $3; //@line 826 "lbfgsb.c"
       $309 = ((($308)) + 8|0); //@line 826 "lbfgsb.c"
       (_dcopy_($305,$307,1236,$309,1236)|0); //@line 826 "lbfgsb.c"
       $310 = $1; //@line 827 "lbfgsb.c"
       $311 = $19; //@line 827 "lbfgsb.c"
       $312 = ((($311)) + 8|0); //@line 827 "lbfgsb.c"
       $313 = $8; //@line 827 "lbfgsb.c"
       $314 = ((($313)) + 8|0); //@line 827 "lbfgsb.c"
       (_dcopy_($310,$312,1236,$314,1236)|0); //@line 827 "lbfgsb.c"
       $315 = +HEAPF64[72>>3]; //@line 828 "lbfgsb.c"
       $316 = $7; //@line 828 "lbfgsb.c"
       HEAPF64[$316>>3] = $315; //@line 828 "lbfgsb.c"
       label = 74;
       break;
      }
      HEAP32[1012>>2] = 1; //@line 838 "lbfgsb.c"
      $318 = $1; //@line 840 "lbfgsb.c"
      $319 = $4; //@line 840 "lbfgsb.c"
      $320 = ((($319)) + 8|0); //@line 840 "lbfgsb.c"
      $321 = $5; //@line 840 "lbfgsb.c"
      $322 = ((($321)) + 8|0); //@line 840 "lbfgsb.c"
      $323 = $6; //@line 840 "lbfgsb.c"
      $324 = ((($323)) + 4|0); //@line 840 "lbfgsb.c"
      $325 = $3; //@line 840 "lbfgsb.c"
      $326 = ((($325)) + 8|0); //@line 840 "lbfgsb.c"
      $327 = $8; //@line 840 "lbfgsb.c"
      $328 = ((($327)) + 8|0); //@line 840 "lbfgsb.c"
      (_projgr_($318,$320,$322,$324,$326,$328,176)|0); //@line 840 "lbfgsb.c"
      $329 = $28; //@line 841 "lbfgsb.c"
      $330 = HEAP32[$329>>2]|0; //@line 841 "lbfgsb.c"
      $331 = ($330|0)>=(1); //@line 841 "lbfgsb.c"
      if ($331) {
       (_s_wsfe(1076)|0); //@line 842 "lbfgsb.c"
       (_do_fio(1236,1020,4)|0); //@line 843 "lbfgsb.c"
       $332 = $7; //@line 844 "lbfgsb.c"
       (_do_fio(1236,$332,8)|0); //@line 844 "lbfgsb.c"
       (_do_fio(1236,176,8)|0); //@line 845 "lbfgsb.c"
       (_e_wsfe()|0); //@line 846 "lbfgsb.c"
      }
      $333 = +HEAPF64[176>>3]; //@line 848 "lbfgsb.c"
      $334 = $10; //@line 848 "lbfgsb.c"
      $335 = +HEAPF64[$334>>3]; //@line 848 "lbfgsb.c"
      $336 = $333 <= $335; //@line 848 "lbfgsb.c"
      if (!($336)) {
       label = 16;
       break;
      }
      //console.log("line 850: " + $333 + " " + $335);
      $337 = $27; //@line 850 "lbfgsb.c"
      _s_copy($337,14398,60,48); //@line 850 "lbfgsb.c"
      label = 74;
      break;
     }
     $668 = +HEAPF64[176>>3]; //@line 1050 "lbfgsb.c"
     $669 = $10; //@line 1050 "lbfgsb.c"
     $670 = +HEAPF64[$669>>3]; //@line 1050 "lbfgsb.c"
     $671 = $668 <= $670; //@line 1050 "lbfgsb.c"
     if ($671) {
      $672 = $27; //@line 1052 "lbfgsb.c"
      _s_copy($672,14398,60,48); //@line 1052 "lbfgsb.c"
      label = 74;
      break;
     }
     $673 = +HEAPF64[72>>3]; //@line 1057 "lbfgsb.c"
     $674 = $673 >= 0.0; //@line 1057 "lbfgsb.c"
     $675 = +HEAPF64[72>>3]; //@line 1057 "lbfgsb.c"
     $676 = -$675; //@line 1057 "lbfgsb.c"
     $677 = $674 ? $675 : $676; //@line 1057 "lbfgsb.c"
     $d__1 = $677; //@line 1057 "lbfgsb.c"
     $678 = $7; //@line 1057 "lbfgsb.c"
     $679 = +HEAPF64[$678>>3]; //@line 1057 "lbfgsb.c"
     $680 = $679 >= 0.0; //@line 1057 "lbfgsb.c"
     $681 = $7; //@line 1057 "lbfgsb.c"
     $682 = +HEAPF64[$681>>3]; //@line 1057 "lbfgsb.c"
     $683 = -$682; //@line 1057 "lbfgsb.c"
     $684 = $680 ? $682 : $683; //@line 1057 "lbfgsb.c"
     $d__2 = $684; //@line 1057 "lbfgsb.c"
     $685 = $d__1; //@line 1057 "lbfgsb.c"
     $686 = $d__2; //@line 1057 "lbfgsb.c"
     $687 = $685 >= $686; //@line 1057 "lbfgsb.c"
     $688 = $d__1; //@line 1057 "lbfgsb.c"
     $689 = $d__2; //@line 1057 "lbfgsb.c"
     $690 = $687 ? $688 : $689; //@line 1057 "lbfgsb.c"
     $d__1 = $690; //@line 1057 "lbfgsb.c"
     $691 = $d__1; //@line 1058 "lbfgsb.c"
     $692 = $691 >= 1.0; //@line 1058 "lbfgsb.c"
     $693 = $d__1; //@line 1058 "lbfgsb.c"
     $694 = $692 ? $693 : 1.0; //@line 1058 "lbfgsb.c"
     HEAPF64[80>>3] = $694; //@line 1058 "lbfgsb.c"
     $695 = +HEAPF64[72>>3]; //@line 1059 "lbfgsb.c"
     $696 = $7; //@line 1059 "lbfgsb.c"
     $697 = +HEAPF64[$696>>3]; //@line 1059 "lbfgsb.c"
     $698 = $695 - $697; //@line 1059 "lbfgsb.c"
     $699 = +HEAPF64[40>>3]; //@line 1059 "lbfgsb.c"
     $700 = +HEAPF64[80>>3]; //@line 1059 "lbfgsb.c"
     $701 = $699 * $700; //@line 1059 "lbfgsb.c"
     $702 = $698 <= $701; //@line 1059 "lbfgsb.c"
     if ($702) {
      $703 = $27; //@line 1061 "lbfgsb.c"
      _s_copy($703,14498,60,47); //@line 1061 "lbfgsb.c"
      $704 = HEAP32[1024>>2]|0; //@line 1063 "lbfgsb.c"
      $705 = ($704|0)>=(10); //@line 1063 "lbfgsb.c"
      if (!($705)) {
       label = 74;
       break;
      }
      HEAP32[1004>>2] = -5; //@line 1064 "lbfgsb.c"
      label = 74;
      break;
     }
     $706 = $1; //@line 1070 "lbfgsb.c"
     $707 = HEAP32[$706>>2]|0; //@line 1070 "lbfgsb.c"
     HEAP32[$i__1>>2] = $707; //@line 1070 "lbfgsb.c"
     HEAP32[980>>2] = 1; //@line 1071 "lbfgsb.c"
     while(1) {
      $708 = HEAP32[980>>2]|0; //@line 1071 "lbfgsb.c"
      $709 = HEAP32[$i__1>>2]|0; //@line 1071 "lbfgsb.c"
      $710 = ($708|0)<=($709|0); //@line 1071 "lbfgsb.c"
      if (!($710)) {
       break;
      }
      $711 = HEAP32[980>>2]|0; //@line 1072 "lbfgsb.c"
      $712 = $8; //@line 1072 "lbfgsb.c"
      $713 = (($712) + ($711<<3)|0); //@line 1072 "lbfgsb.c"
      $714 = +HEAPF64[$713>>3]; //@line 1072 "lbfgsb.c"
      $715 = HEAP32[980>>2]|0; //@line 1072 "lbfgsb.c"
      $716 = $19; //@line 1072 "lbfgsb.c"
      $717 = (($716) + ($715<<3)|0); //@line 1072 "lbfgsb.c"
      $718 = +HEAPF64[$717>>3]; //@line 1072 "lbfgsb.c"
      $719 = $714 - $718; //@line 1072 "lbfgsb.c"
      $720 = HEAP32[980>>2]|0; //@line 1072 "lbfgsb.c"
      $721 = $19; //@line 1072 "lbfgsb.c"
      $722 = (($721) + ($720<<3)|0); //@line 1072 "lbfgsb.c"
      HEAPF64[$722>>3] = $719; //@line 1072 "lbfgsb.c"
      $723 = HEAP32[980>>2]|0; //@line 1071 "lbfgsb.c"
      $724 = (($723) + 1)|0; //@line 1071 "lbfgsb.c"
      HEAP32[980>>2] = $724; //@line 1071 "lbfgsb.c"
     }
     $725 = $1; //@line 1075 "lbfgsb.c"
     $726 = $19; //@line 1075 "lbfgsb.c"
     $727 = ((($726)) + 8|0); //@line 1075 "lbfgsb.c"
     $728 = $19; //@line 1075 "lbfgsb.c"
     $729 = ((($728)) + 8|0); //@line 1075 "lbfgsb.c"
     $730 = (+_ddot_($725,$727,1236,$729,1236)); //@line 1075 "lbfgsb.c"
     HEAPF64[24>>3] = $730; //@line 1075 "lbfgsb.c"
     $731 = +HEAPF64[48>>3]; //@line 1076 "lbfgsb.c"
     $732 = $731 == 1.0; //@line 1076 "lbfgsb.c"
     $733 = +HEAPF64[8>>3]; //@line 1077 "lbfgsb.c"
     $734 = +HEAPF64[112>>3]; //@line 1077 "lbfgsb.c"
     $735 = $733 - $734; //@line 1077 "lbfgsb.c"
     if ($732) {
      HEAPF64[16>>3] = $735; //@line 1077 "lbfgsb.c"
      $736 = +HEAPF64[112>>3]; //@line 1078 "lbfgsb.c"
      $737 = -$736; //@line 1078 "lbfgsb.c"
      HEAPF64[80>>3] = $737; //@line 1078 "lbfgsb.c"
     } else {
      $738 = +HEAPF64[48>>3]; //@line 1080 "lbfgsb.c"
      $739 = $735 * $738; //@line 1080 "lbfgsb.c"
      HEAPF64[16>>3] = $739; //@line 1080 "lbfgsb.c"
      $740 = $1; //@line 1081 "lbfgsb.c"
      $741 = $20; //@line 1081 "lbfgsb.c"
      $742 = ((($741)) + 8|0); //@line 1081 "lbfgsb.c"
      (_dscal_($740,48,$742,1236)|0); //@line 1081 "lbfgsb.c"
      $743 = +HEAPF64[112>>3]; //@line 1082 "lbfgsb.c"
      $744 = -$743; //@line 1082 "lbfgsb.c"
      $745 = +HEAPF64[48>>3]; //@line 1082 "lbfgsb.c"
      $746 = $744 * $745; //@line 1082 "lbfgsb.c"
      HEAPF64[80>>3] = $746; //@line 1082 "lbfgsb.c"
     }
     $747 = +HEAPF64[16>>3]; //@line 1084 "lbfgsb.c"
     $748 = +HEAPF64[160>>3]; //@line 1084 "lbfgsb.c"
     $749 = +HEAPF64[80>>3]; //@line 1084 "lbfgsb.c"
     $750 = $748 * $749; //@line 1084 "lbfgsb.c"
     $751 = $747 <= $750; //@line 1084 "lbfgsb.c"
     if ($751) {
      $752 = HEAP32[1040>>2]|0; //@line 1086 "lbfgsb.c"
      $753 = (($752) + 1)|0; //@line 1086 "lbfgsb.c"
      HEAP32[1040>>2] = $753; //@line 1086 "lbfgsb.c"
      HEAP32[1052>>2] = 0; //@line 1087 "lbfgsb.c"
      $754 = $28; //@line 1088 "lbfgsb.c"
      $755 = HEAP32[$754>>2]|0; //@line 1088 "lbfgsb.c"
      $756 = ($755|0)>=(1); //@line 1088 "lbfgsb.c"
      if (!($756)) {
       label = 16;
       break;
      }
      (_s_wsfe(1196)|0); //@line 1089 "lbfgsb.c"
      (_do_fio(1236,16,8)|0); //@line 1090 "lbfgsb.c"
      (_do_fio(1236,80,8)|0); //@line 1091 "lbfgsb.c"
      (_e_wsfe()|0); //@line 1092 "lbfgsb.c"
      label = 16;
      break;
     }
     HEAP32[1052>>2] = 1; //@line 1101 "lbfgsb.c"
     $757 = HEAP32[1060>>2]|0; //@line 1102 "lbfgsb.c"
     $758 = (($757) + 1)|0; //@line 1102 "lbfgsb.c"
     HEAP32[1060>>2] = $758; //@line 1102 "lbfgsb.c"
     $759 = $1; //@line 1104 "lbfgsb.c"
     $760 = $2; //@line 1104 "lbfgsb.c"
     $761 = $ws_offset; //@line 1104 "lbfgsb.c"
     $762 = $11; //@line 1104 "lbfgsb.c"
     $763 = (($762) + ($761<<3)|0); //@line 1104 "lbfgsb.c"
     $764 = $wy_offset; //@line 1104 "lbfgsb.c"
     $765 = $12; //@line 1104 "lbfgsb.c"
     $766 = (($765) + ($764<<3)|0); //@line 1104 "lbfgsb.c"
     $767 = $sy_offset; //@line 1104 "lbfgsb.c"
     $768 = $13; //@line 1104 "lbfgsb.c"
     $769 = (($768) + ($767<<3)|0); //@line 1104 "lbfgsb.c"
     $770 = $ss_offset; //@line 1105 "lbfgsb.c"
     $771 = $14; //@line 1104 "lbfgsb.c"
     $772 = (($771) + ($770<<3)|0); //@line 1104 "lbfgsb.c"
     $773 = $20; //@line 1105 "lbfgsb.c"
     $774 = ((($773)) + 8|0); //@line 1105 "lbfgsb.c"
     $775 = $19; //@line 1105 "lbfgsb.c"
     $776 = ((($775)) + 8|0); //@line 1105 "lbfgsb.c"
     (_matupd_($759,$760,$763,$766,$769,$772,$774,$776,1036,1060,988,996,120,24,16,48,32)|0); //@line 1104 "lbfgsb.c"
     $777 = $2; //@line 1111 "lbfgsb.c"
     $778 = $wt_offset; //@line 1111 "lbfgsb.c"
     $779 = $15; //@line 1111 "lbfgsb.c"
     $780 = (($779) + ($778<<3)|0); //@line 1111 "lbfgsb.c"
     $781 = $sy_offset; //@line 1111 "lbfgsb.c"
     $782 = $13; //@line 1111 "lbfgsb.c"
     $783 = (($782) + ($781<<3)|0); //@line 1111 "lbfgsb.c"
     $784 = $ss_offset; //@line 1111 "lbfgsb.c"
     $785 = $14; //@line 1111 "lbfgsb.c"
     $786 = (($785) + ($784<<3)|0); //@line 1111 "lbfgsb.c"
     (_formt_($777,$780,$783,$786,988,120,1004)|0); //@line 1111 "lbfgsb.c"
     $787 = HEAP32[1004>>2]|0; //@line 1113 "lbfgsb.c"
     $788 = ($787|0)!=(0); //@line 1113 "lbfgsb.c"
     if ($788) {
      $789 = $28; //@line 1116 "lbfgsb.c"
      $790 = HEAP32[$789>>2]|0; //@line 1116 "lbfgsb.c"
      $791 = ($790|0)>=(1); //@line 1116 "lbfgsb.c"
      if ($791) {
       (_s_wsfe(1216)|0); //@line 1117 "lbfgsb.c"
       (_e_wsfe()|0); //@line 1118 "lbfgsb.c"
      }
      HEAP32[1004>>2] = 0; //@line 1120 "lbfgsb.c"
      HEAP32[988>>2] = 0; //@line 1121 "lbfgsb.c"
      HEAP32[996>>2] = 1; //@line 1122 "lbfgsb.c"
      HEAPF64[120>>3] = 1.0; //@line 1123 "lbfgsb.c"
      HEAP32[1060>>2] = 0; //@line 1124 "lbfgsb.c"
      HEAP32[1052>>2] = 0; //@line 1125 "lbfgsb.c"
      label = 16;
     } else {
      label = 16;
     }
    }
   } while(0);
   L45: while(1) {
    if ((label|0) == 16) {
     label = 0;
     $338 = $28; //@line 856 "lbfgsb.c"
     $339 = HEAP32[$338>>2]|0; //@line 856 "lbfgsb.c"
     $340 = ($339|0)>=(99); //@line 856 "lbfgsb.c"
     if ($340) {
      (_s_wsfe(1096)|0); //@line 857 "lbfgsb.c"
      $341 = HEAP32[1020>>2]|0; //@line 858 "lbfgsb.c"
      $342 = (($341) + 1)|0; //@line 858 "lbfgsb.c"
      HEAP32[$i__1>>2] = $342; //@line 858 "lbfgsb.c"
      (_do_fio(1236,$i__1,4)|0); //@line 859 "lbfgsb.c"
      (_e_wsfe()|0); //@line 860 "lbfgsb.c"
     }
     HEAP32[1044>>2] = -1; //@line 862 "lbfgsb.c"
     $343 = HEAP32[1064>>2]|0; //@line 864 "lbfgsb.c"
     $344 = ($343|0)==(0); //@line 864 "lbfgsb.c"
     $345 = HEAP32[988>>2]|0;
     $346 = ($345|0)>(0); //@line 864 "lbfgsb.c"
     $or$cond = $344 & $346; //@line 864 "lbfgsb.c"
     do {
      if ($or$cond) {
       $347 = $1; //@line 866 "lbfgsb.c"
       $348 = $3; //@line 866 "lbfgsb.c"
       $349 = ((($348)) + 8|0); //@line 866 "lbfgsb.c"
       $350 = $18; //@line 866 "lbfgsb.c"
       $351 = ((($350)) + 8|0); //@line 866 "lbfgsb.c"
       (_dcopy_($347,$349,1236,$351,1236)|0); //@line 866 "lbfgsb.c"
       $352 = HEAP32[1052>>2]|0; //@line 867 "lbfgsb.c"
       HEAP32[992>>2] = $352; //@line 867 "lbfgsb.c"
       HEAP32[1008>>2] = 0; //@line 868 "lbfgsb.c"
      } else {
       (_timer_(56)|0); //@line 876 "lbfgsb.c"
       $353 = $1; //@line 877 "lbfgsb.c"
       $354 = $3; //@line 877 "lbfgsb.c"
       $355 = ((($354)) + 8|0); //@line 877 "lbfgsb.c"
       $356 = $4; //@line 877 "lbfgsb.c"
       $357 = ((($356)) + 8|0); //@line 877 "lbfgsb.c"
       $358 = $5; //@line 877 "lbfgsb.c"
       $359 = ((($358)) + 8|0); //@line 877 "lbfgsb.c"
       $360 = $6; //@line 877 "lbfgsb.c"
       $361 = ((($360)) + 4|0); //@line 877 "lbfgsb.c"
       $362 = $8; //@line 877 "lbfgsb.c"
       $363 = ((($362)) + 8|0); //@line 877 "lbfgsb.c"
       $364 = $26; //@line 877 "lbfgsb.c"
       $365 = ((($364)) + 4|0); //@line 877 "lbfgsb.c"
       $366 = $25; //@line 877 "lbfgsb.c"
       $367 = ((($366)) + 4|0); //@line 877 "lbfgsb.c"
       $368 = $21; //@line 877 "lbfgsb.c"
       $369 = ((($368)) + 8|0); //@line 877 "lbfgsb.c"
       $370 = $20; //@line 878 "lbfgsb.c"
       $371 = ((($370)) + 8|0); //@line 878 "lbfgsb.c"
       $372 = $18; //@line 878 "lbfgsb.c"
       $373 = ((($372)) + 8|0); //@line 878 "lbfgsb.c"
       $374 = $2; //@line 878 "lbfgsb.c"
       $375 = $wy_offset; //@line 878 "lbfgsb.c"
       $376 = $12; //@line 878 "lbfgsb.c"
       $377 = (($376) + ($375<<3)|0); //@line 878 "lbfgsb.c"
       $378 = $ws_offset; //@line 878 "lbfgsb.c"
       $379 = $11; //@line 878 "lbfgsb.c"
       $380 = (($379) + ($378<<3)|0); //@line 878 "lbfgsb.c"
       $381 = $sy_offset; //@line 879 "lbfgsb.c"
       $382 = $13; //@line 878 "lbfgsb.c"
       $383 = (($382) + ($381<<3)|0); //@line 878 "lbfgsb.c"
       $384 = $wt_offset; //@line 879 "lbfgsb.c"
       $385 = $15; //@line 879 "lbfgsb.c"
       $386 = (($385) + ($384<<3)|0); //@line 879 "lbfgsb.c"
       $387 = $23; //@line 879 "lbfgsb.c"
       $388 = ((($387)) + 8|0); //@line 879 "lbfgsb.c"
       $389 = $2; //@line 879 "lbfgsb.c"
       $390 = HEAP32[$389>>2]|0; //@line 879 "lbfgsb.c"
       $391 = $390 << 1; //@line 880 "lbfgsb.c"
       $392 = (($391) + 1)|0; //@line 880 "lbfgsb.c"
       $393 = $23; //@line 879 "lbfgsb.c"
       $394 = (($393) + ($392<<3)|0); //@line 879 "lbfgsb.c"
       $395 = $2; //@line 880 "lbfgsb.c"
       $396 = HEAP32[$395>>2]|0; //@line 880 "lbfgsb.c"
       $397 = $396 << 2; //@line 880 "lbfgsb.c"
       $398 = (($397) + 1)|0; //@line 880 "lbfgsb.c"
       $399 = $23; //@line 880 "lbfgsb.c"
       $400 = (($399) + ($398<<3)|0); //@line 880 "lbfgsb.c"
       $401 = $2; //@line 880 "lbfgsb.c"
       $402 = HEAP32[$401>>2]|0; //@line 880 "lbfgsb.c"
       $403 = ($402*6)|0; //@line 880 "lbfgsb.c"
       $404 = (($403) + 1)|0; //@line 880 "lbfgsb.c"
       $405 = $23; //@line 880 "lbfgsb.c"
       $406 = (($405) + ($404<<3)|0); //@line 880 "lbfgsb.c"
       $407 = $28; //@line 880 "lbfgsb.c"
       (_cauchy_($353,$355,$357,$359,$361,$363,$365,$367,$369,$371,$373,$374,$377,$380,$383,$386,120,988,996,$388,$394,$400,$406,1008,$407,176,1004,160)|0); //@line 877 "lbfgsb.c"
       $408 = HEAP32[1004>>2]|0; //@line 882 "lbfgsb.c"
       $409 = ($408|0)!=(0); //@line 882 "lbfgsb.c"
       if (!($409)) {
        (_timer_(64)|0); //@line 898 "lbfgsb.c"
        $418 = +HEAPF64[152>>3]; //@line 899 "lbfgsb.c"
        $419 = +HEAPF64[64>>3]; //@line 899 "lbfgsb.c"
        $420 = $418 + $419; //@line 899 "lbfgsb.c"
        $421 = +HEAPF64[56>>3]; //@line 899 "lbfgsb.c"
        $422 = $420 - $421; //@line 899 "lbfgsb.c"
        HEAPF64[152>>3] = $422; //@line 899 "lbfgsb.c"
        $423 = HEAP32[1008>>2]|0; //@line 900 "lbfgsb.c"
        $424 = HEAP32[1072>>2]|0; //@line 900 "lbfgsb.c"
        $425 = (($424) + ($423))|0; //@line 900 "lbfgsb.c"
        HEAP32[1072>>2] = $425; //@line 900 "lbfgsb.c"
        $426 = $1; //@line 903 "lbfgsb.c"
        $427 = $24; //@line 903 "lbfgsb.c"
        $428 = ((($427)) + 4|0); //@line 903 "lbfgsb.c"
        $429 = $26; //@line 903 "lbfgsb.c"
        $430 = ((($429)) + 4|0); //@line 903 "lbfgsb.c"
        $431 = $25; //@line 903 "lbfgsb.c"
        $432 = ((($431)) + 4|0); //@line 903 "lbfgsb.c"
        $433 = $28; //@line 904 "lbfgsb.c"
        (_freev_($426,1028,$428,1068,1048,$430,$432,992,1052,1064,$433,1020)|0); //@line 903 "lbfgsb.c"
        $434 = $1; //@line 905 "lbfgsb.c"
        $435 = HEAP32[$434>>2]|0; //@line 905 "lbfgsb.c"
        $436 = HEAP32[1028>>2]|0; //@line 905 "lbfgsb.c"
        $437 = (($435) - ($436))|0; //@line 905 "lbfgsb.c"
        HEAP32[1000>>2] = $437; //@line 905 "lbfgsb.c"
        break;
       }
       $410 = $28; //@line 884 "lbfgsb.c"
       $411 = HEAP32[$410>>2]|0; //@line 884 "lbfgsb.c"
       $412 = ($411|0)>=(1); //@line 884 "lbfgsb.c"
       if ($412) {
        (_s_wsfe(1116)|0); //@line 885 "lbfgsb.c"
        (_e_wsfe()|0); //@line 886 "lbfgsb.c"
       }
       HEAP32[1004>>2] = 0; //@line 888 "lbfgsb.c"
       HEAP32[988>>2] = 0; //@line 889 "lbfgsb.c"
       HEAP32[996>>2] = 1; //@line 890 "lbfgsb.c"
       HEAPF64[120>>3] = 1.0; //@line 891 "lbfgsb.c"
       HEAP32[1060>>2] = 0; //@line 892 "lbfgsb.c"
       HEAP32[1052>>2] = 0; //@line 893 "lbfgsb.c"
       (_timer_(64)|0); //@line 894 "lbfgsb.c"
       $413 = +HEAPF64[152>>3]; //@line 895 "lbfgsb.c"
       $414 = +HEAPF64[64>>3]; //@line 895 "lbfgsb.c"
       $415 = $413 + $414; //@line 895 "lbfgsb.c"
       $416 = +HEAPF64[56>>3]; //@line 895 "lbfgsb.c"
       $417 = $415 - $416; //@line 895 "lbfgsb.c"
       HEAPF64[152>>3] = $417; //@line 895 "lbfgsb.c"
       label = 16;
       continue L45;
      }
     } while(0);
     $438 = HEAP32[1028>>2]|0; //@line 909 "lbfgsb.c"
     $439 = ($438|0)==(0); //@line 909 "lbfgsb.c"
     $440 = HEAP32[988>>2]|0;
     $441 = ($440|0)==(0); //@line 909 "lbfgsb.c"
     $or$cond3 = $439 | $441; //@line 909 "lbfgsb.c"
     do {
      if (!($or$cond3)) {
       (_timer_(56)|0); //@line 917 "lbfgsb.c"
       $442 = HEAP32[992>>2]|0; //@line 923 "lbfgsb.c"
       $443 = ($442|0)!=(0); //@line 923 "lbfgsb.c"
       if ($443) {
        $444 = $1; //@line 924 "lbfgsb.c"
        $445 = $24; //@line 924 "lbfgsb.c"
        $446 = ((($445)) + 4|0); //@line 924 "lbfgsb.c"
        $447 = $26; //@line 924 "lbfgsb.c"
        $448 = ((($447)) + 4|0); //@line 924 "lbfgsb.c"
        $449 = $wn_offset; //@line 925 "lbfgsb.c"
        $450 = $16; //@line 925 "lbfgsb.c"
        $451 = (($450) + ($449<<3)|0); //@line 925 "lbfgsb.c"
        $452 = $snd_offset; //@line 925 "lbfgsb.c"
        $453 = $17; //@line 925 "lbfgsb.c"
        $454 = (($453) + ($452<<3)|0); //@line 925 "lbfgsb.c"
        $455 = $2; //@line 925 "lbfgsb.c"
        $456 = $ws_offset; //@line 925 "lbfgsb.c"
        $457 = $11; //@line 925 "lbfgsb.c"
        $458 = (($457) + ($456<<3)|0); //@line 925 "lbfgsb.c"
        $459 = $wy_offset; //@line 926 "lbfgsb.c"
        $460 = $12; //@line 926 "lbfgsb.c"
        $461 = (($460) + ($459<<3)|0); //@line 926 "lbfgsb.c"
        $462 = $sy_offset; //@line 926 "lbfgsb.c"
        $463 = $13; //@line 926 "lbfgsb.c"
        $464 = (($463) + ($462<<3)|0); //@line 926 "lbfgsb.c"
        (_formk_($444,1028,$446,1068,1048,$448,1060,1052,$451,$454,$455,$458,$461,$464,120,988,996,1004)|0); //@line 924 "lbfgsb.c"
       }
       $465 = HEAP32[1004>>2]|0; //@line 928 "lbfgsb.c"
       $466 = ($465|0)!=(0); //@line 928 "lbfgsb.c"
       if ($466) {
        $467 = $28; //@line 931 "lbfgsb.c"
        $468 = HEAP32[$467>>2]|0; //@line 931 "lbfgsb.c"
        $469 = ($468|0)>=(1); //@line 931 "lbfgsb.c"
        if ($469) {
         (_s_wsfe(1136)|0); //@line 932 "lbfgsb.c"
         (_e_wsfe()|0); //@line 933 "lbfgsb.c"
        }
        HEAP32[1004>>2] = 0; //@line 935 "lbfgsb.c"
        HEAP32[988>>2] = 0; //@line 936 "lbfgsb.c"
        HEAP32[996>>2] = 1; //@line 937 "lbfgsb.c"
        HEAPF64[120>>3] = 1.0; //@line 938 "lbfgsb.c"
        HEAP32[1060>>2] = 0; //@line 939 "lbfgsb.c"
        HEAP32[1052>>2] = 0; //@line 940 "lbfgsb.c"
        (_timer_(64)|0); //@line 941 "lbfgsb.c"
        $470 = +HEAPF64[168>>3]; //@line 942 "lbfgsb.c"
        $471 = +HEAPF64[64>>3]; //@line 942 "lbfgsb.c"
        $472 = $470 + $471; //@line 942 "lbfgsb.c"
        $473 = +HEAPF64[56>>3]; //@line 942 "lbfgsb.c"
        $474 = $472 - $473; //@line 942 "lbfgsb.c"
        HEAPF64[168>>3] = $474; //@line 942 "lbfgsb.c"
        label = 16;
        continue L45;
       }
       $475 = $1; //@line 947 "lbfgsb.c"
       $476 = $2; //@line 947 "lbfgsb.c"
       $477 = $3; //@line 947 "lbfgsb.c"
       $478 = ((($477)) + 8|0); //@line 947 "lbfgsb.c"
       $479 = $8; //@line 947 "lbfgsb.c"
       $480 = ((($479)) + 8|0); //@line 947 "lbfgsb.c"
       $481 = $ws_offset; //@line 947 "lbfgsb.c"
       $482 = $11; //@line 947 "lbfgsb.c"
       $483 = (($482) + ($481<<3)|0); //@line 947 "lbfgsb.c"
       $484 = $wy_offset; //@line 947 "lbfgsb.c"
       $485 = $12; //@line 947 "lbfgsb.c"
       $486 = (($485) + ($484<<3)|0); //@line 947 "lbfgsb.c"
       $487 = $sy_offset; //@line 947 "lbfgsb.c"
       $488 = $13; //@line 947 "lbfgsb.c"
       $489 = (($488) + ($487<<3)|0); //@line 947 "lbfgsb.c"
       $490 = $wt_offset; //@line 948 "lbfgsb.c"
       $491 = $15; //@line 948 "lbfgsb.c"
       $492 = (($491) + ($490<<3)|0); //@line 948 "lbfgsb.c"
       $493 = $18; //@line 948 "lbfgsb.c"
       $494 = ((($493)) + 8|0); //@line 948 "lbfgsb.c"
       $495 = $19; //@line 948 "lbfgsb.c"
       $496 = ((($495)) + 8|0); //@line 948 "lbfgsb.c"
       $497 = $23; //@line 948 "lbfgsb.c"
       $498 = ((($497)) + 8|0); //@line 948 "lbfgsb.c"
       $499 = $24; //@line 948 "lbfgsb.c"
       $500 = ((($499)) + 4|0); //@line 948 "lbfgsb.c"
       (_cmprlb_($475,$476,$478,$480,$483,$486,$489,$492,$494,$496,$498,$500,120,988,996,1028,1064,1004)|0); //@line 947 "lbfgsb.c"
       $501 = HEAP32[1004>>2]|0; //@line 950 "lbfgsb.c"
       $502 = ($501|0)!=(0); //@line 950 "lbfgsb.c"
       if (!($502)) {
        $503 = $1; //@line 954 "lbfgsb.c"
        $504 = $2; //@line 954 "lbfgsb.c"
        $505 = $24; //@line 954 "lbfgsb.c"
        $506 = ((($505)) + 4|0); //@line 954 "lbfgsb.c"
        $507 = $4; //@line 954 "lbfgsb.c"
        $508 = ((($507)) + 8|0); //@line 954 "lbfgsb.c"
        $509 = $5; //@line 954 "lbfgsb.c"
        $510 = ((($509)) + 8|0); //@line 954 "lbfgsb.c"
        $511 = $6; //@line 954 "lbfgsb.c"
        $512 = ((($511)) + 4|0); //@line 954 "lbfgsb.c"
        $513 = $18; //@line 954 "lbfgsb.c"
        $514 = ((($513)) + 8|0); //@line 954 "lbfgsb.c"
        $515 = $19; //@line 954 "lbfgsb.c"
        $516 = ((($515)) + 8|0); //@line 954 "lbfgsb.c"
        $517 = $22; //@line 955 "lbfgsb.c"
        $518 = ((($517)) + 8|0); //@line 955 "lbfgsb.c"
        $519 = $ws_offset; //@line 955 "lbfgsb.c"
        $520 = $11; //@line 955 "lbfgsb.c"
        $521 = (($520) + ($519<<3)|0); //@line 955 "lbfgsb.c"
        $522 = $wy_offset; //@line 955 "lbfgsb.c"
        $523 = $12; //@line 955 "lbfgsb.c"
        $524 = (($523) + ($522<<3)|0); //@line 955 "lbfgsb.c"
        $525 = $3; //@line 955 "lbfgsb.c"
        $526 = ((($525)) + 8|0); //@line 955 "lbfgsb.c"
        $527 = $8; //@line 955 "lbfgsb.c"
        $528 = ((($527)) + 8|0); //@line 955 "lbfgsb.c"
        $529 = $23; //@line 956 "lbfgsb.c"
        $530 = ((($529)) + 8|0); //@line 956 "lbfgsb.c"
        $531 = $wn_offset; //@line 956 "lbfgsb.c"
        $532 = $16; //@line 956 "lbfgsb.c"
        $533 = (($532) + ($531<<3)|0); //@line 956 "lbfgsb.c"
        $534 = $28; //@line 956 "lbfgsb.c"
        (_subsm_($503,$504,1028,$506,$508,$510,$512,$514,$516,$518,$521,$524,120,$526,$528,988,996,1044,$530,$533,$534,1004)|0); //@line 954 "lbfgsb.c"
       }
       $535 = HEAP32[1004>>2]|0; //@line 958 "lbfgsb.c"
       $536 = ($535|0)!=(0); //@line 958 "lbfgsb.c"
       if (!($536)) {
        (_timer_(64)|0); //@line 975 "lbfgsb.c"
        $545 = +HEAPF64[168>>3]; //@line 976 "lbfgsb.c"
        $546 = +HEAPF64[64>>3]; //@line 976 "lbfgsb.c"
        $547 = $545 + $546; //@line 976 "lbfgsb.c"
        $548 = +HEAPF64[56>>3]; //@line 976 "lbfgsb.c"
        $549 = $547 - $548; //@line 976 "lbfgsb.c"
        HEAPF64[168>>3] = $549; //@line 976 "lbfgsb.c"
        break;
       }
       $537 = $28; //@line 961 "lbfgsb.c"
       $538 = HEAP32[$537>>2]|0; //@line 961 "lbfgsb.c"
       $539 = ($538|0)>=(1); //@line 961 "lbfgsb.c"
       if ($539) {
        (_s_wsfe(1156)|0); //@line 962 "lbfgsb.c"
        (_e_wsfe()|0); //@line 963 "lbfgsb.c"
       }
       HEAP32[1004>>2] = 0; //@line 965 "lbfgsb.c"
       HEAP32[988>>2] = 0; //@line 966 "lbfgsb.c"
       HEAP32[996>>2] = 1; //@line 967 "lbfgsb.c"
       HEAPF64[120>>3] = 1.0; //@line 968 "lbfgsb.c"
       HEAP32[1060>>2] = 0; //@line 969 "lbfgsb.c"
       HEAP32[1052>>2] = 0; //@line 970 "lbfgsb.c"
       (_timer_(64)|0); //@line 971 "lbfgsb.c"
       $540 = +HEAPF64[168>>3]; //@line 972 "lbfgsb.c"
       $541 = +HEAPF64[64>>3]; //@line 972 "lbfgsb.c"
       $542 = $540 + $541; //@line 972 "lbfgsb.c"
       $543 = +HEAPF64[56>>3]; //@line 972 "lbfgsb.c"
       $544 = $542 - $543; //@line 972 "lbfgsb.c"
       HEAPF64[168>>3] = $544; //@line 972 "lbfgsb.c"
       label = 16;
       continue L45;
      }
     } while(0);
     $550 = $1; //@line 984 "lbfgsb.c"
     $551 = HEAP32[$550>>2]|0; //@line 984 "lbfgsb.c"
     HEAP32[$i__1>>2] = $551; //@line 984 "lbfgsb.c"
     HEAP32[980>>2] = 1; //@line 985 "lbfgsb.c"
     while(1) {
      $552 = HEAP32[980>>2]|0; //@line 985 "lbfgsb.c"
      $553 = HEAP32[$i__1>>2]|0; //@line 985 "lbfgsb.c"
      $554 = ($552|0)<=($553|0); //@line 985 "lbfgsb.c"
      if (!($554)) {
       break;
      }
      $555 = HEAP32[980>>2]|0; //@line 986 "lbfgsb.c"
      $556 = $18; //@line 986 "lbfgsb.c"
      $557 = (($556) + ($555<<3)|0); //@line 986 "lbfgsb.c"
      $558 = +HEAPF64[$557>>3]; //@line 986 "lbfgsb.c"
      $559 = HEAP32[980>>2]|0; //@line 986 "lbfgsb.c"
      $560 = $3; //@line 986 "lbfgsb.c"
      $561 = (($560) + ($559<<3)|0); //@line 986 "lbfgsb.c"
      $562 = +HEAPF64[$561>>3]; //@line 986 "lbfgsb.c"
      $563 = $558 - $562; //@line 986 "lbfgsb.c"
      $564 = HEAP32[980>>2]|0; //@line 986 "lbfgsb.c"
      $565 = $20; //@line 986 "lbfgsb.c"
      $566 = (($565) + ($564<<3)|0); //@line 986 "lbfgsb.c"
      HEAPF64[$566>>3] = $563; //@line 986 "lbfgsb.c"
      $567 = HEAP32[980>>2]|0; //@line 985 "lbfgsb.c"
      $568 = (($567) + 1)|0; //@line 985 "lbfgsb.c"
      HEAP32[980>>2] = $568; //@line 985 "lbfgsb.c"
     }
     (_timer_(56)|0); //@line 989 "lbfgsb.c"
     label = 43;
     continue;
    }
    else if ((label|0) == 43) {
     label = 0;
     $569 = $1; //@line 991 "lbfgsb.c"
     $570 = $4; //@line 991 "lbfgsb.c"
     $571 = ((($570)) + 8|0); //@line 991 "lbfgsb.c"
     $572 = $5; //@line 991 "lbfgsb.c"
     $573 = ((($572)) + 8|0); //@line 991 "lbfgsb.c"
     $574 = $6; //@line 991 "lbfgsb.c"
     $575 = ((($574)) + 4|0); //@line 991 "lbfgsb.c"
     $576 = $3; //@line 991 "lbfgsb.c"
     $577 = ((($576)) + 8|0); //@line 991 "lbfgsb.c"
     $578 = $7; //@line 991 "lbfgsb.c"
     $579 = $8; //@line 991 "lbfgsb.c"
     $580 = ((($579)) + 8|0); //@line 991 "lbfgsb.c"
     $581 = $20; //@line 992 "lbfgsb.c"
     $582 = ((($581)) + 8|0); //@line 992 "lbfgsb.c"
     $583 = $19; //@line 992 "lbfgsb.c"
     $584 = ((($583)) + 8|0); //@line 992 "lbfgsb.c"
     $585 = $21; //@line 992 "lbfgsb.c"
     $586 = ((($585)) + 8|0); //@line 992 "lbfgsb.c"
     $587 = $18; //@line 992 "lbfgsb.c"
     $588 = ((($587)) + 8|0); //@line 992 "lbfgsb.c"
     $589 = $27; //@line 993 "lbfgsb.c"
     $590 = $29; //@line 994 "lbfgsb.c"
     $591 = $31; //@line 994 "lbfgsb.c"
     $592 = ((($591)) + 88|0); //@line 994 "lbfgsb.c"
     $593 = $32; //@line 994 "lbfgsb.c"
     $594 = ((($593)) + 136|0); //@line 994 "lbfgsb.c"
     $595 = $28; //@line 994 "lbfgsb.c"
     (_lnsrlb_($569,$571,$573,$575,$577,$578,72,8,112,$580,$582,$584,$586,$588,48,128,32,136,144,1020,1016,1024,1012,1004,$589,1032,1064,$590,$592,$594,$595,60,60)|0); //@line 991 "lbfgsb.c"
     $596 = HEAP32[1004>>2]|0; //@line 995 "lbfgsb.c"
     $597 = ($596|0)!=(0); //@line 995 "lbfgsb.c"
     if (!($597)) {
      $598 = HEAP32[1024>>2]|0; //@line 995 "lbfgsb.c"
      $599 = $33; //@line 995 "lbfgsb.c"
      $600 = HEAP32[$599>>2]|0; //@line 995 "lbfgsb.c"
      $601 = ($598|0)>=($600|0); //@line 995 "lbfgsb.c"
      if (!($601)) {
       break;
      }
     }
     $602 = $1; //@line 997 "lbfgsb.c"
     $603 = $21; //@line 997 "lbfgsb.c"
     $604 = ((($603)) + 8|0); //@line 997 "lbfgsb.c"
     $605 = $3; //@line 997 "lbfgsb.c"
     $606 = ((($605)) + 8|0); //@line 997 "lbfgsb.c"
     (_dcopy_($602,$604,1236,$606,1236)|0); //@line 997 "lbfgsb.c"
     $607 = $1; //@line 998 "lbfgsb.c"
     $608 = $19; //@line 998 "lbfgsb.c"
     $609 = ((($608)) + 8|0); //@line 998 "lbfgsb.c"
     $610 = $8; //@line 998 "lbfgsb.c"
     $611 = ((($610)) + 8|0); //@line 998 "lbfgsb.c"
     (_dcopy_($607,$609,1236,$611,1236)|0); //@line 998 "lbfgsb.c"
     $612 = +HEAPF64[72>>3]; //@line 999 "lbfgsb.c"
     $613 = $7; //@line 999 "lbfgsb.c"
     HEAPF64[$613>>3] = $612; //@line 999 "lbfgsb.c"
     $614 = HEAP32[988>>2]|0; //@line 1000 "lbfgsb.c"
     $615 = ($614|0)==(0); //@line 1000 "lbfgsb.c"
     if ($615) {
      $616 = HEAP32[1004>>2]|0; //@line 1002 "lbfgsb.c"
      $617 = ($616|0)==(0); //@line 1002 "lbfgsb.c"
      if ($617) {
       HEAP32[1004>>2] = -9; //@line 1003 "lbfgsb.c"
       $618 = HEAP32[1012>>2]|0; //@line 1005 "lbfgsb.c"
       $619 = (($618) + -1)|0; //@line 1005 "lbfgsb.c"
       HEAP32[1012>>2] = $619; //@line 1005 "lbfgsb.c"
       $620 = HEAP32[1016>>2]|0; //@line 1006 "lbfgsb.c"
       $621 = (($620) + -1)|0; //@line 1006 "lbfgsb.c"
       HEAP32[1016>>2] = $621; //@line 1006 "lbfgsb.c"
       $622 = HEAP32[1024>>2]|0; //@line 1007 "lbfgsb.c"
       $623 = (($622) + -1)|0; //@line 1007 "lbfgsb.c"
       HEAP32[1024>>2] = $623; //@line 1007 "lbfgsb.c"
      }
      $624 = $27; //@line 1009 "lbfgsb.c"
      _s_copy($624,14447,60,30); //@line 1009 "lbfgsb.c"
      $625 = HEAP32[1020>>2]|0; //@line 1011 "lbfgsb.c"
      $626 = (($625) + 1)|0; //@line 1011 "lbfgsb.c"
      HEAP32[1020>>2] = $626; //@line 1011 "lbfgsb.c"
      label = 74;
      continue;
     }
     $627 = $28; //@line 1015 "lbfgsb.c"
     $628 = HEAP32[$627>>2]|0; //@line 1015 "lbfgsb.c"
     $629 = ($628|0)>=(1); //@line 1015 "lbfgsb.c"
     if ($629) {
      (_s_wsfe(1176)|0); //@line 1016 "lbfgsb.c"
      (_e_wsfe()|0); //@line 1017 "lbfgsb.c"
     }
     $630 = HEAP32[1004>>2]|0; //@line 1019 "lbfgsb.c"
     $631 = ($630|0)==(0); //@line 1019 "lbfgsb.c"
     if ($631) {
      $632 = HEAP32[1012>>2]|0; //@line 1020 "lbfgsb.c"
      $633 = (($632) + -1)|0; //@line 1020 "lbfgsb.c"
      HEAP32[1012>>2] = $633; //@line 1020 "lbfgsb.c"
     }
     HEAP32[1004>>2] = 0; //@line 1022 "lbfgsb.c"
     HEAP32[988>>2] = 0; //@line 1023 "lbfgsb.c"
     HEAP32[996>>2] = 1; //@line 1024 "lbfgsb.c"
     HEAPF64[120>>3] = 1.0; //@line 1025 "lbfgsb.c"
     HEAP32[1060>>2] = 0; //@line 1026 "lbfgsb.c"
     HEAP32[1052>>2] = 0; //@line 1027 "lbfgsb.c"
     $634 = $27; //@line 1028 "lbfgsb.c"
     _s_copy($634,14478,60,19); //@line 1028 "lbfgsb.c"
     (_timer_(64)|0); //@line 1029 "lbfgsb.c"
     $635 = +HEAPF64[184>>3]; //@line 1030 "lbfgsb.c"
     $636 = +HEAPF64[64>>3]; //@line 1030 "lbfgsb.c"
     $637 = $635 + $636; //@line 1030 "lbfgsb.c"
     $638 = +HEAPF64[56>>3]; //@line 1030 "lbfgsb.c"
     $639 = $637 - $638; //@line 1030 "lbfgsb.c"
     HEAPF64[184>>3] = $639; //@line 1030 "lbfgsb.c"
     label = 16;
     continue;
    }
    else if ((label|0) == 74) {
     label = 0;
     (_timer_(104)|0); //@line 1135 "lbfgsb.c"
     $792 = +HEAPF64[104>>3]; //@line 1136 "lbfgsb.c"
     $793 = +HEAPF64[96>>3]; //@line 1136 "lbfgsb.c"
     $794 = $792 - $793; //@line 1136 "lbfgsb.c"
     HEAPF64[88>>3] = $794; //@line 1136 "lbfgsb.c"
     $795 = $1; //@line 1137 "lbfgsb.c"
     $796 = $3; //@line 1137 "lbfgsb.c"
     $797 = ((($796)) + 8|0); //@line 1137 "lbfgsb.c"
     $798 = $7; //@line 1137 "lbfgsb.c"
     $799 = $27; //@line 1137 "lbfgsb.c"
     $800 = $28; //@line 1137 "lbfgsb.c"
     (_prn3lb_($795,$797,$798,$799,$800,1004,1020,1012,1072,1040,1000,176,88,1008,14347,1024,48,136,984,152,168,184,60,3)|0); //@line 1137 "lbfgsb.c"
     break L1;
    }
   }
   $640 = $27; //@line 1033 "lbfgsb.c"
   $641 = (_s_cmp($640,14362,5,5)|0); //@line 1033 "lbfgsb.c"
   $642 = ($641|0)==(0); //@line 1033 "lbfgsb.c"
   if (!($642)) {
    (_timer_(64)|0); //@line 1038 "lbfgsb.c"
    $643 = +HEAPF64[184>>3]; //@line 1039 "lbfgsb.c"
    $644 = +HEAPF64[64>>3]; //@line 1039 "lbfgsb.c"
    $645 = $643 + $644; //@line 1039 "lbfgsb.c"
    $646 = +HEAPF64[56>>3]; //@line 1039 "lbfgsb.c"
    $647 = $645 - $646; //@line 1039 "lbfgsb.c"
    HEAPF64[184>>3] = $647; //@line 1039 "lbfgsb.c"
    $648 = HEAP32[1020>>2]|0; //@line 1040 "lbfgsb.c"
    $649 = (($648) + 1)|0; //@line 1040 "lbfgsb.c"
    HEAP32[1020>>2] = $649; //@line 1040 "lbfgsb.c"
    $650 = $1; //@line 1042 "lbfgsb.c"
    $651 = $4; //@line 1042 "lbfgsb.c"
    $652 = ((($651)) + 8|0); //@line 1042 "lbfgsb.c"
    $653 = $5; //@line 1042 "lbfgsb.c"
    $654 = ((($653)) + 8|0); //@line 1042 "lbfgsb.c"
    $655 = $6; //@line 1042 "lbfgsb.c"
    $656 = ((($655)) + 4|0); //@line 1042 "lbfgsb.c"
    $657 = $3; //@line 1042 "lbfgsb.c"
    $658 = ((($657)) + 8|0); //@line 1042 "lbfgsb.c"
    $659 = $8; //@line 1042 "lbfgsb.c"
    $660 = ((($659)) + 8|0); //@line 1042 "lbfgsb.c"
    (_projgr_($650,$652,$654,$656,$658,$660,176)|0); //@line 1042 "lbfgsb.c"
    $661 = $1; //@line 1044 "lbfgsb.c"
    $662 = $3; //@line 1044 "lbfgsb.c"
    $663 = ((($662)) + 8|0); //@line 1044 "lbfgsb.c"
    $664 = $7; //@line 1044 "lbfgsb.c"
    $665 = $8; //@line 1044 "lbfgsb.c"
    $666 = ((($665)) + 8|0); //@line 1044 "lbfgsb.c"
    $667 = $28; //@line 1044 "lbfgsb.c"
    (_prn2lb_($661,$663,$664,$666,$667,1020,1012,1000,176,1008,14347,1044,1024,48,136,3)|0); //@line 1044 "lbfgsb.c"
   }
  }
 } while(0);
 if ((label|0) == 11) {
  $317 = $27; //@line 834 "lbfgsb.c"
  _s_copy($317,14389,60,8); //@line 834 "lbfgsb.c"
 }
 $801 = HEAP32[1056>>2]|0; //@line 1142 "lbfgsb.c"
 $802 = $30; //@line 1142 "lbfgsb.c"
 $803 = ((($802)) + 4|0); //@line 1142 "lbfgsb.c"
 HEAP32[$803>>2] = $801; //@line 1142 "lbfgsb.c"
 $804 = HEAP32[1064>>2]|0; //@line 1143 "lbfgsb.c"
 $805 = $30; //@line 1143 "lbfgsb.c"
 $806 = ((($805)) + 8|0); //@line 1143 "lbfgsb.c"
 HEAP32[$806>>2] = $804; //@line 1143 "lbfgsb.c"
 $807 = HEAP32[1032>>2]|0; //@line 1144 "lbfgsb.c"
 $808 = $30; //@line 1144 "lbfgsb.c"
 $809 = ((($808)) + 12|0); //@line 1144 "lbfgsb.c"
 HEAP32[$809>>2] = $807; //@line 1144 "lbfgsb.c"
 $810 = HEAP32[1052>>2]|0; //@line 1145 "lbfgsb.c"
 $811 = $30; //@line 1145 "lbfgsb.c"
 $812 = ((($811)) + 16|0); //@line 1145 "lbfgsb.c"
 HEAP32[$812>>2] = $810; //@line 1145 "lbfgsb.c"
 $813 = HEAP32[1072>>2]|0; //@line 1146 "lbfgsb.c"
 $814 = $31; //@line 1146 "lbfgsb.c"
 $815 = ((($814)) + 4|0); //@line 1146 "lbfgsb.c"
 HEAP32[$815>>2] = $813; //@line 1146 "lbfgsb.c"
 $816 = HEAP32[1024>>2]|0; //@line 1147 "lbfgsb.c"
 $817 = $31; //@line 1147 "lbfgsb.c"
 $818 = ((($817)) + 16|0); //@line 1147 "lbfgsb.c"
 HEAP32[$818>>2] = $816; //@line 1147 "lbfgsb.c"
 $819 = HEAP32[1040>>2]|0; //@line 1148 "lbfgsb.c"
 $820 = $31; //@line 1148 "lbfgsb.c"
 $821 = ((($820)) + 20|0); //@line 1148 "lbfgsb.c"
 HEAP32[$821>>2] = $819; //@line 1148 "lbfgsb.c"
 $822 = HEAP32[996>>2]|0; //@line 1149 "lbfgsb.c"
 $823 = $31; //@line 1149 "lbfgsb.c"
 $824 = ((($823)) + 24|0); //@line 1149 "lbfgsb.c"
 HEAP32[$824>>2] = $822; //@line 1149 "lbfgsb.c"
 $825 = HEAP32[988>>2]|0; //@line 1150 "lbfgsb.c"
 $826 = $31; //@line 1150 "lbfgsb.c"
 $827 = ((($826)) + 28|0); //@line 1150 "lbfgsb.c"
 HEAP32[$827>>2] = $825; //@line 1150 "lbfgsb.c"
 $828 = HEAP32[1036>>2]|0; //@line 1151 "lbfgsb.c"
 $829 = $31; //@line 1151 "lbfgsb.c"
 $830 = ((($829)) + 32|0); //@line 1151 "lbfgsb.c"
 HEAP32[$830>>2] = $828; //@line 1151 "lbfgsb.c"
 $831 = HEAP32[1020>>2]|0; //@line 1152 "lbfgsb.c"
 $832 = $31; //@line 1152 "lbfgsb.c"
 $833 = ((($832)) + 36|0); //@line 1152 "lbfgsb.c"
 HEAP32[$833>>2] = $831; //@line 1152 "lbfgsb.c"
 $834 = HEAP32[1060>>2]|0; //@line 1153 "lbfgsb.c"
 $835 = $31; //@line 1153 "lbfgsb.c"
 $836 = ((($835)) + 40|0); //@line 1153 "lbfgsb.c"
 HEAP32[$836>>2] = $834; //@line 1153 "lbfgsb.c"
 $837 = HEAP32[1008>>2]|0; //@line 1154 "lbfgsb.c"
 $838 = $31; //@line 1154 "lbfgsb.c"
 $839 = ((($838)) + 48|0); //@line 1154 "lbfgsb.c"
 HEAP32[$839>>2] = $837; //@line 1154 "lbfgsb.c"
 $840 = HEAP32[1012>>2]|0; //@line 1155 "lbfgsb.c"
 $841 = $31; //@line 1155 "lbfgsb.c"
 $842 = ((($841)) + 52|0); //@line 1155 "lbfgsb.c"
 HEAP32[$842>>2] = $840; //@line 1155 "lbfgsb.c"
 $843 = HEAP32[1004>>2]|0; //@line 1156 "lbfgsb.c"
 $844 = $31; //@line 1156 "lbfgsb.c"
 $845 = ((($844)) + 56|0); //@line 1156 "lbfgsb.c"
 HEAP32[$845>>2] = $843; //@line 1156 "lbfgsb.c"
 $846 = HEAP32[1016>>2]|0; //@line 1157 "lbfgsb.c"
 $847 = $31; //@line 1157 "lbfgsb.c"
 $848 = ((($847)) + 60|0); //@line 1157 "lbfgsb.c"
 HEAP32[$848>>2] = $846; //@line 1157 "lbfgsb.c"
 $849 = HEAP32[1044>>2]|0; //@line 1158 "lbfgsb.c"
 $850 = $31; //@line 1158 "lbfgsb.c"
 $851 = ((($850)) + 64|0); //@line 1158 "lbfgsb.c"
 HEAP32[$851>>2] = $849; //@line 1158 "lbfgsb.c"
 $852 = HEAP32[1028>>2]|0; //@line 1159 "lbfgsb.c"
 $853 = $31; //@line 1159 "lbfgsb.c"
 $854 = ((($853)) + 68|0); //@line 1159 "lbfgsb.c"
 HEAP32[$854>>2] = $852; //@line 1159 "lbfgsb.c"
 $855 = HEAP32[1000>>2]|0; //@line 1160 "lbfgsb.c"
 $856 = $31; //@line 1160 "lbfgsb.c"
 $857 = ((($856)) + 72|0); //@line 1160 "lbfgsb.c"
 HEAP32[$857>>2] = $855; //@line 1160 "lbfgsb.c"
 $858 = HEAP32[1048>>2]|0; //@line 1161 "lbfgsb.c"
 $859 = $31; //@line 1161 "lbfgsb.c"
 $860 = ((($859)) + 76|0); //@line 1161 "lbfgsb.c"
 HEAP32[$860>>2] = $858; //@line 1161 "lbfgsb.c"
 $861 = HEAP32[1068>>2]|0; //@line 1162 "lbfgsb.c"
 $862 = $31; //@line 1162 "lbfgsb.c"
 $863 = ((($862)) + 80|0); //@line 1162 "lbfgsb.c"
 HEAP32[$863>>2] = $861; //@line 1162 "lbfgsb.c"
 $864 = +HEAPF64[120>>3]; //@line 1163 "lbfgsb.c"
 $865 = $32; //@line 1163 "lbfgsb.c"
 $866 = ((($865)) + 8|0); //@line 1163 "lbfgsb.c"
 HEAPF64[$866>>3] = $864; //@line 1163 "lbfgsb.c"
 $867 = +HEAPF64[72>>3]; //@line 1164 "lbfgsb.c"
 $868 = $32; //@line 1164 "lbfgsb.c"
 $869 = ((($868)) + 16|0); //@line 1164 "lbfgsb.c"
 HEAPF64[$869>>3] = $867; //@line 1164 "lbfgsb.c"
 $870 = +HEAPF64[40>>3]; //@line 1165 "lbfgsb.c"
 $871 = $32; //@line 1165 "lbfgsb.c"
 $872 = ((($871)) + 24|0); //@line 1165 "lbfgsb.c"
 HEAPF64[$872>>3] = $870; //@line 1165 "lbfgsb.c"
 $873 = +HEAPF64[128>>3]; //@line 1166 "lbfgsb.c"
 $874 = $32; //@line 1166 "lbfgsb.c"
 $875 = ((($874)) + 32|0); //@line 1166 "lbfgsb.c"
 HEAPF64[$875>>3] = $873; //@line 1166 "lbfgsb.c"
 $876 = +HEAPF64[160>>3]; //@line 1167 "lbfgsb.c"
 $877 = $32; //@line 1167 "lbfgsb.c"
 $878 = ((($877)) + 40|0); //@line 1167 "lbfgsb.c"
 HEAPF64[$878>>3] = $876; //@line 1167 "lbfgsb.c"
 $879 = +HEAPF64[56>>3]; //@line 1168 "lbfgsb.c"
 $880 = $32; //@line 1168 "lbfgsb.c"
 $881 = ((($880)) + 48|0); //@line 1168 "lbfgsb.c"
 HEAPF64[$881>>3] = $879; //@line 1168 "lbfgsb.c"
 $882 = +HEAPF64[152>>3]; //@line 1169 "lbfgsb.c"
 $883 = $32; //@line 1169 "lbfgsb.c"
 $884 = ((($883)) + 56|0); //@line 1169 "lbfgsb.c"
 HEAPF64[$884>>3] = $882; //@line 1169 "lbfgsb.c"
 $885 = +HEAPF64[168>>3]; //@line 1170 "lbfgsb.c"
 $886 = $32; //@line 1170 "lbfgsb.c"
 $887 = ((($886)) + 64|0); //@line 1170 "lbfgsb.c"
 HEAPF64[$887>>3] = $885; //@line 1170 "lbfgsb.c"
 $888 = +HEAPF64[184>>3]; //@line 1171 "lbfgsb.c"
 $889 = $32; //@line 1171 "lbfgsb.c"
 $890 = ((($889)) + 72|0); //@line 1171 "lbfgsb.c"
 HEAPF64[$890>>3] = $888; //@line 1171 "lbfgsb.c"
 $891 = +HEAPF64[96>>3]; //@line 1172 "lbfgsb.c"
 $892 = $32; //@line 1172 "lbfgsb.c"
 $893 = ((($892)) + 80|0); //@line 1172 "lbfgsb.c"
 HEAPF64[$893>>3] = $891; //@line 1172 "lbfgsb.c"
 $894 = +HEAPF64[8>>3]; //@line 1173 "lbfgsb.c"
 $895 = $32; //@line 1173 "lbfgsb.c"
 $896 = ((($895)) + 88|0); //@line 1173 "lbfgsb.c"
 HEAPF64[$896>>3] = $894; //@line 1173 "lbfgsb.c"
 $897 = +HEAPF64[144>>3]; //@line 1174 "lbfgsb.c"
 $898 = $32; //@line 1174 "lbfgsb.c"
 $899 = ((($898)) + 96|0); //@line 1174 "lbfgsb.c"
 HEAPF64[$899>>3] = $897; //@line 1174 "lbfgsb.c"
 $900 = +HEAPF64[176>>3]; //@line 1175 "lbfgsb.c"
 $901 = $32; //@line 1175 "lbfgsb.c"
 $902 = ((($901)) + 104|0); //@line 1175 "lbfgsb.c"
 HEAPF64[$902>>3] = $900; //@line 1175 "lbfgsb.c"
 $903 = +HEAPF64[48>>3]; //@line 1176 "lbfgsb.c"
 $904 = $32; //@line 1176 "lbfgsb.c"
 $905 = ((($904)) + 112|0); //@line 1176 "lbfgsb.c"
 HEAPF64[$905>>3] = $903; //@line 1176 "lbfgsb.c"
 $906 = +HEAPF64[112>>3]; //@line 1177 "lbfgsb.c"
 $907 = $32; //@line 1177 "lbfgsb.c"
 $908 = ((($907)) + 120|0); //@line 1177 "lbfgsb.c"
 HEAPF64[$908>>3] = $906; //@line 1177 "lbfgsb.c"
 $909 = +HEAPF64[32>>3]; //@line 1178 "lbfgsb.c"
 $910 = $32; //@line 1178 "lbfgsb.c"
 $911 = ((($910)) + 128|0); //@line 1178 "lbfgsb.c"
 HEAPF64[$911>>3] = $909; //@line 1178 "lbfgsb.c"
 $0 = 0; //@line 1180 "lbfgsb.c"
 $912 = $0; //@line 1181 "lbfgsb.c"
 STACKTOP = sp;return ($912|0); //@line 1181 "lbfgsb.c"
}
function _errclb_($n,$m,$factr,$l,$u,$nbd,$task,$info,$k,$task_len) {
 $n = $n|0;
 $m = $m|0;
 $factr = $factr|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $task = $task|0;
 $info = $info|0;
 $k = $k|0;
 $task_len = $task_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0, $57 = 0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $m;
 $2 = $factr;
 $3 = $l;
 $4 = $u;
 $5 = $nbd;
 $6 = $task;
 $7 = $info;
 $8 = $k;
 $9 = $task_len;
 $10 = $5; //@line 2206 "lbfgsb.c"
 $11 = ((($10)) + -4|0); //@line 2206 "lbfgsb.c"
 $5 = $11; //@line 2206 "lbfgsb.c"
 $12 = $4; //@line 2207 "lbfgsb.c"
 $13 = ((($12)) + -8|0); //@line 2207 "lbfgsb.c"
 $4 = $13; //@line 2207 "lbfgsb.c"
 $14 = $3; //@line 2208 "lbfgsb.c"
 $15 = ((($14)) + -8|0); //@line 2208 "lbfgsb.c"
 $3 = $15; //@line 2208 "lbfgsb.c"
 $16 = $0; //@line 2211 "lbfgsb.c"
 $17 = HEAP32[$16>>2]|0; //@line 2211 "lbfgsb.c"
 $18 = ($17|0)<=(0); //@line 2211 "lbfgsb.c"
 if ($18) {
  $19 = $6; //@line 2212 "lbfgsb.c"
  _s_copy($19,15167,60,15); //@line 2212 "lbfgsb.c"
 }
 $20 = $1; //@line 2214 "lbfgsb.c"
 $21 = HEAP32[$20>>2]|0; //@line 2214 "lbfgsb.c"
 $22 = ($21|0)<=(0); //@line 2214 "lbfgsb.c"
 if ($22) {
  $23 = $6; //@line 2215 "lbfgsb.c"
  _s_copy($23,15183,60,15); //@line 2215 "lbfgsb.c"
 }
 $24 = $2; //@line 2217 "lbfgsb.c"
 $25 = +HEAPF64[$24>>3]; //@line 2217 "lbfgsb.c"
 $26 = $25 < 0.0; //@line 2217 "lbfgsb.c"
 if ($26) {
  $27 = $6; //@line 2218 "lbfgsb.c"
  _s_copy($27,15199,60,19); //@line 2218 "lbfgsb.c"
 }
 $28 = $0; //@line 2221 "lbfgsb.c"
 $29 = HEAP32[$28>>2]|0; //@line 2221 "lbfgsb.c"
 $i__1 = $29; //@line 2221 "lbfgsb.c"
 HEAP32[1680>>2] = 1; //@line 2222 "lbfgsb.c"
 while(1) {
  $30 = HEAP32[1680>>2]|0; //@line 2222 "lbfgsb.c"
  $31 = $i__1; //@line 2222 "lbfgsb.c"
  $32 = ($30|0)<=($31|0); //@line 2222 "lbfgsb.c"
  if (!($32)) {
   break;
  }
  $33 = HEAP32[1680>>2]|0; //@line 2223 "lbfgsb.c"
  $34 = $5; //@line 2223 "lbfgsb.c"
  $35 = (($34) + ($33<<2)|0); //@line 2223 "lbfgsb.c"
  $36 = HEAP32[$35>>2]|0; //@line 2223 "lbfgsb.c"
  $37 = ($36|0)<(0); //@line 2223 "lbfgsb.c"
  if ($37) {
   label = 11;
  } else {
   $38 = HEAP32[1680>>2]|0; //@line 2223 "lbfgsb.c"
   $39 = $5; //@line 2223 "lbfgsb.c"
   $40 = (($39) + ($38<<2)|0); //@line 2223 "lbfgsb.c"
   $41 = HEAP32[$40>>2]|0; //@line 2223 "lbfgsb.c"
   $42 = ($41|0)>(3); //@line 2223 "lbfgsb.c"
   if ($42) {
    label = 11;
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $43 = $6; //@line 2225 "lbfgsb.c"
   _s_copy($43,15219,60,18); //@line 2225 "lbfgsb.c"
   $44 = $7; //@line 2226 "lbfgsb.c"
   HEAP32[$44>>2] = -6; //@line 2226 "lbfgsb.c"
   $45 = HEAP32[1680>>2]|0; //@line 2227 "lbfgsb.c"
   $46 = $8; //@line 2227 "lbfgsb.c"
   HEAP32[$46>>2] = $45; //@line 2227 "lbfgsb.c"
  }
  $47 = HEAP32[1680>>2]|0; //@line 2229 "lbfgsb.c"
  $48 = $5; //@line 2229 "lbfgsb.c"
  $49 = (($48) + ($47<<2)|0); //@line 2229 "lbfgsb.c"
  $50 = HEAP32[$49>>2]|0; //@line 2229 "lbfgsb.c"
  $51 = ($50|0)==(2); //@line 2229 "lbfgsb.c"
  if ($51) {
   $52 = HEAP32[1680>>2]|0; //@line 2230 "lbfgsb.c"
   $53 = $3; //@line 2230 "lbfgsb.c"
   $54 = (($53) + ($52<<3)|0); //@line 2230 "lbfgsb.c"
   $55 = +HEAPF64[$54>>3]; //@line 2230 "lbfgsb.c"
   $56 = HEAP32[1680>>2]|0; //@line 2230 "lbfgsb.c"
   $57 = $4; //@line 2230 "lbfgsb.c"
   $58 = (($57) + ($56<<3)|0); //@line 2230 "lbfgsb.c"
   $59 = +HEAPF64[$58>>3]; //@line 2230 "lbfgsb.c"
   $60 = $55 > $59; //@line 2230 "lbfgsb.c"
   if ($60) {
    $61 = $6; //@line 2232 "lbfgsb.c"
    _s_copy($61,15238,60,27); //@line 2232 "lbfgsb.c"
    $62 = $7; //@line 2234 "lbfgsb.c"
    HEAP32[$62>>2] = -7; //@line 2234 "lbfgsb.c"
    $63 = HEAP32[1680>>2]|0; //@line 2235 "lbfgsb.c"
    $64 = $8; //@line 2235 "lbfgsb.c"
    HEAP32[$64>>2] = $63; //@line 2235 "lbfgsb.c"
   }
  }
  $65 = HEAP32[1680>>2]|0; //@line 2222 "lbfgsb.c"
  $66 = (($65) + 1)|0; //@line 2222 "lbfgsb.c"
  HEAP32[1680>>2] = $66; //@line 2222 "lbfgsb.c"
 }
 STACKTOP = sp;return 0; //@line 2240 "lbfgsb.c"
}
function _prn3lb_($n,$x,$f,$task,$iprint,$info,$iter,$nfgv,$nintol,$nskip,$nact,$sbgnrm,$time,$nseg,$word,$iback,$stp,$xstep,$k,$cachyt,$sbtime,$lnscht,$task_len,$word_len) {
 $n = $n|0;
 $x = $x|0;
 $f = $f|0;
 $task = $task|0;
 $iprint = $iprint|0;
 $info = $info|0;
 $iter = $iter|0;
 $nfgv = $nfgv|0;
 $nintol = $nintol|0;
 $nskip = $nskip|0;
 $nact = $nact|0;
 $sbgnrm = $sbgnrm|0;
 $time = $time|0;
 $nseg = $nseg|0;
 $word = $word|0;
 $iback = $iback|0;
 $stp = $stp|0;
 $xstep = $xstep|0;
 $k = $k|0;
 $cachyt = $cachyt|0;
 $sbtime = $sbtime|0;
 $lnscht = $lnscht|0;
 $task_len = $task_len|0;
 $word_len = $word_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $x;
 $2 = $f;
 $3 = $task;
 $4 = $iprint;
 $5 = $info;
 $6 = $iter;
 $7 = $nfgv;
 $8 = $nintol;
 $9 = $nskip;
 $10 = $nact;
 $11 = $sbgnrm;
 $12 = $time;
 $13 = $nseg;
 $14 = $word;
 $15 = $iback;
 $16 = $stp;
 $17 = $xstep;
 $18 = $k;
 $19 = $cachyt;
 $20 = $sbtime;
 $21 = $lnscht;
 $22 = $task_len;
 $23 = $word_len;
 $24 = $1; //@line 3560 "lbfgsb.c"
 $25 = ((($24)) + -8|0); //@line 3560 "lbfgsb.c"
 $1 = $25; //@line 3560 "lbfgsb.c"
 $26 = $3; //@line 3563 "lbfgsb.c"
 $27 = (_s_cmp($26,14356,5,5)|0); //@line 3563 "lbfgsb.c"
 $28 = ($27|0)==(0); //@line 3563 "lbfgsb.c"
 if (!($28)) {
  $29 = $4; //@line 3566 "lbfgsb.c"
  $30 = HEAP32[$29>>2]|0; //@line 3566 "lbfgsb.c"
  $31 = ($30|0)>=(0); //@line 3566 "lbfgsb.c"
  if ($31) {
   (_s_wsfe(2136)|0); //@line 3567 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3568 "lbfgsb.c"
   (_s_wsfe(2156)|0); //@line 3569 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3570 "lbfgsb.c"
   (_s_wsfe(2176)|0); //@line 3571 "lbfgsb.c"
   $32 = $0; //@line 3572 "lbfgsb.c"
   (_do_fio(1236,$32,4)|0); //@line 3572 "lbfgsb.c"
   $33 = $6; //@line 3573 "lbfgsb.c"
   (_do_fio(1236,$33,4)|0); //@line 3573 "lbfgsb.c"
   $34 = $7; //@line 3574 "lbfgsb.c"
   (_do_fio(1236,$34,4)|0); //@line 3574 "lbfgsb.c"
   $35 = $8; //@line 3575 "lbfgsb.c"
   (_do_fio(1236,$35,4)|0); //@line 3575 "lbfgsb.c"
   $36 = $9; //@line 3576 "lbfgsb.c"
   (_do_fio(1236,$36,4)|0); //@line 3576 "lbfgsb.c"
   $37 = $10; //@line 3577 "lbfgsb.c"
   (_do_fio(1236,$37,4)|0); //@line 3577 "lbfgsb.c"
   $38 = $11; //@line 3578 "lbfgsb.c"
   (_do_fio(1236,$38,8)|0); //@line 3578 "lbfgsb.c"
   $39 = $2; //@line 3579 "lbfgsb.c"
   (_do_fio(1236,$39,8)|0); //@line 3579 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3580 "lbfgsb.c"
   $40 = $4; //@line 3581 "lbfgsb.c"
   $41 = HEAP32[$40>>2]|0; //@line 3581 "lbfgsb.c"
   $42 = ($41|0)>=(100); //@line 3581 "lbfgsb.c"
   if ($42) {
    (_s_wsfe(2196)|0); //@line 3582 "lbfgsb.c"
    (_do_fio(1236,15780,3)|0); //@line 3583 "lbfgsb.c"
    $43 = $0; //@line 3584 "lbfgsb.c"
    $44 = HEAP32[$43>>2]|0; //@line 3584 "lbfgsb.c"
    $i__1 = $44; //@line 3584 "lbfgsb.c"
    HEAP32[2132>>2] = 1; //@line 3585 "lbfgsb.c"
    while(1) {
     $45 = HEAP32[2132>>2]|0; //@line 3585 "lbfgsb.c"
     $46 = $i__1; //@line 3585 "lbfgsb.c"
     $47 = ($45|0)<=($46|0); //@line 3585 "lbfgsb.c"
     if (!($47)) {
      break;
     }
     $48 = HEAP32[2132>>2]|0; //@line 3586 "lbfgsb.c"
     $49 = $1; //@line 3586 "lbfgsb.c"
     $50 = (($49) + ($48<<3)|0); //@line 3586 "lbfgsb.c"
     (_do_fio(1236,$50,8)|0); //@line 3586 "lbfgsb.c"
     $51 = HEAP32[2132>>2]|0; //@line 3585 "lbfgsb.c"
     $52 = (($51) + 1)|0; //@line 3585 "lbfgsb.c"
     HEAP32[2132>>2] = $52; //@line 3585 "lbfgsb.c"
    }
    (_e_wsfe()|0); //@line 3588 "lbfgsb.c"
   }
   $53 = $4; //@line 3590 "lbfgsb.c"
   $54 = HEAP32[$53>>2]|0; //@line 3590 "lbfgsb.c"
   $55 = ($54|0)>=(1); //@line 3590 "lbfgsb.c"
   if ($55) {
    (_s_wsle(2216)|0); //@line 3591 "lbfgsb.c"
    (_do_lio(1308,1236,17485,4)|0); //@line 3592 "lbfgsb.c"
    $56 = $2; //@line 3593 "lbfgsb.c"
    (_do_lio(1908,1236,$56,8)|0); //@line 3593 "lbfgsb.c"
    (_e_wsle()|0); //@line 3594 "lbfgsb.c"
   }
  }
 }
 $57 = $4; //@line 3598 "lbfgsb.c"
 $58 = HEAP32[$57>>2]|0; //@line 3598 "lbfgsb.c"
 $59 = ($58|0)>=(0); //@line 3598 "lbfgsb.c"
 if (!($59)) {
  STACKTOP = sp;return 0; //@line 3660 "lbfgsb.c"
 }
 (_s_wsfe(2236)|0); //@line 3599 "lbfgsb.c"
 $60 = $3; //@line 3600 "lbfgsb.c"
 (_do_fio(1236,$60,60)|0); //@line 3600 "lbfgsb.c"
 (_e_wsfe()|0); //@line 3601 "lbfgsb.c"
 $61 = $5; //@line 3602 "lbfgsb.c"
 $62 = HEAP32[$61>>2]|0; //@line 3602 "lbfgsb.c"
 $63 = ($62|0)!=(0); //@line 3602 "lbfgsb.c"
 if ($63) {
  $64 = $5; //@line 3603 "lbfgsb.c"
  $65 = HEAP32[$64>>2]|0; //@line 3603 "lbfgsb.c"
  $66 = ($65|0)==(-1); //@line 3603 "lbfgsb.c"
  if ($66) {
   (_s_wsfe(2256)|0); //@line 3604 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3605 "lbfgsb.c"
  }
  $67 = $5; //@line 3607 "lbfgsb.c"
  $68 = HEAP32[$67>>2]|0; //@line 3607 "lbfgsb.c"
  $69 = ($68|0)==(-2); //@line 3607 "lbfgsb.c"
  if ($69) {
   (_s_wsfe(2276)|0); //@line 3608 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3609 "lbfgsb.c"
  }
  $70 = $5; //@line 3611 "lbfgsb.c"
  $71 = HEAP32[$70>>2]|0; //@line 3611 "lbfgsb.c"
  $72 = ($71|0)==(-3); //@line 3611 "lbfgsb.c"
  if ($72) {
   (_s_wsfe(2296)|0); //@line 3612 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3613 "lbfgsb.c"
  }
  $73 = $5; //@line 3615 "lbfgsb.c"
  $74 = HEAP32[$73>>2]|0; //@line 3615 "lbfgsb.c"
  $75 = ($74|0)==(-4); //@line 3615 "lbfgsb.c"
  if ($75) {
   (_s_wsfe(2316)|0); //@line 3616 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3617 "lbfgsb.c"
  }
  $76 = $5; //@line 3619 "lbfgsb.c"
  $77 = HEAP32[$76>>2]|0; //@line 3619 "lbfgsb.c"
  $78 = ($77|0)==(-5); //@line 3619 "lbfgsb.c"
  if ($78) {
   (_s_wsfe(2336)|0); //@line 3620 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3621 "lbfgsb.c"
  }
  $79 = $5; //@line 3623 "lbfgsb.c"
  $80 = HEAP32[$79>>2]|0; //@line 3623 "lbfgsb.c"
  $81 = ($80|0)==(-6); //@line 3623 "lbfgsb.c"
  if ($81) {
   (_s_wsle(2356)|0); //@line 3624 "lbfgsb.c"
   (_do_lio(1308,1236,17490,11)|0); //@line 3625 "lbfgsb.c"
   $82 = $18; //@line 3626 "lbfgsb.c"
   (_do_lio(1660,1236,$82,4)|0); //@line 3626 "lbfgsb.c"
   (_do_lio(1308,1236,17502,13)|0); //@line 3627 "lbfgsb.c"
   (_e_wsle()|0); //@line 3628 "lbfgsb.c"
  }
  $83 = $5; //@line 3630 "lbfgsb.c"
  $84 = HEAP32[$83>>2]|0; //@line 3630 "lbfgsb.c"
  $85 = ($84|0)==(-7); //@line 3630 "lbfgsb.c"
  if ($85) {
   (_s_wsle(2376)|0); //@line 3631 "lbfgsb.c"
   (_do_lio(1308,1236,17516,3)|0); //@line 3632 "lbfgsb.c"
   $86 = $18; //@line 3633 "lbfgsb.c"
   (_do_lio(1660,1236,$86,4)|0); //@line 3633 "lbfgsb.c"
   (_do_lio(1308,1236,17520,6)|0); //@line 3634 "lbfgsb.c"
   $87 = $18; //@line 3635 "lbfgsb.c"
   (_do_lio(1660,1236,$87,4)|0); //@line 3635 "lbfgsb.c"
   (_do_lio(1308,1236,17527,25)|0); //@line 3636 "lbfgsb.c"
   (_e_wsle()|0); //@line 3637 "lbfgsb.c"
  }
  $88 = $5; //@line 3639 "lbfgsb.c"
  $89 = HEAP32[$88>>2]|0; //@line 3639 "lbfgsb.c"
  $90 = ($89|0)==(-8); //@line 3639 "lbfgsb.c"
  if ($90) {
   (_s_wsfe(2396)|0); //@line 3640 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3641 "lbfgsb.c"
  }
  $91 = $5; //@line 3643 "lbfgsb.c"
  $92 = HEAP32[$91>>2]|0; //@line 3643 "lbfgsb.c"
  $93 = ($92|0)==(-9); //@line 3643 "lbfgsb.c"
  if ($93) {
   (_s_wsfe(2416)|0); //@line 3644 "lbfgsb.c"
   (_e_wsfe()|0); //@line 3645 "lbfgsb.c"
  }
 }
 $94 = $4; //@line 3648 "lbfgsb.c"
 $95 = HEAP32[$94>>2]|0; //@line 3648 "lbfgsb.c"
 $96 = ($95|0)>=(1); //@line 3648 "lbfgsb.c"
 if ($96) {
  (_s_wsfe(2436)|0); //@line 3649 "lbfgsb.c"
  $97 = $19; //@line 3650 "lbfgsb.c"
  (_do_fio(1236,$97,8)|0); //@line 3650 "lbfgsb.c"
  $98 = $20; //@line 3651 "lbfgsb.c"
  (_do_fio(1236,$98,8)|0); //@line 3651 "lbfgsb.c"
  $99 = $21; //@line 3652 "lbfgsb.c"
  (_do_fio(1236,$99,8)|0); //@line 3652 "lbfgsb.c"
  (_e_wsfe()|0); //@line 3653 "lbfgsb.c"
 }
 (_s_wsfe(2456)|0); //@line 3655 "lbfgsb.c"
 $100 = $12; //@line 3656 "lbfgsb.c"
 (_do_fio(1236,$100,8)|0); //@line 3656 "lbfgsb.c"
 (_e_wsfe()|0); //@line 3657 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 3660 "lbfgsb.c"
}
function _prn1lb_($n,$m,$l,$u,$x,$iprint,$epsmch) {
 //console.log("prn1lb");
 $n = $n|0;
 $m = $m|0;
 $l = $l|0;
 $u = $u|0;
 $x = $x|0;
 $iprint = $iprint|0;
 $epsmch = $epsmch|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $m;
 $2 = $l;
 $3 = $u;
 $4 = $x;
 $5 = $iprint;
 $6 = $epsmch;
 $7 = $4; //@line 3295 "lbfgsb.c"
 $8 = ((($7)) + -8|0); //@line 3295 "lbfgsb.c"
 $4 = $8; //@line 3295 "lbfgsb.c"
 $9 = $3; //@line 3296 "lbfgsb.c"
 $10 = ((($9)) + -8|0); //@line 3296 "lbfgsb.c"
 $3 = $10; //@line 3296 "lbfgsb.c"
 $11 = $2; //@line 3297 "lbfgsb.c"
 $12 = ((($11)) + -8|0); //@line 3297 "lbfgsb.c"
 $2 = $12; //@line 3297 "lbfgsb.c"
 $13 = $5; //@line 3300 "lbfgsb.c"
 $14 = HEAP32[$13>>2]|0; //@line 3300 "lbfgsb.c"
 //console.log("iprint in prn1lb: " + $14);
 $15 = ($14|0)>=(0); //@line 3300 "lbfgsb.c"
 if (!($15)) {
  STACKTOP = sp;return 0; //@line 3341 "lbfgsb.c"
 }
 (_s_wsfe(1924)|0); //@line 3301 "lbfgsb.c"
 $16 = $6; //@line 3302 "lbfgsb.c"
 (_do_fio(1236,$16,8)|0); //@line 3302 "lbfgsb.c"
 (_e_wsfe()|0); //@line 3303 "lbfgsb.c"
 (_s_wsle(1944)|0); //@line 3304 "lbfgsb.c"
 (_do_lio(1308,1236,15598,4)|0); //@line 3305 "lbfgsb.c"
 $17 = $0; //@line 3306 "lbfgsb.c"
 (_do_lio(1660,1236,$17,4)|0); //@line 3306 "lbfgsb.c"
 (_do_lio(1308,1236,15603,8)|0); //@line 3307 "lbfgsb.c"
 $18 = $1; //@line 3308 "lbfgsb.c"
 (_do_lio(1660,1236,$18,4)|0); //@line 3308 "lbfgsb.c"
 (_e_wsle()|0); //@line 3309 "lbfgsb.c"
 $19 = $5; //@line 3310 "lbfgsb.c"
 $20 = HEAP32[$19>>2]|0; //@line 3310 "lbfgsb.c"
 $21 = ($20|0)>=(1); //@line 3310 "lbfgsb.c"
 if (!($21)) {
  STACKTOP = sp;return 0; //@line 3341 "lbfgsb.c"
 }
 $22 = $5; //@line 3311 "lbfgsb.c"
 $23 = HEAP32[$22>>2]|0; //@line 3311 "lbfgsb.c"
 $24 = ($23|0)>(100); //@line 3311 "lbfgsb.c"
 if (!($24)) {
  STACKTOP = sp;return 0; //@line 3341 "lbfgsb.c"
 }
 (_s_wsfe(1964)|0); //@line 3312 "lbfgsb.c"
 (_do_fio(1236,15612,3)|0); //@line 3313 "lbfgsb.c"
 $25 = $0; //@line 3314 "lbfgsb.c"
 $26 = HEAP32[$25>>2]|0; //@line 3314 "lbfgsb.c"
 $i__1 = $26; //@line 3314 "lbfgsb.c"
 HEAP32[1920>>2] = 1; //@line 3315 "lbfgsb.c"
 while(1) {
  $27 = HEAP32[1920>>2]|0; //@line 3315 "lbfgsb.c"
  $28 = $i__1; //@line 3315 "lbfgsb.c"
  $29 = ($27|0)<=($28|0); //@line 3315 "lbfgsb.c"
  if (!($29)) {
   break;
  }
  $30 = HEAP32[1920>>2]|0; //@line 3316 "lbfgsb.c"
  $31 = $2; //@line 3316 "lbfgsb.c"
  $32 = (($31) + ($30<<3)|0); //@line 3316 "lbfgsb.c"
  (_do_fio(1236,$32,8)|0); //@line 3316 "lbfgsb.c"
  $33 = HEAP32[1920>>2]|0; //@line 3315 "lbfgsb.c"
  $34 = (($33) + 1)|0; //@line 3315 "lbfgsb.c"
  HEAP32[1920>>2] = $34; //@line 3315 "lbfgsb.c"
 }
 (_e_wsfe()|0); //@line 3319 "lbfgsb.c"
 (_s_wsfe(1984)|0); //@line 3320 "lbfgsb.c"
 (_do_fio(1236,15616,4)|0); //@line 3321 "lbfgsb.c"
 $35 = $0; //@line 3322 "lbfgsb.c"
 $36 = HEAP32[$35>>2]|0; //@line 3322 "lbfgsb.c"
 $i__1 = $36; //@line 3322 "lbfgsb.c"
 HEAP32[1920>>2] = 1; //@line 3323 "lbfgsb.c"
 while(1) {
  $37 = HEAP32[1920>>2]|0; //@line 3323 "lbfgsb.c"
  $38 = $i__1; //@line 3323 "lbfgsb.c"
  $39 = ($37|0)<=($38|0); //@line 3323 "lbfgsb.c"
  if (!($39)) {
   break;
  }
  $40 = HEAP32[1920>>2]|0; //@line 3324 "lbfgsb.c"
  $41 = $4; //@line 3324 "lbfgsb.c"
  $42 = (($41) + ($40<<3)|0); //@line 3324 "lbfgsb.c"
  (_do_fio(1236,$42,8)|0); //@line 3324 "lbfgsb.c"
  $43 = HEAP32[1920>>2]|0; //@line 3323 "lbfgsb.c"
  $44 = (($43) + 1)|0; //@line 3323 "lbfgsb.c"
  HEAP32[1920>>2] = $44; //@line 3323 "lbfgsb.c"
 }
 (_e_wsfe()|0); //@line 3327 "lbfgsb.c"
 (_s_wsfe(2004)|0); //@line 3328 "lbfgsb.c"
 (_do_fio(1236,15621,3)|0); //@line 3329 "lbfgsb.c"
 $45 = $0; //@line 3330 "lbfgsb.c"
 $46 = HEAP32[$45>>2]|0; //@line 3330 "lbfgsb.c"
 $i__1 = $46; //@line 3330 "lbfgsb.c"
 HEAP32[1920>>2] = 1; //@line 3331 "lbfgsb.c"
 while(1) {
  $47 = HEAP32[1920>>2]|0; //@line 3331 "lbfgsb.c"
  $48 = $i__1; //@line 3331 "lbfgsb.c"
  $49 = ($47|0)<=($48|0); //@line 3331 "lbfgsb.c"
  if (!($49)) {
   break;
  }
  $50 = HEAP32[1920>>2]|0; //@line 3332 "lbfgsb.c"
  $51 = $3; //@line 3332 "lbfgsb.c"
  $52 = (($51) + ($50<<3)|0); //@line 3332 "lbfgsb.c"
  (_do_fio(1236,$52,8)|0); //@line 3332 "lbfgsb.c"
  $53 = HEAP32[1920>>2]|0; //@line 3331 "lbfgsb.c"
  $54 = (($53) + 1)|0; //@line 3331 "lbfgsb.c"
  HEAP32[1920>>2] = $54; //@line 3331 "lbfgsb.c"
 }
 (_e_wsfe()|0); //@line 3335 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 3341 "lbfgsb.c"
}
function _active_($n,$l,$u,$nbd,$x,$iwhere,$iprint,$prjctd,$cnstnd,$boxed) {
 $n = $n|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $x = $x|0;
 $iwhere = $iwhere|0;
 $iprint = $iprint|0;
 $prjctd = $prjctd|0;
 $cnstnd = $cnstnd|0;
 $boxed = $boxed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0.0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0, $43 = 0, $44 = 0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $l;
 $2 = $u;
 $3 = $nbd;
 $4 = $x;
 $5 = $iwhere;
 $6 = $iprint;
 $7 = $prjctd;
 $8 = $cnstnd;
 $9 = $boxed;
 $10 = $5; //@line 1237 "lbfgsb.c"
 $11 = ((($10)) + -4|0); //@line 1237 "lbfgsb.c"
 $5 = $11; //@line 1237 "lbfgsb.c"
 $12 = $4; //@line 1238 "lbfgsb.c"
 $13 = ((($12)) + -8|0); //@line 1238 "lbfgsb.c"
 $4 = $13; //@line 1238 "lbfgsb.c"
 $14 = $3; //@line 1239 "lbfgsb.c"
 $15 = ((($14)) + -4|0); //@line 1239 "lbfgsb.c"
 $3 = $15; //@line 1239 "lbfgsb.c"
 $16 = $2; //@line 1240 "lbfgsb.c"
 $17 = ((($16)) + -8|0); //@line 1240 "lbfgsb.c"
 $2 = $17; //@line 1240 "lbfgsb.c"
 $18 = $1; //@line 1241 "lbfgsb.c"
 $19 = ((($18)) + -8|0); //@line 1241 "lbfgsb.c"
 $1 = $19; //@line 1241 "lbfgsb.c"
 HEAP32[1244>>2] = 0; //@line 1244 "lbfgsb.c"
 $20 = $7; //@line 1245 "lbfgsb.c"
 HEAP32[$20>>2] = 0; //@line 1245 "lbfgsb.c"
 $21 = $8; //@line 1246 "lbfgsb.c"
 HEAP32[$21>>2] = 0; //@line 1246 "lbfgsb.c"
 $22 = $9; //@line 1247 "lbfgsb.c"
 HEAP32[$22>>2] = 1; //@line 1247 "lbfgsb.c"
 $23 = $0; //@line 1249 "lbfgsb.c"
 $24 = HEAP32[$23>>2]|0; //@line 1249 "lbfgsb.c"
 $i__1 = $24; //@line 1249 "lbfgsb.c"
 HEAP32[1240>>2] = 1; //@line 1250 "lbfgsb.c"
 while(1) {
  $25 = HEAP32[1240>>2]|0; //@line 1250 "lbfgsb.c"
  $26 = $i__1; //@line 1250 "lbfgsb.c"
  $27 = ($25|0)<=($26|0); //@line 1250 "lbfgsb.c"
  if (!($27)) {
   break;
  }
  $28 = HEAP32[1240>>2]|0; //@line 1251 "lbfgsb.c"
  $29 = $3; //@line 1251 "lbfgsb.c"
  $30 = (($29) + ($28<<2)|0); //@line 1251 "lbfgsb.c"
  $31 = HEAP32[$30>>2]|0; //@line 1251 "lbfgsb.c"
  $32 = ($31|0)>(0); //@line 1251 "lbfgsb.c"
  do {
   if ($32) {
    $33 = HEAP32[1240>>2]|0; //@line 1252 "lbfgsb.c"
    $34 = $3; //@line 1252 "lbfgsb.c"
    $35 = (($34) + ($33<<2)|0); //@line 1252 "lbfgsb.c"
    $36 = HEAP32[$35>>2]|0; //@line 1252 "lbfgsb.c"
    $37 = ($36|0)<=(2); //@line 1252 "lbfgsb.c"
    if ($37) {
     $38 = HEAP32[1240>>2]|0; //@line 1252 "lbfgsb.c"
     $39 = $4; //@line 1252 "lbfgsb.c"
     $40 = (($39) + ($38<<3)|0); //@line 1252 "lbfgsb.c"
     $41 = +HEAPF64[$40>>3]; //@line 1252 "lbfgsb.c"
     $42 = HEAP32[1240>>2]|0; //@line 1252 "lbfgsb.c"
     $43 = $1; //@line 1252 "lbfgsb.c"
     $44 = (($43) + ($42<<3)|0); //@line 1252 "lbfgsb.c"
     $45 = +HEAPF64[$44>>3]; //@line 1252 "lbfgsb.c"
     $46 = $41 <= $45; //@line 1252 "lbfgsb.c"
     if ($46) {
      $47 = HEAP32[1240>>2]|0; //@line 1253 "lbfgsb.c"
      $48 = $4; //@line 1253 "lbfgsb.c"
      $49 = (($48) + ($47<<3)|0); //@line 1253 "lbfgsb.c"
      $50 = +HEAPF64[$49>>3]; //@line 1253 "lbfgsb.c"
      $51 = HEAP32[1240>>2]|0; //@line 1253 "lbfgsb.c"
      $52 = $1; //@line 1253 "lbfgsb.c"
      $53 = (($52) + ($51<<3)|0); //@line 1253 "lbfgsb.c"
      $54 = +HEAPF64[$53>>3]; //@line 1253 "lbfgsb.c"
      $55 = $50 < $54; //@line 1253 "lbfgsb.c"
      if ($55) {
       $56 = $7; //@line 1254 "lbfgsb.c"
       HEAP32[$56>>2] = 1; //@line 1254 "lbfgsb.c"
       $57 = HEAP32[1240>>2]|0; //@line 1255 "lbfgsb.c"
       $58 = $1; //@line 1255 "lbfgsb.c"
       $59 = (($58) + ($57<<3)|0); //@line 1255 "lbfgsb.c"
       $60 = +HEAPF64[$59>>3]; //@line 1255 "lbfgsb.c"
       $61 = HEAP32[1240>>2]|0; //@line 1255 "lbfgsb.c"
       $62 = $4; //@line 1255 "lbfgsb.c"
       $63 = (($62) + ($61<<3)|0); //@line 1255 "lbfgsb.c"
       HEAPF64[$63>>3] = $60; //@line 1255 "lbfgsb.c"
      }
      $64 = HEAP32[1244>>2]|0; //@line 1257 "lbfgsb.c"
      $65 = (($64) + 1)|0; //@line 1257 "lbfgsb.c"
      HEAP32[1244>>2] = $65; //@line 1257 "lbfgsb.c"
      break;
     }
    }
    $66 = HEAP32[1240>>2]|0; //@line 1258 "lbfgsb.c"
    $67 = $3; //@line 1258 "lbfgsb.c"
    $68 = (($67) + ($66<<2)|0); //@line 1258 "lbfgsb.c"
    $69 = HEAP32[$68>>2]|0; //@line 1258 "lbfgsb.c"
    $70 = ($69|0)>=(2); //@line 1258 "lbfgsb.c"
    if ($70) {
     $71 = HEAP32[1240>>2]|0; //@line 1258 "lbfgsb.c"
     $72 = $4; //@line 1258 "lbfgsb.c"
     $73 = (($72) + ($71<<3)|0); //@line 1258 "lbfgsb.c"
     $74 = +HEAPF64[$73>>3]; //@line 1258 "lbfgsb.c"
     $75 = HEAP32[1240>>2]|0; //@line 1258 "lbfgsb.c"
     $76 = $2; //@line 1258 "lbfgsb.c"
     $77 = (($76) + ($75<<3)|0); //@line 1258 "lbfgsb.c"
     $78 = +HEAPF64[$77>>3]; //@line 1258 "lbfgsb.c"
     $79 = $74 >= $78; //@line 1258 "lbfgsb.c"
     if ($79) {
      $80 = HEAP32[1240>>2]|0; //@line 1259 "lbfgsb.c"
      $81 = $4; //@line 1259 "lbfgsb.c"
      $82 = (($81) + ($80<<3)|0); //@line 1259 "lbfgsb.c"
      $83 = +HEAPF64[$82>>3]; //@line 1259 "lbfgsb.c"
      $84 = HEAP32[1240>>2]|0; //@line 1259 "lbfgsb.c"
      $85 = $2; //@line 1259 "lbfgsb.c"
      $86 = (($85) + ($84<<3)|0); //@line 1259 "lbfgsb.c"
      $87 = +HEAPF64[$86>>3]; //@line 1259 "lbfgsb.c"
      $88 = $83 > $87; //@line 1259 "lbfgsb.c"
      if ($88) {
       $89 = $7; //@line 1260 "lbfgsb.c"
       HEAP32[$89>>2] = 1; //@line 1260 "lbfgsb.c"
       $90 = HEAP32[1240>>2]|0; //@line 1261 "lbfgsb.c"
       $91 = $2; //@line 1261 "lbfgsb.c"
       $92 = (($91) + ($90<<3)|0); //@line 1261 "lbfgsb.c"
       $93 = +HEAPF64[$92>>3]; //@line 1261 "lbfgsb.c"
       $94 = HEAP32[1240>>2]|0; //@line 1261 "lbfgsb.c"
       $95 = $4; //@line 1261 "lbfgsb.c"
       $96 = (($95) + ($94<<3)|0); //@line 1261 "lbfgsb.c"
       HEAPF64[$96>>3] = $93; //@line 1261 "lbfgsb.c"
      }
      $97 = HEAP32[1244>>2]|0; //@line 1263 "lbfgsb.c"
      $98 = (($97) + 1)|0; //@line 1263 "lbfgsb.c"
      HEAP32[1244>>2] = $98; //@line 1263 "lbfgsb.c"
     }
    }
   }
  } while(0);
  $99 = HEAP32[1240>>2]|0; //@line 1250 "lbfgsb.c"
  $100 = (($99) + 1)|0; //@line 1250 "lbfgsb.c"
  HEAP32[1240>>2] = $100; //@line 1250 "lbfgsb.c"
 }
 $101 = $0; //@line 1269 "lbfgsb.c"
 $102 = HEAP32[$101>>2]|0; //@line 1269 "lbfgsb.c"
 $i__1 = $102; //@line 1269 "lbfgsb.c"
 HEAP32[1240>>2] = 1; //@line 1270 "lbfgsb.c"
 while(1) {
  $103 = HEAP32[1240>>2]|0; //@line 1270 "lbfgsb.c"
  $104 = $i__1; //@line 1270 "lbfgsb.c"
  $105 = ($103|0)<=($104|0); //@line 1270 "lbfgsb.c"
  if (!($105)) {
   break;
  }
  $106 = HEAP32[1240>>2]|0; //@line 1271 "lbfgsb.c"
  $107 = $3; //@line 1271 "lbfgsb.c"
  $108 = (($107) + ($106<<2)|0); //@line 1271 "lbfgsb.c"
  $109 = HEAP32[$108>>2]|0; //@line 1271 "lbfgsb.c"
  $110 = ($109|0)!=(2); //@line 1271 "lbfgsb.c"
  if ($110) {
   $111 = $9; //@line 1272 "lbfgsb.c"
   HEAP32[$111>>2] = 0; //@line 1272 "lbfgsb.c"
  }
  $112 = HEAP32[1240>>2]|0; //@line 1274 "lbfgsb.c"
  $113 = $3; //@line 1274 "lbfgsb.c"
  $114 = (($113) + ($112<<2)|0); //@line 1274 "lbfgsb.c"
  $115 = HEAP32[$114>>2]|0; //@line 1274 "lbfgsb.c"
  $116 = ($115|0)==(0); //@line 1274 "lbfgsb.c"
  do {
   if ($116) {
    $117 = HEAP32[1240>>2]|0; //@line 1276 "lbfgsb.c"
    $118 = $5; //@line 1276 "lbfgsb.c"
    $119 = (($118) + ($117<<2)|0); //@line 1276 "lbfgsb.c"
    HEAP32[$119>>2] = -1; //@line 1276 "lbfgsb.c"
   } else {
    $120 = $8; //@line 1279 "lbfgsb.c"
    HEAP32[$120>>2] = 1; //@line 1279 "lbfgsb.c"
    $121 = HEAP32[1240>>2]|0; //@line 1280 "lbfgsb.c"
    $122 = $3; //@line 1280 "lbfgsb.c"
    $123 = (($122) + ($121<<2)|0); //@line 1280 "lbfgsb.c"
    $124 = HEAP32[$123>>2]|0; //@line 1280 "lbfgsb.c"
    $125 = ($124|0)==(2); //@line 1280 "lbfgsb.c"
    if ($125) {
     $126 = HEAP32[1240>>2]|0; //@line 1280 "lbfgsb.c"
     $127 = $2; //@line 1280 "lbfgsb.c"
     $128 = (($127) + ($126<<3)|0); //@line 1280 "lbfgsb.c"
     $129 = +HEAPF64[$128>>3]; //@line 1280 "lbfgsb.c"
     $130 = HEAP32[1240>>2]|0; //@line 1280 "lbfgsb.c"
     $131 = $1; //@line 1280 "lbfgsb.c"
     $132 = (($131) + ($130<<3)|0); //@line 1280 "lbfgsb.c"
     $133 = +HEAPF64[$132>>3]; //@line 1280 "lbfgsb.c"
     $134 = $129 - $133; //@line 1280 "lbfgsb.c"
     $135 = $134 <= 0.0; //@line 1280 "lbfgsb.c"
     if ($135) {
      $136 = HEAP32[1240>>2]|0; //@line 1282 "lbfgsb.c"
      $137 = $5; //@line 1282 "lbfgsb.c"
      $138 = (($137) + ($136<<2)|0); //@line 1282 "lbfgsb.c"
      HEAP32[$138>>2] = 3; //@line 1282 "lbfgsb.c"
      break;
     }
    }
    $139 = HEAP32[1240>>2]|0; //@line 1284 "lbfgsb.c"
    $140 = $5; //@line 1284 "lbfgsb.c"
    $141 = (($140) + ($139<<2)|0); //@line 1284 "lbfgsb.c"
    HEAP32[$141>>2] = 0; //@line 1284 "lbfgsb.c"
   }
  } while(0);
  $142 = HEAP32[1240>>2]|0; //@line 1270 "lbfgsb.c"
  $143 = (($142) + 1)|0; //@line 1270 "lbfgsb.c"
  HEAP32[1240>>2] = $143; //@line 1270 "lbfgsb.c"
 }
 $144 = $6; //@line 1289 "lbfgsb.c"
 $145 = HEAP32[$144>>2]|0; //@line 1289 "lbfgsb.c"
 $146 = ($145|0)>=(0); //@line 1289 "lbfgsb.c"
 if ($146) {
  $147 = $7; //@line 1290 "lbfgsb.c"
  $148 = HEAP32[$147>>2]|0; //@line 1290 "lbfgsb.c"
  $149 = ($148|0)!=(0); //@line 1290 "lbfgsb.c"
  if ($149) {
   (_s_wsle(1248)|0); //@line 1291 "lbfgsb.c"
   (_do_lio(1308,1236,14601,58)|0); //@line 1292 "lbfgsb.c"
   (_e_wsle()|0); //@line 1294 "lbfgsb.c"
  }
  $150 = $8; //@line 1296 "lbfgsb.c"
  $151 = HEAP32[$150>>2]|0; //@line 1296 "lbfgsb.c"
  $152 = ($151|0)!=(0); //@line 1296 "lbfgsb.c"
  if (!($152)) {
   (_s_wsle(1268)|0); //@line 1297 "lbfgsb.c"
   (_do_lio(1308,1236,14660,30)|0); //@line 1298 "lbfgsb.c"
   (_e_wsle()|0); //@line 1300 "lbfgsb.c"
  }
 }
 $153 = $6; //@line 1303 "lbfgsb.c"
 $154 = HEAP32[$153>>2]|0; //@line 1303 "lbfgsb.c"
 $155 = ($154|0)>(0); //@line 1303 "lbfgsb.c"
 if (!($155)) {
  STACKTOP = sp;return 0; //@line 1308 "lbfgsb.c"
 }
 (_s_wsfe(1288)|0); //@line 1304 "lbfgsb.c"
 (_do_fio(1236,1244,4)|0); //@line 1305 "lbfgsb.c"
 (_e_wsfe()|0); //@line 1306 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 1308 "lbfgsb.c"
}
function _projgr_($n,$l,$u,$nbd,$x,$g,$sbgnrm) {
 $n = $n|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $x = $x|0;
 $g = $g|0;
 $sbgnrm = $sbgnrm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0;
 var $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0.0;
 var $81 = 0.0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $d__1 = 0.0, $d__2 = 0.0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $l;
 $2 = $u;
 $3 = $nbd;
 $4 = $x;
 $5 = $g;
 $6 = $sbgnrm;
 $7 = $5; //@line 3695 "lbfgsb.c"
 $8 = ((($7)) + -8|0); //@line 3695 "lbfgsb.c"
 $5 = $8; //@line 3695 "lbfgsb.c"
 $9 = $4; //@line 3696 "lbfgsb.c"
 $10 = ((($9)) + -8|0); //@line 3696 "lbfgsb.c"
 $4 = $10; //@line 3696 "lbfgsb.c"
 $11 = $3; //@line 3697 "lbfgsb.c"
 $12 = ((($11)) + -4|0); //@line 3697 "lbfgsb.c"
 $3 = $12; //@line 3697 "lbfgsb.c"
 $13 = $2; //@line 3698 "lbfgsb.c"
 $14 = ((($13)) + -8|0); //@line 3698 "lbfgsb.c"
 $2 = $14; //@line 3698 "lbfgsb.c"
 $15 = $1; //@line 3699 "lbfgsb.c"
 $16 = ((($15)) + -8|0); //@line 3699 "lbfgsb.c"
 $1 = $16; //@line 3699 "lbfgsb.c"
 $17 = $6; //@line 3702 "lbfgsb.c"
 HEAPF64[$17>>3] = 0.0; //@line 3702 "lbfgsb.c"
 $18 = $0; //@line 3703 "lbfgsb.c"
 $19 = HEAP32[$18>>2]|0; //@line 3703 "lbfgsb.c"
 //console.log("n == " + $19);
 $i__1 = $19; //@line 3703 "lbfgsb.c"
 HEAP32[2476>>2] = 1; //@line 3704 "lbfgsb.c"
 while(1) {
  $20 = HEAP32[2476>>2]|0; //@line 3704 "lbfgsb.c"
  $21 = $i__1; //@line 3704 "lbfgsb.c"
  $22 = ($20|0)<=($21|0); //@line 3704 "lbfgsb.c"
  if (!($22)) {
   break;
  }
  $23 = HEAP32[2476>>2]|0; //@line 3705 "lbfgsb.c"
  $24 = $5; //@line 3705 "lbfgsb.c"
  $25 = (($24) + ($23<<3)|0); //@line 3705 "lbfgsb.c"
  $26 = +HEAPF64[$25>>3]; //@line 3705 "lbfgsb.c"
  HEAPF64[464>>3] = $26; //@line 3705 "lbfgsb.c"
  $27 = HEAP32[2476>>2]|0; //@line 3706 "lbfgsb.c"
  $28 = $3; //@line 3706 "lbfgsb.c"
  $29 = (($28) + ($27<<2)|0); //@line 3706 "lbfgsb.c"
  $30 = HEAP32[$29>>2]|0; //@line 3706 "lbfgsb.c"
  $31 = ($30|0)!=(0); //@line 3706 "lbfgsb.c"
  do {
   if ($31) {
    $32 = +HEAPF64[464>>3]; //@line 3707 "lbfgsb.c"
    $33 = $32 < 0.0; //@line 3707 "lbfgsb.c"
    $34 = HEAP32[2476>>2]|0; //@line 3708 "lbfgsb.c"
    $35 = $3; //@line 3708 "lbfgsb.c"
    $36 = (($35) + ($34<<2)|0); //@line 3708 "lbfgsb.c"
    $37 = HEAP32[$36>>2]|0; //@line 3708 "lbfgsb.c"
    if ($33) {
     $38 = ($37|0)>=(2); //@line 3708 "lbfgsb.c"
     if (!($38)) {
      break;
     }
     $39 = HEAP32[2476>>2]|0; //@line 3710 "lbfgsb.c"
     $40 = $4; //@line 3710 "lbfgsb.c"
     $41 = (($40) + ($39<<3)|0); //@line 3710 "lbfgsb.c"
     $42 = +HEAPF64[$41>>3]; //@line 3710 "lbfgsb.c"
     $43 = HEAP32[2476>>2]|0; //@line 3710 "lbfgsb.c"
     $44 = $2; //@line 3710 "lbfgsb.c"
     $45 = (($44) + ($43<<3)|0); //@line 3710 "lbfgsb.c"
     $46 = +HEAPF64[$45>>3]; //@line 3710 "lbfgsb.c"
     $47 = $42 - $46; //@line 3710 "lbfgsb.c"
     $d__1 = $47; //@line 3710 "lbfgsb.c"
     $48 = $d__1; //@line 3711 "lbfgsb.c"
     $49 = +HEAPF64[464>>3]; //@line 3711 "lbfgsb.c"
     $50 = $48 >= $49; //@line 3711 "lbfgsb.c"
     $51 = $d__1; //@line 3711 "lbfgsb.c"
     $52 = +HEAPF64[464>>3]; //@line 3711 "lbfgsb.c"
     $53 = $50 ? $51 : $52; //@line 3711 "lbfgsb.c"
     HEAPF64[464>>3] = $53; //@line 3711 "lbfgsb.c"
     break;
    } else {
     $54 = ($37|0)<=(2); //@line 3714 "lbfgsb.c"
     if (!($54)) {
      break;
     }
     $55 = HEAP32[2476>>2]|0; //@line 3716 "lbfgsb.c"
     $56 = $4; //@line 3716 "lbfgsb.c"
     $57 = (($56) + ($55<<3)|0); //@line 3716 "lbfgsb.c"
     $58 = +HEAPF64[$57>>3]; //@line 3716 "lbfgsb.c"
     $59 = HEAP32[2476>>2]|0; //@line 3716 "lbfgsb.c"
     $60 = $1; //@line 3716 "lbfgsb.c"
     $61 = (($60) + ($59<<3)|0); //@line 3716 "lbfgsb.c"
     $62 = +HEAPF64[$61>>3]; //@line 3716 "lbfgsb.c"
     $63 = $58 - $62; //@line 3716 "lbfgsb.c"
     $d__1 = $63; //@line 3716 "lbfgsb.c"
     $64 = $d__1; //@line 3717 "lbfgsb.c"
     $65 = +HEAPF64[464>>3]; //@line 3717 "lbfgsb.c"
     $66 = $64 <= $65; //@line 3717 "lbfgsb.c"
     $67 = $d__1; //@line 3717 "lbfgsb.c"
     $68 = +HEAPF64[464>>3]; //@line 3717 "lbfgsb.c"
     $69 = $66 ? $67 : $68; //@line 3717 "lbfgsb.c"
     HEAPF64[464>>3] = $69; //@line 3717 "lbfgsb.c"
     break;
    }
   }
  } while(0);
  $70 = $6; //@line 3722 "lbfgsb.c"
  $71 = +HEAPF64[$70>>3]; //@line 3722 "lbfgsb.c"
  $d__1 = $71; //@line 3722 "lbfgsb.c"
  $72 = +HEAPF64[464>>3]; //@line 3722 "lbfgsb.c"
  $73 = $72 >= 0.0; //@line 3722 "lbfgsb.c"
  $74 = +HEAPF64[464>>3]; //@line 3722 "lbfgsb.c"
  $75 = -$74; //@line 3722 "lbfgsb.c"
  $76 = $73 ? $74 : $75; //@line 3722 "lbfgsb.c"
  $d__2 = $76; //@line 3722 "lbfgsb.c"
  $77 = $d__1; //@line 3723 "lbfgsb.c"
  $78 = $d__2; //@line 3723 "lbfgsb.c"
  $79 = $77 >= $78; //@line 3723 "lbfgsb.c"
  $80 = $d__1; //@line 3723 "lbfgsb.c"
  $81 = $d__2; //@line 3723 "lbfgsb.c"
  $82 = $79 ? $80 : $81; //@line 3723 "lbfgsb.c"
  $83 = $6; //@line 3723 "lbfgsb.c"
  HEAPF64[$83>>3] = $82; //@line 3723 "lbfgsb.c"
  $84 = HEAP32[2476>>2]|0; //@line 3704 "lbfgsb.c"
  $85 = (($84) + 1)|0; //@line 3704 "lbfgsb.c"
  HEAP32[2476>>2] = $85; //@line 3704 "lbfgsb.c"
 }
 STACKTOP = sp;return 0; //@line 3726 "lbfgsb.c"
}
function _cauchy_($n,$x,$l,$u,$nbd,$g,$iorder,$iwhere,$t,$d__,$xcp,$m,$wy,$ws,$sy,$wt,$theta,$col,$head,$p,$c__,$wbp,$v,$nseg,$iprint,$sbgnrm,$info,$epsmch) {
 $n = $n|0;
 $x = $x|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $g = $g|0;
 $iorder = $iorder|0;
 $iwhere = $iwhere|0;
 $t = $t|0;
 $d__ = $d__|0;
 $xcp = $xcp|0;
 $m = $m|0;
 $wy = $wy|0;
 $ws = $ws|0;
 $sy = $sy|0;
 $wt = $wt|0;
 $theta = $theta|0;
 $col = $col|0;
 $head = $head|0;
 $p = $p|0;
 $c__ = $c__|0;
 $wbp = $wbp|0;
 $v = $v|0;
 $nseg = $nseg|0;
 $iprint = $iprint|0;
 $sbgnrm = $sbgnrm|0;
 $info = $info|0;
 $epsmch = $epsmch|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0.0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0.0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0.0, $202 = 0.0, $203 = 0.0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0.0, $228 = 0.0, $229 = 0.0, $23 = 0, $230 = 0.0, $231 = 0.0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0.0, $245 = 0.0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0.0, $26 = 0;
 var $260 = 0.0, $261 = 0.0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0.0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0.0, $307 = 0.0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0.0, $339 = 0.0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0.0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0.0, $353 = 0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0.0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0.0, $406 = 0.0, $407 = 0.0, $408 = 0.0, $409 = 0.0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0.0, $434 = 0.0, $435 = 0.0, $436 = 0.0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0.0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0.0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0.0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0.0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0.0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0.0, $524 = 0.0, $525 = 0, $526 = 0, $527 = 0, $528 = 0.0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0.0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0.0, $542 = 0.0, $543 = 0, $544 = 0, $545 = 0, $546 = 0.0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0.0, $563 = 0, $564 = 0, $565 = 0, $566 = 0.0;
 var $567 = 0.0, $568 = 0.0, $569 = 0.0, $57 = 0, $570 = 0.0, $571 = 0.0, $572 = 0.0, $573 = 0.0, $574 = 0.0, $575 = 0.0, $576 = 0.0, $577 = 0, $578 = 0.0, $579 = 0.0, $58 = 0, $580 = 0.0, $581 = 0.0, $582 = 0.0, $583 = 0.0, $584 = 0;
 var $585 = 0.0, $586 = 0.0, $587 = 0.0, $588 = 0.0, $589 = 0.0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0.0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0.0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0.0, $625 = 0.0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0.0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0.0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0.0, $67 = 0, $670 = 0.0, $671 = 0.0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0.0, $677 = 0.0, $678 = 0.0, $679 = 0.0, $68 = 0, $680 = 0.0, $681 = 0.0, $682 = 0.0, $683 = 0.0, $684 = 0.0, $685 = 0.0, $686 = 0.0, $687 = 0.0, $688 = 0.0, $689 = 0.0, $69 = 0, $690 = 0.0, $691 = 0, $692 = 0.0;
 var $693 = 0.0, $694 = 0.0, $695 = 0.0, $696 = 0.0, $697 = 0, $698 = 0.0, $699 = 0.0, $7 = 0, $70 = 0, $700 = 0.0, $701 = 0, $702 = 0, $703 = 0.0, $704 = 0.0, $705 = 0.0, $706 = 0.0, $707 = 0, $708 = 0, $709 = 0.0, $71 = 0;
 var $710 = 0.0, $711 = 0.0, $712 = 0.0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0.0, $718 = 0, $719 = 0.0, $72 = 0, $720 = 0.0, $721 = 0.0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0.0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d__1 = 0, $i__1 = 0, $i__2 = 0, $or$cond = 0, $or$cond3 = 0, $sy_dim1 = 0, $sy_offset = 0, $ws_dim1 = 0, $ws_offset = 0;
 var $wt_dim1 = 0, $wt_offset = 0, $wy_dim1 = 0, $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 12|0;
 $d__1 = sp;
 $1 = $n;
 $2 = $x;
 $3 = $l;
 $4 = $u;
 $5 = $nbd;
 $6 = $g;
 $7 = $iorder;
 $8 = $iwhere;
 $9 = $t;
 $10 = $d__;
 $11 = $xcp;
 $12 = $m;
 $13 = $wy;
 $14 = $ws;
 $15 = $sy;
 $16 = $wt;
 $17 = $theta;
 $18 = $col;
 $19 = $head;
 $20 = $p;
 $21 = $c__;
 $22 = $wbp;
 $23 = $v;
 $24 = $nseg;
 $25 = $iprint;
 $26 = $sbgnrm;
 $27 = $info;
 $28 = $epsmch;
 $29 = $11; //@line 1716 "lbfgsb.c"
 $30 = ((($29)) + -8|0); //@line 1716 "lbfgsb.c"
 $11 = $30; //@line 1716 "lbfgsb.c"
 $31 = $10; //@line 1717 "lbfgsb.c"
 $32 = ((($31)) + -8|0); //@line 1717 "lbfgsb.c"
 $10 = $32; //@line 1717 "lbfgsb.c"
 $33 = $9; //@line 1718 "lbfgsb.c"
 $34 = ((($33)) + -8|0); //@line 1718 "lbfgsb.c"
 $9 = $34; //@line 1718 "lbfgsb.c"
 $35 = $8; //@line 1719 "lbfgsb.c"
 $36 = ((($35)) + -4|0); //@line 1719 "lbfgsb.c"
 $8 = $36; //@line 1719 "lbfgsb.c"
 $37 = $7; //@line 1720 "lbfgsb.c"
 $38 = ((($37)) + -4|0); //@line 1720 "lbfgsb.c"
 $7 = $38; //@line 1720 "lbfgsb.c"
 $39 = $6; //@line 1721 "lbfgsb.c"
 $40 = ((($39)) + -8|0); //@line 1721 "lbfgsb.c"
 $6 = $40; //@line 1721 "lbfgsb.c"
 $41 = $5; //@line 1722 "lbfgsb.c"
 $42 = ((($41)) + -4|0); //@line 1722 "lbfgsb.c"
 $5 = $42; //@line 1722 "lbfgsb.c"
 $43 = $4; //@line 1723 "lbfgsb.c"
 $44 = ((($43)) + -8|0); //@line 1723 "lbfgsb.c"
 $4 = $44; //@line 1723 "lbfgsb.c"
 $45 = $3; //@line 1724 "lbfgsb.c"
 $46 = ((($45)) + -8|0); //@line 1724 "lbfgsb.c"
 $3 = $46; //@line 1724 "lbfgsb.c"
 $47 = $2; //@line 1725 "lbfgsb.c"
 $48 = ((($47)) + -8|0); //@line 1725 "lbfgsb.c"
 $2 = $48; //@line 1725 "lbfgsb.c"
 $49 = $23; //@line 1726 "lbfgsb.c"
 $50 = ((($49)) + -8|0); //@line 1726 "lbfgsb.c"
 $23 = $50; //@line 1726 "lbfgsb.c"
 $51 = $22; //@line 1727 "lbfgsb.c"
 $52 = ((($51)) + -8|0); //@line 1727 "lbfgsb.c"
 $22 = $52; //@line 1727 "lbfgsb.c"
 $53 = $21; //@line 1728 "lbfgsb.c"
 $54 = ((($53)) + -8|0); //@line 1728 "lbfgsb.c"
 $21 = $54; //@line 1728 "lbfgsb.c"
 $55 = $20; //@line 1729 "lbfgsb.c"
 $56 = ((($55)) + -8|0); //@line 1729 "lbfgsb.c"
 $20 = $56; //@line 1729 "lbfgsb.c"
 $57 = $12; //@line 1730 "lbfgsb.c"
 $58 = HEAP32[$57>>2]|0; //@line 1730 "lbfgsb.c"
 $wt_dim1 = $58; //@line 1730 "lbfgsb.c"
 $59 = $wt_dim1; //@line 1731 "lbfgsb.c"
 $60 = (1 + ($59))|0; //@line 1731 "lbfgsb.c"
 $wt_offset = $60; //@line 1731 "lbfgsb.c"
 $61 = $wt_offset; //@line 1732 "lbfgsb.c"
 $62 = $16; //@line 1732 "lbfgsb.c"
 $63 = (0 - ($61))|0; //@line 1732 "lbfgsb.c"
 $64 = (($62) + ($63<<3)|0); //@line 1732 "lbfgsb.c"
 $16 = $64; //@line 1732 "lbfgsb.c"
 $65 = $12; //@line 1733 "lbfgsb.c"
 $66 = HEAP32[$65>>2]|0; //@line 1733 "lbfgsb.c"
 $sy_dim1 = $66; //@line 1733 "lbfgsb.c"
 $67 = $sy_dim1; //@line 1734 "lbfgsb.c"
 $68 = (1 + ($67))|0; //@line 1734 "lbfgsb.c"
 $sy_offset = $68; //@line 1734 "lbfgsb.c"
 $69 = $sy_offset; //@line 1735 "lbfgsb.c"
 $70 = $15; //@line 1735 "lbfgsb.c"
 $71 = (0 - ($69))|0; //@line 1735 "lbfgsb.c"
 $72 = (($70) + ($71<<3)|0); //@line 1735 "lbfgsb.c"
 $15 = $72; //@line 1735 "lbfgsb.c"
 $73 = $1; //@line 1736 "lbfgsb.c"
 $74 = HEAP32[$73>>2]|0; //@line 1736 "lbfgsb.c"
 $ws_dim1 = $74; //@line 1736 "lbfgsb.c"
 $75 = $ws_dim1; //@line 1737 "lbfgsb.c"
 $76 = (1 + ($75))|0; //@line 1737 "lbfgsb.c"
 $ws_offset = $76; //@line 1737 "lbfgsb.c"
 $77 = $ws_offset; //@line 1738 "lbfgsb.c"
 $78 = $14; //@line 1738 "lbfgsb.c"
 $79 = (0 - ($77))|0; //@line 1738 "lbfgsb.c"
 $80 = (($78) + ($79<<3)|0); //@line 1738 "lbfgsb.c"
 $14 = $80; //@line 1738 "lbfgsb.c"
 $81 = $1; //@line 1739 "lbfgsb.c"
 $82 = HEAP32[$81>>2]|0; //@line 1739 "lbfgsb.c"
 $wy_dim1 = $82; //@line 1739 "lbfgsb.c"
 $83 = $wy_dim1; //@line 1740 "lbfgsb.c"
 $84 = (1 + ($83))|0; //@line 1740 "lbfgsb.c"
 $wy_offset = $84; //@line 1740 "lbfgsb.c"
 $85 = $wy_offset; //@line 1741 "lbfgsb.c"
 $86 = $13; //@line 1741 "lbfgsb.c"
 $87 = (0 - ($85))|0; //@line 1741 "lbfgsb.c"
 $88 = (($86) + ($87<<3)|0); //@line 1741 "lbfgsb.c"
 $13 = $88; //@line 1741 "lbfgsb.c"
 $89 = $26; //@line 1744 "lbfgsb.c"
 $90 = +HEAPF64[$89>>3]; //@line 1744 "lbfgsb.c"
 $91 = $90 <= 0.0; //@line 1744 "lbfgsb.c"
 if ($91) {
  $92 = $25; //@line 1745 "lbfgsb.c"
  $93 = HEAP32[$92>>2]|0; //@line 1745 "lbfgsb.c"
  $94 = ($93|0)>=(0); //@line 1745 "lbfgsb.c"
  if ($94) {
   (_s_wsle(1380)|0); //@line 1746 "lbfgsb.c"
   (_do_lio(1308,1236,15068,23)|0); //@line 1747 "lbfgsb.c"
   (_e_wsle()|0); //@line 1748 "lbfgsb.c"
  }
  $95 = $1; //@line 1750 "lbfgsb.c"
  $96 = $2; //@line 1750 "lbfgsb.c"
  $97 = ((($96)) + 8|0); //@line 1750 "lbfgsb.c"
  $98 = $11; //@line 1750 "lbfgsb.c"
  $99 = ((($98)) + 8|0); //@line 1750 "lbfgsb.c"
  (_dcopy_($95,$97,1236,$99,1236)|0); //@line 1750 "lbfgsb.c"
  $0 = 0; //@line 1751 "lbfgsb.c"
  $750 = $0; //@line 2070 "lbfgsb.c"
  STACKTOP = sp;return ($750|0); //@line 2070 "lbfgsb.c"
 }
 HEAP32[1348>>2] = 1; //@line 1753 "lbfgsb.c"
 $100 = $1; //@line 1754 "lbfgsb.c"
 $101 = HEAP32[$100>>2]|0; //@line 1754 "lbfgsb.c"
 $102 = (($101) + 1)|0; //@line 1754 "lbfgsb.c"
 HEAP32[1352>>2] = $102; //@line 1754 "lbfgsb.c"
 HEAP32[1360>>2] = 0; //@line 1755 "lbfgsb.c"
 HEAP32[1364>>2] = 0; //@line 1756 "lbfgsb.c"
 HEAPF64[336>>3] = 0.0; //@line 1757 "lbfgsb.c"
 $103 = $18; //@line 1758 "lbfgsb.c"
 $104 = HEAP32[$103>>2]|0; //@line 1758 "lbfgsb.c"
 $105 = $104 << 1; //@line 1758 "lbfgsb.c"
 HEAP32[1340>>2] = $105; //@line 1758 "lbfgsb.c"
 HEAPF64[208>>3] = 0.0; //@line 1759 "lbfgsb.c"
 $106 = $25; //@line 1760 "lbfgsb.c"
 $107 = HEAP32[$106>>2]|0; //@line 1760 "lbfgsb.c"
 $108 = ($107|0)>=(99); //@line 1760 "lbfgsb.c"
 if ($108) {
  (_s_wsfe(1400)|0); //@line 1761 "lbfgsb.c"
  (_e_wsfe()|0); //@line 1762 "lbfgsb.c"
 }
 $109 = HEAP32[1340>>2]|0; //@line 1765 "lbfgsb.c"
 HEAP32[$i__1>>2] = $109; //@line 1765 "lbfgsb.c"
 HEAP32[1328>>2] = 1; //@line 1766 "lbfgsb.c"
 while(1) {
  $110 = HEAP32[1328>>2]|0; //@line 1766 "lbfgsb.c"
  $111 = HEAP32[$i__1>>2]|0; //@line 1766 "lbfgsb.c"
  $112 = ($110|0)<=($111|0); //@line 1766 "lbfgsb.c"
  if (!($112)) {
   break;
  }
  $113 = HEAP32[1328>>2]|0; //@line 1767 "lbfgsb.c"
  $114 = $20; //@line 1767 "lbfgsb.c"
  $115 = (($114) + ($113<<3)|0); //@line 1767 "lbfgsb.c"
  HEAPF64[$115>>3] = 0.0; //@line 1767 "lbfgsb.c"
  $116 = HEAP32[1328>>2]|0; //@line 1766 "lbfgsb.c"
  $117 = (($116) + 1)|0; //@line 1766 "lbfgsb.c"
  HEAP32[1328>>2] = $117; //@line 1766 "lbfgsb.c"
 }
 $118 = $1; //@line 1773 "lbfgsb.c"
 $119 = HEAP32[$118>>2]|0; //@line 1773 "lbfgsb.c"
 HEAP32[$i__1>>2] = $119; //@line 1773 "lbfgsb.c"
 HEAP32[1328>>2] = 1; //@line 1774 "lbfgsb.c"
 while(1) {
  $120 = HEAP32[1328>>2]|0; //@line 1774 "lbfgsb.c"
  $121 = HEAP32[$i__1>>2]|0; //@line 1774 "lbfgsb.c"
  $122 = ($120|0)<=($121|0); //@line 1774 "lbfgsb.c"
  if (!($122)) {
   break;
  }
  $123 = HEAP32[1328>>2]|0; //@line 1775 "lbfgsb.c"
  $124 = $6; //@line 1775 "lbfgsb.c"
  $125 = (($124) + ($123<<3)|0); //@line 1775 "lbfgsb.c"
  $126 = +HEAPF64[$125>>3]; //@line 1775 "lbfgsb.c"
  $127 = -$126; //@line 1775 "lbfgsb.c"
  HEAPF64[328>>3] = $127; //@line 1775 "lbfgsb.c"
  $128 = HEAP32[1328>>2]|0; //@line 1776 "lbfgsb.c"
  $129 = $8; //@line 1776 "lbfgsb.c"
  $130 = (($129) + ($128<<2)|0); //@line 1776 "lbfgsb.c"
  $131 = HEAP32[$130>>2]|0; //@line 1776 "lbfgsb.c"
  $132 = ($131|0)!=(3); //@line 1776 "lbfgsb.c"
  do {
   if ($132) {
    $133 = HEAP32[1328>>2]|0; //@line 1776 "lbfgsb.c"
    $134 = $8; //@line 1776 "lbfgsb.c"
    $135 = (($134) + ($133<<2)|0); //@line 1776 "lbfgsb.c"
    $136 = HEAP32[$135>>2]|0; //@line 1776 "lbfgsb.c"
    $137 = ($136|0)!=(-1); //@line 1776 "lbfgsb.c"
    if ($137) {
     $138 = HEAP32[1328>>2]|0; //@line 1779 "lbfgsb.c"
     $139 = $5; //@line 1779 "lbfgsb.c"
     $140 = (($139) + ($138<<2)|0); //@line 1779 "lbfgsb.c"
     $141 = HEAP32[$140>>2]|0; //@line 1779 "lbfgsb.c"
     $142 = ($141|0)<=(2); //@line 1779 "lbfgsb.c"
     if ($142) {
      $143 = HEAP32[1328>>2]|0; //@line 1780 "lbfgsb.c"
      $144 = $2; //@line 1780 "lbfgsb.c"
      $145 = (($144) + ($143<<3)|0); //@line 1780 "lbfgsb.c"
      $146 = +HEAPF64[$145>>3]; //@line 1780 "lbfgsb.c"
      $147 = HEAP32[1328>>2]|0; //@line 1780 "lbfgsb.c"
      $148 = $3; //@line 1780 "lbfgsb.c"
      $149 = (($148) + ($147<<3)|0); //@line 1780 "lbfgsb.c"
      $150 = +HEAPF64[$149>>3]; //@line 1780 "lbfgsb.c"
      $151 = $146 - $150; //@line 1780 "lbfgsb.c"
      HEAPF64[240>>3] = $151; //@line 1780 "lbfgsb.c"
     }
     $152 = HEAP32[1328>>2]|0; //@line 1782 "lbfgsb.c"
     $153 = $5; //@line 1782 "lbfgsb.c"
     $154 = (($153) + ($152<<2)|0); //@line 1782 "lbfgsb.c"
     $155 = HEAP32[$154>>2]|0; //@line 1782 "lbfgsb.c"
     $156 = ($155|0)>=(2); //@line 1782 "lbfgsb.c"
     if ($156) {
      $157 = HEAP32[1328>>2]|0; //@line 1783 "lbfgsb.c"
      $158 = $4; //@line 1783 "lbfgsb.c"
      $159 = (($158) + ($157<<3)|0); //@line 1783 "lbfgsb.c"
      $160 = +HEAPF64[$159>>3]; //@line 1783 "lbfgsb.c"
      $161 = HEAP32[1328>>2]|0; //@line 1783 "lbfgsb.c"
      $162 = $2; //@line 1783 "lbfgsb.c"
      $163 = (($162) + ($161<<3)|0); //@line 1783 "lbfgsb.c"
      $164 = +HEAPF64[$163>>3]; //@line 1783 "lbfgsb.c"
      $165 = $160 - $164; //@line 1783 "lbfgsb.c"
      HEAPF64[248>>3] = $165; //@line 1783 "lbfgsb.c"
     }
     $166 = HEAP32[1328>>2]|0; //@line 1787 "lbfgsb.c"
     $167 = $5; //@line 1787 "lbfgsb.c"
     $168 = (($167) + ($166<<2)|0); //@line 1787 "lbfgsb.c"
     $169 = HEAP32[$168>>2]|0; //@line 1787 "lbfgsb.c"
     $170 = ($169|0)<=(2); //@line 1787 "lbfgsb.c"
     $171 = +HEAPF64[240>>3]; //@line 1787 "lbfgsb.c"
     $172 = $171 <= 0.0; //@line 1787 "lbfgsb.c"
     $173 = $170 ? $172 : 0; //@line 1787 "lbfgsb.c"
     $174 = $173&1; //@line 1787 "lbfgsb.c"
     HEAP32[1372>>2] = $174; //@line 1787 "lbfgsb.c"
     $175 = HEAP32[1328>>2]|0; //@line 1788 "lbfgsb.c"
     $176 = $5; //@line 1788 "lbfgsb.c"
     $177 = (($176) + ($175<<2)|0); //@line 1788 "lbfgsb.c"
     $178 = HEAP32[$177>>2]|0; //@line 1788 "lbfgsb.c"
     $179 = ($178|0)>=(2); //@line 1788 "lbfgsb.c"
     $180 = +HEAPF64[248>>3]; //@line 1788 "lbfgsb.c"
     $181 = $180 <= 0.0; //@line 1788 "lbfgsb.c"
     $182 = $179 ? $181 : 0; //@line 1788 "lbfgsb.c"
     $183 = $182&1; //@line 1788 "lbfgsb.c"
     HEAP32[1376>>2] = $183; //@line 1788 "lbfgsb.c"
     $184 = HEAP32[1328>>2]|0; //@line 1790 "lbfgsb.c"
     $185 = $8; //@line 1790 "lbfgsb.c"
     $186 = (($185) + ($184<<2)|0); //@line 1790 "lbfgsb.c"
     HEAP32[$186>>2] = 0; //@line 1790 "lbfgsb.c"
     $187 = HEAP32[1372>>2]|0; //@line 1791 "lbfgsb.c"
     $188 = ($187|0)!=(0); //@line 1791 "lbfgsb.c"
     if ($188) {
      $189 = +HEAPF64[328>>3]; //@line 1792 "lbfgsb.c"
      $190 = $189 <= 0.0; //@line 1792 "lbfgsb.c"
      if (!($190)) {
       break;
      }
      $191 = HEAP32[1328>>2]|0; //@line 1793 "lbfgsb.c"
      $192 = $8; //@line 1793 "lbfgsb.c"
      $193 = (($192) + ($191<<2)|0); //@line 1793 "lbfgsb.c"
      HEAP32[$193>>2] = 1; //@line 1793 "lbfgsb.c"
      break;
     }
     $194 = HEAP32[1376>>2]|0; //@line 1795 "lbfgsb.c"
     $195 = ($194|0)!=(0); //@line 1795 "lbfgsb.c"
     $196 = +HEAPF64[328>>3]; //@line 1796 "lbfgsb.c"
     $197 = $196 >= 0.0; //@line 1796 "lbfgsb.c"
     if ($195) {
      if (!($197)) {
       break;
      }
      $198 = HEAP32[1328>>2]|0; //@line 1797 "lbfgsb.c"
      $199 = $8; //@line 1797 "lbfgsb.c"
      $200 = (($199) + ($198<<2)|0); //@line 1797 "lbfgsb.c"
      HEAP32[$200>>2] = 2; //@line 1797 "lbfgsb.c"
      break;
     } else {
      $201 = +HEAPF64[328>>3]; //@line 1800 "lbfgsb.c"
      $202 = -$201; //@line 1800 "lbfgsb.c"
      $203 = $197 ? $201 : $202; //@line 1800 "lbfgsb.c"
      $204 = $203 <= 0.0; //@line 1800 "lbfgsb.c"
      if (!($204)) {
       break;
      }
      $205 = HEAP32[1328>>2]|0; //@line 1801 "lbfgsb.c"
      $206 = $8; //@line 1801 "lbfgsb.c"
      $207 = (($206) + ($205<<2)|0); //@line 1801 "lbfgsb.c"
      HEAP32[$207>>2] = -3; //@line 1801 "lbfgsb.c"
      break;
     }
    }
   }
  } while(0);
  $208 = $19; //@line 1805 "lbfgsb.c"
  $209 = HEAP32[$208>>2]|0; //@line 1805 "lbfgsb.c"
  HEAP32[1368>>2] = $209; //@line 1805 "lbfgsb.c"
  $210 = HEAP32[1328>>2]|0; //@line 1806 "lbfgsb.c"
  $211 = $8; //@line 1806 "lbfgsb.c"
  $212 = (($211) + ($210<<2)|0); //@line 1806 "lbfgsb.c"
  $213 = HEAP32[$212>>2]|0; //@line 1806 "lbfgsb.c"
  $214 = ($213|0)!=(0); //@line 1806 "lbfgsb.c"
  if ($214) {
   $215 = HEAP32[1328>>2]|0; //@line 1806 "lbfgsb.c"
   $216 = $8; //@line 1806 "lbfgsb.c"
   $217 = (($216) + ($215<<2)|0); //@line 1806 "lbfgsb.c"
   $218 = HEAP32[$217>>2]|0; //@line 1806 "lbfgsb.c"
   $219 = ($218|0)!=(-1); //@line 1806 "lbfgsb.c"
   if ($219) {
    $220 = HEAP32[1328>>2]|0; //@line 1807 "lbfgsb.c"
    $221 = $10; //@line 1807 "lbfgsb.c"
    $222 = (($221) + ($220<<3)|0); //@line 1807 "lbfgsb.c"
    HEAPF64[$222>>3] = 0.0; //@line 1807 "lbfgsb.c"
   } else {
    label = 29;
   }
  } else {
   label = 29;
  }
  do {
   if ((label|0) == 29) {
    label = 0;
    $223 = +HEAPF64[328>>3]; //@line 1809 "lbfgsb.c"
    $224 = HEAP32[1328>>2]|0; //@line 1809 "lbfgsb.c"
    $225 = $10; //@line 1809 "lbfgsb.c"
    $226 = (($225) + ($224<<3)|0); //@line 1809 "lbfgsb.c"
    HEAPF64[$226>>3] = $223; //@line 1809 "lbfgsb.c"
    $227 = +HEAPF64[328>>3]; //@line 1810 "lbfgsb.c"
    $228 = +HEAPF64[328>>3]; //@line 1810 "lbfgsb.c"
    $229 = $227 * $228; //@line 1810 "lbfgsb.c"
    $230 = +HEAPF64[208>>3]; //@line 1810 "lbfgsb.c"
    $231 = $230 - $229; //@line 1810 "lbfgsb.c"
    HEAPF64[208>>3] = $231; //@line 1810 "lbfgsb.c"
    $232 = $18; //@line 1812 "lbfgsb.c"
    $233 = HEAP32[$232>>2]|0; //@line 1812 "lbfgsb.c"
    $i__2 = $233; //@line 1812 "lbfgsb.c"
    HEAP32[1332>>2] = 1; //@line 1813 "lbfgsb.c"
    while(1) {
     $234 = HEAP32[1332>>2]|0; //@line 1813 "lbfgsb.c"
     $235 = $i__2; //@line 1813 "lbfgsb.c"
     $236 = ($234|0)<=($235|0); //@line 1813 "lbfgsb.c"
     $237 = HEAP32[1328>>2]|0; //@line 1814 "lbfgsb.c"
     if (!($236)) {
      break;
     }
     $238 = HEAP32[1368>>2]|0; //@line 1814 "lbfgsb.c"
     $239 = $wy_dim1; //@line 1814 "lbfgsb.c"
     $240 = Math_imul($238, $239)|0; //@line 1814 "lbfgsb.c"
     $241 = (($237) + ($240))|0; //@line 1814 "lbfgsb.c"
     $242 = $13; //@line 1814 "lbfgsb.c"
     $243 = (($242) + ($241<<3)|0); //@line 1814 "lbfgsb.c"
     $244 = +HEAPF64[$243>>3]; //@line 1814 "lbfgsb.c"
     $245 = +HEAPF64[328>>3]; //@line 1814 "lbfgsb.c"
     $246 = $244 * $245; //@line 1814 "lbfgsb.c"
     $247 = HEAP32[1332>>2]|0; //@line 1814 "lbfgsb.c"
     $248 = $20; //@line 1814 "lbfgsb.c"
     $249 = (($248) + ($247<<3)|0); //@line 1814 "lbfgsb.c"
     $250 = +HEAPF64[$249>>3]; //@line 1814 "lbfgsb.c"
     $251 = $250 + $246; //@line 1814 "lbfgsb.c"
     HEAPF64[$249>>3] = $251; //@line 1814 "lbfgsb.c"
     $252 = HEAP32[1328>>2]|0; //@line 1815 "lbfgsb.c"
     $253 = HEAP32[1368>>2]|0; //@line 1815 "lbfgsb.c"
     $254 = $ws_dim1; //@line 1815 "lbfgsb.c"
     $255 = Math_imul($253, $254)|0; //@line 1815 "lbfgsb.c"
     $256 = (($252) + ($255))|0; //@line 1815 "lbfgsb.c"
     $257 = $14; //@line 1815 "lbfgsb.c"
     $258 = (($257) + ($256<<3)|0); //@line 1815 "lbfgsb.c"
     $259 = +HEAPF64[$258>>3]; //@line 1815 "lbfgsb.c"
     $260 = +HEAPF64[328>>3]; //@line 1815 "lbfgsb.c"
     $261 = $259 * $260; //@line 1815 "lbfgsb.c"
     $262 = $18; //@line 1815 "lbfgsb.c"
     $263 = HEAP32[$262>>2]|0; //@line 1815 "lbfgsb.c"
     $264 = HEAP32[1332>>2]|0; //@line 1815 "lbfgsb.c"
     $265 = (($263) + ($264))|0; //@line 1815 "lbfgsb.c"
     $266 = $20; //@line 1815 "lbfgsb.c"
     $267 = (($266) + ($265<<3)|0); //@line 1815 "lbfgsb.c"
     $268 = +HEAPF64[$267>>3]; //@line 1815 "lbfgsb.c"
     $269 = $268 + $261; //@line 1815 "lbfgsb.c"
     HEAPF64[$267>>3] = $269; //@line 1815 "lbfgsb.c"
     $270 = HEAP32[1368>>2]|0; //@line 1816 "lbfgsb.c"
     $271 = $12; //@line 1816 "lbfgsb.c"
     $272 = HEAP32[$271>>2]|0; //@line 1816 "lbfgsb.c"
     $273 = (($270|0) % ($272|0))&-1; //@line 1816 "lbfgsb.c"
     $274 = (($273) + 1)|0; //@line 1816 "lbfgsb.c"
     HEAP32[1368>>2] = $274; //@line 1816 "lbfgsb.c"
     $275 = HEAP32[1332>>2]|0; //@line 1813 "lbfgsb.c"
     $276 = (($275) + 1)|0; //@line 1813 "lbfgsb.c"
     HEAP32[1332>>2] = $276; //@line 1813 "lbfgsb.c"
    }
    $277 = $5; //@line 1819 "lbfgsb.c"
    $278 = (($277) + ($237<<2)|0); //@line 1819 "lbfgsb.c"
    $279 = HEAP32[$278>>2]|0; //@line 1819 "lbfgsb.c"
    $280 = ($279|0)<=(2); //@line 1819 "lbfgsb.c"
    if ($280) {
     $281 = HEAP32[1328>>2]|0; //@line 1819 "lbfgsb.c"
     $282 = $5; //@line 1819 "lbfgsb.c"
     $283 = (($282) + ($281<<2)|0); //@line 1819 "lbfgsb.c"
     $284 = HEAP32[$283>>2]|0; //@line 1819 "lbfgsb.c"
     $285 = ($284|0)!=(0); //@line 1819 "lbfgsb.c"
     $286 = +HEAPF64[328>>3];
     $287 = $286 < 0.0; //@line 1819 "lbfgsb.c"
     $or$cond = $285 & $287; //@line 1819 "lbfgsb.c"
     if ($or$cond) {
      $288 = HEAP32[1360>>2]|0; //@line 1821 "lbfgsb.c"
      $289 = (($288) + 1)|0; //@line 1821 "lbfgsb.c"
      HEAP32[1360>>2] = $289; //@line 1821 "lbfgsb.c"
      $290 = HEAP32[1328>>2]|0; //@line 1822 "lbfgsb.c"
      $291 = HEAP32[1360>>2]|0; //@line 1822 "lbfgsb.c"
      $292 = $7; //@line 1822 "lbfgsb.c"
      $293 = (($292) + ($291<<2)|0); //@line 1822 "lbfgsb.c"
      HEAP32[$293>>2] = $290; //@line 1822 "lbfgsb.c"
      $294 = +HEAPF64[240>>3]; //@line 1823 "lbfgsb.c"
      $295 = +HEAPF64[328>>3]; //@line 1823 "lbfgsb.c"
      $296 = -$295; //@line 1823 "lbfgsb.c"
      $297 = $294 / $296; //@line 1823 "lbfgsb.c"
      $298 = HEAP32[1360>>2]|0; //@line 1823 "lbfgsb.c"
      $299 = $9; //@line 1823 "lbfgsb.c"
      $300 = (($299) + ($298<<3)|0); //@line 1823 "lbfgsb.c"
      HEAPF64[$300>>3] = $297; //@line 1823 "lbfgsb.c"
      $301 = HEAP32[1360>>2]|0; //@line 1824 "lbfgsb.c"
      $302 = ($301|0)==(1); //@line 1824 "lbfgsb.c"
      if (!($302)) {
       $303 = HEAP32[1360>>2]|0; //@line 1824 "lbfgsb.c"
       $304 = $9; //@line 1824 "lbfgsb.c"
       $305 = (($304) + ($303<<3)|0); //@line 1824 "lbfgsb.c"
       $306 = +HEAPF64[$305>>3]; //@line 1824 "lbfgsb.c"
       $307 = +HEAPF64[336>>3]; //@line 1824 "lbfgsb.c"
       $308 = $306 < $307; //@line 1824 "lbfgsb.c"
       if (!($308)) {
        break;
       }
      }
      $309 = HEAP32[1360>>2]|0; //@line 1825 "lbfgsb.c"
      $310 = $9; //@line 1825 "lbfgsb.c"
      $311 = (($310) + ($309<<3)|0); //@line 1825 "lbfgsb.c"
      $312 = +HEAPF64[$311>>3]; //@line 1825 "lbfgsb.c"
      HEAPF64[336>>3] = $312; //@line 1825 "lbfgsb.c"
      $313 = HEAP32[1360>>2]|0; //@line 1826 "lbfgsb.c"
      HEAP32[1364>>2] = $313; //@line 1826 "lbfgsb.c"
      break;
     }
    }
    $314 = HEAP32[1328>>2]|0; //@line 1828 "lbfgsb.c"
    $315 = $5; //@line 1828 "lbfgsb.c"
    $316 = (($315) + ($314<<2)|0); //@line 1828 "lbfgsb.c"
    $317 = HEAP32[$316>>2]|0; //@line 1828 "lbfgsb.c"
    $318 = ($317|0)>=(2); //@line 1828 "lbfgsb.c"
    $319 = +HEAPF64[328>>3];
    $320 = $319 > 0.0; //@line 1828 "lbfgsb.c"
    $or$cond3 = $318 & $320; //@line 1828 "lbfgsb.c"
    if (!($or$cond3)) {
     $346 = HEAP32[1352>>2]|0; //@line 1839 "lbfgsb.c"
     $347 = (($346) + -1)|0; //@line 1839 "lbfgsb.c"
     HEAP32[1352>>2] = $347; //@line 1839 "lbfgsb.c"
     $348 = HEAP32[1328>>2]|0; //@line 1840 "lbfgsb.c"
     $349 = HEAP32[1352>>2]|0; //@line 1840 "lbfgsb.c"
     $350 = $7; //@line 1840 "lbfgsb.c"
     $351 = (($350) + ($349<<2)|0); //@line 1840 "lbfgsb.c"
     HEAP32[$351>>2] = $348; //@line 1840 "lbfgsb.c"
     $352 = +HEAPF64[328>>3]; //@line 1841 "lbfgsb.c"
     $353 = $352 >= 0.0; //@line 1841 "lbfgsb.c"
     $354 = +HEAPF64[328>>3]; //@line 1841 "lbfgsb.c"
     $355 = -$354; //@line 1841 "lbfgsb.c"
     $356 = $353 ? $354 : $355; //@line 1841 "lbfgsb.c"
     $357 = $356 > 0.0; //@line 1841 "lbfgsb.c"
     if (!($357)) {
      break;
     }
     HEAP32[1348>>2] = 0; //@line 1842 "lbfgsb.c"
     break;
    }
    $321 = HEAP32[1360>>2]|0; //@line 1830 "lbfgsb.c"
    $322 = (($321) + 1)|0; //@line 1830 "lbfgsb.c"
    HEAP32[1360>>2] = $322; //@line 1830 "lbfgsb.c"
    $323 = HEAP32[1328>>2]|0; //@line 1831 "lbfgsb.c"
    $324 = HEAP32[1360>>2]|0; //@line 1831 "lbfgsb.c"
    $325 = $7; //@line 1831 "lbfgsb.c"
    $326 = (($325) + ($324<<2)|0); //@line 1831 "lbfgsb.c"
    HEAP32[$326>>2] = $323; //@line 1831 "lbfgsb.c"
    $327 = +HEAPF64[248>>3]; //@line 1832 "lbfgsb.c"
    $328 = +HEAPF64[328>>3]; //@line 1832 "lbfgsb.c"
    $329 = $327 / $328; //@line 1832 "lbfgsb.c"
    $330 = HEAP32[1360>>2]|0; //@line 1832 "lbfgsb.c"
    $331 = $9; //@line 1832 "lbfgsb.c"
    $332 = (($331) + ($330<<3)|0); //@line 1832 "lbfgsb.c"
    HEAPF64[$332>>3] = $329; //@line 1832 "lbfgsb.c"
    $333 = HEAP32[1360>>2]|0; //@line 1833 "lbfgsb.c"
    $334 = ($333|0)==(1); //@line 1833 "lbfgsb.c"
    if (!($334)) {
     $335 = HEAP32[1360>>2]|0; //@line 1833 "lbfgsb.c"
     $336 = $9; //@line 1833 "lbfgsb.c"
     $337 = (($336) + ($335<<3)|0); //@line 1833 "lbfgsb.c"
     $338 = +HEAPF64[$337>>3]; //@line 1833 "lbfgsb.c"
     $339 = +HEAPF64[336>>3]; //@line 1833 "lbfgsb.c"
     $340 = $338 < $339; //@line 1833 "lbfgsb.c"
     if (!($340)) {
      break;
     }
    }
    $341 = HEAP32[1360>>2]|0; //@line 1834 "lbfgsb.c"
    $342 = $9; //@line 1834 "lbfgsb.c"
    $343 = (($342) + ($341<<3)|0); //@line 1834 "lbfgsb.c"
    $344 = +HEAPF64[$343>>3]; //@line 1834 "lbfgsb.c"
    HEAPF64[336>>3] = $344; //@line 1834 "lbfgsb.c"
    $345 = HEAP32[1360>>2]|0; //@line 1835 "lbfgsb.c"
    HEAP32[1364>>2] = $345; //@line 1835 "lbfgsb.c"
   }
  } while(0);
  $358 = HEAP32[1328>>2]|0; //@line 1774 "lbfgsb.c"
  $359 = (($358) + 1)|0; //@line 1774 "lbfgsb.c"
  HEAP32[1328>>2] = $359; //@line 1774 "lbfgsb.c"
 }
 $360 = $17; //@line 1851 "lbfgsb.c"
 $361 = +HEAPF64[$360>>3]; //@line 1851 "lbfgsb.c"
 $362 = $361 != 1.0; //@line 1851 "lbfgsb.c"
 if ($362) {
  $363 = $18; //@line 1853 "lbfgsb.c"
  $364 = $17; //@line 1853 "lbfgsb.c"
  $365 = $18; //@line 1853 "lbfgsb.c"
  $366 = HEAP32[$365>>2]|0; //@line 1853 "lbfgsb.c"
  $367 = (($366) + 1)|0; //@line 1853 "lbfgsb.c"
  $368 = $20; //@line 1853 "lbfgsb.c"
  $369 = (($368) + ($367<<3)|0); //@line 1853 "lbfgsb.c"
  (_dscal_($363,$364,$369,1236)|0); //@line 1853 "lbfgsb.c"
 }
 $370 = $1; //@line 1856 "lbfgsb.c"
 $371 = $2; //@line 1856 "lbfgsb.c"
 $372 = ((($371)) + 8|0); //@line 1856 "lbfgsb.c"
 $373 = $11; //@line 1856 "lbfgsb.c"
 $374 = ((($373)) + 8|0); //@line 1856 "lbfgsb.c"
 (_dcopy_($370,$372,1236,$374,1236)|0); //@line 1856 "lbfgsb.c"
 $375 = HEAP32[1360>>2]|0; //@line 1857 "lbfgsb.c"
 $376 = ($375|0)==(0); //@line 1857 "lbfgsb.c"
 if ($376) {
  $377 = HEAP32[1352>>2]|0; //@line 1857 "lbfgsb.c"
  $378 = $1; //@line 1857 "lbfgsb.c"
  $379 = HEAP32[$378>>2]|0; //@line 1857 "lbfgsb.c"
  $380 = (($379) + 1)|0; //@line 1857 "lbfgsb.c"
  $381 = ($377|0)==($380|0); //@line 1857 "lbfgsb.c"
  if ($381) {
   $382 = $25; //@line 1859 "lbfgsb.c"
   $383 = HEAP32[$382>>2]|0; //@line 1859 "lbfgsb.c"
   $384 = ($383|0)>(100); //@line 1859 "lbfgsb.c"
   if ($384) {
    (_s_wsfe(1420)|0); //@line 1860 "lbfgsb.c"
    $385 = $1; //@line 1861 "lbfgsb.c"
    $386 = HEAP32[$385>>2]|0; //@line 1861 "lbfgsb.c"
    HEAP32[$i__1>>2] = $386; //@line 1861 "lbfgsb.c"
    HEAP32[1328>>2] = 1; //@line 1862 "lbfgsb.c"
    while(1) {
     $387 = HEAP32[1328>>2]|0; //@line 1862 "lbfgsb.c"
     $388 = HEAP32[$i__1>>2]|0; //@line 1862 "lbfgsb.c"
     $389 = ($387|0)<=($388|0); //@line 1862 "lbfgsb.c"
     if (!($389)) {
      break;
     }
     $390 = HEAP32[1328>>2]|0; //@line 1863 "lbfgsb.c"
     $391 = $11; //@line 1863 "lbfgsb.c"
     $392 = (($391) + ($390<<3)|0); //@line 1863 "lbfgsb.c"
     (_do_fio(1236,$392,8)|0); //@line 1863 "lbfgsb.c"
     $393 = HEAP32[1328>>2]|0; //@line 1862 "lbfgsb.c"
     $394 = (($393) + 1)|0; //@line 1862 "lbfgsb.c"
     HEAP32[1328>>2] = $394; //@line 1862 "lbfgsb.c"
    }
    (_e_wsfe()|0); //@line 1865 "lbfgsb.c"
   }
   $0 = 0; //@line 1867 "lbfgsb.c"
   $750 = $0; //@line 2070 "lbfgsb.c"
   STACKTOP = sp;return ($750|0); //@line 2070 "lbfgsb.c"
  }
 }
 $395 = HEAP32[1340>>2]|0; //@line 1870 "lbfgsb.c"
 HEAP32[$i__1>>2] = $395; //@line 1870 "lbfgsb.c"
 HEAP32[1332>>2] = 1; //@line 1871 "lbfgsb.c"
 while(1) {
  $396 = HEAP32[1332>>2]|0; //@line 1871 "lbfgsb.c"
  $397 = HEAP32[$i__1>>2]|0; //@line 1871 "lbfgsb.c"
  $398 = ($396|0)<=($397|0); //@line 1871 "lbfgsb.c"
  if (!($398)) {
   break;
  }
  $399 = HEAP32[1332>>2]|0; //@line 1872 "lbfgsb.c"
  $400 = $21; //@line 1872 "lbfgsb.c"
  $401 = (($400) + ($399<<3)|0); //@line 1872 "lbfgsb.c"
  HEAPF64[$401>>3] = 0.0; //@line 1872 "lbfgsb.c"
  $402 = HEAP32[1332>>2]|0; //@line 1871 "lbfgsb.c"
  $403 = (($402) + 1)|0; //@line 1871 "lbfgsb.c"
  HEAP32[1332>>2] = $403; //@line 1871 "lbfgsb.c"
 }
 $404 = $17; //@line 1876 "lbfgsb.c"
 $405 = +HEAPF64[$404>>3]; //@line 1876 "lbfgsb.c"
 $406 = -$405; //@line 1876 "lbfgsb.c"
 $407 = +HEAPF64[208>>3]; //@line 1876 "lbfgsb.c"
 $408 = $406 * $407; //@line 1876 "lbfgsb.c"
 HEAPF64[216>>3] = $408; //@line 1876 "lbfgsb.c"
 $409 = +HEAPF64[216>>3]; //@line 1877 "lbfgsb.c"
 HEAPF64[344>>3] = $409; //@line 1877 "lbfgsb.c"
 $410 = $18; //@line 1878 "lbfgsb.c"
 $411 = HEAP32[$410>>2]|0; //@line 1878 "lbfgsb.c"
 $412 = ($411|0)>(0); //@line 1878 "lbfgsb.c"
 do {
  if ($412) {
   $413 = $12; //@line 1879 "lbfgsb.c"
   $414 = $sy_offset; //@line 1879 "lbfgsb.c"
   $415 = $15; //@line 1879 "lbfgsb.c"
   $416 = (($415) + ($414<<3)|0); //@line 1879 "lbfgsb.c"
   $417 = $wt_offset; //@line 1879 "lbfgsb.c"
   $418 = $16; //@line 1879 "lbfgsb.c"
   $419 = (($418) + ($417<<3)|0); //@line 1879 "lbfgsb.c"
   $420 = $18; //@line 1879 "lbfgsb.c"
   $421 = $20; //@line 1879 "lbfgsb.c"
   $422 = ((($421)) + 8|0); //@line 1879 "lbfgsb.c"
   $423 = $23; //@line 1879 "lbfgsb.c"
   $424 = ((($423)) + 8|0); //@line 1879 "lbfgsb.c"
   $425 = $27; //@line 1879 "lbfgsb.c"
   (_bmv_($413,$416,$419,$420,$422,$424,$425)|0); //@line 1879 "lbfgsb.c"
   $426 = $27; //@line 1880 "lbfgsb.c"
   $427 = HEAP32[$426>>2]|0; //@line 1880 "lbfgsb.c"
   $428 = ($427|0)!=(0); //@line 1880 "lbfgsb.c"
   if (!($428)) {
    $429 = $23; //@line 1883 "lbfgsb.c"
    $430 = ((($429)) + 8|0); //@line 1883 "lbfgsb.c"
    $431 = $20; //@line 1883 "lbfgsb.c"
    $432 = ((($431)) + 8|0); //@line 1883 "lbfgsb.c"
    $433 = (+_ddot_(1340,$430,1236,$432,1236)); //@line 1883 "lbfgsb.c"
    $434 = +HEAPF64[216>>3]; //@line 1883 "lbfgsb.c"
    $435 = $434 - $433; //@line 1883 "lbfgsb.c"
    HEAPF64[216>>3] = $435; //@line 1883 "lbfgsb.c"
    break;
   }
   $0 = 0; //@line 1881 "lbfgsb.c"
   $750 = $0; //@line 2070 "lbfgsb.c"
   STACKTOP = sp;return ($750|0); //@line 2070 "lbfgsb.c"
  }
 } while(0);
 $436 = +HEAPF64[208>>3]; //@line 1885 "lbfgsb.c"
 $437 = -$436; //@line 1885 "lbfgsb.c"
 $438 = +HEAPF64[216>>3]; //@line 1885 "lbfgsb.c"
 $439 = $437 / $438; //@line 1885 "lbfgsb.c"
 HEAPF64[264>>3] = $439; //@line 1885 "lbfgsb.c"
 HEAPF64[312>>3] = 0.0; //@line 1886 "lbfgsb.c"
 $440 = $24; //@line 1887 "lbfgsb.c"
 HEAP32[$440>>2] = 1; //@line 1887 "lbfgsb.c"
 $441 = $25; //@line 1888 "lbfgsb.c"
 $442 = HEAP32[$441>>2]|0; //@line 1888 "lbfgsb.c"
 $443 = ($442|0)>=(99); //@line 1888 "lbfgsb.c"
 if ($443) {
  (_s_wsle(1440)|0); //@line 1889 "lbfgsb.c"
  (_do_lio(1308,1236,15092,10)|0); //@line 1890 "lbfgsb.c"
  (_do_lio(1660,1236,1360,4)|0); //@line 1891 "lbfgsb.c"
  (_do_lio(1308,1236,15103,14)|0); //@line 1892 "lbfgsb.c"
  (_e_wsle()|0); //@line 1893 "lbfgsb.c"
 }
 $444 = HEAP32[1360>>2]|0; //@line 1896 "lbfgsb.c"
 $445 = ($444|0)==(0); //@line 1896 "lbfgsb.c"
 L91: do {
  if ($445) {
   label = 95;
  } else {
   $446 = HEAP32[1360>>2]|0; //@line 1899 "lbfgsb.c"
   HEAP32[1356>>2] = $446; //@line 1899 "lbfgsb.c"
   HEAP32[1344>>2] = 1; //@line 1900 "lbfgsb.c"
   HEAPF64[232>>3] = 0.0; //@line 1901 "lbfgsb.c"
   while(1) {
    $447 = +HEAPF64[232>>3]; //@line 1906 "lbfgsb.c"
    HEAPF64[256>>3] = $447; //@line 1906 "lbfgsb.c"
    $448 = HEAP32[1344>>2]|0; //@line 1907 "lbfgsb.c"
    $449 = ($448|0)==(1); //@line 1907 "lbfgsb.c"
    if ($449) {
     $450 = +HEAPF64[336>>3]; //@line 1911 "lbfgsb.c"
     HEAPF64[232>>3] = $450; //@line 1911 "lbfgsb.c"
     $451 = HEAP32[1364>>2]|0; //@line 1912 "lbfgsb.c"
     $452 = $7; //@line 1912 "lbfgsb.c"
     $453 = (($452) + ($451<<2)|0); //@line 1912 "lbfgsb.c"
     $454 = HEAP32[$453>>2]|0; //@line 1912 "lbfgsb.c"
     HEAP32[1336>>2] = $454; //@line 1912 "lbfgsb.c"
    } else {
     $455 = HEAP32[1344>>2]|0; //@line 1914 "lbfgsb.c"
     $456 = ($455|0)==(2); //@line 1914 "lbfgsb.c"
     if ($456) {
      $457 = HEAP32[1364>>2]|0; //@line 1917 "lbfgsb.c"
      $458 = HEAP32[1360>>2]|0; //@line 1917 "lbfgsb.c"
      $459 = ($457|0)!=($458|0); //@line 1917 "lbfgsb.c"
      if ($459) {
       $460 = HEAP32[1360>>2]|0; //@line 1918 "lbfgsb.c"
       $461 = $9; //@line 1918 "lbfgsb.c"
       $462 = (($461) + ($460<<3)|0); //@line 1918 "lbfgsb.c"
       $463 = +HEAPF64[$462>>3]; //@line 1918 "lbfgsb.c"
       $464 = HEAP32[1364>>2]|0; //@line 1918 "lbfgsb.c"
       $465 = $9; //@line 1918 "lbfgsb.c"
       $466 = (($465) + ($464<<3)|0); //@line 1918 "lbfgsb.c"
       HEAPF64[$466>>3] = $463; //@line 1918 "lbfgsb.c"
       $467 = HEAP32[1360>>2]|0; //@line 1919 "lbfgsb.c"
       $468 = $7; //@line 1919 "lbfgsb.c"
       $469 = (($468) + ($467<<2)|0); //@line 1919 "lbfgsb.c"
       $470 = HEAP32[$469>>2]|0; //@line 1919 "lbfgsb.c"
       $471 = HEAP32[1364>>2]|0; //@line 1919 "lbfgsb.c"
       $472 = $7; //@line 1919 "lbfgsb.c"
       $473 = (($472) + ($471<<2)|0); //@line 1919 "lbfgsb.c"
       HEAP32[$473>>2] = $470; //@line 1919 "lbfgsb.c"
      }
     }
     $474 = HEAP32[1344>>2]|0; //@line 1924 "lbfgsb.c"
     $475 = (($474) - 2)|0; //@line 1924 "lbfgsb.c"
     HEAP32[$i__1>>2] = $475; //@line 1924 "lbfgsb.c"
     $476 = $9; //@line 1925 "lbfgsb.c"
     $477 = ((($476)) + 8|0); //@line 1925 "lbfgsb.c"
     $478 = $7; //@line 1925 "lbfgsb.c"
     $479 = ((($478)) + 4|0); //@line 1925 "lbfgsb.c"
     (_hpsolb_(1356,$477,$479,$i__1)|0); //@line 1925 "lbfgsb.c"
     $480 = HEAP32[1356>>2]|0; //@line 1926 "lbfgsb.c"
     $481 = $9; //@line 1926 "lbfgsb.c"
     $482 = (($481) + ($480<<3)|0); //@line 1926 "lbfgsb.c"
     $483 = +HEAPF64[$482>>3]; //@line 1926 "lbfgsb.c"
     HEAPF64[232>>3] = $483; //@line 1926 "lbfgsb.c"
     $484 = HEAP32[1356>>2]|0; //@line 1927 "lbfgsb.c"
     $485 = $7; //@line 1927 "lbfgsb.c"
     $486 = (($485) + ($484<<2)|0); //@line 1927 "lbfgsb.c"
     $487 = HEAP32[$486>>2]|0; //@line 1927 "lbfgsb.c"
     HEAP32[1336>>2] = $487; //@line 1927 "lbfgsb.c"
    }
    $488 = +HEAPF64[232>>3]; //@line 1929 "lbfgsb.c"
    $489 = +HEAPF64[256>>3]; //@line 1929 "lbfgsb.c"
    $490 = $488 - $489; //@line 1929 "lbfgsb.c"
    HEAPF64[224>>3] = $490; //@line 1929 "lbfgsb.c"
    $491 = +HEAPF64[224>>3]; //@line 1930 "lbfgsb.c"
    $492 = $491 != 0.0; //@line 1930 "lbfgsb.c"
    if ($492) {
     $493 = $25; //@line 1930 "lbfgsb.c"
     $494 = HEAP32[$493>>2]|0; //@line 1930 "lbfgsb.c"
     $495 = ($494|0)>=(100); //@line 1930 "lbfgsb.c"
     if ($495) {
      (_s_wsfe(1460)|0); //@line 1931 "lbfgsb.c"
      $496 = $24; //@line 1932 "lbfgsb.c"
      (_do_fio(1236,$496,4)|0); //@line 1932 "lbfgsb.c"
      (_do_fio(1236,208,8)|0); //@line 1933 "lbfgsb.c"
      (_do_fio(1236,216,8)|0); //@line 1934 "lbfgsb.c"
      (_e_wsfe()|0); //@line 1935 "lbfgsb.c"
      (_s_wsfe(1480)|0); //@line 1936 "lbfgsb.c"
      (_do_fio(1236,224,8)|0); //@line 1937 "lbfgsb.c"
      (_e_wsfe()|0); //@line 1938 "lbfgsb.c"
      (_s_wsfe(1500)|0); //@line 1939 "lbfgsb.c"
      (_do_fio(1236,264,8)|0); //@line 1940 "lbfgsb.c"
      (_e_wsfe()|0); //@line 1941 "lbfgsb.c"
     }
    }
    $497 = +HEAPF64[264>>3]; //@line 1944 "lbfgsb.c"
    $498 = +HEAPF64[224>>3]; //@line 1944 "lbfgsb.c"
    $499 = $497 < $498; //@line 1944 "lbfgsb.c"
    if ($499) {
     label = 95;
     break L91;
    }
    $500 = +HEAPF64[224>>3]; //@line 1949 "lbfgsb.c"
    $501 = +HEAPF64[312>>3]; //@line 1949 "lbfgsb.c"
    $502 = $501 + $500; //@line 1949 "lbfgsb.c"
    HEAPF64[312>>3] = $502; //@line 1949 "lbfgsb.c"
    $503 = HEAP32[1356>>2]|0; //@line 1950 "lbfgsb.c"
    $504 = (($503) + -1)|0; //@line 1950 "lbfgsb.c"
    HEAP32[1356>>2] = $504; //@line 1950 "lbfgsb.c"
    $505 = HEAP32[1344>>2]|0; //@line 1951 "lbfgsb.c"
    $506 = (($505) + 1)|0; //@line 1951 "lbfgsb.c"
    HEAP32[1344>>2] = $506; //@line 1951 "lbfgsb.c"
    $507 = HEAP32[1336>>2]|0; //@line 1952 "lbfgsb.c"
    $508 = $10; //@line 1952 "lbfgsb.c"
    $509 = (($508) + ($507<<3)|0); //@line 1952 "lbfgsb.c"
    $510 = +HEAPF64[$509>>3]; //@line 1952 "lbfgsb.c"
    HEAPF64[296>>3] = $510; //@line 1952 "lbfgsb.c"
    $511 = HEAP32[1336>>2]|0; //@line 1953 "lbfgsb.c"
    $512 = $10; //@line 1953 "lbfgsb.c"
    $513 = (($512) + ($511<<3)|0); //@line 1953 "lbfgsb.c"
    HEAPF64[$513>>3] = 0.0; //@line 1953 "lbfgsb.c"
    $514 = +HEAPF64[296>>3]; //@line 1954 "lbfgsb.c"
    $515 = $514 > 0.0; //@line 1954 "lbfgsb.c"
    $516 = HEAP32[1336>>2]|0; //@line 1955 "lbfgsb.c"
    if ($515) {
     $517 = $4; //@line 1955 "lbfgsb.c"
     $518 = (($517) + ($516<<3)|0); //@line 1955 "lbfgsb.c"
     $519 = +HEAPF64[$518>>3]; //@line 1955 "lbfgsb.c"
     $520 = HEAP32[1336>>2]|0; //@line 1955 "lbfgsb.c"
     $521 = $2; //@line 1955 "lbfgsb.c"
     $522 = (($521) + ($520<<3)|0); //@line 1955 "lbfgsb.c"
     $523 = +HEAPF64[$522>>3]; //@line 1955 "lbfgsb.c"
     $524 = $519 - $523; //@line 1955 "lbfgsb.c"
     HEAPF64[304>>3] = $524; //@line 1955 "lbfgsb.c"
     $525 = HEAP32[1336>>2]|0; //@line 1956 "lbfgsb.c"
     $526 = $4; //@line 1956 "lbfgsb.c"
     $527 = (($526) + ($525<<3)|0); //@line 1956 "lbfgsb.c"
     $528 = +HEAPF64[$527>>3]; //@line 1956 "lbfgsb.c"
     $529 = HEAP32[1336>>2]|0; //@line 1956 "lbfgsb.c"
     $530 = $11; //@line 1956 "lbfgsb.c"
     $531 = (($530) + ($529<<3)|0); //@line 1956 "lbfgsb.c"
     HEAPF64[$531>>3] = $528; //@line 1956 "lbfgsb.c"
     $532 = HEAP32[1336>>2]|0; //@line 1957 "lbfgsb.c"
     $533 = $8; //@line 1957 "lbfgsb.c"
     $534 = (($533) + ($532<<2)|0); //@line 1957 "lbfgsb.c"
     HEAP32[$534>>2] = 2; //@line 1957 "lbfgsb.c"
    } else {
     $535 = $3; //@line 1959 "lbfgsb.c"
     $536 = (($535) + ($516<<3)|0); //@line 1959 "lbfgsb.c"
     $537 = +HEAPF64[$536>>3]; //@line 1959 "lbfgsb.c"
     $538 = HEAP32[1336>>2]|0; //@line 1959 "lbfgsb.c"
     $539 = $2; //@line 1959 "lbfgsb.c"
     $540 = (($539) + ($538<<3)|0); //@line 1959 "lbfgsb.c"
     $541 = +HEAPF64[$540>>3]; //@line 1959 "lbfgsb.c"
     $542 = $537 - $541; //@line 1959 "lbfgsb.c"
     HEAPF64[304>>3] = $542; //@line 1959 "lbfgsb.c"
     $543 = HEAP32[1336>>2]|0; //@line 1960 "lbfgsb.c"
     $544 = $3; //@line 1960 "lbfgsb.c"
     $545 = (($544) + ($543<<3)|0); //@line 1960 "lbfgsb.c"
     $546 = +HEAPF64[$545>>3]; //@line 1960 "lbfgsb.c"
     $547 = HEAP32[1336>>2]|0; //@line 1960 "lbfgsb.c"
     $548 = $11; //@line 1960 "lbfgsb.c"
     $549 = (($548) + ($547<<3)|0); //@line 1960 "lbfgsb.c"
     HEAPF64[$549>>3] = $546; //@line 1960 "lbfgsb.c"
     $550 = HEAP32[1336>>2]|0; //@line 1961 "lbfgsb.c"
     $551 = $8; //@line 1961 "lbfgsb.c"
     $552 = (($551) + ($550<<2)|0); //@line 1961 "lbfgsb.c"
     HEAP32[$552>>2] = 1; //@line 1961 "lbfgsb.c"
    }
    $553 = $25; //@line 1963 "lbfgsb.c"
    $554 = HEAP32[$553>>2]|0; //@line 1963 "lbfgsb.c"
    $555 = ($554|0)>=(100); //@line 1963 "lbfgsb.c"
    if ($555) {
     (_s_wsle(1520)|0); //@line 1964 "lbfgsb.c"
     (_do_lio(1308,1236,15118,10)|0); //@line 1965 "lbfgsb.c"
     (_do_lio(1660,1236,1336,4)|0); //@line 1966 "lbfgsb.c"
     (_do_lio(1308,1236,15129,11)|0); //@line 1967 "lbfgsb.c"
     (_e_wsle()|0); //@line 1968 "lbfgsb.c"
    }
    $556 = HEAP32[1356>>2]|0; //@line 1970 "lbfgsb.c"
    $557 = ($556|0)==(0); //@line 1970 "lbfgsb.c"
    if ($557) {
     $558 = HEAP32[1360>>2]|0; //@line 1970 "lbfgsb.c"
     $559 = $1; //@line 1970 "lbfgsb.c"
     $560 = HEAP32[$559>>2]|0; //@line 1970 "lbfgsb.c"
     $561 = ($558|0)==($560|0); //@line 1970 "lbfgsb.c"
     if ($561) {
      label = 82;
      break;
     }
    }
    $563 = $24; //@line 1977 "lbfgsb.c"
    $564 = HEAP32[$563>>2]|0; //@line 1977 "lbfgsb.c"
    $565 = (($564) + 1)|0; //@line 1977 "lbfgsb.c"
    HEAP32[$563>>2] = $565; //@line 1977 "lbfgsb.c"
    $566 = +HEAPF64[296>>3]; //@line 1979 "lbfgsb.c"
    HEAPF64[$d__1>>3] = $566; //@line 1979 "lbfgsb.c"
    $567 = +HEAPF64[$d__1>>3]; //@line 1980 "lbfgsb.c"
    $568 = +HEAPF64[$d__1>>3]; //@line 1980 "lbfgsb.c"
    $569 = $567 * $568; //@line 1980 "lbfgsb.c"
    HEAPF64[320>>3] = $569; //@line 1980 "lbfgsb.c"
    $570 = +HEAPF64[208>>3]; //@line 1983 "lbfgsb.c"
    $571 = +HEAPF64[224>>3]; //@line 1983 "lbfgsb.c"
    $572 = +HEAPF64[216>>3]; //@line 1983 "lbfgsb.c"
    $573 = $571 * $572; //@line 1983 "lbfgsb.c"
    $574 = $570 + $573; //@line 1983 "lbfgsb.c"
    $575 = +HEAPF64[320>>3]; //@line 1983 "lbfgsb.c"
    $576 = $574 + $575; //@line 1983 "lbfgsb.c"
    $577 = $17; //@line 1983 "lbfgsb.c"
    $578 = +HEAPF64[$577>>3]; //@line 1983 "lbfgsb.c"
    $579 = +HEAPF64[296>>3]; //@line 1983 "lbfgsb.c"
    $580 = $578 * $579; //@line 1983 "lbfgsb.c"
    $581 = +HEAPF64[304>>3]; //@line 1983 "lbfgsb.c"
    $582 = $580 * $581; //@line 1983 "lbfgsb.c"
    $583 = $576 - $582; //@line 1983 "lbfgsb.c"
    HEAPF64[208>>3] = $583; //@line 1983 "lbfgsb.c"
    $584 = $17; //@line 1984 "lbfgsb.c"
    $585 = +HEAPF64[$584>>3]; //@line 1984 "lbfgsb.c"
    $586 = +HEAPF64[320>>3]; //@line 1984 "lbfgsb.c"
    $587 = $585 * $586; //@line 1984 "lbfgsb.c"
    $588 = +HEAPF64[216>>3]; //@line 1984 "lbfgsb.c"
    $589 = $588 - $587; //@line 1984 "lbfgsb.c"
    HEAPF64[216>>3] = $589; //@line 1984 "lbfgsb.c"
    $590 = $18; //@line 1985 "lbfgsb.c"
    $591 = HEAP32[$590>>2]|0; //@line 1985 "lbfgsb.c"
    $592 = ($591|0)>(0); //@line 1985 "lbfgsb.c"
    if ($592) {
     $593 = $20; //@line 1987 "lbfgsb.c"
     $594 = ((($593)) + 8|0); //@line 1987 "lbfgsb.c"
     $595 = $21; //@line 1987 "lbfgsb.c"
     $596 = ((($595)) + 8|0); //@line 1987 "lbfgsb.c"
     (_daxpy_(1340,224,$594,1236,$596,1236)|0); //@line 1987 "lbfgsb.c"
     $597 = $19; //@line 1990 "lbfgsb.c"
     $598 = HEAP32[$597>>2]|0; //@line 1990 "lbfgsb.c"
     HEAP32[1368>>2] = $598; //@line 1990 "lbfgsb.c"
     $599 = $18; //@line 1991 "lbfgsb.c"
     $600 = HEAP32[$599>>2]|0; //@line 1991 "lbfgsb.c"
     HEAP32[$i__1>>2] = $600; //@line 1991 "lbfgsb.c"
     HEAP32[1332>>2] = 1; //@line 1992 "lbfgsb.c"
     while(1) {
      $601 = HEAP32[1332>>2]|0; //@line 1992 "lbfgsb.c"
      $602 = HEAP32[$i__1>>2]|0; //@line 1992 "lbfgsb.c"
      $603 = ($601|0)<=($602|0); //@line 1992 "lbfgsb.c"
      if (!($603)) {
       break;
      }
      $604 = HEAP32[1336>>2]|0; //@line 1993 "lbfgsb.c"
      $605 = HEAP32[1368>>2]|0; //@line 1993 "lbfgsb.c"
      $606 = $wy_dim1; //@line 1993 "lbfgsb.c"
      $607 = Math_imul($605, $606)|0; //@line 1993 "lbfgsb.c"
      $608 = (($604) + ($607))|0; //@line 1993 "lbfgsb.c"
      $609 = $13; //@line 1993 "lbfgsb.c"
      $610 = (($609) + ($608<<3)|0); //@line 1993 "lbfgsb.c"
      $611 = +HEAPF64[$610>>3]; //@line 1993 "lbfgsb.c"
      $612 = HEAP32[1332>>2]|0; //@line 1993 "lbfgsb.c"
      $613 = $22; //@line 1993 "lbfgsb.c"
      $614 = (($613) + ($612<<3)|0); //@line 1993 "lbfgsb.c"
      HEAPF64[$614>>3] = $611; //@line 1993 "lbfgsb.c"
      $615 = $17; //@line 1994 "lbfgsb.c"
      $616 = +HEAPF64[$615>>3]; //@line 1994 "lbfgsb.c"
      $617 = HEAP32[1336>>2]|0; //@line 1994 "lbfgsb.c"
      $618 = HEAP32[1368>>2]|0; //@line 1994 "lbfgsb.c"
      $619 = $ws_dim1; //@line 1994 "lbfgsb.c"
      $620 = Math_imul($618, $619)|0; //@line 1994 "lbfgsb.c"
      $621 = (($617) + ($620))|0; //@line 1994 "lbfgsb.c"
      $622 = $14; //@line 1994 "lbfgsb.c"
      $623 = (($622) + ($621<<3)|0); //@line 1994 "lbfgsb.c"
      $624 = +HEAPF64[$623>>3]; //@line 1994 "lbfgsb.c"
      $625 = $616 * $624; //@line 1994 "lbfgsb.c"
      $626 = $18; //@line 1994 "lbfgsb.c"
      $627 = HEAP32[$626>>2]|0; //@line 1994 "lbfgsb.c"
      $628 = HEAP32[1332>>2]|0; //@line 1994 "lbfgsb.c"
      $629 = (($627) + ($628))|0; //@line 1994 "lbfgsb.c"
      $630 = $22; //@line 1994 "lbfgsb.c"
      $631 = (($630) + ($629<<3)|0); //@line 1994 "lbfgsb.c"
      HEAPF64[$631>>3] = $625; //@line 1994 "lbfgsb.c"
      $632 = HEAP32[1368>>2]|0; //@line 1995 "lbfgsb.c"
      $633 = $12; //@line 1995 "lbfgsb.c"
      $634 = HEAP32[$633>>2]|0; //@line 1995 "lbfgsb.c"
      $635 = (($632|0) % ($634|0))&-1; //@line 1995 "lbfgsb.c"
      $636 = (($635) + 1)|0; //@line 1995 "lbfgsb.c"
      HEAP32[1368>>2] = $636; //@line 1995 "lbfgsb.c"
      $637 = HEAP32[1332>>2]|0; //@line 1992 "lbfgsb.c"
      $638 = (($637) + 1)|0; //@line 1992 "lbfgsb.c"
      HEAP32[1332>>2] = $638; //@line 1992 "lbfgsb.c"
     }
     $639 = $12; //@line 1999 "lbfgsb.c"
     $640 = $sy_offset; //@line 1999 "lbfgsb.c"
     $641 = $15; //@line 1999 "lbfgsb.c"
     $642 = (($641) + ($640<<3)|0); //@line 1999 "lbfgsb.c"
     $643 = $wt_offset; //@line 1999 "lbfgsb.c"
     $644 = $16; //@line 1999 "lbfgsb.c"
     $645 = (($644) + ($643<<3)|0); //@line 1999 "lbfgsb.c"
     $646 = $18; //@line 1999 "lbfgsb.c"
     $647 = $22; //@line 1999 "lbfgsb.c"
     $648 = ((($647)) + 8|0); //@line 1999 "lbfgsb.c"
     $649 = $23; //@line 1999 "lbfgsb.c"
     $650 = ((($649)) + 8|0); //@line 1999 "lbfgsb.c"
     $651 = $27; //@line 1999 "lbfgsb.c"
     (_bmv_($639,$642,$645,$646,$648,$650,$651)|0); //@line 1999 "lbfgsb.c"
     $652 = $27; //@line 2000 "lbfgsb.c"
     $653 = HEAP32[$652>>2]|0; //@line 2000 "lbfgsb.c"
     $654 = ($653|0)!=(0); //@line 2000 "lbfgsb.c"
     if ($654) {
      label = 88;
      break;
     }
     $655 = $21; //@line 2003 "lbfgsb.c"
     $656 = ((($655)) + 8|0); //@line 2003 "lbfgsb.c"
     $657 = $23; //@line 2003 "lbfgsb.c"
     $658 = ((($657)) + 8|0); //@line 2003 "lbfgsb.c"
     $659 = (+_ddot_(1340,$656,1236,$658,1236)); //@line 2003 "lbfgsb.c"
     HEAPF64[272>>3] = $659; //@line 2003 "lbfgsb.c"
     $660 = $20; //@line 2004 "lbfgsb.c"
     $661 = ((($660)) + 8|0); //@line 2004 "lbfgsb.c"
     $662 = $23; //@line 2004 "lbfgsb.c"
     $663 = ((($662)) + 8|0); //@line 2004 "lbfgsb.c"
     $664 = (+_ddot_(1340,$661,1236,$663,1236)); //@line 2004 "lbfgsb.c"
     HEAPF64[280>>3] = $664; //@line 2004 "lbfgsb.c"
     $665 = $22; //@line 2005 "lbfgsb.c"
     $666 = ((($665)) + 8|0); //@line 2005 "lbfgsb.c"
     $667 = $23; //@line 2005 "lbfgsb.c"
     $668 = ((($667)) + 8|0); //@line 2005 "lbfgsb.c"
     $669 = (+_ddot_(1340,$666,1236,$668,1236)); //@line 2005 "lbfgsb.c"
     HEAPF64[288>>3] = $669; //@line 2005 "lbfgsb.c"
     $670 = +HEAPF64[296>>3]; //@line 2007 "lbfgsb.c"
     $671 = -$670; //@line 2007 "lbfgsb.c"
     HEAPF64[$d__1>>3] = $671; //@line 2007 "lbfgsb.c"
     $672 = $22; //@line 2008 "lbfgsb.c"
     $673 = ((($672)) + 8|0); //@line 2008 "lbfgsb.c"
     $674 = $20; //@line 2008 "lbfgsb.c"
     $675 = ((($674)) + 8|0); //@line 2008 "lbfgsb.c"
     (_daxpy_(1340,$d__1,$673,1236,$675,1236)|0); //@line 2008 "lbfgsb.c"
     $676 = +HEAPF64[296>>3]; //@line 2010 "lbfgsb.c"
     $677 = +HEAPF64[272>>3]; //@line 2010 "lbfgsb.c"
     $678 = $676 * $677; //@line 2010 "lbfgsb.c"
     $679 = +HEAPF64[208>>3]; //@line 2010 "lbfgsb.c"
     $680 = $679 + $678; //@line 2010 "lbfgsb.c"
     HEAPF64[208>>3] = $680; //@line 2010 "lbfgsb.c"
     $681 = +HEAPF64[216>>3]; //@line 2011 "lbfgsb.c"
     $682 = +HEAPF64[296>>3]; //@line 2011 "lbfgsb.c"
     $683 = $682 * 2.0; //@line 2011 "lbfgsb.c"
     $684 = +HEAPF64[280>>3]; //@line 2011 "lbfgsb.c"
     $685 = $683 * $684; //@line 2011 "lbfgsb.c"
     $686 = $681 + $685; //@line 2011 "lbfgsb.c"
     $687 = +HEAPF64[320>>3]; //@line 2011 "lbfgsb.c"
     $688 = +HEAPF64[288>>3]; //@line 2011 "lbfgsb.c"
     $689 = $687 * $688; //@line 2011 "lbfgsb.c"
     $690 = $686 - $689; //@line 2011 "lbfgsb.c"
     HEAPF64[216>>3] = $690; //@line 2011 "lbfgsb.c"
    }
    $691 = $28; //@line 2014 "lbfgsb.c"
    $692 = +HEAPF64[$691>>3]; //@line 2014 "lbfgsb.c"
    $693 = +HEAPF64[344>>3]; //@line 2014 "lbfgsb.c"
    $694 = $692 * $693; //@line 2014 "lbfgsb.c"
    HEAPF64[$d__1>>3] = $694; //@line 2014 "lbfgsb.c"
    $695 = +HEAPF64[$d__1>>3]; //@line 2015 "lbfgsb.c"
    $696 = +HEAPF64[216>>3]; //@line 2015 "lbfgsb.c"
    $697 = $695 >= $696; //@line 2015 "lbfgsb.c"
    $698 = +HEAPF64[$d__1>>3]; //@line 2015 "lbfgsb.c"
    $699 = +HEAPF64[216>>3]; //@line 2015 "lbfgsb.c"
    $700 = $697 ? $698 : $699; //@line 2015 "lbfgsb.c"
    HEAPF64[216>>3] = $700; //@line 2015 "lbfgsb.c"
    $701 = HEAP32[1356>>2]|0; //@line 2016 "lbfgsb.c"
    $702 = ($701|0)>(0); //@line 2016 "lbfgsb.c"
    if (!($702)) {
     label = 92;
     break;
    }
    $703 = +HEAPF64[208>>3]; //@line 2017 "lbfgsb.c"
    $704 = -$703; //@line 2017 "lbfgsb.c"
    $705 = +HEAPF64[216>>3]; //@line 2017 "lbfgsb.c"
    $706 = $704 / $705; //@line 2017 "lbfgsb.c"
    HEAPF64[264>>3] = $706; //@line 2017 "lbfgsb.c"
   }
   if ((label|0) == 82) {
    $562 = +HEAPF64[224>>3]; //@line 1973 "lbfgsb.c"
    HEAPF64[264>>3] = $562; //@line 1973 "lbfgsb.c"
    break;
   }
   else if ((label|0) == 88) {
    $0 = 0; //@line 2001 "lbfgsb.c"
    $750 = $0; //@line 2070 "lbfgsb.c"
    STACKTOP = sp;return ($750|0); //@line 2070 "lbfgsb.c"
   }
   else if ((label|0) == 92) {
    $707 = HEAP32[1348>>2]|0; //@line 2020 "lbfgsb.c"
    $708 = ($707|0)!=(0); //@line 2020 "lbfgsb.c"
    if ($708) {
     HEAPF64[208>>3] = 0.0; //@line 2021 "lbfgsb.c"
     HEAPF64[216>>3] = 0.0; //@line 2022 "lbfgsb.c"
     HEAPF64[264>>3] = 0.0; //@line 2023 "lbfgsb.c"
     label = 95;
     break;
    } else {
     $709 = +HEAPF64[208>>3]; //@line 2025 "lbfgsb.c"
     $710 = -$709; //@line 2025 "lbfgsb.c"
     $711 = +HEAPF64[216>>3]; //@line 2025 "lbfgsb.c"
     $712 = $710 / $711; //@line 2025 "lbfgsb.c"
     HEAPF64[264>>3] = $712; //@line 2025 "lbfgsb.c"
     label = 95;
     break;
    }
   }
  }
 } while(0);
 if ((label|0) == 95) {
  $713 = $25; //@line 2029 "lbfgsb.c"
  $714 = HEAP32[$713>>2]|0; //@line 2029 "lbfgsb.c"
  $715 = ($714|0)>=(99); //@line 2029 "lbfgsb.c"
  if ($715) {
   (_s_wsle(1540)|0); //@line 2030 "lbfgsb.c"
   (_e_wsle()|0); //@line 2031 "lbfgsb.c"
   (_s_wsle(1560)|0); //@line 2032 "lbfgsb.c"
   (_do_lio(1308,1236,15141,25)|0); //@line 2033 "lbfgsb.c"
   (_e_wsle()|0); //@line 2034 "lbfgsb.c"
   (_s_wsfe(1580)|0); //@line 2035 "lbfgsb.c"
   $716 = $24; //@line 2036 "lbfgsb.c"
   (_do_fio(1236,$716,4)|0); //@line 2036 "lbfgsb.c"
   (_do_fio(1236,208,8)|0); //@line 2037 "lbfgsb.c"
   (_do_fio(1236,216,8)|0); //@line 2038 "lbfgsb.c"
   (_e_wsfe()|0); //@line 2039 "lbfgsb.c"
   (_s_wsfe(1600)|0); //@line 2040 "lbfgsb.c"
   (_do_fio(1236,264,8)|0); //@line 2041 "lbfgsb.c"
   (_e_wsfe()|0); //@line 2042 "lbfgsb.c"
  }
  $717 = +HEAPF64[264>>3]; //@line 2044 "lbfgsb.c"
  $718 = $717 <= 0.0; //@line 2044 "lbfgsb.c"
  if ($718) {
   HEAPF64[264>>3] = 0.0; //@line 2045 "lbfgsb.c"
  }
  $719 = +HEAPF64[264>>3]; //@line 2047 "lbfgsb.c"
  $720 = +HEAPF64[312>>3]; //@line 2047 "lbfgsb.c"
  $721 = $720 + $719; //@line 2047 "lbfgsb.c"
  HEAPF64[312>>3] = $721; //@line 2047 "lbfgsb.c"
  $722 = $1; //@line 2050 "lbfgsb.c"
  $723 = $10; //@line 2050 "lbfgsb.c"
  $724 = ((($723)) + 8|0); //@line 2050 "lbfgsb.c"
  $725 = $11; //@line 2050 "lbfgsb.c"
  $726 = ((($725)) + 8|0); //@line 2050 "lbfgsb.c"
  (_daxpy_($722,312,$724,1236,$726,1236)|0); //@line 2050 "lbfgsb.c"
 }
 $727 = $18; //@line 2054 "lbfgsb.c"
 $728 = HEAP32[$727>>2]|0; //@line 2054 "lbfgsb.c"
 $729 = ($728|0)>(0); //@line 2054 "lbfgsb.c"
 if ($729) {
  $730 = $20; //@line 2055 "lbfgsb.c"
  $731 = ((($730)) + 8|0); //@line 2055 "lbfgsb.c"
  $732 = $21; //@line 2055 "lbfgsb.c"
  $733 = ((($732)) + 8|0); //@line 2055 "lbfgsb.c"
  (_daxpy_(1340,264,$731,1236,$733,1236)|0); //@line 2055 "lbfgsb.c"
 }
 $734 = $25; //@line 2057 "lbfgsb.c"
 $735 = HEAP32[$734>>2]|0; //@line 2057 "lbfgsb.c"
 $736 = ($735|0)>(100); //@line 2057 "lbfgsb.c"
 if ($736) {
  (_s_wsfe(1620)|0); //@line 2058 "lbfgsb.c"
  $737 = $1; //@line 2059 "lbfgsb.c"
  $738 = HEAP32[$737>>2]|0; //@line 2059 "lbfgsb.c"
  HEAP32[$i__1>>2] = $738; //@line 2059 "lbfgsb.c"
  HEAP32[1328>>2] = 1; //@line 2060 "lbfgsb.c"
  while(1) {
   $739 = HEAP32[1328>>2]|0; //@line 2060 "lbfgsb.c"
   $740 = HEAP32[$i__1>>2]|0; //@line 2060 "lbfgsb.c"
   $741 = ($739|0)<=($740|0); //@line 2060 "lbfgsb.c"
   if (!($741)) {
    break;
   }
   $742 = HEAP32[1328>>2]|0; //@line 2061 "lbfgsb.c"
   $743 = $11; //@line 2061 "lbfgsb.c"
   $744 = (($743) + ($742<<3)|0); //@line 2061 "lbfgsb.c"
   (_do_fio(1236,$744,8)|0); //@line 2061 "lbfgsb.c"
   $745 = HEAP32[1328>>2]|0; //@line 2060 "lbfgsb.c"
   $746 = (($745) + 1)|0; //@line 2060 "lbfgsb.c"
   HEAP32[1328>>2] = $746; //@line 2060 "lbfgsb.c"
  }
  (_e_wsfe()|0); //@line 2063 "lbfgsb.c"
 }
 $747 = $25; //@line 2065 "lbfgsb.c"
 $748 = HEAP32[$747>>2]|0; //@line 2065 "lbfgsb.c"
 $749 = ($748|0)>=(99); //@line 2065 "lbfgsb.c"
 if ($749) {
  (_s_wsfe(1640)|0); //@line 2066 "lbfgsb.c"
  (_e_wsfe()|0); //@line 2067 "lbfgsb.c"
 }
 $0 = 0; //@line 2069 "lbfgsb.c"
 $750 = $0; //@line 2070 "lbfgsb.c"
 STACKTOP = sp;return ($750|0); //@line 2070 "lbfgsb.c"
}
function _freev_($n,$nfree,$index,$nenter,$ileave,$indx2,$iwhere,$wrk,$updatd,$cnstnd,$iprint,$iter) {
 $n = $n|0;
 $nfree = $nfree|0;
 $index = $index|0;
 $nenter = $nenter|0;
 $ileave = $ileave|0;
 $indx2 = $indx2|0;
 $iwhere = $iwhere|0;
 $wrk = $wrk|0;
 $updatd = $updatd|0;
 $cnstnd = $cnstnd|0;
 $iprint = $iprint|0;
 $iter = $iter|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp;
 $0 = $n;
 $1 = $nfree;
 $2 = $index;
 $3 = $nenter;
 $4 = $ileave;
 $5 = $indx2;
 $6 = $iwhere;
 $7 = $wrk;
 $8 = $updatd;
 $9 = $cnstnd;
 $10 = $iprint;
 $11 = $iter;
 $12 = $6; //@line 2776 "lbfgsb.c"
 $13 = ((($12)) + -4|0); //@line 2776 "lbfgsb.c"
 $6 = $13; //@line 2776 "lbfgsb.c"
 $14 = $5; //@line 2777 "lbfgsb.c"
 $15 = ((($14)) + -4|0); //@line 2777 "lbfgsb.c"
 $5 = $15; //@line 2777 "lbfgsb.c"
 $16 = $2; //@line 2778 "lbfgsb.c"
 $17 = ((($16)) + -4|0); //@line 2778 "lbfgsb.c"
 $2 = $17; //@line 2778 "lbfgsb.c"
 $18 = $3; //@line 2781 "lbfgsb.c"
 HEAP32[$18>>2] = 0; //@line 2781 "lbfgsb.c"
 $19 = $0; //@line 2782 "lbfgsb.c"
 $20 = HEAP32[$19>>2]|0; //@line 2782 "lbfgsb.c"
 $21 = (($20) + 1)|0; //@line 2782 "lbfgsb.c"
 $22 = $4; //@line 2782 "lbfgsb.c"
 HEAP32[$22>>2] = $21; //@line 2782 "lbfgsb.c"
 $23 = $11; //@line 2783 "lbfgsb.c"
 $24 = HEAP32[$23>>2]|0; //@line 2783 "lbfgsb.c"
 $25 = ($24|0)>(0); //@line 2783 "lbfgsb.c"
 if ($25) {
  $26 = $9; //@line 2783 "lbfgsb.c"
  $27 = HEAP32[$26>>2]|0; //@line 2783 "lbfgsb.c"
  $28 = ($27|0)!=(0); //@line 2783 "lbfgsb.c"
  if ($28) {
   $29 = $1; //@line 2785 "lbfgsb.c"
   $30 = HEAP32[$29>>2]|0; //@line 2785 "lbfgsb.c"
   HEAP32[$i__1>>2] = $30; //@line 2785 "lbfgsb.c"
   HEAP32[1772>>2] = 1; //@line 2786 "lbfgsb.c"
   while(1) {
    $31 = HEAP32[1772>>2]|0; //@line 2786 "lbfgsb.c"
    $32 = HEAP32[$i__1>>2]|0; //@line 2786 "lbfgsb.c"
    $33 = ($31|0)<=($32|0); //@line 2786 "lbfgsb.c"
    if (!($33)) {
     break;
    }
    $34 = HEAP32[1772>>2]|0; //@line 2787 "lbfgsb.c"
    $35 = $2; //@line 2787 "lbfgsb.c"
    $36 = (($35) + ($34<<2)|0); //@line 2787 "lbfgsb.c"
    $37 = HEAP32[$36>>2]|0; //@line 2787 "lbfgsb.c"
    HEAP32[1776>>2] = $37; //@line 2787 "lbfgsb.c"
    $38 = HEAP32[1776>>2]|0; //@line 2790 "lbfgsb.c"
    $39 = $6; //@line 2790 "lbfgsb.c"
    $40 = (($39) + ($38<<2)|0); //@line 2790 "lbfgsb.c"
    $41 = HEAP32[$40>>2]|0; //@line 2790 "lbfgsb.c"
    $42 = ($41|0)>(0); //@line 2790 "lbfgsb.c"
    if ($42) {
     $43 = $4; //@line 2791 "lbfgsb.c"
     $44 = HEAP32[$43>>2]|0; //@line 2791 "lbfgsb.c"
     $45 = (($44) + -1)|0; //@line 2791 "lbfgsb.c"
     HEAP32[$43>>2] = $45; //@line 2791 "lbfgsb.c"
     $46 = HEAP32[1776>>2]|0; //@line 2792 "lbfgsb.c"
     $47 = $4; //@line 2792 "lbfgsb.c"
     $48 = HEAP32[$47>>2]|0; //@line 2792 "lbfgsb.c"
     $49 = $5; //@line 2792 "lbfgsb.c"
     $50 = (($49) + ($48<<2)|0); //@line 2792 "lbfgsb.c"
     HEAP32[$50>>2] = $46; //@line 2792 "lbfgsb.c"
     $51 = $10; //@line 2793 "lbfgsb.c"
     $52 = HEAP32[$51>>2]|0; //@line 2793 "lbfgsb.c"
     $53 = ($52|0)>=(100); //@line 2793 "lbfgsb.c"
     if ($53) {
      (_s_wsle(1784)|0); //@line 2794 "lbfgsb.c"
      (_do_lio(1308,1236,15266,9)|0); //@line 2795 "lbfgsb.c"
      (_do_lio(1660,1236,1776,4)|0); //@line 2796 "lbfgsb.c"
      (_do_lio(1308,1236,15276,33)|0); //@line 2797 "lbfgsb.c"
      (_e_wsle()|0); //@line 2799 "lbfgsb.c"
     }
    }
    $54 = HEAP32[1772>>2]|0; //@line 2786 "lbfgsb.c"
    $55 = (($54) + 1)|0; //@line 2786 "lbfgsb.c"
    HEAP32[1772>>2] = $55; //@line 2786 "lbfgsb.c"
   }
   $56 = $0; //@line 2804 "lbfgsb.c"
   $57 = HEAP32[$56>>2]|0; //@line 2804 "lbfgsb.c"
   HEAP32[$i__1>>2] = $57; //@line 2804 "lbfgsb.c"
   $58 = $1; //@line 2805 "lbfgsb.c"
   $59 = HEAP32[$58>>2]|0; //@line 2805 "lbfgsb.c"
   $60 = (($59) + 1)|0; //@line 2805 "lbfgsb.c"
   HEAP32[1772>>2] = $60; //@line 2805 "lbfgsb.c"
   while(1) {
    $61 = HEAP32[1772>>2]|0; //@line 2805 "lbfgsb.c"
    $62 = HEAP32[$i__1>>2]|0; //@line 2805 "lbfgsb.c"
    $63 = ($61|0)<=($62|0); //@line 2805 "lbfgsb.c"
    if (!($63)) {
     break;
    }
    $64 = HEAP32[1772>>2]|0; //@line 2806 "lbfgsb.c"
    $65 = $2; //@line 2806 "lbfgsb.c"
    $66 = (($65) + ($64<<2)|0); //@line 2806 "lbfgsb.c"
    $67 = HEAP32[$66>>2]|0; //@line 2806 "lbfgsb.c"
    HEAP32[1776>>2] = $67; //@line 2806 "lbfgsb.c"
    $68 = HEAP32[1776>>2]|0; //@line 2807 "lbfgsb.c"
    $69 = $6; //@line 2807 "lbfgsb.c"
    $70 = (($69) + ($68<<2)|0); //@line 2807 "lbfgsb.c"
    $71 = HEAP32[$70>>2]|0; //@line 2807 "lbfgsb.c"
    $72 = ($71|0)<=(0); //@line 2807 "lbfgsb.c"
    if ($72) {
     $73 = $3; //@line 2808 "lbfgsb.c"
     $74 = HEAP32[$73>>2]|0; //@line 2808 "lbfgsb.c"
     $75 = (($74) + 1)|0; //@line 2808 "lbfgsb.c"
     HEAP32[$73>>2] = $75; //@line 2808 "lbfgsb.c"
     $76 = HEAP32[1776>>2]|0; //@line 2809 "lbfgsb.c"
     $77 = $3; //@line 2809 "lbfgsb.c"
     $78 = HEAP32[$77>>2]|0; //@line 2809 "lbfgsb.c"
     $79 = $5; //@line 2809 "lbfgsb.c"
     $80 = (($79) + ($78<<2)|0); //@line 2809 "lbfgsb.c"
     HEAP32[$80>>2] = $76; //@line 2809 "lbfgsb.c"
     $81 = $10; //@line 2810 "lbfgsb.c"
     $82 = HEAP32[$81>>2]|0; //@line 2810 "lbfgsb.c"
     $83 = ($82|0)>=(100); //@line 2810 "lbfgsb.c"
     if ($83) {
      (_s_wsle(1804)|0); //@line 2811 "lbfgsb.c"
      (_do_lio(1308,1236,15266,9)|0); //@line 2812 "lbfgsb.c"
      (_do_lio(1660,1236,1776,4)|0); //@line 2813 "lbfgsb.c"
      (_do_lio(1308,1236,15310,33)|0); //@line 2814 "lbfgsb.c"
      (_e_wsle()|0); //@line 2816 "lbfgsb.c"
     }
    }
    $84 = HEAP32[1772>>2]|0; //@line 2805 "lbfgsb.c"
    $85 = (($84) + 1)|0; //@line 2805 "lbfgsb.c"
    HEAP32[1772>>2] = $85; //@line 2805 "lbfgsb.c"
   }
   $86 = $10; //@line 2821 "lbfgsb.c"
   $87 = HEAP32[$86>>2]|0; //@line 2821 "lbfgsb.c"
   $88 = ($87|0)>=(99); //@line 2821 "lbfgsb.c"
   if ($88) {
    (_s_wsle(1824)|0); //@line 2822 "lbfgsb.c"
    $89 = $0; //@line 2823 "lbfgsb.c"
    $90 = HEAP32[$89>>2]|0; //@line 2823 "lbfgsb.c"
    $91 = (($90) + 1)|0; //@line 2823 "lbfgsb.c"
    $92 = $4; //@line 2823 "lbfgsb.c"
    $93 = HEAP32[$92>>2]|0; //@line 2823 "lbfgsb.c"
    $94 = (($91) - ($93))|0; //@line 2823 "lbfgsb.c"
    HEAP32[$i__1>>2] = $94; //@line 2823 "lbfgsb.c"
    (_do_lio(1660,1236,$i__1,4)|0); //@line 2824 "lbfgsb.c"
    (_do_lio(1308,1236,15344,18)|0); //@line 2825 "lbfgsb.c"
    $95 = $3; //@line 2826 "lbfgsb.c"
    (_do_lio(1660,1236,$95,4)|0); //@line 2826 "lbfgsb.c"
    (_do_lio(1308,1236,15363,16)|0); //@line 2827 "lbfgsb.c"
    (_e_wsle()|0); //@line 2828 "lbfgsb.c"
   }
  }
 }
 $96 = $4; //@line 2831 "lbfgsb.c"
 $97 = HEAP32[$96>>2]|0; //@line 2831 "lbfgsb.c"
 $98 = $0; //@line 2831 "lbfgsb.c"
 $99 = HEAP32[$98>>2]|0; //@line 2831 "lbfgsb.c"
 $100 = (($99) + 1)|0; //@line 2831 "lbfgsb.c"
 $101 = ($97|0)<($100|0); //@line 2831 "lbfgsb.c"
 if ($101) {
  $109 = 1;
 } else {
  $102 = $3; //@line 2831 "lbfgsb.c"
  $103 = HEAP32[$102>>2]|0; //@line 2831 "lbfgsb.c"
  $104 = ($103|0)>(0); //@line 2831 "lbfgsb.c"
  if ($104) {
   $109 = 1;
  } else {
   $105 = $8; //@line 2831 "lbfgsb.c"
   $106 = HEAP32[$105>>2]|0; //@line 2831 "lbfgsb.c"
   $107 = ($106|0)!=(0); //@line 2831 "lbfgsb.c"
   $109 = $107;
  }
 }
 $108 = $109&1; //@line 2831 "lbfgsb.c"
 $110 = $7; //@line 2831 "lbfgsb.c"
 HEAP32[$110>>2] = $108; //@line 2831 "lbfgsb.c"
 $111 = $1; //@line 2833 "lbfgsb.c"
 HEAP32[$111>>2] = 0; //@line 2833 "lbfgsb.c"
 $112 = $0; //@line 2834 "lbfgsb.c"
 $113 = HEAP32[$112>>2]|0; //@line 2834 "lbfgsb.c"
 $114 = (($113) + 1)|0; //@line 2834 "lbfgsb.c"
 HEAP32[1780>>2] = $114; //@line 2834 "lbfgsb.c"
 $115 = $0; //@line 2835 "lbfgsb.c"
 $116 = HEAP32[$115>>2]|0; //@line 2835 "lbfgsb.c"
 HEAP32[$i__1>>2] = $116; //@line 2835 "lbfgsb.c"
 HEAP32[1772>>2] = 1; //@line 2836 "lbfgsb.c"
 while(1) {
  $117 = HEAP32[1772>>2]|0; //@line 2836 "lbfgsb.c"
  $118 = HEAP32[$i__1>>2]|0; //@line 2836 "lbfgsb.c"
  $119 = ($117|0)<=($118|0); //@line 2836 "lbfgsb.c"
  if (!($119)) {
   break;
  }
  $120 = HEAP32[1772>>2]|0; //@line 2837 "lbfgsb.c"
  $121 = $6; //@line 2837 "lbfgsb.c"
  $122 = (($121) + ($120<<2)|0); //@line 2837 "lbfgsb.c"
  $123 = HEAP32[$122>>2]|0; //@line 2837 "lbfgsb.c"
  $124 = ($123|0)<=(0); //@line 2837 "lbfgsb.c"
  if ($124) {
   $125 = $1; //@line 2838 "lbfgsb.c"
   $126 = HEAP32[$125>>2]|0; //@line 2838 "lbfgsb.c"
   $127 = (($126) + 1)|0; //@line 2838 "lbfgsb.c"
   HEAP32[$125>>2] = $127; //@line 2838 "lbfgsb.c"
   $128 = HEAP32[1772>>2]|0; //@line 2839 "lbfgsb.c"
   $129 = $1; //@line 2839 "lbfgsb.c"
   $130 = HEAP32[$129>>2]|0; //@line 2839 "lbfgsb.c"
   $131 = $2; //@line 2839 "lbfgsb.c"
   $132 = (($131) + ($130<<2)|0); //@line 2839 "lbfgsb.c"
   HEAP32[$132>>2] = $128; //@line 2839 "lbfgsb.c"
  } else {
   $133 = HEAP32[1780>>2]|0; //@line 2841 "lbfgsb.c"
   $134 = (($133) + -1)|0; //@line 2841 "lbfgsb.c"
   HEAP32[1780>>2] = $134; //@line 2841 "lbfgsb.c"
   $135 = HEAP32[1772>>2]|0; //@line 2842 "lbfgsb.c"
   $136 = HEAP32[1780>>2]|0; //@line 2842 "lbfgsb.c"
   $137 = $2; //@line 2842 "lbfgsb.c"
   $138 = (($137) + ($136<<2)|0); //@line 2842 "lbfgsb.c"
   HEAP32[$138>>2] = $135; //@line 2842 "lbfgsb.c"
  }
  $139 = HEAP32[1772>>2]|0; //@line 2836 "lbfgsb.c"
  $140 = (($139) + 1)|0; //@line 2836 "lbfgsb.c"
  HEAP32[1772>>2] = $140; //@line 2836 "lbfgsb.c"
 }
 $141 = $10; //@line 2846 "lbfgsb.c"
 $142 = HEAP32[$141>>2]|0; //@line 2846 "lbfgsb.c"
 $143 = ($142|0)>=(99); //@line 2846 "lbfgsb.c"
 if (!($143)) {
  STACKTOP = sp;return 0; //@line 2854 "lbfgsb.c"
 }
 (_s_wsle(1844)|0); //@line 2847 "lbfgsb.c"
 $144 = $1; //@line 2848 "lbfgsb.c"
 (_do_lio(1660,1236,$144,4)|0); //@line 2848 "lbfgsb.c"
 (_do_lio(1308,1236,15380,27)|0); //@line 2849 "lbfgsb.c"
 $145 = $11; //@line 2850 "lbfgsb.c"
 $146 = HEAP32[$145>>2]|0; //@line 2850 "lbfgsb.c"
 $147 = (($146) + 1)|0; //@line 2850 "lbfgsb.c"
 HEAP32[$i__1>>2] = $147; //@line 2850 "lbfgsb.c"
 (_do_lio(1660,1236,$i__1,4)|0); //@line 2851 "lbfgsb.c"
 (_e_wsle()|0); //@line 2852 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 2854 "lbfgsb.c"
}
function _formk_($n,$nsub,$ind,$nenter,$ileave,$indx2,$iupdat,$updatd,$wn,$wn1,$m,$ws,$wy,$sy,$theta,$col,$head,$info) {
 $n = $n|0;
 $nsub = $nsub|0;
 $ind = $ind|0;
 $nenter = $nenter|0;
 $ileave = $ileave|0;
 $indx2 = $indx2|0;
 $iupdat = $iupdat|0;
 $updatd = $updatd|0;
 $wn = $wn|0;
 $wn1 = $wn1|0;
 $m = $m|0;
 $ws = $ws|0;
 $wy = $wy|0;
 $sy = $sy|0;
 $theta = $theta|0;
 $col = $col|0;
 $head = $head|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0.0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0.0, $211 = 0.0, $212 = 0.0, $213 = 0.0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0.0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0.0, $241 = 0.0;
 var $242 = 0.0, $243 = 0.0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0.0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0.0, $26 = 0;
 var $260 = 0.0, $261 = 0.0, $262 = 0.0, $263 = 0, $264 = 0, $265 = 0.0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0.0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0.0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0.0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0.0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0.0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0.0, $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0.0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0.0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0.0, $469 = 0.0, $47 = 0, $470 = 0.0, $471 = 0.0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0.0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0.0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0.0, $504 = 0.0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0.0, $521 = 0.0, $522 = 0.0, $523 = 0.0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0.0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0.0, $588 = 0.0, $589 = 0.0, $59 = 0, $590 = 0.0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0.0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0.0, $62 = 0;
 var $620 = 0.0, $621 = 0.0, $622 = 0.0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0.0;
 var $639 = 0.0, $64 = 0, $640 = 0.0, $641 = 0.0, $642 = 0.0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0.0, $651 = 0.0, $652 = 0.0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0.0, $71 = 0;
 var $710 = 0, $711 = 0.0, $712 = 0.0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0.0, $728 = 0;
 var $729 = 0.0, $73 = 0, $730 = 0.0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0.0, $753 = 0.0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0.0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0.0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0.0, $802 = 0.0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0.0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0.0, $871 = 0.0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, $sy_dim1 = 0, $sy_offset = 0, $wn1_dim1 = 0;
 var $wn1_offset = 0, $wn_dim1 = 0, $wn_offset = 0, $ws_dim1 = 0, $ws_offset = 0, $wy_dim1 = 0, $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__2 = sp + 4|0;
 $1 = $n;
 $2 = $nsub;
 $3 = $ind;
 $4 = $nenter;
 $5 = $ileave;
 $6 = $indx2;
 $7 = $iupdat;
 $8 = $updatd;
 $9 = $wn;
 $10 = $wn1;
 $11 = $m;
 $12 = $ws;
 $13 = $wy;
 $14 = $sy;
 $15 = $theta;
 $16 = $col;
 $17 = $head;
 $18 = $info;
 $19 = $6; //@line 2394 "lbfgsb.c"
 $20 = ((($19)) + -4|0); //@line 2394 "lbfgsb.c"
 $6 = $20; //@line 2394 "lbfgsb.c"
 $21 = $3; //@line 2395 "lbfgsb.c"
 $22 = ((($21)) + -4|0); //@line 2395 "lbfgsb.c"
 $3 = $22; //@line 2395 "lbfgsb.c"
 $23 = $11; //@line 2396 "lbfgsb.c"
 $24 = HEAP32[$23>>2]|0; //@line 2396 "lbfgsb.c"
 $sy_dim1 = $24; //@line 2396 "lbfgsb.c"
 $25 = $sy_dim1; //@line 2397 "lbfgsb.c"
 $26 = (1 + ($25))|0; //@line 2397 "lbfgsb.c"
 $sy_offset = $26; //@line 2397 "lbfgsb.c"
 $27 = $sy_offset; //@line 2398 "lbfgsb.c"
 $28 = $14; //@line 2398 "lbfgsb.c"
 $29 = (0 - ($27))|0; //@line 2398 "lbfgsb.c"
 $30 = (($28) + ($29<<3)|0); //@line 2398 "lbfgsb.c"
 $14 = $30; //@line 2398 "lbfgsb.c"
 $31 = $1; //@line 2399 "lbfgsb.c"
 $32 = HEAP32[$31>>2]|0; //@line 2399 "lbfgsb.c"
 $wy_dim1 = $32; //@line 2399 "lbfgsb.c"
 $33 = $wy_dim1; //@line 2400 "lbfgsb.c"
 $34 = (1 + ($33))|0; //@line 2400 "lbfgsb.c"
 $wy_offset = $34; //@line 2400 "lbfgsb.c"
 $35 = $wy_offset; //@line 2401 "lbfgsb.c"
 $36 = $13; //@line 2401 "lbfgsb.c"
 $37 = (0 - ($35))|0; //@line 2401 "lbfgsb.c"
 $38 = (($36) + ($37<<3)|0); //@line 2401 "lbfgsb.c"
 $13 = $38; //@line 2401 "lbfgsb.c"
 $39 = $1; //@line 2402 "lbfgsb.c"
 $40 = HEAP32[$39>>2]|0; //@line 2402 "lbfgsb.c"
 $ws_dim1 = $40; //@line 2402 "lbfgsb.c"
 $41 = $ws_dim1; //@line 2403 "lbfgsb.c"
 $42 = (1 + ($41))|0; //@line 2403 "lbfgsb.c"
 $ws_offset = $42; //@line 2403 "lbfgsb.c"
 $43 = $ws_offset; //@line 2404 "lbfgsb.c"
 $44 = $12; //@line 2404 "lbfgsb.c"
 $45 = (0 - ($43))|0; //@line 2404 "lbfgsb.c"
 $46 = (($44) + ($45<<3)|0); //@line 2404 "lbfgsb.c"
 $12 = $46; //@line 2404 "lbfgsb.c"
 $47 = $11; //@line 2405 "lbfgsb.c"
 $48 = HEAP32[$47>>2]|0; //@line 2405 "lbfgsb.c"
 $49 = $48<<1; //@line 2405 "lbfgsb.c"
 $wn1_dim1 = $49; //@line 2405 "lbfgsb.c"
 $50 = $wn1_dim1; //@line 2406 "lbfgsb.c"
 $51 = (1 + ($50))|0; //@line 2406 "lbfgsb.c"
 $wn1_offset = $51; //@line 2406 "lbfgsb.c"
 $52 = $wn1_offset; //@line 2407 "lbfgsb.c"
 $53 = $10; //@line 2407 "lbfgsb.c"
 $54 = (0 - ($52))|0; //@line 2407 "lbfgsb.c"
 $55 = (($53) + ($54<<3)|0); //@line 2407 "lbfgsb.c"
 $10 = $55; //@line 2407 "lbfgsb.c"
 $56 = $11; //@line 2408 "lbfgsb.c"
 $57 = HEAP32[$56>>2]|0; //@line 2408 "lbfgsb.c"
 $58 = $57<<1; //@line 2408 "lbfgsb.c"
 $wn_dim1 = $58; //@line 2408 "lbfgsb.c"
 $59 = $wn_dim1; //@line 2409 "lbfgsb.c"
 $60 = (1 + ($59))|0; //@line 2409 "lbfgsb.c"
 $wn_offset = $60; //@line 2409 "lbfgsb.c"
 $61 = $wn_offset; //@line 2410 "lbfgsb.c"
 $62 = $9; //@line 2410 "lbfgsb.c"
 $63 = (0 - ($61))|0; //@line 2410 "lbfgsb.c"
 $64 = (($62) + ($63<<3)|0); //@line 2410 "lbfgsb.c"
 $9 = $64; //@line 2410 "lbfgsb.c"
 $65 = $8; //@line 2413 "lbfgsb.c"
 $66 = HEAP32[$65>>2]|0; //@line 2413 "lbfgsb.c"
 $67 = ($66|0)!=(0); //@line 2413 "lbfgsb.c"
 if ($67) {
  $68 = $7; //@line 2414 "lbfgsb.c"
  $69 = HEAP32[$68>>2]|0; //@line 2414 "lbfgsb.c"
  $70 = $11; //@line 2414 "lbfgsb.c"
  $71 = HEAP32[$70>>2]|0; //@line 2414 "lbfgsb.c"
  $72 = ($69|0)>($71|0); //@line 2414 "lbfgsb.c"
  L3: do {
   if ($72) {
    $73 = $11; //@line 2416 "lbfgsb.c"
    $74 = HEAP32[$73>>2]|0; //@line 2416 "lbfgsb.c"
    $75 = (($74) - 1)|0; //@line 2416 "lbfgsb.c"
    $i__1 = $75; //@line 2416 "lbfgsb.c"
    HEAP32[1712>>2] = 1; //@line 2417 "lbfgsb.c"
    while(1) {
     $76 = HEAP32[1712>>2]|0; //@line 2417 "lbfgsb.c"
     $77 = $i__1; //@line 2417 "lbfgsb.c"
     $78 = ($76|0)<=($77|0); //@line 2417 "lbfgsb.c"
     if (!($78)) {
      break L3;
     }
     $79 = $11; //@line 2418 "lbfgsb.c"
     $80 = HEAP32[$79>>2]|0; //@line 2418 "lbfgsb.c"
     $81 = HEAP32[1712>>2]|0; //@line 2418 "lbfgsb.c"
     $82 = (($80) + ($81))|0; //@line 2418 "lbfgsb.c"
     HEAP32[1704>>2] = $82; //@line 2418 "lbfgsb.c"
     $83 = $11; //@line 2419 "lbfgsb.c"
     $84 = HEAP32[$83>>2]|0; //@line 2419 "lbfgsb.c"
     $85 = HEAP32[1712>>2]|0; //@line 2419 "lbfgsb.c"
     $86 = (($84) - ($85))|0; //@line 2419 "lbfgsb.c"
     HEAP32[$i__2>>2] = $86; //@line 2419 "lbfgsb.c"
     $87 = HEAP32[1712>>2]|0; //@line 2420 "lbfgsb.c"
     $88 = (($87) + 1)|0; //@line 2420 "lbfgsb.c"
     $89 = HEAP32[1712>>2]|0; //@line 2420 "lbfgsb.c"
     $90 = (($89) + 1)|0; //@line 2420 "lbfgsb.c"
     $91 = $wn1_dim1; //@line 2420 "lbfgsb.c"
     $92 = Math_imul($90, $91)|0; //@line 2420 "lbfgsb.c"
     $93 = (($88) + ($92))|0; //@line 2420 "lbfgsb.c"
     $94 = $10; //@line 2420 "lbfgsb.c"
     $95 = (($94) + ($93<<3)|0); //@line 2420 "lbfgsb.c"
     $96 = HEAP32[1712>>2]|0; //@line 2421 "lbfgsb.c"
     $97 = HEAP32[1712>>2]|0; //@line 2421 "lbfgsb.c"
     $98 = $wn1_dim1; //@line 2421 "lbfgsb.c"
     $99 = Math_imul($97, $98)|0; //@line 2421 "lbfgsb.c"
     $100 = (($96) + ($99))|0; //@line 2421 "lbfgsb.c"
     $101 = $10; //@line 2420 "lbfgsb.c"
     $102 = (($101) + ($100<<3)|0); //@line 2420 "lbfgsb.c"
     (_dcopy_($i__2,$95,1236,$102,1236)|0); //@line 2420 "lbfgsb.c"
     $103 = $11; //@line 2422 "lbfgsb.c"
     $104 = HEAP32[$103>>2]|0; //@line 2422 "lbfgsb.c"
     $105 = HEAP32[1712>>2]|0; //@line 2422 "lbfgsb.c"
     $106 = (($104) - ($105))|0; //@line 2422 "lbfgsb.c"
     HEAP32[$i__2>>2] = $106; //@line 2422 "lbfgsb.c"
     $107 = HEAP32[1704>>2]|0; //@line 2423 "lbfgsb.c"
     $108 = (($107) + 1)|0; //@line 2423 "lbfgsb.c"
     $109 = HEAP32[1704>>2]|0; //@line 2423 "lbfgsb.c"
     $110 = (($109) + 1)|0; //@line 2423 "lbfgsb.c"
     $111 = $wn1_dim1; //@line 2423 "lbfgsb.c"
     $112 = Math_imul($110, $111)|0; //@line 2423 "lbfgsb.c"
     $113 = (($108) + ($112))|0; //@line 2423 "lbfgsb.c"
     $114 = $10; //@line 2423 "lbfgsb.c"
     $115 = (($114) + ($113<<3)|0); //@line 2423 "lbfgsb.c"
     $116 = HEAP32[1704>>2]|0; //@line 2424 "lbfgsb.c"
     $117 = HEAP32[1704>>2]|0; //@line 2424 "lbfgsb.c"
     $118 = $wn1_dim1; //@line 2424 "lbfgsb.c"
     $119 = Math_imul($117, $118)|0; //@line 2424 "lbfgsb.c"
     $120 = (($116) + ($119))|0; //@line 2424 "lbfgsb.c"
     $121 = $10; //@line 2423 "lbfgsb.c"
     $122 = (($121) + ($120<<3)|0); //@line 2423 "lbfgsb.c"
     (_dcopy_($i__2,$115,1236,$122,1236)|0); //@line 2423 "lbfgsb.c"
     $123 = $11; //@line 2425 "lbfgsb.c"
     $124 = HEAP32[$123>>2]|0; //@line 2425 "lbfgsb.c"
     $125 = (($124) - 1)|0; //@line 2425 "lbfgsb.c"
     HEAP32[$i__2>>2] = $125; //@line 2425 "lbfgsb.c"
     $126 = $11; //@line 2426 "lbfgsb.c"
     $127 = HEAP32[$126>>2]|0; //@line 2426 "lbfgsb.c"
     $128 = (($127) + 2)|0; //@line 2426 "lbfgsb.c"
     $129 = HEAP32[1712>>2]|0; //@line 2426 "lbfgsb.c"
     $130 = (($129) + 1)|0; //@line 2426 "lbfgsb.c"
     $131 = $wn1_dim1; //@line 2426 "lbfgsb.c"
     $132 = Math_imul($130, $131)|0; //@line 2426 "lbfgsb.c"
     $133 = (($128) + ($132))|0; //@line 2426 "lbfgsb.c"
     $134 = $10; //@line 2426 "lbfgsb.c"
     $135 = (($134) + ($133<<3)|0); //@line 2426 "lbfgsb.c"
     $136 = $11; //@line 2427 "lbfgsb.c"
     $137 = HEAP32[$136>>2]|0; //@line 2427 "lbfgsb.c"
     $138 = (($137) + 1)|0; //@line 2427 "lbfgsb.c"
     $139 = HEAP32[1712>>2]|0; //@line 2427 "lbfgsb.c"
     $140 = $wn1_dim1; //@line 2427 "lbfgsb.c"
     $141 = Math_imul($139, $140)|0; //@line 2427 "lbfgsb.c"
     $142 = (($138) + ($141))|0; //@line 2427 "lbfgsb.c"
     $143 = $10; //@line 2426 "lbfgsb.c"
     $144 = (($143) + ($142<<3)|0); //@line 2426 "lbfgsb.c"
     (_dcopy_($i__2,$135,1236,$144,1236)|0); //@line 2426 "lbfgsb.c"
     $145 = HEAP32[1712>>2]|0; //@line 2417 "lbfgsb.c"
     $146 = (($145) + 1)|0; //@line 2417 "lbfgsb.c"
     HEAP32[1712>>2] = $146; //@line 2417 "lbfgsb.c"
    }
   }
  } while(0);
  HEAP32[1752>>2] = 1; //@line 2432 "lbfgsb.c"
  $147 = $2; //@line 2433 "lbfgsb.c"
  $148 = HEAP32[$147>>2]|0; //@line 2433 "lbfgsb.c"
  HEAP32[1732>>2] = $148; //@line 2433 "lbfgsb.c"
  $149 = $2; //@line 2434 "lbfgsb.c"
  $150 = HEAP32[$149>>2]|0; //@line 2434 "lbfgsb.c"
  $151 = (($150) + 1)|0; //@line 2434 "lbfgsb.c"
  HEAP32[1748>>2] = $151; //@line 2434 "lbfgsb.c"
  $152 = $1; //@line 2435 "lbfgsb.c"
  $153 = HEAP32[$152>>2]|0; //@line 2435 "lbfgsb.c"
  HEAP32[1728>>2] = $153; //@line 2435 "lbfgsb.c"
  $154 = $16; //@line 2436 "lbfgsb.c"
  $155 = HEAP32[$154>>2]|0; //@line 2436 "lbfgsb.c"
  HEAP32[1708>>2] = $155; //@line 2436 "lbfgsb.c"
  $156 = $11; //@line 2437 "lbfgsb.c"
  $157 = HEAP32[$156>>2]|0; //@line 2437 "lbfgsb.c"
  $158 = $16; //@line 2437 "lbfgsb.c"
  $159 = HEAP32[$158>>2]|0; //@line 2437 "lbfgsb.c"
  $160 = (($157) + ($159))|0; //@line 2437 "lbfgsb.c"
  HEAP32[1700>>2] = $160; //@line 2437 "lbfgsb.c"
  $161 = $17; //@line 2438 "lbfgsb.c"
  $162 = HEAP32[$161>>2]|0; //@line 2438 "lbfgsb.c"
  $163 = $16; //@line 2438 "lbfgsb.c"
  $164 = HEAP32[$163>>2]|0; //@line 2438 "lbfgsb.c"
  $165 = (($162) + ($164))|0; //@line 2438 "lbfgsb.c"
  $166 = (($165) - 1)|0; //@line 2438 "lbfgsb.c"
  HEAP32[1740>>2] = $166; //@line 2438 "lbfgsb.c"
  $167 = HEAP32[1740>>2]|0; //@line 2439 "lbfgsb.c"
  $168 = $11; //@line 2439 "lbfgsb.c"
  $169 = HEAP32[$168>>2]|0; //@line 2439 "lbfgsb.c"
  $170 = ($167|0)>($169|0); //@line 2439 "lbfgsb.c"
  if ($170) {
   $171 = $11; //@line 2440 "lbfgsb.c"
   $172 = HEAP32[$171>>2]|0; //@line 2440 "lbfgsb.c"
   $173 = HEAP32[1740>>2]|0; //@line 2440 "lbfgsb.c"
   $174 = (($173) - ($172))|0; //@line 2440 "lbfgsb.c"
   HEAP32[1740>>2] = $174; //@line 2440 "lbfgsb.c"
  }
  $175 = $17; //@line 2442 "lbfgsb.c"
  $176 = HEAP32[$175>>2]|0; //@line 2442 "lbfgsb.c"
  HEAP32[1744>>2] = $176; //@line 2442 "lbfgsb.c"
  $177 = $16; //@line 2443 "lbfgsb.c"
  $178 = HEAP32[$177>>2]|0; //@line 2443 "lbfgsb.c"
  $i__1 = $178; //@line 2443 "lbfgsb.c"
  HEAP32[1712>>2] = 1; //@line 2444 "lbfgsb.c"
  while(1) {
   $179 = HEAP32[1712>>2]|0; //@line 2444 "lbfgsb.c"
   $180 = $i__1; //@line 2444 "lbfgsb.c"
   $181 = ($179|0)<=($180|0); //@line 2444 "lbfgsb.c"
   if (!($181)) {
    break;
   }
   $182 = $11; //@line 2445 "lbfgsb.c"
   $183 = HEAP32[$182>>2]|0; //@line 2445 "lbfgsb.c"
   $184 = HEAP32[1712>>2]|0; //@line 2445 "lbfgsb.c"
   $185 = (($183) + ($184))|0; //@line 2445 "lbfgsb.c"
   HEAP32[1704>>2] = $185; //@line 2445 "lbfgsb.c"
   HEAPF64[368>>3] = 0.0; //@line 2446 "lbfgsb.c"
   HEAPF64[376>>3] = 0.0; //@line 2447 "lbfgsb.c"
   HEAPF64[384>>3] = 0.0; //@line 2448 "lbfgsb.c"
   $186 = HEAP32[1732>>2]|0; //@line 2450 "lbfgsb.c"
   HEAP32[$i__2>>2] = $186; //@line 2450 "lbfgsb.c"
   $187 = HEAP32[1752>>2]|0; //@line 2451 "lbfgsb.c"
   HEAP32[1688>>2] = $187; //@line 2451 "lbfgsb.c"
   while(1) {
    $188 = HEAP32[1688>>2]|0; //@line 2451 "lbfgsb.c"
    $189 = HEAP32[$i__2>>2]|0; //@line 2451 "lbfgsb.c"
    $190 = ($188|0)<=($189|0); //@line 2451 "lbfgsb.c"
    if (!($190)) {
     break;
    }
    $191 = HEAP32[1688>>2]|0; //@line 2452 "lbfgsb.c"
    $192 = $3; //@line 2452 "lbfgsb.c"
    $193 = (($192) + ($191<<2)|0); //@line 2452 "lbfgsb.c"
    $194 = HEAP32[$193>>2]|0; //@line 2452 "lbfgsb.c"
    HEAP32[1692>>2] = $194; //@line 2452 "lbfgsb.c"
    $195 = HEAP32[1692>>2]|0; //@line 2453 "lbfgsb.c"
    $196 = HEAP32[1740>>2]|0; //@line 2453 "lbfgsb.c"
    $197 = $wy_dim1; //@line 2453 "lbfgsb.c"
    $198 = Math_imul($196, $197)|0; //@line 2453 "lbfgsb.c"
    $199 = (($195) + ($198))|0; //@line 2453 "lbfgsb.c"
    $200 = $13; //@line 2453 "lbfgsb.c"
    $201 = (($200) + ($199<<3)|0); //@line 2453 "lbfgsb.c"
    $202 = +HEAPF64[$201>>3]; //@line 2453 "lbfgsb.c"
    $203 = HEAP32[1692>>2]|0; //@line 2453 "lbfgsb.c"
    $204 = HEAP32[1744>>2]|0; //@line 2453 "lbfgsb.c"
    $205 = $wy_dim1; //@line 2453 "lbfgsb.c"
    $206 = Math_imul($204, $205)|0; //@line 2453 "lbfgsb.c"
    $207 = (($203) + ($206))|0; //@line 2453 "lbfgsb.c"
    $208 = $13; //@line 2453 "lbfgsb.c"
    $209 = (($208) + ($207<<3)|0); //@line 2453 "lbfgsb.c"
    $210 = +HEAPF64[$209>>3]; //@line 2453 "lbfgsb.c"
    $211 = $202 * $210; //@line 2453 "lbfgsb.c"
    $212 = +HEAPF64[368>>3]; //@line 2453 "lbfgsb.c"
    $213 = $212 + $211; //@line 2453 "lbfgsb.c"
    HEAPF64[368>>3] = $213; //@line 2453 "lbfgsb.c"
    $214 = HEAP32[1688>>2]|0; //@line 2451 "lbfgsb.c"
    $215 = (($214) + 1)|0; //@line 2451 "lbfgsb.c"
    HEAP32[1688>>2] = $215; //@line 2451 "lbfgsb.c"
   }
   $216 = HEAP32[1728>>2]|0; //@line 2457 "lbfgsb.c"
   HEAP32[$i__2>>2] = $216; //@line 2457 "lbfgsb.c"
   $217 = HEAP32[1748>>2]|0; //@line 2458 "lbfgsb.c"
   HEAP32[1688>>2] = $217; //@line 2458 "lbfgsb.c"
   while(1) {
    $218 = HEAP32[1688>>2]|0; //@line 2458 "lbfgsb.c"
    $219 = HEAP32[$i__2>>2]|0; //@line 2458 "lbfgsb.c"
    $220 = ($218|0)<=($219|0); //@line 2458 "lbfgsb.c"
    if (!($220)) {
     break;
    }
    $221 = HEAP32[1688>>2]|0; //@line 2459 "lbfgsb.c"
    $222 = $3; //@line 2459 "lbfgsb.c"
    $223 = (($222) + ($221<<2)|0); //@line 2459 "lbfgsb.c"
    $224 = HEAP32[$223>>2]|0; //@line 2459 "lbfgsb.c"
    HEAP32[1692>>2] = $224; //@line 2459 "lbfgsb.c"
    $225 = HEAP32[1692>>2]|0; //@line 2460 "lbfgsb.c"
    $226 = HEAP32[1740>>2]|0; //@line 2460 "lbfgsb.c"
    $227 = $ws_dim1; //@line 2460 "lbfgsb.c"
    $228 = Math_imul($226, $227)|0; //@line 2460 "lbfgsb.c"
    $229 = (($225) + ($228))|0; //@line 2460 "lbfgsb.c"
    $230 = $12; //@line 2460 "lbfgsb.c"
    $231 = (($230) + ($229<<3)|0); //@line 2460 "lbfgsb.c"
    $232 = +HEAPF64[$231>>3]; //@line 2460 "lbfgsb.c"
    $233 = HEAP32[1692>>2]|0; //@line 2460 "lbfgsb.c"
    $234 = HEAP32[1744>>2]|0; //@line 2460 "lbfgsb.c"
    $235 = $ws_dim1; //@line 2460 "lbfgsb.c"
    $236 = Math_imul($234, $235)|0; //@line 2460 "lbfgsb.c"
    $237 = (($233) + ($236))|0; //@line 2460 "lbfgsb.c"
    $238 = $12; //@line 2460 "lbfgsb.c"
    $239 = (($238) + ($237<<3)|0); //@line 2460 "lbfgsb.c"
    $240 = +HEAPF64[$239>>3]; //@line 2460 "lbfgsb.c"
    $241 = $232 * $240; //@line 2460 "lbfgsb.c"
    $242 = +HEAPF64[376>>3]; //@line 2460 "lbfgsb.c"
    $243 = $242 + $241; //@line 2460 "lbfgsb.c"
    HEAPF64[376>>3] = $243; //@line 2460 "lbfgsb.c"
    $244 = HEAP32[1692>>2]|0; //@line 2461 "lbfgsb.c"
    $245 = HEAP32[1740>>2]|0; //@line 2461 "lbfgsb.c"
    $246 = $ws_dim1; //@line 2461 "lbfgsb.c"
    $247 = Math_imul($245, $246)|0; //@line 2461 "lbfgsb.c"
    $248 = (($244) + ($247))|0; //@line 2461 "lbfgsb.c"
    $249 = $12; //@line 2461 "lbfgsb.c"
    $250 = (($249) + ($248<<3)|0); //@line 2461 "lbfgsb.c"
    $251 = +HEAPF64[$250>>3]; //@line 2461 "lbfgsb.c"
    $252 = HEAP32[1692>>2]|0; //@line 2461 "lbfgsb.c"
    $253 = HEAP32[1744>>2]|0; //@line 2461 "lbfgsb.c"
    $254 = $wy_dim1; //@line 2461 "lbfgsb.c"
    $255 = Math_imul($253, $254)|0; //@line 2461 "lbfgsb.c"
    $256 = (($252) + ($255))|0; //@line 2461 "lbfgsb.c"
    $257 = $13; //@line 2461 "lbfgsb.c"
    $258 = (($257) + ($256<<3)|0); //@line 2461 "lbfgsb.c"
    $259 = +HEAPF64[$258>>3]; //@line 2461 "lbfgsb.c"
    $260 = $251 * $259; //@line 2461 "lbfgsb.c"
    $261 = +HEAPF64[384>>3]; //@line 2461 "lbfgsb.c"
    $262 = $261 + $260; //@line 2461 "lbfgsb.c"
    HEAPF64[384>>3] = $262; //@line 2461 "lbfgsb.c"
    $263 = HEAP32[1688>>2]|0; //@line 2458 "lbfgsb.c"
    $264 = (($263) + 1)|0; //@line 2458 "lbfgsb.c"
    HEAP32[1688>>2] = $264; //@line 2458 "lbfgsb.c"
   }
   $265 = +HEAPF64[368>>3]; //@line 2464 "lbfgsb.c"
   $266 = HEAP32[1708>>2]|0; //@line 2464 "lbfgsb.c"
   $267 = HEAP32[1712>>2]|0; //@line 2464 "lbfgsb.c"
   $268 = $wn1_dim1; //@line 2464 "lbfgsb.c"
   $269 = Math_imul($267, $268)|0; //@line 2464 "lbfgsb.c"
   $270 = (($266) + ($269))|0; //@line 2464 "lbfgsb.c"
   $271 = $10; //@line 2464 "lbfgsb.c"
   $272 = (($271) + ($270<<3)|0); //@line 2464 "lbfgsb.c"
   HEAPF64[$272>>3] = $265; //@line 2464 "lbfgsb.c"
   $273 = +HEAPF64[376>>3]; //@line 2465 "lbfgsb.c"
   $274 = HEAP32[1700>>2]|0; //@line 2465 "lbfgsb.c"
   $275 = HEAP32[1704>>2]|0; //@line 2465 "lbfgsb.c"
   $276 = $wn1_dim1; //@line 2465 "lbfgsb.c"
   $277 = Math_imul($275, $276)|0; //@line 2465 "lbfgsb.c"
   $278 = (($274) + ($277))|0; //@line 2465 "lbfgsb.c"
   $279 = $10; //@line 2465 "lbfgsb.c"
   $280 = (($279) + ($278<<3)|0); //@line 2465 "lbfgsb.c"
   HEAPF64[$280>>3] = $273; //@line 2465 "lbfgsb.c"
   $281 = +HEAPF64[384>>3]; //@line 2466 "lbfgsb.c"
   $282 = HEAP32[1700>>2]|0; //@line 2466 "lbfgsb.c"
   $283 = HEAP32[1712>>2]|0; //@line 2466 "lbfgsb.c"
   $284 = $wn1_dim1; //@line 2466 "lbfgsb.c"
   $285 = Math_imul($283, $284)|0; //@line 2466 "lbfgsb.c"
   $286 = (($282) + ($285))|0; //@line 2466 "lbfgsb.c"
   $287 = $10; //@line 2466 "lbfgsb.c"
   $288 = (($287) + ($286<<3)|0); //@line 2466 "lbfgsb.c"
   HEAPF64[$288>>3] = $281; //@line 2466 "lbfgsb.c"
   $289 = HEAP32[1744>>2]|0; //@line 2467 "lbfgsb.c"
   $290 = $11; //@line 2467 "lbfgsb.c"
   $291 = HEAP32[$290>>2]|0; //@line 2467 "lbfgsb.c"
   $292 = (($289|0) % ($291|0))&-1; //@line 2467 "lbfgsb.c"
   $293 = (($292) + 1)|0; //@line 2467 "lbfgsb.c"
   HEAP32[1744>>2] = $293; //@line 2467 "lbfgsb.c"
   $294 = HEAP32[1712>>2]|0; //@line 2444 "lbfgsb.c"
   $295 = (($294) + 1)|0; //@line 2444 "lbfgsb.c"
   HEAP32[1712>>2] = $295; //@line 2444 "lbfgsb.c"
  }
  $296 = $16; //@line 2471 "lbfgsb.c"
  $297 = HEAP32[$296>>2]|0; //@line 2471 "lbfgsb.c"
  HEAP32[1712>>2] = $297; //@line 2471 "lbfgsb.c"
  $298 = $17; //@line 2472 "lbfgsb.c"
  $299 = HEAP32[$298>>2]|0; //@line 2472 "lbfgsb.c"
  $300 = $16; //@line 2472 "lbfgsb.c"
  $301 = HEAP32[$300>>2]|0; //@line 2472 "lbfgsb.c"
  $302 = (($299) + ($301))|0; //@line 2472 "lbfgsb.c"
  $303 = (($302) - 1)|0; //@line 2472 "lbfgsb.c"
  HEAP32[1744>>2] = $303; //@line 2472 "lbfgsb.c"
  $304 = HEAP32[1744>>2]|0; //@line 2473 "lbfgsb.c"
  $305 = $11; //@line 2473 "lbfgsb.c"
  $306 = HEAP32[$305>>2]|0; //@line 2473 "lbfgsb.c"
  $307 = ($304|0)>($306|0); //@line 2473 "lbfgsb.c"
  if ($307) {
   $308 = $11; //@line 2474 "lbfgsb.c"
   $309 = HEAP32[$308>>2]|0; //@line 2474 "lbfgsb.c"
   $310 = HEAP32[1744>>2]|0; //@line 2474 "lbfgsb.c"
   $311 = (($310) - ($309))|0; //@line 2474 "lbfgsb.c"
   HEAP32[1744>>2] = $311; //@line 2474 "lbfgsb.c"
  }
  $312 = $17; //@line 2476 "lbfgsb.c"
  $313 = HEAP32[$312>>2]|0; //@line 2476 "lbfgsb.c"
  HEAP32[1740>>2] = $313; //@line 2476 "lbfgsb.c"
  $314 = $16; //@line 2477 "lbfgsb.c"
  $315 = HEAP32[$314>>2]|0; //@line 2477 "lbfgsb.c"
  $i__1 = $315; //@line 2477 "lbfgsb.c"
  HEAP32[1684>>2] = 1; //@line 2478 "lbfgsb.c"
  while(1) {
   $316 = HEAP32[1684>>2]|0; //@line 2478 "lbfgsb.c"
   $317 = $i__1; //@line 2478 "lbfgsb.c"
   $318 = ($316|0)<=($317|0); //@line 2478 "lbfgsb.c"
   if (!($318)) {
    break;
   }
   $319 = $11; //@line 2479 "lbfgsb.c"
   $320 = HEAP32[$319>>2]|0; //@line 2479 "lbfgsb.c"
   $321 = HEAP32[1684>>2]|0; //@line 2479 "lbfgsb.c"
   $322 = (($320) + ($321))|0; //@line 2479 "lbfgsb.c"
   HEAP32[1700>>2] = $322; //@line 2479 "lbfgsb.c"
   HEAPF64[384>>3] = 0.0; //@line 2480 "lbfgsb.c"
   $323 = HEAP32[1732>>2]|0; //@line 2482 "lbfgsb.c"
   HEAP32[$i__2>>2] = $323; //@line 2482 "lbfgsb.c"
   $324 = HEAP32[1752>>2]|0; //@line 2483 "lbfgsb.c"
   HEAP32[1688>>2] = $324; //@line 2483 "lbfgsb.c"
   while(1) {
    $325 = HEAP32[1688>>2]|0; //@line 2483 "lbfgsb.c"
    $326 = HEAP32[$i__2>>2]|0; //@line 2483 "lbfgsb.c"
    $327 = ($325|0)<=($326|0); //@line 2483 "lbfgsb.c"
    if (!($327)) {
     break;
    }
    $328 = HEAP32[1688>>2]|0; //@line 2484 "lbfgsb.c"
    $329 = $3; //@line 2484 "lbfgsb.c"
    $330 = (($329) + ($328<<2)|0); //@line 2484 "lbfgsb.c"
    $331 = HEAP32[$330>>2]|0; //@line 2484 "lbfgsb.c"
    HEAP32[1692>>2] = $331; //@line 2484 "lbfgsb.c"
    $332 = HEAP32[1692>>2]|0; //@line 2485 "lbfgsb.c"
    $333 = HEAP32[1740>>2]|0; //@line 2485 "lbfgsb.c"
    $334 = $ws_dim1; //@line 2485 "lbfgsb.c"
    $335 = Math_imul($333, $334)|0; //@line 2485 "lbfgsb.c"
    $336 = (($332) + ($335))|0; //@line 2485 "lbfgsb.c"
    $337 = $12; //@line 2485 "lbfgsb.c"
    $338 = (($337) + ($336<<3)|0); //@line 2485 "lbfgsb.c"
    $339 = +HEAPF64[$338>>3]; //@line 2485 "lbfgsb.c"
    $340 = HEAP32[1692>>2]|0; //@line 2485 "lbfgsb.c"
    $341 = HEAP32[1744>>2]|0; //@line 2485 "lbfgsb.c"
    $342 = $wy_dim1; //@line 2485 "lbfgsb.c"
    $343 = Math_imul($341, $342)|0; //@line 2485 "lbfgsb.c"
    $344 = (($340) + ($343))|0; //@line 2485 "lbfgsb.c"
    $345 = $13; //@line 2485 "lbfgsb.c"
    $346 = (($345) + ($344<<3)|0); //@line 2485 "lbfgsb.c"
    $347 = +HEAPF64[$346>>3]; //@line 2485 "lbfgsb.c"
    $348 = $339 * $347; //@line 2485 "lbfgsb.c"
    $349 = +HEAPF64[384>>3]; //@line 2485 "lbfgsb.c"
    $350 = $349 + $348; //@line 2485 "lbfgsb.c"
    HEAPF64[384>>3] = $350; //@line 2485 "lbfgsb.c"
    $351 = HEAP32[1688>>2]|0; //@line 2483 "lbfgsb.c"
    $352 = (($351) + 1)|0; //@line 2483 "lbfgsb.c"
    HEAP32[1688>>2] = $352; //@line 2483 "lbfgsb.c"
   }
   $353 = HEAP32[1740>>2]|0; //@line 2488 "lbfgsb.c"
   $354 = $11; //@line 2488 "lbfgsb.c"
   $355 = HEAP32[$354>>2]|0; //@line 2488 "lbfgsb.c"
   $356 = (($353|0) % ($355|0))&-1; //@line 2488 "lbfgsb.c"
   $357 = (($356) + 1)|0; //@line 2488 "lbfgsb.c"
   HEAP32[1740>>2] = $357; //@line 2488 "lbfgsb.c"
   $358 = +HEAPF64[384>>3]; //@line 2489 "lbfgsb.c"
   $359 = HEAP32[1700>>2]|0; //@line 2489 "lbfgsb.c"
   $360 = HEAP32[1712>>2]|0; //@line 2489 "lbfgsb.c"
   $361 = $wn1_dim1; //@line 2489 "lbfgsb.c"
   $362 = Math_imul($360, $361)|0; //@line 2489 "lbfgsb.c"
   $363 = (($359) + ($362))|0; //@line 2489 "lbfgsb.c"
   $364 = $10; //@line 2489 "lbfgsb.c"
   $365 = (($364) + ($363<<3)|0); //@line 2489 "lbfgsb.c"
   HEAPF64[$365>>3] = $358; //@line 2489 "lbfgsb.c"
   $366 = HEAP32[1684>>2]|0; //@line 2478 "lbfgsb.c"
   $367 = (($366) + 1)|0; //@line 2478 "lbfgsb.c"
   HEAP32[1684>>2] = $367; //@line 2478 "lbfgsb.c"
  }
  $368 = $16; //@line 2492 "lbfgsb.c"
  $369 = HEAP32[$368>>2]|0; //@line 2492 "lbfgsb.c"
  $370 = (($369) - 1)|0; //@line 2492 "lbfgsb.c"
  HEAP32[1736>>2] = $370; //@line 2492 "lbfgsb.c"
 } else {
  $371 = $16; //@line 2494 "lbfgsb.c"
  $372 = HEAP32[$371>>2]|0; //@line 2494 "lbfgsb.c"
  HEAP32[1736>>2] = $372; //@line 2494 "lbfgsb.c"
 }
 $373 = $17; //@line 2498 "lbfgsb.c"
 $374 = HEAP32[$373>>2]|0; //@line 2498 "lbfgsb.c"
 HEAP32[1740>>2] = $374; //@line 2498 "lbfgsb.c"
 $375 = HEAP32[1736>>2]|0; //@line 2499 "lbfgsb.c"
 $i__1 = $375; //@line 2499 "lbfgsb.c"
 HEAP32[1708>>2] = 1; //@line 2500 "lbfgsb.c"
 while(1) {
  $376 = HEAP32[1708>>2]|0; //@line 2500 "lbfgsb.c"
  $377 = $i__1; //@line 2500 "lbfgsb.c"
  $378 = ($376|0)<=($377|0); //@line 2500 "lbfgsb.c"
  if (!($378)) {
   break;
  }
  $379 = $11; //@line 2501 "lbfgsb.c"
  $380 = HEAP32[$379>>2]|0; //@line 2501 "lbfgsb.c"
  $381 = HEAP32[1708>>2]|0; //@line 2501 "lbfgsb.c"
  $382 = (($380) + ($381))|0; //@line 2501 "lbfgsb.c"
  HEAP32[1700>>2] = $382; //@line 2501 "lbfgsb.c"
  $383 = $17; //@line 2502 "lbfgsb.c"
  $384 = HEAP32[$383>>2]|0; //@line 2502 "lbfgsb.c"
  HEAP32[1744>>2] = $384; //@line 2502 "lbfgsb.c"
  $385 = HEAP32[1708>>2]|0; //@line 2503 "lbfgsb.c"
  HEAP32[$i__2>>2] = $385; //@line 2503 "lbfgsb.c"
  HEAP32[1712>>2] = 1; //@line 2504 "lbfgsb.c"
  while(1) {
   $386 = HEAP32[1712>>2]|0; //@line 2504 "lbfgsb.c"
   $387 = HEAP32[$i__2>>2]|0; //@line 2504 "lbfgsb.c"
   $388 = ($386|0)<=($387|0); //@line 2504 "lbfgsb.c"
   if (!($388)) {
    break;
   }
   $389 = $11; //@line 2505 "lbfgsb.c"
   $390 = HEAP32[$389>>2]|0; //@line 2505 "lbfgsb.c"
   $391 = HEAP32[1712>>2]|0; //@line 2505 "lbfgsb.c"
   $392 = (($390) + ($391))|0; //@line 2505 "lbfgsb.c"
   HEAP32[1704>>2] = $392; //@line 2505 "lbfgsb.c"
   HEAPF64[368>>3] = 0.0; //@line 2506 "lbfgsb.c"
   HEAPF64[376>>3] = 0.0; //@line 2507 "lbfgsb.c"
   HEAPF64[384>>3] = 0.0; //@line 2508 "lbfgsb.c"
   HEAPF64[392>>3] = 0.0; //@line 2509 "lbfgsb.c"
   $393 = $4; //@line 2510 "lbfgsb.c"
   $394 = HEAP32[$393>>2]|0; //@line 2510 "lbfgsb.c"
   $i__3 = $394; //@line 2510 "lbfgsb.c"
   HEAP32[1688>>2] = 1; //@line 2511 "lbfgsb.c"
   while(1) {
    $395 = HEAP32[1688>>2]|0; //@line 2511 "lbfgsb.c"
    $396 = $i__3; //@line 2511 "lbfgsb.c"
    $397 = ($395|0)<=($396|0); //@line 2511 "lbfgsb.c"
    if (!($397)) {
     break;
    }
    $398 = HEAP32[1688>>2]|0; //@line 2512 "lbfgsb.c"
    $399 = $6; //@line 2512 "lbfgsb.c"
    $400 = (($399) + ($398<<2)|0); //@line 2512 "lbfgsb.c"
    $401 = HEAP32[$400>>2]|0; //@line 2512 "lbfgsb.c"
    HEAP32[1692>>2] = $401; //@line 2512 "lbfgsb.c"
    $402 = HEAP32[1692>>2]|0; //@line 2513 "lbfgsb.c"
    $403 = HEAP32[1740>>2]|0; //@line 2513 "lbfgsb.c"
    $404 = $wy_dim1; //@line 2513 "lbfgsb.c"
    $405 = Math_imul($403, $404)|0; //@line 2513 "lbfgsb.c"
    $406 = (($402) + ($405))|0; //@line 2513 "lbfgsb.c"
    $407 = $13; //@line 2513 "lbfgsb.c"
    $408 = (($407) + ($406<<3)|0); //@line 2513 "lbfgsb.c"
    $409 = +HEAPF64[$408>>3]; //@line 2513 "lbfgsb.c"
    $410 = HEAP32[1692>>2]|0; //@line 2513 "lbfgsb.c"
    $411 = HEAP32[1744>>2]|0; //@line 2513 "lbfgsb.c"
    $412 = $wy_dim1; //@line 2513 "lbfgsb.c"
    $413 = Math_imul($411, $412)|0; //@line 2513 "lbfgsb.c"
    $414 = (($410) + ($413))|0; //@line 2513 "lbfgsb.c"
    $415 = $13; //@line 2513 "lbfgsb.c"
    $416 = (($415) + ($414<<3)|0); //@line 2513 "lbfgsb.c"
    $417 = +HEAPF64[$416>>3]; //@line 2513 "lbfgsb.c"
    $418 = $409 * $417; //@line 2513 "lbfgsb.c"
    $419 = +HEAPF64[368>>3]; //@line 2513 "lbfgsb.c"
    $420 = $419 + $418; //@line 2513 "lbfgsb.c"
    HEAPF64[368>>3] = $420; //@line 2513 "lbfgsb.c"
    $421 = HEAP32[1692>>2]|0; //@line 2514 "lbfgsb.c"
    $422 = HEAP32[1740>>2]|0; //@line 2514 "lbfgsb.c"
    $423 = $ws_dim1; //@line 2514 "lbfgsb.c"
    $424 = Math_imul($422, $423)|0; //@line 2514 "lbfgsb.c"
    $425 = (($421) + ($424))|0; //@line 2514 "lbfgsb.c"
    $426 = $12; //@line 2514 "lbfgsb.c"
    $427 = (($426) + ($425<<3)|0); //@line 2514 "lbfgsb.c"
    $428 = +HEAPF64[$427>>3]; //@line 2514 "lbfgsb.c"
    $429 = HEAP32[1692>>2]|0; //@line 2514 "lbfgsb.c"
    $430 = HEAP32[1744>>2]|0; //@line 2514 "lbfgsb.c"
    $431 = $ws_dim1; //@line 2514 "lbfgsb.c"
    $432 = Math_imul($430, $431)|0; //@line 2514 "lbfgsb.c"
    $433 = (($429) + ($432))|0; //@line 2514 "lbfgsb.c"
    $434 = $12; //@line 2514 "lbfgsb.c"
    $435 = (($434) + ($433<<3)|0); //@line 2514 "lbfgsb.c"
    $436 = +HEAPF64[$435>>3]; //@line 2514 "lbfgsb.c"
    $437 = $428 * $436; //@line 2514 "lbfgsb.c"
    $438 = +HEAPF64[376>>3]; //@line 2514 "lbfgsb.c"
    $439 = $438 + $437; //@line 2514 "lbfgsb.c"
    HEAPF64[376>>3] = $439; //@line 2514 "lbfgsb.c"
    $440 = HEAP32[1688>>2]|0; //@line 2511 "lbfgsb.c"
    $441 = (($440) + 1)|0; //@line 2511 "lbfgsb.c"
    HEAP32[1688>>2] = $441; //@line 2511 "lbfgsb.c"
   }
   $442 = $1; //@line 2517 "lbfgsb.c"
   $443 = HEAP32[$442>>2]|0; //@line 2517 "lbfgsb.c"
   $i__3 = $443; //@line 2517 "lbfgsb.c"
   $444 = $5; //@line 2518 "lbfgsb.c"
   $445 = HEAP32[$444>>2]|0; //@line 2518 "lbfgsb.c"
   HEAP32[1688>>2] = $445; //@line 2518 "lbfgsb.c"
   while(1) {
    $446 = HEAP32[1688>>2]|0; //@line 2518 "lbfgsb.c"
    $447 = $i__3; //@line 2518 "lbfgsb.c"
    $448 = ($446|0)<=($447|0); //@line 2518 "lbfgsb.c"
    if (!($448)) {
     break;
    }
    $449 = HEAP32[1688>>2]|0; //@line 2519 "lbfgsb.c"
    $450 = $6; //@line 2519 "lbfgsb.c"
    $451 = (($450) + ($449<<2)|0); //@line 2519 "lbfgsb.c"
    $452 = HEAP32[$451>>2]|0; //@line 2519 "lbfgsb.c"
    HEAP32[1692>>2] = $452; //@line 2519 "lbfgsb.c"
    $453 = HEAP32[1692>>2]|0; //@line 2520 "lbfgsb.c"
    $454 = HEAP32[1740>>2]|0; //@line 2520 "lbfgsb.c"
    $455 = $wy_dim1; //@line 2520 "lbfgsb.c"
    $456 = Math_imul($454, $455)|0; //@line 2520 "lbfgsb.c"
    $457 = (($453) + ($456))|0; //@line 2520 "lbfgsb.c"
    $458 = $13; //@line 2520 "lbfgsb.c"
    $459 = (($458) + ($457<<3)|0); //@line 2520 "lbfgsb.c"
    $460 = +HEAPF64[$459>>3]; //@line 2520 "lbfgsb.c"
    $461 = HEAP32[1692>>2]|0; //@line 2520 "lbfgsb.c"
    $462 = HEAP32[1744>>2]|0; //@line 2520 "lbfgsb.c"
    $463 = $wy_dim1; //@line 2520 "lbfgsb.c"
    $464 = Math_imul($462, $463)|0; //@line 2520 "lbfgsb.c"
    $465 = (($461) + ($464))|0; //@line 2520 "lbfgsb.c"
    $466 = $13; //@line 2520 "lbfgsb.c"
    $467 = (($466) + ($465<<3)|0); //@line 2520 "lbfgsb.c"
    $468 = +HEAPF64[$467>>3]; //@line 2520 "lbfgsb.c"
    $469 = $460 * $468; //@line 2520 "lbfgsb.c"
    $470 = +HEAPF64[384>>3]; //@line 2520 "lbfgsb.c"
    $471 = $470 + $469; //@line 2520 "lbfgsb.c"
    HEAPF64[384>>3] = $471; //@line 2520 "lbfgsb.c"
    $472 = HEAP32[1692>>2]|0; //@line 2521 "lbfgsb.c"
    $473 = HEAP32[1740>>2]|0; //@line 2521 "lbfgsb.c"
    $474 = $ws_dim1; //@line 2521 "lbfgsb.c"
    $475 = Math_imul($473, $474)|0; //@line 2521 "lbfgsb.c"
    $476 = (($472) + ($475))|0; //@line 2521 "lbfgsb.c"
    $477 = $12; //@line 2521 "lbfgsb.c"
    $478 = (($477) + ($476<<3)|0); //@line 2521 "lbfgsb.c"
    $479 = +HEAPF64[$478>>3]; //@line 2521 "lbfgsb.c"
    $480 = HEAP32[1692>>2]|0; //@line 2521 "lbfgsb.c"
    $481 = HEAP32[1744>>2]|0; //@line 2521 "lbfgsb.c"
    $482 = $ws_dim1; //@line 2521 "lbfgsb.c"
    $483 = Math_imul($481, $482)|0; //@line 2521 "lbfgsb.c"
    $484 = (($480) + ($483))|0; //@line 2521 "lbfgsb.c"
    $485 = $12; //@line 2521 "lbfgsb.c"
    $486 = (($485) + ($484<<3)|0); //@line 2521 "lbfgsb.c"
    $487 = +HEAPF64[$486>>3]; //@line 2521 "lbfgsb.c"
    $488 = $479 * $487; //@line 2521 "lbfgsb.c"
    $489 = +HEAPF64[392>>3]; //@line 2521 "lbfgsb.c"
    $490 = $489 + $488; //@line 2521 "lbfgsb.c"
    HEAPF64[392>>3] = $490; //@line 2521 "lbfgsb.c"
    $491 = HEAP32[1688>>2]|0; //@line 2518 "lbfgsb.c"
    $492 = (($491) + 1)|0; //@line 2518 "lbfgsb.c"
    HEAP32[1688>>2] = $492; //@line 2518 "lbfgsb.c"
   }
   $493 = HEAP32[1708>>2]|0; //@line 2524 "lbfgsb.c"
   $494 = HEAP32[1712>>2]|0; //@line 2524 "lbfgsb.c"
   $495 = $wn1_dim1; //@line 2524 "lbfgsb.c"
   $496 = Math_imul($494, $495)|0; //@line 2524 "lbfgsb.c"
   $497 = (($493) + ($496))|0; //@line 2524 "lbfgsb.c"
   $498 = $10; //@line 2524 "lbfgsb.c"
   $499 = (($498) + ($497<<3)|0); //@line 2524 "lbfgsb.c"
   $500 = +HEAPF64[$499>>3]; //@line 2524 "lbfgsb.c"
   $501 = +HEAPF64[368>>3]; //@line 2524 "lbfgsb.c"
   $502 = $500 + $501; //@line 2524 "lbfgsb.c"
   $503 = +HEAPF64[384>>3]; //@line 2524 "lbfgsb.c"
   $504 = $502 - $503; //@line 2524 "lbfgsb.c"
   $505 = HEAP32[1708>>2]|0; //@line 2524 "lbfgsb.c"
   $506 = HEAP32[1712>>2]|0; //@line 2524 "lbfgsb.c"
   $507 = $wn1_dim1; //@line 2524 "lbfgsb.c"
   $508 = Math_imul($506, $507)|0; //@line 2524 "lbfgsb.c"
   $509 = (($505) + ($508))|0; //@line 2524 "lbfgsb.c"
   $510 = $10; //@line 2524 "lbfgsb.c"
   $511 = (($510) + ($509<<3)|0); //@line 2524 "lbfgsb.c"
   HEAPF64[$511>>3] = $504; //@line 2524 "lbfgsb.c"
   $512 = HEAP32[1700>>2]|0; //@line 2525 "lbfgsb.c"
   $513 = HEAP32[1704>>2]|0; //@line 2525 "lbfgsb.c"
   $514 = $wn1_dim1; //@line 2525 "lbfgsb.c"
   $515 = Math_imul($513, $514)|0; //@line 2525 "lbfgsb.c"
   $516 = (($512) + ($515))|0; //@line 2525 "lbfgsb.c"
   $517 = $10; //@line 2525 "lbfgsb.c"
   $518 = (($517) + ($516<<3)|0); //@line 2525 "lbfgsb.c"
   $519 = +HEAPF64[$518>>3]; //@line 2525 "lbfgsb.c"
   $520 = +HEAPF64[376>>3]; //@line 2525 "lbfgsb.c"
   $521 = $519 - $520; //@line 2525 "lbfgsb.c"
   $522 = +HEAPF64[392>>3]; //@line 2525 "lbfgsb.c"
   $523 = $521 + $522; //@line 2525 "lbfgsb.c"
   $524 = HEAP32[1700>>2]|0; //@line 2525 "lbfgsb.c"
   $525 = HEAP32[1704>>2]|0; //@line 2525 "lbfgsb.c"
   $526 = $wn1_dim1; //@line 2525 "lbfgsb.c"
   $527 = Math_imul($525, $526)|0; //@line 2525 "lbfgsb.c"
   $528 = (($524) + ($527))|0; //@line 2525 "lbfgsb.c"
   $529 = $10; //@line 2525 "lbfgsb.c"
   $530 = (($529) + ($528<<3)|0); //@line 2525 "lbfgsb.c"
   HEAPF64[$530>>3] = $523; //@line 2525 "lbfgsb.c"
   $531 = HEAP32[1744>>2]|0; //@line 2526 "lbfgsb.c"
   $532 = $11; //@line 2526 "lbfgsb.c"
   $533 = HEAP32[$532>>2]|0; //@line 2526 "lbfgsb.c"
   $534 = (($531|0) % ($533|0))&-1; //@line 2526 "lbfgsb.c"
   $535 = (($534) + 1)|0; //@line 2526 "lbfgsb.c"
   HEAP32[1744>>2] = $535; //@line 2526 "lbfgsb.c"
   $536 = HEAP32[1712>>2]|0; //@line 2504 "lbfgsb.c"
   $537 = (($536) + 1)|0; //@line 2504 "lbfgsb.c"
   HEAP32[1712>>2] = $537; //@line 2504 "lbfgsb.c"
  }
  $538 = HEAP32[1740>>2]|0; //@line 2529 "lbfgsb.c"
  $539 = $11; //@line 2529 "lbfgsb.c"
  $540 = HEAP32[$539>>2]|0; //@line 2529 "lbfgsb.c"
  $541 = (($538|0) % ($540|0))&-1; //@line 2529 "lbfgsb.c"
  $542 = (($541) + 1)|0; //@line 2529 "lbfgsb.c"
  HEAP32[1740>>2] = $542; //@line 2529 "lbfgsb.c"
  $543 = HEAP32[1708>>2]|0; //@line 2500 "lbfgsb.c"
  $544 = (($543) + 1)|0; //@line 2500 "lbfgsb.c"
  HEAP32[1708>>2] = $544; //@line 2500 "lbfgsb.c"
 }
 $545 = $17; //@line 2533 "lbfgsb.c"
 $546 = HEAP32[$545>>2]|0; //@line 2533 "lbfgsb.c"
 HEAP32[1740>>2] = $546; //@line 2533 "lbfgsb.c"
 $547 = $11; //@line 2534 "lbfgsb.c"
 $548 = HEAP32[$547>>2]|0; //@line 2534 "lbfgsb.c"
 $549 = HEAP32[1736>>2]|0; //@line 2534 "lbfgsb.c"
 $550 = (($548) + ($549))|0; //@line 2534 "lbfgsb.c"
 $i__1 = $550; //@line 2534 "lbfgsb.c"
 $551 = $11; //@line 2535 "lbfgsb.c"
 $552 = HEAP32[$551>>2]|0; //@line 2535 "lbfgsb.c"
 $553 = (($552) + 1)|0; //@line 2535 "lbfgsb.c"
 HEAP32[1700>>2] = $553; //@line 2535 "lbfgsb.c"
 while(1) {
  $554 = HEAP32[1700>>2]|0; //@line 2535 "lbfgsb.c"
  $555 = $i__1; //@line 2535 "lbfgsb.c"
  $556 = ($554|0)<=($555|0); //@line 2535 "lbfgsb.c"
  if (!($556)) {
   break;
  }
  $557 = $17; //@line 2536 "lbfgsb.c"
  $558 = HEAP32[$557>>2]|0; //@line 2536 "lbfgsb.c"
  HEAP32[1744>>2] = $558; //@line 2536 "lbfgsb.c"
  $559 = HEAP32[1736>>2]|0; //@line 2537 "lbfgsb.c"
  HEAP32[$i__2>>2] = $559; //@line 2537 "lbfgsb.c"
  HEAP32[1712>>2] = 1; //@line 2538 "lbfgsb.c"
  while(1) {
   $560 = HEAP32[1712>>2]|0; //@line 2538 "lbfgsb.c"
   $561 = HEAP32[$i__2>>2]|0; //@line 2538 "lbfgsb.c"
   $562 = ($560|0)<=($561|0); //@line 2538 "lbfgsb.c"
   if (!($562)) {
    break;
   }
   HEAPF64[368>>3] = 0.0; //@line 2539 "lbfgsb.c"
   HEAPF64[384>>3] = 0.0; //@line 2540 "lbfgsb.c"
   $563 = $4; //@line 2541 "lbfgsb.c"
   $564 = HEAP32[$563>>2]|0; //@line 2541 "lbfgsb.c"
   $i__3 = $564; //@line 2541 "lbfgsb.c"
   HEAP32[1688>>2] = 1; //@line 2542 "lbfgsb.c"
   while(1) {
    $565 = HEAP32[1688>>2]|0; //@line 2542 "lbfgsb.c"
    $566 = $i__3; //@line 2542 "lbfgsb.c"
    $567 = ($565|0)<=($566|0); //@line 2542 "lbfgsb.c"
    if (!($567)) {
     break;
    }
    $568 = HEAP32[1688>>2]|0; //@line 2543 "lbfgsb.c"
    $569 = $6; //@line 2543 "lbfgsb.c"
    $570 = (($569) + ($568<<2)|0); //@line 2543 "lbfgsb.c"
    $571 = HEAP32[$570>>2]|0; //@line 2543 "lbfgsb.c"
    HEAP32[1692>>2] = $571; //@line 2543 "lbfgsb.c"
    $572 = HEAP32[1692>>2]|0; //@line 2544 "lbfgsb.c"
    $573 = HEAP32[1740>>2]|0; //@line 2544 "lbfgsb.c"
    $574 = $ws_dim1; //@line 2544 "lbfgsb.c"
    $575 = Math_imul($573, $574)|0; //@line 2544 "lbfgsb.c"
    $576 = (($572) + ($575))|0; //@line 2544 "lbfgsb.c"
    $577 = $12; //@line 2544 "lbfgsb.c"
    $578 = (($577) + ($576<<3)|0); //@line 2544 "lbfgsb.c"
    $579 = +HEAPF64[$578>>3]; //@line 2544 "lbfgsb.c"
    $580 = HEAP32[1692>>2]|0; //@line 2544 "lbfgsb.c"
    $581 = HEAP32[1744>>2]|0; //@line 2544 "lbfgsb.c"
    $582 = $wy_dim1; //@line 2544 "lbfgsb.c"
    $583 = Math_imul($581, $582)|0; //@line 2544 "lbfgsb.c"
    $584 = (($580) + ($583))|0; //@line 2544 "lbfgsb.c"
    $585 = $13; //@line 2544 "lbfgsb.c"
    $586 = (($585) + ($584<<3)|0); //@line 2544 "lbfgsb.c"
    $587 = +HEAPF64[$586>>3]; //@line 2544 "lbfgsb.c"
    $588 = $579 * $587; //@line 2544 "lbfgsb.c"
    $589 = +HEAPF64[368>>3]; //@line 2544 "lbfgsb.c"
    $590 = $589 + $588; //@line 2544 "lbfgsb.c"
    HEAPF64[368>>3] = $590; //@line 2544 "lbfgsb.c"
    $591 = HEAP32[1688>>2]|0; //@line 2542 "lbfgsb.c"
    $592 = (($591) + 1)|0; //@line 2542 "lbfgsb.c"
    HEAP32[1688>>2] = $592; //@line 2542 "lbfgsb.c"
   }
   $593 = $1; //@line 2547 "lbfgsb.c"
   $594 = HEAP32[$593>>2]|0; //@line 2547 "lbfgsb.c"
   $i__3 = $594; //@line 2547 "lbfgsb.c"
   $595 = $5; //@line 2548 "lbfgsb.c"
   $596 = HEAP32[$595>>2]|0; //@line 2548 "lbfgsb.c"
   HEAP32[1688>>2] = $596; //@line 2548 "lbfgsb.c"
   while(1) {
    $597 = HEAP32[1688>>2]|0; //@line 2548 "lbfgsb.c"
    $598 = $i__3; //@line 2548 "lbfgsb.c"
    $599 = ($597|0)<=($598|0); //@line 2548 "lbfgsb.c"
    if (!($599)) {
     break;
    }
    $600 = HEAP32[1688>>2]|0; //@line 2549 "lbfgsb.c"
    $601 = $6; //@line 2549 "lbfgsb.c"
    $602 = (($601) + ($600<<2)|0); //@line 2549 "lbfgsb.c"
    $603 = HEAP32[$602>>2]|0; //@line 2549 "lbfgsb.c"
    HEAP32[1692>>2] = $603; //@line 2549 "lbfgsb.c"
    $604 = HEAP32[1692>>2]|0; //@line 2550 "lbfgsb.c"
    $605 = HEAP32[1740>>2]|0; //@line 2550 "lbfgsb.c"
    $606 = $ws_dim1; //@line 2550 "lbfgsb.c"
    $607 = Math_imul($605, $606)|0; //@line 2550 "lbfgsb.c"
    $608 = (($604) + ($607))|0; //@line 2550 "lbfgsb.c"
    $609 = $12; //@line 2550 "lbfgsb.c"
    $610 = (($609) + ($608<<3)|0); //@line 2550 "lbfgsb.c"
    $611 = +HEAPF64[$610>>3]; //@line 2550 "lbfgsb.c"
    $612 = HEAP32[1692>>2]|0; //@line 2550 "lbfgsb.c"
    $613 = HEAP32[1744>>2]|0; //@line 2550 "lbfgsb.c"
    $614 = $wy_dim1; //@line 2550 "lbfgsb.c"
    $615 = Math_imul($613, $614)|0; //@line 2550 "lbfgsb.c"
    $616 = (($612) + ($615))|0; //@line 2550 "lbfgsb.c"
    $617 = $13; //@line 2550 "lbfgsb.c"
    $618 = (($617) + ($616<<3)|0); //@line 2550 "lbfgsb.c"
    $619 = +HEAPF64[$618>>3]; //@line 2550 "lbfgsb.c"
    $620 = $611 * $619; //@line 2550 "lbfgsb.c"
    $621 = +HEAPF64[384>>3]; //@line 2550 "lbfgsb.c"
    $622 = $621 + $620; //@line 2550 "lbfgsb.c"
    HEAPF64[384>>3] = $622; //@line 2550 "lbfgsb.c"
    $623 = HEAP32[1688>>2]|0; //@line 2548 "lbfgsb.c"
    $624 = (($623) + 1)|0; //@line 2548 "lbfgsb.c"
    HEAP32[1688>>2] = $624; //@line 2548 "lbfgsb.c"
   }
   $625 = HEAP32[1700>>2]|0; //@line 2553 "lbfgsb.c"
   $626 = HEAP32[1712>>2]|0; //@line 2553 "lbfgsb.c"
   $627 = $11; //@line 2553 "lbfgsb.c"
   $628 = HEAP32[$627>>2]|0; //@line 2553 "lbfgsb.c"
   $629 = (($626) + ($628))|0; //@line 2553 "lbfgsb.c"
   $630 = ($625|0)<=($629|0); //@line 2553 "lbfgsb.c"
   $631 = HEAP32[1700>>2]|0; //@line 2554 "lbfgsb.c"
   $632 = HEAP32[1712>>2]|0; //@line 2554 "lbfgsb.c"
   $633 = $wn1_dim1; //@line 2554 "lbfgsb.c"
   $634 = Math_imul($632, $633)|0; //@line 2554 "lbfgsb.c"
   $635 = (($631) + ($634))|0; //@line 2554 "lbfgsb.c"
   $636 = $10; //@line 2554 "lbfgsb.c"
   $637 = (($636) + ($635<<3)|0); //@line 2554 "lbfgsb.c"
   $638 = +HEAPF64[$637>>3]; //@line 2554 "lbfgsb.c"
   $639 = +HEAPF64[368>>3]; //@line 2554 "lbfgsb.c"
   if ($630) {
    $640 = $638 + $639; //@line 2554 "lbfgsb.c"
    $641 = +HEAPF64[384>>3]; //@line 2555 "lbfgsb.c"
    $642 = $640 - $641; //@line 2554 "lbfgsb.c"
    $643 = HEAP32[1700>>2]|0; //@line 2554 "lbfgsb.c"
    $644 = HEAP32[1712>>2]|0; //@line 2554 "lbfgsb.c"
    $645 = $wn1_dim1; //@line 2554 "lbfgsb.c"
    $646 = Math_imul($644, $645)|0; //@line 2554 "lbfgsb.c"
    $647 = (($643) + ($646))|0; //@line 2554 "lbfgsb.c"
    $648 = $10; //@line 2554 "lbfgsb.c"
    $649 = (($648) + ($647<<3)|0); //@line 2554 "lbfgsb.c"
    HEAPF64[$649>>3] = $642; //@line 2554 "lbfgsb.c"
   } else {
    $650 = $638 - $639; //@line 2557 "lbfgsb.c"
    $651 = +HEAPF64[384>>3]; //@line 2558 "lbfgsb.c"
    $652 = $650 + $651; //@line 2557 "lbfgsb.c"
    $653 = HEAP32[1700>>2]|0; //@line 2557 "lbfgsb.c"
    $654 = HEAP32[1712>>2]|0; //@line 2557 "lbfgsb.c"
    $655 = $wn1_dim1; //@line 2557 "lbfgsb.c"
    $656 = Math_imul($654, $655)|0; //@line 2557 "lbfgsb.c"
    $657 = (($653) + ($656))|0; //@line 2557 "lbfgsb.c"
    $658 = $10; //@line 2557 "lbfgsb.c"
    $659 = (($658) + ($657<<3)|0); //@line 2557 "lbfgsb.c"
    HEAPF64[$659>>3] = $652; //@line 2557 "lbfgsb.c"
   }
   $660 = HEAP32[1744>>2]|0; //@line 2560 "lbfgsb.c"
   $661 = $11; //@line 2560 "lbfgsb.c"
   $662 = HEAP32[$661>>2]|0; //@line 2560 "lbfgsb.c"
   $663 = (($660|0) % ($662|0))&-1; //@line 2560 "lbfgsb.c"
   $664 = (($663) + 1)|0; //@line 2560 "lbfgsb.c"
   HEAP32[1744>>2] = $664; //@line 2560 "lbfgsb.c"
   $665 = HEAP32[1712>>2]|0; //@line 2538 "lbfgsb.c"
   $666 = (($665) + 1)|0; //@line 2538 "lbfgsb.c"
   HEAP32[1712>>2] = $666; //@line 2538 "lbfgsb.c"
  }
  $667 = HEAP32[1740>>2]|0; //@line 2563 "lbfgsb.c"
  $668 = $11; //@line 2563 "lbfgsb.c"
  $669 = HEAP32[$668>>2]|0; //@line 2563 "lbfgsb.c"
  $670 = (($667|0) % ($669|0))&-1; //@line 2563 "lbfgsb.c"
  $671 = (($670) + 1)|0; //@line 2563 "lbfgsb.c"
  HEAP32[1740>>2] = $671; //@line 2563 "lbfgsb.c"
  $672 = HEAP32[1700>>2]|0; //@line 2535 "lbfgsb.c"
  $673 = (($672) + 1)|0; //@line 2535 "lbfgsb.c"
  HEAP32[1700>>2] = $673; //@line 2535 "lbfgsb.c"
 }
 $674 = $11; //@line 2568 "lbfgsb.c"
 $675 = HEAP32[$674>>2]|0; //@line 2568 "lbfgsb.c"
 $676 = $675 << 1; //@line 2568 "lbfgsb.c"
 HEAP32[1696>>2] = $676; //@line 2568 "lbfgsb.c"
 $677 = $16; //@line 2569 "lbfgsb.c"
 $678 = HEAP32[$677>>2]|0; //@line 2569 "lbfgsb.c"
 $i__1 = $678; //@line 2569 "lbfgsb.c"
 HEAP32[1708>>2] = 1; //@line 2570 "lbfgsb.c"
 while(1) {
  $679 = HEAP32[1708>>2]|0; //@line 2570 "lbfgsb.c"
  $680 = $i__1; //@line 2570 "lbfgsb.c"
  $681 = ($679|0)<=($680|0); //@line 2570 "lbfgsb.c"
  if (!($681)) {
   break;
  }
  $682 = $16; //@line 2571 "lbfgsb.c"
  $683 = HEAP32[$682>>2]|0; //@line 2571 "lbfgsb.c"
  $684 = HEAP32[1708>>2]|0; //@line 2571 "lbfgsb.c"
  $685 = (($683) + ($684))|0; //@line 2571 "lbfgsb.c"
  HEAP32[1700>>2] = $685; //@line 2571 "lbfgsb.c"
  $686 = $11; //@line 2572 "lbfgsb.c"
  $687 = HEAP32[$686>>2]|0; //@line 2572 "lbfgsb.c"
  $688 = HEAP32[1708>>2]|0; //@line 2572 "lbfgsb.c"
  $689 = (($687) + ($688))|0; //@line 2572 "lbfgsb.c"
  HEAP32[1716>>2] = $689; //@line 2572 "lbfgsb.c"
  $690 = HEAP32[1708>>2]|0; //@line 2573 "lbfgsb.c"
  HEAP32[$i__2>>2] = $690; //@line 2573 "lbfgsb.c"
  HEAP32[1712>>2] = 1; //@line 2574 "lbfgsb.c"
  while(1) {
   $691 = HEAP32[1712>>2]|0; //@line 2574 "lbfgsb.c"
   $692 = HEAP32[$i__2>>2]|0; //@line 2574 "lbfgsb.c"
   $693 = ($691|0)<=($692|0); //@line 2574 "lbfgsb.c"
   if (!($693)) {
    break;
   }
   $694 = $16; //@line 2575 "lbfgsb.c"
   $695 = HEAP32[$694>>2]|0; //@line 2575 "lbfgsb.c"
   $696 = HEAP32[1712>>2]|0; //@line 2575 "lbfgsb.c"
   $697 = (($695) + ($696))|0; //@line 2575 "lbfgsb.c"
   HEAP32[1704>>2] = $697; //@line 2575 "lbfgsb.c"
   $698 = $11; //@line 2576 "lbfgsb.c"
   $699 = HEAP32[$698>>2]|0; //@line 2576 "lbfgsb.c"
   $700 = HEAP32[1712>>2]|0; //@line 2576 "lbfgsb.c"
   $701 = (($699) + ($700))|0; //@line 2576 "lbfgsb.c"
   HEAP32[1720>>2] = $701; //@line 2576 "lbfgsb.c"
   $702 = HEAP32[1708>>2]|0; //@line 2577 "lbfgsb.c"
   $703 = HEAP32[1712>>2]|0; //@line 2577 "lbfgsb.c"
   $704 = $wn1_dim1; //@line 2577 "lbfgsb.c"
   $705 = Math_imul($703, $704)|0; //@line 2577 "lbfgsb.c"
   $706 = (($702) + ($705))|0; //@line 2577 "lbfgsb.c"
   $707 = $10; //@line 2577 "lbfgsb.c"
   $708 = (($707) + ($706<<3)|0); //@line 2577 "lbfgsb.c"
   $709 = +HEAPF64[$708>>3]; //@line 2577 "lbfgsb.c"
   $710 = $15; //@line 2577 "lbfgsb.c"
   $711 = +HEAPF64[$710>>3]; //@line 2577 "lbfgsb.c"
   $712 = $709 / $711; //@line 2577 "lbfgsb.c"
   $713 = HEAP32[1712>>2]|0; //@line 2577 "lbfgsb.c"
   $714 = HEAP32[1708>>2]|0; //@line 2577 "lbfgsb.c"
   $715 = $wn_dim1; //@line 2577 "lbfgsb.c"
   $716 = Math_imul($714, $715)|0; //@line 2577 "lbfgsb.c"
   $717 = (($713) + ($716))|0; //@line 2577 "lbfgsb.c"
   $718 = $9; //@line 2577 "lbfgsb.c"
   $719 = (($718) + ($717<<3)|0); //@line 2577 "lbfgsb.c"
   HEAPF64[$719>>3] = $712; //@line 2577 "lbfgsb.c"
   $720 = HEAP32[1716>>2]|0; //@line 2578 "lbfgsb.c"
   $721 = HEAP32[1720>>2]|0; //@line 2578 "lbfgsb.c"
   $722 = $wn1_dim1; //@line 2578 "lbfgsb.c"
   $723 = Math_imul($721, $722)|0; //@line 2578 "lbfgsb.c"
   $724 = (($720) + ($723))|0; //@line 2578 "lbfgsb.c"
   $725 = $10; //@line 2578 "lbfgsb.c"
   $726 = (($725) + ($724<<3)|0); //@line 2578 "lbfgsb.c"
   $727 = +HEAPF64[$726>>3]; //@line 2578 "lbfgsb.c"
   $728 = $15; //@line 2578 "lbfgsb.c"
   $729 = +HEAPF64[$728>>3]; //@line 2578 "lbfgsb.c"
   $730 = $727 * $729; //@line 2578 "lbfgsb.c"
   $731 = HEAP32[1704>>2]|0; //@line 2578 "lbfgsb.c"
   $732 = HEAP32[1700>>2]|0; //@line 2578 "lbfgsb.c"
   $733 = $wn_dim1; //@line 2578 "lbfgsb.c"
   $734 = Math_imul($732, $733)|0; //@line 2578 "lbfgsb.c"
   $735 = (($731) + ($734))|0; //@line 2578 "lbfgsb.c"
   $736 = $9; //@line 2578 "lbfgsb.c"
   $737 = (($736) + ($735<<3)|0); //@line 2578 "lbfgsb.c"
   HEAPF64[$737>>3] = $730; //@line 2578 "lbfgsb.c"
   $738 = HEAP32[1712>>2]|0; //@line 2574 "lbfgsb.c"
   $739 = (($738) + 1)|0; //@line 2574 "lbfgsb.c"
   HEAP32[1712>>2] = $739; //@line 2574 "lbfgsb.c"
  }
  $740 = HEAP32[1708>>2]|0; //@line 2581 "lbfgsb.c"
  $741 = (($740) - 1)|0; //@line 2581 "lbfgsb.c"
  HEAP32[$i__2>>2] = $741; //@line 2581 "lbfgsb.c"
  HEAP32[1712>>2] = 1; //@line 2582 "lbfgsb.c"
  while(1) {
   $742 = HEAP32[1712>>2]|0; //@line 2582 "lbfgsb.c"
   $743 = HEAP32[$i__2>>2]|0; //@line 2582 "lbfgsb.c"
   $744 = ($742|0)<=($743|0); //@line 2582 "lbfgsb.c"
   if (!($744)) {
    break;
   }
   $745 = HEAP32[1716>>2]|0; //@line 2583 "lbfgsb.c"
   $746 = HEAP32[1712>>2]|0; //@line 2583 "lbfgsb.c"
   $747 = $wn1_dim1; //@line 2583 "lbfgsb.c"
   $748 = Math_imul($746, $747)|0; //@line 2583 "lbfgsb.c"
   $749 = (($745) + ($748))|0; //@line 2583 "lbfgsb.c"
   $750 = $10; //@line 2583 "lbfgsb.c"
   $751 = (($750) + ($749<<3)|0); //@line 2583 "lbfgsb.c"
   $752 = +HEAPF64[$751>>3]; //@line 2583 "lbfgsb.c"
   $753 = -$752; //@line 2583 "lbfgsb.c"
   $754 = HEAP32[1712>>2]|0; //@line 2583 "lbfgsb.c"
   $755 = HEAP32[1700>>2]|0; //@line 2583 "lbfgsb.c"
   $756 = $wn_dim1; //@line 2583 "lbfgsb.c"
   $757 = Math_imul($755, $756)|0; //@line 2583 "lbfgsb.c"
   $758 = (($754) + ($757))|0; //@line 2583 "lbfgsb.c"
   $759 = $9; //@line 2583 "lbfgsb.c"
   $760 = (($759) + ($758<<3)|0); //@line 2583 "lbfgsb.c"
   HEAPF64[$760>>3] = $753; //@line 2583 "lbfgsb.c"
   $761 = HEAP32[1712>>2]|0; //@line 2582 "lbfgsb.c"
   $762 = (($761) + 1)|0; //@line 2582 "lbfgsb.c"
   HEAP32[1712>>2] = $762; //@line 2582 "lbfgsb.c"
  }
  $763 = $16; //@line 2586 "lbfgsb.c"
  $764 = HEAP32[$763>>2]|0; //@line 2586 "lbfgsb.c"
  HEAP32[$i__2>>2] = $764; //@line 2586 "lbfgsb.c"
  $765 = HEAP32[1708>>2]|0; //@line 2587 "lbfgsb.c"
  HEAP32[1712>>2] = $765; //@line 2587 "lbfgsb.c"
  while(1) {
   $766 = HEAP32[1712>>2]|0; //@line 2587 "lbfgsb.c"
   $767 = HEAP32[$i__2>>2]|0; //@line 2587 "lbfgsb.c"
   $768 = ($766|0)<=($767|0); //@line 2587 "lbfgsb.c"
   if (!($768)) {
    break;
   }
   $769 = HEAP32[1716>>2]|0; //@line 2588 "lbfgsb.c"
   $770 = HEAP32[1712>>2]|0; //@line 2588 "lbfgsb.c"
   $771 = $wn1_dim1; //@line 2588 "lbfgsb.c"
   $772 = Math_imul($770, $771)|0; //@line 2588 "lbfgsb.c"
   $773 = (($769) + ($772))|0; //@line 2588 "lbfgsb.c"
   $774 = $10; //@line 2588 "lbfgsb.c"
   $775 = (($774) + ($773<<3)|0); //@line 2588 "lbfgsb.c"
   $776 = +HEAPF64[$775>>3]; //@line 2588 "lbfgsb.c"
   $777 = HEAP32[1712>>2]|0; //@line 2588 "lbfgsb.c"
   $778 = HEAP32[1700>>2]|0; //@line 2588 "lbfgsb.c"
   $779 = $wn_dim1; //@line 2588 "lbfgsb.c"
   $780 = Math_imul($778, $779)|0; //@line 2588 "lbfgsb.c"
   $781 = (($777) + ($780))|0; //@line 2588 "lbfgsb.c"
   $782 = $9; //@line 2588 "lbfgsb.c"
   $783 = (($782) + ($781<<3)|0); //@line 2588 "lbfgsb.c"
   HEAPF64[$783>>3] = $776; //@line 2588 "lbfgsb.c"
   $784 = HEAP32[1712>>2]|0; //@line 2587 "lbfgsb.c"
   $785 = (($784) + 1)|0; //@line 2587 "lbfgsb.c"
   HEAP32[1712>>2] = $785; //@line 2587 "lbfgsb.c"
  }
  $786 = HEAP32[1708>>2]|0; //@line 2591 "lbfgsb.c"
  $787 = HEAP32[1708>>2]|0; //@line 2591 "lbfgsb.c"
  $788 = $sy_dim1; //@line 2591 "lbfgsb.c"
  $789 = Math_imul($787, $788)|0; //@line 2591 "lbfgsb.c"
  $790 = (($786) + ($789))|0; //@line 2591 "lbfgsb.c"
  $791 = $14; //@line 2591 "lbfgsb.c"
  $792 = (($791) + ($790<<3)|0); //@line 2591 "lbfgsb.c"
  $793 = +HEAPF64[$792>>3]; //@line 2591 "lbfgsb.c"
  $794 = HEAP32[1708>>2]|0; //@line 2591 "lbfgsb.c"
  $795 = HEAP32[1708>>2]|0; //@line 2591 "lbfgsb.c"
  $796 = $wn_dim1; //@line 2591 "lbfgsb.c"
  $797 = Math_imul($795, $796)|0; //@line 2591 "lbfgsb.c"
  $798 = (($794) + ($797))|0; //@line 2591 "lbfgsb.c"
  $799 = $9; //@line 2591 "lbfgsb.c"
  $800 = (($799) + ($798<<3)|0); //@line 2591 "lbfgsb.c"
  $801 = +HEAPF64[$800>>3]; //@line 2591 "lbfgsb.c"
  $802 = $801 + $793; //@line 2591 "lbfgsb.c"
  HEAPF64[$800>>3] = $802; //@line 2591 "lbfgsb.c"
  $803 = HEAP32[1708>>2]|0; //@line 2570 "lbfgsb.c"
  $804 = (($803) + 1)|0; //@line 2570 "lbfgsb.c"
  HEAP32[1708>>2] = $804; //@line 2570 "lbfgsb.c"
 }
 $805 = $wn_offset; //@line 2598 "lbfgsb.c"
 $806 = $9; //@line 2598 "lbfgsb.c"
 $807 = (($806) + ($805<<3)|0); //@line 2598 "lbfgsb.c"
 $808 = $16; //@line 2598 "lbfgsb.c"
 $809 = $18; //@line 2598 "lbfgsb.c"
 (_dpofa_($807,1696,$808,$809)|0); //@line 2598 "lbfgsb.c"
 $810 = $18; //@line 2599 "lbfgsb.c"
 $811 = HEAP32[$810>>2]|0; //@line 2599 "lbfgsb.c"
 $812 = ($811|0)!=(0); //@line 2599 "lbfgsb.c"
 if ($812) {
  $813 = $18; //@line 2600 "lbfgsb.c"
  HEAP32[$813>>2] = -1; //@line 2600 "lbfgsb.c"
  $0 = 0; //@line 2601 "lbfgsb.c"
  $893 = $0; //@line 2629 "lbfgsb.c"
  STACKTOP = sp;return ($893|0); //@line 2629 "lbfgsb.c"
 }
 $814 = $16; //@line 2604 "lbfgsb.c"
 $815 = HEAP32[$814>>2]|0; //@line 2604 "lbfgsb.c"
 $816 = $815 << 1; //@line 2604 "lbfgsb.c"
 HEAP32[1724>>2] = $816; //@line 2604 "lbfgsb.c"
 $817 = HEAP32[1724>>2]|0; //@line 2605 "lbfgsb.c"
 $i__1 = $817; //@line 2605 "lbfgsb.c"
 $818 = $16; //@line 2606 "lbfgsb.c"
 $819 = HEAP32[$818>>2]|0; //@line 2606 "lbfgsb.c"
 $820 = (($819) + 1)|0; //@line 2606 "lbfgsb.c"
 HEAP32[1704>>2] = $820; //@line 2606 "lbfgsb.c"
 while(1) {
  $821 = HEAP32[1704>>2]|0; //@line 2606 "lbfgsb.c"
  $822 = $i__1; //@line 2606 "lbfgsb.c"
  $823 = ($821|0)<=($822|0); //@line 2606 "lbfgsb.c"
  if (!($823)) {
   break;
  }
  $824 = $wn_offset; //@line 2607 "lbfgsb.c"
  $825 = $9; //@line 2607 "lbfgsb.c"
  $826 = (($825) + ($824<<3)|0); //@line 2607 "lbfgsb.c"
  $827 = $16; //@line 2607 "lbfgsb.c"
  $828 = HEAP32[1704>>2]|0; //@line 2607 "lbfgsb.c"
  $829 = $wn_dim1; //@line 2607 "lbfgsb.c"
  $830 = Math_imul($828, $829)|0; //@line 2607 "lbfgsb.c"
  $831 = (($830) + 1)|0; //@line 2607 "lbfgsb.c"
  $832 = $9; //@line 2607 "lbfgsb.c"
  $833 = (($832) + ($831<<3)|0); //@line 2607 "lbfgsb.c"
  $834 = $18; //@line 2607 "lbfgsb.c"
  (_dtrsl_($826,1696,$827,$833,1324,$834)|0); //@line 2607 "lbfgsb.c"
  $835 = HEAP32[1704>>2]|0; //@line 2606 "lbfgsb.c"
  $836 = (($835) + 1)|0; //@line 2606 "lbfgsb.c"
  HEAP32[1704>>2] = $836; //@line 2606 "lbfgsb.c"
 }
 $837 = HEAP32[1724>>2]|0; //@line 2612 "lbfgsb.c"
 $i__1 = $837; //@line 2612 "lbfgsb.c"
 $838 = $16; //@line 2613 "lbfgsb.c"
 $839 = HEAP32[$838>>2]|0; //@line 2613 "lbfgsb.c"
 $840 = (($839) + 1)|0; //@line 2613 "lbfgsb.c"
 HEAP32[1700>>2] = $840; //@line 2613 "lbfgsb.c"
 while(1) {
  $841 = HEAP32[1700>>2]|0; //@line 2613 "lbfgsb.c"
  $842 = $i__1; //@line 2613 "lbfgsb.c"
  $843 = ($841|0)<=($842|0); //@line 2613 "lbfgsb.c"
  if (!($843)) {
   break;
  }
  $844 = HEAP32[1724>>2]|0; //@line 2614 "lbfgsb.c"
  HEAP32[$i__2>>2] = $844; //@line 2614 "lbfgsb.c"
  $845 = HEAP32[1700>>2]|0; //@line 2615 "lbfgsb.c"
  HEAP32[1704>>2] = $845; //@line 2615 "lbfgsb.c"
  while(1) {
   $846 = HEAP32[1704>>2]|0; //@line 2615 "lbfgsb.c"
   $847 = HEAP32[$i__2>>2]|0; //@line 2615 "lbfgsb.c"
   $848 = ($846|0)<=($847|0); //@line 2615 "lbfgsb.c"
   if (!($848)) {
    break;
   }
   $849 = $16; //@line 2616 "lbfgsb.c"
   $850 = HEAP32[1700>>2]|0; //@line 2616 "lbfgsb.c"
   $851 = $wn_dim1; //@line 2616 "lbfgsb.c"
   $852 = Math_imul($850, $851)|0; //@line 2616 "lbfgsb.c"
   $853 = (($852) + 1)|0; //@line 2616 "lbfgsb.c"
   $854 = $9; //@line 2616 "lbfgsb.c"
   $855 = (($854) + ($853<<3)|0); //@line 2616 "lbfgsb.c"
   $856 = HEAP32[1704>>2]|0; //@line 2617 "lbfgsb.c"
   $857 = $wn_dim1; //@line 2617 "lbfgsb.c"
   $858 = Math_imul($856, $857)|0; //@line 2617 "lbfgsb.c"
   $859 = (($858) + 1)|0; //@line 2617 "lbfgsb.c"
   $860 = $9; //@line 2617 "lbfgsb.c"
   $861 = (($860) + ($859<<3)|0); //@line 2617 "lbfgsb.c"
   $862 = (+_ddot_($849,$855,1236,$861,1236)); //@line 2616 "lbfgsb.c"
   $863 = HEAP32[1700>>2]|0; //@line 2616 "lbfgsb.c"
   $864 = HEAP32[1704>>2]|0; //@line 2616 "lbfgsb.c"
   $865 = $wn_dim1; //@line 2616 "lbfgsb.c"
   $866 = Math_imul($864, $865)|0; //@line 2616 "lbfgsb.c"
   $867 = (($863) + ($866))|0; //@line 2616 "lbfgsb.c"
   $868 = $9; //@line 2616 "lbfgsb.c"
   $869 = (($868) + ($867<<3)|0); //@line 2616 "lbfgsb.c"
   $870 = +HEAPF64[$869>>3]; //@line 2616 "lbfgsb.c"
   $871 = $870 + $862; //@line 2616 "lbfgsb.c"
   HEAPF64[$869>>3] = $871; //@line 2616 "lbfgsb.c"
   $872 = HEAP32[1704>>2]|0; //@line 2615 "lbfgsb.c"
   $873 = (($872) + 1)|0; //@line 2615 "lbfgsb.c"
   HEAP32[1704>>2] = $873; //@line 2615 "lbfgsb.c"
  }
  $874 = HEAP32[1700>>2]|0; //@line 2613 "lbfgsb.c"
  $875 = (($874) + 1)|0; //@line 2613 "lbfgsb.c"
  HEAP32[1700>>2] = $875; //@line 2613 "lbfgsb.c"
 }
 $876 = $16; //@line 2623 "lbfgsb.c"
 $877 = HEAP32[$876>>2]|0; //@line 2623 "lbfgsb.c"
 $878 = (($877) + 1)|0; //@line 2623 "lbfgsb.c"
 $879 = $16; //@line 2623 "lbfgsb.c"
 $880 = HEAP32[$879>>2]|0; //@line 2623 "lbfgsb.c"
 $881 = (($880) + 1)|0; //@line 2623 "lbfgsb.c"
 $882 = $wn_dim1; //@line 2623 "lbfgsb.c"
 $883 = Math_imul($881, $882)|0; //@line 2623 "lbfgsb.c"
 $884 = (($878) + ($883))|0; //@line 2623 "lbfgsb.c"
 $885 = $9; //@line 2623 "lbfgsb.c"
 $886 = (($885) + ($884<<3)|0); //@line 2623 "lbfgsb.c"
 $887 = $16; //@line 2623 "lbfgsb.c"
 $888 = $18; //@line 2623 "lbfgsb.c"
 (_dpofa_($886,1696,$887,$888)|0); //@line 2623 "lbfgsb.c"
 $889 = $18; //@line 2624 "lbfgsb.c"
 $890 = HEAP32[$889>>2]|0; //@line 2624 "lbfgsb.c"
 $891 = ($890|0)!=(0); //@line 2624 "lbfgsb.c"
 if ($891) {
  $892 = $18; //@line 2625 "lbfgsb.c"
  HEAP32[$892>>2] = -2; //@line 2625 "lbfgsb.c"
  $0 = 0; //@line 2626 "lbfgsb.c"
  $893 = $0; //@line 2629 "lbfgsb.c"
  STACKTOP = sp;return ($893|0); //@line 2629 "lbfgsb.c"
 } else {
  $0 = 0; //@line 2628 "lbfgsb.c"
  $893 = $0; //@line 2629 "lbfgsb.c"
  STACKTOP = sp;return ($893|0); //@line 2629 "lbfgsb.c"
 }
 return (0)|0;
}
function _cmprlb_($n,$m,$x,$g,$ws,$wy,$sy,$wt,$z__,$r__,$wa,$index,$theta,$col,$head,$nfree,$cnstnd,$info) {
 $n = $n|0;
 $m = $m|0;
 $x = $x|0;
 $g = $g|0;
 $ws = $ws|0;
 $wy = $wy|0;
 $sy = $sy|0;
 $wt = $wt|0;
 $z__ = $z__|0;
 $r__ = $r__|0;
 $wa = $wa|0;
 $index = $index|0;
 $theta = $theta|0;
 $col = $col|0;
 $head = $head|0;
 $nfree = $nfree|0;
 $cnstnd = $cnstnd|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0.0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0.0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0.0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0.0, $i__1 = 0, $i__2 = 0, $sy_dim1 = 0, $sy_offset = 0, $ws_dim1 = 0, $ws_offset = 0, $wt_dim1 = 0, $wt_offset = 0, $wy_dim1 = 0, $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $m;
 $3 = $x;
 $4 = $g;
 $5 = $ws;
 $6 = $wy;
 $7 = $sy;
 $8 = $wt;
 $9 = $z__;
 $10 = $r__;
 $11 = $wa;
 $12 = $index;
 $13 = $theta;
 $14 = $col;
 $15 = $head;
 $16 = $nfree;
 $17 = $cnstnd;
 $18 = $info;
 $19 = $12; //@line 2114 "lbfgsb.c"
 $20 = ((($19)) + -4|0); //@line 2114 "lbfgsb.c"
 $12 = $20; //@line 2114 "lbfgsb.c"
 $21 = $10; //@line 2115 "lbfgsb.c"
 $22 = ((($21)) + -8|0); //@line 2115 "lbfgsb.c"
 $10 = $22; //@line 2115 "lbfgsb.c"
 $23 = $9; //@line 2116 "lbfgsb.c"
 $24 = ((($23)) + -8|0); //@line 2116 "lbfgsb.c"
 $9 = $24; //@line 2116 "lbfgsb.c"
 $25 = $4; //@line 2117 "lbfgsb.c"
 $26 = ((($25)) + -8|0); //@line 2117 "lbfgsb.c"
 $4 = $26; //@line 2117 "lbfgsb.c"
 $27 = $3; //@line 2118 "lbfgsb.c"
 $28 = ((($27)) + -8|0); //@line 2118 "lbfgsb.c"
 $3 = $28; //@line 2118 "lbfgsb.c"
 $29 = $11; //@line 2119 "lbfgsb.c"
 $30 = ((($29)) + -8|0); //@line 2119 "lbfgsb.c"
 $11 = $30; //@line 2119 "lbfgsb.c"
 $31 = $2; //@line 2120 "lbfgsb.c"
 $32 = HEAP32[$31>>2]|0; //@line 2120 "lbfgsb.c"
 $wt_dim1 = $32; //@line 2120 "lbfgsb.c"
 $33 = $wt_dim1; //@line 2121 "lbfgsb.c"
 $34 = (1 + ($33))|0; //@line 2121 "lbfgsb.c"
 $wt_offset = $34; //@line 2121 "lbfgsb.c"
 $35 = $wt_offset; //@line 2122 "lbfgsb.c"
 $36 = $8; //@line 2122 "lbfgsb.c"
 $37 = (0 - ($35))|0; //@line 2122 "lbfgsb.c"
 $38 = (($36) + ($37<<3)|0); //@line 2122 "lbfgsb.c"
 $8 = $38; //@line 2122 "lbfgsb.c"
 $39 = $2; //@line 2123 "lbfgsb.c"
 $40 = HEAP32[$39>>2]|0; //@line 2123 "lbfgsb.c"
 $sy_dim1 = $40; //@line 2123 "lbfgsb.c"
 $41 = $sy_dim1; //@line 2124 "lbfgsb.c"
 $42 = (1 + ($41))|0; //@line 2124 "lbfgsb.c"
 $sy_offset = $42; //@line 2124 "lbfgsb.c"
 $43 = $sy_offset; //@line 2125 "lbfgsb.c"
 $44 = $7; //@line 2125 "lbfgsb.c"
 $45 = (0 - ($43))|0; //@line 2125 "lbfgsb.c"
 $46 = (($44) + ($45<<3)|0); //@line 2125 "lbfgsb.c"
 $7 = $46; //@line 2125 "lbfgsb.c"
 $47 = $1; //@line 2126 "lbfgsb.c"
 $48 = HEAP32[$47>>2]|0; //@line 2126 "lbfgsb.c"
 $wy_dim1 = $48; //@line 2126 "lbfgsb.c"
 $49 = $wy_dim1; //@line 2127 "lbfgsb.c"
 $50 = (1 + ($49))|0; //@line 2127 "lbfgsb.c"
 $wy_offset = $50; //@line 2127 "lbfgsb.c"
 $51 = $wy_offset; //@line 2128 "lbfgsb.c"
 $52 = $6; //@line 2128 "lbfgsb.c"
 $53 = (0 - ($51))|0; //@line 2128 "lbfgsb.c"
 $54 = (($52) + ($53<<3)|0); //@line 2128 "lbfgsb.c"
 $6 = $54; //@line 2128 "lbfgsb.c"
 $55 = $1; //@line 2129 "lbfgsb.c"
 $56 = HEAP32[$55>>2]|0; //@line 2129 "lbfgsb.c"
 $ws_dim1 = $56; //@line 2129 "lbfgsb.c"
 $57 = $ws_dim1; //@line 2130 "lbfgsb.c"
 $58 = (1 + ($57))|0; //@line 2130 "lbfgsb.c"
 $ws_offset = $58; //@line 2130 "lbfgsb.c"
 $59 = $ws_offset; //@line 2131 "lbfgsb.c"
 $60 = $5; //@line 2131 "lbfgsb.c"
 $61 = (0 - ($59))|0; //@line 2131 "lbfgsb.c"
 $62 = (($60) + ($61<<3)|0); //@line 2131 "lbfgsb.c"
 $5 = $62; //@line 2131 "lbfgsb.c"
 $63 = $17; //@line 2134 "lbfgsb.c"
 $64 = HEAP32[$63>>2]|0; //@line 2134 "lbfgsb.c"
 $65 = ($64|0)!=(0); //@line 2134 "lbfgsb.c"
 L1: do {
  if ($65) {
   label = 6;
  } else {
   $66 = $14; //@line 2134 "lbfgsb.c"
   $67 = HEAP32[$66>>2]|0; //@line 2134 "lbfgsb.c"
   $68 = ($67|0)>(0); //@line 2134 "lbfgsb.c"
   if ($68) {
    $69 = $1; //@line 2135 "lbfgsb.c"
    $70 = HEAP32[$69>>2]|0; //@line 2135 "lbfgsb.c"
    $i__1 = $70; //@line 2135 "lbfgsb.c"
    HEAP32[1664>>2] = 1; //@line 2136 "lbfgsb.c"
    while(1) {
     $71 = HEAP32[1664>>2]|0; //@line 2136 "lbfgsb.c"
     $72 = $i__1; //@line 2136 "lbfgsb.c"
     $73 = ($71|0)<=($72|0); //@line 2136 "lbfgsb.c"
     if (!($73)) {
      break L1;
     }
     $74 = HEAP32[1664>>2]|0; //@line 2137 "lbfgsb.c"
     $75 = $4; //@line 2137 "lbfgsb.c"
     $76 = (($75) + ($74<<3)|0); //@line 2137 "lbfgsb.c"
     $77 = +HEAPF64[$76>>3]; //@line 2137 "lbfgsb.c"
     $78 = -$77; //@line 2137 "lbfgsb.c"
     $79 = HEAP32[1664>>2]|0; //@line 2137 "lbfgsb.c"
     $80 = $10; //@line 2137 "lbfgsb.c"
     $81 = (($80) + ($79<<3)|0); //@line 2137 "lbfgsb.c"
     HEAPF64[$81>>3] = $78; //@line 2137 "lbfgsb.c"
     $82 = HEAP32[1664>>2]|0; //@line 2136 "lbfgsb.c"
     $83 = (($82) + 1)|0; //@line 2136 "lbfgsb.c"
     HEAP32[1664>>2] = $83; //@line 2136 "lbfgsb.c"
    }
   } else {
    label = 6;
   }
  }
 } while(0);
 L7: do {
  if ((label|0) == 6) {
   $84 = $16; //@line 2141 "lbfgsb.c"
   $85 = HEAP32[$84>>2]|0; //@line 2141 "lbfgsb.c"
   $i__1 = $85; //@line 2141 "lbfgsb.c"
   HEAP32[1664>>2] = 1; //@line 2142 "lbfgsb.c"
   while(1) {
    $86 = HEAP32[1664>>2]|0; //@line 2142 "lbfgsb.c"
    $87 = $i__1; //@line 2142 "lbfgsb.c"
    $88 = ($86|0)<=($87|0); //@line 2142 "lbfgsb.c"
    if (!($88)) {
     break;
    }
    $89 = HEAP32[1664>>2]|0; //@line 2143 "lbfgsb.c"
    $90 = $12; //@line 2143 "lbfgsb.c"
    $91 = (($90) + ($89<<2)|0); //@line 2143 "lbfgsb.c"
    $92 = HEAP32[$91>>2]|0; //@line 2143 "lbfgsb.c"
    HEAP32[1672>>2] = $92; //@line 2143 "lbfgsb.c"
    $93 = $13; //@line 2144 "lbfgsb.c"
    $94 = +HEAPF64[$93>>3]; //@line 2144 "lbfgsb.c"
    $95 = -$94; //@line 2144 "lbfgsb.c"
    $96 = HEAP32[1672>>2]|0; //@line 2144 "lbfgsb.c"
    $97 = $9; //@line 2144 "lbfgsb.c"
    $98 = (($97) + ($96<<3)|0); //@line 2144 "lbfgsb.c"
    $99 = +HEAPF64[$98>>3]; //@line 2144 "lbfgsb.c"
    $100 = HEAP32[1672>>2]|0; //@line 2144 "lbfgsb.c"
    $101 = $3; //@line 2144 "lbfgsb.c"
    $102 = (($101) + ($100<<3)|0); //@line 2144 "lbfgsb.c"
    $103 = +HEAPF64[$102>>3]; //@line 2144 "lbfgsb.c"
    $104 = $99 - $103; //@line 2144 "lbfgsb.c"
    $105 = $95 * $104; //@line 2144 "lbfgsb.c"
    $106 = HEAP32[1672>>2]|0; //@line 2144 "lbfgsb.c"
    $107 = $4; //@line 2144 "lbfgsb.c"
    $108 = (($107) + ($106<<3)|0); //@line 2144 "lbfgsb.c"
    $109 = +HEAPF64[$108>>3]; //@line 2144 "lbfgsb.c"
    $110 = $105 - $109; //@line 2144 "lbfgsb.c"
    $111 = HEAP32[1664>>2]|0; //@line 2144 "lbfgsb.c"
    $112 = $10; //@line 2144 "lbfgsb.c"
    $113 = (($112) + ($111<<3)|0); //@line 2144 "lbfgsb.c"
    HEAPF64[$113>>3] = $110; //@line 2144 "lbfgsb.c"
    $114 = HEAP32[1664>>2]|0; //@line 2142 "lbfgsb.c"
    $115 = (($114) + 1)|0; //@line 2142 "lbfgsb.c"
    HEAP32[1664>>2] = $115; //@line 2142 "lbfgsb.c"
   }
   $116 = $2; //@line 2147 "lbfgsb.c"
   $117 = $sy_offset; //@line 2147 "lbfgsb.c"
   $118 = $7; //@line 2147 "lbfgsb.c"
   $119 = (($118) + ($117<<3)|0); //@line 2147 "lbfgsb.c"
   $120 = $wt_offset; //@line 2147 "lbfgsb.c"
   $121 = $8; //@line 2147 "lbfgsb.c"
   $122 = (($121) + ($120<<3)|0); //@line 2147 "lbfgsb.c"
   $123 = $14; //@line 2147 "lbfgsb.c"
   $124 = $2; //@line 2147 "lbfgsb.c"
   $125 = HEAP32[$124>>2]|0; //@line 2147 "lbfgsb.c"
   $126 = $125 << 1; //@line 2147 "lbfgsb.c"
   $127 = (($126) + 1)|0; //@line 2147 "lbfgsb.c"
   $128 = $11; //@line 2147 "lbfgsb.c"
   $129 = (($128) + ($127<<3)|0); //@line 2147 "lbfgsb.c"
   $130 = $11; //@line 2147 "lbfgsb.c"
   $131 = ((($130)) + 8|0); //@line 2147 "lbfgsb.c"
   $132 = $18; //@line 2148 "lbfgsb.c"
   (_bmv_($116,$119,$122,$123,$129,$131,$132)|0); //@line 2147 "lbfgsb.c"
   $133 = $18; //@line 2149 "lbfgsb.c"
   $134 = HEAP32[$133>>2]|0; //@line 2149 "lbfgsb.c"
   $135 = ($134|0)!=(0); //@line 2149 "lbfgsb.c"
   if ($135) {
    $136 = $18; //@line 2150 "lbfgsb.c"
    HEAP32[$136>>2] = -8; //@line 2150 "lbfgsb.c"
    $0 = 0; //@line 2151 "lbfgsb.c"
    $205 = $0; //@line 2170 "lbfgsb.c"
    STACKTOP = sp;return ($205|0); //@line 2170 "lbfgsb.c"
   }
   $137 = $15; //@line 2153 "lbfgsb.c"
   $138 = HEAP32[$137>>2]|0; //@line 2153 "lbfgsb.c"
   HEAP32[1676>>2] = $138; //@line 2153 "lbfgsb.c"
   $139 = $14; //@line 2154 "lbfgsb.c"
   $140 = HEAP32[$139>>2]|0; //@line 2154 "lbfgsb.c"
   $i__1 = $140; //@line 2154 "lbfgsb.c"
   HEAP32[1668>>2] = 1; //@line 2155 "lbfgsb.c"
   while(1) {
    $141 = HEAP32[1668>>2]|0; //@line 2155 "lbfgsb.c"
    $142 = $i__1; //@line 2155 "lbfgsb.c"
    $143 = ($141|0)<=($142|0); //@line 2155 "lbfgsb.c"
    if (!($143)) {
     break L7;
    }
    $144 = HEAP32[1668>>2]|0; //@line 2156 "lbfgsb.c"
    $145 = $11; //@line 2156 "lbfgsb.c"
    $146 = (($145) + ($144<<3)|0); //@line 2156 "lbfgsb.c"
    $147 = +HEAPF64[$146>>3]; //@line 2156 "lbfgsb.c"
    HEAPF64[352>>3] = $147; //@line 2156 "lbfgsb.c"
    $148 = $13; //@line 2157 "lbfgsb.c"
    $149 = +HEAPF64[$148>>3]; //@line 2157 "lbfgsb.c"
    $150 = $14; //@line 2157 "lbfgsb.c"
    $151 = HEAP32[$150>>2]|0; //@line 2157 "lbfgsb.c"
    $152 = HEAP32[1668>>2]|0; //@line 2157 "lbfgsb.c"
    $153 = (($151) + ($152))|0; //@line 2157 "lbfgsb.c"
    $154 = $11; //@line 2157 "lbfgsb.c"
    $155 = (($154) + ($153<<3)|0); //@line 2157 "lbfgsb.c"
    $156 = +HEAPF64[$155>>3]; //@line 2157 "lbfgsb.c"
    $157 = $149 * $156; //@line 2157 "lbfgsb.c"
    HEAPF64[360>>3] = $157; //@line 2157 "lbfgsb.c"
    $158 = $16; //@line 2158 "lbfgsb.c"
    $159 = HEAP32[$158>>2]|0; //@line 2158 "lbfgsb.c"
    $i__2 = $159; //@line 2158 "lbfgsb.c"
    HEAP32[1664>>2] = 1; //@line 2159 "lbfgsb.c"
    while(1) {
     $160 = HEAP32[1664>>2]|0; //@line 2159 "lbfgsb.c"
     $161 = $i__2; //@line 2159 "lbfgsb.c"
     $162 = ($160|0)<=($161|0); //@line 2159 "lbfgsb.c"
     if (!($162)) {
      break;
     }
     $163 = HEAP32[1664>>2]|0; //@line 2160 "lbfgsb.c"
     $164 = $12; //@line 2160 "lbfgsb.c"
     $165 = (($164) + ($163<<2)|0); //@line 2160 "lbfgsb.c"
     $166 = HEAP32[$165>>2]|0; //@line 2160 "lbfgsb.c"
     HEAP32[1672>>2] = $166; //@line 2160 "lbfgsb.c"
     $167 = HEAP32[1664>>2]|0; //@line 2161 "lbfgsb.c"
     $168 = $10; //@line 2161 "lbfgsb.c"
     $169 = (($168) + ($167<<3)|0); //@line 2161 "lbfgsb.c"
     $170 = +HEAPF64[$169>>3]; //@line 2161 "lbfgsb.c"
     $171 = HEAP32[1672>>2]|0; //@line 2161 "lbfgsb.c"
     $172 = HEAP32[1676>>2]|0; //@line 2161 "lbfgsb.c"
     $173 = $wy_dim1; //@line 2161 "lbfgsb.c"
     $174 = Math_imul($172, $173)|0; //@line 2161 "lbfgsb.c"
     $175 = (($171) + ($174))|0; //@line 2161 "lbfgsb.c"
     $176 = $6; //@line 2161 "lbfgsb.c"
     $177 = (($176) + ($175<<3)|0); //@line 2161 "lbfgsb.c"
     $178 = +HEAPF64[$177>>3]; //@line 2161 "lbfgsb.c"
     $179 = +HEAPF64[352>>3]; //@line 2161 "lbfgsb.c"
     $180 = $178 * $179; //@line 2161 "lbfgsb.c"
     $181 = $170 + $180; //@line 2161 "lbfgsb.c"
     $182 = HEAP32[1672>>2]|0; //@line 2161 "lbfgsb.c"
     $183 = HEAP32[1676>>2]|0; //@line 2162 "lbfgsb.c"
     $184 = $ws_dim1; //@line 2162 "lbfgsb.c"
     $185 = Math_imul($183, $184)|0; //@line 2162 "lbfgsb.c"
     $186 = (($182) + ($185))|0; //@line 2161 "lbfgsb.c"
     $187 = $5; //@line 2161 "lbfgsb.c"
     $188 = (($187) + ($186<<3)|0); //@line 2161 "lbfgsb.c"
     $189 = +HEAPF64[$188>>3]; //@line 2161 "lbfgsb.c"
     $190 = +HEAPF64[360>>3]; //@line 2162 "lbfgsb.c"
     $191 = $189 * $190; //@line 2162 "lbfgsb.c"
     $192 = $181 + $191; //@line 2161 "lbfgsb.c"
     $193 = HEAP32[1664>>2]|0; //@line 2161 "lbfgsb.c"
     $194 = $10; //@line 2161 "lbfgsb.c"
     $195 = (($194) + ($193<<3)|0); //@line 2161 "lbfgsb.c"
     HEAPF64[$195>>3] = $192; //@line 2161 "lbfgsb.c"
     $196 = HEAP32[1664>>2]|0; //@line 2159 "lbfgsb.c"
     $197 = (($196) + 1)|0; //@line 2159 "lbfgsb.c"
     HEAP32[1664>>2] = $197; //@line 2159 "lbfgsb.c"
    }
    $198 = HEAP32[1676>>2]|0; //@line 2165 "lbfgsb.c"
    $199 = $2; //@line 2165 "lbfgsb.c"
    $200 = HEAP32[$199>>2]|0; //@line 2165 "lbfgsb.c"
    $201 = (($198|0) % ($200|0))&-1; //@line 2165 "lbfgsb.c"
    $202 = (($201) + 1)|0; //@line 2165 "lbfgsb.c"
    HEAP32[1676>>2] = $202; //@line 2165 "lbfgsb.c"
    $203 = HEAP32[1668>>2]|0; //@line 2155 "lbfgsb.c"
    $204 = (($203) + 1)|0; //@line 2155 "lbfgsb.c"
    HEAP32[1668>>2] = $204; //@line 2155 "lbfgsb.c"
   }
  }
 } while(0);
 $0 = 0; //@line 2169 "lbfgsb.c"
 $205 = $0; //@line 2170 "lbfgsb.c"
 STACKTOP = sp;return ($205|0); //@line 2170 "lbfgsb.c"
}
function _subsm_($n,$m,$nsub,$ind,$l,$u,$nbd,$x,$d__,$xp,$ws,$wy,$theta,$xx,$gg,$col,$head,$iword,$wv,$wn,$iprint,$info) {
 $n = $n|0;
 $m = $m|0;
 $nsub = $nsub|0;
 $ind = $ind|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $x = $x|0;
 $d__ = $d__|0;
 $xp = $xp|0;
 $ws = $ws|0;
 $wy = $wy|0;
 $theta = $theta|0;
 $xx = $xx|0;
 $gg = $gg|0;
 $col = $col|0;
 $head = $head|0;
 $iword = $iword|0;
 $wv = $wv|0;
 $wn = $wn|0;
 $iprint = $iprint|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0.0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0.0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0, $215 = 0, $216 = 0, $217 = 0.0, $218 = 0.0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0, $234 = 0.0, $235 = 0.0, $236 = 0.0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0.0, $274 = 0, $275 = 0, $276 = 0, $277 = 0.0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0.0, $292 = 0.0, $293 = 0.0, $294 = 0.0, $295 = 0.0, $296 = 0.0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0.0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0.0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0.0, $322 = 0.0, $323 = 0.0, $324 = 0.0, $325 = 0.0, $326 = 0.0, $327 = 0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0.0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0.0, $335 = 0.0, $336 = 0.0, $337 = 0, $338 = 0.0, $339 = 0.0, $34 = 0, $340 = 0.0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0.0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0.0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0.0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0.0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0.0, $371 = 0.0, $372 = 0.0, $373 = 0.0, $374 = 0.0, $375 = 0.0, $376 = 0, $377 = 0.0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0.0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0.0, $391 = 0, $392 = 0, $393 = 0.0, $394 = 0.0, $395 = 0.0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0.0, $413 = 0, $414 = 0, $415 = 0, $416 = 0.0, $417 = 0.0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0.0;
 var $422 = 0.0, $423 = 0.0, $424 = 0.0, $425 = 0, $426 = 0, $427 = 0.0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0.0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0.0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0.0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0.0, $471 = 0.0, $472 = 0.0, $473 = 0, $474 = 0.0, $475 = 0.0, $476 = 0.0;
 var $477 = 0.0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0.0, $482 = 0.0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0.0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0.0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0.0, $504 = 0, $505 = 0.0, $506 = 0.0, $507 = 0.0, $508 = 0.0, $509 = 0.0, $51 = 0, $510 = 0, $511 = 0.0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0.0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0.0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0.0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0.0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0.0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0.0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0.0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0.0, $563 = 0.0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0.0, $568 = 0.0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0.0, $98 = 0, $99 = 0, $d__1 = 0, $d__2 = 0.0, $i__1 = 0, $i__2 = 0, $wn_dim1 = 0;
 var $wn_offset = 0, $ws_dim1 = 0, $ws_offset = 0, $wy_dim1 = 0, $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $d__1 = sp + 8|0;
 $1 = $n;
 $2 = $m;
 $3 = $nsub;
 $4 = $ind;
 $5 = $l;
 $6 = $u;
 $7 = $nbd;
 $8 = $x;
 $9 = $d__;
 $10 = $xp;
 $11 = $ws;
 $12 = $wy;
 $13 = $theta;
 $14 = $xx;
 $15 = $gg;
 $16 = $col;
 $17 = $head;
 $18 = $iword;
 $19 = $wv;
 $20 = $wn;
 $21 = $iprint;
 $22 = $info;
 $23 = $15; //@line 3950 "lbfgsb.c"
 $24 = ((($23)) + -8|0); //@line 3950 "lbfgsb.c"
 $15 = $24; //@line 3950 "lbfgsb.c"
 $25 = $14; //@line 3951 "lbfgsb.c"
 $26 = ((($25)) + -8|0); //@line 3951 "lbfgsb.c"
 $14 = $26; //@line 3951 "lbfgsb.c"
 $27 = $10; //@line 3952 "lbfgsb.c"
 $28 = ((($27)) + -8|0); //@line 3952 "lbfgsb.c"
 $10 = $28; //@line 3952 "lbfgsb.c"
 $29 = $9; //@line 3953 "lbfgsb.c"
 $30 = ((($29)) + -8|0); //@line 3953 "lbfgsb.c"
 $9 = $30; //@line 3953 "lbfgsb.c"
 $31 = $8; //@line 3954 "lbfgsb.c"
 $32 = ((($31)) + -8|0); //@line 3954 "lbfgsb.c"
 $8 = $32; //@line 3954 "lbfgsb.c"
 $33 = $7; //@line 3955 "lbfgsb.c"
 $34 = ((($33)) + -4|0); //@line 3955 "lbfgsb.c"
 $7 = $34; //@line 3955 "lbfgsb.c"
 $35 = $6; //@line 3956 "lbfgsb.c"
 $36 = ((($35)) + -8|0); //@line 3956 "lbfgsb.c"
 $6 = $36; //@line 3956 "lbfgsb.c"
 $37 = $5; //@line 3957 "lbfgsb.c"
 $38 = ((($37)) + -8|0); //@line 3957 "lbfgsb.c"
 $5 = $38; //@line 3957 "lbfgsb.c"
 $39 = $2; //@line 3958 "lbfgsb.c"
 $40 = HEAP32[$39>>2]|0; //@line 3958 "lbfgsb.c"
 $41 = $40<<1; //@line 3958 "lbfgsb.c"
 $wn_dim1 = $41; //@line 3958 "lbfgsb.c"
 $42 = $wn_dim1; //@line 3959 "lbfgsb.c"
 $43 = (1 + ($42))|0; //@line 3959 "lbfgsb.c"
 $wn_offset = $43; //@line 3959 "lbfgsb.c"
 $44 = $wn_offset; //@line 3960 "lbfgsb.c"
 $45 = $20; //@line 3960 "lbfgsb.c"
 $46 = (0 - ($44))|0; //@line 3960 "lbfgsb.c"
 $47 = (($45) + ($46<<3)|0); //@line 3960 "lbfgsb.c"
 $20 = $47; //@line 3960 "lbfgsb.c"
 $48 = $19; //@line 3961 "lbfgsb.c"
 $49 = ((($48)) + -8|0); //@line 3961 "lbfgsb.c"
 $19 = $49; //@line 3961 "lbfgsb.c"
 $50 = $1; //@line 3962 "lbfgsb.c"
 $51 = HEAP32[$50>>2]|0; //@line 3962 "lbfgsb.c"
 $wy_dim1 = $51; //@line 3962 "lbfgsb.c"
 $52 = $wy_dim1; //@line 3963 "lbfgsb.c"
 $53 = (1 + ($52))|0; //@line 3963 "lbfgsb.c"
 $wy_offset = $53; //@line 3963 "lbfgsb.c"
 $54 = $wy_offset; //@line 3964 "lbfgsb.c"
 $55 = $12; //@line 3964 "lbfgsb.c"
 $56 = (0 - ($54))|0; //@line 3964 "lbfgsb.c"
 $57 = (($55) + ($56<<3)|0); //@line 3964 "lbfgsb.c"
 $12 = $57; //@line 3964 "lbfgsb.c"
 $58 = $1; //@line 3965 "lbfgsb.c"
 $59 = HEAP32[$58>>2]|0; //@line 3965 "lbfgsb.c"
 $ws_dim1 = $59; //@line 3965 "lbfgsb.c"
 $60 = $ws_dim1; //@line 3966 "lbfgsb.c"
 $61 = (1 + ($60))|0; //@line 3966 "lbfgsb.c"
 $ws_offset = $61; //@line 3966 "lbfgsb.c"
 $62 = $ws_offset; //@line 3967 "lbfgsb.c"
 $63 = $11; //@line 3967 "lbfgsb.c"
 $64 = (0 - ($62))|0; //@line 3967 "lbfgsb.c"
 $65 = (($63) + ($64<<3)|0); //@line 3967 "lbfgsb.c"
 $11 = $65; //@line 3967 "lbfgsb.c"
 $66 = $4; //@line 3968 "lbfgsb.c"
 $67 = ((($66)) + -4|0); //@line 3968 "lbfgsb.c"
 $4 = $67; //@line 3968 "lbfgsb.c"
 $68 = $3; //@line 3971 "lbfgsb.c"
 $69 = HEAP32[$68>>2]|0; //@line 3971 "lbfgsb.c"
 $70 = ($69|0)<=(0); //@line 3971 "lbfgsb.c"
 if ($70) {
  $0 = 0; //@line 3972 "lbfgsb.c"
  $574 = $0; //@line 4167 "lbfgsb.c"
  STACKTOP = sp;return ($574|0); //@line 4167 "lbfgsb.c"
 }
 $71 = $21; //@line 3974 "lbfgsb.c"
 $72 = HEAP32[$71>>2]|0; //@line 3974 "lbfgsb.c"
 $73 = ($72|0)>=(99); //@line 3974 "lbfgsb.c"
 if ($73) {
  (_s_wsfe(2516)|0); //@line 3975 "lbfgsb.c"
  (_e_wsfe()|0); //@line 3976 "lbfgsb.c"
 }
 $74 = $17; //@line 3979 "lbfgsb.c"
 $75 = HEAP32[$74>>2]|0; //@line 3979 "lbfgsb.c"
 HEAP32[2512>>2] = $75; //@line 3979 "lbfgsb.c"
 $76 = $16; //@line 3980 "lbfgsb.c"
 $77 = HEAP32[$76>>2]|0; //@line 3980 "lbfgsb.c"
 $i__1 = $77; //@line 3980 "lbfgsb.c"
 HEAP32[2480>>2] = 1; //@line 3981 "lbfgsb.c"
 while(1) {
  $78 = HEAP32[2480>>2]|0; //@line 3981 "lbfgsb.c"
  $79 = $i__1; //@line 3981 "lbfgsb.c"
  $80 = ($78|0)<=($79|0); //@line 3981 "lbfgsb.c"
  if (!($80)) {
   break;
  }
  HEAPF64[496>>3] = 0.0; //@line 3982 "lbfgsb.c"
  HEAPF64[504>>3] = 0.0; //@line 3983 "lbfgsb.c"
  $81 = $3; //@line 3984 "lbfgsb.c"
  $82 = HEAP32[$81>>2]|0; //@line 3984 "lbfgsb.c"
  $i__2 = $82; //@line 3984 "lbfgsb.c"
  HEAP32[2484>>2] = 1; //@line 3985 "lbfgsb.c"
  while(1) {
   $83 = HEAP32[2484>>2]|0; //@line 3985 "lbfgsb.c"
   $84 = $i__2; //@line 3985 "lbfgsb.c"
   $85 = ($83|0)<=($84|0); //@line 3985 "lbfgsb.c"
   if (!($85)) {
    break;
   }
   $86 = HEAP32[2484>>2]|0; //@line 3986 "lbfgsb.c"
   $87 = $4; //@line 3986 "lbfgsb.c"
   $88 = (($87) + ($86<<2)|0); //@line 3986 "lbfgsb.c"
   $89 = HEAP32[$88>>2]|0; //@line 3986 "lbfgsb.c"
   HEAP32[2488>>2] = $89; //@line 3986 "lbfgsb.c"
   $90 = HEAP32[2488>>2]|0; //@line 3987 "lbfgsb.c"
   $91 = HEAP32[2512>>2]|0; //@line 3987 "lbfgsb.c"
   $92 = $wy_dim1; //@line 3987 "lbfgsb.c"
   $93 = Math_imul($91, $92)|0; //@line 3987 "lbfgsb.c"
   $94 = (($90) + ($93))|0; //@line 3987 "lbfgsb.c"
   $95 = $12; //@line 3987 "lbfgsb.c"
   $96 = (($95) + ($94<<3)|0); //@line 3987 "lbfgsb.c"
   $97 = +HEAPF64[$96>>3]; //@line 3987 "lbfgsb.c"
   $98 = HEAP32[2484>>2]|0; //@line 3987 "lbfgsb.c"
   $99 = $9; //@line 3987 "lbfgsb.c"
   $100 = (($99) + ($98<<3)|0); //@line 3987 "lbfgsb.c"
   $101 = +HEAPF64[$100>>3]; //@line 3987 "lbfgsb.c"
   $102 = $97 * $101; //@line 3987 "lbfgsb.c"
   $103 = +HEAPF64[496>>3]; //@line 3987 "lbfgsb.c"
   $104 = $103 + $102; //@line 3987 "lbfgsb.c"
   HEAPF64[496>>3] = $104; //@line 3987 "lbfgsb.c"
   $105 = HEAP32[2488>>2]|0; //@line 3988 "lbfgsb.c"
   $106 = HEAP32[2512>>2]|0; //@line 3988 "lbfgsb.c"
   $107 = $ws_dim1; //@line 3988 "lbfgsb.c"
   $108 = Math_imul($106, $107)|0; //@line 3988 "lbfgsb.c"
   $109 = (($105) + ($108))|0; //@line 3988 "lbfgsb.c"
   $110 = $11; //@line 3988 "lbfgsb.c"
   $111 = (($110) + ($109<<3)|0); //@line 3988 "lbfgsb.c"
   $112 = +HEAPF64[$111>>3]; //@line 3988 "lbfgsb.c"
   $113 = HEAP32[2484>>2]|0; //@line 3988 "lbfgsb.c"
   $114 = $9; //@line 3988 "lbfgsb.c"
   $115 = (($114) + ($113<<3)|0); //@line 3988 "lbfgsb.c"
   $116 = +HEAPF64[$115>>3]; //@line 3988 "lbfgsb.c"
   $117 = $112 * $116; //@line 3988 "lbfgsb.c"
   $118 = +HEAPF64[504>>3]; //@line 3988 "lbfgsb.c"
   $119 = $118 + $117; //@line 3988 "lbfgsb.c"
   HEAPF64[504>>3] = $119; //@line 3988 "lbfgsb.c"
   $120 = HEAP32[2484>>2]|0; //@line 3985 "lbfgsb.c"
   $121 = (($120) + 1)|0; //@line 3985 "lbfgsb.c"
   HEAP32[2484>>2] = $121; //@line 3985 "lbfgsb.c"
  }
  $122 = +HEAPF64[496>>3]; //@line 3991 "lbfgsb.c"
  $123 = HEAP32[2480>>2]|0; //@line 3991 "lbfgsb.c"
  $124 = $19; //@line 3991 "lbfgsb.c"
  $125 = (($124) + ($123<<3)|0); //@line 3991 "lbfgsb.c"
  HEAPF64[$125>>3] = $122; //@line 3991 "lbfgsb.c"
  $126 = $13; //@line 3992 "lbfgsb.c"
  $127 = +HEAPF64[$126>>3]; //@line 3992 "lbfgsb.c"
  $128 = +HEAPF64[504>>3]; //@line 3992 "lbfgsb.c"
  $129 = $127 * $128; //@line 3992 "lbfgsb.c"
  $130 = $16; //@line 3992 "lbfgsb.c"
  $131 = HEAP32[$130>>2]|0; //@line 3992 "lbfgsb.c"
  $132 = HEAP32[2480>>2]|0; //@line 3992 "lbfgsb.c"
  $133 = (($131) + ($132))|0; //@line 3992 "lbfgsb.c"
  $134 = $19; //@line 3992 "lbfgsb.c"
  $135 = (($134) + ($133<<3)|0); //@line 3992 "lbfgsb.c"
  HEAPF64[$135>>3] = $129; //@line 3992 "lbfgsb.c"
  $136 = HEAP32[2512>>2]|0; //@line 3993 "lbfgsb.c"
  $137 = $2; //@line 3993 "lbfgsb.c"
  $138 = HEAP32[$137>>2]|0; //@line 3993 "lbfgsb.c"
  $139 = (($136|0) % ($138|0))&-1; //@line 3993 "lbfgsb.c"
  $140 = (($139) + 1)|0; //@line 3993 "lbfgsb.c"
  HEAP32[2512>>2] = $140; //@line 3993 "lbfgsb.c"
  $141 = HEAP32[2480>>2]|0; //@line 3981 "lbfgsb.c"
  $142 = (($141) + 1)|0; //@line 3981 "lbfgsb.c"
  HEAP32[2480>>2] = $142; //@line 3981 "lbfgsb.c"
 }
 $143 = $2; //@line 3997 "lbfgsb.c"
 $144 = HEAP32[$143>>2]|0; //@line 3997 "lbfgsb.c"
 $145 = $144 << 1; //@line 3997 "lbfgsb.c"
 HEAP32[2492>>2] = $145; //@line 3997 "lbfgsb.c"
 $146 = $16; //@line 3998 "lbfgsb.c"
 $147 = HEAP32[$146>>2]|0; //@line 3998 "lbfgsb.c"
 $148 = $147 << 1; //@line 3998 "lbfgsb.c"
 HEAP32[2508>>2] = $148; //@line 3998 "lbfgsb.c"
 $149 = $wn_offset; //@line 3999 "lbfgsb.c"
 $150 = $20; //@line 3999 "lbfgsb.c"
 $151 = (($150) + ($149<<3)|0); //@line 3999 "lbfgsb.c"
 $152 = $19; //@line 3999 "lbfgsb.c"
 $153 = ((($152)) + 8|0); //@line 3999 "lbfgsb.c"
 $154 = $22; //@line 3999 "lbfgsb.c"
 (_dtrsl_($151,2492,2508,$153,1324,$154)|0); //@line 3999 "lbfgsb.c"
 $155 = $22; //@line 4000 "lbfgsb.c"
 $156 = HEAP32[$155>>2]|0; //@line 4000 "lbfgsb.c"
 $157 = ($156|0)!=(0); //@line 4000 "lbfgsb.c"
 if ($157) {
  $0 = 0; //@line 4001 "lbfgsb.c"
  $574 = $0; //@line 4167 "lbfgsb.c"
  STACKTOP = sp;return ($574|0); //@line 4167 "lbfgsb.c"
 }
 $158 = $16; //@line 4003 "lbfgsb.c"
 $159 = HEAP32[$158>>2]|0; //@line 4003 "lbfgsb.c"
 $i__1 = $159; //@line 4003 "lbfgsb.c"
 HEAP32[2480>>2] = 1; //@line 4004 "lbfgsb.c"
 while(1) {
  $160 = HEAP32[2480>>2]|0; //@line 4004 "lbfgsb.c"
  $161 = $i__1; //@line 4004 "lbfgsb.c"
  $162 = ($160|0)<=($161|0); //@line 4004 "lbfgsb.c"
  if (!($162)) {
   break;
  }
  $163 = HEAP32[2480>>2]|0; //@line 4005 "lbfgsb.c"
  $164 = $19; //@line 4005 "lbfgsb.c"
  $165 = (($164) + ($163<<3)|0); //@line 4005 "lbfgsb.c"
  $166 = +HEAPF64[$165>>3]; //@line 4005 "lbfgsb.c"
  $167 = -$166; //@line 4005 "lbfgsb.c"
  $168 = HEAP32[2480>>2]|0; //@line 4005 "lbfgsb.c"
  $169 = $19; //@line 4005 "lbfgsb.c"
  $170 = (($169) + ($168<<3)|0); //@line 4005 "lbfgsb.c"
  HEAPF64[$170>>3] = $167; //@line 4005 "lbfgsb.c"
  $171 = HEAP32[2480>>2]|0; //@line 4004 "lbfgsb.c"
  $172 = (($171) + 1)|0; //@line 4004 "lbfgsb.c"
  HEAP32[2480>>2] = $172; //@line 4004 "lbfgsb.c"
 }
 $173 = $wn_offset; //@line 4008 "lbfgsb.c"
 $174 = $20; //@line 4008 "lbfgsb.c"
 $175 = (($174) + ($173<<3)|0); //@line 4008 "lbfgsb.c"
 $176 = $19; //@line 4008 "lbfgsb.c"
 $177 = ((($176)) + 8|0); //@line 4008 "lbfgsb.c"
 $178 = $22; //@line 4008 "lbfgsb.c"
 (_dtrsl_($175,2492,2508,$177,1236,$178)|0); //@line 4008 "lbfgsb.c"
 $179 = $22; //@line 4009 "lbfgsb.c"
 $180 = HEAP32[$179>>2]|0; //@line 4009 "lbfgsb.c"
 $181 = ($180|0)!=(0); //@line 4009 "lbfgsb.c"
 if ($181) {
  $0 = 0; //@line 4010 "lbfgsb.c"
  $574 = $0; //@line 4167 "lbfgsb.c"
  STACKTOP = sp;return ($574|0); //@line 4167 "lbfgsb.c"
 }
 $182 = $17; //@line 4013 "lbfgsb.c"
 $183 = HEAP32[$182>>2]|0; //@line 4013 "lbfgsb.c"
 HEAP32[2512>>2] = $183; //@line 4013 "lbfgsb.c"
 $184 = $16; //@line 4014 "lbfgsb.c"
 $185 = HEAP32[$184>>2]|0; //@line 4014 "lbfgsb.c"
 $i__1 = $185; //@line 4014 "lbfgsb.c"
 HEAP32[2500>>2] = 1; //@line 4015 "lbfgsb.c"
 while(1) {
  $186 = HEAP32[2500>>2]|0; //@line 4015 "lbfgsb.c"
  $187 = $i__1; //@line 4015 "lbfgsb.c"
  $188 = ($186|0)<=($187|0); //@line 4015 "lbfgsb.c"
  if (!($188)) {
   break;
  }
  $189 = $16; //@line 4016 "lbfgsb.c"
  $190 = HEAP32[$189>>2]|0; //@line 4016 "lbfgsb.c"
  $191 = HEAP32[2500>>2]|0; //@line 4016 "lbfgsb.c"
  $192 = (($190) + ($191))|0; //@line 4016 "lbfgsb.c"
  HEAP32[2496>>2] = $192; //@line 4016 "lbfgsb.c"
  $193 = $3; //@line 4017 "lbfgsb.c"
  $194 = HEAP32[$193>>2]|0; //@line 4017 "lbfgsb.c"
  $i__2 = $194; //@line 4017 "lbfgsb.c"
  HEAP32[2480>>2] = 1; //@line 4018 "lbfgsb.c"
  while(1) {
   $195 = HEAP32[2480>>2]|0; //@line 4018 "lbfgsb.c"
   $196 = $i__2; //@line 4018 "lbfgsb.c"
   $197 = ($195|0)<=($196|0); //@line 4018 "lbfgsb.c"
   if (!($197)) {
    break;
   }
   $198 = HEAP32[2480>>2]|0; //@line 4019 "lbfgsb.c"
   $199 = $4; //@line 4019 "lbfgsb.c"
   $200 = (($199) + ($198<<2)|0); //@line 4019 "lbfgsb.c"
   $201 = HEAP32[$200>>2]|0; //@line 4019 "lbfgsb.c"
   HEAP32[2488>>2] = $201; //@line 4019 "lbfgsb.c"
   $202 = HEAP32[2480>>2]|0; //@line 4020 "lbfgsb.c"
   $203 = $9; //@line 4020 "lbfgsb.c"
   $204 = (($203) + ($202<<3)|0); //@line 4020 "lbfgsb.c"
   $205 = +HEAPF64[$204>>3]; //@line 4020 "lbfgsb.c"
   $206 = HEAP32[2488>>2]|0; //@line 4020 "lbfgsb.c"
   $207 = HEAP32[2512>>2]|0; //@line 4020 "lbfgsb.c"
   $208 = $wy_dim1; //@line 4020 "lbfgsb.c"
   $209 = Math_imul($207, $208)|0; //@line 4020 "lbfgsb.c"
   $210 = (($206) + ($209))|0; //@line 4020 "lbfgsb.c"
   $211 = $12; //@line 4020 "lbfgsb.c"
   $212 = (($211) + ($210<<3)|0); //@line 4020 "lbfgsb.c"
   $213 = +HEAPF64[$212>>3]; //@line 4020 "lbfgsb.c"
   $214 = HEAP32[2500>>2]|0; //@line 4020 "lbfgsb.c"
   $215 = $19; //@line 4020 "lbfgsb.c"
   $216 = (($215) + ($214<<3)|0); //@line 4020 "lbfgsb.c"
   $217 = +HEAPF64[$216>>3]; //@line 4020 "lbfgsb.c"
   $218 = $213 * $217; //@line 4020 "lbfgsb.c"
   $219 = $13; //@line 4020 "lbfgsb.c"
   $220 = +HEAPF64[$219>>3]; //@line 4020 "lbfgsb.c"
   $221 = $218 / $220; //@line 4020 "lbfgsb.c"
   $222 = $205 + $221; //@line 4020 "lbfgsb.c"
   $223 = HEAP32[2488>>2]|0; //@line 4021 "lbfgsb.c"
   $224 = HEAP32[2512>>2]|0; //@line 4021 "lbfgsb.c"
   $225 = $ws_dim1; //@line 4021 "lbfgsb.c"
   $226 = Math_imul($224, $225)|0; //@line 4021 "lbfgsb.c"
   $227 = (($223) + ($226))|0; //@line 4021 "lbfgsb.c"
   $228 = $11; //@line 4021 "lbfgsb.c"
   $229 = (($228) + ($227<<3)|0); //@line 4021 "lbfgsb.c"
   $230 = +HEAPF64[$229>>3]; //@line 4021 "lbfgsb.c"
   $231 = HEAP32[2496>>2]|0; //@line 4021 "lbfgsb.c"
   $232 = $19; //@line 4021 "lbfgsb.c"
   $233 = (($232) + ($231<<3)|0); //@line 4021 "lbfgsb.c"
   $234 = +HEAPF64[$233>>3]; //@line 4021 "lbfgsb.c"
   $235 = $230 * $234; //@line 4021 "lbfgsb.c"
   $236 = $222 + $235; //@line 4021 "lbfgsb.c"
   $237 = HEAP32[2480>>2]|0; //@line 4020 "lbfgsb.c"
   $238 = $9; //@line 4020 "lbfgsb.c"
   $239 = (($238) + ($237<<3)|0); //@line 4020 "lbfgsb.c"
   HEAPF64[$239>>3] = $236; //@line 4020 "lbfgsb.c"
   $240 = HEAP32[2480>>2]|0; //@line 4018 "lbfgsb.c"
   $241 = (($240) + 1)|0; //@line 4018 "lbfgsb.c"
   HEAP32[2480>>2] = $241; //@line 4018 "lbfgsb.c"
  }
  $242 = HEAP32[2512>>2]|0; //@line 4024 "lbfgsb.c"
  $243 = $2; //@line 4024 "lbfgsb.c"
  $244 = HEAP32[$243>>2]|0; //@line 4024 "lbfgsb.c"
  $245 = (($242|0) % ($244|0))&-1; //@line 4024 "lbfgsb.c"
  $246 = (($245) + 1)|0; //@line 4024 "lbfgsb.c"
  HEAP32[2512>>2] = $246; //@line 4024 "lbfgsb.c"
  $247 = HEAP32[2500>>2]|0; //@line 4015 "lbfgsb.c"
  $248 = (($247) + 1)|0; //@line 4015 "lbfgsb.c"
  HEAP32[2500>>2] = $248; //@line 4015 "lbfgsb.c"
 }
 $249 = $13; //@line 4027 "lbfgsb.c"
 $250 = +HEAPF64[$249>>3]; //@line 4027 "lbfgsb.c"
 $251 = 1.0 / $250; //@line 4027 "lbfgsb.c"
 HEAPF64[$d__1>>3] = $251; //@line 4027 "lbfgsb.c"
 $252 = $3; //@line 4028 "lbfgsb.c"
 $253 = $9; //@line 4028 "lbfgsb.c"
 $254 = ((($253)) + 8|0); //@line 4028 "lbfgsb.c"
 (_dscal_($252,$d__1,$254,1236)|0); //@line 4028 "lbfgsb.c"
 $255 = $18; //@line 4032 "lbfgsb.c"
 HEAP32[$255>>2] = 0; //@line 4032 "lbfgsb.c"
 $256 = $1; //@line 4033 "lbfgsb.c"
 $257 = $8; //@line 4033 "lbfgsb.c"
 $258 = ((($257)) + 8|0); //@line 4033 "lbfgsb.c"
 $259 = $10; //@line 4033 "lbfgsb.c"
 $260 = ((($259)) + 8|0); //@line 4033 "lbfgsb.c"
 (_dcopy_($256,$258,1236,$260,1236)|0); //@line 4033 "lbfgsb.c"
 $261 = $3; //@line 4035 "lbfgsb.c"
 $262 = HEAP32[$261>>2]|0; //@line 4035 "lbfgsb.c"
 $i__1 = $262; //@line 4035 "lbfgsb.c"
 HEAP32[2480>>2] = 1; //@line 4036 "lbfgsb.c"
 while(1) {
  $263 = HEAP32[2480>>2]|0; //@line 4036 "lbfgsb.c"
  $264 = $i__1; //@line 4036 "lbfgsb.c"
  $265 = ($263|0)<=($264|0); //@line 4036 "lbfgsb.c"
  if (!($265)) {
   break;
  }
  $266 = HEAP32[2480>>2]|0; //@line 4037 "lbfgsb.c"
  $267 = $4; //@line 4037 "lbfgsb.c"
  $268 = (($267) + ($266<<2)|0); //@line 4037 "lbfgsb.c"
  $269 = HEAP32[$268>>2]|0; //@line 4037 "lbfgsb.c"
  HEAP32[2488>>2] = $269; //@line 4037 "lbfgsb.c"
  $270 = HEAP32[2480>>2]|0; //@line 4038 "lbfgsb.c"
  $271 = $9; //@line 4038 "lbfgsb.c"
  $272 = (($271) + ($270<<3)|0); //@line 4038 "lbfgsb.c"
  $273 = +HEAPF64[$272>>3]; //@line 4038 "lbfgsb.c"
  HEAPF64[472>>3] = $273; //@line 4038 "lbfgsb.c"
  $274 = HEAP32[2488>>2]|0; //@line 4039 "lbfgsb.c"
  $275 = $8; //@line 4039 "lbfgsb.c"
  $276 = (($275) + ($274<<3)|0); //@line 4039 "lbfgsb.c"
  $277 = +HEAPF64[$276>>3]; //@line 4039 "lbfgsb.c"
  HEAPF64[480>>3] = $277; //@line 4039 "lbfgsb.c"
  $278 = HEAP32[2488>>2]|0; //@line 4040 "lbfgsb.c"
  $279 = $7; //@line 4040 "lbfgsb.c"
  $280 = (($279) + ($278<<2)|0); //@line 4040 "lbfgsb.c"
  $281 = HEAP32[$280>>2]|0; //@line 4040 "lbfgsb.c"
  $282 = ($281|0)!=(0); //@line 4040 "lbfgsb.c"
  do {
   if ($282) {
    $283 = HEAP32[2488>>2]|0; //@line 4042 "lbfgsb.c"
    $284 = $7; //@line 4042 "lbfgsb.c"
    $285 = (($284) + ($283<<2)|0); //@line 4042 "lbfgsb.c"
    $286 = HEAP32[$285>>2]|0; //@line 4042 "lbfgsb.c"
    $287 = ($286|0)==(1); //@line 4042 "lbfgsb.c"
    $288 = HEAP32[2488>>2]|0; //@line 4045 "lbfgsb.c"
    if ($287) {
     $289 = $5; //@line 4045 "lbfgsb.c"
     $290 = (($289) + ($288<<3)|0); //@line 4045 "lbfgsb.c"
     $291 = +HEAPF64[$290>>3]; //@line 4045 "lbfgsb.c"
     HEAPF64[$d__1>>3] = $291; //@line 4045 "lbfgsb.c"
     $292 = +HEAPF64[480>>3]; //@line 4045 "lbfgsb.c"
     $293 = +HEAPF64[472>>3]; //@line 4045 "lbfgsb.c"
     $294 = $292 + $293; //@line 4045 "lbfgsb.c"
     $d__2 = $294; //@line 4045 "lbfgsb.c"
     $295 = +HEAPF64[$d__1>>3]; //@line 4046 "lbfgsb.c"
     $296 = $d__2; //@line 4046 "lbfgsb.c"
     $297 = $295 >= $296; //@line 4046 "lbfgsb.c"
     $298 = +HEAPF64[$d__1>>3]; //@line 4046 "lbfgsb.c"
     $299 = $d__2; //@line 4046 "lbfgsb.c"
     $300 = $297 ? $298 : $299; //@line 4046 "lbfgsb.c"
     $301 = HEAP32[2488>>2]|0; //@line 4046 "lbfgsb.c"
     $302 = $8; //@line 4046 "lbfgsb.c"
     $303 = (($302) + ($301<<3)|0); //@line 4046 "lbfgsb.c"
     HEAPF64[$303>>3] = $300; //@line 4046 "lbfgsb.c"
     $304 = HEAP32[2488>>2]|0; //@line 4047 "lbfgsb.c"
     $305 = $8; //@line 4047 "lbfgsb.c"
     $306 = (($305) + ($304<<3)|0); //@line 4047 "lbfgsb.c"
     $307 = +HEAPF64[$306>>3]; //@line 4047 "lbfgsb.c"
     $308 = HEAP32[2488>>2]|0; //@line 4047 "lbfgsb.c"
     $309 = $5; //@line 4047 "lbfgsb.c"
     $310 = (($309) + ($308<<3)|0); //@line 4047 "lbfgsb.c"
     $311 = +HEAPF64[$310>>3]; //@line 4047 "lbfgsb.c"
     $312 = $307 == $311; //@line 4047 "lbfgsb.c"
     if (!($312)) {
      break;
     }
     $313 = $18; //@line 4048 "lbfgsb.c"
     HEAP32[$313>>2] = 1; //@line 4048 "lbfgsb.c"
     break;
    }
    $314 = $7; //@line 4052 "lbfgsb.c"
    $315 = (($314) + ($288<<2)|0); //@line 4052 "lbfgsb.c"
    $316 = HEAP32[$315>>2]|0; //@line 4052 "lbfgsb.c"
    $317 = ($316|0)==(2); //@line 4052 "lbfgsb.c"
    $318 = HEAP32[2488>>2]|0; //@line 4055 "lbfgsb.c"
    if ($317) {
     $319 = $5; //@line 4055 "lbfgsb.c"
     $320 = (($319) + ($318<<3)|0); //@line 4055 "lbfgsb.c"
     $321 = +HEAPF64[$320>>3]; //@line 4055 "lbfgsb.c"
     HEAPF64[$d__1>>3] = $321; //@line 4055 "lbfgsb.c"
     $322 = +HEAPF64[480>>3]; //@line 4055 "lbfgsb.c"
     $323 = +HEAPF64[472>>3]; //@line 4055 "lbfgsb.c"
     $324 = $322 + $323; //@line 4055 "lbfgsb.c"
     $d__2 = $324; //@line 4055 "lbfgsb.c"
     $325 = +HEAPF64[$d__1>>3]; //@line 4056 "lbfgsb.c"
     $326 = $d__2; //@line 4056 "lbfgsb.c"
     $327 = $325 >= $326; //@line 4056 "lbfgsb.c"
     $328 = +HEAPF64[$d__1>>3]; //@line 4056 "lbfgsb.c"
     $329 = $d__2; //@line 4056 "lbfgsb.c"
     $330 = $327 ? $328 : $329; //@line 4056 "lbfgsb.c"
     HEAPF64[480>>3] = $330; //@line 4056 "lbfgsb.c"
     $331 = HEAP32[2488>>2]|0; //@line 4058 "lbfgsb.c"
     $332 = $6; //@line 4058 "lbfgsb.c"
     $333 = (($332) + ($331<<3)|0); //@line 4058 "lbfgsb.c"
     $334 = +HEAPF64[$333>>3]; //@line 4058 "lbfgsb.c"
     HEAPF64[$d__1>>3] = $334; //@line 4058 "lbfgsb.c"
     $335 = +HEAPF64[$d__1>>3]; //@line 4059 "lbfgsb.c"
     $336 = +HEAPF64[480>>3]; //@line 4059 "lbfgsb.c"
     $337 = $335 <= $336; //@line 4059 "lbfgsb.c"
     $338 = +HEAPF64[$d__1>>3]; //@line 4059 "lbfgsb.c"
     $339 = +HEAPF64[480>>3]; //@line 4059 "lbfgsb.c"
     $340 = $337 ? $338 : $339; //@line 4059 "lbfgsb.c"
     $341 = HEAP32[2488>>2]|0; //@line 4059 "lbfgsb.c"
     $342 = $8; //@line 4059 "lbfgsb.c"
     $343 = (($342) + ($341<<3)|0); //@line 4059 "lbfgsb.c"
     HEAPF64[$343>>3] = $340; //@line 4059 "lbfgsb.c"
     $344 = HEAP32[2488>>2]|0; //@line 4060 "lbfgsb.c"
     $345 = $8; //@line 4060 "lbfgsb.c"
     $346 = (($345) + ($344<<3)|0); //@line 4060 "lbfgsb.c"
     $347 = +HEAPF64[$346>>3]; //@line 4060 "lbfgsb.c"
     $348 = HEAP32[2488>>2]|0; //@line 4060 "lbfgsb.c"
     $349 = $5; //@line 4060 "lbfgsb.c"
     $350 = (($349) + ($348<<3)|0); //@line 4060 "lbfgsb.c"
     $351 = +HEAPF64[$350>>3]; //@line 4060 "lbfgsb.c"
     $352 = $347 == $351; //@line 4060 "lbfgsb.c"
     if (!($352)) {
      $353 = HEAP32[2488>>2]|0; //@line 4060 "lbfgsb.c"
      $354 = $8; //@line 4060 "lbfgsb.c"
      $355 = (($354) + ($353<<3)|0); //@line 4060 "lbfgsb.c"
      $356 = +HEAPF64[$355>>3]; //@line 4060 "lbfgsb.c"
      $357 = HEAP32[2488>>2]|0; //@line 4060 "lbfgsb.c"
      $358 = $6; //@line 4060 "lbfgsb.c"
      $359 = (($358) + ($357<<3)|0); //@line 4060 "lbfgsb.c"
      $360 = +HEAPF64[$359>>3]; //@line 4060 "lbfgsb.c"
      $361 = $356 == $360; //@line 4060 "lbfgsb.c"
      if (!($361)) {
       break;
      }
     }
     $362 = $18; //@line 4061 "lbfgsb.c"
     HEAP32[$362>>2] = 1; //@line 4061 "lbfgsb.c"
     break;
    } else {
     $363 = $7; //@line 4065 "lbfgsb.c"
     $364 = (($363) + ($318<<2)|0); //@line 4065 "lbfgsb.c"
     $365 = HEAP32[$364>>2]|0; //@line 4065 "lbfgsb.c"
     $366 = ($365|0)==(3); //@line 4065 "lbfgsb.c"
     if (!($366)) {
      break;
     }
     $367 = HEAP32[2488>>2]|0; //@line 4068 "lbfgsb.c"
     $368 = $6; //@line 4068 "lbfgsb.c"
     $369 = (($368) + ($367<<3)|0); //@line 4068 "lbfgsb.c"
     $370 = +HEAPF64[$369>>3]; //@line 4068 "lbfgsb.c"
     HEAPF64[$d__1>>3] = $370; //@line 4068 "lbfgsb.c"
     $371 = +HEAPF64[480>>3]; //@line 4068 "lbfgsb.c"
     $372 = +HEAPF64[472>>3]; //@line 4068 "lbfgsb.c"
     $373 = $371 + $372; //@line 4068 "lbfgsb.c"
     $d__2 = $373; //@line 4068 "lbfgsb.c"
     $374 = +HEAPF64[$d__1>>3]; //@line 4069 "lbfgsb.c"
     $375 = $d__2; //@line 4069 "lbfgsb.c"
     $376 = $374 <= $375; //@line 4069 "lbfgsb.c"
     $377 = +HEAPF64[$d__1>>3]; //@line 4069 "lbfgsb.c"
     $378 = $d__2; //@line 4069 "lbfgsb.c"
     $379 = $376 ? $377 : $378; //@line 4069 "lbfgsb.c"
     $380 = HEAP32[2488>>2]|0; //@line 4069 "lbfgsb.c"
     $381 = $8; //@line 4069 "lbfgsb.c"
     $382 = (($381) + ($380<<3)|0); //@line 4069 "lbfgsb.c"
     HEAPF64[$382>>3] = $379; //@line 4069 "lbfgsb.c"
     $383 = HEAP32[2488>>2]|0; //@line 4070 "lbfgsb.c"
     $384 = $8; //@line 4070 "lbfgsb.c"
     $385 = (($384) + ($383<<3)|0); //@line 4070 "lbfgsb.c"
     $386 = +HEAPF64[$385>>3]; //@line 4070 "lbfgsb.c"
     $387 = HEAP32[2488>>2]|0; //@line 4070 "lbfgsb.c"
     $388 = $6; //@line 4070 "lbfgsb.c"
     $389 = (($388) + ($387<<3)|0); //@line 4070 "lbfgsb.c"
     $390 = +HEAPF64[$389>>3]; //@line 4070 "lbfgsb.c"
     $391 = $386 == $390; //@line 4070 "lbfgsb.c"
     if (!($391)) {
      break;
     }
     $392 = $18; //@line 4071 "lbfgsb.c"
     HEAP32[$392>>2] = 1; //@line 4071 "lbfgsb.c"
     break;
    }
   } else {
    $393 = +HEAPF64[480>>3]; //@line 4079 "lbfgsb.c"
    $394 = +HEAPF64[472>>3]; //@line 4079 "lbfgsb.c"
    $395 = $393 + $394; //@line 4079 "lbfgsb.c"
    $396 = HEAP32[2488>>2]|0; //@line 4079 "lbfgsb.c"
    $397 = $8; //@line 4079 "lbfgsb.c"
    $398 = (($397) + ($396<<3)|0); //@line 4079 "lbfgsb.c"
    HEAPF64[$398>>3] = $395; //@line 4079 "lbfgsb.c"
   }
  } while(0);
  $399 = HEAP32[2480>>2]|0; //@line 4036 "lbfgsb.c"
  $400 = (($399) + 1)|0; //@line 4036 "lbfgsb.c"
  HEAP32[2480>>2] = $400; //@line 4036 "lbfgsb.c"
 }
 $401 = $18; //@line 4084 "lbfgsb.c"
 $402 = HEAP32[$401>>2]|0; //@line 4084 "lbfgsb.c"
 $403 = ($402|0)==(0); //@line 4084 "lbfgsb.c"
 L56: do {
  if (!($403)) {
   HEAPF64[488>>3] = 0.0; //@line 4090 "lbfgsb.c"
   $404 = $1; //@line 4091 "lbfgsb.c"
   $405 = HEAP32[$404>>2]|0; //@line 4091 "lbfgsb.c"
   $i__1 = $405; //@line 4091 "lbfgsb.c"
   HEAP32[2480>>2] = 1; //@line 4092 "lbfgsb.c"
   while(1) {
    $406 = HEAP32[2480>>2]|0; //@line 4092 "lbfgsb.c"
    $407 = $i__1; //@line 4092 "lbfgsb.c"
    $408 = ($406|0)<=($407|0); //@line 4092 "lbfgsb.c"
    if (!($408)) {
     break;
    }
    $409 = HEAP32[2480>>2]|0; //@line 4093 "lbfgsb.c"
    $410 = $8; //@line 4093 "lbfgsb.c"
    $411 = (($410) + ($409<<3)|0); //@line 4093 "lbfgsb.c"
    $412 = +HEAPF64[$411>>3]; //@line 4093 "lbfgsb.c"
    $413 = HEAP32[2480>>2]|0; //@line 4093 "lbfgsb.c"
    $414 = $14; //@line 4093 "lbfgsb.c"
    $415 = (($414) + ($413<<3)|0); //@line 4093 "lbfgsb.c"
    $416 = +HEAPF64[$415>>3]; //@line 4093 "lbfgsb.c"
    $417 = $412 - $416; //@line 4093 "lbfgsb.c"
    $418 = HEAP32[2480>>2]|0; //@line 4093 "lbfgsb.c"
    $419 = $15; //@line 4093 "lbfgsb.c"
    $420 = (($419) + ($418<<3)|0); //@line 4093 "lbfgsb.c"
    $421 = +HEAPF64[$420>>3]; //@line 4093 "lbfgsb.c"
    $422 = $417 * $421; //@line 4093 "lbfgsb.c"
    $423 = +HEAPF64[488>>3]; //@line 4093 "lbfgsb.c"
    $424 = $423 + $422; //@line 4093 "lbfgsb.c"
    HEAPF64[488>>3] = $424; //@line 4093 "lbfgsb.c"
    $425 = HEAP32[2480>>2]|0; //@line 4092 "lbfgsb.c"
    $426 = (($425) + 1)|0; //@line 4092 "lbfgsb.c"
    HEAP32[2480>>2] = $426; //@line 4092 "lbfgsb.c"
   }
   $427 = +HEAPF64[488>>3]; //@line 4096 "lbfgsb.c"
   $428 = $427 > 0.0; //@line 4096 "lbfgsb.c"
   if ($428) {
    $429 = $1; //@line 4097 "lbfgsb.c"
    $430 = $10; //@line 4097 "lbfgsb.c"
    $431 = ((($430)) + 8|0); //@line 4097 "lbfgsb.c"
    $432 = $8; //@line 4097 "lbfgsb.c"
    $433 = ((($432)) + 8|0); //@line 4097 "lbfgsb.c"
    (_dcopy_($429,$431,1236,$433,1236)|0); //@line 4097 "lbfgsb.c"
    $434 = $21; //@line 4098 "lbfgsb.c"
    $435 = HEAP32[$434>>2]|0; //@line 4098 "lbfgsb.c"
    $436 = ($435|0)>=(0); //@line 4098 "lbfgsb.c"
    if ($436) {
     (_s_wsle(2536)|0); //@line 4099 "lbfgsb.c"
     (_do_lio(1308,1236,17664,39)|0); //@line 4100 "lbfgsb.c"
     (_e_wsle()|0); //@line 4102 "lbfgsb.c"
     (_s_wsle(2556)|0); //@line 4103 "lbfgsb.c"
     (_do_lio(1308,1236,17704,29)|0); //@line 4104 "lbfgsb.c"
     (_e_wsle()|0); //@line 4105 "lbfgsb.c"
    }
    HEAPF64[512>>3] = 1.0; //@line 4113 "lbfgsb.c"
    $437 = +HEAPF64[512>>3]; //@line 4114 "lbfgsb.c"
    HEAPF64[496>>3] = $437; //@line 4114 "lbfgsb.c"
    HEAP32[2504>>2] = 0; //@line 4115 "lbfgsb.c"
    $438 = $3; //@line 4116 "lbfgsb.c"
    $439 = HEAP32[$438>>2]|0; //@line 4116 "lbfgsb.c"
    $i__1 = $439; //@line 4116 "lbfgsb.c"
    HEAP32[2480>>2] = 1; //@line 4117 "lbfgsb.c"
    while(1) {
     $440 = HEAP32[2480>>2]|0; //@line 4117 "lbfgsb.c"
     $441 = $i__1; //@line 4117 "lbfgsb.c"
     $442 = ($440|0)<=($441|0); //@line 4117 "lbfgsb.c"
     if (!($442)) {
      break;
     }
     $443 = HEAP32[2480>>2]|0; //@line 4118 "lbfgsb.c"
     $444 = $4; //@line 4118 "lbfgsb.c"
     $445 = (($444) + ($443<<2)|0); //@line 4118 "lbfgsb.c"
     $446 = HEAP32[$445>>2]|0; //@line 4118 "lbfgsb.c"
     HEAP32[2488>>2] = $446; //@line 4118 "lbfgsb.c"
     $447 = HEAP32[2480>>2]|0; //@line 4119 "lbfgsb.c"
     $448 = $9; //@line 4119 "lbfgsb.c"
     $449 = (($448) + ($447<<3)|0); //@line 4119 "lbfgsb.c"
     $450 = +HEAPF64[$449>>3]; //@line 4119 "lbfgsb.c"
     HEAPF64[472>>3] = $450; //@line 4119 "lbfgsb.c"
     $451 = HEAP32[2488>>2]|0; //@line 4120 "lbfgsb.c"
     $452 = $7; //@line 4120 "lbfgsb.c"
     $453 = (($452) + ($451<<2)|0); //@line 4120 "lbfgsb.c"
     $454 = HEAP32[$453>>2]|0; //@line 4120 "lbfgsb.c"
     $455 = ($454|0)!=(0); //@line 4120 "lbfgsb.c"
     if ($455) {
      $456 = +HEAPF64[472>>3]; //@line 4121 "lbfgsb.c"
      $457 = $456 < 0.0; //@line 4121 "lbfgsb.c"
      do {
       if ($457) {
        $458 = HEAP32[2488>>2]|0; //@line 4121 "lbfgsb.c"
        $459 = $7; //@line 4121 "lbfgsb.c"
        $460 = (($459) + ($458<<2)|0); //@line 4121 "lbfgsb.c"
        $461 = HEAP32[$460>>2]|0; //@line 4121 "lbfgsb.c"
        $462 = ($461|0)<=(2); //@line 4121 "lbfgsb.c"
        if ($462) {
         $463 = HEAP32[2488>>2]|0; //@line 4122 "lbfgsb.c"
         $464 = $5; //@line 4122 "lbfgsb.c"
         $465 = (($464) + ($463<<3)|0); //@line 4122 "lbfgsb.c"
         $466 = +HEAPF64[$465>>3]; //@line 4122 "lbfgsb.c"
         $467 = HEAP32[2488>>2]|0; //@line 4122 "lbfgsb.c"
         $468 = $8; //@line 4122 "lbfgsb.c"
         $469 = (($468) + ($467<<3)|0); //@line 4122 "lbfgsb.c"
         $470 = +HEAPF64[$469>>3]; //@line 4122 "lbfgsb.c"
         $471 = $466 - $470; //@line 4122 "lbfgsb.c"
         HEAPF64[504>>3] = $471; //@line 4122 "lbfgsb.c"
         $472 = +HEAPF64[504>>3]; //@line 4123 "lbfgsb.c"
         $473 = $472 >= 0.0; //@line 4123 "lbfgsb.c"
         if ($473) {
          HEAPF64[496>>3] = 0.0; //@line 4124 "lbfgsb.c"
          break;
         }
         $474 = +HEAPF64[472>>3]; //@line 4125 "lbfgsb.c"
         $475 = +HEAPF64[512>>3]; //@line 4125 "lbfgsb.c"
         $476 = $474 * $475; //@line 4125 "lbfgsb.c"
         $477 = +HEAPF64[504>>3]; //@line 4125 "lbfgsb.c"
         $478 = $476 < $477; //@line 4125 "lbfgsb.c"
         if ($478) {
          $479 = +HEAPF64[504>>3]; //@line 4126 "lbfgsb.c"
          $480 = +HEAPF64[472>>3]; //@line 4126 "lbfgsb.c"
          $481 = $479 / $480; //@line 4126 "lbfgsb.c"
          HEAPF64[496>>3] = $481; //@line 4126 "lbfgsb.c"
         }
        } else {
         label = 55;
        }
       } else {
        label = 55;
       }
      } while(0);
      do {
       if ((label|0) == 55) {
        label = 0;
        $482 = +HEAPF64[472>>3]; //@line 4128 "lbfgsb.c"
        $483 = $482 > 0.0; //@line 4128 "lbfgsb.c"
        if ($483) {
         $484 = HEAP32[2488>>2]|0; //@line 4128 "lbfgsb.c"
         $485 = $7; //@line 4128 "lbfgsb.c"
         $486 = (($485) + ($484<<2)|0); //@line 4128 "lbfgsb.c"
         $487 = HEAP32[$486>>2]|0; //@line 4128 "lbfgsb.c"
         $488 = ($487|0)>=(2); //@line 4128 "lbfgsb.c"
         if ($488) {
          $489 = HEAP32[2488>>2]|0; //@line 4129 "lbfgsb.c"
          $490 = $6; //@line 4129 "lbfgsb.c"
          $491 = (($490) + ($489<<3)|0); //@line 4129 "lbfgsb.c"
          $492 = +HEAPF64[$491>>3]; //@line 4129 "lbfgsb.c"
          $493 = HEAP32[2488>>2]|0; //@line 4129 "lbfgsb.c"
          $494 = $8; //@line 4129 "lbfgsb.c"
          $495 = (($494) + ($493<<3)|0); //@line 4129 "lbfgsb.c"
          $496 = +HEAPF64[$495>>3]; //@line 4129 "lbfgsb.c"
          $497 = $492 - $496; //@line 4129 "lbfgsb.c"
          HEAPF64[504>>3] = $497; //@line 4129 "lbfgsb.c"
          $498 = +HEAPF64[504>>3]; //@line 4130 "lbfgsb.c"
          $499 = $498 <= 0.0; //@line 4130 "lbfgsb.c"
          if ($499) {
           HEAPF64[496>>3] = 0.0; //@line 4131 "lbfgsb.c"
           break;
          }
          $500 = +HEAPF64[472>>3]; //@line 4132 "lbfgsb.c"
          $501 = +HEAPF64[512>>3]; //@line 4132 "lbfgsb.c"
          $502 = $500 * $501; //@line 4132 "lbfgsb.c"
          $503 = +HEAPF64[504>>3]; //@line 4132 "lbfgsb.c"
          $504 = $502 > $503; //@line 4132 "lbfgsb.c"
          if ($504) {
           $505 = +HEAPF64[504>>3]; //@line 4133 "lbfgsb.c"
           $506 = +HEAPF64[472>>3]; //@line 4133 "lbfgsb.c"
           $507 = $505 / $506; //@line 4133 "lbfgsb.c"
           HEAPF64[496>>3] = $507; //@line 4133 "lbfgsb.c"
          }
         }
        }
       }
      } while(0);
      $508 = +HEAPF64[496>>3]; //@line 4136 "lbfgsb.c"
      $509 = +HEAPF64[512>>3]; //@line 4136 "lbfgsb.c"
      $510 = $508 < $509; //@line 4136 "lbfgsb.c"
      if ($510) {
       $511 = +HEAPF64[496>>3]; //@line 4137 "lbfgsb.c"
       HEAPF64[512>>3] = $511; //@line 4137 "lbfgsb.c"
       $512 = HEAP32[2480>>2]|0; //@line 4138 "lbfgsb.c"
       HEAP32[2504>>2] = $512; //@line 4138 "lbfgsb.c"
      }
     }
     $513 = HEAP32[2480>>2]|0; //@line 4117 "lbfgsb.c"
     $514 = (($513) + 1)|0; //@line 4117 "lbfgsb.c"
     HEAP32[2480>>2] = $514; //@line 4117 "lbfgsb.c"
    }
    $515 = +HEAPF64[512>>3]; //@line 4143 "lbfgsb.c"
    $516 = $515 < 1.0; //@line 4143 "lbfgsb.c"
    do {
     if ($516) {
      $517 = HEAP32[2504>>2]|0; //@line 4144 "lbfgsb.c"
      $518 = $9; //@line 4144 "lbfgsb.c"
      $519 = (($518) + ($517<<3)|0); //@line 4144 "lbfgsb.c"
      $520 = +HEAPF64[$519>>3]; //@line 4144 "lbfgsb.c"
      HEAPF64[472>>3] = $520; //@line 4144 "lbfgsb.c"
      $521 = HEAP32[2504>>2]|0; //@line 4145 "lbfgsb.c"
      $522 = $4; //@line 4145 "lbfgsb.c"
      $523 = (($522) + ($521<<2)|0); //@line 4145 "lbfgsb.c"
      $524 = HEAP32[$523>>2]|0; //@line 4145 "lbfgsb.c"
      HEAP32[2488>>2] = $524; //@line 4145 "lbfgsb.c"
      $525 = +HEAPF64[472>>3]; //@line 4146 "lbfgsb.c"
      $526 = $525 > 0.0; //@line 4146 "lbfgsb.c"
      if ($526) {
       $527 = HEAP32[2488>>2]|0; //@line 4147 "lbfgsb.c"
       $528 = $6; //@line 4147 "lbfgsb.c"
       $529 = (($528) + ($527<<3)|0); //@line 4147 "lbfgsb.c"
       $530 = +HEAPF64[$529>>3]; //@line 4147 "lbfgsb.c"
       $531 = HEAP32[2488>>2]|0; //@line 4147 "lbfgsb.c"
       $532 = $8; //@line 4147 "lbfgsb.c"
       $533 = (($532) + ($531<<3)|0); //@line 4147 "lbfgsb.c"
       HEAPF64[$533>>3] = $530; //@line 4147 "lbfgsb.c"
       $534 = HEAP32[2504>>2]|0; //@line 4148 "lbfgsb.c"
       $535 = $9; //@line 4148 "lbfgsb.c"
       $536 = (($535) + ($534<<3)|0); //@line 4148 "lbfgsb.c"
       HEAPF64[$536>>3] = 0.0; //@line 4148 "lbfgsb.c"
       break;
      }
      $537 = +HEAPF64[472>>3]; //@line 4149 "lbfgsb.c"
      $538 = $537 < 0.0; //@line 4149 "lbfgsb.c"
      if ($538) {
       $539 = HEAP32[2488>>2]|0; //@line 4150 "lbfgsb.c"
       $540 = $5; //@line 4150 "lbfgsb.c"
       $541 = (($540) + ($539<<3)|0); //@line 4150 "lbfgsb.c"
       $542 = +HEAPF64[$541>>3]; //@line 4150 "lbfgsb.c"
       $543 = HEAP32[2488>>2]|0; //@line 4150 "lbfgsb.c"
       $544 = $8; //@line 4150 "lbfgsb.c"
       $545 = (($544) + ($543<<3)|0); //@line 4150 "lbfgsb.c"
       HEAPF64[$545>>3] = $542; //@line 4150 "lbfgsb.c"
       $546 = HEAP32[2504>>2]|0; //@line 4151 "lbfgsb.c"
       $547 = $9; //@line 4151 "lbfgsb.c"
       $548 = (($547) + ($546<<3)|0); //@line 4151 "lbfgsb.c"
       HEAPF64[$548>>3] = 0.0; //@line 4151 "lbfgsb.c"
      }
     }
    } while(0);
    $549 = $3; //@line 4154 "lbfgsb.c"
    $550 = HEAP32[$549>>2]|0; //@line 4154 "lbfgsb.c"
    $i__1 = $550; //@line 4154 "lbfgsb.c"
    HEAP32[2480>>2] = 1; //@line 4155 "lbfgsb.c"
    while(1) {
     $551 = HEAP32[2480>>2]|0; //@line 4155 "lbfgsb.c"
     $552 = $i__1; //@line 4155 "lbfgsb.c"
     $553 = ($551|0)<=($552|0); //@line 4155 "lbfgsb.c"
     if (!($553)) {
      break L56;
     }
     $554 = HEAP32[2480>>2]|0; //@line 4156 "lbfgsb.c"
     $555 = $4; //@line 4156 "lbfgsb.c"
     $556 = (($555) + ($554<<2)|0); //@line 4156 "lbfgsb.c"
     $557 = HEAP32[$556>>2]|0; //@line 4156 "lbfgsb.c"
     HEAP32[2488>>2] = $557; //@line 4156 "lbfgsb.c"
     $558 = +HEAPF64[512>>3]; //@line 4157 "lbfgsb.c"
     $559 = HEAP32[2480>>2]|0; //@line 4157 "lbfgsb.c"
     $560 = $9; //@line 4157 "lbfgsb.c"
     $561 = (($560) + ($559<<3)|0); //@line 4157 "lbfgsb.c"
     $562 = +HEAPF64[$561>>3]; //@line 4157 "lbfgsb.c"
     $563 = $558 * $562; //@line 4157 "lbfgsb.c"
     $564 = HEAP32[2488>>2]|0; //@line 4157 "lbfgsb.c"
     $565 = $8; //@line 4157 "lbfgsb.c"
     $566 = (($565) + ($564<<3)|0); //@line 4157 "lbfgsb.c"
     $567 = +HEAPF64[$566>>3]; //@line 4157 "lbfgsb.c"
     $568 = $567 + $563; //@line 4157 "lbfgsb.c"
     HEAPF64[$566>>3] = $568; //@line 4157 "lbfgsb.c"
     $569 = HEAP32[2480>>2]|0; //@line 4155 "lbfgsb.c"
     $570 = (($569) + 1)|0; //@line 4155 "lbfgsb.c"
     HEAP32[2480>>2] = $570; //@line 4155 "lbfgsb.c"
    }
   }
  }
 } while(0);
 $571 = $21; //@line 4162 "lbfgsb.c"
 $572 = HEAP32[$571>>2]|0; //@line 4162 "lbfgsb.c"
 $573 = ($572|0)>=(99); //@line 4162 "lbfgsb.c"
 if ($573) {
  (_s_wsfe(2576)|0); //@line 4163 "lbfgsb.c"
  (_e_wsfe()|0); //@line 4164 "lbfgsb.c"
 }
 $0 = 0; //@line 4166 "lbfgsb.c"
 $574 = $0; //@line 4167 "lbfgsb.c"
 STACKTOP = sp;return ($574|0); //@line 4167 "lbfgsb.c"
}
function _lnsrlb_($n,$l,$u,$nbd,$x,$f,$fold,$gd,$gdold,$g,$d__,$r__,$t,$z__,$stp,$dnorm,$dtd,$xstep,$stpmx,$iter,$ifun,$iback,$nfgv,$info,$task,$boxed,$cnstnd,$csave,$isave,$dsave,$iprint,$task_len,$csave_len) {
 $n = $n|0;
 $l = $l|0;
 $u = $u|0;
 $nbd = $nbd|0;
 $x = $x|0;
 $f = $f|0;
 $fold = $fold|0;
 $gd = $gd|0;
 $gdold = $gdold|0;
 $g = $g|0;
 $d__ = $d__|0;
 $r__ = $r__|0;
 $t = $t|0;
 $z__ = $z__|0;
 $stp = $stp|0;
 $dnorm = $dnorm|0;
 $dtd = $dtd|0;
 $xstep = $xstep|0;
 $stpmx = $stpmx|0;
 $iter = $iter|0;
 $ifun = $ifun|0;
 $iback = $iback|0;
 $nfgv = $nfgv|0;
 $info = $info|0;
 $task = $task|0;
 $boxed = $boxed|0;
 $cnstnd = $cnstnd|0;
 $csave = $csave|0;
 $isave = $isave|0;
 $dsave = $dsave|0;
 $iprint = $iprint|0;
 $task_len = $task_len|0;
 $csave_len = $csave_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0.0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0.0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0.0, $143 = 0.0, $144 = 0.0, $145 = 0, $146 = 0.0, $147 = 0.0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0, $163 = 0.0, $164 = 0, $165 = 0.0, $166 = 0, $167 = 0.0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0.0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0.0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0.0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0.0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0.0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0.0, $263 = 0.0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d__1 = 0.0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $l;
 $3 = $u;
 $4 = $nbd;
 $5 = $x;
 $6 = $f;
 $7 = $fold;
 $8 = $gd;
 $9 = $gdold;
 $10 = $g;
 $11 = $d__;
 $12 = $r__;
 $13 = $t;
 $14 = $z__;
 $15 = $stp;
 $16 = $dnorm;
 $17 = $dtd;
 $18 = $xstep;
 $19 = $stpmx;
 $20 = $iter;
 $21 = $ifun;
 $22 = $iback;
 $23 = $nfgv;
 $24 = $info;
 $25 = $task;
 $26 = $boxed;
 $27 = $cnstnd;
 $28 = $csave;
 $29 = $isave;
 $30 = $dsave;
 $31 = $iprint;
 $32 = $task_len;
 $33 = $csave_len;
 $34 = $14; //@line 3034 "lbfgsb.c"
 $35 = ((($34)) + -8|0); //@line 3034 "lbfgsb.c"
 $14 = $35; //@line 3034 "lbfgsb.c"
 $36 = $13; //@line 3035 "lbfgsb.c"
 $37 = ((($36)) + -8|0); //@line 3035 "lbfgsb.c"
 $13 = $37; //@line 3035 "lbfgsb.c"
 $38 = $12; //@line 3036 "lbfgsb.c"
 $39 = ((($38)) + -8|0); //@line 3036 "lbfgsb.c"
 $12 = $39; //@line 3036 "lbfgsb.c"
 $40 = $11; //@line 3037 "lbfgsb.c"
 $41 = ((($40)) + -8|0); //@line 3037 "lbfgsb.c"
 $11 = $41; //@line 3037 "lbfgsb.c"
 $42 = $10; //@line 3038 "lbfgsb.c"
 $43 = ((($42)) + -8|0); //@line 3038 "lbfgsb.c"
 $10 = $43; //@line 3038 "lbfgsb.c"
 $44 = $5; //@line 3039 "lbfgsb.c"
 $45 = ((($44)) + -8|0); //@line 3039 "lbfgsb.c"
 $5 = $45; //@line 3039 "lbfgsb.c"
 $46 = $4; //@line 3040 "lbfgsb.c"
 $47 = ((($46)) + -4|0); //@line 3040 "lbfgsb.c"
 $4 = $47; //@line 3040 "lbfgsb.c"
 $48 = $3; //@line 3041 "lbfgsb.c"
 $49 = ((($48)) + -8|0); //@line 3041 "lbfgsb.c"
 $3 = $49; //@line 3041 "lbfgsb.c"
 $50 = $2; //@line 3042 "lbfgsb.c"
 $51 = ((($50)) + -8|0); //@line 3042 "lbfgsb.c"
 $2 = $51; //@line 3042 "lbfgsb.c"
 $52 = $29; //@line 3043 "lbfgsb.c"
 $53 = ((($52)) + -4|0); //@line 3043 "lbfgsb.c"
 $29 = $53; //@line 3043 "lbfgsb.c"
 $54 = $30; //@line 3044 "lbfgsb.c"
 $55 = ((($54)) + -8|0); //@line 3044 "lbfgsb.c"
 $30 = $55; //@line 3044 "lbfgsb.c"
 $56 = $25; //@line 3047 "lbfgsb.c"
 $57 = (_s_cmp($56,14362,5,5)|0); //@line 3047 "lbfgsb.c"
 $58 = ($57|0)==(0); //@line 3047 "lbfgsb.c"
 if (!($58)) {
  $59 = $1; //@line 3050 "lbfgsb.c"
  $60 = $11; //@line 3050 "lbfgsb.c"
  $61 = ((($60)) + 8|0); //@line 3050 "lbfgsb.c"
  $62 = $11; //@line 3050 "lbfgsb.c"
  $63 = ((($62)) + 8|0); //@line 3050 "lbfgsb.c"
  $64 = (+_ddot_($59,$61,1236,$63,1236)); //@line 3050 "lbfgsb.c"
  $65 = $17; //@line 3050 "lbfgsb.c"
  HEAPF64[$65>>3] = $64; //@line 3050 "lbfgsb.c"
  $66 = $17; //@line 3051 "lbfgsb.c"
  $67 = +HEAPF64[$66>>3]; //@line 3051 "lbfgsb.c"
  $68 = (+Math_sqrt((+$67))); //@line 3051 "lbfgsb.c"
  $69 = $16; //@line 3051 "lbfgsb.c"
  HEAPF64[$69>>3] = $68; //@line 3051 "lbfgsb.c"
  $70 = $19; //@line 3053 "lbfgsb.c"
  HEAPF64[$70>>3] = 1.0E+10; //@line 3053 "lbfgsb.c"
  $71 = $27; //@line 3054 "lbfgsb.c"
  $72 = HEAP32[$71>>2]|0; //@line 3054 "lbfgsb.c"
  $73 = ($72|0)!=(0); //@line 3054 "lbfgsb.c"
  L3: do {
   if ($73) {
    $74 = $20; //@line 3055 "lbfgsb.c"
    $75 = HEAP32[$74>>2]|0; //@line 3055 "lbfgsb.c"
    $76 = ($75|0)==(0); //@line 3055 "lbfgsb.c"
    if ($76) {
     $77 = $19; //@line 3056 "lbfgsb.c"
     HEAPF64[$77>>3] = 1.0; //@line 3056 "lbfgsb.c"
     break;
    }
    $78 = $1; //@line 3058 "lbfgsb.c"
    $79 = HEAP32[$78>>2]|0; //@line 3058 "lbfgsb.c"
    $i__1 = $79; //@line 3058 "lbfgsb.c"
    HEAP32[1884>>2] = 1; //@line 3059 "lbfgsb.c"
    while(1) {
     $80 = HEAP32[1884>>2]|0; //@line 3059 "lbfgsb.c"
     $81 = $i__1; //@line 3059 "lbfgsb.c"
     $82 = ($80|0)<=($81|0); //@line 3059 "lbfgsb.c"
     if (!($82)) {
      break L3;
     }
     $83 = HEAP32[1884>>2]|0; //@line 3060 "lbfgsb.c"
     $84 = $11; //@line 3060 "lbfgsb.c"
     $85 = (($84) + ($83<<3)|0); //@line 3060 "lbfgsb.c"
     $86 = +HEAPF64[$85>>3]; //@line 3060 "lbfgsb.c"
     HEAPF64[424>>3] = $86; //@line 3060 "lbfgsb.c"
     $87 = HEAP32[1884>>2]|0; //@line 3061 "lbfgsb.c"
     $88 = $4; //@line 3061 "lbfgsb.c"
     $89 = (($88) + ($87<<2)|0); //@line 3061 "lbfgsb.c"
     $90 = HEAP32[$89>>2]|0; //@line 3061 "lbfgsb.c"
     $91 = ($90|0)!=(0); //@line 3061 "lbfgsb.c"
     do {
      if ($91) {
       $92 = +HEAPF64[424>>3]; //@line 3062 "lbfgsb.c"
       $93 = $92 < 0.0; //@line 3062 "lbfgsb.c"
       if ($93) {
        $94 = HEAP32[1884>>2]|0; //@line 3062 "lbfgsb.c"
        $95 = $4; //@line 3062 "lbfgsb.c"
        $96 = (($95) + ($94<<2)|0); //@line 3062 "lbfgsb.c"
        $97 = HEAP32[$96>>2]|0; //@line 3062 "lbfgsb.c"
        $98 = ($97|0)<=(2); //@line 3062 "lbfgsb.c"
        if ($98) {
         $99 = HEAP32[1884>>2]|0; //@line 3063 "lbfgsb.c"
         $100 = $2; //@line 3063 "lbfgsb.c"
         $101 = (($100) + ($99<<3)|0); //@line 3063 "lbfgsb.c"
         $102 = +HEAPF64[$101>>3]; //@line 3063 "lbfgsb.c"
         $103 = HEAP32[1884>>2]|0; //@line 3063 "lbfgsb.c"
         $104 = $5; //@line 3063 "lbfgsb.c"
         $105 = (($104) + ($103<<3)|0); //@line 3063 "lbfgsb.c"
         $106 = +HEAPF64[$105>>3]; //@line 3063 "lbfgsb.c"
         $107 = $102 - $106; //@line 3063 "lbfgsb.c"
         HEAPF64[432>>3] = $107; //@line 3063 "lbfgsb.c"
         $108 = +HEAPF64[432>>3]; //@line 3064 "lbfgsb.c"
         $109 = $108 >= 0.0; //@line 3064 "lbfgsb.c"
         if ($109) {
          $110 = $19; //@line 3065 "lbfgsb.c"
          HEAPF64[$110>>3] = 0.0; //@line 3065 "lbfgsb.c"
          break;
         }
         $111 = +HEAPF64[424>>3]; //@line 3066 "lbfgsb.c"
         $112 = $19; //@line 3066 "lbfgsb.c"
         $113 = +HEAPF64[$112>>3]; //@line 3066 "lbfgsb.c"
         $114 = $111 * $113; //@line 3066 "lbfgsb.c"
         $115 = +HEAPF64[432>>3]; //@line 3066 "lbfgsb.c"
         $116 = $114 < $115; //@line 3066 "lbfgsb.c"
         if (!($116)) {
          break;
         }
         $117 = +HEAPF64[432>>3]; //@line 3067 "lbfgsb.c"
         $118 = +HEAPF64[424>>3]; //@line 3067 "lbfgsb.c"
         $119 = $117 / $118; //@line 3067 "lbfgsb.c"
         $120 = $19; //@line 3067 "lbfgsb.c"
         HEAPF64[$120>>3] = $119; //@line 3067 "lbfgsb.c"
         break;
        }
       }
       $121 = +HEAPF64[424>>3]; //@line 3069 "lbfgsb.c"
       $122 = $121 > 0.0; //@line 3069 "lbfgsb.c"
       if ($122) {
        $123 = HEAP32[1884>>2]|0; //@line 3069 "lbfgsb.c"
        $124 = $4; //@line 3069 "lbfgsb.c"
        $125 = (($124) + ($123<<2)|0); //@line 3069 "lbfgsb.c"
        $126 = HEAP32[$125>>2]|0; //@line 3069 "lbfgsb.c"
        $127 = ($126|0)>=(2); //@line 3069 "lbfgsb.c"
        if ($127) {
         $128 = HEAP32[1884>>2]|0; //@line 3070 "lbfgsb.c"
         $129 = $3; //@line 3070 "lbfgsb.c"
         $130 = (($129) + ($128<<3)|0); //@line 3070 "lbfgsb.c"
         $131 = +HEAPF64[$130>>3]; //@line 3070 "lbfgsb.c"
         $132 = HEAP32[1884>>2]|0; //@line 3070 "lbfgsb.c"
         $133 = $5; //@line 3070 "lbfgsb.c"
         $134 = (($133) + ($132<<3)|0); //@line 3070 "lbfgsb.c"
         $135 = +HEAPF64[$134>>3]; //@line 3070 "lbfgsb.c"
         $136 = $131 - $135; //@line 3070 "lbfgsb.c"
         HEAPF64[432>>3] = $136; //@line 3070 "lbfgsb.c"
         $137 = +HEAPF64[432>>3]; //@line 3071 "lbfgsb.c"
         $138 = $137 <= 0.0; //@line 3071 "lbfgsb.c"
         if ($138) {
          $139 = $19; //@line 3072 "lbfgsb.c"
          HEAPF64[$139>>3] = 0.0; //@line 3072 "lbfgsb.c"
          break;
         }
         $140 = +HEAPF64[424>>3]; //@line 3073 "lbfgsb.c"
         $141 = $19; //@line 3073 "lbfgsb.c"
         $142 = +HEAPF64[$141>>3]; //@line 3073 "lbfgsb.c"
         $143 = $140 * $142; //@line 3073 "lbfgsb.c"
         $144 = +HEAPF64[432>>3]; //@line 3073 "lbfgsb.c"
         $145 = $143 > $144; //@line 3073 "lbfgsb.c"
         if ($145) {
          $146 = +HEAPF64[432>>3]; //@line 3074 "lbfgsb.c"
          $147 = +HEAPF64[424>>3]; //@line 3074 "lbfgsb.c"
          $148 = $146 / $147; //@line 3074 "lbfgsb.c"
          $149 = $19; //@line 3074 "lbfgsb.c"
          HEAPF64[$149>>3] = $148; //@line 3074 "lbfgsb.c"
         }
        }
       }
      }
     } while(0);
     $150 = HEAP32[1884>>2]|0; //@line 3059 "lbfgsb.c"
     $151 = (($150) + 1)|0; //@line 3059 "lbfgsb.c"
     HEAP32[1884>>2] = $151; //@line 3059 "lbfgsb.c"
    }
   }
  } while(0);
  $152 = $20; //@line 3082 "lbfgsb.c"
  $153 = HEAP32[$152>>2]|0; //@line 3082 "lbfgsb.c"
  $154 = ($153|0)==(0); //@line 3082 "lbfgsb.c"
  if ($154) {
   $155 = $26; //@line 3082 "lbfgsb.c"
   $156 = HEAP32[$155>>2]|0; //@line 3082 "lbfgsb.c"
   $157 = ($156|0)!=(0); //@line 3082 "lbfgsb.c"
   if ($157) {
    label = 27;
   } else {
    $158 = $16; //@line 3084 "lbfgsb.c"
    $159 = +HEAPF64[$158>>3]; //@line 3084 "lbfgsb.c"
    $160 = 1.0 / $159; //@line 3084 "lbfgsb.c"
    $d__1 = $160; //@line 3084 "lbfgsb.c"
    $161 = $d__1; //@line 3085 "lbfgsb.c"
    $162 = $19; //@line 3085 "lbfgsb.c"
    $163 = +HEAPF64[$162>>3]; //@line 3085 "lbfgsb.c"
    $164 = $161 <= $163; //@line 3085 "lbfgsb.c"
    if ($164) {
     $165 = $d__1; //@line 3085 "lbfgsb.c"
     $169 = $165;
    } else {
     $166 = $19; //@line 3085 "lbfgsb.c"
     $167 = +HEAPF64[$166>>3]; //@line 3085 "lbfgsb.c"
     $169 = $167;
    }
    $168 = $15; //@line 3085 "lbfgsb.c"
    HEAPF64[$168>>3] = $169; //@line 3085 "lbfgsb.c"
   }
  } else {
   label = 27;
  }
  if ((label|0) == 27) {
   $170 = $15; //@line 3087 "lbfgsb.c"
   HEAPF64[$170>>3] = 1.0; //@line 3087 "lbfgsb.c"
  }
  $171 = $1; //@line 3089 "lbfgsb.c"
  $172 = $5; //@line 3089 "lbfgsb.c"
  $173 = ((($172)) + 8|0); //@line 3089 "lbfgsb.c"
  $174 = $13; //@line 3089 "lbfgsb.c"
  $175 = ((($174)) + 8|0); //@line 3089 "lbfgsb.c"
  (_dcopy_($171,$173,1236,$175,1236)|0); //@line 3089 "lbfgsb.c"
  $176 = $1; //@line 3090 "lbfgsb.c"
  $177 = $10; //@line 3090 "lbfgsb.c"
  $178 = ((($177)) + 8|0); //@line 3090 "lbfgsb.c"
  $179 = $12; //@line 3090 "lbfgsb.c"
  $180 = ((($179)) + 8|0); //@line 3090 "lbfgsb.c"
  (_dcopy_($176,$178,1236,$180,1236)|0); //@line 3090 "lbfgsb.c"
  $181 = $6; //@line 3091 "lbfgsb.c"
  $182 = +HEAPF64[$181>>3]; //@line 3091 "lbfgsb.c"
  $183 = $7; //@line 3091 "lbfgsb.c"
  HEAPF64[$183>>3] = $182; //@line 3091 "lbfgsb.c"
  $184 = $21; //@line 3092 "lbfgsb.c"
  HEAP32[$184>>2] = 0; //@line 3092 "lbfgsb.c"
  $185 = $22; //@line 3093 "lbfgsb.c"
  HEAP32[$185>>2] = 0; //@line 3093 "lbfgsb.c"
  $186 = $28; //@line 3094 "lbfgsb.c"
  _s_copy($186,13740,60,5); //@line 3094 "lbfgsb.c"
 }
 $187 = $1; //@line 3096 "lbfgsb.c"
 $188 = $10; //@line 3096 "lbfgsb.c"
 //console.log('g at 3096: ' + HEAPF64[($10+8)>>3]);
 $189 = ((($188)) + 8|0); //@line 3096 "lbfgsb.c"
 $190 = $11; //@line 3096 "lbfgsb.c"
 //console.log('d__ at 3096: ' + HEAPF64[($11+8)>>3]);
 $191 = ((($190)) + 8|0); //@line 3096 "lbfgsb.c"
 $192 = (+_ddot_($187,$189,1236,$191,1236)); //@line 3096 "lbfgsb.c"
 $193 = $8; //@line 3096 "lbfgsb.c"
 HEAPF64[$193>>3] = $192; //@line 3096 "lbfgsb.c"
 $194 = $21; //@line 3097 "lbfgsb.c"
 $195 = HEAP32[$194>>2]|0; //@line 3097 "lbfgsb.c"
 $196 = ($195|0)==(0); //@line 3097 "lbfgsb.c"
 if ($196) {
  $197 = $8; //@line 3098 "lbfgsb.c"
  $198 = +HEAPF64[$197>>3]; //@line 3098 "lbfgsb.c"
  $199 = $9; //@line 3098 "lbfgsb.c"
  HEAPF64[$199>>3] = $198; //@line 3098 "lbfgsb.c"
  $200 = $8; //@line 3099 "lbfgsb.c"
  $201 = +HEAPF64[$200>>3]; //@line 3099 "lbfgsb.c"
  $202 = $201 >= 0.0; //@line 3099 "lbfgsb.c"
  if ($202) {
   $203 = $31; //@line 3102 "lbfgsb.c"
   $204 = HEAP32[$203>>2]|0; //@line 3102 "lbfgsb.c"
   $205 = ($204|0)>=(0); //@line 3102 "lbfgsb.c"
   if ($205) {
    (_s_wsle(1888)|0); //@line 3103 "lbfgsb.c"
    (_do_lio(1308,1236,15408,37)|0); //@line 3104 "lbfgsb.c"
    $206 = $8; //@line 3106 "lbfgsb.c"
    (_do_lio(1908,1236,$206,8)|0); //@line 3106 "lbfgsb.c"
    (_e_wsle()|0); //@line 3108 "lbfgsb.c"
   }
   $207 = $24; //@line 3110 "lbfgsb.c"
   HEAP32[$207>>2] = -4; //@line 3110 "lbfgsb.c"
   $0 = 0; //@line 3111 "lbfgsb.c"
   $270 = $0; //@line 3136 "lbfgsb.c"
   STACKTOP = sp;return ($270|0); //@line 3136 "lbfgsb.c"
  }
 }
 $208 = $6; //@line 3114 "lbfgsb.c"
 $209 = $8; //@line 3114 "lbfgsb.c"
 $210 = $15; //@line 3114 "lbfgsb.c"
 $211 = $19; //@line 3114 "lbfgsb.c"
 $212 = $28; //@line 3114 "lbfgsb.c"
 $213 = $29; //@line 3115 "lbfgsb.c"
 $214 = ((($213)) + 4|0); //@line 3115 "lbfgsb.c"
 $215 = $30; //@line 3115 "lbfgsb.c"
 $216 = ((($215)) + 8|0); //@line 3115 "lbfgsb.c"
 (_dcsrch_($208,$209,$210,440,448,456,192,$211,$212,$214,$216,60)|0); //@line 3114 "lbfgsb.c"
 $217 = $15; //@line 3116 "lbfgsb.c"
 $218 = +HEAPF64[$217>>3]; //@line 3116 "lbfgsb.c"
 $219 = $16; //@line 3116 "lbfgsb.c"
 $220 = +HEAPF64[$219>>3]; //@line 3116 "lbfgsb.c"
 $221 = $218 * $220; //@line 3116 "lbfgsb.c"
 $222 = $18; //@line 3116 "lbfgsb.c"
 HEAPF64[$222>>3] = $221; //@line 3116 "lbfgsb.c"
 $223 = $28; //@line 3117 "lbfgsb.c"
 $224 = (_s_cmp($223,15446,4,4)|0); //@line 3117 "lbfgsb.c"
 $225 = ($224|0)!=(0); //@line 3117 "lbfgsb.c"
 L48: do {
  if ($225) {
   $226 = $28; //@line 3117 "lbfgsb.c"
   $227 = (_s_cmp($226,15451,4,4)|0); //@line 3117 "lbfgsb.c"
   $228 = ($227|0)!=(0); //@line 3118 "lbfgsb.c"
   if ($228) {
    $229 = $25; //@line 3119 "lbfgsb.c"
    _s_copy($229,15456,60,9); //@line 3119 "lbfgsb.c"
    $230 = $21; //@line 3120 "lbfgsb.c"
    $231 = HEAP32[$230>>2]|0; //@line 3120 "lbfgsb.c"
    $232 = (($231) + 1)|0; //@line 3120 "lbfgsb.c"
    HEAP32[$230>>2] = $232; //@line 3120 "lbfgsb.c"
    $233 = $23; //@line 3121 "lbfgsb.c"
    $234 = HEAP32[$233>>2]|0; //@line 3121 "lbfgsb.c"
    $235 = (($234) + 1)|0; //@line 3121 "lbfgsb.c"
    HEAP32[$233>>2] = $235; //@line 3121 "lbfgsb.c"
    $236 = $21; //@line 3122 "lbfgsb.c"
    $237 = HEAP32[$236>>2]|0; //@line 3122 "lbfgsb.c"
    $238 = (($237) - 1)|0; //@line 3122 "lbfgsb.c"
    $239 = $22; //@line 3122 "lbfgsb.c"
    HEAP32[$239>>2] = $238; //@line 3122 "lbfgsb.c"
    $240 = $15; //@line 3123 "lbfgsb.c"
    $241 = +HEAPF64[$240>>3]; //@line 3123 "lbfgsb.c"
    $242 = $241 == 1.0; //@line 3123 "lbfgsb.c"
    $243 = $1; //@line 3124 "lbfgsb.c"
    if ($242) {
     $244 = $14; //@line 3124 "lbfgsb.c"
     $245 = ((($244)) + 8|0); //@line 3124 "lbfgsb.c"
     $246 = $5; //@line 3124 "lbfgsb.c"
     $247 = ((($246)) + 8|0); //@line 3124 "lbfgsb.c"
     (_dcopy_($243,$245,1236,$247,1236)|0); //@line 3124 "lbfgsb.c"
     break;
    }
    $248 = HEAP32[$243>>2]|0; //@line 3126 "lbfgsb.c"
    $i__1 = $248; //@line 3126 "lbfgsb.c"
    HEAP32[1884>>2] = 1; //@line 3127 "lbfgsb.c"
    while(1) {
     $249 = HEAP32[1884>>2]|0; //@line 3127 "lbfgsb.c"
     $250 = $i__1; //@line 3127 "lbfgsb.c"
     $251 = ($249|0)<=($250|0); //@line 3127 "lbfgsb.c"
     if (!($251)) {
      break L48;
     }
     $252 = $15; //@line 3128 "lbfgsb.c"
     $253 = +HEAPF64[$252>>3]; //@line 3128 "lbfgsb.c"
     $254 = HEAP32[1884>>2]|0; //@line 3128 "lbfgsb.c"
     $255 = $11; //@line 3128 "lbfgsb.c"
     $256 = (($255) + ($254<<3)|0); //@line 3128 "lbfgsb.c"
     $257 = +HEAPF64[$256>>3]; //@line 3128 "lbfgsb.c"
     $258 = $253 * $257; //@line 3128 "lbfgsb.c"
     $259 = HEAP32[1884>>2]|0; //@line 3128 "lbfgsb.c"
     $260 = $13; //@line 3128 "lbfgsb.c"
     $261 = (($260) + ($259<<3)|0); //@line 3128 "lbfgsb.c"
     $262 = +HEAPF64[$261>>3]; //@line 3128 "lbfgsb.c"
     $263 = $258 + $262; //@line 3128 "lbfgsb.c"
     $264 = HEAP32[1884>>2]|0; //@line 3128 "lbfgsb.c"
     $265 = $5; //@line 3128 "lbfgsb.c"
     $266 = (($265) + ($264<<3)|0); //@line 3128 "lbfgsb.c"
     HEAPF64[$266>>3] = $263; //@line 3128 "lbfgsb.c"
     $267 = HEAP32[1884>>2]|0; //@line 3127 "lbfgsb.c"
     $268 = (($267) + 1)|0; //@line 3127 "lbfgsb.c"
     HEAP32[1884>>2] = $268; //@line 3127 "lbfgsb.c"
    }
   } else {
    label = 41;
   }
  } else {
   label = 41;
  }
 } while(0);
 if ((label|0) == 41) {
  $269 = $25; //@line 3133 "lbfgsb.c"
  _s_copy($269,14368,60,5); //@line 3133 "lbfgsb.c"
 }
 $0 = 0; //@line 3135 "lbfgsb.c"
 $270 = $0; //@line 3136 "lbfgsb.c"
 STACKTOP = sp;return ($270|0); //@line 3136 "lbfgsb.c"
}
function _prn2lb_($n,$x,$f,$g,$iprint,$iter,$nfgv,$nact,$sbgnrm,$nseg,$word,$iword,$iback,$stp,$xstep,$word_len) {
 $n = $n|0;
 $x = $x|0;
 $f = $f|0;
 $g = $g|0;
 $iprint = $iprint|0;
 $iter = $iter|0;
 $nfgv = $nfgv|0;
 $nact = $nact|0;
 $sbgnrm = $sbgnrm|0;
 $nseg = $nseg|0;
 $word = $word|0;
 $iword = $iword|0;
 $iback = $iback|0;
 $stp = $stp|0;
 $xstep = $xstep|0;
 $word_len = $word_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $x;
 $2 = $f;
 $3 = $g;
 $4 = $iprint;
 $5 = $iter;
 $6 = $nfgv;
 $7 = $nact;
 $8 = $sbgnrm;
 $9 = $nseg;
 $10 = $word;
 $11 = $iword;
 $12 = $iback;
 $13 = $stp;
 $14 = $xstep;
 $15 = $word_len;
 $16 = $3; //@line 3396 "lbfgsb.c"
 $17 = ((($16)) + -8|0); //@line 3396 "lbfgsb.c"
 $3 = $17; //@line 3396 "lbfgsb.c"
 $18 = $1; //@line 3397 "lbfgsb.c"
 $19 = ((($18)) + -8|0); //@line 3397 "lbfgsb.c"
 $1 = $19; //@line 3397 "lbfgsb.c"
 $20 = $11; //@line 3400 "lbfgsb.c"
 $21 = HEAP32[$20>>2]|0; //@line 3400 "lbfgsb.c"
 $22 = ($21|0)==(0); //@line 3400 "lbfgsb.c"
 do {
  if ($22) {
   $23 = $10; //@line 3402 "lbfgsb.c"
   _s_copy($23,15732,3,3); //@line 3402 "lbfgsb.c"
  } else {
   $24 = $11; //@line 3403 "lbfgsb.c"
   $25 = HEAP32[$24>>2]|0; //@line 3403 "lbfgsb.c"
   $26 = ($25|0)==(1); //@line 3403 "lbfgsb.c"
   if ($26) {
    $27 = $10; //@line 3405 "lbfgsb.c"
    _s_copy($27,15736,3,3); //@line 3405 "lbfgsb.c"
    break;
   }
   $28 = $11; //@line 3406 "lbfgsb.c"
   $29 = HEAP32[$28>>2]|0; //@line 3406 "lbfgsb.c"
   $30 = ($29|0)==(5); //@line 3406 "lbfgsb.c"
   $31 = $10; //@line 3408 "lbfgsb.c"
   if ($30) {
    _s_copy($31,15740,3,3); //@line 3408 "lbfgsb.c"
    break;
   } else {
    _s_copy($31,14352,3,3); //@line 3410 "lbfgsb.c"
    break;
   }
  }
 } while(0);
 $32 = $4; //@line 3412 "lbfgsb.c"
 $33 = HEAP32[$32>>2]|0; //@line 3412 "lbfgsb.c"
 $34 = ($33|0)>=(99); //@line 3412 "lbfgsb.c"
 if (!($34)) {
  $63 = $4; //@line 3440 "lbfgsb.c"
  $64 = HEAP32[$63>>2]|0; //@line 3440 "lbfgsb.c"
  $65 = ($64|0)>(0); //@line 3440 "lbfgsb.c"
  if (!($65)) {
   STACKTOP = sp;return 0; //@line 3451 "lbfgsb.c"
  }
  $66 = $5; //@line 3441 "lbfgsb.c"
  $67 = HEAP32[$66>>2]|0; //@line 3441 "lbfgsb.c"
  $68 = $4; //@line 3441 "lbfgsb.c"
  $69 = HEAP32[$68>>2]|0; //@line 3441 "lbfgsb.c"
  $70 = (($67|0) % ($69|0))&-1; //@line 3441 "lbfgsb.c"
  HEAP32[2028>>2] = $70; //@line 3441 "lbfgsb.c"
  $71 = HEAP32[2028>>2]|0; //@line 3442 "lbfgsb.c"
  $72 = ($71|0)==(0); //@line 3442 "lbfgsb.c"
  if (!($72)) {
   STACKTOP = sp;return 0; //@line 3451 "lbfgsb.c"
  }
  (_s_wsfe(2112)|0); //@line 3443 "lbfgsb.c"
  $73 = $5; //@line 3444 "lbfgsb.c"
  (_do_fio(1236,$73,4)|0); //@line 3444 "lbfgsb.c"
  $74 = $2; //@line 3445 "lbfgsb.c"
  (_do_fio(1236,$74,8)|0); //@line 3445 "lbfgsb.c"
  $75 = $8; //@line 3446 "lbfgsb.c"
  (_do_fio(1236,$75,8)|0); //@line 3446 "lbfgsb.c"
  (_e_wsfe()|0); //@line 3447 "lbfgsb.c"
  STACKTOP = sp;return 0; //@line 3451 "lbfgsb.c"
 }
 (_s_wsle(2032)|0); //@line 3413 "lbfgsb.c"
 (_do_lio(1308,1236,15744,11)|0); //@line 3414 "lbfgsb.c"
 $35 = $12; //@line 3415 "lbfgsb.c"
 (_do_lio(1660,1236,$35,4)|0); //@line 3415 "lbfgsb.c"
 (_do_lio(1308,1236,15756,23)|0); //@line 3416 "lbfgsb.c"
 $36 = $14; //@line 3417 "lbfgsb.c"
 (_do_lio(1908,1236,$36,8)|0); //@line 3417 "lbfgsb.c"
 (_e_wsle()|0); //@line 3418 "lbfgsb.c"
 (_s_wsfe(2052)|0); //@line 3419 "lbfgsb.c"
 $37 = $5; //@line 3420 "lbfgsb.c"
 (_do_fio(1236,$37,4)|0); //@line 3420 "lbfgsb.c"
 $38 = $2; //@line 3421 "lbfgsb.c"
 (_do_fio(1236,$38,8)|0); //@line 3421 "lbfgsb.c"
 $39 = $8; //@line 3422 "lbfgsb.c"
 (_do_fio(1236,$39,8)|0); //@line 3422 "lbfgsb.c"
 (_e_wsfe()|0); //@line 3423 "lbfgsb.c"
 $40 = $4; //@line 3424 "lbfgsb.c"
 $41 = HEAP32[$40>>2]|0; //@line 3424 "lbfgsb.c"
 $42 = ($41|0)>(100); //@line 3424 "lbfgsb.c"
 if (!($42)) {
  STACKTOP = sp;return 0; //@line 3451 "lbfgsb.c"
 }
 (_s_wsfe(2072)|0); //@line 3425 "lbfgsb.c"
 (_do_fio(1236,15780,3)|0); //@line 3426 "lbfgsb.c"
 $43 = $0; //@line 3427 "lbfgsb.c"
 $44 = HEAP32[$43>>2]|0; //@line 3427 "lbfgsb.c"
 $i__1 = $44; //@line 3427 "lbfgsb.c"
 HEAP32[2024>>2] = 1; //@line 3428 "lbfgsb.c"
 while(1) {
  $45 = HEAP32[2024>>2]|0; //@line 3428 "lbfgsb.c"
  $46 = $i__1; //@line 3428 "lbfgsb.c"
  $47 = ($45|0)<=($46|0); //@line 3428 "lbfgsb.c"
  if (!($47)) {
   break;
  }
  $48 = HEAP32[2024>>2]|0; //@line 3429 "lbfgsb.c"
  $49 = $1; //@line 3429 "lbfgsb.c"
  $50 = (($49) + ($48<<3)|0); //@line 3429 "lbfgsb.c"
  (_do_fio(1236,$50,8)|0); //@line 3429 "lbfgsb.c"
  $51 = HEAP32[2024>>2]|0; //@line 3428 "lbfgsb.c"
  $52 = (($51) + 1)|0; //@line 3428 "lbfgsb.c"
  HEAP32[2024>>2] = $52; //@line 3428 "lbfgsb.c"
 }
 (_e_wsfe()|0); //@line 3431 "lbfgsb.c"
 (_s_wsfe(2092)|0); //@line 3432 "lbfgsb.c"
 (_do_fio(1236,15784,3)|0); //@line 3433 "lbfgsb.c"
 $53 = $0; //@line 3434 "lbfgsb.c"
 $54 = HEAP32[$53>>2]|0; //@line 3434 "lbfgsb.c"
 $i__1 = $54; //@line 3434 "lbfgsb.c"
 HEAP32[2024>>2] = 1; //@line 3435 "lbfgsb.c"
 while(1) {
  $55 = HEAP32[2024>>2]|0; //@line 3435 "lbfgsb.c"
  $56 = $i__1; //@line 3435 "lbfgsb.c"
  $57 = ($55|0)<=($56|0); //@line 3435 "lbfgsb.c"
  if (!($57)) {
   break;
  }
  $58 = HEAP32[2024>>2]|0; //@line 3436 "lbfgsb.c"
  $59 = $3; //@line 3436 "lbfgsb.c"
  $60 = (($59) + ($58<<3)|0); //@line 3436 "lbfgsb.c"
  (_do_fio(1236,$60,8)|0); //@line 3436 "lbfgsb.c"
  $61 = HEAP32[2024>>2]|0; //@line 3435 "lbfgsb.c"
  $62 = (($61) + 1)|0; //@line 3435 "lbfgsb.c"
  HEAP32[2024>>2] = $62; //@line 3435 "lbfgsb.c"
 }
 (_e_wsfe()|0); //@line 3438 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 3451 "lbfgsb.c"
}
function _matupd_($n,$m,$ws,$wy,$sy,$ss,$d__,$r__,$itail,$iupdat,$col,$head,$theta,$rr,$dr,$stp,$dtd) {
 $n = $n|0;
 $m = $m|0;
 $ws = $ws|0;
 $wy = $wy|0;
 $sy = $sy|0;
 $ss = $ss|0;
 $d__ = $d__|0;
 $r__ = $r__|0;
 $itail = $itail|0;
 $iupdat = $iupdat|0;
 $col = $col|0;
 $head = $head|0;
 $theta = $theta|0;
 $rr = $rr|0;
 $dr = $dr|0;
 $stp = $stp|0;
 $dtd = $dtd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0.0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0.0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0.0, $225 = 0, $226 = 0.0, $227 = 0.0, $228 = 0, $229 = 0.0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0.0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, $i__2 = 0, $ss_dim1 = 0, $ss_offset = 0, $sy_dim1 = 0, $sy_offset = 0, $ws_dim1 = 0, $ws_offset = 0, $wy_dim1 = 0;
 var $wy_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__2 = sp;
 $0 = $n;
 $1 = $m;
 $2 = $ws;
 $3 = $wy;
 $4 = $sy;
 $5 = $ss;
 $6 = $d__;
 $7 = $r__;
 $8 = $itail;
 $9 = $iupdat;
 $10 = $col;
 $11 = $head;
 $12 = $theta;
 $13 = $rr;
 $14 = $dr;
 $15 = $stp;
 $16 = $dtd;
 $17 = $7; //@line 3182 "lbfgsb.c"
 $18 = ((($17)) + -8|0); //@line 3182 "lbfgsb.c"
 $7 = $18; //@line 3182 "lbfgsb.c"
 $19 = $6; //@line 3183 "lbfgsb.c"
 $20 = ((($19)) + -8|0); //@line 3183 "lbfgsb.c"
 $6 = $20; //@line 3183 "lbfgsb.c"
 $21 = $1; //@line 3184 "lbfgsb.c"
 $22 = HEAP32[$21>>2]|0; //@line 3184 "lbfgsb.c"
 $ss_dim1 = $22; //@line 3184 "lbfgsb.c"
 $23 = $ss_dim1; //@line 3185 "lbfgsb.c"
 $24 = (1 + ($23))|0; //@line 3185 "lbfgsb.c"
 $ss_offset = $24; //@line 3185 "lbfgsb.c"
 $25 = $ss_offset; //@line 3186 "lbfgsb.c"
 $26 = $5; //@line 3186 "lbfgsb.c"
 $27 = (0 - ($25))|0; //@line 3186 "lbfgsb.c"
 $28 = (($26) + ($27<<3)|0); //@line 3186 "lbfgsb.c"
 $5 = $28; //@line 3186 "lbfgsb.c"
 $29 = $1; //@line 3187 "lbfgsb.c"
 $30 = HEAP32[$29>>2]|0; //@line 3187 "lbfgsb.c"
 $sy_dim1 = $30; //@line 3187 "lbfgsb.c"
 $31 = $sy_dim1; //@line 3188 "lbfgsb.c"
 $32 = (1 + ($31))|0; //@line 3188 "lbfgsb.c"
 $sy_offset = $32; //@line 3188 "lbfgsb.c"
 $33 = $sy_offset; //@line 3189 "lbfgsb.c"
 $34 = $4; //@line 3189 "lbfgsb.c"
 $35 = (0 - ($33))|0; //@line 3189 "lbfgsb.c"
 $36 = (($34) + ($35<<3)|0); //@line 3189 "lbfgsb.c"
 $4 = $36; //@line 3189 "lbfgsb.c"
 $37 = $0; //@line 3190 "lbfgsb.c"
 $38 = HEAP32[$37>>2]|0; //@line 3190 "lbfgsb.c"
 $wy_dim1 = $38; //@line 3190 "lbfgsb.c"
 $39 = $wy_dim1; //@line 3191 "lbfgsb.c"
 $40 = (1 + ($39))|0; //@line 3191 "lbfgsb.c"
 $wy_offset = $40; //@line 3191 "lbfgsb.c"
 $41 = $wy_offset; //@line 3192 "lbfgsb.c"
 $42 = $3; //@line 3192 "lbfgsb.c"
 $43 = (0 - ($41))|0; //@line 3192 "lbfgsb.c"
 $44 = (($42) + ($43<<3)|0); //@line 3192 "lbfgsb.c"
 $3 = $44; //@line 3192 "lbfgsb.c"
 $45 = $0; //@line 3193 "lbfgsb.c"
 $46 = HEAP32[$45>>2]|0; //@line 3193 "lbfgsb.c"
 $ws_dim1 = $46; //@line 3193 "lbfgsb.c"
 $47 = $ws_dim1; //@line 3194 "lbfgsb.c"
 $48 = (1 + ($47))|0; //@line 3194 "lbfgsb.c"
 $ws_offset = $48; //@line 3194 "lbfgsb.c"
 $49 = $ws_offset; //@line 3195 "lbfgsb.c"
 $50 = $2; //@line 3195 "lbfgsb.c"
 $51 = (0 - ($49))|0; //@line 3195 "lbfgsb.c"
 $52 = (($50) + ($51<<3)|0); //@line 3195 "lbfgsb.c"
 $2 = $52; //@line 3195 "lbfgsb.c"
 $53 = $9; //@line 3198 "lbfgsb.c"
 $54 = HEAP32[$53>>2]|0; //@line 3198 "lbfgsb.c"
 $55 = $1; //@line 3198 "lbfgsb.c"
 $56 = HEAP32[$55>>2]|0; //@line 3198 "lbfgsb.c"
 $57 = ($54|0)<=($56|0); //@line 3198 "lbfgsb.c"
 if ($57) {
  $58 = $9; //@line 3199 "lbfgsb.c"
  $59 = HEAP32[$58>>2]|0; //@line 3199 "lbfgsb.c"
  $60 = $10; //@line 3199 "lbfgsb.c"
  HEAP32[$60>>2] = $59; //@line 3199 "lbfgsb.c"
  $61 = $11; //@line 3200 "lbfgsb.c"
  $62 = HEAP32[$61>>2]|0; //@line 3200 "lbfgsb.c"
  $63 = $9; //@line 3200 "lbfgsb.c"
  $64 = HEAP32[$63>>2]|0; //@line 3200 "lbfgsb.c"
  $65 = (($62) + ($64))|0; //@line 3200 "lbfgsb.c"
  $66 = (($65) - 2)|0; //@line 3200 "lbfgsb.c"
  $67 = $1; //@line 3200 "lbfgsb.c"
  $68 = HEAP32[$67>>2]|0; //@line 3200 "lbfgsb.c"
  $69 = (($66|0) % ($68|0))&-1; //@line 3200 "lbfgsb.c"
  $70 = (($69) + 1)|0; //@line 3200 "lbfgsb.c"
  $71 = $8; //@line 3200 "lbfgsb.c"
  HEAP32[$71>>2] = $70; //@line 3200 "lbfgsb.c"
 } else {
  $72 = $8; //@line 3202 "lbfgsb.c"
  $73 = HEAP32[$72>>2]|0; //@line 3202 "lbfgsb.c"
  $74 = $1; //@line 3202 "lbfgsb.c"
  $75 = HEAP32[$74>>2]|0; //@line 3202 "lbfgsb.c"
  $76 = (($73|0) % ($75|0))&-1; //@line 3202 "lbfgsb.c"
  $77 = (($76) + 1)|0; //@line 3202 "lbfgsb.c"
  $78 = $8; //@line 3202 "lbfgsb.c"
  HEAP32[$78>>2] = $77; //@line 3202 "lbfgsb.c"
  $79 = $11; //@line 3203 "lbfgsb.c"
  $80 = HEAP32[$79>>2]|0; //@line 3203 "lbfgsb.c"
  $81 = $1; //@line 3203 "lbfgsb.c"
  $82 = HEAP32[$81>>2]|0; //@line 3203 "lbfgsb.c"
  $83 = (($80|0) % ($82|0))&-1; //@line 3203 "lbfgsb.c"
  $84 = (($83) + 1)|0; //@line 3203 "lbfgsb.c"
  $85 = $11; //@line 3203 "lbfgsb.c"
  HEAP32[$85>>2] = $84; //@line 3203 "lbfgsb.c"
 }
 $86 = $0; //@line 3206 "lbfgsb.c"
 $87 = $6; //@line 3206 "lbfgsb.c"
 $88 = ((($87)) + 8|0); //@line 3206 "lbfgsb.c"
 $89 = $8; //@line 3206 "lbfgsb.c"
 $90 = HEAP32[$89>>2]|0; //@line 3206 "lbfgsb.c"
 $91 = $ws_dim1; //@line 3206 "lbfgsb.c"
 $92 = Math_imul($90, $91)|0; //@line 3206 "lbfgsb.c"
 $93 = (($92) + 1)|0; //@line 3206 "lbfgsb.c"
 $94 = $2; //@line 3206 "lbfgsb.c"
 $95 = (($94) + ($93<<3)|0); //@line 3206 "lbfgsb.c"
 (_dcopy_($86,$88,1236,$95,1236)|0); //@line 3206 "lbfgsb.c"
 $96 = $0; //@line 3207 "lbfgsb.c"
 $97 = $7; //@line 3207 "lbfgsb.c"
 $98 = ((($97)) + 8|0); //@line 3207 "lbfgsb.c"
 $99 = $8; //@line 3207 "lbfgsb.c"
 $100 = HEAP32[$99>>2]|0; //@line 3207 "lbfgsb.c"
 $101 = $wy_dim1; //@line 3207 "lbfgsb.c"
 $102 = Math_imul($100, $101)|0; //@line 3207 "lbfgsb.c"
 $103 = (($102) + 1)|0; //@line 3207 "lbfgsb.c"
 $104 = $3; //@line 3207 "lbfgsb.c"
 $105 = (($104) + ($103<<3)|0); //@line 3207 "lbfgsb.c"
 (_dcopy_($96,$98,1236,$105,1236)|0); //@line 3207 "lbfgsb.c"
 $106 = $13; //@line 3209 "lbfgsb.c"
 $107 = +HEAPF64[$106>>3]; //@line 3209 "lbfgsb.c"
 $108 = $14; //@line 3209 "lbfgsb.c"
 $109 = +HEAPF64[$108>>3]; //@line 3209 "lbfgsb.c"
 $110 = $107 / $109; //@line 3209 "lbfgsb.c"
 $111 = $12; //@line 3209 "lbfgsb.c"
 HEAPF64[$111>>3] = $110; //@line 3209 "lbfgsb.c"
 $112 = $9; //@line 3213 "lbfgsb.c"
 $113 = HEAP32[$112>>2]|0; //@line 3213 "lbfgsb.c"
 $114 = $1; //@line 3213 "lbfgsb.c"
 $115 = HEAP32[$114>>2]|0; //@line 3213 "lbfgsb.c"
 $116 = ($113|0)>($115|0); //@line 3213 "lbfgsb.c"
 L5: do {
  if ($116) {
   $117 = $10; //@line 3215 "lbfgsb.c"
   $118 = HEAP32[$117>>2]|0; //@line 3215 "lbfgsb.c"
   $119 = (($118) - 1)|0; //@line 3215 "lbfgsb.c"
   $i__1 = $119; //@line 3215 "lbfgsb.c"
   HEAP32[1912>>2] = 1; //@line 3216 "lbfgsb.c"
   while(1) {
    $120 = HEAP32[1912>>2]|0; //@line 3216 "lbfgsb.c"
    $121 = $i__1; //@line 3216 "lbfgsb.c"
    $122 = ($120|0)<=($121|0); //@line 3216 "lbfgsb.c"
    if (!($122)) {
     break L5;
    }
    $123 = HEAP32[1912>>2]|0; //@line 3217 "lbfgsb.c"
    $124 = (($123) + 1)|0; //@line 3217 "lbfgsb.c"
    $125 = $ss_dim1; //@line 3217 "lbfgsb.c"
    $126 = Math_imul($124, $125)|0; //@line 3217 "lbfgsb.c"
    $127 = (($126) + 2)|0; //@line 3217 "lbfgsb.c"
    $128 = $5; //@line 3217 "lbfgsb.c"
    $129 = (($128) + ($127<<3)|0); //@line 3217 "lbfgsb.c"
    $130 = HEAP32[1912>>2]|0; //@line 3217 "lbfgsb.c"
    $131 = $ss_dim1; //@line 3217 "lbfgsb.c"
    $132 = Math_imul($130, $131)|0; //@line 3217 "lbfgsb.c"
    $133 = (($132) + 1)|0; //@line 3217 "lbfgsb.c"
    $134 = $5; //@line 3217 "lbfgsb.c"
    $135 = (($134) + ($133<<3)|0); //@line 3217 "lbfgsb.c"
    (_dcopy_(1912,$129,1236,$135,1236)|0); //@line 3217 "lbfgsb.c"
    $136 = $10; //@line 3219 "lbfgsb.c"
    $137 = HEAP32[$136>>2]|0; //@line 3219 "lbfgsb.c"
    $138 = HEAP32[1912>>2]|0; //@line 3219 "lbfgsb.c"
    $139 = (($137) - ($138))|0; //@line 3219 "lbfgsb.c"
    HEAP32[$i__2>>2] = $139; //@line 3219 "lbfgsb.c"
    $140 = HEAP32[1912>>2]|0; //@line 3220 "lbfgsb.c"
    $141 = (($140) + 1)|0; //@line 3220 "lbfgsb.c"
    $142 = HEAP32[1912>>2]|0; //@line 3220 "lbfgsb.c"
    $143 = (($142) + 1)|0; //@line 3220 "lbfgsb.c"
    $144 = $sy_dim1; //@line 3220 "lbfgsb.c"
    $145 = Math_imul($143, $144)|0; //@line 3220 "lbfgsb.c"
    $146 = (($141) + ($145))|0; //@line 3220 "lbfgsb.c"
    $147 = $4; //@line 3220 "lbfgsb.c"
    $148 = (($147) + ($146<<3)|0); //@line 3220 "lbfgsb.c"
    $149 = HEAP32[1912>>2]|0; //@line 3220 "lbfgsb.c"
    $150 = HEAP32[1912>>2]|0; //@line 3220 "lbfgsb.c"
    $151 = $sy_dim1; //@line 3221 "lbfgsb.c"
    $152 = Math_imul($150, $151)|0; //@line 3220 "lbfgsb.c"
    $153 = (($149) + ($152))|0; //@line 3220 "lbfgsb.c"
    $154 = $4; //@line 3220 "lbfgsb.c"
    $155 = (($154) + ($153<<3)|0); //@line 3220 "lbfgsb.c"
    (_dcopy_($i__2,$148,1236,$155,1236)|0); //@line 3220 "lbfgsb.c"
    $156 = HEAP32[1912>>2]|0; //@line 3216 "lbfgsb.c"
    $157 = (($156) + 1)|0; //@line 3216 "lbfgsb.c"
    HEAP32[1912>>2] = $157; //@line 3216 "lbfgsb.c"
   }
  }
 } while(0);
 $158 = $11; //@line 3227 "lbfgsb.c"
 $159 = HEAP32[$158>>2]|0; //@line 3227 "lbfgsb.c"
 HEAP32[1916>>2] = $159; //@line 3227 "lbfgsb.c"
 $160 = $10; //@line 3228 "lbfgsb.c"
 $161 = HEAP32[$160>>2]|0; //@line 3228 "lbfgsb.c"
 $162 = (($161) - 1)|0; //@line 3228 "lbfgsb.c"
 $i__1 = $162; //@line 3228 "lbfgsb.c"
 HEAP32[1912>>2] = 1; //@line 3229 "lbfgsb.c"
 while(1) {
  $163 = HEAP32[1912>>2]|0; //@line 3229 "lbfgsb.c"
  $164 = $i__1; //@line 3229 "lbfgsb.c"
  $165 = ($163|0)<=($164|0); //@line 3229 "lbfgsb.c"
  if (!($165)) {
   break;
  }
  $166 = $0; //@line 3230 "lbfgsb.c"
  $167 = $6; //@line 3230 "lbfgsb.c"
  $168 = ((($167)) + 8|0); //@line 3230 "lbfgsb.c"
  $169 = HEAP32[1916>>2]|0; //@line 3230 "lbfgsb.c"
  $170 = $wy_dim1; //@line 3231 "lbfgsb.c"
  $171 = Math_imul($169, $170)|0; //@line 3230 "lbfgsb.c"
  $172 = (($171) + 1)|0; //@line 3231 "lbfgsb.c"
  $173 = $3; //@line 3230 "lbfgsb.c"
  $174 = (($173) + ($172<<3)|0); //@line 3230 "lbfgsb.c"
  $175 = (+_ddot_($166,$168,1236,$174,1236)); //@line 3230 "lbfgsb.c"
  $176 = $10; //@line 3230 "lbfgsb.c"
  $177 = HEAP32[$176>>2]|0; //@line 3230 "lbfgsb.c"
  $178 = HEAP32[1912>>2]|0; //@line 3230 "lbfgsb.c"
  $179 = $sy_dim1; //@line 3230 "lbfgsb.c"
  $180 = Math_imul($178, $179)|0; //@line 3230 "lbfgsb.c"
  $181 = (($177) + ($180))|0; //@line 3230 "lbfgsb.c"
  $182 = $4; //@line 3230 "lbfgsb.c"
  $183 = (($182) + ($181<<3)|0); //@line 3230 "lbfgsb.c"
  HEAPF64[$183>>3] = $175; //@line 3230 "lbfgsb.c"
  $184 = $0; //@line 3232 "lbfgsb.c"
  $185 = HEAP32[1916>>2]|0; //@line 3232 "lbfgsb.c"
  $186 = $ws_dim1; //@line 3232 "lbfgsb.c"
  $187 = Math_imul($185, $186)|0; //@line 3232 "lbfgsb.c"
  $188 = (($187) + 1)|0; //@line 3232 "lbfgsb.c"
  $189 = $2; //@line 3232 "lbfgsb.c"
  $190 = (($189) + ($188<<3)|0); //@line 3232 "lbfgsb.c"
  $191 = $6; //@line 3233 "lbfgsb.c"
  $192 = ((($191)) + 8|0); //@line 3233 "lbfgsb.c"
  $193 = (+_ddot_($184,$190,1236,$192,1236)); //@line 3232 "lbfgsb.c"
  $194 = HEAP32[1912>>2]|0; //@line 3232 "lbfgsb.c"
  $195 = $10; //@line 3232 "lbfgsb.c"
  $196 = HEAP32[$195>>2]|0; //@line 3232 "lbfgsb.c"
  $197 = $ss_dim1; //@line 3232 "lbfgsb.c"
  $198 = Math_imul($196, $197)|0; //@line 3232 "lbfgsb.c"
  $199 = (($194) + ($198))|0; //@line 3232 "lbfgsb.c"
  $200 = $5; //@line 3232 "lbfgsb.c"
  $201 = (($200) + ($199<<3)|0); //@line 3232 "lbfgsb.c"
  HEAPF64[$201>>3] = $193; //@line 3232 "lbfgsb.c"
  $202 = HEAP32[1916>>2]|0; //@line 3234 "lbfgsb.c"
  $203 = $1; //@line 3234 "lbfgsb.c"
  $204 = HEAP32[$203>>2]|0; //@line 3234 "lbfgsb.c"
  $205 = (($202|0) % ($204|0))&-1; //@line 3234 "lbfgsb.c"
  $206 = (($205) + 1)|0; //@line 3234 "lbfgsb.c"
  HEAP32[1916>>2] = $206; //@line 3234 "lbfgsb.c"
  $207 = HEAP32[1912>>2]|0; //@line 3229 "lbfgsb.c"
  $208 = (($207) + 1)|0; //@line 3229 "lbfgsb.c"
  HEAP32[1912>>2] = $208; //@line 3229 "lbfgsb.c"
 }
 $209 = $15; //@line 3237 "lbfgsb.c"
 $210 = +HEAPF64[$209>>3]; //@line 3237 "lbfgsb.c"
 $211 = $210 == 1.0; //@line 3237 "lbfgsb.c"
 if ($211) {
  $212 = $16; //@line 3238 "lbfgsb.c"
  $213 = +HEAPF64[$212>>3]; //@line 3238 "lbfgsb.c"
  $214 = $10; //@line 3238 "lbfgsb.c"
  $215 = HEAP32[$214>>2]|0; //@line 3238 "lbfgsb.c"
  $216 = $10; //@line 3238 "lbfgsb.c"
  $217 = HEAP32[$216>>2]|0; //@line 3238 "lbfgsb.c"
  $218 = $ss_dim1; //@line 3238 "lbfgsb.c"
  $219 = Math_imul($217, $218)|0; //@line 3238 "lbfgsb.c"
  $220 = (($215) + ($219))|0; //@line 3238 "lbfgsb.c"
  $221 = $5; //@line 3238 "lbfgsb.c"
  $222 = (($221) + ($220<<3)|0); //@line 3238 "lbfgsb.c"
  HEAPF64[$222>>3] = $213; //@line 3238 "lbfgsb.c"
  $240 = $14; //@line 3242 "lbfgsb.c"
  $241 = +HEAPF64[$240>>3]; //@line 3242 "lbfgsb.c"
  $242 = $10; //@line 3242 "lbfgsb.c"
  $243 = HEAP32[$242>>2]|0; //@line 3242 "lbfgsb.c"
  $244 = $10; //@line 3242 "lbfgsb.c"
  $245 = HEAP32[$244>>2]|0; //@line 3242 "lbfgsb.c"
  $246 = $sy_dim1; //@line 3242 "lbfgsb.c"
  $247 = Math_imul($245, $246)|0; //@line 3242 "lbfgsb.c"
  $248 = (($243) + ($247))|0; //@line 3242 "lbfgsb.c"
  $249 = $4; //@line 3242 "lbfgsb.c"
  $250 = (($249) + ($248<<3)|0); //@line 3242 "lbfgsb.c"
  HEAPF64[$250>>3] = $241; //@line 3242 "lbfgsb.c"
  STACKTOP = sp;return 0; //@line 3243 "lbfgsb.c"
 } else {
  $223 = $15; //@line 3240 "lbfgsb.c"
  $224 = +HEAPF64[$223>>3]; //@line 3240 "lbfgsb.c"
  $225 = $15; //@line 3240 "lbfgsb.c"
  $226 = +HEAPF64[$225>>3]; //@line 3240 "lbfgsb.c"
  $227 = $224 * $226; //@line 3240 "lbfgsb.c"
  $228 = $16; //@line 3240 "lbfgsb.c"
  $229 = +HEAPF64[$228>>3]; //@line 3240 "lbfgsb.c"
  $230 = $227 * $229; //@line 3240 "lbfgsb.c"
  $231 = $10; //@line 3240 "lbfgsb.c"
  $232 = HEAP32[$231>>2]|0; //@line 3240 "lbfgsb.c"
  $233 = $10; //@line 3240 "lbfgsb.c"
  $234 = HEAP32[$233>>2]|0; //@line 3240 "lbfgsb.c"
  $235 = $ss_dim1; //@line 3240 "lbfgsb.c"
  $236 = Math_imul($234, $235)|0; //@line 3240 "lbfgsb.c"
  $237 = (($232) + ($236))|0; //@line 3240 "lbfgsb.c"
  $238 = $5; //@line 3240 "lbfgsb.c"
  $239 = (($238) + ($237<<3)|0); //@line 3240 "lbfgsb.c"
  HEAPF64[$239>>3] = $230; //@line 3240 "lbfgsb.c"
  $240 = $14; //@line 3242 "lbfgsb.c"
  $241 = +HEAPF64[$240>>3]; //@line 3242 "lbfgsb.c"
  $242 = $10; //@line 3242 "lbfgsb.c"
  $243 = HEAP32[$242>>2]|0; //@line 3242 "lbfgsb.c"
  $244 = $10; //@line 3242 "lbfgsb.c"
  $245 = HEAP32[$244>>2]|0; //@line 3242 "lbfgsb.c"
  $246 = $sy_dim1; //@line 3242 "lbfgsb.c"
  $247 = Math_imul($245, $246)|0; //@line 3242 "lbfgsb.c"
  $248 = (($243) + ($247))|0; //@line 3242 "lbfgsb.c"
  $249 = $4; //@line 3242 "lbfgsb.c"
  $250 = (($249) + ($248<<3)|0); //@line 3242 "lbfgsb.c"
  HEAPF64[$250>>3] = $241; //@line 3242 "lbfgsb.c"
  STACKTOP = sp;return 0; //@line 3243 "lbfgsb.c"
 }
 return (0)|0;
}
function _formt_($m,$wt,$sy,$ss,$col,$theta,$info) {
 $m = $m|0;
 $wt = $wt|0;
 $sy = $sy|0;
 $ss = $ss|0;
 $col = $col|0;
 $theta = $theta|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0.0, $45 = 0.0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0;
 var $i__2 = 0, $i__3 = 0, $ss_dim1 = 0, $ss_offset = 0, $sy_dim1 = 0, $sy_offset = 0, $wt_dim1 = 0, $wt_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $wt;
 $2 = $sy;
 $3 = $ss;
 $4 = $col;
 $5 = $theta;
 $6 = $info;
 $7 = $0; //@line 2673 "lbfgsb.c"
 $8 = HEAP32[$7>>2]|0; //@line 2673 "lbfgsb.c"
 $ss_dim1 = $8; //@line 2673 "lbfgsb.c"
 $9 = $ss_dim1; //@line 2674 "lbfgsb.c"
 $10 = (1 + ($9))|0; //@line 2674 "lbfgsb.c"
 $ss_offset = $10; //@line 2674 "lbfgsb.c"
 $11 = $ss_offset; //@line 2675 "lbfgsb.c"
 $12 = $3; //@line 2675 "lbfgsb.c"
 $13 = (0 - ($11))|0; //@line 2675 "lbfgsb.c"
 $14 = (($12) + ($13<<3)|0); //@line 2675 "lbfgsb.c"
 $3 = $14; //@line 2675 "lbfgsb.c"
 $15 = $0; //@line 2676 "lbfgsb.c"
 $16 = HEAP32[$15>>2]|0; //@line 2676 "lbfgsb.c"
 $sy_dim1 = $16; //@line 2676 "lbfgsb.c"
 $17 = $sy_dim1; //@line 2677 "lbfgsb.c"
 $18 = (1 + ($17))|0; //@line 2677 "lbfgsb.c"
 $sy_offset = $18; //@line 2677 "lbfgsb.c"
 $19 = $sy_offset; //@line 2678 "lbfgsb.c"
 $20 = $2; //@line 2678 "lbfgsb.c"
 $21 = (0 - ($19))|0; //@line 2678 "lbfgsb.c"
 $22 = (($20) + ($21<<3)|0); //@line 2678 "lbfgsb.c"
 $2 = $22; //@line 2678 "lbfgsb.c"
 $23 = $0; //@line 2679 "lbfgsb.c"
 $24 = HEAP32[$23>>2]|0; //@line 2679 "lbfgsb.c"
 $wt_dim1 = $24; //@line 2679 "lbfgsb.c"
 $25 = $wt_dim1; //@line 2680 "lbfgsb.c"
 $26 = (1 + ($25))|0; //@line 2680 "lbfgsb.c"
 $wt_offset = $26; //@line 2680 "lbfgsb.c"
 $27 = $wt_offset; //@line 2681 "lbfgsb.c"
 $28 = $1; //@line 2681 "lbfgsb.c"
 $29 = (0 - ($27))|0; //@line 2681 "lbfgsb.c"
 $30 = (($28) + ($29<<3)|0); //@line 2681 "lbfgsb.c"
 $1 = $30; //@line 2681 "lbfgsb.c"
 $31 = $4; //@line 2684 "lbfgsb.c"
 $32 = HEAP32[$31>>2]|0; //@line 2684 "lbfgsb.c"
 $i__1 = $32; //@line 2684 "lbfgsb.c"
 HEAP32[1760>>2] = 1; //@line 2685 "lbfgsb.c"
 while(1) {
  $33 = HEAP32[1760>>2]|0; //@line 2685 "lbfgsb.c"
  $34 = $i__1; //@line 2685 "lbfgsb.c"
  $35 = ($33|0)<=($34|0); //@line 2685 "lbfgsb.c"
  if (!($35)) {
   break;
  }
  $36 = $5; //@line 2686 "lbfgsb.c"
  $37 = +HEAPF64[$36>>3]; //@line 2686 "lbfgsb.c"
  $38 = HEAP32[1760>>2]|0; //@line 2686 "lbfgsb.c"
  $39 = $ss_dim1; //@line 2686 "lbfgsb.c"
  $40 = Math_imul($38, $39)|0; //@line 2686 "lbfgsb.c"
  $41 = (($40) + 1)|0; //@line 2686 "lbfgsb.c"
  $42 = $3; //@line 2686 "lbfgsb.c"
  $43 = (($42) + ($41<<3)|0); //@line 2686 "lbfgsb.c"
  $44 = +HEAPF64[$43>>3]; //@line 2686 "lbfgsb.c"
  $45 = $37 * $44; //@line 2686 "lbfgsb.c"
  $46 = HEAP32[1760>>2]|0; //@line 2686 "lbfgsb.c"
  $47 = $wt_dim1; //@line 2686 "lbfgsb.c"
  $48 = Math_imul($46, $47)|0; //@line 2686 "lbfgsb.c"
  $49 = (($48) + 1)|0; //@line 2686 "lbfgsb.c"
  $50 = $1; //@line 2686 "lbfgsb.c"
  $51 = (($50) + ($49<<3)|0); //@line 2686 "lbfgsb.c"
  HEAPF64[$51>>3] = $45; //@line 2686 "lbfgsb.c"
  $52 = HEAP32[1760>>2]|0; //@line 2685 "lbfgsb.c"
  $53 = (($52) + 1)|0; //@line 2685 "lbfgsb.c"
  HEAP32[1760>>2] = $53; //@line 2685 "lbfgsb.c"
 }
 $54 = $4; //@line 2689 "lbfgsb.c"
 $55 = HEAP32[$54>>2]|0; //@line 2689 "lbfgsb.c"
 $i__1 = $55; //@line 2689 "lbfgsb.c"
 HEAP32[1756>>2] = 2; //@line 2690 "lbfgsb.c"
 while(1) {
  $56 = HEAP32[1756>>2]|0; //@line 2690 "lbfgsb.c"
  $57 = $i__1; //@line 2690 "lbfgsb.c"
  $58 = ($56|0)<=($57|0); //@line 2690 "lbfgsb.c"
  if (!($58)) {
   break;
  }
  $59 = $4; //@line 2691 "lbfgsb.c"
  $60 = HEAP32[$59>>2]|0; //@line 2691 "lbfgsb.c"
  $i__2 = $60; //@line 2691 "lbfgsb.c"
  $61 = HEAP32[1756>>2]|0; //@line 2692 "lbfgsb.c"
  HEAP32[1760>>2] = $61; //@line 2692 "lbfgsb.c"
  while(1) {
   $62 = HEAP32[1760>>2]|0; //@line 2692 "lbfgsb.c"
   $63 = $i__2; //@line 2692 "lbfgsb.c"
   $64 = ($62|0)<=($63|0); //@line 2692 "lbfgsb.c"
   $65 = HEAP32[1756>>2]|0; //@line 2693 "lbfgsb.c"
   if (!($64)) {
    break;
   }
   $66 = HEAP32[1760>>2]|0; //@line 2693 "lbfgsb.c"
   $67 = ($65|0)<=($66|0); //@line 2693 "lbfgsb.c"
   $68 = HEAP32[1756>>2]|0; //@line 2693 "lbfgsb.c"
   $69 = HEAP32[1760>>2]|0; //@line 2693 "lbfgsb.c"
   $70 = $67 ? $68 : $69; //@line 2693 "lbfgsb.c"
   $71 = (($70) - 1)|0; //@line 2693 "lbfgsb.c"
   HEAP32[1768>>2] = $71; //@line 2693 "lbfgsb.c"
   HEAPF64[400>>3] = 0.0; //@line 2694 "lbfgsb.c"
   $72 = HEAP32[1768>>2]|0; //@line 2695 "lbfgsb.c"
   $i__3 = $72; //@line 2695 "lbfgsb.c"
   HEAP32[1764>>2] = 1; //@line 2696 "lbfgsb.c"
   while(1) {
    $73 = HEAP32[1764>>2]|0; //@line 2696 "lbfgsb.c"
    $74 = $i__3; //@line 2696 "lbfgsb.c"
    $75 = ($73|0)<=($74|0); //@line 2696 "lbfgsb.c"
    if (!($75)) {
     break;
    }
    $76 = HEAP32[1756>>2]|0; //@line 2697 "lbfgsb.c"
    $77 = HEAP32[1764>>2]|0; //@line 2697 "lbfgsb.c"
    $78 = $sy_dim1; //@line 2697 "lbfgsb.c"
    $79 = Math_imul($77, $78)|0; //@line 2697 "lbfgsb.c"
    $80 = (($76) + ($79))|0; //@line 2697 "lbfgsb.c"
    $81 = $2; //@line 2697 "lbfgsb.c"
    $82 = (($81) + ($80<<3)|0); //@line 2697 "lbfgsb.c"
    $83 = +HEAPF64[$82>>3]; //@line 2697 "lbfgsb.c"
    $84 = HEAP32[1760>>2]|0; //@line 2697 "lbfgsb.c"
    $85 = HEAP32[1764>>2]|0; //@line 2697 "lbfgsb.c"
    $86 = $sy_dim1; //@line 2697 "lbfgsb.c"
    $87 = Math_imul($85, $86)|0; //@line 2697 "lbfgsb.c"
    $88 = (($84) + ($87))|0; //@line 2697 "lbfgsb.c"
    $89 = $2; //@line 2697 "lbfgsb.c"
    $90 = (($89) + ($88<<3)|0); //@line 2697 "lbfgsb.c"
    $91 = +HEAPF64[$90>>3]; //@line 2697 "lbfgsb.c"
    $92 = $83 * $91; //@line 2697 "lbfgsb.c"
    $93 = HEAP32[1764>>2]|0; //@line 2697 "lbfgsb.c"
    $94 = HEAP32[1764>>2]|0; //@line 2698 "lbfgsb.c"
    $95 = $sy_dim1; //@line 2698 "lbfgsb.c"
    $96 = Math_imul($94, $95)|0; //@line 2698 "lbfgsb.c"
    $97 = (($93) + ($96))|0; //@line 2697 "lbfgsb.c"
    $98 = $2; //@line 2697 "lbfgsb.c"
    $99 = (($98) + ($97<<3)|0); //@line 2697 "lbfgsb.c"
    $100 = +HEAPF64[$99>>3]; //@line 2697 "lbfgsb.c"
    $101 = $92 / $100; //@line 2697 "lbfgsb.c"
    $102 = +HEAPF64[400>>3]; //@line 2697 "lbfgsb.c"
    $103 = $102 + $101; //@line 2697 "lbfgsb.c"
    HEAPF64[400>>3] = $103; //@line 2697 "lbfgsb.c"
    $104 = HEAP32[1764>>2]|0; //@line 2696 "lbfgsb.c"
    $105 = (($104) + 1)|0; //@line 2696 "lbfgsb.c"
    HEAP32[1764>>2] = $105; //@line 2696 "lbfgsb.c"
   }
   $106 = +HEAPF64[400>>3]; //@line 2701 "lbfgsb.c"
   $107 = $5; //@line 2701 "lbfgsb.c"
   $108 = +HEAPF64[$107>>3]; //@line 2701 "lbfgsb.c"
   $109 = HEAP32[1756>>2]|0; //@line 2701 "lbfgsb.c"
   $110 = HEAP32[1760>>2]|0; //@line 2701 "lbfgsb.c"
   $111 = $ss_dim1; //@line 2701 "lbfgsb.c"
   $112 = Math_imul($110, $111)|0; //@line 2701 "lbfgsb.c"
   $113 = (($109) + ($112))|0; //@line 2701 "lbfgsb.c"
   $114 = $3; //@line 2701 "lbfgsb.c"
   $115 = (($114) + ($113<<3)|0); //@line 2701 "lbfgsb.c"
   $116 = +HEAPF64[$115>>3]; //@line 2701 "lbfgsb.c"
   $117 = $108 * $116; //@line 2701 "lbfgsb.c"
   $118 = $106 + $117; //@line 2701 "lbfgsb.c"
   $119 = HEAP32[1756>>2]|0; //@line 2701 "lbfgsb.c"
   $120 = HEAP32[1760>>2]|0; //@line 2701 "lbfgsb.c"
   $121 = $wt_dim1; //@line 2701 "lbfgsb.c"
   $122 = Math_imul($120, $121)|0; //@line 2701 "lbfgsb.c"
   $123 = (($119) + ($122))|0; //@line 2701 "lbfgsb.c"
   $124 = $1; //@line 2701 "lbfgsb.c"
   $125 = (($124) + ($123<<3)|0); //@line 2701 "lbfgsb.c"
   HEAPF64[$125>>3] = $118; //@line 2701 "lbfgsb.c"
   $126 = HEAP32[1760>>2]|0; //@line 2692 "lbfgsb.c"
   $127 = (($126) + 1)|0; //@line 2692 "lbfgsb.c"
   HEAP32[1760>>2] = $127; //@line 2692 "lbfgsb.c"
  }
  $128 = (($65) + 1)|0; //@line 2690 "lbfgsb.c"
  HEAP32[1756>>2] = $128; //@line 2690 "lbfgsb.c"
 }
 $129 = $wt_offset; //@line 2708 "lbfgsb.c"
 $130 = $1; //@line 2708 "lbfgsb.c"
 $131 = (($130) + ($129<<3)|0); //@line 2708 "lbfgsb.c"
 $132 = $0; //@line 2708 "lbfgsb.c"
 $133 = $4; //@line 2708 "lbfgsb.c"
 $134 = $6; //@line 2708 "lbfgsb.c"
 (_dpofa_($131,$132,$133,$134)|0); //@line 2708 "lbfgsb.c"
 $135 = $6; //@line 2709 "lbfgsb.c"
 $136 = HEAP32[$135>>2]|0; //@line 2709 "lbfgsb.c"
 $137 = ($136|0)!=(0); //@line 2709 "lbfgsb.c"
 if (!($137)) {
  STACKTOP = sp;return 0; //@line 2712 "lbfgsb.c"
 }
 $138 = $6; //@line 2710 "lbfgsb.c"
 HEAP32[$138>>2] = -3; //@line 2710 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 2712 "lbfgsb.c"
}
function _bmv_($m,$sy,$wt,$col,$v,$p,$info) {
 $m = $m|0;
 $sy = $sy|0;
 $wt = $wt|0;
 $col = $col|0;
 $v = $v|0;
 $p = $p|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0.0, $125 = 0.0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0.0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0.0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0, $21 = 0, $210 = 0, $211 = 0.0, $212 = 0, $213 = 0, $214 = 0, $215 = 0.0, $216 = 0.0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0.0, $64 = 0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i__1 = 0, $i__2 = 0, $sy_dim1 = 0, $sy_offset = 0, $wt_dim1 = 0, $wt_offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $m;
 $2 = $sy;
 $3 = $wt;
 $4 = $col;
 $5 = $v;
 $6 = $p;
 $7 = $info;
 $8 = $1; //@line 1385 "lbfgsb.c"
 $9 = HEAP32[$8>>2]|0; //@line 1385 "lbfgsb.c"
 $wt_dim1 = $9; //@line 1385 "lbfgsb.c"
 $10 = $wt_dim1; //@line 1386 "lbfgsb.c"
 $11 = (1 + ($10))|0; //@line 1386 "lbfgsb.c"
 $wt_offset = $11; //@line 1386 "lbfgsb.c"
 $12 = $wt_offset; //@line 1387 "lbfgsb.c"
 $13 = $3; //@line 1387 "lbfgsb.c"
 $14 = (0 - ($12))|0; //@line 1387 "lbfgsb.c"
 $15 = (($13) + ($14<<3)|0); //@line 1387 "lbfgsb.c"
 $3 = $15; //@line 1387 "lbfgsb.c"
 $16 = $1; //@line 1388 "lbfgsb.c"
 $17 = HEAP32[$16>>2]|0; //@line 1388 "lbfgsb.c"
 $sy_dim1 = $17; //@line 1388 "lbfgsb.c"
 $18 = $sy_dim1; //@line 1389 "lbfgsb.c"
 $19 = (1 + ($18))|0; //@line 1389 "lbfgsb.c"
 $sy_offset = $19; //@line 1389 "lbfgsb.c"
 $20 = $sy_offset; //@line 1390 "lbfgsb.c"
 $21 = $2; //@line 1390 "lbfgsb.c"
 $22 = (0 - ($20))|0; //@line 1390 "lbfgsb.c"
 $23 = (($21) + ($22<<3)|0); //@line 1390 "lbfgsb.c"
 $2 = $23; //@line 1390 "lbfgsb.c"
 $24 = $6; //@line 1391 "lbfgsb.c"
 $25 = ((($24)) + -8|0); //@line 1391 "lbfgsb.c"
 $6 = $25; //@line 1391 "lbfgsb.c"
 $26 = $5; //@line 1392 "lbfgsb.c"
 $27 = ((($26)) + -8|0); //@line 1392 "lbfgsb.c"
 $5 = $27; //@line 1392 "lbfgsb.c"
 $28 = $4; //@line 1395 "lbfgsb.c"
 $29 = HEAP32[$28>>2]|0; //@line 1395 "lbfgsb.c"
 $30 = ($29|0)==(0); //@line 1395 "lbfgsb.c"
 if ($30) {
  $0 = 0; //@line 1396 "lbfgsb.c"
  $219 = $0; //@line 1452 "lbfgsb.c"
  STACKTOP = sp;return ($219|0); //@line 1452 "lbfgsb.c"
 }
 $31 = $4; //@line 1401 "lbfgsb.c"
 $32 = HEAP32[$31>>2]|0; //@line 1401 "lbfgsb.c"
 $33 = (($32) + 1)|0; //@line 1401 "lbfgsb.c"
 $34 = $5; //@line 1401 "lbfgsb.c"
 $35 = (($34) + ($33<<3)|0); //@line 1401 "lbfgsb.c"
 $36 = +HEAPF64[$35>>3]; //@line 1401 "lbfgsb.c"
 $37 = $4; //@line 1401 "lbfgsb.c"
 $38 = HEAP32[$37>>2]|0; //@line 1401 "lbfgsb.c"
 $39 = (($38) + 1)|0; //@line 1401 "lbfgsb.c"
 $40 = $6; //@line 1401 "lbfgsb.c"
 $41 = (($40) + ($39<<3)|0); //@line 1401 "lbfgsb.c"
 HEAPF64[$41>>3] = $36; //@line 1401 "lbfgsb.c"
 $42 = $4; //@line 1402 "lbfgsb.c"
 $43 = HEAP32[$42>>2]|0; //@line 1402 "lbfgsb.c"
 $i__1 = $43; //@line 1402 "lbfgsb.c"
 HEAP32[1312>>2] = 2; //@line 1403 "lbfgsb.c"
 while(1) {
  $44 = HEAP32[1312>>2]|0; //@line 1403 "lbfgsb.c"
  $45 = $i__1; //@line 1403 "lbfgsb.c"
  $46 = ($44|0)<=($45|0); //@line 1403 "lbfgsb.c"
  if (!($46)) {
   break;
  }
  $47 = $4; //@line 1404 "lbfgsb.c"
  $48 = HEAP32[$47>>2]|0; //@line 1404 "lbfgsb.c"
  $49 = HEAP32[1312>>2]|0; //@line 1404 "lbfgsb.c"
  $50 = (($48) + ($49))|0; //@line 1404 "lbfgsb.c"
  HEAP32[1320>>2] = $50; //@line 1404 "lbfgsb.c"
  HEAPF64[200>>3] = 0.0; //@line 1405 "lbfgsb.c"
  $51 = HEAP32[1312>>2]|0; //@line 1406 "lbfgsb.c"
  $52 = (($51) - 1)|0; //@line 1406 "lbfgsb.c"
  $i__2 = $52; //@line 1406 "lbfgsb.c"
  HEAP32[1316>>2] = 1; //@line 1407 "lbfgsb.c"
  while(1) {
   $53 = HEAP32[1316>>2]|0; //@line 1407 "lbfgsb.c"
   $54 = $i__2; //@line 1407 "lbfgsb.c"
   $55 = ($53|0)<=($54|0); //@line 1407 "lbfgsb.c"
   if (!($55)) {
    break;
   }
   $56 = HEAP32[1312>>2]|0; //@line 1408 "lbfgsb.c"
   $57 = HEAP32[1316>>2]|0; //@line 1408 "lbfgsb.c"
   $58 = $sy_dim1; //@line 1408 "lbfgsb.c"
   $59 = Math_imul($57, $58)|0; //@line 1408 "lbfgsb.c"
   $60 = (($56) + ($59))|0; //@line 1408 "lbfgsb.c"
   $61 = $2; //@line 1408 "lbfgsb.c"
   $62 = (($61) + ($60<<3)|0); //@line 1408 "lbfgsb.c"
   $63 = +HEAPF64[$62>>3]; //@line 1408 "lbfgsb.c"
   $64 = HEAP32[1316>>2]|0; //@line 1408 "lbfgsb.c"
   $65 = $5; //@line 1408 "lbfgsb.c"
   $66 = (($65) + ($64<<3)|0); //@line 1408 "lbfgsb.c"
   $67 = +HEAPF64[$66>>3]; //@line 1408 "lbfgsb.c"
   $68 = $63 * $67; //@line 1408 "lbfgsb.c"
   $69 = HEAP32[1316>>2]|0; //@line 1408 "lbfgsb.c"
   $70 = HEAP32[1316>>2]|0; //@line 1408 "lbfgsb.c"
   $71 = $sy_dim1; //@line 1408 "lbfgsb.c"
   $72 = Math_imul($70, $71)|0; //@line 1408 "lbfgsb.c"
   $73 = (($69) + ($72))|0; //@line 1408 "lbfgsb.c"
   $74 = $2; //@line 1408 "lbfgsb.c"
   $75 = (($74) + ($73<<3)|0); //@line 1408 "lbfgsb.c"
   $76 = +HEAPF64[$75>>3]; //@line 1408 "lbfgsb.c"
   $77 = $68 / $76; //@line 1408 "lbfgsb.c"
   $78 = +HEAPF64[200>>3]; //@line 1408 "lbfgsb.c"
   $79 = $78 + $77; //@line 1408 "lbfgsb.c"
   HEAPF64[200>>3] = $79; //@line 1408 "lbfgsb.c"
   $80 = HEAP32[1316>>2]|0; //@line 1407 "lbfgsb.c"
   $81 = (($80) + 1)|0; //@line 1407 "lbfgsb.c"
   HEAP32[1316>>2] = $81; //@line 1407 "lbfgsb.c"
  }
  $82 = HEAP32[1320>>2]|0; //@line 1411 "lbfgsb.c"
  $83 = $5; //@line 1411 "lbfgsb.c"
  $84 = (($83) + ($82<<3)|0); //@line 1411 "lbfgsb.c"
  $85 = +HEAPF64[$84>>3]; //@line 1411 "lbfgsb.c"
  $86 = +HEAPF64[200>>3]; //@line 1411 "lbfgsb.c"
  $87 = $85 + $86; //@line 1411 "lbfgsb.c"
  $88 = HEAP32[1320>>2]|0; //@line 1411 "lbfgsb.c"
  $89 = $6; //@line 1411 "lbfgsb.c"
  $90 = (($89) + ($88<<3)|0); //@line 1411 "lbfgsb.c"
  HEAPF64[$90>>3] = $87; //@line 1411 "lbfgsb.c"
  $91 = HEAP32[1312>>2]|0; //@line 1403 "lbfgsb.c"
  $92 = (($91) + 1)|0; //@line 1403 "lbfgsb.c"
  HEAP32[1312>>2] = $92; //@line 1403 "lbfgsb.c"
 }
 $93 = $wt_offset; //@line 1415 "lbfgsb.c"
 $94 = $3; //@line 1415 "lbfgsb.c"
 $95 = (($94) + ($93<<3)|0); //@line 1415 "lbfgsb.c"
 $96 = $1; //@line 1415 "lbfgsb.c"
 $97 = $4; //@line 1415 "lbfgsb.c"
 $98 = $4; //@line 1415 "lbfgsb.c"
 $99 = HEAP32[$98>>2]|0; //@line 1415 "lbfgsb.c"
 $100 = (($99) + 1)|0; //@line 1415 "lbfgsb.c"
 $101 = $6; //@line 1415 "lbfgsb.c"
 $102 = (($101) + ($100<<3)|0); //@line 1415 "lbfgsb.c"
 $103 = $7; //@line 1415 "lbfgsb.c"
 (_dtrsl_($95,$96,$97,$102,1324,$103)|0); //@line 1415 "lbfgsb.c"
 $104 = $7; //@line 1416 "lbfgsb.c"
 $105 = HEAP32[$104>>2]|0; //@line 1416 "lbfgsb.c"
 $106 = ($105|0)!=(0); //@line 1416 "lbfgsb.c"
 if ($106) {
  $0 = 0; //@line 1417 "lbfgsb.c"
  $219 = $0; //@line 1452 "lbfgsb.c"
  STACKTOP = sp;return ($219|0); //@line 1452 "lbfgsb.c"
 }
 $107 = $4; //@line 1420 "lbfgsb.c"
 $108 = HEAP32[$107>>2]|0; //@line 1420 "lbfgsb.c"
 $i__1 = $108; //@line 1420 "lbfgsb.c"
 HEAP32[1312>>2] = 1; //@line 1421 "lbfgsb.c"
 while(1) {
  $109 = HEAP32[1312>>2]|0; //@line 1421 "lbfgsb.c"
  $110 = $i__1; //@line 1421 "lbfgsb.c"
  $111 = ($109|0)<=($110|0); //@line 1421 "lbfgsb.c"
  if (!($111)) {
   break;
  }
  $112 = HEAP32[1312>>2]|0; //@line 1422 "lbfgsb.c"
  $113 = $5; //@line 1422 "lbfgsb.c"
  $114 = (($113) + ($112<<3)|0); //@line 1422 "lbfgsb.c"
  $115 = +HEAPF64[$114>>3]; //@line 1422 "lbfgsb.c"
  $116 = HEAP32[1312>>2]|0; //@line 1422 "lbfgsb.c"
  $117 = HEAP32[1312>>2]|0; //@line 1422 "lbfgsb.c"
  $118 = $sy_dim1; //@line 1422 "lbfgsb.c"
  $119 = Math_imul($117, $118)|0; //@line 1422 "lbfgsb.c"
  $120 = (($116) + ($119))|0; //@line 1422 "lbfgsb.c"
  $121 = $2; //@line 1422 "lbfgsb.c"
  $122 = (($121) + ($120<<3)|0); //@line 1422 "lbfgsb.c"
  $123 = +HEAPF64[$122>>3]; //@line 1422 "lbfgsb.c"
  $124 = (+Math_sqrt((+$123))); //@line 1422 "lbfgsb.c"
  $125 = $115 / $124; //@line 1422 "lbfgsb.c"
  $126 = HEAP32[1312>>2]|0; //@line 1422 "lbfgsb.c"
  $127 = $6; //@line 1422 "lbfgsb.c"
  $128 = (($127) + ($126<<3)|0); //@line 1422 "lbfgsb.c"
  HEAPF64[$128>>3] = $125; //@line 1422 "lbfgsb.c"
  $129 = HEAP32[1312>>2]|0; //@line 1421 "lbfgsb.c"
  $130 = (($129) + 1)|0; //@line 1421 "lbfgsb.c"
  HEAP32[1312>>2] = $130; //@line 1421 "lbfgsb.c"
 }
 $131 = $wt_offset; //@line 1428 "lbfgsb.c"
 $132 = $3; //@line 1428 "lbfgsb.c"
 $133 = (($132) + ($131<<3)|0); //@line 1428 "lbfgsb.c"
 $134 = $1; //@line 1428 "lbfgsb.c"
 $135 = $4; //@line 1428 "lbfgsb.c"
 $136 = $4; //@line 1428 "lbfgsb.c"
 $137 = HEAP32[$136>>2]|0; //@line 1428 "lbfgsb.c"
 $138 = (($137) + 1)|0; //@line 1428 "lbfgsb.c"
 $139 = $6; //@line 1428 "lbfgsb.c"
 $140 = (($139) + ($138<<3)|0); //@line 1428 "lbfgsb.c"
 $141 = $7; //@line 1428 "lbfgsb.c"
 (_dtrsl_($133,$134,$135,$140,1236,$141)|0); //@line 1428 "lbfgsb.c"
 $142 = $7; //@line 1429 "lbfgsb.c"
 $143 = HEAP32[$142>>2]|0; //@line 1429 "lbfgsb.c"
 $144 = ($143|0)!=(0); //@line 1429 "lbfgsb.c"
 if ($144) {
  $0 = 0; //@line 1430 "lbfgsb.c"
  $219 = $0; //@line 1452 "lbfgsb.c"
  STACKTOP = sp;return ($219|0); //@line 1452 "lbfgsb.c"
 }
 $145 = $4; //@line 1434 "lbfgsb.c"
 $146 = HEAP32[$145>>2]|0; //@line 1434 "lbfgsb.c"
 $i__1 = $146; //@line 1434 "lbfgsb.c"
 HEAP32[1312>>2] = 1; //@line 1435 "lbfgsb.c"
 while(1) {
  $147 = HEAP32[1312>>2]|0; //@line 1435 "lbfgsb.c"
  $148 = $i__1; //@line 1435 "lbfgsb.c"
  $149 = ($147|0)<=($148|0); //@line 1435 "lbfgsb.c"
  if (!($149)) {
   break;
  }
  $150 = HEAP32[1312>>2]|0; //@line 1436 "lbfgsb.c"
  $151 = $6; //@line 1436 "lbfgsb.c"
  $152 = (($151) + ($150<<3)|0); //@line 1436 "lbfgsb.c"
  $153 = +HEAPF64[$152>>3]; //@line 1436 "lbfgsb.c"
  $154 = -$153; //@line 1436 "lbfgsb.c"
  $155 = HEAP32[1312>>2]|0; //@line 1436 "lbfgsb.c"
  $156 = HEAP32[1312>>2]|0; //@line 1436 "lbfgsb.c"
  $157 = $sy_dim1; //@line 1436 "lbfgsb.c"
  $158 = Math_imul($156, $157)|0; //@line 1436 "lbfgsb.c"
  $159 = (($155) + ($158))|0; //@line 1436 "lbfgsb.c"
  $160 = $2; //@line 1436 "lbfgsb.c"
  $161 = (($160) + ($159<<3)|0); //@line 1436 "lbfgsb.c"
  $162 = +HEAPF64[$161>>3]; //@line 1436 "lbfgsb.c"
  $163 = (+Math_sqrt((+$162))); //@line 1436 "lbfgsb.c"
  $164 = $154 / $163; //@line 1436 "lbfgsb.c"
  $165 = HEAP32[1312>>2]|0; //@line 1436 "lbfgsb.c"
  $166 = $6; //@line 1436 "lbfgsb.c"
  $167 = (($166) + ($165<<3)|0); //@line 1436 "lbfgsb.c"
  HEAPF64[$167>>3] = $164; //@line 1436 "lbfgsb.c"
  $168 = HEAP32[1312>>2]|0; //@line 1435 "lbfgsb.c"
  $169 = (($168) + 1)|0; //@line 1435 "lbfgsb.c"
  HEAP32[1312>>2] = $169; //@line 1435 "lbfgsb.c"
 }
 $170 = $4; //@line 1439 "lbfgsb.c"
 $171 = HEAP32[$170>>2]|0; //@line 1439 "lbfgsb.c"
 $i__1 = $171; //@line 1439 "lbfgsb.c"
 HEAP32[1312>>2] = 1; //@line 1440 "lbfgsb.c"
 while(1) {
  $172 = HEAP32[1312>>2]|0; //@line 1440 "lbfgsb.c"
  $173 = $i__1; //@line 1440 "lbfgsb.c"
  $174 = ($172|0)<=($173|0); //@line 1440 "lbfgsb.c"
  if (!($174)) {
   break;
  }
  HEAPF64[200>>3] = 0.0; //@line 1441 "lbfgsb.c"
  $175 = $4; //@line 1442 "lbfgsb.c"
  $176 = HEAP32[$175>>2]|0; //@line 1442 "lbfgsb.c"
  $i__2 = $176; //@line 1442 "lbfgsb.c"
  $177 = HEAP32[1312>>2]|0; //@line 1443 "lbfgsb.c"
  $178 = (($177) + 1)|0; //@line 1443 "lbfgsb.c"
  HEAP32[1316>>2] = $178; //@line 1443 "lbfgsb.c"
  while(1) {
   $179 = HEAP32[1316>>2]|0; //@line 1443 "lbfgsb.c"
   $180 = $i__2; //@line 1443 "lbfgsb.c"
   $181 = ($179|0)<=($180|0); //@line 1443 "lbfgsb.c"
   if (!($181)) {
    break;
   }
   $182 = HEAP32[1316>>2]|0; //@line 1444 "lbfgsb.c"
   $183 = HEAP32[1312>>2]|0; //@line 1444 "lbfgsb.c"
   $184 = $sy_dim1; //@line 1444 "lbfgsb.c"
   $185 = Math_imul($183, $184)|0; //@line 1444 "lbfgsb.c"
   $186 = (($182) + ($185))|0; //@line 1444 "lbfgsb.c"
   $187 = $2; //@line 1444 "lbfgsb.c"
   $188 = (($187) + ($186<<3)|0); //@line 1444 "lbfgsb.c"
   $189 = +HEAPF64[$188>>3]; //@line 1444 "lbfgsb.c"
   $190 = $4; //@line 1444 "lbfgsb.c"
   $191 = HEAP32[$190>>2]|0; //@line 1444 "lbfgsb.c"
   $192 = HEAP32[1316>>2]|0; //@line 1444 "lbfgsb.c"
   $193 = (($191) + ($192))|0; //@line 1444 "lbfgsb.c"
   $194 = $6; //@line 1444 "lbfgsb.c"
   $195 = (($194) + ($193<<3)|0); //@line 1444 "lbfgsb.c"
   $196 = +HEAPF64[$195>>3]; //@line 1444 "lbfgsb.c"
   $197 = $189 * $196; //@line 1444 "lbfgsb.c"
   $198 = HEAP32[1312>>2]|0; //@line 1444 "lbfgsb.c"
   $199 = HEAP32[1312>>2]|0; //@line 1444 "lbfgsb.c"
   $200 = $sy_dim1; //@line 1445 "lbfgsb.c"
   $201 = Math_imul($199, $200)|0; //@line 1444 "lbfgsb.c"
   $202 = (($198) + ($201))|0; //@line 1444 "lbfgsb.c"
   $203 = $2; //@line 1444 "lbfgsb.c"
   $204 = (($203) + ($202<<3)|0); //@line 1444 "lbfgsb.c"
   $205 = +HEAPF64[$204>>3]; //@line 1444 "lbfgsb.c"
   $206 = $197 / $205; //@line 1444 "lbfgsb.c"
   $207 = +HEAPF64[200>>3]; //@line 1444 "lbfgsb.c"
   $208 = $207 + $206; //@line 1444 "lbfgsb.c"
   HEAPF64[200>>3] = $208; //@line 1444 "lbfgsb.c"
   $209 = HEAP32[1316>>2]|0; //@line 1443 "lbfgsb.c"
   $210 = (($209) + 1)|0; //@line 1443 "lbfgsb.c"
   HEAP32[1316>>2] = $210; //@line 1443 "lbfgsb.c"
  }
  $211 = +HEAPF64[200>>3]; //@line 1448 "lbfgsb.c"
  $212 = HEAP32[1312>>2]|0; //@line 1448 "lbfgsb.c"
  $213 = $6; //@line 1448 "lbfgsb.c"
  $214 = (($213) + ($212<<3)|0); //@line 1448 "lbfgsb.c"
  $215 = +HEAPF64[$214>>3]; //@line 1448 "lbfgsb.c"
  $216 = $215 + $211; //@line 1448 "lbfgsb.c"
  HEAPF64[$214>>3] = $216; //@line 1448 "lbfgsb.c"
  $217 = HEAP32[1312>>2]|0; //@line 1440 "lbfgsb.c"
  $218 = (($217) + 1)|0; //@line 1440 "lbfgsb.c"
  HEAP32[1312>>2] = $218; //@line 1440 "lbfgsb.c"
 }
 $0 = 0; //@line 1451 "lbfgsb.c"
 $219 = $0; //@line 1452 "lbfgsb.c"
 STACKTOP = sp;return ($219|0); //@line 1452 "lbfgsb.c"
}
function _hpsolb_($n,$t,$iorder,$iheap) {
 $n = $n|0;
 $t = $t|0;
 $iorder = $iorder|0;
 $iheap = $iheap|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0.0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $t;
 $2 = $iorder;
 $3 = $iheap;
 $4 = $2; //@line 2911 "lbfgsb.c"
 $5 = ((($4)) + -4|0); //@line 2911 "lbfgsb.c"
 $2 = $5; //@line 2911 "lbfgsb.c"
 $6 = $1; //@line 2912 "lbfgsb.c"
 $7 = ((($6)) + -8|0); //@line 2912 "lbfgsb.c"
 $1 = $7; //@line 2912 "lbfgsb.c"
 $8 = $3; //@line 2915 "lbfgsb.c"
 $9 = HEAP32[$8>>2]|0; //@line 2915 "lbfgsb.c"
 $10 = ($9|0)==(0); //@line 2915 "lbfgsb.c"
 L1: do {
  if ($10) {
   $11 = $0; //@line 2917 "lbfgsb.c"
   $12 = HEAP32[$11>>2]|0; //@line 2917 "lbfgsb.c"
   $i__1 = $12; //@line 2917 "lbfgsb.c"
   HEAP32[1872>>2] = 2; //@line 2918 "lbfgsb.c"
   while(1) {
    $13 = HEAP32[1872>>2]|0; //@line 2918 "lbfgsb.c"
    $14 = $i__1; //@line 2918 "lbfgsb.c"
    $15 = ($13|0)<=($14|0); //@line 2918 "lbfgsb.c"
    if (!($15)) {
     break L1;
    }
    $16 = HEAP32[1872>>2]|0; //@line 2919 "lbfgsb.c"
    $17 = $1; //@line 2919 "lbfgsb.c"
    $18 = (($17) + ($16<<3)|0); //@line 2919 "lbfgsb.c"
    $19 = +HEAPF64[$18>>3]; //@line 2919 "lbfgsb.c"
    HEAPF64[416>>3] = $19; //@line 2919 "lbfgsb.c"
    $20 = HEAP32[1872>>2]|0; //@line 2920 "lbfgsb.c"
    $21 = $2; //@line 2920 "lbfgsb.c"
    $22 = (($21) + ($20<<2)|0); //@line 2920 "lbfgsb.c"
    $23 = HEAP32[$22>>2]|0; //@line 2920 "lbfgsb.c"
    HEAP32[1876>>2] = $23; //@line 2920 "lbfgsb.c"
    $24 = HEAP32[1872>>2]|0; //@line 2922 "lbfgsb.c"
    HEAP32[1864>>2] = $24; //@line 2922 "lbfgsb.c"
    while(1) {
     $25 = HEAP32[1864>>2]|0; //@line 2924 "lbfgsb.c"
     $26 = ($25|0)>(1); //@line 2924 "lbfgsb.c"
     if (!($26)) {
      break;
     }
     $27 = HEAP32[1864>>2]|0; //@line 2925 "lbfgsb.c"
     $28 = (($27|0) / 2)&-1; //@line 2925 "lbfgsb.c"
     HEAP32[1868>>2] = $28; //@line 2925 "lbfgsb.c"
     $29 = +HEAPF64[416>>3]; //@line 2926 "lbfgsb.c"
     $30 = HEAP32[1868>>2]|0; //@line 2926 "lbfgsb.c"
     $31 = $1; //@line 2926 "lbfgsb.c"
     $32 = (($31) + ($30<<3)|0); //@line 2926 "lbfgsb.c"
     $33 = +HEAPF64[$32>>3]; //@line 2926 "lbfgsb.c"
     $34 = $29 < $33; //@line 2926 "lbfgsb.c"
     if (!($34)) {
      break;
     }
     $35 = HEAP32[1868>>2]|0; //@line 2927 "lbfgsb.c"
     $36 = $1; //@line 2927 "lbfgsb.c"
     $37 = (($36) + ($35<<3)|0); //@line 2927 "lbfgsb.c"
     $38 = +HEAPF64[$37>>3]; //@line 2927 "lbfgsb.c"
     $39 = HEAP32[1864>>2]|0; //@line 2927 "lbfgsb.c"
     $40 = $1; //@line 2927 "lbfgsb.c"
     $41 = (($40) + ($39<<3)|0); //@line 2927 "lbfgsb.c"
     HEAPF64[$41>>3] = $38; //@line 2927 "lbfgsb.c"
     $42 = HEAP32[1868>>2]|0; //@line 2928 "lbfgsb.c"
     $43 = $2; //@line 2928 "lbfgsb.c"
     $44 = (($43) + ($42<<2)|0); //@line 2928 "lbfgsb.c"
     $45 = HEAP32[$44>>2]|0; //@line 2928 "lbfgsb.c"
     $46 = HEAP32[1864>>2]|0; //@line 2928 "lbfgsb.c"
     $47 = $2; //@line 2928 "lbfgsb.c"
     $48 = (($47) + ($46<<2)|0); //@line 2928 "lbfgsb.c"
     HEAP32[$48>>2] = $45; //@line 2928 "lbfgsb.c"
     $49 = HEAP32[1868>>2]|0; //@line 2929 "lbfgsb.c"
     HEAP32[1864>>2] = $49; //@line 2929 "lbfgsb.c"
    }
    $50 = +HEAPF64[416>>3]; //@line 2933 "lbfgsb.c"
    $51 = HEAP32[1864>>2]|0; //@line 2933 "lbfgsb.c"
    $52 = $1; //@line 2933 "lbfgsb.c"
    $53 = (($52) + ($51<<3)|0); //@line 2933 "lbfgsb.c"
    HEAPF64[$53>>3] = $50; //@line 2933 "lbfgsb.c"
    $54 = HEAP32[1876>>2]|0; //@line 2934 "lbfgsb.c"
    $55 = HEAP32[1864>>2]|0; //@line 2934 "lbfgsb.c"
    $56 = $2; //@line 2934 "lbfgsb.c"
    $57 = (($56) + ($55<<2)|0); //@line 2934 "lbfgsb.c"
    HEAP32[$57>>2] = $54; //@line 2934 "lbfgsb.c"
    $58 = HEAP32[1872>>2]|0; //@line 2918 "lbfgsb.c"
    $59 = (($58) + 1)|0; //@line 2918 "lbfgsb.c"
    HEAP32[1872>>2] = $59; //@line 2918 "lbfgsb.c"
   }
  }
 } while(0);
 $60 = $0; //@line 2941 "lbfgsb.c"
 $61 = HEAP32[$60>>2]|0; //@line 2941 "lbfgsb.c"
 $62 = ($61|0)>(1); //@line 2941 "lbfgsb.c"
 if (!($62)) {
  STACKTOP = sp;return 0; //@line 2967 "lbfgsb.c"
 }
 HEAP32[1864>>2] = 1; //@line 2942 "lbfgsb.c"
 $63 = $1; //@line 2943 "lbfgsb.c"
 $64 = ((($63)) + 8|0); //@line 2943 "lbfgsb.c"
 $65 = +HEAPF64[$64>>3]; //@line 2943 "lbfgsb.c"
 HEAPF64[408>>3] = $65; //@line 2943 "lbfgsb.c"
 $66 = $2; //@line 2944 "lbfgsb.c"
 $67 = ((($66)) + 4|0); //@line 2944 "lbfgsb.c"
 $68 = HEAP32[$67>>2]|0; //@line 2944 "lbfgsb.c"
 HEAP32[1880>>2] = $68; //@line 2944 "lbfgsb.c"
 $69 = $0; //@line 2945 "lbfgsb.c"
 $70 = HEAP32[$69>>2]|0; //@line 2945 "lbfgsb.c"
 $71 = $1; //@line 2945 "lbfgsb.c"
 $72 = (($71) + ($70<<3)|0); //@line 2945 "lbfgsb.c"
 $73 = +HEAPF64[$72>>3]; //@line 2945 "lbfgsb.c"
 HEAPF64[416>>3] = $73; //@line 2945 "lbfgsb.c"
 $74 = $0; //@line 2946 "lbfgsb.c"
 $75 = HEAP32[$74>>2]|0; //@line 2946 "lbfgsb.c"
 $76 = $2; //@line 2946 "lbfgsb.c"
 $77 = (($76) + ($75<<2)|0); //@line 2946 "lbfgsb.c"
 $78 = HEAP32[$77>>2]|0; //@line 2946 "lbfgsb.c"
 HEAP32[1876>>2] = $78; //@line 2946 "lbfgsb.c"
 while(1) {
  $79 = HEAP32[1864>>2]|0; //@line 2949 "lbfgsb.c"
  $80 = HEAP32[1864>>2]|0; //@line 2949 "lbfgsb.c"
  $81 = (($79) + ($80))|0; //@line 2949 "lbfgsb.c"
  HEAP32[1868>>2] = $81; //@line 2949 "lbfgsb.c"
  $82 = HEAP32[1868>>2]|0; //@line 2950 "lbfgsb.c"
  $83 = $0; //@line 2950 "lbfgsb.c"
  $84 = HEAP32[$83>>2]|0; //@line 2950 "lbfgsb.c"
  $85 = (($84) - 1)|0; //@line 2950 "lbfgsb.c"
  $86 = ($82|0)<=($85|0); //@line 2950 "lbfgsb.c"
  if (!($86)) {
   break;
  }
  $87 = HEAP32[1868>>2]|0; //@line 2951 "lbfgsb.c"
  $88 = (($87) + 1)|0; //@line 2951 "lbfgsb.c"
  $89 = $1; //@line 2951 "lbfgsb.c"
  $90 = (($89) + ($88<<3)|0); //@line 2951 "lbfgsb.c"
  $91 = +HEAPF64[$90>>3]; //@line 2951 "lbfgsb.c"
  $92 = HEAP32[1868>>2]|0; //@line 2951 "lbfgsb.c"
  $93 = $1; //@line 2951 "lbfgsb.c"
  $94 = (($93) + ($92<<3)|0); //@line 2951 "lbfgsb.c"
  $95 = +HEAPF64[$94>>3]; //@line 2951 "lbfgsb.c"
  $96 = $91 < $95; //@line 2951 "lbfgsb.c"
  if ($96) {
   $97 = HEAP32[1868>>2]|0; //@line 2952 "lbfgsb.c"
   $98 = (($97) + 1)|0; //@line 2952 "lbfgsb.c"
   HEAP32[1868>>2] = $98; //@line 2952 "lbfgsb.c"
  }
  $99 = HEAP32[1868>>2]|0; //@line 2954 "lbfgsb.c"
  $100 = $1; //@line 2954 "lbfgsb.c"
  $101 = (($100) + ($99<<3)|0); //@line 2954 "lbfgsb.c"
  $102 = +HEAPF64[$101>>3]; //@line 2954 "lbfgsb.c"
  $103 = +HEAPF64[416>>3]; //@line 2954 "lbfgsb.c"
  $104 = $102 < $103; //@line 2954 "lbfgsb.c"
  if (!($104)) {
   break;
  }
  $105 = HEAP32[1868>>2]|0; //@line 2955 "lbfgsb.c"
  $106 = $1; //@line 2955 "lbfgsb.c"
  $107 = (($106) + ($105<<3)|0); //@line 2955 "lbfgsb.c"
  $108 = +HEAPF64[$107>>3]; //@line 2955 "lbfgsb.c"
  $109 = HEAP32[1864>>2]|0; //@line 2955 "lbfgsb.c"
  $110 = $1; //@line 2955 "lbfgsb.c"
  $111 = (($110) + ($109<<3)|0); //@line 2955 "lbfgsb.c"
  HEAPF64[$111>>3] = $108; //@line 2955 "lbfgsb.c"
  $112 = HEAP32[1868>>2]|0; //@line 2956 "lbfgsb.c"
  $113 = $2; //@line 2956 "lbfgsb.c"
  $114 = (($113) + ($112<<2)|0); //@line 2956 "lbfgsb.c"
  $115 = HEAP32[$114>>2]|0; //@line 2956 "lbfgsb.c"
  $116 = HEAP32[1864>>2]|0; //@line 2956 "lbfgsb.c"
  $117 = $2; //@line 2956 "lbfgsb.c"
  $118 = (($117) + ($116<<2)|0); //@line 2956 "lbfgsb.c"
  HEAP32[$118>>2] = $115; //@line 2956 "lbfgsb.c"
  $119 = HEAP32[1868>>2]|0; //@line 2957 "lbfgsb.c"
  HEAP32[1864>>2] = $119; //@line 2957 "lbfgsb.c"
 }
 $120 = +HEAPF64[416>>3]; //@line 2961 "lbfgsb.c"
 $121 = HEAP32[1864>>2]|0; //@line 2961 "lbfgsb.c"
 $122 = $1; //@line 2961 "lbfgsb.c"
 $123 = (($122) + ($121<<3)|0); //@line 2961 "lbfgsb.c"
 HEAPF64[$123>>3] = $120; //@line 2961 "lbfgsb.c"
 $124 = HEAP32[1876>>2]|0; //@line 2962 "lbfgsb.c"
 $125 = HEAP32[1864>>2]|0; //@line 2962 "lbfgsb.c"
 $126 = $2; //@line 2962 "lbfgsb.c"
 $127 = (($126) + ($125<<2)|0); //@line 2962 "lbfgsb.c"
 HEAP32[$127>>2] = $124; //@line 2962 "lbfgsb.c"
 $128 = +HEAPF64[408>>3]; //@line 2964 "lbfgsb.c"
 $129 = $0; //@line 2964 "lbfgsb.c"
 $130 = HEAP32[$129>>2]|0; //@line 2964 "lbfgsb.c"
 $131 = $1; //@line 2964 "lbfgsb.c"
 $132 = (($131) + ($130<<3)|0); //@line 2964 "lbfgsb.c"
 HEAPF64[$132>>3] = $128; //@line 2964 "lbfgsb.c"
 $133 = HEAP32[1880>>2]|0; //@line 2965 "lbfgsb.c"
 $134 = $0; //@line 2965 "lbfgsb.c"
 $135 = HEAP32[$134>>2]|0; //@line 2965 "lbfgsb.c"
 $136 = $2; //@line 2965 "lbfgsb.c"
 $137 = (($136) + ($135<<2)|0); //@line 2965 "lbfgsb.c"
 HEAP32[$137>>2] = $133; //@line 2965 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 2967 "lbfgsb.c"
}
function _dcsrch_($f,$g,$stp,$ftol,$gtol,$xtol,$stpmin,$stpmax,$task,$isave,$dsave,$task_len) {
 $f = $f|0;
 $g = $g|0;
 $stp = $stp|0;
 $ftol = $ftol|0;
 $gtol = $gtol|0;
 $xtol = $xtol|0;
 $stpmin = $stpmin|0;
 $stpmax = $stpmax|0;
 $task = $task|0;
 $isave = $isave|0;
 $dsave = $dsave|0;
 $task_len = $task_len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0.0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0.0, $118 = 0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0, $137 = 0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0.0, $145 = 0.0, $146 = 0.0, $147 = 0.0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0.0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0, $184 = 0.0, $185 = 0, $186 = 0, $187 = 0.0, $188 = 0.0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0.0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0.0, $198 = 0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0.0, $203 = 0.0, $204 = 0, $205 = 0;
 var $206 = 0.0, $207 = 0.0, $208 = 0, $209 = 0, $21 = 0.0, $210 = 0, $211 = 0, $212 = 0.0, $213 = 0.0, $214 = 0, $215 = 0, $216 = 0.0, $217 = 0, $218 = 0, $219 = 0.0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0.0, $226 = 0.0, $227 = 0, $228 = 0, $229 = 0, $23 = 0.0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0.0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0.0, $244 = 0.0, $245 = 0, $246 = 0, $247 = 0.0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0.0, $256 = 0.0, $257 = 0.0, $258 = 0.0, $259 = 0.0, $26 = 0;
 var $260 = 0.0, $261 = 0.0, $262 = 0.0, $263 = 0, $264 = 0.0, $265 = 0.0, $266 = 0.0, $267 = 0.0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0.0, $271 = 0.0, $272 = 0.0, $273 = 0, $274 = 0.0, $275 = 0.0, $276 = 0.0, $277 = 0.0, $278 = 0.0;
 var $279 = 0.0, $28 = 0.0, $280 = 0.0, $281 = 0.0, $282 = 0.0, $283 = 0.0, $284 = 0.0, $285 = 0.0, $286 = 0.0, $287 = 0.0, $288 = 0.0, $289 = 0.0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0.0, $296 = 0.0;
 var $297 = 0.0, $298 = 0.0, $299 = 0, $3 = 0, $30 = 0.0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0.0, $304 = 0.0, $305 = 0, $306 = 0.0, $307 = 0.0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0.0, $311 = 0.0, $312 = 0, $313 = 0.0;
 var $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0.0, $321 = 0.0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0.0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0.0, $331 = 0.0;
 var $332 = 0, $333 = 0.0, $334 = 0.0, $335 = 0.0, $336 = 0, $337 = 0.0, $338 = 0, $339 = 0.0, $34 = 0, $340 = 0.0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0, $345 = 0.0, $346 = 0, $347 = 0.0, $348 = 0.0, $349 = 0.0, $35 = 0.0;
 var $350 = 0.0, $351 = 0.0, $352 = 0, $353 = 0.0, $354 = 0, $355 = 0.0, $356 = 0, $357 = 0, $358 = 0.0, $359 = 0, $36 = 0, $360 = 0.0, $361 = 0, $362 = 0.0, $363 = 0, $364 = 0.0, $365 = 0, $366 = 0.0, $367 = 0, $368 = 0;
 var $369 = 0.0, $37 = 0, $370 = 0, $371 = 0.0, $372 = 0, $373 = 0.0, $374 = 0, $375 = 0, $376 = 0, $377 = 0.0, $378 = 0.0, $379 = 0, $38 = 0, $380 = 0, $381 = 0.0, $382 = 0.0, $383 = 0, $384 = 0, $385 = 0, $386 = 0.0;
 var $387 = 0.0, $388 = 0.0, $389 = 0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0.0, $393 = 0, $394 = 0.0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0.0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0.0, $406 = 0, $407 = 0, $408 = 0.0, $409 = 0, $41 = 0, $410 = 0, $411 = 0.0, $412 = 0, $413 = 0, $414 = 0.0, $415 = 0, $416 = 0, $417 = 0.0, $418 = 0, $419 = 0, $42 = 0, $420 = 0.0, $421 = 0;
 var $422 = 0, $423 = 0.0, $424 = 0, $425 = 0, $426 = 0.0, $427 = 0, $428 = 0, $429 = 0.0, $43 = 0, $430 = 0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0.0, $436 = 0, $437 = 0, $438 = 0.0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0.0, $442 = 0, $443 = 0, $444 = 0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0.0, $73 = 0, $74 = 0.0, $75 = 0.0, $76 = 0.0;
 var $77 = 0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d__1 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $f;
 $2 = $g;
 $3 = $stp;
 $4 = $ftol;
 $5 = $gtol;
 $6 = $xtol;
 $7 = $stpmin;
 $8 = $stpmax;
 $9 = $task;
 $10 = $isave;
 $11 = $dsave;
 $12 = $task_len;
 $13 = $11; //@line 4326 "lbfgsb.c"
 $14 = ((($13)) + -8|0); //@line 4326 "lbfgsb.c"
 $11 = $14; //@line 4326 "lbfgsb.c"
 $15 = $10; //@line 4327 "lbfgsb.c"
 $16 = ((($15)) + -4|0); //@line 4327 "lbfgsb.c"
 $10 = $16; //@line 4327 "lbfgsb.c"
 $17 = $9; //@line 4330 "lbfgsb.c"
 $18 = (_s_cmp($17,13740,5,5)|0); //@line 4330 "lbfgsb.c"
 $19 = ($18|0)==(0); //@line 4330 "lbfgsb.c"
 do {
  if ($19) {
   $20 = $3; //@line 4332 "lbfgsb.c"
   $21 = +HEAPF64[$20>>3]; //@line 4332 "lbfgsb.c"
   $22 = $7; //@line 4332 "lbfgsb.c"
   $23 = +HEAPF64[$22>>3]; //@line 4332 "lbfgsb.c"
   $24 = $21 < $23; //@line 4332 "lbfgsb.c"
   if ($24) {
    $25 = $9; //@line 4333 "lbfgsb.c"
    $26 = $12; //@line 4333 "lbfgsb.c"
    _s_copy($25,17734,$26,22); //@line 4333 "lbfgsb.c"
   }
   $27 = $3; //@line 4335 "lbfgsb.c"
   $28 = +HEAPF64[$27>>3]; //@line 4335 "lbfgsb.c"
   $29 = $8; //@line 4335 "lbfgsb.c"
   $30 = +HEAPF64[$29>>3]; //@line 4335 "lbfgsb.c"
   $31 = $28 > $30; //@line 4335 "lbfgsb.c"
   if ($31) {
    $32 = $9; //@line 4336 "lbfgsb.c"
    $33 = $12; //@line 4336 "lbfgsb.c"
    _s_copy($32,17757,$33,22); //@line 4336 "lbfgsb.c"
   }
   $34 = $2; //@line 4338 "lbfgsb.c"
   $35 = +HEAPF64[$34>>3]; //@line 4338 "lbfgsb.c"
   $36 = $35 >= 0.0; //@line 4338 "lbfgsb.c"
   if ($36) {
    $37 = $9; //@line 4339 "lbfgsb.c"
    $38 = $12; //@line 4339 "lbfgsb.c"
    _s_copy($37,17780,$38,26); //@line 4339 "lbfgsb.c"
   }
   $39 = $4; //@line 4341 "lbfgsb.c"
   $40 = +HEAPF64[$39>>3]; //@line 4341 "lbfgsb.c"
   $41 = $40 < 0.0; //@line 4341 "lbfgsb.c"
   if ($41) {
    $42 = $9; //@line 4342 "lbfgsb.c"
    $43 = $12; //@line 4342 "lbfgsb.c"
    _s_copy($42,17807,$43,21); //@line 4342 "lbfgsb.c"
   }
   $44 = $5; //@line 4344 "lbfgsb.c"
   $45 = +HEAPF64[$44>>3]; //@line 4344 "lbfgsb.c"
   $46 = $45 < 0.0; //@line 4344 "lbfgsb.c"
   if ($46) {
    $47 = $9; //@line 4345 "lbfgsb.c"
    $48 = $12; //@line 4345 "lbfgsb.c"
    _s_copy($47,17829,$48,21); //@line 4345 "lbfgsb.c"
   }
   $49 = $6; //@line 4347 "lbfgsb.c"
   $50 = +HEAPF64[$49>>3]; //@line 4347 "lbfgsb.c"
   $51 = $50 < 0.0; //@line 4347 "lbfgsb.c"
   if ($51) {
    $52 = $9; //@line 4348 "lbfgsb.c"
    $53 = $12; //@line 4348 "lbfgsb.c"
    _s_copy($52,17851,$53,21); //@line 4348 "lbfgsb.c"
   }
   $54 = $7; //@line 4350 "lbfgsb.c"
   $55 = +HEAPF64[$54>>3]; //@line 4350 "lbfgsb.c"
   $56 = $55 < 0.0; //@line 4350 "lbfgsb.c"
   if ($56) {
    $57 = $9; //@line 4351 "lbfgsb.c"
    $58 = $12; //@line 4351 "lbfgsb.c"
    _s_copy($57,17873,$58,23); //@line 4351 "lbfgsb.c"
   }
   $59 = $8; //@line 4353 "lbfgsb.c"
   $60 = +HEAPF64[$59>>3]; //@line 4353 "lbfgsb.c"
   $61 = $7; //@line 4353 "lbfgsb.c"
   $62 = +HEAPF64[$61>>3]; //@line 4353 "lbfgsb.c"
   $63 = $60 < $62; //@line 4353 "lbfgsb.c"
   if ($63) {
    $64 = $9; //@line 4354 "lbfgsb.c"
    $65 = $12; //@line 4354 "lbfgsb.c"
    _s_copy($64,17897,$65,25); //@line 4354 "lbfgsb.c"
   }
   $66 = $9; //@line 4357 "lbfgsb.c"
   $67 = (_s_cmp($66,14356,5,5)|0); //@line 4357 "lbfgsb.c"
   $68 = ($67|0)==(0); //@line 4357 "lbfgsb.c"
   if (!($68)) {
    HEAP32[2600>>2] = 0; //@line 4361 "lbfgsb.c"
    HEAP32[2596>>2] = 1; //@line 4362 "lbfgsb.c"
    $69 = $1; //@line 4363 "lbfgsb.c"
    $70 = +HEAPF64[$69>>3]; //@line 4363 "lbfgsb.c"
    HEAPF64[616>>3] = $70; //@line 4363 "lbfgsb.c"
    $71 = $2; //@line 4364 "lbfgsb.c"
    $72 = +HEAPF64[$71>>3]; //@line 4364 "lbfgsb.c"
    HEAPF64[624>>3] = $72; //@line 4364 "lbfgsb.c"
    $73 = $4; //@line 4365 "lbfgsb.c"
    $74 = +HEAPF64[$73>>3]; //@line 4365 "lbfgsb.c"
    $75 = +HEAPF64[624>>3]; //@line 4365 "lbfgsb.c"
    $76 = $74 * $75; //@line 4365 "lbfgsb.c"
    HEAPF64[648>>3] = $76; //@line 4365 "lbfgsb.c"
    $77 = $8; //@line 4366 "lbfgsb.c"
    $78 = +HEAPF64[$77>>3]; //@line 4366 "lbfgsb.c"
    $79 = $7; //@line 4366 "lbfgsb.c"
    $80 = +HEAPF64[$79>>3]; //@line 4366 "lbfgsb.c"
    $81 = $78 - $80; //@line 4366 "lbfgsb.c"
    HEAPF64[632>>3] = $81; //@line 4366 "lbfgsb.c"
    $82 = +HEAPF64[632>>3]; //@line 4367 "lbfgsb.c"
    $83 = $82 / 0.5; //@line 4367 "lbfgsb.c"
    HEAPF64[672>>3] = $83; //@line 4367 "lbfgsb.c"
    HEAPF64[600>>3] = 0.0; //@line 4374 "lbfgsb.c"
    $84 = +HEAPF64[616>>3]; //@line 4375 "lbfgsb.c"
    HEAPF64[536>>3] = $84; //@line 4375 "lbfgsb.c"
    $85 = +HEAPF64[624>>3]; //@line 4376 "lbfgsb.c"
    HEAPF64[552>>3] = $85; //@line 4376 "lbfgsb.c"
    HEAPF64[608>>3] = 0.0; //@line 4377 "lbfgsb.c"
    $86 = +HEAPF64[616>>3]; //@line 4378 "lbfgsb.c"
    HEAPF64[544>>3] = $86; //@line 4378 "lbfgsb.c"
    $87 = +HEAPF64[624>>3]; //@line 4379 "lbfgsb.c"
    HEAPF64[560>>3] = $87; //@line 4379 "lbfgsb.c"
    HEAPF64[656>>3] = 0.0; //@line 4380 "lbfgsb.c"
    $88 = $3; //@line 4381 "lbfgsb.c"
    $89 = +HEAPF64[$88>>3]; //@line 4381 "lbfgsb.c"
    $90 = $3; //@line 4381 "lbfgsb.c"
    $91 = +HEAPF64[$90>>3]; //@line 4381 "lbfgsb.c"
    $92 = $91 * 4.0; //@line 4381 "lbfgsb.c"
    $93 = $89 + $92; //@line 4381 "lbfgsb.c"
    HEAPF64[664>>3] = $93; //@line 4381 "lbfgsb.c"
    $94 = $9; //@line 4382 "lbfgsb.c"
    $95 = $12; //@line 4382 "lbfgsb.c"
    _s_copy($94,17923,$95,2); //@line 4382 "lbfgsb.c"
    break;
   }
   $0 = 0; //@line 4358 "lbfgsb.c"
   $444 = $0; //@line 4508 "lbfgsb.c"
   STACKTOP = sp;return ($444|0); //@line 4508 "lbfgsb.c"
  } else {
   $96 = $10; //@line 4386 "lbfgsb.c"
   $97 = ((($96)) + 4|0); //@line 4386 "lbfgsb.c"
   $98 = HEAP32[$97>>2]|0; //@line 4386 "lbfgsb.c"
   $99 = ($98|0)==(1); //@line 4386 "lbfgsb.c"
   if ($99) {
    HEAP32[2600>>2] = 1; //@line 4387 "lbfgsb.c"
   } else {
    HEAP32[2600>>2] = 0; //@line 4389 "lbfgsb.c"
   }
   $100 = $10; //@line 4391 "lbfgsb.c"
   $101 = ((($100)) + 8|0); //@line 4391 "lbfgsb.c"
   $102 = HEAP32[$101>>2]|0; //@line 4391 "lbfgsb.c"
   HEAP32[2596>>2] = $102; //@line 4391 "lbfgsb.c"
   $103 = $11; //@line 4392 "lbfgsb.c"
   $104 = ((($103)) + 8|0); //@line 4392 "lbfgsb.c"
   $105 = +HEAPF64[$104>>3]; //@line 4392 "lbfgsb.c"
   HEAPF64[624>>3] = $105; //@line 4392 "lbfgsb.c"
   $106 = $11; //@line 4393 "lbfgsb.c"
   $107 = ((($106)) + 16|0); //@line 4393 "lbfgsb.c"
   $108 = +HEAPF64[$107>>3]; //@line 4393 "lbfgsb.c"
   HEAPF64[648>>3] = $108; //@line 4393 "lbfgsb.c"
   $109 = $11; //@line 4394 "lbfgsb.c"
   $110 = ((($109)) + 24|0); //@line 4394 "lbfgsb.c"
   $111 = +HEAPF64[$110>>3]; //@line 4394 "lbfgsb.c"
   HEAPF64[552>>3] = $111; //@line 4394 "lbfgsb.c"
   $112 = $11; //@line 4395 "lbfgsb.c"
   $113 = ((($112)) + 32|0); //@line 4395 "lbfgsb.c"
   $114 = +HEAPF64[$113>>3]; //@line 4395 "lbfgsb.c"
   HEAPF64[560>>3] = $114; //@line 4395 "lbfgsb.c"
   $115 = $11; //@line 4396 "lbfgsb.c"
   $116 = ((($115)) + 40|0); //@line 4396 "lbfgsb.c"
   $117 = +HEAPF64[$116>>3]; //@line 4396 "lbfgsb.c"
   HEAPF64[616>>3] = $117; //@line 4396 "lbfgsb.c"
   $118 = $11; //@line 4397 "lbfgsb.c"
   $119 = ((($118)) + 48|0); //@line 4397 "lbfgsb.c"
   $120 = +HEAPF64[$119>>3]; //@line 4397 "lbfgsb.c"
   HEAPF64[536>>3] = $120; //@line 4397 "lbfgsb.c"
   $121 = $11; //@line 4398 "lbfgsb.c"
   $122 = ((($121)) + 56|0); //@line 4398 "lbfgsb.c"
   $123 = +HEAPF64[$122>>3]; //@line 4398 "lbfgsb.c"
   HEAPF64[544>>3] = $123; //@line 4398 "lbfgsb.c"
   $124 = $11; //@line 4399 "lbfgsb.c"
   $125 = ((($124)) + 64|0); //@line 4399 "lbfgsb.c"
   $126 = +HEAPF64[$125>>3]; //@line 4399 "lbfgsb.c"
   HEAPF64[600>>3] = $126; //@line 4399 "lbfgsb.c"
   $127 = $11; //@line 4400 "lbfgsb.c"
   $128 = ((($127)) + 72|0); //@line 4400 "lbfgsb.c"
   $129 = +HEAPF64[$128>>3]; //@line 4400 "lbfgsb.c"
   HEAPF64[608>>3] = $129; //@line 4400 "lbfgsb.c"
   $130 = $11; //@line 4401 "lbfgsb.c"
   $131 = ((($130)) + 80|0); //@line 4401 "lbfgsb.c"
   $132 = +HEAPF64[$131>>3]; //@line 4401 "lbfgsb.c"
   HEAPF64[656>>3] = $132; //@line 4401 "lbfgsb.c"
   $133 = $11; //@line 4402 "lbfgsb.c"
   $134 = ((($133)) + 88|0); //@line 4402 "lbfgsb.c"
   $135 = +HEAPF64[$134>>3]; //@line 4402 "lbfgsb.c"
   HEAPF64[664>>3] = $135; //@line 4402 "lbfgsb.c"
   $136 = $11; //@line 4403 "lbfgsb.c"
   $137 = ((($136)) + 96|0); //@line 4403 "lbfgsb.c"
   $138 = +HEAPF64[$137>>3]; //@line 4403 "lbfgsb.c"
   HEAPF64[632>>3] = $138; //@line 4403 "lbfgsb.c"
   $139 = $11; //@line 4404 "lbfgsb.c"
   $140 = ((($139)) + 104|0); //@line 4404 "lbfgsb.c"
   $141 = +HEAPF64[$140>>3]; //@line 4404 "lbfgsb.c"
   HEAPF64[672>>3] = $141; //@line 4404 "lbfgsb.c"
   $142 = +HEAPF64[616>>3]; //@line 4408 "lbfgsb.c"
   $143 = $3; //@line 4408 "lbfgsb.c"
   $144 = +HEAPF64[$143>>3]; //@line 4408 "lbfgsb.c"
   $145 = +HEAPF64[648>>3]; //@line 4408 "lbfgsb.c"
   $146 = $144 * $145; //@line 4408 "lbfgsb.c"
   $147 = $142 + $146; //@line 4408 "lbfgsb.c"
   HEAPF64[640>>3] = $147; //@line 4408 "lbfgsb.c"
   $148 = HEAP32[2596>>2]|0; //@line 4409 "lbfgsb.c"
   $149 = ($148|0)==(1); //@line 4409 "lbfgsb.c"
   if ($149) {
    $150 = $1; //@line 4409 "lbfgsb.c"
    $151 = +HEAPF64[$150>>3]; //@line 4409 "lbfgsb.c"
    $152 = +HEAPF64[640>>3]; //@line 4409 "lbfgsb.c"
    $153 = $151 <= $152; //@line 4409 "lbfgsb.c"
    if ($153) {
     $154 = $2; //@line 4409 "lbfgsb.c"
     $155 = +HEAPF64[$154>>3]; //@line 4409 "lbfgsb.c"
     $156 = $155 >= 0.0; //@line 4409 "lbfgsb.c"
     if ($156) {
      HEAP32[2596>>2] = 2; //@line 4410 "lbfgsb.c"
     }
    }
   }
   $157 = HEAP32[2600>>2]|0; //@line 4413 "lbfgsb.c"
   $158 = ($157|0)!=(0); //@line 4413 "lbfgsb.c"
   do {
    if ($158) {
     $159 = $3; //@line 4413 "lbfgsb.c"
     $160 = +HEAPF64[$159>>3]; //@line 4413 "lbfgsb.c"
     $161 = +HEAPF64[656>>3]; //@line 4413 "lbfgsb.c"
     $162 = $160 <= $161; //@line 4413 "lbfgsb.c"
     if (!($162)) {
      $163 = $3; //@line 4413 "lbfgsb.c"
      $164 = +HEAPF64[$163>>3]; //@line 4413 "lbfgsb.c"
      $165 = +HEAPF64[664>>3]; //@line 4413 "lbfgsb.c"
      $166 = $164 >= $165; //@line 4413 "lbfgsb.c"
      if (!($166)) {
       break;
      }
     }
     $167 = $9; //@line 4414 "lbfgsb.c"
     $168 = $12; //@line 4414 "lbfgsb.c"
     _s_copy($167,17926,$168,41); //@line 4414 "lbfgsb.c"
    }
   } while(0);
   $169 = HEAP32[2600>>2]|0; //@line 4417 "lbfgsb.c"
   $170 = ($169|0)!=(0); //@line 4417 "lbfgsb.c"
   if ($170) {
    $171 = +HEAPF64[664>>3]; //@line 4417 "lbfgsb.c"
    $172 = +HEAPF64[656>>3]; //@line 4417 "lbfgsb.c"
    $173 = $171 - $172; //@line 4417 "lbfgsb.c"
    $174 = $6; //@line 4417 "lbfgsb.c"
    $175 = +HEAPF64[$174>>3]; //@line 4417 "lbfgsb.c"
    $176 = +HEAPF64[664>>3]; //@line 4417 "lbfgsb.c"
    $177 = $175 * $176; //@line 4417 "lbfgsb.c"
    $178 = $173 <= $177; //@line 4417 "lbfgsb.c"
    if ($178) {
     $179 = $9; //@line 4418 "lbfgsb.c"
     $180 = $12; //@line 4418 "lbfgsb.c"
     _s_copy($179,17968,$180,28); //@line 4418 "lbfgsb.c"
    }
   }
   $181 = $3; //@line 4420 "lbfgsb.c"
   $182 = +HEAPF64[$181>>3]; //@line 4420 "lbfgsb.c"
   $183 = $8; //@line 4420 "lbfgsb.c"
   $184 = +HEAPF64[$183>>3]; //@line 4420 "lbfgsb.c"
   $185 = $182 == $184; //@line 4420 "lbfgsb.c"
   if ($185) {
    $186 = $1; //@line 4420 "lbfgsb.c"
    $187 = +HEAPF64[$186>>3]; //@line 4420 "lbfgsb.c"
    $188 = +HEAPF64[640>>3]; //@line 4420 "lbfgsb.c"
    $189 = $187 <= $188; //@line 4420 "lbfgsb.c"
    if ($189) {
     $190 = $2; //@line 4420 "lbfgsb.c"
     $191 = +HEAPF64[$190>>3]; //@line 4420 "lbfgsb.c"
     $192 = +HEAPF64[648>>3]; //@line 4420 "lbfgsb.c"
     $193 = $191 <= $192; //@line 4420 "lbfgsb.c"
     if ($193) {
      $194 = $9; //@line 4421 "lbfgsb.c"
      $195 = $12; //@line 4421 "lbfgsb.c"
      _s_copy($194,17997,$195,21); //@line 4421 "lbfgsb.c"
     }
    }
   }
   $196 = $3; //@line 4423 "lbfgsb.c"
   $197 = +HEAPF64[$196>>3]; //@line 4423 "lbfgsb.c"
   $198 = $7; //@line 4423 "lbfgsb.c"
   $199 = +HEAPF64[$198>>3]; //@line 4423 "lbfgsb.c"
   $200 = $197 == $199; //@line 4423 "lbfgsb.c"
   do {
    if ($200) {
     $201 = $1; //@line 4423 "lbfgsb.c"
     $202 = +HEAPF64[$201>>3]; //@line 4423 "lbfgsb.c"
     $203 = +HEAPF64[640>>3]; //@line 4423 "lbfgsb.c"
     $204 = $202 > $203; //@line 4423 "lbfgsb.c"
     if (!($204)) {
      $205 = $2; //@line 4423 "lbfgsb.c"
      $206 = +HEAPF64[$205>>3]; //@line 4423 "lbfgsb.c"
      $207 = +HEAPF64[648>>3]; //@line 4423 "lbfgsb.c"
      $208 = $206 >= $207; //@line 4423 "lbfgsb.c"
      if (!($208)) {
       break;
      }
     }
     $209 = $9; //@line 4424 "lbfgsb.c"
     $210 = $12; //@line 4424 "lbfgsb.c"
     _s_copy($209,18019,$210,21); //@line 4424 "lbfgsb.c"
    }
   } while(0);
   $211 = $1; //@line 4427 "lbfgsb.c"
   $212 = +HEAPF64[$211>>3]; //@line 4427 "lbfgsb.c"
   $213 = +HEAPF64[640>>3]; //@line 4427 "lbfgsb.c"
   $214 = $212 <= $213; //@line 4427 "lbfgsb.c"
   if ($214) {
    $215 = $2; //@line 4427 "lbfgsb.c"
    $216 = +HEAPF64[$215>>3]; //@line 4427 "lbfgsb.c"
    $217 = $216 >= 0.0; //@line 4427 "lbfgsb.c"
    $218 = $2; //@line 4427 "lbfgsb.c"
    $219 = +HEAPF64[$218>>3]; //@line 4427 "lbfgsb.c"
    $220 = -$219; //@line 4427 "lbfgsb.c"
    $221 = $217 ? $219 : $220; //@line 4427 "lbfgsb.c"
    $222 = $5; //@line 4427 "lbfgsb.c"
    $223 = +HEAPF64[$222>>3]; //@line 4427 "lbfgsb.c"
    $224 = +HEAPF64[624>>3]; //@line 4427 "lbfgsb.c"
    $225 = -$224; //@line 4427 "lbfgsb.c"
    $226 = $223 * $225; //@line 4427 "lbfgsb.c"
    $227 = $221 <= $226; //@line 4427 "lbfgsb.c"
    if ($227) {
     $228 = $9; //@line 4428 "lbfgsb.c"
     $229 = $12; //@line 4428 "lbfgsb.c"
     _s_copy($228,18041,$229,11); //@line 4428 "lbfgsb.c"
    }
   }
   $230 = $9; //@line 4431 "lbfgsb.c"
   $231 = (_s_cmp($230,15451,4,4)|0); //@line 4431 "lbfgsb.c"
   $232 = ($231|0)==(0); //@line 4431 "lbfgsb.c"
   if (!($232)) {
    $233 = $9; //@line 4431 "lbfgsb.c"
    $234 = (_s_cmp($233,15446,4,4)|0); //@line 4431 "lbfgsb.c"
    $235 = ($234|0)==(0); //@line 4432 "lbfgsb.c"
    if (!($235)) {
     $236 = HEAP32[2596>>2]|0; //@line 4438 "lbfgsb.c"
     $237 = ($236|0)==(1); //@line 4438 "lbfgsb.c"
     if ($237) {
      $238 = $1; //@line 4438 "lbfgsb.c"
      $239 = +HEAPF64[$238>>3]; //@line 4438 "lbfgsb.c"
      $240 = +HEAPF64[536>>3]; //@line 4438 "lbfgsb.c"
      $241 = $239 <= $240; //@line 4438 "lbfgsb.c"
      if ($241) {
       $242 = $1; //@line 4438 "lbfgsb.c"
       $243 = +HEAPF64[$242>>3]; //@line 4438 "lbfgsb.c"
       $244 = +HEAPF64[640>>3]; //@line 4438 "lbfgsb.c"
       $245 = $243 > $244; //@line 4438 "lbfgsb.c"
       if ($245) {
        $246 = $1; //@line 4440 "lbfgsb.c"
        $247 = +HEAPF64[$246>>3]; //@line 4440 "lbfgsb.c"
        $248 = $3; //@line 4440 "lbfgsb.c"
        $249 = +HEAPF64[$248>>3]; //@line 4440 "lbfgsb.c"
        $250 = +HEAPF64[648>>3]; //@line 4440 "lbfgsb.c"
        $251 = $249 * $250; //@line 4440 "lbfgsb.c"
        $252 = $247 - $251; //@line 4440 "lbfgsb.c"
        HEAPF64[520>>3] = $252; //@line 4440 "lbfgsb.c"
        $253 = +HEAPF64[536>>3]; //@line 4441 "lbfgsb.c"
        $254 = +HEAPF64[600>>3]; //@line 4441 "lbfgsb.c"
        $255 = +HEAPF64[648>>3]; //@line 4441 "lbfgsb.c"
        $256 = $254 * $255; //@line 4441 "lbfgsb.c"
        $257 = $253 - $256; //@line 4441 "lbfgsb.c"
        HEAPF64[568>>3] = $257; //@line 4441 "lbfgsb.c"
        $258 = +HEAPF64[544>>3]; //@line 4442 "lbfgsb.c"
        $259 = +HEAPF64[608>>3]; //@line 4442 "lbfgsb.c"
        $260 = +HEAPF64[648>>3]; //@line 4442 "lbfgsb.c"
        $261 = $259 * $260; //@line 4442 "lbfgsb.c"
        $262 = $258 - $261; //@line 4442 "lbfgsb.c"
        HEAPF64[576>>3] = $262; //@line 4442 "lbfgsb.c"
        $263 = $2; //@line 4443 "lbfgsb.c"
        $264 = +HEAPF64[$263>>3]; //@line 4443 "lbfgsb.c"
        $265 = +HEAPF64[648>>3]; //@line 4443 "lbfgsb.c"
        $266 = $264 - $265; //@line 4443 "lbfgsb.c"
        HEAPF64[528>>3] = $266; //@line 4443 "lbfgsb.c"
        $267 = +HEAPF64[552>>3]; //@line 4444 "lbfgsb.c"
        $268 = +HEAPF64[648>>3]; //@line 4444 "lbfgsb.c"
        $269 = $267 - $268; //@line 4444 "lbfgsb.c"
        HEAPF64[584>>3] = $269; //@line 4444 "lbfgsb.c"
        $270 = +HEAPF64[560>>3]; //@line 4445 "lbfgsb.c"
        $271 = +HEAPF64[648>>3]; //@line 4445 "lbfgsb.c"
        $272 = $270 - $271; //@line 4445 "lbfgsb.c"
        HEAPF64[592>>3] = $272; //@line 4445 "lbfgsb.c"
        $273 = $3; //@line 4447 "lbfgsb.c"
        (_dcstep_(600,568,584,608,576,592,$273,520,528,2600,656,664)|0); //@line 4447 "lbfgsb.c"
        $274 = +HEAPF64[568>>3]; //@line 4450 "lbfgsb.c"
        $275 = +HEAPF64[600>>3]; //@line 4450 "lbfgsb.c"
        $276 = +HEAPF64[648>>3]; //@line 4450 "lbfgsb.c"
        $277 = $275 * $276; //@line 4450 "lbfgsb.c"
        $278 = $274 + $277; //@line 4450 "lbfgsb.c"
        HEAPF64[536>>3] = $278; //@line 4450 "lbfgsb.c"
        $279 = +HEAPF64[576>>3]; //@line 4451 "lbfgsb.c"
        $280 = +HEAPF64[608>>3]; //@line 4451 "lbfgsb.c"
        $281 = +HEAPF64[648>>3]; //@line 4451 "lbfgsb.c"
        $282 = $280 * $281; //@line 4451 "lbfgsb.c"
        $283 = $279 + $282; //@line 4451 "lbfgsb.c"
        HEAPF64[544>>3] = $283; //@line 4451 "lbfgsb.c"
        $284 = +HEAPF64[584>>3]; //@line 4452 "lbfgsb.c"
        $285 = +HEAPF64[648>>3]; //@line 4452 "lbfgsb.c"
        $286 = $284 + $285; //@line 4452 "lbfgsb.c"
        HEAPF64[552>>3] = $286; //@line 4452 "lbfgsb.c"
        $287 = +HEAPF64[592>>3]; //@line 4453 "lbfgsb.c"
        $288 = +HEAPF64[648>>3]; //@line 4453 "lbfgsb.c"
        $289 = $287 + $288; //@line 4453 "lbfgsb.c"
        HEAPF64[560>>3] = $289; //@line 4453 "lbfgsb.c"
       } else {
        label = 52;
       }
      } else {
       label = 52;
      }
     } else {
      label = 52;
     }
     if ((label|0) == 52) {
      $290 = $3; //@line 4456 "lbfgsb.c"
      $291 = $1; //@line 4456 "lbfgsb.c"
      $292 = $2; //@line 4456 "lbfgsb.c"
      (_dcstep_(600,536,552,608,544,560,$290,$291,$292,2600,656,664)|0); //@line 4456 "lbfgsb.c"
     }
     $293 = HEAP32[2600>>2]|0; //@line 4460 "lbfgsb.c"
     $294 = ($293|0)!=(0); //@line 4460 "lbfgsb.c"
     if ($294) {
      $295 = +HEAPF64[608>>3]; //@line 4461 "lbfgsb.c"
      $296 = +HEAPF64[600>>3]; //@line 4461 "lbfgsb.c"
      $297 = $295 - $296; //@line 4461 "lbfgsb.c"
      $d__1 = $297; //@line 4461 "lbfgsb.c"
      $298 = $d__1; //@line 4461 "lbfgsb.c"
      $299 = $298 >= 0.0; //@line 4461 "lbfgsb.c"
      $300 = $d__1; //@line 4461 "lbfgsb.c"
      $301 = -$300; //@line 4461 "lbfgsb.c"
      $302 = $299 ? $300 : $301; //@line 4461 "lbfgsb.c"
      $303 = +HEAPF64[672>>3]; //@line 4461 "lbfgsb.c"
      $304 = $303 * 0.66000000000000003; //@line 4461 "lbfgsb.c"
      $305 = $302 >= $304; //@line 4461 "lbfgsb.c"
      if ($305) {
       $306 = +HEAPF64[600>>3]; //@line 4462 "lbfgsb.c"
       $307 = +HEAPF64[608>>3]; //@line 4462 "lbfgsb.c"
       $308 = +HEAPF64[600>>3]; //@line 4462 "lbfgsb.c"
       $309 = $307 - $308; //@line 4462 "lbfgsb.c"
       $310 = $309 * 0.5; //@line 4462 "lbfgsb.c"
       $311 = $306 + $310; //@line 4462 "lbfgsb.c"
       $312 = $3; //@line 4462 "lbfgsb.c"
       HEAPF64[$312>>3] = $311; //@line 4462 "lbfgsb.c"
      }
      $313 = +HEAPF64[632>>3]; //@line 4464 "lbfgsb.c"
      HEAPF64[672>>3] = $313; //@line 4464 "lbfgsb.c"
      $314 = +HEAPF64[608>>3]; //@line 4465 "lbfgsb.c"
      $315 = +HEAPF64[600>>3]; //@line 4465 "lbfgsb.c"
      $316 = $314 - $315; //@line 4465 "lbfgsb.c"
      $d__1 = $316; //@line 4465 "lbfgsb.c"
      $317 = $d__1; //@line 4465 "lbfgsb.c"
      $318 = $317 >= 0.0; //@line 4465 "lbfgsb.c"
      $319 = $d__1; //@line 4465 "lbfgsb.c"
      $320 = -$319; //@line 4465 "lbfgsb.c"
      $321 = $318 ? $319 : $320; //@line 4465 "lbfgsb.c"
      HEAPF64[632>>3] = $321; //@line 4465 "lbfgsb.c"
     }
     $322 = HEAP32[2600>>2]|0; //@line 4468 "lbfgsb.c"
     $323 = ($322|0)!=(0); //@line 4468 "lbfgsb.c"
     if ($323) {
      $324 = +HEAPF64[600>>3]; //@line 4469 "lbfgsb.c"
      $325 = +HEAPF64[608>>3]; //@line 4469 "lbfgsb.c"
      $326 = $324 <= $325; //@line 4469 "lbfgsb.c"
      $327 = +HEAPF64[600>>3]; //@line 4469 "lbfgsb.c"
      $328 = +HEAPF64[608>>3]; //@line 4469 "lbfgsb.c"
      $329 = $326 ? $327 : $328; //@line 4469 "lbfgsb.c"
      HEAPF64[656>>3] = $329; //@line 4469 "lbfgsb.c"
      $330 = +HEAPF64[600>>3]; //@line 4470 "lbfgsb.c"
      $331 = +HEAPF64[608>>3]; //@line 4470 "lbfgsb.c"
      $332 = $330 >= $331; //@line 4470 "lbfgsb.c"
      $333 = +HEAPF64[600>>3]; //@line 4470 "lbfgsb.c"
      $334 = +HEAPF64[608>>3]; //@line 4470 "lbfgsb.c"
      $335 = $332 ? $333 : $334; //@line 4470 "lbfgsb.c"
      HEAPF64[664>>3] = $335; //@line 4470 "lbfgsb.c"
     } else {
      $336 = $3; //@line 4472 "lbfgsb.c"
      $337 = +HEAPF64[$336>>3]; //@line 4472 "lbfgsb.c"
      $338 = $3; //@line 4472 "lbfgsb.c"
      $339 = +HEAPF64[$338>>3]; //@line 4472 "lbfgsb.c"
      $340 = +HEAPF64[600>>3]; //@line 4472 "lbfgsb.c"
      $341 = $339 - $340; //@line 4472 "lbfgsb.c"
      $342 = $341 * 1.1000000000000001; //@line 4472 "lbfgsb.c"
      $343 = $337 + $342; //@line 4472 "lbfgsb.c"
      HEAPF64[656>>3] = $343; //@line 4472 "lbfgsb.c"
      $344 = $3; //@line 4473 "lbfgsb.c"
      $345 = +HEAPF64[$344>>3]; //@line 4473 "lbfgsb.c"
      $346 = $3; //@line 4473 "lbfgsb.c"
      $347 = +HEAPF64[$346>>3]; //@line 4473 "lbfgsb.c"
      $348 = +HEAPF64[600>>3]; //@line 4473 "lbfgsb.c"
      $349 = $347 - $348; //@line 4473 "lbfgsb.c"
      $350 = $349 * 4.0; //@line 4473 "lbfgsb.c"
      $351 = $345 + $350; //@line 4473 "lbfgsb.c"
      HEAPF64[664>>3] = $351; //@line 4473 "lbfgsb.c"
     }
     $352 = $3; //@line 4476 "lbfgsb.c"
     $353 = +HEAPF64[$352>>3]; //@line 4476 "lbfgsb.c"
     $354 = $7; //@line 4476 "lbfgsb.c"
     $355 = +HEAPF64[$354>>3]; //@line 4476 "lbfgsb.c"
     $356 = $353 >= $355; //@line 4476 "lbfgsb.c"
     if ($356) {
      $357 = $3; //@line 4476 "lbfgsb.c"
      $358 = +HEAPF64[$357>>3]; //@line 4476 "lbfgsb.c"
      $362 = $358;
     } else {
      $359 = $7; //@line 4476 "lbfgsb.c"
      $360 = +HEAPF64[$359>>3]; //@line 4476 "lbfgsb.c"
      $362 = $360;
     }
     $361 = $3; //@line 4476 "lbfgsb.c"
     HEAPF64[$361>>3] = $362; //@line 4476 "lbfgsb.c"
     $363 = $3; //@line 4477 "lbfgsb.c"
     $364 = +HEAPF64[$363>>3]; //@line 4477 "lbfgsb.c"
     $365 = $8; //@line 4477 "lbfgsb.c"
     $366 = +HEAPF64[$365>>3]; //@line 4477 "lbfgsb.c"
     $367 = $364 <= $366; //@line 4477 "lbfgsb.c"
     if ($367) {
      $368 = $3; //@line 4477 "lbfgsb.c"
      $369 = +HEAPF64[$368>>3]; //@line 4477 "lbfgsb.c"
      $373 = $369;
     } else {
      $370 = $8; //@line 4477 "lbfgsb.c"
      $371 = +HEAPF64[$370>>3]; //@line 4477 "lbfgsb.c"
      $373 = $371;
     }
     $372 = $3; //@line 4477 "lbfgsb.c"
     HEAPF64[$372>>3] = $373; //@line 4477 "lbfgsb.c"
     $374 = HEAP32[2600>>2]|0; //@line 4480 "lbfgsb.c"
     $375 = ($374|0)!=(0); //@line 4480 "lbfgsb.c"
     if ($375) {
      $376 = $3; //@line 4480 "lbfgsb.c"
      $377 = +HEAPF64[$376>>3]; //@line 4480 "lbfgsb.c"
      $378 = +HEAPF64[656>>3]; //@line 4480 "lbfgsb.c"
      $379 = $377 <= $378; //@line 4480 "lbfgsb.c"
      if ($379) {
       label = 71;
      } else {
       $380 = $3; //@line 4480 "lbfgsb.c"
       $381 = +HEAPF64[$380>>3]; //@line 4480 "lbfgsb.c"
       $382 = +HEAPF64[664>>3]; //@line 4480 "lbfgsb.c"
       $383 = $381 >= $382; //@line 4480 "lbfgsb.c"
       if ($383) {
        label = 71;
       } else {
        label = 69;
       }
      }
     } else {
      label = 69;
     }
     if ((label|0) == 69) {
      $384 = HEAP32[2600>>2]|0; //@line 4480 "lbfgsb.c"
      $385 = ($384|0)!=(0); //@line 4480 "lbfgsb.c"
      if ($385) {
       $386 = +HEAPF64[664>>3]; //@line 4481 "lbfgsb.c"
       $387 = +HEAPF64[656>>3]; //@line 4481 "lbfgsb.c"
       $388 = $386 - $387; //@line 4481 "lbfgsb.c"
       $389 = $6; //@line 4481 "lbfgsb.c"
       $390 = +HEAPF64[$389>>3]; //@line 4481 "lbfgsb.c"
       $391 = +HEAPF64[664>>3]; //@line 4481 "lbfgsb.c"
       $392 = $390 * $391; //@line 4481 "lbfgsb.c"
       $393 = $388 <= $392; //@line 4481 "lbfgsb.c"
       if ($393) {
        label = 71;
       }
      }
     }
     if ((label|0) == 71) {
      $394 = +HEAPF64[600>>3]; //@line 4482 "lbfgsb.c"
      $395 = $3; //@line 4482 "lbfgsb.c"
      HEAPF64[$395>>3] = $394; //@line 4482 "lbfgsb.c"
     }
     $396 = $9; //@line 4485 "lbfgsb.c"
     $397 = $12; //@line 4485 "lbfgsb.c"
     _s_copy($396,17923,$397,2); //@line 4485 "lbfgsb.c"
    }
   }
  }
 } while(0);
 $398 = HEAP32[2600>>2]|0; //@line 4488 "lbfgsb.c"
 $399 = ($398|0)!=(0); //@line 4488 "lbfgsb.c"
 $400 = $10; //@line 4489 "lbfgsb.c"
 $401 = ((($400)) + 4|0); //@line 4489 "lbfgsb.c"
 if ($399) {
  HEAP32[$401>>2] = 1; //@line 4489 "lbfgsb.c"
 } else {
  HEAP32[$401>>2] = 0; //@line 4491 "lbfgsb.c"
 }
 $402 = HEAP32[2596>>2]|0; //@line 4493 "lbfgsb.c"
 $403 = $10; //@line 4493 "lbfgsb.c"
 $404 = ((($403)) + 8|0); //@line 4493 "lbfgsb.c"
 HEAP32[$404>>2] = $402; //@line 4493 "lbfgsb.c"
 $405 = +HEAPF64[624>>3]; //@line 4494 "lbfgsb.c"
 $406 = $11; //@line 4494 "lbfgsb.c"
 $407 = ((($406)) + 8|0); //@line 4494 "lbfgsb.c"
 HEAPF64[$407>>3] = $405; //@line 4494 "lbfgsb.c"
 $408 = +HEAPF64[648>>3]; //@line 4495 "lbfgsb.c"
 $409 = $11; //@line 4495 "lbfgsb.c"
 $410 = ((($409)) + 16|0); //@line 4495 "lbfgsb.c"
 HEAPF64[$410>>3] = $408; //@line 4495 "lbfgsb.c"
 $411 = +HEAPF64[552>>3]; //@line 4496 "lbfgsb.c"
 $412 = $11; //@line 4496 "lbfgsb.c"
 $413 = ((($412)) + 24|0); //@line 4496 "lbfgsb.c"
 HEAPF64[$413>>3] = $411; //@line 4496 "lbfgsb.c"
 $414 = +HEAPF64[560>>3]; //@line 4497 "lbfgsb.c"
 $415 = $11; //@line 4497 "lbfgsb.c"
 $416 = ((($415)) + 32|0); //@line 4497 "lbfgsb.c"
 HEAPF64[$416>>3] = $414; //@line 4497 "lbfgsb.c"
 $417 = +HEAPF64[616>>3]; //@line 4498 "lbfgsb.c"
 $418 = $11; //@line 4498 "lbfgsb.c"
 $419 = ((($418)) + 40|0); //@line 4498 "lbfgsb.c"
 HEAPF64[$419>>3] = $417; //@line 4498 "lbfgsb.c"
 $420 = +HEAPF64[536>>3]; //@line 4499 "lbfgsb.c"
 $421 = $11; //@line 4499 "lbfgsb.c"
 $422 = ((($421)) + 48|0); //@line 4499 "lbfgsb.c"
 HEAPF64[$422>>3] = $420; //@line 4499 "lbfgsb.c"
 $423 = +HEAPF64[544>>3]; //@line 4500 "lbfgsb.c"
 $424 = $11; //@line 4500 "lbfgsb.c"
 $425 = ((($424)) + 56|0); //@line 4500 "lbfgsb.c"
 HEAPF64[$425>>3] = $423; //@line 4500 "lbfgsb.c"
 $426 = +HEAPF64[600>>3]; //@line 4501 "lbfgsb.c"
 $427 = $11; //@line 4501 "lbfgsb.c"
 $428 = ((($427)) + 64|0); //@line 4501 "lbfgsb.c"
 HEAPF64[$428>>3] = $426; //@line 4501 "lbfgsb.c"
 $429 = +HEAPF64[608>>3]; //@line 4502 "lbfgsb.c"
 $430 = $11; //@line 4502 "lbfgsb.c"
 $431 = ((($430)) + 72|0); //@line 4502 "lbfgsb.c"
 HEAPF64[$431>>3] = $429; //@line 4502 "lbfgsb.c"
 $432 = +HEAPF64[656>>3]; //@line 4503 "lbfgsb.c"
 $433 = $11; //@line 4503 "lbfgsb.c"
 $434 = ((($433)) + 80|0); //@line 4503 "lbfgsb.c"
 HEAPF64[$434>>3] = $432; //@line 4503 "lbfgsb.c"
 $435 = +HEAPF64[664>>3]; //@line 4504 "lbfgsb.c"
 $436 = $11; //@line 4504 "lbfgsb.c"
 $437 = ((($436)) + 88|0); //@line 4504 "lbfgsb.c"
 HEAPF64[$437>>3] = $435; //@line 4504 "lbfgsb.c"
 $438 = +HEAPF64[632>>3]; //@line 4505 "lbfgsb.c"
 $439 = $11; //@line 4505 "lbfgsb.c"
 $440 = ((($439)) + 96|0); //@line 4505 "lbfgsb.c"
 HEAPF64[$440>>3] = $438; //@line 4505 "lbfgsb.c"
 $441 = +HEAPF64[672>>3]; //@line 4506 "lbfgsb.c"
 $442 = $11; //@line 4506 "lbfgsb.c"
 $443 = ((($442)) + 104|0); //@line 4506 "lbfgsb.c"
 HEAPF64[$443>>3] = $441; //@line 4506 "lbfgsb.c"
 $0 = 0; //@line 4507 "lbfgsb.c"
 $444 = $0; //@line 4508 "lbfgsb.c"
 STACKTOP = sp;return ($444|0); //@line 4508 "lbfgsb.c"
}
function _dcstep_($stx,$fx,$dx,$sty,$fy,$dy,$stp,$fp,$dp,$brackt,$stpmin,$stpmax) {
 $stx = $stx|0;
 $fx = $fx|0;
 $dx = $dx|0;
 $sty = $sty|0;
 $fy = $fy|0;
 $dy = $dy|0;
 $stp = $stp|0;
 $fp = $fp|0;
 $dp = $dp|0;
 $brackt = $brackt|0;
 $stpmin = $stpmin|0;
 $stpmax = $stpmax|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0.0, $125 = 0.0, $126 = 0, $127 = 0.0, $128 = 0, $129 = 0.0, $13 = 0.0, $130 = 0.0, $131 = 0.0, $132 = 0.0, $133 = 0;
 var $134 = 0.0, $135 = 0, $136 = 0.0, $137 = 0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0, $143 = 0.0, $144 = 0, $145 = 0.0, $146 = 0.0, $147 = 0.0, $148 = 0, $149 = 0.0, $15 = 0.0, $150 = 0.0, $151 = 0.0;
 var $152 = 0.0, $153 = 0, $154 = 0.0, $155 = 0, $156 = 0.0, $157 = 0.0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0, $166 = 0.0, $167 = 0.0, $168 = 0.0, $169 = 0.0, $17 = 0.0;
 var $170 = 0, $171 = 0.0, $172 = 0.0, $173 = 0.0, $174 = 0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0, $186 = 0.0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0, $195 = 0.0, $196 = 0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0.0, $200 = 0, $201 = 0.0, $202 = 0.0, $203 = 0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0.0, $210 = 0.0, $211 = 0, $212 = 0.0, $213 = 0, $214 = 0, $215 = 0.0, $216 = 0.0, $217 = 0.0, $218 = 0.0, $219 = 0.0, $22 = 0.0, $220 = 0, $221 = 0.0, $222 = 0.0, $223 = 0.0;
 var $224 = 0, $225 = 0.0, $226 = 0, $227 = 0, $228 = 0.0, $229 = 0.0, $23 = 0.0, $230 = 0.0, $231 = 0.0, $232 = 0.0, $233 = 0, $234 = 0.0, $235 = 0.0, $236 = 0.0, $237 = 0.0, $238 = 0.0, $239 = 0.0, $24 = 0.0, $240 = 0.0, $241 = 0.0;
 var $242 = 0.0, $243 = 0.0, $244 = 0, $245 = 0.0, $246 = 0.0, $247 = 0.0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0.0, $251 = 0.0, $252 = 0.0, $253 = 0.0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0.0, $258 = 0, $259 = 0.0, $26 = 0.0;
 var $260 = 0, $261 = 0.0, $262 = 0.0, $263 = 0.0, $264 = 0, $265 = 0.0, $266 = 0.0, $267 = 0.0, $268 = 0.0, $269 = 0.0, $27 = 0, $270 = 0, $271 = 0.0, $272 = 0.0, $273 = 0.0, $274 = 0.0, $275 = 0, $276 = 0.0, $277 = 0.0, $278 = 0.0;
 var $279 = 0.0, $28 = 0.0, $280 = 0.0, $281 = 0, $282 = 0.0, $283 = 0.0, $284 = 0, $285 = 0.0, $286 = 0, $287 = 0.0, $288 = 0.0, $289 = 0.0, $29 = 0, $290 = 0.0, $291 = 0, $292 = 0.0, $293 = 0, $294 = 0.0, $295 = 0, $296 = 0.0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0, $302 = 0.0, $303 = 0, $304 = 0.0, $305 = 0.0, $306 = 0.0, $307 = 0.0, $308 = 0.0, $309 = 0, $31 = 0.0, $310 = 0.0, $311 = 0.0, $312 = 0.0, $313 = 0;
 var $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0.0, $32 = 0, $320 = 0.0, $321 = 0.0, $322 = 0, $323 = 0.0, $324 = 0.0, $325 = 0.0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0, $33 = 0.0, $330 = 0, $331 = 0.0;
 var $332 = 0, $333 = 0, $334 = 0.0, $335 = 0.0, $336 = 0.0, $337 = 0, $338 = 0.0, $339 = 0, $34 = 0.0, $340 = 0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0, $345 = 0, $346 = 0.0, $347 = 0, $348 = 0.0, $349 = 0.0, $35 = 0.0;
 var $350 = 0.0, $351 = 0, $352 = 0.0, $353 = 0, $354 = 0.0, $355 = 0.0, $356 = 0.0, $357 = 0, $358 = 0.0, $359 = 0.0, $36 = 0, $360 = 0, $361 = 0.0, $362 = 0.0, $363 = 0.0, $364 = 0, $365 = 0.0, $366 = 0.0, $367 = 0.0, $368 = 0;
 var $369 = 0.0, $37 = 0.0, $370 = 0, $371 = 0, $372 = 0.0, $373 = 0.0, $374 = 0.0, $375 = 0.0, $376 = 0.0, $377 = 0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0.0, $381 = 0, $382 = 0.0, $383 = 0, $384 = 0, $385 = 0.0, $386 = 0.0;
 var $387 = 0.0, $388 = 0.0, $389 = 0.0, $39 = 0.0, $390 = 0, $391 = 0.0, $392 = 0.0, $393 = 0.0, $394 = 0.0, $395 = 0.0, $396 = 0.0, $397 = 0.0, $398 = 0.0, $399 = 0.0, $4 = 0, $40 = 0.0, $400 = 0, $401 = 0.0, $402 = 0.0, $403 = 0.0;
 var $404 = 0, $405 = 0.0, $406 = 0.0, $407 = 0.0, $408 = 0.0, $409 = 0.0, $41 = 0.0, $410 = 0.0, $411 = 0.0, $412 = 0.0, $413 = 0, $414 = 0.0, $415 = 0.0, $416 = 0.0, $417 = 0.0, $418 = 0.0, $419 = 0, $42 = 0, $420 = 0.0, $421 = 0;
 var $422 = 0.0, $423 = 0, $424 = 0.0, $425 = 0.0, $426 = 0.0, $427 = 0, $428 = 0.0, $429 = 0.0, $43 = 0.0, $430 = 0.0, $431 = 0.0, $432 = 0.0, $433 = 0, $434 = 0.0, $435 = 0, $436 = 0.0, $437 = 0.0, $438 = 0.0, $439 = 0.0, $44 = 0.0;
 var $440 = 0.0, $441 = 0.0, $442 = 0.0, $443 = 0.0, $444 = 0.0, $445 = 0, $446 = 0.0, $447 = 0, $448 = 0, $449 = 0.0, $45 = 0, $450 = 0.0, $451 = 0, $452 = 0.0, $453 = 0, $454 = 0.0, $455 = 0.0, $456 = 0.0, $457 = 0.0, $458 = 0;
 var $459 = 0.0, $46 = 0.0, $460 = 0, $461 = 0, $462 = 0.0, $463 = 0, $464 = 0.0, $465 = 0, $466 = 0.0, $467 = 0, $468 = 0.0, $469 = 0, $47 = 0.0, $470 = 0.0, $471 = 0, $472 = 0.0, $473 = 0.0, $474 = 0.0, $475 = 0, $476 = 0.0;
 var $477 = 0, $478 = 0.0, $479 = 0.0, $48 = 0.0, $480 = 0.0, $481 = 0.0, $482 = 0, $483 = 0, $484 = 0, $485 = 0.0, $486 = 0, $487 = 0.0, $488 = 0.0, $489 = 0.0, $49 = 0, $490 = 0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0.0;
 var $495 = 0, $496 = 0.0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0.0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0, $504 = 0.0, $505 = 0.0, $506 = 0, $507 = 0.0, $508 = 0, $509 = 0.0, $51 = 0.0, $510 = 0, $511 = 0;
 var $512 = 0.0, $513 = 0, $514 = 0.0, $515 = 0, $516 = 0.0, $517 = 0.0, $518 = 0.0, $519 = 0.0, $52 = 0.0, $520 = 0.0, $521 = 0.0, $522 = 0, $523 = 0.0, $524 = 0.0, $525 = 0.0, $526 = 0, $527 = 0.0, $528 = 0.0, $529 = 0.0, $53 = 0;
 var $530 = 0, $531 = 0.0, $532 = 0.0, $533 = 0, $534 = 0.0, $535 = 0.0, $536 = 0, $537 = 0, $538 = 0.0, $539 = 0.0, $54 = 0.0, $540 = 0.0, $541 = 0, $542 = 0.0, $543 = 0.0, $544 = 0, $545 = 0, $546 = 0.0, $547 = 0.0, $548 = 0.0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0.0, $554 = 0, $555 = 0.0, $556 = 0.0, $557 = 0.0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0, $561 = 0.0, $562 = 0.0, $563 = 0.0, $564 = 0, $565 = 0.0, $566 = 0.0;
 var $567 = 0, $568 = 0.0, $569 = 0.0, $57 = 0.0, $570 = 0.0, $571 = 0, $572 = 0.0, $573 = 0.0, $574 = 0.0, $575 = 0, $576 = 0.0, $577 = 0, $578 = 0, $579 = 0.0, $58 = 0.0, $580 = 0.0, $581 = 0.0, $582 = 0.0, $583 = 0.0, $584 = 0;
 var $585 = 0.0, $586 = 0.0, $587 = 0.0, $588 = 0, $589 = 0.0, $59 = 0.0, $590 = 0, $591 = 0, $592 = 0.0, $593 = 0.0, $594 = 0.0, $595 = 0.0, $596 = 0.0, $597 = 0, $598 = 0.0, $599 = 0.0, $6 = 0, $60 = 0.0, $600 = 0.0, $601 = 0.0;
 var $602 = 0.0, $603 = 0.0, $604 = 0.0, $605 = 0.0, $606 = 0.0, $607 = 0.0, $608 = 0, $609 = 0.0, $61 = 0.0, $610 = 0.0, $611 = 0.0, $612 = 0, $613 = 0.0, $614 = 0.0, $615 = 0.0, $616 = 0.0, $617 = 0.0, $618 = 0.0, $619 = 0.0, $62 = 0;
 var $620 = 0, $621 = 0.0, $622 = 0, $623 = 0.0, $624 = 0, $625 = 0.0, $626 = 0.0, $627 = 0.0, $628 = 0, $629 = 0.0, $63 = 0.0, $630 = 0.0, $631 = 0.0, $632 = 0.0, $633 = 0.0, $634 = 0, $635 = 0.0, $636 = 0.0, $637 = 0.0, $638 = 0.0;
 var $639 = 0, $64 = 0.0, $640 = 0.0, $641 = 0.0, $642 = 0.0, $643 = 0.0, $644 = 0.0, $645 = 0, $646 = 0.0, $647 = 0.0, $648 = 0, $649 = 0.0, $65 = 0.0, $650 = 0, $651 = 0.0, $652 = 0.0, $653 = 0.0, $654 = 0.0, $655 = 0.0, $656 = 0;
 var $657 = 0.0, $658 = 0, $659 = 0.0, $66 = 0, $660 = 0, $661 = 0, $662 = 0.0, $663 = 0, $664 = 0.0, $665 = 0, $666 = 0.0, $667 = 0, $668 = 0.0, $669 = 0, $67 = 0.0, $670 = 0, $671 = 0.0, $672 = 0, $673 = 0, $674 = 0.0;
 var $675 = 0, $676 = 0, $677 = 0.0, $678 = 0, $679 = 0.0, $68 = 0, $680 = 0, $681 = 0, $682 = 0.0, $683 = 0, $684 = 0, $685 = 0.0, $686 = 0, $687 = 0, $688 = 0.0, $689 = 0, $69 = 0, $690 = 0, $691 = 0.0, $692 = 0;
 var $693 = 0, $694 = 0.0, $695 = 0, $696 = 0, $697 = 0.0, $698 = 0, $699 = 0.0, $7 = 0, $70 = 0.0, $700 = 0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0;
 var $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $94 = 0.0, $95 = 0.0, $96 = 0.0, $97 = 0.0, $98 = 0;
 var $99 = 0.0, $d__1 = 0.0, $d__2 = 0.0, $d__3 = 0.0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $stx;
 $1 = $fx;
 $2 = $dx;
 $3 = $sty;
 $4 = $fy;
 $5 = $dy;
 $6 = $stp;
 $7 = $fp;
 $8 = $dp;
 $9 = $brackt;
 $10 = $stpmin;
 $11 = $stpmax;
 $12 = $8; //@line 4617 "lbfgsb.c"
 $13 = +HEAPF64[$12>>3]; //@line 4617 "lbfgsb.c"
 $14 = $2; //@line 4617 "lbfgsb.c"
 $15 = +HEAPF64[$14>>3]; //@line 4617 "lbfgsb.c"
 $16 = $2; //@line 4617 "lbfgsb.c"
 $17 = +HEAPF64[$16>>3]; //@line 4617 "lbfgsb.c"
 $18 = $17 >= 0.0; //@line 4617 "lbfgsb.c"
 $19 = $2; //@line 4617 "lbfgsb.c"
 $20 = +HEAPF64[$19>>3]; //@line 4617 "lbfgsb.c"
 $21 = -$20; //@line 4617 "lbfgsb.c"
 $22 = $18 ? $20 : $21; //@line 4617 "lbfgsb.c"
 $23 = $15 / $22; //@line 4617 "lbfgsb.c"
 $24 = $13 * $23; //@line 4617 "lbfgsb.c"
 HEAPF64[712>>3] = $24; //@line 4617 "lbfgsb.c"
 $25 = $7; //@line 4622 "lbfgsb.c"
 $26 = +HEAPF64[$25>>3]; //@line 4622 "lbfgsb.c"
 $27 = $1; //@line 4622 "lbfgsb.c"
 $28 = +HEAPF64[$27>>3]; //@line 4622 "lbfgsb.c"
 $29 = $26 > $28; //@line 4622 "lbfgsb.c"
 do {
  if ($29) {
   $30 = $1; //@line 4623 "lbfgsb.c"
   $31 = +HEAPF64[$30>>3]; //@line 4623 "lbfgsb.c"
   $32 = $7; //@line 4623 "lbfgsb.c"
   $33 = +HEAPF64[$32>>3]; //@line 4623 "lbfgsb.c"
   $34 = $31 - $33; //@line 4623 "lbfgsb.c"
   $35 = $34 * 3.0; //@line 4623 "lbfgsb.c"
   $36 = $6; //@line 4623 "lbfgsb.c"
   $37 = +HEAPF64[$36>>3]; //@line 4623 "lbfgsb.c"
   $38 = $0; //@line 4623 "lbfgsb.c"
   $39 = +HEAPF64[$38>>3]; //@line 4623 "lbfgsb.c"
   $40 = $37 - $39; //@line 4623 "lbfgsb.c"
   $41 = $35 / $40; //@line 4623 "lbfgsb.c"
   $42 = $2; //@line 4623 "lbfgsb.c"
   $43 = +HEAPF64[$42>>3]; //@line 4623 "lbfgsb.c"
   $44 = $41 + $43; //@line 4623 "lbfgsb.c"
   $45 = $8; //@line 4623 "lbfgsb.c"
   $46 = +HEAPF64[$45>>3]; //@line 4623 "lbfgsb.c"
   $47 = $44 + $46; //@line 4623 "lbfgsb.c"
   HEAPF64[752>>3] = $47; //@line 4623 "lbfgsb.c"
   $48 = +HEAPF64[752>>3]; //@line 4625 "lbfgsb.c"
   $49 = $48 >= 0.0; //@line 4625 "lbfgsb.c"
   $50 = +HEAPF64[752>>3]; //@line 4625 "lbfgsb.c"
   $51 = -$50; //@line 4625 "lbfgsb.c"
   $52 = $49 ? $50 : $51; //@line 4625 "lbfgsb.c"
   $d__1 = $52; //@line 4625 "lbfgsb.c"
   $53 = $2; //@line 4625 "lbfgsb.c"
   $54 = +HEAPF64[$53>>3]; //@line 4625 "lbfgsb.c"
   $55 = $54 >= 0.0; //@line 4625 "lbfgsb.c"
   $56 = $2; //@line 4625 "lbfgsb.c"
   $57 = +HEAPF64[$56>>3]; //@line 4625 "lbfgsb.c"
   $58 = -$57; //@line 4625 "lbfgsb.c"
   $59 = $55 ? $57 : $58; //@line 4625 "lbfgsb.c"
   $d__2 = $59; //@line 4625 "lbfgsb.c"
   $60 = $d__1; //@line 4625 "lbfgsb.c"
   $61 = $d__2; //@line 4625 "lbfgsb.c"
   $62 = $60 >= $61; //@line 4625 "lbfgsb.c"
   $63 = $d__1; //@line 4625 "lbfgsb.c"
   $64 = $d__2; //@line 4625 "lbfgsb.c"
   $65 = $62 ? $63 : $64; //@line 4625 "lbfgsb.c"
   $d__1 = $65; //@line 4625 "lbfgsb.c"
   $66 = $8; //@line 4625 "lbfgsb.c"
   $67 = +HEAPF64[$66>>3]; //@line 4625 "lbfgsb.c"
   $68 = $67 >= 0.0; //@line 4625 "lbfgsb.c"
   $69 = $8; //@line 4625 "lbfgsb.c"
   $70 = +HEAPF64[$69>>3]; //@line 4625 "lbfgsb.c"
   $71 = -$70; //@line 4625 "lbfgsb.c"
   $72 = $68 ? $70 : $71; //@line 4625 "lbfgsb.c"
   $d__2 = $72; //@line 4625 "lbfgsb.c"
   $73 = $d__1; //@line 4627 "lbfgsb.c"
   $74 = $d__2; //@line 4627 "lbfgsb.c"
   $75 = $73 >= $74; //@line 4627 "lbfgsb.c"
   $76 = $d__1; //@line 4627 "lbfgsb.c"
   $77 = $d__2; //@line 4627 "lbfgsb.c"
   $78 = $75 ? $76 : $77; //@line 4627 "lbfgsb.c"
   HEAPF64[704>>3] = $78; //@line 4627 "lbfgsb.c"
   $79 = +HEAPF64[752>>3]; //@line 4629 "lbfgsb.c"
   $80 = +HEAPF64[704>>3]; //@line 4629 "lbfgsb.c"
   $81 = $79 / $80; //@line 4629 "lbfgsb.c"
   $d__1 = $81; //@line 4629 "lbfgsb.c"
   $82 = +HEAPF64[704>>3]; //@line 4630 "lbfgsb.c"
   $83 = $d__1; //@line 4630 "lbfgsb.c"
   $84 = $d__1; //@line 4630 "lbfgsb.c"
   $85 = $83 * $84; //@line 4630 "lbfgsb.c"
   $86 = $2; //@line 4630 "lbfgsb.c"
   $87 = +HEAPF64[$86>>3]; //@line 4630 "lbfgsb.c"
   $88 = +HEAPF64[704>>3]; //@line 4630 "lbfgsb.c"
   $89 = $87 / $88; //@line 4630 "lbfgsb.c"
   $90 = $8; //@line 4630 "lbfgsb.c"
   $91 = +HEAPF64[$90>>3]; //@line 4630 "lbfgsb.c"
   $92 = +HEAPF64[704>>3]; //@line 4630 "lbfgsb.c"
   $93 = $91 / $92; //@line 4630 "lbfgsb.c"
   $94 = $89 * $93; //@line 4630 "lbfgsb.c"
   $95 = $85 - $94; //@line 4630 "lbfgsb.c"
   $96 = (+Math_sqrt((+$95))); //@line 4630 "lbfgsb.c"
   $97 = $82 * $96; //@line 4630 "lbfgsb.c"
   HEAPF64[744>>3] = $97; //@line 4630 "lbfgsb.c"
   $98 = $6; //@line 4631 "lbfgsb.c"
   $99 = +HEAPF64[$98>>3]; //@line 4631 "lbfgsb.c"
   $100 = $0; //@line 4631 "lbfgsb.c"
   $101 = +HEAPF64[$100>>3]; //@line 4631 "lbfgsb.c"
   $102 = $99 < $101; //@line 4631 "lbfgsb.c"
   if ($102) {
    $103 = +HEAPF64[744>>3]; //@line 4632 "lbfgsb.c"
    $104 = -$103; //@line 4632 "lbfgsb.c"
    HEAPF64[744>>3] = $104; //@line 4632 "lbfgsb.c"
   }
   $105 = +HEAPF64[744>>3]; //@line 4634 "lbfgsb.c"
   $106 = $2; //@line 4634 "lbfgsb.c"
   $107 = +HEAPF64[$106>>3]; //@line 4634 "lbfgsb.c"
   $108 = $105 - $107; //@line 4634 "lbfgsb.c"
   $109 = +HEAPF64[752>>3]; //@line 4634 "lbfgsb.c"
   $110 = $108 + $109; //@line 4634 "lbfgsb.c"
   HEAPF64[680>>3] = $110; //@line 4634 "lbfgsb.c"
   $111 = +HEAPF64[744>>3]; //@line 4635 "lbfgsb.c"
   $112 = $2; //@line 4635 "lbfgsb.c"
   $113 = +HEAPF64[$112>>3]; //@line 4635 "lbfgsb.c"
   $114 = $111 - $113; //@line 4635 "lbfgsb.c"
   $115 = +HEAPF64[744>>3]; //@line 4635 "lbfgsb.c"
   $116 = $114 + $115; //@line 4635 "lbfgsb.c"
   $117 = $8; //@line 4635 "lbfgsb.c"
   $118 = +HEAPF64[$117>>3]; //@line 4635 "lbfgsb.c"
   $119 = $116 + $118; //@line 4635 "lbfgsb.c"
   HEAPF64[688>>3] = $119; //@line 4635 "lbfgsb.c"
   $120 = +HEAPF64[680>>3]; //@line 4636 "lbfgsb.c"
   $121 = +HEAPF64[688>>3]; //@line 4636 "lbfgsb.c"
   $122 = $120 / $121; //@line 4636 "lbfgsb.c"
   HEAPF64[696>>3] = $122; //@line 4636 "lbfgsb.c"
   $123 = $0; //@line 4637 "lbfgsb.c"
   $124 = +HEAPF64[$123>>3]; //@line 4637 "lbfgsb.c"
   $125 = +HEAPF64[696>>3]; //@line 4637 "lbfgsb.c"
   $126 = $6; //@line 4637 "lbfgsb.c"
   $127 = +HEAPF64[$126>>3]; //@line 4637 "lbfgsb.c"
   $128 = $0; //@line 4637 "lbfgsb.c"
   $129 = +HEAPF64[$128>>3]; //@line 4637 "lbfgsb.c"
   $130 = $127 - $129; //@line 4637 "lbfgsb.c"
   $131 = $125 * $130; //@line 4637 "lbfgsb.c"
   $132 = $124 + $131; //@line 4637 "lbfgsb.c"
   HEAPF64[720>>3] = $132; //@line 4637 "lbfgsb.c"
   $133 = $0; //@line 4638 "lbfgsb.c"
   $134 = +HEAPF64[$133>>3]; //@line 4638 "lbfgsb.c"
   $135 = $2; //@line 4638 "lbfgsb.c"
   $136 = +HEAPF64[$135>>3]; //@line 4638 "lbfgsb.c"
   $137 = $1; //@line 4638 "lbfgsb.c"
   $138 = +HEAPF64[$137>>3]; //@line 4638 "lbfgsb.c"
   $139 = $7; //@line 4638 "lbfgsb.c"
   $140 = +HEAPF64[$139>>3]; //@line 4638 "lbfgsb.c"
   $141 = $138 - $140; //@line 4638 "lbfgsb.c"
   $142 = $6; //@line 4638 "lbfgsb.c"
   $143 = +HEAPF64[$142>>3]; //@line 4638 "lbfgsb.c"
   $144 = $0; //@line 4638 "lbfgsb.c"
   $145 = +HEAPF64[$144>>3]; //@line 4638 "lbfgsb.c"
   $146 = $143 - $145; //@line 4638 "lbfgsb.c"
   $147 = $141 / $146; //@line 4638 "lbfgsb.c"
   $148 = $2; //@line 4638 "lbfgsb.c"
   $149 = +HEAPF64[$148>>3]; //@line 4638 "lbfgsb.c"
   $150 = $147 + $149; //@line 4638 "lbfgsb.c"
   $151 = $136 / $150; //@line 4638 "lbfgsb.c"
   $152 = $151 / 2.0; //@line 4638 "lbfgsb.c"
   $153 = $6; //@line 4638 "lbfgsb.c"
   $154 = +HEAPF64[$153>>3]; //@line 4638 "lbfgsb.c"
   $155 = $0; //@line 4639 "lbfgsb.c"
   $156 = +HEAPF64[$155>>3]; //@line 4639 "lbfgsb.c"
   $157 = $154 - $156; //@line 4639 "lbfgsb.c"
   $158 = $152 * $157; //@line 4638 "lbfgsb.c"
   $159 = $134 + $158; //@line 4638 "lbfgsb.c"
   HEAPF64[736>>3] = $159; //@line 4638 "lbfgsb.c"
   $160 = +HEAPF64[720>>3]; //@line 4640 "lbfgsb.c"
   $161 = $0; //@line 4640 "lbfgsb.c"
   $162 = +HEAPF64[$161>>3]; //@line 4640 "lbfgsb.c"
   $163 = $160 - $162; //@line 4640 "lbfgsb.c"
   $d__1 = $163; //@line 4640 "lbfgsb.c"
   $164 = $d__1; //@line 4640 "lbfgsb.c"
   $165 = $164 >= 0.0; //@line 4640 "lbfgsb.c"
   $166 = $d__1; //@line 4640 "lbfgsb.c"
   $167 = -$166; //@line 4640 "lbfgsb.c"
   $168 = $165 ? $166 : $167; //@line 4640 "lbfgsb.c"
   $169 = +HEAPF64[736>>3]; //@line 4640 "lbfgsb.c"
   $170 = $0; //@line 4640 "lbfgsb.c"
   $171 = +HEAPF64[$170>>3]; //@line 4640 "lbfgsb.c"
   $172 = $169 - $171; //@line 4640 "lbfgsb.c"
   $d__2 = $172; //@line 4640 "lbfgsb.c"
   $173 = $d__2; //@line 4640 "lbfgsb.c"
   $174 = $173 >= 0.0; //@line 4640 "lbfgsb.c"
   $175 = $d__2; //@line 4640 "lbfgsb.c"
   $176 = -$175; //@line 4640 "lbfgsb.c"
   $177 = $174 ? $175 : $176; //@line 4640 "lbfgsb.c"
   $178 = $168 < $177; //@line 4640 "lbfgsb.c"
   $179 = +HEAPF64[720>>3]; //@line 4642 "lbfgsb.c"
   if ($178) {
    HEAPF64[728>>3] = $179; //@line 4642 "lbfgsb.c"
   } else {
    $180 = +HEAPF64[736>>3]; //@line 4644 "lbfgsb.c"
    $181 = +HEAPF64[720>>3]; //@line 4644 "lbfgsb.c"
    $182 = $180 - $181; //@line 4644 "lbfgsb.c"
    $183 = $182 / 2.0; //@line 4644 "lbfgsb.c"
    $184 = $179 + $183; //@line 4644 "lbfgsb.c"
    HEAPF64[728>>3] = $184; //@line 4644 "lbfgsb.c"
   }
   $185 = $9; //@line 4646 "lbfgsb.c"
   HEAP32[$185>>2] = 1; //@line 4646 "lbfgsb.c"
  } else {
   $186 = +HEAPF64[712>>3]; //@line 4651 "lbfgsb.c"
   $187 = $186 < 0.0; //@line 4651 "lbfgsb.c"
   if ($187) {
    $188 = $1; //@line 4652 "lbfgsb.c"
    $189 = +HEAPF64[$188>>3]; //@line 4652 "lbfgsb.c"
    $190 = $7; //@line 4652 "lbfgsb.c"
    $191 = +HEAPF64[$190>>3]; //@line 4652 "lbfgsb.c"
    $192 = $189 - $191; //@line 4652 "lbfgsb.c"
    $193 = $192 * 3.0; //@line 4652 "lbfgsb.c"
    $194 = $6; //@line 4652 "lbfgsb.c"
    $195 = +HEAPF64[$194>>3]; //@line 4652 "lbfgsb.c"
    $196 = $0; //@line 4652 "lbfgsb.c"
    $197 = +HEAPF64[$196>>3]; //@line 4652 "lbfgsb.c"
    $198 = $195 - $197; //@line 4652 "lbfgsb.c"
    $199 = $193 / $198; //@line 4652 "lbfgsb.c"
    $200 = $2; //@line 4652 "lbfgsb.c"
    $201 = +HEAPF64[$200>>3]; //@line 4652 "lbfgsb.c"
    $202 = $199 + $201; //@line 4652 "lbfgsb.c"
    $203 = $8; //@line 4652 "lbfgsb.c"
    $204 = +HEAPF64[$203>>3]; //@line 4652 "lbfgsb.c"
    $205 = $202 + $204; //@line 4652 "lbfgsb.c"
    HEAPF64[752>>3] = $205; //@line 4652 "lbfgsb.c"
    $206 = +HEAPF64[752>>3]; //@line 4654 "lbfgsb.c"
    $207 = $206 >= 0.0; //@line 4654 "lbfgsb.c"
    $208 = +HEAPF64[752>>3]; //@line 4654 "lbfgsb.c"
    $209 = -$208; //@line 4654 "lbfgsb.c"
    $210 = $207 ? $208 : $209; //@line 4654 "lbfgsb.c"
    $d__1 = $210; //@line 4654 "lbfgsb.c"
    $211 = $2; //@line 4654 "lbfgsb.c"
    $212 = +HEAPF64[$211>>3]; //@line 4654 "lbfgsb.c"
    $213 = $212 >= 0.0; //@line 4654 "lbfgsb.c"
    $214 = $2; //@line 4654 "lbfgsb.c"
    $215 = +HEAPF64[$214>>3]; //@line 4654 "lbfgsb.c"
    $216 = -$215; //@line 4654 "lbfgsb.c"
    $217 = $213 ? $215 : $216; //@line 4654 "lbfgsb.c"
    $d__2 = $217; //@line 4654 "lbfgsb.c"
    $218 = $d__1; //@line 4654 "lbfgsb.c"
    $219 = $d__2; //@line 4654 "lbfgsb.c"
    $220 = $218 >= $219; //@line 4654 "lbfgsb.c"
    $221 = $d__1; //@line 4654 "lbfgsb.c"
    $222 = $d__2; //@line 4654 "lbfgsb.c"
    $223 = $220 ? $221 : $222; //@line 4654 "lbfgsb.c"
    $d__1 = $223; //@line 4654 "lbfgsb.c"
    $224 = $8; //@line 4654 "lbfgsb.c"
    $225 = +HEAPF64[$224>>3]; //@line 4654 "lbfgsb.c"
    $226 = $225 >= 0.0; //@line 4654 "lbfgsb.c"
    $227 = $8; //@line 4654 "lbfgsb.c"
    $228 = +HEAPF64[$227>>3]; //@line 4654 "lbfgsb.c"
    $229 = -$228; //@line 4654 "lbfgsb.c"
    $230 = $226 ? $228 : $229; //@line 4654 "lbfgsb.c"
    $d__2 = $230; //@line 4654 "lbfgsb.c"
    $231 = $d__1; //@line 4656 "lbfgsb.c"
    $232 = $d__2; //@line 4656 "lbfgsb.c"
    $233 = $231 >= $232; //@line 4656 "lbfgsb.c"
    $234 = $d__1; //@line 4656 "lbfgsb.c"
    $235 = $d__2; //@line 4656 "lbfgsb.c"
    $236 = $233 ? $234 : $235; //@line 4656 "lbfgsb.c"
    HEAPF64[704>>3] = $236; //@line 4656 "lbfgsb.c"
    $237 = +HEAPF64[752>>3]; //@line 4658 "lbfgsb.c"
    $238 = +HEAPF64[704>>3]; //@line 4658 "lbfgsb.c"
    $239 = $237 / $238; //@line 4658 "lbfgsb.c"
    $d__1 = $239; //@line 4658 "lbfgsb.c"
    $240 = +HEAPF64[704>>3]; //@line 4659 "lbfgsb.c"
    $241 = $d__1; //@line 4659 "lbfgsb.c"
    $242 = $d__1; //@line 4659 "lbfgsb.c"
    $243 = $241 * $242; //@line 4659 "lbfgsb.c"
    $244 = $2; //@line 4659 "lbfgsb.c"
    $245 = +HEAPF64[$244>>3]; //@line 4659 "lbfgsb.c"
    $246 = +HEAPF64[704>>3]; //@line 4659 "lbfgsb.c"
    $247 = $245 / $246; //@line 4659 "lbfgsb.c"
    $248 = $8; //@line 4659 "lbfgsb.c"
    $249 = +HEAPF64[$248>>3]; //@line 4659 "lbfgsb.c"
    $250 = +HEAPF64[704>>3]; //@line 4659 "lbfgsb.c"
    $251 = $249 / $250; //@line 4659 "lbfgsb.c"
    $252 = $247 * $251; //@line 4659 "lbfgsb.c"
    $253 = $243 - $252; //@line 4659 "lbfgsb.c"
    $254 = (+Math_sqrt((+$253))); //@line 4659 "lbfgsb.c"
    $255 = $240 * $254; //@line 4659 "lbfgsb.c"
    HEAPF64[744>>3] = $255; //@line 4659 "lbfgsb.c"
    $256 = $6; //@line 4660 "lbfgsb.c"
    $257 = +HEAPF64[$256>>3]; //@line 4660 "lbfgsb.c"
    $258 = $0; //@line 4660 "lbfgsb.c"
    $259 = +HEAPF64[$258>>3]; //@line 4660 "lbfgsb.c"
    $260 = $257 > $259; //@line 4660 "lbfgsb.c"
    if ($260) {
     $261 = +HEAPF64[744>>3]; //@line 4661 "lbfgsb.c"
     $262 = -$261; //@line 4661 "lbfgsb.c"
     HEAPF64[744>>3] = $262; //@line 4661 "lbfgsb.c"
    }
    $263 = +HEAPF64[744>>3]; //@line 4663 "lbfgsb.c"
    $264 = $8; //@line 4663 "lbfgsb.c"
    $265 = +HEAPF64[$264>>3]; //@line 4663 "lbfgsb.c"
    $266 = $263 - $265; //@line 4663 "lbfgsb.c"
    $267 = +HEAPF64[752>>3]; //@line 4663 "lbfgsb.c"
    $268 = $266 + $267; //@line 4663 "lbfgsb.c"
    HEAPF64[680>>3] = $268; //@line 4663 "lbfgsb.c"
    $269 = +HEAPF64[744>>3]; //@line 4664 "lbfgsb.c"
    $270 = $8; //@line 4664 "lbfgsb.c"
    $271 = +HEAPF64[$270>>3]; //@line 4664 "lbfgsb.c"
    $272 = $269 - $271; //@line 4664 "lbfgsb.c"
    $273 = +HEAPF64[744>>3]; //@line 4664 "lbfgsb.c"
    $274 = $272 + $273; //@line 4664 "lbfgsb.c"
    $275 = $2; //@line 4664 "lbfgsb.c"
    $276 = +HEAPF64[$275>>3]; //@line 4664 "lbfgsb.c"
    $277 = $274 + $276; //@line 4664 "lbfgsb.c"
    HEAPF64[688>>3] = $277; //@line 4664 "lbfgsb.c"
    $278 = +HEAPF64[680>>3]; //@line 4665 "lbfgsb.c"
    $279 = +HEAPF64[688>>3]; //@line 4665 "lbfgsb.c"
    $280 = $278 / $279; //@line 4665 "lbfgsb.c"
    HEAPF64[696>>3] = $280; //@line 4665 "lbfgsb.c"
    $281 = $6; //@line 4666 "lbfgsb.c"
    $282 = +HEAPF64[$281>>3]; //@line 4666 "lbfgsb.c"
    $283 = +HEAPF64[696>>3]; //@line 4666 "lbfgsb.c"
    $284 = $0; //@line 4666 "lbfgsb.c"
    $285 = +HEAPF64[$284>>3]; //@line 4666 "lbfgsb.c"
    $286 = $6; //@line 4666 "lbfgsb.c"
    $287 = +HEAPF64[$286>>3]; //@line 4666 "lbfgsb.c"
    $288 = $285 - $287; //@line 4666 "lbfgsb.c"
    $289 = $283 * $288; //@line 4666 "lbfgsb.c"
    $290 = $282 + $289; //@line 4666 "lbfgsb.c"
    HEAPF64[720>>3] = $290; //@line 4666 "lbfgsb.c"
    $291 = $6; //@line 4667 "lbfgsb.c"
    $292 = +HEAPF64[$291>>3]; //@line 4667 "lbfgsb.c"
    $293 = $8; //@line 4667 "lbfgsb.c"
    $294 = +HEAPF64[$293>>3]; //@line 4667 "lbfgsb.c"
    $295 = $8; //@line 4667 "lbfgsb.c"
    $296 = +HEAPF64[$295>>3]; //@line 4667 "lbfgsb.c"
    $297 = $2; //@line 4667 "lbfgsb.c"
    $298 = +HEAPF64[$297>>3]; //@line 4667 "lbfgsb.c"
    $299 = $296 - $298; //@line 4667 "lbfgsb.c"
    $300 = $294 / $299; //@line 4667 "lbfgsb.c"
    $301 = $0; //@line 4667 "lbfgsb.c"
    $302 = +HEAPF64[$301>>3]; //@line 4667 "lbfgsb.c"
    $303 = $6; //@line 4667 "lbfgsb.c"
    $304 = +HEAPF64[$303>>3]; //@line 4667 "lbfgsb.c"
    $305 = $302 - $304; //@line 4667 "lbfgsb.c"
    $306 = $300 * $305; //@line 4667 "lbfgsb.c"
    $307 = $292 + $306; //@line 4667 "lbfgsb.c"
    HEAPF64[736>>3] = $307; //@line 4667 "lbfgsb.c"
    $308 = +HEAPF64[720>>3]; //@line 4668 "lbfgsb.c"
    $309 = $6; //@line 4668 "lbfgsb.c"
    $310 = +HEAPF64[$309>>3]; //@line 4668 "lbfgsb.c"
    $311 = $308 - $310; //@line 4668 "lbfgsb.c"
    $d__1 = $311; //@line 4668 "lbfgsb.c"
    $312 = $d__1; //@line 4668 "lbfgsb.c"
    $313 = $312 >= 0.0; //@line 4668 "lbfgsb.c"
    $314 = $d__1; //@line 4668 "lbfgsb.c"
    $315 = -$314; //@line 4668 "lbfgsb.c"
    $316 = $313 ? $314 : $315; //@line 4668 "lbfgsb.c"
    $317 = +HEAPF64[736>>3]; //@line 4668 "lbfgsb.c"
    $318 = $6; //@line 4668 "lbfgsb.c"
    $319 = +HEAPF64[$318>>3]; //@line 4668 "lbfgsb.c"
    $320 = $317 - $319; //@line 4668 "lbfgsb.c"
    $d__2 = $320; //@line 4668 "lbfgsb.c"
    $321 = $d__2; //@line 4668 "lbfgsb.c"
    $322 = $321 >= 0.0; //@line 4668 "lbfgsb.c"
    $323 = $d__2; //@line 4668 "lbfgsb.c"
    $324 = -$323; //@line 4668 "lbfgsb.c"
    $325 = $322 ? $323 : $324; //@line 4668 "lbfgsb.c"
    $326 = $316 > $325; //@line 4668 "lbfgsb.c"
    if ($326) {
     $327 = +HEAPF64[720>>3]; //@line 4670 "lbfgsb.c"
     HEAPF64[728>>3] = $327; //@line 4670 "lbfgsb.c"
    } else {
     $328 = +HEAPF64[736>>3]; //@line 4672 "lbfgsb.c"
     HEAPF64[728>>3] = $328; //@line 4672 "lbfgsb.c"
    }
    $329 = $9; //@line 4674 "lbfgsb.c"
    HEAP32[$329>>2] = 1; //@line 4674 "lbfgsb.c"
    break;
   }
   $330 = $8; //@line 4677 "lbfgsb.c"
   $331 = +HEAPF64[$330>>3]; //@line 4677 "lbfgsb.c"
   $332 = $331 >= 0.0; //@line 4677 "lbfgsb.c"
   $333 = $8; //@line 4677 "lbfgsb.c"
   $334 = +HEAPF64[$333>>3]; //@line 4677 "lbfgsb.c"
   $335 = -$334; //@line 4677 "lbfgsb.c"
   $336 = $332 ? $334 : $335; //@line 4677 "lbfgsb.c"
   $337 = $2; //@line 4677 "lbfgsb.c"
   $338 = +HEAPF64[$337>>3]; //@line 4677 "lbfgsb.c"
   $339 = $338 >= 0.0; //@line 4677 "lbfgsb.c"
   $340 = $2; //@line 4677 "lbfgsb.c"
   $341 = +HEAPF64[$340>>3]; //@line 4677 "lbfgsb.c"
   $342 = -$341; //@line 4677 "lbfgsb.c"
   $343 = $339 ? $341 : $342; //@line 4677 "lbfgsb.c"
   $344 = $336 < $343; //@line 4677 "lbfgsb.c"
   if (!($344)) {
    $549 = $9; //@line 4745 "lbfgsb.c"
    $550 = HEAP32[$549>>2]|0; //@line 4745 "lbfgsb.c"
    $551 = ($550|0)!=(0); //@line 4745 "lbfgsb.c"
    if ($551) {
     $552 = $7; //@line 4746 "lbfgsb.c"
     $553 = +HEAPF64[$552>>3]; //@line 4746 "lbfgsb.c"
     $554 = $4; //@line 4746 "lbfgsb.c"
     $555 = +HEAPF64[$554>>3]; //@line 4746 "lbfgsb.c"
     $556 = $553 - $555; //@line 4746 "lbfgsb.c"
     $557 = $556 * 3.0; //@line 4746 "lbfgsb.c"
     $558 = $3; //@line 4746 "lbfgsb.c"
     $559 = +HEAPF64[$558>>3]; //@line 4746 "lbfgsb.c"
     $560 = $6; //@line 4746 "lbfgsb.c"
     $561 = +HEAPF64[$560>>3]; //@line 4746 "lbfgsb.c"
     $562 = $559 - $561; //@line 4746 "lbfgsb.c"
     $563 = $557 / $562; //@line 4746 "lbfgsb.c"
     $564 = $5; //@line 4746 "lbfgsb.c"
     $565 = +HEAPF64[$564>>3]; //@line 4746 "lbfgsb.c"
     $566 = $563 + $565; //@line 4746 "lbfgsb.c"
     $567 = $8; //@line 4746 "lbfgsb.c"
     $568 = +HEAPF64[$567>>3]; //@line 4746 "lbfgsb.c"
     $569 = $566 + $568; //@line 4746 "lbfgsb.c"
     HEAPF64[752>>3] = $569; //@line 4746 "lbfgsb.c"
     $570 = +HEAPF64[752>>3]; //@line 4748 "lbfgsb.c"
     $571 = $570 >= 0.0; //@line 4748 "lbfgsb.c"
     $572 = +HEAPF64[752>>3]; //@line 4748 "lbfgsb.c"
     $573 = -$572; //@line 4748 "lbfgsb.c"
     $574 = $571 ? $572 : $573; //@line 4748 "lbfgsb.c"
     $d__1 = $574; //@line 4748 "lbfgsb.c"
     $575 = $5; //@line 4748 "lbfgsb.c"
     $576 = +HEAPF64[$575>>3]; //@line 4748 "lbfgsb.c"
     $577 = $576 >= 0.0; //@line 4748 "lbfgsb.c"
     $578 = $5; //@line 4748 "lbfgsb.c"
     $579 = +HEAPF64[$578>>3]; //@line 4748 "lbfgsb.c"
     $580 = -$579; //@line 4748 "lbfgsb.c"
     $581 = $577 ? $579 : $580; //@line 4748 "lbfgsb.c"
     $d__2 = $581; //@line 4748 "lbfgsb.c"
     $582 = $d__1; //@line 4748 "lbfgsb.c"
     $583 = $d__2; //@line 4748 "lbfgsb.c"
     $584 = $582 >= $583; //@line 4748 "lbfgsb.c"
     $585 = $d__1; //@line 4748 "lbfgsb.c"
     $586 = $d__2; //@line 4748 "lbfgsb.c"
     $587 = $584 ? $585 : $586; //@line 4748 "lbfgsb.c"
     $d__1 = $587; //@line 4748 "lbfgsb.c"
     $588 = $8; //@line 4749 "lbfgsb.c"
     $589 = +HEAPF64[$588>>3]; //@line 4749 "lbfgsb.c"
     $590 = $589 >= 0.0; //@line 4749 "lbfgsb.c"
     $591 = $8; //@line 4749 "lbfgsb.c"
     $592 = +HEAPF64[$591>>3]; //@line 4749 "lbfgsb.c"
     $593 = -$592; //@line 4749 "lbfgsb.c"
     $594 = $590 ? $592 : $593; //@line 4749 "lbfgsb.c"
     $d__2 = $594; //@line 4748 "lbfgsb.c"
     $595 = $d__1; //@line 4750 "lbfgsb.c"
     $596 = $d__2; //@line 4750 "lbfgsb.c"
     $597 = $595 >= $596; //@line 4750 "lbfgsb.c"
     $598 = $d__1; //@line 4750 "lbfgsb.c"
     $599 = $d__2; //@line 4750 "lbfgsb.c"
     $600 = $597 ? $598 : $599; //@line 4750 "lbfgsb.c"
     HEAPF64[704>>3] = $600; //@line 4750 "lbfgsb.c"
     $601 = +HEAPF64[752>>3]; //@line 4752 "lbfgsb.c"
     $602 = +HEAPF64[704>>3]; //@line 4752 "lbfgsb.c"
     $603 = $601 / $602; //@line 4752 "lbfgsb.c"
     $d__1 = $603; //@line 4752 "lbfgsb.c"
     $604 = +HEAPF64[704>>3]; //@line 4753 "lbfgsb.c"
     $605 = $d__1; //@line 4753 "lbfgsb.c"
     $606 = $d__1; //@line 4753 "lbfgsb.c"
     $607 = $605 * $606; //@line 4753 "lbfgsb.c"
     $608 = $5; //@line 4753 "lbfgsb.c"
     $609 = +HEAPF64[$608>>3]; //@line 4753 "lbfgsb.c"
     $610 = +HEAPF64[704>>3]; //@line 4753 "lbfgsb.c"
     $611 = $609 / $610; //@line 4753 "lbfgsb.c"
     $612 = $8; //@line 4753 "lbfgsb.c"
     $613 = +HEAPF64[$612>>3]; //@line 4753 "lbfgsb.c"
     $614 = +HEAPF64[704>>3]; //@line 4753 "lbfgsb.c"
     $615 = $613 / $614; //@line 4753 "lbfgsb.c"
     $616 = $611 * $615; //@line 4753 "lbfgsb.c"
     $617 = $607 - $616; //@line 4753 "lbfgsb.c"
     $618 = (+Math_sqrt((+$617))); //@line 4753 "lbfgsb.c"
     $619 = $604 * $618; //@line 4753 "lbfgsb.c"
     HEAPF64[744>>3] = $619; //@line 4753 "lbfgsb.c"
     $620 = $6; //@line 4754 "lbfgsb.c"
     $621 = +HEAPF64[$620>>3]; //@line 4754 "lbfgsb.c"
     $622 = $3; //@line 4754 "lbfgsb.c"
     $623 = +HEAPF64[$622>>3]; //@line 4754 "lbfgsb.c"
     $624 = $621 > $623; //@line 4754 "lbfgsb.c"
     if ($624) {
      $625 = +HEAPF64[744>>3]; //@line 4755 "lbfgsb.c"
      $626 = -$625; //@line 4755 "lbfgsb.c"
      HEAPF64[744>>3] = $626; //@line 4755 "lbfgsb.c"
     }
     $627 = +HEAPF64[744>>3]; //@line 4757 "lbfgsb.c"
     $628 = $8; //@line 4757 "lbfgsb.c"
     $629 = +HEAPF64[$628>>3]; //@line 4757 "lbfgsb.c"
     $630 = $627 - $629; //@line 4757 "lbfgsb.c"
     $631 = +HEAPF64[752>>3]; //@line 4757 "lbfgsb.c"
     $632 = $630 + $631; //@line 4757 "lbfgsb.c"
     HEAPF64[680>>3] = $632; //@line 4757 "lbfgsb.c"
     $633 = +HEAPF64[744>>3]; //@line 4758 "lbfgsb.c"
     $634 = $8; //@line 4758 "lbfgsb.c"
     $635 = +HEAPF64[$634>>3]; //@line 4758 "lbfgsb.c"
     $636 = $633 - $635; //@line 4758 "lbfgsb.c"
     $637 = +HEAPF64[744>>3]; //@line 4758 "lbfgsb.c"
     $638 = $636 + $637; //@line 4758 "lbfgsb.c"
     $639 = $5; //@line 4758 "lbfgsb.c"
     $640 = +HEAPF64[$639>>3]; //@line 4758 "lbfgsb.c"
     $641 = $638 + $640; //@line 4758 "lbfgsb.c"
     HEAPF64[688>>3] = $641; //@line 4758 "lbfgsb.c"
     $642 = +HEAPF64[680>>3]; //@line 4759 "lbfgsb.c"
     $643 = +HEAPF64[688>>3]; //@line 4759 "lbfgsb.c"
     $644 = $642 / $643; //@line 4759 "lbfgsb.c"
     HEAPF64[696>>3] = $644; //@line 4759 "lbfgsb.c"
     $645 = $6; //@line 4760 "lbfgsb.c"
     $646 = +HEAPF64[$645>>3]; //@line 4760 "lbfgsb.c"
     $647 = +HEAPF64[696>>3]; //@line 4760 "lbfgsb.c"
     $648 = $3; //@line 4760 "lbfgsb.c"
     $649 = +HEAPF64[$648>>3]; //@line 4760 "lbfgsb.c"
     $650 = $6; //@line 4760 "lbfgsb.c"
     $651 = +HEAPF64[$650>>3]; //@line 4760 "lbfgsb.c"
     $652 = $649 - $651; //@line 4760 "lbfgsb.c"
     $653 = $647 * $652; //@line 4760 "lbfgsb.c"
     $654 = $646 + $653; //@line 4760 "lbfgsb.c"
     HEAPF64[720>>3] = $654; //@line 4760 "lbfgsb.c"
     $655 = +HEAPF64[720>>3]; //@line 4761 "lbfgsb.c"
     HEAPF64[728>>3] = $655; //@line 4761 "lbfgsb.c"
     break;
    } else {
     $656 = $6; //@line 4762 "lbfgsb.c"
     $657 = +HEAPF64[$656>>3]; //@line 4762 "lbfgsb.c"
     $658 = $0; //@line 4762 "lbfgsb.c"
     $659 = +HEAPF64[$658>>3]; //@line 4762 "lbfgsb.c"
     $660 = $657 > $659; //@line 4762 "lbfgsb.c"
     if ($660) {
      $661 = $11; //@line 4763 "lbfgsb.c"
      $662 = +HEAPF64[$661>>3]; //@line 4763 "lbfgsb.c"
      HEAPF64[728>>3] = $662; //@line 4763 "lbfgsb.c"
      break;
     } else {
      $663 = $10; //@line 4765 "lbfgsb.c"
      $664 = +HEAPF64[$663>>3]; //@line 4765 "lbfgsb.c"
      HEAPF64[728>>3] = $664; //@line 4765 "lbfgsb.c"
      break;
     }
    }
   }
   $345 = $1; //@line 4682 "lbfgsb.c"
   $346 = +HEAPF64[$345>>3]; //@line 4682 "lbfgsb.c"
   $347 = $7; //@line 4682 "lbfgsb.c"
   $348 = +HEAPF64[$347>>3]; //@line 4682 "lbfgsb.c"
   $349 = $346 - $348; //@line 4682 "lbfgsb.c"
   $350 = $349 * 3.0; //@line 4682 "lbfgsb.c"
   $351 = $6; //@line 4682 "lbfgsb.c"
   $352 = +HEAPF64[$351>>3]; //@line 4682 "lbfgsb.c"
   $353 = $0; //@line 4682 "lbfgsb.c"
   $354 = +HEAPF64[$353>>3]; //@line 4682 "lbfgsb.c"
   $355 = $352 - $354; //@line 4682 "lbfgsb.c"
   $356 = $350 / $355; //@line 4682 "lbfgsb.c"
   $357 = $2; //@line 4682 "lbfgsb.c"
   $358 = +HEAPF64[$357>>3]; //@line 4682 "lbfgsb.c"
   $359 = $356 + $358; //@line 4682 "lbfgsb.c"
   $360 = $8; //@line 4682 "lbfgsb.c"
   $361 = +HEAPF64[$360>>3]; //@line 4682 "lbfgsb.c"
   $362 = $359 + $361; //@line 4682 "lbfgsb.c"
   HEAPF64[752>>3] = $362; //@line 4682 "lbfgsb.c"
   $363 = +HEAPF64[752>>3]; //@line 4684 "lbfgsb.c"
   $364 = $363 >= 0.0; //@line 4684 "lbfgsb.c"
   $365 = +HEAPF64[752>>3]; //@line 4684 "lbfgsb.c"
   $366 = -$365; //@line 4684 "lbfgsb.c"
   $367 = $364 ? $365 : $366; //@line 4684 "lbfgsb.c"
   $d__1 = $367; //@line 4684 "lbfgsb.c"
   $368 = $2; //@line 4684 "lbfgsb.c"
   $369 = +HEAPF64[$368>>3]; //@line 4684 "lbfgsb.c"
   $370 = $369 >= 0.0; //@line 4684 "lbfgsb.c"
   $371 = $2; //@line 4684 "lbfgsb.c"
   $372 = +HEAPF64[$371>>3]; //@line 4684 "lbfgsb.c"
   $373 = -$372; //@line 4684 "lbfgsb.c"
   $374 = $370 ? $372 : $373; //@line 4684 "lbfgsb.c"
   $d__2 = $374; //@line 4684 "lbfgsb.c"
   $375 = $d__1; //@line 4684 "lbfgsb.c"
   $376 = $d__2; //@line 4684 "lbfgsb.c"
   $377 = $375 >= $376; //@line 4684 "lbfgsb.c"
   $378 = $d__1; //@line 4684 "lbfgsb.c"
   $379 = $d__2; //@line 4684 "lbfgsb.c"
   $380 = $377 ? $378 : $379; //@line 4684 "lbfgsb.c"
   $d__1 = $380; //@line 4684 "lbfgsb.c"
   $381 = $8; //@line 4684 "lbfgsb.c"
   $382 = +HEAPF64[$381>>3]; //@line 4684 "lbfgsb.c"
   $383 = $382 >= 0.0; //@line 4684 "lbfgsb.c"
   $384 = $8; //@line 4684 "lbfgsb.c"
   $385 = +HEAPF64[$384>>3]; //@line 4684 "lbfgsb.c"
   $386 = -$385; //@line 4684 "lbfgsb.c"
   $387 = $383 ? $385 : $386; //@line 4684 "lbfgsb.c"
   $d__2 = $387; //@line 4684 "lbfgsb.c"
   $388 = $d__1; //@line 4686 "lbfgsb.c"
   $389 = $d__2; //@line 4686 "lbfgsb.c"
   $390 = $388 >= $389; //@line 4686 "lbfgsb.c"
   $391 = $d__1; //@line 4686 "lbfgsb.c"
   $392 = $d__2; //@line 4686 "lbfgsb.c"
   $393 = $390 ? $391 : $392; //@line 4686 "lbfgsb.c"
   HEAPF64[704>>3] = $393; //@line 4686 "lbfgsb.c"
   $394 = +HEAPF64[752>>3]; //@line 4691 "lbfgsb.c"
   $395 = +HEAPF64[704>>3]; //@line 4691 "lbfgsb.c"
   $396 = $394 / $395; //@line 4691 "lbfgsb.c"
   $d__3 = $396; //@line 4691 "lbfgsb.c"
   $d__1 = 0.0; //@line 4692 "lbfgsb.c"
   $397 = $d__3; //@line 4692 "lbfgsb.c"
   $398 = $d__3; //@line 4692 "lbfgsb.c"
   $399 = $397 * $398; //@line 4692 "lbfgsb.c"
   $400 = $2; //@line 4692 "lbfgsb.c"
   $401 = +HEAPF64[$400>>3]; //@line 4692 "lbfgsb.c"
   $402 = +HEAPF64[704>>3]; //@line 4692 "lbfgsb.c"
   $403 = $401 / $402; //@line 4692 "lbfgsb.c"
   $404 = $8; //@line 4692 "lbfgsb.c"
   $405 = +HEAPF64[$404>>3]; //@line 4692 "lbfgsb.c"
   $406 = +HEAPF64[704>>3]; //@line 4692 "lbfgsb.c"
   $407 = $405 / $406; //@line 4692 "lbfgsb.c"
   $408 = $403 * $407; //@line 4692 "lbfgsb.c"
   $409 = $399 - $408; //@line 4692 "lbfgsb.c"
   $d__2 = $409; //@line 4692 "lbfgsb.c"
   $410 = +HEAPF64[704>>3]; //@line 4693 "lbfgsb.c"
   $411 = $d__1; //@line 4693 "lbfgsb.c"
   $412 = $d__2; //@line 4693 "lbfgsb.c"
   $413 = $411 >= $412; //@line 4693 "lbfgsb.c"
   $414 = $d__1; //@line 4693 "lbfgsb.c"
   $415 = $d__2; //@line 4693 "lbfgsb.c"
   $416 = $413 ? $414 : $415; //@line 4693 "lbfgsb.c"
   $417 = (+Math_sqrt((+$416))); //@line 4693 "lbfgsb.c"
   $418 = $410 * $417; //@line 4693 "lbfgsb.c"
   HEAPF64[744>>3] = $418; //@line 4693 "lbfgsb.c"
   $419 = $6; //@line 4694 "lbfgsb.c"
   $420 = +HEAPF64[$419>>3]; //@line 4694 "lbfgsb.c"
   $421 = $0; //@line 4694 "lbfgsb.c"
   $422 = +HEAPF64[$421>>3]; //@line 4694 "lbfgsb.c"
   $423 = $420 > $422; //@line 4694 "lbfgsb.c"
   if ($423) {
    $424 = +HEAPF64[744>>3]; //@line 4695 "lbfgsb.c"
    $425 = -$424; //@line 4695 "lbfgsb.c"
    HEAPF64[744>>3] = $425; //@line 4695 "lbfgsb.c"
   }
   $426 = +HEAPF64[744>>3]; //@line 4697 "lbfgsb.c"
   $427 = $8; //@line 4697 "lbfgsb.c"
   $428 = +HEAPF64[$427>>3]; //@line 4697 "lbfgsb.c"
   $429 = $426 - $428; //@line 4697 "lbfgsb.c"
   $430 = +HEAPF64[752>>3]; //@line 4697 "lbfgsb.c"
   $431 = $429 + $430; //@line 4697 "lbfgsb.c"
   HEAPF64[680>>3] = $431; //@line 4697 "lbfgsb.c"
   $432 = +HEAPF64[744>>3]; //@line 4698 "lbfgsb.c"
   $433 = $2; //@line 4698 "lbfgsb.c"
   $434 = +HEAPF64[$433>>3]; //@line 4698 "lbfgsb.c"
   $435 = $8; //@line 4698 "lbfgsb.c"
   $436 = +HEAPF64[$435>>3]; //@line 4698 "lbfgsb.c"
   $437 = $434 - $436; //@line 4698 "lbfgsb.c"
   $438 = $432 + $437; //@line 4698 "lbfgsb.c"
   $439 = +HEAPF64[744>>3]; //@line 4698 "lbfgsb.c"
   $440 = $438 + $439; //@line 4698 "lbfgsb.c"
   HEAPF64[688>>3] = $440; //@line 4698 "lbfgsb.c"
   $441 = +HEAPF64[680>>3]; //@line 4699 "lbfgsb.c"
   $442 = +HEAPF64[688>>3]; //@line 4699 "lbfgsb.c"
   $443 = $441 / $442; //@line 4699 "lbfgsb.c"
   HEAPF64[696>>3] = $443; //@line 4699 "lbfgsb.c"
   $444 = +HEAPF64[696>>3]; //@line 4700 "lbfgsb.c"
   $445 = $444 < 0.0; //@line 4700 "lbfgsb.c"
   $446 = +HEAPF64[744>>3];
   $447 = $446 != 0.0; //@line 4700 "lbfgsb.c"
   $or$cond = $445 & $447; //@line 4700 "lbfgsb.c"
   $448 = $6; //@line 4701 "lbfgsb.c"
   $449 = +HEAPF64[$448>>3]; //@line 4701 "lbfgsb.c"
   do {
    if ($or$cond) {
     $450 = +HEAPF64[696>>3]; //@line 4701 "lbfgsb.c"
     $451 = $0; //@line 4701 "lbfgsb.c"
     $452 = +HEAPF64[$451>>3]; //@line 4701 "lbfgsb.c"
     $453 = $6; //@line 4701 "lbfgsb.c"
     $454 = +HEAPF64[$453>>3]; //@line 4701 "lbfgsb.c"
     $455 = $452 - $454; //@line 4701 "lbfgsb.c"
     $456 = $450 * $455; //@line 4701 "lbfgsb.c"
     $457 = $449 + $456; //@line 4701 "lbfgsb.c"
     HEAPF64[720>>3] = $457; //@line 4701 "lbfgsb.c"
    } else {
     $458 = $0; //@line 4702 "lbfgsb.c"
     $459 = +HEAPF64[$458>>3]; //@line 4702 "lbfgsb.c"
     $460 = $449 > $459; //@line 4702 "lbfgsb.c"
     if ($460) {
      $461 = $11; //@line 4703 "lbfgsb.c"
      $462 = +HEAPF64[$461>>3]; //@line 4703 "lbfgsb.c"
      HEAPF64[720>>3] = $462; //@line 4703 "lbfgsb.c"
      break;
     } else {
      $463 = $10; //@line 4705 "lbfgsb.c"
      $464 = +HEAPF64[$463>>3]; //@line 4705 "lbfgsb.c"
      HEAPF64[720>>3] = $464; //@line 4705 "lbfgsb.c"
      break;
     }
    }
   } while(0);
   $465 = $6; //@line 4707 "lbfgsb.c"
   $466 = +HEAPF64[$465>>3]; //@line 4707 "lbfgsb.c"
   $467 = $8; //@line 4707 "lbfgsb.c"
   $468 = +HEAPF64[$467>>3]; //@line 4707 "lbfgsb.c"
   $469 = $8; //@line 4707 "lbfgsb.c"
   $470 = +HEAPF64[$469>>3]; //@line 4707 "lbfgsb.c"
   $471 = $2; //@line 4707 "lbfgsb.c"
   $472 = +HEAPF64[$471>>3]; //@line 4707 "lbfgsb.c"
   $473 = $470 - $472; //@line 4707 "lbfgsb.c"
   $474 = $468 / $473; //@line 4707 "lbfgsb.c"
   $475 = $0; //@line 4707 "lbfgsb.c"
   $476 = +HEAPF64[$475>>3]; //@line 4707 "lbfgsb.c"
   $477 = $6; //@line 4707 "lbfgsb.c"
   $478 = +HEAPF64[$477>>3]; //@line 4707 "lbfgsb.c"
   $479 = $476 - $478; //@line 4707 "lbfgsb.c"
   $480 = $474 * $479; //@line 4707 "lbfgsb.c"
   $481 = $466 + $480; //@line 4707 "lbfgsb.c"
   HEAPF64[736>>3] = $481; //@line 4707 "lbfgsb.c"
   $482 = $9; //@line 4708 "lbfgsb.c"
   $483 = HEAP32[$482>>2]|0; //@line 4708 "lbfgsb.c"
   $484 = ($483|0)!=(0); //@line 4708 "lbfgsb.c"
   $485 = +HEAPF64[720>>3]; //@line 4712 "lbfgsb.c"
   $486 = $6; //@line 4712 "lbfgsb.c"
   $487 = +HEAPF64[$486>>3]; //@line 4712 "lbfgsb.c"
   $488 = $485 - $487; //@line 4712 "lbfgsb.c"
   $d__1 = $488; //@line 4712 "lbfgsb.c"
   $489 = $d__1; //@line 4712 "lbfgsb.c"
   $490 = $489 >= 0.0; //@line 4712 "lbfgsb.c"
   $491 = $d__1; //@line 4712 "lbfgsb.c"
   $492 = -$491; //@line 4712 "lbfgsb.c"
   $493 = $490 ? $491 : $492; //@line 4712 "lbfgsb.c"
   $494 = +HEAPF64[736>>3]; //@line 4712 "lbfgsb.c"
   $495 = $6; //@line 4712 "lbfgsb.c"
   $496 = +HEAPF64[$495>>3]; //@line 4712 "lbfgsb.c"
   $497 = $494 - $496; //@line 4712 "lbfgsb.c"
   $d__2 = $497; //@line 4712 "lbfgsb.c"
   $498 = $d__2; //@line 4712 "lbfgsb.c"
   $499 = $498 >= 0.0; //@line 4712 "lbfgsb.c"
   $500 = $d__2; //@line 4712 "lbfgsb.c"
   $501 = -$500; //@line 4712 "lbfgsb.c"
   $502 = $499 ? $500 : $501; //@line 4712 "lbfgsb.c"
   if ($484) {
    $503 = $493 < $502; //@line 4712 "lbfgsb.c"
    if ($503) {
     $504 = +HEAPF64[720>>3]; //@line 4714 "lbfgsb.c"
     HEAPF64[728>>3] = $504; //@line 4714 "lbfgsb.c"
    } else {
     $505 = +HEAPF64[736>>3]; //@line 4716 "lbfgsb.c"
     HEAPF64[728>>3] = $505; //@line 4716 "lbfgsb.c"
    }
    $506 = $6; //@line 4718 "lbfgsb.c"
    $507 = +HEAPF64[$506>>3]; //@line 4718 "lbfgsb.c"
    $508 = $0; //@line 4718 "lbfgsb.c"
    $509 = +HEAPF64[$508>>3]; //@line 4718 "lbfgsb.c"
    $510 = $507 > $509; //@line 4718 "lbfgsb.c"
    $511 = $6; //@line 4720 "lbfgsb.c"
    $512 = +HEAPF64[$511>>3]; //@line 4720 "lbfgsb.c"
    $513 = $3; //@line 4720 "lbfgsb.c"
    $514 = +HEAPF64[$513>>3]; //@line 4720 "lbfgsb.c"
    $515 = $6; //@line 4720 "lbfgsb.c"
    $516 = +HEAPF64[$515>>3]; //@line 4720 "lbfgsb.c"
    $517 = $514 - $516; //@line 4720 "lbfgsb.c"
    $518 = $517 * 0.66000000000000003; //@line 4720 "lbfgsb.c"
    $519 = $512 + $518; //@line 4720 "lbfgsb.c"
    $d__1 = $519; //@line 4720 "lbfgsb.c"
    $520 = $d__1; //@line 4721 "lbfgsb.c"
    $521 = +HEAPF64[728>>3]; //@line 4721 "lbfgsb.c"
    if ($510) {
     $522 = $520 <= $521; //@line 4721 "lbfgsb.c"
     $523 = $d__1; //@line 4721 "lbfgsb.c"
     $524 = +HEAPF64[728>>3]; //@line 4721 "lbfgsb.c"
     $525 = $522 ? $523 : $524; //@line 4721 "lbfgsb.c"
     HEAPF64[728>>3] = $525; //@line 4721 "lbfgsb.c"
     break;
    } else {
     $526 = $520 >= $521; //@line 4725 "lbfgsb.c"
     $527 = $d__1; //@line 4725 "lbfgsb.c"
     $528 = +HEAPF64[728>>3]; //@line 4725 "lbfgsb.c"
     $529 = $526 ? $527 : $528; //@line 4725 "lbfgsb.c"
     HEAPF64[728>>3] = $529; //@line 4725 "lbfgsb.c"
     break;
    }
   }
   $530 = $493 > $502; //@line 4731 "lbfgsb.c"
   if ($530) {
    $531 = +HEAPF64[720>>3]; //@line 4733 "lbfgsb.c"
    HEAPF64[728>>3] = $531; //@line 4733 "lbfgsb.c"
   } else {
    $532 = +HEAPF64[736>>3]; //@line 4735 "lbfgsb.c"
    HEAPF64[728>>3] = $532; //@line 4735 "lbfgsb.c"
   }
   $533 = $11; //@line 4737 "lbfgsb.c"
   $534 = +HEAPF64[$533>>3]; //@line 4737 "lbfgsb.c"
   $535 = +HEAPF64[728>>3]; //@line 4737 "lbfgsb.c"
   $536 = $534 <= $535; //@line 4737 "lbfgsb.c"
   if ($536) {
    $537 = $11; //@line 4737 "lbfgsb.c"
    $538 = +HEAPF64[$537>>3]; //@line 4737 "lbfgsb.c"
    $540 = $538;
   } else {
    $539 = +HEAPF64[728>>3]; //@line 4737 "lbfgsb.c"
    $540 = $539;
   }
   HEAPF64[728>>3] = $540; //@line 4737 "lbfgsb.c"
   $541 = $10; //@line 4738 "lbfgsb.c"
   $542 = +HEAPF64[$541>>3]; //@line 4738 "lbfgsb.c"
   $543 = +HEAPF64[728>>3]; //@line 4738 "lbfgsb.c"
   $544 = $542 >= $543; //@line 4738 "lbfgsb.c"
   if ($544) {
    $545 = $10; //@line 4738 "lbfgsb.c"
    $546 = +HEAPF64[$545>>3]; //@line 4738 "lbfgsb.c"
    $548 = $546;
   } else {
    $547 = +HEAPF64[728>>3]; //@line 4738 "lbfgsb.c"
    $548 = $547;
   }
   HEAPF64[728>>3] = $548; //@line 4738 "lbfgsb.c"
  }
 } while(0);
 $665 = $7; //@line 4769 "lbfgsb.c"
 $666 = +HEAPF64[$665>>3]; //@line 4769 "lbfgsb.c"
 $667 = $1; //@line 4769 "lbfgsb.c"
 $668 = +HEAPF64[$667>>3]; //@line 4769 "lbfgsb.c"
 $669 = $666 > $668; //@line 4769 "lbfgsb.c"
 if ($669) {
  $670 = $6; //@line 4770 "lbfgsb.c"
  $671 = +HEAPF64[$670>>3]; //@line 4770 "lbfgsb.c"
  $672 = $3; //@line 4770 "lbfgsb.c"
  HEAPF64[$672>>3] = $671; //@line 4770 "lbfgsb.c"
  $673 = $7; //@line 4771 "lbfgsb.c"
  $674 = +HEAPF64[$673>>3]; //@line 4771 "lbfgsb.c"
  $675 = $4; //@line 4771 "lbfgsb.c"
  HEAPF64[$675>>3] = $674; //@line 4771 "lbfgsb.c"
  $676 = $8; //@line 4772 "lbfgsb.c"
  $677 = +HEAPF64[$676>>3]; //@line 4772 "lbfgsb.c"
  $678 = $5; //@line 4772 "lbfgsb.c"
  HEAPF64[$678>>3] = $677; //@line 4772 "lbfgsb.c"
  $699 = +HEAPF64[728>>3]; //@line 4784 "lbfgsb.c"
  $700 = $6; //@line 4784 "lbfgsb.c"
  HEAPF64[$700>>3] = $699; //@line 4784 "lbfgsb.c"
  STACKTOP = sp;return 0; //@line 4785 "lbfgsb.c"
 }
 $679 = +HEAPF64[712>>3]; //@line 4774 "lbfgsb.c"
 $680 = $679 < 0.0; //@line 4774 "lbfgsb.c"
 if ($680) {
  $681 = $0; //@line 4775 "lbfgsb.c"
  $682 = +HEAPF64[$681>>3]; //@line 4775 "lbfgsb.c"
  $683 = $3; //@line 4775 "lbfgsb.c"
  HEAPF64[$683>>3] = $682; //@line 4775 "lbfgsb.c"
  $684 = $1; //@line 4776 "lbfgsb.c"
  $685 = +HEAPF64[$684>>3]; //@line 4776 "lbfgsb.c"
  $686 = $4; //@line 4776 "lbfgsb.c"
  HEAPF64[$686>>3] = $685; //@line 4776 "lbfgsb.c"
  $687 = $2; //@line 4777 "lbfgsb.c"
  $688 = +HEAPF64[$687>>3]; //@line 4777 "lbfgsb.c"
  $689 = $5; //@line 4777 "lbfgsb.c"
  HEAPF64[$689>>3] = $688; //@line 4777 "lbfgsb.c"
 }
 $690 = $6; //@line 4779 "lbfgsb.c"
 $691 = +HEAPF64[$690>>3]; //@line 4779 "lbfgsb.c"
 $692 = $0; //@line 4779 "lbfgsb.c"
 HEAPF64[$692>>3] = $691; //@line 4779 "lbfgsb.c"
 $693 = $7; //@line 4780 "lbfgsb.c"
 $694 = +HEAPF64[$693>>3]; //@line 4780 "lbfgsb.c"
 $695 = $1; //@line 4780 "lbfgsb.c"
 HEAPF64[$695>>3] = $694; //@line 4780 "lbfgsb.c"
 $696 = $8; //@line 4781 "lbfgsb.c"
 $697 = +HEAPF64[$696>>3]; //@line 4781 "lbfgsb.c"
 $698 = $2; //@line 4781 "lbfgsb.c"
 HEAPF64[$698>>3] = $697; //@line 4781 "lbfgsb.c"
 $699 = +HEAPF64[728>>3]; //@line 4784 "lbfgsb.c"
 $700 = $6; //@line 4784 "lbfgsb.c"
 HEAPF64[$700>>3] = $699; //@line 4784 "lbfgsb.c"
 STACKTOP = sp;return 0; //@line 4785 "lbfgsb.c"
}
function _dgesv_($n,$nrhs,$a,$lda,$ipiv,$b,$ldb,$info) {
 $n = $n|0;
 $nrhs = $nrhs|0;
 $a = $a|0;
 $lda = $lda|0;
 $ipiv = $ipiv|0;
 $b = $b|0;
 $ldb = $ldb|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $a_dim1 = 0, $a_offset = 0, $b_dim1 = 0, $b_offset = 0, $i__1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp;
 $1 = $n;
 $2 = $nrhs;
 $3 = $a;
 $4 = $lda;
 $5 = $ipiv;
 $6 = $b;
 $7 = $ldb;
 $8 = $info;
 $9 = $4;
 $10 = HEAP32[$9>>2]|0;
 $a_dim1 = $10;
 $11 = $a_dim1;
 $12 = (1 + ($11))|0;
 $a_offset = $12;
 $13 = $a_offset;
 $14 = $3;
 $15 = (0 - ($13))|0;
 $16 = (($14) + ($15<<3)|0);
 $3 = $16;
 $17 = $5;
 $18 = ((($17)) + -4|0);
 $5 = $18;
 $19 = $7;
 $20 = HEAP32[$19>>2]|0;
 $b_dim1 = $20;
 $21 = $b_dim1;
 $22 = (1 + ($21))|0;
 $b_offset = $22;
 $23 = $b_offset;
 $24 = $6;
 $25 = (0 - ($23))|0;
 $26 = (($24) + ($25<<3)|0);
 $6 = $26;
 $27 = $8;
 HEAP32[$27>>2] = 0;
 $28 = $1;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)<(0);
 do {
  if ($30) {
   $31 = $8;
   HEAP32[$31>>2] = -1;
  } else {
   $32 = $2;
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)<(0);
   if ($34) {
    $35 = $8;
    HEAP32[$35>>2] = -2;
    break;
   }
   $36 = $4;
   $37 = HEAP32[$36>>2]|0;
   $38 = $1;
   $39 = HEAP32[$38>>2]|0;
   $40 = (1)>=($39|0);
   if ($40) {
    $44 = 1;
   } else {
    $41 = $1;
    $42 = HEAP32[$41>>2]|0;
    $44 = $42;
   }
   $43 = ($37|0)<($44|0);
   if ($43) {
    $45 = $8;
    HEAP32[$45>>2] = -4;
    break;
   }
   $46 = $7;
   $47 = HEAP32[$46>>2]|0;
   $48 = $1;
   $49 = HEAP32[$48>>2]|0;
   $50 = (1)>=($49|0);
   if ($50) {
    $54 = 1;
   } else {
    $51 = $1;
    $52 = HEAP32[$51>>2]|0;
    $54 = $52;
   }
   $53 = ($47|0)<($54|0);
   if ($53) {
    $55 = $8;
    HEAP32[$55>>2] = -7;
   }
  }
 } while(0);
 $56 = $8;
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = $8;
  $60 = HEAP32[$59>>2]|0;
  $61 = (0 - ($60))|0;
  HEAP32[$i__1>>2] = $61;
  (_xerbla_(18053,$i__1)|0);
  $0 = 0;
  $87 = $0;
  STACKTOP = sp;return ($87|0);
 }
 $62 = $1;
 $63 = $1;
 $64 = $a_offset;
 $65 = $3;
 $66 = (($65) + ($64<<3)|0);
 $67 = $4;
 $68 = $5;
 $69 = ((($68)) + 4|0);
 $70 = $8;
 (_dgetrf_($62,$63,$66,$67,$69,$70)|0);
 $71 = $8;
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)==(0);
 if ($73) {
  $74 = $1;
  $75 = $2;
  $76 = $a_offset;
  $77 = $3;
  $78 = (($77) + ($76<<3)|0);
  $79 = $4;
  $80 = $5;
  $81 = ((($80)) + 4|0);
  $82 = $b_offset;
  $83 = $6;
  $84 = (($83) + ($82<<3)|0);
  $85 = $7;
  $86 = $8;
  (_dgetrs_(18087,$74,$75,$78,$79,$81,$84,$85,$86)|0);
 }
 $0 = 0;
 $87 = $0;
 STACKTOP = sp;return ($87|0);
}
function _dgetrf_($m,$n,$a,$lda,$ipiv,$info) {
 $m = $m|0;
 $n = $n|0;
 $a = $a|0;
 $lda = $lda|0;
 $ipiv = $ipiv|0;
 $info = $info|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0, $a_offset = 0, $i__ = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, $i__4 = 0, $i__5 = 0, $iinfo = 0, $j = 0, $jb = 0, $nb = 0, $or$cond = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 36|0;
 $i__3 = sp + 28|0;
 $i__4 = sp + 24|0;
 $j = sp + 12|0;
 $jb = sp + 8|0;
 $iinfo = sp;
 $1 = $m;
 $2 = $n;
 $3 = $a;
 $4 = $lda;
 $5 = $ipiv;
 $6 = $info;
 $7 = $4;
 $8 = HEAP32[$7>>2]|0;
 $a_dim1 = $8;
 $9 = $a_dim1;
 $10 = (1 + ($9))|0;
 $a_offset = $10;
 $11 = $a_offset;
 $12 = $3;
 $13 = (0 - ($11))|0;
 $14 = (($12) + ($13<<3)|0);
 $3 = $14;
 $15 = $5;
 $16 = ((($15)) + -4|0);
 $5 = $16;
 $17 = $6;
 HEAP32[$17>>2] = 0;
 $18 = $1;
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)<(0);
 do {
  if ($20) {
   $21 = $6;
   HEAP32[$21>>2] = -1;
  } else {
   $22 = $2;
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)<(0);
   if ($24) {
    $25 = $6;
    HEAP32[$25>>2] = -2;
    break;
   }
   $26 = $4;
   $27 = HEAP32[$26>>2]|0;
   $28 = $1;
   $29 = HEAP32[$28>>2]|0;
   $30 = (1)>=($29|0);
   if ($30) {
    $34 = 1;
   } else {
    $31 = $1;
    $32 = HEAP32[$31>>2]|0;
    $34 = $32;
   }
   $33 = ($27|0)<($34|0);
   if ($33) {
    $35 = $6;
    HEAP32[$35>>2] = -4;
   }
  }
 } while(0);
 $36 = $6;
 $37 = HEAP32[$36>>2]|0;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $6;
  $40 = HEAP32[$39>>2]|0;
  $41 = (0 - ($40))|0;
  HEAP32[$i__1>>2] = $41;
  (_xerbla_(18060,$i__1)|0);
  $0 = 0;
  $284 = $0;
  STACKTOP = sp;return ($284|0);
 }
 $42 = $1;
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if (!($44)) {
  $45 = $2;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)==(0);
  if (!($47)) {
   $48 = $1;
   $49 = $2;
   $50 = (_ilaenv_(2604,18060,18067,$48,$49,2608,2608)|0);
   $nb = $50;
   $51 = $nb;
   $52 = ($51|0)<=(1);
   L19: do {
    if ($52) {
     label = 19;
    } else {
     $53 = $nb;
     $54 = $1;
     $55 = HEAP32[$54>>2]|0;
     $56 = $2;
     $57 = HEAP32[$56>>2]|0;
     $58 = ($55|0)<=($57|0);
     if ($58) {
      $59 = $1;
      $60 = HEAP32[$59>>2]|0;
      $64 = $60;
     } else {
      $61 = $2;
      $62 = HEAP32[$61>>2]|0;
      $64 = $62;
     }
     $63 = ($53|0)>=($64|0);
     if ($63) {
      label = 19;
     } else {
      $74 = $1;
      $75 = HEAP32[$74>>2]|0;
      $76 = $2;
      $77 = HEAP32[$76>>2]|0;
      $78 = ($75|0)<=($77|0);
      if ($78) {
       $79 = $1;
       $80 = HEAP32[$79>>2]|0;
       $83 = $80;
      } else {
       $81 = $2;
       $82 = HEAP32[$81>>2]|0;
       $83 = $82;
      }
      HEAP32[$i__1>>2] = $83;
      $84 = $nb;
      $i__2 = $84;
      HEAP32[$j>>2] = 1;
      while(1) {
       $85 = $i__2;
       $86 = ($85|0)<(0);
       $87 = HEAP32[$j>>2]|0;
       $88 = HEAP32[$i__1>>2]|0;
       $89 = ($87|0)>=($88|0);
       $90 = ($87|0)<=($88|0);
       $$sink = $86 ? $89 : $90;
       $91 = $$sink&1;
       $92 = ($91|0)!=(0);
       if (!($92)) {
        break L19;
       }
       $93 = $1;
       $94 = HEAP32[$93>>2]|0;
       $95 = $2;
       $96 = HEAP32[$95>>2]|0;
       $97 = ($94|0)<=($96|0);
       if ($97) {
        $98 = $1;
        $99 = HEAP32[$98>>2]|0;
        $104 = $99;
       } else {
        $100 = $2;
        $101 = HEAP32[$100>>2]|0;
        $104 = $101;
       }
       $102 = HEAP32[$j>>2]|0;
       $103 = (($104) - ($102))|0;
       $105 = (($103) + 1)|0;
       HEAP32[$i__3>>2] = $105;
       $106 = HEAP32[$i__3>>2]|0;
       $107 = $nb;
       $108 = ($106|0)<=($107|0);
       $109 = HEAP32[$i__3>>2]|0;
       $110 = $nb;
       $111 = $108 ? $109 : $110;
       HEAP32[$jb>>2] = $111;
       $112 = $1;
       $113 = HEAP32[$112>>2]|0;
       $114 = HEAP32[$j>>2]|0;
       $115 = (($113) - ($114))|0;
       $116 = (($115) + 1)|0;
       HEAP32[$i__3>>2] = $116;
       $117 = HEAP32[$j>>2]|0;
       $118 = HEAP32[$j>>2]|0;
       $119 = $a_dim1;
       $120 = Math_imul($118, $119)|0;
       $121 = (($117) + ($120))|0;
       $122 = $3;
       $123 = (($122) + ($121<<3)|0);
       $124 = $4;
       $125 = HEAP32[$j>>2]|0;
       $126 = $5;
       $127 = (($126) + ($125<<2)|0);
       (_dgetf2_($i__3,$jb,$123,$124,$127,$iinfo)|0);
       $128 = $6;
       $129 = HEAP32[$128>>2]|0;
       $130 = ($129|0)==(0);
       $131 = HEAP32[$iinfo>>2]|0;
       $132 = ($131|0)>(0);
       $or$cond = $130 & $132;
       if ($or$cond) {
        $133 = HEAP32[$iinfo>>2]|0;
        $134 = HEAP32[$j>>2]|0;
        $135 = (($133) + ($134))|0;
        $136 = (($135) - 1)|0;
        $137 = $6;
        HEAP32[$137>>2] = $136;
       }
       $138 = $1;
       $139 = HEAP32[$138>>2]|0;
       HEAP32[$i__4>>2] = $139;
       $140 = HEAP32[$j>>2]|0;
       $141 = HEAP32[$jb>>2]|0;
       $142 = (($140) + ($141))|0;
       $143 = (($142) - 1)|0;
       $i__5 = $143;
       $144 = HEAP32[$i__4>>2]|0;
       $145 = $i__5;
       $146 = ($144|0)<=($145|0);
       $147 = HEAP32[$i__4>>2]|0;
       $148 = $i__5;
       $149 = $146 ? $147 : $148;
       HEAP32[$i__3>>2] = $149;
       $150 = HEAP32[$j>>2]|0;
       $i__ = $150;
       while(1) {
        $151 = $i__;
        $152 = HEAP32[$i__3>>2]|0;
        $153 = ($151|0)<=($152|0);
        $154 = HEAP32[$j>>2]|0;
        $155 = (($154) - 1)|0;
        if (!($153)) {
         break;
        }
        $156 = $i__;
        $157 = $5;
        $158 = (($157) + ($156<<2)|0);
        $159 = HEAP32[$158>>2]|0;
        $160 = (($155) + ($159))|0;
        $161 = $i__;
        $162 = $5;
        $163 = (($162) + ($161<<2)|0);
        HEAP32[$163>>2] = $160;
        $164 = $i__;
        $165 = (($164) + 1)|0;
        $i__ = $165;
       }
       HEAP32[$i__3>>2] = $155;
       $166 = HEAP32[$j>>2]|0;
       $167 = HEAP32[$jb>>2]|0;
       $168 = (($166) + ($167))|0;
       $169 = (($168) - 1)|0;
       HEAP32[$i__4>>2] = $169;
       $170 = $a_offset;
       $171 = $3;
       $172 = (($171) + ($170<<3)|0);
       $173 = $4;
       $174 = $5;
       $175 = ((($174)) + 4|0);
       (_dlaswp_($i__3,$172,$173,$j,$i__4,$175,2604)|0);
       $176 = HEAP32[$j>>2]|0;
       $177 = HEAP32[$jb>>2]|0;
       $178 = (($176) + ($177))|0;
       $179 = $2;
       $180 = HEAP32[$179>>2]|0;
       $181 = ($178|0)<=($180|0);
       if ($181) {
        $182 = $2;
        $183 = HEAP32[$182>>2]|0;
        $184 = HEAP32[$j>>2]|0;
        $185 = (($183) - ($184))|0;
        $186 = HEAP32[$jb>>2]|0;
        $187 = (($185) - ($186))|0;
        $188 = (($187) + 1)|0;
        HEAP32[$i__3>>2] = $188;
        $189 = HEAP32[$j>>2]|0;
        $190 = HEAP32[$jb>>2]|0;
        $191 = (($189) + ($190))|0;
        $192 = (($191) - 1)|0;
        HEAP32[$i__4>>2] = $192;
        $193 = HEAP32[$j>>2]|0;
        $194 = HEAP32[$jb>>2]|0;
        $195 = (($193) + ($194))|0;
        $196 = $a_dim1;
        $197 = Math_imul($195, $196)|0;
        $198 = (($197) + 1)|0;
        $199 = $3;
        $200 = (($199) + ($198<<3)|0);
        $201 = $4;
        $202 = $5;
        $203 = ((($202)) + 4|0);
        (_dlaswp_($i__3,$200,$201,$j,$i__4,$203,2604)|0);
        $204 = $2;
        $205 = HEAP32[$204>>2]|0;
        $206 = HEAP32[$j>>2]|0;
        $207 = (($205) - ($206))|0;
        $208 = HEAP32[$jb>>2]|0;
        $209 = (($207) - ($208))|0;
        $210 = (($209) + 1)|0;
        HEAP32[$i__3>>2] = $210;
        $211 = HEAP32[$j>>2]|0;
        $212 = HEAP32[$j>>2]|0;
        $213 = $a_dim1;
        $214 = Math_imul($212, $213)|0;
        $215 = (($211) + ($214))|0;
        $216 = $3;
        $217 = (($216) + ($215<<3)|0);
        $218 = $4;
        $219 = HEAP32[$j>>2]|0;
        $220 = HEAP32[$j>>2]|0;
        $221 = HEAP32[$jb>>2]|0;
        $222 = (($220) + ($221))|0;
        $223 = $a_dim1;
        $224 = Math_imul($222, $223)|0;
        $225 = (($219) + ($224))|0;
        $226 = $3;
        $227 = (($226) + ($225<<3)|0);
        $228 = $4;
        (_dtrsm_(18076,18081,18087,18100,$jb,$i__3,760,$217,$218,$227,$228)|0);
        $229 = HEAP32[$j>>2]|0;
        $230 = HEAP32[$jb>>2]|0;
        $231 = (($229) + ($230))|0;
        $232 = $1;
        $233 = HEAP32[$232>>2]|0;
        $234 = ($231|0)<=($233|0);
        if ($234) {
         $235 = $1;
         $236 = HEAP32[$235>>2]|0;
         $237 = HEAP32[$j>>2]|0;
         $238 = (($236) - ($237))|0;
         $239 = HEAP32[$jb>>2]|0;
         $240 = (($238) - ($239))|0;
         $241 = (($240) + 1)|0;
         HEAP32[$i__3>>2] = $241;
         $242 = $2;
         $243 = HEAP32[$242>>2]|0;
         $244 = HEAP32[$j>>2]|0;
         $245 = (($243) - ($244))|0;
         $246 = HEAP32[$jb>>2]|0;
         $247 = (($245) - ($246))|0;
         $248 = (($247) + 1)|0;
         HEAP32[$i__4>>2] = $248;
         $249 = HEAP32[$j>>2]|0;
         $250 = HEAP32[$jb>>2]|0;
         $251 = (($249) + ($250))|0;
         $252 = HEAP32[$j>>2]|0;
         $253 = $a_dim1;
         $254 = Math_imul($252, $253)|0;
         $255 = (($251) + ($254))|0;
         $256 = $3;
         $257 = (($256) + ($255<<3)|0);
         $258 = $4;
         $259 = HEAP32[$j>>2]|0;
         $260 = HEAP32[$j>>2]|0;
         $261 = HEAP32[$jb>>2]|0;
         $262 = (($260) + ($261))|0;
         $263 = $a_dim1;
         $264 = Math_imul($262, $263)|0;
         $265 = (($259) + ($264))|0;
         $266 = $3;
         $267 = (($266) + ($265<<3)|0);
         $268 = $4;
         $269 = HEAP32[$j>>2]|0;
         $270 = HEAP32[$jb>>2]|0;
         $271 = (($269) + ($270))|0;
         $272 = HEAP32[$j>>2]|0;
         $273 = HEAP32[$jb>>2]|0;
         $274 = (($272) + ($273))|0;
         $275 = $a_dim1;
         $276 = Math_imul($274, $275)|0;
         $277 = (($271) + ($276))|0;
         $278 = $3;
         $279 = (($278) + ($277<<3)|0);
         $280 = $4;
         (_dgemm_(18087,18087,$i__3,$i__4,$jb,768,$257,$258,$267,$268,760,$279,$280)|0);
        }
       }
       $281 = $i__2;
       $282 = HEAP32[$j>>2]|0;
       $283 = (($282) + ($281))|0;
       HEAP32[$j>>2] = $283;
      }
     }
    }
   } while(0);
   if ((label|0) == 19) {
    $65 = $1;
    $66 = $2;
    $67 = $a_offset;
    $68 = $3;
    $69 = (($68) + ($67<<3)|0);
    $70 = $4;
    $71 = $5;
    $72 = ((($71)) + 4|0);
    $73 = $6;
    (_dgetf2_($65,$66,$69,$70,$72,$73)|0);
   }
   $0 = 0;
   $284 = $0;
   STACKTOP = sp;return ($284|0);
  }
 }
 $0 = 0;
 $284 = $0;
 STACKTOP = sp;return ($284|0);
}
function _dgetrs_($trans,$n,$nrhs,$a,$lda,$ipiv,$b,$ldb,$info) {
 $trans = $trans|0;
 $n = $n|0;
 $nrhs = $nrhs|0;
 $a = $a|0;
 $lda = $lda|0;
 $ipiv = $ipiv|0;
 $b = $b|0;
 $ldb = $ldb|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0;
 var $a_offset = 0, $b_dim1 = 0, $b_offset = 0, $i__1 = 0, $notran = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 4|0;
 $1 = $trans;
 $2 = $n;
 $3 = $nrhs;
 $4 = $a;
 $5 = $lda;
 $6 = $ipiv;
 $7 = $b;
 $8 = $ldb;
 $9 = $info;
 $10 = $5;
 $11 = HEAP32[$10>>2]|0;
 $a_dim1 = $11;
 $12 = $a_dim1;
 $13 = (1 + ($12))|0;
 $a_offset = $13;
 $14 = $a_offset;
 $15 = $4;
 $16 = (0 - ($14))|0;
 $17 = (($15) + ($16<<3)|0);
 $4 = $17;
 $18 = $6;
 $19 = ((($18)) + -4|0);
 $6 = $19;
 $20 = $8;
 $21 = HEAP32[$20>>2]|0;
 $b_dim1 = $21;
 $22 = $b_dim1;
 $23 = (1 + ($22))|0;
 $b_offset = $23;
 $24 = $b_offset;
 $25 = $7;
 $26 = (0 - ($24))|0;
 $27 = (($25) + ($26<<3)|0);
 $7 = $27;
 $28 = $9;
 HEAP32[$28>>2] = 0;
 $29 = $1;
 $30 = (_lsame_($29,20460)|0);
 $notran = $30;
 $31 = $notran;
 $32 = ($31|0)!=(0);
 if ($32) {
  label = 5;
 } else {
  $33 = $1;
  $34 = (_lsame_($33,20466)|0);
  $35 = ($34|0)!=(0);
  if ($35) {
   label = 5;
  } else {
   $36 = $1;
   $37 = (_lsame_($36,20468)|0);
   $38 = ($37|0)!=(0);
   if ($38) {
    label = 5;
   } else {
    $39 = $9;
    HEAP32[$39>>2] = -1;
   }
  }
 }
 do {
  if ((label|0) == 5) {
   $40 = $2;
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)<(0);
   if ($42) {
    $43 = $9;
    HEAP32[$43>>2] = -2;
    break;
   }
   $44 = $3;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)<(0);
   if ($46) {
    $47 = $9;
    HEAP32[$47>>2] = -3;
    break;
   }
   $48 = $5;
   $49 = HEAP32[$48>>2]|0;
   $50 = $2;
   $51 = HEAP32[$50>>2]|0;
   $52 = (1)>=($51|0);
   if ($52) {
    $56 = 1;
   } else {
    $53 = $2;
    $54 = HEAP32[$53>>2]|0;
    $56 = $54;
   }
   $55 = ($49|0)<($56|0);
   if ($55) {
    $57 = $9;
    HEAP32[$57>>2] = -5;
    break;
   }
   $58 = $8;
   $59 = HEAP32[$58>>2]|0;
   $60 = $2;
   $61 = HEAP32[$60>>2]|0;
   $62 = (1)>=($61|0);
   if ($62) {
    $66 = 1;
   } else {
    $63 = $2;
    $64 = HEAP32[$63>>2]|0;
    $66 = $64;
   }
   $65 = ($59|0)<($66|0);
   if ($65) {
    $67 = $9;
    HEAP32[$67>>2] = -8;
   }
  }
 } while(0);
 $68 = $9;
 $69 = HEAP32[$68>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = $9;
  $72 = HEAP32[$71>>2]|0;
  $73 = (0 - ($72))|0;
  HEAP32[$i__1>>2] = $73;
  (_xerbla_(18069,$i__1)|0);
  $0 = 0;
  $138 = $0;
  STACKTOP = sp;return ($138|0);
 }
 $74 = $2;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)==(0);
 if (!($76)) {
  $77 = $3;
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)==(0);
  if (!($79)) {
   $80 = $notran;
   $81 = ($80|0)!=(0);
   if ($81) {
    $82 = $3;
    $83 = $b_offset;
    $84 = $7;
    $85 = (($84) + ($83<<3)|0);
    $86 = $8;
    $87 = $2;
    $88 = $6;
    $89 = ((($88)) + 4|0);
    (_dlaswp_($82,$85,$86,2612,$87,$89,2612)|0);
    $90 = $2;
    $91 = $3;
    $92 = $a_offset;
    $93 = $4;
    $94 = (($93) + ($92<<3)|0);
    $95 = $5;
    $96 = $b_offset;
    $97 = $7;
    $98 = (($97) + ($96<<3)|0);
    $99 = $8;
    (_dtrsm_(18076,18081,18087,18100,$90,$91,776,$94,$95,$98,$99)|0);
    $100 = $2;
    $101 = $3;
    $102 = $a_offset;
    $103 = $4;
    $104 = (($103) + ($102<<3)|0);
    $105 = $5;
    $106 = $b_offset;
    $107 = $7;
    $108 = (($107) + ($106<<3)|0);
    $109 = $8;
    (_dtrsm_(18076,18105,18087,18111,$100,$101,776,$104,$105,$108,$109)|0);
   } else {
    $110 = $2;
    $111 = $3;
    $112 = $a_offset;
    $113 = $4;
    $114 = (($113) + ($112<<3)|0);
    $115 = $5;
    $116 = $b_offset;
    $117 = $7;
    $118 = (($117) + ($116<<3)|0);
    $119 = $8;
    (_dtrsm_(18076,18105,18120,18111,$110,$111,776,$114,$115,$118,$119)|0);
    $120 = $2;
    $121 = $3;
    $122 = $a_offset;
    $123 = $4;
    $124 = (($123) + ($122<<3)|0);
    $125 = $5;
    $126 = $b_offset;
    $127 = $7;
    $128 = (($127) + ($126<<3)|0);
    $129 = $8;
    (_dtrsm_(18076,18081,18120,18100,$120,$121,776,$124,$125,$128,$129)|0);
    $130 = $3;
    $131 = $b_offset;
    $132 = $7;
    $133 = (($132) + ($131<<3)|0);
    $134 = $8;
    $135 = $2;
    $136 = $6;
    $137 = ((($136)) + 4|0);
    (_dlaswp_($130,$133,$134,2612,$135,$137,2616)|0);
   }
   $0 = 0;
   $138 = $0;
   STACKTOP = sp;return ($138|0);
  }
 }
 $0 = 0;
 $138 = $0;
 STACKTOP = sp;return ($138|0);
}
function _dlaswp_($n,$a,$lda,$k1,$k2,$ipiv,$incx) {
 $n = $n|0;
 $a = $a|0;
 $lda = $lda|0;
 $k1 = $k1|0;
 $k2 = $k2|0;
 $ipiv = $ipiv|0;
 $incx = $incx|0;
 var $$sink = 0, $$sink1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0.0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0.0, $a_dim1 = 0, $a_offset = 0, $i1 = 0, $i2 = 0, $i__ = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0;
 var $i__4 = 0, $inc = 0, $ip = 0, $ix = 0, $ix0 = 0, $j = 0, $k = 0, $n32 = 0, $temp = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $a;
 $3 = $lda;
 $4 = $k1;
 $5 = $k2;
 $6 = $ipiv;
 $7 = $incx;
 $8 = $3;
 $9 = HEAP32[$8>>2]|0;
 $a_dim1 = $9;
 $10 = $a_dim1;
 $11 = (1 + ($10))|0;
 $a_offset = $11;
 $12 = $a_offset;
 $13 = $2;
 $14 = (0 - ($12))|0;
 $15 = (($13) + ($14<<3)|0);
 $2 = $15;
 $16 = $6;
 $17 = ((($16)) + -4|0);
 $6 = $17;
 $18 = $7;
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)>(0);
 do {
  if ($20) {
   $21 = $4;
   $22 = HEAP32[$21>>2]|0;
   $ix0 = $22;
   $23 = $4;
   $24 = HEAP32[$23>>2]|0;
   $i1 = $24;
   $25 = $5;
   $26 = HEAP32[$25>>2]|0;
   $i2 = $26;
   $inc = 1;
  } else {
   $27 = $7;
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)<(0);
   if ($29) {
    $30 = $5;
    $31 = HEAP32[$30>>2]|0;
    $32 = (1 - ($31))|0;
    $33 = $7;
    $34 = HEAP32[$33>>2]|0;
    $35 = Math_imul($32, $34)|0;
    $36 = (($35) + 1)|0;
    $ix0 = $36;
    $37 = $5;
    $38 = HEAP32[$37>>2]|0;
    $i1 = $38;
    $39 = $4;
    $40 = HEAP32[$39>>2]|0;
    $i2 = $40;
    $inc = -1;
    break;
   }
   $0 = 0;
   $189 = $0;
   STACKTOP = sp;return ($189|0);
  }
 } while(0);
 $41 = $1;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42|0) / 32)&-1;
 $44 = $43 << 5;
 $n32 = $44;
 $45 = $n32;
 $46 = ($45|0)!=(0);
 L9: do {
  if ($46) {
   $47 = $n32;
   $i__1 = $47;
   $j = 1;
   while(1) {
    $48 = $j;
    $49 = $i__1;
    $50 = ($48|0)<=($49|0);
    if (!($50)) {
     break L9;
    }
    $51 = $ix0;
    $ix = $51;
    $52 = $i2;
    $i__2 = $52;
    $53 = $inc;
    $i__3 = $53;
    $54 = $i1;
    $i__ = $54;
    while(1) {
     $55 = $i__3;
     $56 = ($55|0)<(0);
     $57 = $i__;
     $58 = $i__2;
     $59 = ($57|0)>=($58|0);
     $60 = ($57|0)<=($58|0);
     $$sink = $56 ? $59 : $60;
     $61 = $$sink&1;
     $62 = ($61|0)!=(0);
     if (!($62)) {
      break;
     }
     $63 = $ix;
     $64 = $6;
     $65 = (($64) + ($63<<2)|0);
     $66 = HEAP32[$65>>2]|0;
     $ip = $66;
     $67 = $ip;
     $68 = $i__;
     $69 = ($67|0)!=($68|0);
     L17: do {
      if ($69) {
       $70 = $j;
       $71 = (($70) + 31)|0;
       $i__4 = $71;
       $72 = $j;
       $k = $72;
       while(1) {
        $73 = $k;
        $74 = $i__4;
        $75 = ($73|0)<=($74|0);
        if (!($75)) {
         break L17;
        }
        $76 = $i__;
        $77 = $k;
        $78 = $a_dim1;
        $79 = Math_imul($77, $78)|0;
        $80 = (($76) + ($79))|0;
        $81 = $2;
        $82 = (($81) + ($80<<3)|0);
        $83 = +HEAPF64[$82>>3];
        $temp = $83;
        $84 = $ip;
        $85 = $k;
        $86 = $a_dim1;
        $87 = Math_imul($85, $86)|0;
        $88 = (($84) + ($87))|0;
        $89 = $2;
        $90 = (($89) + ($88<<3)|0);
        $91 = +HEAPF64[$90>>3];
        $92 = $i__;
        $93 = $k;
        $94 = $a_dim1;
        $95 = Math_imul($93, $94)|0;
        $96 = (($92) + ($95))|0;
        $97 = $2;
        $98 = (($97) + ($96<<3)|0);
        HEAPF64[$98>>3] = $91;
        $99 = $temp;
        $100 = $ip;
        $101 = $k;
        $102 = $a_dim1;
        $103 = Math_imul($101, $102)|0;
        $104 = (($100) + ($103))|0;
        $105 = $2;
        $106 = (($105) + ($104<<3)|0);
        HEAPF64[$106>>3] = $99;
        $107 = $k;
        $108 = (($107) + 1)|0;
        $k = $108;
       }
      }
     } while(0);
     $109 = $7;
     $110 = HEAP32[$109>>2]|0;
     $111 = $ix;
     $112 = (($111) + ($110))|0;
     $ix = $112;
     $113 = $i__3;
     $114 = $i__;
     $115 = (($114) + ($113))|0;
     $i__ = $115;
    }
    $116 = $j;
    $117 = (($116) + 32)|0;
    $j = $117;
   }
  }
 } while(0);
 $118 = $n32;
 $119 = $1;
 $120 = HEAP32[$119>>2]|0;
 $121 = ($118|0)!=($120|0);
 L25: do {
  if ($121) {
   $122 = $n32;
   $123 = (($122) + 1)|0;
   $n32 = $123;
   $124 = $ix0;
   $ix = $124;
   $125 = $i2;
   $i__1 = $125;
   $126 = $inc;
   $i__3 = $126;
   $127 = $i1;
   $i__ = $127;
   while(1) {
    $128 = $i__3;
    $129 = ($128|0)<(0);
    $130 = $i__;
    $131 = $i__1;
    $132 = ($130|0)>=($131|0);
    $133 = ($130|0)<=($131|0);
    $$sink1 = $129 ? $132 : $133;
    $134 = $$sink1&1;
    $135 = ($134|0)!=(0);
    if (!($135)) {
     break L25;
    }
    $136 = $ix;
    $137 = $6;
    $138 = (($137) + ($136<<2)|0);
    $139 = HEAP32[$138>>2]|0;
    $ip = $139;
    $140 = $ip;
    $141 = $i__;
    $142 = ($140|0)!=($141|0);
    L30: do {
     if ($142) {
      $143 = $1;
      $144 = HEAP32[$143>>2]|0;
      $i__2 = $144;
      $145 = $n32;
      $k = $145;
      while(1) {
       $146 = $k;
       $147 = $i__2;
       $148 = ($146|0)<=($147|0);
       if (!($148)) {
        break L30;
       }
       $149 = $i__;
       $150 = $k;
       $151 = $a_dim1;
       $152 = Math_imul($150, $151)|0;
       $153 = (($149) + ($152))|0;
       $154 = $2;
       $155 = (($154) + ($153<<3)|0);
       $156 = +HEAPF64[$155>>3];
       $temp = $156;
       $157 = $ip;
       $158 = $k;
       $159 = $a_dim1;
       $160 = Math_imul($158, $159)|0;
       $161 = (($157) + ($160))|0;
       $162 = $2;
       $163 = (($162) + ($161<<3)|0);
       $164 = +HEAPF64[$163>>3];
       $165 = $i__;
       $166 = $k;
       $167 = $a_dim1;
       $168 = Math_imul($166, $167)|0;
       $169 = (($165) + ($168))|0;
       $170 = $2;
       $171 = (($170) + ($169<<3)|0);
       HEAPF64[$171>>3] = $164;
       $172 = $temp;
       $173 = $ip;
       $174 = $k;
       $175 = $a_dim1;
       $176 = Math_imul($174, $175)|0;
       $177 = (($173) + ($176))|0;
       $178 = $2;
       $179 = (($178) + ($177<<3)|0);
       HEAPF64[$179>>3] = $172;
       $180 = $k;
       $181 = (($180) + 1)|0;
       $k = $181;
      }
     }
    } while(0);
    $182 = $7;
    $183 = HEAP32[$182>>2]|0;
    $184 = $ix;
    $185 = (($184) + ($183))|0;
    $ix = $185;
    $186 = $i__3;
    $187 = $i__;
    $188 = (($187) + ($186))|0;
    $i__ = $188;
   }
  }
 } while(0);
 $0 = 0;
 $189 = $0;
 STACKTOP = sp;return ($189|0);
}
function _dlamch_($cmach) {
 $cmach = $cmach|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0.0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0, $44 = 0.0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0.0, $60 = 0.0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0.0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $8 = 0, $9 = 0, $beta = 0, $i__1 = 0, $imax = 0, $imin = 0, $it = 0;
 var $lrnd = 0, $ret_val = 0.0, $rmach = 0.0, $small = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 44|0;
 $it = sp + 40|0;
 $beta = sp + 36|0;
 $imin = sp + 32|0;
 $imax = sp + 28|0;
 $lrnd = sp + 24|0;
 $0 = $cmach;
 $1 = HEAP32[2620>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  (_dlamc2_($beta,$it,$lrnd,784,$imin,792,$imax,800)|0);
  $3 = HEAP32[$beta>>2]|0;
  $4 = (+($3|0));
  HEAPF64[808>>3] = $4;
  $5 = HEAP32[$it>>2]|0;
  $6 = (+($5|0));
  HEAPF64[816>>3] = $6;
  $7 = HEAP32[$lrnd>>2]|0;
  $8 = ($7|0)!=(0);
  if ($8) {
   HEAPF64[824>>3] = 1.0;
   $9 = HEAP32[$it>>2]|0;
   $10 = (1 - ($9))|0;
   HEAP32[$i__1>>2] = $10;
   $11 = (+_pow_di(808,$i__1));
   $12 = $11 / 2.0;
   HEAPF64[784>>3] = $12;
  } else {
   HEAPF64[824>>3] = 0.0;
   $13 = HEAP32[$it>>2]|0;
   $14 = (1 - ($13))|0;
   HEAP32[$i__1>>2] = $14;
   $15 = (+_pow_di(808,$i__1));
   HEAPF64[784>>3] = $15;
  }
  $16 = +HEAPF64[784>>3];
  $17 = +HEAPF64[808>>3];
  $18 = $16 * $17;
  HEAPF64[832>>3] = $18;
  $19 = HEAP32[$imin>>2]|0;
  $20 = (+($19|0));
  HEAPF64[840>>3] = $20;
  $21 = HEAP32[$imax>>2]|0;
  $22 = (+($21|0));
  HEAPF64[848>>3] = $22;
  $23 = +HEAPF64[792>>3];
  HEAPF64[856>>3] = $23;
  $24 = +HEAPF64[800>>3];
  $25 = 1.0 / $24;
  $small = $25;
  $26 = $small;
  $27 = +HEAPF64[856>>3];
  $28 = $26 >= $27;
  if ($28) {
   $29 = $small;
   $30 = +HEAPF64[784>>3];
   $31 = $30 + 1.0;
   $32 = $29 * $31;
   HEAPF64[856>>3] = $32;
  }
 }
 $33 = $0;
 $34 = (_lsame_($33,18130)|0);
 $35 = ($34|0)!=(0);
 do {
  if ($35) {
   $36 = +HEAPF64[784>>3];
   $rmach = $36;
  } else {
   $37 = $0;
   $38 = (_lsame_($37,18497)|0);
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = +HEAPF64[856>>3];
    $rmach = $40;
    break;
   }
   $41 = $0;
   $42 = (_lsame_($41,18132)|0);
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = +HEAPF64[808>>3];
    $rmach = $44;
    break;
   }
   $45 = $0;
   $46 = (_lsame_($45,18134)|0);
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = +HEAPF64[832>>3];
    $rmach = $48;
    break;
   }
   $49 = $0;
   $50 = (_lsame_($49,20460)|0);
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = +HEAPF64[816>>3];
    $rmach = $52;
    break;
   }
   $53 = $0;
   $54 = (_lsame_($53,20464)|0);
   $55 = ($54|0)!=(0);
   if ($55) {
    $56 = +HEAPF64[824>>3];
    $rmach = $56;
    break;
   }
   $57 = $0;
   $58 = (_lsame_($57,18136)|0);
   $59 = ($58|0)!=(0);
   if ($59) {
    $60 = +HEAPF64[840>>3];
    $rmach = $60;
    break;
   }
   $61 = $0;
   $62 = (_lsame_($61,20462)|0);
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = +HEAPF64[792>>3];
    $rmach = $64;
    break;
   }
   $65 = $0;
   $66 = (_lsame_($65,20458)|0);
   $67 = ($66|0)!=(0);
   if ($67) {
    $68 = +HEAPF64[848>>3];
    $rmach = $68;
    break;
   }
   $69 = $0;
   $70 = (_lsame_($69,18138)|0);
   $71 = ($70|0)!=(0);
   if ($71) {
    $72 = +HEAPF64[800>>3];
    $rmach = $72;
   }
  }
 } while(0);
 $73 = $rmach;
 $ret_val = $73;
 HEAP32[2620>>2] = 0;
 $74 = $ret_val;
 STACKTOP = sp;return (+$74);
}
function _dlamc2_($beta,$t,$rnd,$eps,$emin,$rmin,$emax,$rmax) {
 $beta = $beta|0;
 $t = $t|0;
 $rnd = $rnd|0;
 $eps = $eps|0;
 $emin = $emin|0;
 $rmin = $rmin|0;
 $emax = $emax|0;
 $rmax = $rmax|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0.0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0.0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0.0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0.0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0.0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0.0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0.0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0.0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0.0, $219 = 0, $22 = 0.0, $220 = 0, $221 = 0, $222 = 0.0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0.0, $227 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0.0, $37 = 0.0;
 var $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0.0, $55 = 0.0;
 var $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0.0, $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0;
 var $74 = 0.0, $75 = 0.0, $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0, $81 = 0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0;
 var $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c__ = 0, $d__1 = 0, $d__2 = 0, $d__3 = 0.0, $d__4 = 0.0, $d__5 = 0.0, $gnmin = 0, $gpmin = 0, $half = 0, $i__ = 0;
 var $i__1 = 0, $ieee = 0, $lieee1 = 0, $lrnd = 0, $ngnmin = 0, $ngpmin = 0, $one = 0, $rbase = 0.0, $sixth = 0, $small = 0, $third = 0, $two = 0.0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 160|0;
 $d__1 = sp + 120|0;
 $d__2 = sp + 112|0;
 $a = sp + 80|0;
 $b = sp + 72|0;
 $c__ = sp + 64|0;
 $one = sp + 56|0;
 $ieee = sp + 152|0;
 $half = sp + 40|0;
 $lrnd = sp + 148|0;
 $zero = sp + 32|0;
 $gnmin = sp + 144|0;
 $small = sp + 16|0;
 $gpmin = sp + 140|0;
 $third = sp + 8|0;
 $sixth = sp;
 $lieee1 = sp + 136|0;
 $ngnmin = sp + 132|0;
 $ngpmin = sp + 128|0;
 $0 = $beta;
 $1 = $t;
 $2 = $rnd;
 $3 = $eps;
 $4 = $emin;
 $5 = $rmin;
 $6 = $emax;
 $7 = $rmax;
 $8 = HEAP32[2624>>2]|0;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $212 = HEAP32[2628>>2]|0;
  $213 = $0;
  HEAP32[$213>>2] = $212;
  $214 = HEAP32[2632>>2]|0;
  $215 = $1;
  HEAP32[$215>>2] = $214;
  $216 = HEAP32[$lrnd>>2]|0;
  $217 = $2;
  HEAP32[$217>>2] = $216;
  $218 = +HEAPF64[864>>3];
  $219 = $3;
  HEAPF64[$219>>3] = $218;
  $220 = HEAP32[2636>>2]|0;
  $221 = $4;
  HEAP32[$221>>2] = $220;
  $222 = +HEAPF64[872>>3];
  $223 = $5;
  HEAPF64[$223>>3] = $222;
  $224 = HEAP32[2668>>2]|0;
  $225 = $6;
  HEAP32[$225>>2] = $224;
  $226 = +HEAPF64[880>>3];
  $227 = $7;
  HEAPF64[$227>>3] = $226;
  STACKTOP = sp;return 0;
 }
 HEAPF64[$zero>>3] = 0.0;
 HEAPF64[$one>>3] = 1.0;
 $two = 2.0;
 (_dlamc1_(2628,2632,$lrnd,$lieee1)|0);
 $10 = HEAP32[2628>>2]|0;
 $11 = (+($10|0));
 HEAPF64[$b>>3] = $11;
 $12 = HEAP32[2632>>2]|0;
 $13 = (0 - ($12))|0;
 HEAP32[$i__1>>2] = $13;
 $14 = (+_pow_di($b,$i__1));
 HEAPF64[$a>>3] = $14;
 $15 = +HEAPF64[$a>>3];
 HEAPF64[864>>3] = $15;
 $16 = $two;
 $17 = $16 / 3.0;
 HEAPF64[$b>>3] = $17;
 $18 = +HEAPF64[$one>>3];
 $19 = $18 / 2.0;
 HEAPF64[$half>>3] = $19;
 $20 = +HEAPF64[$half>>3];
 $21 = -$20;
 HEAPF64[$d__1>>3] = $21;
 $22 = (+_dlamc3_($b,$d__1));
 HEAPF64[$sixth>>3] = $22;
 $23 = (+_dlamc3_($sixth,$sixth));
 HEAPF64[$third>>3] = $23;
 $24 = +HEAPF64[$half>>3];
 $25 = -$24;
 HEAPF64[$d__1>>3] = $25;
 $26 = (+_dlamc3_($third,$d__1));
 HEAPF64[$b>>3] = $26;
 $27 = (+_dlamc3_($b,$sixth));
 HEAPF64[$b>>3] = $27;
 $28 = +HEAPF64[$b>>3];
 $29 = $28 >= 0.0;
 $30 = +HEAPF64[$b>>3];
 $31 = -$30;
 $32 = $29 ? $30 : $31;
 HEAPF64[$b>>3] = $32;
 $33 = +HEAPF64[$b>>3];
 $34 = +HEAPF64[864>>3];
 $35 = $33 < $34;
 if ($35) {
  $36 = +HEAPF64[864>>3];
  HEAPF64[$b>>3] = $36;
 }
 HEAPF64[864>>3] = 1.0;
 while(1) {
  $37 = +HEAPF64[864>>3];
  $38 = +HEAPF64[$b>>3];
  $39 = $37 > $38;
  if (!($39)) {
   break;
  }
  $40 = +HEAPF64[$b>>3];
  $41 = +HEAPF64[$zero>>3];
  $42 = $40 > $41;
  if (!($42)) {
   break;
  }
  $43 = +HEAPF64[$b>>3];
  HEAPF64[864>>3] = $43;
  $44 = +HEAPF64[$half>>3];
  $45 = +HEAPF64[864>>3];
  $46 = $44 * $45;
  HEAPF64[$d__1>>3] = $46;
  $47 = $two;
  $d__3 = $47;
  $48 = $d__3;
  $d__4 = $48;
  $49 = $d__3;
  $50 = $d__3;
  $51 = $50 * $49;
  $d__3 = $51;
  $52 = +HEAPF64[864>>3];
  $d__5 = $52;
  $53 = $d__4;
  $54 = $d__3;
  $55 = $d__3;
  $56 = $54 * $55;
  $57 = $53 * $56;
  $58 = $d__5;
  $59 = $d__5;
  $60 = $58 * $59;
  $61 = $57 * $60;
  HEAPF64[$d__2>>3] = $61;
  $62 = (+_dlamc3_($d__1,$d__2));
  HEAPF64[$c__>>3] = $62;
  $63 = +HEAPF64[$c__>>3];
  $64 = -$63;
  HEAPF64[$d__1>>3] = $64;
  $65 = (+_dlamc3_($half,$d__1));
  HEAPF64[$c__>>3] = $65;
  $66 = (+_dlamc3_($half,$c__));
  HEAPF64[$b>>3] = $66;
  $67 = +HEAPF64[$b>>3];
  $68 = -$67;
  HEAPF64[$d__1>>3] = $68;
  $69 = (+_dlamc3_($half,$d__1));
  HEAPF64[$c__>>3] = $69;
  $70 = (+_dlamc3_($half,$c__));
  HEAPF64[$b>>3] = $70;
 }
 $71 = +HEAPF64[$a>>3];
 $72 = +HEAPF64[864>>3];
 $73 = $71 < $72;
 if ($73) {
  $74 = +HEAPF64[$a>>3];
  HEAPF64[864>>3] = $74;
 }
 $75 = +HEAPF64[$one>>3];
 $76 = HEAP32[2628>>2]|0;
 $77 = (+($76|0));
 $78 = $75 / $77;
 $rbase = $78;
 $79 = +HEAPF64[$one>>3];
 HEAPF64[$small>>3] = $79;
 $i__ = 1;
 while(1) {
  $80 = $i__;
  $81 = ($80|0)<=(3);
  if (!($81)) {
   break;
  }
  $82 = +HEAPF64[$small>>3];
  $83 = $rbase;
  $84 = $82 * $83;
  HEAPF64[$d__1>>3] = $84;
  $85 = (+_dlamc3_($d__1,$zero));
  HEAPF64[$small>>3] = $85;
  $86 = $i__;
  $87 = (($86) + 1)|0;
  $i__ = $87;
 }
 $88 = (+_dlamc3_($one,$small));
 HEAPF64[$a>>3] = $88;
 (_dlamc4_($ngpmin,$one,2628)|0);
 $89 = +HEAPF64[$one>>3];
 $90 = -$89;
 HEAPF64[$d__1>>3] = $90;
 (_dlamc4_($ngnmin,$d__1,2628)|0);
 (_dlamc4_($gpmin,$a,2628)|0);
 $91 = +HEAPF64[$a>>3];
 $92 = -$91;
 HEAPF64[$d__1>>3] = $92;
 (_dlamc4_($gnmin,$d__1,2628)|0);
 HEAP32[$ieee>>2] = 0;
 $93 = HEAP32[$ngpmin>>2]|0;
 $94 = HEAP32[$ngnmin>>2]|0;
 $95 = ($93|0)==($94|0);
 do {
  if ($95) {
   $96 = HEAP32[$gpmin>>2]|0;
   $97 = HEAP32[$gnmin>>2]|0;
   $98 = ($96|0)==($97|0);
   if ($98) {
    $99 = HEAP32[$ngpmin>>2]|0;
    $100 = HEAP32[$gpmin>>2]|0;
    $101 = ($99|0)==($100|0);
    if ($101) {
     $102 = HEAP32[$ngpmin>>2]|0;
     HEAP32[2636>>2] = $102;
     break;
    }
    $103 = HEAP32[$gpmin>>2]|0;
    $104 = HEAP32[$ngpmin>>2]|0;
    $105 = (($103) - ($104))|0;
    $106 = ($105|0)==(3);
    $107 = HEAP32[$ngpmin>>2]|0;
    if ($106) {
     $108 = (($107) - 1)|0;
     $109 = HEAP32[2632>>2]|0;
     $110 = (($108) + ($109))|0;
     HEAP32[2636>>2] = $110;
     HEAP32[$ieee>>2] = 1;
     break;
    } else {
     $111 = HEAP32[$gpmin>>2]|0;
     $112 = ($107|0)<=($111|0);
     $113 = HEAP32[$ngpmin>>2]|0;
     $114 = HEAP32[$gpmin>>2]|0;
     $115 = $112 ? $113 : $114;
     HEAP32[2636>>2] = $115;
     HEAP32[2640>>2] = 1;
     break;
    }
   } else {
    label = 20;
   }
  } else {
   label = 20;
  }
 } while(0);
 do {
  if ((label|0) == 20) {
   $116 = HEAP32[$ngpmin>>2]|0;
   $117 = HEAP32[$gpmin>>2]|0;
   $118 = ($116|0)==($117|0);
   if ($118) {
    $119 = HEAP32[$ngnmin>>2]|0;
    $120 = HEAP32[$gnmin>>2]|0;
    $121 = ($119|0)==($120|0);
    if ($121) {
     $122 = HEAP32[$ngpmin>>2]|0;
     $123 = HEAP32[$ngnmin>>2]|0;
     $124 = (($122) - ($123))|0;
     HEAP32[$i__1>>2] = $124;
     $125 = HEAP32[$i__1>>2]|0;
     $126 = ($125|0)>=(0);
     $127 = HEAP32[$i__1>>2]|0;
     $128 = (0 - ($127))|0;
     $129 = $126 ? $127 : $128;
     $130 = ($129|0)==(1);
     $131 = HEAP32[$ngpmin>>2]|0;
     $132 = HEAP32[$ngnmin>>2]|0;
     if ($130) {
      $133 = ($131|0)>=($132|0);
      $134 = HEAP32[$ngpmin>>2]|0;
      $135 = HEAP32[$ngnmin>>2]|0;
      $136 = $133 ? $134 : $135;
      HEAP32[2636>>2] = $136;
      break;
     } else {
      $137 = ($131|0)<=($132|0);
      $138 = HEAP32[$ngpmin>>2]|0;
      $139 = HEAP32[$ngnmin>>2]|0;
      $140 = $137 ? $138 : $139;
      HEAP32[2636>>2] = $140;
      HEAP32[2640>>2] = 1;
      break;
     }
    }
   }
   $141 = HEAP32[$ngpmin>>2]|0;
   $142 = HEAP32[$ngnmin>>2]|0;
   $143 = (($141) - ($142))|0;
   HEAP32[$i__1>>2] = $143;
   $144 = HEAP32[$i__1>>2]|0;
   $145 = ($144|0)>=(0);
   $146 = HEAP32[$i__1>>2]|0;
   $147 = (0 - ($146))|0;
   $148 = $145 ? $146 : $147;
   $149 = ($148|0)==(1);
   if ($149) {
    $150 = HEAP32[$gpmin>>2]|0;
    $151 = HEAP32[$gnmin>>2]|0;
    $152 = ($150|0)==($151|0);
    if ($152) {
     $153 = HEAP32[$gpmin>>2]|0;
     $154 = HEAP32[$ngpmin>>2]|0;
     $155 = HEAP32[$ngnmin>>2]|0;
     $156 = ($154|0)<=($155|0);
     $157 = HEAP32[$ngpmin>>2]|0;
     $158 = HEAP32[$ngnmin>>2]|0;
     $159 = $156 ? $157 : $158;
     $160 = (($153) - ($159))|0;
     $161 = ($160|0)==(3);
     $162 = HEAP32[$ngpmin>>2]|0;
     $163 = HEAP32[$ngnmin>>2]|0;
     if ($161) {
      $164 = ($162|0)>=($163|0);
      $165 = HEAP32[$ngpmin>>2]|0;
      $166 = HEAP32[$ngnmin>>2]|0;
      $167 = $164 ? $165 : $166;
      $168 = (($167) - 1)|0;
      $169 = HEAP32[2632>>2]|0;
      $170 = (($168) + ($169))|0;
      HEAP32[2636>>2] = $170;
      break;
     } else {
      $171 = ($162|0)<=($163|0);
      $172 = HEAP32[$ngpmin>>2]|0;
      $173 = HEAP32[$ngnmin>>2]|0;
      $174 = $171 ? $172 : $173;
      HEAP32[2636>>2] = $174;
      HEAP32[2640>>2] = 1;
      break;
     }
    }
   }
   $175 = HEAP32[$ngpmin>>2]|0;
   $176 = HEAP32[$ngnmin>>2]|0;
   $177 = ($175|0)<=($176|0);
   $178 = HEAP32[$ngpmin>>2]|0;
   $179 = HEAP32[$ngnmin>>2]|0;
   $180 = $177 ? $178 : $179;
   HEAP32[$i__1>>2] = $180;
   $181 = HEAP32[$i__1>>2]|0;
   $182 = HEAP32[$gpmin>>2]|0;
   $183 = ($181|0)<=($182|0);
   $184 = HEAP32[$i__1>>2]|0;
   $185 = HEAP32[$gpmin>>2]|0;
   $186 = $183 ? $184 : $185;
   HEAP32[$i__1>>2] = $186;
   $187 = HEAP32[$i__1>>2]|0;
   $188 = HEAP32[$gnmin>>2]|0;
   $189 = ($187|0)<=($188|0);
   $190 = HEAP32[$i__1>>2]|0;
   $191 = HEAP32[$gnmin>>2]|0;
   $192 = $189 ? $190 : $191;
   HEAP32[2636>>2] = $192;
   HEAP32[2640>>2] = 1;
  }
 } while(0);
 HEAP32[2624>>2] = 0;
 $193 = HEAP32[2640>>2]|0;
 $194 = ($193|0)!=(0);
 if ($194) {
  HEAP32[2624>>2] = 1;
  (_s_wsfe(2644)|0);
  (_do_fio(2664,2636,4)|0);
  (_e_wsfe()|0);
 }
 $195 = HEAP32[$ieee>>2]|0;
 $196 = ($195|0)!=(0);
 $197 = HEAP32[$lieee1>>2]|0;
 $198 = ($197|0)!=(0);
 $199 = $196 ? 1 : $198;
 $200 = $199&1;
 HEAP32[$ieee>>2] = $200;
 HEAPF64[872>>3] = 1.0;
 $201 = HEAP32[2636>>2]|0;
 $202 = (1 - ($201))|0;
 HEAP32[$i__1>>2] = $202;
 $i__ = 1;
 while(1) {
  $203 = $i__;
  $204 = HEAP32[$i__1>>2]|0;
  $205 = ($203|0)<=($204|0);
  if (!($205)) {
   break;
  }
  $206 = +HEAPF64[872>>3];
  $207 = $rbase;
  $208 = $206 * $207;
  HEAPF64[$d__1>>3] = $208;
  $209 = (+_dlamc3_($d__1,$zero));
  HEAPF64[872>>3] = $209;
  $210 = $i__;
  $211 = (($210) + 1)|0;
  $i__ = $211;
 }
 (_dlamc5_(2628,2632,2636,$ieee,2668,880)|0);
 $212 = HEAP32[2628>>2]|0;
 $213 = $0;
 HEAP32[$213>>2] = $212;
 $214 = HEAP32[2632>>2]|0;
 $215 = $1;
 HEAP32[$215>>2] = $214;
 $216 = HEAP32[$lrnd>>2]|0;
 $217 = $2;
 HEAP32[$217>>2] = $216;
 $218 = +HEAPF64[864>>3];
 $219 = $3;
 HEAPF64[$219>>3] = $218;
 $220 = HEAP32[2636>>2]|0;
 $221 = $4;
 HEAP32[$221>>2] = $220;
 $222 = +HEAPF64[872>>3];
 $223 = $5;
 HEAPF64[$223>>3] = $222;
 $224 = HEAP32[2668>>2]|0;
 $225 = $6;
 HEAP32[$225>>2] = $224;
 $226 = +HEAPF64[880>>3];
 $227 = $7;
 HEAPF64[$227>>3] = $226;
 STACKTOP = sp;return 0;
}
function _dlamc1_($beta,$t,$rnd,$ieee1) {
 $beta = $beta|0;
 $t = $t|0;
 $rnd = $rnd|0;
 $ieee1 = $ieee1|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0.0, $60 = 0.0, $61 = 0.0, $62 = 0.0;
 var $63 = 0, $64 = 0.0, $65 = 0.0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0.0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0.0, $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0;
 var $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0.0, $90 = 0, $91 = 0, $a = 0, $b = 0, $c__ = 0, $d__1 = 0, $d__2 = 0, $f = 0, $one = 0, $qtr = 0.0;
 var $savec = 0, $t1 = 0.0, $t2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $d__1 = sp + 80|0;
 $d__2 = sp + 72|0;
 $a = sp + 64|0;
 $b = sp + 56|0;
 $c__ = sp + 48|0;
 $f = sp + 40|0;
 $one = sp + 16|0;
 $savec = sp;
 $0 = $beta;
 $1 = $t;
 $2 = $rnd;
 $3 = $ieee1;
 $4 = HEAP32[2672>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $84 = HEAP32[2676>>2]|0;
  $85 = $0;
  HEAP32[$85>>2] = $84;
  $86 = HEAP32[2688>>2]|0;
  $87 = $1;
  HEAP32[$87>>2] = $86;
  $88 = HEAP32[2680>>2]|0;
  $89 = $2;
  HEAP32[$89>>2] = $88;
  $90 = HEAP32[2684>>2]|0;
  $91 = $3;
  HEAP32[$91>>2] = $90;
  HEAP32[2672>>2] = 0;
  STACKTOP = sp;return 0;
 }
 HEAPF64[$one>>3] = 1.0;
 HEAPF64[$a>>3] = 1.0;
 HEAPF64[$c__>>3] = 1.0;
 while(1) {
  $6 = +HEAPF64[$c__>>3];
  $7 = +HEAPF64[$one>>3];
  $8 = $6 == $7;
  if (!($8)) {
   break;
  }
  $9 = +HEAPF64[$a>>3];
  $10 = $9 * 2.0;
  HEAPF64[$a>>3] = $10;
  $11 = (+_dlamc3_($a,$one));
  HEAPF64[$c__>>3] = $11;
  $12 = +HEAPF64[$a>>3];
  $13 = -$12;
  HEAPF64[$d__1>>3] = $13;
  $14 = (+_dlamc3_($c__,$d__1));
  HEAPF64[$c__>>3] = $14;
 }
 HEAPF64[$b>>3] = 1.0;
 $15 = (+_dlamc3_($a,$b));
 HEAPF64[$c__>>3] = $15;
 while(1) {
  $16 = +HEAPF64[$c__>>3];
  $17 = +HEAPF64[$a>>3];
  $18 = $16 == $17;
  if (!($18)) {
   break;
  }
  $19 = +HEAPF64[$b>>3];
  $20 = $19 * 2.0;
  HEAPF64[$b>>3] = $20;
  $21 = (+_dlamc3_($a,$b));
  HEAPF64[$c__>>3] = $21;
 }
 $22 = +HEAPF64[$one>>3];
 $23 = $22 / 4.0;
 $qtr = $23;
 $24 = +HEAPF64[$c__>>3];
 HEAPF64[$savec>>3] = $24;
 $25 = +HEAPF64[$a>>3];
 $26 = -$25;
 HEAPF64[$d__1>>3] = $26;
 $27 = (+_dlamc3_($c__,$d__1));
 HEAPF64[$c__>>3] = $27;
 $28 = +HEAPF64[$c__>>3];
 $29 = $qtr;
 $30 = $28 + $29;
 $31 = (~~(($30)));
 HEAP32[2676>>2] = $31;
 $32 = HEAP32[2676>>2]|0;
 $33 = (+($32|0));
 HEAPF64[$b>>3] = $33;
 $34 = +HEAPF64[$b>>3];
 $35 = $34 / 2.0;
 HEAPF64[$d__1>>3] = $35;
 $36 = +HEAPF64[$b>>3];
 $37 = -$36;
 $38 = $37 / 100.0;
 HEAPF64[$d__2>>3] = $38;
 $39 = (+_dlamc3_($d__1,$d__2));
 HEAPF64[$f>>3] = $39;
 $40 = (+_dlamc3_($f,$a));
 HEAPF64[$c__>>3] = $40;
 $41 = +HEAPF64[$c__>>3];
 $42 = +HEAPF64[$a>>3];
 $43 = $41 == $42;
 if ($43) {
  HEAP32[2680>>2] = 1;
 } else {
  HEAP32[2680>>2] = 0;
 }
 $44 = +HEAPF64[$b>>3];
 $45 = $44 / 2.0;
 HEAPF64[$d__1>>3] = $45;
 $46 = +HEAPF64[$b>>3];
 $47 = $46 / 100.0;
 HEAPF64[$d__2>>3] = $47;
 $48 = (+_dlamc3_($d__1,$d__2));
 HEAPF64[$f>>3] = $48;
 $49 = (+_dlamc3_($f,$a));
 HEAPF64[$c__>>3] = $49;
 $50 = HEAP32[2680>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = +HEAPF64[$c__>>3];
  $53 = +HEAPF64[$a>>3];
  $54 = $52 == $53;
  if ($54) {
   HEAP32[2680>>2] = 0;
  }
 }
 $55 = +HEAPF64[$b>>3];
 $56 = $55 / 2.0;
 HEAPF64[$d__1>>3] = $56;
 $57 = (+_dlamc3_($d__1,$a));
 $t1 = $57;
 $58 = +HEAPF64[$b>>3];
 $59 = $58 / 2.0;
 HEAPF64[$d__1>>3] = $59;
 $60 = (+_dlamc3_($d__1,$savec));
 $t2 = $60;
 $61 = $t1;
 $62 = +HEAPF64[$a>>3];
 $63 = $61 == $62;
 if ($63) {
  $64 = $t2;
  $65 = +HEAPF64[$savec>>3];
  $66 = $64 > $65;
  if ($66) {
   $67 = HEAP32[2680>>2]|0;
   $68 = ($67|0)!=(0);
   $70 = $68;
  } else {
   $70 = 0;
  }
 } else {
  $70 = 0;
 }
 $69 = $70&1;
 HEAP32[2684>>2] = $69;
 HEAP32[2688>>2] = 0;
 HEAPF64[$a>>3] = 1.0;
 HEAPF64[$c__>>3] = 1.0;
 while(1) {
  $71 = +HEAPF64[$c__>>3];
  $72 = +HEAPF64[$one>>3];
  $73 = $71 == $72;
  if (!($73)) {
   break;
  }
  $74 = HEAP32[2688>>2]|0;
  $75 = (($74) + 1)|0;
  HEAP32[2688>>2] = $75;
  $76 = HEAP32[2676>>2]|0;
  $77 = (+($76|0));
  $78 = +HEAPF64[$a>>3];
  $79 = $78 * $77;
  HEAPF64[$a>>3] = $79;
  $80 = (+_dlamc3_($a,$one));
  HEAPF64[$c__>>3] = $80;
  $81 = +HEAPF64[$a>>3];
  $82 = -$81;
  HEAPF64[$d__1>>3] = $82;
  $83 = (+_dlamc3_($c__,$d__1));
  HEAPF64[$c__>>3] = $83;
 }
 $84 = HEAP32[2676>>2]|0;
 $85 = $0;
 HEAP32[$85>>2] = $84;
 $86 = HEAP32[2688>>2]|0;
 $87 = $1;
 HEAP32[$87>>2] = $86;
 $88 = HEAP32[2680>>2]|0;
 $89 = $2;
 HEAP32[$89>>2] = $88;
 $90 = HEAP32[2684>>2]|0;
 $91 = $3;
 HEAP32[$91>>2] = $90;
 HEAP32[2672>>2] = 0;
 STACKTOP = sp;return 0;
}
function _dlamc3_($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0.0, $6 = 0.0, $7 = 0.0, $ret_val = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = +HEAPF64[$2>>3];
 $4 = $1;
 $5 = +HEAPF64[$4>>3];
 $6 = $3 + $5;
 $ret_val = $6;
 $7 = $ret_val;
 STACKTOP = sp;return (+$7);
}
function _dlamc4_($emin,$start,$base) {
 $emin = $emin|0;
 $start = $start|0;
 $base = $base|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0.0, $26 = 0.0;
 var $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0.0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0.0, $4 = 0.0, $40 = 0.0, $41 = 0.0, $42 = 0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0.0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0.0;
 var $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0, $8 = 0.0, $9 = 0.0, $a = 0.0, $b1 = 0.0, $b2 = 0.0;
 var $c1 = 0.0, $c2 = 0.0, $d1 = 0.0, $d2 = 0.0, $d__1 = 0, $i__ = 0, $i__1 = 0, $one = 0.0, $rbase = 0.0, $zero = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $d__1 = sp + 80|0;
 $zero = sp + 8|0;
 $0 = $emin;
 $1 = $start;
 $2 = $base;
 $3 = $1;
 $4 = +HEAPF64[$3>>3];
 $a = $4;
 $one = 1.0;
 $5 = $one;
 $6 = $2;
 $7 = HEAP32[$6>>2]|0;
 $8 = (+($7|0));
 $9 = $5 / $8;
 $rbase = $9;
 HEAPF64[$zero>>3] = 0.0;
 $10 = $0;
 HEAP32[$10>>2] = 1;
 $11 = $a;
 $12 = $rbase;
 $13 = $11 * $12;
 HEAPF64[$d__1>>3] = $13;
 $14 = (+_dlamc3_($d__1,$zero));
 $b1 = $14;
 $15 = $a;
 $c1 = $15;
 $16 = $a;
 $c2 = $16;
 $17 = $a;
 $d1 = $17;
 $18 = $a;
 $d2 = $18;
 L1: while(1) {
  $19 = $c1;
  $20 = $a;
  $21 = $19 == $20;
  if (!($21)) {
   label = 12;
   break;
  }
  $22 = $c2;
  $23 = $a;
  $24 = $22 == $23;
  if (!($24)) {
   label = 12;
   break;
  }
  $25 = $d1;
  $26 = $a;
  $27 = $25 == $26;
  if (!($27)) {
   label = 12;
   break;
  }
  $28 = $d2;
  $29 = $a;
  $30 = $28 == $29;
  if (!($30)) {
   label = 12;
   break;
  }
  $31 = $0;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + -1)|0;
  HEAP32[$31>>2] = $33;
  $34 = $b1;
  $a = $34;
  $35 = $a;
  $36 = $2;
  $37 = HEAP32[$36>>2]|0;
  $38 = (+($37|0));
  $39 = $35 / $38;
  HEAPF64[$d__1>>3] = $39;
  $40 = (+_dlamc3_($d__1,$zero));
  $b1 = $40;
  $41 = $b1;
  $42 = $2;
  $43 = HEAP32[$42>>2]|0;
  $44 = (+($43|0));
  $45 = $41 * $44;
  HEAPF64[$d__1>>3] = $45;
  $46 = (+_dlamc3_($d__1,$zero));
  $c1 = $46;
  $47 = +HEAPF64[$zero>>3];
  $d1 = $47;
  $48 = $2;
  $49 = HEAP32[$48>>2]|0;
  $i__1 = $49;
  $i__ = 1;
  while(1) {
   $50 = $i__;
   $51 = $i__1;
   $52 = ($50|0)<=($51|0);
   if (!($52)) {
    break;
   }
   $53 = $b1;
   $54 = $d1;
   $55 = $54 + $53;
   $d1 = $55;
   $56 = $i__;
   $57 = (($56) + 1)|0;
   $i__ = $57;
  }
  $58 = $a;
  $59 = $rbase;
  $60 = $58 * $59;
  HEAPF64[$d__1>>3] = $60;
  $61 = (+_dlamc3_($d__1,$zero));
  $b2 = $61;
  $62 = $b2;
  $63 = $rbase;
  $64 = $62 / $63;
  HEAPF64[$d__1>>3] = $64;
  $65 = (+_dlamc3_($d__1,$zero));
  $c2 = $65;
  $66 = +HEAPF64[$zero>>3];
  $d2 = $66;
  $67 = $2;
  $68 = HEAP32[$67>>2]|0;
  $i__1 = $68;
  $i__ = 1;
  while(1) {
   $69 = $i__;
   $70 = $i__1;
   $71 = ($69|0)<=($70|0);
   if (!($71)) {
    continue L1;
   }
   $72 = $b2;
   $73 = $d2;
   $74 = $73 + $72;
   $d2 = $74;
   $75 = $i__;
   $76 = (($75) + 1)|0;
   $i__ = $76;
  }
 }
 if ((label|0) == 12) {
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _dlamc5_($beta,$p,$emin,$ieee,$emax,$rmax) {
 $beta = $beta|0;
 $p = $p|0;
 $emin = $emin|0;
 $ieee = $ieee|0;
 $emax = $emax|0;
 $rmax = $rmax|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0.0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0, $83 = 0.0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0.0;
 var $96 = 0, $97 = 0, $98 = 0.0, $99 = 0.0, $d__1 = 0, $exbits = 0, $expsum = 0, $i__ = 0, $i__1 = 0, $lexp = 0, $nbits = 0, $oldy = 0.0, $recbas = 0.0, $try__ = 0, $uexp = 0, $y = 0, $z__ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $d__1 = sp + 32|0;
 $y = sp + 24|0;
 $z__ = sp + 16|0;
 $0 = $beta;
 $1 = $p;
 $2 = $emin;
 $3 = $ieee;
 $4 = $emax;
 $5 = $rmax;
 $lexp = 1;
 $exbits = 1;
 while(1) {
  $6 = $lexp;
  $7 = $6 << 1;
  $try__ = $7;
  $8 = $try__;
  $9 = $2;
  $10 = HEAP32[$9>>2]|0;
  $11 = (0 - ($10))|0;
  $12 = ($8|0)<=($11|0);
  if (!($12)) {
   break;
  }
  $13 = $try__;
  $lexp = $13;
  $14 = $exbits;
  $15 = (($14) + 1)|0;
  $exbits = $15;
 }
 $16 = $lexp;
 $17 = $2;
 $18 = HEAP32[$17>>2]|0;
 $19 = (0 - ($18))|0;
 $20 = ($16|0)==($19|0);
 if ($20) {
  $21 = $lexp;
  $uexp = $21;
 } else {
  $22 = $try__;
  $uexp = $22;
  $23 = $exbits;
  $24 = (($23) + 1)|0;
  $exbits = $24;
 }
 $25 = $uexp;
 $26 = $2;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + ($27))|0;
 $29 = $lexp;
 $30 = (0 - ($29))|0;
 $31 = $2;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) - ($32))|0;
 $34 = ($28|0)>($33|0);
 if ($34) {
  $35 = $lexp;
  $36 = $35 << 1;
  $expsum = $36;
 } else {
  $37 = $uexp;
  $38 = $37 << 1;
  $expsum = $38;
 }
 $39 = $expsum;
 $40 = $2;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) + ($41))|0;
 $43 = (($42) - 1)|0;
 $44 = $4;
 HEAP32[$44>>2] = $43;
 $45 = $exbits;
 $46 = (($45) + 1)|0;
 $47 = $1;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + ($48))|0;
 $nbits = $49;
 $50 = $nbits;
 $51 = (($50|0) % 2)&-1;
 $52 = ($51|0)==(1);
 if ($52) {
  $53 = $0;
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)==(2);
  if ($55) {
   $56 = $4;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + -1)|0;
   HEAP32[$56>>2] = $58;
  }
 }
 $59 = $3;
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)!=(0);
 if ($61) {
  $62 = $4;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + -1)|0;
  HEAP32[$62>>2] = $64;
 }
 $65 = $0;
 $66 = HEAP32[$65>>2]|0;
 $67 = (+($66|0));
 $68 = 1.0 / $67;
 $recbas = $68;
 $69 = $0;
 $70 = HEAP32[$69>>2]|0;
 $71 = (+($70|0));
 $72 = $71 - 1.0;
 HEAPF64[$z__>>3] = $72;
 HEAPF64[$y>>3] = 0.0;
 $73 = $1;
 $74 = HEAP32[$73>>2]|0;
 $i__1 = $74;
 $i__ = 1;
 while(1) {
  $75 = $i__;
  $76 = $i__1;
  $77 = ($75|0)<=($76|0);
  if (!($77)) {
   break;
  }
  $78 = $recbas;
  $79 = +HEAPF64[$z__>>3];
  $80 = $79 * $78;
  HEAPF64[$z__>>3] = $80;
  $81 = +HEAPF64[$y>>3];
  $82 = $81 < 1.0;
  if ($82) {
   $83 = +HEAPF64[$y>>3];
   $oldy = $83;
  }
  $84 = (+_dlamc3_($y,$z__));
  HEAPF64[$y>>3] = $84;
  $85 = $i__;
  $86 = (($85) + 1)|0;
  $i__ = $86;
 }
 $87 = +HEAPF64[$y>>3];
 $88 = $87 >= 1.0;
 if ($88) {
  $89 = $oldy;
  HEAPF64[$y>>3] = $89;
 }
 $90 = $4;
 $91 = HEAP32[$90>>2]|0;
 $i__1 = $91;
 $i__ = 1;
 while(1) {
  $92 = $i__;
  $93 = $i__1;
  $94 = ($92|0)<=($93|0);
  $95 = +HEAPF64[$y>>3];
  if (!($94)) {
   break;
  }
  $96 = $0;
  $97 = HEAP32[$96>>2]|0;
  $98 = (+($97|0));
  $99 = $95 * $98;
  HEAPF64[$d__1>>3] = $99;
  $100 = (+_dlamc3_($d__1,888));
  HEAPF64[$y>>3] = $100;
  $101 = $i__;
  $102 = (($101) + 1)|0;
  $i__ = $102;
 }
 $103 = $5;
 HEAPF64[$103>>3] = $95;
 STACKTOP = sp;return 0;
}
function _ilaenv_($ispec,$name__,$opts,$n1,$n2,$n3,$n4) {
 $ispec = $ispec|0;
 $name__ = $name__|0;
 $opts = $opts|0;
 $n1 = $n1|0;
 $n2 = $n2|0;
 $n3 = $n3|0;
 $n4 = $n4|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0.0, $496 = 0, $497 = 0.0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $c1 = 0, $c2 = 0, $c3 = 0, $c4 = 0, $cname = 0, $i__ = 0, $ic = 0, $iz = 0, $name_len = 0, $nb = 0, $nbmin = 0, $nx = 0, $opts_len = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0;
 var $or$cond21 = 0, $or$cond23 = 0, $or$cond25 = 0, $or$cond27 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $ret_val = 0, $sname = 0, $subnam = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $c1 = sp + 90|0;
 $c2 = sp + 88|0;
 $c3 = sp + 84|0;
 $c4 = sp + 82|0;
 $subnam = sp + 76|0;
 $1 = $ispec;
 $2 = $name__;
 $3 = $opts;
 $4 = $n1;
 $5 = $n2;
 $6 = $n3;
 $7 = $n4;
 $8 = $2;
 $9 = (_strlen($8)|0);
 $name_len = $9;
 $10 = $3;
 $11 = (_strlen($10)|0);
 $opts_len = $11;
 $12 = $1;
 $13 = HEAP32[$12>>2]|0;
 do {
  switch ($13|0) {
  case 3: case 2: case 1:  {
   $ret_val = 1;
   $15 = $2;
   $16 = $name_len;
   _s_copy($subnam,$15,1,$16);
   $17 = HEAP8[$subnam>>0]|0;
   $18 = $17&255;
   $ic = $18;
   $iz = 90;
   $19 = $iz;
   $20 = ($19|0)==(90);
   $21 = $iz;
   $22 = ($21|0)==(122);
   $or$cond = $20 | $22;
   L5: do {
    if ($or$cond) {
     $23 = $ic;
     $24 = ($23|0)>=(97);
     $25 = $ic;
     $26 = ($25|0)<=(122);
     $or$cond3 = $24 & $26;
     if ($or$cond3) {
      $27 = $ic;
      $28 = (($27) - 32)|0;
      $29 = $28&255;
      HEAP8[$subnam>>0] = $29;
      $i__ = 2;
      while(1) {
       $30 = $i__;
       $31 = ($30|0)<=(6);
       if (!($31)) {
        break L5;
       }
       $32 = $i__;
       $33 = (($32) - 1)|0;
       $34 = (($subnam) + ($33)|0);
       $35 = HEAP8[$34>>0]|0;
       $36 = $35&255;
       $ic = $36;
       $37 = $ic;
       $38 = ($37|0)>=(97);
       $39 = $ic;
       $40 = ($39|0)<=(122);
       $or$cond5 = $38 & $40;
       if ($or$cond5) {
        $41 = $ic;
        $42 = (($41) - 32)|0;
        $43 = $42&255;
        $44 = $i__;
        $45 = (($44) - 1)|0;
        $46 = (($subnam) + ($45)|0);
        HEAP8[$46>>0] = $43;
       }
       $47 = $i__;
       $48 = (($47) + 1)|0;
       $i__ = $48;
      }
     }
    } else {
     $49 = $iz;
     $50 = ($49|0)==(233);
     $51 = $iz;
     $52 = ($51|0)==(169);
     $or$cond7 = $50 | $52;
     if (!($or$cond7)) {
      $95 = $iz;
      $96 = ($95|0)==(218);
      $97 = $iz;
      $98 = ($97|0)==(250);
      $or$cond21 = $96 | $98;
      if (!($or$cond21)) {
       break;
      }
      $99 = $ic;
      $100 = ($99|0)>=(225);
      $101 = $ic;
      $102 = ($101|0)<=(250);
      $or$cond23 = $100 & $102;
      if (!($or$cond23)) {
       break;
      }
      $103 = $ic;
      $104 = (($103) - 32)|0;
      $105 = $104&255;
      HEAP8[$subnam>>0] = $105;
      $i__ = 2;
      while(1) {
       $106 = $i__;
       $107 = ($106|0)<=(6);
       if (!($107)) {
        break L5;
       }
       $108 = $i__;
       $109 = (($108) - 1)|0;
       $110 = (($subnam) + ($109)|0);
       $111 = HEAP8[$110>>0]|0;
       $112 = $111&255;
       $ic = $112;
       $113 = $ic;
       $114 = ($113|0)>=(225);
       $115 = $ic;
       $116 = ($115|0)<=(250);
       $or$cond25 = $114 & $116;
       if ($or$cond25) {
        $117 = $ic;
        $118 = (($117) - 32)|0;
        $119 = $118&255;
        $120 = $i__;
        $121 = (($120) - 1)|0;
        $122 = (($subnam) + ($121)|0);
        HEAP8[$122>>0] = $119;
       }
       $123 = $i__;
       $124 = (($123) + 1)|0;
       $i__ = $124;
      }
     }
     $53 = $ic;
     $54 = ($53|0)>=(129);
     $55 = $ic;
     $56 = ($55|0)<=(137);
     $or$cond9 = $54 & $56;
     if (!($or$cond9)) {
      $57 = $ic;
      $58 = ($57|0)>=(145);
      $59 = $ic;
      $60 = ($59|0)<=(153);
      $or$cond11 = $58 & $60;
      if (!($or$cond11)) {
       $61 = $ic;
       $62 = ($61|0)>=(162);
       $63 = $ic;
       $64 = ($63|0)<=(169);
       $or$cond13 = $62 & $64;
       if (!($or$cond13)) {
        break;
       }
      }
     }
     $65 = $ic;
     $66 = (($65) + 64)|0;
     $67 = $66&255;
     HEAP8[$subnam>>0] = $67;
     $i__ = 2;
     while(1) {
      $68 = $i__;
      $69 = ($68|0)<=(6);
      if (!($69)) {
       break L5;
      }
      $70 = $i__;
      $71 = (($70) - 1)|0;
      $72 = (($subnam) + ($71)|0);
      $73 = HEAP8[$72>>0]|0;
      $74 = $73&255;
      $ic = $74;
      $75 = $ic;
      $76 = ($75|0)>=(129);
      $77 = $ic;
      $78 = ($77|0)<=(137);
      $or$cond15 = $76 & $78;
      if ($or$cond15) {
       label = 19;
      } else {
       $79 = $ic;
       $80 = ($79|0)>=(145);
       $81 = $ic;
       $82 = ($81|0)<=(153);
       $or$cond17 = $80 & $82;
       if ($or$cond17) {
        label = 19;
       } else {
        $83 = $ic;
        $84 = ($83|0)>=(162);
        $85 = $ic;
        $86 = ($85|0)<=(169);
        $or$cond19 = $84 & $86;
        if ($or$cond19) {
         label = 19;
        }
       }
      }
      if ((label|0) == 19) {
       label = 0;
       $87 = $ic;
       $88 = (($87) + 64)|0;
       $89 = $88&255;
       $90 = $i__;
       $91 = (($90) - 1)|0;
       $92 = (($subnam) + ($91)|0);
       HEAP8[$92>>0] = $89;
      }
      $93 = $i__;
      $94 = (($93) + 1)|0;
      $i__ = $94;
     }
    }
   } while(0);
   $125 = HEAP8[$subnam>>0]|0;
   HEAP8[$c1>>0] = $125;
   $126 = HEAP8[$c1>>0]|0;
   $127 = $126&255;
   $128 = ($127|0)==(83);
   if ($128) {
    $133 = 1;
   } else {
    $129 = HEAP8[$c1>>0]|0;
    $130 = $129&255;
    $131 = ($130|0)==(68);
    $133 = $131;
   }
   $132 = $133&1;
   $sname = $132;
   $134 = HEAP8[$c1>>0]|0;
   $135 = $134&255;
   $136 = ($135|0)==(67);
   if ($136) {
    $141 = 1;
   } else {
    $137 = HEAP8[$c1>>0]|0;
    $138 = $137&255;
    $139 = ($138|0)==(90);
    $141 = $139;
   }
   $140 = $141&1;
   $cname = $140;
   $142 = $cname;
   $143 = ($142|0)!=(0);
   $144 = $sname;
   $145 = ($144|0)!=(0);
   $or$cond27 = $143 | $145;
   if (!($or$cond27)) {
    $146 = $ret_val;
    $0 = $146;
    $520 = $0;
    STACKTOP = sp;return ($520|0);
   }
   $147 = ((($subnam)) + 1|0);
   _s_copy($c2,$147,1,2);
   $148 = ((($subnam)) + 3|0);
   _s_copy($c3,$148,1,3);
   $149 = ((($c3)) + 1|0);
   _s_copy($c4,$149,1,2);
   $150 = $1;
   $151 = HEAP32[$150>>2]|0;
   switch ($151|0) {
   case 3:  {
    $nx = 0;
    $414 = (_s_cmp($c2,18391,1,2)|0);
    $415 = ($414|0)==(0);
    L186: do {
     if ($415) {
      $416 = (_s_cmp($c3,18398,1,3)|0);
      $417 = ($416|0)==(0);
      if (!($417)) {
       $418 = (_s_cmp($c3,18402,1,3)|0);
       $419 = ($418|0)==(0);
       if (!($419)) {
        $420 = (_s_cmp($c3,18406,1,3)|0);
        $421 = ($420|0)==(0);
        if (!($421)) {
         $422 = (_s_cmp($c3,18410,1,3)|0);
         $423 = ($422|0)==(0);
         if (!($423)) {
          $424 = (_s_cmp($c3,18414,1,3)|0);
          $425 = ($424|0)==(0);
          if ($425) {
           $nx = 128;
           break;
          }
          $426 = (_s_cmp($c3,18418,1,3)|0);
          $427 = ($426|0)==(0);
          if (!($427)) {
           break;
          }
          $nx = 128;
          break;
         }
        }
       }
      }
      $nx = 128;
     } else {
      $428 = (_s_cmp($c2,18429,1,2)|0);
      $429 = ($428|0)==(0);
      if ($429) {
       $430 = $sname;
       $431 = ($430|0)!=(0);
       if (!($431)) {
        break;
       }
       $432 = (_s_cmp($c3,18432,1,3)|0);
       $433 = ($432|0)==(0);
       if (!($433)) {
        break;
       }
       $nx = 32;
       break;
      }
      $434 = $cname;
      $435 = ($434|0)!=(0);
      if ($435) {
       $436 = (_s_cmp($c2,18440,1,2)|0);
       $437 = ($436|0)==(0);
       if ($437) {
        $438 = (_s_cmp($c3,18432,1,3)|0);
        $439 = ($438|0)==(0);
        if (!($439)) {
         break;
        }
        $nx = 32;
        break;
       }
      }
      $440 = $sname;
      $441 = ($440|0)!=(0);
      if ($441) {
       $442 = (_s_cmp($c2,18443,1,2)|0);
       $443 = ($442|0)==(0);
       if ($443) {
        $444 = HEAP8[$c3>>0]|0;
        $445 = $444&255;
        $446 = ($445|0)==(71);
        if (!($446)) {
         break;
        }
        $447 = (_s_cmp($c4,18446,1,2)|0);
        $448 = ($447|0)==(0);
        do {
         if (!($448)) {
          $449 = (_s_cmp($c4,18449,1,2)|0);
          $450 = ($449|0)==(0);
          if (!($450)) {
           $451 = (_s_cmp($c4,18452,1,2)|0);
           $452 = ($451|0)==(0);
           if (!($452)) {
            $453 = (_s_cmp($c4,18455,1,2)|0);
            $454 = ($453|0)==(0);
            if (!($454)) {
             $455 = (_s_cmp($c4,18458,1,2)|0);
             $456 = ($455|0)==(0);
             if (!($456)) {
              $457 = (_s_cmp($c4,18461,1,2)|0);
              $458 = ($457|0)==(0);
              if ($458) {
               break;
              }
              $459 = (_s_cmp($c4,18464,1,2)|0);
              $460 = ($459|0)==(0);
              if (!($460)) {
               break L186;
              }
             }
            }
           }
          }
         }
        } while(0);
        $nx = 128;
        break;
       }
      }
      $461 = $cname;
      $462 = ($461|0)!=(0);
      if ($462) {
       $463 = (_s_cmp($c2,18467,1,2)|0);
       $464 = ($463|0)==(0);
       if ($464) {
        $465 = HEAP8[$c3>>0]|0;
        $466 = $465&255;
        $467 = ($466|0)==(71);
        if ($467) {
         $468 = (_s_cmp($c4,18446,1,2)|0);
         $469 = ($468|0)==(0);
         do {
          if (!($469)) {
           $470 = (_s_cmp($c4,18449,1,2)|0);
           $471 = ($470|0)==(0);
           if (!($471)) {
            $472 = (_s_cmp($c4,18452,1,2)|0);
            $473 = ($472|0)==(0);
            if (!($473)) {
             $474 = (_s_cmp($c4,18455,1,2)|0);
             $475 = ($474|0)==(0);
             if (!($475)) {
              $476 = (_s_cmp($c4,18458,1,2)|0);
              $477 = ($476|0)==(0);
              if (!($477)) {
               $478 = (_s_cmp($c4,18461,1,2)|0);
               $479 = ($478|0)==(0);
               if ($479) {
                break;
               }
               $480 = (_s_cmp($c4,18464,1,2)|0);
               $481 = ($480|0)==(0);
               if (!($481)) {
                break L186;
               }
              }
             }
            }
           }
          }
         } while(0);
         $nx = 128;
        }
       }
      }
     }
    } while(0);
    $482 = $nx;
    $ret_val = $482;
    $483 = $ret_val;
    $0 = $483;
    $520 = $0;
    STACKTOP = sp;return ($520|0);
    break;
   }
   case 2:  {
    $nbmin = 2;
    $306 = (_s_cmp($c2,18391,1,2)|0);
    $307 = ($306|0)==(0);
    L236: do {
     if ($307) {
      $308 = (_s_cmp($c3,18398,1,3)|0);
      $309 = ($308|0)==(0);
      if (!($309)) {
       $310 = (_s_cmp($c3,18402,1,3)|0);
       $311 = ($310|0)==(0);
       if (!($311)) {
        $312 = (_s_cmp($c3,18406,1,3)|0);
        $313 = ($312|0)==(0);
        if (!($313)) {
         $314 = (_s_cmp($c3,18410,1,3)|0);
         $315 = ($314|0)==(0);
         if (!($315)) {
          $316 = (_s_cmp($c3,18414,1,3)|0);
          $317 = ($316|0)==(0);
          if ($317) {
           $nbmin = 2;
           break;
          }
          $318 = (_s_cmp($c3,18418,1,3)|0);
          $319 = ($318|0)==(0);
          if ($319) {
           $nbmin = 2;
           break;
          }
          $320 = (_s_cmp($c3,18422,1,3)|0);
          $321 = ($320|0)==(0);
          if (!($321)) {
           break;
          }
          $nbmin = 2;
          break;
         }
        }
       }
      }
      $nbmin = 2;
     } else {
      $322 = (_s_cmp($c2,18429,1,2)|0);
      $323 = ($322|0)==(0);
      if ($323) {
       $324 = (_s_cmp($c3,18394,1,3)|0);
       $325 = ($324|0)==(0);
       $326 = $sname;
       $327 = ($326|0)!=(0);
       if ($325) {
        $nbmin = 8;
        break;
       }
       if (!($327)) {
        break;
       }
       $328 = (_s_cmp($c3,18432,1,3)|0);
       $329 = ($328|0)==(0);
       if (!($329)) {
        break;
       }
       $nbmin = 2;
       break;
      }
      $330 = $cname;
      $331 = ($330|0)!=(0);
      if ($331) {
       $332 = (_s_cmp($c2,18440,1,2)|0);
       $333 = ($332|0)==(0);
       if ($333) {
        $334 = (_s_cmp($c3,18432,1,3)|0);
        $335 = ($334|0)==(0);
        if (!($335)) {
         break;
        }
        $nbmin = 2;
        break;
       }
      }
      $336 = $sname;
      $337 = ($336|0)!=(0);
      if ($337) {
       $338 = (_s_cmp($c2,18443,1,2)|0);
       $339 = ($338|0)==(0);
       if ($339) {
        $340 = HEAP8[$c3>>0]|0;
        $341 = $340&255;
        $342 = ($341|0)==(71);
        if ($342) {
         $343 = (_s_cmp($c4,18446,1,2)|0);
         $344 = ($343|0)==(0);
         do {
          if (!($344)) {
           $345 = (_s_cmp($c4,18449,1,2)|0);
           $346 = ($345|0)==(0);
           if (!($346)) {
            $347 = (_s_cmp($c4,18452,1,2)|0);
            $348 = ($347|0)==(0);
            if (!($348)) {
             $349 = (_s_cmp($c4,18455,1,2)|0);
             $350 = ($349|0)==(0);
             if (!($350)) {
              $351 = (_s_cmp($c4,18458,1,2)|0);
              $352 = ($351|0)==(0);
              if ($352) {
               break;
              }
              $353 = (_s_cmp($c4,18461,1,2)|0);
              $354 = ($353|0)==(0);
              if ($354) {
               break;
              }
              $355 = (_s_cmp($c4,18464,1,2)|0);
              $356 = ($355|0)==(0);
              if (!($356)) {
               break L236;
              }
             }
            }
           }
          }
         } while(0);
         $nbmin = 2;
         break;
        }
        $357 = HEAP8[$c3>>0]|0;
        $358 = $357&255;
        $359 = ($358|0)==(77);
        if (!($359)) {
         break;
        }
        $360 = (_s_cmp($c4,18446,1,2)|0);
        $361 = ($360|0)==(0);
        do {
         if (!($361)) {
          $362 = (_s_cmp($c4,18449,1,2)|0);
          $363 = ($362|0)==(0);
          if (!($363)) {
           $364 = (_s_cmp($c4,18452,1,2)|0);
           $365 = ($364|0)==(0);
           if (!($365)) {
            $366 = (_s_cmp($c4,18455,1,2)|0);
            $367 = ($366|0)==(0);
            if (!($367)) {
             $368 = (_s_cmp($c4,18458,1,2)|0);
             $369 = ($368|0)==(0);
             if ($369) {
              break;
             }
             $370 = (_s_cmp($c4,18461,1,2)|0);
             $371 = ($370|0)==(0);
             if ($371) {
              break;
             }
             $372 = (_s_cmp($c4,18464,1,2)|0);
             $373 = ($372|0)==(0);
             if (!($373)) {
              break L236;
             }
            }
           }
          }
         }
        } while(0);
        $nbmin = 2;
        break;
       }
      }
      $374 = $cname;
      $375 = ($374|0)!=(0);
      if ($375) {
       $376 = (_s_cmp($c2,18467,1,2)|0);
       $377 = ($376|0)==(0);
       if ($377) {
        $378 = HEAP8[$c3>>0]|0;
        $379 = $378&255;
        $380 = ($379|0)==(71);
        if ($380) {
         $381 = (_s_cmp($c4,18446,1,2)|0);
         $382 = ($381|0)==(0);
         do {
          if (!($382)) {
           $383 = (_s_cmp($c4,18449,1,2)|0);
           $384 = ($383|0)==(0);
           if (!($384)) {
            $385 = (_s_cmp($c4,18452,1,2)|0);
            $386 = ($385|0)==(0);
            if (!($386)) {
             $387 = (_s_cmp($c4,18455,1,2)|0);
             $388 = ($387|0)==(0);
             if (!($388)) {
              $389 = (_s_cmp($c4,18458,1,2)|0);
              $390 = ($389|0)==(0);
              if ($390) {
               break;
              }
              $391 = (_s_cmp($c4,18461,1,2)|0);
              $392 = ($391|0)==(0);
              if ($392) {
               break;
              }
              $393 = (_s_cmp($c4,18464,1,2)|0);
              $394 = ($393|0)==(0);
              if (!($394)) {
               break L236;
              }
             }
            }
           }
          }
         } while(0);
         $nbmin = 2;
         break;
        }
        $395 = HEAP8[$c3>>0]|0;
        $396 = $395&255;
        $397 = ($396|0)==(77);
        if ($397) {
         $398 = (_s_cmp($c4,18446,1,2)|0);
         $399 = ($398|0)==(0);
         do {
          if (!($399)) {
           $400 = (_s_cmp($c4,18449,1,2)|0);
           $401 = ($400|0)==(0);
           if (!($401)) {
            $402 = (_s_cmp($c4,18452,1,2)|0);
            $403 = ($402|0)==(0);
            if (!($403)) {
             $404 = (_s_cmp($c4,18455,1,2)|0);
             $405 = ($404|0)==(0);
             if (!($405)) {
              $406 = (_s_cmp($c4,18458,1,2)|0);
              $407 = ($406|0)==(0);
              if ($407) {
               break;
              }
              $408 = (_s_cmp($c4,18461,1,2)|0);
              $409 = ($408|0)==(0);
              if ($409) {
               break;
              }
              $410 = (_s_cmp($c4,18464,1,2)|0);
              $411 = ($410|0)==(0);
              if (!($411)) {
               break L236;
              }
             }
            }
           }
          }
         } while(0);
         $nbmin = 2;
        }
       }
      }
     }
    } while(0);
    $412 = $nbmin;
    $ret_val = $412;
    $413 = $ret_val;
    $0 = $413;
    $520 = $0;
    STACKTOP = sp;return ($520|0);
    break;
   }
   default: {
    $nb = 1;
    $152 = (_s_cmp($c2,18391,1,2)|0);
    $153 = ($152|0)==(0);
    L52: do {
     if ($153) {
      $154 = (_s_cmp($c3,18394,1,3)|0);
      $155 = ($154|0)==(0);
      if ($155) {
       $nb = 64;
       break;
      }
      $156 = (_s_cmp($c3,18398,1,3)|0);
      $157 = ($156|0)==(0);
      if (!($157)) {
       $158 = (_s_cmp($c3,18402,1,3)|0);
       $159 = ($158|0)==(0);
       if (!($159)) {
        $160 = (_s_cmp($c3,18406,1,3)|0);
        $161 = ($160|0)==(0);
        if (!($161)) {
         $162 = (_s_cmp($c3,18410,1,3)|0);
         $163 = ($162|0)==(0);
         if (!($163)) {
          $164 = (_s_cmp($c3,18414,1,3)|0);
          $165 = ($164|0)==(0);
          if ($165) {
           $nb = 32;
           break;
          }
          $166 = (_s_cmp($c3,18418,1,3)|0);
          $167 = ($166|0)==(0);
          if ($167) {
           $nb = 32;
           break;
          }
          $168 = (_s_cmp($c3,18422,1,3)|0);
          $169 = ($168|0)==(0);
          if (!($169)) {
           break;
          }
          $nb = 64;
          break;
         }
        }
       }
      }
      $nb = 32;
     } else {
      $170 = (_s_cmp($c2,18426,1,2)|0);
      $171 = ($170|0)==(0);
      if ($171) {
       $172 = (_s_cmp($c3,18394,1,3)|0);
       $173 = ($172|0)==(0);
       if (!($173)) {
        break;
       }
       $nb = 64;
       break;
      }
      $174 = (_s_cmp($c2,18429,1,2)|0);
      $175 = ($174|0)==(0);
      if ($175) {
       $176 = (_s_cmp($c3,18394,1,3)|0);
       $177 = ($176|0)==(0);
       $178 = $sname;
       $179 = ($178|0)!=(0);
       if ($177) {
        $nb = 64;
        break;
       }
       if ($179) {
        $180 = (_s_cmp($c3,18432,1,3)|0);
        $181 = ($180|0)==(0);
        if ($181) {
         $nb = 32;
         break;
        }
       }
       $182 = $sname;
       $183 = ($182|0)!=(0);
       if (!($183)) {
        break;
       }
       $184 = (_s_cmp($c3,18436,1,3)|0);
       $185 = ($184|0)==(0);
       if (!($185)) {
        break;
       }
       $nb = 64;
       break;
      }
      $186 = $cname;
      $187 = ($186|0)!=(0);
      if ($187) {
       $188 = (_s_cmp($c2,18440,1,2)|0);
       $189 = ($188|0)==(0);
       if ($189) {
        $190 = (_s_cmp($c3,18394,1,3)|0);
        $191 = ($190|0)==(0);
        if ($191) {
         $nb = 64;
         break;
        }
        $192 = (_s_cmp($c3,18432,1,3)|0);
        $193 = ($192|0)==(0);
        if ($193) {
         $nb = 32;
         break;
        }
        $194 = (_s_cmp($c3,18436,1,3)|0);
        $195 = ($194|0)==(0);
        if (!($195)) {
         break;
        }
        $nb = 64;
        break;
       }
      }
      $196 = $sname;
      $197 = ($196|0)!=(0);
      if ($197) {
       $198 = (_s_cmp($c2,18443,1,2)|0);
       $199 = ($198|0)==(0);
       if ($199) {
        $200 = HEAP8[$c3>>0]|0;
        $201 = $200&255;
        $202 = ($201|0)==(71);
        if ($202) {
         $203 = (_s_cmp($c4,18446,1,2)|0);
         $204 = ($203|0)==(0);
         do {
          if (!($204)) {
           $205 = (_s_cmp($c4,18449,1,2)|0);
           $206 = ($205|0)==(0);
           if (!($206)) {
            $207 = (_s_cmp($c4,18452,1,2)|0);
            $208 = ($207|0)==(0);
            if (!($208)) {
             $209 = (_s_cmp($c4,18455,1,2)|0);
             $210 = ($209|0)==(0);
             if ($210) {
              break;
             }
             $211 = (_s_cmp($c4,18458,1,2)|0);
             $212 = ($211|0)==(0);
             if ($212) {
              break;
             }
             $213 = (_s_cmp($c4,18461,1,2)|0);
             $214 = ($213|0)==(0);
             if ($214) {
              break;
             }
             $215 = (_s_cmp($c4,18464,1,2)|0);
             $216 = ($215|0)==(0);
             if (!($216)) {
              break L52;
             }
            }
           }
          }
         } while(0);
         $nb = 32;
         break;
        }
        $217 = HEAP8[$c3>>0]|0;
        $218 = $217&255;
        $219 = ($218|0)==(77);
        if (!($219)) {
         break;
        }
        $220 = (_s_cmp($c4,18446,1,2)|0);
        $221 = ($220|0)==(0);
        do {
         if (!($221)) {
          $222 = (_s_cmp($c4,18449,1,2)|0);
          $223 = ($222|0)==(0);
          if (!($223)) {
           $224 = (_s_cmp($c4,18452,1,2)|0);
           $225 = ($224|0)==(0);
           if (!($225)) {
            $226 = (_s_cmp($c4,18455,1,2)|0);
            $227 = ($226|0)==(0);
            if ($227) {
             break;
            }
            $228 = (_s_cmp($c4,18458,1,2)|0);
            $229 = ($228|0)==(0);
            if ($229) {
             break;
            }
            $230 = (_s_cmp($c4,18461,1,2)|0);
            $231 = ($230|0)==(0);
            if ($231) {
             break;
            }
            $232 = (_s_cmp($c4,18464,1,2)|0);
            $233 = ($232|0)==(0);
            if (!($233)) {
             break L52;
            }
           }
          }
         }
        } while(0);
        $nb = 32;
        break;
       }
      }
      $234 = $cname;
      $235 = ($234|0)!=(0);
      if ($235) {
       $236 = (_s_cmp($c2,18467,1,2)|0);
       $237 = ($236|0)==(0);
       if ($237) {
        $238 = HEAP8[$c3>>0]|0;
        $239 = $238&255;
        $240 = ($239|0)==(71);
        if ($240) {
         $241 = (_s_cmp($c4,18446,1,2)|0);
         $242 = ($241|0)==(0);
         do {
          if (!($242)) {
           $243 = (_s_cmp($c4,18449,1,2)|0);
           $244 = ($243|0)==(0);
           if (!($244)) {
            $245 = (_s_cmp($c4,18452,1,2)|0);
            $246 = ($245|0)==(0);
            if ($246) {
             break;
            }
            $247 = (_s_cmp($c4,18455,1,2)|0);
            $248 = ($247|0)==(0);
            if ($248) {
             break;
            }
            $249 = (_s_cmp($c4,18458,1,2)|0);
            $250 = ($249|0)==(0);
            if ($250) {
             break;
            }
            $251 = (_s_cmp($c4,18461,1,2)|0);
            $252 = ($251|0)==(0);
            if ($252) {
             break;
            }
            $253 = (_s_cmp($c4,18464,1,2)|0);
            $254 = ($253|0)==(0);
            if (!($254)) {
             break L52;
            }
           }
          }
         } while(0);
         $nb = 32;
         break;
        }
        $255 = HEAP8[$c3>>0]|0;
        $256 = $255&255;
        $257 = ($256|0)==(77);
        if (!($257)) {
         break;
        }
        $258 = (_s_cmp($c4,18446,1,2)|0);
        $259 = ($258|0)==(0);
        do {
         if (!($259)) {
          $260 = (_s_cmp($c4,18449,1,2)|0);
          $261 = ($260|0)==(0);
          if (!($261)) {
           $262 = (_s_cmp($c4,18452,1,2)|0);
           $263 = ($262|0)==(0);
           if ($263) {
            break;
           }
           $264 = (_s_cmp($c4,18455,1,2)|0);
           $265 = ($264|0)==(0);
           if ($265) {
            break;
           }
           $266 = (_s_cmp($c4,18458,1,2)|0);
           $267 = ($266|0)==(0);
           if ($267) {
            break;
           }
           $268 = (_s_cmp($c4,18461,1,2)|0);
           $269 = ($268|0)==(0);
           if ($269) {
            break;
           }
           $270 = (_s_cmp($c4,18464,1,2)|0);
           $271 = ($270|0)==(0);
           if (!($271)) {
            break L52;
           }
          }
         }
        } while(0);
        $nb = 32;
        break;
       }
      }
      $272 = (_s_cmp($c2,18470,1,2)|0);
      $273 = ($272|0)==(0);
      if ($273) {
       $274 = (_s_cmp($c3,18394,1,3)|0);
       $275 = ($274|0)==(0);
       if (!($275)) {
        break;
       }
       $276 = $sname;
       $277 = ($276|0)!=(0);
       $278 = $7;
       $279 = HEAP32[$278>>2]|0;
       $280 = ($279|0)<=(64);
       if ($277) {
        if ($280) {
         $nb = 1;
         break;
        } else {
         $nb = 32;
         break;
        }
       } else {
        if ($280) {
         $nb = 1;
         break;
        } else {
         $nb = 32;
         break;
        }
       }
      }
      $281 = (_s_cmp($c2,18473,1,2)|0);
      $282 = ($281|0)==(0);
      if ($282) {
       $283 = (_s_cmp($c3,18394,1,3)|0);
       $284 = ($283|0)==(0);
       if (!($284)) {
        break;
       }
       $285 = $sname;
       $286 = ($285|0)!=(0);
       $287 = $5;
       $288 = HEAP32[$287>>2]|0;
       $289 = ($288|0)<=(64);
       if ($286) {
        if ($289) {
         $nb = 1;
         break;
        } else {
         $nb = 32;
         break;
        }
       } else {
        if ($289) {
         $nb = 1;
         break;
        } else {
         $nb = 32;
         break;
        }
       }
      }
      $290 = (_s_cmp($c2,18461,1,2)|0);
      $291 = ($290|0)==(0);
      if ($291) {
       $292 = (_s_cmp($c3,18422,1,3)|0);
       $293 = ($292|0)==(0);
       if (!($293)) {
        break;
       }
       $nb = 64;
       break;
      }
      $294 = (_s_cmp($c2,18476,1,2)|0);
      $295 = ($294|0)==(0);
      if ($295) {
       $296 = (_s_cmp($c3,18479,1,3)|0);
       $297 = ($296|0)==(0);
       if (!($297)) {
        break;
       }
       $nb = 64;
       break;
      }
      $298 = $sname;
      $299 = ($298|0)!=(0);
      if ($299) {
       $300 = (_s_cmp($c2,18483,1,2)|0);
       $301 = ($300|0)==(0);
       if ($301) {
        $302 = (_s_cmp($c3,18486,1,3)|0);
        $303 = ($302|0)==(0);
        if ($303) {
         $nb = 1;
        }
       }
      }
     }
    } while(0);
    $304 = $nb;
    $ret_val = $304;
    $305 = $ret_val;
    $0 = $305;
    $520 = $0;
    STACKTOP = sp;return ($520|0);
   }
   }
   break;
  }
  case 4:  {
   $ret_val = 6;
   $484 = $ret_val;
   $0 = $484;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 5:  {
   $ret_val = 2;
   $485 = $ret_val;
   $0 = $485;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 6:  {
   $486 = $4;
   $487 = HEAP32[$486>>2]|0;
   $488 = $5;
   $489 = HEAP32[$488>>2]|0;
   $490 = ($487|0)<=($489|0);
   if ($490) {
    $491 = $4;
    $492 = HEAP32[$491>>2]|0;
    $496 = $492;
   } else {
    $493 = $5;
    $494 = HEAP32[$493>>2]|0;
    $496 = $494;
   }
   $495 = (+($496|0));
   $497 = $495 * 1.6000000238418579;
   $498 = (~~(($497)));
   $ret_val = $498;
   $499 = $ret_val;
   $0 = $499;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 7:  {
   $ret_val = 1;
   $500 = $ret_val;
   $0 = $500;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 8:  {
   $ret_val = 50;
   $501 = $ret_val;
   $0 = $501;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 9:  {
   $ret_val = 25;
   $502 = $ret_val;
   $0 = $502;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 10:  {
   $ret_val = 1;
   $503 = $ret_val;
   $504 = ($503|0)==(1);
   if ($504) {
    $505 = (_ieeeck_(2692,2696,2700)|0);
    $ret_val = $505;
   }
   $506 = $ret_val;
   $0 = $506;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 11:  {
   $ret_val = 1;
   $507 = $ret_val;
   $508 = ($507|0)==(1);
   if ($508) {
    $509 = (_ieeeck_(2704,2696,2700)|0);
    $ret_val = $509;
   }
   $510 = $ret_val;
   $0 = $510;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  case 16: case 15: case 14: case 13: case 12:  {
   $511 = $1;
   $512 = $2;
   $513 = $3;
   $514 = $4;
   $515 = $5;
   $516 = $6;
   $517 = $7;
   $518 = (_iparmq_($511,$512,$513,$514,$515,$516,$517)|0);
   $ret_val = $518;
   $519 = $ret_val;
   $0 = $519;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
   break;
  }
  default: {
   $ret_val = -1;
   $14 = $ret_val;
   $0 = $14;
   $520 = $0;
   STACKTOP = sp;return ($520|0);
  }
  }
 } while(0);
 return (0)|0;
}
function _ieeeck_($ispec,$zero,$one) {
 $ispec = $ispec|0;
 $zero = $zero|0;
 $one = $one|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0.0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0.0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0;
 var $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0, $27 = 0.0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0.0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0.0, $50 = 0.0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0, $69 = 0.0, $7 = 0.0, $70 = 0, $71 = 0, $72 = 0.0, $73 = 0.0;
 var $74 = 0.0, $75 = 0.0, $76 = 0, $77 = 0.0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0.0, $90 = 0.0, $91 = 0.0;
 var $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0.0, $96 = 0.0, $97 = 0.0, $98 = 0.0, $99 = 0.0, $nan1 = 0.0, $nan2 = 0.0, $nan3 = 0.0, $nan4 = 0.0, $nan5 = 0.0, $nan6 = 0.0, $neginf = 0.0, $negzro = 0.0, $newzro = 0.0, $posinf = 0.0, $ret_val = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $ispec;
 $2 = $zero;
 $3 = $one;
 $ret_val = 1;
 $4 = $3;
 $5 = +HEAPF32[$4>>2];
 $6 = $2;
 $7 = +HEAPF32[$6>>2];
 $8 = $5 / $7;
 $posinf = $8;
 $9 = $posinf;
 $10 = $3;
 $11 = +HEAPF32[$10>>2];
 $12 = $9 <= $11;
 do {
  if ($12) {
   $ret_val = 0;
   $13 = $ret_val;
   $0 = $13;
  } else {
   $14 = $3;
   $15 = +HEAPF32[$14>>2];
   $16 = -$15;
   $17 = $2;
   $18 = +HEAPF32[$17>>2];
   $19 = $16 / $18;
   $neginf = $19;
   $20 = $neginf;
   $21 = $2;
   $22 = +HEAPF32[$21>>2];
   $23 = $20 >= $22;
   if ($23) {
    $ret_val = 0;
    $24 = $ret_val;
    $0 = $24;
    break;
   }
   $25 = $3;
   $26 = +HEAPF32[$25>>2];
   $27 = $neginf;
   $28 = $3;
   $29 = +HEAPF32[$28>>2];
   $30 = $27 + $29;
   $31 = $26 / $30;
   $negzro = $31;
   $32 = $negzro;
   $33 = $2;
   $34 = +HEAPF32[$33>>2];
   $35 = $32 != $34;
   if ($35) {
    $ret_val = 0;
    $36 = $ret_val;
    $0 = $36;
    break;
   }
   $37 = $3;
   $38 = +HEAPF32[$37>>2];
   $39 = $negzro;
   $40 = $38 / $39;
   $neginf = $40;
   $41 = $neginf;
   $42 = $2;
   $43 = +HEAPF32[$42>>2];
   $44 = $41 >= $43;
   if ($44) {
    $ret_val = 0;
    $45 = $ret_val;
    $0 = $45;
    break;
   }
   $46 = $negzro;
   $47 = $2;
   $48 = +HEAPF32[$47>>2];
   $49 = $46 + $48;
   $newzro = $49;
   $50 = $newzro;
   $51 = $2;
   $52 = +HEAPF32[$51>>2];
   $53 = $50 != $52;
   if ($53) {
    $ret_val = 0;
    $54 = $ret_val;
    $0 = $54;
    break;
   }
   $55 = $3;
   $56 = +HEAPF32[$55>>2];
   $57 = $newzro;
   $58 = $56 / $57;
   $posinf = $58;
   $59 = $posinf;
   $60 = $3;
   $61 = +HEAPF32[$60>>2];
   $62 = $59 <= $61;
   if ($62) {
    $ret_val = 0;
    $63 = $ret_val;
    $0 = $63;
    break;
   }
   $64 = $posinf;
   $65 = $neginf;
   $66 = $65 * $64;
   $neginf = $66;
   $67 = $neginf;
   $68 = $2;
   $69 = +HEAPF32[$68>>2];
   $70 = $67 >= $69;
   if ($70) {
    $ret_val = 0;
    $71 = $ret_val;
    $0 = $71;
    break;
   }
   $72 = $posinf;
   $73 = $posinf;
   $74 = $73 * $72;
   $posinf = $74;
   $75 = $posinf;
   $76 = $3;
   $77 = +HEAPF32[$76>>2];
   $78 = $75 <= $77;
   if ($78) {
    $ret_val = 0;
    $79 = $ret_val;
    $0 = $79;
    break;
   }
   $80 = $1;
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)==(0);
   if ($82) {
    $83 = $ret_val;
    $0 = $83;
    break;
   }
   $84 = $posinf;
   $85 = $neginf;
   $86 = $84 + $85;
   $nan1 = $86;
   $87 = $posinf;
   $88 = $neginf;
   $89 = $87 / $88;
   $nan2 = $89;
   $90 = $posinf;
   $91 = $posinf;
   $92 = $90 / $91;
   $nan3 = $92;
   $93 = $posinf;
   $94 = $2;
   $95 = +HEAPF32[$94>>2];
   $96 = $93 * $95;
   $nan4 = $96;
   $97 = $neginf;
   $98 = $negzro;
   $99 = $97 * $98;
   $nan5 = $99;
   $100 = $nan5;
   $101 = $100 * 0.0;
   $nan6 = $101;
   $102 = $nan1;
   $103 = $nan1;
   $104 = $102 == $103;
   if ($104) {
    $ret_val = 0;
    $105 = $ret_val;
    $0 = $105;
    break;
   }
   $106 = $nan2;
   $107 = $nan2;
   $108 = $106 == $107;
   if ($108) {
    $ret_val = 0;
    $109 = $ret_val;
    $0 = $109;
    break;
   }
   $110 = $nan3;
   $111 = $nan3;
   $112 = $110 == $111;
   if ($112) {
    $ret_val = 0;
    $113 = $ret_val;
    $0 = $113;
    break;
   }
   $114 = $nan4;
   $115 = $nan4;
   $116 = $114 == $115;
   if ($116) {
    $ret_val = 0;
    $117 = $ret_val;
    $0 = $117;
    break;
   }
   $118 = $nan5;
   $119 = $nan5;
   $120 = $118 == $119;
   if ($120) {
    $ret_val = 0;
    $121 = $ret_val;
    $0 = $121;
    break;
   }
   $122 = $nan6;
   $123 = $nan6;
   $124 = $122 == $123;
   if ($124) {
    $ret_val = 0;
    $125 = $ret_val;
    $0 = $125;
    break;
   } else {
    $126 = $ret_val;
    $0 = $126;
    break;
   }
  }
 } while(0);
 $127 = $0;
 STACKTOP = sp;return ($127|0);
}
function _iparmq_($ispec,$name__,$opts,$n,$ilo,$ihi,$lwork) {
 $ispec = $ispec|0;
 $name__ = $name__|0;
 $opts = $opts|0;
 $n = $n|0;
 $ilo = $ilo|0;
 $ihi = $ihi|0;
 $lwork = $lwork|0;
 var $$ = 0, $$$ = 0, $$$1 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $i__1 = 0, $i__2 = 0, $nh = 0, $ns = 0, $r__1 = 0, $ret_val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $r__1 = sp + 8|0;
 $0 = $ispec;
 $1 = $name__;
 $2 = $opts;
 $3 = $n;
 $4 = $ilo;
 $5 = $ihi;
 $6 = $lwork;
 $7 = $0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(15);
 if ($9) {
  label = 4;
 } else {
  $10 = $0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(13);
  if ($12) {
   label = 4;
  } else {
   $13 = $0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(16);
   if ($15) {
    label = 4;
   }
  }
 }
 if ((label|0) == 4) {
  $16 = $5;
  $17 = HEAP32[$16>>2]|0;
  $18 = $4;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) - ($19))|0;
  $21 = (($20) + 1)|0;
  $nh = $21;
  $ns = 2;
  $22 = $nh;
  $23 = ($22|0)>=(30);
  $$ = $23 ? 4 : 2;
  $ns = $$;
  $24 = $nh;
  $25 = ($24|0)>=(60);
  $$$ = $25 ? 10 : $$;
  $ns = $$$;
  $26 = $nh;
  $27 = ($26|0)>=(150);
  if ($27) {
   $28 = $nh;
   $29 = (+($28|0));
   $30 = $29;
   $31 = (+Math_log((+$30)));
   $32 = (+Math_log(2.0));
   $33 = $31 / $32;
   $34 = $33;
   HEAPF32[$r__1>>2] = $34;
   $i__1 = 10;
   $35 = $nh;
   $36 = (_i_nint($r__1)|0);
   $37 = (($35|0) / ($36|0))&-1;
   $i__2 = $37;
   $38 = $i__1;
   $39 = $i__2;
   $40 = ($38|0)>=($39|0);
   $41 = $i__1;
   $42 = $i__2;
   $43 = $40 ? $41 : $42;
   $ns = $43;
  }
  $44 = $nh;
  $45 = ($44|0)>=(590);
  if ($45) {
   $ns = 64;
  }
  $46 = $nh;
  $47 = ($46|0)>=(3000);
  if ($47) {
   $ns = 128;
  }
  $48 = $nh;
  $49 = ($48|0)>=(6000);
  if ($49) {
   $ns = 256;
  }
  $i__1 = 2;
  $50 = $ns;
  $51 = $ns;
  $52 = (($51|0) % 2)&-1;
  $53 = (($50) - ($52))|0;
  $i__2 = $53;
  $54 = $i__1;
  $55 = $i__2;
  $56 = ($54|0)>=($55|0);
  $57 = $i__1;
  $58 = $i__2;
  $59 = $56 ? $57 : $58;
  $ns = $59;
 }
 $60 = $0;
 $61 = HEAP32[$60>>2]|0;
 $62 = ($61|0)==(12);
 if ($62) {
  $ret_val = 75;
  $85 = $ret_val;
  STACKTOP = sp;return ($85|0);
 }
 $63 = $0;
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)==(14);
 if ($65) {
  $ret_val = 14;
  $85 = $ret_val;
  STACKTOP = sp;return ($85|0);
 }
 $66 = $0;
 $67 = HEAP32[$66>>2]|0;
 $68 = ($67|0)==(15);
 if ($68) {
  $69 = $ns;
  $ret_val = $69;
  $85 = $ret_val;
  STACKTOP = sp;return ($85|0);
 }
 $70 = $0;
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)==(13);
 if ($72) {
  $73 = $nh;
  $74 = ($73|0)<=(500);
  $75 = $ns;
  if ($74) {
   $ret_val = $75;
   $85 = $ret_val;
   STACKTOP = sp;return ($85|0);
  } else {
   $76 = ($75*3)|0;
   $77 = (($76|0) / 2)&-1;
   $ret_val = $77;
   $85 = $ret_val;
   STACKTOP = sp;return ($85|0);
  }
 } else {
  $78 = $0;
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)==(16);
  if ($80) {
   $ret_val = 0;
   $81 = $ns;
   $82 = ($81|0)>=(14);
   $$1 = $82 ? 1 : 0;
   $ret_val = $$1;
   $83 = $ns;
   $84 = ($83|0)>=(14);
   $$$1 = $84 ? 2 : $$1;
   $ret_val = $$$1;
   $85 = $ret_val;
   STACKTOP = sp;return ($85|0);
  } else {
   $ret_val = -1;
   $85 = $ret_val;
   STACKTOP = sp;return ($85|0);
  }
 }
 return (0)|0;
}
function _lsame_($ca,$cb) {
 $ca = $ca|0;
 $cb = $cb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $inta = 0, $intb = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0;
 var $or$cond25 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $ret_val = 0, $zcode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $ca;
 $2 = $cb;
 $3 = $1;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $2;
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($5|0)==($8|0);
 $10 = $9&1;
 $ret_val = $10;
 $11 = $ret_val;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $ret_val;
  $0 = $13;
  $89 = $0;
  STACKTOP = sp;return ($89|0);
 }
 $zcode = 90;
 $14 = $1;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $inta = $16;
 $17 = $2;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $intb = $19;
 $20 = $zcode;
 $21 = ($20|0)==(90);
 $22 = $zcode;
 $23 = ($22|0)==(122);
 $or$cond = $21 | $23;
 do {
  if ($or$cond) {
   $24 = $inta;
   $25 = ($24|0)>=(97);
   $26 = $inta;
   $27 = ($26|0)<=(122);
   $or$cond3 = $25 & $27;
   if ($or$cond3) {
    $28 = $inta;
    $29 = (($28) + -32)|0;
    $inta = $29;
   }
   $30 = $intb;
   $31 = ($30|0)>=(97);
   $32 = $intb;
   $33 = ($32|0)<=(122);
   $or$cond5 = $31 & $33;
   if ($or$cond5) {
    $34 = $intb;
    $35 = (($34) + -32)|0;
    $intb = $35;
   }
  } else {
   $36 = $zcode;
   $37 = ($36|0)==(233);
   $38 = $zcode;
   $39 = ($38|0)==(169);
   $or$cond7 = $37 | $39;
   if (!($or$cond7)) {
    $68 = $zcode;
    $69 = ($68|0)==(218);
    $70 = $zcode;
    $71 = ($70|0)==(250);
    $or$cond21 = $69 | $71;
    if (!($or$cond21)) {
     break;
    }
    $72 = $inta;
    $73 = ($72|0)>=(225);
    $74 = $inta;
    $75 = ($74|0)<=(250);
    $or$cond23 = $73 & $75;
    if ($or$cond23) {
     $76 = $inta;
     $77 = (($76) + -32)|0;
     $inta = $77;
    }
    $78 = $intb;
    $79 = ($78|0)>=(225);
    $80 = $intb;
    $81 = ($80|0)<=(250);
    $or$cond25 = $79 & $81;
    if (!($or$cond25)) {
     break;
    }
    $82 = $intb;
    $83 = (($82) + -32)|0;
    $intb = $83;
    break;
   }
   $40 = $inta;
   $41 = ($40|0)>=(129);
   $42 = $inta;
   $43 = ($42|0)<=(137);
   $or$cond9 = $41 & $43;
   if ($or$cond9) {
    label = 12;
   } else {
    $44 = $inta;
    $45 = ($44|0)>=(145);
    $46 = $inta;
    $47 = ($46|0)<=(153);
    $or$cond11 = $45 & $47;
    if ($or$cond11) {
     label = 12;
    } else {
     $48 = $inta;
     $49 = ($48|0)>=(162);
     $50 = $inta;
     $51 = ($50|0)<=(169);
     $or$cond13 = $49 & $51;
     if ($or$cond13) {
      label = 12;
     }
    }
   }
   if ((label|0) == 12) {
    $52 = $inta;
    $53 = (($52) + 64)|0;
    $inta = $53;
   }
   $54 = $intb;
   $55 = ($54|0)>=(129);
   $56 = $intb;
   $57 = ($56|0)<=(137);
   $or$cond15 = $55 & $57;
   if (!($or$cond15)) {
    $58 = $intb;
    $59 = ($58|0)>=(145);
    $60 = $intb;
    $61 = ($60|0)<=(153);
    $or$cond17 = $59 & $61;
    if (!($or$cond17)) {
     $62 = $intb;
     $63 = ($62|0)>=(162);
     $64 = $intb;
     $65 = ($64|0)<=(169);
     $or$cond19 = $63 & $65;
     if (!($or$cond19)) {
      break;
     }
    }
   }
   $66 = $intb;
   $67 = (($66) + 64)|0;
   $intb = $67;
  }
 } while(0);
 $84 = $inta;
 $85 = $intb;
 $86 = ($84|0)==($85|0);
 $87 = $86&1;
 $ret_val = $87;
 $88 = $ret_val;
 $0 = $88;
 $89 = $0;
 STACKTOP = sp;return ($89|0);
}
function _dgetf2_($m,$n,$a,$lda,$ipiv,$info) {
 $m = $m|0;
 $n = $n|0;
 $a = $a|0;
 $lda = $lda|0;
 $ipiv = $ipiv|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0.0, $123 = 0.0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0.0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0.0, $168 = 0.0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0, $a_offset = 0, $d__1 = 0, $i__ = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, $j = 0, $jp = 0, $sfmin = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__1 = sp + 36|0;
 $i__2 = sp + 32|0;
 $i__3 = sp + 28|0;
 $d__1 = sp + 8|0;
 $1 = $m;
 $2 = $n;
 $3 = $a;
 $4 = $lda;
 $5 = $ipiv;
 $6 = $info;
 $7 = $4;
 $8 = HEAP32[$7>>2]|0;
 $a_dim1 = $8;
 $9 = $a_dim1;
 $10 = (1 + ($9))|0;
 $a_offset = $10;
 $11 = $a_offset;
 $12 = $3;
 $13 = (0 - ($11))|0;
 $14 = (($12) + ($13<<3)|0);
 $3 = $14;
 $15 = $5;
 $16 = ((($15)) + -4|0);
 $5 = $16;
 $17 = $6;
 HEAP32[$17>>2] = 0;
 $18 = $1;
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)<(0);
 do {
  if ($20) {
   $21 = $6;
   HEAP32[$21>>2] = -1;
  } else {
   $22 = $2;
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)<(0);
   if ($24) {
    $25 = $6;
    HEAP32[$25>>2] = -2;
    break;
   }
   $26 = $4;
   $27 = HEAP32[$26>>2]|0;
   $28 = $1;
   $29 = HEAP32[$28>>2]|0;
   $30 = (1)>=($29|0);
   if ($30) {
    $34 = 1;
   } else {
    $31 = $1;
    $32 = HEAP32[$31>>2]|0;
    $34 = $32;
   }
   $33 = ($27|0)<($34|0);
   if ($33) {
    $35 = $6;
    HEAP32[$35>>2] = -4;
   }
  }
 } while(0);
 $36 = $6;
 $37 = HEAP32[$36>>2]|0;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = $6;
  $40 = HEAP32[$39>>2]|0;
  $41 = (0 - ($40))|0;
  HEAP32[$i__1>>2] = $41;
  (_xerbla_(18490,$i__1)|0);
  $0 = 0;
  $225 = $0;
  STACKTOP = sp;return ($225|0);
 }
 $42 = $1;
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if (!($44)) {
  $45 = $2;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)==(0);
  if (!($47)) {
   $48 = (+_dlamch_(18497));
   $sfmin = $48;
   $49 = $1;
   $50 = HEAP32[$49>>2]|0;
   $51 = $2;
   $52 = HEAP32[$51>>2]|0;
   $53 = ($50|0)<=($52|0);
   if ($53) {
    $54 = $1;
    $55 = HEAP32[$54>>2]|0;
    $58 = $55;
   } else {
    $56 = $2;
    $57 = HEAP32[$56>>2]|0;
    $58 = $57;
   }
   HEAP32[$i__1>>2] = $58;
   $j = 1;
   while(1) {
    $59 = $j;
    $60 = HEAP32[$i__1>>2]|0;
    $61 = ($59|0)<=($60|0);
    if (!($61)) {
     break;
    }
    $62 = $1;
    $63 = HEAP32[$62>>2]|0;
    $64 = $j;
    $65 = (($63) - ($64))|0;
    $66 = (($65) + 1)|0;
    HEAP32[$i__2>>2] = $66;
    $67 = $j;
    $68 = (($67) - 1)|0;
    $69 = $j;
    $70 = $j;
    $71 = $a_dim1;
    $72 = Math_imul($70, $71)|0;
    $73 = (($69) + ($72))|0;
    $74 = $3;
    $75 = (($74) + ($73<<3)|0);
    $76 = (_idamax_($i__2,$75,2708)|0);
    $77 = (($68) + ($76))|0;
    $jp = $77;
    $78 = $jp;
    $79 = $j;
    $80 = $5;
    $81 = (($80) + ($79<<2)|0);
    HEAP32[$81>>2] = $78;
    $82 = $jp;
    $83 = $j;
    $84 = $a_dim1;
    $85 = Math_imul($83, $84)|0;
    $86 = (($82) + ($85))|0;
    $87 = $3;
    $88 = (($87) + ($86<<3)|0);
    $89 = +HEAPF64[$88>>3];
    $90 = $89 != 0.0;
    L26: do {
     if ($90) {
      $91 = $jp;
      $92 = $j;
      $93 = ($91|0)!=($92|0);
      if ($93) {
       $94 = $2;
       $95 = $j;
       $96 = $a_dim1;
       $97 = (($95) + ($96))|0;
       $98 = $3;
       $99 = (($98) + ($97<<3)|0);
       $100 = $4;
       $101 = $jp;
       $102 = $a_dim1;
       $103 = (($101) + ($102))|0;
       $104 = $3;
       $105 = (($104) + ($103<<3)|0);
       $106 = $4;
       (_dswap_($94,$99,$100,$105,$106)|0);
      }
      $107 = $j;
      $108 = $1;
      $109 = HEAP32[$108>>2]|0;
      $110 = ($107|0)<($109|0);
      if ($110) {
       $111 = $j;
       $112 = $j;
       $113 = $a_dim1;
       $114 = Math_imul($112, $113)|0;
       $115 = (($111) + ($114))|0;
       $116 = $3;
       $117 = (($116) + ($115<<3)|0);
       $118 = +HEAPF64[$117>>3];
       HEAPF64[$d__1>>3] = $118;
       $119 = +HEAPF64[$d__1>>3];
       $120 = $119 >= 0.0;
       $121 = +HEAPF64[$d__1>>3];
       $122 = -$121;
       $123 = $120 ? $121 : $122;
       $124 = $sfmin;
       $125 = $123 >= $124;
       $126 = $1;
       $127 = HEAP32[$126>>2]|0;
       $128 = $j;
       $129 = (($127) - ($128))|0;
       HEAP32[$i__2>>2] = $129;
       if ($125) {
        $130 = $j;
        $131 = $j;
        $132 = $a_dim1;
        $133 = Math_imul($131, $132)|0;
        $134 = (($130) + ($133))|0;
        $135 = $3;
        $136 = (($135) + ($134<<3)|0);
        $137 = +HEAPF64[$136>>3];
        $138 = 1.0 / $137;
        HEAPF64[$d__1>>3] = $138;
        $139 = $j;
        $140 = (($139) + 1)|0;
        $141 = $j;
        $142 = $a_dim1;
        $143 = Math_imul($141, $142)|0;
        $144 = (($140) + ($143))|0;
        $145 = $3;
        $146 = (($145) + ($144<<3)|0);
        (_dscal_($i__2,$d__1,$146,2708)|0);
        break;
       }
       $i__ = 1;
       while(1) {
        $147 = $i__;
        $148 = HEAP32[$i__2>>2]|0;
        $149 = ($147|0)<=($148|0);
        if (!($149)) {
         break L26;
        }
        $150 = $j;
        $151 = $j;
        $152 = $a_dim1;
        $153 = Math_imul($151, $152)|0;
        $154 = (($150) + ($153))|0;
        $155 = $3;
        $156 = (($155) + ($154<<3)|0);
        $157 = +HEAPF64[$156>>3];
        $158 = $j;
        $159 = $i__;
        $160 = (($158) + ($159))|0;
        $161 = $j;
        $162 = $a_dim1;
        $163 = Math_imul($161, $162)|0;
        $164 = (($160) + ($163))|0;
        $165 = $3;
        $166 = (($165) + ($164<<3)|0);
        $167 = +HEAPF64[$166>>3];
        $168 = $167 / $157;
        HEAPF64[$166>>3] = $168;
        $169 = $i__;
        $170 = (($169) + 1)|0;
        $i__ = $170;
       }
      }
     } else {
      $171 = $6;
      $172 = HEAP32[$171>>2]|0;
      $173 = ($172|0)==(0);
      if ($173) {
       $174 = $j;
       $175 = $6;
       HEAP32[$175>>2] = $174;
      }
     }
    } while(0);
    $176 = $j;
    $177 = $1;
    $178 = HEAP32[$177>>2]|0;
    $179 = $2;
    $180 = HEAP32[$179>>2]|0;
    $181 = ($178|0)<=($180|0);
    if ($181) {
     $182 = $1;
     $183 = HEAP32[$182>>2]|0;
     $187 = $183;
    } else {
     $184 = $2;
     $185 = HEAP32[$184>>2]|0;
     $187 = $185;
    }
    $186 = ($176|0)<($187|0);
    if ($186) {
     $188 = $1;
     $189 = HEAP32[$188>>2]|0;
     $190 = $j;
     $191 = (($189) - ($190))|0;
     HEAP32[$i__2>>2] = $191;
     $192 = $2;
     $193 = HEAP32[$192>>2]|0;
     $194 = $j;
     $195 = (($193) - ($194))|0;
     HEAP32[$i__3>>2] = $195;
     $196 = $j;
     $197 = (($196) + 1)|0;
     $198 = $j;
     $199 = $a_dim1;
     $200 = Math_imul($198, $199)|0;
     $201 = (($197) + ($200))|0;
     $202 = $3;
     $203 = (($202) + ($201<<3)|0);
     $204 = $j;
     $205 = $j;
     $206 = (($205) + 1)|0;
     $207 = $a_dim1;
     $208 = Math_imul($206, $207)|0;
     $209 = (($204) + ($208))|0;
     $210 = $3;
     $211 = (($210) + ($209<<3)|0);
     $212 = $4;
     $213 = $j;
     $214 = (($213) + 1)|0;
     $215 = $j;
     $216 = (($215) + 1)|0;
     $217 = $a_dim1;
     $218 = Math_imul($216, $217)|0;
     $219 = (($214) + ($218))|0;
     $220 = $3;
     $221 = (($220) + ($219<<3)|0);
     $222 = $4;
     (_dger_($i__2,$i__3,896,$203,2708,$211,$212,$221,$222)|0);
    }
    $223 = $j;
    $224 = (($223) + 1)|0;
    $j = $224;
   }
   $0 = 0;
   $225 = $0;
   STACKTOP = sp;return ($225|0);
  }
 }
 $0 = 0;
 $225 = $0;
 STACKTOP = sp;return ($225|0);
}
function _pow_di($ap,$bp) {
 $ap = $ap|0;
 $bp = $bp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0.0, $26 = 0.0;
 var $27 = 0.0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pow = 0.0, $u = 0, $x = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ap;
 $1 = $bp;
 $pow = 1.0;
 $2 = $0;
 $3 = +HEAPF64[$2>>3];
 $x = $3;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $n = $5;
 $6 = $n;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $27 = $pow;
  STACKTOP = sp;return (+$27);
 }
 $8 = $n;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = $n;
  $11 = (0 - ($10))|0;
  $n = $11;
  $12 = $x;
  $13 = 1.0 / $12;
  $x = $13;
 }
 $14 = $n;
 $u = $14;
 while(1) {
  $15 = $u;
  $16 = $15 & 1;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = $x;
   $19 = $pow;
   $20 = $19 * $18;
   $pow = $20;
  }
  $21 = $u;
  $22 = $21 >>> 1;
  $u = $22;
  $23 = ($22|0)!=(0);
  if (!($23)) {
   break;
  }
  $24 = $x;
  $25 = $x;
  $26 = $25 * $24;
  $x = $26;
 }
 $27 = $pow;
 STACKTOP = sp;return (+$27);
}
function _i_nint($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0, $13 = 0.0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $x;
 $1 = $0;
 $2 = +HEAPF32[$1>>2];
 $3 = $2 >= 0.0;
 $4 = $0;
 $5 = +HEAPF32[$4>>2];
 $6 = $5;
 if ($3) {
  $7 = $6 + 0.5;
  $8 = (+Math_floor((+$7)));
  $13 = $8;
 } else {
  $9 = 0.5 - $6;
  $10 = (+Math_floor((+$9)));
  $11 = -$10;
  $13 = $11;
 }
 $12 = (~~(($13)));
 STACKTOP = sp;return ($12|0);
}
/*
 * Javantea: This is absolutely ridiculous. FIXME FIXME FIXME!!!
 */
function _s_cmp($a0,$b0,$la,$lb) {
 $a0 = $a0|0;
 $b0 = $b0|0;
 $la = $la|0;
 $lb = $lb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $a = 0, $aend = 0, $b = 0, $bend = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a0;
 $2 = $b0;
 $3 = $la;
 $4 = $lb;
 $5 = $1;
 $a = $5;
 $6 = $2;
 $b = $6;
 $7 = $a;
 $8 = $3;
 $9 = (($7) + ($8)|0);
 $aend = $9;
 $10 = $b;
 $11 = $4;
 $12 = (($10) + ($11)|0);
 $bend = $12;
 $13 = $3;
 $14 = $4;
 $15 = ($13|0)<=($14|0);
 L1: do {
  if ($15) {
   while(1) {
    $16 = $a;
    $17 = $aend;
    $18 = ($16>>>0)<($17>>>0);
    if (!($18)) {
     break;
    }
    $19 = $a;
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    $22 = $b;
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
	//console.log("maybe comparing: " + String.fromCharCode($20) + ", " + String.fromCharCode($23) + "...");
    $25 = ($21|0)!=($24|0);
    $26 = $a;
    if ($25) {
     //console.log("returning label 4");
     label = 4;
     break;
    }
    $33 = ((($26)) + 1|0);
    $a = $33;
    $34 = $b;
    $35 = ((($34)) + 1|0);
    $b = $35;
   }
   if ((label|0) == 4) {
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = $b;
    $30 = HEAP8[$29>>0]|0;
    $31 = $30&255;
    $32 = (($28) - ($31))|0;
    $0 = $32;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   while(1) {
    $36 = $b;
    $37 = $bend;
    $38 = ($36>>>0)<($37>>>0);
    if (!($38)) {
     break L1;
    }
    $39 = $b;
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = ($41|0)!=(32);
    $43 = $b;
    if ($42) {
     break;
    }
    $47 = ((($43)) + 1|0);
    $b = $47;
   }
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = (32 - ($45))|0;
   $0 = $46;
   $80 = $0;
   STACKTOP = sp;return ($80|0);
  } else {
   while(1) {
    $48 = $b;
    $49 = $bend;
    $50 = ($48>>>0)<($49>>>0);
    if (!($50)) {
     break;
    }
    $51 = $a;
    $52 = HEAP8[$51>>0]|0;
    $53 = $52&255;
    $54 = $b;
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
	//console.log("maybe comparing2: " + String.fromCharCode($53) + ", " + String.fromCharCode($56) + "...");
    $57 = ($53|0)==($56|0);
    $58 = $a;
    if (!($57)) {
     label = 13;
     break;
    }
    $59 = ((($58)) + 1|0);
    $a = $59;
    $60 = $b;
    $61 = ((($60)) + 1|0);
    $b = $61;
   }
   if ((label|0) == 13) {
    $62 = HEAP8[$58>>0]|0;
    $63 = $62&255;
    $64 = $b;
    $65 = HEAP8[$64>>0]|0;
    $66 = $65&255;
    $67 = (($63) - ($66))|0;
    $0 = $67;
    $80 = $0;
    STACKTOP = sp;return ($80|0);
   }
   while(1) {
    $68 = $a;
    $69 = $aend;
    $70 = ($68>>>0)<($69>>>0);
    if (!($70)) {
     break L1;
    }
    $71 = $a;
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = ($73|0)!=(32);
    $75 = $a;
    if ($74) {
     break;
    }
    $79 = ((($75)) + 1|0);
    $a = $79;
   }
   $76 = HEAP8[$75>>0]|0;
   $77 = $76&255;
   $78 = (($77) - 32)|0;
   $0 = $78;
   $80 = $0;
   //console.log("a return " + $80);
   // FIXME: This is a huge hack. No idea why it works.
   STACKTOP = sp;return 0;//($80|0);
  }
 } while(0);
 $0 = 0;
 $80 = $0;
 //console.log("last return " + $80);
 STACKTOP = sp;return ($80|0);
}
function _s_copy($a,$b,$la,$lb) {
 $a = $a|0;
 $b = $b|0;
 $la = $la|0;
 $lb = $lb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $aend = 0, $bend = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $b;
 $2 = $la;
 $3 = $lb;
 $4 = $0;
 $5 = $2;
 $6 = (($4) + ($5)|0);
 $aend = $6;
 $7 = $2;
 $8 = $3;
 $9 = ($7|0)<=($8|0);
 if ($9) {
  $10 = $0;
  $11 = $1;
  $12 = ($10>>>0)<=($11>>>0);
  if (!($12)) {
   $13 = $0;
   $14 = $1;
   $15 = $2;
   $16 = (($14) + ($15)|0);
   $17 = ($13>>>0)>=($16>>>0);
   if (!($17)) {
    $26 = $2;
    $27 = $1;
    $28 = (($27) + ($26)|0);
    $1 = $28;
    while(1) {
     $29 = $0;
     $30 = $aend;
     $31 = ($29>>>0)<($30>>>0);
     if (!($31)) {
      break;
     }
     $32 = $1;
     $33 = ((($32)) + -1|0);
     $1 = $33;
     $34 = HEAP8[$33>>0]|0;
     $35 = $aend;
     $36 = ((($35)) + -1|0);
     $aend = $36;
     HEAP8[$36>>0] = $34;
    }
    STACKTOP = sp;return;
   }
  }
  while(1) {
   $18 = $0;
   $19 = $aend;
   $20 = ($18>>>0)<($19>>>0);
   if (!($20)) {
    break;
   }
   $21 = $1;
   $22 = ((($21)) + 1|0);
   $1 = $22;
   $23 = HEAP8[$21>>0]|0;
   $24 = $0;
   $25 = ((($24)) + 1|0);
   $0 = $25;
   HEAP8[$24>>0] = $23;
  }
  STACKTOP = sp;return;
 }
 $37 = $1;
 $38 = $3;
 $39 = (($37) + ($38)|0);
 $bend = $39;
 $40 = $0;
 $41 = $1;
 $42 = ($40>>>0)<=($41>>>0);
 if ($42) {
  label = 11;
 } else {
  $43 = $0;
  $44 = $bend;
  $45 = ($43>>>0)>=($44>>>0);
  if ($45) {
   label = 11;
  } else {
   $54 = $3;
   $55 = $0;
   $56 = (($55) + ($54)|0);
   $0 = $56;
   while(1) {
    $57 = $1;
    $58 = $bend;
    $59 = ($57>>>0)<($58>>>0);
    if (!($59)) {
     break;
    }
    $60 = $bend;
    $61 = ((($60)) + -1|0);
    $bend = $61;
    $62 = HEAP8[$61>>0]|0;
    $63 = $0;
    $64 = ((($63)) + -1|0);
    $0 = $64;
    HEAP8[$64>>0] = $62;
   }
   $65 = $3;
   $66 = $0;
   $67 = (($66) + ($65)|0);
   $0 = $67;
  }
 }
 L22: do {
  if ((label|0) == 11) {
   while(1) {
    label = 0;
    $46 = $1;
    $47 = $bend;
    $48 = ($46>>>0)<($47>>>0);
    if (!($48)) {
     break L22;
    }
    $49 = $1;
    $50 = ((($49)) + 1|0);
    $1 = $50;
    $51 = HEAP8[$49>>0]|0;
    $52 = $0;
    $53 = ((($52)) + 1|0);
    $0 = $53;
    HEAP8[$52>>0] = $51;
    label = 11;
   }
  }
 } while(0);
 while(1) {
  $68 = $0;
  $69 = $aend;
  $70 = ($68>>>0)<($69>>>0);
  if (!($70)) {
   break;
  }
  $71 = $0;
  $72 = ((($71)) + 1|0);
  $0 = $72;
  HEAP8[$71>>0] = 32;
 }
 STACKTOP = sp;return;
}
function _do_lio($type,$number,$ptr,$len) {
 $type = $type|0;
 $number = $number|0;
 $ptr = $ptr|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $type;
 $1 = $number;
 $2 = $ptr;
 $3 = $len;
 $4 = HEAP32[12652>>2]|0;
 $5 = $1;
 $6 = $2;
 $7 = $3;
 $8 = $0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (FUNCTION_TABLE_iiiii[$4 & 15]($5,$6,$7,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _pars_f($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $s;
 HEAP32[2712>>2] = 0;
 HEAP32[2716>>2] = 0;
 HEAP32[2720>>2] = 0;
 $2 = $1;
 $3 = (_f_s($2,0)|0);
 $4 = ($3|0)==(0|0);
 if ($4) {
  $0 = -1;
 } else {
  $0 = 0;
 }
 $5 = $0;
 STACKTOP = sp;return ($5|0);
}
function _do_fio($number,$ptr,$len) {
 $number = $number|0;
 $ptr = $ptr|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $n = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $1 = $number;
 $2 = $ptr;
 $3 = $len;
 $i = 0;
 L1: while(1) {
  $4 = $i;
  $5 = $1;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   label = 41;
   break;
  }
  L3: while(1) {
   $8 = HEAP32[2712>>2]|0;
   $9 = (2820 + ($8<<4)|0);
   $p = $9;
   $10 = HEAP32[$9>>2]|0;
   $11 = (_type_f($10)|0);
   do {
    switch ($11|0) {
    case 9:  {
     $20 = HEAP32[12620>>2]|0;
     $21 = $p;
     $22 = (FUNCTION_TABLE_ii[$20 & 15]($21)|0);
     $23 = ($22|0)!=(0);
     $24 = HEAP32[2712>>2]|0;
     $25 = (($24) + 1)|0;
     HEAP32[2712>>2] = $25;
     if ($23) {
      continue L3;
     } else {
      break L3;
     }
     break;
    }
    case 8:  {
     $26 = HEAP32[2724>>2]|0;
     $27 = (2728 + ($26<<2)|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ($28|0)<=(0);
     if (!($29)) {
      label = 11;
      break L3;
     }
     $30 = HEAP32[2724>>2]|0;
     $31 = (($30) + -1)|0;
     HEAP32[2724>>2] = $31;
     $32 = HEAP32[2712>>2]|0;
     $33 = (($32) + 1)|0;
     HEAP32[2712>>2] = $33;
     continue L3;
     break;
    }
    case 6:  {
     $60 = $p;
     $61 = ((($60)) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[2724>>2]|0;
     $64 = (($63) + 1)|0;
     HEAP32[2724>>2] = $64;
     $65 = (2728 + ($64<<2)|0);
     HEAP32[$65>>2] = $62;
     $66 = HEAP32[2712>>2]|0;
     $67 = (($66) + 1)|0;
     HEAP32[2712>>2] = $67;
     continue L3;
     break;
    }
    case 1:  {
     $68 = $p;
     $69 = ((($68)) + 4|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = HEAP32[2772>>2]|0;
     $72 = (($71) + 1)|0;
     HEAP32[2772>>2] = $72;
     $73 = (2776 + ($72<<2)|0);
     HEAP32[$73>>2] = $70;
     $74 = HEAP32[2712>>2]|0;
     $75 = (($74) + 1)|0;
     HEAP32[2712>>2] = $75;
     continue L3;
     break;
    }
    case 3:  {
     $76 = HEAP32[2724>>2]|0;
     $77 = (2728 + ($76<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (($78) + -1)|0;
     HEAP32[$77>>2] = $79;
     $80 = ($79|0)<=(0);
     if ($80) {
      $81 = HEAP32[2724>>2]|0;
      $82 = (($81) + -1)|0;
      HEAP32[2724>>2] = $82;
      $83 = HEAP32[2772>>2]|0;
      $84 = (($83) + -1)|0;
      HEAP32[2772>>2] = $84;
      $85 = HEAP32[2712>>2]|0;
      $86 = (($85) + 1)|0;
      HEAP32[2712>>2] = $86;
      continue L3;
     } else {
      $87 = HEAP32[2772>>2]|0;
      $88 = (($87) + -1)|0;
      HEAP32[2772>>2] = $88;
      $89 = (2776 + ($87<<2)|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (1 + ($90))|0;
      HEAP32[2712>>2] = $91;
      continue L3;
     }
     break;
    }
    case 2:  {
     HEAP32[2724>>2] = 0;
     HEAP32[2772>>2] = 0;
     $92 = $p;
     $93 = ((($92)) + 4|0);
     $94 = HEAP32[$93>>2]|0;
     HEAP32[2712>>2] = $94;
     $95 = $2;
     $96 = ($95|0)==(0|0);
     if ($96) {
      label = 26;
      break L1;
     }
     $99 = HEAP32[2768>>2]|0;
     $100 = ($99|0)!=(0);
     if (!($100)) {
      label = 28;
      break L1;
     }
     $101 = HEAP32[12624>>2]|0;
     $102 = (FUNCTION_TABLE_i[$101 & 15]()|0);
     $n = $102;
     $103 = ($102|0)!=(0);
     if ($103) {
      label = 30;
      break L1;
     } else {
      continue L3;
     }
     break;
    }
    case 16:  {
     $105 = $2;
     $106 = ($105|0)==(0|0);
     if ($106) {
      label = 32;
      break L1;
     }
     $109 = HEAP32[2712>>2]|0;
     $110 = (($109) + 1)|0;
     HEAP32[2712>>2] = $110;
     continue L3;
     break;
    }
    case 33:  {
     HEAP32[2816>>2] = 1;
     $111 = HEAP32[2712>>2]|0;
     $112 = (($111) + 1)|0;
     HEAP32[2712>>2] = $112;
     continue L3;
     break;
    }
    case 19: case 17:  {
     HEAP32[12604>>2] = 0;
     $113 = HEAP32[2712>>2]|0;
     $114 = (($113) + 1)|0;
     HEAP32[2712>>2] = $114;
     continue L3;
     break;
    }
    case 18:  {
     HEAP32[12604>>2] = 1;
     $115 = HEAP32[2712>>2]|0;
     $116 = (($115) + 1)|0;
     HEAP32[2712>>2] = $116;
     continue L3;
     break;
    }
    case 20:  {
     $117 = $p;
     $118 = ((($117)) + 4|0);
     $119 = HEAP32[$118>>2]|0;
     HEAP32[12644>>2] = $119;
     $120 = HEAP32[2712>>2]|0;
     $121 = (($120) + 1)|0;
     HEAP32[2712>>2] = $121;
     continue L3;
     break;
    }
    case 21:  {
     HEAP32[12608>>2] = 0;
     $122 = HEAP32[2712>>2]|0;
     $123 = (($122) + 1)|0;
     HEAP32[2712>>2] = $123;
     continue L3;
     break;
    }
    case 22:  {
     HEAP32[12608>>2] = 1;
     $124 = HEAP32[2712>>2]|0;
     $125 = (($124) + 1)|0;
     HEAP32[2712>>2] = $125;
     continue L3;
     break;
    }
    default: {
     label = 4;
     break L1;
    }
    }
   } while(0);
  }
  if ((label|0) == 11) {
   label = 0;
   $34 = $2;
   $35 = ($34|0)==(0|0);
   if ($35) {
    label = 12;
    break;
   }
   $38 = HEAP32[2724>>2]|0;
   $39 = (2728 + ($38<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + -1)|0;
   HEAP32[$39>>2] = $41;
   HEAP32[2768>>2] = 1;
   $42 = HEAP32[12616>>2]|0;
   $43 = $p;
   $44 = $2;
   $45 = $3;
   $46 = (FUNCTION_TABLE_iiii[$42 & 15]($43,$44,$45)|0);
   $n = $46;
   $47 = ($46|0)>(0);
   if ($47) {
    label = 14;
    break;
   }
   $53 = $n;
   $54 = ($53|0)<(0);
   if ($54) {
    label = 16;
    break;
   }
  }
  $126 = $i;
  $127 = (($126) + 1)|0;
  $i = $127;
  $128 = $3;
  $129 = $2;
  $130 = (($129) + ($128)|0);
  $2 = $130;
 }
 if ((label|0) == 4) {
  $12 = HEAP32[12736>>2]|0;
  $13 = $p;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[12564>>2]|0;
  HEAP32[$vararg_buffer>>2] = $14;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $15;
  (_fprintf($12,18499,$vararg_buffer)|0);
  $16 = HEAP32[12596>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0);
  if ($18) {
   $19 = (___errno_location()|0);
   HEAP32[$19>>2] = 100;
  } else {
   _f__fatal(100,18530);
  }
  $0 = 100;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 12) {
  $36 = HEAP32[12592>>2]|0;
  $37 = (FUNCTION_TABLE_i[$36 & 15]()|0);
  $0 = $37;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 14) {
  $48 = HEAP32[12596>>2]|0;
  $49 = HEAP32[$48>>2]|0;
  $50 = (___errno_location()|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (_err__fl($49,$51,18537)|0);
  $0 = $52;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 16) {
  $55 = HEAP32[12596>>2]|0;
  $56 = ((($55)) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = (___errno_location()|0);
   HEAP32[$59>>2] = -1;
  } else {
   _f__fatal(-1,18537);
  }
  $0 = -1;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 26) {
  $97 = HEAP32[12592>>2]|0;
  $98 = (FUNCTION_TABLE_i[$97 & 15]()|0);
  $0 = $98;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 28) {
  $0 = 0;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 30) {
  $104 = $n;
  $0 = $104;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 32) {
  $107 = HEAP32[12592>>2]|0;
  $108 = (FUNCTION_TABLE_i[$107 & 15]()|0);
  $0 = $108;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 41) {
  $0 = 0;
  $131 = $0;
  STACKTOP = sp;return ($131|0);
 }
 return (0)|0;
}
function _en_fio() {
 var $0 = 0, $one = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $one = sp;
 HEAP32[$one>>2] = 1;
 $0 = (_do_fio($one,0,0)|0);
 STACKTOP = sp;return ($0|0);
}
function _fmt_bg() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 HEAP32[12636>>2] = 0;
 HEAP32[2712>>2] = 0;
 HEAP32[2772>>2] = 0;
 HEAP32[2724>>2] = 0;
 HEAP32[2768>>2] = 0;
 HEAP32[2776>>2] = 0;
 HEAP32[2728>>2] = 0;
 return;
}
function _f_s($s,$curloc) {
 $s = $s|0;
 $curloc = $curloc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $s;
 $2 = $curloc;
 while(1) {
  $3 = $1;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($5|0)==(32);
  $7 = $1;
  $8 = ((($7)) + 1|0);
  $1 = $8;
  if (!($6)) {
   break;
  }
 }
 $9 = HEAP8[$7>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($10|0)!=(40);
 if ($11) {
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 $12 = HEAP32[2720>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[2720>>2] = $13;
 $14 = ($12|0)==(1);
 if ($14) {
  $15 = $2;
  HEAP32[2716>>2] = $15;
 }
 $16 = $2;
 $17 = (_op_gen(1,$16,0,0)|0);
 $18 = ($17|0)<(0);
 if (!($18)) {
  $19 = $1;
  $20 = (_f_list($19)|0);
  $1 = $20;
  $21 = ($20|0)==(0|0);
  if (!($21)) {
   while(1) {
    $22 = $1;
    $23 = HEAP8[$22>>0]|0;
    $24 = $23 << 24 >> 24;
    $25 = ($24|0)==(32);
    $26 = $1;
    if (!($25)) {
     break;
    }
    $27 = ((($26)) + 1|0);
    $1 = $27;
   }
   $0 = $26;
   $28 = $0;
   STACKTOP = sp;return ($28|0);
  }
 }
 $0 = 0;
 $28 = $0;
 STACKTOP = sp;return ($28|0);
}
function _type_f($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $1;
 switch ($2|0) {
 case 1:  {
  $0 = 1;
  break;
 }
 case 2:  {
  $0 = 2;
  break;
 }
 case 3:  {
  $0 = 3;
  break;
 }
 case 6:  {
  $0 = 6;
  break;
 }
 case 14: case 13: case 15: case 12: case 11: case 5: case 4:  {
  $0 = 9;
  break;
 }
 case 36: case 35: case 28: case 27: case 26: case 25: case 24: case 29: case 34: case 32: case 31: case 30: case 10: case 7: case 23:  {
  $0 = 8;
  break;
 }
 default: {
  $3 = $1;
  $0 = $3;
 }
 }
 $4 = $0;
 STACKTOP = sp;return ($4|0);
}
function _op_gen($a,$b,$c,$d) {
 $a = $a|0;
 $b = $b|0;
 $c = $c|0;
 $d = $d|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = $a;
 $1 = $b;
 $2 = $c;
 $3 = $d;
 $4 = HEAP32[2712>>2]|0;
 $5 = (2820 + ($4<<4)|0);
 $p = $5;
 $6 = HEAP32[2712>>2]|0;
 $7 = ($6|0)>=(300);
 if ($7) {
  $8 = HEAP32[12736>>2]|0;
  (_fprintf($8,18552,$vararg_buffer)|0);
  $9 = HEAP32[12564>>2]|0;
  _sig_die($9,1);
 }
 $10 = $0;
 $11 = $p;
 HEAP32[$11>>2] = $10;
 $12 = $1;
 $13 = $p;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = $2;
 $16 = $p;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = $3;
 $19 = $p;
 $20 = ((($19)) + 8|0);
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = $18;
 $22 = HEAP32[2712>>2]|0;
 $23 = (($22) + 1)|0;
 HEAP32[2712>>2] = $23;
 STACKTOP = sp;return ($22|0);
}
function _f_list($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $s;
 while(1) {
  $2 = $1;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3 << 24 >> 24;
  $5 = ($4|0)!=(0);
  if (!($5)) {
   label = 15;
   break;
  }
  while(1) {
   $6 = $1;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)==(32);
   $10 = $1;
   if (!($9)) {
    break;
   }
   $11 = ((($10)) + 1|0);
   $1 = $11;
  }
  $12 = (_i_tem($10)|0);
  $1 = $12;
  $13 = ($12|0)==(0|0);
  if ($13) {
   label = 6;
   break;
  }
  while(1) {
   $14 = $1;
   $15 = HEAP8[$14>>0]|0;
   $16 = $15 << 24 >> 24;
   $17 = ($16|0)==(32);
   $18 = $1;
   if (!($17)) {
    break;
   }
   $19 = ((($18)) + 1|0);
   $1 = $19;
  }
  $20 = HEAP8[$18>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)==(44);
  $23 = $1;
  if ($22) {
   $24 = ((($23)) + 1|0);
   $1 = $24;
   continue;
  } else {
   $25 = HEAP8[$23>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)==(41);
   if ($27) {
    label = 12;
    break;
   } else {
    continue;
   }
  }
 }
 if ((label|0) == 6) {
  $0 = 0;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 else if ((label|0) == 12) {
  $28 = HEAP32[2720>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[2720>>2] = $29;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = HEAP32[2716>>2]|0;
   (_op_gen(2,$31,0,0)|0);
   $32 = $1;
   $33 = ((($32)) + 1|0);
   $1 = $33;
   $0 = $33;
   $36 = $0;
   STACKTOP = sp;return ($36|0);
  } else {
   (_op_gen(3,0,0,0)|0);
   $34 = $1;
   $35 = ((($34)) + 1|0);
   $1 = $35;
   $0 = $35;
   $36 = $0;
   STACKTOP = sp;return ($36|0);
  }
 }
 else if ((label|0) == 15) {
  $0 = 0;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 return (0)|0;
}
function _i_tem($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $curloc = 0, $n = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $t = sp + 8|0;
 $n = sp + 4|0;
 $1 = $s;
 $2 = $1;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(41);
 $6 = $1;
 do {
  if ($5) {
   $0 = $6;
  } else {
   $7 = (_ne_d($6,$t)|0);
   $8 = ($7|0)!=(0);
   if ($8) {
    $9 = HEAP32[$t>>2]|0;
    $0 = $9;
    break;
   }
   $10 = $1;
   $11 = (_e_d($10,$t)|0);
   $12 = ($11|0)!=(0);
   if ($12) {
    $13 = HEAP32[$t>>2]|0;
    $0 = $13;
    break;
   }
   $14 = $1;
   $15 = (_gt_num($14,$n,1)|0);
   $1 = $15;
   $16 = HEAP32[$n>>2]|0;
   $17 = (_op_gen(6,$16,0,0)|0);
   $curloc = $17;
   $18 = ($17|0)<(0);
   if ($18) {
    $0 = 0;
    break;
   } else {
    $19 = $1;
    $20 = $curloc;
    $21 = (_f_s($19,$20)|0);
    $0 = $21;
    break;
   }
  }
 } while(0);
 $22 = $0;
 STACKTOP = sp;return ($22|0);
}
function _ne_d($s,$p) {
 $s = $s|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $n = 0, $sign = 0, $sp = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $n = sp + 12|0;
 $1 = $s;
 $2 = $p;
 $sign = 0;
 $3 = $1;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 L1: do {
  switch ($5|0) {
  case 58:  {
   (_op_gen(16,0,0,0)|0);
   label = 43;
   break;
  }
  case 36:  {
   (_op_gen(33,0,0,0)|0);
   label = 43;
   break;
  }
  case 98: case 66:  {
   $6 = $1;
   $7 = ((($6)) + 1|0);
   $1 = $7;
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)==(122);
   if (!($10)) {
    $11 = $1;
    $12 = HEAP8[$11>>0]|0;
    $13 = $12 << 24 >> 24;
    $14 = ($13|0)==(90);
    if (!($14)) {
     (_op_gen(21,0,0,0)|0);
     label = 43;
     break L1;
    }
   }
   (_op_gen(22,0,0,0)|0);
   label = 43;
   break;
  }
  case 115: case 83:  {
   $15 = $1;
   $16 = ((($15)) + 1|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = ($18|0)==(115);
   do {
    if ($19) {
     label = 11;
    } else {
     $20 = $1;
     $21 = ((($20)) + 1|0);
     $22 = HEAP8[$21>>0]|0;
     $23 = $22 << 24 >> 24;
     $24 = ($23|0)==(83);
     if ($24) {
      label = 11;
     } else {
      $27 = $1;
      $28 = ((($27)) + 1|0);
      $29 = HEAP8[$28>>0]|0;
      $30 = $29 << 24 >> 24;
      $31 = ($30|0)==(112);
      if (!($31)) {
       $32 = $1;
       $33 = ((($32)) + 1|0);
       $34 = HEAP8[$33>>0]|0;
       $35 = $34 << 24 >> 24;
       $36 = ($35|0)==(80);
       if (!($36)) {
        $x = 17;
        break;
       }
      }
      $x = 18;
      $37 = $1;
      $38 = ((($37)) + 1|0);
      $1 = $38;
     }
    }
   } while(0);
   if ((label|0) == 11) {
    $x = 19;
    $25 = $1;
    $26 = ((($25)) + 1|0);
    $1 = $26;
   }
   $39 = $x;
   (_op_gen($39,0,0,0)|0);
   label = 43;
   break;
  }
  case 47:  {
   (_op_gen(5,0,0,0)|0);
   label = 43;
   break;
  }
  case 45:  {
   $sign = 1;
   label = 19;
   break;
  }
  case 43:  {
   label = 19;
   break;
  }
  case 57: case 56: case 55: case 54: case 53: case 52: case 51: case 50: case 49: case 48:  {
   label = 20;
   break;
  }
  case 39: case 34: case 2:  {
   $65 = (_op_gen(11,0,0,0)|0);
   $66 = (2820 + ($65<<4)|0);
   $sp = $66;
   $67 = $1;
   $68 = $sp;
   $69 = ((($68)) + 8|0);
   HEAP32[$69>>2] = $67;
   $70 = $1;
   $71 = (_ap_end($70)|0);
   $72 = $2;
   HEAP32[$72>>2] = $71;
   $73 = ($71|0)==(0|0);
   if ($73) {
    $0 = 0;
    $110 = $0;
    STACKTOP = sp;return ($110|0);
   } else {
    $0 = 1;
    $110 = $0;
    STACKTOP = sp;return ($110|0);
   }
   break;
  }
  case 116: case 84:  {
   $74 = $1;
   $75 = ((($74)) + 1|0);
   $76 = HEAP8[$75>>0]|0;
   $77 = $76 << 24 >> 24;
   $78 = ($77|0)==(108);
   do {
    if ($78) {
     label = 34;
    } else {
     $79 = $1;
     $80 = ((($79)) + 1|0);
     $81 = HEAP8[$80>>0]|0;
     $82 = $81 << 24 >> 24;
     $83 = ($82|0)==(76);
     if ($83) {
      label = 34;
     } else {
      $86 = $1;
      $87 = ((($86)) + 1|0);
      $88 = HEAP8[$87>>0]|0;
      $89 = $88 << 24 >> 24;
      $90 = ($89|0)==(114);
      if (!($90)) {
       $91 = $1;
       $92 = ((($91)) + 1|0);
       $93 = HEAP8[$92>>0]|0;
       $94 = $93 << 24 >> 24;
       $95 = ($94|0)==(82);
       if (!($95)) {
        $x = 15;
        break;
       }
      }
      $x = 14;
      $96 = $1;
      $97 = ((($96)) + 1|0);
      $1 = $97;
     }
    }
   } while(0);
   if ((label|0) == 34) {
    $x = 13;
    $84 = $1;
    $85 = ((($84)) + 1|0);
    $1 = $85;
   }
   $98 = $1;
   $99 = ((($98)) + 1|0);
   $100 = (_gt_num($99,$n,0)|0);
   $1 = $100;
   $101 = ($100|0)!=(0|0);
   if ($101) {
    $102 = $1;
    $103 = ((($102)) + -1|0);
    $1 = $103;
    $104 = $x;
    $105 = HEAP32[$n>>2]|0;
    (_op_gen($104,$105,0,0)|0);
    label = 43;
   } else {
    label = 21;
   }
   break;
  }
  case 120: case 88:  {
   (_op_gen(4,1,0,0)|0);
   label = 43;
   break;
  }
  case 112: case 80:  {
   (_op_gen(20,1,0,0)|0);
   label = 43;
   break;
  }
  default: {
   $0 = 0;
   $110 = $0;
   STACKTOP = sp;return ($110|0);
  }
  }
 } while(0);
 if ((label|0) == 19) {
  $40 = $1;
  $41 = ((($40)) + 1|0);
  $1 = $41;
  label = 20;
 }
 L46: do {
  if ((label|0) == 20) {
   $42 = $1;
   $43 = (_gt_num($42,$n,0)|0);
   $1 = $43;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $46 = $1;
    $47 = HEAP8[$46>>0]|0;
    $48 = $47 << 24 >> 24;
    switch ($48|0) {
    case 112: case 80:  {
     $49 = $sign;
     $50 = ($49|0)!=(0);
     if ($50) {
      $51 = HEAP32[$n>>2]|0;
      $52 = (0 - ($51))|0;
      HEAP32[$n>>2] = $52;
     }
     $53 = HEAP32[$n>>2]|0;
     (_op_gen(20,$53,0,0)|0);
     label = 43;
     break L46;
     break;
    }
    case 120: case 88:  {
     $54 = HEAP32[$n>>2]|0;
     (_op_gen(4,$54,0,0)|0);
     label = 43;
     break L46;
     break;
    }
    case 104: case 72:  {
     $55 = HEAP32[$n>>2]|0;
     $56 = (_op_gen(12,$55,0,0)|0);
     $57 = (2820 + ($56<<4)|0);
     $sp = $57;
     $58 = $1;
     $59 = ((($58)) + 1|0);
     $60 = $sp;
     $61 = ((($60)) + 8|0);
     HEAP32[$61>>2] = $59;
     $62 = HEAP32[$n>>2]|0;
     $63 = $1;
     $64 = (($63) + ($62)|0);
     $1 = $64;
     label = 43;
     break L46;
     break;
    }
    default: {
     $0 = 0;
     $110 = $0;
     STACKTOP = sp;return ($110|0);
    }
    }
   } else {
    label = 21;
   }
  }
 } while(0);
 if ((label|0) == 21) {
  $45 = $2;
  HEAP32[$45>>2] = 0;
  $0 = 1;
  $110 = $0;
  STACKTOP = sp;return ($110|0);
 }
 else if ((label|0) == 43) {
  $106 = $1;
  $107 = ((($106)) + 1|0);
  $1 = $107;
  $108 = $1;
  $109 = $2;
  HEAP32[$109>>2] = $108;
  $0 = 1;
  $110 = $0;
  STACKTOP = sp;return ($110|0);
 }
 return (0)|0;
}
function _e_d($s,$p) {
 $s = $s|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d = 0, $e = 0, $found = 0, $i = 0, $im = 0, $n = 0, $sv = 0, $w = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $n = sp + 24|0;
 $w = sp + 20|0;
 $d = sp + 16|0;
 $e = sp + 12|0;
 $1 = $s;
 $2 = $p;
 $found = 0;
 $x = 0;
 $3 = $1;
 $sv = $3;
 $4 = $1;
 $5 = (_gt_num($4,$n,1)|0);
 $1 = $5;
 $6 = HEAP32[$n>>2]|0;
 (_op_gen(6,$6,0,0)|0);
 $7 = $1;
 $8 = ((($7)) + 1|0);
 $1 = $8;
 $9 = HEAP8[$7>>0]|0;
 $10 = $9 << 24 >> 24;
 L1: do {
  switch ($10|0) {
  case 101: case 69:  {
   $x = 1;
   label = 3;
   break;
  }
  case 103: case 71:  {
   label = 3;
   break;
  }
  case 111: case 79:  {
   $i = 32;
   $im = 34;
   label = 40;
   break;
  }
  case 122: case 90:  {
   $i = 35;
   $im = 36;
   label = 40;
   break;
  }
  case 108: case 76:  {
   $found = 1;
   $48 = $1;
   $49 = (_gt_num($48,$w,0)|0);
   $1 = $49;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    $51 = HEAP32[$w>>2]|0;
    $52 = ($51|0)==(0);
    if (!($52)) {
     $53 = HEAP32[$w>>2]|0;
     (_op_gen(29,$53,0,0)|0);
    }
   } else {
    label = 4;
   }
   break;
  }
  case 97: case 65:  {
   $found = 1;
   while(1) {
    $54 = $1;
    $55 = HEAP8[$54>>0]|0;
    $56 = $55 << 24 >> 24;
    $57 = ($56|0)==(32);
    $58 = $1;
    if (!($57)) {
     break;
    }
    $59 = ((($58)) + 1|0);
    $1 = $59;
   }
   $60 = HEAP8[$58>>0]|0;
   $61 = $60 << 24 >> 24;
   $62 = ($61|0)>=(48);
   if ($62) {
    $63 = $1;
    $64 = HEAP8[$63>>0]|0;
    $65 = $64 << 24 >> 24;
    $66 = ($65|0)<=(57);
    if ($66) {
     $67 = $1;
     $68 = (_gt_num($67,$w,1)|0);
     $1 = $68;
     $69 = HEAP32[$w>>2]|0;
     $70 = ($69|0)==(0);
     if ($70) {
      break L1;
     }
     $71 = HEAP32[$w>>2]|0;
     (_op_gen(31,$71,0,0)|0);
     break L1;
    }
   }
   (_op_gen(30,0,0,0)|0);
   break;
  }
  case 102: case 70:  {
   $72 = $1;
   $73 = (_gt_num($72,$w,0)|0);
   $1 = $73;
   $74 = ($73|0)!=(0|0);
   if ($74) {
    $found = 1;
    $75 = HEAP32[$w>>2]|0;
    $76 = ($75|0)==(0);
    if (!($76)) {
     $77 = $1;
     $78 = HEAP8[$77>>0]|0;
     $79 = $78 << 24 >> 24;
     $80 = ($79|0)==(46);
     if ($80) {
      $81 = $1;
      $82 = ((($81)) + 1|0);
      $83 = (_gt_num($82,$d,0)|0);
      $1 = $83;
      $84 = ($83|0)!=(0|0);
      if (!($84)) {
       label = 4;
       break L1;
      }
     } else {
      HEAP32[$d>>2] = 0;
     }
     $85 = HEAP32[$w>>2]|0;
     $86 = HEAP32[$d>>2]|0;
     (_op_gen(23,$85,$86,0)|0);
    }
   } else {
    label = 4;
   }
   break;
  }
  case 100: case 68:  {
   $found = 1;
   $87 = $1;
   $88 = (_gt_num($87,$w,0)|0);
   $1 = $88;
   $89 = ($88|0)!=(0|0);
   if ($89) {
    $90 = HEAP32[$w>>2]|0;
    $91 = ($90|0)==(0);
    if (!($91)) {
     $92 = $1;
     $93 = HEAP8[$92>>0]|0;
     $94 = $93 << 24 >> 24;
     $95 = ($94|0)==(46);
     if ($95) {
      $96 = $1;
      $97 = ((($96)) + 1|0);
      $98 = (_gt_num($97,$d,0)|0);
      $1 = $98;
      $99 = ($98|0)!=(0|0);
      if (!($99)) {
       label = 4;
       break L1;
      }
     } else {
      HEAP32[$d>>2] = 0;
     }
     $100 = HEAP32[$w>>2]|0;
     $101 = HEAP32[$d>>2]|0;
     (_op_gen(26,$100,$101,0)|0);
    }
   } else {
    label = 4;
   }
   break;
  }
  case 105: case 73:  {
   $i = 7;
   $im = 10;
   label = 40;
   break;
  }
  default: {
  }
  }
 } while(0);
 do {
  if ((label|0) == 3) {
   $found = 1;
   $11 = $1;
   $12 = (_gt_num($11,$w,0)|0);
   $1 = $12;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $15 = HEAP32[$w>>2]|0;
    $16 = ($15|0)==(0);
    if (!($16)) {
     $17 = $1;
     $18 = HEAP8[$17>>0]|0;
     $19 = $18 << 24 >> 24;
     $20 = ($19|0)==(46);
     if ($20) {
      $21 = $1;
      $22 = ((($21)) + 1|0);
      $23 = (_gt_num($22,$d,0)|0);
      $1 = $23;
      $24 = ($23|0)!=(0|0);
      if (!($24)) {
       label = 4;
       break;
      }
     } else {
      HEAP32[$d>>2] = 0;
     }
     $25 = $1;
     $26 = HEAP8[$25>>0]|0;
     $27 = $26 << 24 >> 24;
     $28 = ($27|0)!=(69);
     if ($28) {
      $29 = $1;
      $30 = HEAP8[$29>>0]|0;
      $31 = $30 << 24 >> 24;
      $32 = ($31|0)!=(101);
      if ($32) {
       $33 = $x;
       $34 = ($33|0)==(1);
       $35 = $34 ? 24 : 27;
       $36 = HEAP32[$w>>2]|0;
       $37 = HEAP32[$d>>2]|0;
       (_op_gen($35,$36,$37,0)|0);
       break;
      }
     }
     $38 = $1;
     $39 = ((($38)) + 1|0);
     $40 = (_gt_num($39,$e,0)|0);
     $1 = $40;
     $41 = ($40|0)!=(0|0);
     if ($41) {
      $42 = $x;
      $43 = ($42|0)==(1);
      $44 = $43 ? 25 : 28;
      $45 = HEAP32[$w>>2]|0;
      $46 = HEAP32[$d>>2]|0;
      $47 = HEAP32[$e>>2]|0;
      (_op_gen($44,$45,$46,$47)|0);
     } else {
      label = 4;
     }
    }
   } else {
    label = 4;
   }
  }
  else if ((label|0) == 40) {
   $102 = $1;
   $103 = (_gt_num($102,$w,0)|0);
   $1 = $103;
   $104 = ($103|0)!=(0|0);
   if ($104) {
    $found = 1;
    $105 = HEAP32[$w>>2]|0;
    $106 = ($105|0)==(0);
    if (!($106)) {
     $107 = $1;
     $108 = HEAP8[$107>>0]|0;
     $109 = $108 << 24 >> 24;
     $110 = ($109|0)!=(46);
     if ($110) {
      $111 = $i;
      $112 = HEAP32[$w>>2]|0;
      (_op_gen($111,$112,0,0)|0);
      break;
     }
     $113 = $1;
     $114 = ((($113)) + 1|0);
     $115 = (_gt_num($114,$d,0)|0);
     $1 = $115;
     $116 = ($115|0)!=(0|0);
     if ($116) {
      $117 = $im;
      $118 = HEAP32[$w>>2]|0;
      $119 = HEAP32[$d>>2]|0;
      (_op_gen($117,$118,$119,0)|0);
     } else {
      label = 4;
     }
    }
   } else {
    label = 4;
   }
  }
 } while(0);
 if ((label|0) == 4) {
  $14 = $2;
  HEAP32[$14>>2] = 0;
  $0 = 1;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 $120 = $found;
 $121 = ($120|0)==(0);
 if ($121) {
  $122 = HEAP32[2712>>2]|0;
  $123 = (($122) + -1)|0;
  HEAP32[2712>>2] = $123;
  $124 = $sv;
  $125 = $2;
  HEAP32[$125>>2] = $124;
  $0 = 0;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 } else {
  $126 = $1;
  $127 = $2;
  HEAP32[$127>>2] = $126;
  $0 = 1;
  $128 = $0;
  STACKTOP = sp;return ($128|0);
 }
 return (0)|0;
}
function _gt_num($s,$n,$n1) {
 $s = $s|0;
 $n = $n|0;
 $n1 = $n1|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $f__cnt = 0, $m = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $s;
 $1 = $n;
 $2 = $n1;
 $m = 0;
 $f__cnt = 0;
 $3 = $0;
 $4 = HEAP8[$3>>0]|0;
 $c = $4;
 while(1) {
  $5 = $c;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)==(32);
  if ($7) {
   $8 = $0;
   $9 = ((($8)) + 1|0);
   $0 = $9;
  } else {
   $10 = $c;
   $11 = $10 << 24 >> 24;
   $12 = ($11|0)>(57);
   if ($12) {
    break;
   }
   $13 = $c;
   $14 = $13 << 24 >> 24;
   $15 = ($14|0)<(48);
   if ($15) {
    break;
   }
   $16 = $m;
   $17 = ($16*10)|0;
   $18 = $c;
   $19 = $18 << 24 >> 24;
   $20 = (($17) + ($19))|0;
   $21 = (($20) - 48)|0;
   $m = $21;
   $22 = $f__cnt;
   $23 = (($22) + 1)|0;
   $f__cnt = $23;
   $24 = $0;
   $25 = ((($24)) + 1|0);
   $0 = $25;
  }
  $26 = $0;
  $27 = HEAP8[$26>>0]|0;
  $c = $27;
 }
 $28 = $f__cnt;
 $29 = ($28|0)==(0);
 if (!($29)) {
  $34 = $m;
  $35 = $1;
  HEAP32[$35>>2] = $34;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $30 = $2;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  $0 = 0;
 }
 $32 = $2;
 $33 = $1;
 HEAP32[$33>>2] = $32;
 $36 = $0;
 STACKTOP = sp;return ($36|0);
}
function _ap_end($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $quote = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $s;
 $2 = $1;
 $3 = ((($2)) + 1|0);
 $1 = $3;
 $4 = HEAP8[$2>>0]|0;
 $quote = $4;
 while(1) {
  $5 = $1;
  $6 = HEAP8[$5>>0]|0;
  $7 = ($6<<24>>24)!=(0);
  if (!($7)) {
   break;
  }
  $8 = $1;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = $quote;
  $12 = $11 << 24 >> 24;
  $13 = ($10|0)!=($12|0);
  if (!($13)) {
   $14 = $1;
   $15 = ((($14)) + 1|0);
   $1 = $15;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16 << 24 >> 24;
   $18 = $quote;
   $19 = $18 << 24 >> 24;
   $20 = ($17|0)!=($19|0);
   if ($20) {
    label = 5;
    break;
   }
  }
  $22 = $1;
  $23 = ((($22)) + 1|0);
  $1 = $23;
 }
 if ((label|0) == 5) {
  $21 = $1;
  $0 = $21;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 $24 = HEAP32[12596>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = (___errno_location()|0);
  HEAP32[$27>>2] = 100;
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 } else {
  _f__fatal(100,18541);
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _c_sfe($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = $1;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (7764 + (($4*48)|0)|0);
 $p = $5;
 HEAP32[7760>>2] = $5;
 $6 = $1;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)>=(100);
 if (!($9)) {
  $10 = $1;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)<(0);
  if (!($13)) {
   $18 = $p;
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    $21 = $1;
    $22 = ((($21)) + 4|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = (_fk_open(3,5,$23)|0);
    $25 = ($24|0)!=(0);
    if ($25) {
     $26 = $1;
     $27 = HEAP32[$26>>2]|0;
     $28 = ($27|0)!=(0);
     if ($28) {
      $29 = (___errno_location()|0);
      HEAP32[$29>>2] = 114;
     } else {
      _f__fatal(114,18577);
     }
     $0 = 114;
     $38 = $0;
     STACKTOP = sp;return ($38|0);
    }
   }
   $30 = $p;
   $31 = ((($30)) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ($32|0)!=(0);
   if ($33) {
    $0 = 0;
    $38 = $0;
    STACKTOP = sp;return ($38|0);
   }
   $34 = $1;
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = (___errno_location()|0);
    HEAP32[$37>>2] = 102;
   } else {
    _f__fatal(102,18577);
   }
   $0 = 102;
   $38 = $0;
   STACKTOP = sp;return ($38|0);
  }
 }
 $14 = $1;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = (___errno_location()|0);
  HEAP32[$17>>2] = 101;
 } else {
  _f__fatal(101,18581);
 }
 $0 = 101;
 $38 = $0;
 STACKTOP = sp;return ($38|0);
}
function _e_wsfe() {
 var $0 = 0, $1 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = (_en_fio()|0);
 $n = $0;
 HEAP32[12564>>2] = 0;
 $1 = $n;
 STACKTOP = sp;return ($1|0);
}
function _x_wSL() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = (_f__putbuf(10)|0);
 $n = $0;
 HEAP32[12636>>2] = 0;
 HEAP32[12632>>2] = 0;
 HEAP32[12640>>2] = 0;
 $1 = $n;
 $2 = ($1|0)==(0);
 $3 = $2&1;
 STACKTOP = sp;return ($3|0);
}
function _s_wsfe($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = HEAP32[12584>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  _f_init();
 }
 HEAP32[12568>>2] = 0;
 HEAP32[12572>>2] = 1;
 HEAP32[12576>>2] = 1;
 HEAP32[12580>>2] = 1;
 $4 = $1;
 $5 = (_c_sfe($4)|0);
 $n = $5;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = $n;
  $0 = $7;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 $8 = $1;
 HEAP32[12596>>2] = $8;
 HEAP32[12632>>2] = 0;
 HEAP32[12636>>2] = 0;
 HEAP32[12640>>2] = 0;
 HEAP32[2816>>2] = 0;
 HEAP32[12644>>2] = 0;
 $9 = $1;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[12564>>2] = $11;
 $12 = HEAP32[7760>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[12588>>2] = $13;
 $14 = HEAP32[12564>>2]|0;
 $15 = (_pars_f($14)|0);
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = $1;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = (___errno_location()|0);
   HEAP32[$20>>2] = 100;
  } else {
   _f__fatal(100,18581);
  }
  $0 = 100;
  $39 = $0;
  STACKTOP = sp;return ($39|0);
 }
 HEAP32[12612>>2] = 7;
 HEAP32[12616>>2] = 8;
 HEAP32[12620>>2] = 9;
 HEAP32[12592>>2] = 10;
 HEAP32[12624>>2] = 11;
 HEAP32[12628>>2] = 12;
 _fmt_bg();
 HEAP32[12604>>2] = 0;
 $21 = HEAP32[7760>>2]|0;
 $22 = ((($21)) + 32|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[12608>>2] = $23;
 $24 = HEAP32[7760>>2]|0;
 $25 = ((($24)) + 40|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)!=(1);
 if ($27) {
  $28 = HEAP32[7760>>2]|0;
  $29 = (_f__nowwriting($28)|0);
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = $1;
   $32 = HEAP32[$31>>2]|0;
   $33 = ($32|0)!=(0);
   $34 = (___errno_location()|0);
   $35 = HEAP32[$34>>2]|0;
   if ($33) {
    $36 = (___errno_location()|0);
    HEAP32[$36>>2] = $35;
   } else {
    _f__fatal($35,18589);
   }
   $37 = (___errno_location()|0);
   $38 = HEAP32[$37>>2]|0;
   $0 = $38;
   $39 = $0;
   STACKTOP = sp;return ($39|0);
  }
 }
 $0 = 0;
 $39 = $0;
 STACKTOP = sp;return ($39|0);
}
function _xw_end() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = HEAP32[2816>>2]|0;
 $1 = ($0|0)!=(0);
 if ($1) {
  $n = 0;
  (_f__putbuf(0)|0);
  $2 = HEAP32[12588>>2]|0;
  (_fflush($2)|0);
 } else {
  $3 = (_f__putbuf(10)|0);
  $n = $3;
 }
 HEAP32[12636>>2] = 0;
 HEAP32[12632>>2] = 0;
 HEAP32[12640>>2] = 0;
 $4 = $n;
 STACKTOP = sp;return ($4|0);
}
function _xw_rev() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $n = 0;
 $0 = HEAP32[2768>>2]|0;
 $1 = ($0|0)!=(0);
 if ($1) {
  $2 = (_f__putbuf(10)|0);
  $n = $2;
  HEAP32[2768>>2] = 0;
 }
 HEAP32[12636>>2] = 0;
 HEAP32[12632>>2] = 0;
 HEAP32[12640>>2] = 0;
 $3 = $n;
 STACKTOP = sp;return ($3|0);
}
function _s_wsle($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = $1;
 $3 = (_c_le($2)|0);
 $n = $3;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = $n;
  $0 = $5;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 HEAP32[12568>>2] = 0;
 HEAP32[12580>>2] = 1;
 HEAP32[12576>>2] = 1;
 HEAP32[12612>>2] = 7;
 HEAP32[12652>>2] = 13;
 HEAP32[12656>>2] = 80;
 HEAP32[12628>>2] = 12;
 $6 = HEAP32[7760>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(1);
 if ($9) {
  $10 = HEAP32[7760>>2]|0;
  $11 = (_f__nowwriting($10)|0);
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = $1;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(0);
   $16 = (___errno_location()|0);
   $17 = HEAP32[$16>>2]|0;
   if ($15) {
    $18 = (___errno_location()|0);
    HEAP32[$18>>2] = $17;
   } else {
    _f__fatal($17,18601);
   }
   $19 = (___errno_location()|0);
   $20 = HEAP32[$19>>2]|0;
   $0 = $20;
   $21 = $0;
   STACKTOP = sp;return ($21|0);
  }
 }
 $0 = 0;
 $21 = $0;
 STACKTOP = sp;return ($21|0);
}
function _e_wsle() {
 var $0 = 0, $1 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = (_f__putbuf(10)|0);
 $n = $0;
 HEAP32[12632>>2] = 0;
 $1 = $n;
 STACKTOP = sp;return ($1|0);
}
function _sig_die($s,$kill) {
 $s = $s|0;
 $kill = $kill|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = $s;
 $1 = $kill;
 $2 = HEAP32[12736>>2]|0;
 $3 = $0;
 HEAP32[$vararg_buffer>>2] = $3;
 (_fprintf($2,18619,$vararg_buffer)|0);
 $4 = $1;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[12736>>2]|0;
  (_fflush($6)|0);
  _f_exit();
  $7 = HEAP32[12736>>2]|0;
  (_fflush($7)|0);
  (_signal(6,(0|0))|0);
  _abort();
  // unreachable;
 } else {
  _exit(1);
  // unreachable;
 }
}
function _f_clos($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = $1;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(100);
 if ($5) {
  $0 = 0;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 $6 = $1;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (7764 + (($8*48)|0)|0);
 $b = $9;
 $10 = $b;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0|0);
 if (!($12)) {
  $13 = $b;
  $14 = ((($13)) + 44|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)==(1);
  L7: do {
   if ($16) {
    label = 11;
   } else {
    $17 = $1;
    $18 = ((($17)) + 8|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = ($19|0)!=(0|0);
    if ($20) {
     $21 = $1;
     $22 = ((($21)) + 8|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = HEAP8[$23>>0]|0;
     $25 = $24 << 24 >> 24;
     switch ($25|0) {
     case 100: case 68:  {
      label = 11;
      break L7;
      break;
     }
     default: {
     }
     }
    }
    $26 = $b;
    $27 = ((($26)) + 40|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = ($28|0)==(1);
    if ($29) {
     $30 = $1;
     (_t_runc($30)|0);
    }
    $31 = $b;
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($33|0)!=(0|0);
    if ($34) {
     $35 = $b;
     $36 = HEAP32[$35>>2]|0;
     (_fclose($36)|0);
     $37 = $b;
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     _free($39);
    }
   }
  } while(0);
  if ((label|0) == 11) {
   $40 = $b;
   $41 = HEAP32[$40>>2]|0;
   (_fclose($41)|0);
   $42 = $b;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(0|0);
   if ($45) {
    $46 = $b;
    $47 = ((($46)) + 4|0);
    $48 = HEAP32[$47>>2]|0;
    (_unlink($48)|0);
    $49 = $b;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    _free($51);
   }
  }
  $52 = $b;
  HEAP32[$52>>2] = 0;
 }
 $53 = $b;
 $54 = ((($53)) + 36|0);
 HEAP32[$54>>2] = 0;
 $55 = $b;
 $56 = ((($55)) + 4|0);
 HEAP32[$56>>2] = 0;
 $0 = 0;
 $57 = $0;
 STACKTOP = sp;return ($57|0);
}
function _f_exit() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = HEAP32[7620>>2]|0;
 $1 = ($0|0)!=(0);
 if ($1) {
  STACKTOP = sp;return;
 }
 HEAP32[7620>>2] = 1;
 HEAP32[(7628)>>2] = 0;
 $i = 0;
 while(1) {
  $2 = $i;
  $3 = ($2|0)<(100);
  if (!($3)) {
   break;
  }
  $4 = $i;
  HEAP32[(7624)>>2] = $4;
  (_f_clos(7620)|0);
  $5 = $i;
  $6 = (($5) + 1)|0;
  $i = $6;
 }
 STACKTOP = sp;return;
}
function _t_runc($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $bf = 0, $len = 0, $loc = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = $1;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (7764 + (($4*48)|0)|0);
 $b = $5;
 $6 = $b;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $0 = 0;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 $10 = $b;
 $11 = HEAP32[$10>>2]|0;
 $bf = $11;
 $12 = (_ftell($11)|0);
 $loc = $12;
 $13 = $bf;
 (_fseek($13,0,2)|0);
 $14 = $bf;
 $15 = (_ftell($14)|0);
 $len = $15;
 $16 = $loc;
 $17 = $len;
 $18 = ($16|0)>=($17|0);
 if (!($18)) {
  $19 = $b;
  $20 = ((($19)) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)==(0);
  if (!($22)) {
   $23 = $b;
   $24 = ((($23)) + 28|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25 & 2;
   $27 = ($26|0)!=(0);
   if ($27) {
    $28 = $b;
    $29 = HEAP32[$28>>2]|0;
    (_fflush($29)|0);
   }
   $30 = $b;
   $31 = HEAP32[$30>>2]|0;
   $32 = (_fileno($31)|0);
   $33 = $loc;
   $34 = (_ftruncate($32,$33)|0);
   $rc = $34;
   $35 = $b;
   $36 = HEAP32[$35>>2]|0;
   (_fseek($36,0,2)|0);
   $37 = $rc;
   $38 = ($37|0)!=(0);
   if (!($38)) {
    $0 = 0;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
   $39 = $1;
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = (___errno_location()|0);
    HEAP32[$42>>2] = 111;
   } else {
    _f__fatal(111,18623);
   }
   $0 = 111;
   $43 = $0;
   STACKTOP = sp;return ($43|0);
  }
 }
 $0 = 0;
 $43 = $0;
 STACKTOP = sp;return ($43|0);
}
function _f__canseek($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $switch$split12D = 0, $switch$split2D = 0, $switch$split42D = 0;
 var $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $x = sp;
 $1 = $f;
 $2 = $1;
 $3 = (_fileno($2)|0);
 $4 = (_fstat($3,$x)|0);
 $5 = ($4|0)<(0);
 L1: do {
  if ($5) {
   $0 = 0;
  } else {
   $6 = ((($x)) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = $7 & 61440;
   $switch$split2D = ($8|0)<(24576);
   L4: do {
    if ($switch$split2D) {
     $switch$split12D = ($8|0)<(16384);
     if (!($switch$split12D)) {
      switch ($8|0) {
      case 16384:  {
       break L4;
       break;
      }
      default: {
       label = 11;
       break L4;
      }
      }
     }
     switch ($8|0) {
     case 8192:  {
      break;
     }
     default: {
      label = 11;
      break L4;
     }
     }
     $12 = $1;
     $13 = (_fileno($12)|0);
     $14 = (_isatty($13)|0);
     $15 = ($14|0)!=(0);
     if ($15) {
      $0 = 0;
      break L1;
     } else {
      $0 = 1;
      break L1;
     }
    } else {
     $switch$split42D = ($8|0)<(32768);
     if (!($switch$split42D)) {
      switch ($8|0) {
      case 32768:  {
       break L4;
       break;
      }
      default: {
       label = 11;
       break L4;
      }
      }
     }
     switch ($8|0) {
     case 24576:  {
      break;
     }
     default: {
      label = 11;
      break L4;
     }
     }
     $0 = 1;
     break L1;
    }
   } while(0);
   if ((label|0) == 11) {
    $0 = 0;
    break;
   }
   $9 = ((($x)) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10>>>0)>(0);
   if ($11) {
    $0 = 1;
    break;
   } else {
    $0 = 0;
    break;
   }
  }
 } while(0);
 $16 = $0;
 STACKTOP = sp;return ($16|0);
}
function _f__fatal($n,$s) {
 $n = $n|0;
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0;
 var $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer23 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr28 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer23 = sp + 56|0;
 $vararg_buffer20 = sp + 48|0;
 $vararg_buffer18 = sp + 40|0;
 $vararg_buffer15 = sp + 32|0;
 $vararg_buffer12 = sp + 24|0;
 $vararg_buffer8 = sp + 16|0;
 $vararg_buffer5 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $n;
 $1 = $s;
 $2 = $0;
 $3 = ($2|0)<(100);
 $4 = $0;
 $5 = ($4|0)>=(0);
 $or$cond = $3 & $5;
 do {
  if ($or$cond) {
   $6 = $1;
   _perror($6);
  } else {
   $7 = $0;
   $8 = ($7|0)>=(132);
   $9 = $0;
   $10 = ($9|0)<(-1);
   $or$cond3 = $8 | $10;
   if ($or$cond3) {
    $11 = HEAP32[12736>>2]|0;
    $12 = $1;
    $13 = $0;
    HEAP32[$vararg_buffer>>2] = $12;
    $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $13;
    (_fprintf($11,18631,$vararg_buffer)|0);
    break;
   }
   $14 = $0;
   $15 = ($14|0)==(-1);
   $16 = HEAP32[12736>>2]|0;
   $17 = $1;
   if ($15) {
    HEAP32[$vararg_buffer5>>2] = $17;
    (_fprintf($16,18660,$vararg_buffer5)|0);
    break;
   } else {
    $18 = $0;
    $19 = (($18) - 100)|0;
    $20 = (7632 + ($19<<2)|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $17;
    $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
    HEAP32[$vararg_ptr11>>2] = $21;
    (_fprintf($16,18677,$vararg_buffer8)|0);
    break;
   }
  }
 } while(0);
 $22 = HEAP32[7760>>2]|0;
 $23 = ($22|0)!=(0|0);
 $24 = HEAP32[12736>>2]|0;
 if ($23) {
  $25 = HEAP32[7760>>2]|0;
  $26 = $25;
  $27 = (($26) - (7764))|0;
  $28 = (($27|0) / 48)&-1;
  HEAP32[$vararg_buffer12>>2] = $28;
  (_fprintf($24,18685,$vararg_buffer12)|0);
  $29 = HEAP32[12736>>2]|0;
  $30 = HEAP32[7760>>2]|0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)!=(0|0);
  $34 = $33 ? 18710 : 18720;
  $35 = HEAP32[7760>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$vararg_buffer15>>2] = $37;
  (_fprintf($29,$34,$vararg_buffer15)|0);
 } else {
  (_fprintf($24,18731,$vararg_buffer18)|0);
 }
 $38 = HEAP32[12564>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = HEAP32[12736>>2]|0;
  $41 = HEAP32[12564>>2]|0;
  HEAP32[$vararg_buffer20>>2] = $41;
  (_fprintf($40,18761,$vararg_buffer20)|0);
 }
 $42 = HEAP32[12736>>2]|0;
 $43 = HEAP32[12568>>2]|0;
 $44 = ($43|0)!=(0);
 $45 = $44 ? 18778 : 18786;
 $46 = HEAP32[12572>>2]|0;
 $47 = ($46|0)!=(0);
 $48 = $47 ? 18794 : 18805;
 $49 = HEAP32[12576>>2]|0;
 $50 = ($49|0)!=(0);
 $51 = $50 ? 18812 : 18822;
 $52 = HEAP32[12580>>2]|0;
 $53 = ($52|0)!=(0);
 $54 = $53 ? 18834 : 18843;
 HEAP32[$vararg_buffer23>>2] = $45;
 $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
 HEAP32[$vararg_ptr26>>2] = $48;
 $vararg_ptr27 = ((($vararg_buffer23)) + 8|0);
 HEAP32[$vararg_ptr27>>2] = $51;
 $vararg_ptr28 = ((($vararg_buffer23)) + 12|0);
 HEAP32[$vararg_ptr28>>2] = $54;
 (_fprintf($42,18852,$vararg_buffer23)|0);
 _sig_die(18871,1);
 STACKTOP = sp;return;
}
function _f_init() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 HEAP32[12584>>2] = 1;
 $p = 7764;
 $0 = HEAP32[12736>>2]|0;
 $1 = $p;
 HEAP32[$1>>2] = $0;
 $2 = HEAP32[12736>>2]|0;
 $3 = (_f__canseek($2)|0);
 $4 = $p;
 $5 = ((($4)) + 20|0);
 HEAP32[$5>>2] = $3;
 $6 = $p;
 $7 = ((($6)) + 24|0);
 HEAP32[$7>>2] = 1;
 $8 = $p;
 $9 = ((($8)) + 40|0);
 HEAP32[$9>>2] = 1;
 $p = (8004);
 $10 = HEAP32[12740>>2]|0;
 $11 = $p;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[12740>>2]|0;
 $13 = (_f__canseek($12)|0);
 $14 = $p;
 $15 = ((($14)) + 20|0);
 HEAP32[$15>>2] = $13;
 $16 = $p;
 $17 = ((($16)) + 24|0);
 HEAP32[$17>>2] = 1;
 $18 = $p;
 $19 = ((($18)) + 40|0);
 HEAP32[$19>>2] = 0;
 $p = (8052);
 $20 = HEAP32[12728>>2]|0;
 $21 = $p;
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[12728>>2]|0;
 $23 = (_f__canseek($22)|0);
 $24 = $p;
 $25 = ((($24)) + 20|0);
 HEAP32[$25>>2] = $23;
 $26 = $p;
 $27 = ((($26)) + 24|0);
 HEAP32[$27>>2] = 1;
 $28 = $p;
 $29 = ((($28)) + 40|0);
 HEAP32[$29>>2] = 1;
 STACKTOP = sp;return;
}
function _f__nowwriting($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $loc = 0, $ufmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $x;
 $2 = $1;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 $7 = $1;
 L1: do {
  if ($6) {
   $8 = ((($7)) + 28|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 1;
   $11 = ($10|0)!=(0);
   if ($11) {
    $12 = $1;
    $13 = HEAP32[$12>>2]|0;
    (_fseek($13,0,1)|0);
   }
  } else {
   $14 = ((($7)) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   do {
    if ($16) {
     $17 = $1;
     $18 = ((($17)) + 16|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = ($19|0)!=(0);
     if ($20) {
      $24 = 0;
     } else {
      $21 = $1;
      $22 = ((($21)) + 24|0);
      $23 = HEAP32[$22>>2]|0;
      $24 = $23;
     }
     $ufmt = $24;
     $25 = $1;
     $26 = ((($25)) + 40|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = ($27|0)==(3);
     $29 = $1;
     if ($28) {
      $30 = ((($29)) + 4|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = $ufmt;
      $33 = (12672 + ($32<<2)|0);
      $34 = HEAP32[$33>>2]|0;
      $35 = $1;
      $36 = HEAP32[$35>>2]|0;
      $37 = (_freopen($31,$34,$36)|0);
      $38 = $1;
      HEAP32[$38>>2] = $37;
      HEAP32[12588>>2] = $37;
      $39 = ($37|0)!=(0|0);
      if (!($39)) {
       break;
      }
      $40 = $1;
      $41 = ((($40)) + 28|0);
      HEAP32[$41>>2] = 2;
      break L1;
     }
     $42 = HEAP32[$29>>2]|0;
     $43 = (_ftell($42)|0);
     $loc = $43;
     $44 = $1;
     $45 = ((($44)) + 4|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = $ufmt;
     $48 = $47 | 2;
     $49 = (12672 + ($48<<2)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = $1;
     $52 = HEAP32[$51>>2]|0;
     $53 = (_freopen($46,$50,$52)|0);
     $54 = $1;
     HEAP32[$54>>2] = $53;
     HEAP32[12588>>2] = $53;
     $55 = ($53|0)!=(0|0);
     $56 = $1;
     if ($55) {
      $58 = ((($56)) + 28|0);
      HEAP32[$58>>2] = 3;
      $59 = $1;
      $60 = HEAP32[$59>>2]|0;
      $61 = $loc;
      (_fseek($60,$61,0)|0);
      break L1;
     } else {
      HEAP32[$56>>2] = 0;
      break;
     }
    }
   } while(0);
   $57 = (___errno_location()|0);
   HEAP32[$57>>2] = 127;
   $0 = 1;
   $64 = $0;
   STACKTOP = sp;return ($64|0);
  }
 } while(0);
 $62 = $1;
 $63 = ((($62)) + 40|0);
 HEAP32[$63>>2] = 1;
 $0 = 0;
 $64 = $0;
 STACKTOP = sp;return ($64|0);
}
function _err__fl($f,$m,$s) {
 $f = $f|0;
 $m = $m|0;
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $f;
 $1 = $m;
 $2 = $s;
 $3 = $0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  $5 = $1;
  $6 = $2;
  _f__fatal($5,$6);
 }
 $7 = HEAP32[12592>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[12592>>2]|0;
  (FUNCTION_TABLE_i[$9 & 15]()|0);
 }
 $10 = $1;
 $11 = (___errno_location()|0);
 HEAP32[$11>>2] = $10;
 STACKTOP = sp;return ($10|0);
}
function _c_le($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = HEAP32[12584>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  _f_init();
 }
 HEAP32[12564>>2] = 19540;
 $4 = $1;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (7764 + (($6*48)|0)|0);
 HEAP32[7760>>2] = $7;
 $8 = $1;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)>=(100);
 if (!($11)) {
  $12 = $1;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)<(0);
  if (!($15)) {
   HEAP32[12632>>2] = 0;
   HEAP32[12644>>2] = 0;
   $20 = $1;
   HEAP32[12596>>2] = $20;
   $21 = HEAP32[7760>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)==(0|0);
   if ($23) {
    $24 = $1;
    $25 = ((($24)) + 4|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = (_fk_open(3,5,$26)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     $29 = $1;
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)!=(0);
     if ($31) {
      $32 = (___errno_location()|0);
      HEAP32[$32>>2] = 102;
     } else {
      _f__fatal(102,19554);
     }
     $0 = 102;
     $43 = $0;
     STACKTOP = sp;return ($43|0);
    }
   }
   $33 = HEAP32[7760>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   HEAP32[12588>>2] = $34;
   $35 = HEAP32[7760>>2]|0;
   $36 = ((($35)) + 24|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = ($37|0)!=(0);
   if ($38) {
    $0 = 0;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
   $39 = $1;
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = (___errno_location()|0);
    HEAP32[$42>>2] = 103;
   } else {
    _f__fatal(103,19554);
   }
   $0 = 103;
   $43 = $0;
   STACKTOP = sp;return ($43|0);
  }
 }
 $16 = $1;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = (___errno_location()|0);
  HEAP32[$19>>2] = 101;
 } else {
  _f__fatal(101,19548);
 }
 $0 = 101;
 $43 = $0;
 STACKTOP = sp;return ($43|0);
}
function _l_write($number,$ptr,$len,$type) {
 $number = $number|0;
 $ptr = $ptr|0;
 $len = $len|0;
 $type = $type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $x = 0;
 var $xx = 0, $y = 0.0, $yy = 0, $z = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $number;
 $1 = $ptr;
 $2 = $len;
 $3 = $type;
 $i = 0;
 while(1) {
  $4 = $i;
  $5 = $0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $3;
  do {
   switch ($8|0) {
   case 11:  {
    label = 5;
    break;
   }
   case 2:  {
    $12 = $1;
    $13 = HEAP16[$12>>1]|0;
    $14 = $13 << 16 >> 16;
    $x = $14;
    label = 8;
    break;
   }
   case 3:  {
    $15 = $1;
    $16 = HEAP32[$15>>2]|0;
    $x = $16;
    label = 8;
    break;
   }
   case 4:  {
    $18 = $1;
    $19 = +HEAPF32[$18>>2];
    $20 = $19;
    $y = $20;
    label = 11;
    break;
   }
   case 5:  {
    $21 = $1;
    $22 = +HEAPF64[$21>>3];
    $y = $22;
    label = 11;
    break;
   }
   case 6:  {
    $24 = $1;
    $xx = $24;
    $25 = $xx;
    $26 = ((($25)) + 4|0);
    $xx = $26;
    $27 = +HEAPF32[$25>>2];
    $28 = $27;
    $y = $28;
    $29 = $xx;
    $30 = +HEAPF32[$29>>2];
    $31 = $30;
    $z = $31;
    label = 14;
    break;
   }
   case 7:  {
    $32 = $1;
    $yy = $32;
    $33 = $yy;
    $34 = ((($33)) + 8|0);
    $yy = $34;
    $35 = +HEAPF64[$33>>3];
    $y = $35;
    $36 = $yy;
    $37 = +HEAPF64[$36>>3];
    $z = $37;
    label = 14;
    break;
   }
   case 12:  {
    $40 = $1;
    $41 = HEAP8[$40>>0]|0;
    $42 = $41 << 24 >> 24;
    $x = $42;
    label = 18;
    break;
   }
   case 13:  {
    $43 = $1;
    $44 = HEAP16[$43>>1]|0;
    $45 = $44 << 16 >> 16;
    $x = $45;
    label = 18;
    break;
   }
   case 8:  {
    $46 = $1;
    $47 = HEAP32[$46>>2]|0;
    $x = $47;
    label = 18;
    break;
   }
   case 9:  {
    $51 = $1;
    $52 = $2;
    _lwrt_A($51,$52);
    break;
   }
   default: {
    _f__fatal(117,19558);
    label = 5;
   }
   }
  } while(0);
  if ((label|0) == 5) {
   label = 0;
   $9 = $1;
   $10 = HEAP8[$9>>0]|0;
   $11 = $10 << 24 >> 24;
   $x = $11;
   label = 8;
  }
  else if ((label|0) == 11) {
   label = 0;
   $23 = $y;
   _lwrt_F($23);
  }
  else if ((label|0) == 14) {
   label = 0;
   $38 = $y;
   $39 = $z;
   _lwrt_C($38,$39);
  }
  else if ((label|0) == 18) {
   label = 0;
   $48 = $1;
   $49 = HEAP32[$48>>2]|0;
   $50 = $2;
   _lwrt_L($49,$50);
  }
  if ((label|0) == 8) {
   label = 0;
   $17 = $x;
   _lwrt_I($17);
  }
  $53 = $2;
  $54 = $1;
  $55 = (($54) + ($53)|0);
  $1 = $55;
  $56 = $i;
  $57 = (($56) + 1)|0;
  $i = $57;
 }
 STACKTOP = sp;return 0;
}
function _lwrt_I($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ndigit = 0, $p = 0, $sign = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ndigit = sp + 4|0;
 $sign = sp;
 $0 = $n;
 $1 = $0;
 $2 = (_f__icvt($1,$ndigit,$sign,10)|0);
 $p = $2;
 $3 = HEAP32[12632>>2]|0;
 $4 = HEAP32[$ndigit>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = HEAP32[12656>>2]|0;
 $7 = ($5|0)>=($6|0);
 if ($7) {
  _donewrec();
 }
 $8 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$8 & 15](32);
 $9 = HEAP32[$sign>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$11 & 15](45);
 }
 while(1) {
  $12 = $p;
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[12612>>2]|0;
  $16 = $p;
  $17 = ((($16)) + 1|0);
  $p = $17;
  $18 = HEAP8[$16>>0]|0;
  $19 = $18 << 24 >> 24;
  FUNCTION_TABLE_vi[$15 & 15]($19);
 }
 STACKTOP = sp;return;
}
function _lwrt_F($n) {
 $n = +$n;
 var $0 = 0.0, $1 = 0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $buf = sp + 8|0;
 $0 = $n;
 $1 = HEAP32[12632>>2]|0;
 $2 = $0;
 $3 = (_l_g($buf,$2)|0);
 $4 = (($1) + ($3))|0;
 $5 = HEAP32[12656>>2]|0;
 $6 = ($4|0)>=($5|0);
 if (!($6)) {
  _l_put($buf);
  STACKTOP = sp;return;
 }
 _donewrec();
 _l_put($buf);
 STACKTOP = sp;return;
}
function _lwrt_C($a,$b) {
 $a = +$a;
 $b = +$b;
 var $0 = 0.0, $1 = 0.0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $al = 0, $ba = 0, $bb = 0, $bl = 0, $bufa = 0, $bufb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $bufa = sp + 56|0;
 $bufb = sp + 32|0;
 $0 = $a;
 $1 = $b;
 $2 = $0;
 $3 = (_l_g($bufa,$2)|0);
 $al = $3;
 $ba = $bufa;
 while(1) {
  $4 = $ba;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)==(32);
  if (!($7)) {
   break;
  }
  $8 = $al;
  $9 = (($8) + -1)|0;
  $al = $9;
  $10 = $ba;
  $11 = ((($10)) + 1|0);
  $ba = $11;
 }
 $12 = $1;
 $13 = (_l_g($bufb,$12)|0);
 $14 = (($13) + 1)|0;
 $bl = $14;
 $bb = $bufb;
 while(1) {
  $15 = $bb;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)==(32);
  if (!($18)) {
   break;
  }
  $19 = $bl;
  $20 = (($19) + -1)|0;
  $bl = $20;
  $21 = $bb;
  $22 = ((($21)) + 1|0);
  $bb = $22;
 }
 $23 = HEAP32[12632>>2]|0;
 $24 = $al;
 $25 = (($23) + ($24))|0;
 $26 = $bl;
 $27 = (($25) + ($26))|0;
 $28 = (($27) + 3)|0;
 $29 = HEAP32[12656>>2]|0;
 $30 = ($28|0)>=($29|0);
 if ($30) {
  _donewrec();
 }
 $31 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$31 & 15](32);
 $32 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$32 & 15](40);
 $33 = $ba;
 _l_put($33);
 $34 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$34 & 15](44);
 $35 = HEAP32[12632>>2]|0;
 $36 = $bl;
 $37 = (($35) + ($36))|0;
 $38 = HEAP32[12656>>2]|0;
 $39 = ($37|0)>=($38|0);
 if (!($39)) {
  $42 = $bb;
  _l_put($42);
  $43 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$43 & 15](41);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[12628>>2]|0;
 (FUNCTION_TABLE_i[$40 & 15]()|0);
 $41 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$41 & 15](32);
 $42 = $bb;
 _l_put($42);
 $43 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$43 & 15](41);
 STACKTOP = sp;return;
}
function _lwrt_L($n,$len) {
 $n = $n|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $n;
 $1 = $len;
 $2 = HEAP32[12632>>2]|0;
 $3 = (($2) + 2)|0;
 $4 = HEAP32[12656>>2]|0;
 $5 = ($3|0)>=($4|0);
 if ($5) {
  _donewrec();
 }
 $6 = $1;
 (_wrt_L($0,2,$6)|0);
 STACKTOP = sp;return;
}
function _lwrt_A($p,$len) {
 $p = $p|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $a = 0, $or$cond = 0, $p1 = 0;
 var $pe = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $p;
 $1 = $len;
 $a = 0;
 $2 = $0;
 $3 = $1;
 $4 = (($2) + ($3)|0);
 $pe = $4;
 $5 = HEAP32[12660>>2]|0;
 $6 = ($5|0)!=(0);
 L1: do {
  if ($6) {
   $a = 3;
   $7 = $1;
   $8 = ($7|0)>(1);
   if ($8) {
    $9 = $1;
    $10 = (($9) - 1)|0;
    $11 = $0;
    $12 = (($11) + ($10)|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $15 = ($14|0)==(32);
    if ($15) {
     while(1) {
      $16 = $1;
      $17 = (($16) + -1)|0;
      $1 = $17;
      $18 = ($17|0)>(1);
      if (!($18)) {
       break;
      }
      $19 = $1;
      $20 = (($19) - 1)|0;
      $21 = $0;
      $22 = (($21) + ($20)|0);
      $23 = HEAP8[$22>>0]|0;
      $24 = $23 << 24 >> 24;
      $25 = ($24|0)==(32);
      if (!($25)) {
       break;
      }
     }
     $26 = $0;
     $27 = $1;
     $28 = (($26) + ($27)|0);
     $pe = $28;
    }
   }
   $29 = $0;
   $p1 = $29;
   while(1) {
    $30 = $p1;
    $31 = $pe;
    $32 = ($30>>>0)<($31>>>0);
    if (!($32)) {
     break L1;
    }
    $33 = $p1;
    $34 = ((($33)) + 1|0);
    $p1 = $34;
    $35 = HEAP8[$33>>0]|0;
    $36 = $35 << 24 >> 24;
    $37 = ($36|0)==(39);
    if (!($37)) {
     continue;
    }
    $38 = $a;
    $39 = (($38) + 1)|0;
    $a = $39;
   }
  }
 } while(0);
 $40 = HEAP32[12632>>2]|0;
 $41 = $1;
 $42 = (($40) + ($41))|0;
 $43 = $a;
 $44 = (($42) + ($43))|0;
 $45 = HEAP32[12656>>2]|0;
 $46 = ($44|0)>=($45|0);
 if ($46) {
  _donewrec();
 }
 $47 = $a;
 $48 = ($47|0)==(0);
 $49 = HEAP32[12632>>2]|0;
 $50 = ($49|0)!=(0);
 $or$cond = $48 & $50;
 if (!($or$cond)) {
  $51 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$51 & 15](32);
 }
 $52 = $a;
 $53 = ($52|0)!=(0);
 if (!($53)) {
  while(1) {
   $69 = $0;
   $70 = $pe;
   $71 = ($69>>>0)<($70>>>0);
   if (!($71)) {
    break;
   }
   $72 = HEAP32[12612>>2]|0;
   $73 = $0;
   $74 = ((($73)) + 1|0);
   $0 = $74;
   $75 = HEAP8[$73>>0]|0;
   $76 = $75 << 24 >> 24;
   FUNCTION_TABLE_vi[$72 & 15]($76);
  }
  STACKTOP = sp;return;
 }
 $54 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$54 & 15](39);
 while(1) {
  $55 = $0;
  $56 = $pe;
  $57 = ($55>>>0)<($56>>>0);
  if (!($57)) {
   break;
  }
  $58 = $0;
  $59 = HEAP8[$58>>0]|0;
  $60 = $59 << 24 >> 24;
  $61 = ($60|0)==(39);
  if ($61) {
   $62 = HEAP32[12612>>2]|0;
   FUNCTION_TABLE_vi[$62 & 15](39);
  }
  $63 = HEAP32[12612>>2]|0;
  $64 = $0;
  $65 = ((($64)) + 1|0);
  $0 = $65;
  $66 = HEAP8[$64>>0]|0;
  $67 = $66 << 24 >> 24;
  FUNCTION_TABLE_vi[$63 & 15]($67);
 }
 $68 = HEAP32[12612>>2]|0;
 FUNCTION_TABLE_vi[$68 & 15](39);
 STACKTOP = sp;return;
}
function _donewrec() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[12632>>2]|0;
 $1 = ($0|0)!=(0);
 if (!($1)) {
  return;
 }
 $2 = HEAP32[12628>>2]|0;
 (FUNCTION_TABLE_i[$2 & 15]()|0);
 return;
}
function _l_g($buf,$n) {
 $buf = $buf|0;
 $n = +$n;
 var $0 = 0, $1 = 0.0, $10 = 0.0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0.0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $b = 0, $c = 0, $c1 = 0;
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 8|0;
 $0 = $buf;
 $1 = $n;
 $2 = $0;
 $b = $2;
 $3 = $b;
 $4 = ((($3)) + 1|0);
 $b = $4;
 HEAP8[$3>>0] = 32;
 $5 = $1;
 $6 = $5 < 0.0;
 $7 = $b;
 $8 = ((($7)) + 1|0);
 $b = $8;
 if ($6) {
  HEAP8[$7>>0] = 45;
  $9 = $1;
  $10 = -$9;
  $1 = $10;
 } else {
  HEAP8[$7>>0] = 32;
 }
 $11 = $1;
 $12 = $11 == 0.0;
 $13 = $b;
 L5: do {
  if ($12) {
   $14 = ((($13)) + 1|0);
   $b = $14;
   HEAP8[$13>>0] = 48;
   $15 = $b;
   $16 = ((($15)) + 1|0);
   $b = $16;
   HEAP8[$15>>0] = 46;
   $17 = $b;
   HEAP8[$17>>0] = 0;
  } else {
   $18 = $1;
   HEAPF64[$vararg_buffer>>3] = $18;
   (_sprintf($13,19578,$vararg_buffer)|0);
   $19 = $b;
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   switch ($21|0) {
   case 48:  {
    while(1) {
     $22 = $b;
     $23 = ((($22)) + 1|0);
     $24 = HEAP8[$23>>0]|0;
     $25 = $b;
     HEAP8[$25>>0] = $24;
     $26 = ($24<<24>>24)!=(0);
     if (!($26)) {
      break L5;
     }
     $27 = $b;
     $28 = ((($27)) + 1|0);
     $b = $28;
    }
    break;
   }
   case 78: case 110: case 73: case 105:  {
    while(1) {
     $29 = $b;
     $30 = ((($29)) + 1|0);
     $b = $30;
     $31 = HEAP8[$30>>0]|0;
     $32 = ($31<<24>>24)!=(0);
     if (!($32)) {
      break;
     }
    }
    break;
   }
   default: {
    L9: while(1) {
     $33 = $b;
     $34 = HEAP8[$33>>0]|0;
     $35 = $34 << 24 >> 24;
     switch ($35|0) {
     case 0:  {
      label = 11;
      break L9;
      break;
     }
     case 46:  {
      label = 12;
      break L9;
      break;
     }
     case 69:  {
      label = 13;
      break L9;
      break;
     }
     default: {
     }
     }
     $50 = $b;
     $51 = ((($50)) + 1|0);
     $b = $51;
    }
    if ((label|0) == 11) {
     $36 = $b;
     $37 = ((($36)) + 1|0);
     $b = $37;
     HEAP8[$36>>0] = 46;
     $38 = $b;
     HEAP8[$38>>0] = 0;
     break L5;
    }
    else if ((label|0) == 12) {
     while(1) {
      label = 0;
      $39 = $b;
      $40 = ((($39)) + 1|0);
      $b = $40;
      $41 = HEAP8[$40>>0]|0;
      $42 = ($41<<24>>24)!=(0);
      if ($42) {
       label = 12;
      } else {
       break;
      }
     }
    }
    else if ((label|0) == 13) {
     $c1 = 46;
     $c = 69;
     while(1) {
      $43 = $c1;
      $44 = $b;
      HEAP8[$44>>0] = $43;
      $45 = ($43<<24>>24)!=(0);
      if (!($45)) {
       break L5;
      }
      $46 = $c;
      $c1 = $46;
      $47 = $b;
      $48 = ((($47)) + 1|0);
      $b = $48;
      $49 = HEAP8[$48>>0]|0;
      $c = $49;
     }
    }
   }
   }
  }
 } while(0);
 $52 = $b;
 $53 = $0;
 $54 = $52;
 $55 = $53;
 $56 = (($54) - ($55))|0;
 STACKTOP = sp;return ($56|0);
}
function _l_put($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $c = 0, $pn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $s;
 $1 = HEAP32[12612>>2]|0;
 $pn = $1;
 while(1) {
  $2 = $0;
  $3 = ((($2)) + 1|0);
  $0 = $3;
  $4 = HEAP8[$2>>0]|0;
  $5 = $4 << 24 >> 24;
  $c = $5;
  $6 = ($5|0)!=(0);
  if (!($6)) {
   break;
  }
  $7 = $pn;
  $8 = $c;
  FUNCTION_TABLE_vi[$7 & 15]($8);
 }
 STACKTOP = sp;return;
}
function _f__putbuf($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $s = 0, $se = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $c;
 $1 = HEAP32[12640>>2]|0;
 $2 = HEAP32[12632>>2]|0;
 $3 = ($1|0)>($2|0);
 if ($3) {
  $4 = HEAP32[12640>>2]|0;
  HEAP32[12632>>2] = $4;
 }
 $5 = HEAP32[12632>>2]|0;
 $6 = (($5) + 1)|0;
 $n = $6;
 $7 = $n;
 $8 = HEAP32[12688>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  $10 = $n;
  $11 = HEAP32[12632>>2]|0;
  _f__bufadj($10,$11);
 }
 $12 = HEAP32[12692>>2]|0;
 $s = $12;
 $13 = $s;
 $14 = HEAP32[12632>>2]|0;
 $15 = (($13) + ($14)|0);
 $se = $15;
 $16 = $0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = $0;
  $19 = $18&255;
  $20 = $se;
  $21 = ((($20)) + 1|0);
  $se = $21;
  HEAP8[$20>>0] = $19;
 }
 $22 = $se;
 HEAP8[$22>>0] = 0;
 while(1) {
  $23 = $s;
  $24 = HEAP32[12588>>2]|0;
  (_fputs($23,$24)|0);
  $25 = $s;
  $26 = (_strlen($25)|0);
  $27 = $s;
  $28 = (($27) + ($26)|0);
  $s = $28;
  $29 = $s;
  $30 = $se;
  $31 = ($29>>>0)>=($30>>>0);
  if ($31) {
   break;
  }
  $32 = $s;
  $33 = ((($32)) + 1|0);
  $s = $33;
  $34 = HEAP8[$32>>0]|0;
  $35 = $34 << 24 >> 24;
  $36 = HEAP32[12588>>2]|0;
  (_putc($35,$36)|0);
 }
 STACKTOP = sp;return 0;
}
function _x_putc($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $c;
 $1 = HEAP32[12632>>2]|0;
 $2 = HEAP32[12688>>2]|0;
 $3 = ($1|0)>=($2|0);
 if ($3) {
  $4 = HEAP32[12632>>2]|0;
  $5 = HEAP32[12688>>2]|0;
  _f__bufadj($4,$5);
 }
 $6 = $0;
 $7 = $6&255;
 $8 = HEAP32[12632>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[12632>>2] = $9;
 $10 = HEAP32[12692>>2]|0;
 $11 = (($10) + ($8)|0);
 HEAP8[$11>>0] = $7;
 STACKTOP = sp;return;
}
function _f_open($a) {
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $b = 0, $buf = 0, $n = 0, $rv = 0, $s = 0, $tf = 0, $ufmt = 0;
 var $vararg_buffer = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $buf = sp + 48|0;
 $x = sp + 16|0;
 $n = sp + 4|0;
 $1 = $a;
 HEAP32[12580>>2] = 1;
 $2 = $1;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(100);
 if (!($5)) {
  $6 = $1;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)<(0);
  if (!($9)) {
   $14 = HEAP32[12584>>2]|0;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    _f_init();
   }
   $16 = $1;
   $17 = ((($16)) + 4|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (7764 + (($18*48)|0)|0);
   $b = $19;
   HEAP32[7760>>2] = $19;
   $20 = $b;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   L7: do {
    if ($22) {
     $23 = $1;
     $24 = ((($23)) + 8|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = ($25|0)==(0|0);
     do {
      if (!($26)) {
       $47 = $1;
       $48 = ((($47)) + 8|0);
       $49 = HEAP32[$48>>2]|0;
       $50 = $1;
       $51 = ((($50)) + 12|0);
       $52 = HEAP32[$51>>2]|0;
       _g_char($49,$52,$buf);
       $53 = (_f__inode($buf,$n)|0);
       $54 = $b;
       $55 = ((($54)) + 8|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = ($53|0)==($56|0);
       if ($57) {
        $58 = HEAP32[$n>>2]|0;
        $59 = $b;
        $60 = ((($59)) + 12|0);
        $61 = HEAP32[$60>>2]|0;
        $62 = ($58|0)==($61|0);
        if ($62) {
         break;
        }
       }
       $63 = $1;
       $64 = ((($63)) + 4|0);
       $65 = HEAP32[$64>>2]|0;
       $66 = ((($x)) + 4|0);
       HEAP32[$66>>2] = $65;
       $67 = ((($x)) + 8|0);
       HEAP32[$67>>2] = 0;
       $68 = $1;
       $69 = HEAP32[$68>>2]|0;
       HEAP32[$x>>2] = $69;
       $70 = (_f_clos($x)|0);
       $rv = $70;
       $71 = ($70|0)!=(0);
       if (!($71)) {
        break L7;
       }
       $72 = $rv;
       $0 = $72;
       $312 = $0;
       STACKTOP = sp;return ($312|0);
      }
     } while(0);
     $27 = $1;
     $28 = ((($27)) + 32|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = ($29|0)!=(0|0);
     if ($30) {
      $31 = $1;
      $32 = ((($31)) + 32|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = HEAP8[$33>>0]|0;
      $35 = $34 << 24 >> 24;
      $36 = ($35|0)==(122);
      if ($36) {
       $44 = 1;
      } else {
       $37 = $1;
       $38 = ((($37)) + 32|0);
       $39 = HEAP32[$38>>2]|0;
       $40 = HEAP8[$39>>0]|0;
       $41 = $40 << 24 >> 24;
       $42 = ($41|0)==(90);
       $44 = $42;
      }
      $43 = $44&1;
      $45 = $b;
      $46 = ((($45)) + 32|0);
      HEAP32[$46>>2] = $43;
     }
     $0 = 0;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
   } while(0);
   $73 = $1;
   $74 = ((($73)) + 28|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = $b;
   $77 = ((($76)) + 16|0);
   HEAP32[$77>>2] = $75;
   $78 = $1;
   $79 = ((($78)) + 32|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = ($80|0)!=(0|0);
   if ($81) {
    $82 = $1;
    $83 = ((($82)) + 32|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP8[$84>>0]|0;
    $86 = $85 << 24 >> 24;
    $87 = ($86|0)==(122);
    if ($87) {
     $95 = 1;
    } else {
     $88 = $1;
     $89 = ((($88)) + 32|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP8[$90>>0]|0;
     $92 = $91 << 24 >> 24;
     $93 = ($92|0)==(90);
     $95 = $93;
    }
   } else {
    $95 = 0;
   }
   $94 = $95&1;
   $96 = $b;
   $97 = ((($96)) + 32|0);
   HEAP32[$97>>2] = $94;
   $98 = $1;
   $99 = ((($98)) + 24|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($100|0)==(0|0);
   do {
    if ($101) {
     $102 = $b;
     $103 = ((($102)) + 16|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = ($104|0)>(0);
     $106 = $b;
     $107 = ((($106)) + 24|0);
     if ($105) {
      HEAP32[$107>>2] = 0;
      break;
     } else {
      HEAP32[$107>>2] = 1;
      break;
     }
    } else {
     $108 = $1;
     $109 = ((($108)) + 24|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = HEAP8[$110>>0]|0;
     $112 = $111 << 24 >> 24;
     $113 = ($112|0)==(102);
     if (!($113)) {
      $114 = $1;
      $115 = ((($114)) + 24|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = HEAP8[$116>>0]|0;
      $118 = $117 << 24 >> 24;
      $119 = ($118|0)==(70);
      if (!($119)) {
       $122 = $b;
       $123 = ((($122)) + 24|0);
       HEAP32[$123>>2] = 0;
       break;
      }
     }
     $120 = $b;
     $121 = ((($120)) + 24|0);
     HEAP32[$121>>2] = 1;
    }
   } while(0);
   $124 = $b;
   $125 = ((($124)) + 24|0);
   $126 = HEAP32[$125>>2]|0;
   $ufmt = $126;
   $127 = $1;
   $128 = ((($127)) + 8|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = ($129|0)!=(0|0);
   if ($130) {
    $131 = $1;
    $132 = ((($131)) + 8|0);
    $133 = HEAP32[$132>>2]|0;
    $134 = $1;
    $135 = ((($134)) + 12|0);
    $136 = HEAP32[$135>>2]|0;
    _g_char($133,$136,$buf);
    $137 = HEAP8[$buf>>0]|0;
    $138 = ($137<<24>>24)!=(0);
    if (!($138)) {
     $139 = $1;
     $140 = HEAP32[$139>>2]|0;
     $141 = ($140|0)!=(0);
     if ($141) {
      $142 = (___errno_location()|0);
      HEAP32[$142>>2] = 107;
     } else {
      $143 = $1;
      _opn_err(107,19583,$143);
     }
     $0 = 107;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
   } else {
    $144 = $1;
    $145 = ((($144)) + 4|0);
    $146 = HEAP32[$145>>2]|0;
    HEAP32[$vararg_buffer>>2] = $146;
    (_sprintf($buf,19588,$vararg_buffer)|0);
   }
   $147 = $b;
   $148 = ((($147)) + 44|0);
   HEAP32[$148>>2] = 0;
   $149 = $b;
   $150 = ((($149)) + 36|0);
   HEAP32[$150>>2] = 0;
   $151 = $b;
   $152 = ((($151)) + 40|0);
   HEAP32[$152>>2] = 0;
   $153 = $b;
   HEAP32[$153>>2] = 0;
   $154 = $b;
   $155 = ((($154)) + 28|0);
   HEAP32[$155>>2] = 3;
   $156 = $1;
   $157 = ((($156)) + 16|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = ($158|0)!=(0|0);
   if ($159) {
    $160 = $1;
    $161 = ((($160)) + 16|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP8[$162>>0]|0;
    $164 = $163 << 24 >> 24;
    $165 = $164;
   } else {
    $165 = 117;
   }
   switch ($165|0) {
   case 79: case 111:  {
    $166 = (_access($buf,0)|0);
    $167 = ($166|0)!=(0);
    if ($167) {
     $168 = $1;
     $169 = HEAP32[$168>>2]|0;
     $170 = ($169|0)!=(0);
     $171 = (___errno_location()|0);
     $172 = HEAP32[$171>>2]|0;
     if ($170) {
      $173 = (___errno_location()|0);
      HEAP32[$173>>2] = $172;
     } else {
      $174 = $1;
      _opn_err($172,19583,$174);
     }
     $175 = (___errno_location()|0);
     $176 = HEAP32[$175>>2]|0;
     $0 = $176;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
    break;
   }
   case 83: case 115:  {
    $177 = $b;
    $178 = ((($177)) + 44|0);
    HEAP32[$178>>2] = 1;
    $179 = (_tmpfile()|0);
    $180 = $b;
    HEAP32[$180>>2] = $179;
    $181 = ($179|0)!=(0|0);
    if ($181) {
     $191 = $b;
     $192 = ((($191)) + 4|0);
     HEAP32[$192>>2] = 0;
     $193 = $b;
     $194 = ((($193)) + 12|0);
     HEAP32[$194>>2] = -1;
     $195 = $b;
     $196 = ((($195)) + 8|0);
     HEAP32[$196>>2] = -1;
     $197 = $b;
     $198 = ((($197)) + 20|0);
     HEAP32[$198>>2] = 1;
     $0 = 0;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
    $182 = $1;
    $183 = HEAP32[$182>>2]|0;
    $184 = ($183|0)!=(0);
    $185 = (___errno_location()|0);
    $186 = HEAP32[$185>>2]|0;
    if ($184) {
     $187 = (___errno_location()|0);
     HEAP32[$187>>2] = $186;
    } else {
     $188 = $1;
     _opn_err($186,19583,$188);
    }
    $189 = (___errno_location()|0);
    $190 = HEAP32[$189>>2]|0;
    $0 = $190;
    $312 = $0;
    STACKTOP = sp;return ($312|0);
    break;
   }
   case 78: case 110:  {
    $199 = (_access($buf,0)|0);
    $200 = ($199|0)!=(0);
    if ($200) {
     label = 57;
    } else {
     $201 = $1;
     $202 = HEAP32[$201>>2]|0;
     $203 = ($202|0)!=(0);
     if ($203) {
      $204 = (___errno_location()|0);
      HEAP32[$204>>2] = 128;
     } else {
      $205 = $1;
      _opn_err(128,19583,$205);
     }
     $0 = 128;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
    break;
   }
   case 82: case 114:  {
    label = 57;
    break;
   }
   default: {
   }
   }
   if ((label|0) == 57) {
    $206 = HEAP32[12672>>2]|0;
    $207 = (_fopen($buf,$206)|0);
    $tf = $207;
    $208 = ($207|0)!=(0|0);
    if ($208) {
     $209 = $tf;
     (_fclose($209)|0);
    }
   }
   $210 = (_strlen($buf)|0);
   $211 = (($210) + 1)|0;
   $212 = (_malloc($211)|0);
   $213 = $b;
   $214 = ((($213)) + 4|0);
   HEAP32[$214>>2] = $212;
   $215 = $b;
   $216 = ((($215)) + 4|0);
   $217 = HEAP32[$216>>2]|0;
   $218 = ($217|0)==(0|0);
   if ($218) {
    $219 = $1;
    $220 = HEAP32[$219>>2]|0;
    $221 = ($220|0)!=(0);
    if ($221) {
     $222 = (___errno_location()|0);
     HEAP32[$222>>2] = 113;
    } else {
     $223 = $1;
     _opn_err(113,19597,$223);
    }
    $0 = 113;
    $312 = $0;
    STACKTOP = sp;return ($312|0);
   }
   $224 = $b;
   $225 = ((($224)) + 4|0);
   $226 = HEAP32[$225>>2]|0;
   (_strcpy($226,$buf)|0);
   $227 = $1;
   $228 = ((($227)) + 20|0);
   $229 = HEAP32[$228>>2]|0;
   $s = $229;
   $230 = ($229|0)!=(0|0);
   if ($230) {
    $231 = $b;
    $232 = ((($231)) + 16|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = ($233|0)!=(0);
    if ($234) {
     $ufmt = 0;
    }
   }
   $235 = $ufmt;
   $236 = $235 | 2;
   $237 = (12672 + ($236<<2)|0);
   $238 = HEAP32[$237>>2]|0;
   $239 = (_fopen($buf,$238)|0);
   $tf = $239;
   $240 = ($239|0)!=(0|0);
   do {
    if (!($240)) {
     $241 = $ufmt;
     $242 = (12664 + ($241<<2)|0);
     $243 = HEAP32[$242>>2]|0;
     $244 = (_fopen($buf,$243)|0);
     $tf = $244;
     $245 = ($244|0)!=(0|0);
     if ($245) {
      $246 = $b;
      $247 = ((($246)) + 28|0);
      HEAP32[$247>>2] = 1;
      break;
     }
     $248 = $ufmt;
     $249 = (12672 + ($248<<2)|0);
     $250 = HEAP32[$249>>2]|0;
     $251 = (_fopen($buf,$250)|0);
     $tf = $251;
     $252 = ($251|0)!=(0|0);
     if ($252) {
      $253 = $b;
      $254 = ((($253)) + 40|0);
      HEAP32[$254>>2] = 1;
      $255 = $b;
      $256 = ((($255)) + 28|0);
      HEAP32[$256>>2] = 2;
      break;
     }
     $257 = $1;
     $258 = HEAP32[$257>>2]|0;
     $259 = ($258|0)!=(0);
     $260 = (___errno_location()|0);
     $261 = HEAP32[$260>>2]|0;
     if ($259) {
      $262 = (___errno_location()|0);
      HEAP32[$262>>2] = $261;
     } else {
      _f__fatal($261,19583);
     }
     $263 = (___errno_location()|0);
     $264 = HEAP32[$263>>2]|0;
     $0 = $264;
     $312 = $0;
     STACKTOP = sp;return ($312|0);
    }
   } while(0);
   $265 = $tf;
   $266 = $b;
   HEAP32[$266>>2] = $265;
   $267 = (_f__canseek($265)|0);
   $268 = $b;
   $269 = ((($268)) + 20|0);
   HEAP32[$269>>2] = $267;
   $270 = $b;
   $271 = ((($270)) + 12|0);
   $272 = (_f__inode($buf,$271)|0);
   $273 = $b;
   $274 = ((($273)) + 8|0);
   HEAP32[$274>>2] = $272;
   $275 = ($272|0)==(-1);
   if ($275) {
    $276 = $1;
    $277 = HEAP32[$276>>2]|0;
    $278 = ($277|0)!=(0);
    if ($278) {
     $279 = (___errno_location()|0);
     HEAP32[$279>>2] = 108;
    } else {
     $280 = $1;
     _opn_err(108,19583,$280);
    }
    $0 = 108;
    $312 = $0;
    STACKTOP = sp;return ($312|0);
   }
   $281 = $b;
   $282 = ((($281)) + 20|0);
   $283 = HEAP32[$282>>2]|0;
   $284 = ($283|0)!=(0);
   do {
    if ($284) {
     $285 = $1;
     $286 = ((($285)) + 28|0);
     $287 = HEAP32[$286>>2]|0;
     $288 = ($287|0)!=(0);
     if ($288) {
      $289 = $b;
      $290 = HEAP32[$289>>2]|0;
      _rewind($290);
      break;
     }
     $291 = $1;
     $292 = ((($291)) + 20|0);
     $293 = HEAP32[$292>>2]|0;
     $s = $293;
     $294 = ($293|0)!=(0|0);
     if ($294) {
      $295 = $s;
      $296 = HEAP8[$295>>0]|0;
      $297 = $296 << 24 >> 24;
      $298 = ($297|0)==(97);
      if (!($298)) {
       $299 = $s;
       $300 = HEAP8[$299>>0]|0;
       $301 = $300 << 24 >> 24;
       $302 = ($301|0)==(65);
       if (!($302)) {
        break;
       }
      }
      $303 = $b;
      $304 = HEAP32[$303>>2]|0;
      $305 = (_fseek($304,0,2)|0);
      $306 = ($305|0)!=(0);
      if ($306) {
       $307 = $1;
       $308 = HEAP32[$307>>2]|0;
       $309 = ($308|0)!=(0);
       if ($309) {
        $310 = (___errno_location()|0);
        HEAP32[$310>>2] = 129;
       } else {
        $311 = $1;
        _opn_err(129,19583,$311);
       }
       $0 = 129;
       $312 = $0;
       STACKTOP = sp;return ($312|0);
      }
     }
    }
   } while(0);
   $0 = 0;
   $312 = $0;
   STACKTOP = sp;return ($312|0);
  }
 }
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = (___errno_location()|0);
  HEAP32[$13>>2] = 101;
 } else {
  _f__fatal(101,19583);
 }
 $0 = 101;
 $312 = $0;
 STACKTOP = sp;return ($312|0);
}
function _fk_open($seq,$fmt,$n) {
 $seq = $seq|0;
 $fmt = $fmt|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $nbuf = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $nbuf = sp + 52|0;
 $a = sp + 4|0;
 $0 = $seq;
 $1 = $fmt;
 $2 = $n;
 $3 = $2;
 HEAP32[$vararg_buffer>>2] = $3;
 (_sprintf($nbuf,19588,$vararg_buffer)|0);
 HEAP32[$a>>2] = 1;
 $4 = $2;
 $5 = ((($a)) + 4|0);
 HEAP32[$5>>2] = $4;
 $6 = ((($a)) + 8|0);
 HEAP32[$6>>2] = $nbuf;
 $7 = (_strlen($nbuf)|0);
 $8 = ((($a)) + 12|0);
 HEAP32[$8>>2] = $7;
 $9 = ((($a)) + 16|0);
 HEAP32[$9>>2] = 0;
 $10 = $0;
 $11 = ($10|0)==(3);
 $12 = $11 ? 19606 : 19608;
 $13 = ((($a)) + 20|0);
 HEAP32[$13>>2] = $12;
 $14 = $1;
 $15 = ($14|0)==(5);
 $16 = $15 ? 19610 : 19612;
 $17 = ((($a)) + 24|0);
 HEAP32[$17>>2] = $16;
 $18 = $0;
 $19 = ($18|0)==(4);
 $20 = $19 ? 1 : 0;
 $21 = ((($a)) + 28|0);
 HEAP32[$21>>2] = $20;
 $22 = ((($a)) + 32|0);
 HEAP32[$22>>2] = 0;
 $23 = (_f_open($a)|0);
 STACKTOP = sp;return ($23|0);
}
function _f__bufadj($n,$c) {
 $n = $n|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $len = 0, $nbuf = 0, $s = 0, $t = 0, $te = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $c;
 $2 = HEAP32[12692>>2]|0;
 $3 = ($2|0)==(19631|0);
 if ($3) {
  HEAP32[12688>>2] = 1024;
 }
 while(1) {
  $4 = HEAP32[12688>>2]|0;
  $5 = $0;
  $6 = ($4|0)<=($5|0);
  $7 = HEAP32[12688>>2]|0;
  if (!($6)) {
   break;
  }
  $8 = $7 << 1;
  HEAP32[12688>>2] = $8;
 }
 $len = $7;
 $9 = $len;
 $10 = HEAP32[12688>>2]|0;
 $11 = ($9|0)!=($10|0);
 if ($11) {
  label = 7;
 } else {
  $12 = $len;
  $13 = (_malloc($12)|0);
  $nbuf = $13;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  _f__fatal(113,20031);
 }
 $15 = $nbuf;
 $s = $15;
 $16 = HEAP32[12692>>2]|0;
 $t = $16;
 $17 = $t;
 $18 = $1;
 $19 = (($17) + ($18)|0);
 $te = $19;
 while(1) {
  $20 = $t;
  $21 = $te;
  $22 = ($20>>>0)<($21>>>0);
  if (!($22)) {
   break;
  }
  $23 = $t;
  $24 = ((($23)) + 1|0);
  $t = $24;
  $25 = HEAP8[$23>>0]|0;
  $26 = $s;
  $27 = ((($26)) + 1|0);
  $s = $27;
  HEAP8[$26>>0] = $25;
 }
 $28 = HEAP32[12692>>2]|0;
 $29 = ($28|0)!=(19631|0);
 if (!($29)) {
  $31 = $nbuf;
  HEAP32[12692>>2] = $31;
  STACKTOP = sp;return;
 }
 $30 = HEAP32[12692>>2]|0;
 _free($30);
 $31 = $nbuf;
 HEAP32[12692>>2] = $31;
 STACKTOP = sp;return;
}
function _opn_err($m,$s,$a) {
 $m = $m|0;
 $s = $s|0;
 $a = $a|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $m;
 $1 = $s;
 $2 = $a;
 $3 = $2;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $24 = $0;
  $25 = $1;
  _f__fatal($24,$25);
  STACKTOP = sp;return;
 }
 $7 = $2;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[12688>>2]|0;
 $11 = ($9|0)>=($10|0);
 if ($11) {
  $12 = $2;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  _f__bufadj($14,0);
 }
 $15 = $2;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $2;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[12692>>2]|0;
 $22 = HEAP32[7760>>2]|0;
 $23 = ((($22)) + 4|0);
 HEAP32[$23>>2] = $21;
 _g_char($17,$20,$21);
 $24 = $0;
 $25 = $1;
 _f__fatal($24,$25);
 STACKTOP = sp;return;
}
function _g_char($a,$alen,$b) {
 $a = $a|0;
 $alen = $alen|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $a;
 $1 = $alen;
 $2 = $b;
 $3 = $0;
 $4 = $1;
 $5 = (($3) + ($4)|0);
 $x = $5;
 $6 = $2;
 $7 = $1;
 $8 = (($6) + ($7)|0);
 $y = $8;
 while(1) {
  $9 = $x;
  $10 = $0;
  $11 = ($9>>>0)<=($10>>>0);
  if ($11) {
   label = 3;
   break;
  }
  $13 = $x;
  $14 = ((($13)) + -1|0);
  $x = $14;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = ($16|0)!=(32);
  $18 = $y;
  $19 = ((($18)) + -1|0);
  $y = $19;
  if ($17) {
   break;
  }
 }
 if ((label|0) == 3) {
  $12 = $2;
  HEAP8[$12>>0] = 0;
  STACKTOP = sp;return;
 }
 HEAP8[$18>>0] = 0;
 while(1) {
  $20 = $x;
  $21 = HEAP8[$20>>0]|0;
  $22 = $y;
  $23 = ((($22)) + -1|0);
  $y = $23;
  HEAP8[$22>>0] = $21;
  $24 = $x;
  $25 = ((($24)) + -1|0);
  $x = $25;
  $26 = $0;
  $27 = ($24>>>0)>($26>>>0);
  if (!($27)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _f__inode($a,$dev) {
 $a = $a|0;
 $dev = $dev|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $x = sp;
 $1 = $a;
 $2 = $dev;
 $3 = $1;
 $4 = (_stat($3,$x)|0);
 $5 = ($4|0)<(0);
 if ($5) {
  $0 = -1;
  $10 = $0;
  STACKTOP = sp;return ($10|0);
 } else {
  $6 = HEAP32[$x>>2]|0;
  $7 = $2;
  HEAP32[$7>>2] = $6;
  $8 = ((($x)) + 72|0);
  $9 = HEAP32[$8>>2]|0;
  $0 = $9;
  $10 = $0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _wrt_L($n,$len,$sz) {
 $n = $n|0;
 $len = $len|0;
 $sz = $sz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $n;
 $1 = $len;
 $2 = $sz;
 $3 = $2;
 $4 = (4)==($3|0);
 do {
  if ($4) {
   $5 = $0;
   $6 = HEAP32[$5>>2]|0;
   $x = $6;
  } else {
   $7 = $2;
   $8 = ($7|0)==(1);
   $9 = $0;
   if ($8) {
    $10 = HEAP8[$9>>0]|0;
    $11 = $10 << 24 >> 24;
    $x = $11;
    break;
   } else {
    $12 = HEAP16[$9>>1]|0;
    $13 = $12 << 16 >> 16;
    $x = $13;
    break;
   }
  }
 } while(0);
 $i = 0;
 while(1) {
  $14 = $i;
  $15 = $1;
  $16 = (($15) - 1)|0;
  $17 = ($14|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$18 & 15](32);
  $19 = $i;
  $20 = (($19) + 1)|0;
  $i = $20;
 }
 $21 = $x;
 $22 = ($21|0)!=(0);
 $23 = HEAP32[12612>>2]|0;
 if ($22) {
  FUNCTION_TABLE_vi[$23 & 15](84);
  STACKTOP = sp;return 0;
 } else {
  FUNCTION_TABLE_vi[$23 & 15](70);
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _w_ed($p,$ptr,$len) {
 $p = $p|0;
 $ptr = $ptr|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $1 = $p;
 $2 = $ptr;
 $3 = $len;
 $4 = HEAP32[12636>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_mv_cur()|0);
  $i = $6;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = $i;
   $0 = $8;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 L4: do {
  if ((label|0) == 4) {
   $9 = $1;
   $10 = HEAP32[$9>>2]|0;
   do {
    switch ($10|0) {
    case 7:  {
     break;
    }
    case 10:  {
     $21 = $2;
     $22 = $1;
     $23 = ((($22)) + 4|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = $1;
     $26 = ((($25)) + 8|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = $3;
     $29 = (_wrt_IM($21,$24,$27,$28,10)|0);
     $0 = $29;
     break L4;
     break;
    }
    case 32:  {
     $30 = $2;
     $31 = $1;
     $32 = ((($31)) + 4|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = $3;
     $35 = (_wrt_I($30,$33,$34,8)|0);
     $0 = $35;
     break L4;
     break;
    }
    case 34:  {
     $36 = $2;
     $37 = $1;
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = $1;
     $41 = ((($40)) + 8|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = $3;
     $44 = (_wrt_IM($36,$39,$42,$43,8)|0);
     $0 = $44;
     break L4;
     break;
    }
    case 29:  {
     $45 = $2;
     $46 = $1;
     $47 = ((($46)) + 4|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = $3;
     $50 = (_wrt_L($45,$48,$49)|0);
     $0 = $50;
     break L4;
     break;
    }
    case 30:  {
     $51 = $2;
     $52 = $3;
     $53 = (_wrt_A($51,$52)|0);
     $0 = $53;
     break L4;
     break;
    }
    case 31:  {
     $54 = $2;
     $55 = $1;
     $56 = ((($55)) + 4|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = $3;
     $59 = (_wrt_AW($54,$57,$58)|0);
     $0 = $59;
     break L4;
     break;
    }
    case 25: case 24: case 26:  {
     $60 = $2;
     $61 = $1;
     $62 = ((($61)) + 4|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = $1;
     $65 = ((($64)) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = $1;
     $68 = ((($67)) + 8|0);
     $69 = ((($68)) + 4|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = $3;
     $72 = (_wrt_E($60,$63,$66,$70,$71)|0);
     $0 = $72;
     break L4;
     break;
    }
    case 28: case 27:  {
     $73 = $2;
     $74 = $1;
     $75 = ((($74)) + 4|0);
     $76 = HEAP32[$75>>2]|0;
     $77 = $1;
     $78 = ((($77)) + 8|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = $1;
     $81 = ((($80)) + 8|0);
     $82 = ((($81)) + 4|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = $3;
     $85 = (_wrt_G($73,$76,$79,$83,$84)|0);
     $0 = $85;
     break L4;
     break;
    }
    case 23:  {
     $86 = $2;
     $87 = $1;
     $88 = ((($87)) + 4|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = $1;
     $91 = ((($90)) + 8|0);
     $92 = HEAP32[$91>>2]|0;
     $93 = $3;
     $94 = (_wrt_F($86,$89,$92,$93)|0);
     $0 = $94;
     break L4;
     break;
    }
    case 35:  {
     $95 = $2;
     $96 = $1;
     $97 = ((($96)) + 4|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = $3;
     $100 = (_wrt_Z($95,$98,0,$99)|0);
     $0 = $100;
     break L4;
     break;
    }
    case 36:  {
     $101 = $2;
     $102 = $1;
     $103 = ((($102)) + 4|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = $1;
     $106 = ((($105)) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = $3;
     $109 = (_wrt_Z($101,$104,$107,$108)|0);
     $0 = $109;
     break L4;
     break;
    }
    default: {
     $11 = HEAP32[12736>>2]|0;
     $12 = $1;
     $13 = HEAP32[$12>>2]|0;
     HEAP32[$vararg_buffer>>2] = $13;
     (_fprintf($11,20046,$vararg_buffer)|0);
     $14 = HEAP32[12564>>2]|0;
     _sig_die($14,1);
    }
    }
   } while(0);
   $15 = $2;
   $16 = $1;
   $17 = ((($16)) + 4|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = $3;
   $20 = (_wrt_I($15,$18,$19,10)|0);
   $0 = $20;
  }
 } while(0);
 $110 = $0;
 STACKTOP = sp;return ($110|0);
}
function _w_ned($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $1 = $p;
 $2 = $1;
 $3 = HEAP32[$2>>2]|0;
 switch ($3|0) {
 case 5:  {
  break;
 }
 case 15:  {
  $10 = $1;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[12632>>2]|0;
  $14 = (($12) - ($13))|0;
  $15 = (($14) - 1)|0;
  HEAP32[12636>>2] = $15;
  $0 = 1;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
  break;
 }
 case 13:  {
  $16 = $1;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[12636>>2]|0;
  $20 = (($19) - ($18))|0;
  HEAP32[12636>>2] = $20;
  $21 = HEAP32[12636>>2]|0;
  $22 = HEAP32[12632>>2]|0;
  $23 = (0 - ($22))|0;
  $24 = ($21|0)<($23|0);
  if ($24) {
   $25 = HEAP32[12632>>2]|0;
   $26 = (0 - ($25))|0;
   HEAP32[12636>>2] = $26;
  }
  $0 = 1;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
  break;
 }
 case 4: case 14:  {
  $27 = $1;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[12636>>2]|0;
  $31 = (($30) + ($29))|0;
  HEAP32[12636>>2] = $31;
  $0 = 1;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
  break;
 }
 case 11:  {
  $32 = $1;
  $33 = ((($32)) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (_wrt_AP($34)|0);
  $0 = $35;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
  break;
 }
 case 12:  {
  $36 = $1;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $1;
  $40 = ((($39)) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (_wrt_H($38,$41)|0);
  $0 = $42;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
  break;
 }
 default: {
  $4 = HEAP32[12736>>2]|0;
  $5 = $1;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$vararg_buffer>>2] = $6;
  (_fprintf($4,20073,$vararg_buffer)|0);
  $7 = HEAP32[12564>>2]|0;
  _sig_die($7,1);
 }
 }
 $8 = HEAP32[12628>>2]|0;
 $9 = (FUNCTION_TABLE_i[$8 & 15]()|0);
 $0 = $9;
 $43 = $0;
 STACKTOP = sp;return ($43|0);
}
function _mv_cur() {
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cursor = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = HEAP32[12636>>2]|0;
 $cursor = $1;
 HEAP32[12636>>2] = 0;
 $2 = HEAP32[12580>>2]|0;
 $3 = ($2|0)==(0);
 $4 = $cursor;
 if (!($3)) {
  $70 = ($4|0)>(0);
  L3: do {
   if ($70) {
    $71 = HEAP32[12640>>2]|0;
    $72 = HEAP32[12632>>2]|0;
    $73 = ($71|0)<=($72|0);
    if ($73) {
     while(1) {
      $74 = $cursor;
      $75 = ($74|0)>(0);
      if (!($75)) {
       break L3;
      }
      $76 = HEAP32[12612>>2]|0;
      FUNCTION_TABLE_vi[$76 & 15](32);
      $77 = $cursor;
      $78 = (($77) + -1)|0;
      $cursor = $78;
     }
    }
    $79 = HEAP32[12640>>2]|0;
    $80 = HEAP32[12632>>2]|0;
    $81 = $cursor;
    $82 = (($80) + ($81))|0;
    $83 = ($79|0)<=($82|0);
    if (!($83)) {
     $95 = $cursor;
     $96 = HEAP32[12632>>2]|0;
     $97 = (($96) + ($95))|0;
     HEAP32[12632>>2] = $97;
     break;
    }
    $84 = HEAP32[12640>>2]|0;
    $85 = HEAP32[12632>>2]|0;
    $86 = (($84) - ($85))|0;
    $87 = $cursor;
    $88 = (($87) - ($86))|0;
    $cursor = $88;
    $89 = HEAP32[12640>>2]|0;
    HEAP32[12632>>2] = $89;
    while(1) {
     $90 = $cursor;
     $91 = ($90|0)>(0);
     if (!($91)) {
      break L3;
     }
     $92 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$92 & 15](32);
     $93 = $cursor;
     $94 = (($93) + -1)|0;
     $cursor = $94;
    }
   } else {
    $98 = $cursor;
    $99 = ($98|0)<(0);
    if ($99) {
     $100 = $cursor;
     $101 = HEAP32[12632>>2]|0;
     $102 = (($100) + ($101))|0;
     $103 = ($102|0)<(0);
     if (!($103)) {
      $108 = HEAP32[12640>>2]|0;
      $109 = HEAP32[12632>>2]|0;
      $110 = ($108|0)<($109|0);
      if ($110) {
       $111 = HEAP32[12632>>2]|0;
       HEAP32[12640>>2] = $111;
      }
      $112 = $cursor;
      $113 = HEAP32[12632>>2]|0;
      $114 = (($113) + ($112))|0;
      HEAP32[12632>>2] = $114;
      break;
     }
     $104 = HEAP32[12596>>2]|0;
     $105 = HEAP32[$104>>2]|0;
     $106 = ($105|0)!=(0);
     if ($106) {
      $107 = (___errno_location()|0);
      HEAP32[$107>>2] = 110;
     } else {
      _f__fatal(110,20118);
     }
     $0 = 110;
     $115 = $0;
     STACKTOP = sp;return ($115|0);
    }
   }
  } while(0);
  $0 = 0;
  $115 = $0;
  STACKTOP = sp;return ($115|0);
 }
 $5 = ($4|0)<(0);
 L32: do {
  if ($5) {
   $6 = HEAP32[12640>>2]|0;
   $7 = HEAP32[12632>>2]|0;
   $8 = ($6|0)<($7|0);
   if ($8) {
    $9 = HEAP32[12632>>2]|0;
    HEAP32[12640>>2] = $9;
   }
   $10 = $cursor;
   $11 = HEAP32[12632>>2]|0;
   $12 = (($11) + ($10))|0;
   HEAP32[12632>>2] = $12;
   $13 = $cursor;
   $14 = HEAP32[12648>>2]|0;
   $15 = (($14) + ($13)|0);
   HEAP32[12648>>2] = $15;
   $16 = HEAP32[12632>>2]|0;
   $17 = ($16|0)<(0);
   if ($17) {
    $18 = HEAP32[12596>>2]|0;
    $19 = HEAP32[$18>>2]|0;
    $20 = ($19|0)!=(0);
    if ($20) {
     $21 = (___errno_location()|0);
     HEAP32[$21>>2] = 110;
    } else {
     _f__fatal(110,20118);
    }
    $0 = 110;
    $115 = $0;
    STACKTOP = sp;return ($115|0);
   }
  } else {
   $22 = $cursor;
   $23 = ($22|0)>(0);
   if ($23) {
    $24 = HEAP32[12632>>2]|0;
    $25 = $cursor;
    $26 = (($24) + ($25))|0;
    $27 = HEAP32[12600>>2]|0;
    $28 = ((($27)) + 16|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($26|0)>=($29|0);
    if ($30) {
     $31 = HEAP32[12596>>2]|0;
     $32 = HEAP32[$31>>2]|0;
     $33 = ($32|0)!=(0);
     if ($33) {
      $34 = (___errno_location()|0);
      HEAP32[$34>>2] = 110;
     } else {
      _f__fatal(110,20127);
     }
     $0 = 110;
     $115 = $0;
     STACKTOP = sp;return ($115|0);
    }
    $35 = HEAP32[12640>>2]|0;
    $36 = HEAP32[12632>>2]|0;
    $37 = ($35|0)<=($36|0);
    if ($37) {
     while(1) {
      $38 = $cursor;
      $39 = ($38|0)>(0);
      if (!($39)) {
       break L32;
      }
      $40 = HEAP32[12612>>2]|0;
      FUNCTION_TABLE_vi[$40 & 15](32);
      $41 = $cursor;
      $42 = (($41) + -1)|0;
      $cursor = $42;
     }
    }
    $43 = HEAP32[12640>>2]|0;
    $44 = HEAP32[12632>>2]|0;
    $45 = $cursor;
    $46 = (($44) + ($45))|0;
    $47 = ($43|0)<=($46|0);
    if (!($47)) {
     $64 = $cursor;
     $65 = HEAP32[12648>>2]|0;
     $66 = (($65) + ($64)|0);
     HEAP32[12648>>2] = $66;
     $67 = $cursor;
     $68 = HEAP32[12632>>2]|0;
     $69 = (($68) + ($67))|0;
     HEAP32[12632>>2] = $69;
     break;
    }
    $48 = HEAP32[12640>>2]|0;
    $49 = HEAP32[12632>>2]|0;
    $50 = (($48) - ($49))|0;
    $51 = $cursor;
    $52 = (($51) - ($50))|0;
    $cursor = $52;
    $53 = HEAP32[12640>>2]|0;
    $54 = HEAP32[12632>>2]|0;
    $55 = (($53) - ($54))|0;
    $56 = HEAP32[12648>>2]|0;
    $57 = (($56) + ($55)|0);
    HEAP32[12648>>2] = $57;
    $58 = HEAP32[12640>>2]|0;
    HEAP32[12632>>2] = $58;
    while(1) {
     $59 = $cursor;
     $60 = ($59|0)>(0);
     if (!($60)) {
      break L32;
     }
     $61 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$61 & 15](32);
     $62 = $cursor;
     $63 = (($62) + -1)|0;
     $cursor = $63;
    }
   }
  }
 } while(0);
 $0 = 0;
 $115 = $0;
 STACKTOP = sp;return ($115|0);
}
function _wrt_I($n,$w,$len,$base) {
 $n = $n|0;
 $w = $w|0;
 $len = $len|0;
 $base = $base|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ans = 0, $i = 0, $ndigit = 0;
 var $or$cond = 0, $sign = 0, $spare = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ndigit = sp + 20|0;
 $sign = sp + 16|0;
 $0 = $n;
 $1 = $w;
 $2 = $len;
 $3 = $base;
 $4 = $2;
 $5 = ($4|0)==(4);
 do {
  if ($5) {
   $6 = $0;
   $7 = HEAP32[$6>>2]|0;
   $x = $7;
  } else {
   $8 = $2;
   $9 = ($8|0)==(1);
   $10 = $0;
   if ($9) {
    $11 = HEAP8[$10>>0]|0;
    $12 = $11 << 24 >> 24;
    $x = $12;
    break;
   } else {
    $13 = HEAP16[$10>>1]|0;
    $14 = $13 << 16 >> 16;
    $x = $14;
    break;
   }
  }
 } while(0);
 $15 = $x;
 $16 = $3;
 $17 = (_f__icvt($15,$ndigit,$sign,$16)|0);
 $ans = $17;
 $18 = $1;
 $19 = HEAP32[$ndigit>>2]|0;
 $20 = (($18) - ($19))|0;
 $spare = $20;
 $21 = HEAP32[$sign>>2]|0;
 $22 = ($21|0)!=(0);
 $23 = HEAP32[12604>>2]|0;
 $24 = ($23|0)!=(0);
 $or$cond = $22 | $24;
 if ($or$cond) {
  $25 = $spare;
  $26 = (($25) + -1)|0;
  $spare = $26;
 }
 $27 = $spare;
 $28 = ($27|0)<(0);
 $i = 0;
 if ($28) {
  while(1) {
   $29 = $i;
   $30 = $1;
   $31 = ($29|0)<($30|0);
   if (!($31)) {
    break;
   }
   $32 = HEAP32[12612>>2]|0;
   FUNCTION_TABLE_vi[$32 & 15](42);
   $33 = $i;
   $34 = (($33) + 1)|0;
   $i = $34;
  }
  STACKTOP = sp;return 0;
 }
 while(1) {
  $35 = $i;
  $36 = $spare;
  $37 = ($35|0)<($36|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$38 & 15](32);
  $39 = $i;
  $40 = (($39) + 1)|0;
  $i = $40;
 }
 $41 = HEAP32[$sign>>2]|0;
 $42 = ($41|0)!=(0);
 if ($42) {
  $43 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$43 & 15](45);
 } else {
  $44 = HEAP32[12604>>2]|0;
  $45 = ($44|0)!=(0);
  if ($45) {
   $46 = HEAP32[12612>>2]|0;
   FUNCTION_TABLE_vi[$46 & 15](43);
  }
 }
 $i = 0;
 while(1) {
  $47 = $i;
  $48 = HEAP32[$ndigit>>2]|0;
  $49 = ($47|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[12612>>2]|0;
  $51 = $ans;
  $52 = ((($51)) + 1|0);
  $ans = $52;
  $53 = HEAP8[$51>>0]|0;
  $54 = $53 << 24 >> 24;
  FUNCTION_TABLE_vi[$50 & 15]($54);
  $55 = $i;
  $56 = (($55) + 1)|0;
  $i = $56;
 }
 STACKTOP = sp;return 0;
}
function _wrt_IM($n,$w,$m,$len,$base) {
 $n = $n|0;
 $w = $w|0;
 $m = $m|0;
 $len = $len|0;
 $base = $base|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $ans = 0, $i = 0, $ndigit = 0, $or$cond = 0, $or$cond3 = 0, $sign = 0, $spare = 0;
 var $x = 0, $xsign = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ndigit = sp + 24|0;
 $sign = sp + 20|0;
 $1 = $n;
 $2 = $w;
 $3 = $m;
 $4 = $len;
 $5 = $base;
 $6 = $4;
 $7 = (4)==($6|0);
 do {
  if ($7) {
   $8 = $1;
   $9 = HEAP32[$8>>2]|0;
   $x = $9;
  } else {
   $10 = $4;
   $11 = ($10|0)==(1);
   $12 = $1;
   if ($11) {
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $x = $14;
    break;
   } else {
    $15 = HEAP16[$12>>1]|0;
    $16 = $15 << 16 >> 16;
    $x = $16;
    break;
   }
  }
 } while(0);
 $17 = $x;
 $18 = $5;
 $19 = (_f__icvt($17,$ndigit,$sign,$18)|0);
 $ans = $19;
 $20 = HEAP32[$sign>>2]|0;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[12604>>2]|0;
 $23 = ($22|0)!=(0);
 $or$cond = $21 | $23;
 if ($or$cond) {
  $xsign = 1;
 } else {
  $xsign = 0;
 }
 $24 = HEAP32[$ndigit>>2]|0;
 $25 = $xsign;
 $26 = (($24) + ($25))|0;
 $27 = $2;
 $28 = ($26|0)>($27|0);
 if (!($28)) {
  $29 = $3;
  $30 = $xsign;
  $31 = (($29) + ($30))|0;
  $32 = $2;
  $33 = ($31|0)>($32|0);
  if (!($33)) {
   $40 = $x;
   $41 = ($40|0)==(0);
   $42 = $3;
   $43 = ($42|0)==(0);
   $or$cond3 = $41 & $43;
   if ($or$cond3) {
    $i = 0;
    while(1) {
     $44 = $i;
     $45 = $2;
     $46 = ($44|0)<($45|0);
     if (!($46)) {
      break;
     }
     $47 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$47 & 15](32);
     $48 = $i;
     $49 = (($48) + 1)|0;
     $i = $49;
    }
    $0 = 0;
    $92 = $0;
    STACKTOP = sp;return ($92|0);
   }
   $50 = HEAP32[$ndigit>>2]|0;
   $51 = $3;
   $52 = ($50|0)>=($51|0);
   $53 = $2;
   if ($52) {
    $54 = HEAP32[$ndigit>>2]|0;
    $55 = (($53) - ($54))|0;
    $56 = $xsign;
    $57 = (($55) - ($56))|0;
    $spare = $57;
   } else {
    $58 = $3;
    $59 = (($53) - ($58))|0;
    $60 = $xsign;
    $61 = (($59) - ($60))|0;
    $spare = $61;
   }
   $i = 0;
   while(1) {
    $62 = $i;
    $63 = $spare;
    $64 = ($62|0)<($63|0);
    if (!($64)) {
     break;
    }
    $65 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$65 & 15](32);
    $66 = $i;
    $67 = (($66) + 1)|0;
    $i = $67;
   }
   $68 = HEAP32[$sign>>2]|0;
   $69 = ($68|0)!=(0);
   if ($69) {
    $70 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$70 & 15](45);
   } else {
    $71 = HEAP32[12604>>2]|0;
    $72 = ($71|0)!=(0);
    if ($72) {
     $73 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$73 & 15](43);
    }
   }
   $i = 0;
   while(1) {
    $74 = $i;
    $75 = $3;
    $76 = HEAP32[$ndigit>>2]|0;
    $77 = (($75) - ($76))|0;
    $78 = ($74|0)<($77|0);
    if (!($78)) {
     break;
    }
    $79 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$79 & 15](48);
    $80 = $i;
    $81 = (($80) + 1)|0;
    $i = $81;
   }
   $i = 0;
   while(1) {
    $82 = $i;
    $83 = HEAP32[$ndigit>>2]|0;
    $84 = ($82|0)<($83|0);
    if (!($84)) {
     break;
    }
    $85 = HEAP32[12612>>2]|0;
    $86 = $ans;
    $87 = ((($86)) + 1|0);
    $ans = $87;
    $88 = HEAP8[$86>>0]|0;
    $89 = $88 << 24 >> 24;
    FUNCTION_TABLE_vi[$85 & 15]($89);
    $90 = $i;
    $91 = (($90) + 1)|0;
    $i = $91;
   }
   $0 = 0;
   $92 = $0;
   STACKTOP = sp;return ($92|0);
  }
 }
 $i = 0;
 while(1) {
  $34 = $i;
  $35 = $2;
  $36 = ($34|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$37 & 15](42);
  $38 = $i;
  $39 = (($38) + 1)|0;
  $i = $39;
 }
 $0 = 0;
 $92 = $0;
 STACKTOP = sp;return ($92|0);
}
function _wrt_A($p,$len) {
 $p = $p|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $p;
 $1 = $len;
 while(1) {
  $2 = $1;
  $3 = (($2) + -1)|0;
  $1 = $3;
  $4 = ($2|0)>(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[12612>>2]|0;
  $6 = $0;
  $7 = ((($6)) + 1|0);
  $0 = $7;
  $8 = HEAP8[$6>>0]|0;
  $9 = $8 << 24 >> 24;
  FUNCTION_TABLE_vi[$5 & 15]($9);
 }
 STACKTOP = sp;return 0;
}
function _wrt_AW($p,$w,$len) {
 $p = $p|0;
 $w = $w|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $p;
 $1 = $w;
 $2 = $len;
 while(1) {
  $3 = $1;
  $4 = $2;
  $5 = ($3|0)>($4|0);
  if (!($5)) {
   break;
  }
  $6 = $1;
  $7 = (($6) + -1)|0;
  $1 = $7;
  $8 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$8 & 15](32);
 }
 while(1) {
  $9 = $1;
  $10 = (($9) + -1)|0;
  $1 = $10;
  $11 = ($9|0)>(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[12612>>2]|0;
  $13 = $0;
  $14 = ((($13)) + 1|0);
  $0 = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15 << 24 >> 24;
  FUNCTION_TABLE_vi[$12 & 15]($16);
 }
 STACKTOP = sp;return 0;
}
function _wrt_G($p,$w,$d,$e,$len) {
 $p = $p|0;
 $w = $w|0;
 $d = $d|0;
 $e = $e|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0.0, $i = 0, $j = 0, $n = 0, $oldscale = 0, $up = 0.0, $x = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $p;
 $2 = $w;
 $3 = $d;
 $4 = $e;
 $5 = $len;
 $up = 1.0;
 $i = 0;
 $6 = $5;
 $7 = ($6|0)==(4);
 $8 = $1;
 if ($7) {
  $9 = +HEAPF32[$8>>2];
  $10 = $9;
  $12 = $10;
 } else {
  $11 = +HEAPF64[$8>>3];
  $12 = $11;
 }
 $x = $12;
 $13 = $x;
 $14 = $13 < 0.0;
 if ($14) {
  $15 = $x;
  $16 = -$15;
  $x = $16;
 }
 $17 = $x;
 $18 = $17 < 0.10000000000000001;
 L8: do {
  if ($18) {
   $19 = $x;
   $20 = $19 != 0.0;
   if (!($20)) {
    $i = 1;
    break;
   }
   $21 = $1;
   $22 = $2;
   $23 = $3;
   $24 = $4;
   $25 = $5;
   $26 = (_wrt_E($21,$22,$23,$24,$25)|0);
   $0 = $26;
   $65 = $0;
   STACKTOP = sp;return ($65|0);
  } else {
   while(1) {
    $27 = $i;
    $28 = $3;
    $29 = ($27|0)<=($28|0);
    if (!($29)) {
     break;
    }
    $30 = $x;
    $31 = $up;
    $32 = $30 >= $31;
    if (!($32)) {
     break L8;
    }
    $55 = $i;
    $56 = (($55) + 1)|0;
    $i = $56;
    $57 = $up;
    $58 = $57 * 10.0;
    $up = $58;
   }
   $59 = $1;
   $60 = $2;
   $61 = $3;
   $62 = $4;
   $63 = $5;
   $64 = (_wrt_E($59,$60,$61,$62,$63)|0);
   $0 = $64;
   $65 = $0;
   STACKTOP = sp;return ($65|0);
  }
 } while(0);
 $33 = HEAP32[12644>>2]|0;
 $oldscale = $33;
 HEAP32[12644>>2] = 0;
 $34 = $4;
 $35 = ($34|0)==(0);
 if ($35) {
  $n = 4;
 } else {
  $36 = $4;
  $37 = (($36) + 2)|0;
  $n = $37;
 }
 $38 = $1;
 $39 = $2;
 $40 = $n;
 $41 = (($39) - ($40))|0;
 $42 = $3;
 $43 = $i;
 $44 = (($42) - ($43))|0;
 $45 = $5;
 $46 = (_wrt_F($38,$41,$44,$45)|0);
 $i = $46;
 $j = 0;
 while(1) {
  $47 = $j;
  $48 = $n;
  $49 = ($47|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$50 & 15](32);
  $51 = $j;
  $52 = (($51) + 1)|0;
  $j = $52;
 }
 $53 = $oldscale;
 HEAP32[12644>>2] = $53;
 $54 = $i;
 $0 = $54;
 $65 = $0;
 STACKTOP = sp;return ($65|0);
}
function _wrt_Z($n,$w,$minlen,$len) {
 $n = $n|0;
 $w = $w|0;
 $minlen = $minlen|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $s = 0, $se = 0, $w1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $w;
 $3 = $minlen;
 $4 = $len;
 $5 = $1;
 $s = $5;
 $6 = $4;
 $7 = (($6) + -1)|0;
 $4 = $7;
 $8 = HEAP8[12696>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 $10 = $s;
 if ($9) {
  $se = $10;
  $11 = $4;
  $12 = $s;
  $13 = (($12) + ($11)|0);
  $s = $13;
  $i = -1;
 } else {
  $14 = $4;
  $15 = (($10) + ($14)|0);
  $se = $15;
  $i = 1;
 }
 while(1) {
  $16 = $s;
  $17 = $se;
  $18 = ($16|0)==($17|0);
  if ($18) {
   break;
  }
  $19 = $s;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)!=(0);
  if ($22) {
   break;
  }
  $23 = $i;
  $24 = $s;
  $25 = (($24) + ($23)|0);
  $s = $25;
 }
 $26 = $i;
 $27 = $se;
 $28 = $s;
 $29 = $27;
 $30 = $28;
 $31 = (($29) - ($30))|0;
 $32 = Math_imul($26, $31)|0;
 $33 = $32 << 1;
 $34 = (($33) + 1)|0;
 $w1 = $34;
 $35 = $s;
 $36 = HEAP8[$35>>0]|0;
 $37 = $36 << 24 >> 24;
 $38 = $37 & 240;
 $39 = ($38|0)!=(0);
 if ($39) {
  $40 = $w1;
  $41 = (($40) + 1)|0;
  $w1 = $41;
 }
 $42 = $w1;
 $43 = $2;
 $44 = ($42|0)>($43|0);
 L12: do {
  if ($44) {
   $i = 0;
   while(1) {
    $45 = $i;
    $46 = $2;
    $47 = ($45|0)<($46|0);
    if (!($47)) {
     break L12;
    }
    $48 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$48 & 15](42);
    $49 = $i;
    $50 = (($49) + 1)|0;
    $i = $50;
   }
  } else {
   $51 = $w1;
   $52 = $3;
   $53 = (($52) - ($51))|0;
   $3 = $53;
   $54 = ($53|0)>(0);
   if ($54) {
    $55 = $3;
    $56 = $w1;
    $57 = (($56) + ($55))|0;
    $w1 = $57;
   }
   while(1) {
    $58 = $2;
    $59 = (($58) + -1)|0;
    $2 = $59;
    $60 = $w1;
    $61 = ($59|0)>=($60|0);
    if (!($61)) {
     break;
    }
    $62 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$62 & 15](32);
   }
   while(1) {
    $63 = $3;
    $64 = (($63) + -1)|0;
    $3 = $64;
    $65 = ($64|0)>=(0);
    if (!($65)) {
     break;
    }
    $66 = HEAP32[12612>>2]|0;
    FUNCTION_TABLE_vi[$66 & 15](48);
   }
   $67 = $s;
   $68 = HEAP8[$67>>0]|0;
   $69 = $68 << 24 >> 24;
   $70 = $69 & 240;
   $71 = ($70|0)!=(0);
   do {
    if (!($71)) {
     $72 = HEAP32[12612>>2]|0;
     $73 = $s;
     $74 = HEAP8[$73>>0]|0;
     $75 = $74 << 24 >> 24;
     $76 = $75 & 15;
     $77 = (20101 + ($76)|0);
     $78 = HEAP8[$77>>0]|0;
     $79 = $78 << 24 >> 24;
     FUNCTION_TABLE_vi[$72 & 15]($79);
     $80 = $s;
     $81 = $se;
     $82 = ($80|0)==($81|0);
     if (!($82)) {
      $83 = $i;
      $84 = $s;
      $85 = (($84) + ($83)|0);
      $s = $85;
      break;
     }
     $0 = 0;
     $109 = $0;
     STACKTOP = sp;return ($109|0);
    }
   } while(0);
   while(1) {
    $86 = HEAP32[12612>>2]|0;
    $87 = $s;
    $88 = HEAP8[$87>>0]|0;
    $89 = $88 << 24 >> 24;
    $90 = $89 >> 4;
    $91 = $90 & 15;
    $92 = (20101 + ($91)|0);
    $93 = HEAP8[$92>>0]|0;
    $94 = $93 << 24 >> 24;
    FUNCTION_TABLE_vi[$86 & 15]($94);
    $95 = HEAP32[12612>>2]|0;
    $96 = $s;
    $97 = HEAP8[$96>>0]|0;
    $98 = $97 << 24 >> 24;
    $99 = $98 & 15;
    $100 = (20101 + ($99)|0);
    $101 = HEAP8[$100>>0]|0;
    $102 = $101 << 24 >> 24;
    FUNCTION_TABLE_vi[$95 & 15]($102);
    $103 = $s;
    $104 = $se;
    $105 = ($103|0)==($104|0);
    if ($105) {
     break L12;
    }
    $106 = $i;
    $107 = $s;
    $108 = (($107) + ($106)|0);
    $s = $108;
   }
  }
 } while(0);
 $0 = 0;
 $109 = $0;
 STACKTOP = sp;return ($109|0);
}
function _wrt_AP($s) {
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $quote = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $s;
 $2 = HEAP32[12636>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = (_mv_cur()|0);
  $i = $4;
  $5 = ($4|0)!=(0);
  if ($5) {
   $6 = $i;
   $0 = $6;
   $36 = $0;
   STACKTOP = sp;return ($36|0);
  }
 }
 $7 = $1;
 $8 = ((($7)) + 1|0);
 $1 = $8;
 $9 = HEAP8[$7>>0]|0;
 $quote = $9;
 while(1) {
  $10 = $1;
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  if (!($12)) {
   label = 12;
   break;
  }
  $13 = $1;
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $quote;
  $17 = $16 << 24 >> 24;
  $18 = ($15|0)!=($17|0);
  if ($18) {
   $19 = HEAP32[12612>>2]|0;
   $20 = $1;
   $21 = HEAP8[$20>>0]|0;
   $22 = $21 << 24 >> 24;
   FUNCTION_TABLE_vi[$19 & 15]($22);
  } else {
   $23 = $1;
   $24 = ((($23)) + 1|0);
   $1 = $24;
   $25 = HEAP8[$24>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = $quote;
   $28 = $27 << 24 >> 24;
   $29 = ($26|0)==($28|0);
   if (!($29)) {
    label = 10;
    break;
   }
   $30 = HEAP32[12612>>2]|0;
   $31 = $1;
   $32 = HEAP8[$31>>0]|0;
   $33 = $32 << 24 >> 24;
   FUNCTION_TABLE_vi[$30 & 15]($33);
  }
  $34 = $1;
  $35 = ((($34)) + 1|0);
  $1 = $35;
 }
 if ((label|0) == 10) {
  $0 = 1;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 else if ((label|0) == 12) {
  $0 = 1;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 return (0)|0;
}
function _wrt_H($a,$s) {
 $a = $a|0;
 $s = $s|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $a;
 $2 = $s;
 $3 = HEAP32[12636>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = (_mv_cur()|0);
  $i = $5;
  $6 = ($5|0)!=(0);
  if ($6) {
   $7 = $i;
   $0 = $7;
   $16 = $0;
   STACKTOP = sp;return ($16|0);
  }
 }
 while(1) {
  $8 = $1;
  $9 = (($8) + -1)|0;
  $1 = $9;
  $10 = ($8|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[12612>>2]|0;
  $12 = $2;
  $13 = ((($12)) + 1|0);
  $2 = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14 << 24 >> 24;
  FUNCTION_TABLE_vi[$11 & 15]($15);
 }
 $0 = 1;
 $16 = $0;
 STACKTOP = sp;return ($16|0);
}
function _f__icvt($value,$ndigit,$sign,$base) {
 $value = $value|0;
 $ndigit = $ndigit|0;
 $sign = $sign|0;
 $base = $base|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $uvalue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $value;
 $2 = $ndigit;
 $3 = $sign;
 $4 = $base;
 $5 = $1;
 $6 = ($5|0)>(0);
 $7 = $1;
 do {
  if ($6) {
   $uvalue = $7;
   $8 = $3;
   HEAP32[$8>>2] = 0;
  } else {
   $9 = ($7|0)<(0);
   if ($9) {
    $10 = $1;
    $11 = (0 - ($10))|0;
    $uvalue = $11;
    $12 = $3;
    HEAP32[$12>>2] = 1;
    break;
   }
   $13 = $3;
   HEAP32[$13>>2] = 0;
   $14 = $2;
   HEAP32[$14>>2] = 1;
   HEAP8[(20420)>>0] = 48;
   $0 = (20420);
   $33 = $0;
   STACKTOP = sp;return ($33|0);
  }
 } while(0);
 $i = 23;
 while(1) {
  $15 = $uvalue;
  $16 = $4;
  $17 = (($15>>>0) % ($16>>>0))&-1;
  $18 = (($17) + 48)|0;
  $19 = $18&255;
  $20 = $i;
  $21 = (($20) + -1)|0;
  $i = $21;
  $22 = (20398 + ($21)|0);
  HEAP8[$22>>0] = $19;
  $23 = $4;
  $24 = $uvalue;
  $25 = (($24>>>0) / ($23>>>0))&-1;
  $uvalue = $25;
  $26 = $uvalue;
  $27 = ($26>>>0)>(0);
  if (!($27)) {
   break;
  }
 }
 $28 = $i;
 $29 = (23 - ($28))|0;
 $30 = $2;
 HEAP32[$30>>2] = $29;
 $31 = $i;
 $32 = (20398 + ($31)|0);
 $0 = $32;
 $33 = $0;
 STACKTOP = sp;return ($33|0);
}
function _wrt_E($p,$w,$d,$e,$len) {
 $p = $p|0;
 $w = $w|0;
 $d = $d|0;
 $e = $e|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0;
 var $33 = 0, $34 = 0.0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0.0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0, $d1 = 0, $dd = 0.0, $delta = 0, $e0 = 0, $e1 = 0, $i = 0;
 var $s = 0, $se = 0, $sign = 0, $signspace = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer2 = sp + 24|0;
 $vararg_buffer = sp + 8|0;
 $buf = sp + 88|0;
 $1 = $p;
 $2 = $w;
 $3 = $d;
 $4 = $e;
 $5 = $len;
 $6 = $4;
 $e0 = $6;
 $7 = $4;
 $8 = ($7|0)<=(0);
 if ($8) {
  $4 = 2;
 }
 $9 = HEAP32[12644>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[12644>>2]|0;
  $12 = $3;
  $13 = (($12) + 2)|0;
  $14 = ($11|0)>=($13|0);
  if (!($14)) {
   $15 = HEAP32[12644>>2]|0;
   $16 = $3;
   $17 = (0 - ($16))|0;
   $18 = ($15|0)<=($17|0);
   if (!($18)) {
    label = 6;
   }
  }
 } else {
  label = 6;
 }
 L7: do {
  if ((label|0) == 6) {
   $19 = HEAP32[12644>>2]|0;
   $20 = ($19|0)<=(0);
   if ($20) {
    $21 = $3;
    $22 = (($21) + -1)|0;
    $3 = $22;
   }
   $23 = $5;
   $24 = ($23|0)==(4);
   $25 = $1;
   if ($24) {
    $26 = +HEAPF32[$25>>2];
    $27 = $26;
    $dd = $27;
   } else {
    $28 = +HEAPF64[$25>>3];
    $dd = $28;
   }
   $29 = $dd;
   $30 = $29 < 0.0;
   if ($30) {
    $sign = 1;
    $signspace = 1;
    $31 = $dd;
    $32 = -$31;
    $dd = $32;
   } else {
    $sign = 0;
    $33 = HEAP32[12604>>2]|0;
    $signspace = $33;
    $34 = $dd;
    $35 = $34 != 0.0;
    if (!($35)) {
     $dd = 0.0;
    }
   }
   $36 = $2;
   $37 = $signspace;
   $38 = (4 + ($37))|0;
   $39 = $3;
   $40 = (($38) + ($39))|0;
   $41 = $4;
   $42 = (($40) + ($41))|0;
   $43 = (($36) - ($42))|0;
   $delta = $43;
   $44 = $delta;
   $45 = ($44|0)<(0);
   if (!($45)) {
    $50 = HEAP32[12644>>2]|0;
    $51 = ($50|0)<(0);
    if ($51) {
     $52 = HEAP32[12644>>2]|0;
     $53 = $3;
     $54 = (($53) + ($52))|0;
     $3 = $54;
    }
    $55 = $3;
    $56 = ($55|0)>(40);
    if ($56) {
     $57 = $3;
     $58 = (($57) - 40)|0;
     $d1 = $58;
     $3 = 40;
    } else {
     $d1 = 0;
    }
    $59 = $3;
    $60 = $dd;
    HEAP32[$vararg_buffer>>2] = $59;
    $vararg_ptr1 = ((($vararg_buffer)) + 8|0);
    HEAPF64[$vararg_ptr1>>3] = $60;
    (_sprintf($buf,20422,$vararg_buffer)|0);
    $61 = HEAP8[$buf>>0]|0;
    $62 = $61 << 24 >> 24;
    $63 = (($62) + 8)|0;
    $64 = (20134 + ($63)|0);
    $65 = HEAP8[$64>>0]|0;
    $66 = $65 << 24 >> 24;
    $67 = $66 & 1;
    $68 = ($67|0)!=(0);
    if (!($68)) {
     $69 = HEAP8[$buf>>0]|0;
     $70 = $69 << 24 >> 24;
     switch ($70|0) {
     case 78: case 110:  {
      $signspace = 0;
      break;
     }
     default: {
     }
     }
     $71 = $2;
     $72 = (_strlen($buf)|0);
     $73 = (($71) - ($72))|0;
     $74 = $signspace;
     $75 = (($73) - ($74))|0;
     $delta = $75;
     $76 = $delta;
     $77 = ($76|0)<(0);
     if ($77) {
      break;
     }
     while(1) {
      $78 = $delta;
      $79 = (($78) + -1)|0;
      $delta = $79;
      $80 = ($79|0)>=(0);
      if (!($80)) {
       break;
      }
      $81 = HEAP32[12612>>2]|0;
      FUNCTION_TABLE_vi[$81 & 15](32);
     }
     $82 = $signspace;
     $83 = ($82|0)!=(0);
     if ($83) {
      $84 = HEAP32[12612>>2]|0;
      $85 = $sign;
      $86 = ($85|0)!=(0);
      $87 = $86 ? 45 : 43;
      FUNCTION_TABLE_vi[$84 & 15]($87);
     }
     $s = $buf;
     while(1) {
      $88 = $s;
      $89 = HEAP8[$88>>0]|0;
      $90 = ($89<<24>>24)!=(0);
      if (!($90)) {
       break;
      }
      $91 = HEAP32[12612>>2]|0;
      $92 = $s;
      $93 = HEAP8[$92>>0]|0;
      $94 = $93 << 24 >> 24;
      FUNCTION_TABLE_vi[$91 & 15]($94);
      $95 = $s;
      $96 = ((($95)) + 1|0);
      $s = $96;
     }
     $0 = 0;
     $232 = $0;
     STACKTOP = sp;return ($232|0);
    }
    $97 = $3;
    $98 = (($buf) + ($97)|0);
    $99 = ((($98)) + 3|0);
    $se = $99;
    $100 = $dd;
    $101 = $100 != 0.0;
    $102 = $se;
    if ($101) {
     $103 = $se;
     $104 = (_atoi($103)|0);
     $105 = (($104) + 1)|0;
     $106 = HEAP32[12644>>2]|0;
     $107 = (($105) - ($106))|0;
     HEAP32[$vararg_buffer2>>2] = $107;
     (_sprintf($102,20428,$vararg_buffer2)|0);
    } else {
     (_strcpy($102,20434)|0);
    }
    $108 = $se;
    $109 = ((($108)) + 1|0);
    $se = $109;
    $s = $109;
    $110 = $4;
    $111 = ($110|0)<(2);
    if ($111) {
     $112 = $s;
     $113 = HEAP8[$112>>0]|0;
     $114 = $113 << 24 >> 24;
     $115 = ($114|0)!=(48);
     if ($115) {
      break;
     }
    }
    $116 = $s;
    $117 = ((($116)) + 2|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = ($118<<24>>24)!=(0);
    L54: do {
     if ($119) {
      $120 = $e0;
      $121 = ($120|0)!=(0);
      if ($121) {
       $131 = $e0;
       $132 = ($131|0)>=(0);
       if ($132) {
        label = 48;
        break;
       }
       $133 = $4;
       $e1 = $133;
       break;
      }
      $122 = $s;
      $123 = ((($122)) + -2|0);
      $s = $123;
      $e1 = 2;
      while(1) {
       $124 = $s;
       $125 = ((($124)) + 1|0);
       $126 = HEAP8[$125>>0]|0;
       $127 = $s;
       HEAP8[$127>>0] = $126;
       $128 = ($126<<24>>24)!=(0);
       if (!($128)) {
        break L54;
       }
       $129 = $s;
       $130 = ((($129)) + 1|0);
       $s = $130;
      }
     } else {
      label = 48;
     }
    } while(0);
    L63: do {
     if ((label|0) == 48) {
      $134 = $s;
      $135 = ((($134)) + 2|0);
      $s = $135;
      $e1 = 2;
      while(1) {
       $136 = $s;
       $137 = HEAP8[$136>>0]|0;
       $138 = ($137<<24>>24)!=(0);
       if (!($138)) {
        break L63;
       }
       $139 = $e1;
       $140 = $4;
       $141 = ($139|0)>=($140|0);
       if ($141) {
        break L7;
       }
       $142 = $e1;
       $143 = (($142) + 1)|0;
       $e1 = $143;
       $144 = $s;
       $145 = ((($144)) + 1|0);
       $s = $145;
      }
     }
    } while(0);
    while(1) {
     $146 = $delta;
     $147 = (($146) + -1)|0;
     $delta = $147;
     $148 = ($147|0)>=(0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$149 & 15](32);
    }
    $150 = $signspace;
    $151 = ($150|0)!=(0);
    if ($151) {
     $152 = HEAP32[12612>>2]|0;
     $153 = $sign;
     $154 = ($153|0)!=(0);
     $155 = $154 ? 45 : 43;
     FUNCTION_TABLE_vi[$152 & 15]($155);
    }
    $s = $buf;
    $156 = HEAP32[12644>>2]|0;
    $i = $156;
    $157 = HEAP32[12644>>2]|0;
    $158 = ($157|0)<=(0);
    if ($158) {
     $159 = HEAP32[12612>>2]|0;
     FUNCTION_TABLE_vi[$159 & 15](46);
     while(1) {
      $160 = $i;
      $161 = ($160|0)<(0);
      $162 = HEAP32[12612>>2]|0;
      if (!($161)) {
       break;
      }
      FUNCTION_TABLE_vi[$162 & 15](48);
      $163 = $i;
      $164 = (($163) + 1)|0;
      $i = $164;
     }
     $165 = $s;
     $166 = HEAP8[$165>>0]|0;
     $167 = $166 << 24 >> 24;
     FUNCTION_TABLE_vi[$162 & 15]($167);
     $168 = $s;
     $169 = ((($168)) + 2|0);
     $s = $169;
    } else {
     $170 = HEAP32[12644>>2]|0;
     $171 = ($170|0)>(1);
     if ($171) {
      $172 = HEAP32[12612>>2]|0;
      $173 = $s;
      $174 = HEAP8[$173>>0]|0;
      $175 = $174 << 24 >> 24;
      FUNCTION_TABLE_vi[$172 & 15]($175);
      $176 = $s;
      $177 = ((($176)) + 2|0);
      $s = $177;
      while(1) {
       $178 = $i;
       $179 = (($178) + -1)|0;
       $i = $179;
       $180 = ($179|0)>(0);
       $181 = HEAP32[12612>>2]|0;
       if (!($180)) {
        break;
       }
       $182 = $s;
       $183 = ((($182)) + 1|0);
       $s = $183;
       $184 = HEAP8[$182>>0]|0;
       $185 = $184 << 24 >> 24;
       FUNCTION_TABLE_vi[$181 & 15]($185);
      }
      FUNCTION_TABLE_vi[$181 & 15](46);
     }
    }
    $186 = $d1;
    $187 = ($186|0)!=(0);
    if ($187) {
     $188 = $se;
     $189 = ((($188)) + -2|0);
     $se = $189;
     while(1) {
      $190 = $s;
      $191 = $se;
      $192 = ($190>>>0)<($191>>>0);
      if (!($192)) {
       break;
      }
      $193 = HEAP32[12612>>2]|0;
      $194 = $s;
      $195 = ((($194)) + 1|0);
      $s = $195;
      $196 = HEAP8[$194>>0]|0;
      $197 = $196 << 24 >> 24;
      FUNCTION_TABLE_vi[$193 & 15]($197);
     }
     $198 = $se;
     $199 = ((($198)) + 2|0);
     $se = $199;
     while(1) {
      $200 = HEAP32[12612>>2]|0;
      FUNCTION_TABLE_vi[$200 & 15](48);
      $201 = $d1;
      $202 = (($201) + -1)|0;
      $d1 = $202;
      $203 = ($202|0)>(0);
      if (!($203)) {
       break;
      }
     }
    }
    while(1) {
     $204 = $s;
     $205 = $se;
     $206 = ($204>>>0)<($205>>>0);
     if (!($206)) {
      break;
     }
     $207 = HEAP32[12612>>2]|0;
     $208 = $s;
     $209 = ((($208)) + 1|0);
     $s = $209;
     $210 = HEAP8[$208>>0]|0;
     $211 = $210 << 24 >> 24;
     FUNCTION_TABLE_vi[$207 & 15]($211);
    }
    $212 = $4;
    $213 = ($212|0)<(2);
    L101: do {
     if ($213) {
      $214 = HEAP32[12612>>2]|0;
      $215 = $s;
      $216 = ((($215)) + 1|0);
      $217 = HEAP8[$216>>0]|0;
      $218 = $217 << 24 >> 24;
      FUNCTION_TABLE_vi[$214 & 15]($218);
     } else {
      while(1) {
       $219 = $e1;
       $220 = (($219) + 1)|0;
       $e1 = $220;
       $221 = $4;
       $222 = ($220|0)<=($221|0);
       if (!($222)) {
        break;
       }
       $223 = HEAP32[12612>>2]|0;
       FUNCTION_TABLE_vi[$223 & 15](48);
      }
      while(1) {
       $224 = $s;
       $225 = HEAP8[$224>>0]|0;
       $226 = ($225<<24>>24)!=(0);
       if (!($226)) {
        break L101;
       }
       $227 = HEAP32[12612>>2]|0;
       $228 = $s;
       $229 = ((($228)) + 1|0);
       $s = $229;
       $230 = HEAP8[$228>>0]|0;
       $231 = $230 << 24 >> 24;
       FUNCTION_TABLE_vi[$227 & 15]($231);
      }
     }
    } while(0);
    $0 = 0;
    $232 = $0;
    STACKTOP = sp;return ($232|0);
   }
  }
 } while(0);
 while(1) {
  $46 = $2;
  $47 = (($46) + -1)|0;
  $2 = $47;
  $48 = ($47|0)>=(0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$49 & 15](42);
 }
 $0 = 0;
 $232 = $0;
 STACKTOP = sp;return ($232|0);
}
function _wrt_F($p,$w,$d,$len) {
 $p = $p|0;
 $w = $w|0;
 $d = $d|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0.0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0.0, $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $b = 0, $buf = 0, $d1 = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $s = 0, $sign = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $x = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 720|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp + 8|0;
 $buf = sp + 64|0;
 $1 = $p;
 $2 = $w;
 $3 = $d;
 $4 = $len;
 $5 = $4;
 $6 = ($5|0)==(4);
 $7 = $1;
 if ($6) {
  $8 = +HEAPF32[$7>>2];
  $9 = $8;
  $11 = $9;
 } else {
  $10 = +HEAPF64[$7>>3];
  $11 = $10;
 }
 $x = $11;
 $12 = $3;
 $13 = ($12|0)<(344);
 if ($13) {
  $d1 = 0;
 } else {
  $14 = $3;
  $15 = (($14) - 344)|0;
  $d1 = $15;
  $3 = 344;
 }
 $16 = $x;
 $17 = $16 < 0.0;
 if ($17) {
  $18 = $x;
  $19 = -$18;
  $x = $19;
  $sign = 1;
 } else {
  $sign = 0;
  $20 = $x;
  $21 = $20 != 0.0;
  if (!($21)) {
   $x = 0.0;
  }
 }
 $22 = HEAP32[12644>>2]|0;
 $n = $22;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = $n;
  $25 = ($24|0)>(0);
  if ($25) {
   while(1) {
    $26 = $x;
    $27 = $26 * 10.0;
    $x = $27;
    $28 = $n;
    $29 = (($28) + -1)|0;
    $n = $29;
    $30 = ($29|0)>(0);
    if (!($30)) {
     break;
    }
   }
  } else {
   while(1) {
    $31 = $x;
    $32 = $31 * 0.10000000000000001;
    $x = $32;
    $33 = $n;
    $34 = (($33) + 1)|0;
    $n = $34;
    $35 = ($34|0)<(0);
    if (!($35)) {
     break;
    }
   }
  }
 }
 $b = $buf;
 $36 = $3;
 $37 = $x;
 HEAP32[$vararg_buffer>>2] = $36;
 $vararg_ptr4 = ((($vararg_buffer)) + 8|0);
 HEAPF64[$vararg_ptr4>>3] = $37;
 $38 = (_sprintf($buf,20438,$vararg_buffer)|0);
 $39 = $d1;
 $40 = (($38) + ($39))|0;
 $n = $40;
 $41 = HEAP8[$buf>>0]|0;
 $42 = $41 << 24 >> 24;
 $43 = ($42|0)==(48);
 $44 = $3;
 $45 = ($44|0)!=(0);
 $or$cond = $43 & $45;
 if ($or$cond) {
  $46 = $b;
  $47 = ((($46)) + 1|0);
  $b = $47;
  $48 = $n;
  $49 = (($48) + -1)|0;
  $n = $49;
 }
 $50 = $sign;
 $51 = ($50|0)==(1);
 L25: do {
  if ($51) {
   $52 = $b;
   $s = $52;
   L27: while(1) {
    $53 = $s;
    $54 = HEAP8[$53>>0]|0;
    $55 = $54 << 24 >> 24;
    $56 = ($55|0)==(48);
    $57 = $s;
    if ($56) {
     $58 = ((($57)) + 1|0);
     $s = $58;
     continue;
    }
    $59 = HEAP8[$57>>0]|0;
    $60 = $59 << 24 >> 24;
    switch ($60|0) {
    case 0:  {
     break L27;
     break;
    }
    case 46:  {
     break;
    }
    default: {
     break L25;
    }
    }
    $61 = $s;
    $62 = ((($61)) + 1|0);
    $s = $62;
   }
   $sign = 0;
  }
 } while(0);
 $63 = $sign;
 $64 = ($63|0)!=(0);
 $65 = HEAP32[12604>>2]|0;
 $66 = ($65|0)!=(0);
 $or$cond3 = $64 | $66;
 if ($or$cond3) {
  $67 = $n;
  $68 = (($67) + 1)|0;
  $n = $68;
 }
 $69 = $n;
 $70 = $2;
 $71 = ($69|0)>($70|0);
 if ($71) {
  while(1) {
   $72 = $2;
   $73 = (($72) + -1)|0;
   $2 = $73;
   $74 = ($73|0)>=(0);
   if (!($74)) {
    break;
   }
   $75 = HEAP32[12612>>2]|0;
   FUNCTION_TABLE_vi[$75 & 15](42);
  }
  $0 = 0;
  $100 = $0;
  STACKTOP = sp;return ($100|0);
 }
 $76 = $n;
 $77 = $2;
 $78 = (($77) - ($76))|0;
 $2 = $78;
 while(1) {
  $79 = $2;
  $80 = (($79) + -1)|0;
  $2 = $80;
  $81 = ($80|0)>=(0);
  if (!($81)) {
   break;
  }
  $82 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$82 & 15](32);
 }
 $83 = $sign;
 $84 = ($83|0)!=(0);
 if ($84) {
  $85 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$85 & 15](45);
 } else {
  $86 = HEAP32[12604>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[12612>>2]|0;
   FUNCTION_TABLE_vi[$88 & 15](43);
  }
 }
 while(1) {
  $89 = $b;
  $90 = ((($89)) + 1|0);
  $b = $90;
  $91 = HEAP8[$89>>0]|0;
  $92 = $91 << 24 >> 24;
  $n = $92;
  $93 = ($92|0)!=(0);
  if (!($93)) {
   break;
  }
  $94 = HEAP32[12612>>2]|0;
  $95 = $n;
  FUNCTION_TABLE_vi[$94 & 15]($95);
 }
 while(1) {
  $96 = $d1;
  $97 = (($96) + -1)|0;
  $d1 = $97;
  $98 = ($97|0)>=(0);
  if (!($98)) {
   break;
  }
  $99 = HEAP32[12612>>2]|0;
  FUNCTION_TABLE_vi[$99 & 15](48);
 }
 $0 = 0;
 $100 = $0;
 STACKTOP = sp;return ($100|0);
}
function _idamax_($n,$dx,$incx) {
 $n = $n|0;
 $dx = $dx|0;
 $incx = $incx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0.0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0.0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0.0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0.0, $96 = 0.0, $97 = 0, $98 = 0.0, $99 = 0.0, $d__1 = 0.0, $dmax__ = 0.0, $i__ = 0, $i__1 = 0, $ix = 0, $ret_val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $dx;
 $3 = $incx;
 $4 = $2;
 $5 = ((($4)) + -8|0);
 $2 = $5;
 $ret_val = 0;
 $6 = $1;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<(1);
 if (!($8)) {
  $9 = $3;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)<=(0);
  if (!($11)) {
   $ret_val = 1;
   $13 = $1;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(1);
   if ($15) {
    $16 = $ret_val;
    $0 = $16;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
   $17 = $3;
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)==(1);
   if ($19) {
    $66 = $2;
    $67 = ((($66)) + 8|0);
    $68 = +HEAPF64[$67>>3];
    $69 = $68 >= 0.0;
    $70 = $2;
    $71 = ((($70)) + 8|0);
    $72 = +HEAPF64[$71>>3];
    $73 = -$72;
    $74 = $69 ? $72 : $73;
    $dmax__ = $74;
    $75 = $1;
    $76 = HEAP32[$75>>2]|0;
    $i__1 = $76;
    $i__ = 2;
    while(1) {
     $77 = $i__;
     $78 = $i__1;
     $79 = ($77|0)<=($78|0);
     if (!($79)) {
      break;
     }
     $80 = $i__;
     $81 = $2;
     $82 = (($81) + ($80<<3)|0);
     $83 = +HEAPF64[$82>>3];
     $d__1 = $83;
     $84 = $d__1;
     $85 = $84 >= 0.0;
     $86 = $d__1;
     $87 = -$86;
     $88 = $85 ? $86 : $87;
     $89 = $dmax__;
     $90 = $88 <= $89;
     if (!($90)) {
      $91 = $i__;
      $ret_val = $91;
      $92 = $i__;
      $93 = $2;
      $94 = (($93) + ($92<<3)|0);
      $95 = +HEAPF64[$94>>3];
      $d__1 = $95;
      $96 = $d__1;
      $97 = $96 >= 0.0;
      $98 = $d__1;
      $99 = -$98;
      $100 = $97 ? $98 : $99;
      $dmax__ = $100;
     }
     $101 = $i__;
     $102 = (($101) + 1)|0;
     $i__ = $102;
    }
    $103 = $ret_val;
    $0 = $103;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   } else {
    $ix = 1;
    $20 = $2;
    $21 = ((($20)) + 8|0);
    $22 = +HEAPF64[$21>>3];
    $23 = $22 >= 0.0;
    $24 = $2;
    $25 = ((($24)) + 8|0);
    $26 = +HEAPF64[$25>>3];
    $27 = -$26;
    $28 = $23 ? $26 : $27;
    $dmax__ = $28;
    $29 = $3;
    $30 = HEAP32[$29>>2]|0;
    $31 = $ix;
    $32 = (($31) + ($30))|0;
    $ix = $32;
    $33 = $1;
    $34 = HEAP32[$33>>2]|0;
    $i__1 = $34;
    $i__ = 2;
    while(1) {
     $35 = $i__;
     $36 = $i__1;
     $37 = ($35|0)<=($36|0);
     if (!($37)) {
      break;
     }
     $38 = $ix;
     $39 = $2;
     $40 = (($39) + ($38<<3)|0);
     $41 = +HEAPF64[$40>>3];
     $d__1 = $41;
     $42 = $d__1;
     $43 = $42 >= 0.0;
     $44 = $d__1;
     $45 = -$44;
     $46 = $43 ? $44 : $45;
     $47 = $dmax__;
     $48 = $46 <= $47;
     if (!($48)) {
      $49 = $i__;
      $ret_val = $49;
      $50 = $ix;
      $51 = $2;
      $52 = (($51) + ($50<<3)|0);
      $53 = +HEAPF64[$52>>3];
      $d__1 = $53;
      $54 = $d__1;
      $55 = $54 >= 0.0;
      $56 = $d__1;
      $57 = -$56;
      $58 = $55 ? $56 : $57;
      $dmax__ = $58;
     }
     $59 = $3;
     $60 = HEAP32[$59>>2]|0;
     $61 = $ix;
     $62 = (($61) + ($60))|0;
     $ix = $62;
     $63 = $i__;
     $64 = (($63) + 1)|0;
     $i__ = $64;
    }
    $65 = $ret_val;
    $0 = $65;
    $104 = $0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $12 = $ret_val;
 $0 = $12;
 $104 = $0;
 STACKTOP = sp;return ($104|0);
}
function _daxpy_($n,$da,$dx,$incx,$dy,$incy) {
 $n = $n|0;
 $da = $da|0;
 $dx = $dx|0;
 $incx = $incx|0;
 $dy = $dy|0;
 $incy = $incy|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0.0, $117 = 0.0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0.0, $125 = 0.0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $132 = 0, $133 = 0.0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0.0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0.0, $145 = 0.0, $146 = 0, $147 = 0.0, $148 = 0, $149 = 0, $15 = 0.0, $150 = 0, $151 = 0;
 var $152 = 0.0, $153 = 0.0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0.0, $61 = 0.0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $i__ = 0, $i__1 = 0, $ix = 0, $iy = 0, $m = 0, $mp1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $da;
 $3 = $dx;
 $4 = $incx;
 $5 = $dy;
 $6 = $incy;
 $7 = $5;
 $8 = ((($7)) + -8|0);
 $5 = $8;
 $9 = $3;
 $10 = ((($9)) + -8|0);
 $3 = $10;
 $11 = $1;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)<=(0);
 if ($13) {
  $0 = 0;
  $162 = $0;
  STACKTOP = sp;return ($162|0);
 }
 $14 = $2;
 $15 = +HEAPF64[$14>>3];
 $16 = $15 == 0.0;
 if ($16) {
  $0 = 0;
  $162 = $0;
  STACKTOP = sp;return ($162|0);
 }
 $17 = $4;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(1);
 if ($19) {
  $20 = $6;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)==(1);
  if ($22) {
   $72 = $1;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($73|0) % 4)&-1;
   $m = $74;
   $75 = $m;
   $76 = ($75|0)==(0);
   if (!($76)) {
    $77 = $m;
    $i__1 = $77;
    $i__ = 1;
    while(1) {
     $78 = $i__;
     $79 = $i__1;
     $80 = ($78|0)<=($79|0);
     if (!($80)) {
      break;
     }
     $81 = $2;
     $82 = +HEAPF64[$81>>3];
     $83 = $i__;
     $84 = $3;
     $85 = (($84) + ($83<<3)|0);
     $86 = +HEAPF64[$85>>3];
     $87 = $82 * $86;
     $88 = $i__;
     $89 = $5;
     $90 = (($89) + ($88<<3)|0);
     $91 = +HEAPF64[$90>>3];
     $92 = $91 + $87;
     HEAPF64[$90>>3] = $92;
     $93 = $i__;
     $94 = (($93) + 1)|0;
     $i__ = $94;
    }
    $95 = $1;
    $96 = HEAP32[$95>>2]|0;
    $97 = ($96|0)<(4);
    if ($97) {
     $0 = 0;
     $162 = $0;
     STACKTOP = sp;return ($162|0);
    }
   }
   $98 = $m;
   $99 = (($98) + 1)|0;
   $mp1 = $99;
   $100 = $1;
   $101 = HEAP32[$100>>2]|0;
   $i__1 = $101;
   $102 = $mp1;
   $i__ = $102;
   while(1) {
    $103 = $i__;
    $104 = $i__1;
    $105 = ($103|0)<=($104|0);
    if (!($105)) {
     break;
    }
    $106 = $2;
    $107 = +HEAPF64[$106>>3];
    $108 = $i__;
    $109 = $3;
    $110 = (($109) + ($108<<3)|0);
    $111 = +HEAPF64[$110>>3];
    $112 = $107 * $111;
    $113 = $i__;
    $114 = $5;
    $115 = (($114) + ($113<<3)|0);
    $116 = +HEAPF64[$115>>3];
    $117 = $116 + $112;
    HEAPF64[$115>>3] = $117;
    $118 = $2;
    $119 = +HEAPF64[$118>>3];
    $120 = $i__;
    $121 = (($120) + 1)|0;
    $122 = $3;
    $123 = (($122) + ($121<<3)|0);
    $124 = +HEAPF64[$123>>3];
    $125 = $119 * $124;
    $126 = $i__;
    $127 = (($126) + 1)|0;
    $128 = $5;
    $129 = (($128) + ($127<<3)|0);
    $130 = +HEAPF64[$129>>3];
    $131 = $130 + $125;
    HEAPF64[$129>>3] = $131;
    $132 = $2;
    $133 = +HEAPF64[$132>>3];
    $134 = $i__;
    $135 = (($134) + 2)|0;
    $136 = $3;
    $137 = (($136) + ($135<<3)|0);
    $138 = +HEAPF64[$137>>3];
    $139 = $133 * $138;
    $140 = $i__;
    $141 = (($140) + 2)|0;
    $142 = $5;
    $143 = (($142) + ($141<<3)|0);
    $144 = +HEAPF64[$143>>3];
    $145 = $144 + $139;
    HEAPF64[$143>>3] = $145;
    $146 = $2;
    $147 = +HEAPF64[$146>>3];
    $148 = $i__;
    $149 = (($148) + 3)|0;
    $150 = $3;
    $151 = (($150) + ($149<<3)|0);
    $152 = +HEAPF64[$151>>3];
    $153 = $147 * $152;
    $154 = $i__;
    $155 = (($154) + 3)|0;
    $156 = $5;
    $157 = (($156) + ($155<<3)|0);
    $158 = +HEAPF64[$157>>3];
    $159 = $158 + $153;
    HEAPF64[$157>>3] = $159;
    $160 = $i__;
    $161 = (($160) + 4)|0;
    $i__ = $161;
   }
   $0 = 0;
   $162 = $0;
   STACKTOP = sp;return ($162|0);
  }
 }
 $ix = 1;
 $iy = 1;
 $23 = $4;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)<(0);
 if ($25) {
  $26 = $1;
  $27 = HEAP32[$26>>2]|0;
  $28 = (0 - ($27))|0;
  $29 = (($28) + 1)|0;
  $30 = $4;
  $31 = HEAP32[$30>>2]|0;
  $32 = Math_imul($29, $31)|0;
  $33 = (($32) + 1)|0;
  $ix = $33;
 }
 $34 = $6;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)<(0);
 if ($36) {
  $37 = $1;
  $38 = HEAP32[$37>>2]|0;
  $39 = (0 - ($38))|0;
  $40 = (($39) + 1)|0;
  $41 = $6;
  $42 = HEAP32[$41>>2]|0;
  $43 = Math_imul($40, $42)|0;
  $44 = (($43) + 1)|0;
  $iy = $44;
 }
 $45 = $1;
 $46 = HEAP32[$45>>2]|0;
 $i__1 = $46;
 $i__ = 1;
 while(1) {
  $47 = $i__;
  $48 = $i__1;
  $49 = ($47|0)<=($48|0);
  if (!($49)) {
   break;
  }
  $50 = $2;
  $51 = +HEAPF64[$50>>3];
  $52 = $ix;
  $53 = $3;
  $54 = (($53) + ($52<<3)|0);
  $55 = +HEAPF64[$54>>3];
  $56 = $51 * $55;
  $57 = $iy;
  $58 = $5;
  $59 = (($58) + ($57<<3)|0);
  $60 = +HEAPF64[$59>>3];
  $61 = $60 + $56;
  HEAPF64[$59>>3] = $61;
  $62 = $4;
  $63 = HEAP32[$62>>2]|0;
  $64 = $ix;
  $65 = (($64) + ($63))|0;
  $ix = $65;
  $66 = $6;
  $67 = HEAP32[$66>>2]|0;
  $68 = $iy;
  $69 = (($68) + ($67))|0;
  $iy = $69;
  $70 = $i__;
  $71 = (($70) + 1)|0;
  $i__ = $71;
 }
 $0 = 0;
 $162 = $0;
 STACKTOP = sp;return ($162|0);
}
function _dcopy_($n,$dx,$incx,$dy,$incy) {
 $n = $n|0;
 $dx = $dx|0;
 $incx = $incx|0;
 $dy = $dy|0;
 $incy = $incy|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i__ = 0, $i__1 = 0, $ix = 0, $iy = 0;
 var $m = 0, $mp1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $dx;
 $3 = $incx;
 $4 = $dy;
 $5 = $incy;
 $6 = $4;
 $7 = ((($6)) + -8|0);
 $4 = $7;
 $8 = $2;
 $9 = ((($8)) + -8|0);
 $2 = $9;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)<=(0);
 if ($12) {
  $0 = 0;
  $155 = $0;
  STACKTOP = sp;return ($155|0);
 }
 $13 = $3;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(1);
 if ($15) {
  $16 = $5;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(1);
  if ($18) {
   $63 = $1;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($64|0) % 7)&-1;
   $m = $65;
   $66 = $m;
   $67 = ($66|0)==(0);
   if (!($67)) {
    $68 = $m;
    $i__1 = $68;
    $i__ = 1;
    while(1) {
     $69 = $i__;
     $70 = $i__1;
     $71 = ($69|0)<=($70|0);
     if (!($71)) {
      break;
     }
     $72 = $i__;
     $73 = $2;
     $74 = (($73) + ($72<<3)|0);
     $75 = +HEAPF64[$74>>3];
     $76 = $i__;
     $77 = $4;
     $78 = (($77) + ($76<<3)|0);
     HEAPF64[$78>>3] = $75;
     $79 = $i__;
     $80 = (($79) + 1)|0;
     $i__ = $80;
    }
    $81 = $1;
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)<(7);
    if ($83) {
     $0 = 0;
     $155 = $0;
     STACKTOP = sp;return ($155|0);
    }
   }
   $84 = $m;
   $85 = (($84) + 1)|0;
   $mp1 = $85;
   $86 = $1;
   $87 = HEAP32[$86>>2]|0;
   $i__1 = $87;
   $88 = $mp1;
   $i__ = $88;
   while(1) {
    $89 = $i__;
    $90 = $i__1;
    $91 = ($89|0)<=($90|0);
    if (!($91)) {
     break;
    }
    $92 = $i__;
    $93 = $2;
    $94 = (($93) + ($92<<3)|0);
    $95 = +HEAPF64[$94>>3];
    $96 = $i__;
    $97 = $4;
    $98 = (($97) + ($96<<3)|0);
    HEAPF64[$98>>3] = $95;
    $99 = $i__;
    $100 = (($99) + 1)|0;
    $101 = $2;
    $102 = (($101) + ($100<<3)|0);
    $103 = +HEAPF64[$102>>3];
    $104 = $i__;
    $105 = (($104) + 1)|0;
    $106 = $4;
    $107 = (($106) + ($105<<3)|0);
    HEAPF64[$107>>3] = $103;
    $108 = $i__;
    $109 = (($108) + 2)|0;
    $110 = $2;
    $111 = (($110) + ($109<<3)|0);
    $112 = +HEAPF64[$111>>3];
    $113 = $i__;
    $114 = (($113) + 2)|0;
    $115 = $4;
    $116 = (($115) + ($114<<3)|0);
    HEAPF64[$116>>3] = $112;
    $117 = $i__;
    $118 = (($117) + 3)|0;
    $119 = $2;
    $120 = (($119) + ($118<<3)|0);
    $121 = +HEAPF64[$120>>3];
    $122 = $i__;
    $123 = (($122) + 3)|0;
    $124 = $4;
    $125 = (($124) + ($123<<3)|0);
    HEAPF64[$125>>3] = $121;
    $126 = $i__;
    $127 = (($126) + 4)|0;
    $128 = $2;
    $129 = (($128) + ($127<<3)|0);
    $130 = +HEAPF64[$129>>3];
    $131 = $i__;
    $132 = (($131) + 4)|0;
    $133 = $4;
    $134 = (($133) + ($132<<3)|0);
    HEAPF64[$134>>3] = $130;
    $135 = $i__;
    $136 = (($135) + 5)|0;
    $137 = $2;
    $138 = (($137) + ($136<<3)|0);
    $139 = +HEAPF64[$138>>3];
    $140 = $i__;
    $141 = (($140) + 5)|0;
    $142 = $4;
    $143 = (($142) + ($141<<3)|0);
    HEAPF64[$143>>3] = $139;
    $144 = $i__;
    $145 = (($144) + 6)|0;
    $146 = $2;
    $147 = (($146) + ($145<<3)|0);
    $148 = +HEAPF64[$147>>3];
    $149 = $i__;
    $150 = (($149) + 6)|0;
    $151 = $4;
    $152 = (($151) + ($150<<3)|0);
    HEAPF64[$152>>3] = $148;
    $153 = $i__;
    $154 = (($153) + 7)|0;
    $i__ = $154;
   }
   $0 = 0;
   $155 = $0;
   STACKTOP = sp;return ($155|0);
  }
 }
 $ix = 1;
 $iy = 1;
 $19 = $3;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)<(0);
 if ($21) {
  $22 = $1;
  $23 = HEAP32[$22>>2]|0;
  $24 = (0 - ($23))|0;
  $25 = (($24) + 1)|0;
  $26 = $3;
  $27 = HEAP32[$26>>2]|0;
  $28 = Math_imul($25, $27)|0;
  $29 = (($28) + 1)|0;
  $ix = $29;
 }
 $30 = $5;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)<(0);
 if ($32) {
  $33 = $1;
  $34 = HEAP32[$33>>2]|0;
  $35 = (0 - ($34))|0;
  $36 = (($35) + 1)|0;
  $37 = $5;
  $38 = HEAP32[$37>>2]|0;
  $39 = Math_imul($36, $38)|0;
  $40 = (($39) + 1)|0;
  $iy = $40;
 }
 $41 = $1;
 $42 = HEAP32[$41>>2]|0;
 $i__1 = $42;
 $i__ = 1;
 while(1) {
  $43 = $i__;
  $44 = $i__1;
  $45 = ($43|0)<=($44|0);
  if (!($45)) {
   break;
  }
  $46 = $ix;
  $47 = $2;
  $48 = (($47) + ($46<<3)|0);
  $49 = +HEAPF64[$48>>3];
  $50 = $iy;
  $51 = $4;
  $52 = (($51) + ($50<<3)|0);
  HEAPF64[$52>>3] = $49;
  $53 = $3;
  $54 = HEAP32[$53>>2]|0;
  $55 = $ix;
  $56 = (($55) + ($54))|0;
  $ix = $56;
  $57 = $5;
  $58 = HEAP32[$57>>2]|0;
  $59 = $iy;
  $60 = (($59) + ($58))|0;
  $iy = $60;
  $61 = $i__;
  $62 = (($61) + 1)|0;
  $i__ = $62;
 }
 $0 = 0;
 $155 = $0;
 STACKTOP = sp;return ($155|0);
}
function _ddot_($n,$dx,$incx,$dy,$incy) {
 $n = $n|0;
 $dx = $dx|0;
 $incx = $incx|0;
 $dy = $dy|0;
 $incy = $incy|0;
 var $0 = 0.0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0.0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0.0, $124 = 0.0, $125 = 0.0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0.0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0.0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0.0, $16 = 0, $160 = 0.0, $161 = 0.0, $162 = 0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0.0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0;
 var $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dtemp = 0.0, $i__ = 0, $i__1 = 0, $ix = 0, $iy = 0, $m = 0, $mp1 = 0, $ret_val = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $dx;
 $3 = $incx;
 $4 = $dy;
 $5 = $incy;
 $6 = $4;
 $7 = ((($6)) + -8|0);
 $4 = $7;
 $8 = $2;
 $9 = ((($8)) + -8|0);
 $2 = $9;
 $ret_val = 0.0;
 $dtemp = 0.0;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)<=(0);
 if ($12) {
  $13 = $ret_val;
  $0 = $13;
  $166 = $0;
  STACKTOP = sp;return (+$166);
 }
 $14 = $3;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = $5;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(1);
  if ($19) {
   $70 = $1;
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71|0) % 5)&-1;
   $m = $72;
   $73 = $m;
   $74 = ($73|0)==(0);
   if ($74) {
    label = 18;
   } else {
    $75 = $m;
    $i__1 = $75;
    $i__ = 1;
    while(1) {
     $76 = $i__;
     $77 = $i__1;
     $78 = ($76|0)<=($77|0);
     if (!($78)) {
      break;
     }
     $79 = $i__;
     $80 = $2;
     $81 = (($80) + ($79<<3)|0);
     $82 = +HEAPF64[$81>>3];
     $83 = $i__;
     $84 = $4;
     $85 = (($84) + ($83<<3)|0);
     $86 = +HEAPF64[$85>>3];
     $87 = $82 * $86;
     $88 = $dtemp;
     $89 = $88 + $87;
     $dtemp = $89;
     $90 = $i__;
     $91 = (($90) + 1)|0;
     $i__ = $91;
    }
    $92 = $1;
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)<(5);
    if (!($94)) {
     label = 18;
    }
   }
   L14: do {
    if ((label|0) == 18) {
     $95 = $m;
     $96 = (($95) + 1)|0;
     $mp1 = $96;
     $97 = $1;
     $98 = HEAP32[$97>>2]|0;
     $i__1 = $98;
     $99 = $mp1;
     $i__ = $99;
     while(1) {
      $100 = $i__;
      $101 = $i__1;
      $102 = ($100|0)<=($101|0);
      if (!($102)) {
       break L14;
      }
      $103 = $dtemp;
      $104 = $i__;
      $105 = $2;
      $106 = (($105) + ($104<<3)|0);
      $107 = +HEAPF64[$106>>3];
      $108 = $i__;
      $109 = $4;
      $110 = (($109) + ($108<<3)|0);
      $111 = +HEAPF64[$110>>3];
      $112 = $107 * $111;
      $113 = $103 + $112;
      $114 = $i__;
      $115 = (($114) + 1)|0;
      $116 = $2;
      $117 = (($116) + ($115<<3)|0);
      $118 = +HEAPF64[$117>>3];
      $119 = $i__;
      $120 = (($119) + 1)|0;
      $121 = $4;
      $122 = (($121) + ($120<<3)|0);
      $123 = +HEAPF64[$122>>3];
      $124 = $118 * $123;
      $125 = $113 + $124;
      $126 = $i__;
      $127 = (($126) + 2)|0;
      $128 = $2;
      $129 = (($128) + ($127<<3)|0);
      $130 = +HEAPF64[$129>>3];
      $131 = $i__;
      $132 = (($131) + 2)|0;
      $133 = $4;
      $134 = (($133) + ($132<<3)|0);
      $135 = +HEAPF64[$134>>3];
      $136 = $130 * $135;
      $137 = $125 + $136;
      $138 = $i__;
      $139 = (($138) + 3)|0;
      $140 = $2;
      $141 = (($140) + ($139<<3)|0);
      $142 = +HEAPF64[$141>>3];
      $143 = $i__;
      $144 = (($143) + 3)|0;
      $145 = $4;
      $146 = (($145) + ($144<<3)|0);
      $147 = +HEAPF64[$146>>3];
      $148 = $142 * $147;
      $149 = $137 + $148;
      $150 = $i__;
      $151 = (($150) + 4)|0;
      $152 = $2;
      $153 = (($152) + ($151<<3)|0);
      $154 = +HEAPF64[$153>>3];
      $155 = $i__;
      $156 = (($155) + 4)|0;
      $157 = $4;
      $158 = (($157) + ($156<<3)|0);
      $159 = +HEAPF64[$158>>3];
      $160 = $154 * $159;
      $161 = $149 + $160;
      $dtemp = $161;
      $162 = $i__;
      $163 = (($162) + 5)|0;
      $i__ = $163;
     }
    }
   } while(0);
   $164 = $dtemp;
   $ret_val = $164;
   $165 = $ret_val;
   $0 = $165;
   $166 = $0;
   STACKTOP = sp;return (+$166);
  }
 }
 $ix = 1;
 $iy = 1;
 $20 = $3;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = $1;
  $24 = HEAP32[$23>>2]|0;
  $25 = (0 - ($24))|0;
  $26 = (($25) + 1)|0;
  $27 = $3;
  $28 = HEAP32[$27>>2]|0;
  $29 = Math_imul($26, $28)|0;
  $30 = (($29) + 1)|0;
  $ix = $30;
 }
 $31 = $5;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)<(0);
 if ($33) {
  $34 = $1;
  $35 = HEAP32[$34>>2]|0;
  $36 = (0 - ($35))|0;
  $37 = (($36) + 1)|0;
  $38 = $5;
  $39 = HEAP32[$38>>2]|0;
  $40 = Math_imul($37, $39)|0;
  $41 = (($40) + 1)|0;
  $iy = $41;
 }
 $42 = $1;
 $43 = HEAP32[$42>>2]|0;
 $i__1 = $43;
 $i__ = 1;
 while(1) {
  $44 = $i__;
  $45 = $i__1;
  $46 = ($44|0)<=($45|0);
  if (!($46)) {
   break;
  }
  $47 = $ix;
  $48 = $2;
  $49 = (($48) + ($47<<3)|0);
  $50 = +HEAPF64[$49>>3];
  $51 = $iy;
  $52 = $4;
  $53 = (($52) + ($51<<3)|0);
  $54 = +HEAPF64[$53>>3];
  $55 = $50 * $54;
  $56 = $dtemp;
  $57 = $56 + $55;
  $dtemp = $57;
  $58 = $3;
  $59 = HEAP32[$58>>2]|0;
  $60 = $ix;
  $61 = (($60) + ($59))|0;
  $ix = $61;
  $62 = $5;
  $63 = HEAP32[$62>>2]|0;
  $64 = $iy;
  $65 = (($64) + ($63))|0;
  $iy = $65;
  $66 = $i__;
  $67 = (($66) + 1)|0;
  $i__ = $67;
 }
 $68 = $dtemp;
 $ret_val = $68;
 $69 = $ret_val;
 $0 = $69;
 $166 = $0;
 STACKTOP = sp;return (+$166);
}
function _dscal_($n,$da,$dx,$incx) {
 $n = $n|0;
 $da = $da|0;
 $dx = $dx|0;
 $incx = $incx|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0.0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0.0, $116 = 0.0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0.0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0.0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0.0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0.0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0.0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0, $i__ = 0, $i__1 = 0, $i__2 = 0;
 var $m = 0, $mp1 = 0, $nincx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $da;
 $3 = $dx;
 $4 = $incx;
 $5 = $3;
 $6 = ((($5)) + -8|0);
 $3 = $6;
 $7 = $1;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<=(0);
 if (!($9)) {
  $10 = $4;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)<=(0);
  if (!($12)) {
   $13 = $4;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(1);
   $16 = $1;
   $17 = HEAP32[$16>>2]|0;
   if (!($15)) {
    $18 = $4;
    $19 = HEAP32[$18>>2]|0;
    $20 = Math_imul($17, $19)|0;
    $nincx = $20;
    $21 = $nincx;
    $i__1 = $21;
    $22 = $4;
    $23 = HEAP32[$22>>2]|0;
    $i__2 = $23;
    $i__ = 1;
    while(1) {
     $24 = $i__2;
     $25 = ($24|0)<(0);
     $26 = $i__;
     $27 = $i__1;
     $28 = ($26|0)>=($27|0);
     $29 = ($26|0)<=($27|0);
     $$sink = $25 ? $28 : $29;
     $30 = $$sink&1;
     $31 = ($30|0)!=(0);
     if (!($31)) {
      break;
     }
     $32 = $2;
     $33 = +HEAPF64[$32>>3];
     $34 = $i__;
     $35 = $3;
     $36 = (($35) + ($34<<3)|0);
     $37 = +HEAPF64[$36>>3];
     $38 = $33 * $37;
     $39 = $i__;
     $40 = $3;
     $41 = (($40) + ($39<<3)|0);
     HEAPF64[$41>>3] = $38;
     $42 = $i__2;
     $43 = $i__;
     $44 = (($43) + ($42))|0;
     $i__ = $44;
    }
    $0 = 0;
    $135 = $0;
    STACKTOP = sp;return ($135|0);
   }
   $45 = (($17|0) % 5)&-1;
   $m = $45;
   $46 = $m;
   $47 = ($46|0)==(0);
   if (!($47)) {
    $48 = $m;
    $i__2 = $48;
    $i__ = 1;
    while(1) {
     $49 = $i__;
     $50 = $i__2;
     $51 = ($49|0)<=($50|0);
     if (!($51)) {
      break;
     }
     $52 = $2;
     $53 = +HEAPF64[$52>>3];
     $54 = $i__;
     $55 = $3;
     $56 = (($55) + ($54<<3)|0);
     $57 = +HEAPF64[$56>>3];
     $58 = $53 * $57;
     $59 = $i__;
     $60 = $3;
     $61 = (($60) + ($59<<3)|0);
     HEAPF64[$61>>3] = $58;
     $62 = $i__;
     $63 = (($62) + 1)|0;
     $i__ = $63;
    }
    $64 = $1;
    $65 = HEAP32[$64>>2]|0;
    $66 = ($65|0)<(5);
    if ($66) {
     $0 = 0;
     $135 = $0;
     STACKTOP = sp;return ($135|0);
    }
   }
   $67 = $m;
   $68 = (($67) + 1)|0;
   $mp1 = $68;
   $69 = $1;
   $70 = HEAP32[$69>>2]|0;
   $i__2 = $70;
   $71 = $mp1;
   $i__ = $71;
   while(1) {
    $72 = $i__;
    $73 = $i__2;
    $74 = ($72|0)<=($73|0);
    if (!($74)) {
     break;
    }
    $75 = $2;
    $76 = +HEAPF64[$75>>3];
    $77 = $i__;
    $78 = $3;
    $79 = (($78) + ($77<<3)|0);
    $80 = +HEAPF64[$79>>3];
    $81 = $76 * $80;
    $82 = $i__;
    $83 = $3;
    $84 = (($83) + ($82<<3)|0);
    HEAPF64[$84>>3] = $81;
    $85 = $2;
    $86 = +HEAPF64[$85>>3];
    $87 = $i__;
    $88 = (($87) + 1)|0;
    $89 = $3;
    $90 = (($89) + ($88<<3)|0);
    $91 = +HEAPF64[$90>>3];
    $92 = $86 * $91;
    $93 = $i__;
    $94 = (($93) + 1)|0;
    $95 = $3;
    $96 = (($95) + ($94<<3)|0);
    HEAPF64[$96>>3] = $92;
    $97 = $2;
    $98 = +HEAPF64[$97>>3];
    $99 = $i__;
    $100 = (($99) + 2)|0;
    $101 = $3;
    $102 = (($101) + ($100<<3)|0);
    $103 = +HEAPF64[$102>>3];
    $104 = $98 * $103;
    $105 = $i__;
    $106 = (($105) + 2)|0;
    $107 = $3;
    $108 = (($107) + ($106<<3)|0);
    HEAPF64[$108>>3] = $104;
    $109 = $2;
    $110 = +HEAPF64[$109>>3];
    $111 = $i__;
    $112 = (($111) + 3)|0;
    $113 = $3;
    $114 = (($113) + ($112<<3)|0);
    $115 = +HEAPF64[$114>>3];
    $116 = $110 * $115;
    $117 = $i__;
    $118 = (($117) + 3)|0;
    $119 = $3;
    $120 = (($119) + ($118<<3)|0);
    HEAPF64[$120>>3] = $116;
    $121 = $2;
    $122 = +HEAPF64[$121>>3];
    $123 = $i__;
    $124 = (($123) + 4)|0;
    $125 = $3;
    $126 = (($125) + ($124<<3)|0);
    $127 = +HEAPF64[$126>>3];
    $128 = $122 * $127;
    $129 = $i__;
    $130 = (($129) + 4)|0;
    $131 = $3;
    $132 = (($131) + ($130<<3)|0);
    HEAPF64[$132>>3] = $128;
    $133 = $i__;
    $134 = (($133) + 5)|0;
    $i__ = $134;
   }
   $0 = 0;
   $135 = $0;
   STACKTOP = sp;return ($135|0);
  }
 }
 $0 = 0;
 $135 = $0;
 STACKTOP = sp;return ($135|0);
}
function _dswap_($n,$dx,$incx,$dy,$incy) {
 $n = $n|0;
 $dx = $dx|0;
 $incx = $incx|0;
 $dy = $dy|0;
 $incy = $incy|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0.0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0.0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0.0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dtemp = 0.0, $i__ = 0, $i__1 = 0, $ix = 0, $iy = 0, $m = 0, $mp1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $1 = $n;
 $2 = $dx;
 $3 = $incx;
 $4 = $dy;
 $5 = $incy;
 $6 = $4;
 $7 = ((($6)) + -8|0);
 $4 = $7;
 $8 = $2;
 $9 = ((($8)) + -8|0);
 $2 = $9;
 $10 = $1;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)<=(0);
 if ($12) {
  $0 = 0;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $13 = $3;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(1);
 if ($15) {
  $16 = $5;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(1);
  if ($18) {
   $71 = $1;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72|0) % 3)&-1;
   $m = $73;
   $74 = $m;
   $75 = ($74|0)==(0);
   if (!($75)) {
    $76 = $m;
    $i__1 = $76;
    $i__ = 1;
    while(1) {
     $77 = $i__;
     $78 = $i__1;
     $79 = ($77|0)<=($78|0);
     if (!($79)) {
      break;
     }
     $80 = $i__;
     $81 = $2;
     $82 = (($81) + ($80<<3)|0);
     $83 = +HEAPF64[$82>>3];
     $dtemp = $83;
     $84 = $i__;
     $85 = $4;
     $86 = (($85) + ($84<<3)|0);
     $87 = +HEAPF64[$86>>3];
     $88 = $i__;
     $89 = $2;
     $90 = (($89) + ($88<<3)|0);
     HEAPF64[$90>>3] = $87;
     $91 = $dtemp;
     $92 = $i__;
     $93 = $4;
     $94 = (($93) + ($92<<3)|0);
     HEAPF64[$94>>3] = $91;
     $95 = $i__;
     $96 = (($95) + 1)|0;
     $i__ = $96;
    }
    $97 = $1;
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)<(3);
    if ($99) {
     $0 = 0;
     $163 = $0;
     STACKTOP = sp;return ($163|0);
    }
   }
   $100 = $m;
   $101 = (($100) + 1)|0;
   $mp1 = $101;
   $102 = $1;
   $103 = HEAP32[$102>>2]|0;
   $i__1 = $103;
   $104 = $mp1;
   $i__ = $104;
   while(1) {
    $105 = $i__;
    $106 = $i__1;
    $107 = ($105|0)<=($106|0);
    if (!($107)) {
     break;
    }
    $108 = $i__;
    $109 = $2;
    $110 = (($109) + ($108<<3)|0);
    $111 = +HEAPF64[$110>>3];
    $dtemp = $111;
    $112 = $i__;
    $113 = $4;
    $114 = (($113) + ($112<<3)|0);
    $115 = +HEAPF64[$114>>3];
    $116 = $i__;
    $117 = $2;
    $118 = (($117) + ($116<<3)|0);
    HEAPF64[$118>>3] = $115;
    $119 = $dtemp;
    $120 = $i__;
    $121 = $4;
    $122 = (($121) + ($120<<3)|0);
    HEAPF64[$122>>3] = $119;
    $123 = $i__;
    $124 = (($123) + 1)|0;
    $125 = $2;
    $126 = (($125) + ($124<<3)|0);
    $127 = +HEAPF64[$126>>3];
    $dtemp = $127;
    $128 = $i__;
    $129 = (($128) + 1)|0;
    $130 = $4;
    $131 = (($130) + ($129<<3)|0);
    $132 = +HEAPF64[$131>>3];
    $133 = $i__;
    $134 = (($133) + 1)|0;
    $135 = $2;
    $136 = (($135) + ($134<<3)|0);
    HEAPF64[$136>>3] = $132;
    $137 = $dtemp;
    $138 = $i__;
    $139 = (($138) + 1)|0;
    $140 = $4;
    $141 = (($140) + ($139<<3)|0);
    HEAPF64[$141>>3] = $137;
    $142 = $i__;
    $143 = (($142) + 2)|0;
    $144 = $2;
    $145 = (($144) + ($143<<3)|0);
    $146 = +HEAPF64[$145>>3];
    $dtemp = $146;
    $147 = $i__;
    $148 = (($147) + 2)|0;
    $149 = $4;
    $150 = (($149) + ($148<<3)|0);
    $151 = +HEAPF64[$150>>3];
    $152 = $i__;
    $153 = (($152) + 2)|0;
    $154 = $2;
    $155 = (($154) + ($153<<3)|0);
    HEAPF64[$155>>3] = $151;
    $156 = $dtemp;
    $157 = $i__;
    $158 = (($157) + 2)|0;
    $159 = $4;
    $160 = (($159) + ($158<<3)|0);
    HEAPF64[$160>>3] = $156;
    $161 = $i__;
    $162 = (($161) + 3)|0;
    $i__ = $162;
   }
   $0 = 0;
   $163 = $0;
   STACKTOP = sp;return ($163|0);
  }
 }
 $ix = 1;
 $iy = 1;
 $19 = $3;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)<(0);
 if ($21) {
  $22 = $1;
  $23 = HEAP32[$22>>2]|0;
  $24 = (0 - ($23))|0;
  $25 = (($24) + 1)|0;
  $26 = $3;
  $27 = HEAP32[$26>>2]|0;
  $28 = Math_imul($25, $27)|0;
  $29 = (($28) + 1)|0;
  $ix = $29;
 }
 $30 = $5;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)<(0);
 if ($32) {
  $33 = $1;
  $34 = HEAP32[$33>>2]|0;
  $35 = (0 - ($34))|0;
  $36 = (($35) + 1)|0;
  $37 = $5;
  $38 = HEAP32[$37>>2]|0;
  $39 = Math_imul($36, $38)|0;
  $40 = (($39) + 1)|0;
  $iy = $40;
 }
 $41 = $1;
 $42 = HEAP32[$41>>2]|0;
 $i__1 = $42;
 $i__ = 1;
 while(1) {
  $43 = $i__;
  $44 = $i__1;
  $45 = ($43|0)<=($44|0);
  if (!($45)) {
   break;
  }
  $46 = $ix;
  $47 = $2;
  $48 = (($47) + ($46<<3)|0);
  $49 = +HEAPF64[$48>>3];
  $dtemp = $49;
  $50 = $iy;
  $51 = $4;
  $52 = (($51) + ($50<<3)|0);
  $53 = +HEAPF64[$52>>3];
  $54 = $ix;
  $55 = $2;
  $56 = (($55) + ($54<<3)|0);
  HEAPF64[$56>>3] = $53;
  $57 = $dtemp;
  $58 = $iy;
  $59 = $4;
  $60 = (($59) + ($58<<3)|0);
  HEAPF64[$60>>3] = $57;
  $61 = $3;
  $62 = HEAP32[$61>>2]|0;
  $63 = $ix;
  $64 = (($63) + ($62))|0;
  $ix = $64;
  $65 = $5;
  $66 = HEAP32[$65>>2]|0;
  $67 = $iy;
  $68 = (($67) + ($66))|0;
  $iy = $68;
  $69 = $i__;
  $70 = (($69) + 1)|0;
  $i__ = $70;
 }
 $0 = 0;
 $163 = $0;
 STACKTOP = sp;return ($163|0);
}
function _dger_($m,$n,$alpha,$x,$incx,$y,$incy,$a,$lda) {
 $m = $m|0;
 $n = $n|0;
 $alpha = $alpha|0;
 $x = $x|0;
 $incx = $incx|0;
 $y = $y|0;
 $incy = $incy|0;
 $a = $a|0;
 $lda = $lda|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0.0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0.0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0.0, $138 = 0.0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0.0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0.0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0.0, $93 = 0.0, $94 = 0.0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0, $a_offset = 0, $i__ = 0, $i__1 = 0, $i__2 = 0, $info = 0, $ix = 0;
 var $j = 0, $jy = 0, $kx = 0, $temp = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $info = sp + 8|0;
 $1 = $m;
 $2 = $n;
 $3 = $alpha;
 $4 = $x;
 $5 = $incx;
 $6 = $y;
 $7 = $incy;
 $8 = $a;
 $9 = $lda;
 $10 = $4;
 $11 = ((($10)) + -8|0);
 $4 = $11;
 $12 = $6;
 $13 = ((($12)) + -8|0);
 $6 = $13;
 $14 = $9;
 $15 = HEAP32[$14>>2]|0;
 $a_dim1 = $15;
 $16 = $a_dim1;
 $17 = (1 + ($16))|0;
 $a_offset = $17;
 $18 = $a_offset;
 $19 = $8;
 $20 = (0 - ($18))|0;
 $21 = (($19) + ($20<<3)|0);
 $8 = $21;
 HEAP32[$info>>2] = 0;
 $22 = $1;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)<(0);
 do {
  if ($24) {
   HEAP32[$info>>2] = 1;
  } else {
   $25 = $2;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)<(0);
   if ($27) {
    HEAP32[$info>>2] = 2;
    break;
   }
   $28 = $5;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0);
   if ($30) {
    HEAP32[$info>>2] = 5;
    break;
   }
   $31 = $7;
   $32 = HEAP32[$31>>2]|0;
   $33 = ($32|0)==(0);
   if ($33) {
    HEAP32[$info>>2] = 7;
    break;
   }
   $34 = $9;
   $35 = HEAP32[$34>>2]|0;
   $36 = $1;
   $37 = HEAP32[$36>>2]|0;
   $38 = (1)>=($37|0);
   if ($38) {
    $42 = 1;
   } else {
    $39 = $1;
    $40 = HEAP32[$39>>2]|0;
    $42 = $40;
   }
   $41 = ($35|0)<($42|0);
   if ($41) {
    HEAP32[$info>>2] = 9;
   }
  }
 } while(0);
 $43 = HEAP32[$info>>2]|0;
 $44 = ($43|0)!=(0);
 if ($44) {
  (_xerbla_(20444,$info)|0);
  $0 = 0;
  $172 = $0;
  STACKTOP = sp;return ($172|0);
 }
 $45 = $1;
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0);
 if (!($47)) {
  $48 = $2;
  $49 = HEAP32[$48>>2]|0;
  $50 = ($49|0)==(0);
  if (!($50)) {
   $51 = $3;
   $52 = +HEAPF64[$51>>3];
   $53 = $52 == 0.0;
   if (!($53)) {
    $54 = $7;
    $55 = HEAP32[$54>>2]|0;
    $56 = ($55|0)>(0);
    if ($56) {
     $jy = 1;
    } else {
     $57 = $2;
     $58 = HEAP32[$57>>2]|0;
     $59 = (($58) - 1)|0;
     $60 = $7;
     $61 = HEAP32[$60>>2]|0;
     $62 = Math_imul($59, $61)|0;
     $63 = (1 - ($62))|0;
     $jy = $63;
    }
    $64 = $5;
    $65 = HEAP32[$64>>2]|0;
    $66 = ($65|0)==(1);
    L30: do {
     if ($66) {
      $67 = $2;
      $68 = HEAP32[$67>>2]|0;
      $i__1 = $68;
      $j = 1;
      while(1) {
       $69 = $j;
       $70 = $i__1;
       $71 = ($69|0)<=($70|0);
       if (!($71)) {
        break L30;
       }
       $72 = $jy;
       $73 = $6;
       $74 = (($73) + ($72<<3)|0);
       $75 = +HEAPF64[$74>>3];
       $76 = $75 != 0.0;
       L35: do {
        if ($76) {
         $77 = $3;
         $78 = +HEAPF64[$77>>3];
         $79 = $jy;
         $80 = $6;
         $81 = (($80) + ($79<<3)|0);
         $82 = +HEAPF64[$81>>3];
         $83 = $78 * $82;
         $temp = $83;
         $84 = $1;
         $85 = HEAP32[$84>>2]|0;
         $i__2 = $85;
         $i__ = 1;
         while(1) {
          $86 = $i__;
          $87 = $i__2;
          $88 = ($86|0)<=($87|0);
          if (!($88)) {
           break L35;
          }
          $89 = $i__;
          $90 = $4;
          $91 = (($90) + ($89<<3)|0);
          $92 = +HEAPF64[$91>>3];
          $93 = $temp;
          $94 = $92 * $93;
          $95 = $i__;
          $96 = $j;
          $97 = $a_dim1;
          $98 = Math_imul($96, $97)|0;
          $99 = (($95) + ($98))|0;
          $100 = $8;
          $101 = (($100) + ($99<<3)|0);
          $102 = +HEAPF64[$101>>3];
          $103 = $102 + $94;
          HEAPF64[$101>>3] = $103;
          $104 = $i__;
          $105 = (($104) + 1)|0;
          $i__ = $105;
         }
        }
       } while(0);
       $106 = $7;
       $107 = HEAP32[$106>>2]|0;
       $108 = $jy;
       $109 = (($108) + ($107))|0;
       $jy = $109;
       $110 = $j;
       $111 = (($110) + 1)|0;
       $j = $111;
      }
     } else {
      $112 = $5;
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)>(0);
      if ($114) {
       $kx = 1;
      } else {
       $115 = $1;
       $116 = HEAP32[$115>>2]|0;
       $117 = (($116) - 1)|0;
       $118 = $5;
       $119 = HEAP32[$118>>2]|0;
       $120 = Math_imul($117, $119)|0;
       $121 = (1 - ($120))|0;
       $kx = $121;
      }
      $122 = $2;
      $123 = HEAP32[$122>>2]|0;
      $i__1 = $123;
      $j = 1;
      while(1) {
       $124 = $j;
       $125 = $i__1;
       $126 = ($124|0)<=($125|0);
       if (!($126)) {
        break L30;
       }
       $127 = $jy;
       $128 = $6;
       $129 = (($128) + ($127<<3)|0);
       $130 = +HEAPF64[$129>>3];
       $131 = $130 != 0.0;
       L49: do {
        if ($131) {
         $132 = $3;
         $133 = +HEAPF64[$132>>3];
         $134 = $jy;
         $135 = $6;
         $136 = (($135) + ($134<<3)|0);
         $137 = +HEAPF64[$136>>3];
         $138 = $133 * $137;
         $temp = $138;
         $139 = $kx;
         $ix = $139;
         $140 = $1;
         $141 = HEAP32[$140>>2]|0;
         $i__2 = $141;
         $i__ = 1;
         while(1) {
          $142 = $i__;
          $143 = $i__2;
          $144 = ($142|0)<=($143|0);
          if (!($144)) {
           break L49;
          }
          $145 = $ix;
          $146 = $4;
          $147 = (($146) + ($145<<3)|0);
          $148 = +HEAPF64[$147>>3];
          $149 = $temp;
          $150 = $148 * $149;
          $151 = $i__;
          $152 = $j;
          $153 = $a_dim1;
          $154 = Math_imul($152, $153)|0;
          $155 = (($151) + ($154))|0;
          $156 = $8;
          $157 = (($156) + ($155<<3)|0);
          $158 = +HEAPF64[$157>>3];
          $159 = $158 + $150;
          HEAPF64[$157>>3] = $159;
          $160 = $5;
          $161 = HEAP32[$160>>2]|0;
          $162 = $ix;
          $163 = (($162) + ($161))|0;
          $ix = $163;
          $164 = $i__;
          $165 = (($164) + 1)|0;
          $i__ = $165;
         }
        }
       } while(0);
       $166 = $7;
       $167 = HEAP32[$166>>2]|0;
       $168 = $jy;
       $169 = (($168) + ($167))|0;
       $jy = $169;
       $170 = $j;
       $171 = (($170) + 1)|0;
       $j = $171;
      }
     }
    } while(0);
    $0 = 0;
    $172 = $0;
    STACKTOP = sp;return ($172|0);
   }
  }
 }
 $0 = 0;
 $172 = $0;
 STACKTOP = sp;return ($172|0);
}
function _dgemm_($transa,$transb,$m,$n,$k,$alpha,$a,$lda,$b,$ldb,$beta,$c__,$ldc) {
 $transa = $transa|0;
 $transb = $transb|0;
 $m = $m|0;
 $n = $n|0;
 $k = $k|0;
 $alpha = $alpha|0;
 $a = $a|0;
 $lda = $lda|0;
 $b = $b|0;
 $ldb = $ldb|0;
 $beta = $beta|0;
 $c__ = $c__|0;
 $ldc = $ldc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0.0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0, $123 = 0, $124 = 0.0, $125 = 0, $126 = 0, $127 = 0.0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0.0, $168 = 0.0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0.0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0.0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0.0;
 var $224 = 0.0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0.0, $247 = 0, $248 = 0, $249 = 0.0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0.0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0.0, $273 = 0.0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0.0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0.0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0.0, $318 = 0.0, $319 = 0.0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0.0, $328 = 0.0, $329 = 0.0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0.0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0.0, $347 = 0.0, $348 = 0.0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0.0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0.0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0.0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0.0, $398 = 0.0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0.0, $421 = 0;
 var $422 = 0, $423 = 0.0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0.0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0.0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0.0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0.0, $456 = 0.0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0.0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0.0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0.0, $501 = 0.0, $502 = 0.0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0.0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0.0, $52 = 0, $520 = 0.0, $521 = 0.0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0, $a_offset = 0, $b_dim1 = 0, $b_offset = 0, $c_dim1 = 0, $c_offset = 0;
 var $i__ = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, $info = 0, $j = 0, $l = 0, $ncola = 0, $nota = 0, $notb = 0, $nrowa = 0, $nrowb = 0, $temp = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $info = sp + 28|0;
 $1 = $transa;
 $2 = $transb;
 $3 = $m;
 $4 = $n;
 $5 = $k;
 $6 = $alpha;
 $7 = $a;
 $8 = $lda;
 $9 = $b;
 $10 = $ldb;
 $11 = $beta;
 $12 = $c__;
 $13 = $ldc;
 $14 = $8;
 $15 = HEAP32[$14>>2]|0;
 $a_dim1 = $15;
 $16 = $a_dim1;
 $17 = (1 + ($16))|0;
 $a_offset = $17;
 $18 = $a_offset;
 $19 = $7;
 $20 = (0 - ($18))|0;
 $21 = (($19) + ($20<<3)|0);
 $7 = $21;
 $22 = $10;
 $23 = HEAP32[$22>>2]|0;
 $b_dim1 = $23;
 $24 = $b_dim1;
 $25 = (1 + ($24))|0;
 $b_offset = $25;
 $26 = $b_offset;
 $27 = $9;
 $28 = (0 - ($26))|0;
 $29 = (($27) + ($28<<3)|0);
 $9 = $29;
 $30 = $13;
 $31 = HEAP32[$30>>2]|0;
 $c_dim1 = $31;
 $32 = $c_dim1;
 $33 = (1 + ($32))|0;
 $c_offset = $33;
 $34 = $c_offset;
 $35 = $12;
 $36 = (0 - ($34))|0;
 $37 = (($35) + ($36<<3)|0);
 $12 = $37;
 $38 = $1;
 $39 = (_lsame_($38,20460)|0);
 $nota = $39;
 $40 = $2;
 $41 = (_lsame_($40,20460)|0);
 $notb = $41;
 $42 = $nota;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = $3;
  $45 = HEAP32[$44>>2]|0;
  $nrowa = $45;
  $46 = $5;
  $47 = HEAP32[$46>>2]|0;
  $ncola = $47;
 } else {
  $48 = $5;
  $49 = HEAP32[$48>>2]|0;
  $nrowa = $49;
  $50 = $3;
  $51 = HEAP32[$50>>2]|0;
  $ncola = $51;
 }
 $52 = $notb;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = $5;
  $55 = HEAP32[$54>>2]|0;
  $nrowb = $55;
 } else {
  $56 = $4;
  $57 = HEAP32[$56>>2]|0;
  $nrowb = $57;
 }
 HEAP32[$info>>2] = 0;
 $58 = $nota;
 $59 = ($58|0)!=(0);
 if ($59) {
  label = 11;
 } else {
  $60 = $1;
  $61 = (_lsame_($60,20468)|0);
  $62 = ($61|0)!=(0);
  if ($62) {
   label = 11;
  } else {
   $63 = $1;
   $64 = (_lsame_($63,20466)|0);
   $65 = ($64|0)!=(0);
   if ($65) {
    label = 11;
   } else {
    HEAP32[$info>>2] = 1;
   }
  }
 }
 do {
  if ((label|0) == 11) {
   $66 = $notb;
   $67 = ($66|0)!=(0);
   if (!($67)) {
    $68 = $2;
    $69 = (_lsame_($68,20468)|0);
    $70 = ($69|0)!=(0);
    if (!($70)) {
     $71 = $2;
     $72 = (_lsame_($71,20466)|0);
     $73 = ($72|0)!=(0);
     if (!($73)) {
      HEAP32[$info>>2] = 2;
      break;
     }
    }
   }
   $74 = $3;
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)<(0);
   if ($76) {
    HEAP32[$info>>2] = 3;
    break;
   }
   $77 = $4;
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)<(0);
   if ($79) {
    HEAP32[$info>>2] = 4;
    break;
   }
   $80 = $5;
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)<(0);
   if ($82) {
    HEAP32[$info>>2] = 5;
    break;
   }
   $83 = $8;
   $84 = HEAP32[$83>>2]|0;
   $85 = $nrowa;
   $86 = (1)>=($85|0);
   $87 = $nrowa;
   $88 = $86 ? 1 : $87;
   $89 = ($84|0)<($88|0);
   if ($89) {
    HEAP32[$info>>2] = 8;
    break;
   }
   $90 = $10;
   $91 = HEAP32[$90>>2]|0;
   $92 = $nrowb;
   $93 = (1)>=($92|0);
   $94 = $nrowb;
   $95 = $93 ? 1 : $94;
   $96 = ($91|0)<($95|0);
   if ($96) {
    HEAP32[$info>>2] = 10;
    break;
   }
   $97 = $13;
   $98 = HEAP32[$97>>2]|0;
   $99 = $3;
   $100 = HEAP32[$99>>2]|0;
   $101 = (1)>=($100|0);
   if ($101) {
    $105 = 1;
   } else {
    $102 = $3;
    $103 = HEAP32[$102>>2]|0;
    $105 = $103;
   }
   $104 = ($98|0)<($105|0);
   if ($104) {
    HEAP32[$info>>2] = 13;
   }
  }
 } while(0);
 $106 = HEAP32[$info>>2]|0;
 $107 = ($106|0)!=(0);
 if ($107) {
  (_xerbla_(20451,$info)|0);
  $0 = 0;
  $533 = $0;
  STACKTOP = sp;return ($533|0);
 }
 $108 = $3;
 $109 = HEAP32[$108>>2]|0;
 $110 = ($109|0)==(0);
 do {
  if (!($110)) {
   $111 = $4;
   $112 = HEAP32[$111>>2]|0;
   $113 = ($112|0)==(0);
   if (!($113)) {
    $114 = $6;
    $115 = +HEAPF64[$114>>3];
    $116 = $115 == 0.0;
    if ($116) {
     label = 35;
    } else {
     $117 = $5;
     $118 = HEAP32[$117>>2]|0;
     $119 = ($118|0)==(0);
     if ($119) {
      label = 35;
     }
    }
    if ((label|0) == 35) {
     $120 = $11;
     $121 = +HEAPF64[$120>>3];
     $122 = $121 == 1.0;
     if ($122) {
      break;
     }
    }
    $123 = $6;
    $124 = +HEAPF64[$123>>3];
    $125 = $124 == 0.0;
    if ($125) {
     $126 = $11;
     $127 = +HEAPF64[$126>>3];
     $128 = $127 == 0.0;
     $129 = $4;
     $130 = HEAP32[$129>>2]|0;
     $i__1 = $130;
     $j = 1;
     L54: do {
      if ($128) {
       while(1) {
        $131 = $j;
        $132 = $i__1;
        $133 = ($131|0)<=($132|0);
        if (!($133)) {
         break L54;
        }
        $134 = $3;
        $135 = HEAP32[$134>>2]|0;
        $i__2 = $135;
        $i__ = 1;
        while(1) {
         $136 = $i__;
         $137 = $i__2;
         $138 = ($136|0)<=($137|0);
         if (!($138)) {
          break;
         }
         $139 = $i__;
         $140 = $j;
         $141 = $c_dim1;
         $142 = Math_imul($140, $141)|0;
         $143 = (($139) + ($142))|0;
         $144 = $12;
         $145 = (($144) + ($143<<3)|0);
         HEAPF64[$145>>3] = 0.0;
         $146 = $i__;
         $147 = (($146) + 1)|0;
         $i__ = $147;
        }
        $148 = $j;
        $149 = (($148) + 1)|0;
        $j = $149;
       }
      } else {
       while(1) {
        $150 = $j;
        $151 = $i__1;
        $152 = ($150|0)<=($151|0);
        if (!($152)) {
         break L54;
        }
        $153 = $3;
        $154 = HEAP32[$153>>2]|0;
        $i__2 = $154;
        $i__ = 1;
        while(1) {
         $155 = $i__;
         $156 = $i__2;
         $157 = ($155|0)<=($156|0);
         if (!($157)) {
          break;
         }
         $158 = $11;
         $159 = +HEAPF64[$158>>3];
         $160 = $i__;
         $161 = $j;
         $162 = $c_dim1;
         $163 = Math_imul($161, $162)|0;
         $164 = (($160) + ($163))|0;
         $165 = $12;
         $166 = (($165) + ($164<<3)|0);
         $167 = +HEAPF64[$166>>3];
         $168 = $159 * $167;
         $169 = $i__;
         $170 = $j;
         $171 = $c_dim1;
         $172 = Math_imul($170, $171)|0;
         $173 = (($169) + ($172))|0;
         $174 = $12;
         $175 = (($174) + ($173<<3)|0);
         HEAPF64[$175>>3] = $168;
         $176 = $i__;
         $177 = (($176) + 1)|0;
         $i__ = $177;
        }
        $178 = $j;
        $179 = (($178) + 1)|0;
        $j = $179;
       }
      }
     } while(0);
     $0 = 0;
     $533 = $0;
     STACKTOP = sp;return ($533|0);
    }
    $180 = $notb;
    $181 = ($180|0)!=(0);
    $182 = $nota;
    $183 = ($182|0)!=(0);
    $184 = $4;
    $185 = HEAP32[$184>>2]|0;
    $i__1 = $185;
    $j = 1;
    L72: do {
     if ($181) {
      if (!($183)) {
       while(1) {
        $289 = $j;
        $290 = $i__1;
        $291 = ($289|0)<=($290|0);
        if (!($291)) {
         break L72;
        }
        $292 = $3;
        $293 = HEAP32[$292>>2]|0;
        $i__2 = $293;
        $i__ = 1;
        while(1) {
         $294 = $i__;
         $295 = $i__2;
         $296 = ($294|0)<=($295|0);
         if (!($296)) {
          break;
         }
         $temp = 0.0;
         $297 = $5;
         $298 = HEAP32[$297>>2]|0;
         $i__3 = $298;
         $l = 1;
         while(1) {
          $299 = $l;
          $300 = $i__3;
          $301 = ($299|0)<=($300|0);
          if (!($301)) {
           break;
          }
          $302 = $l;
          $303 = $i__;
          $304 = $a_dim1;
          $305 = Math_imul($303, $304)|0;
          $306 = (($302) + ($305))|0;
          $307 = $7;
          $308 = (($307) + ($306<<3)|0);
          $309 = +HEAPF64[$308>>3];
          $310 = $l;
          $311 = $j;
          $312 = $b_dim1;
          $313 = Math_imul($311, $312)|0;
          $314 = (($310) + ($313))|0;
          $315 = $9;
          $316 = (($315) + ($314<<3)|0);
          $317 = +HEAPF64[$316>>3];
          $318 = $309 * $317;
          $319 = $temp;
          $320 = $319 + $318;
          $temp = $320;
          $321 = $l;
          $322 = (($321) + 1)|0;
          $l = $322;
         }
         $323 = $11;
         $324 = +HEAPF64[$323>>3];
         $325 = $324 == 0.0;
         $326 = $6;
         $327 = +HEAPF64[$326>>3];
         $328 = $temp;
         $329 = $327 * $328;
         if ($325) {
          $330 = $i__;
          $331 = $j;
          $332 = $c_dim1;
          $333 = Math_imul($331, $332)|0;
          $334 = (($330) + ($333))|0;
          $335 = $12;
          $336 = (($335) + ($334<<3)|0);
          HEAPF64[$336>>3] = $329;
         } else {
          $337 = $11;
          $338 = +HEAPF64[$337>>3];
          $339 = $i__;
          $340 = $j;
          $341 = $c_dim1;
          $342 = Math_imul($340, $341)|0;
          $343 = (($339) + ($342))|0;
          $344 = $12;
          $345 = (($344) + ($343<<3)|0);
          $346 = +HEAPF64[$345>>3];
          $347 = $338 * $346;
          $348 = $329 + $347;
          $349 = $i__;
          $350 = $j;
          $351 = $c_dim1;
          $352 = Math_imul($350, $351)|0;
          $353 = (($349) + ($352))|0;
          $354 = $12;
          $355 = (($354) + ($353<<3)|0);
          HEAPF64[$355>>3] = $348;
         }
         $356 = $i__;
         $357 = (($356) + 1)|0;
         $i__ = $357;
        }
        $358 = $j;
        $359 = (($358) + 1)|0;
        $j = $359;
       }
      }
      while(1) {
       $186 = $j;
       $187 = $i__1;
       $188 = ($186|0)<=($187|0);
       if (!($188)) {
        break L72;
       }
       $189 = $11;
       $190 = +HEAPF64[$189>>3];
       $191 = $190 == 0.0;
       L93: do {
        if ($191) {
         $192 = $3;
         $193 = HEAP32[$192>>2]|0;
         $i__2 = $193;
         $i__ = 1;
         while(1) {
          $194 = $i__;
          $195 = $i__2;
          $196 = ($194|0)<=($195|0);
          if (!($196)) {
           break L93;
          }
          $197 = $i__;
          $198 = $j;
          $199 = $c_dim1;
          $200 = Math_imul($198, $199)|0;
          $201 = (($197) + ($200))|0;
          $202 = $12;
          $203 = (($202) + ($201<<3)|0);
          HEAPF64[$203>>3] = 0.0;
          $204 = $i__;
          $205 = (($204) + 1)|0;
          $i__ = $205;
         }
        } else {
         $206 = $11;
         $207 = +HEAPF64[$206>>3];
         $208 = $207 != 1.0;
         if ($208) {
          $209 = $3;
          $210 = HEAP32[$209>>2]|0;
          $i__2 = $210;
          $i__ = 1;
          while(1) {
           $211 = $i__;
           $212 = $i__2;
           $213 = ($211|0)<=($212|0);
           if (!($213)) {
            break L93;
           }
           $214 = $11;
           $215 = +HEAPF64[$214>>3];
           $216 = $i__;
           $217 = $j;
           $218 = $c_dim1;
           $219 = Math_imul($217, $218)|0;
           $220 = (($216) + ($219))|0;
           $221 = $12;
           $222 = (($221) + ($220<<3)|0);
           $223 = +HEAPF64[$222>>3];
           $224 = $215 * $223;
           $225 = $i__;
           $226 = $j;
           $227 = $c_dim1;
           $228 = Math_imul($226, $227)|0;
           $229 = (($225) + ($228))|0;
           $230 = $12;
           $231 = (($230) + ($229<<3)|0);
           HEAPF64[$231>>3] = $224;
           $232 = $i__;
           $233 = (($232) + 1)|0;
           $i__ = $233;
          }
         }
        }
       } while(0);
       $234 = $5;
       $235 = HEAP32[$234>>2]|0;
       $i__2 = $235;
       $l = 1;
       while(1) {
        $236 = $l;
        $237 = $i__2;
        $238 = ($236|0)<=($237|0);
        if (!($238)) {
         break;
        }
        $239 = $l;
        $240 = $j;
        $241 = $b_dim1;
        $242 = Math_imul($240, $241)|0;
        $243 = (($239) + ($242))|0;
        $244 = $9;
        $245 = (($244) + ($243<<3)|0);
        $246 = +HEAPF64[$245>>3];
        $247 = $246 != 0.0;
        L107: do {
         if ($247) {
          $248 = $6;
          $249 = +HEAPF64[$248>>3];
          $250 = $l;
          $251 = $j;
          $252 = $b_dim1;
          $253 = Math_imul($251, $252)|0;
          $254 = (($250) + ($253))|0;
          $255 = $9;
          $256 = (($255) + ($254<<3)|0);
          $257 = +HEAPF64[$256>>3];
          $258 = $249 * $257;
          $temp = $258;
          $259 = $3;
          $260 = HEAP32[$259>>2]|0;
          $i__3 = $260;
          $i__ = 1;
          while(1) {
           $261 = $i__;
           $262 = $i__3;
           $263 = ($261|0)<=($262|0);
           if (!($263)) {
            break L107;
           }
           $264 = $temp;
           $265 = $i__;
           $266 = $l;
           $267 = $a_dim1;
           $268 = Math_imul($266, $267)|0;
           $269 = (($265) + ($268))|0;
           $270 = $7;
           $271 = (($270) + ($269<<3)|0);
           $272 = +HEAPF64[$271>>3];
           $273 = $264 * $272;
           $274 = $i__;
           $275 = $j;
           $276 = $c_dim1;
           $277 = Math_imul($275, $276)|0;
           $278 = (($274) + ($277))|0;
           $279 = $12;
           $280 = (($279) + ($278<<3)|0);
           $281 = +HEAPF64[$280>>3];
           $282 = $281 + $273;
           HEAPF64[$280>>3] = $282;
           $283 = $i__;
           $284 = (($283) + 1)|0;
           $i__ = $284;
          }
         }
        } while(0);
        $285 = $l;
        $286 = (($285) + 1)|0;
        $l = $286;
       }
       $287 = $j;
       $288 = (($287) + 1)|0;
       $j = $288;
      }
     } else {
      if (!($183)) {
       while(1) {
        $462 = $j;
        $463 = $i__1;
        $464 = ($462|0)<=($463|0);
        if (!($464)) {
         break L72;
        }
        $465 = $3;
        $466 = HEAP32[$465>>2]|0;
        $i__2 = $466;
        $i__ = 1;
        while(1) {
         $467 = $i__;
         $468 = $i__2;
         $469 = ($467|0)<=($468|0);
         if (!($469)) {
          break;
         }
         $temp = 0.0;
         $470 = $5;
         $471 = HEAP32[$470>>2]|0;
         $i__3 = $471;
         $l = 1;
         while(1) {
          $472 = $l;
          $473 = $i__3;
          $474 = ($472|0)<=($473|0);
          if (!($474)) {
           break;
          }
          $475 = $l;
          $476 = $i__;
          $477 = $a_dim1;
          $478 = Math_imul($476, $477)|0;
          $479 = (($475) + ($478))|0;
          $480 = $7;
          $481 = (($480) + ($479<<3)|0);
          $482 = +HEAPF64[$481>>3];
          $483 = $j;
          $484 = $l;
          $485 = $b_dim1;
          $486 = Math_imul($484, $485)|0;
          $487 = (($483) + ($486))|0;
          $488 = $9;
          $489 = (($488) + ($487<<3)|0);
          $490 = +HEAPF64[$489>>3];
          $491 = $482 * $490;
          $492 = $temp;
          $493 = $492 + $491;
          $temp = $493;
          $494 = $l;
          $495 = (($494) + 1)|0;
          $l = $495;
         }
         $496 = $11;
         $497 = +HEAPF64[$496>>3];
         $498 = $497 == 0.0;
         $499 = $6;
         $500 = +HEAPF64[$499>>3];
         $501 = $temp;
         $502 = $500 * $501;
         if ($498) {
          $503 = $i__;
          $504 = $j;
          $505 = $c_dim1;
          $506 = Math_imul($504, $505)|0;
          $507 = (($503) + ($506))|0;
          $508 = $12;
          $509 = (($508) + ($507<<3)|0);
          HEAPF64[$509>>3] = $502;
         } else {
          $510 = $11;
          $511 = +HEAPF64[$510>>3];
          $512 = $i__;
          $513 = $j;
          $514 = $c_dim1;
          $515 = Math_imul($513, $514)|0;
          $516 = (($512) + ($515))|0;
          $517 = $12;
          $518 = (($517) + ($516<<3)|0);
          $519 = +HEAPF64[$518>>3];
          $520 = $511 * $519;
          $521 = $502 + $520;
          $522 = $i__;
          $523 = $j;
          $524 = $c_dim1;
          $525 = Math_imul($523, $524)|0;
          $526 = (($522) + ($525))|0;
          $527 = $12;
          $528 = (($527) + ($526<<3)|0);
          HEAPF64[$528>>3] = $521;
         }
         $529 = $i__;
         $530 = (($529) + 1)|0;
         $i__ = $530;
        }
        $531 = $j;
        $532 = (($531) + 1)|0;
        $j = $532;
       }
      }
      while(1) {
       $360 = $j;
       $361 = $i__1;
       $362 = ($360|0)<=($361|0);
       if (!($362)) {
        break L72;
       }
       $363 = $11;
       $364 = +HEAPF64[$363>>3];
       $365 = $364 == 0.0;
       L134: do {
        if ($365) {
         $366 = $3;
         $367 = HEAP32[$366>>2]|0;
         $i__2 = $367;
         $i__ = 1;
         while(1) {
          $368 = $i__;
          $369 = $i__2;
          $370 = ($368|0)<=($369|0);
          if (!($370)) {
           break L134;
          }
          $371 = $i__;
          $372 = $j;
          $373 = $c_dim1;
          $374 = Math_imul($372, $373)|0;
          $375 = (($371) + ($374))|0;
          $376 = $12;
          $377 = (($376) + ($375<<3)|0);
          HEAPF64[$377>>3] = 0.0;
          $378 = $i__;
          $379 = (($378) + 1)|0;
          $i__ = $379;
         }
        } else {
         $380 = $11;
         $381 = +HEAPF64[$380>>3];
         $382 = $381 != 1.0;
         if ($382) {
          $383 = $3;
          $384 = HEAP32[$383>>2]|0;
          $i__2 = $384;
          $i__ = 1;
          while(1) {
           $385 = $i__;
           $386 = $i__2;
           $387 = ($385|0)<=($386|0);
           if (!($387)) {
            break L134;
           }
           $388 = $11;
           $389 = +HEAPF64[$388>>3];
           $390 = $i__;
           $391 = $j;
           $392 = $c_dim1;
           $393 = Math_imul($391, $392)|0;
           $394 = (($390) + ($393))|0;
           $395 = $12;
           $396 = (($395) + ($394<<3)|0);
           $397 = +HEAPF64[$396>>3];
           $398 = $389 * $397;
           $399 = $i__;
           $400 = $j;
           $401 = $c_dim1;
           $402 = Math_imul($400, $401)|0;
           $403 = (($399) + ($402))|0;
           $404 = $12;
           $405 = (($404) + ($403<<3)|0);
           HEAPF64[$405>>3] = $398;
           $406 = $i__;
           $407 = (($406) + 1)|0;
           $i__ = $407;
          }
         }
        }
       } while(0);
       $408 = $5;
       $409 = HEAP32[$408>>2]|0;
       $i__2 = $409;
       $l = 1;
       while(1) {
        $410 = $l;
        $411 = $i__2;
        $412 = ($410|0)<=($411|0);
        $413 = $j;
        if (!($412)) {
         break;
        }
        $414 = $l;
        $415 = $b_dim1;
        $416 = Math_imul($414, $415)|0;
        $417 = (($413) + ($416))|0;
        $418 = $9;
        $419 = (($418) + ($417<<3)|0);
        $420 = +HEAPF64[$419>>3];
        $421 = $420 != 0.0;
        L148: do {
         if ($421) {
          $422 = $6;
          $423 = +HEAPF64[$422>>3];
          $424 = $j;
          $425 = $l;
          $426 = $b_dim1;
          $427 = Math_imul($425, $426)|0;
          $428 = (($424) + ($427))|0;
          $429 = $9;
          $430 = (($429) + ($428<<3)|0);
          $431 = +HEAPF64[$430>>3];
          $432 = $423 * $431;
          $temp = $432;
          $433 = $3;
          $434 = HEAP32[$433>>2]|0;
          $i__3 = $434;
          $i__ = 1;
          while(1) {
           $435 = $i__;
           $436 = $i__3;
           $437 = ($435|0)<=($436|0);
           if (!($437)) {
            break L148;
           }
           $438 = $temp;
           $439 = $i__;
           $440 = $l;
           $441 = $a_dim1;
           $442 = Math_imul($440, $441)|0;
           $443 = (($439) + ($442))|0;
           $444 = $7;
           $445 = (($444) + ($443<<3)|0);
           $446 = +HEAPF64[$445>>3];
           $447 = $438 * $446;
           $448 = $i__;
           $449 = $j;
           $450 = $c_dim1;
           $451 = Math_imul($449, $450)|0;
           $452 = (($448) + ($451))|0;
           $453 = $12;
           $454 = (($453) + ($452<<3)|0);
           $455 = +HEAPF64[$454>>3];
           $456 = $455 + $447;
           HEAPF64[$454>>3] = $456;
           $457 = $i__;
           $458 = (($457) + 1)|0;
           $i__ = $458;
          }
         }
        } while(0);
        $459 = $l;
        $460 = (($459) + 1)|0;
        $l = $460;
       }
       $461 = (($413) + 1)|0;
       $j = $461;
      }
     }
    } while(0);
    $0 = 0;
    $533 = $0;
    STACKTOP = sp;return ($533|0);
   }
  }
 } while(0);
 $0 = 0;
 $533 = $0;
 STACKTOP = sp;return ($533|0);
}
function _dtrsm_($side,$uplo,$transa,$diag,$m,$n,$alpha,$a,$lda,$b,$ldb) {
 $side = $side|0;
 $uplo = $uplo|0;
 $transa = $transa|0;
 $diag = $diag|0;
 $m = $m|0;
 $n = $n|0;
 $alpha = $alpha|0;
 $a = $a|0;
 $lda = $lda|0;
 $b = $b|0;
 $ldb = $ldb|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0.0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0.0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0.0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0.0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0.0, $212 = 0.0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0.0, $221 = 0.0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0.0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0.0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0.0, $249 = 0.0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0.0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0.0, $291 = 0.0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0.0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0.0, $315 = 0.0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0.0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0.0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0.0, $349 = 0.0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0.0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0.0, $371 = 0.0, $372 = 0.0, $373 = 0.0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0.0, $386 = 0.0;
 var $387 = 0.0, $388 = 0.0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0.0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0.0, $417 = 0.0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0.0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0.0, $441 = 0.0, $442 = 0.0, $443 = 0.0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0.0, $456 = 0.0, $457 = 0.0, $458 = 0.0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0.0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0.0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0.0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0.0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0.0, $536 = 0.0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0.0, $545 = 0.0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0.0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0.0, $575 = 0.0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0.0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0.0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0.0, $607 = 0.0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0.0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0.0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0.0, $654 = 0.0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0.0, $663 = 0.0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0.0, $678 = 0.0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0.0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0.0;
 var $693 = 0.0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0.0, $717 = 0.0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0.0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0.0, $732 = 0.0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0.0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0.0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0.0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0.0, $778 = 0.0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0.0, $787 = 0.0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0.0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0.0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0.0, $81 = 0, $810 = 0.0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0.0, $835 = 0.0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0.0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0.0, $85 = 0, $850 = 0.0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0.0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0.0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0.0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0.0, $898 = 0.0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0.0, $907 = 0.0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0.0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0.0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0.0, $93 = 0, $930 = 0.0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $95 = 0, $96 = 0.0;
 var $97 = 0, $98 = 0, $99 = 0, $a_dim1 = 0, $a_offset = 0, $b_dim1 = 0, $b_offset = 0, $i__ = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, $info = 0, $j = 0, $k = 0, $lside = 0, $nounit = 0, $nrowa = 0, $temp = 0.0, $upper = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $info = sp + 24|0;
 $1 = $side;
 $2 = $uplo;
 $3 = $transa;
 $4 = $diag;
 $5 = $m;
 $6 = $n;
 $7 = $alpha;
 $8 = $a;
 $9 = $lda;
 $10 = $b;
 $11 = $ldb;
 $12 = $9;
 $13 = HEAP32[$12>>2]|0;
 $a_dim1 = $13;
 $14 = $a_dim1;
 $15 = (1 + ($14))|0;
 $a_offset = $15;
 $16 = $a_offset;
 $17 = $8;
 $18 = (0 - ($16))|0;
 $19 = (($17) + ($18<<3)|0);
 $8 = $19;
 $20 = $11;
 $21 = HEAP32[$20>>2]|0;
 $b_dim1 = $21;
 $22 = $b_dim1;
 $23 = (1 + ($22))|0;
 $b_offset = $23;
 $24 = $b_offset;
 $25 = $10;
 $26 = (0 - ($24))|0;
 $27 = (($25) + ($26<<3)|0);
 $10 = $27;
 $28 = $1;
 $29 = (_lsame_($28,20458)|0);
 $lside = $29;
 $30 = $lside;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = $5;
  $33 = HEAP32[$32>>2]|0;
  $nrowa = $33;
 } else {
  $34 = $6;
  $35 = HEAP32[$34>>2]|0;
  $nrowa = $35;
 }
 $36 = $4;
 $37 = (_lsame_($36,20460)|0);
 $nounit = $37;
 $38 = $2;
 $39 = (_lsame_($38,20462)|0);
 $upper = $39;
 HEAP32[$info>>2] = 0;
 $40 = $lside;
 $41 = ($40|0)!=(0);
 if ($41) {
  label = 7;
 } else {
  $42 = $1;
  $43 = (_lsame_($42,20464)|0);
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 7;
  } else {
   HEAP32[$info>>2] = 1;
  }
 }
 do {
  if ((label|0) == 7) {
   $45 = $upper;
   $46 = ($45|0)!=(0);
   if (!($46)) {
    $47 = $2;
    $48 = (_lsame_($47,20458)|0);
    $49 = ($48|0)!=(0);
    if (!($49)) {
     HEAP32[$info>>2] = 2;
     break;
    }
   }
   $50 = $3;
   $51 = (_lsame_($50,20460)|0);
   $52 = ($51|0)!=(0);
   if (!($52)) {
    $53 = $3;
    $54 = (_lsame_($53,20466)|0);
    $55 = ($54|0)!=(0);
    if (!($55)) {
     $56 = $3;
     $57 = (_lsame_($56,20468)|0);
     $58 = ($57|0)!=(0);
     if (!($58)) {
      HEAP32[$info>>2] = 3;
      break;
     }
    }
   }
   $59 = $4;
   $60 = (_lsame_($59,20462)|0);
   $61 = ($60|0)!=(0);
   if (!($61)) {
    $62 = $4;
    $63 = (_lsame_($62,20460)|0);
    $64 = ($63|0)!=(0);
    if (!($64)) {
     HEAP32[$info>>2] = 4;
     break;
    }
   }
   $65 = $5;
   $66 = HEAP32[$65>>2]|0;
   $67 = ($66|0)<(0);
   if ($67) {
    HEAP32[$info>>2] = 5;
    break;
   }
   $68 = $6;
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)<(0);
   if ($70) {
    HEAP32[$info>>2] = 6;
    break;
   }
   $71 = $9;
   $72 = HEAP32[$71>>2]|0;
   $73 = $nrowa;
   $74 = (1)>=($73|0);
   $75 = $nrowa;
   $76 = $74 ? 1 : $75;
   $77 = ($72|0)<($76|0);
   if ($77) {
    HEAP32[$info>>2] = 9;
    break;
   }
   $78 = $11;
   $79 = HEAP32[$78>>2]|0;
   $80 = $5;
   $81 = HEAP32[$80>>2]|0;
   $82 = (1)>=($81|0);
   if ($82) {
    $86 = 1;
   } else {
    $83 = $5;
    $84 = HEAP32[$83>>2]|0;
    $86 = $84;
   }
   $85 = ($79|0)<($86|0);
   if ($85) {
    HEAP32[$info>>2] = 11;
   }
  }
 } while(0);
 $87 = HEAP32[$info>>2]|0;
 $88 = ($87|0)!=(0);
 if ($88) {
  (_xerbla_(20470,$info)|0);
  $0 = 0;
  $942 = $0;
  STACKTOP = sp;return ($942|0);
 }
 $89 = $5;
 $90 = HEAP32[$89>>2]|0;
 $91 = ($90|0)==(0);
 if (!($91)) {
  $92 = $6;
  $93 = HEAP32[$92>>2]|0;
  $94 = ($93|0)==(0);
  if (!($94)) {
   $95 = $7;
   $96 = +HEAPF64[$95>>3];
   $97 = $96 == 0.0;
   if ($97) {
    $98 = $6;
    $99 = HEAP32[$98>>2]|0;
    $i__1 = $99;
    $j = 1;
    while(1) {
     $100 = $j;
     $101 = $i__1;
     $102 = ($100|0)<=($101|0);
     if (!($102)) {
      break;
     }
     $103 = $5;
     $104 = HEAP32[$103>>2]|0;
     $i__2 = $104;
     $i__ = 1;
     while(1) {
      $105 = $i__;
      $106 = $i__2;
      $107 = ($105|0)<=($106|0);
      if (!($107)) {
       break;
      }
      $108 = $i__;
      $109 = $j;
      $110 = $b_dim1;
      $111 = Math_imul($109, $110)|0;
      $112 = (($108) + ($111))|0;
      $113 = $10;
      $114 = (($113) + ($112<<3)|0);
      HEAPF64[$114>>3] = 0.0;
      $115 = $i__;
      $116 = (($115) + 1)|0;
      $i__ = $116;
     }
     $117 = $j;
     $118 = (($117) + 1)|0;
     $j = $118;
    }
    $0 = 0;
    $942 = $0;
    STACKTOP = sp;return ($942|0);
   }
   $119 = $lside;
   $120 = ($119|0)!=(0);
   $121 = $3;
   $122 = (_lsame_($121,20460)|0);
   $123 = ($122|0)!=(0);
   $124 = $upper;
   $125 = ($124|0)!=(0);
   $126 = $6;
   $127 = HEAP32[$126>>2]|0;
   L56: do {
    if ($120) {
     $i__1 = $127;
     $j = 1;
     if (!($123)) {
      if ($125) {
       while(1) {
        $331 = $j;
        $332 = $i__1;
        $333 = ($331|0)<=($332|0);
        if (!($333)) {
         break L56;
        }
        $334 = $5;
        $335 = HEAP32[$334>>2]|0;
        $i__2 = $335;
        $i__ = 1;
        while(1) {
         $336 = $i__;
         $337 = $i__2;
         $338 = ($336|0)<=($337|0);
         if (!($338)) {
          break;
         }
         $339 = $7;
         $340 = +HEAPF64[$339>>3];
         $341 = $i__;
         $342 = $j;
         $343 = $b_dim1;
         $344 = Math_imul($342, $343)|0;
         $345 = (($341) + ($344))|0;
         $346 = $10;
         $347 = (($346) + ($345<<3)|0);
         $348 = +HEAPF64[$347>>3];
         $349 = $340 * $348;
         $temp = $349;
         $350 = $i__;
         $351 = (($350) - 1)|0;
         $i__3 = $351;
         $k = 1;
         while(1) {
          $352 = $k;
          $353 = $i__3;
          $354 = ($352|0)<=($353|0);
          if (!($354)) {
           break;
          }
          $355 = $k;
          $356 = $i__;
          $357 = $a_dim1;
          $358 = Math_imul($356, $357)|0;
          $359 = (($355) + ($358))|0;
          $360 = $8;
          $361 = (($360) + ($359<<3)|0);
          $362 = +HEAPF64[$361>>3];
          $363 = $k;
          $364 = $j;
          $365 = $b_dim1;
          $366 = Math_imul($364, $365)|0;
          $367 = (($363) + ($366))|0;
          $368 = $10;
          $369 = (($368) + ($367<<3)|0);
          $370 = +HEAPF64[$369>>3];
          $371 = $362 * $370;
          $372 = $temp;
          $373 = $372 - $371;
          $temp = $373;
          $374 = $k;
          $375 = (($374) + 1)|0;
          $k = $375;
         }
         $376 = $nounit;
         $377 = ($376|0)!=(0);
         if ($377) {
          $378 = $i__;
          $379 = $i__;
          $380 = $a_dim1;
          $381 = Math_imul($379, $380)|0;
          $382 = (($378) + ($381))|0;
          $383 = $8;
          $384 = (($383) + ($382<<3)|0);
          $385 = +HEAPF64[$384>>3];
          $386 = $temp;
          $387 = $386 / $385;
          $temp = $387;
         }
         $388 = $temp;
         $389 = $i__;
         $390 = $j;
         $391 = $b_dim1;
         $392 = Math_imul($390, $391)|0;
         $393 = (($389) + ($392))|0;
         $394 = $10;
         $395 = (($394) + ($393<<3)|0);
         HEAPF64[$395>>3] = $388;
         $396 = $i__;
         $397 = (($396) + 1)|0;
         $i__ = $397;
        }
        $398 = $j;
        $399 = (($398) + 1)|0;
        $j = $399;
       }
      } else {
       while(1) {
        $400 = $j;
        $401 = $i__1;
        $402 = ($400|0)<=($401|0);
        if (!($402)) {
         break L56;
        }
        $403 = $5;
        $404 = HEAP32[$403>>2]|0;
        $i__ = $404;
        while(1) {
         $405 = $i__;
         $406 = ($405|0)>=(1);
         if (!($406)) {
          break;
         }
         $407 = $7;
         $408 = +HEAPF64[$407>>3];
         $409 = $i__;
         $410 = $j;
         $411 = $b_dim1;
         $412 = Math_imul($410, $411)|0;
         $413 = (($409) + ($412))|0;
         $414 = $10;
         $415 = (($414) + ($413<<3)|0);
         $416 = +HEAPF64[$415>>3];
         $417 = $408 * $416;
         $temp = $417;
         $418 = $5;
         $419 = HEAP32[$418>>2]|0;
         $i__2 = $419;
         $420 = $i__;
         $421 = (($420) + 1)|0;
         $k = $421;
         while(1) {
          $422 = $k;
          $423 = $i__2;
          $424 = ($422|0)<=($423|0);
          if (!($424)) {
           break;
          }
          $425 = $k;
          $426 = $i__;
          $427 = $a_dim1;
          $428 = Math_imul($426, $427)|0;
          $429 = (($425) + ($428))|0;
          $430 = $8;
          $431 = (($430) + ($429<<3)|0);
          $432 = +HEAPF64[$431>>3];
          $433 = $k;
          $434 = $j;
          $435 = $b_dim1;
          $436 = Math_imul($434, $435)|0;
          $437 = (($433) + ($436))|0;
          $438 = $10;
          $439 = (($438) + ($437<<3)|0);
          $440 = +HEAPF64[$439>>3];
          $441 = $432 * $440;
          $442 = $temp;
          $443 = $442 - $441;
          $temp = $443;
          $444 = $k;
          $445 = (($444) + 1)|0;
          $k = $445;
         }
         $446 = $nounit;
         $447 = ($446|0)!=(0);
         if ($447) {
          $448 = $i__;
          $449 = $i__;
          $450 = $a_dim1;
          $451 = Math_imul($449, $450)|0;
          $452 = (($448) + ($451))|0;
          $453 = $8;
          $454 = (($453) + ($452<<3)|0);
          $455 = +HEAPF64[$454>>3];
          $456 = $temp;
          $457 = $456 / $455;
          $temp = $457;
         }
         $458 = $temp;
         $459 = $i__;
         $460 = $j;
         $461 = $b_dim1;
         $462 = Math_imul($460, $461)|0;
         $463 = (($459) + ($462))|0;
         $464 = $10;
         $465 = (($464) + ($463<<3)|0);
         HEAPF64[$465>>3] = $458;
         $466 = $i__;
         $467 = (($466) + -1)|0;
         $i__ = $467;
        }
        $468 = $j;
        $469 = (($468) + 1)|0;
        $j = $469;
       }
      }
     }
     if ($125) {
      while(1) {
       $128 = $j;
       $129 = $i__1;
       $130 = ($128|0)<=($129|0);
       if (!($130)) {
        break L56;
       }
       $131 = $7;
       $132 = +HEAPF64[$131>>3];
       $133 = $132 != 1.0;
       L94: do {
        if ($133) {
         $134 = $5;
         $135 = HEAP32[$134>>2]|0;
         $i__2 = $135;
         $i__ = 1;
         while(1) {
          $136 = $i__;
          $137 = $i__2;
          $138 = ($136|0)<=($137|0);
          if (!($138)) {
           break L94;
          }
          $139 = $7;
          $140 = +HEAPF64[$139>>3];
          $141 = $i__;
          $142 = $j;
          $143 = $b_dim1;
          $144 = Math_imul($142, $143)|0;
          $145 = (($141) + ($144))|0;
          $146 = $10;
          $147 = (($146) + ($145<<3)|0);
          $148 = +HEAPF64[$147>>3];
          $149 = $140 * $148;
          $150 = $i__;
          $151 = $j;
          $152 = $b_dim1;
          $153 = Math_imul($151, $152)|0;
          $154 = (($150) + ($153))|0;
          $155 = $10;
          $156 = (($155) + ($154<<3)|0);
          HEAPF64[$156>>3] = $149;
          $157 = $i__;
          $158 = (($157) + 1)|0;
          $i__ = $158;
         }
        }
       } while(0);
       $159 = $5;
       $160 = HEAP32[$159>>2]|0;
       $k = $160;
       while(1) {
        $161 = $k;
        $162 = ($161|0)>=(1);
        if (!($162)) {
         break;
        }
        $163 = $k;
        $164 = $j;
        $165 = $b_dim1;
        $166 = Math_imul($164, $165)|0;
        $167 = (($163) + ($166))|0;
        $168 = $10;
        $169 = (($168) + ($167<<3)|0);
        $170 = +HEAPF64[$169>>3];
        $171 = $170 != 0.0;
        L103: do {
         if ($171) {
          $172 = $nounit;
          $173 = ($172|0)!=(0);
          if ($173) {
           $174 = $k;
           $175 = $k;
           $176 = $a_dim1;
           $177 = Math_imul($175, $176)|0;
           $178 = (($174) + ($177))|0;
           $179 = $8;
           $180 = (($179) + ($178<<3)|0);
           $181 = +HEAPF64[$180>>3];
           $182 = $k;
           $183 = $j;
           $184 = $b_dim1;
           $185 = Math_imul($183, $184)|0;
           $186 = (($182) + ($185))|0;
           $187 = $10;
           $188 = (($187) + ($186<<3)|0);
           $189 = +HEAPF64[$188>>3];
           $190 = $189 / $181;
           HEAPF64[$188>>3] = $190;
          }
          $191 = $k;
          $192 = (($191) - 1)|0;
          $i__2 = $192;
          $i__ = 1;
          while(1) {
           $193 = $i__;
           $194 = $i__2;
           $195 = ($193|0)<=($194|0);
           if (!($195)) {
            break L103;
           }
           $196 = $k;
           $197 = $j;
           $198 = $b_dim1;
           $199 = Math_imul($197, $198)|0;
           $200 = (($196) + ($199))|0;
           $201 = $10;
           $202 = (($201) + ($200<<3)|0);
           $203 = +HEAPF64[$202>>3];
           $204 = $i__;
           $205 = $k;
           $206 = $a_dim1;
           $207 = Math_imul($205, $206)|0;
           $208 = (($204) + ($207))|0;
           $209 = $8;
           $210 = (($209) + ($208<<3)|0);
           $211 = +HEAPF64[$210>>3];
           $212 = $203 * $211;
           $213 = $i__;
           $214 = $j;
           $215 = $b_dim1;
           $216 = Math_imul($214, $215)|0;
           $217 = (($213) + ($216))|0;
           $218 = $10;
           $219 = (($218) + ($217<<3)|0);
           $220 = +HEAPF64[$219>>3];
           $221 = $220 - $212;
           HEAPF64[$219>>3] = $221;
           $222 = $i__;
           $223 = (($222) + 1)|0;
           $i__ = $223;
          }
         }
        } while(0);
        $224 = $k;
        $225 = (($224) + -1)|0;
        $k = $225;
       }
       $226 = $j;
       $227 = (($226) + 1)|0;
       $j = $227;
      }
     } else {
      while(1) {
       $228 = $j;
       $229 = $i__1;
       $230 = ($228|0)<=($229|0);
       if (!($230)) {
        break L56;
       }
       $231 = $7;
       $232 = +HEAPF64[$231>>3];
       $233 = $232 != 1.0;
       L116: do {
        if ($233) {
         $234 = $5;
         $235 = HEAP32[$234>>2]|0;
         $i__2 = $235;
         $i__ = 1;
         while(1) {
          $236 = $i__;
          $237 = $i__2;
          $238 = ($236|0)<=($237|0);
          if (!($238)) {
           break L116;
          }
          $239 = $7;
          $240 = +HEAPF64[$239>>3];
          $241 = $i__;
          $242 = $j;
          $243 = $b_dim1;
          $244 = Math_imul($242, $243)|0;
          $245 = (($241) + ($244))|0;
          $246 = $10;
          $247 = (($246) + ($245<<3)|0);
          $248 = +HEAPF64[$247>>3];
          $249 = $240 * $248;
          $250 = $i__;
          $251 = $j;
          $252 = $b_dim1;
          $253 = Math_imul($251, $252)|0;
          $254 = (($250) + ($253))|0;
          $255 = $10;
          $256 = (($255) + ($254<<3)|0);
          HEAPF64[$256>>3] = $249;
          $257 = $i__;
          $258 = (($257) + 1)|0;
          $i__ = $258;
         }
        }
       } while(0);
       $259 = $5;
       $260 = HEAP32[$259>>2]|0;
       $i__2 = $260;
       $k = 1;
       while(1) {
        $261 = $k;
        $262 = $i__2;
        $263 = ($261|0)<=($262|0);
        if (!($263)) {
         break;
        }
        $264 = $k;
        $265 = $j;
        $266 = $b_dim1;
        $267 = Math_imul($265, $266)|0;
        $268 = (($264) + ($267))|0;
        $269 = $10;
        $270 = (($269) + ($268<<3)|0);
        $271 = +HEAPF64[$270>>3];
        $272 = $271 != 0.0;
        L125: do {
         if ($272) {
          $273 = $nounit;
          $274 = ($273|0)!=(0);
          if ($274) {
           $275 = $k;
           $276 = $k;
           $277 = $a_dim1;
           $278 = Math_imul($276, $277)|0;
           $279 = (($275) + ($278))|0;
           $280 = $8;
           $281 = (($280) + ($279<<3)|0);
           $282 = +HEAPF64[$281>>3];
           $283 = $k;
           $284 = $j;
           $285 = $b_dim1;
           $286 = Math_imul($284, $285)|0;
           $287 = (($283) + ($286))|0;
           $288 = $10;
           $289 = (($288) + ($287<<3)|0);
           $290 = +HEAPF64[$289>>3];
           $291 = $290 / $282;
           HEAPF64[$289>>3] = $291;
          }
          $292 = $5;
          $293 = HEAP32[$292>>2]|0;
          $i__3 = $293;
          $294 = $k;
          $295 = (($294) + 1)|0;
          $i__ = $295;
          while(1) {
           $296 = $i__;
           $297 = $i__3;
           $298 = ($296|0)<=($297|0);
           if (!($298)) {
            break L125;
           }
           $299 = $k;
           $300 = $j;
           $301 = $b_dim1;
           $302 = Math_imul($300, $301)|0;
           $303 = (($299) + ($302))|0;
           $304 = $10;
           $305 = (($304) + ($303<<3)|0);
           $306 = +HEAPF64[$305>>3];
           $307 = $i__;
           $308 = $k;
           $309 = $a_dim1;
           $310 = Math_imul($308, $309)|0;
           $311 = (($307) + ($310))|0;
           $312 = $8;
           $313 = (($312) + ($311<<3)|0);
           $314 = +HEAPF64[$313>>3];
           $315 = $306 * $314;
           $316 = $i__;
           $317 = $j;
           $318 = $b_dim1;
           $319 = Math_imul($317, $318)|0;
           $320 = (($316) + ($319))|0;
           $321 = $10;
           $322 = (($321) + ($320<<3)|0);
           $323 = +HEAPF64[$322>>3];
           $324 = $323 - $315;
           HEAPF64[$322>>3] = $324;
           $325 = $i__;
           $326 = (($325) + 1)|0;
           $i__ = $326;
          }
         }
        } while(0);
        $327 = $k;
        $328 = (($327) + 1)|0;
        $k = $328;
       }
       $329 = $j;
       $330 = (($329) + 1)|0;
       $j = $330;
      }
     }
    } else {
     if ($123) {
      if ($125) {
       $i__1 = $127;
       $j = 1;
       while(1) {
        $470 = $j;
        $471 = $i__1;
        $472 = ($470|0)<=($471|0);
        if (!($472)) {
         break L56;
        }
        $473 = $7;
        $474 = +HEAPF64[$473>>3];
        $475 = $474 != 1.0;
        L143: do {
         if ($475) {
          $476 = $5;
          $477 = HEAP32[$476>>2]|0;
          $i__2 = $477;
          $i__ = 1;
          while(1) {
           $478 = $i__;
           $479 = $i__2;
           $480 = ($478|0)<=($479|0);
           if (!($480)) {
            break L143;
           }
           $481 = $7;
           $482 = +HEAPF64[$481>>3];
           $483 = $i__;
           $484 = $j;
           $485 = $b_dim1;
           $486 = Math_imul($484, $485)|0;
           $487 = (($483) + ($486))|0;
           $488 = $10;
           $489 = (($488) + ($487<<3)|0);
           $490 = +HEAPF64[$489>>3];
           $491 = $482 * $490;
           $492 = $i__;
           $493 = $j;
           $494 = $b_dim1;
           $495 = Math_imul($493, $494)|0;
           $496 = (($492) + ($495))|0;
           $497 = $10;
           $498 = (($497) + ($496<<3)|0);
           HEAPF64[$498>>3] = $491;
           $499 = $i__;
           $500 = (($499) + 1)|0;
           $i__ = $500;
          }
         }
        } while(0);
        $501 = $j;
        $502 = (($501) - 1)|0;
        $i__2 = $502;
        $k = 1;
        while(1) {
         $503 = $k;
         $504 = $i__2;
         $505 = ($503|0)<=($504|0);
         if (!($505)) {
          break;
         }
         $506 = $k;
         $507 = $j;
         $508 = $a_dim1;
         $509 = Math_imul($507, $508)|0;
         $510 = (($506) + ($509))|0;
         $511 = $8;
         $512 = (($511) + ($510<<3)|0);
         $513 = +HEAPF64[$512>>3];
         $514 = $513 != 0.0;
         L152: do {
          if ($514) {
           $515 = $5;
           $516 = HEAP32[$515>>2]|0;
           $i__3 = $516;
           $i__ = 1;
           while(1) {
            $517 = $i__;
            $518 = $i__3;
            $519 = ($517|0)<=($518|0);
            if (!($519)) {
             break L152;
            }
            $520 = $k;
            $521 = $j;
            $522 = $a_dim1;
            $523 = Math_imul($521, $522)|0;
            $524 = (($520) + ($523))|0;
            $525 = $8;
            $526 = (($525) + ($524<<3)|0);
            $527 = +HEAPF64[$526>>3];
            $528 = $i__;
            $529 = $k;
            $530 = $b_dim1;
            $531 = Math_imul($529, $530)|0;
            $532 = (($528) + ($531))|0;
            $533 = $10;
            $534 = (($533) + ($532<<3)|0);
            $535 = +HEAPF64[$534>>3];
            $536 = $527 * $535;
            $537 = $i__;
            $538 = $j;
            $539 = $b_dim1;
            $540 = Math_imul($538, $539)|0;
            $541 = (($537) + ($540))|0;
            $542 = $10;
            $543 = (($542) + ($541<<3)|0);
            $544 = +HEAPF64[$543>>3];
            $545 = $544 - $536;
            HEAPF64[$543>>3] = $545;
            $546 = $i__;
            $547 = (($546) + 1)|0;
            $i__ = $547;
           }
          }
         } while(0);
         $548 = $k;
         $549 = (($548) + 1)|0;
         $k = $549;
        }
        $550 = $nounit;
        $551 = ($550|0)!=(0);
        L159: do {
         if ($551) {
          $552 = $j;
          $553 = $j;
          $554 = $a_dim1;
          $555 = Math_imul($553, $554)|0;
          $556 = (($552) + ($555))|0;
          $557 = $8;
          $558 = (($557) + ($556<<3)|0);
          $559 = +HEAPF64[$558>>3];
          $560 = 1.0 / $559;
          $temp = $560;
          $561 = $5;
          $562 = HEAP32[$561>>2]|0;
          $i__2 = $562;
          $i__ = 1;
          while(1) {
           $563 = $i__;
           $564 = $i__2;
           $565 = ($563|0)<=($564|0);
           if (!($565)) {
            break L159;
           }
           $566 = $temp;
           $567 = $i__;
           $568 = $j;
           $569 = $b_dim1;
           $570 = Math_imul($568, $569)|0;
           $571 = (($567) + ($570))|0;
           $572 = $10;
           $573 = (($572) + ($571<<3)|0);
           $574 = +HEAPF64[$573>>3];
           $575 = $566 * $574;
           $576 = $i__;
           $577 = $j;
           $578 = $b_dim1;
           $579 = Math_imul($577, $578)|0;
           $580 = (($576) + ($579))|0;
           $581 = $10;
           $582 = (($581) + ($580<<3)|0);
           HEAPF64[$582>>3] = $575;
           $583 = $i__;
           $584 = (($583) + 1)|0;
           $i__ = $584;
          }
         }
        } while(0);
        $585 = $j;
        $586 = (($585) + 1)|0;
        $j = $586;
       }
      } else {
       $j = $127;
       while(1) {
        $587 = $j;
        $588 = ($587|0)>=(1);
        if (!($588)) {
         break L56;
        }
        $589 = $7;
        $590 = +HEAPF64[$589>>3];
        $591 = $590 != 1.0;
        L169: do {
         if ($591) {
          $592 = $5;
          $593 = HEAP32[$592>>2]|0;
          $i__1 = $593;
          $i__ = 1;
          while(1) {
           $594 = $i__;
           $595 = $i__1;
           $596 = ($594|0)<=($595|0);
           if (!($596)) {
            break L169;
           }
           $597 = $7;
           $598 = +HEAPF64[$597>>3];
           $599 = $i__;
           $600 = $j;
           $601 = $b_dim1;
           $602 = Math_imul($600, $601)|0;
           $603 = (($599) + ($602))|0;
           $604 = $10;
           $605 = (($604) + ($603<<3)|0);
           $606 = +HEAPF64[$605>>3];
           $607 = $598 * $606;
           $608 = $i__;
           $609 = $j;
           $610 = $b_dim1;
           $611 = Math_imul($609, $610)|0;
           $612 = (($608) + ($611))|0;
           $613 = $10;
           $614 = (($613) + ($612<<3)|0);
           HEAPF64[$614>>3] = $607;
           $615 = $i__;
           $616 = (($615) + 1)|0;
           $i__ = $616;
          }
         }
        } while(0);
        $617 = $6;
        $618 = HEAP32[$617>>2]|0;
        $i__1 = $618;
        $619 = $j;
        $620 = (($619) + 1)|0;
        $k = $620;
        while(1) {
         $621 = $k;
         $622 = $i__1;
         $623 = ($621|0)<=($622|0);
         if (!($623)) {
          break;
         }
         $624 = $k;
         $625 = $j;
         $626 = $a_dim1;
         $627 = Math_imul($625, $626)|0;
         $628 = (($624) + ($627))|0;
         $629 = $8;
         $630 = (($629) + ($628<<3)|0);
         $631 = +HEAPF64[$630>>3];
         $632 = $631 != 0.0;
         L178: do {
          if ($632) {
           $633 = $5;
           $634 = HEAP32[$633>>2]|0;
           $i__2 = $634;
           $i__ = 1;
           while(1) {
            $635 = $i__;
            $636 = $i__2;
            $637 = ($635|0)<=($636|0);
            if (!($637)) {
             break L178;
            }
            $638 = $k;
            $639 = $j;
            $640 = $a_dim1;
            $641 = Math_imul($639, $640)|0;
            $642 = (($638) + ($641))|0;
            $643 = $8;
            $644 = (($643) + ($642<<3)|0);
            $645 = +HEAPF64[$644>>3];
            $646 = $i__;
            $647 = $k;
            $648 = $b_dim1;
            $649 = Math_imul($647, $648)|0;
            $650 = (($646) + ($649))|0;
            $651 = $10;
            $652 = (($651) + ($650<<3)|0);
            $653 = +HEAPF64[$652>>3];
            $654 = $645 * $653;
            $655 = $i__;
            $656 = $j;
            $657 = $b_dim1;
            $658 = Math_imul($656, $657)|0;
            $659 = (($655) + ($658))|0;
            $660 = $10;
            $661 = (($660) + ($659<<3)|0);
            $662 = +HEAPF64[$661>>3];
            $663 = $662 - $654;
            HEAPF64[$661>>3] = $663;
            $664 = $i__;
            $665 = (($664) + 1)|0;
            $i__ = $665;
           }
          }
         } while(0);
         $666 = $k;
         $667 = (($666) + 1)|0;
         $k = $667;
        }
        $668 = $nounit;
        $669 = ($668|0)!=(0);
        L185: do {
         if ($669) {
          $670 = $j;
          $671 = $j;
          $672 = $a_dim1;
          $673 = Math_imul($671, $672)|0;
          $674 = (($670) + ($673))|0;
          $675 = $8;
          $676 = (($675) + ($674<<3)|0);
          $677 = +HEAPF64[$676>>3];
          $678 = 1.0 / $677;
          $temp = $678;
          $679 = $5;
          $680 = HEAP32[$679>>2]|0;
          $i__1 = $680;
          $i__ = 1;
          while(1) {
           $681 = $i__;
           $682 = $i__1;
           $683 = ($681|0)<=($682|0);
           if (!($683)) {
            break L185;
           }
           $684 = $temp;
           $685 = $i__;
           $686 = $j;
           $687 = $b_dim1;
           $688 = Math_imul($686, $687)|0;
           $689 = (($685) + ($688))|0;
           $690 = $10;
           $691 = (($690) + ($689<<3)|0);
           $692 = +HEAPF64[$691>>3];
           $693 = $684 * $692;
           $694 = $i__;
           $695 = $j;
           $696 = $b_dim1;
           $697 = Math_imul($695, $696)|0;
           $698 = (($694) + ($697))|0;
           $699 = $10;
           $700 = (($699) + ($698<<3)|0);
           HEAPF64[$700>>3] = $693;
           $701 = $i__;
           $702 = (($701) + 1)|0;
           $i__ = $702;
          }
         }
        } while(0);
        $703 = $j;
        $704 = (($703) + -1)|0;
        $j = $704;
       }
      }
     } else {
      if ($125) {
       $k = $127;
       while(1) {
        $705 = $k;
        $706 = ($705|0)>=(1);
        if (!($706)) {
         break L56;
        }
        $707 = $nounit;
        $708 = ($707|0)!=(0);
        L197: do {
         if ($708) {
          $709 = $k;
          $710 = $k;
          $711 = $a_dim1;
          $712 = Math_imul($710, $711)|0;
          $713 = (($709) + ($712))|0;
          $714 = $8;
          $715 = (($714) + ($713<<3)|0);
          $716 = +HEAPF64[$715>>3];
          $717 = 1.0 / $716;
          $temp = $717;
          $718 = $5;
          $719 = HEAP32[$718>>2]|0;
          $i__1 = $719;
          $i__ = 1;
          while(1) {
           $720 = $i__;
           $721 = $i__1;
           $722 = ($720|0)<=($721|0);
           if (!($722)) {
            break L197;
           }
           $723 = $temp;
           $724 = $i__;
           $725 = $k;
           $726 = $b_dim1;
           $727 = Math_imul($725, $726)|0;
           $728 = (($724) + ($727))|0;
           $729 = $10;
           $730 = (($729) + ($728<<3)|0);
           $731 = +HEAPF64[$730>>3];
           $732 = $723 * $731;
           $733 = $i__;
           $734 = $k;
           $735 = $b_dim1;
           $736 = Math_imul($734, $735)|0;
           $737 = (($733) + ($736))|0;
           $738 = $10;
           $739 = (($738) + ($737<<3)|0);
           HEAPF64[$739>>3] = $732;
           $740 = $i__;
           $741 = (($740) + 1)|0;
           $i__ = $741;
          }
         }
        } while(0);
        $742 = $k;
        $743 = (($742) - 1)|0;
        $i__1 = $743;
        $j = 1;
        while(1) {
         $744 = $j;
         $745 = $i__1;
         $746 = ($744|0)<=($745|0);
         if (!($746)) {
          break;
         }
         $747 = $j;
         $748 = $k;
         $749 = $a_dim1;
         $750 = Math_imul($748, $749)|0;
         $751 = (($747) + ($750))|0;
         $752 = $8;
         $753 = (($752) + ($751<<3)|0);
         $754 = +HEAPF64[$753>>3];
         $755 = $754 != 0.0;
         L206: do {
          if ($755) {
           $756 = $j;
           $757 = $k;
           $758 = $a_dim1;
           $759 = Math_imul($757, $758)|0;
           $760 = (($756) + ($759))|0;
           $761 = $8;
           $762 = (($761) + ($760<<3)|0);
           $763 = +HEAPF64[$762>>3];
           $temp = $763;
           $764 = $5;
           $765 = HEAP32[$764>>2]|0;
           $i__2 = $765;
           $i__ = 1;
           while(1) {
            $766 = $i__;
            $767 = $i__2;
            $768 = ($766|0)<=($767|0);
            if (!($768)) {
             break L206;
            }
            $769 = $temp;
            $770 = $i__;
            $771 = $k;
            $772 = $b_dim1;
            $773 = Math_imul($771, $772)|0;
            $774 = (($770) + ($773))|0;
            $775 = $10;
            $776 = (($775) + ($774<<3)|0);
            $777 = +HEAPF64[$776>>3];
            $778 = $769 * $777;
            $779 = $i__;
            $780 = $j;
            $781 = $b_dim1;
            $782 = Math_imul($780, $781)|0;
            $783 = (($779) + ($782))|0;
            $784 = $10;
            $785 = (($784) + ($783<<3)|0);
            $786 = +HEAPF64[$785>>3];
            $787 = $786 - $778;
            HEAPF64[$785>>3] = $787;
            $788 = $i__;
            $789 = (($788) + 1)|0;
            $i__ = $789;
           }
          }
         } while(0);
         $790 = $j;
         $791 = (($790) + 1)|0;
         $j = $791;
        }
        $792 = $7;
        $793 = +HEAPF64[$792>>3];
        $794 = $793 != 1.0;
        L213: do {
         if ($794) {
          $795 = $5;
          $796 = HEAP32[$795>>2]|0;
          $i__1 = $796;
          $i__ = 1;
          while(1) {
           $797 = $i__;
           $798 = $i__1;
           $799 = ($797|0)<=($798|0);
           if (!($799)) {
            break L213;
           }
           $800 = $7;
           $801 = +HEAPF64[$800>>3];
           $802 = $i__;
           $803 = $k;
           $804 = $b_dim1;
           $805 = Math_imul($803, $804)|0;
           $806 = (($802) + ($805))|0;
           $807 = $10;
           $808 = (($807) + ($806<<3)|0);
           $809 = +HEAPF64[$808>>3];
           $810 = $801 * $809;
           $811 = $i__;
           $812 = $k;
           $813 = $b_dim1;
           $814 = Math_imul($812, $813)|0;
           $815 = (($811) + ($814))|0;
           $816 = $10;
           $817 = (($816) + ($815<<3)|0);
           HEAPF64[$817>>3] = $810;
           $818 = $i__;
           $819 = (($818) + 1)|0;
           $i__ = $819;
          }
         }
        } while(0);
        $820 = $k;
        $821 = (($820) + -1)|0;
        $k = $821;
       }
      } else {
       $i__1 = $127;
       $k = 1;
       while(1) {
        $822 = $k;
        $823 = $i__1;
        $824 = ($822|0)<=($823|0);
        if (!($824)) {
         break L56;
        }
        $825 = $nounit;
        $826 = ($825|0)!=(0);
        L223: do {
         if ($826) {
          $827 = $k;
          $828 = $k;
          $829 = $a_dim1;
          $830 = Math_imul($828, $829)|0;
          $831 = (($827) + ($830))|0;
          $832 = $8;
          $833 = (($832) + ($831<<3)|0);
          $834 = +HEAPF64[$833>>3];
          $835 = 1.0 / $834;
          $temp = $835;
          $836 = $5;
          $837 = HEAP32[$836>>2]|0;
          $i__2 = $837;
          $i__ = 1;
          while(1) {
           $838 = $i__;
           $839 = $i__2;
           $840 = ($838|0)<=($839|0);
           if (!($840)) {
            break L223;
           }
           $841 = $temp;
           $842 = $i__;
           $843 = $k;
           $844 = $b_dim1;
           $845 = Math_imul($843, $844)|0;
           $846 = (($842) + ($845))|0;
           $847 = $10;
           $848 = (($847) + ($846<<3)|0);
           $849 = +HEAPF64[$848>>3];
           $850 = $841 * $849;
           $851 = $i__;
           $852 = $k;
           $853 = $b_dim1;
           $854 = Math_imul($852, $853)|0;
           $855 = (($851) + ($854))|0;
           $856 = $10;
           $857 = (($856) + ($855<<3)|0);
           HEAPF64[$857>>3] = $850;
           $858 = $i__;
           $859 = (($858) + 1)|0;
           $i__ = $859;
          }
         }
        } while(0);
        $860 = $6;
        $861 = HEAP32[$860>>2]|0;
        $i__2 = $861;
        $862 = $k;
        $863 = (($862) + 1)|0;
        $j = $863;
        while(1) {
         $864 = $j;
         $865 = $i__2;
         $866 = ($864|0)<=($865|0);
         if (!($866)) {
          break;
         }
         $867 = $j;
         $868 = $k;
         $869 = $a_dim1;
         $870 = Math_imul($868, $869)|0;
         $871 = (($867) + ($870))|0;
         $872 = $8;
         $873 = (($872) + ($871<<3)|0);
         $874 = +HEAPF64[$873>>3];
         $875 = $874 != 0.0;
         L232: do {
          if ($875) {
           $876 = $j;
           $877 = $k;
           $878 = $a_dim1;
           $879 = Math_imul($877, $878)|0;
           $880 = (($876) + ($879))|0;
           $881 = $8;
           $882 = (($881) + ($880<<3)|0);
           $883 = +HEAPF64[$882>>3];
           $temp = $883;
           $884 = $5;
           $885 = HEAP32[$884>>2]|0;
           $i__3 = $885;
           $i__ = 1;
           while(1) {
            $886 = $i__;
            $887 = $i__3;
            $888 = ($886|0)<=($887|0);
            if (!($888)) {
             break L232;
            }
            $889 = $temp;
            $890 = $i__;
            $891 = $k;
            $892 = $b_dim1;
            $893 = Math_imul($891, $892)|0;
            $894 = (($890) + ($893))|0;
            $895 = $10;
            $896 = (($895) + ($894<<3)|0);
            $897 = +HEAPF64[$896>>3];
            $898 = $889 * $897;
            $899 = $i__;
            $900 = $j;
            $901 = $b_dim1;
            $902 = Math_imul($900, $901)|0;
            $903 = (($899) + ($902))|0;
            $904 = $10;
            $905 = (($904) + ($903<<3)|0);
            $906 = +HEAPF64[$905>>3];
            $907 = $906 - $898;
            HEAPF64[$905>>3] = $907;
            $908 = $i__;
            $909 = (($908) + 1)|0;
            $i__ = $909;
           }
          }
         } while(0);
         $910 = $j;
         $911 = (($910) + 1)|0;
         $j = $911;
        }
        $912 = $7;
        $913 = +HEAPF64[$912>>3];
        $914 = $913 != 1.0;
        L239: do {
         if ($914) {
          $915 = $5;
          $916 = HEAP32[$915>>2]|0;
          $i__2 = $916;
          $i__ = 1;
          while(1) {
           $917 = $i__;
           $918 = $i__2;
           $919 = ($917|0)<=($918|0);
           if (!($919)) {
            break L239;
           }
           $920 = $7;
           $921 = +HEAPF64[$920>>3];
           $922 = $i__;
           $923 = $k;
           $924 = $b_dim1;
           $925 = Math_imul($923, $924)|0;
           $926 = (($922) + ($925))|0;
           $927 = $10;
           $928 = (($927) + ($926<<3)|0);
           $929 = +HEAPF64[$928>>3];
           $930 = $921 * $929;
           $931 = $i__;
           $932 = $k;
           $933 = $b_dim1;
           $934 = Math_imul($932, $933)|0;
           $935 = (($931) + ($934))|0;
           $936 = $10;
           $937 = (($936) + ($935<<3)|0);
           HEAPF64[$937>>3] = $930;
           $938 = $i__;
           $939 = (($938) + 1)|0;
           $i__ = $939;
          }
         }
        } while(0);
        $940 = $k;
        $941 = (($940) + 1)|0;
        $k = $941;
       }
      }
     }
    }
   } while(0);
   $0 = 0;
   $942 = $0;
   STACKTOP = sp;return ($942|0);
  }
 }
 $0 = 0;
 $942 = $0;
 STACKTOP = sp;return ($942|0);
}
function _xerbla_($srname,$info) {
 $srname = $srname|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = $srname;
 $1 = $info;
 $2 = $0;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $2;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $4;
 (_printf(20477,$vararg_buffer)|0);
 STACKTOP = sp;return 0;
}
function _timer_($ttime) {
 $ttime = $ttime|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $0 = $ttime;
 $1 = $0; //@line 34 "timer.c"
 HEAPF64[$1>>3] = 0.0; //@line 34 "timer.c"
 STACKTOP = sp;return 0; //@line 35 "timer.c"
}
function _dpofa_($a,$lda,$n,$info) {
 $a = $a|0;
 $lda = $lda|0;
 $n = $n|0;
 $info = $info|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $a_dim1 = 0, $a_offset = 0, $i__1 = 0, $i__2 = 0, $i__3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__3 = sp;
 $0 = $a;
 $1 = $lda;
 $2 = $n;
 $3 = $info;
 $4 = $1; //@line 81 "linpack.c"
 $5 = HEAP32[$4>>2]|0; //@line 81 "linpack.c"
 $a_dim1 = $5; //@line 81 "linpack.c"
 $6 = $a_dim1; //@line 82 "linpack.c"
 $7 = (1 + ($6))|0; //@line 82 "linpack.c"
 $a_offset = $7; //@line 82 "linpack.c"
 $8 = $a_offset; //@line 83 "linpack.c"
 $9 = $0; //@line 83 "linpack.c"
 $10 = (0 - ($8))|0; //@line 83 "linpack.c"
 $11 = (($9) + ($10<<3)|0); //@line 83 "linpack.c"
 $0 = $11; //@line 83 "linpack.c"
 $12 = $2; //@line 86 "linpack.c"
 $13 = HEAP32[$12>>2]|0; //@line 86 "linpack.c"
 $i__1 = $13; //@line 86 "linpack.c"
 HEAP32[12700>>2] = 1; //@line 87 "linpack.c"
 while(1) {
  $14 = HEAP32[12700>>2]|0; //@line 87 "linpack.c"
  $15 = $i__1; //@line 87 "linpack.c"
  $16 = ($14|0)<=($15|0); //@line 87 "linpack.c"
  if (!($16)) {
   break;
  }
  $17 = HEAP32[12700>>2]|0; //@line 88 "linpack.c"
  $18 = $3; //@line 88 "linpack.c"
  HEAP32[$18>>2] = $17; //@line 88 "linpack.c"
  HEAPF64[904>>3] = 0.0; //@line 89 "linpack.c"
  $19 = HEAP32[12700>>2]|0; //@line 90 "linpack.c"
  $20 = (($19) - 1)|0; //@line 90 "linpack.c"
  HEAP32[12704>>2] = $20; //@line 90 "linpack.c"
  $21 = HEAP32[12704>>2]|0; //@line 91 "linpack.c"
  $22 = ($21|0)<(1); //@line 91 "linpack.c"
  L4: do {
   if (!($22)) {
    $23 = HEAP32[12704>>2]|0; //@line 94 "linpack.c"
    $i__2 = $23; //@line 94 "linpack.c"
    HEAP32[12708>>2] = 1; //@line 95 "linpack.c"
    while(1) {
     $24 = HEAP32[12708>>2]|0; //@line 95 "linpack.c"
     $25 = $i__2; //@line 95 "linpack.c"
     $26 = ($24|0)<=($25|0); //@line 95 "linpack.c"
     if (!($26)) {
      break L4;
     }
     $27 = HEAP32[12708>>2]|0; //@line 96 "linpack.c"
     $28 = (($27) - 1)|0; //@line 96 "linpack.c"
     HEAP32[$i__3>>2] = $28; //@line 96 "linpack.c"
     $29 = HEAP32[12708>>2]|0; //@line 97 "linpack.c"
     $30 = HEAP32[12700>>2]|0; //@line 97 "linpack.c"
     $31 = $a_dim1; //@line 97 "linpack.c"
     $32 = Math_imul($30, $31)|0; //@line 97 "linpack.c"
     $33 = (($29) + ($32))|0; //@line 97 "linpack.c"
     $34 = $0; //@line 97 "linpack.c"
     $35 = (($34) + ($33<<3)|0); //@line 97 "linpack.c"
     $36 = +HEAPF64[$35>>3]; //@line 97 "linpack.c"
     $37 = HEAP32[12708>>2]|0; //@line 97 "linpack.c"
     $38 = $a_dim1; //@line 97 "linpack.c"
     $39 = Math_imul($37, $38)|0; //@line 97 "linpack.c"
     $40 = (($39) + 1)|0; //@line 97 "linpack.c"
     $41 = $0; //@line 97 "linpack.c"
     $42 = (($41) + ($40<<3)|0); //@line 97 "linpack.c"
     $43 = HEAP32[12700>>2]|0; //@line 98 "linpack.c"
     $44 = $a_dim1; //@line 98 "linpack.c"
     $45 = Math_imul($43, $44)|0; //@line 98 "linpack.c"
     $46 = (($45) + 1)|0; //@line 98 "linpack.c"
     $47 = $0; //@line 98 "linpack.c"
     $48 = (($47) + ($46<<3)|0); //@line 98 "linpack.c"
     $49 = (+_ddot_($i__3,$42,12712,$48,12712)); //@line 97 "linpack.c"
     $50 = $36 - $49; //@line 97 "linpack.c"
     HEAPF64[912>>3] = $50; //@line 97 "linpack.c"
     $51 = HEAP32[12708>>2]|0; //@line 99 "linpack.c"
     $52 = HEAP32[12708>>2]|0; //@line 99 "linpack.c"
     $53 = $a_dim1; //@line 99 "linpack.c"
     $54 = Math_imul($52, $53)|0; //@line 99 "linpack.c"
     $55 = (($51) + ($54))|0; //@line 99 "linpack.c"
     $56 = $0; //@line 99 "linpack.c"
     $57 = (($56) + ($55<<3)|0); //@line 99 "linpack.c"
     $58 = +HEAPF64[$57>>3]; //@line 99 "linpack.c"
     $59 = +HEAPF64[912>>3]; //@line 99 "linpack.c"
     $60 = $59 / $58; //@line 99 "linpack.c"
     HEAPF64[912>>3] = $60; //@line 99 "linpack.c"
     $61 = +HEAPF64[912>>3]; //@line 100 "linpack.c"
     $62 = HEAP32[12708>>2]|0; //@line 100 "linpack.c"
     $63 = HEAP32[12700>>2]|0; //@line 100 "linpack.c"
     $64 = $a_dim1; //@line 100 "linpack.c"
     $65 = Math_imul($63, $64)|0; //@line 100 "linpack.c"
     $66 = (($62) + ($65))|0; //@line 100 "linpack.c"
     $67 = $0; //@line 100 "linpack.c"
     $68 = (($67) + ($66<<3)|0); //@line 100 "linpack.c"
     HEAPF64[$68>>3] = $61; //@line 100 "linpack.c"
     $69 = +HEAPF64[912>>3]; //@line 101 "linpack.c"
     $70 = +HEAPF64[912>>3]; //@line 101 "linpack.c"
     $71 = $69 * $70; //@line 101 "linpack.c"
     $72 = +HEAPF64[904>>3]; //@line 101 "linpack.c"
     $73 = $72 + $71; //@line 101 "linpack.c"
     HEAPF64[904>>3] = $73; //@line 101 "linpack.c"
     $74 = HEAP32[12708>>2]|0; //@line 95 "linpack.c"
     $75 = (($74) + 1)|0; //@line 95 "linpack.c"
     HEAP32[12708>>2] = $75; //@line 95 "linpack.c"
    }
   }
  } while(0);
  $76 = HEAP32[12700>>2]|0; //@line 105 "linpack.c"
  $77 = HEAP32[12700>>2]|0; //@line 105 "linpack.c"
  $78 = $a_dim1; //@line 105 "linpack.c"
  $79 = Math_imul($77, $78)|0; //@line 105 "linpack.c"
  $80 = (($76) + ($79))|0; //@line 105 "linpack.c"
  $81 = $0; //@line 105 "linpack.c"
  $82 = (($81) + ($80<<3)|0); //@line 105 "linpack.c"
  $83 = +HEAPF64[$82>>3]; //@line 105 "linpack.c"
  $84 = +HEAPF64[904>>3]; //@line 105 "linpack.c"
  $85 = $83 - $84; //@line 105 "linpack.c"
  HEAPF64[904>>3] = $85; //@line 105 "linpack.c"
  $86 = +HEAPF64[904>>3]; //@line 107 "linpack.c"
  $87 = $86 <= 0.0; //@line 107 "linpack.c"
  if ($87) {
   label = 10;
   break;
  }
  $88 = +HEAPF64[904>>3]; //@line 110 "linpack.c"
  $89 = (+Math_sqrt((+$88))); //@line 110 "linpack.c"
  $90 = HEAP32[12700>>2]|0; //@line 110 "linpack.c"
  $91 = HEAP32[12700>>2]|0; //@line 110 "linpack.c"
  $92 = $a_dim1; //@line 110 "linpack.c"
  $93 = Math_imul($91, $92)|0; //@line 110 "linpack.c"
  $94 = (($90) + ($93))|0; //@line 110 "linpack.c"
  $95 = $0; //@line 110 "linpack.c"
  $96 = (($95) + ($94<<3)|0); //@line 110 "linpack.c"
  HEAPF64[$96>>3] = $89; //@line 110 "linpack.c"
  $97 = HEAP32[12700>>2]|0; //@line 87 "linpack.c"
  $98 = (($97) + 1)|0; //@line 87 "linpack.c"
  HEAP32[12700>>2] = $98; //@line 87 "linpack.c"
 }
 if ((label|0) == 10) {
  STACKTOP = sp;return 0; //@line 115 "linpack.c"
 }
 $99 = $3; //@line 113 "linpack.c"
 HEAP32[$99>>2] = 0; //@line 113 "linpack.c"
 STACKTOP = sp;return 0; //@line 115 "linpack.c"
}
function _dtrsl_($t,$ldt,$n,$b,$job,$info) {
 $t = $t|0;
 $ldt = $ldt|0;
 $n = $n|0;
 $b = $b|0;
 $job = $job|0;
 $info = $info|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0.0, $137 = 0.0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0.0, $155 = 0, $156 = 0, $157 = 0, $158 = 0.0, $159 = 0.0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0.0, $177 = 0.0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0.0, $206 = 0, $207 = 0, $208 = 0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0.0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0.0;
 var $223 = 0.0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0.0, $231 = 0, $232 = 0, $233 = 0.0, $234 = 0.0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0.0, $254 = 0, $255 = 0, $256 = 0, $257 = 0.0, $258 = 0.0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0.0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0.0, $271 = 0.0, $272 = 0, $273 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0.0, $i__1 = 0, $i__2 = 0, $t_dim1 = 0, $t_offset = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $i__2 = sp;
 $0 = $t;
 $1 = $ldt;
 $2 = $n;
 $3 = $b;
 $4 = $job;
 $5 = $info;
 $6 = $1; //@line 196 "linpack.c"
 $7 = HEAP32[$6>>2]|0; //@line 196 "linpack.c"
 $t_dim1 = $7; //@line 196 "linpack.c"
 $8 = $t_dim1; //@line 197 "linpack.c"
 $9 = (1 + ($8))|0; //@line 197 "linpack.c"
 $t_offset = $9; //@line 197 "linpack.c"
 $10 = $t_offset; //@line 198 "linpack.c"
 $11 = $0; //@line 198 "linpack.c"
 $12 = (0 - ($10))|0; //@line 198 "linpack.c"
 $13 = (($11) + ($12<<3)|0); //@line 198 "linpack.c"
 $0 = $13; //@line 198 "linpack.c"
 $14 = $3; //@line 199 "linpack.c"
 $15 = ((($14)) + -8|0); //@line 199 "linpack.c"
 $3 = $15; //@line 199 "linpack.c"
 $16 = $2; //@line 202 "linpack.c"
 $17 = HEAP32[$16>>2]|0; //@line 202 "linpack.c"
 $i__1 = $17; //@line 202 "linpack.c"
 $18 = $5; //@line 203 "linpack.c"
 HEAP32[$18>>2] = 1; //@line 203 "linpack.c"
 while(1) {
  $19 = $5; //@line 203 "linpack.c"
  $20 = HEAP32[$19>>2]|0; //@line 203 "linpack.c"
  $21 = $i__1; //@line 203 "linpack.c"
  $22 = ($20|0)<=($21|0); //@line 203 "linpack.c"
  $23 = $5; //@line 205 "linpack.c"
  if (!($22)) {
   break;
  }
  $24 = HEAP32[$23>>2]|0; //@line 205 "linpack.c"
  $25 = $5; //@line 205 "linpack.c"
  $26 = HEAP32[$25>>2]|0; //@line 205 "linpack.c"
  $27 = $t_dim1; //@line 205 "linpack.c"
  $28 = Math_imul($26, $27)|0; //@line 205 "linpack.c"
  $29 = (($24) + ($28))|0; //@line 205 "linpack.c"
  $30 = $0; //@line 205 "linpack.c"
  $31 = (($30) + ($29<<3)|0); //@line 205 "linpack.c"
  $32 = +HEAPF64[$31>>3]; //@line 205 "linpack.c"
  $33 = $32 == 0.0; //@line 205 "linpack.c"
  if ($33) {
   label = 24;
   break;
  }
  $34 = $5; //@line 203 "linpack.c"
  $35 = HEAP32[$34>>2]|0; //@line 203 "linpack.c"
  $36 = (($35) + 1)|0; //@line 203 "linpack.c"
  HEAP32[$34>>2] = $36; //@line 203 "linpack.c"
 }
 if ((label|0) == 24) {
  STACKTOP = sp;return 0; //@line 299 "linpack.c"
 }
 HEAP32[$23>>2] = 0; //@line 210 "linpack.c"
 HEAP32[12716>>2] = 1; //@line 214 "linpack.c"
 $37 = $4; //@line 215 "linpack.c"
 $38 = HEAP32[$37>>2]|0; //@line 215 "linpack.c"
 $39 = (($38|0) % 10)&-1; //@line 215 "linpack.c"
 $40 = ($39|0)!=(0); //@line 215 "linpack.c"
 $$ = $40 ? 2 : 1; //@line 215 "linpack.c"
 HEAP32[12716>>2] = $$; //@line 216 "linpack.c"
 $41 = $4; //@line 218 "linpack.c"
 $42 = HEAP32[$41>>2]|0; //@line 218 "linpack.c"
 $43 = (($42|0) % 100)&-1; //@line 218 "linpack.c"
 $44 = (($43|0) / 10)&-1; //@line 218 "linpack.c"
 $45 = ($44|0)!=(0); //@line 218 "linpack.c"
 if ($45) {
  $46 = HEAP32[12716>>2]|0; //@line 219 "linpack.c"
  $47 = (($46) + 2)|0; //@line 219 "linpack.c"
  HEAP32[12716>>2] = $47; //@line 219 "linpack.c"
 }
 $48 = HEAP32[12716>>2]|0; //@line 221 "linpack.c"
 switch ($48|0) {
 case 4:  {
  $226 = $t_dim1; //@line 285 "linpack.c"
  $227 = (($226) + 1)|0; //@line 285 "linpack.c"
  $228 = $0; //@line 285 "linpack.c"
  $229 = (($228) + ($227<<3)|0); //@line 285 "linpack.c"
  $230 = +HEAPF64[$229>>3]; //@line 285 "linpack.c"
  $231 = $3; //@line 285 "linpack.c"
  $232 = ((($231)) + 8|0); //@line 285 "linpack.c"
  $233 = +HEAPF64[$232>>3]; //@line 285 "linpack.c"
  $234 = $233 / $230; //@line 285 "linpack.c"
  HEAPF64[$232>>3] = $234; //@line 285 "linpack.c"
  $235 = $2; //@line 286 "linpack.c"
  $236 = HEAP32[$235>>2]|0; //@line 286 "linpack.c"
  $237 = ($236|0)<(2); //@line 286 "linpack.c"
  if ($237) {
   STACKTOP = sp;return 0; //@line 299 "linpack.c"
  }
  $238 = $2; //@line 289 "linpack.c"
  $239 = HEAP32[$238>>2]|0; //@line 289 "linpack.c"
  $i__1 = $239; //@line 289 "linpack.c"
  HEAP32[12720>>2] = 2; //@line 290 "linpack.c"
  while(1) {
   $240 = HEAP32[12720>>2]|0; //@line 290 "linpack.c"
   $241 = $i__1; //@line 290 "linpack.c"
   $242 = ($240|0)<=($241|0); //@line 290 "linpack.c"
   if (!($242)) {
    break;
   }
   $243 = HEAP32[12720>>2]|0; //@line 291 "linpack.c"
   $244 = (($243) - 1)|0; //@line 291 "linpack.c"
   HEAP32[$i__2>>2] = $244; //@line 291 "linpack.c"
   $245 = HEAP32[12720>>2]|0; //@line 292 "linpack.c"
   $246 = $t_dim1; //@line 292 "linpack.c"
   $247 = Math_imul($245, $246)|0; //@line 292 "linpack.c"
   $248 = (($247) + 1)|0; //@line 292 "linpack.c"
   $249 = $0; //@line 292 "linpack.c"
   $250 = (($249) + ($248<<3)|0); //@line 292 "linpack.c"
   $251 = $3; //@line 292 "linpack.c"
   $252 = ((($251)) + 8|0); //@line 292 "linpack.c"
   $253 = (+_ddot_($i__2,$250,12712,$252,12712)); //@line 292 "linpack.c"
   $254 = HEAP32[12720>>2]|0; //@line 292 "linpack.c"
   $255 = $3; //@line 292 "linpack.c"
   $256 = (($255) + ($254<<3)|0); //@line 292 "linpack.c"
   $257 = +HEAPF64[$256>>3]; //@line 292 "linpack.c"
   $258 = $257 - $253; //@line 292 "linpack.c"
   HEAPF64[$256>>3] = $258; //@line 292 "linpack.c"
   $259 = HEAP32[12720>>2]|0; //@line 293 "linpack.c"
   $260 = HEAP32[12720>>2]|0; //@line 293 "linpack.c"
   $261 = $t_dim1; //@line 293 "linpack.c"
   $262 = Math_imul($260, $261)|0; //@line 293 "linpack.c"
   $263 = (($259) + ($262))|0; //@line 293 "linpack.c"
   $264 = $0; //@line 293 "linpack.c"
   $265 = (($264) + ($263<<3)|0); //@line 293 "linpack.c"
   $266 = +HEAPF64[$265>>3]; //@line 293 "linpack.c"
   $267 = HEAP32[12720>>2]|0; //@line 293 "linpack.c"
   $268 = $3; //@line 293 "linpack.c"
   $269 = (($268) + ($267<<3)|0); //@line 293 "linpack.c"
   $270 = +HEAPF64[$269>>3]; //@line 293 "linpack.c"
   $271 = $270 / $266; //@line 293 "linpack.c"
   HEAPF64[$269>>3] = $271; //@line 293 "linpack.c"
   $272 = HEAP32[12720>>2]|0; //@line 290 "linpack.c"
   $273 = (($272) + 1)|0; //@line 290 "linpack.c"
   HEAP32[12720>>2] = $273; //@line 290 "linpack.c"
  }
  STACKTOP = sp;return 0; //@line 299 "linpack.c"
  break;
 }
 case 2:  {
  $103 = $2; //@line 249 "linpack.c"
  $104 = HEAP32[$103>>2]|0; //@line 249 "linpack.c"
  $105 = $2; //@line 249 "linpack.c"
  $106 = HEAP32[$105>>2]|0; //@line 249 "linpack.c"
  $107 = $t_dim1; //@line 249 "linpack.c"
  $108 = Math_imul($106, $107)|0; //@line 249 "linpack.c"
  $109 = (($104) + ($108))|0; //@line 249 "linpack.c"
  $110 = $0; //@line 249 "linpack.c"
  $111 = (($110) + ($109<<3)|0); //@line 249 "linpack.c"
  $112 = +HEAPF64[$111>>3]; //@line 249 "linpack.c"
  $113 = $2; //@line 249 "linpack.c"
  $114 = HEAP32[$113>>2]|0; //@line 249 "linpack.c"
  $115 = $3; //@line 249 "linpack.c"
  $116 = (($115) + ($114<<3)|0); //@line 249 "linpack.c"
  $117 = +HEAPF64[$116>>3]; //@line 249 "linpack.c"
  $118 = $117 / $112; //@line 249 "linpack.c"
  HEAPF64[$116>>3] = $118; //@line 249 "linpack.c"
  $119 = $2; //@line 250 "linpack.c"
  $120 = HEAP32[$119>>2]|0; //@line 250 "linpack.c"
  $121 = ($120|0)<(2); //@line 250 "linpack.c"
  if ($121) {
   STACKTOP = sp;return 0; //@line 299 "linpack.c"
  }
  $122 = $2; //@line 253 "linpack.c"
  $123 = HEAP32[$122>>2]|0; //@line 253 "linpack.c"
  $i__1 = $123; //@line 253 "linpack.c"
  HEAP32[12724>>2] = 2; //@line 254 "linpack.c"
  while(1) {
   $124 = HEAP32[12724>>2]|0; //@line 254 "linpack.c"
   $125 = $i__1; //@line 254 "linpack.c"
   $126 = ($124|0)<=($125|0); //@line 254 "linpack.c"
   if (!($126)) {
    break;
   }
   $127 = $2; //@line 255 "linpack.c"
   $128 = HEAP32[$127>>2]|0; //@line 255 "linpack.c"
   $129 = HEAP32[12724>>2]|0; //@line 255 "linpack.c"
   $130 = (($128) - ($129))|0; //@line 255 "linpack.c"
   $131 = (($130) + 1)|0; //@line 255 "linpack.c"
   HEAP32[12720>>2] = $131; //@line 255 "linpack.c"
   $132 = HEAP32[12720>>2]|0; //@line 256 "linpack.c"
   $133 = (($132) + 1)|0; //@line 256 "linpack.c"
   $134 = $3; //@line 256 "linpack.c"
   $135 = (($134) + ($133<<3)|0); //@line 256 "linpack.c"
   $136 = +HEAPF64[$135>>3]; //@line 256 "linpack.c"
   $137 = -$136; //@line 256 "linpack.c"
   HEAPF64[920>>3] = $137; //@line 256 "linpack.c"
   $138 = HEAP32[12720>>2]|0; //@line 257 "linpack.c"
   $139 = (($138) + 1)|0; //@line 257 "linpack.c"
   $140 = $t_dim1; //@line 257 "linpack.c"
   $141 = Math_imul($139, $140)|0; //@line 257 "linpack.c"
   $142 = (($141) + 1)|0; //@line 257 "linpack.c"
   $143 = $0; //@line 257 "linpack.c"
   $144 = (($143) + ($142<<3)|0); //@line 257 "linpack.c"
   $145 = $3; //@line 257 "linpack.c"
   $146 = ((($145)) + 8|0); //@line 257 "linpack.c"
   (_daxpy_(12720,920,$144,12712,$146,12712)|0); //@line 257 "linpack.c"
   $147 = HEAP32[12720>>2]|0; //@line 258 "linpack.c"
   $148 = HEAP32[12720>>2]|0; //@line 258 "linpack.c"
   $149 = $t_dim1; //@line 258 "linpack.c"
   $150 = Math_imul($148, $149)|0; //@line 258 "linpack.c"
   $151 = (($147) + ($150))|0; //@line 258 "linpack.c"
   $152 = $0; //@line 258 "linpack.c"
   $153 = (($152) + ($151<<3)|0); //@line 258 "linpack.c"
   $154 = +HEAPF64[$153>>3]; //@line 258 "linpack.c"
   $155 = HEAP32[12720>>2]|0; //@line 258 "linpack.c"
   $156 = $3; //@line 258 "linpack.c"
   $157 = (($156) + ($155<<3)|0); //@line 258 "linpack.c"
   $158 = +HEAPF64[$157>>3]; //@line 258 "linpack.c"
   $159 = $158 / $154; //@line 258 "linpack.c"
   HEAPF64[$157>>3] = $159; //@line 258 "linpack.c"
   $160 = HEAP32[12724>>2]|0; //@line 254 "linpack.c"
   $161 = (($160) + 1)|0; //@line 254 "linpack.c"
   HEAP32[12724>>2] = $161; //@line 254 "linpack.c"
  }
  STACKTOP = sp;return 0; //@line 299 "linpack.c"
  break;
 }
 case 3:  {
  $162 = $2; //@line 267 "linpack.c"
  $163 = HEAP32[$162>>2]|0; //@line 267 "linpack.c"
  $164 = $2; //@line 267 "linpack.c"
  $165 = HEAP32[$164>>2]|0; //@line 267 "linpack.c"
  $166 = $t_dim1; //@line 267 "linpack.c"
  $167 = Math_imul($165, $166)|0; //@line 267 "linpack.c"
  $168 = (($163) + ($167))|0; //@line 267 "linpack.c"
  $169 = $0; //@line 267 "linpack.c"
  $170 = (($169) + ($168<<3)|0); //@line 267 "linpack.c"
  $171 = +HEAPF64[$170>>3]; //@line 267 "linpack.c"
  $172 = $2; //@line 267 "linpack.c"
  $173 = HEAP32[$172>>2]|0; //@line 267 "linpack.c"
  $174 = $3; //@line 267 "linpack.c"
  $175 = (($174) + ($173<<3)|0); //@line 267 "linpack.c"
  $176 = +HEAPF64[$175>>3]; //@line 267 "linpack.c"
  $177 = $176 / $171; //@line 267 "linpack.c"
  HEAPF64[$175>>3] = $177; //@line 267 "linpack.c"
  $178 = $2; //@line 268 "linpack.c"
  $179 = HEAP32[$178>>2]|0; //@line 268 "linpack.c"
  $180 = ($179|0)<(2); //@line 268 "linpack.c"
  if ($180) {
   STACKTOP = sp;return 0; //@line 299 "linpack.c"
  }
  $181 = $2; //@line 271 "linpack.c"
  $182 = HEAP32[$181>>2]|0; //@line 271 "linpack.c"
  $i__1 = $182; //@line 271 "linpack.c"
  HEAP32[12724>>2] = 2; //@line 272 "linpack.c"
  while(1) {
   $183 = HEAP32[12724>>2]|0; //@line 272 "linpack.c"
   $184 = $i__1; //@line 272 "linpack.c"
   $185 = ($183|0)<=($184|0); //@line 272 "linpack.c"
   if (!($185)) {
    break;
   }
   $186 = $2; //@line 273 "linpack.c"
   $187 = HEAP32[$186>>2]|0; //@line 273 "linpack.c"
   $188 = HEAP32[12724>>2]|0; //@line 273 "linpack.c"
   $189 = (($187) - ($188))|0; //@line 273 "linpack.c"
   $190 = (($189) + 1)|0; //@line 273 "linpack.c"
   HEAP32[12720>>2] = $190; //@line 273 "linpack.c"
   $191 = HEAP32[12724>>2]|0; //@line 274 "linpack.c"
   $192 = (($191) - 1)|0; //@line 274 "linpack.c"
   HEAP32[$i__2>>2] = $192; //@line 274 "linpack.c"
   $193 = HEAP32[12720>>2]|0; //@line 275 "linpack.c"
   $194 = (($193) + 1)|0; //@line 275 "linpack.c"
   $195 = HEAP32[12720>>2]|0; //@line 275 "linpack.c"
   $196 = $t_dim1; //@line 275 "linpack.c"
   $197 = Math_imul($195, $196)|0; //@line 275 "linpack.c"
   $198 = (($194) + ($197))|0; //@line 275 "linpack.c"
   $199 = $0; //@line 275 "linpack.c"
   $200 = (($199) + ($198<<3)|0); //@line 275 "linpack.c"
   $201 = HEAP32[12720>>2]|0; //@line 275 "linpack.c"
   $202 = (($201) + 1)|0; //@line 275 "linpack.c"
   $203 = $3; //@line 275 "linpack.c"
   $204 = (($203) + ($202<<3)|0); //@line 275 "linpack.c"
   $205 = (+_ddot_($i__2,$200,12712,$204,12712)); //@line 275 "linpack.c"
   $206 = HEAP32[12720>>2]|0; //@line 275 "linpack.c"
   $207 = $3; //@line 275 "linpack.c"
   $208 = (($207) + ($206<<3)|0); //@line 275 "linpack.c"
   $209 = +HEAPF64[$208>>3]; //@line 275 "linpack.c"
   $210 = $209 - $205; //@line 275 "linpack.c"
   HEAPF64[$208>>3] = $210; //@line 275 "linpack.c"
   $211 = HEAP32[12720>>2]|0; //@line 276 "linpack.c"
   $212 = HEAP32[12720>>2]|0; //@line 276 "linpack.c"
   $213 = $t_dim1; //@line 276 "linpack.c"
   $214 = Math_imul($212, $213)|0; //@line 276 "linpack.c"
   $215 = (($211) + ($214))|0; //@line 276 "linpack.c"
   $216 = $0; //@line 276 "linpack.c"
   $217 = (($216) + ($215<<3)|0); //@line 276 "linpack.c"
   $218 = +HEAPF64[$217>>3]; //@line 276 "linpack.c"
   $219 = HEAP32[12720>>2]|0; //@line 276 "linpack.c"
   $220 = $3; //@line 276 "linpack.c"
   $221 = (($220) + ($219<<3)|0); //@line 276 "linpack.c"
   $222 = +HEAPF64[$221>>3]; //@line 276 "linpack.c"
   $223 = $222 / $218; //@line 276 "linpack.c"
   HEAPF64[$221>>3] = $223; //@line 276 "linpack.c"
   $224 = HEAP32[12724>>2]|0; //@line 272 "linpack.c"
   $225 = (($224) + 1)|0; //@line 272 "linpack.c"
   HEAP32[12724>>2] = $225; //@line 272 "linpack.c"
  }
  STACKTOP = sp;return 0; //@line 299 "linpack.c"
  break;
 }
 default: {
  $49 = $t_dim1; //@line 231 "linpack.c"
  $50 = (($49) + 1)|0; //@line 231 "linpack.c"
  $51 = $0; //@line 231 "linpack.c"
  $52 = (($51) + ($50<<3)|0); //@line 231 "linpack.c"
  $53 = +HEAPF64[$52>>3]; //@line 231 "linpack.c"
  $54 = $3; //@line 231 "linpack.c"
  $55 = ((($54)) + 8|0); //@line 231 "linpack.c"
  $56 = +HEAPF64[$55>>3]; //@line 231 "linpack.c"
  $57 = $56 / $53; //@line 231 "linpack.c"
  HEAPF64[$55>>3] = $57; //@line 231 "linpack.c"
  $58 = $2; //@line 232 "linpack.c"
  $59 = HEAP32[$58>>2]|0; //@line 232 "linpack.c"
  $60 = ($59|0)<(2); //@line 232 "linpack.c"
  if ($60) {
   STACKTOP = sp;return 0; //@line 299 "linpack.c"
  }
  $61 = $2; //@line 235 "linpack.c"
  $62 = HEAP32[$61>>2]|0; //@line 235 "linpack.c"
  $i__1 = $62; //@line 235 "linpack.c"
  HEAP32[12720>>2] = 2; //@line 236 "linpack.c"
  while(1) {
   $63 = HEAP32[12720>>2]|0; //@line 236 "linpack.c"
   $64 = $i__1; //@line 236 "linpack.c"
   $65 = ($63|0)<=($64|0); //@line 236 "linpack.c"
   if (!($65)) {
    break;
   }
   $66 = HEAP32[12720>>2]|0; //@line 237 "linpack.c"
   $67 = (($66) - 1)|0; //@line 237 "linpack.c"
   $68 = $3; //@line 237 "linpack.c"
   $69 = (($68) + ($67<<3)|0); //@line 237 "linpack.c"
   $70 = +HEAPF64[$69>>3]; //@line 237 "linpack.c"
   $71 = -$70; //@line 237 "linpack.c"
   HEAPF64[920>>3] = $71; //@line 237 "linpack.c"
   $72 = $2; //@line 238 "linpack.c"
   $73 = HEAP32[$72>>2]|0; //@line 238 "linpack.c"
   $74 = HEAP32[12720>>2]|0; //@line 238 "linpack.c"
   $75 = (($73) - ($74))|0; //@line 238 "linpack.c"
   $76 = (($75) + 1)|0; //@line 238 "linpack.c"
   HEAP32[$i__2>>2] = $76; //@line 238 "linpack.c"
   $77 = HEAP32[12720>>2]|0; //@line 239 "linpack.c"
   $78 = HEAP32[12720>>2]|0; //@line 239 "linpack.c"
   $79 = (($78) - 1)|0; //@line 239 "linpack.c"
   $80 = $t_dim1; //@line 239 "linpack.c"
   $81 = Math_imul($79, $80)|0; //@line 239 "linpack.c"
   $82 = (($77) + ($81))|0; //@line 239 "linpack.c"
   $83 = $0; //@line 239 "linpack.c"
   $84 = (($83) + ($82<<3)|0); //@line 239 "linpack.c"
   $85 = HEAP32[12720>>2]|0; //@line 239 "linpack.c"
   $86 = $3; //@line 239 "linpack.c"
   $87 = (($86) + ($85<<3)|0); //@line 239 "linpack.c"
   (_daxpy_($i__2,920,$84,12712,$87,12712)|0); //@line 239 "linpack.c"
   $88 = HEAP32[12720>>2]|0; //@line 240 "linpack.c"
   $89 = HEAP32[12720>>2]|0; //@line 240 "linpack.c"
   $90 = $t_dim1; //@line 240 "linpack.c"
   $91 = Math_imul($89, $90)|0; //@line 240 "linpack.c"
   $92 = (($88) + ($91))|0; //@line 240 "linpack.c"
   $93 = $0; //@line 240 "linpack.c"
   $94 = (($93) + ($92<<3)|0); //@line 240 "linpack.c"
   $95 = +HEAPF64[$94>>3]; //@line 240 "linpack.c"
   $96 = HEAP32[12720>>2]|0; //@line 240 "linpack.c"
   $97 = $3; //@line 240 "linpack.c"
   $98 = (($97) + ($96<<3)|0); //@line 240 "linpack.c"
   $99 = +HEAPF64[$98>>3]; //@line 240 "linpack.c"
   $100 = $99 / $95; //@line 240 "linpack.c"
   HEAPF64[$98>>3] = $100; //@line 240 "linpack.c"
   $101 = HEAP32[12720>>2]|0; //@line 236 "linpack.c"
   $102 = (($101) + 1)|0; //@line 236 "linpack.c"
   HEAP32[12720>>2] = $102; //@line 236 "linpack.c"
  }
  STACKTOP = sp;return 0; //@line 299 "linpack.c"
 }
 }
 return (0)|0;
}
function _unlink($path) {
 $path = $path|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $0 = (___syscall10(10,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function ___dup3($old,$new,$flags) {
 $old = $old|0;
 $new = $new|0;
 $flags = $flags|0;
 var $$0 = 0, $$lcssa = 0, $$lcssa12 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = ($old|0)==($new|0);
 L1: do {
  if ($0) {
   $1 = (___syscall_ret(-22)|0);
   $$0 = $1;
  } else {
   $2 = $flags & 524288;
   $3 = ($2|0)!=(0);
   L4: do {
    if ($3) {
     L5: while(1) {
      HEAP32[$vararg_buffer>>2] = $old;
      $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
      HEAP32[$vararg_ptr1>>2] = $new;
      $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
      HEAP32[$vararg_ptr2>>2] = $flags;
      $4 = (___syscall330(330,($vararg_buffer|0))|0);
      switch ($4|0) {
      case -38:  {
       break L4;
       break;
      }
      case -16:  {
       break;
      }
      default: {
       $$lcssa12 = $4;
       break L5;
      }
      }
     }
     $5 = (___syscall_ret($$lcssa12)|0);
     $$0 = $5;
     break L1;
    }
   } while(0);
   while(1) {
    HEAP32[$vararg_buffer3>>2] = $old;
    $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
    HEAP32[$vararg_ptr6>>2] = $new;
    $6 = (___syscall63(63,($vararg_buffer3|0))|0);
    $7 = ($6|0)==(-16);
    if (!($7)) {
     $$lcssa = $6;
     break;
    }
   }
   if ($3) {
    HEAP32[$vararg_buffer7>>2] = $new;
    $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
    HEAP32[$vararg_ptr10>>2] = 2;
    $vararg_ptr11 = ((($vararg_buffer7)) + 8|0);
    HEAP32[$vararg_ptr11>>2] = 1;
    (___syscall221(221,($vararg_buffer7|0))|0);
   }
   $8 = (___syscall_ret($$lcssa)|0);
   $$0 = $8;
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function _ftruncate($fd,$length) {
 $fd = $fd|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = ($length|0)<(0);
 $1 = $0 << 31 >> 31;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $length;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $1;
 $2 = (___syscall194(194,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _isatty($fd) {
 $fd = $fd|0;
 var $0 = 0, $1 = 0, $2 = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $t = sp;
 $0 = (_tcgetattr($fd,$t)|0);
 $1 = ($0|0)==(0);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _access($filename,$amode) {
 $filename = $filename|0;
 $amode = $amode|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $filename;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $amode;
 $0 = (___syscall33(33,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _printf($fmt,$varargs) {
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = HEAP32[12728>>2]|0;
 $1 = (_vfprintf($0,$fmt,$ap)|0);
 STACKTOP = sp;return ($1|0);
}
function _vsnprintf($s,$n,$fmt,$ap) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$$02 = 0, $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $f = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $b = sp + 112|0;
 $f = sp;
 dest=$f; src=12788; stop=dest+112|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $0 = (($n) + -1)|0;
 $1 = ($0>>>0)>(2147483646);
 if ($1) {
  $2 = ($n|0)==(0);
  if ($2) {
   $$01 = $b;$$02 = 1;
   label = 4;
  } else {
   $3 = (___errno_location()|0);
   HEAP32[$3>>2] = 75;
   $$0 = -1;
  }
 } else {
  $$01 = $s;$$02 = $n;
  label = 4;
 }
 if ((label|0) == 4) {
  $4 = $$01;
  $5 = (-2 - ($4))|0;
  $6 = ($$02>>>0)>($5>>>0);
  $$$02 = $6 ? $5 : $$02;
  $7 = ((($f)) + 48|0);
  HEAP32[$7>>2] = $$$02;
  $8 = ((($f)) + 20|0);
  HEAP32[$8>>2] = $$01;
  $9 = ((($f)) + 44|0);
  HEAP32[$9>>2] = $$01;
  $10 = (($$01) + ($$$02)|0);
  $11 = ((($f)) + 16|0);
  HEAP32[$11>>2] = $10;
  $12 = ((($f)) + 28|0);
  HEAP32[$12>>2] = $10;
  $13 = (_vfprintf($f,$fmt,$ap)|0);
  $14 = ($$$02|0)==(0);
  if ($14) {
   $$0 = $13;
  } else {
   $15 = HEAP32[$8>>2]|0;
   $16 = HEAP32[$11>>2]|0;
   $17 = ($15|0)==($16|0);
   $18 = $17 << 31 >> 31;
   $19 = (($15) + ($18)|0);
   HEAP8[$19>>0] = 0;
   $$0 = $13;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function _fflush($f) {
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$012 = 0, $$014 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $phitmp = 0, $r$0$lcssa = 0, $r$03 = 0, $r$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($f|0)==(0|0);
 do {
  if ($0) {
   $7 = HEAP32[12732>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $27 = 0;
   } else {
    $9 = HEAP32[12732>>2]|0;
    $10 = (_fflush($9)|0);
    $27 = $10;
   }
   ___lock(((12772)|0));
   $$012 = HEAP32[(12768)>>2]|0;
   $11 = ($$012|0)==(0|0);
   if ($11) {
    $r$0$lcssa = $27;
   } else {
    $$014 = $$012;$r$03 = $27;
    while(1) {
     $12 = ((($$014)) + 76|0);
     $13 = HEAP32[$12>>2]|0;
     $14 = ($13|0)>(-1);
     if ($14) {
      $15 = (___lockfile($$014)|0);
      $23 = $15;
     } else {
      $23 = 0;
     }
     $16 = ((($$014)) + 20|0);
     $17 = HEAP32[$16>>2]|0;
     $18 = ((($$014)) + 28|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = ($17>>>0)>($19>>>0);
     if ($20) {
      $21 = (___fflush_unlocked($$014)|0);
      $22 = $21 | $r$03;
      $r$1 = $22;
     } else {
      $r$1 = $r$03;
     }
     $24 = ($23|0)==(0);
     if (!($24)) {
      ___unlockfile($$014);
     }
     $25 = ((($$014)) + 56|0);
     $$01 = HEAP32[$25>>2]|0;
     $26 = ($$01|0)==(0|0);
     if ($26) {
      $r$0$lcssa = $r$1;
      break;
     } else {
      $$014 = $$01;$r$03 = $r$1;
     }
    }
   }
   ___unlock(((12772)|0));
   $$0 = $r$0$lcssa;
  } else {
   $1 = ((($f)) + 76|0);
   $2 = HEAP32[$1>>2]|0;
   $3 = ($2|0)>(-1);
   if (!($3)) {
    $4 = (___fflush_unlocked($f)|0);
    $$0 = $4;
    break;
   }
   $5 = (___lockfile($f)|0);
   $phitmp = ($5|0)==(0);
   $6 = (___fflush_unlocked($f)|0);
   if ($phitmp) {
    $$0 = $6;
   } else {
    ___unlockfile($f);
    $$0 = $6;
   }
  }
 } while(0);
 return ($$0|0);
}
function ___towrite($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = HEAP32[$f>>2]|0;
 $7 = $6 & 8;
 $8 = ($7|0)==(0);
 if ($8) {
  $10 = ((($f)) + 8|0);
  HEAP32[$10>>2] = 0;
  $11 = ((($f)) + 4|0);
  HEAP32[$11>>2] = 0;
  $12 = ((($f)) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($f)) + 28|0);
  HEAP32[$14>>2] = $13;
  $15 = ((($f)) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = $13;
  $17 = ((($f)) + 48|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + ($18)|0);
  $20 = ((($f)) + 16|0);
  HEAP32[$20>>2] = $19;
  $$0 = 0;
 } else {
  $9 = $6 | 32;
  HEAP32[$f>>2] = $9;
  $$0 = -1;
 }
 return ($$0|0);
}
function ___ftello_unlocked($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 40|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$f>>2]|0;
 $3 = $2 & 128;
 $4 = ($3|0)==(0);
 if ($4) {
  $10 = 1;
 } else {
  $5 = ((($f)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ((($f)) + 28|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6>>>0)>($8>>>0);
  $phitmp = $9 ? 2 : 1;
  $10 = $phitmp;
 }
 $11 = (FUNCTION_TABLE_iiii[$1 & 15]($f,0,$10)|0);
 $12 = ($11|0)<(0);
 if ($12) {
  $$0 = $11;
 } else {
  $13 = ((($f)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($f)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = $14;
  $18 = $16;
  $19 = ((($f)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ((($f)) + 28|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $20;
  $24 = $22;
  $25 = (($11) - ($17))|0;
  $26 = (($25) + ($18))|0;
  $27 = (($26) + ($23))|0;
  $28 = (($27) - ($24))|0;
  $$0 = $28;
 }
 return ($$0|0);
}
function ___ftello($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>(-1);
 if ($2) {
  $4 = (___lockfile($f)|0);
  $phitmp = ($4|0)==(0);
  $5 = (___ftello_unlocked($f)|0);
  if ($phitmp) {
   $6 = $5;
  } else {
   ___unlockfile($f);
   $6 = $5;
  }
 } else {
  $3 = (___ftello_unlocked($f)|0);
  $6 = $3;
 }
 return ($6|0);
}
function _ftell($f) {
 $f = $f|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___ftello($f)|0);
 return ($0|0);
}
function ___stdout_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tio = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $tio = sp + 12|0;
 $0 = ((($f)) + 36|0);
 HEAP32[$0>>2] = 5;
 $1 = HEAP32[$f>>2]|0;
 $2 = $1 & 64;
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = ((($f)) + 60|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$vararg_buffer>>2] = $5;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 21505;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $tio;
  $6 = (___syscall54(54,($vararg_buffer|0))|0);
  $7 = ($6|0)==(0);
  if (!($7)) {
   $8 = ((($f)) + 75|0);
   HEAP8[$8>>0] = -1;
  }
 }
 $9 = (___stdio_write($f,$buf,$len)|0);
 STACKTOP = sp;return ($9|0);
}
function ___fdopen($fd,$mode) {
 $fd = $fd|0;
 $mode = $mode|0;
 var $$0 = 0, $$pre = 0, $$pre1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $memchr = 0, $tio = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $tio = sp + 52|0;
 $0 = HEAP8[$mode>>0]|0;
 $1 = $0 << 24 >> 24;
 $memchr = (_memchr(20543,$1,4)|0);
 $2 = ($memchr|0)==(0|0);
 if ($2) {
  $3 = (___errno_location()|0);
  HEAP32[$3>>2] = 22;
  $$0 = 0;
 } else {
  $4 = (_malloc(1144)|0);
  $5 = ($4|0)==(0|0);
  if ($5) {
   $$0 = 0;
  } else {
   dest=$4; stop=dest+112|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $6 = (_strchr($mode,43)|0);
   $7 = ($6|0)==(0|0);
   if ($7) {
    $8 = ($0<<24>>24)==(114);
    $9 = $8 ? 8 : 4;
    HEAP32[$4>>2] = $9;
   }
   $10 = (_strchr($mode,101)|0);
   $11 = ($10|0)==(0|0);
   if ($11) {
    $12 = $0;
   } else {
    HEAP32[$vararg_buffer>>2] = $fd;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = 2;
    $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
    HEAP32[$vararg_ptr2>>2] = 1;
    (___syscall221(221,($vararg_buffer|0))|0);
    $$pre = HEAP8[$mode>>0]|0;
    $12 = $$pre;
   }
   $13 = ($12<<24>>24)==(97);
   if ($13) {
    HEAP32[$vararg_buffer3>>2] = $fd;
    $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
    HEAP32[$vararg_ptr6>>2] = 3;
    $14 = (___syscall221(221,($vararg_buffer3|0))|0);
    $15 = $14 & 1024;
    $16 = ($15|0)==(0);
    if ($16) {
     $17 = $14 | 1024;
     HEAP32[$vararg_buffer7>>2] = $fd;
     $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
     HEAP32[$vararg_ptr10>>2] = 4;
     $vararg_ptr11 = ((($vararg_buffer7)) + 8|0);
     HEAP32[$vararg_ptr11>>2] = $17;
     (___syscall221(221,($vararg_buffer7|0))|0);
    }
    $18 = HEAP32[$4>>2]|0;
    $19 = $18 | 128;
    HEAP32[$4>>2] = $19;
    $26 = $19;
   } else {
    $$pre1 = HEAP32[$4>>2]|0;
    $26 = $$pre1;
   }
   $20 = ((($4)) + 60|0);
   HEAP32[$20>>2] = $fd;
   $21 = ((($4)) + 120|0);
   $22 = ((($4)) + 44|0);
   HEAP32[$22>>2] = $21;
   $23 = ((($4)) + 48|0);
   HEAP32[$23>>2] = 1024;
   $24 = ((($4)) + 75|0);
   HEAP8[$24>>0] = -1;
   $25 = $26 & 8;
   $27 = ($25|0)==(0);
   if ($27) {
    HEAP32[$vararg_buffer12>>2] = $fd;
    $vararg_ptr15 = ((($vararg_buffer12)) + 4|0);
    HEAP32[$vararg_ptr15>>2] = 21505;
    $vararg_ptr16 = ((($vararg_buffer12)) + 8|0);
    HEAP32[$vararg_ptr16>>2] = $tio;
    $28 = (___syscall54(54,($vararg_buffer12|0))|0);
    $29 = ($28|0)==(0);
    if ($29) {
     HEAP8[$24>>0] = 10;
    }
   }
   $30 = ((($4)) + 32|0);
   HEAP32[$30>>2] = 6;
   $31 = ((($4)) + 36|0);
   HEAP32[$31>>2] = 5;
   $32 = ((($4)) + 40|0);
   HEAP32[$32>>2] = 4;
   $33 = ((($4)) + 12|0);
   HEAP32[$33>>2] = 2;
   $34 = HEAP32[(12748)>>2]|0;
   $35 = ($34|0)==(0);
   if ($35) {
    $36 = ((($4)) + 76|0);
    HEAP32[$36>>2] = -1;
   }
   ___lock(((12772)|0));
   $37 = HEAP32[(12768)>>2]|0;
   $38 = ((($4)) + 56|0);
   HEAP32[$38>>2] = $37;
   $39 = ($37|0)==(0);
   if (!($39)) {
    $40 = $37;
    $41 = ((($40)) + 52|0);
    HEAP32[$41>>2] = $4;
   }
   HEAP32[(12768)>>2] = $4;
   ___unlock(((12772)|0));
   $$0 = $4;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function _sprintf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsprintf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _fputc($c,$f) {
 $c = $c|0;
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)<(0);
 if ($2) {
  label = 3;
 } else {
  $3 = (___lockfile($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   label = 3;
  } else {
   $18 = ((($f)) + 75|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19 << 24 >> 24;
   $21 = ($20|0)==($c|0);
   if ($21) {
    label = 10;
   } else {
    $22 = ((($f)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ((($f)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($23>>>0)<($25>>>0);
    if ($26) {
     $27 = $c&255;
     $28 = ((($23)) + 1|0);
     HEAP32[$22>>2] = $28;
     HEAP8[$23>>0] = $27;
     $29 = $c & 255;
     $31 = $29;
    } else {
     label = 10;
    }
   }
   if ((label|0) == 10) {
    $30 = (___overflow($f,$c)|0);
    $31 = $30;
   }
   ___unlockfile($f);
   $$0 = $31;
  }
 }
 do {
  if ((label|0) == 3) {
   $5 = ((($f)) + 75|0);
   $6 = HEAP8[$5>>0]|0;
   $7 = $6 << 24 >> 24;
   $8 = ($7|0)==($c|0);
   if (!($8)) {
    $9 = ((($f)) + 20|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ((($f)) + 16|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = ($10>>>0)<($12>>>0);
    if ($13) {
     $14 = $c&255;
     $15 = ((($10)) + 1|0);
     HEAP32[$9>>2] = $15;
     HEAP8[$10>>0] = $14;
     $16 = $c & 255;
     $$0 = $16;
     break;
    }
   }
   $17 = (___overflow($f,$c)|0);
   $$0 = $17;
  }
 } while(0);
 return ($$0|0);
}
function _fileno($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>(-1);
 if ($2) {
  $3 = (___lockfile($f)|0);
  $phitmp = ($3|0)==(0);
  if (!($phitmp)) {
   ___unlockfile($f);
  }
 }
 $4 = ((($f)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 return ($5|0);
}
function ___fseeko_unlocked($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $$0 = 0, $$01 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($whence|0)==(1);
 if ($0) {
  $1 = ((($f)) + 8|0);
  $2 = HEAP32[$1>>2]|0;
  $3 = ((($f)) + 4|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = $2;
  $6 = $4;
  $7 = (($off) - ($5))|0;
  $8 = (($7) + ($6))|0;
  $$01 = $8;
 } else {
  $$01 = $off;
 }
 $9 = ((($f)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($f)) + 28|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($10>>>0)>($12>>>0);
 if ($13) {
  $14 = ((($f)) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  (FUNCTION_TABLE_iiii[$15 & 15]($f,0,0)|0);
  $16 = HEAP32[$9>>2]|0;
  $17 = ($16|0)==(0|0);
  if ($17) {
   $$0 = -1;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $18 = ((($f)) + 16|0);
  HEAP32[$18>>2] = 0;
  HEAP32[$11>>2] = 0;
  HEAP32[$9>>2] = 0;
  $19 = ((($f)) + 40|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (FUNCTION_TABLE_iiii[$20 & 15]($f,$$01,$whence)|0);
  $22 = ($21|0)<(0);
  if ($22) {
   $$0 = -1;
  } else {
   $23 = ((($f)) + 8|0);
   HEAP32[$23>>2] = 0;
   $24 = ((($f)) + 4|0);
   HEAP32[$24>>2] = 0;
   $25 = HEAP32[$f>>2]|0;
   $26 = $25 & -17;
   HEAP32[$f>>2] = $26;
   $$0 = 0;
  }
 }
 return ($$0|0);
}
function ___fseeko($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>(-1);
 if ($2) {
  $4 = (___lockfile($f)|0);
  $phitmp = ($4|0)==(0);
  $5 = (___fseeko_unlocked($f,$off,$whence)|0);
  if ($phitmp) {
   $6 = $5;
  } else {
   ___unlockfile($f);
   $6 = $5;
  }
 } else {
  $3 = (___fseeko_unlocked($f,$off,$whence)|0);
  $6 = $3;
 }
 return ($6|0);
}
function _fseek($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___fseeko($f,$off,$whence)|0);
 return ($0|0);
}
function _tmpfile() {
 var $$0 = 0, $$lcssa = 0, $$lcssa7 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $buf = 0, $try$02 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $buf = sp + 20|0;
 $try$02 = 0;
 while(1) {
  $2 = (_tmpnam($buf)|0);
  $3 = ($2|0)==(0|0);
  if ($3) {
   $$0 = 0;
   break;
  }
  HEAP32[$vararg_buffer>>2] = $2;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 32962;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 384;
  $4 = (___syscall5(5,($vararg_buffer|0))|0);
  $5 = (___syscall_ret($4)|0);
  $6 = ($5|0)>(-1);
  $0 = (($try$02) + 1)|0;
  if ($6) {
   $$lcssa = $5;$$lcssa7 = $2;
   label = 5;
   break;
  }
  $1 = ($0|0)<(100);
  if ($1) {
   $try$02 = $0;
  } else {
   $$0 = 0;
   break;
  }
 }
 if ((label|0) == 5) {
  $7 = (___fdopen($$lcssa,20540)|0);
  HEAP32[$vararg_buffer3>>2] = $$lcssa7;
  (___syscall10(10,($vararg_buffer3|0))|0);
  $$0 = $7;
 }
 STACKTOP = sp;return ($$0|0);
}
function ___overflow($f,$_c) {
 $f = $f|0;
 $_c = $_c|0;
 var $$0 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $c = sp;
 $0 = $_c&255;
 HEAP8[$c>>0] = $0;
 $1 = ((($f)) + 16|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  $4 = (___towrite($f)|0);
  $5 = ($4|0)==(0);
  if ($5) {
   $$pre = HEAP32[$1>>2]|0;
   $9 = $$pre;
   label = 4;
  } else {
   $$0 = -1;
  }
 } else {
  $9 = $2;
  label = 4;
 }
 do {
  if ((label|0) == 4) {
   $6 = ((($f)) + 20|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7>>>0)<($9>>>0);
   if ($8) {
    $10 = $_c & 255;
    $11 = ((($f)) + 75|0);
    $12 = HEAP8[$11>>0]|0;
    $13 = $12 << 24 >> 24;
    $14 = ($10|0)==($13|0);
    if (!($14)) {
     $15 = ((($7)) + 1|0);
     HEAP32[$6>>2] = $15;
     HEAP8[$7>>0] = $0;
     $$0 = $10;
     break;
    }
   }
   $16 = ((($f)) + 36|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (FUNCTION_TABLE_iiii[$17 & 15]($f,$c,1)|0);
   $19 = ($18|0)==(1);
   if ($19) {
    $20 = HEAP8[$c>>0]|0;
    $21 = $20&255;
    $$0 = $21;
   } else {
    $$0 = -1;
   }
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function _perror($msg) {
 $msg = $msg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[12736>>2]|0;
 $1 = (___errno_location()|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = (_strerror($2)|0);
 $4 = ((($0)) + 76|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(-1);
 if ($6) {
  $7 = (___lockfile($0)|0);
  $13 = $7;
 } else {
  $13 = 0;
 }
 $8 = ($msg|0)==(0|0);
 if (!($8)) {
  $9 = HEAP8[$msg>>0]|0;
  $10 = ($9<<24>>24)==(0);
  if (!($10)) {
   $11 = (_strlen($msg)|0);
   (_fwrite($msg,$11,1,$0)|0);
   (_fputc(58,$0)|0);
   (_fputc(32,$0)|0);
  }
 }
 $12 = (_strlen($3)|0);
 (_fwrite($3,$12,1,$0)|0);
 (_fputc(10,$0)|0);
 $14 = ($13|0)==(0);
 if (!($14)) {
  ___unlockfile($0);
 }
 return;
}
function _snprintf($s,$n,$fmt,$varargs) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsnprintf($s,$n,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function ___stdio_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt$0 = 0, $cnt$1 = 0, $iov$0 = 0, $iov$0$lcssa11 = 0, $iov$1 = 0, $iovcnt$0 = 0;
 var $iovcnt$0$lcssa12 = 0, $iovcnt$1 = 0, $iovs = 0, $rem$0 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $iovs = sp + 32|0;
 $0 = ((($f)) + 28|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$iovs>>2] = $1;
 $2 = ((($iovs)) + 4|0);
 $3 = ((($f)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4;
 $6 = (($5) - ($1))|0;
 HEAP32[$2>>2] = $6;
 $7 = ((($iovs)) + 8|0);
 HEAP32[$7>>2] = $buf;
 $8 = ((($iovs)) + 12|0);
 HEAP32[$8>>2] = $len;
 $9 = (($6) + ($len))|0;
 $10 = ((($f)) + 60|0);
 $11 = ((($f)) + 44|0);
 $iov$0 = $iovs;$iovcnt$0 = 2;$rem$0 = $9;
 while(1) {
  $12 = HEAP32[12744>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $17 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $17;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = $iov$0;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $iovcnt$0;
   $18 = (___syscall146(146,($vararg_buffer3|0))|0);
   $19 = (___syscall_ret($18)|0);
   $cnt$0 = $19;
  } else {
   _pthread_cleanup_push((14|0),($f|0));
   $14 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $iov$0;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $iovcnt$0;
   $15 = (___syscall146(146,($vararg_buffer|0))|0);
   $16 = (___syscall_ret($15)|0);
   _pthread_cleanup_pop(0);
   $cnt$0 = $16;
  }
  $20 = ($rem$0|0)==($cnt$0|0);
  if ($20) {
   label = 6;
   break;
  }
  $27 = ($cnt$0|0)<(0);
  if ($27) {
   $iov$0$lcssa11 = $iov$0;$iovcnt$0$lcssa12 = $iovcnt$0;
   label = 8;
   break;
  }
  $35 = (($rem$0) - ($cnt$0))|0;
  $36 = ((($iov$0)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($cnt$0>>>0)>($37>>>0);
  if ($38) {
   $39 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $39;
   HEAP32[$3>>2] = $39;
   $40 = (($cnt$0) - ($37))|0;
   $41 = ((($iov$0)) + 8|0);
   $42 = (($iovcnt$0) + -1)|0;
   $$phi$trans$insert = ((($iov$0)) + 12|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   $50 = $$pre;$cnt$1 = $40;$iov$1 = $41;$iovcnt$1 = $42;
  } else {
   $43 = ($iovcnt$0|0)==(2);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = (($44) + ($cnt$0)|0);
    HEAP32[$0>>2] = $45;
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = 2;
   } else {
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = $iovcnt$0;
   }
  }
  $46 = HEAP32[$iov$1>>2]|0;
  $47 = (($46) + ($cnt$1)|0);
  HEAP32[$iov$1>>2] = $47;
  $48 = ((($iov$1)) + 4|0);
  $49 = (($50) - ($cnt$1))|0;
  HEAP32[$48>>2] = $49;
  $iov$0 = $iov$1;$iovcnt$0 = $iovcnt$1;$rem$0 = $35;
 }
 if ((label|0) == 6) {
  $21 = HEAP32[$11>>2]|0;
  $22 = ((($f)) + 48|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + ($23)|0);
  $25 = ((($f)) + 16|0);
  HEAP32[$25>>2] = $24;
  $26 = $21;
  HEAP32[$0>>2] = $26;
  HEAP32[$3>>2] = $26;
  $$0 = $len;
 }
 else if ((label|0) == 8) {
  $28 = ((($f)) + 16|0);
  HEAP32[$28>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$3>>2] = 0;
  $29 = HEAP32[$f>>2]|0;
  $30 = $29 | 32;
  HEAP32[$f>>2] = $30;
  $31 = ($iovcnt$0$lcssa12|0)==(2);
  if ($31) {
   $$0 = 0;
  } else {
   $32 = ((($iov$0$lcssa11)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($len) - ($33))|0;
   $$0 = $34;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function _fopen($filename,$mode) {
 $filename = $filename|0;
 $mode = $mode|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $memchr = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = HEAP8[$mode>>0]|0;
 $1 = $0 << 24 >> 24;
 $memchr = (_memchr(20543,$1,4)|0);
 $2 = ($memchr|0)==(0|0);
 if ($2) {
  $3 = (___errno_location()|0);
  HEAP32[$3>>2] = 22;
  $$0 = 0;
 } else {
  $4 = (___fmodeflags($mode)|0);
  $5 = $4 | 32768;
  HEAP32[$vararg_buffer>>2] = $filename;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $5;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 438;
  $6 = (___syscall5(5,($vararg_buffer|0))|0);
  $7 = (___syscall_ret($6)|0);
  $8 = ($7|0)<(0);
  if ($8) {
   $$0 = 0;
  } else {
   $9 = (___fdopen($7,$mode)|0);
   $10 = ($9|0)==(0|0);
   if ($10) {
    HEAP32[$vararg_buffer3>>2] = $7;
    (___syscall6(6,($vararg_buffer3|0))|0);
    $$0 = 0;
   } else {
    $$0 = $9;
   }
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___fmodeflags($mode) {
 $mode = $mode|0;
 var $$ = 0, $$flags$4 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flags$0 = 0, $flags$0$ = 0, $flags$2 = 0;
 var $flags$2$ = 0, $flags$4 = 0, $not$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strchr($mode,43)|0);
 $1 = ($0|0)==(0|0);
 $2 = HEAP8[$mode>>0]|0;
 $not$ = ($2<<24>>24)!=(114);
 $$ = $not$&1;
 $flags$0 = $1 ? $$ : 2;
 $3 = (_strchr($mode,120)|0);
 $4 = ($3|0)==(0|0);
 $5 = $flags$0 | 128;
 $flags$0$ = $4 ? $flags$0 : $5;
 $6 = (_strchr($mode,101)|0);
 $7 = ($6|0)==(0|0);
 $8 = $flags$0$ | 524288;
 $flags$2 = $7 ? $flags$0$ : $8;
 $9 = ($2<<24>>24)==(114);
 $10 = $flags$2 | 64;
 $flags$2$ = $9 ? $flags$2 : $10;
 $11 = ($2<<24>>24)==(119);
 $12 = $flags$2$ | 512;
 $flags$4 = $11 ? $12 : $flags$2$;
 $13 = ($2<<24>>24)==(97);
 $14 = $flags$4 | 1024;
 $$flags$4 = $13 ? $14 : $flags$4;
 return ($$flags$4|0);
}
function ___lockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 0;
}
function ___unlockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function ___stdio_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $cnt$0 = 0, $iov = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $iov = sp + 32|0;
 HEAP32[$iov>>2] = $buf;
 $0 = ((($iov)) + 4|0);
 $1 = ((($f)) + 48|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 $4 = $3&1;
 $5 = (($len) - ($4))|0;
 HEAP32[$0>>2] = $5;
 $6 = ((($iov)) + 8|0);
 $7 = ((($f)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$6>>2] = $8;
 $9 = ((($iov)) + 12|0);
 HEAP32[$9>>2] = $2;
 $10 = HEAP32[12744>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $16 = ((($f)) + 60|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $17;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $iov;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = 2;
  $18 = (___syscall145(145,($vararg_buffer3|0))|0);
  $19 = (___syscall_ret($18)|0);
  $cnt$0 = $19;
 } else {
  _pthread_cleanup_push((15|0),($f|0));
  $12 = ((($f)) + 60|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $iov;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = 2;
  $14 = (___syscall145(145,($vararg_buffer|0))|0);
  $15 = (___syscall_ret($14)|0);
  _pthread_cleanup_pop(0);
  $cnt$0 = $15;
 }
 $20 = ($cnt$0|0)<(1);
 if ($20) {
  $21 = $cnt$0 & 48;
  $22 = $21 ^ 16;
  $23 = HEAP32[$f>>2]|0;
  $24 = $23 | $22;
  HEAP32[$f>>2] = $24;
  $25 = ((($f)) + 8|0);
  HEAP32[$25>>2] = 0;
  $26 = ((($f)) + 4|0);
  HEAP32[$26>>2] = 0;
  $$0 = $cnt$0;
 } else {
  $27 = HEAP32[$0>>2]|0;
  $28 = ($cnt$0>>>0)>($27>>>0);
  if ($28) {
   $29 = (($cnt$0) - ($27))|0;
   $30 = HEAP32[$7>>2]|0;
   $31 = ((($f)) + 4|0);
   HEAP32[$31>>2] = $30;
   $32 = $30;
   $33 = (($32) + ($29)|0);
   $34 = ((($f)) + 8|0);
   HEAP32[$34>>2] = $33;
   $35 = HEAP32[$1>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $$0 = $len;
   } else {
    $37 = ((($32)) + 1|0);
    HEAP32[$31>>2] = $37;
    $38 = HEAP8[$32>>0]|0;
    $39 = (($len) + -1)|0;
    $40 = (($buf) + ($39)|0);
    HEAP8[$40>>0] = $38;
    $$0 = $len;
   }
  } else {
   $$0 = $cnt$0;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function _vsprintf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_vsnprintf($s,2147483647,$fmt,$ap)|0);
 return ($0|0);
}
function _putc($c,$f) {
 $c = $c|0;
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)<(0);
 if ($2) {
  label = 3;
 } else {
  $3 = (___lockfile($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   label = 3;
  } else {
   $18 = ((($f)) + 75|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19 << 24 >> 24;
   $21 = ($20|0)==($c|0);
   if ($21) {
    label = 10;
   } else {
    $22 = ((($f)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ((($f)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($23>>>0)<($25>>>0);
    if ($26) {
     $27 = $c&255;
     $28 = ((($23)) + 1|0);
     HEAP32[$22>>2] = $28;
     HEAP8[$23>>0] = $27;
     $29 = $c & 255;
     $31 = $29;
    } else {
     label = 10;
    }
   }
   if ((label|0) == 10) {
    $30 = (___overflow($f,$c)|0);
    $31 = $30;
   }
   ___unlockfile($f);
   $$0 = $31;
  }
 }
 do {
  if ((label|0) == 3) {
   $5 = ((($f)) + 75|0);
   $6 = HEAP8[$5>>0]|0;
   $7 = $6 << 24 >> 24;
   $8 = ($7|0)==($c|0);
   if (!($8)) {
    $9 = ((($f)) + 20|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ((($f)) + 16|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = ($10>>>0)<($12>>>0);
    if ($13) {
     $14 = $c&255;
     $15 = ((($10)) + 1|0);
     HEAP32[$9>>2] = $15;
     HEAP8[$10>>0] = $14;
     $16 = $c & 255;
     $$0 = $16;
     break;
    }
   }
   $17 = (___overflow($f,$c)|0);
   $$0 = $17;
  }
 } while(0);
 return ($$0|0);
}
function _tmpnam($s) {
 $s = $s|0;
 var $$0 = 0, $$s = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $try$0 = 0, $try$1 = 0, $ts = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer2 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $ts = sp + 40|0;
 $1 = ($s|0)==(0|0);
 $$s = $1 ? 20547 : $s;
 HEAP32[$0>>2] = $$s;
 HEAP32[$vararg_buffer>>2] = 20567;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 7;
 $2 = (___syscall33(33,($vararg_buffer|0))|0);
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = ((($ts)) + 4|0);
  $5 = $0;
  $try$0 = 0;
  while(1) {
   HEAP32[$vararg_buffer2>>2] = 0;
   $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $ts;
   $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
   HEAP32[$vararg_ptr6>>2] = 0;
   (___syscall265(265,($vararg_buffer2|0))|0);
   $6 = HEAP32[$4>>2]|0;
   $7 = $6 ^ $5;
   $8 = HEAP32[$0>>2]|0;
   $9 = $8;
   $10 = $7 ^ $9;
   $11 = HEAP32[12900>>2]|0;HEAP32[12900>>2] = (($11+1)|0);
   HEAP32[$vararg_buffer7>>2] = $11;
   $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
   HEAP32[$vararg_ptr10>>2] = $10;
   (_snprintf($8,20,20572,$vararg_buffer7)|0);
   $12 = HEAP32[$0>>2]|0;
   HEAP32[$vararg_buffer11>>2] = $12;
   $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
   HEAP32[$vararg_ptr14>>2] = 0;
   $13 = (___syscall33(33,($vararg_buffer11|0))|0);
   $14 = ($13|0)==(0);
   if (!($14)) {
    $try$1 = $try$0;
    break;
   }
   $15 = (($try$0) + 1)|0;
   $16 = ($try$0|0)<(100);
   if ($16) {
    $try$0 = $15;
   } else {
    $try$1 = $15;
    break;
   }
  }
  $17 = ($try$1|0)>(99);
  $18 = HEAP32[$0>>2]|0;
  $19 = $17 ? 0 : $18;
  $$0 = $19;
 } else {
  $$0 = 0;
 }
 STACKTOP = sp;return ($$0|0);
}
function _fclose($f) {
 $f = $f|0;
 var $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>(-1);
 if ($2) {
 }
 $3 = HEAP32[$f>>2]|0;
 $4 = $3 & 1;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  ___lock(((12772)|0));
  $6 = ((($f)) + 52|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0|0);
  $9 = $7;
  $$pre = ((($f)) + 56|0);
  if (!($8)) {
   $10 = HEAP32[$$pre>>2]|0;
   $11 = ((($7)) + 56|0);
   HEAP32[$11>>2] = $10;
  }
  $12 = HEAP32[$$pre>>2]|0;
  $13 = ($12|0)==(0|0);
  $14 = $12;
  if (!($13)) {
   $15 = ((($12)) + 52|0);
   HEAP32[$15>>2] = $9;
  }
  $16 = HEAP32[(12768)>>2]|0;
  $17 = ($16|0)==($f|0);
  if ($17) {
   HEAP32[(12768)>>2] = $14;
  }
  ___unlock(((12772)|0));
 }
 $18 = (_fflush($f)|0);
 $19 = ((($f)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (FUNCTION_TABLE_ii[$20 & 15]($f)|0);
 $22 = $21 | $18;
 $23 = ((($f)) + 92|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)==(0|0);
 if (!($25)) {
  _free($24);
 }
 if (!($5)) {
  _free($f);
 }
 return ($22|0);
}
function _freopen($filename,$mode,$f) {
 $filename = $filename|0;
 $mode = $mode|0;
 $f = $f|0;
 var $$0 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = (___fmodeflags($mode)|0);
 $1 = ((($f)) + 76|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(-1);
 if ($3) {
  $4 = (___lockfile($f)|0);
  $40 = $4;
 } else {
  $40 = 0;
 }
 (_fflush($f)|0);
 $5 = ($filename|0)==(0|0);
 do {
  if ($5) {
   $6 = $0 & 524288;
   $7 = ($6|0)==(0);
   $$pre = ((($f)) + 60|0);
   if (!($7)) {
    $8 = HEAP32[$$pre>>2]|0;
    HEAP32[$vararg_buffer>>2] = $8;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = 2;
    $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
    HEAP32[$vararg_ptr2>>2] = 1;
    (___syscall221(221,($vararg_buffer|0))|0);
   }
   $9 = $0 & -524481;
   $10 = HEAP32[$$pre>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $10;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = 4;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $9;
   $11 = (___syscall221(221,($vararg_buffer3|0))|0);
   $12 = (___syscall_ret($11)|0);
   $13 = ($12|0)<(0);
   if ($13) {
    label = 15;
   } else {
    label = 12;
   }
  } else {
   $14 = (_fopen($filename,$mode)|0);
   $15 = ($14|0)==(0|0);
   if ($15) {
    label = 15;
   } else {
    $16 = ((($14)) + 60|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ((($f)) + 60|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = ($17|0)==($19|0);
    if ($20) {
     HEAP32[$16>>2] = -1;
    } else {
     $21 = $0 & 524288;
     $22 = (___dup3($17,$19,$21)|0);
     $23 = ($22|0)<(0);
     if ($23) {
      (_fclose($14)|0);
      label = 15;
      break;
     }
    }
    $24 = HEAP32[$f>>2]|0;
    $25 = $24 & 1;
    $26 = HEAP32[$14>>2]|0;
    $27 = $25 | $26;
    HEAP32[$f>>2] = $27;
    $28 = ((($14)) + 32|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ((($f)) + 32|0);
    HEAP32[$30>>2] = $29;
    $31 = ((($14)) + 36|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ((($f)) + 36|0);
    HEAP32[$33>>2] = $32;
    $34 = ((($14)) + 40|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ((($f)) + 40|0);
    HEAP32[$36>>2] = $35;
    $37 = ((($14)) + 12|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ((($f)) + 12|0);
    HEAP32[$39>>2] = $38;
    (_fclose($14)|0);
    label = 12;
   }
  }
 } while(0);
 if ((label|0) == 12) {
  $41 = ($40|0)==(0);
  if ($41) {
   $$0 = $f;
  } else {
   ___unlockfile($f);
   $$0 = $f;
  }
 }
 else if ((label|0) == 15) {
  (_fclose($f)|0);
  $$0 = 0;
 }
 STACKTOP = sp;return ($$0|0);
}
function _rewind($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 76|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>(-1);
 if ($2) {
  $3 = (___lockfile($f)|0);
  $phitmp = ($3|0)==(0);
  (___fseeko_unlocked($f,0,0)|0);
  $4 = HEAP32[$f>>2]|0;
  $5 = $4 & -33;
  HEAP32[$f>>2] = $5;
  if (!($phitmp)) {
   ___unlockfile($f);
  }
 } else {
  (___fseeko_unlocked($f,0,0)|0);
  $6 = HEAP32[$f>>2]|0;
  $7 = $6 & -33;
  HEAP32[$f>>2] = $7;
 }
 return;
}
function _vfprintf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap2 = 0, $internal_buf = 0, $nl_arg = 0, $nl_type = 0;
 var $ret$1 = 0, $ret$1$ = 0, $vacopy_currentptr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap2 = sp + 120|0;
 $nl_type = sp + 80|0;
 $nl_arg = sp;
 $internal_buf = sp + 136|0;
 dest=$nl_type; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $vacopy_currentptr = HEAP32[$ap>>2]|0;
 HEAP32[$ap2>>2] = $vacopy_currentptr;
 $0 = (_printf_core(0,$fmt,$ap2,$nl_arg,$nl_type)|0);
 $1 = ($0|0)<(0);
 if ($1) {
  $$0 = -1;
 } else {
  $2 = ((($f)) + 76|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)>(-1);
  if ($4) {
   $5 = (___lockfile($f)|0);
   $32 = $5;
  } else {
   $32 = 0;
  }
  $6 = HEAP32[$f>>2]|0;
  $7 = $6 & 32;
  $8 = ((($f)) + 74|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = ($9<<24>>24)<(1);
  if ($10) {
   $11 = $6 & -33;
   HEAP32[$f>>2] = $11;
  }
  $12 = ((($f)) + 48|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)==(0);
  if ($14) {
   $16 = ((($f)) + 44|0);
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$16>>2] = $internal_buf;
   $18 = ((($f)) + 28|0);
   HEAP32[$18>>2] = $internal_buf;
   $19 = ((($f)) + 20|0);
   HEAP32[$19>>2] = $internal_buf;
   HEAP32[$12>>2] = 80;
   $20 = ((($internal_buf)) + 80|0);
   $21 = ((($f)) + 16|0);
   HEAP32[$21>>2] = $20;
   $22 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
   $23 = ($17|0)==(0|0);
   if ($23) {
    $ret$1 = $22;
   } else {
    $24 = ((($f)) + 36|0);
    $25 = HEAP32[$24>>2]|0;
    (FUNCTION_TABLE_iiii[$25 & 15]($f,0,0)|0);
    $26 = HEAP32[$19>>2]|0;
    $27 = ($26|0)==(0|0);
    $$ = $27 ? -1 : $22;
    HEAP32[$16>>2] = $17;
    HEAP32[$12>>2] = 0;
    HEAP32[$21>>2] = 0;
    HEAP32[$18>>2] = 0;
    HEAP32[$19>>2] = 0;
    $ret$1 = $$;
   }
  } else {
   $15 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
   $ret$1 = $15;
  }
  $28 = HEAP32[$f>>2]|0;
  $29 = $28 & 32;
  $30 = ($29|0)==(0);
  $ret$1$ = $30 ? $ret$1 : -1;
  $31 = $28 | $7;
  HEAP32[$f>>2] = $31;
  $33 = ($32|0)==(0);
  if (!($33)) {
   ___unlockfile($f);
  }
  $$0 = $ret$1$;
 }
 STACKTOP = sp;return ($$0|0);
}
function ___stdio_seek($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $$pre = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $ret = sp + 20|0;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $off;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $ret;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $whence;
 $2 = (___syscall140(140,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 $4 = ($3|0)<(0);
 if ($4) {
  HEAP32[$ret>>2] = -1;
  $5 = -1;
 } else {
  $$pre = HEAP32[$ret>>2]|0;
  $5 = $$pre;
 }
 STACKTOP = sp;return ($5|0);
}
function _fputs($s,$f) {
 $s = $s|0;
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen($s)|0);
 $1 = (_fwrite($s,$0,1,$f)|0);
 $2 = (($1) + -1)|0;
 return ($2|0);
}
function ___fwritex($s,$l,$f) {
 $s = $s|0;
 $l = $l|0;
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$pre = 0, $$pre6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$0 = 0, $i$0$lcssa10 = 0;
 var $i$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  $3 = (___towrite($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   $$pre = HEAP32[$0>>2]|0;
   $7 = $$pre;
   label = 4;
  } else {
   $$0 = 0;
  }
 } else {
  $7 = $1;
  label = 4;
 }
 L4: do {
  if ((label|0) == 4) {
   $5 = ((($f)) + 20|0);
   $6 = HEAP32[$5>>2]|0;
   $8 = $7;
   $9 = $6;
   $10 = (($8) - ($9))|0;
   $11 = ($10>>>0)<($l>>>0);
   if ($11) {
    $12 = ((($f)) + 36|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = (FUNCTION_TABLE_iiii[$13 & 15]($f,$s,$l)|0);
    $$0 = $14;
    break;
   }
   $15 = ((($f)) + 75|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)>(-1);
   L9: do {
    if ($17) {
     $i$0 = $l;
     while(1) {
      $18 = ($i$0|0)==(0);
      if ($18) {
       $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
       break L9;
      }
      $19 = (($i$0) + -1)|0;
      $20 = (($s) + ($19)|0);
      $21 = HEAP8[$20>>0]|0;
      $22 = ($21<<24>>24)==(10);
      if ($22) {
       $i$0$lcssa10 = $i$0;
       break;
      } else {
       $i$0 = $19;
      }
     }
     $23 = ((($f)) + 36|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = (FUNCTION_TABLE_iiii[$24 & 15]($f,$s,$i$0$lcssa10)|0);
     $26 = ($25>>>0)<($i$0$lcssa10>>>0);
     if ($26) {
      $$0 = $i$0$lcssa10;
      break L4;
     }
     $27 = (($s) + ($i$0$lcssa10)|0);
     $28 = (($l) - ($i$0$lcssa10))|0;
     $$pre6 = HEAP32[$5>>2]|0;
     $$01 = $28;$$02 = $27;$29 = $$pre6;$i$1 = $i$0$lcssa10;
    } else {
     $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
    }
   } while(0);
   _memcpy(($29|0),($$02|0),($$01|0))|0;
   $30 = HEAP32[$5>>2]|0;
   $31 = (($30) + ($$01)|0);
   HEAP32[$5>>2] = $31;
   $32 = (($i$1) + ($$01))|0;
   $$0 = $32;
  }
 } while(0);
 return ($$0|0);
}
function _fwrite($src,$size,$nmemb,$f) {
 $src = $src|0;
 $size = $size|0;
 $nmemb = $nmemb|0;
 $f = $f|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $phitmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = Math_imul($nmemb, $size)|0;
 $1 = ((($f)) + 76|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(-1);
 if ($3) {
  $5 = (___lockfile($f)|0);
  $phitmp = ($5|0)==(0);
  $6 = (___fwritex($src,$0,$f)|0);
  if ($phitmp) {
   $7 = $6;
  } else {
   ___unlockfile($f);
   $7 = $6;
  }
 } else {
  $4 = (___fwritex($src,$0,$f)|0);
  $7 = $4;
 }
 $8 = ($7|0)==($0|0);
 if ($8) {
  $10 = $nmemb;
 } else {
  $9 = (($7>>>0) / ($size>>>0))&-1;
  $10 = $9;
 }
 return ($10|0);
}
function _fprintf($f,$fmt,$varargs) {
 $f = $f|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vfprintf($f,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function ___stdio_close($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $2 = (___syscall6(6,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function ___errno_location() {
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[12744>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  $$0 = 12904;
 } else {
  $2 = (_pthread_self()|0);
  $3 = ((($2)) + 60|0);
  $4 = HEAP32[$3>>2]|0;
  $$0 = $4;
 }
 return ($$0|0);
}
function _strerror($e) {
 $e = $e|0;
 var $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$03 = 0, $i$03$lcssa = 0, $i$12 = 0, $s$0$lcssa = 0, $s$01 = 0, $s$1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $i$03 = 0;
 while(1) {
  $1 = (20584 + ($i$03)|0);
  $2 = HEAP8[$1>>0]|0;
  $3 = $2&255;
  $4 = ($3|0)==($e|0);
  if ($4) {
   $i$03$lcssa = $i$03;
   label = 2;
   break;
  }
  $5 = (($i$03) + 1)|0;
  $6 = ($5|0)==(87);
  if ($6) {
   $i$12 = 87;$s$01 = 20672;
   label = 5;
   break;
  } else {
   $i$03 = $5;
  }
 }
 if ((label|0) == 2) {
  $0 = ($i$03$lcssa|0)==(0);
  if ($0) {
   $s$0$lcssa = 20672;
  } else {
   $i$12 = $i$03$lcssa;$s$01 = 20672;
   label = 5;
  }
 }
 if ((label|0) == 5) {
  while(1) {
   label = 0;
   $s$1 = $s$01;
   while(1) {
    $7 = HEAP8[$s$1>>0]|0;
    $8 = ($7<<24>>24)==(0);
    $9 = ((($s$1)) + 1|0);
    if ($8) {
     $$lcssa = $9;
     break;
    } else {
     $s$1 = $9;
    }
   }
   $10 = (($i$12) + -1)|0;
   $11 = ($10|0)==(0);
   if ($11) {
    $s$0$lcssa = $$lcssa;
    break;
   } else {
    $i$12 = $10;$s$01 = $$lcssa;
    label = 5;
   }
  }
 }
 return ($s$0$lcssa|0);
}
function _stat($path,$buf) {
 $path = $path|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $0 = (___syscall195(195,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _fstat($fd,$st) {
 $fd = $fd|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $st;
 $0 = (___syscall197(197,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer2>>2] = $fd;
  $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer2|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer6>>2] = $buf;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $st;
   $5 = (___syscall195(195,($vararg_buffer6|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _atoi($s) {
 $s = $s|0;
 var $$0 = 0, $$0$lcssa = 0, $$1$ph = 0, $$13 = 0, $$lcssa9 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $isdigit = 0, $isdigit2 = 0, $isdigittmp = 0, $isdigittmp1 = 0, $isdigittmp5 = 0, $n$0$lcssa = 0, $n$04 = 0, $neg$0 = 0, $neg$1$ph = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $$0 = $s;
 while(1) {
  $0 = HEAP8[$$0>>0]|0;
  $1 = $0 << 24 >> 24;
  $2 = (_isspace($1)|0);
  $3 = ($2|0)==(0);
  $4 = ((($$0)) + 1|0);
  if ($3) {
   $$0$lcssa = $$0;$$lcssa9 = $4;
   break;
  } else {
   $$0 = $4;
  }
 }
 $5 = HEAP8[$$0$lcssa>>0]|0;
 $6 = $5 << 24 >> 24;
 switch ($6|0) {
 case 45:  {
  $neg$0 = 1;
  label = 5;
  break;
 }
 case 43:  {
  $neg$0 = 0;
  label = 5;
  break;
 }
 default: {
  $$1$ph = $$0$lcssa;$8 = $5;$neg$1$ph = 0;
 }
 }
 if ((label|0) == 5) {
  $$pre = HEAP8[$$lcssa9>>0]|0;
  $$1$ph = $$lcssa9;$8 = $$pre;$neg$1$ph = $neg$0;
 }
 $7 = $8 << 24 >> 24;
 $isdigittmp1 = (($7) + -48)|0;
 $isdigit2 = ($isdigittmp1>>>0)<(10);
 if ($isdigit2) {
  $$13 = $$1$ph;$isdigittmp5 = $isdigittmp1;$n$04 = 0;
  while(1) {
   $9 = ($n$04*10)|0;
   $10 = ((($$13)) + 1|0);
   $11 = (($9) - ($isdigittmp5))|0;
   $12 = HEAP8[$10>>0]|0;
   $13 = $12 << 24 >> 24;
   $isdigittmp = (($13) + -48)|0;
   $isdigit = ($isdigittmp>>>0)<(10);
   if ($isdigit) {
    $$13 = $10;$isdigittmp5 = $isdigittmp;$n$04 = $11;
   } else {
    $n$0$lcssa = $11;
    break;
   }
  }
 } else {
  $n$0$lcssa = 0;
 }
 $14 = ($neg$1$ph|0)!=(0);
 $15 = (0 - ($n$0$lcssa))|0;
 $16 = $14 ? $n$0$lcssa : $15;
 return ($16|0);
}
function _frexp($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $$0 = 0.0, $$01 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0, $storemerge = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 switch ($4|0) {
 case 0:  {
  $5 = $x != 0.0;
  if ($5) {
   $6 = $x * 1.8446744073709552E+19;
   $7 = (+_frexp($6,$e));
   $8 = HEAP32[$e>>2]|0;
   $9 = (($8) + -64)|0;
   $$01 = $7;$storemerge = $9;
  } else {
   $$01 = $x;$storemerge = 0;
  }
  HEAP32[$e>>2] = $storemerge;
  $$0 = $$01;
  break;
 }
 case 2047:  {
  $$0 = $x;
  break;
 }
 default: {
  $10 = (($4) + -1022)|0;
  HEAP32[$e>>2] = $10;
  $11 = $1 & -2146435073;
  $12 = $11 | 1071644672;
  HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $12;$13 = +HEAPF64[tempDoublePtr>>3];
  $$0 = $13;
 }
 }
 return (+$$0);
}
function _frexpl($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_frexp($x,$e));
 return (+$0);
}
function ___syscall_ret($r) {
 $r = $r|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($r>>>0)>(4294963200);
 if ($0) {
  $1 = (0 - ($r))|0;
  $2 = (___errno_location()|0);
  HEAP32[$2>>2] = $1;
  $$0 = -1;
 } else {
  $$0 = $r;
 }
 return ($$0|0);
}
function ___procfdname($buf,$fd) {
 $buf = $buf|0;
 $fd = $fd|0;
 var $$01 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i$0 = 0, $i$13 = 0, $i$22 = 0, $j$04 = 0, $scevgep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $i$0 = 0;
 while(1) {
  $0 = (22476 + ($i$0)|0);
  $1 = HEAP8[$0>>0]|0;
  $2 = (($buf) + ($i$0)|0);
  HEAP8[$2>>0] = $1;
  $3 = ($i$0|0)==(14);
  $4 = (($i$0) + 1)|0;
  if ($3) {
   break;
  } else {
   $i$0 = $4;
  }
 }
 $5 = ($fd|0)==(0);
 if ($5) {
  $scevgep = ((($buf)) + 14|0);
  HEAP8[$scevgep>>0] = 48;
  $6 = ((($buf)) + 15|0);
  HEAP8[$6>>0] = 0;
 } else {
  $i$13 = 14;$j$04 = $fd;
  while(1) {
   $7 = (($j$04>>>0) / 10)&-1;
   $8 = (($i$13) + 1)|0;
   $9 = ($j$04>>>0)<(10);
   if ($9) {
    $$lcssa = $8;
    break;
   } else {
    $i$13 = $8;$j$04 = $7;
   }
  }
  $10 = (($buf) + ($$lcssa)|0);
  HEAP8[$10>>0] = 0;
  $$01 = $fd;$i$22 = $$lcssa;
  while(1) {
   $11 = (($$01>>>0) % 10)&-1;
   $12 = $11 | 48;
   $13 = $12&255;
   $14 = (($i$22) + -1)|0;
   $15 = (($buf) + ($14)|0);
   HEAP8[$15>>0] = $13;
   $16 = (($$01>>>0) / 10)&-1;
   $17 = ($$01>>>0)<(10);
   if ($17) {
    break;
   } else {
    $$01 = $16;$i$22 = $14;
   }
  }
 }
 return;
}
function _wctomb($s,$wc) {
 $s = $s|0;
 $wc = $wc|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 0;
 } else {
  $1 = (_wcrtomb($s,$wc,0)|0);
  $$0 = $1;
 }
 return ($$0|0);
}
function _wcrtomb($s,$wc,$st) {
 $s = $s|0;
 $wc = $wc|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 do {
  if ($0) {
   $$0 = 1;
  } else {
   $1 = ($wc>>>0)<(128);
   if ($1) {
    $2 = $wc&255;
    HEAP8[$s>>0] = $2;
    $$0 = 1;
    break;
   }
   $3 = ($wc>>>0)<(2048);
   if ($3) {
    $4 = $wc >>> 6;
    $5 = $4 | 192;
    $6 = $5&255;
    $7 = ((($s)) + 1|0);
    HEAP8[$s>>0] = $6;
    $8 = $wc & 63;
    $9 = $8 | 128;
    $10 = $9&255;
    HEAP8[$7>>0] = $10;
    $$0 = 2;
    break;
   }
   $11 = ($wc>>>0)<(55296);
   $12 = $wc & -8192;
   $13 = ($12|0)==(57344);
   $or$cond = $11 | $13;
   if ($or$cond) {
    $14 = $wc >>> 12;
    $15 = $14 | 224;
    $16 = $15&255;
    $17 = ((($s)) + 1|0);
    HEAP8[$s>>0] = $16;
    $18 = $wc >>> 6;
    $19 = $18 & 63;
    $20 = $19 | 128;
    $21 = $20&255;
    $22 = ((($s)) + 2|0);
    HEAP8[$17>>0] = $21;
    $23 = $wc & 63;
    $24 = $23 | 128;
    $25 = $24&255;
    HEAP8[$22>>0] = $25;
    $$0 = 3;
    break;
   }
   $26 = (($wc) + -65536)|0;
   $27 = ($26>>>0)<(1048576);
   if ($27) {
    $28 = $wc >>> 18;
    $29 = $28 | 240;
    $30 = $29&255;
    $31 = ((($s)) + 1|0);
    HEAP8[$s>>0] = $30;
    $32 = $wc >>> 12;
    $33 = $32 & 63;
    $34 = $33 | 128;
    $35 = $34&255;
    $36 = ((($s)) + 2|0);
    HEAP8[$31>>0] = $35;
    $37 = $wc >>> 6;
    $38 = $37 & 63;
    $39 = $38 | 128;
    $40 = $39&255;
    $41 = ((($s)) + 3|0);
    HEAP8[$36>>0] = $40;
    $42 = $wc & 63;
    $43 = $42 | 128;
    $44 = $43&255;
    HEAP8[$41>>0] = $44;
    $$0 = 4;
    break;
   } else {
    $45 = (___errno_location()|0);
    HEAP32[$45>>2] = 84;
    $$0 = -1;
    break;
   }
  }
 } while(0);
 return ($$0|0);
}
function _ioctl($fd,$req,$varargs) {
 $fd = $fd|0;
 $req = $req|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 $ap = sp + 16|0;
 HEAP32[$ap>>2] = $varargs;
 $arglist_current = HEAP32[$ap>>2]|0;
 $0 = $arglist_current;
 $1 = ((0) + 4|0);
 $expanded2 = $1;
 $expanded = (($expanded2) - 1)|0;
 $2 = (($0) + ($expanded))|0;
 $3 = ((0) + 4|0);
 $expanded6 = $3;
 $expanded5 = (($expanded6) - 1)|0;
 $expanded4 = $expanded5 ^ -1;
 $4 = $2 & $expanded4;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $arglist_next = ((($5)) + 4|0);
 HEAP32[$ap>>2] = $arglist_next;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $req;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $6;
 $7 = (___syscall54(54,($vararg_buffer|0))|0);
 $8 = (___syscall_ret($7)|0);
 STACKTOP = sp;return ($8|0);
}
function _tcgetattr($fd,$tio) {
 $fd = $fd|0;
 $tio = $tio|0;
 var $$ = 0, $0 = 0, $not$ = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $tio;
 $0 = (_ioctl($fd,21505,$vararg_buffer)|0);
 $not$ = ($0|0)!=(0);
 $$ = $not$ << 31 >> 31;
 STACKTOP = sp;return ($$|0);
}
function _isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($c|0)==(32);
 $1 = (($c) + -9)|0;
 $2 = ($1>>>0)<(5);
 $3 = $0 | $2;
 $4 = $3&1;
 return ($4|0);
}
function _strlen($s) {
 $s = $s|0;
 var $$0 = 0, $$01$lcssa = 0, $$014 = 0, $$1$lcssa = 0, $$lcssa20 = 0, $$pn = 0, $$pn15 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w$0 = 0, $w$0$lcssa = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $0 & 3;
 $2 = ($1|0)==(0);
 L1: do {
  if ($2) {
   $$01$lcssa = $s;
   label = 4;
  } else {
   $$014 = $s;$21 = $0;
   while(1) {
    $3 = HEAP8[$$014>>0]|0;
    $4 = ($3<<24>>24)==(0);
    if ($4) {
     $$pn = $21;
     break L1;
    }
    $5 = ((($$014)) + 1|0);
    $6 = $5;
    $7 = $6 & 3;
    $8 = ($7|0)==(0);
    if ($8) {
     $$01$lcssa = $5;
     label = 4;
     break;
    } else {
     $$014 = $5;$21 = $6;
    }
   }
  }
 } while(0);
 if ((label|0) == 4) {
  $w$0 = $$01$lcssa;
  while(1) {
   $9 = HEAP32[$w$0>>2]|0;
   $10 = (($9) + -16843009)|0;
   $11 = $9 & -2139062144;
   $12 = $11 ^ -2139062144;
   $13 = $12 & $10;
   $14 = ($13|0)==(0);
   $15 = ((($w$0)) + 4|0);
   if ($14) {
    $w$0 = $15;
   } else {
    $$lcssa20 = $9;$w$0$lcssa = $w$0;
    break;
   }
  }
  $16 = $$lcssa20&255;
  $17 = ($16<<24>>24)==(0);
  if ($17) {
   $$1$lcssa = $w$0$lcssa;
  } else {
   $$pn15 = $w$0$lcssa;
   while(1) {
    $18 = ((($$pn15)) + 1|0);
    $$pre = HEAP8[$18>>0]|0;
    $19 = ($$pre<<24>>24)==(0);
    if ($19) {
     $$1$lcssa = $18;
     break;
    } else {
     $$pn15 = $18;
    }
   }
  }
  $20 = $$1$lcssa;
  $$pn = $20;
 }
 $$0 = (($$pn) - ($0))|0;
 return ($$0|0);
}
function ___strchrnul($s,$c) {
 $s = $s|0;
 $c = $c|0;
 var $$0 = 0, $$02$lcssa = 0, $$0211 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond5 = 0, $w$0$lcssa = 0, $w$08 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = ($0|0)==(0);
 L1: do {
  if ($1) {
   $6 = (_strlen($s)|0);
   $7 = (($s) + ($6)|0);
   $$0 = $7;
  } else {
   $2 = $s;
   $3 = $2 & 3;
   $4 = ($3|0)==(0);
   if ($4) {
    $$02$lcssa = $s;
   } else {
    $5 = $c&255;
    $$0211 = $s;
    while(1) {
     $8 = HEAP8[$$0211>>0]|0;
     $9 = ($8<<24>>24)==(0);
     $10 = ($8<<24>>24)==($5<<24>>24);
     $or$cond = $9 | $10;
     if ($or$cond) {
      $$0 = $$0211;
      break L1;
     }
     $11 = ((($$0211)) + 1|0);
     $12 = $11;
     $13 = $12 & 3;
     $14 = ($13|0)==(0);
     if ($14) {
      $$02$lcssa = $11;
      break;
     } else {
      $$0211 = $11;
     }
    }
   }
   $15 = Math_imul($0, 16843009)|0;
   $16 = HEAP32[$$02$lcssa>>2]|0;
   $17 = (($16) + -16843009)|0;
   $18 = $16 & -2139062144;
   $19 = $18 ^ -2139062144;
   $20 = $19 & $17;
   $21 = ($20|0)==(0);
   L10: do {
    if ($21) {
     $23 = $16;$w$08 = $$02$lcssa;
     while(1) {
      $22 = $23 ^ $15;
      $24 = (($22) + -16843009)|0;
      $25 = $22 & -2139062144;
      $26 = $25 ^ -2139062144;
      $27 = $26 & $24;
      $28 = ($27|0)==(0);
      if (!($28)) {
       $w$0$lcssa = $w$08;
       break L10;
      }
      $29 = ((($w$08)) + 4|0);
      $30 = HEAP32[$29>>2]|0;
      $31 = (($30) + -16843009)|0;
      $32 = $30 & -2139062144;
      $33 = $32 ^ -2139062144;
      $34 = $33 & $31;
      $35 = ($34|0)==(0);
      if ($35) {
       $23 = $30;$w$08 = $29;
      } else {
       $w$0$lcssa = $29;
       break;
      }
     }
    } else {
     $w$0$lcssa = $$02$lcssa;
    }
   } while(0);
   $36 = $c&255;
   $$1 = $w$0$lcssa;
   while(1) {
    $37 = HEAP8[$$1>>0]|0;
    $38 = ($37<<24>>24)==(0);
    $39 = ($37<<24>>24)==($36<<24>>24);
    $or$cond5 = $38 | $39;
    $40 = ((($$1)) + 1|0);
    if ($or$cond5) {
     $$0 = $$1;
     break;
    } else {
     $$1 = $40;
    }
   }
  }
 } while(0);
 return ($$0|0);
}
function _memchr($src,$c,$n) {
 $src = $src|0;
 $c = $c|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa44 = 0, $$019 = 0, $$1$lcssa = 0, $$110 = 0, $$110$lcssa = 0, $$24 = 0, $$3 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond18 = 0, $s$0$lcssa = 0, $s$0$lcssa43 = 0, $s$020 = 0, $s$15 = 0, $s$2 = 0, $w$0$lcssa = 0, $w$011 = 0, $w$011$lcssa = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = $src;
 $2 = $1 & 3;
 $3 = ($2|0)!=(0);
 $4 = ($n|0)!=(0);
 $or$cond18 = $4 & $3;
 L1: do {
  if ($or$cond18) {
   $5 = $c&255;
   $$019 = $n;$s$020 = $src;
   while(1) {
    $6 = HEAP8[$s$020>>0]|0;
    $7 = ($6<<24>>24)==($5<<24>>24);
    if ($7) {
     $$0$lcssa44 = $$019;$s$0$lcssa43 = $s$020;
     label = 6;
     break L1;
    }
    $8 = ((($s$020)) + 1|0);
    $9 = (($$019) + -1)|0;
    $10 = $8;
    $11 = $10 & 3;
    $12 = ($11|0)!=(0);
    $13 = ($9|0)!=(0);
    $or$cond = $13 & $12;
    if ($or$cond) {
     $$019 = $9;$s$020 = $8;
    } else {
     $$0$lcssa = $9;$$lcssa = $13;$s$0$lcssa = $8;
     label = 5;
     break;
    }
   }
  } else {
   $$0$lcssa = $n;$$lcssa = $4;$s$0$lcssa = $src;
   label = 5;
  }
 } while(0);
 if ((label|0) == 5) {
  if ($$lcssa) {
   $$0$lcssa44 = $$0$lcssa;$s$0$lcssa43 = $s$0$lcssa;
   label = 6;
  } else {
   $$3 = 0;$s$2 = $s$0$lcssa;
  }
 }
 L8: do {
  if ((label|0) == 6) {
   $14 = HEAP8[$s$0$lcssa43>>0]|0;
   $15 = $c&255;
   $16 = ($14<<24>>24)==($15<<24>>24);
   if ($16) {
    $$3 = $$0$lcssa44;$s$2 = $s$0$lcssa43;
   } else {
    $17 = Math_imul($0, 16843009)|0;
    $18 = ($$0$lcssa44>>>0)>(3);
    L11: do {
     if ($18) {
      $$110 = $$0$lcssa44;$w$011 = $s$0$lcssa43;
      while(1) {
       $19 = HEAP32[$w$011>>2]|0;
       $20 = $19 ^ $17;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$110$lcssa = $$110;$w$011$lcssa = $w$011;
        break;
       }
       $26 = ((($w$011)) + 4|0);
       $27 = (($$110) + -4)|0;
       $28 = ($27>>>0)>(3);
       if ($28) {
        $$110 = $27;$w$011 = $26;
       } else {
        $$1$lcssa = $27;$w$0$lcssa = $26;
        label = 11;
        break L11;
       }
      }
      $$24 = $$110$lcssa;$s$15 = $w$011$lcssa;
     } else {
      $$1$lcssa = $$0$lcssa44;$w$0$lcssa = $s$0$lcssa43;
      label = 11;
     }
    } while(0);
    if ((label|0) == 11) {
     $29 = ($$1$lcssa|0)==(0);
     if ($29) {
      $$3 = 0;$s$2 = $w$0$lcssa;
      break;
     } else {
      $$24 = $$1$lcssa;$s$15 = $w$0$lcssa;
     }
    }
    while(1) {
     $30 = HEAP8[$s$15>>0]|0;
     $31 = ($30<<24>>24)==($15<<24>>24);
     if ($31) {
      $$3 = $$24;$s$2 = $s$15;
      break L8;
     }
     $32 = ((($s$15)) + 1|0);
     $33 = (($$24) + -1)|0;
     $34 = ($33|0)==(0);
     if ($34) {
      $$3 = 0;$s$2 = $32;
      break;
     } else {
      $$24 = $33;$s$15 = $32;
     }
    }
   }
  }
 } while(0);
 $35 = ($$3|0)!=(0);
 $36 = $35 ? $s$2 : 0;
 return ($36|0);
}
function _strcpy($dest,$src) {
 $dest = $dest|0;
 $src = $src|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 (___stpcpy($dest,$src)|0);
 return ($dest|0);
}
function _strchr($s,$c) {
 $s = $s|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___strchrnul($s,$c)|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $c&255;
 $3 = ($1<<24>>24)==($2<<24>>24);
 $4 = $3 ? $0 : 0;
 return ($4|0);
}
function ___stpcpy($d,$s) {
 $d = $d|0;
 $s = $s|0;
 var $$0$lcssa = 0, $$01$lcssa = 0, $$0115 = 0, $$016 = 0, $$03 = 0, $$1$ph = 0, $$12$ph = 0, $$128 = 0, $$19 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $wd$0$lcssa = 0, $wd$010 = 0, $ws$0$lcssa = 0, $ws$011 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $d;
 $2 = $0 ^ $1;
 $3 = $2 & 3;
 $4 = ($3|0)==(0);
 L1: do {
  if ($4) {
   $5 = $0 & 3;
   $6 = ($5|0)==(0);
   if ($6) {
    $$0$lcssa = $s;$$01$lcssa = $d;
   } else {
    $$0115 = $d;$$016 = $s;
    while(1) {
     $7 = HEAP8[$$016>>0]|0;
     HEAP8[$$0115>>0] = $7;
     $8 = ($7<<24>>24)==(0);
     if ($8) {
      $$03 = $$0115;
      break L1;
     }
     $9 = ((($$016)) + 1|0);
     $10 = ((($$0115)) + 1|0);
     $11 = $9;
     $12 = $11 & 3;
     $13 = ($12|0)==(0);
     if ($13) {
      $$0$lcssa = $9;$$01$lcssa = $10;
      break;
     } else {
      $$0115 = $10;$$016 = $9;
     }
    }
   }
   $14 = HEAP32[$$0$lcssa>>2]|0;
   $15 = (($14) + -16843009)|0;
   $16 = $14 & -2139062144;
   $17 = $16 ^ -2139062144;
   $18 = $17 & $15;
   $19 = ($18|0)==(0);
   if ($19) {
    $22 = $14;$wd$010 = $$01$lcssa;$ws$011 = $$0$lcssa;
    while(1) {
     $20 = ((($ws$011)) + 4|0);
     $21 = ((($wd$010)) + 4|0);
     HEAP32[$wd$010>>2] = $22;
     $23 = HEAP32[$20>>2]|0;
     $24 = (($23) + -16843009)|0;
     $25 = $23 & -2139062144;
     $26 = $25 ^ -2139062144;
     $27 = $26 & $24;
     $28 = ($27|0)==(0);
     if ($28) {
      $22 = $23;$wd$010 = $21;$ws$011 = $20;
     } else {
      $wd$0$lcssa = $21;$ws$0$lcssa = $20;
      break;
     }
    }
   } else {
    $wd$0$lcssa = $$01$lcssa;$ws$0$lcssa = $$0$lcssa;
   }
   $$1$ph = $ws$0$lcssa;$$12$ph = $wd$0$lcssa;
   label = 8;
  } else {
   $$1$ph = $s;$$12$ph = $d;
   label = 8;
  }
 } while(0);
 if ((label|0) == 8) {
  $29 = HEAP8[$$1$ph>>0]|0;
  HEAP8[$$12$ph>>0] = $29;
  $30 = ($29<<24>>24)==(0);
  if ($30) {
   $$03 = $$12$ph;
  } else {
   $$128 = $$12$ph;$$19 = $$1$ph;
   while(1) {
    $31 = ((($$19)) + 1|0);
    $32 = ((($$128)) + 1|0);
    $33 = HEAP8[$31>>0]|0;
    HEAP8[$32>>0] = $33;
    $34 = ($33<<24>>24)==(0);
    if ($34) {
     $$03 = $32;
     break;
    } else {
     $$128 = $32;$$19 = $31;
    }
   }
  }
 }
 return ($$03|0);
}
function ___fflush_unlocked($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 20|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($f)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)>($3>>>0);
 if ($4) {
  $5 = ((($f)) + 36|0);
  $6 = HEAP32[$5>>2]|0;
  (FUNCTION_TABLE_iiii[$6 & 15]($f,0,0)|0);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)==(0|0);
  if ($8) {
   $$0 = -1;
  } else {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $9 = ((($f)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($f)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10>>>0)<($12>>>0);
  if ($13) {
   $14 = ((($f)) + 40|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $10;
   $17 = $12;
   $18 = (($16) - ($17))|0;
   (FUNCTION_TABLE_iiii[$15 & 15]($f,$18,1)|0);
  }
  $19 = ((($f)) + 16|0);
  HEAP32[$19>>2] = 0;
  HEAP32[$2>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$11>>2] = 0;
  HEAP32[$9>>2] = 0;
  $$0 = 0;
 }
 return ($$0|0);
}
function _cleanup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 68|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  ___unlockfile($p);
 }
 return;
}
function _cleanup52($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 68|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  ___unlockfile($p);
 }
 return;
}
function _printf_core($f,$fmt,$ap,$nl_arg,$nl_type) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 $nl_arg = $nl_arg|0;
 $nl_type = $nl_type|0;
 var $$ = 0, $$$i = 0, $$0 = 0, $$0$i = 0, $$0$lcssa$i = 0, $$012$i = 0, $$013$i = 0, $$03$i33 = 0, $$07$i = 0.0, $$1$i = 0.0, $$114$i = 0, $$2$i = 0.0, $$20$i = 0.0, $$21$i = 0, $$210$$22$i = 0, $$210$$24$i = 0, $$210$i = 0, $$23$i = 0, $$3$i = 0.0, $$31$i = 0;
 var $$311$i = 0, $$4$i = 0.0, $$412$lcssa$i = 0, $$41276$i = 0, $$43 = 0, $$5$lcssa$i = 0, $$587$i = 0, $$a$3$i = 0, $$a$3185$i = 0, $$a$3186$i = 0, $$fl$4 = 0, $$l10n$0 = 0, $$lcssa159$i = 0, $$lcssa321 = 0, $$lcssa322 = 0, $$lcssa326 = 0, $$lcssa328 = 0, $$lcssa329 = 0, $$lcssa330 = 0, $$lcssa331 = 0;
 var $$lcssa332 = 0, $$lcssa334 = 0, $$lcssa344 = 0, $$lcssa347 = 0.0, $$lcssa349 = 0, $$lcssa52 = 0, $$neg52$i = 0, $$neg53$i = 0, $$p$$i = 0, $$p$0 = 0, $$p$5 = 0, $$p$i = 0, $$pn$i = 0, $$pr$i = 0, $$pr47$i = 0, $$pre = 0, $$pre$i = 0, $$pre$phi184$iZ2D = 0, $$pre179$i = 0, $$pre182$i = 0;
 var $$pre183$i = 0, $$pre190 = 0, $$sum$i = 0, $$sum15$i = 0, $$sum16$i = 0, $$z$3$i = 0, $$z$4$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0;
 var $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0;
 var $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0;
 var $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0;
 var $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0;
 var $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0;
 var $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0;
 var $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0;
 var $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0;
 var $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0;
 var $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0;
 var $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0;
 var $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0;
 var $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0;
 var $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0;
 var $362 = 0, $363 = 0, $364 = 0.0, $365 = 0, $366 = 0, $367 = 0, $368 = 0.0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0;
 var $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0.0, $397 = 0.0, $398 = 0;
 var $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0.0, $413 = 0, $414 = 0, $415 = 0;
 var $416 = 0.0, $417 = 0.0, $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0.0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0;
 var $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0.0, $448 = 0.0, $449 = 0.0, $45 = 0, $450 = 0, $451 = 0;
 var $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0;
 var $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0.0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0;
 var $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0.0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0;
 var $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0;
 var $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0;
 var $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0;
 var $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0;
 var $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0;
 var $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0.0, $602 = 0.0, $603 = 0, $604 = 0.0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0;
 var $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0;
 var $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0;
 var $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0;
 var $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0;
 var $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0;
 var $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0;
 var $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0;
 var $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0;
 var $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0;
 var $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0;
 var $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a$0 = 0, $a$1 = 0, $a$1$lcssa$i = 0, $a$1147$i = 0, $a$2 = 0, $a$2$ph$i = 0, $a$3$lcssa$i = 0, $a$3134$i = 0, $a$5$lcssa$i = 0, $a$5109$i = 0;
 var $a$6$i = 0, $a$7$i = 0, $a$8$ph$i = 0, $arg = 0, $arglist_current = 0, $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $argpos$0 = 0, $big$i = 0, $buf = 0, $buf$i = 0, $carry$0140$i = 0, $carry3$0128$i = 0, $cnt$0 = 0, $cnt$1 = 0, $cnt$1$lcssa = 0, $d$0$i = 0, $d$0139$i = 0, $d$0141$i = 0;
 var $d$1127$i = 0, $d$2$lcssa$i = 0, $d$2108$i = 0, $d$3$i = 0, $d$482$i = 0, $d$575$i = 0, $d$686$i = 0, $e$0123$i = 0, $e$1$i = 0, $e$2104$i = 0, $e$3$i = 0, $e$4$ph$i = 0, $e2$i = 0, $ebuf0$i = 0, $estr$0$i = 0, $estr$1$lcssa$i = 0, $estr$193$i = 0, $estr$2$i = 0, $exitcond$i = 0, $expanded = 0;
 var $expanded10 = 0, $expanded11 = 0, $expanded13 = 0, $expanded14 = 0, $expanded15 = 0, $expanded4 = 0, $expanded6 = 0, $expanded7 = 0, $expanded8 = 0, $fl$0103 = 0, $fl$056 = 0, $fl$1 = 0, $fl$1$ = 0, $fl$3 = 0, $fl$4 = 0, $fl$6 = 0, $i$0$lcssa = 0, $i$0$lcssa197 = 0, $i$0108 = 0, $i$0122$i = 0;
 var $i$03$i = 0, $i$03$i25 = 0, $i$1$lcssa$i = 0, $i$1116$i = 0, $i$1119 = 0, $i$2103$i = 0, $i$295 = 0, $i$295$lcssa = 0, $i$393 = 0, $i$399$i = 0, $isdigit = 0, $isdigit$i = 0, $isdigit$i27 = 0, $isdigit10 = 0, $isdigit12 = 0, $isdigit2$i = 0, $isdigit2$i23 = 0, $isdigittmp = 0, $isdigittmp$ = 0, $isdigittmp$i = 0;
 var $isdigittmp$i26 = 0, $isdigittmp1$i = 0, $isdigittmp1$i22 = 0, $isdigittmp11 = 0, $isdigittmp4$i = 0, $isdigittmp4$i24 = 0, $isdigittmp9 = 0, $j$0$i = 0, $j$0115$i = 0, $j$0117$i = 0, $j$1100$i = 0, $j$2$i = 0, $l$0 = 0, $l$0$i = 0, $l$1$i = 0, $l$1107 = 0, $l$2 = 0, $l10n$0 = 0, $l10n$0$lcssa = 0, $l10n$0$phi = 0;
 var $l10n$1 = 0, $l10n$2 = 0, $l10n$3 = 0, $mb = 0, $notlhs$i = 0, $notrhs$i = 0, $or$cond = 0, $or$cond$i = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond20 = 0, $or$cond239 = 0, $or$cond29$i = 0, $or$cond3$not$i = 0, $or$cond6$i = 0, $p$0 = 0, $p$1 = 0, $p$2 = 0, $p$2$ = 0, $p$3 = 0;
 var $p$4195 = 0, $p$5 = 0, $pl$0 = 0, $pl$0$i = 0, $pl$1 = 0, $pl$1$i = 0, $pl$2 = 0, $prefix$0 = 0, $prefix$0$$i = 0, $prefix$0$i = 0, $prefix$1 = 0, $prefix$2 = 0, $r$0$a$8$i = 0, $re$169$i = 0, $round$068$i = 0.0, $round6$1$i = 0.0, $s$0$i = 0, $s$1$i = 0, $s$1$i$lcssa = 0, $s1$0$i = 0;
 var $s7$079$i = 0, $s7$1$i = 0, $s8$0$lcssa$i = 0, $s8$070$i = 0, $s9$0$i = 0, $s9$183$i = 0, $s9$2$i = 0, $small$0$i = 0.0, $small$1$i = 0.0, $st$0 = 0, $st$0$lcssa327 = 0, $storemerge = 0, $storemerge13 = 0, $storemerge8102 = 0, $storemerge854 = 0, $sum = 0, $t$0 = 0, $t$1 = 0, $w$$i = 0, $w$0 = 0;
 var $w$1 = 0, $w$2 = 0, $w$30$i = 0, $wc = 0, $ws$0109 = 0, $ws$1120 = 0, $z$0$i = 0, $z$0$lcssa = 0, $z$096 = 0, $z$1 = 0, $z$1$lcssa$i = 0, $z$1146$i = 0, $z$2 = 0, $z$2$i = 0, $z$2$i$lcssa = 0, $z$3$lcssa$i = 0, $z$3133$i = 0, $z$4$i = 0, $z$6$$i = 0, $z$6$i = 0;
 var $z$6$i$lcssa = 0, $z$6$ph$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $big$i = sp + 24|0;
 $e2$i = sp + 16|0;
 $buf$i = sp + 588|0;
 $ebuf0$i = sp + 576|0;
 $arg = sp;
 $buf = sp + 536|0;
 $wc = sp + 8|0;
 $mb = sp + 528|0;
 $0 = ($f|0)!=(0|0);
 $1 = ((($buf)) + 40|0);
 $2 = $1;
 $3 = ((($buf)) + 39|0);
 $4 = ((($wc)) + 4|0);
 $5 = ((($ebuf0$i)) + 12|0);
 $6 = ((($ebuf0$i)) + 11|0);
 $7 = $buf$i;
 $8 = $5;
 $9 = (($8) - ($7))|0;
 $10 = (-2 - ($7))|0;
 $11 = (($8) + 2)|0;
 $12 = ((($big$i)) + 288|0);
 $13 = ((($buf$i)) + 9|0);
 $14 = $13;
 $15 = ((($buf$i)) + 8|0);
 $22 = $fmt;$cnt$0 = 0;$l$0 = 0;$l10n$0 = 0;
 L1: while(1) {
  $16 = ($cnt$0|0)>(-1);
  do {
   if ($16) {
    $17 = (2147483647 - ($cnt$0))|0;
    $18 = ($l$0|0)>($17|0);
    if ($18) {
     $19 = (___errno_location()|0);
     HEAP32[$19>>2] = 75;
     $cnt$1 = -1;
     break;
    } else {
     $20 = (($l$0) + ($cnt$0))|0;
     $cnt$1 = $20;
     break;
    }
   } else {
    $cnt$1 = $cnt$0;
   }
  } while(0);
  $21 = HEAP8[$22>>0]|0;
  $23 = ($21<<24>>24)==(0);
  if ($23) {
   $cnt$1$lcssa = $cnt$1;$l10n$0$lcssa = $l10n$0;
   label = 245;
   break;
  } else {
   $24 = $21;$26 = $22;
  }
  L9: while(1) {
   switch ($24<<24>>24) {
   case 37:  {
    $28 = $26;$z$096 = $26;
    label = 9;
    break L9;
    break;
   }
   case 0:  {
    $$lcssa52 = $26;$z$0$lcssa = $26;
    break L9;
    break;
   }
   default: {
   }
   }
   $25 = ((($26)) + 1|0);
   $$pre = HEAP8[$25>>0]|0;
   $24 = $$pre;$26 = $25;
  }
  L12: do {
   if ((label|0) == 9) {
    while(1) {
     label = 0;
     $27 = ((($28)) + 1|0);
     $29 = HEAP8[$27>>0]|0;
     $30 = ($29<<24>>24)==(37);
     if (!($30)) {
      $$lcssa52 = $28;$z$0$lcssa = $z$096;
      break L12;
     }
     $31 = ((($z$096)) + 1|0);
     $32 = ((($28)) + 2|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = ($33<<24>>24)==(37);
     if ($34) {
      $28 = $32;$z$096 = $31;
      label = 9;
     } else {
      $$lcssa52 = $32;$z$0$lcssa = $31;
      break;
     }
    }
   }
  } while(0);
  $35 = $z$0$lcssa;
  $36 = $22;
  $37 = (($35) - ($36))|0;
  if ($0) {
   $38 = HEAP32[$f>>2]|0;
   $39 = $38 & 32;
   $40 = ($39|0)==(0);
   if ($40) {
    (___fwritex($22,$37,$f)|0);
   }
  }
  $41 = ($z$0$lcssa|0)==($22|0);
  if (!($41)) {
   $l10n$0$phi = $l10n$0;$22 = $$lcssa52;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$0$phi;
   continue;
  }
  $42 = ((($$lcssa52)) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43 << 24 >> 24;
  $isdigittmp = (($44) + -48)|0;
  $isdigit = ($isdigittmp>>>0)<(10);
  if ($isdigit) {
   $45 = ((($$lcssa52)) + 2|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = ($46<<24>>24)==(36);
   $48 = ((($$lcssa52)) + 3|0);
   $$43 = $47 ? $48 : $42;
   $$l10n$0 = $47 ? 1 : $l10n$0;
   $isdigittmp$ = $47 ? $isdigittmp : -1;
   $$pre190 = HEAP8[$$43>>0]|0;
   $50 = $$pre190;$argpos$0 = $isdigittmp$;$l10n$1 = $$l10n$0;$storemerge = $$43;
  } else {
   $50 = $43;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $42;
  }
  $49 = $50 << 24 >> 24;
  $51 = $49 & -32;
  $52 = ($51|0)==(32);
  L25: do {
   if ($52) {
    $54 = $49;$59 = $50;$fl$0103 = 0;$storemerge8102 = $storemerge;
    while(1) {
     $53 = (($54) + -32)|0;
     $55 = 1 << $53;
     $56 = $55 & 75913;
     $57 = ($56|0)==(0);
     if ($57) {
      $68 = $59;$fl$056 = $fl$0103;$storemerge854 = $storemerge8102;
      break L25;
     }
     $58 = $59 << 24 >> 24;
     $60 = (($58) + -32)|0;
     $61 = 1 << $60;
     $62 = $61 | $fl$0103;
     $63 = ((($storemerge8102)) + 1|0);
     $64 = HEAP8[$63>>0]|0;
     $65 = $64 << 24 >> 24;
     $66 = $65 & -32;
     $67 = ($66|0)==(32);
     if ($67) {
      $54 = $65;$59 = $64;$fl$0103 = $62;$storemerge8102 = $63;
     } else {
      $68 = $64;$fl$056 = $62;$storemerge854 = $63;
      break;
     }
    }
   } else {
    $68 = $50;$fl$056 = 0;$storemerge854 = $storemerge;
   }
  } while(0);
  $69 = ($68<<24>>24)==(42);
  do {
   if ($69) {
    $70 = ((($storemerge854)) + 1|0);
    $71 = HEAP8[$70>>0]|0;
    $72 = $71 << 24 >> 24;
    $isdigittmp11 = (($72) + -48)|0;
    $isdigit12 = ($isdigittmp11>>>0)<(10);
    if ($isdigit12) {
     $73 = ((($storemerge854)) + 2|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = ($74<<24>>24)==(36);
     if ($75) {
      $76 = (($nl_type) + ($isdigittmp11<<2)|0);
      HEAP32[$76>>2] = 10;
      $77 = HEAP8[$70>>0]|0;
      $78 = $77 << 24 >> 24;
      $79 = (($78) + -48)|0;
      $80 = (($nl_arg) + ($79<<3)|0);
      $81 = $80;
      $82 = $81;
      $83 = HEAP32[$82>>2]|0;
      $84 = (($81) + 4)|0;
      $85 = $84;
      $86 = HEAP32[$85>>2]|0;
      $87 = ((($storemerge854)) + 3|0);
      $l10n$2 = 1;$storemerge13 = $87;$w$0 = $83;
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     label = 0;
     $88 = ($l10n$1|0)==(0);
     if (!($88)) {
      $$0 = -1;
      break L1;
     }
     if (!($0)) {
      $108 = $70;$fl$1 = $fl$056;$l10n$3 = 0;$w$1 = 0;
      break;
     }
     $arglist_current = HEAP32[$ap>>2]|0;
     $89 = $arglist_current;
     $90 = ((0) + 4|0);
     $expanded4 = $90;
     $expanded = (($expanded4) - 1)|0;
     $91 = (($89) + ($expanded))|0;
     $92 = ((0) + 4|0);
     $expanded8 = $92;
     $expanded7 = (($expanded8) - 1)|0;
     $expanded6 = $expanded7 ^ -1;
     $93 = $91 & $expanded6;
     $94 = $93;
     $95 = HEAP32[$94>>2]|0;
     $arglist_next = ((($94)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     $l10n$2 = 0;$storemerge13 = $70;$w$0 = $95;
    }
    $96 = ($w$0|0)<(0);
    if ($96) {
     $97 = $fl$056 | 8192;
     $98 = (0 - ($w$0))|0;
     $108 = $storemerge13;$fl$1 = $97;$l10n$3 = $l10n$2;$w$1 = $98;
    } else {
     $108 = $storemerge13;$fl$1 = $fl$056;$l10n$3 = $l10n$2;$w$1 = $w$0;
    }
   } else {
    $99 = $68 << 24 >> 24;
    $isdigittmp1$i = (($99) + -48)|0;
    $isdigit2$i = ($isdigittmp1$i>>>0)<(10);
    if ($isdigit2$i) {
     $103 = $storemerge854;$i$03$i = 0;$isdigittmp4$i = $isdigittmp1$i;
     while(1) {
      $100 = ($i$03$i*10)|0;
      $101 = (($100) + ($isdigittmp4$i))|0;
      $102 = ((($103)) + 1|0);
      $104 = HEAP8[$102>>0]|0;
      $105 = $104 << 24 >> 24;
      $isdigittmp$i = (($105) + -48)|0;
      $isdigit$i = ($isdigittmp$i>>>0)<(10);
      if ($isdigit$i) {
       $103 = $102;$i$03$i = $101;$isdigittmp4$i = $isdigittmp$i;
      } else {
       $$lcssa321 = $101;$$lcssa322 = $102;
       break;
      }
     }
     $106 = ($$lcssa321|0)<(0);
     if ($106) {
      $$0 = -1;
      break L1;
     } else {
      $108 = $$lcssa322;$fl$1 = $fl$056;$l10n$3 = $l10n$1;$w$1 = $$lcssa321;
     }
    } else {
     $108 = $storemerge854;$fl$1 = $fl$056;$l10n$3 = $l10n$1;$w$1 = 0;
    }
   }
  } while(0);
  $107 = HEAP8[$108>>0]|0;
  $109 = ($107<<24>>24)==(46);
  L46: do {
   if ($109) {
    $110 = ((($108)) + 1|0);
    $111 = HEAP8[$110>>0]|0;
    $112 = ($111<<24>>24)==(42);
    if (!($112)) {
     $139 = $111 << 24 >> 24;
     $isdigittmp1$i22 = (($139) + -48)|0;
     $isdigit2$i23 = ($isdigittmp1$i22>>>0)<(10);
     if ($isdigit2$i23) {
      $143 = $110;$i$03$i25 = 0;$isdigittmp4$i24 = $isdigittmp1$i22;
     } else {
      $802 = $110;$p$0 = 0;
      break;
     }
     while(1) {
      $140 = ($i$03$i25*10)|0;
      $141 = (($140) + ($isdigittmp4$i24))|0;
      $142 = ((($143)) + 1|0);
      $144 = HEAP8[$142>>0]|0;
      $145 = $144 << 24 >> 24;
      $isdigittmp$i26 = (($145) + -48)|0;
      $isdigit$i27 = ($isdigittmp$i26>>>0)<(10);
      if ($isdigit$i27) {
       $143 = $142;$i$03$i25 = $141;$isdigittmp4$i24 = $isdigittmp$i26;
      } else {
       $802 = $142;$p$0 = $141;
       break L46;
      }
     }
    }
    $113 = ((($108)) + 2|0);
    $114 = HEAP8[$113>>0]|0;
    $115 = $114 << 24 >> 24;
    $isdigittmp9 = (($115) + -48)|0;
    $isdigit10 = ($isdigittmp9>>>0)<(10);
    if ($isdigit10) {
     $116 = ((($108)) + 3|0);
     $117 = HEAP8[$116>>0]|0;
     $118 = ($117<<24>>24)==(36);
     if ($118) {
      $119 = (($nl_type) + ($isdigittmp9<<2)|0);
      HEAP32[$119>>2] = 10;
      $120 = HEAP8[$113>>0]|0;
      $121 = $120 << 24 >> 24;
      $122 = (($121) + -48)|0;
      $123 = (($nl_arg) + ($122<<3)|0);
      $124 = $123;
      $125 = $124;
      $126 = HEAP32[$125>>2]|0;
      $127 = (($124) + 4)|0;
      $128 = $127;
      $129 = HEAP32[$128>>2]|0;
      $130 = ((($108)) + 4|0);
      $802 = $130;$p$0 = $126;
      break;
     }
    }
    $131 = ($l10n$3|0)==(0);
    if (!($131)) {
     $$0 = -1;
     break L1;
    }
    if ($0) {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $132 = $arglist_current2;
     $133 = ((0) + 4|0);
     $expanded11 = $133;
     $expanded10 = (($expanded11) - 1)|0;
     $134 = (($132) + ($expanded10))|0;
     $135 = ((0) + 4|0);
     $expanded15 = $135;
     $expanded14 = (($expanded15) - 1)|0;
     $expanded13 = $expanded14 ^ -1;
     $136 = $134 & $expanded13;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $arglist_next3 = ((($137)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $802 = $113;$p$0 = $138;
    } else {
     $802 = $113;$p$0 = 0;
    }
   } else {
    $802 = $108;$p$0 = -1;
   }
  } while(0);
  $147 = $802;$st$0 = 0;
  while(1) {
   $146 = HEAP8[$147>>0]|0;
   $148 = $146 << 24 >> 24;
   $149 = (($148) + -65)|0;
   $150 = ($149>>>0)>(57);
   if ($150) {
    $$0 = -1;
    break L1;
   }
   $151 = ((($147)) + 1|0);
   $152 = ((24563 + (($st$0*58)|0)|0) + ($149)|0);
   $153 = HEAP8[$152>>0]|0;
   $154 = $153&255;
   $155 = (($154) + -1)|0;
   $156 = ($155>>>0)<(8);
   if ($156) {
    $147 = $151;$st$0 = $154;
   } else {
    $$lcssa326 = $147;$$lcssa328 = $151;$$lcssa329 = $153;$$lcssa330 = $154;$st$0$lcssa327 = $st$0;
    break;
   }
  }
  $157 = ($$lcssa329<<24>>24)==(0);
  if ($157) {
   $$0 = -1;
   break;
  }
  $158 = ($$lcssa329<<24>>24)==(19);
  $159 = ($argpos$0|0)>(-1);
  do {
   if ($158) {
    if ($159) {
     $$0 = -1;
     break L1;
    } else {
     label = 52;
    }
   } else {
    if ($159) {
     $160 = (($nl_type) + ($argpos$0<<2)|0);
     HEAP32[$160>>2] = $$lcssa330;
     $161 = (($nl_arg) + ($argpos$0<<3)|0);
     $162 = $161;
     $163 = $162;
     $164 = HEAP32[$163>>2]|0;
     $165 = (($162) + 4)|0;
     $166 = $165;
     $167 = HEAP32[$166>>2]|0;
     $168 = $arg;
     $169 = $168;
     HEAP32[$169>>2] = $164;
     $170 = (($168) + 4)|0;
     $171 = $170;
     HEAP32[$171>>2] = $167;
     label = 52;
     break;
    }
    if (!($0)) {
     $$0 = 0;
     break L1;
    }
    _pop_arg80($arg,$$lcssa330,$ap);
   }
  } while(0);
  if ((label|0) == 52) {
   label = 0;
   if (!($0)) {
    $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
    continue;
   }
  }
  $172 = HEAP8[$$lcssa326>>0]|0;
  $173 = $172 << 24 >> 24;
  $174 = ($st$0$lcssa327|0)!=(0);
  $175 = $173 & 15;
  $176 = ($175|0)==(3);
  $or$cond15 = $174 & $176;
  $177 = $173 & -33;
  $t$0 = $or$cond15 ? $177 : $173;
  $178 = $fl$1 & 8192;
  $179 = ($178|0)==(0);
  $180 = $fl$1 & -65537;
  $fl$1$ = $179 ? $fl$1 : $180;
  L75: do {
   switch ($t$0|0) {
   case 110:  {
    switch ($st$0$lcssa327|0) {
    case 0:  {
     $187 = HEAP32[$arg>>2]|0;
     HEAP32[$187>>2] = $cnt$1;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 1:  {
     $188 = HEAP32[$arg>>2]|0;
     HEAP32[$188>>2] = $cnt$1;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 2:  {
     $189 = ($cnt$1|0)<(0);
     $190 = $189 << 31 >> 31;
     $191 = HEAP32[$arg>>2]|0;
     $192 = $191;
     $193 = $192;
     HEAP32[$193>>2] = $cnt$1;
     $194 = (($192) + 4)|0;
     $195 = $194;
     HEAP32[$195>>2] = $190;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 3:  {
     $196 = $cnt$1&65535;
     $197 = HEAP32[$arg>>2]|0;
     HEAP16[$197>>1] = $196;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 4:  {
     $198 = $cnt$1&255;
     $199 = HEAP32[$arg>>2]|0;
     HEAP8[$199>>0] = $198;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 6:  {
     $200 = HEAP32[$arg>>2]|0;
     HEAP32[$200>>2] = $cnt$1;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 7:  {
     $201 = ($cnt$1|0)<(0);
     $202 = $201 << 31 >> 31;
     $203 = HEAP32[$arg>>2]|0;
     $204 = $203;
     $205 = $204;
     HEAP32[$205>>2] = $cnt$1;
     $206 = (($204) + 4)|0;
     $207 = $206;
     HEAP32[$207>>2] = $202;
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    default: {
     $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $37;$l10n$0 = $l10n$3;
     continue L1;
    }
    }
    break;
   }
   case 112:  {
    $208 = ($p$0>>>0)>(8);
    $209 = $208 ? $p$0 : 8;
    $210 = $fl$1$ | 8;
    $fl$3 = $210;$p$1 = $209;$t$1 = 120;
    label = 64;
    break;
   }
   case 88: case 120:  {
    $fl$3 = $fl$1$;$p$1 = $p$0;$t$1 = $t$0;
    label = 64;
    break;
   }
   case 111:  {
    $248 = $arg;
    $249 = $248;
    $250 = HEAP32[$249>>2]|0;
    $251 = (($248) + 4)|0;
    $252 = $251;
    $253 = HEAP32[$252>>2]|0;
    $254 = ($250|0)==(0);
    $255 = ($253|0)==(0);
    $256 = $254 & $255;
    if ($256) {
     $$0$lcssa$i = $1;
    } else {
     $$03$i33 = $1;$258 = $250;$262 = $253;
     while(1) {
      $257 = $258 & 7;
      $259 = $257 | 48;
      $260 = $259&255;
      $261 = ((($$03$i33)) + -1|0);
      HEAP8[$261>>0] = $260;
      $263 = (_bitshift64Lshr(($258|0),($262|0),3)|0);
      $264 = tempRet0;
      $265 = ($263|0)==(0);
      $266 = ($264|0)==(0);
      $267 = $265 & $266;
      if ($267) {
       $$0$lcssa$i = $261;
       break;
      } else {
       $$03$i33 = $261;$258 = $263;$262 = $264;
      }
     }
    }
    $268 = $fl$1$ & 8;
    $269 = ($268|0)==(0);
    if ($269) {
     $a$0 = $$0$lcssa$i;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = 0;$prefix$1 = 25043;
     label = 77;
    } else {
     $270 = $$0$lcssa$i;
     $271 = (($2) - ($270))|0;
     $272 = (($271) + 1)|0;
     $273 = ($p$0|0)<($272|0);
     $$p$0 = $273 ? $272 : $p$0;
     $a$0 = $$0$lcssa$i;$fl$4 = $fl$1$;$p$2 = $$p$0;$pl$1 = 0;$prefix$1 = 25043;
     label = 77;
    }
    break;
   }
   case 105: case 100:  {
    $274 = $arg;
    $275 = $274;
    $276 = HEAP32[$275>>2]|0;
    $277 = (($274) + 4)|0;
    $278 = $277;
    $279 = HEAP32[$278>>2]|0;
    $280 = ($279|0)<(0);
    if ($280) {
     $281 = (_i64Subtract(0,0,($276|0),($279|0))|0);
     $282 = tempRet0;
     $283 = $arg;
     $284 = $283;
     HEAP32[$284>>2] = $281;
     $285 = (($283) + 4)|0;
     $286 = $285;
     HEAP32[$286>>2] = $282;
     $291 = $281;$292 = $282;$pl$0 = 1;$prefix$0 = 25043;
     label = 76;
     break L75;
    }
    $287 = $fl$1$ & 2048;
    $288 = ($287|0)==(0);
    if ($288) {
     $289 = $fl$1$ & 1;
     $290 = ($289|0)==(0);
     $$ = $290 ? 25043 : (25045);
     $291 = $276;$292 = $279;$pl$0 = $289;$prefix$0 = $$;
     label = 76;
    } else {
     $291 = $276;$292 = $279;$pl$0 = 1;$prefix$0 = (25044);
     label = 76;
    }
    break;
   }
   case 117:  {
    $181 = $arg;
    $182 = $181;
    $183 = HEAP32[$182>>2]|0;
    $184 = (($181) + 4)|0;
    $185 = $184;
    $186 = HEAP32[$185>>2]|0;
    $291 = $183;$292 = $186;$pl$0 = 0;$prefix$0 = 25043;
    label = 76;
    break;
   }
   case 99:  {
    $312 = $arg;
    $313 = $312;
    $314 = HEAP32[$313>>2]|0;
    $315 = (($312) + 4)|0;
    $316 = $315;
    $317 = HEAP32[$316>>2]|0;
    $318 = $314&255;
    HEAP8[$3>>0] = $318;
    $a$2 = $3;$fl$6 = $180;$p$5 = 1;$pl$2 = 0;$prefix$2 = 25043;$z$2 = $1;
    break;
   }
   case 109:  {
    $319 = (___errno_location()|0);
    $320 = HEAP32[$319>>2]|0;
    $321 = (_strerror($320)|0);
    $a$1 = $321;
    label = 82;
    break;
   }
   case 115:  {
    $322 = HEAP32[$arg>>2]|0;
    $323 = ($322|0)!=(0|0);
    $324 = $323 ? $322 : 25053;
    $a$1 = $324;
    label = 82;
    break;
   }
   case 67:  {
    $331 = $arg;
    $332 = $331;
    $333 = HEAP32[$332>>2]|0;
    $334 = (($331) + 4)|0;
    $335 = $334;
    $336 = HEAP32[$335>>2]|0;
    HEAP32[$wc>>2] = $333;
    HEAP32[$4>>2] = 0;
    HEAP32[$arg>>2] = $wc;
    $p$4195 = -1;
    label = 86;
    break;
   }
   case 83:  {
    $337 = ($p$0|0)==(0);
    if ($337) {
     _pad($f,32,$w$1,0,$fl$1$);
     $i$0$lcssa197 = 0;
     label = 98;
    } else {
     $p$4195 = $p$0;
     label = 86;
    }
    break;
   }
   case 65: case 71: case 70: case 69: case 97: case 103: case 102: case 101:  {
    $364 = +HEAPF64[$arg>>3];
    HEAP32[$e2$i>>2] = 0;
    HEAPF64[tempDoublePtr>>3] = $364;$365 = HEAP32[tempDoublePtr>>2]|0;
    $366 = HEAP32[tempDoublePtr+4>>2]|0;
    $367 = ($366|0)<(0);
    if ($367) {
     $368 = -$364;
     $$07$i = $368;$pl$0$i = 1;$prefix$0$i = 25060;
    } else {
     $369 = $fl$1$ & 2048;
     $370 = ($369|0)==(0);
     if ($370) {
      $371 = $fl$1$ & 1;
      $372 = ($371|0)==(0);
      $$$i = $372 ? (25061) : (25066);
      $$07$i = $364;$pl$0$i = $371;$prefix$0$i = $$$i;
     } else {
      $$07$i = $364;$pl$0$i = 1;$prefix$0$i = (25063);
     }
    }
    HEAPF64[tempDoublePtr>>3] = $$07$i;$373 = HEAP32[tempDoublePtr>>2]|0;
    $374 = HEAP32[tempDoublePtr+4>>2]|0;
    $375 = $374 & 2146435072;
    $376 = ($375>>>0)<(2146435072);
    $377 = (0)<(0);
    $378 = ($375|0)==(2146435072);
    $379 = $378 & $377;
    $380 = $376 | $379;
    do {
     if ($380) {
      $396 = (+_frexpl($$07$i,$e2$i));
      $397 = $396 * 2.0;
      $398 = $397 != 0.0;
      if ($398) {
       $399 = HEAP32[$e2$i>>2]|0;
       $400 = (($399) + -1)|0;
       HEAP32[$e2$i>>2] = $400;
      }
      $401 = $t$0 | 32;
      $402 = ($401|0)==(97);
      if ($402) {
       $403 = $t$0 & 32;
       $404 = ($403|0)==(0);
       $405 = ((($prefix$0$i)) + 9|0);
       $prefix$0$$i = $404 ? $prefix$0$i : $405;
       $406 = $pl$0$i | 2;
       $407 = ($p$0>>>0)>(11);
       $408 = (12 - ($p$0))|0;
       $409 = ($408|0)==(0);
       $410 = $407 | $409;
       do {
        if ($410) {
         $$1$i = $397;
        } else {
         $re$169$i = $408;$round$068$i = 8.0;
         while(1) {
          $411 = (($re$169$i) + -1)|0;
          $412 = $round$068$i * 16.0;
          $413 = ($411|0)==(0);
          if ($413) {
           $$lcssa347 = $412;
           break;
          } else {
           $re$169$i = $411;$round$068$i = $412;
          }
         }
         $414 = HEAP8[$prefix$0$$i>>0]|0;
         $415 = ($414<<24>>24)==(45);
         if ($415) {
          $416 = -$397;
          $417 = $416 - $$lcssa347;
          $418 = $$lcssa347 + $417;
          $419 = -$418;
          $$1$i = $419;
          break;
         } else {
          $420 = $397 + $$lcssa347;
          $421 = $420 - $$lcssa347;
          $$1$i = $421;
          break;
         }
        }
       } while(0);
       $422 = HEAP32[$e2$i>>2]|0;
       $423 = ($422|0)<(0);
       $424 = (0 - ($422))|0;
       $425 = $423 ? $424 : $422;
       $426 = ($425|0)<(0);
       $427 = $426 << 31 >> 31;
       $428 = (_fmt_u($425,$427,$5)|0);
       $429 = ($428|0)==($5|0);
       if ($429) {
        HEAP8[$6>>0] = 48;
        $estr$0$i = $6;
       } else {
        $estr$0$i = $428;
       }
       $430 = $422 >> 31;
       $431 = $430 & 2;
       $432 = (($431) + 43)|0;
       $433 = $432&255;
       $434 = ((($estr$0$i)) + -1|0);
       HEAP8[$434>>0] = $433;
       $435 = (($t$0) + 15)|0;
       $436 = $435&255;
       $437 = ((($estr$0$i)) + -2|0);
       HEAP8[$437>>0] = $436;
       $notrhs$i = ($p$0|0)<(1);
       $438 = $fl$1$ & 8;
       $439 = ($438|0)==(0);
       $$2$i = $$1$i;$s$0$i = $buf$i;
       while(1) {
        $440 = (~~(($$2$i)));
        $441 = (25027 + ($440)|0);
        $442 = HEAP8[$441>>0]|0;
        $443 = $442&255;
        $444 = $443 | $403;
        $445 = $444&255;
        $446 = ((($s$0$i)) + 1|0);
        HEAP8[$s$0$i>>0] = $445;
        $447 = (+($440|0));
        $448 = $$2$i - $447;
        $449 = $448 * 16.0;
        $450 = $446;
        $451 = (($450) - ($7))|0;
        $452 = ($451|0)==(1);
        do {
         if ($452) {
          $notlhs$i = $449 == 0.0;
          $or$cond3$not$i = $notrhs$i & $notlhs$i;
          $or$cond$i = $439 & $or$cond3$not$i;
          if ($or$cond$i) {
           $s$1$i = $446;
           break;
          }
          $453 = ((($s$0$i)) + 2|0);
          HEAP8[$446>>0] = 46;
          $s$1$i = $453;
         } else {
          $s$1$i = $446;
         }
        } while(0);
        $454 = $449 != 0.0;
        if ($454) {
         $$2$i = $449;$s$0$i = $s$1$i;
        } else {
         $s$1$i$lcssa = $s$1$i;
         break;
        }
       }
       $455 = ($p$0|0)!=(0);
       $$pre182$i = $s$1$i$lcssa;
       $456 = (($10) + ($$pre182$i))|0;
       $457 = ($456|0)<($p$0|0);
       $or$cond239 = $455 & $457;
       $458 = $437;
       $459 = (($11) + ($p$0))|0;
       $460 = (($459) - ($458))|0;
       $461 = $437;
       $462 = (($9) - ($461))|0;
       $463 = (($462) + ($$pre182$i))|0;
       $l$0$i = $or$cond239 ? $460 : $463;
       $464 = (($l$0$i) + ($406))|0;
       _pad($f,32,$w$1,$464,$fl$1$);
       $465 = HEAP32[$f>>2]|0;
       $466 = $465 & 32;
       $467 = ($466|0)==(0);
       if ($467) {
        (___fwritex($prefix$0$$i,$406,$f)|0);
       }
       $468 = $fl$1$ ^ 65536;
       _pad($f,48,$w$1,$464,$468);
       $469 = (($$pre182$i) - ($7))|0;
       $470 = HEAP32[$f>>2]|0;
       $471 = $470 & 32;
       $472 = ($471|0)==(0);
       if ($472) {
        (___fwritex($buf$i,$469,$f)|0);
       }
       $473 = $437;
       $474 = (($8) - ($473))|0;
       $sum = (($469) + ($474))|0;
       $475 = (($l$0$i) - ($sum))|0;
       _pad($f,48,$475,0,0);
       $476 = HEAP32[$f>>2]|0;
       $477 = $476 & 32;
       $478 = ($477|0)==(0);
       if ($478) {
        (___fwritex($437,$474,$f)|0);
       }
       $479 = $fl$1$ ^ 8192;
       _pad($f,32,$w$1,$464,$479);
       $480 = ($464|0)<($w$1|0);
       $w$$i = $480 ? $w$1 : $464;
       $$0$i = $w$$i;
       break;
      }
      $481 = ($p$0|0)<(0);
      $$p$i = $481 ? 6 : $p$0;
      if ($398) {
       $482 = $397 * 268435456.0;
       $483 = HEAP32[$e2$i>>2]|0;
       $484 = (($483) + -28)|0;
       HEAP32[$e2$i>>2] = $484;
       $$3$i = $482;$485 = $484;
      } else {
       $$pre179$i = HEAP32[$e2$i>>2]|0;
       $$3$i = $397;$485 = $$pre179$i;
      }
      $486 = ($485|0)<(0);
      $$31$i = $486 ? $big$i : $12;
      $487 = $$31$i;
      $$4$i = $$3$i;$z$0$i = $$31$i;
      while(1) {
       $488 = (~~(($$4$i))>>>0);
       HEAP32[$z$0$i>>2] = $488;
       $489 = ((($z$0$i)) + 4|0);
       $490 = (+($488>>>0));
       $491 = $$4$i - $490;
       $492 = $491 * 1.0E+9;
       $493 = $492 != 0.0;
       if ($493) {
        $$4$i = $492;$z$0$i = $489;
       } else {
        $$lcssa331 = $489;
        break;
       }
      }
      $$pr$i = HEAP32[$e2$i>>2]|0;
      $494 = ($$pr$i|0)>(0);
      if ($494) {
       $495 = $$pr$i;$a$1147$i = $$31$i;$z$1146$i = $$lcssa331;
       while(1) {
        $496 = ($495|0)>(29);
        $497 = $496 ? 29 : $495;
        $d$0139$i = ((($z$1146$i)) + -4|0);
        $498 = ($d$0139$i>>>0)<($a$1147$i>>>0);
        do {
         if ($498) {
          $a$2$ph$i = $a$1147$i;
         } else {
          $carry$0140$i = 0;$d$0141$i = $d$0139$i;
          while(1) {
           $499 = HEAP32[$d$0141$i>>2]|0;
           $500 = (_bitshift64Shl(($499|0),0,($497|0))|0);
           $501 = tempRet0;
           $502 = (_i64Add(($500|0),($501|0),($carry$0140$i|0),0)|0);
           $503 = tempRet0;
           $504 = (___uremdi3(($502|0),($503|0),1000000000,0)|0);
           $505 = tempRet0;
           HEAP32[$d$0141$i>>2] = $504;
           $506 = (___udivdi3(($502|0),($503|0),1000000000,0)|0);
           $507 = tempRet0;
           $d$0$i = ((($d$0141$i)) + -4|0);
           $508 = ($d$0$i>>>0)<($a$1147$i>>>0);
           if ($508) {
            $$lcssa332 = $506;
            break;
           } else {
            $carry$0140$i = $506;$d$0141$i = $d$0$i;
           }
          }
          $509 = ($$lcssa332|0)==(0);
          if ($509) {
           $a$2$ph$i = $a$1147$i;
           break;
          }
          $510 = ((($a$1147$i)) + -4|0);
          HEAP32[$510>>2] = $$lcssa332;
          $a$2$ph$i = $510;
         }
        } while(0);
        $z$2$i = $z$1146$i;
        while(1) {
         $511 = ($z$2$i>>>0)>($a$2$ph$i>>>0);
         if (!($511)) {
          $z$2$i$lcssa = $z$2$i;
          break;
         }
         $512 = ((($z$2$i)) + -4|0);
         $513 = HEAP32[$512>>2]|0;
         $514 = ($513|0)==(0);
         if ($514) {
          $z$2$i = $512;
         } else {
          $z$2$i$lcssa = $z$2$i;
          break;
         }
        }
        $515 = HEAP32[$e2$i>>2]|0;
        $516 = (($515) - ($497))|0;
        HEAP32[$e2$i>>2] = $516;
        $517 = ($516|0)>(0);
        if ($517) {
         $495 = $516;$a$1147$i = $a$2$ph$i;$z$1146$i = $z$2$i$lcssa;
        } else {
         $$pr47$i = $516;$a$1$lcssa$i = $a$2$ph$i;$z$1$lcssa$i = $z$2$i$lcssa;
         break;
        }
       }
      } else {
       $$pr47$i = $$pr$i;$a$1$lcssa$i = $$31$i;$z$1$lcssa$i = $$lcssa331;
      }
      $518 = ($$pr47$i|0)<(0);
      if ($518) {
       $519 = (($$p$i) + 25)|0;
       $520 = (($519|0) / 9)&-1;
       $521 = (($520) + 1)|0;
       $522 = ($401|0)==(102);
       $524 = $$pr47$i;$a$3134$i = $a$1$lcssa$i;$z$3133$i = $z$1$lcssa$i;
       while(1) {
        $523 = (0 - ($524))|0;
        $525 = ($523|0)>(9);
        $526 = $525 ? 9 : $523;
        $527 = ($a$3134$i>>>0)<($z$3133$i>>>0);
        do {
         if ($527) {
          $531 = 1 << $526;
          $532 = (($531) + -1)|0;
          $533 = 1000000000 >>> $526;
          $carry3$0128$i = 0;$d$1127$i = $a$3134$i;
          while(1) {
           $534 = HEAP32[$d$1127$i>>2]|0;
           $535 = $534 & $532;
           $536 = $534 >>> $526;
           $537 = (($536) + ($carry3$0128$i))|0;
           HEAP32[$d$1127$i>>2] = $537;
           $538 = Math_imul($535, $533)|0;
           $539 = ((($d$1127$i)) + 4|0);
           $540 = ($539>>>0)<($z$3133$i>>>0);
           if ($540) {
            $carry3$0128$i = $538;$d$1127$i = $539;
           } else {
            $$lcssa334 = $538;
            break;
           }
          }
          $541 = HEAP32[$a$3134$i>>2]|0;
          $542 = ($541|0)==(0);
          $543 = ((($a$3134$i)) + 4|0);
          $$a$3$i = $542 ? $543 : $a$3134$i;
          $544 = ($$lcssa334|0)==(0);
          if ($544) {
           $$a$3186$i = $$a$3$i;$z$4$i = $z$3133$i;
           break;
          }
          $545 = ((($z$3133$i)) + 4|0);
          HEAP32[$z$3133$i>>2] = $$lcssa334;
          $$a$3186$i = $$a$3$i;$z$4$i = $545;
         } else {
          $528 = HEAP32[$a$3134$i>>2]|0;
          $529 = ($528|0)==(0);
          $530 = ((($a$3134$i)) + 4|0);
          $$a$3185$i = $529 ? $530 : $a$3134$i;
          $$a$3186$i = $$a$3185$i;$z$4$i = $z$3133$i;
         }
        } while(0);
        $546 = $522 ? $$31$i : $$a$3186$i;
        $547 = $z$4$i;
        $548 = $546;
        $549 = (($547) - ($548))|0;
        $550 = $549 >> 2;
        $551 = ($550|0)>($521|0);
        $552 = (($546) + ($521<<2)|0);
        $$z$4$i = $551 ? $552 : $z$4$i;
        $553 = HEAP32[$e2$i>>2]|0;
        $554 = (($553) + ($526))|0;
        HEAP32[$e2$i>>2] = $554;
        $555 = ($554|0)<(0);
        if ($555) {
         $524 = $554;$a$3134$i = $$a$3186$i;$z$3133$i = $$z$4$i;
        } else {
         $a$3$lcssa$i = $$a$3186$i;$z$3$lcssa$i = $$z$4$i;
         break;
        }
       }
      } else {
       $a$3$lcssa$i = $a$1$lcssa$i;$z$3$lcssa$i = $z$1$lcssa$i;
      }
      $556 = ($a$3$lcssa$i>>>0)<($z$3$lcssa$i>>>0);
      do {
       if ($556) {
        $557 = $a$3$lcssa$i;
        $558 = (($487) - ($557))|0;
        $559 = $558 >> 2;
        $560 = ($559*9)|0;
        $561 = HEAP32[$a$3$lcssa$i>>2]|0;
        $562 = ($561>>>0)<(10);
        if ($562) {
         $e$1$i = $560;
         break;
        } else {
         $e$0123$i = $560;$i$0122$i = 10;
        }
        while(1) {
         $563 = ($i$0122$i*10)|0;
         $564 = (($e$0123$i) + 1)|0;
         $565 = ($561>>>0)<($563>>>0);
         if ($565) {
          $e$1$i = $564;
          break;
         } else {
          $e$0123$i = $564;$i$0122$i = $563;
         }
        }
       } else {
        $e$1$i = 0;
       }
      } while(0);
      $566 = ($401|0)!=(102);
      $567 = $566 ? $e$1$i : 0;
      $568 = (($$p$i) - ($567))|0;
      $569 = ($401|0)==(103);
      $570 = ($$p$i|0)!=(0);
      $571 = $570 & $569;
      $$neg52$i = $571 << 31 >> 31;
      $572 = (($568) + ($$neg52$i))|0;
      $573 = $z$3$lcssa$i;
      $574 = (($573) - ($487))|0;
      $575 = $574 >> 2;
      $576 = ($575*9)|0;
      $577 = (($576) + -9)|0;
      $578 = ($572|0)<($577|0);
      if ($578) {
       $579 = (($572) + 9216)|0;
       $580 = (($579|0) / 9)&-1;
       $$sum$i = (($580) + -1023)|0;
       $581 = (($$31$i) + ($$sum$i<<2)|0);
       $582 = (($579|0) % 9)&-1;
       $j$0115$i = (($582) + 1)|0;
       $583 = ($j$0115$i|0)<(9);
       if ($583) {
        $i$1116$i = 10;$j$0117$i = $j$0115$i;
        while(1) {
         $584 = ($i$1116$i*10)|0;
         $j$0$i = (($j$0117$i) + 1)|0;
         $exitcond$i = ($j$0$i|0)==(9);
         if ($exitcond$i) {
          $i$1$lcssa$i = $584;
          break;
         } else {
          $i$1116$i = $584;$j$0117$i = $j$0$i;
         }
        }
       } else {
        $i$1$lcssa$i = 10;
       }
       $585 = HEAP32[$581>>2]|0;
       $586 = (($585>>>0) % ($i$1$lcssa$i>>>0))&-1;
       $587 = ($586|0)==(0);
       if ($587) {
        $$sum15$i = (($580) + -1022)|0;
        $588 = (($$31$i) + ($$sum15$i<<2)|0);
        $589 = ($588|0)==($z$3$lcssa$i|0);
        if ($589) {
         $a$7$i = $a$3$lcssa$i;$d$3$i = $581;$e$3$i = $e$1$i;
        } else {
         label = 163;
        }
       } else {
        label = 163;
       }
       do {
        if ((label|0) == 163) {
         label = 0;
         $590 = (($585>>>0) / ($i$1$lcssa$i>>>0))&-1;
         $591 = $590 & 1;
         $592 = ($591|0)==(0);
         $$20$i = $592 ? 9007199254740992.0 : 9007199254740994.0;
         $593 = (($i$1$lcssa$i|0) / 2)&-1;
         $594 = ($586>>>0)<($593>>>0);
         do {
          if ($594) {
           $small$0$i = 0.5;
          } else {
           $595 = ($586|0)==($593|0);
           if ($595) {
            $$sum16$i = (($580) + -1022)|0;
            $596 = (($$31$i) + ($$sum16$i<<2)|0);
            $597 = ($596|0)==($z$3$lcssa$i|0);
            if ($597) {
             $small$0$i = 1.0;
             break;
            }
           }
           $small$0$i = 1.5;
          }
         } while(0);
         $598 = ($pl$0$i|0)==(0);
         do {
          if ($598) {
           $round6$1$i = $$20$i;$small$1$i = $small$0$i;
          } else {
           $599 = HEAP8[$prefix$0$i>>0]|0;
           $600 = ($599<<24>>24)==(45);
           if (!($600)) {
            $round6$1$i = $$20$i;$small$1$i = $small$0$i;
            break;
           }
           $601 = -$$20$i;
           $602 = -$small$0$i;
           $round6$1$i = $601;$small$1$i = $602;
          }
         } while(0);
         $603 = (($585) - ($586))|0;
         HEAP32[$581>>2] = $603;
         $604 = $round6$1$i + $small$1$i;
         $605 = $604 != $round6$1$i;
         if (!($605)) {
          $a$7$i = $a$3$lcssa$i;$d$3$i = $581;$e$3$i = $e$1$i;
          break;
         }
         $606 = (($603) + ($i$1$lcssa$i))|0;
         HEAP32[$581>>2] = $606;
         $607 = ($606>>>0)>(999999999);
         if ($607) {
          $a$5109$i = $a$3$lcssa$i;$d$2108$i = $581;
          while(1) {
           $608 = ((($d$2108$i)) + -4|0);
           HEAP32[$d$2108$i>>2] = 0;
           $609 = ($608>>>0)<($a$5109$i>>>0);
           if ($609) {
            $610 = ((($a$5109$i)) + -4|0);
            HEAP32[$610>>2] = 0;
            $a$6$i = $610;
           } else {
            $a$6$i = $a$5109$i;
           }
           $611 = HEAP32[$608>>2]|0;
           $612 = (($611) + 1)|0;
           HEAP32[$608>>2] = $612;
           $613 = ($612>>>0)>(999999999);
           if ($613) {
            $a$5109$i = $a$6$i;$d$2108$i = $608;
           } else {
            $a$5$lcssa$i = $a$6$i;$d$2$lcssa$i = $608;
            break;
           }
          }
         } else {
          $a$5$lcssa$i = $a$3$lcssa$i;$d$2$lcssa$i = $581;
         }
         $614 = $a$5$lcssa$i;
         $615 = (($487) - ($614))|0;
         $616 = $615 >> 2;
         $617 = ($616*9)|0;
         $618 = HEAP32[$a$5$lcssa$i>>2]|0;
         $619 = ($618>>>0)<(10);
         if ($619) {
          $a$7$i = $a$5$lcssa$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $617;
          break;
         } else {
          $e$2104$i = $617;$i$2103$i = 10;
         }
         while(1) {
          $620 = ($i$2103$i*10)|0;
          $621 = (($e$2104$i) + 1)|0;
          $622 = ($618>>>0)<($620>>>0);
          if ($622) {
           $a$7$i = $a$5$lcssa$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $621;
           break;
          } else {
           $e$2104$i = $621;$i$2103$i = $620;
          }
         }
        }
       } while(0);
       $623 = ((($d$3$i)) + 4|0);
       $624 = ($z$3$lcssa$i>>>0)>($623>>>0);
       $$z$3$i = $624 ? $623 : $z$3$lcssa$i;
       $a$8$ph$i = $a$7$i;$e$4$ph$i = $e$3$i;$z$6$ph$i = $$z$3$i;
      } else {
       $a$8$ph$i = $a$3$lcssa$i;$e$4$ph$i = $e$1$i;$z$6$ph$i = $z$3$lcssa$i;
      }
      $625 = (0 - ($e$4$ph$i))|0;
      $z$6$i = $z$6$ph$i;
      while(1) {
       $626 = ($z$6$i>>>0)>($a$8$ph$i>>>0);
       if (!($626)) {
        $$lcssa159$i = 0;$z$6$i$lcssa = $z$6$i;
        break;
       }
       $627 = ((($z$6$i)) + -4|0);
       $628 = HEAP32[$627>>2]|0;
       $629 = ($628|0)==(0);
       if ($629) {
        $z$6$i = $627;
       } else {
        $$lcssa159$i = 1;$z$6$i$lcssa = $z$6$i;
        break;
       }
      }
      do {
       if ($569) {
        $630 = $570&1;
        $631 = $630 ^ 1;
        $$p$$i = (($631) + ($$p$i))|0;
        $632 = ($$p$$i|0)>($e$4$ph$i|0);
        $633 = ($e$4$ph$i|0)>(-5);
        $or$cond6$i = $632 & $633;
        if ($or$cond6$i) {
         $634 = (($t$0) + -1)|0;
         $$neg53$i = (($$p$$i) + -1)|0;
         $635 = (($$neg53$i) - ($e$4$ph$i))|0;
         $$013$i = $634;$$210$i = $635;
        } else {
         $636 = (($t$0) + -2)|0;
         $637 = (($$p$$i) + -1)|0;
         $$013$i = $636;$$210$i = $637;
        }
        $638 = $fl$1$ & 8;
        $639 = ($638|0)==(0);
        if (!($639)) {
         $$114$i = $$013$i;$$311$i = $$210$i;$$pre$phi184$iZ2D = $638;
         break;
        }
        do {
         if ($$lcssa159$i) {
          $640 = ((($z$6$i$lcssa)) + -4|0);
          $641 = HEAP32[$640>>2]|0;
          $642 = ($641|0)==(0);
          if ($642) {
           $j$2$i = 9;
           break;
          }
          $643 = (($641>>>0) % 10)&-1;
          $644 = ($643|0)==(0);
          if ($644) {
           $i$399$i = 10;$j$1100$i = 0;
          } else {
           $j$2$i = 0;
           break;
          }
          while(1) {
           $645 = ($i$399$i*10)|0;
           $646 = (($j$1100$i) + 1)|0;
           $647 = (($641>>>0) % ($645>>>0))&-1;
           $648 = ($647|0)==(0);
           if ($648) {
            $i$399$i = $645;$j$1100$i = $646;
           } else {
            $j$2$i = $646;
            break;
           }
          }
         } else {
          $j$2$i = 9;
         }
        } while(0);
        $649 = $$013$i | 32;
        $650 = ($649|0)==(102);
        $651 = $z$6$i$lcssa;
        $652 = (($651) - ($487))|0;
        $653 = $652 >> 2;
        $654 = ($653*9)|0;
        $655 = (($654) + -9)|0;
        if ($650) {
         $656 = (($655) - ($j$2$i))|0;
         $657 = ($656|0)<(0);
         $$21$i = $657 ? 0 : $656;
         $658 = ($$210$i|0)<($$21$i|0);
         $$210$$22$i = $658 ? $$210$i : $$21$i;
         $$114$i = $$013$i;$$311$i = $$210$$22$i;$$pre$phi184$iZ2D = 0;
         break;
        } else {
         $659 = (($655) + ($e$4$ph$i))|0;
         $660 = (($659) - ($j$2$i))|0;
         $661 = ($660|0)<(0);
         $$23$i = $661 ? 0 : $660;
         $662 = ($$210$i|0)<($$23$i|0);
         $$210$$24$i = $662 ? $$210$i : $$23$i;
         $$114$i = $$013$i;$$311$i = $$210$$24$i;$$pre$phi184$iZ2D = 0;
         break;
        }
       } else {
        $$pre183$i = $fl$1$ & 8;
        $$114$i = $t$0;$$311$i = $$p$i;$$pre$phi184$iZ2D = $$pre183$i;
       }
      } while(0);
      $663 = $$311$i | $$pre$phi184$iZ2D;
      $664 = ($663|0)!=(0);
      $665 = $664&1;
      $666 = $$114$i | 32;
      $667 = ($666|0)==(102);
      if ($667) {
       $668 = ($e$4$ph$i|0)>(0);
       $669 = $668 ? $e$4$ph$i : 0;
       $$pn$i = $669;$estr$2$i = 0;
      } else {
       $670 = ($e$4$ph$i|0)<(0);
       $671 = $670 ? $625 : $e$4$ph$i;
       $672 = ($671|0)<(0);
       $673 = $672 << 31 >> 31;
       $674 = (_fmt_u($671,$673,$5)|0);
       $675 = $674;
       $676 = (($8) - ($675))|0;
       $677 = ($676|0)<(2);
       if ($677) {
        $estr$193$i = $674;
        while(1) {
         $678 = ((($estr$193$i)) + -1|0);
         HEAP8[$678>>0] = 48;
         $679 = $678;
         $680 = (($8) - ($679))|0;
         $681 = ($680|0)<(2);
         if ($681) {
          $estr$193$i = $678;
         } else {
          $estr$1$lcssa$i = $678;
          break;
         }
        }
       } else {
        $estr$1$lcssa$i = $674;
       }
       $682 = $e$4$ph$i >> 31;
       $683 = $682 & 2;
       $684 = (($683) + 43)|0;
       $685 = $684&255;
       $686 = ((($estr$1$lcssa$i)) + -1|0);
       HEAP8[$686>>0] = $685;
       $687 = $$114$i&255;
       $688 = ((($estr$1$lcssa$i)) + -2|0);
       HEAP8[$688>>0] = $687;
       $689 = $688;
       $690 = (($8) - ($689))|0;
       $$pn$i = $690;$estr$2$i = $688;
      }
      $691 = (($pl$0$i) + 1)|0;
      $692 = (($691) + ($$311$i))|0;
      $l$1$i = (($692) + ($665))|0;
      $693 = (($l$1$i) + ($$pn$i))|0;
      _pad($f,32,$w$1,$693,$fl$1$);
      $694 = HEAP32[$f>>2]|0;
      $695 = $694 & 32;
      $696 = ($695|0)==(0);
      if ($696) {
       (___fwritex($prefix$0$i,$pl$0$i,$f)|0);
      }
      $697 = $fl$1$ ^ 65536;
      _pad($f,48,$w$1,$693,$697);
      do {
       if ($667) {
        $698 = ($a$8$ph$i>>>0)>($$31$i>>>0);
        $r$0$a$8$i = $698 ? $$31$i : $a$8$ph$i;
        $d$482$i = $r$0$a$8$i;
        while(1) {
         $699 = HEAP32[$d$482$i>>2]|0;
         $700 = (_fmt_u($699,0,$13)|0);
         $701 = ($d$482$i|0)==($r$0$a$8$i|0);
         do {
          if ($701) {
           $705 = ($700|0)==($13|0);
           if (!($705)) {
            $s7$1$i = $700;
            break;
           }
           HEAP8[$15>>0] = 48;
           $s7$1$i = $15;
          } else {
           $702 = ($700>>>0)>($buf$i>>>0);
           if ($702) {
            $s7$079$i = $700;
           } else {
            $s7$1$i = $700;
            break;
           }
           while(1) {
            $703 = ((($s7$079$i)) + -1|0);
            HEAP8[$703>>0] = 48;
            $704 = ($703>>>0)>($buf$i>>>0);
            if ($704) {
             $s7$079$i = $703;
            } else {
             $s7$1$i = $703;
             break;
            }
           }
          }
         } while(0);
         $706 = HEAP32[$f>>2]|0;
         $707 = $706 & 32;
         $708 = ($707|0)==(0);
         if ($708) {
          $709 = $s7$1$i;
          $710 = (($14) - ($709))|0;
          (___fwritex($s7$1$i,$710,$f)|0);
         }
         $711 = ((($d$482$i)) + 4|0);
         $712 = ($711>>>0)>($$31$i>>>0);
         if ($712) {
          $$lcssa344 = $711;
          break;
         } else {
          $d$482$i = $711;
         }
        }
        $713 = ($663|0)==(0);
        do {
         if (!($713)) {
          $714 = HEAP32[$f>>2]|0;
          $715 = $714 & 32;
          $716 = ($715|0)==(0);
          if (!($716)) {
           break;
          }
          (___fwritex(25095,1,$f)|0);
         }
        } while(0);
        $717 = ($$lcssa344>>>0)<($z$6$i$lcssa>>>0);
        $718 = ($$311$i|0)>(0);
        $719 = $718 & $717;
        if ($719) {
         $$41276$i = $$311$i;$d$575$i = $$lcssa344;
         while(1) {
          $720 = HEAP32[$d$575$i>>2]|0;
          $721 = (_fmt_u($720,0,$13)|0);
          $722 = ($721>>>0)>($buf$i>>>0);
          if ($722) {
           $s8$070$i = $721;
           while(1) {
            $723 = ((($s8$070$i)) + -1|0);
            HEAP8[$723>>0] = 48;
            $724 = ($723>>>0)>($buf$i>>>0);
            if ($724) {
             $s8$070$i = $723;
            } else {
             $s8$0$lcssa$i = $723;
             break;
            }
           }
          } else {
           $s8$0$lcssa$i = $721;
          }
          $725 = HEAP32[$f>>2]|0;
          $726 = $725 & 32;
          $727 = ($726|0)==(0);
          if ($727) {
           $728 = ($$41276$i|0)>(9);
           $729 = $728 ? 9 : $$41276$i;
           (___fwritex($s8$0$lcssa$i,$729,$f)|0);
          }
          $730 = ((($d$575$i)) + 4|0);
          $731 = (($$41276$i) + -9)|0;
          $732 = ($730>>>0)<($z$6$i$lcssa>>>0);
          $733 = ($$41276$i|0)>(9);
          $734 = $733 & $732;
          if ($734) {
           $$41276$i = $731;$d$575$i = $730;
          } else {
           $$412$lcssa$i = $731;
           break;
          }
         }
        } else {
         $$412$lcssa$i = $$311$i;
        }
        $735 = (($$412$lcssa$i) + 9)|0;
        _pad($f,48,$735,9,0);
       } else {
        $736 = ((($a$8$ph$i)) + 4|0);
        $z$6$$i = $$lcssa159$i ? $z$6$i$lcssa : $736;
        $737 = ($$311$i|0)>(-1);
        if ($737) {
         $738 = ($$pre$phi184$iZ2D|0)==(0);
         $$587$i = $$311$i;$d$686$i = $a$8$ph$i;
         while(1) {
          $739 = HEAP32[$d$686$i>>2]|0;
          $740 = (_fmt_u($739,0,$13)|0);
          $741 = ($740|0)==($13|0);
          if ($741) {
           HEAP8[$15>>0] = 48;
           $s9$0$i = $15;
          } else {
           $s9$0$i = $740;
          }
          $742 = ($d$686$i|0)==($a$8$ph$i|0);
          do {
           if ($742) {
            $746 = ((($s9$0$i)) + 1|0);
            $747 = HEAP32[$f>>2]|0;
            $748 = $747 & 32;
            $749 = ($748|0)==(0);
            if ($749) {
             (___fwritex($s9$0$i,1,$f)|0);
            }
            $750 = ($$587$i|0)<(1);
            $or$cond29$i = $738 & $750;
            if ($or$cond29$i) {
             $s9$2$i = $746;
             break;
            }
            $751 = HEAP32[$f>>2]|0;
            $752 = $751 & 32;
            $753 = ($752|0)==(0);
            if (!($753)) {
             $s9$2$i = $746;
             break;
            }
            (___fwritex(25095,1,$f)|0);
            $s9$2$i = $746;
           } else {
            $743 = ($s9$0$i>>>0)>($buf$i>>>0);
            if ($743) {
             $s9$183$i = $s9$0$i;
            } else {
             $s9$2$i = $s9$0$i;
             break;
            }
            while(1) {
             $744 = ((($s9$183$i)) + -1|0);
             HEAP8[$744>>0] = 48;
             $745 = ($744>>>0)>($buf$i>>>0);
             if ($745) {
              $s9$183$i = $744;
             } else {
              $s9$2$i = $744;
              break;
             }
            }
           }
          } while(0);
          $754 = $s9$2$i;
          $755 = (($14) - ($754))|0;
          $756 = HEAP32[$f>>2]|0;
          $757 = $756 & 32;
          $758 = ($757|0)==(0);
          if ($758) {
           $759 = ($$587$i|0)>($755|0);
           $760 = $759 ? $755 : $$587$i;
           (___fwritex($s9$2$i,$760,$f)|0);
          }
          $761 = (($$587$i) - ($755))|0;
          $762 = ((($d$686$i)) + 4|0);
          $763 = ($762>>>0)<($z$6$$i>>>0);
          $764 = ($761|0)>(-1);
          $765 = $763 & $764;
          if ($765) {
           $$587$i = $761;$d$686$i = $762;
          } else {
           $$5$lcssa$i = $761;
           break;
          }
         }
        } else {
         $$5$lcssa$i = $$311$i;
        }
        $766 = (($$5$lcssa$i) + 18)|0;
        _pad($f,48,$766,18,0);
        $767 = HEAP32[$f>>2]|0;
        $768 = $767 & 32;
        $769 = ($768|0)==(0);
        if (!($769)) {
         break;
        }
        $770 = $estr$2$i;
        $771 = (($8) - ($770))|0;
        (___fwritex($estr$2$i,$771,$f)|0);
       }
      } while(0);
      $772 = $fl$1$ ^ 8192;
      _pad($f,32,$w$1,$693,$772);
      $773 = ($693|0)<($w$1|0);
      $w$30$i = $773 ? $w$1 : $693;
      $$0$i = $w$30$i;
     } else {
      $381 = $t$0 & 32;
      $382 = ($381|0)!=(0);
      $383 = $382 ? 25079 : 25083;
      $384 = ($$07$i != $$07$i) | (0.0 != 0.0);
      $385 = $382 ? 25087 : 25091;
      $pl$1$i = $384 ? 0 : $pl$0$i;
      $s1$0$i = $384 ? $385 : $383;
      $386 = (($pl$1$i) + 3)|0;
      _pad($f,32,$w$1,$386,$180);
      $387 = HEAP32[$f>>2]|0;
      $388 = $387 & 32;
      $389 = ($388|0)==(0);
      if ($389) {
       (___fwritex($prefix$0$i,$pl$1$i,$f)|0);
       $$pre$i = HEAP32[$f>>2]|0;
       $391 = $$pre$i;
      } else {
       $391 = $387;
      }
      $390 = $391 & 32;
      $392 = ($390|0)==(0);
      if ($392) {
       (___fwritex($s1$0$i,3,$f)|0);
      }
      $393 = $fl$1$ ^ 8192;
      _pad($f,32,$w$1,$386,$393);
      $394 = ($386|0)<($w$1|0);
      $395 = $394 ? $w$1 : $386;
      $$0$i = $395;
     }
    } while(0);
    $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $$0$i;$l10n$0 = $l10n$3;
    continue L1;
    break;
   }
   default: {
    $a$2 = $22;$fl$6 = $fl$1$;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 25043;$z$2 = $1;
   }
   }
  } while(0);
  L313: do {
   if ((label|0) == 64) {
    label = 0;
    $211 = $arg;
    $212 = $211;
    $213 = HEAP32[$212>>2]|0;
    $214 = (($211) + 4)|0;
    $215 = $214;
    $216 = HEAP32[$215>>2]|0;
    $217 = $t$1 & 32;
    $218 = ($213|0)==(0);
    $219 = ($216|0)==(0);
    $220 = $218 & $219;
    if ($220) {
     $a$0 = $1;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 25043;
     label = 77;
    } else {
     $$012$i = $1;$222 = $213;$229 = $216;
     while(1) {
      $221 = $222 & 15;
      $223 = (25027 + ($221)|0);
      $224 = HEAP8[$223>>0]|0;
      $225 = $224&255;
      $226 = $225 | $217;
      $227 = $226&255;
      $228 = ((($$012$i)) + -1|0);
      HEAP8[$228>>0] = $227;
      $230 = (_bitshift64Lshr(($222|0),($229|0),4)|0);
      $231 = tempRet0;
      $232 = ($230|0)==(0);
      $233 = ($231|0)==(0);
      $234 = $232 & $233;
      if ($234) {
       $$lcssa349 = $228;
       break;
      } else {
       $$012$i = $228;$222 = $230;$229 = $231;
      }
     }
     $235 = $arg;
     $236 = $235;
     $237 = HEAP32[$236>>2]|0;
     $238 = (($235) + 4)|0;
     $239 = $238;
     $240 = HEAP32[$239>>2]|0;
     $241 = ($237|0)==(0);
     $242 = ($240|0)==(0);
     $243 = $241 & $242;
     $244 = $fl$3 & 8;
     $245 = ($244|0)==(0);
     $or$cond17 = $245 | $243;
     if ($or$cond17) {
      $a$0 = $$lcssa349;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 25043;
      label = 77;
     } else {
      $246 = $t$1 >> 4;
      $247 = (25043 + ($246)|0);
      $a$0 = $$lcssa349;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 2;$prefix$1 = $247;
      label = 77;
     }
    }
   }
   else if ((label|0) == 76) {
    label = 0;
    $293 = (_fmt_u($291,$292,$1)|0);
    $a$0 = $293;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
    label = 77;
   }
   else if ((label|0) == 82) {
    label = 0;
    $325 = (_memchr($a$1,0,$p$0)|0);
    $326 = ($325|0)==(0|0);
    $327 = $325;
    $328 = $a$1;
    $329 = (($327) - ($328))|0;
    $330 = (($a$1) + ($p$0)|0);
    $z$1 = $326 ? $330 : $325;
    $p$3 = $326 ? $p$0 : $329;
    $a$2 = $a$1;$fl$6 = $180;$p$5 = $p$3;$pl$2 = 0;$prefix$2 = 25043;$z$2 = $z$1;
   }
   else if ((label|0) == 86) {
    label = 0;
    $338 = HEAP32[$arg>>2]|0;
    $i$0108 = 0;$l$1107 = 0;$ws$0109 = $338;
    while(1) {
     $339 = HEAP32[$ws$0109>>2]|0;
     $340 = ($339|0)==(0);
     if ($340) {
      $i$0$lcssa = $i$0108;$l$2 = $l$1107;
      break;
     }
     $341 = (_wctomb($mb,$339)|0);
     $342 = ($341|0)<(0);
     $343 = (($p$4195) - ($i$0108))|0;
     $344 = ($341>>>0)>($343>>>0);
     $or$cond20 = $342 | $344;
     if ($or$cond20) {
      $i$0$lcssa = $i$0108;$l$2 = $341;
      break;
     }
     $345 = ((($ws$0109)) + 4|0);
     $346 = (($341) + ($i$0108))|0;
     $347 = ($p$4195>>>0)>($346>>>0);
     if ($347) {
      $i$0108 = $346;$l$1107 = $341;$ws$0109 = $345;
     } else {
      $i$0$lcssa = $346;$l$2 = $341;
      break;
     }
    }
    $348 = ($l$2|0)<(0);
    if ($348) {
     $$0 = -1;
     break L1;
    }
    _pad($f,32,$w$1,$i$0$lcssa,$fl$1$);
    $349 = ($i$0$lcssa|0)==(0);
    if ($349) {
     $i$0$lcssa197 = 0;
     label = 98;
    } else {
     $350 = HEAP32[$arg>>2]|0;
     $i$1119 = 0;$ws$1120 = $350;
     while(1) {
      $351 = HEAP32[$ws$1120>>2]|0;
      $352 = ($351|0)==(0);
      if ($352) {
       $i$0$lcssa197 = $i$0$lcssa;
       label = 98;
       break L313;
      }
      $353 = ((($ws$1120)) + 4|0);
      $354 = (_wctomb($mb,$351)|0);
      $355 = (($354) + ($i$1119))|0;
      $356 = ($355|0)>($i$0$lcssa|0);
      if ($356) {
       $i$0$lcssa197 = $i$0$lcssa;
       label = 98;
       break L313;
      }
      $357 = HEAP32[$f>>2]|0;
      $358 = $357 & 32;
      $359 = ($358|0)==(0);
      if ($359) {
       (___fwritex($mb,$354,$f)|0);
      }
      $360 = ($355>>>0)<($i$0$lcssa>>>0);
      if ($360) {
       $i$1119 = $355;$ws$1120 = $353;
      } else {
       $i$0$lcssa197 = $i$0$lcssa;
       label = 98;
       break;
      }
     }
    }
   }
  } while(0);
  if ((label|0) == 98) {
   label = 0;
   $361 = $fl$1$ ^ 8192;
   _pad($f,32,$w$1,$i$0$lcssa197,$361);
   $362 = ($w$1|0)>($i$0$lcssa197|0);
   $363 = $362 ? $w$1 : $i$0$lcssa197;
   $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $363;$l10n$0 = $l10n$3;
   continue;
  }
  if ((label|0) == 77) {
   label = 0;
   $294 = ($p$2|0)>(-1);
   $295 = $fl$4 & -65537;
   $$fl$4 = $294 ? $295 : $fl$4;
   $296 = $arg;
   $297 = $296;
   $298 = HEAP32[$297>>2]|0;
   $299 = (($296) + 4)|0;
   $300 = $299;
   $301 = HEAP32[$300>>2]|0;
   $302 = ($298|0)!=(0);
   $303 = ($301|0)!=(0);
   $304 = $302 | $303;
   $305 = ($p$2|0)!=(0);
   $or$cond = $305 | $304;
   if ($or$cond) {
    $306 = $a$0;
    $307 = (($2) - ($306))|0;
    $308 = $304&1;
    $309 = $308 ^ 1;
    $310 = (($309) + ($307))|0;
    $311 = ($p$2|0)>($310|0);
    $p$2$ = $311 ? $p$2 : $310;
    $a$2 = $a$0;$fl$6 = $$fl$4;$p$5 = $p$2$;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $1;
   } else {
    $a$2 = $1;$fl$6 = $$fl$4;$p$5 = 0;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $1;
   }
  }
  $774 = $z$2;
  $775 = $a$2;
  $776 = (($774) - ($775))|0;
  $777 = ($p$5|0)<($776|0);
  $$p$5 = $777 ? $776 : $p$5;
  $778 = (($pl$2) + ($$p$5))|0;
  $779 = ($w$1|0)<($778|0);
  $w$2 = $779 ? $778 : $w$1;
  _pad($f,32,$w$2,$778,$fl$6);
  $780 = HEAP32[$f>>2]|0;
  $781 = $780 & 32;
  $782 = ($781|0)==(0);
  if ($782) {
   (___fwritex($prefix$2,$pl$2,$f)|0);
  }
  $783 = $fl$6 ^ 65536;
  _pad($f,48,$w$2,$778,$783);
  _pad($f,48,$$p$5,$776,0);
  $784 = HEAP32[$f>>2]|0;
  $785 = $784 & 32;
  $786 = ($785|0)==(0);
  if ($786) {
   (___fwritex($a$2,$776,$f)|0);
  }
  $787 = $fl$6 ^ 8192;
  _pad($f,32,$w$2,$778,$787);
  $22 = $$lcssa328;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
 }
 L348: do {
  if ((label|0) == 245) {
   $788 = ($f|0)==(0|0);
   if ($788) {
    $789 = ($l10n$0$lcssa|0)==(0);
    if ($789) {
     $$0 = 0;
    } else {
     $i$295 = 1;
     while(1) {
      $790 = (($nl_type) + ($i$295<<2)|0);
      $791 = HEAP32[$790>>2]|0;
      $792 = ($791|0)==(0);
      if ($792) {
       $i$295$lcssa = $i$295;
       break;
      }
      $794 = (($nl_arg) + ($i$295<<3)|0);
      _pop_arg80($794,$791,$ap);
      $795 = (($i$295) + 1)|0;
      $796 = ($795|0)<(10);
      if ($796) {
       $i$295 = $795;
      } else {
       $$0 = 1;
       break L348;
      }
     }
     $793 = ($i$295$lcssa|0)<(10);
     if ($793) {
      $i$393 = $i$295$lcssa;
      while(1) {
       $799 = (($nl_type) + ($i$393<<2)|0);
       $800 = HEAP32[$799>>2]|0;
       $801 = ($800|0)==(0);
       $797 = (($i$393) + 1)|0;
       if (!($801)) {
        $$0 = -1;
        break L348;
       }
       $798 = ($797|0)<(10);
       if ($798) {
        $i$393 = $797;
       } else {
        $$0 = 1;
        break;
       }
      }
     } else {
      $$0 = 1;
     }
    }
   } else {
    $$0 = $cnt$1$lcssa;
   }
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function _pop_arg80($arg,$type,$ap) {
 $arg = $arg|0;
 $type = $type|0;
 $ap = $ap|0;
 var $$mask = 0, $$mask1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0.0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0.0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current17 = 0;
 var $arglist_current2 = 0, $arglist_current20 = 0, $arglist_current23 = 0, $arglist_current26 = 0, $arglist_current5 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next18 = 0, $arglist_next21 = 0, $arglist_next24 = 0, $arglist_next27 = 0, $arglist_next3 = 0, $arglist_next6 = 0, $arglist_next9 = 0, $expanded = 0, $expanded28 = 0, $expanded30 = 0, $expanded31 = 0;
 var $expanded32 = 0, $expanded34 = 0, $expanded35 = 0, $expanded37 = 0, $expanded38 = 0, $expanded39 = 0, $expanded41 = 0, $expanded42 = 0, $expanded44 = 0, $expanded45 = 0, $expanded46 = 0, $expanded48 = 0, $expanded49 = 0, $expanded51 = 0, $expanded52 = 0, $expanded53 = 0, $expanded55 = 0, $expanded56 = 0, $expanded58 = 0, $expanded59 = 0;
 var $expanded60 = 0, $expanded62 = 0, $expanded63 = 0, $expanded65 = 0, $expanded66 = 0, $expanded67 = 0, $expanded69 = 0, $expanded70 = 0, $expanded72 = 0, $expanded73 = 0, $expanded74 = 0, $expanded76 = 0, $expanded77 = 0, $expanded79 = 0, $expanded80 = 0, $expanded81 = 0, $expanded83 = 0, $expanded84 = 0, $expanded86 = 0, $expanded87 = 0;
 var $expanded88 = 0, $expanded90 = 0, $expanded91 = 0, $expanded93 = 0, $expanded94 = 0, $expanded95 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($type>>>0)>(20);
 L1: do {
  if (!($0)) {
   do {
    switch ($type|0) {
    case 9:  {
     $arglist_current = HEAP32[$ap>>2]|0;
     $1 = $arglist_current;
     $2 = ((0) + 4|0);
     $expanded28 = $2;
     $expanded = (($expanded28) - 1)|0;
     $3 = (($1) + ($expanded))|0;
     $4 = ((0) + 4|0);
     $expanded32 = $4;
     $expanded31 = (($expanded32) - 1)|0;
     $expanded30 = $expanded31 ^ -1;
     $5 = $3 & $expanded30;
     $6 = $5;
     $7 = HEAP32[$6>>2]|0;
     $arglist_next = ((($6)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     HEAP32[$arg>>2] = $7;
     break L1;
     break;
    }
    case 10:  {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $8 = $arglist_current2;
     $9 = ((0) + 4|0);
     $expanded35 = $9;
     $expanded34 = (($expanded35) - 1)|0;
     $10 = (($8) + ($expanded34))|0;
     $11 = ((0) + 4|0);
     $expanded39 = $11;
     $expanded38 = (($expanded39) - 1)|0;
     $expanded37 = $expanded38 ^ -1;
     $12 = $10 & $expanded37;
     $13 = $12;
     $14 = HEAP32[$13>>2]|0;
     $arglist_next3 = ((($13)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $15 = ($14|0)<(0);
     $16 = $15 << 31 >> 31;
     $17 = $arg;
     $18 = $17;
     HEAP32[$18>>2] = $14;
     $19 = (($17) + 4)|0;
     $20 = $19;
     HEAP32[$20>>2] = $16;
     break L1;
     break;
    }
    case 11:  {
     $arglist_current5 = HEAP32[$ap>>2]|0;
     $21 = $arglist_current5;
     $22 = ((0) + 4|0);
     $expanded42 = $22;
     $expanded41 = (($expanded42) - 1)|0;
     $23 = (($21) + ($expanded41))|0;
     $24 = ((0) + 4|0);
     $expanded46 = $24;
     $expanded45 = (($expanded46) - 1)|0;
     $expanded44 = $expanded45 ^ -1;
     $25 = $23 & $expanded44;
     $26 = $25;
     $27 = HEAP32[$26>>2]|0;
     $arglist_next6 = ((($26)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next6;
     $28 = $arg;
     $29 = $28;
     HEAP32[$29>>2] = $27;
     $30 = (($28) + 4)|0;
     $31 = $30;
     HEAP32[$31>>2] = 0;
     break L1;
     break;
    }
    case 12:  {
     $arglist_current8 = HEAP32[$ap>>2]|0;
     $32 = $arglist_current8;
     $33 = ((0) + 8|0);
     $expanded49 = $33;
     $expanded48 = (($expanded49) - 1)|0;
     $34 = (($32) + ($expanded48))|0;
     $35 = ((0) + 8|0);
     $expanded53 = $35;
     $expanded52 = (($expanded53) - 1)|0;
     $expanded51 = $expanded52 ^ -1;
     $36 = $34 & $expanded51;
     $37 = $36;
     $38 = $37;
     $39 = $38;
     $40 = HEAP32[$39>>2]|0;
     $41 = (($38) + 4)|0;
     $42 = $41;
     $43 = HEAP32[$42>>2]|0;
     $arglist_next9 = ((($37)) + 8|0);
     HEAP32[$ap>>2] = $arglist_next9;
     $44 = $arg;
     $45 = $44;
     HEAP32[$45>>2] = $40;
     $46 = (($44) + 4)|0;
     $47 = $46;
     HEAP32[$47>>2] = $43;
     break L1;
     break;
    }
    case 13:  {
     $arglist_current11 = HEAP32[$ap>>2]|0;
     $48 = $arglist_current11;
     $49 = ((0) + 4|0);
     $expanded56 = $49;
     $expanded55 = (($expanded56) - 1)|0;
     $50 = (($48) + ($expanded55))|0;
     $51 = ((0) + 4|0);
     $expanded60 = $51;
     $expanded59 = (($expanded60) - 1)|0;
     $expanded58 = $expanded59 ^ -1;
     $52 = $50 & $expanded58;
     $53 = $52;
     $54 = HEAP32[$53>>2]|0;
     $arglist_next12 = ((($53)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next12;
     $55 = $54&65535;
     $56 = $55 << 16 >> 16;
     $57 = ($56|0)<(0);
     $58 = $57 << 31 >> 31;
     $59 = $arg;
     $60 = $59;
     HEAP32[$60>>2] = $56;
     $61 = (($59) + 4)|0;
     $62 = $61;
     HEAP32[$62>>2] = $58;
     break L1;
     break;
    }
    case 14:  {
     $arglist_current14 = HEAP32[$ap>>2]|0;
     $63 = $arglist_current14;
     $64 = ((0) + 4|0);
     $expanded63 = $64;
     $expanded62 = (($expanded63) - 1)|0;
     $65 = (($63) + ($expanded62))|0;
     $66 = ((0) + 4|0);
     $expanded67 = $66;
     $expanded66 = (($expanded67) - 1)|0;
     $expanded65 = $expanded66 ^ -1;
     $67 = $65 & $expanded65;
     $68 = $67;
     $69 = HEAP32[$68>>2]|0;
     $arglist_next15 = ((($68)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next15;
     $$mask1 = $69 & 65535;
     $70 = $arg;
     $71 = $70;
     HEAP32[$71>>2] = $$mask1;
     $72 = (($70) + 4)|0;
     $73 = $72;
     HEAP32[$73>>2] = 0;
     break L1;
     break;
    }
    case 15:  {
     $arglist_current17 = HEAP32[$ap>>2]|0;
     $74 = $arglist_current17;
     $75 = ((0) + 4|0);
     $expanded70 = $75;
     $expanded69 = (($expanded70) - 1)|0;
     $76 = (($74) + ($expanded69))|0;
     $77 = ((0) + 4|0);
     $expanded74 = $77;
     $expanded73 = (($expanded74) - 1)|0;
     $expanded72 = $expanded73 ^ -1;
     $78 = $76 & $expanded72;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $arglist_next18 = ((($79)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next18;
     $81 = $80&255;
     $82 = $81 << 24 >> 24;
     $83 = ($82|0)<(0);
     $84 = $83 << 31 >> 31;
     $85 = $arg;
     $86 = $85;
     HEAP32[$86>>2] = $82;
     $87 = (($85) + 4)|0;
     $88 = $87;
     HEAP32[$88>>2] = $84;
     break L1;
     break;
    }
    case 16:  {
     $arglist_current20 = HEAP32[$ap>>2]|0;
     $89 = $arglist_current20;
     $90 = ((0) + 4|0);
     $expanded77 = $90;
     $expanded76 = (($expanded77) - 1)|0;
     $91 = (($89) + ($expanded76))|0;
     $92 = ((0) + 4|0);
     $expanded81 = $92;
     $expanded80 = (($expanded81) - 1)|0;
     $expanded79 = $expanded80 ^ -1;
     $93 = $91 & $expanded79;
     $94 = $93;
     $95 = HEAP32[$94>>2]|0;
     $arglist_next21 = ((($94)) + 4|0);
     HEAP32[$ap>>2] = $arglist_next21;
     $$mask = $95 & 255;
     $96 = $arg;
     $97 = $96;
     HEAP32[$97>>2] = $$mask;
     $98 = (($96) + 4)|0;
     $99 = $98;
     HEAP32[$99>>2] = 0;
     break L1;
     break;
    }
    case 17:  {
     $arglist_current23 = HEAP32[$ap>>2]|0;
     $100 = $arglist_current23;
     $101 = ((0) + 8|0);
     $expanded84 = $101;
     $expanded83 = (($expanded84) - 1)|0;
     $102 = (($100) + ($expanded83))|0;
     $103 = ((0) + 8|0);
     $expanded88 = $103;
     $expanded87 = (($expanded88) - 1)|0;
     $expanded86 = $expanded87 ^ -1;
     $104 = $102 & $expanded86;
     $105 = $104;
     $106 = +HEAPF64[$105>>3];
     $arglist_next24 = ((($105)) + 8|0);
     HEAP32[$ap>>2] = $arglist_next24;
     HEAPF64[$arg>>3] = $106;
     break L1;
     break;
    }
    case 18:  {
     $arglist_current26 = HEAP32[$ap>>2]|0;
     $107 = $arglist_current26;
     $108 = ((0) + 8|0);
     $expanded91 = $108;
     $expanded90 = (($expanded91) - 1)|0;
     $109 = (($107) + ($expanded90))|0;
     $110 = ((0) + 8|0);
     $expanded95 = $110;
     $expanded94 = (($expanded95) - 1)|0;
     $expanded93 = $expanded94 ^ -1;
     $111 = $109 & $expanded93;
     $112 = $111;
     $113 = +HEAPF64[$112>>3];
     $arglist_next27 = ((($112)) + 8|0);
     HEAP32[$ap>>2] = $arglist_next27;
     HEAPF64[$arg>>3] = $113;
     break L1;
     break;
    }
    default: {
     break L1;
    }
    }
   } while(0);
  }
 } while(0);
 return;
}
function _fmt_u($0,$1,$s) {
 $0 = $0|0;
 $1 = $1|0;
 $s = $s|0;
 var $$0$lcssa = 0, $$01$lcssa$off0 = 0, $$05 = 0, $$1$lcssa = 0, $$12 = 0, $$lcssa20 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $y$03 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $2 = ($1>>>0)>(0);
 $3 = ($0>>>0)>(4294967295);
 $4 = ($1|0)==(0);
 $5 = $4 & $3;
 $6 = $2 | $5;
 if ($6) {
  $$05 = $s;$7 = $0;$8 = $1;
  while(1) {
   $9 = (___uremdi3(($7|0),($8|0),10,0)|0);
   $10 = tempRet0;
   $11 = $9 | 48;
   $12 = $11&255;
   $13 = ((($$05)) + -1|0);
   HEAP8[$13>>0] = $12;
   $14 = (___udivdi3(($7|0),($8|0),10,0)|0);
   $15 = tempRet0;
   $16 = ($8>>>0)>(9);
   $17 = ($7>>>0)>(4294967295);
   $18 = ($8|0)==(9);
   $19 = $18 & $17;
   $20 = $16 | $19;
   if ($20) {
    $$05 = $13;$7 = $14;$8 = $15;
   } else {
    $$lcssa20 = $13;$28 = $14;$29 = $15;
    break;
   }
  }
  $$0$lcssa = $$lcssa20;$$01$lcssa$off0 = $28;
 } else {
  $$0$lcssa = $s;$$01$lcssa$off0 = $0;
 }
 $21 = ($$01$lcssa$off0|0)==(0);
 if ($21) {
  $$1$lcssa = $$0$lcssa;
 } else {
  $$12 = $$0$lcssa;$y$03 = $$01$lcssa$off0;
  while(1) {
   $22 = (($y$03>>>0) % 10)&-1;
   $23 = $22 | 48;
   $24 = $23&255;
   $25 = ((($$12)) + -1|0);
   HEAP8[$25>>0] = $24;
   $26 = (($y$03>>>0) / 10)&-1;
   $27 = ($y$03>>>0)<(10);
   if ($27) {
    $$1$lcssa = $25;
    break;
   } else {
    $$12 = $25;$y$03 = $26;
   }
  }
 }
 return ($$1$lcssa|0);
}
function _pad($f,$c,$w,$l,$fl) {
 $f = $f|0;
 $c = $c|0;
 $w = $w|0;
 $l = $l|0;
 $fl = $fl|0;
 var $$0$lcssa6 = 0, $$02 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pad = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0; if ((STACKTOP|0) >= (STACK_MAX|0)) abort();
 $pad = sp;
 $0 = $fl & 73728;
 $1 = ($0|0)==(0);
 $2 = ($w|0)>($l|0);
 $or$cond = $2 & $1;
 do {
  if ($or$cond) {
   $3 = (($w) - ($l))|0;
   $4 = ($3>>>0)>(256);
   $5 = $4 ? 256 : $3;
   _memset(($pad|0),($c|0),($5|0))|0;
   $6 = ($3>>>0)>(255);
   $7 = HEAP32[$f>>2]|0;
   $8 = $7 & 32;
   $9 = ($8|0)==(0);
   if ($6) {
    $10 = (($w) - ($l))|0;
    $$02 = $3;$17 = $7;$18 = $9;
    while(1) {
     if ($18) {
      (___fwritex($pad,256,$f)|0);
      $$pre = HEAP32[$f>>2]|0;
      $14 = $$pre;
     } else {
      $14 = $17;
     }
     $11 = (($$02) + -256)|0;
     $12 = ($11>>>0)>(255);
     $13 = $14 & 32;
     $15 = ($13|0)==(0);
     if ($12) {
      $$02 = $11;$17 = $14;$18 = $15;
     } else {
      break;
     }
    }
    $16 = $10 & 255;
    if ($15) {
     $$0$lcssa6 = $16;
    } else {
     break;
    }
   } else {
    if ($9) {
     $$0$lcssa6 = $3;
    } else {
     break;
    }
   }
   (___fwritex($pad,$$0$lcssa6,$f)|0);
  }
 } while(0);
 STACKTOP = sp;return;
}
function _sn_write($f,$s,$l) {
 $f = $f|0;
 $s = $s|0;
 $l = $l|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($f)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = ($6>>>0)>($l>>>0);
 $l$ = $7 ? $l : $6;
 _memcpy(($3|0),($s|0),($l$|0))|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($l$)|0);
 HEAP32[$2>>2] = $9;
 return ($l|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$3$i = 0, $$lcssa = 0, $$lcssa211 = 0, $$lcssa215 = 0, $$lcssa216 = 0, $$lcssa217 = 0, $$lcssa219 = 0, $$lcssa222 = 0, $$lcssa224 = 0, $$lcssa226 = 0, $$lcssa228 = 0, $$lcssa230 = 0, $$lcssa232 = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i22$i = 0, $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i23$iZ2D = 0;
 var $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre106 = 0, $$pre14$i$i = 0, $$pre43$i = 0, $$pre56$i$i = 0, $$pre57$i$i = 0, $$pre8$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i13$i = 0, $$sum$i14$i = 0, $$sum$i17$i = 0, $$sum$i19$i = 0;
 var $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i15$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0;
 var $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum121$i = 0, $$sum122$i = 0, $$sum123$i = 0, $$sum124$i = 0, $$sum125$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0;
 var $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i16$i = 0, $$sum2$i18$i = 0, $$sum2$i21$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i27 = 0;
 var $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0;
 var $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0;
 var $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0;
 var $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0;
 var $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0;
 var $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0;
 var $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0;
 var $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0;
 var $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0;
 var $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0;
 var $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0;
 var $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0;
 var $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0;
 var $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0;
 var $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0;
 var $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0;
 var $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0;
 var $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0;
 var $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0;
 var $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0;
 var $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0;
 var $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0;
 var $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0;
 var $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0;
 var $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0;
 var $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0;
 var $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0;
 var $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0;
 var $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0;
 var $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0;
 var $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0;
 var $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0;
 var $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0;
 var $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0;
 var $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0;
 var $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0;
 var $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0;
 var $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0;
 var $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0;
 var $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0;
 var $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0;
 var $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0;
 var $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0;
 var $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0;
 var $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0;
 var $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0;
 var $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0;
 var $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0;
 var $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0;
 var $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0;
 var $F5$0$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$029$i = 0, $K2$07$i$i = 0, $K8$051$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i$i$lcssa = 0, $R$0$i$lcssa = 0, $R$0$i18 = 0, $R$0$i18$lcssa = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i$i$lcssa = 0, $RP$0$i$lcssa = 0;
 var $RP$0$i17 = 0, $RP$0$i17$lcssa = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i25$i = 0, $T$028$i = 0, $T$028$i$lcssa = 0, $T$050$i$i = 0, $T$050$i$i$lcssa = 0, $T$06$i$i = 0, $T$06$i$i$lcssa = 0, $br$0$ph$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0;
 var $not$$i = 0, $not$$i$i = 0, $not$$i26$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i30 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond3$i = 0, $or$cond5$i = 0, $or$cond57$i = 0, $or$cond6$i = 0, $or$cond8$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i$lcssa = 0, $rsize$0$i15 = 0, $rsize$1$i = 0;
 var $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$084$i = 0, $sp$084$i$lcssa = 0, $sp$183$i = 0, $sp$183$i$lcssa = 0, $ssize$0$$i = 0, $ssize$0$i = 0, $ssize$1$ph$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0;
 var $t$2$v$3$i = 0, $t$230$i = 0, $tbase$255$i = 0, $tsize$0$ph$i = 0, $tsize$0323944$i = 0, $tsize$1$i = 0, $tsize$254$i = 0, $v$0$i = 0, $v$0$i$lcssa = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0, $v$3$ph$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   $2 = (($bytes) + 11)|0;
   $3 = $2 & -8;
   $4 = $1 ? 16 : $3;
   $5 = $4 >>> 3;
   $6 = HEAP32[13244>>2]|0;
   $7 = $6 >>> $5;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($5))|0;
    $13 = $12 << 1;
    $14 = (13284 + ($13<<2)|0);
    $$sum10 = (($13) + 2)|0;
    $15 = (13284 + ($$sum10<<2)|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[13244>>2] = $22;
     } else {
      $23 = HEAP32[(13260)>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = ((($18)) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = ((($16)) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    return ($mem$0|0);
   }
   $34 = HEAP32[(13252)>>2]|0;
   $35 = ($4>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $5;
     $38 = 2 << $5;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = (13284 + ($65<<2)|0);
     $$sum4 = (($65) + 2)|0;
     $67 = (13284 + ($$sum4<<2)|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[13244>>2] = $74;
       $88 = $34;
      } else {
       $75 = HEAP32[(13260)>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = ((($70)) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        $$pre = HEAP32[(13252)>>2]|0;
        $88 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($4))|0;
     $82 = $4 | 3;
     $83 = ((($68)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($4)|0);
     $85 = $81 | 1;
     $$sum56 = $4 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $89 = ($88|0)==(0);
     if (!($89)) {
      $90 = HEAP32[(13264)>>2]|0;
      $91 = $88 >>> 3;
      $92 = $91 << 1;
      $93 = (13284 + ($92<<2)|0);
      $94 = HEAP32[13244>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[13244>>2] = $98;
       $$pre105 = (($92) + 2)|0;
       $$pre106 = (13284 + ($$pre105<<2)|0);
       $$pre$phiZ2D = $$pre106;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = (13284 + ($$sum9<<2)|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[(13260)>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = ((($F4$0)) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = ((($90)) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = ((($90)) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[(13252)>>2] = $81;
     HEAP32[(13264)>>2] = $84;
     $mem$0 = $69;
     return ($mem$0|0);
    }
    $106 = HEAP32[(13248)>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $4;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = (13548 + ($130<<2)|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($4))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = ((($t$0$i)) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = ((($t$0$i)) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        $rsize$0$i$lcssa = $rsize$0$i;$v$0$i$lcssa = $v$0$i;
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = ((($144)) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($4))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[(13260)>>2]|0;
     $150 = ($v$0$i$lcssa>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i$lcssa) + ($4)|0);
     $152 = ($v$0$i$lcssa>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = ((($v$0$i$lcssa)) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = ((($v$0$i$lcssa)) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i$lcssa|0);
     do {
      if ($157) {
       $167 = ((($v$0$i$lcssa)) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = ((($v$0$i$lcssa)) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = ((($R$0$i)) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = ((($R$0$i)) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         $R$0$i$lcssa = $R$0$i;$RP$0$i$lcssa = $RP$0$i;
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i$lcssa>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i$lcssa>>2] = 0;
        $R$1$i = $R$0$i$lcssa;
        break;
       }
      } else {
       $158 = ((($v$0$i$lcssa)) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = ((($159)) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i$lcssa|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = ((($156)) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i$lcssa|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = ((($v$0$i$lcssa)) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = (13548 + ($182<<2)|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i$lcssa|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[(13248)>>2]|0;
         $189 = $188 & $187;
         HEAP32[(13248)>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[(13260)>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = ((($154)) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i$lcssa|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = ((($154)) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[(13260)>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = ((($R$1$i)) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = ((($v$0$i$lcssa)) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = ($201>>>0)<($197>>>0);
         if ($203) {
          _abort();
          // unreachable;
         } else {
          $204 = ((($R$1$i)) + 16|0);
          HEAP32[$204>>2] = $201;
          $205 = ((($201)) + 24|0);
          HEAP32[$205>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $206 = ((($v$0$i$lcssa)) + 20|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = ($207|0)==(0|0);
       if (!($208)) {
        $209 = HEAP32[(13260)>>2]|0;
        $210 = ($207>>>0)<($209>>>0);
        if ($210) {
         _abort();
         // unreachable;
        } else {
         $211 = ((($R$1$i)) + 20|0);
         HEAP32[$211>>2] = $207;
         $212 = ((($207)) + 24|0);
         HEAP32[$212>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $213 = ($rsize$0$i$lcssa>>>0)<(16);
     if ($213) {
      $214 = (($rsize$0$i$lcssa) + ($4))|0;
      $215 = $214 | 3;
      $216 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$216>>2] = $215;
      $$sum4$i = (($214) + 4)|0;
      $217 = (($v$0$i$lcssa) + ($$sum4$i)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $218 | 1;
      HEAP32[$217>>2] = $219;
     } else {
      $220 = $4 | 3;
      $221 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$221>>2] = $220;
      $222 = $rsize$0$i$lcssa | 1;
      $$sum$i35 = $4 | 4;
      $223 = (($v$0$i$lcssa) + ($$sum$i35)|0);
      HEAP32[$223>>2] = $222;
      $$sum1$i = (($rsize$0$i$lcssa) + ($4))|0;
      $224 = (($v$0$i$lcssa) + ($$sum1$i)|0);
      HEAP32[$224>>2] = $rsize$0$i$lcssa;
      $225 = HEAP32[(13252)>>2]|0;
      $226 = ($225|0)==(0);
      if (!($226)) {
       $227 = HEAP32[(13264)>>2]|0;
       $228 = $225 >>> 3;
       $229 = $228 << 1;
       $230 = (13284 + ($229<<2)|0);
       $231 = HEAP32[13244>>2]|0;
       $232 = 1 << $228;
       $233 = $231 & $232;
       $234 = ($233|0)==(0);
       if ($234) {
        $235 = $231 | $232;
        HEAP32[13244>>2] = $235;
        $$pre$i = (($229) + 2)|0;
        $$pre8$i = (13284 + ($$pre$i<<2)|0);
        $$pre$phi$iZ2D = $$pre8$i;$F1$0$i = $230;
       } else {
        $$sum3$i = (($229) + 2)|0;
        $236 = (13284 + ($$sum3$i<<2)|0);
        $237 = HEAP32[$236>>2]|0;
        $238 = HEAP32[(13260)>>2]|0;
        $239 = ($237>>>0)<($238>>>0);
        if ($239) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $236;$F1$0$i = $237;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $227;
       $240 = ((($F1$0$i)) + 12|0);
       HEAP32[$240>>2] = $227;
       $241 = ((($227)) + 8|0);
       HEAP32[$241>>2] = $F1$0$i;
       $242 = ((($227)) + 12|0);
       HEAP32[$242>>2] = $230;
      }
      HEAP32[(13252)>>2] = $rsize$0$i$lcssa;
      HEAP32[(13264)>>2] = $151;
     }
     $243 = ((($v$0$i$lcssa)) + 8|0);
     $mem$0 = $243;
     return ($mem$0|0);
    }
   } else {
    $nb$0 = $4;
   }
  } else {
   $244 = ($bytes>>>0)>(4294967231);
   if ($244) {
    $nb$0 = -1;
   } else {
    $245 = (($bytes) + 11)|0;
    $246 = $245 & -8;
    $247 = HEAP32[(13248)>>2]|0;
    $248 = ($247|0)==(0);
    if ($248) {
     $nb$0 = $246;
    } else {
     $249 = (0 - ($246))|0;
     $250 = $245 >>> 8;
     $251 = ($250|0)==(0);
     if ($251) {
      $idx$0$i = 0;
     } else {
      $252 = ($246>>>0)>(16777215);
      if ($252) {
       $idx$0$i = 31;
      } else {
       $253 = (($250) + 1048320)|0;
       $254 = $253 >>> 16;
       $255 = $254 & 8;
       $256 = $250 << $255;
       $257 = (($256) + 520192)|0;
       $258 = $257 >>> 16;
       $259 = $258 & 4;
       $260 = $259 | $255;
       $261 = $256 << $259;
       $262 = (($261) + 245760)|0;
       $263 = $262 >>> 16;
       $264 = $263 & 2;
       $265 = $260 | $264;
       $266 = (14 - ($265))|0;
       $267 = $261 << $264;
       $268 = $267 >>> 15;
       $269 = (($266) + ($268))|0;
       $270 = $269 << 1;
       $271 = (($269) + 7)|0;
       $272 = $246 >>> $271;
       $273 = $272 & 1;
       $274 = $273 | $270;
       $idx$0$i = $274;
      }
     }
     $275 = (13548 + ($idx$0$i<<2)|0);
     $276 = HEAP32[$275>>2]|0;
     $277 = ($276|0)==(0|0);
     L123: do {
      if ($277) {
       $rsize$2$i = $249;$t$1$i = 0;$v$2$i = 0;
       label = 86;
      } else {
       $278 = ($idx$0$i|0)==(31);
       $279 = $idx$0$i >>> 1;
       $280 = (25 - ($279))|0;
       $281 = $278 ? 0 : $280;
       $282 = $246 << $281;
       $rsize$0$i15 = $249;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $276;$v$0$i16 = 0;
       while(1) {
        $283 = ((($t$0$i14)) + 4|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 & -8;
        $286 = (($285) - ($246))|0;
        $287 = ($286>>>0)<($rsize$0$i15>>>0);
        if ($287) {
         $288 = ($285|0)==($246|0);
         if ($288) {
          $rsize$331$i = $286;$t$230$i = $t$0$i14;$v$332$i = $t$0$i14;
          label = 90;
          break L123;
         } else {
          $rsize$1$i = $286;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $289 = ((($t$0$i14)) + 20|0);
        $290 = HEAP32[$289>>2]|0;
        $291 = $sizebits$0$i >>> 31;
        $292 = (((($t$0$i14)) + 16|0) + ($291<<2)|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = ($290|0)==(0|0);
        $295 = ($290|0)==($293|0);
        $or$cond19$i = $294 | $295;
        $rst$1$i = $or$cond19$i ? $rst$0$i : $290;
        $296 = ($293|0)==(0|0);
        $297 = $sizebits$0$i << 1;
        if ($296) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         label = 86;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $297;$t$0$i14 = $293;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     if ((label|0) == 86) {
      $298 = ($t$1$i|0)==(0|0);
      $299 = ($v$2$i|0)==(0|0);
      $or$cond$i = $298 & $299;
      if ($or$cond$i) {
       $300 = 2 << $idx$0$i;
       $301 = (0 - ($300))|0;
       $302 = $300 | $301;
       $303 = $247 & $302;
       $304 = ($303|0)==(0);
       if ($304) {
        $nb$0 = $246;
        break;
       }
       $305 = (0 - ($303))|0;
       $306 = $303 & $305;
       $307 = (($306) + -1)|0;
       $308 = $307 >>> 12;
       $309 = $308 & 16;
       $310 = $307 >>> $309;
       $311 = $310 >>> 5;
       $312 = $311 & 8;
       $313 = $312 | $309;
       $314 = $310 >>> $312;
       $315 = $314 >>> 2;
       $316 = $315 & 4;
       $317 = $313 | $316;
       $318 = $314 >>> $316;
       $319 = $318 >>> 1;
       $320 = $319 & 2;
       $321 = $317 | $320;
       $322 = $318 >>> $320;
       $323 = $322 >>> 1;
       $324 = $323 & 1;
       $325 = $321 | $324;
       $326 = $322 >>> $324;
       $327 = (($325) + ($326))|0;
       $328 = (13548 + ($327<<2)|0);
       $329 = HEAP32[$328>>2]|0;
       $t$2$ph$i = $329;$v$3$ph$i = 0;
      } else {
       $t$2$ph$i = $t$1$i;$v$3$ph$i = $v$2$i;
      }
      $330 = ($t$2$ph$i|0)==(0|0);
      if ($330) {
       $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$3$ph$i;
      } else {
       $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$3$ph$i;
       label = 90;
      }
     }
     if ((label|0) == 90) {
      while(1) {
       label = 0;
       $331 = ((($t$230$i)) + 4|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = $332 & -8;
       $334 = (($333) - ($246))|0;
       $335 = ($334>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $335 ? $334 : $rsize$331$i;
       $t$2$v$3$i = $335 ? $t$230$i : $v$332$i;
       $336 = ((($t$230$i)) + 16|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = ($337|0)==(0|0);
       if (!($338)) {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $337;$v$332$i = $t$2$v$3$i;
        label = 90;
        continue;
       }
       $339 = ((($t$230$i)) + 20|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $340;$v$332$i = $t$2$v$3$i;
        label = 90;
       }
      }
     }
     $342 = ($v$3$lcssa$i|0)==(0|0);
     if ($342) {
      $nb$0 = $246;
     } else {
      $343 = HEAP32[(13252)>>2]|0;
      $344 = (($343) - ($246))|0;
      $345 = ($rsize$3$lcssa$i>>>0)<($344>>>0);
      if ($345) {
       $346 = HEAP32[(13260)>>2]|0;
       $347 = ($v$3$lcssa$i>>>0)<($346>>>0);
       if ($347) {
        _abort();
        // unreachable;
       }
       $348 = (($v$3$lcssa$i) + ($246)|0);
       $349 = ($v$3$lcssa$i>>>0)<($348>>>0);
       if (!($349)) {
        _abort();
        // unreachable;
       }
       $350 = ((($v$3$lcssa$i)) + 24|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = ((($v$3$lcssa$i)) + 12|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($353|0)==($v$3$lcssa$i|0);
       do {
        if ($354) {
         $364 = ((($v$3$lcssa$i)) + 20|0);
         $365 = HEAP32[$364>>2]|0;
         $366 = ($365|0)==(0|0);
         if ($366) {
          $367 = ((($v$3$lcssa$i)) + 16|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = ($368|0)==(0|0);
          if ($369) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $368;$RP$0$i17 = $367;
          }
         } else {
          $R$0$i18 = $365;$RP$0$i17 = $364;
         }
         while(1) {
          $370 = ((($R$0$i18)) + 20|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = ($371|0)==(0|0);
          if (!($372)) {
           $R$0$i18 = $371;$RP$0$i17 = $370;
           continue;
          }
          $373 = ((($R$0$i18)) + 16|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ($374|0)==(0|0);
          if ($375) {
           $R$0$i18$lcssa = $R$0$i18;$RP$0$i17$lcssa = $RP$0$i17;
           break;
          } else {
           $R$0$i18 = $374;$RP$0$i17 = $373;
          }
         }
         $376 = ($RP$0$i17$lcssa>>>0)<($346>>>0);
         if ($376) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17$lcssa>>2] = 0;
          $R$1$i20 = $R$0$i18$lcssa;
          break;
         }
        } else {
         $355 = ((($v$3$lcssa$i)) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         $357 = ($356>>>0)<($346>>>0);
         if ($357) {
          _abort();
          // unreachable;
         }
         $358 = ((($356)) + 12|0);
         $359 = HEAP32[$358>>2]|0;
         $360 = ($359|0)==($v$3$lcssa$i|0);
         if (!($360)) {
          _abort();
          // unreachable;
         }
         $361 = ((($353)) + 8|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = ($362|0)==($v$3$lcssa$i|0);
         if ($363) {
          HEAP32[$358>>2] = $353;
          HEAP32[$361>>2] = $356;
          $R$1$i20 = $353;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $377 = ($351|0)==(0|0);
       do {
        if (!($377)) {
         $378 = ((($v$3$lcssa$i)) + 28|0);
         $379 = HEAP32[$378>>2]|0;
         $380 = (13548 + ($379<<2)|0);
         $381 = HEAP32[$380>>2]|0;
         $382 = ($v$3$lcssa$i|0)==($381|0);
         if ($382) {
          HEAP32[$380>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $383 = 1 << $379;
           $384 = $383 ^ -1;
           $385 = HEAP32[(13248)>>2]|0;
           $386 = $385 & $384;
           HEAP32[(13248)>>2] = $386;
           break;
          }
         } else {
          $387 = HEAP32[(13260)>>2]|0;
          $388 = ($351>>>0)<($387>>>0);
          if ($388) {
           _abort();
           // unreachable;
          }
          $389 = ((($351)) + 16|0);
          $390 = HEAP32[$389>>2]|0;
          $391 = ($390|0)==($v$3$lcssa$i|0);
          if ($391) {
           HEAP32[$389>>2] = $R$1$i20;
          } else {
           $392 = ((($351)) + 20|0);
           HEAP32[$392>>2] = $R$1$i20;
          }
          $393 = ($R$1$i20|0)==(0|0);
          if ($393) {
           break;
          }
         }
         $394 = HEAP32[(13260)>>2]|0;
         $395 = ($R$1$i20>>>0)<($394>>>0);
         if ($395) {
          _abort();
          // unreachable;
         }
         $396 = ((($R$1$i20)) + 24|0);
         HEAP32[$396>>2] = $351;
         $397 = ((($v$3$lcssa$i)) + 16|0);
         $398 = HEAP32[$397>>2]|0;
         $399 = ($398|0)==(0|0);
         do {
          if (!($399)) {
           $400 = ($398>>>0)<($394>>>0);
           if ($400) {
            _abort();
            // unreachable;
           } else {
            $401 = ((($R$1$i20)) + 16|0);
            HEAP32[$401>>2] = $398;
            $402 = ((($398)) + 24|0);
            HEAP32[$402>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $403 = ((($v$3$lcssa$i)) + 20|0);
         $404 = HEAP32[$403>>2]|0;
         $405 = ($404|0)==(0|0);
         if (!($405)) {
          $406 = HEAP32[(13260)>>2]|0;
          $407 = ($404>>>0)<($406>>>0);
          if ($407) {
           _abort();
           // unreachable;
          } else {
           $408 = ((($R$1$i20)) + 20|0);
           HEAP32[$408>>2] = $404;
           $409 = ((($404)) + 24|0);
           HEAP32[$409>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $410 = ($rsize$3$lcssa$i>>>0)<(16);
       L199: do {
        if ($410) {
         $411 = (($rsize$3$lcssa$i) + ($246))|0;
         $412 = $411 | 3;
         $413 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$413>>2] = $412;
         $$sum18$i = (($411) + 4)|0;
         $414 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $415 = HEAP32[$414>>2]|0;
         $416 = $415 | 1;
         HEAP32[$414>>2] = $416;
        } else {
         $417 = $246 | 3;
         $418 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$418>>2] = $417;
         $419 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $246 | 4;
         $420 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$420>>2] = $419;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($246))|0;
         $421 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$421>>2] = $rsize$3$lcssa$i;
         $422 = $rsize$3$lcssa$i >>> 3;
         $423 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($423) {
          $424 = $422 << 1;
          $425 = (13284 + ($424<<2)|0);
          $426 = HEAP32[13244>>2]|0;
          $427 = 1 << $422;
          $428 = $426 & $427;
          $429 = ($428|0)==(0);
          if ($429) {
           $430 = $426 | $427;
           HEAP32[13244>>2] = $430;
           $$pre$i25 = (($424) + 2)|0;
           $$pre43$i = (13284 + ($$pre$i25<<2)|0);
           $$pre$phi$i26Z2D = $$pre43$i;$F5$0$i = $425;
          } else {
           $$sum17$i = (($424) + 2)|0;
           $431 = (13284 + ($$sum17$i<<2)|0);
           $432 = HEAP32[$431>>2]|0;
           $433 = HEAP32[(13260)>>2]|0;
           $434 = ($432>>>0)<($433>>>0);
           if ($434) {
            _abort();
            // unreachable;
           } else {
            $$pre$phi$i26Z2D = $431;$F5$0$i = $432;
           }
          }
          HEAP32[$$pre$phi$i26Z2D>>2] = $348;
          $435 = ((($F5$0$i)) + 12|0);
          HEAP32[$435>>2] = $348;
          $$sum15$i = (($246) + 8)|0;
          $436 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$436>>2] = $F5$0$i;
          $$sum16$i = (($246) + 12)|0;
          $437 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$437>>2] = $425;
          break;
         }
         $438 = $rsize$3$lcssa$i >>> 8;
         $439 = ($438|0)==(0);
         if ($439) {
          $I7$0$i = 0;
         } else {
          $440 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($440) {
           $I7$0$i = 31;
          } else {
           $441 = (($438) + 1048320)|0;
           $442 = $441 >>> 16;
           $443 = $442 & 8;
           $444 = $438 << $443;
           $445 = (($444) + 520192)|0;
           $446 = $445 >>> 16;
           $447 = $446 & 4;
           $448 = $447 | $443;
           $449 = $444 << $447;
           $450 = (($449) + 245760)|0;
           $451 = $450 >>> 16;
           $452 = $451 & 2;
           $453 = $448 | $452;
           $454 = (14 - ($453))|0;
           $455 = $449 << $452;
           $456 = $455 >>> 15;
           $457 = (($454) + ($456))|0;
           $458 = $457 << 1;
           $459 = (($457) + 7)|0;
           $460 = $rsize$3$lcssa$i >>> $459;
           $461 = $460 & 1;
           $462 = $461 | $458;
           $I7$0$i = $462;
          }
         }
         $463 = (13548 + ($I7$0$i<<2)|0);
         $$sum2$i = (($246) + 28)|0;
         $464 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$464>>2] = $I7$0$i;
         $$sum3$i27 = (($246) + 16)|0;
         $465 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($246) + 20)|0;
         $466 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$466>>2] = 0;
         HEAP32[$465>>2] = 0;
         $467 = HEAP32[(13248)>>2]|0;
         $468 = 1 << $I7$0$i;
         $469 = $467 & $468;
         $470 = ($469|0)==(0);
         if ($470) {
          $471 = $467 | $468;
          HEAP32[(13248)>>2] = $471;
          HEAP32[$463>>2] = $348;
          $$sum5$i = (($246) + 24)|0;
          $472 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$472>>2] = $463;
          $$sum6$i = (($246) + 12)|0;
          $473 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$473>>2] = $348;
          $$sum7$i = (($246) + 8)|0;
          $474 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$474>>2] = $348;
          break;
         }
         $475 = HEAP32[$463>>2]|0;
         $476 = ((($475)) + 4|0);
         $477 = HEAP32[$476>>2]|0;
         $478 = $477 & -8;
         $479 = ($478|0)==($rsize$3$lcssa$i|0);
         L217: do {
          if ($479) {
           $T$0$lcssa$i = $475;
          } else {
           $480 = ($I7$0$i|0)==(31);
           $481 = $I7$0$i >>> 1;
           $482 = (25 - ($481))|0;
           $483 = $480 ? 0 : $482;
           $484 = $rsize$3$lcssa$i << $483;
           $K12$029$i = $484;$T$028$i = $475;
           while(1) {
            $491 = $K12$029$i >>> 31;
            $492 = (((($T$028$i)) + 16|0) + ($491<<2)|0);
            $487 = HEAP32[$492>>2]|0;
            $493 = ($487|0)==(0|0);
            if ($493) {
             $$lcssa232 = $492;$T$028$i$lcssa = $T$028$i;
             break;
            }
            $485 = $K12$029$i << 1;
            $486 = ((($487)) + 4|0);
            $488 = HEAP32[$486>>2]|0;
            $489 = $488 & -8;
            $490 = ($489|0)==($rsize$3$lcssa$i|0);
            if ($490) {
             $T$0$lcssa$i = $487;
             break L217;
            } else {
             $K12$029$i = $485;$T$028$i = $487;
            }
           }
           $494 = HEAP32[(13260)>>2]|0;
           $495 = ($$lcssa232>>>0)<($494>>>0);
           if ($495) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$$lcssa232>>2] = $348;
            $$sum11$i = (($246) + 24)|0;
            $496 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$496>>2] = $T$028$i$lcssa;
            $$sum12$i = (($246) + 12)|0;
            $497 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$497>>2] = $348;
            $$sum13$i = (($246) + 8)|0;
            $498 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$498>>2] = $348;
            break L199;
           }
          }
         } while(0);
         $499 = ((($T$0$lcssa$i)) + 8|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[(13260)>>2]|0;
         $502 = ($500>>>0)>=($501>>>0);
         $not$$i = ($T$0$lcssa$i>>>0)>=($501>>>0);
         $503 = $502 & $not$$i;
         if ($503) {
          $504 = ((($500)) + 12|0);
          HEAP32[$504>>2] = $348;
          HEAP32[$499>>2] = $348;
          $$sum8$i = (($246) + 8)|0;
          $505 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$505>>2] = $500;
          $$sum9$i = (($246) + 12)|0;
          $506 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$506>>2] = $T$0$lcssa$i;
          $$sum10$i = (($246) + 24)|0;
          $507 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$507>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $508 = ((($v$3$lcssa$i)) + 8|0);
       $mem$0 = $508;
       return ($mem$0|0);
      } else {
       $nb$0 = $246;
      }
     }
    }
   }
  }
 } while(0);
 $509 = HEAP32[(13252)>>2]|0;
 $510 = ($509>>>0)<($nb$0>>>0);
 if (!($510)) {
  $511 = (($509) - ($nb$0))|0;
  $512 = HEAP32[(13264)>>2]|0;
  $513 = ($511>>>0)>(15);
  if ($513) {
   $514 = (($512) + ($nb$0)|0);
   HEAP32[(13264)>>2] = $514;
   HEAP32[(13252)>>2] = $511;
   $515 = $511 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $516 = (($512) + ($$sum2)|0);
   HEAP32[$516>>2] = $515;
   $517 = (($512) + ($509)|0);
   HEAP32[$517>>2] = $511;
   $518 = $nb$0 | 3;
   $519 = ((($512)) + 4|0);
   HEAP32[$519>>2] = $518;
  } else {
   HEAP32[(13252)>>2] = 0;
   HEAP32[(13264)>>2] = 0;
   $520 = $509 | 3;
   $521 = ((($512)) + 4|0);
   HEAP32[$521>>2] = $520;
   $$sum1 = (($509) + 4)|0;
   $522 = (($512) + ($$sum1)|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $523 | 1;
   HEAP32[$522>>2] = $524;
  }
  $525 = ((($512)) + 8|0);
  $mem$0 = $525;
  return ($mem$0|0);
 }
 $526 = HEAP32[(13256)>>2]|0;
 $527 = ($526>>>0)>($nb$0>>>0);
 if ($527) {
  $528 = (($526) - ($nb$0))|0;
  HEAP32[(13256)>>2] = $528;
  $529 = HEAP32[(13268)>>2]|0;
  $530 = (($529) + ($nb$0)|0);
  HEAP32[(13268)>>2] = $530;
  $531 = $528 | 1;
  $$sum = (($nb$0) + 4)|0;
  $532 = (($529) + ($$sum)|0);
  HEAP32[$532>>2] = $531;
  $533 = $nb$0 | 3;
  $534 = ((($529)) + 4|0);
  HEAP32[$534>>2] = $533;
  $535 = ((($529)) + 8|0);
  $mem$0 = $535;
  return ($mem$0|0);
 }
 $536 = HEAP32[13716>>2]|0;
 $537 = ($536|0)==(0);
 do {
  if ($537) {
   $538 = (_sysconf(30)|0);
   $539 = (($538) + -1)|0;
   $540 = $539 & $538;
   $541 = ($540|0)==(0);
   if ($541) {
    HEAP32[(13724)>>2] = $538;
    HEAP32[(13720)>>2] = $538;
    HEAP32[(13728)>>2] = -1;
    HEAP32[(13732)>>2] = -1;
    HEAP32[(13736)>>2] = 0;
    HEAP32[(13688)>>2] = 0;
    $542 = (_time((0|0))|0);
    $543 = $542 & -16;
    $544 = $543 ^ 1431655768;
    HEAP32[13716>>2] = $544;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $545 = (($nb$0) + 48)|0;
 $546 = HEAP32[(13724)>>2]|0;
 $547 = (($nb$0) + 47)|0;
 $548 = (($546) + ($547))|0;
 $549 = (0 - ($546))|0;
 $550 = $548 & $549;
 $551 = ($550>>>0)>($nb$0>>>0);
 if (!($551)) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $552 = HEAP32[(13684)>>2]|0;
 $553 = ($552|0)==(0);
 if (!($553)) {
  $554 = HEAP32[(13676)>>2]|0;
  $555 = (($554) + ($550))|0;
  $556 = ($555>>>0)<=($554>>>0);
  $557 = ($555>>>0)>($552>>>0);
  $or$cond1$i = $556 | $557;
  if ($or$cond1$i) {
   $mem$0 = 0;
   return ($mem$0|0);
  }
 }
 $558 = HEAP32[(13688)>>2]|0;
 $559 = $558 & 4;
 $560 = ($559|0)==(0);
 L258: do {
  if ($560) {
   $561 = HEAP32[(13268)>>2]|0;
   $562 = ($561|0)==(0|0);
   L260: do {
    if ($562) {
     label = 174;
    } else {
     $sp$0$i$i = (13692);
     while(1) {
      $563 = HEAP32[$sp$0$i$i>>2]|0;
      $564 = ($563>>>0)>($561>>>0);
      if (!($564)) {
       $565 = ((($sp$0$i$i)) + 4|0);
       $566 = HEAP32[$565>>2]|0;
       $567 = (($563) + ($566)|0);
       $568 = ($567>>>0)>($561>>>0);
       if ($568) {
        $$lcssa228 = $sp$0$i$i;$$lcssa230 = $565;
        break;
       }
      }
      $569 = ((($sp$0$i$i)) + 8|0);
      $570 = HEAP32[$569>>2]|0;
      $571 = ($570|0)==(0|0);
      if ($571) {
       label = 174;
       break L260;
      } else {
       $sp$0$i$i = $570;
      }
     }
     $594 = HEAP32[(13256)>>2]|0;
     $595 = (($548) - ($594))|0;
     $596 = $595 & $549;
     $597 = ($596>>>0)<(2147483647);
     if ($597) {
      $598 = (_sbrk(($596|0))|0);
      $599 = HEAP32[$$lcssa228>>2]|0;
      $600 = HEAP32[$$lcssa230>>2]|0;
      $601 = (($599) + ($600)|0);
      $602 = ($598|0)==($601|0);
      $$3$i = $602 ? $596 : 0;
      if ($602) {
       $603 = ($598|0)==((-1)|0);
       if ($603) {
        $tsize$0323944$i = $$3$i;
       } else {
        $tbase$255$i = $598;$tsize$254$i = $$3$i;
        label = 194;
        break L258;
       }
      } else {
       $br$0$ph$i = $598;$ssize$1$ph$i = $596;$tsize$0$ph$i = $$3$i;
       label = 184;
      }
     } else {
      $tsize$0323944$i = 0;
     }
    }
   } while(0);
   do {
    if ((label|0) == 174) {
     $572 = (_sbrk(0)|0);
     $573 = ($572|0)==((-1)|0);
     if ($573) {
      $tsize$0323944$i = 0;
     } else {
      $574 = $572;
      $575 = HEAP32[(13720)>>2]|0;
      $576 = (($575) + -1)|0;
      $577 = $576 & $574;
      $578 = ($577|0)==(0);
      if ($578) {
       $ssize$0$i = $550;
      } else {
       $579 = (($576) + ($574))|0;
       $580 = (0 - ($575))|0;
       $581 = $579 & $580;
       $582 = (($550) - ($574))|0;
       $583 = (($582) + ($581))|0;
       $ssize$0$i = $583;
      }
      $584 = HEAP32[(13676)>>2]|0;
      $585 = (($584) + ($ssize$0$i))|0;
      $586 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $587 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i30 = $586 & $587;
      if ($or$cond$i30) {
       $588 = HEAP32[(13684)>>2]|0;
       $589 = ($588|0)==(0);
       if (!($589)) {
        $590 = ($585>>>0)<=($584>>>0);
        $591 = ($585>>>0)>($588>>>0);
        $or$cond2$i = $590 | $591;
        if ($or$cond2$i) {
         $tsize$0323944$i = 0;
         break;
        }
       }
       $592 = (_sbrk(($ssize$0$i|0))|0);
       $593 = ($592|0)==($572|0);
       $ssize$0$$i = $593 ? $ssize$0$i : 0;
       if ($593) {
        $tbase$255$i = $572;$tsize$254$i = $ssize$0$$i;
        label = 194;
        break L258;
       } else {
        $br$0$ph$i = $592;$ssize$1$ph$i = $ssize$0$i;$tsize$0$ph$i = $ssize$0$$i;
        label = 184;
       }
      } else {
       $tsize$0323944$i = 0;
      }
     }
    }
   } while(0);
   L280: do {
    if ((label|0) == 184) {
     $604 = (0 - ($ssize$1$ph$i))|0;
     $605 = ($br$0$ph$i|0)!=((-1)|0);
     $606 = ($ssize$1$ph$i>>>0)<(2147483647);
     $or$cond5$i = $606 & $605;
     $607 = ($545>>>0)>($ssize$1$ph$i>>>0);
     $or$cond6$i = $607 & $or$cond5$i;
     do {
      if ($or$cond6$i) {
       $608 = HEAP32[(13724)>>2]|0;
       $609 = (($547) - ($ssize$1$ph$i))|0;
       $610 = (($609) + ($608))|0;
       $611 = (0 - ($608))|0;
       $612 = $610 & $611;
       $613 = ($612>>>0)<(2147483647);
       if ($613) {
        $614 = (_sbrk(($612|0))|0);
        $615 = ($614|0)==((-1)|0);
        if ($615) {
         (_sbrk(($604|0))|0);
         $tsize$0323944$i = $tsize$0$ph$i;
         break L280;
        } else {
         $616 = (($612) + ($ssize$1$ph$i))|0;
         $ssize$2$i = $616;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$ph$i;
       }
      } else {
       $ssize$2$i = $ssize$1$ph$i;
      }
     } while(0);
     $617 = ($br$0$ph$i|0)==((-1)|0);
     if ($617) {
      $tsize$0323944$i = $tsize$0$ph$i;
     } else {
      $tbase$255$i = $br$0$ph$i;$tsize$254$i = $ssize$2$i;
      label = 194;
      break L258;
     }
    }
   } while(0);
   $618 = HEAP32[(13688)>>2]|0;
   $619 = $618 | 4;
   HEAP32[(13688)>>2] = $619;
   $tsize$1$i = $tsize$0323944$i;
   label = 191;
  } else {
   $tsize$1$i = 0;
   label = 191;
  }
 } while(0);
 if ((label|0) == 191) {
  $620 = ($550>>>0)<(2147483647);
  if ($620) {
   $621 = (_sbrk(($550|0))|0);
   $622 = (_sbrk(0)|0);
   $623 = ($621|0)!=((-1)|0);
   $624 = ($622|0)!=((-1)|0);
   $or$cond3$i = $623 & $624;
   $625 = ($621>>>0)<($622>>>0);
   $or$cond8$i = $625 & $or$cond3$i;
   if ($or$cond8$i) {
    $626 = $622;
    $627 = $621;
    $628 = (($626) - ($627))|0;
    $629 = (($nb$0) + 40)|0;
    $630 = ($628>>>0)>($629>>>0);
    $$tsize$1$i = $630 ? $628 : $tsize$1$i;
    if ($630) {
     $tbase$255$i = $621;$tsize$254$i = $$tsize$1$i;
     label = 194;
    }
   }
  }
 }
 if ((label|0) == 194) {
  $631 = HEAP32[(13676)>>2]|0;
  $632 = (($631) + ($tsize$254$i))|0;
  HEAP32[(13676)>>2] = $632;
  $633 = HEAP32[(13680)>>2]|0;
  $634 = ($632>>>0)>($633>>>0);
  if ($634) {
   HEAP32[(13680)>>2] = $632;
  }
  $635 = HEAP32[(13268)>>2]|0;
  $636 = ($635|0)==(0|0);
  L299: do {
   if ($636) {
    $637 = HEAP32[(13260)>>2]|0;
    $638 = ($637|0)==(0|0);
    $639 = ($tbase$255$i>>>0)<($637>>>0);
    $or$cond9$i = $638 | $639;
    if ($or$cond9$i) {
     HEAP32[(13260)>>2] = $tbase$255$i;
    }
    HEAP32[(13692)>>2] = $tbase$255$i;
    HEAP32[(13696)>>2] = $tsize$254$i;
    HEAP32[(13704)>>2] = 0;
    $640 = HEAP32[13716>>2]|0;
    HEAP32[(13280)>>2] = $640;
    HEAP32[(13276)>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $641 = $i$02$i$i << 1;
     $642 = (13284 + ($641<<2)|0);
     $$sum$i$i = (($641) + 3)|0;
     $643 = (13284 + ($$sum$i$i<<2)|0);
     HEAP32[$643>>2] = $642;
     $$sum1$i$i = (($641) + 2)|0;
     $644 = (13284 + ($$sum1$i$i<<2)|0);
     HEAP32[$644>>2] = $642;
     $645 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($645|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $645;
     }
    }
    $646 = (($tsize$254$i) + -40)|0;
    $647 = ((($tbase$255$i)) + 8|0);
    $648 = $647;
    $649 = $648 & 7;
    $650 = ($649|0)==(0);
    $651 = (0 - ($648))|0;
    $652 = $651 & 7;
    $653 = $650 ? 0 : $652;
    $654 = (($tbase$255$i) + ($653)|0);
    $655 = (($646) - ($653))|0;
    HEAP32[(13268)>>2] = $654;
    HEAP32[(13256)>>2] = $655;
    $656 = $655 | 1;
    $$sum$i13$i = (($653) + 4)|0;
    $657 = (($tbase$255$i) + ($$sum$i13$i)|0);
    HEAP32[$657>>2] = $656;
    $$sum2$i$i = (($tsize$254$i) + -36)|0;
    $658 = (($tbase$255$i) + ($$sum2$i$i)|0);
    HEAP32[$658>>2] = 40;
    $659 = HEAP32[(13732)>>2]|0;
    HEAP32[(13272)>>2] = $659;
   } else {
    $sp$084$i = (13692);
    while(1) {
     $660 = HEAP32[$sp$084$i>>2]|0;
     $661 = ((($sp$084$i)) + 4|0);
     $662 = HEAP32[$661>>2]|0;
     $663 = (($660) + ($662)|0);
     $664 = ($tbase$255$i|0)==($663|0);
     if ($664) {
      $$lcssa222 = $660;$$lcssa224 = $661;$$lcssa226 = $662;$sp$084$i$lcssa = $sp$084$i;
      label = 204;
      break;
     }
     $665 = ((($sp$084$i)) + 8|0);
     $666 = HEAP32[$665>>2]|0;
     $667 = ($666|0)==(0|0);
     if ($667) {
      break;
     } else {
      $sp$084$i = $666;
     }
    }
    if ((label|0) == 204) {
     $668 = ((($sp$084$i$lcssa)) + 12|0);
     $669 = HEAP32[$668>>2]|0;
     $670 = $669 & 8;
     $671 = ($670|0)==(0);
     if ($671) {
      $672 = ($635>>>0)>=($$lcssa222>>>0);
      $673 = ($635>>>0)<($tbase$255$i>>>0);
      $or$cond57$i = $673 & $672;
      if ($or$cond57$i) {
       $674 = (($$lcssa226) + ($tsize$254$i))|0;
       HEAP32[$$lcssa224>>2] = $674;
       $675 = HEAP32[(13256)>>2]|0;
       $676 = (($675) + ($tsize$254$i))|0;
       $677 = ((($635)) + 8|0);
       $678 = $677;
       $679 = $678 & 7;
       $680 = ($679|0)==(0);
       $681 = (0 - ($678))|0;
       $682 = $681 & 7;
       $683 = $680 ? 0 : $682;
       $684 = (($635) + ($683)|0);
       $685 = (($676) - ($683))|0;
       HEAP32[(13268)>>2] = $684;
       HEAP32[(13256)>>2] = $685;
       $686 = $685 | 1;
       $$sum$i17$i = (($683) + 4)|0;
       $687 = (($635) + ($$sum$i17$i)|0);
       HEAP32[$687>>2] = $686;
       $$sum2$i18$i = (($676) + 4)|0;
       $688 = (($635) + ($$sum2$i18$i)|0);
       HEAP32[$688>>2] = 40;
       $689 = HEAP32[(13732)>>2]|0;
       HEAP32[(13272)>>2] = $689;
       break;
      }
     }
    }
    $690 = HEAP32[(13260)>>2]|0;
    $691 = ($tbase$255$i>>>0)<($690>>>0);
    if ($691) {
     HEAP32[(13260)>>2] = $tbase$255$i;
     $755 = $tbase$255$i;
    } else {
     $755 = $690;
    }
    $692 = (($tbase$255$i) + ($tsize$254$i)|0);
    $sp$183$i = (13692);
    while(1) {
     $693 = HEAP32[$sp$183$i>>2]|0;
     $694 = ($693|0)==($692|0);
     if ($694) {
      $$lcssa219 = $sp$183$i;$sp$183$i$lcssa = $sp$183$i;
      label = 212;
      break;
     }
     $695 = ((($sp$183$i)) + 8|0);
     $696 = HEAP32[$695>>2]|0;
     $697 = ($696|0)==(0|0);
     if ($697) {
      $sp$0$i$i$i = (13692);
      break;
     } else {
      $sp$183$i = $696;
     }
    }
    if ((label|0) == 212) {
     $698 = ((($sp$183$i$lcssa)) + 12|0);
     $699 = HEAP32[$698>>2]|0;
     $700 = $699 & 8;
     $701 = ($700|0)==(0);
     if ($701) {
      HEAP32[$$lcssa219>>2] = $tbase$255$i;
      $702 = ((($sp$183$i$lcssa)) + 4|0);
      $703 = HEAP32[$702>>2]|0;
      $704 = (($703) + ($tsize$254$i))|0;
      HEAP32[$702>>2] = $704;
      $705 = ((($tbase$255$i)) + 8|0);
      $706 = $705;
      $707 = $706 & 7;
      $708 = ($707|0)==(0);
      $709 = (0 - ($706))|0;
      $710 = $709 & 7;
      $711 = $708 ? 0 : $710;
      $712 = (($tbase$255$i) + ($711)|0);
      $$sum112$i = (($tsize$254$i) + 8)|0;
      $713 = (($tbase$255$i) + ($$sum112$i)|0);
      $714 = $713;
      $715 = $714 & 7;
      $716 = ($715|0)==(0);
      $717 = (0 - ($714))|0;
      $718 = $717 & 7;
      $719 = $716 ? 0 : $718;
      $$sum113$i = (($719) + ($tsize$254$i))|0;
      $720 = (($tbase$255$i) + ($$sum113$i)|0);
      $721 = $720;
      $722 = $712;
      $723 = (($721) - ($722))|0;
      $$sum$i19$i = (($711) + ($nb$0))|0;
      $724 = (($tbase$255$i) + ($$sum$i19$i)|0);
      $725 = (($723) - ($nb$0))|0;
      $726 = $nb$0 | 3;
      $$sum1$i20$i = (($711) + 4)|0;
      $727 = (($tbase$255$i) + ($$sum1$i20$i)|0);
      HEAP32[$727>>2] = $726;
      $728 = ($720|0)==($635|0);
      L324: do {
       if ($728) {
        $729 = HEAP32[(13256)>>2]|0;
        $730 = (($729) + ($725))|0;
        HEAP32[(13256)>>2] = $730;
        HEAP32[(13268)>>2] = $724;
        $731 = $730 | 1;
        $$sum42$i$i = (($$sum$i19$i) + 4)|0;
        $732 = (($tbase$255$i) + ($$sum42$i$i)|0);
        HEAP32[$732>>2] = $731;
       } else {
        $733 = HEAP32[(13264)>>2]|0;
        $734 = ($720|0)==($733|0);
        if ($734) {
         $735 = HEAP32[(13252)>>2]|0;
         $736 = (($735) + ($725))|0;
         HEAP32[(13252)>>2] = $736;
         HEAP32[(13264)>>2] = $724;
         $737 = $736 | 1;
         $$sum40$i$i = (($$sum$i19$i) + 4)|0;
         $738 = (($tbase$255$i) + ($$sum40$i$i)|0);
         HEAP32[$738>>2] = $737;
         $$sum41$i$i = (($736) + ($$sum$i19$i))|0;
         $739 = (($tbase$255$i) + ($$sum41$i$i)|0);
         HEAP32[$739>>2] = $736;
         break;
        }
        $$sum2$i21$i = (($tsize$254$i) + 4)|0;
        $$sum114$i = (($$sum2$i21$i) + ($719))|0;
        $740 = (($tbase$255$i) + ($$sum114$i)|0);
        $741 = HEAP32[$740>>2]|0;
        $742 = $741 & 3;
        $743 = ($742|0)==(1);
        if ($743) {
         $744 = $741 & -8;
         $745 = $741 >>> 3;
         $746 = ($741>>>0)<(256);
         L332: do {
          if ($746) {
           $$sum3738$i$i = $719 | 8;
           $$sum124$i = (($$sum3738$i$i) + ($tsize$254$i))|0;
           $747 = (($tbase$255$i) + ($$sum124$i)|0);
           $748 = HEAP32[$747>>2]|0;
           $$sum39$i$i = (($tsize$254$i) + 12)|0;
           $$sum125$i = (($$sum39$i$i) + ($719))|0;
           $749 = (($tbase$255$i) + ($$sum125$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $751 = $745 << 1;
           $752 = (13284 + ($751<<2)|0);
           $753 = ($748|0)==($752|0);
           do {
            if (!($753)) {
             $754 = ($748>>>0)<($755>>>0);
             if ($754) {
              _abort();
              // unreachable;
             }
             $756 = ((($748)) + 12|0);
             $757 = HEAP32[$756>>2]|0;
             $758 = ($757|0)==($720|0);
             if ($758) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $759 = ($750|0)==($748|0);
           if ($759) {
            $760 = 1 << $745;
            $761 = $760 ^ -1;
            $762 = HEAP32[13244>>2]|0;
            $763 = $762 & $761;
            HEAP32[13244>>2] = $763;
            break;
           }
           $764 = ($750|0)==($752|0);
           do {
            if ($764) {
             $$pre57$i$i = ((($750)) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $765 = ($750>>>0)<($755>>>0);
             if ($765) {
              _abort();
              // unreachable;
             }
             $766 = ((($750)) + 8|0);
             $767 = HEAP32[$766>>2]|0;
             $768 = ($767|0)==($720|0);
             if ($768) {
              $$pre$phi58$i$iZ2D = $766;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $769 = ((($748)) + 12|0);
           HEAP32[$769>>2] = $750;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $748;
          } else {
           $$sum34$i$i = $719 | 24;
           $$sum115$i = (($$sum34$i$i) + ($tsize$254$i))|0;
           $770 = (($tbase$255$i) + ($$sum115$i)|0);
           $771 = HEAP32[$770>>2]|0;
           $$sum5$i$i = (($tsize$254$i) + 12)|0;
           $$sum116$i = (($$sum5$i$i) + ($719))|0;
           $772 = (($tbase$255$i) + ($$sum116$i)|0);
           $773 = HEAP32[$772>>2]|0;
           $774 = ($773|0)==($720|0);
           do {
            if ($774) {
             $$sum67$i$i = $719 | 16;
             $$sum122$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
             $784 = (($tbase$255$i) + ($$sum122$i)|0);
             $785 = HEAP32[$784>>2]|0;
             $786 = ($785|0)==(0|0);
             if ($786) {
              $$sum123$i = (($$sum67$i$i) + ($tsize$254$i))|0;
              $787 = (($tbase$255$i) + ($$sum123$i)|0);
              $788 = HEAP32[$787>>2]|0;
              $789 = ($788|0)==(0|0);
              if ($789) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $788;$RP$0$i$i = $787;
              }
             } else {
              $R$0$i$i = $785;$RP$0$i$i = $784;
             }
             while(1) {
              $790 = ((($R$0$i$i)) + 20|0);
              $791 = HEAP32[$790>>2]|0;
              $792 = ($791|0)==(0|0);
              if (!($792)) {
               $R$0$i$i = $791;$RP$0$i$i = $790;
               continue;
              }
              $793 = ((($R$0$i$i)) + 16|0);
              $794 = HEAP32[$793>>2]|0;
              $795 = ($794|0)==(0|0);
              if ($795) {
               $R$0$i$i$lcssa = $R$0$i$i;$RP$0$i$i$lcssa = $RP$0$i$i;
               break;
              } else {
               $R$0$i$i = $794;$RP$0$i$i = $793;
              }
             }
             $796 = ($RP$0$i$i$lcssa>>>0)<($755>>>0);
             if ($796) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i$lcssa>>2] = 0;
              $R$1$i$i = $R$0$i$i$lcssa;
              break;
             }
            } else {
             $$sum3536$i$i = $719 | 8;
             $$sum117$i = (($$sum3536$i$i) + ($tsize$254$i))|0;
             $775 = (($tbase$255$i) + ($$sum117$i)|0);
             $776 = HEAP32[$775>>2]|0;
             $777 = ($776>>>0)<($755>>>0);
             if ($777) {
              _abort();
              // unreachable;
             }
             $778 = ((($776)) + 12|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = ($779|0)==($720|0);
             if (!($780)) {
              _abort();
              // unreachable;
             }
             $781 = ((($773)) + 8|0);
             $782 = HEAP32[$781>>2]|0;
             $783 = ($782|0)==($720|0);
             if ($783) {
              HEAP32[$778>>2] = $773;
              HEAP32[$781>>2] = $776;
              $R$1$i$i = $773;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $797 = ($771|0)==(0|0);
           if ($797) {
            break;
           }
           $$sum30$i$i = (($tsize$254$i) + 28)|0;
           $$sum118$i = (($$sum30$i$i) + ($719))|0;
           $798 = (($tbase$255$i) + ($$sum118$i)|0);
           $799 = HEAP32[$798>>2]|0;
           $800 = (13548 + ($799<<2)|0);
           $801 = HEAP32[$800>>2]|0;
           $802 = ($720|0)==($801|0);
           do {
            if ($802) {
             HEAP32[$800>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $803 = 1 << $799;
             $804 = $803 ^ -1;
             $805 = HEAP32[(13248)>>2]|0;
             $806 = $805 & $804;
             HEAP32[(13248)>>2] = $806;
             break L332;
            } else {
             $807 = HEAP32[(13260)>>2]|0;
             $808 = ($771>>>0)<($807>>>0);
             if ($808) {
              _abort();
              // unreachable;
             }
             $809 = ((($771)) + 16|0);
             $810 = HEAP32[$809>>2]|0;
             $811 = ($810|0)==($720|0);
             if ($811) {
              HEAP32[$809>>2] = $R$1$i$i;
             } else {
              $812 = ((($771)) + 20|0);
              HEAP32[$812>>2] = $R$1$i$i;
             }
             $813 = ($R$1$i$i|0)==(0|0);
             if ($813) {
              break L332;
             }
            }
           } while(0);
           $814 = HEAP32[(13260)>>2]|0;
           $815 = ($R$1$i$i>>>0)<($814>>>0);
           if ($815) {
            _abort();
            // unreachable;
           }
           $816 = ((($R$1$i$i)) + 24|0);
           HEAP32[$816>>2] = $771;
           $$sum3132$i$i = $719 | 16;
           $$sum119$i = (($$sum3132$i$i) + ($tsize$254$i))|0;
           $817 = (($tbase$255$i) + ($$sum119$i)|0);
           $818 = HEAP32[$817>>2]|0;
           $819 = ($818|0)==(0|0);
           do {
            if (!($819)) {
             $820 = ($818>>>0)<($814>>>0);
             if ($820) {
              _abort();
              // unreachable;
             } else {
              $821 = ((($R$1$i$i)) + 16|0);
              HEAP32[$821>>2] = $818;
              $822 = ((($818)) + 24|0);
              HEAP32[$822>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum120$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
           $823 = (($tbase$255$i) + ($$sum120$i)|0);
           $824 = HEAP32[$823>>2]|0;
           $825 = ($824|0)==(0|0);
           if ($825) {
            break;
           }
           $826 = HEAP32[(13260)>>2]|0;
           $827 = ($824>>>0)<($826>>>0);
           if ($827) {
            _abort();
            // unreachable;
           } else {
            $828 = ((($R$1$i$i)) + 20|0);
            HEAP32[$828>>2] = $824;
            $829 = ((($824)) + 24|0);
            HEAP32[$829>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $744 | $719;
         $$sum121$i = (($$sum9$i$i) + ($tsize$254$i))|0;
         $830 = (($tbase$255$i) + ($$sum121$i)|0);
         $831 = (($744) + ($725))|0;
         $oldfirst$0$i$i = $830;$qsize$0$i$i = $831;
        } else {
         $oldfirst$0$i$i = $720;$qsize$0$i$i = $725;
        }
        $832 = ((($oldfirst$0$i$i)) + 4|0);
        $833 = HEAP32[$832>>2]|0;
        $834 = $833 & -2;
        HEAP32[$832>>2] = $834;
        $835 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i19$i) + 4)|0;
        $836 = (($tbase$255$i) + ($$sum10$i$i)|0);
        HEAP32[$836>>2] = $835;
        $$sum11$i$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
        $837 = (($tbase$255$i) + ($$sum11$i$i)|0);
        HEAP32[$837>>2] = $qsize$0$i$i;
        $838 = $qsize$0$i$i >>> 3;
        $839 = ($qsize$0$i$i>>>0)<(256);
        if ($839) {
         $840 = $838 << 1;
         $841 = (13284 + ($840<<2)|0);
         $842 = HEAP32[13244>>2]|0;
         $843 = 1 << $838;
         $844 = $842 & $843;
         $845 = ($844|0)==(0);
         do {
          if ($845) {
           $846 = $842 | $843;
           HEAP32[13244>>2] = $846;
           $$pre$i22$i = (($840) + 2)|0;
           $$pre56$i$i = (13284 + ($$pre$i22$i<<2)|0);
           $$pre$phi$i23$iZ2D = $$pre56$i$i;$F4$0$i$i = $841;
          } else {
           $$sum29$i$i = (($840) + 2)|0;
           $847 = (13284 + ($$sum29$i$i<<2)|0);
           $848 = HEAP32[$847>>2]|0;
           $849 = HEAP32[(13260)>>2]|0;
           $850 = ($848>>>0)<($849>>>0);
           if (!($850)) {
            $$pre$phi$i23$iZ2D = $847;$F4$0$i$i = $848;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i23$iZ2D>>2] = $724;
         $851 = ((($F4$0$i$i)) + 12|0);
         HEAP32[$851>>2] = $724;
         $$sum27$i$i = (($$sum$i19$i) + 8)|0;
         $852 = (($tbase$255$i) + ($$sum27$i$i)|0);
         HEAP32[$852>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i19$i) + 12)|0;
         $853 = (($tbase$255$i) + ($$sum28$i$i)|0);
         HEAP32[$853>>2] = $841;
         break;
        }
        $854 = $qsize$0$i$i >>> 8;
        $855 = ($854|0)==(0);
        do {
         if ($855) {
          $I7$0$i$i = 0;
         } else {
          $856 = ($qsize$0$i$i>>>0)>(16777215);
          if ($856) {
           $I7$0$i$i = 31;
           break;
          }
          $857 = (($854) + 1048320)|0;
          $858 = $857 >>> 16;
          $859 = $858 & 8;
          $860 = $854 << $859;
          $861 = (($860) + 520192)|0;
          $862 = $861 >>> 16;
          $863 = $862 & 4;
          $864 = $863 | $859;
          $865 = $860 << $863;
          $866 = (($865) + 245760)|0;
          $867 = $866 >>> 16;
          $868 = $867 & 2;
          $869 = $864 | $868;
          $870 = (14 - ($869))|0;
          $871 = $865 << $868;
          $872 = $871 >>> 15;
          $873 = (($870) + ($872))|0;
          $874 = $873 << 1;
          $875 = (($873) + 7)|0;
          $876 = $qsize$0$i$i >>> $875;
          $877 = $876 & 1;
          $878 = $877 | $874;
          $I7$0$i$i = $878;
         }
        } while(0);
        $879 = (13548 + ($I7$0$i$i<<2)|0);
        $$sum12$i$i = (($$sum$i19$i) + 28)|0;
        $880 = (($tbase$255$i) + ($$sum12$i$i)|0);
        HEAP32[$880>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i19$i) + 16)|0;
        $881 = (($tbase$255$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i19$i) + 20)|0;
        $882 = (($tbase$255$i) + ($$sum14$i$i)|0);
        HEAP32[$882>>2] = 0;
        HEAP32[$881>>2] = 0;
        $883 = HEAP32[(13248)>>2]|0;
        $884 = 1 << $I7$0$i$i;
        $885 = $883 & $884;
        $886 = ($885|0)==(0);
        if ($886) {
         $887 = $883 | $884;
         HEAP32[(13248)>>2] = $887;
         HEAP32[$879>>2] = $724;
         $$sum15$i$i = (($$sum$i19$i) + 24)|0;
         $888 = (($tbase$255$i) + ($$sum15$i$i)|0);
         HEAP32[$888>>2] = $879;
         $$sum16$i$i = (($$sum$i19$i) + 12)|0;
         $889 = (($tbase$255$i) + ($$sum16$i$i)|0);
         HEAP32[$889>>2] = $724;
         $$sum17$i$i = (($$sum$i19$i) + 8)|0;
         $890 = (($tbase$255$i) + ($$sum17$i$i)|0);
         HEAP32[$890>>2] = $724;
         break;
        }
        $891 = HEAP32[$879>>2]|0;
        $892 = ((($891)) + 4|0);
        $893 = HEAP32[$892>>2]|0;
        $894 = $893 & -8;
        $895 = ($894|0)==($qsize$0$i$i|0);
        L418: do {
         if ($895) {
          $T$0$lcssa$i25$i = $891;
         } else {
          $896 = ($I7$0$i$i|0)==(31);
          $897 = $I7$0$i$i >>> 1;
          $898 = (25 - ($897))|0;
          $899 = $896 ? 0 : $898;
          $900 = $qsize$0$i$i << $899;
          $K8$051$i$i = $900;$T$050$i$i = $891;
          while(1) {
           $907 = $K8$051$i$i >>> 31;
           $908 = (((($T$050$i$i)) + 16|0) + ($907<<2)|0);
           $903 = HEAP32[$908>>2]|0;
           $909 = ($903|0)==(0|0);
           if ($909) {
            $$lcssa = $908;$T$050$i$i$lcssa = $T$050$i$i;
            break;
           }
           $901 = $K8$051$i$i << 1;
           $902 = ((($903)) + 4|0);
           $904 = HEAP32[$902>>2]|0;
           $905 = $904 & -8;
           $906 = ($905|0)==($qsize$0$i$i|0);
           if ($906) {
            $T$0$lcssa$i25$i = $903;
            break L418;
           } else {
            $K8$051$i$i = $901;$T$050$i$i = $903;
           }
          }
          $910 = HEAP32[(13260)>>2]|0;
          $911 = ($$lcssa>>>0)<($910>>>0);
          if ($911) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$$lcssa>>2] = $724;
           $$sum23$i$i = (($$sum$i19$i) + 24)|0;
           $912 = (($tbase$255$i) + ($$sum23$i$i)|0);
           HEAP32[$912>>2] = $T$050$i$i$lcssa;
           $$sum24$i$i = (($$sum$i19$i) + 12)|0;
           $913 = (($tbase$255$i) + ($$sum24$i$i)|0);
           HEAP32[$913>>2] = $724;
           $$sum25$i$i = (($$sum$i19$i) + 8)|0;
           $914 = (($tbase$255$i) + ($$sum25$i$i)|0);
           HEAP32[$914>>2] = $724;
           break L324;
          }
         }
        } while(0);
        $915 = ((($T$0$lcssa$i25$i)) + 8|0);
        $916 = HEAP32[$915>>2]|0;
        $917 = HEAP32[(13260)>>2]|0;
        $918 = ($916>>>0)>=($917>>>0);
        $not$$i26$i = ($T$0$lcssa$i25$i>>>0)>=($917>>>0);
        $919 = $918 & $not$$i26$i;
        if ($919) {
         $920 = ((($916)) + 12|0);
         HEAP32[$920>>2] = $724;
         HEAP32[$915>>2] = $724;
         $$sum20$i$i = (($$sum$i19$i) + 8)|0;
         $921 = (($tbase$255$i) + ($$sum20$i$i)|0);
         HEAP32[$921>>2] = $916;
         $$sum21$i$i = (($$sum$i19$i) + 12)|0;
         $922 = (($tbase$255$i) + ($$sum21$i$i)|0);
         HEAP32[$922>>2] = $T$0$lcssa$i25$i;
         $$sum22$i$i = (($$sum$i19$i) + 24)|0;
         $923 = (($tbase$255$i) + ($$sum22$i$i)|0);
         HEAP32[$923>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $$sum1819$i$i = $711 | 8;
      $924 = (($tbase$255$i) + ($$sum1819$i$i)|0);
      $mem$0 = $924;
      return ($mem$0|0);
     } else {
      $sp$0$i$i$i = (13692);
     }
    }
    while(1) {
     $925 = HEAP32[$sp$0$i$i$i>>2]|0;
     $926 = ($925>>>0)>($635>>>0);
     if (!($926)) {
      $927 = ((($sp$0$i$i$i)) + 4|0);
      $928 = HEAP32[$927>>2]|0;
      $929 = (($925) + ($928)|0);
      $930 = ($929>>>0)>($635>>>0);
      if ($930) {
       $$lcssa215 = $925;$$lcssa216 = $928;$$lcssa217 = $929;
       break;
      }
     }
     $931 = ((($sp$0$i$i$i)) + 8|0);
     $932 = HEAP32[$931>>2]|0;
     $sp$0$i$i$i = $932;
    }
    $$sum$i14$i = (($$lcssa216) + -47)|0;
    $$sum1$i15$i = (($$lcssa216) + -39)|0;
    $933 = (($$lcssa215) + ($$sum1$i15$i)|0);
    $934 = $933;
    $935 = $934 & 7;
    $936 = ($935|0)==(0);
    $937 = (0 - ($934))|0;
    $938 = $937 & 7;
    $939 = $936 ? 0 : $938;
    $$sum2$i16$i = (($$sum$i14$i) + ($939))|0;
    $940 = (($$lcssa215) + ($$sum2$i16$i)|0);
    $941 = ((($635)) + 16|0);
    $942 = ($940>>>0)<($941>>>0);
    $943 = $942 ? $635 : $940;
    $944 = ((($943)) + 8|0);
    $945 = (($tsize$254$i) + -40)|0;
    $946 = ((($tbase$255$i)) + 8|0);
    $947 = $946;
    $948 = $947 & 7;
    $949 = ($948|0)==(0);
    $950 = (0 - ($947))|0;
    $951 = $950 & 7;
    $952 = $949 ? 0 : $951;
    $953 = (($tbase$255$i) + ($952)|0);
    $954 = (($945) - ($952))|0;
    HEAP32[(13268)>>2] = $953;
    HEAP32[(13256)>>2] = $954;
    $955 = $954 | 1;
    $$sum$i$i$i = (($952) + 4)|0;
    $956 = (($tbase$255$i) + ($$sum$i$i$i)|0);
    HEAP32[$956>>2] = $955;
    $$sum2$i$i$i = (($tsize$254$i) + -36)|0;
    $957 = (($tbase$255$i) + ($$sum2$i$i$i)|0);
    HEAP32[$957>>2] = 40;
    $958 = HEAP32[(13732)>>2]|0;
    HEAP32[(13272)>>2] = $958;
    $959 = ((($943)) + 4|0);
    HEAP32[$959>>2] = 27;
    ;HEAP32[$944>>2]=HEAP32[(13692)>>2]|0;HEAP32[$944+4>>2]=HEAP32[(13692)+4>>2]|0;HEAP32[$944+8>>2]=HEAP32[(13692)+8>>2]|0;HEAP32[$944+12>>2]=HEAP32[(13692)+12>>2]|0;
    HEAP32[(13692)>>2] = $tbase$255$i;
    HEAP32[(13696)>>2] = $tsize$254$i;
    HEAP32[(13704)>>2] = 0;
    HEAP32[(13700)>>2] = $944;
    $960 = ((($943)) + 28|0);
    HEAP32[$960>>2] = 7;
    $961 = ((($943)) + 32|0);
    $962 = ($961>>>0)<($$lcssa217>>>0);
    if ($962) {
     $964 = $960;
     while(1) {
      $963 = ((($964)) + 4|0);
      HEAP32[$963>>2] = 7;
      $965 = ((($964)) + 8|0);
      $966 = ($965>>>0)<($$lcssa217>>>0);
      if ($966) {
       $964 = $963;
      } else {
       break;
      }
     }
    }
    $967 = ($943|0)==($635|0);
    if (!($967)) {
     $968 = $943;
     $969 = $635;
     $970 = (($968) - ($969))|0;
     $971 = HEAP32[$959>>2]|0;
     $972 = $971 & -2;
     HEAP32[$959>>2] = $972;
     $973 = $970 | 1;
     $974 = ((($635)) + 4|0);
     HEAP32[$974>>2] = $973;
     HEAP32[$943>>2] = $970;
     $975 = $970 >>> 3;
     $976 = ($970>>>0)<(256);
     if ($976) {
      $977 = $975 << 1;
      $978 = (13284 + ($977<<2)|0);
      $979 = HEAP32[13244>>2]|0;
      $980 = 1 << $975;
      $981 = $979 & $980;
      $982 = ($981|0)==(0);
      if ($982) {
       $983 = $979 | $980;
       HEAP32[13244>>2] = $983;
       $$pre$i$i = (($977) + 2)|0;
       $$pre14$i$i = (13284 + ($$pre$i$i<<2)|0);
       $$pre$phi$i$iZ2D = $$pre14$i$i;$F$0$i$i = $978;
      } else {
       $$sum4$i$i = (($977) + 2)|0;
       $984 = (13284 + ($$sum4$i$i<<2)|0);
       $985 = HEAP32[$984>>2]|0;
       $986 = HEAP32[(13260)>>2]|0;
       $987 = ($985>>>0)<($986>>>0);
       if ($987) {
        _abort();
        // unreachable;
       } else {
        $$pre$phi$i$iZ2D = $984;$F$0$i$i = $985;
       }
      }
      HEAP32[$$pre$phi$i$iZ2D>>2] = $635;
      $988 = ((($F$0$i$i)) + 12|0);
      HEAP32[$988>>2] = $635;
      $989 = ((($635)) + 8|0);
      HEAP32[$989>>2] = $F$0$i$i;
      $990 = ((($635)) + 12|0);
      HEAP32[$990>>2] = $978;
      break;
     }
     $991 = $970 >>> 8;
     $992 = ($991|0)==(0);
     if ($992) {
      $I1$0$i$i = 0;
     } else {
      $993 = ($970>>>0)>(16777215);
      if ($993) {
       $I1$0$i$i = 31;
      } else {
       $994 = (($991) + 1048320)|0;
       $995 = $994 >>> 16;
       $996 = $995 & 8;
       $997 = $991 << $996;
       $998 = (($997) + 520192)|0;
       $999 = $998 >>> 16;
       $1000 = $999 & 4;
       $1001 = $1000 | $996;
       $1002 = $997 << $1000;
       $1003 = (($1002) + 245760)|0;
       $1004 = $1003 >>> 16;
       $1005 = $1004 & 2;
       $1006 = $1001 | $1005;
       $1007 = (14 - ($1006))|0;
       $1008 = $1002 << $1005;
       $1009 = $1008 >>> 15;
       $1010 = (($1007) + ($1009))|0;
       $1011 = $1010 << 1;
       $1012 = (($1010) + 7)|0;
       $1013 = $970 >>> $1012;
       $1014 = $1013 & 1;
       $1015 = $1014 | $1011;
       $I1$0$i$i = $1015;
      }
     }
     $1016 = (13548 + ($I1$0$i$i<<2)|0);
     $1017 = ((($635)) + 28|0);
     HEAP32[$1017>>2] = $I1$0$i$i;
     $1018 = ((($635)) + 20|0);
     HEAP32[$1018>>2] = 0;
     HEAP32[$941>>2] = 0;
     $1019 = HEAP32[(13248)>>2]|0;
     $1020 = 1 << $I1$0$i$i;
     $1021 = $1019 & $1020;
     $1022 = ($1021|0)==(0);
     if ($1022) {
      $1023 = $1019 | $1020;
      HEAP32[(13248)>>2] = $1023;
      HEAP32[$1016>>2] = $635;
      $1024 = ((($635)) + 24|0);
      HEAP32[$1024>>2] = $1016;
      $1025 = ((($635)) + 12|0);
      HEAP32[$1025>>2] = $635;
      $1026 = ((($635)) + 8|0);
      HEAP32[$1026>>2] = $635;
      break;
     }
     $1027 = HEAP32[$1016>>2]|0;
     $1028 = ((($1027)) + 4|0);
     $1029 = HEAP32[$1028>>2]|0;
     $1030 = $1029 & -8;
     $1031 = ($1030|0)==($970|0);
     L459: do {
      if ($1031) {
       $T$0$lcssa$i$i = $1027;
      } else {
       $1032 = ($I1$0$i$i|0)==(31);
       $1033 = $I1$0$i$i >>> 1;
       $1034 = (25 - ($1033))|0;
       $1035 = $1032 ? 0 : $1034;
       $1036 = $970 << $1035;
       $K2$07$i$i = $1036;$T$06$i$i = $1027;
       while(1) {
        $1043 = $K2$07$i$i >>> 31;
        $1044 = (((($T$06$i$i)) + 16|0) + ($1043<<2)|0);
        $1039 = HEAP32[$1044>>2]|0;
        $1045 = ($1039|0)==(0|0);
        if ($1045) {
         $$lcssa211 = $1044;$T$06$i$i$lcssa = $T$06$i$i;
         break;
        }
        $1037 = $K2$07$i$i << 1;
        $1038 = ((($1039)) + 4|0);
        $1040 = HEAP32[$1038>>2]|0;
        $1041 = $1040 & -8;
        $1042 = ($1041|0)==($970|0);
        if ($1042) {
         $T$0$lcssa$i$i = $1039;
         break L459;
        } else {
         $K2$07$i$i = $1037;$T$06$i$i = $1039;
        }
       }
       $1046 = HEAP32[(13260)>>2]|0;
       $1047 = ($$lcssa211>>>0)<($1046>>>0);
       if ($1047) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$$lcssa211>>2] = $635;
        $1048 = ((($635)) + 24|0);
        HEAP32[$1048>>2] = $T$06$i$i$lcssa;
        $1049 = ((($635)) + 12|0);
        HEAP32[$1049>>2] = $635;
        $1050 = ((($635)) + 8|0);
        HEAP32[$1050>>2] = $635;
        break L299;
       }
      }
     } while(0);
     $1051 = ((($T$0$lcssa$i$i)) + 8|0);
     $1052 = HEAP32[$1051>>2]|0;
     $1053 = HEAP32[(13260)>>2]|0;
     $1054 = ($1052>>>0)>=($1053>>>0);
     $not$$i$i = ($T$0$lcssa$i$i>>>0)>=($1053>>>0);
     $1055 = $1054 & $not$$i$i;
     if ($1055) {
      $1056 = ((($1052)) + 12|0);
      HEAP32[$1056>>2] = $635;
      HEAP32[$1051>>2] = $635;
      $1057 = ((($635)) + 8|0);
      HEAP32[$1057>>2] = $1052;
      $1058 = ((($635)) + 12|0);
      HEAP32[$1058>>2] = $T$0$lcssa$i$i;
      $1059 = ((($635)) + 24|0);
      HEAP32[$1059>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   }
  } while(0);
  $1060 = HEAP32[(13256)>>2]|0;
  $1061 = ($1060>>>0)>($nb$0>>>0);
  if ($1061) {
   $1062 = (($1060) - ($nb$0))|0;
   HEAP32[(13256)>>2] = $1062;
   $1063 = HEAP32[(13268)>>2]|0;
   $1064 = (($1063) + ($nb$0)|0);
   HEAP32[(13268)>>2] = $1064;
   $1065 = $1062 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1066 = (($1063) + ($$sum$i32)|0);
   HEAP32[$1066>>2] = $1065;
   $1067 = $nb$0 | 3;
   $1068 = ((($1063)) + 4|0);
   HEAP32[$1068>>2] = $1067;
   $1069 = ((($1063)) + 8|0);
   $mem$0 = $1069;
   return ($mem$0|0);
  }
 }
 $1070 = (___errno_location()|0);
 HEAP32[$1070>>2] = 12;
 $mem$0 = 0;
 return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$lcssa = 0, $$pre = 0, $$pre$phi59Z2D = 0, $$pre$phi61Z2D = 0, $$pre$phiZ2D = 0, $$pre57 = 0, $$pre58 = 0, $$pre60 = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum1718 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0;
 var $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0;
 var $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0;
 var $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0;
 var $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0;
 var $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0;
 var $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0;
 var $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0;
 var $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0;
 var $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0;
 var $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0;
 var $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0;
 var $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0;
 var $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0;
 var $321 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $K19$052 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0;
 var $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$051 = 0, $T$051$lcssa = 0, $cond = 0, $cond47 = 0, $not$ = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  return;
 }
 $1 = ((($mem)) + -8|0);
 $2 = HEAP32[(13260)>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = ((($mem)) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[(13264)>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $103 = (($mem) + ($$sum3)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $104 & 3;
    $106 = ($105|0)==(3);
    if (!($106)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[(13252)>>2] = $15;
    $107 = $104 & -2;
    HEAP32[$103>>2] = $107;
    $108 = $15 | 1;
    $$sum20 = (($$sum2) + 4)|0;
    $109 = (($mem) + ($$sum20)|0);
    HEAP32[$109>>2] = $108;
    HEAP32[$9>>2] = $15;
    return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum30 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum30)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum31 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum31)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = (13284 + ($25<<2)|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = ((($22)) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[13244>>2]|0;
     $36 = $35 & $34;
     HEAP32[13244>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre60 = ((($24)) + 8|0);
     $$pre$phi61Z2D = $$pre60;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = ((($24)) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi61Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = ((($22)) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi61Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum22 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum22)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum23 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum23)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum25 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum25)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum24 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum24)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = ((($R$0)) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = ((($R$0)) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0$lcssa>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum29 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum29)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = ((($49)) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = ((($46)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum26 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum26)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (13548 + ($72<<2)|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[(13248)>>2]|0;
      $79 = $78 & $77;
      HEAP32[(13248)>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[(13260)>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = ((($44)) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = ((($44)) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[(13260)>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = ((($R$1)) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum27 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum27)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = ($91>>>0)<($87>>>0);
      if ($93) {
       _abort();
       // unreachable;
      } else {
       $94 = ((($R$1)) + 16|0);
       HEAP32[$94>>2] = $91;
       $95 = ((($91)) + 24|0);
       HEAP32[$95>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum2) + 20)|0;
    $96 = (($mem) + ($$sum28)|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)==(0|0);
    if ($98) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $99 = HEAP32[(13260)>>2]|0;
     $100 = ($97>>>0)<($99>>>0);
     if ($100) {
      _abort();
      // unreachable;
     } else {
      $101 = ((($R$1)) + 20|0);
      HEAP32[$101>>2] = $97;
      $102 = ((($97)) + 24|0);
      HEAP32[$102>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $110 = ($p$0>>>0)<($9>>>0);
 if (!($110)) {
  _abort();
  // unreachable;
 }
 $$sum19 = (($8) + -4)|0;
 $111 = (($mem) + ($$sum19)|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 1;
 $114 = ($113|0)==(0);
 if ($114) {
  _abort();
  // unreachable;
 }
 $115 = $112 & 2;
 $116 = ($115|0)==(0);
 if ($116) {
  $117 = HEAP32[(13268)>>2]|0;
  $118 = ($9|0)==($117|0);
  if ($118) {
   $119 = HEAP32[(13256)>>2]|0;
   $120 = (($119) + ($psize$0))|0;
   HEAP32[(13256)>>2] = $120;
   HEAP32[(13268)>>2] = $p$0;
   $121 = $120 | 1;
   $122 = ((($p$0)) + 4|0);
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[(13264)>>2]|0;
   $124 = ($p$0|0)==($123|0);
   if (!($124)) {
    return;
   }
   HEAP32[(13264)>>2] = 0;
   HEAP32[(13252)>>2] = 0;
   return;
  }
  $125 = HEAP32[(13264)>>2]|0;
  $126 = ($9|0)==($125|0);
  if ($126) {
   $127 = HEAP32[(13252)>>2]|0;
   $128 = (($127) + ($psize$0))|0;
   HEAP32[(13252)>>2] = $128;
   HEAP32[(13264)>>2] = $p$0;
   $129 = $128 | 1;
   $130 = ((($p$0)) + 4|0);
   HEAP32[$130>>2] = $129;
   $131 = (($p$0) + ($128)|0);
   HEAP32[$131>>2] = $128;
   return;
  }
  $132 = $112 & -8;
  $133 = (($132) + ($psize$0))|0;
  $134 = $112 >>> 3;
  $135 = ($112>>>0)<(256);
  do {
   if ($135) {
    $136 = (($mem) + ($8)|0);
    $137 = HEAP32[$136>>2]|0;
    $$sum1718 = $8 | 4;
    $138 = (($mem) + ($$sum1718)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = $134 << 1;
    $141 = (13284 + ($140<<2)|0);
    $142 = ($137|0)==($141|0);
    if (!($142)) {
     $143 = HEAP32[(13260)>>2]|0;
     $144 = ($137>>>0)<($143>>>0);
     if ($144) {
      _abort();
      // unreachable;
     }
     $145 = ((($137)) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = ($146|0)==($9|0);
     if (!($147)) {
      _abort();
      // unreachable;
     }
    }
    $148 = ($139|0)==($137|0);
    if ($148) {
     $149 = 1 << $134;
     $150 = $149 ^ -1;
     $151 = HEAP32[13244>>2]|0;
     $152 = $151 & $150;
     HEAP32[13244>>2] = $152;
     break;
    }
    $153 = ($139|0)==($141|0);
    if ($153) {
     $$pre58 = ((($139)) + 8|0);
     $$pre$phi59Z2D = $$pre58;
    } else {
     $154 = HEAP32[(13260)>>2]|0;
     $155 = ($139>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     }
     $156 = ((($139)) + 8|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($157|0)==($9|0);
     if ($158) {
      $$pre$phi59Z2D = $156;
     } else {
      _abort();
      // unreachable;
     }
    }
    $159 = ((($137)) + 12|0);
    HEAP32[$159>>2] = $139;
    HEAP32[$$pre$phi59Z2D>>2] = $137;
   } else {
    $$sum5 = (($8) + 16)|0;
    $160 = (($mem) + ($$sum5)|0);
    $161 = HEAP32[$160>>2]|0;
    $$sum67 = $8 | 4;
    $162 = (($mem) + ($$sum67)|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = ($163|0)==($9|0);
    do {
     if ($164) {
      $$sum9 = (($8) + 12)|0;
      $175 = (($mem) + ($$sum9)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = ($176|0)==(0|0);
      if ($177) {
       $$sum8 = (($8) + 8)|0;
       $178 = (($mem) + ($$sum8)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      } else {
       $R7$0 = $176;$RP9$0 = $175;
      }
      while(1) {
       $181 = ((($R7$0)) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ($182|0)==(0|0);
       if (!($183)) {
        $R7$0 = $182;$RP9$0 = $181;
        continue;
       }
       $184 = ((($R7$0)) + 16|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = ($185|0)==(0|0);
       if ($186) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $185;$RP9$0 = $184;
       }
      }
      $187 = HEAP32[(13260)>>2]|0;
      $188 = ($RP9$0$lcssa>>>0)<($187>>>0);
      if ($188) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $165 = (($mem) + ($8)|0);
      $166 = HEAP32[$165>>2]|0;
      $167 = HEAP32[(13260)>>2]|0;
      $168 = ($166>>>0)<($167>>>0);
      if ($168) {
       _abort();
       // unreachable;
      }
      $169 = ((($166)) + 12|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==($9|0);
      if (!($171)) {
       _abort();
       // unreachable;
      }
      $172 = ((($163)) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = ($173|0)==($9|0);
      if ($174) {
       HEAP32[$169>>2] = $163;
       HEAP32[$172>>2] = $166;
       $R7$1 = $163;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $189 = ($161|0)==(0|0);
    if (!($189)) {
     $$sum12 = (($8) + 20)|0;
     $190 = (($mem) + ($$sum12)|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (13548 + ($191<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($9|0)==($193|0);
     if ($194) {
      HEAP32[$192>>2] = $R7$1;
      $cond47 = ($R7$1|0)==(0|0);
      if ($cond47) {
       $195 = 1 << $191;
       $196 = $195 ^ -1;
       $197 = HEAP32[(13248)>>2]|0;
       $198 = $197 & $196;
       HEAP32[(13248)>>2] = $198;
       break;
      }
     } else {
      $199 = HEAP32[(13260)>>2]|0;
      $200 = ($161>>>0)<($199>>>0);
      if ($200) {
       _abort();
       // unreachable;
      }
      $201 = ((($161)) + 16|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)==($9|0);
      if ($203) {
       HEAP32[$201>>2] = $R7$1;
      } else {
       $204 = ((($161)) + 20|0);
       HEAP32[$204>>2] = $R7$1;
      }
      $205 = ($R7$1|0)==(0|0);
      if ($205) {
       break;
      }
     }
     $206 = HEAP32[(13260)>>2]|0;
     $207 = ($R7$1>>>0)<($206>>>0);
     if ($207) {
      _abort();
      // unreachable;
     }
     $208 = ((($R7$1)) + 24|0);
     HEAP32[$208>>2] = $161;
     $$sum13 = (($8) + 8)|0;
     $209 = (($mem) + ($$sum13)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     do {
      if (!($211)) {
       $212 = ($210>>>0)<($206>>>0);
       if ($212) {
        _abort();
        // unreachable;
       } else {
        $213 = ((($R7$1)) + 16|0);
        HEAP32[$213>>2] = $210;
        $214 = ((($210)) + 24|0);
        HEAP32[$214>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum14 = (($8) + 12)|0;
     $215 = (($mem) + ($$sum14)|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = ($216|0)==(0|0);
     if (!($217)) {
      $218 = HEAP32[(13260)>>2]|0;
      $219 = ($216>>>0)<($218>>>0);
      if ($219) {
       _abort();
       // unreachable;
      } else {
       $220 = ((($R7$1)) + 20|0);
       HEAP32[$220>>2] = $216;
       $221 = ((($216)) + 24|0);
       HEAP32[$221>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $222 = $133 | 1;
  $223 = ((($p$0)) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($p$0) + ($133)|0);
  HEAP32[$224>>2] = $133;
  $225 = HEAP32[(13264)>>2]|0;
  $226 = ($p$0|0)==($225|0);
  if ($226) {
   HEAP32[(13252)>>2] = $133;
   return;
  } else {
   $psize$1 = $133;
  }
 } else {
  $227 = $112 & -2;
  HEAP32[$111>>2] = $227;
  $228 = $psize$0 | 1;
  $229 = ((($p$0)) + 4|0);
  HEAP32[$229>>2] = $228;
  $230 = (($p$0) + ($psize$0)|0);
  HEAP32[$230>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $231 = $psize$1 >>> 3;
 $232 = ($psize$1>>>0)<(256);
 if ($232) {
  $233 = $231 << 1;
  $234 = (13284 + ($233<<2)|0);
  $235 = HEAP32[13244>>2]|0;
  $236 = 1 << $231;
  $237 = $235 & $236;
  $238 = ($237|0)==(0);
  if ($238) {
   $239 = $235 | $236;
   HEAP32[13244>>2] = $239;
   $$pre = (($233) + 2)|0;
   $$pre57 = (13284 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre57;$F16$0 = $234;
  } else {
   $$sum11 = (($233) + 2)|0;
   $240 = (13284 + ($$sum11<<2)|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[(13260)>>2]|0;
   $243 = ($241>>>0)<($242>>>0);
   if ($243) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $240;$F16$0 = $241;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $244 = ((($F16$0)) + 12|0);
  HEAP32[$244>>2] = $p$0;
  $245 = ((($p$0)) + 8|0);
  HEAP32[$245>>2] = $F16$0;
  $246 = ((($p$0)) + 12|0);
  HEAP32[$246>>2] = $234;
  return;
 }
 $247 = $psize$1 >>> 8;
 $248 = ($247|0)==(0);
 if ($248) {
  $I18$0 = 0;
 } else {
  $249 = ($psize$1>>>0)>(16777215);
  if ($249) {
   $I18$0 = 31;
  } else {
   $250 = (($247) + 1048320)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 8;
   $253 = $247 << $252;
   $254 = (($253) + 520192)|0;
   $255 = $254 >>> 16;
   $256 = $255 & 4;
   $257 = $256 | $252;
   $258 = $253 << $256;
   $259 = (($258) + 245760)|0;
   $260 = $259 >>> 16;
   $261 = $260 & 2;
   $262 = $257 | $261;
   $263 = (14 - ($262))|0;
   $264 = $258 << $261;
   $265 = $264 >>> 15;
   $266 = (($263) + ($265))|0;
   $267 = $266 << 1;
   $268 = (($266) + 7)|0;
   $269 = $psize$1 >>> $268;
   $270 = $269 & 1;
   $271 = $270 | $267;
   $I18$0 = $271;
  }
 }
 $272 = (13548 + ($I18$0<<2)|0);
 $273 = ((($p$0)) + 28|0);
 HEAP32[$273>>2] = $I18$0;
 $274 = ((($p$0)) + 16|0);
 $275 = ((($p$0)) + 20|0);
 HEAP32[$275>>2] = 0;
 HEAP32[$274>>2] = 0;
 $276 = HEAP32[(13248)>>2]|0;
 $277 = 1 << $I18$0;
 $278 = $276 & $277;
 $279 = ($278|0)==(0);
 L199: do {
  if ($279) {
   $280 = $276 | $277;
   HEAP32[(13248)>>2] = $280;
   HEAP32[$272>>2] = $p$0;
   $281 = ((($p$0)) + 24|0);
   HEAP32[$281>>2] = $272;
   $282 = ((($p$0)) + 12|0);
   HEAP32[$282>>2] = $p$0;
   $283 = ((($p$0)) + 8|0);
   HEAP32[$283>>2] = $p$0;
  } else {
   $284 = HEAP32[$272>>2]|0;
   $285 = ((($284)) + 4|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = $286 & -8;
   $288 = ($287|0)==($psize$1|0);
   L202: do {
    if ($288) {
     $T$0$lcssa = $284;
    } else {
     $289 = ($I18$0|0)==(31);
     $290 = $I18$0 >>> 1;
     $291 = (25 - ($290))|0;
     $292 = $289 ? 0 : $291;
     $293 = $psize$1 << $292;
     $K19$052 = $293;$T$051 = $284;
     while(1) {
      $300 = $K19$052 >>> 31;
      $301 = (((($T$051)) + 16|0) + ($300<<2)|0);
      $296 = HEAP32[$301>>2]|0;
      $302 = ($296|0)==(0|0);
      if ($302) {
       $$lcssa = $301;$T$051$lcssa = $T$051;
       break;
      }
      $294 = $K19$052 << 1;
      $295 = ((($296)) + 4|0);
      $297 = HEAP32[$295>>2]|0;
      $298 = $297 & -8;
      $299 = ($298|0)==($psize$1|0);
      if ($299) {
       $T$0$lcssa = $296;
       break L202;
      } else {
       $K19$052 = $294;$T$051 = $296;
      }
     }
     $303 = HEAP32[(13260)>>2]|0;
     $304 = ($$lcssa>>>0)<($303>>>0);
     if ($304) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$$lcssa>>2] = $p$0;
      $305 = ((($p$0)) + 24|0);
      HEAP32[$305>>2] = $T$051$lcssa;
      $306 = ((($p$0)) + 12|0);
      HEAP32[$306>>2] = $p$0;
      $307 = ((($p$0)) + 8|0);
      HEAP32[$307>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $308 = ((($T$0$lcssa)) + 8|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[(13260)>>2]|0;
   $311 = ($309>>>0)>=($310>>>0);
   $not$ = ($T$0$lcssa>>>0)>=($310>>>0);
   $312 = $311 & $not$;
   if ($312) {
    $313 = ((($309)) + 12|0);
    HEAP32[$313>>2] = $p$0;
    HEAP32[$308>>2] = $p$0;
    $314 = ((($p$0)) + 8|0);
    HEAP32[$314>>2] = $309;
    $315 = ((($p$0)) + 12|0);
    HEAP32[$315>>2] = $T$0$lcssa;
    $316 = ((($p$0)) + 24|0);
    HEAP32[$316>>2] = 0;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $317 = HEAP32[(13276)>>2]|0;
 $318 = (($317) + -1)|0;
 HEAP32[(13276)>>2] = $318;
 $319 = ($318|0)==(0);
 if ($319) {
  $sp$0$in$i = (13700);
 } else {
  return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $320 = ($sp$0$i|0)==(0|0);
  $321 = ((($sp$0$i)) + 8|0);
  if ($320) {
   break;
  } else {
   $sp$0$in$i = $321;
  }
 }
 HEAP32[(13276)>>2] = -1;
 return;
}
function runPostSets() {
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memcpy(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return $10$0 | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return $1$0 | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (Math_clz32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0 | 0, $d_sroa_0_0_insert_insert99$1 | 0, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



  
function dynCall_i(index) {
  index = index|0;
  
  return FUNCTION_TABLE_i[index&15]()|0;
}


function dynCall_ii(index,a1) {
  index = index|0;
  a1=a1|0;
  return FUNCTION_TABLE_ii[index&15](a1|0)|0;
}


function dynCall_iiii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  return FUNCTION_TABLE_iiii[index&15](a1|0,a2|0,a3|0)|0;
}


function dynCall_iiiii(index,a1,a2,a3,a4) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return FUNCTION_TABLE_iiiii[index&15](a1|0,a2|0,a3|0,a4|0)|0;
}


function dynCall_vi(index,a1) {
  index = index|0;
  a1=a1|0;
  FUNCTION_TABLE_vi[index&15](a1|0);
}

function b0() {
 ; nullFunc_i(0);return 0;
}
function b1(p0) {
 p0 = p0|0; nullFunc_ii(1);return 0;
}
function b2(p0,p1,p2) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(2);return 0;
}
function b3(p0,p1,p2,p3) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; nullFunc_iiiii(3);return 0;
}
function b4(p0) {
 p0 = p0|0; nullFunc_vi(4);
}

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_i = [b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,_xw_end,_xw_rev,_x_wSL,b0,b0,b0];
var FUNCTION_TABLE_ii = [b1,b1,___stdio_close,b1,b1,b1,b1,b1,b1,_w_ned,b1,b1,b1,b1,b1,b1];
var FUNCTION_TABLE_iiii = [b2,_sn_write,b2,___stdout_write,___stdio_seek,___stdio_write,___stdio_read,b2,_w_ed,b2,b2,b2,b2,b2,b2,b2];
var FUNCTION_TABLE_iiiii = [b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,_l_write,b3,b3];
var FUNCTION_TABLE_vi = [b4,b4,b4,b4,b4,b4,b4,_x_putc,b4,b4,b4,b4,b4,b4,_cleanup,_cleanup52];

  return { _i64Subtract: _i64Subtract, _free: _free, _i64Add: _i64Add, _dgesv_: _dgesv_, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, _setulb_: _setulb_, _bitshift64Lshr: _bitshift64Lshr, _fflush: _fflush, ___errno_location: ___errno_location, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_i: dynCall_i, dynCall_ii: dynCall_ii, dynCall_iiii: dynCall_iiii, dynCall_iiiii: dynCall_iiiii, dynCall_vi: dynCall_vi };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var real__i64Subtract = asm["_i64Subtract"]; asm["_i64Subtract"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Subtract.apply(null, arguments);
};

var real__free = asm["_free"]; asm["_free"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__free.apply(null, arguments);
};

var real__i64Add = asm["_i64Add"]; asm["_i64Add"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__i64Add.apply(null, arguments);
};

var real__dgesv_ = asm["_dgesv_"]; asm["_dgesv_"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__dgesv_.apply(null, arguments);
};

var real__malloc = asm["_malloc"]; asm["_malloc"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__malloc.apply(null, arguments);
};

var real__setulb_ = asm["_setulb_"]; asm["_setulb_"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__setulb_.apply(null, arguments);
};

var real__bitshift64Lshr = asm["_bitshift64Lshr"]; asm["_bitshift64Lshr"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Lshr.apply(null, arguments);
};

var real__fflush = asm["_fflush"]; asm["_fflush"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__fflush.apply(null, arguments);
};

var real____errno_location = asm["___errno_location"]; asm["___errno_location"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real____errno_location.apply(null, arguments);
};

var real__bitshift64Shl = asm["_bitshift64Shl"]; asm["_bitshift64Shl"] = function() {
assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
assert(!runtimeExited, 'the runtime was exited (use NO_EXIT_RUNTIME to keep it alive after main() exits)');
return real__bitshift64Shl.apply(null, arguments);
};
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _free = Module["_free"] = asm["_free"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _dgesv_ = Module["_dgesv_"] = asm["_dgesv_"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _setulb_ = Module["_setulb_"] = asm["_setulb_"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _fflush = Module["_fflush"] = asm["_fflush"];
var ___errno_location = Module["___errno_location"] = asm["___errno_location"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
;

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];

Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];



// === Auto-generated postamble setup entry stuff ===


function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun']) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);


  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret, /* implicit = */ true);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

    if (Module['_main'] && shouldRunNow) Module['callMain'](args);

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status, implicit) {
  if (implicit && Module['noExitRuntime']) {
    Module.printErr('exit(' + status + ') implicitly called by end of main(), but noExitRuntime, so not exiting the runtime (you can use emscripten_force_exit, if you want to force a true shutdown)');
    return;
  }

  if (Module['noExitRuntime']) {
    Module.printErr('exit(' + status + ') called, but noExitRuntime, so halting execution but not exiting the runtime or preventing further async execution (you can use emscripten_force_exit, if you want to force a true shutdown)');
  } else {

    ABORT = true;
    EXITSTATUS = status;
    STACKTOP = initialStackTop;

    exitRuntime();

    if (Module['onExit']) Module['onExit'](status);
  }

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
  if (what !== undefined) {
    Module.print(what);
    Module.printErr(what);
    what = JSON.stringify(what)
  } else {
    what = '';
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  var output = 'abort(' + what + ') at ' + stackTrace() + extra;
  if (abortDecorators) {
    abortDecorators.forEach(function(decorator) {
      output = decorator(output, what);
    });
  }
  throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



//# sourceMappingURL=lbfgsb.c.4.js.map